var api;
(function (api) {
    var Class = (function () {
        function Class(name, fn) {
            this.name = name;
            this.fn = fn;
        }
        Class.prototype.getName = function () {
            return this.name;
        };
        Class.prototype.newInstance = function (constructorParams) {
            var newInstance = Object.create(this.fn.prototype);
            newInstance.constructor.call(newInstance, constructorParams);
            return newInstance;
        };
        return Class;
    })();
    api.Class = Class;
})(api || (api = {}));
var api;
(function (api) {
    var ObjectHelper = (function () {
        function ObjectHelper() {
        }
        ObjectHelper.iFrameSafeInstanceOf = function (obj, fn) {
            if (!fn) {
                console.warn('Undefined fn passed to iFrameSafeInstanceOf, returning false', obj, fn);
                return false;
            }
            if (!obj) {
                return false;
            }
            if (obj instanceof fn) {
                return true;
            }
            if (obj.constructor.name && obj.constructor.name === fn.name) {
                return true;
            }
            else if (Object.prototype.toString.call(obj) === fn.name) {
                return true;
            }
            if (!(typeof obj === 'object')) {
                return false;
            }
            var prototype = Object.getPrototypeOf(obj);
            if (prototype.constructor.name) {
                do {
                    prototype = Object.getPrototypeOf(prototype);
                    if (!prototype) {
                        return false;
                    }
                } while (prototype.constructor.name !== fn.name);
            }
            else {
                do {
                    prototype = Object.getPrototypeOf(prototype);
                    if (!prototype) {
                        return false;
                    }
                } while (Object.prototype.toString.call(prototype) !== fn.name);
            }
            return true;
        };
        ObjectHelper.equals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a.equals(b);
        };
        ObjectHelper.arrayEquals = function (arrayA, arrayB) {
            if (!arrayA && !arrayB) {
                return true;
            }
            else if (!arrayA && arrayB) {
                return false;
            }
            else if (arrayA && !arrayB) {
                return false;
            }
            if (arrayA.length != arrayB.length) {
                return false;
            }
            for (var i = 0; i < arrayA.length; i++) {
                if (!ObjectHelper.equals(arrayA[i], arrayB[i])) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.anyArrayEquals = function (arrayA, arrayB) {
            if (!arrayA && !arrayB) {
                return true;
            }
            else if (!arrayA && arrayB) {
                return false;
            }
            else if (arrayA && !arrayB) {
                return false;
            }
            if (arrayA.length != arrayB.length) {
                return false;
            }
            for (var i = 0; i < arrayA.length; i++) {
                if (!ObjectHelper.objectEquals(arrayA[i], arrayB[i])) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.mapEquals = function (mapA, mapB) {
            if (!mapA && !mapB) {
                return true;
            }
            else if (!mapA && mapB) {
                return false;
            }
            else if (mapA && !mapB) {
                return false;
            }
            var keysA = [];
            for (var keyA in mapA) {
                if (mapA.hasOwnProperty(keyA)) {
                    keysA.push(keyA);
                }
            }
            var keysB = [];
            for (var keyB in mapB) {
                if (mapB.hasOwnProperty(keyB)) {
                    keysB.push(keyB);
                }
            }
            if (!ObjectHelper.stringArrayEquals(keysA, keysB)) {
                return false;
            }
            for (var keyA in keysA) {
                var valueA = mapA[keysA[keyA]];
                var valueB = mapB[keysA[keyA]];
                if (!ObjectHelper.equals(valueA, valueB)) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.stringEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a.toString() === b.toString();
        };
        ObjectHelper.stringArrayEquals = function (arrayA, arrayB) {
            if (!arrayA && !arrayB) {
                return true;
            }
            else if (!arrayA && arrayB) {
                return false;
            }
            else if (arrayA && !arrayB) {
                return false;
            }
            if (arrayA.length != arrayB.length) {
                return false;
            }
            for (var i = 0; i < arrayA.length; i++) {
                if (!ObjectHelper.stringEquals(arrayA[i], arrayB[i])) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.booleanEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a == b;
        };
        ObjectHelper.numberEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a == b;
        };
        ObjectHelper.dateEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a.toISOString() == b.toISOString();
        };
        ObjectHelper.anyEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            var aString = JSON.stringify(a);
            var bString = JSON.stringify(b);
            return aString == bString;
        };
        ObjectHelper.objectEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            var aString = JSON.stringify(a, function (key, value) {
                return (!!key && a == value) ? undefined : value;
            });
            var bString = JSON.stringify(b, function (key, value) {
                return (!!key && b == value) ? undefined : value;
            });
            return aString == bString;
        };
        ObjectHelper.objectPropertyIterator = function (object, callback) {
            var index = 0;
            for (var name in object) {
                if (object.hasOwnProperty(name)) {
                    var property = object[name];
                    callback(name, property, index++);
                }
            }
        };
        return ObjectHelper;
    })();
    api.ObjectHelper = ObjectHelper;
})(api || (api = {}));
var api;
(function (api) {
    var ClassHelper = (function () {
        function ClassHelper() {
        }
        ClassHelper.getFunctionName = function (func) {
            if (func.name) {
                return func.name;
            }
            else {
                var funcNameRegex = /function (.+)\(/;
                var results = (funcNameRegex).exec(func.toString());
                return (results && results.length > 1) ? results[1] : "";
            }
        };
        ClassHelper.getClassName = function (instance) {
            return ClassHelper.getFunctionName(instance["constructor"]);
        };
        ClassHelper.getClass = function (instance) {
            return instance["constructor"];
        };
        ClassHelper.getModuleName = function (instance) {
            var fullName = ClassHelper.getFullName(instance);
            return fullName ? fullName.substr(0, fullName.lastIndexOf(".")) : "";
        };
        ClassHelper.getFullName = function (instance) {
            var constructor = (typeof instance === 'function') ? instance : instance["constructor"];
            return ClassHelper.findPath(window, constructor) || "";
        };
        ClassHelper.findPath = function (obj, constructor, nestLevel) {
            var value, path, nestLevel = nestLevel || 1;
            if (nestLevel > ClassHelper.MAX_NEST_LEVEL) {
                return null;
            }
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (nestLevel == 1 && ClassHelper.ALLOWED_PACKAGES.indexOf(key) < 0) {
                        continue;
                    }
                    value = obj[key];
                    if (!value || value == obj) {
                        continue;
                    }
                    if (typeof value === 'object') {
                        path = ClassHelper.findPath(value, constructor, nestLevel + 1);
                        if (path) {
                            return key + "." + path;
                        }
                    }
                    else if (typeof value === 'function') {
                        if (value == constructor) {
                            return ClassHelper.getFunctionName(constructor);
                        }
                    }
                }
            }
            return path;
        };
        ClassHelper.distanceTo = function (instance, clazz) {
            if (instance.constructor.name === clazz.name) {
                return 0;
            }
            var distance = 0;
            var prototype = Object.getPrototypeOf(instance);
            do {
                prototype = Object.getPrototypeOf(prototype);
                if (!prototype) {
                    return distance;
                }
                distance++;
            } while (prototype.constructor.name !== clazz.name);
            return distance;
        };
        ClassHelper.MAX_NEST_LEVEL = 7;
        ClassHelper.ALLOWED_PACKAGES = ['api', 'app', 'LiveEdit'];
        return ClassHelper;
    })();
    api.ClassHelper = ClassHelper;
})(api || (api = {}));
var api;
(function (api) {
    var BrowserName;
    (function (BrowserName) {
        BrowserName[BrowserName["CHROME"] = 0] = "CHROME";
        BrowserName[BrowserName["FIREFOX"] = 1] = "FIREFOX";
        BrowserName[BrowserName["OPERA"] = 2] = "OPERA";
        BrowserName[BrowserName["SAFARI"] = 3] = "SAFARI";
        BrowserName[BrowserName["MSIE"] = 4] = "MSIE";
        BrowserName[BrowserName["TRIDENT"] = 5] = "TRIDENT";
    })(BrowserName || (BrowserName = {}));
    var BrowserHelper = (function () {
        function BrowserHelper() {
        }
        BrowserHelper.isAvailableBrowser = function () {
            if (!BrowserHelper.BROWSER_NAME) {
                this.init();
            }
            switch (BrowserHelper.BROWSER_NAME) {
                case 0 /* CHROME */:
                    if (BrowserHelper.BROWSER_VERSION >= BrowserHelper.AVAILABLE_VERSIONS[BrowserHelper.BROWSER_NAME]) {
                        return true;
                    }
                    break;
                case 1 /* FIREFOX */:
                    if (BrowserHelper.BROWSER_VERSION >= BrowserHelper.AVAILABLE_VERSIONS[BrowserHelper.BROWSER_NAME]) {
                        return true;
                    }
                    break;
            }
            return false;
        };
        BrowserHelper.isOldBrowser = function () {
            if (!BrowserHelper.BROWSER_NAME) {
                this.init();
            }
            if (BrowserHelper.AVAILABLE_VERSIONS[BrowserHelper.BROWSER_NAME] > BrowserHelper.BROWSER_VERSION) {
                return true;
            }
            return false;
        };
        BrowserHelper.init = function () {
            var M = navigator.userAgent.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i) || [];
            BrowserHelper.BROWSER_NAME = BrowserName[M[1].toLocaleUpperCase()];
            BrowserHelper.BROWSER_VERSION = M[2];
            BrowserHelper.AVAILABLE_VERSIONS[0 /* CHROME */] = "39";
            BrowserHelper.AVAILABLE_VERSIONS[1 /* FIREFOX */] = "27";
        };
        BrowserHelper.AVAILABLE_VERSIONS = {};
        return BrowserHelper;
    })();
    api.BrowserHelper = BrowserHelper;
})(api || (api = {}));
var api;
(function (api) {
    var Name = (function () {
        function Name(name) {
            api.util.assertNotNull(name, "Name cannot be null");
            api.util.assert(!api.util.StringHelper.isEmpty(name), "Name cannot be empty");
            this.value = name;
        }
        Name.prototype.getValue = function () {
            return this.value;
        };
        Name.prototype.toString = function () {
            return this.value;
        };
        Name.prototype.equals = function (o) {
            if (!(api.ObjectHelper.iFrameSafeInstanceOf(o, Name))) {
                return false;
            }
            var other = o;
            if (this.value != other.value) {
                return false;
            }
            return true;
        };
        Name.ensureValidName = function (possibleInvalidName) {
            if (!possibleInvalidName) {
                return "";
            }
            var generated = possibleInvalidName.replace(/[\s+\.\/]/ig, '-').replace(/-{2,}/g, '-').replace(/^-|-$/g, '').toLowerCase();
            return (generated || '').replace(Name.FORBIDDEN_CHARS, '');
        };
        Name.FORBIDDEN_CHARS = /[^a-z0-9\-]+/ig;
        return Name;
    })();
    api.Name = Name;
})(api || (api = {}));
var api;
(function (api) {
    (function (ExceptionType) {
        ExceptionType[ExceptionType["INFO"] = 0] = "INFO";
        ExceptionType[ExceptionType["ERROR"] = 1] = "ERROR";
        ExceptionType[ExceptionType["WARNING"] = 2] = "WARNING";
    })(api.ExceptionType || (api.ExceptionType = {}));
    var ExceptionType = api.ExceptionType;
    var Exception = (function () {
        function Exception(message, type) {
            if (type === void 0) { type = 1 /* ERROR */; }
            this.message = message;
            this.type = type;
        }
        Exception.prototype.getMessage = function () {
            return this.message;
        };
        Exception.prototype.getType = function () {
            return this.type;
        };
        return Exception;
    })();
    api.Exception = Exception;
})(api || (api = {}));
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var api;
(function (api) {
    var AccessDeniedException = (function (_super) {
        __extends(AccessDeniedException, _super);
        function AccessDeniedException(message) {
            _super.call(this, message, 2 /* WARNING */);
        }
        return AccessDeniedException;
    })(api.Exception);
    api.AccessDeniedException = AccessDeniedException;
})(api || (api = {}));
var api;
(function (api) {
    var DefaultErrorHandler = (function () {
        function DefaultErrorHandler() {
        }
        DefaultErrorHandler.handle = function (error) {
            if (api.ObjectHelper.iFrameSafeInstanceOf(error, Error)) {
                console.error(error);
                throw error;
            }
            else if (api.ObjectHelper.iFrameSafeInstanceOf(error, api.AccessDeniedException)) {
                var application = api.app.Application.getApplication();
                var wnd = application.getWindow();
                new api.app.ShowAppLauncherEvent(application, true).fire(wnd.parent);
                new api.app.ShowAppLauncherEvent(application, true).fire(wnd);
            }
            else if (api.ObjectHelper.iFrameSafeInstanceOf(error, api.Exception)) {
                var message = error.getMessage();
                switch (error.getType()) {
                    case 1 /* ERROR */:
                        console.error(message);
                        api.notify.showError(message);
                        break;
                    case 2 /* WARNING */:
                        console.warn(message);
                        api.notify.showWarning(message);
                        break;
                    case 0 /* INFO */:
                        console.info(message);
                        api.notify.showFeedback(message);
                        break;
                }
            }
            else {
                console.error(error);
                api.notify.showError(error.toString());
                throw error;
            }
        };
        return DefaultErrorHandler;
    })();
    api.DefaultErrorHandler = DefaultErrorHandler;
})(api || (api = {}));
var api;
(function (api) {
    var PropertyChangedEvent = (function () {
        function PropertyChangedEvent(propertyName, oldValue, newValue, source) {
            this.propertyName = propertyName;
            this.oldValue = oldValue;
            this.newValue = newValue;
            this.source = source;
        }
        PropertyChangedEvent.prototype.getPropertyName = function () {
            return this.propertyName;
        };
        PropertyChangedEvent.prototype.getOldValue = function () {
            return this.oldValue;
        };
        PropertyChangedEvent.prototype.getNewValue = function () {
            return this.newValue;
        };
        PropertyChangedEvent.prototype.getSource = function () {
            return this.source;
        };
        return PropertyChangedEvent;
    })();
    api.PropertyChangedEvent = PropertyChangedEvent;
})(api || (api = {}));
var api;
(function (api) {
    var ValidityChangedEvent = (function () {
        function ValidityChangedEvent(valid) {
            this.valid = valid;
        }
        ValidityChangedEvent.prototype.isValid = function () {
            return this.valid;
        };
        return ValidityChangedEvent;
    })();
    api.ValidityChangedEvent = ValidityChangedEvent;
})(api || (api = {}));
var api;
(function (api) {
    var Client = (function () {
        function Client() {
            this.id = Date.now().toString();
            this.propertyIdProvider = new api.data.DefaultPropertyIdProvider(this.id);
        }
        Client.get = function () {
            var w = api.dom.WindowDOM.get();
            var topWindow = w.getTopParent().asWindow();
            if (!topWindow.api.Client.instance) {
                topWindow.api.Client.instance = new Client();
            }
            return topWindow.api.Client.instance;
        };
        Client.prototype.getId = function () {
            return this.id;
        };
        Client.prototype.getPropertyIdProvider = function () {
            return this.propertyIdProvider;
        };
        return Client;
    })();
    api.Client = Client;
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        function assert(expression, message) {
            if (!expression) {
                console.error(message);
                throw new Error(message || 'Assertion failed');
            }
        }
        util.assert = assert;
        function assertState(expression, message) {
            assert(expression, 'Illegal state: ' + message);
        }
        util.assertState = assertState;
        function assertNotNull(value, message) {
            assert(value != null, message || 'Value may not be null');
            return value;
        }
        util.assertNotNull = assertNotNull;
        function assertNull(value, message) {
            assert(value == null, message);
        }
        util.assertNull = assertNull;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var ArrayHelper = (function () {
            function ArrayHelper() {
            }
            ArrayHelper.moveElement = function (oldIndex, newIndex, array) {
                var element = array.splice(oldIndex, 1)[0];
                if (newIndex >= array.length) {
                    array[newIndex] = element;
                }
                else {
                    array.splice(newIndex, 0, element);
                }
            };
            ArrayHelper.addUnique = function (value, array) {
                if (array.indexOf(value) === -1) {
                    array.push(value);
                }
            };
            ArrayHelper.removeValue = function (value, array) {
                var index = array.indexOf(value);
                if (index > -1) {
                    array.splice(index, 1);
                }
            };
            ArrayHelper.removeDuplicates = function (array, keyFunction) {
                var seen = {};
                return array.filter(function (item) {
                    var key = keyFunction(item);
                    return seen.hasOwnProperty(key) ? false : (seen[key] = true);
                });
            };
            return ArrayHelper;
        })();
        util.ArrayHelper = ArrayHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var BasePath = (function () {
            function BasePath(elements, elementDivider, absolute) {
                this.elementDivider = elementDivider != null ? elementDivider : BasePath.DEFAULT_ELEMENT_DIVIDER;
                this.absolute = absolute == undefined ? true : absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? this.elementDivider : "") + this.elements.join(this.elementDivider);
            }
            BasePath.prototype.isAbsolute = function () {
                return this.absolute;
            };
            BasePath.prototype.getElements = function () {
                return this.elements;
            };
            BasePath.prototype.getElement = function (index) {
                return this.elements[index];
            };
            BasePath.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            BasePath.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElemements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElemements.push(element);
                    }
                });
                return this.newInstance(parentElemements, this.absolute);
            };
            BasePath.prototype.newInstance = function (elements, absolute) {
                throw new Error("Must be implemented by inheritor");
            };
            BasePath.prototype.toString = function () {
                return this.refString;
            };
            BasePath.removeEmptyElements = function (elements) {
                var filteredElements = [];
                elements.forEach(function (element) {
                    if (element.length > 0) {
                        filteredElements.push(element);
                    }
                });
                return filteredElements;
            };
            BasePath.DEFAULT_ELEMENT_DIVIDER = "/";
            return BasePath;
        })();
        util.BasePath = BasePath;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var UriHelper = (function () {
            function UriHelper() {
            }
            UriHelper.getUri = function (path) {
                var basePath = window['CONFIG'] && window['CONFIG']['baseUri'] || UriHelper.DEFAULT_URI;
                return UriHelper.joinPath(basePath, UriHelper.relativePath(path));
            };
            UriHelper.getAdminUri = function (path) {
                return UriHelper.getUri(UriHelper.joinPath('admin', UriHelper.relativePath(path)));
            };
            UriHelper.getRestUri = function (path) {
                return UriHelper.getAdminUri(UriHelper.joinPath('rest', UriHelper.relativePath(path)));
            };
            UriHelper.getPortalUri = function (path) {
                return UriHelper.getUri(UriHelper.joinPath('admin', '/portal', UriHelper.relativePath(path)));
            };
            UriHelper.relativePath = function (path) {
                if (util.StringHelper.isBlank(path)) {
                    return util.StringHelper.EMPTY_STRING;
                }
                return path.charAt(0) == '/' ? path.substring(1) : path;
            };
            UriHelper.joinPath = function () {
                var paths = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    paths[_i - 0] = arguments[_i];
                }
                return util.StringHelper.removeEmptyStrings(paths).join('/').replace(/(^|[^:])\/{2,}/g, '$1/');
            };
            UriHelper.getUrlLocation = function (url) {
                return util.StringHelper.isBlank(url) ? util.StringHelper.EMPTY_STRING : url.split(/\?|&/i)[0];
            };
            UriHelper.decodeUrlParams = function (url) {
                if (util.StringHelper.isBlank(url)) {
                    return {};
                }
                var array = url.split(/\?|&/i);
                var params = {};
                var param;
                if (array.length > 1) {
                    for (var i = 1; i < array.length; i++) {
                        param = array[i].split('=');
                        params[param[0]] = param.length > 1 ? decodeURIComponent(param[1]) : undefined;
                    }
                }
                return params;
            };
            UriHelper.encodeUrlParams = function (params, prefix) {
                if (!params) {
                    return util.StringHelper.EMPTY_STRING;
                }
                var urlArray = [];
                for (var key in params) {
                    if (params.hasOwnProperty(key) && params[key] != undefined) {
                        var value = params[key];
                        var prefixedKey = prefix ? prefix + "[" + key + "]" : key;
                        if (typeof value == "object") {
                            urlArray.push(this.encodeUrlParams(value, prefixedKey));
                        }
                        else {
                            urlArray.push(encodeURIComponent(prefixedKey) + "=" + encodeURIComponent(value));
                        }
                    }
                }
                return urlArray.join("&");
            };
            UriHelper.DEFAULT_URI = '/';
            return UriHelper;
        })();
        util.UriHelper = UriHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var StringHelper = (function () {
            function StringHelper() {
            }
            StringHelper.limit = function (str, length, ending) {
                if (ending === void 0) { ending = "\u2026"; }
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.substring(0, length) + ending;
            };
            StringHelper.capitalize = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.charAt(0).toUpperCase() + str.slice(1);
            };
            StringHelper.capitalizeAll = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/(?:^|\s)\S/g, function (ch) {
                    return ch.toUpperCase();
                });
            };
            StringHelper.isUpperCase = function (str) {
                if (StringHelper.isEmpty(str)) {
                    return false;
                }
                return str.toUpperCase() == str;
            };
            StringHelper.isLowerCase = function (str) {
                if (StringHelper.isEmpty(str)) {
                    return false;
                }
                return str.toLowerCase() == str;
            };
            StringHelper.isMixedCase = function (str) {
                if (StringHelper.isEmpty(str)) {
                    return false;
                }
                return !StringHelper.isLowerCase(str) && !StringHelper.isUpperCase(str);
            };
            StringHelper.isEmpty = function (str) {
                return !str || str.length == 0;
            };
            StringHelper.isBlank = function (str) {
                return StringHelper.isEmpty(str) || str.trim().length == 0;
            };
            StringHelper.removeCarriageChars = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/\r/g, "");
            };
            StringHelper.removeWhitespaces = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/\s/g, "");
            };
            StringHelper.removeEmptyStrings = function (elements) {
                return !elements ? [] : elements.filter(function (element) {
                    return !StringHelper.isEmpty(element);
                });
            };
            StringHelper.substringBetween = function (str, left, right) {
                if (StringHelper.isEmpty(str) || StringHelper.isEmpty(left) || StringHelper.isEmpty(right)) {
                    return StringHelper.EMPTY_STRING;
                }
                var start = str.indexOf(left);
                if (start !== -1) {
                    var end = str.indexOf(right, start + left.length);
                    if (end !== -1) {
                        return str.substring(start + left.length, end);
                    }
                }
                return StringHelper.EMPTY_STRING;
            };
            StringHelper.format = function (str) {
                var tokens = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    tokens[_i - 1] = arguments[_i];
                }
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/\{\{|\}\}|\{(\d+)\}/g, function (m, n) {
                    if (m == "{{") {
                        return "{";
                    }
                    if (m == "}}") {
                        return "}";
                    }
                    return tokens[n];
                });
            };
            StringHelper.EMPTY_STRING = "";
            return StringHelper;
        })();
        util.StringHelper = StringHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Animation = (function () {
            function Animation(duration, interval) {
                if (interval === void 0) { interval = Animation.DEFAULT_INTERVAL; }
                this.running = false;
                this.duration = duration;
                this.interval = interval;
            }
            Animation.prototype.onStep = function (doStep) {
                this.doStep = doStep;
            };
            Animation.prototype.start = function () {
                var _this = this;
                var startTime = this.getCurrentTime();
                this.id = setInterval(function () {
                    var progress = Math.min((_this.getCurrentTime() - startTime) / _this.duration, 1);
                    if (_this.doStep) {
                        _this.doStep(progress);
                    }
                    if (progress == 1) {
                        _this.stop();
                    }
                }, this.interval);
                this.running = true;
            };
            Animation.prototype.stop = function () {
                clearInterval(this.id);
                this.running = false;
            };
            Animation.prototype.isRunning = function () {
                return this.running;
            };
            Animation.prototype.getCurrentTime = function () {
                return new Date().getTime();
            };
            Animation.DEFAULT_INTERVAL = 10;
            return Animation;
        })();
        util.Animation = Animation;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var CookieHelper = (function () {
            function CookieHelper() {
            }
            CookieHelper.setCookie = function (name, value, days) {
                if (days === void 0) { days = 1; }
                if (days) {
                    var date = new Date();
                    date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
                    var expires = '; expires=' + date.toUTCString();
                }
                else {
                    var expires = '';
                }
                document.cookie = CookieHelper.escape(name) + '=' + CookieHelper.escape(value) + expires + '; path=/';
            };
            CookieHelper.getCookie = function (name) {
                var nameEQ = CookieHelper.escape(name) + '=';
                var ca = document.cookie.split(';');
                for (var i = 0; i < ca.length; i++) {
                    var c = ca[i];
                    while (c.charAt(0) === ' ') {
                        c = c.substring(1, c.length);
                    }
                    if (c.indexOf(nameEQ) === 0) {
                        return CookieHelper.unescape(c.substring(nameEQ.length, c.length));
                    }
                }
                return null;
            };
            CookieHelper.removeCookie = function (name) {
                CookieHelper.setCookie(name, '', -1);
            };
            CookieHelper.escape = function (value) {
                return encodeURIComponent(value);
            };
            CookieHelper.unescape = function (value) {
                return decodeURIComponent(value);
            };
            return CookieHelper;
        })();
        util.CookieHelper = CookieHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var DateHelper = (function () {
            function DateHelper() {
            }
            DateHelper.isInvalidDate = function (value) {
                return isNaN(value.getTime());
            };
            DateHelper.getTZOffset = function () {
                return (new Date().getTimezoneOffset() / 60) * -1;
            };
            DateHelper.parseUTCTime = function (localTime) {
                var values = localTime.split(':');
                var date = new Date();
                var localHours = Number(values[0]);
                if (values.length == 3) {
                    date.setHours(localHours, Number(values[1]), Number(values[2]));
                }
                else {
                    date.setHours(localHours, Number(values[1]));
                }
                var hoursAsString = "" + date.getUTCHours();
                var minutesAsString = "" + date.getUTCMinutes();
                return hoursAsString + ":" + minutesAsString;
            };
            DateHelper.parseUTCDate = function (value) {
                var parsedYear = Number(value.substring(0, 4));
                var parsedMonth = Number(value.substring(5, 7));
                var parsedDayOfMonth = Number(value.substring(8, 10));
                return api.util.DateHelper.newUTCDate(parsedYear, parsedMonth - 1, parsedDayOfMonth);
            };
            DateHelper.parseUTCDateTime = function (value) {
                var parsedYear = Number(value.substring(0, 4));
                var parsedMonth = Number(value.substring(5, 7));
                var parsedDayOfMonth = Number(value.substring(8, 10));
                var parsedHours = Number(value.substring(11, 13));
                var parsedMinutes = Number(value.substring(14, 16));
                var parsedSeconds = Number(value.substring(17, 19));
                return api.util.DateHelper.newUTCDateTime(parsedYear, parsedMonth - 1, parsedDayOfMonth, parsedHours, parsedMinutes, parsedSeconds);
            };
            DateHelper.newUTCDate = function (year, month, date) {
                return new Date(Date.UTC(year, month, date));
            };
            DateHelper.newUTCDateTime = function (year, month, date, hours, minutes, seconds) {
                if (seconds === void 0) { seconds = 0; }
                return new Date(Date.UTC(year, month, date, hours, minutes, seconds));
            };
            DateHelper.formatUTCDate = function (date) {
                var yearAsString = "" + date.getUTCFullYear();
                return yearAsString + "-" + this.padNumber(date.getUTCMonth() + 1) + "-" + this.padNumber(date.getUTCDate());
            };
            DateHelper.padNumber = function (num) {
                return (num < 10 ? '0' : '') + num;
            };
            DateHelper.formatUTCDateTime = function (date) {
                var dateAsString = DateHelper.formatUTCDate(date);
                return dateAsString + "T" + this.padNumber(date.getUTCHours()) + ":" + this.padNumber(date.getUTCMinutes()) + ":" + this.padNumber(date.getUTCSeconds());
            };
            return DateHelper;
        })();
        util.DateHelper = DateHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var NumberHelper = (function () {
            function NumberHelper() {
            }
            NumberHelper.isWholeNumber = function (value) {
                return NumberHelper.isNumber(value) && value % 1 == 0;
            };
            NumberHelper.isNumber = function (value) {
                return typeof value === 'number' && !isNaN(value);
            };
            NumberHelper.randomBetween = function (from, to) {
                return from + Math.round(Math.random() * (to - from));
            };
            return NumberHelper;
        })();
        util.NumberHelper = NumberHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Flow = (function () {
            function Flow(thisOfProducer) {
                this.thisOfProducer = thisOfProducer;
            }
            Flow.prototype.getThisOfProducer = function () {
                return this.thisOfProducer;
            };
            Flow.prototype.doExecute = function (context) {
                return this.doExecuteNext(context);
            };
            Flow.prototype.doExecuteNext = function (context) {
                throw new Error("Must be implemented by inheritor");
            };
            return Flow;
        })();
        util.Flow = Flow;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Link = (function () {
            function Link(value) {
                this.path = value;
            }
            Link.prototype.getPath = function () {
                return this.path;
            };
            Link.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Link)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.path, other.path)) {
                    return false;
                }
            };
            Link.prototype.toString = function () {
                return this.path;
            };
            return Link;
        })();
        util.Link = Link;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var DelayedFunctionCall = (function () {
            function DelayedFunctionCall(functionToCall, context, delay) {
                if (delay === void 0) { delay = 500; }
                this.functionToCall = functionToCall;
                this.delay = delay;
                this.context = context;
            }
            DelayedFunctionCall.prototype.delayCall = function () {
                var _this = this;
                if (this.timerId) {
                    window.clearTimeout(this.timerId);
                }
                this.timerId = window.setTimeout(function () {
                    _this.functionToCall.call(_this.context);
                    _this.timerId = null;
                }, this.delay);
            };
            return DelayedFunctionCall;
        })();
        util.DelayedFunctionCall = DelayedFunctionCall;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var GeoPoint = (function () {
            function GeoPoint(latitude, longitude) {
                this.latitude = latitude;
                this.longitude = longitude;
            }
            GeoPoint.prototype.getLatitude = function () {
                return this.latitude;
            };
            GeoPoint.prototype.getLongitude = function () {
                return this.longitude;
            };
            GeoPoint.prototype.toString = function () {
                return "" + this.latitude + "," + this.longitude;
            };
            GeoPoint.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, GeoPoint)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.latitude, other.latitude)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.longitude, other.longitude)) {
                    return false;
                }
                return true;
            };
            GeoPoint.isValidString = function (s) {
                if (util.StringHelper.isBlank(s)) {
                    return false;
                }
                var indexOfComma = s.indexOf(',');
                if (indexOfComma < 1 || s.split(',').length != 2) {
                    return false;
                }
                else if (indexOfComma == s.length - 1) {
                    return false;
                }
                var coordinates = s.split(',');
                var latitude = Number(coordinates[0]);
                var longitude = Number(coordinates[1]);
                return typeof latitude === 'number' && !isNaN(latitude) && typeof longitude === 'number' && !isNaN(longitude);
            };
            GeoPoint.fromString = function (s) {
                if (!GeoPoint.isValidString(s)) {
                    throw new Error("Cannot parse GeoPoint from string: " + s);
                }
                var coordinates = s.split(',');
                var latitude = Number(coordinates[0]);
                var longitude = Number(coordinates[1]);
                return new GeoPoint(latitude, longitude);
            };
            return GeoPoint;
        })();
        util.GeoPoint = GeoPoint;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var LocalTime = (function () {
            function LocalTime(hours, minutes, tzo, seconds) {
                this.hours = hours;
                this.minutes = minutes;
                this.tzo = tzo;
                if (seconds) {
                    this.seconds = seconds;
                }
            }
            LocalTime.prototype.getHours = function () {
                return this.hours;
            };
            LocalTime.prototype.getMinutes = function () {
                return this.minutes;
            };
            LocalTime.prototype.getSeconds = function () {
                return this.seconds;
            };
            LocalTime.prototype.getTZO = function () {
                return this.tzo;
            };
            LocalTime.prototype.toString = function () {
                if (this.seconds) {
                    return this.padNumber(this.hours) + ":" + this.padNumber(this.minutes) + ":" + this.padNumber(this.seconds);
                }
                else {
                    return this.padNumber(this.hours) + ":" + this.padNumber(this.minutes);
                }
            };
            LocalTime.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, LocalTime)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.hours, other.hours)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.minutes, other.minutes)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.seconds, other.seconds)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.tzo, other.tzo)) {
                    return false;
                }
                return true;
            };
            LocalTime.prototype.padNumber = function (num) {
                return (num < 10 ? '0' : '') + num;
            };
            LocalTime.isValidString = function (s) {
                if (util.StringHelper.isBlank(s)) {
                    return false;
                }
                var re = /^[0-2]?\d:[0-5]?\d$/;
                return re.test(s);
            };
            LocalTime.fromString = function (s) {
                if (!LocalTime.isValidString(s)) {
                    throw new Error("Cannot parse LocalTime from string: " + s);
                }
                var tzo = api.util.DateHelper.getTZOffset();
                var localTime = s.split(':');
                var hours = Number(localTime[0]);
                var minutes = Number(localTime[1]);
                return new LocalTime(hours, minutes, tzo);
            };
            LocalTime.prototype.getAdjustedTime = function () {
                var date = new Date();
                date.setHours(this.getHours() + util.DateHelper.getTZOffset(), this.getMinutes());
                return {
                    hour: date.getHours(),
                    minute: date.getMinutes()
                };
            };
            return LocalTime;
        })();
        util.LocalTime = LocalTime;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var BinaryReference = (function () {
            function BinaryReference(value) {
                this.value = value;
            }
            BinaryReference.prototype.getValue = function () {
                return this.value;
            };
            BinaryReference.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, BinaryReference)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                    return false;
                }
            };
            BinaryReference.prototype.toString = function () {
                return this.value;
            };
            return BinaryReference;
        })();
        util.BinaryReference = BinaryReference;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Reference = (function () {
            function Reference(value) {
                this.referenceId = value;
            }
            Reference.from = function (value) {
                return new Reference(value.toString());
            };
            Reference.prototype.getNodeId = function () {
                return this.referenceId;
            };
            Reference.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Reference)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.referenceId, other.referenceId)) {
                    return false;
                }
                return true;
            };
            Reference.prototype.toString = function () {
                return this.referenceId;
            };
            return Reference;
        })();
        util.Reference = Reference;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var PromiseHelper = (function () {
            function PromiseHelper() {
            }
            PromiseHelper.newResolvedVoidPromise = function () {
                var deferred = wemQ.defer();
                deferred.resolve(null);
                return deferred.promise;
            };
            return PromiseHelper;
        })();
        util.PromiseHelper = PromiseHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                var LoaderEvent = (function () {
                    function LoaderEvent() {
                    }
                    return LoaderEvent;
                })();
                event.LoaderEvent = LoaderEvent;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                var LoadedDataEvent = (function (_super) {
                    __extends(LoadedDataEvent, _super);
                    function LoadedDataEvent(data) {
                        _super.call(this);
                        this.data = data;
                    }
                    LoadedDataEvent.prototype.getData = function () {
                        return this.data;
                    };
                    return LoadedDataEvent;
                })(event.LoaderEvent);
                event.LoadedDataEvent = LoadedDataEvent;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                var LoadingDataEvent = (function (_super) {
                    __extends(LoadingDataEvent, _super);
                    function LoadingDataEvent() {
                        _super.call(this);
                    }
                    return LoadingDataEvent;
                })(event.LoaderEvent);
                event.LoadingDataEvent = LoadingDataEvent;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                (function (LoaderEvents) {
                    LoaderEvents[LoaderEvents["LoadedData"] = 0] = "LoadedData";
                    LoaderEvents[LoaderEvents["LoadingData"] = 1] = "LoadingData";
                })(event.LoaderEvents || (event.LoaderEvents = {}));
                var LoaderEvents = event.LoaderEvents;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var LoadedDataEvent = api.util.loader.event.LoadedDataEvent;
            var LoadingDataEvent = api.util.loader.event.LoadingDataEvent;
            var LoaderStatus;
            (function (LoaderStatus) {
                LoaderStatus[LoaderStatus["NOT_STARTED"] = 0] = "NOT_STARTED";
                LoaderStatus[LoaderStatus["LOADING"] = 1] = "LOADING";
                LoaderStatus[LoaderStatus["LOADED"] = 2] = "LOADED";
            })(LoaderStatus || (LoaderStatus = {}));
            var BaseLoader = (function () {
                function BaseLoader(request) {
                    this.status = 0 /* NOT_STARTED */;
                    this.loadedDataListeners = [];
                    this.loadingDataListeners = [];
                    this.setRequest(request);
                }
                BaseLoader.prototype.sendRequest = function () {
                    return this.request.sendAndParse();
                };
                BaseLoader.prototype.load = function () {
                    var _this = this;
                    this.notifyLoadingData();
                    return this.sendRequest().then(function (results) {
                        _this.results = results;
                        if (_this.comparator) {
                            _this.results = results.sort(_this.comparator.compare);
                        }
                        _this.notifyLoadedData(results);
                        return _this.results;
                    });
                };
                BaseLoader.prototype.isLoading = function () {
                    return this.status == 1 /* LOADING */;
                };
                BaseLoader.prototype.isLoaded = function () {
                    return this.status == 2 /* LOADED */;
                };
                BaseLoader.prototype.isNotStarted = function () {
                    return this.status == 0 /* NOT_STARTED */;
                };
                BaseLoader.prototype.setComparator = function (comparator) {
                    this.comparator = comparator;
                    return this;
                };
                BaseLoader.prototype.setRequest = function (request) {
                    this.request = request;
                };
                BaseLoader.prototype.getRequest = function () {
                    return this.request;
                };
                BaseLoader.prototype.search = function (searchString) {
                    var deferred = wemQ.defer();
                    this.searchString = searchString;
                    if (this.results) {
                        var filtered = this.results.filter(this.filterFn, this);
                        this.notifyLoadedData(filtered);
                        deferred.resolve(this.results);
                    }
                    else {
                        deferred.resolve(null);
                    }
                    return deferred.promise;
                };
                BaseLoader.prototype.getSearchString = function () {
                    return this.searchString;
                };
                BaseLoader.prototype.filterFn = function (result) {
                    throw Error("must be implemented");
                };
                BaseLoader.prototype.notifyLoadedData = function (results) {
                    var _this = this;
                    this.status = 2 /* LOADED */;
                    this.loadedDataListeners.forEach(function (listener) {
                        listener.call(_this, new LoadedDataEvent(results));
                    });
                };
                BaseLoader.prototype.notifyLoadingData = function () {
                    var _this = this;
                    this.status = 1 /* LOADING */;
                    this.loadingDataListeners.forEach(function (listener) {
                        listener.call(_this, new LoadingDataEvent());
                    });
                };
                BaseLoader.prototype.onLoadedData = function (listener) {
                    this.loadedDataListeners.push(listener);
                };
                BaseLoader.prototype.onLoadingData = function (listener) {
                    this.loadingDataListeners.push(listener);
                };
                BaseLoader.prototype.unLoadedData = function (listener) {
                    this.loadedDataListeners = this.loadedDataListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                BaseLoader.prototype.unLoadingData = function (listener) {
                    this.loadingDataListeners = this.loadingDataListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                return BaseLoader;
            })();
            loader.BaseLoader = BaseLoader;
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var ImageLoader = (function () {
                function ImageLoader() {
                }
                ImageLoader.get = function (url, width, height) {
                    var imageFound = false;
                    var returnImage;
                    url = encodeURI(url);
                    for (var i in ImageLoader.cachedImages) {
                        if (ImageLoader.cachedImages[i].src == url) {
                            imageFound = true;
                            returnImage = ImageLoader.cachedImages[i];
                        }
                    }
                    if (!imageFound) {
                        var image = new Image(width, height);
                        image.src = url;
                        ImageLoader.cachedImages[ImageLoader.cachedImages.length + 1] = image;
                        returnImage = image;
                    }
                    return returnImage;
                };
                ImageLoader.cachedImages = [];
                return ImageLoader;
            })();
            loader.ImageLoader = ImageLoader;
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var i18n;
    (function (i18n) {
        var messages = {};
        var currentLocale = 'en';
        function setLocale(locale) {
            currentLocale = locale;
        }
        i18n.setLocale = setLocale;
        function addBundle(locale, bundle) {
            if (messages[locale] == undefined) {
                messages[locale] = bundle;
            }
            else {
                wemjq.extend(messages[locale], messages[locale], bundle);
            }
        }
        i18n.addBundle = addBundle;
        function message(key, args) {
            var message = key;
            var current = messages[currentLocale];
            if ((current != undefined) && (current[key] != undefined)) {
                message = current[key];
            }
            return message.replace(/\$(\d+)/g, function () {
                return args[arguments[1] - 1];
            });
        }
        i18n.message = message;
    })(i18n = api.i18n || (api.i18n = {}));
})(api || (api = {}));
var _i18n = function (key) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    return api.i18n.message(key, args);
};
var api;
(function (api) {
    var i18n;
    (function (i18n) {
        var messages = {};
        i18n.addBundle('no', messages);
    })(i18n = api.i18n || (api.i18n = {}));
})(api || (api = {}));
var api;
(function (api) {
    var cache;
    (function (cache) {
        var Cache = (function () {
            function Cache() {
                this.objectsByKey = {};
            }
            Cache.prototype.getAll = function () {
                var all = [];
                for (var key in this.objectsByKey) {
                    if (this.objectsByKey.hasOwnProperty(key)) {
                        all.push(this.objectsByKey[key]);
                    }
                }
                return all;
            };
            Cache.prototype.copy = function (object) {
                throw new Error("Must be implemented by inheritor");
            };
            Cache.prototype.getKeyFromObject = function (object) {
                throw new Error("Must be implemented by inheritor");
            };
            Cache.prototype.getKeyAsString = function (object) {
                throw new Error("Must be implemented by inheritor");
            };
            Cache.prototype.put = function (object) {
                var copy = this.copy(object);
                var keyAsString = this.getKeyAsString(this.getKeyFromObject(object));
                this.objectsByKey[keyAsString] = copy;
            };
            Cache.prototype.deleteByKey = function (key) {
                var keyAsString = this.getKeyAsString(key);
                delete this.objectsByKey[keyAsString];
            };
            Cache.prototype.getByKey = function (key) {
                var keyAsString = this.getKeyAsString(key);
                var object = this.objectsByKey[keyAsString];
                if (!object) {
                    return null;
                }
                return this.copy(object);
            };
            return Cache;
        })();
        cache.Cache = Cache;
    })(cache = api.cache || (api.cache = {}));
})(api || (api = {}));
var api;
(function (api) {
    var event;
    (function (_event) {
        var Event = (function () {
            function Event(name) {
                this.name = name || api.ClassHelper.getFullName(this);
            }
            Event.prototype.getName = function () {
                return this.name;
            };
            Event.prototype.fire = function (contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                _event.EventBus.fireEvent(this, contextWindow);
            };
            Event.bind = function (name, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                _event.EventBus.onEvent(name, handler, contextWindow);
            };
            Event.unbind = function (name, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                _event.EventBus.unEvent(name, handler, contextWindow);
            };
            return Event;
        })();
        _event.Event = Event;
    })(event = api.event || (api.event = {}));
})(api || (api = {}));
var api;
(function (api) {
    var event;
    (function (_event) {
        var EventBus = (function () {
            function EventBus() {
            }
            EventBus.onEvent = function (eventName, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                var customEventHandler = function (customEvent) { return handler(customEvent['apiEventObj']); };
                if (!EventBus.handlersMap[eventName]) {
                    EventBus.handlersMap[eventName] = [];
                }
                EventBus.handlersMap[eventName].push({
                    customEventHandler: customEventHandler,
                    apiEventHandler: handler
                });
                contextWindow.addEventListener(eventName, customEventHandler);
            };
            EventBus.unEvent = function (eventName, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                if (handler) {
                    var customEventHandler;
                    EventBus.handlersMap[eventName] = EventBus.handlersMap[eventName].filter(function (entry) {
                        if (entry.apiEventHandler === handler) {
                            customEventHandler = entry.customEventHandler;
                        }
                        return entry.apiEventHandler != handler;
                    });
                    contextWindow.removeEventListener(eventName, customEventHandler);
                }
                else {
                    (EventBus.handlersMap[eventName] || []).forEach(function (entry) {
                        contextWindow.removeEventListener(eventName, entry.customEventHandler);
                    });
                    EventBus.handlersMap[eventName] = [];
                }
            };
            EventBus.fireEvent = function (apiEventObj, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                var customEvent = contextWindow.document.createEvent('Event');
                customEvent.initEvent(apiEventObj.getName(), true, true);
                customEvent['apiEventObj'] = apiEventObj;
                contextWindow.dispatchEvent(customEvent);
            };
            EventBus.handlersMap = {};
            return EventBus;
        })();
        _event.EventBus = EventBus;
    })(event = api.event || (api.event = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var WindowDOM = (function () {
            function WindowDOM() {
                this.el = window;
            }
            WindowDOM.get = function () {
                return WindowDOM.instance;
            };
            WindowDOM.prototype.asWindow = function () {
                return this.el;
            };
            WindowDOM.prototype.getTopParent = function () {
                var parent = this.getParent();
                if (!parent) {
                    return null;
                }
                var i = 0;
                do {
                    var next = parent.getParent();
                    if (!next) {
                        return parent;
                    }
                    parent = next;
                    i++;
                } while (i < 10);
                return null;
            };
            WindowDOM.prototype.getParent = function () {
                var parent = this.el.parent;
                if (parent === this.el) {
                    return null;
                }
                return parent.api.dom.WindowDOM.get();
            };
            WindowDOM.prototype.getHTMLElement = function () {
                return this.el;
            };
            WindowDOM.prototype.getScrollTop = function () {
                return wemjq(this.el).scrollTop();
            };
            WindowDOM.prototype.onResized = function (listener, element) {
                var _this = this;
                this.el.addEventListener("resize", listener);
                if (element) {
                    element.onRemoved(function () { return _this.unResized(listener); });
                }
            };
            WindowDOM.prototype.unResized = function (listener) {
                this.el.removeEventListener("resize", listener);
            };
            WindowDOM.prototype.getWidth = function () {
                return wemjq(this.el).innerWidth();
            };
            WindowDOM.prototype.getHeight = function () {
                return wemjq(this.el).innerHeight();
            };
            WindowDOM.prototype.onScroll = function (listener, element) {
                var _this = this;
                this.el.addEventListener('scroll', listener);
                if (element) {
                    element.onRemoved(function () { return _this.unScroll(listener); });
                }
            };
            WindowDOM.prototype.unScroll = function (listener) {
                this.el.removeEventListener('scroll', listener);
            };
            WindowDOM.instance = new WindowDOM();
            return WindowDOM;
        })();
        dom.WindowDOM = WindowDOM;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementHelper = (function () {
            function ElementHelper(element) {
                api.util.assertNotNull(element, 'Element cannot be null');
                this.el = element;
            }
            ElementHelper.fromName = function (name) {
                api.util.assert(!api.util.StringHelper.isEmpty(name), 'Tag name cannot be empty');
                return new ElementHelper(document.createElement(name));
            };
            ElementHelper.prototype.getHTMLElement = function () {
                return this.el;
            };
            ElementHelper.prototype.insertBefore = function (newEl, existingEl) {
                api.util.assertNotNull(newEl, 'New element cannot be null');
                api.util.assertNotNull(existingEl, 'Existing element cannot be null');
                this.el.insertBefore(newEl.getHTMLElement(), existingEl ? existingEl.getHTMLElement() : null);
            };
            ElementHelper.prototype.insertBeforeEl = function (existingEl) {
                existingEl.el.parentElement.insertBefore(this.el, existingEl.el);
            };
            ElementHelper.prototype.insertAfterEl = function (existingEl) {
                api.util.assertNotNull(existingEl, 'Existing element cannot be null');
                api.util.assertNotNull(existingEl.el.parentElement, 'Existing element\'s parentElement cannot be null');
                existingEl.el.parentElement.insertBefore(this.el, existingEl.el.nextElementSibling);
            };
            ElementHelper.prototype.insertAfterThisEl = function (toInsert) {
                api.util.assertNotNull(toInsert, 'Existing element cannot be null');
                this.el.parentElement.insertBefore(toInsert.el, this.el.nextElementSibling);
            };
            ElementHelper.prototype.getPrevious = function () {
                var previous = this.el.previousSibling;
                while (previous && previous.nodeType != Node.ELEMENT_NODE) {
                    previous = previous.previousSibling;
                }
                return previous ? new ElementHelper(previous) : null;
            };
            ElementHelper.prototype.getNext = function () {
                var next = this.el.nextSibling;
                while (next && next.nodeType != Node.ELEMENT_NODE) {
                    next = next.nextSibling;
                }
                return next ? new ElementHelper(next) : null;
            };
            ElementHelper.prototype.setDisabled = function (value) {
                this.el.disabled = value;
                return this;
            };
            ElementHelper.prototype.isDisabled = function () {
                return this.el.disabled;
            };
            ElementHelper.prototype.getId = function () {
                return this.el.id;
            };
            ElementHelper.prototype.setId = function (value) {
                this.el.id = value;
                return this;
            };
            ElementHelper.prototype.setInnerHtml = function (value) {
                wemjq(this.el).html(value);
                return this;
            };
            ElementHelper.prototype.getInnerHtml = function () {
                return this.el.innerHTML;
            };
            ElementHelper.prototype.setText = function (value) {
                wemjq(this.el).text(value);
                return this;
            };
            ElementHelper.prototype.getText = function () {
                return this.el.innerText || this.el.textContent;
            };
            ElementHelper.prototype.setAttribute = function (name, value) {
                this.el.setAttribute(name, value);
                return this;
            };
            ElementHelper.prototype.getAttribute = function (name) {
                return this.el.getAttribute(name);
            };
            ElementHelper.prototype.hasAttribute = function (name) {
                return this.el.hasAttribute(name);
            };
            ElementHelper.prototype.removeAttribute = function (name) {
                this.el.removeAttribute(name);
                return this;
            };
            ElementHelper.prototype.setData = function (name, value) {
                api.util.assert(!api.util.StringHelper.isEmpty(name), 'Name cannot be empty');
                api.util.assert(!api.util.StringHelper.isEmpty(value), 'Value cannot be empty');
                this.el.setAttribute('data-' + name, value);
                wemjq(this.el).data(name, value);
                return this;
            };
            ElementHelper.prototype.getData = function (name) {
                return wemjq(this.el).data(name);
            };
            ElementHelper.prototype.getValue = function () {
                return this.el['value'];
            };
            ElementHelper.prototype.setValue = function (value) {
                this.el['value'] = value;
                return this;
            };
            ElementHelper.prototype.addClass = function (clsName) {
                var _this = this;
                api.util.assert(!api.util.StringHelper.isEmpty(clsName), 'Class name cannot be empty');
                var classList = clsName.split(" ");
                classList.forEach(function (classItem) {
                    if (_this.el.classList && !_this.hasClass(classItem)) {
                        _this.el.classList.add(classItem);
                    }
                });
                return this;
            };
            ElementHelper.prototype.setClass = function (value) {
                this.el.className = value;
                return this;
            };
            ElementHelper.prototype.getClass = function () {
                return this.el.className;
            };
            ElementHelper.prototype.setTitle = function (value) {
                this.el.title = value;
                return this;
            };
            ElementHelper.prototype.getTitle = function () {
                return this.el.title;
            };
            ElementHelper.prototype.hasClass = function (clsName) {
                api.util.assert(!api.util.StringHelper.isEmpty(clsName), 'Class name cannot be empty');
                var classList = clsName.split(" ");
                for (var i = 0; i < classList.length; i++) {
                    var classItem = classList[i];
                    if (!this.el.classList || !this.el.classList.contains(classItem)) {
                        return false;
                    }
                }
                return true;
            };
            ElementHelper.prototype.removeClass = function (clsName) {
                var _this = this;
                api.util.assert(!api.util.StringHelper.isEmpty(clsName), 'Class name cannot be empty');
                var classList = clsName.split(" ");
                classList.forEach(function (classItem) {
                    if (_this.el.classList) {
                        _this.el.classList.remove(classItem);
                    }
                });
                return this;
            };
            ElementHelper.prototype.addEventListener = function (eventName, f) {
                this.el.addEventListener(eventName, f);
                return this;
            };
            ElementHelper.prototype.removeEventListener = function (eventName, f) {
                this.el.removeEventListener(eventName, f);
                return this;
            };
            ElementHelper.prototype.appendChild = function (child) {
                return this.insertChild(child, this.el.children.length);
            };
            ElementHelper.prototype.appendChildren = function (children) {
                var _this = this;
                children.forEach(function (child) {
                    _this.el.appendChild(child);
                });
                return this;
            };
            ElementHelper.prototype.insertChild = function (child, index) {
                if (index > this.el.children.length - 1) {
                    this.el.appendChild(child);
                }
                else {
                    this.el.insertBefore(child, this.el.children.item(index));
                }
                return this;
            };
            ElementHelper.prototype.getTagName = function () {
                return this.el.tagName;
            };
            ElementHelper.prototype.getDisplay = function () {
                return this.el.style.display;
            };
            ElementHelper.prototype.setDisplay = function (value) {
                this.el.style.display = value;
                return this;
            };
            ElementHelper.prototype.getVisibility = function () {
                return this.el.style.visibility;
            };
            ElementHelper.prototype.setVisibility = function (value) {
                this.el.style.visibility = value;
                return this;
            };
            ElementHelper.prototype.getPosition = function () {
                return this.getComputedProperty('position');
            };
            ElementHelper.prototype.setPosition = function (value) {
                this.el.style.position = value;
                return this;
            };
            ElementHelper.prototype.setWidth = function (value) {
                this.el.style.width = value;
                return this;
            };
            ElementHelper.prototype.setWidthPx = function (value) {
                this.setWidth(value + "px");
                return this;
            };
            ElementHelper.prototype.setMaxWidth = function (value) {
                this.el.style.maxWidth = value;
                return this;
            };
            ElementHelper.prototype.setMaxWidthPx = function (value) {
                this.setMaxWidth(value + "px");
                return this;
            };
            ElementHelper.prototype.getWidth = function () {
                return wemjq(this.el).innerWidth();
            };
            ElementHelper.prototype.getWidthWithoutPadding = function () {
                return wemjq(this.el).width();
            };
            ElementHelper.prototype.getWidthWithBorder = function () {
                return wemjq(this.el).outerWidth();
            };
            ElementHelper.prototype.getWidthWithMargin = function () {
                return wemjq(this.el).outerWidth(true);
            };
            ElementHelper.prototype.getMinWidth = function () {
                return parseFloat(this.getComputedProperty('min-width')) || 0;
            };
            ElementHelper.prototype.setHeight = function (value) {
                this.el.style.height = value;
                return this;
            };
            ElementHelper.prototype.setHeightPx = function (value) {
                this.setHeight(value + "px");
                return this;
            };
            ElementHelper.prototype.getHeight = function () {
                return wemjq(this.el).innerHeight();
            };
            ElementHelper.prototype.getHeightWithoutPadding = function () {
                return wemjq(this.el).height();
            };
            ElementHelper.prototype.getHeightWithBorder = function () {
                return wemjq(this.el).outerHeight();
            };
            ElementHelper.prototype.getHeightWithMargin = function () {
                return wemjq(this.el).outerHeight(true);
            };
            ElementHelper.prototype.setTop = function (value) {
                this.el.style.top = value;
                return this;
            };
            ElementHelper.prototype.setTopPx = function (value) {
                return this.setTop(value + "px");
            };
            ElementHelper.prototype.getTopPx = function () {
                return parseFloat(this.getTop());
            };
            ElementHelper.prototype.getTop = function () {
                return this.el.style.top;
            };
            ElementHelper.prototype.setBottom = function (value) {
                this.el.style.bottom = value;
                return this;
            };
            ElementHelper.prototype.setBottomPx = function (value) {
                return this.setBottom(value + "px");
            };
            ElementHelper.prototype.setLeftPx = function (value) {
                return this.setLeft(value + "px");
            };
            ElementHelper.prototype.setLeft = function (value) {
                this.el.style.left = value;
                return this;
            };
            ElementHelper.prototype.setRight = function (value) {
                this.el.style.right = value;
                return this;
            };
            ElementHelper.prototype.setRightPx = function (value) {
                return this.setRight(value + "px");
            };
            ElementHelper.prototype.getMarginLeft = function () {
                return parseFloat(this.getComputedProperty('margin-left')) || 0;
            };
            ElementHelper.prototype.setMarginLeft = function (value) {
                this.el.style.marginLeft = value;
                return this;
            };
            ElementHelper.prototype.getMarginRight = function () {
                return parseFloat(this.getComputedProperty('margin-right'));
            };
            ElementHelper.prototype.setMarginRight = function (value) {
                this.el.style.marginRight = value;
                return this;
            };
            ElementHelper.prototype.getMarginTop = function () {
                return parseFloat(this.getComputedProperty('margin-top'));
            };
            ElementHelper.prototype.setMarginTop = function (value) {
                this.el.style.marginTop = value;
                return this;
            };
            ElementHelper.prototype.getMarginBottom = function () {
                return parseFloat(this.getComputedProperty('margin-bottom'));
            };
            ElementHelper.prototype.setMarginBottom = function (value) {
                this.el.style.marginBottom = value;
                return this;
            };
            ElementHelper.prototype.setStroke = function (value) {
                this.el.style.stroke = value;
                return this;
            };
            ElementHelper.prototype.getStroke = function () {
                return this.getComputedProperty('stroke');
            };
            ElementHelper.prototype.setStrokeDasharray = function (value) {
                this.el.style.strokeDasharray = value;
                return this;
            };
            ElementHelper.prototype.getStrokeDasharray = function () {
                return this.getComputedProperty('stroke-dasharray');
            };
            ElementHelper.prototype.setFill = function (value) {
                this.el.style.fill = value;
                return this;
            };
            ElementHelper.prototype.getFill = function () {
                return this.getComputedProperty('fill');
            };
            ElementHelper.prototype.getPaddingLeft = function () {
                return parseFloat(this.getComputedProperty('padding-left')) || 0;
            };
            ElementHelper.prototype.setPaddingLeft = function (value) {
                this.el.style.paddingLeft = value;
                return this;
            };
            ElementHelper.prototype.getPaddingRight = function () {
                return parseFloat(this.getComputedProperty('padding-right'));
            };
            ElementHelper.prototype.setPaddingRight = function (value) {
                this.el.style.paddingRight = value;
                return this;
            };
            ElementHelper.prototype.getPaddingTop = function () {
                return parseFloat(this.getComputedProperty('padding-top'));
            };
            ElementHelper.prototype.setPaddingTop = function (value) {
                this.el.style.paddingTop = value;
                return this;
            };
            ElementHelper.prototype.getPaddingBottom = function () {
                return parseFloat(this.getComputedProperty('padding-bottom'));
            };
            ElementHelper.prototype.setPaddingBottom = function (value) {
                this.el.style.paddingBottom = value;
                return this;
            };
            ElementHelper.prototype.getBorderTopWidth = function () {
                return parseFloat(this.getComputedProperty('border-top-width'));
            };
            ElementHelper.prototype.getBorderBottomWidth = function () {
                return parseFloat(this.getComputedProperty('border-bottom-width'));
            };
            ElementHelper.prototype.getBorderRightWidth = function () {
                return parseFloat(this.getComputedProperty('border-right-width'));
            };
            ElementHelper.prototype.getBorderLeftWidth = function () {
                return parseFloat(this.getComputedProperty('border-left-width'));
            };
            ElementHelper.prototype.setZindex = function (value) {
                this.el.style.zIndex = value.toString();
                return this;
            };
            ElementHelper.prototype.getBoundingClientRect = function () {
                return this.el.getBoundingClientRect();
            };
            ElementHelper.prototype.scrollIntoView = function (top) {
                this.el.scrollIntoView(top);
                return this;
            };
            ElementHelper.prototype.getScrollTop = function () {
                return this.el.scrollTop;
            };
            ElementHelper.prototype.setScrollTop = function (top) {
                this.el.scrollTop = top;
                return this;
            };
            ElementHelper.prototype.getTabIndex = function () {
                return this.el.tabIndex;
            };
            ElementHelper.prototype.setTabIndex = function (tabIndex) {
                this.el.tabIndex = tabIndex;
                return this;
            };
            ElementHelper.prototype.getFontSize = function () {
                return this.getComputedProperty('font-size');
            };
            ElementHelper.prototype.setFontSize = function (value) {
                this.el.style.fontSize = value;
                return this;
            };
            ElementHelper.prototype.setBackgroundImage = function (value) {
                this.el.style.backgroundImage = value;
                return this;
            };
            ElementHelper.prototype.setCursor = function (value) {
                this.el.style.cursor = value;
                return this;
            };
            ElementHelper.prototype.getCursor = function () {
                return this.el.style.cursor;
            };
            ElementHelper.prototype.remove = function () {
                var parent = this.el.parentElement;
                if (parent) {
                    parent.removeChild(this.el);
                }
            };
            ElementHelper.prototype.contains = function (element) {
                return this.el.contains ? this.el.contains(element) : !!(this.el.compareDocumentPosition(element) & 16);
            };
            ElementHelper.prototype.getOffset = function () {
                return wemjq(this.el).offset();
            };
            ElementHelper.prototype.setOffset = function (offset) {
                wemjq(this.el).offset(offset);
                return this;
            };
            ElementHelper.prototype.getOffsetParent = function () {
                return wemjq(this.el).offsetParent()[0];
            };
            ElementHelper.prototype.getOffsetToParent = function () {
                return wemjq(this.el).position();
            };
            ElementHelper.prototype.getOffsetTop = function () {
                return this.getOffset().top;
            };
            ElementHelper.prototype.getOffsetTopRelativeToParent = function () {
                return this.el.offsetTop;
            };
            ElementHelper.prototype.getOffsetLeft = function () {
                return this.getOffset().left;
            };
            ElementHelper.prototype.getOffsetLeftRelativeToParent = function () {
                return this.el.offsetLeft;
            };
            ElementHelper.prototype.getComputedProperty = function (name, pseudoElement) {
                if (pseudoElement === void 0) { pseudoElement = null; }
                return window.getComputedStyle(this.el, pseudoElement).getPropertyValue(name);
            };
            ElementHelper.prototype.focus = function () {
                this.el.focus();
            };
            ElementHelper.prototype.blur = function () {
                this.el.blur();
            };
            ElementHelper.prototype.getSiblingIndex = function () {
                var i = 0;
                var prev = this.el;
                while ((prev = prev.previousElementSibling) != null) {
                    i++;
                }
                return i;
            };
            ElementHelper.prototype.isVisible = function () {
                return wemjq(this.el).is(':visible');
            };
            return ElementHelper;
        })();
        dom.ElementHelper = ElementHelper;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ImgHelper = (function (_super) {
            __extends(ImgHelper, _super);
            function ImgHelper(element) {
                _super.call(this, element);
                this.imgEl = element;
            }
            ImgHelper.create = function () {
                return new ImgHelper(document.createElement("img"));
            };
            ImgHelper.prototype.getHTMLElement = function () {
                return this.imgEl;
            };
            ImgHelper.prototype.setSrc = function (value) {
                this.imgEl.src = value;
                return this;
            };
            ImgHelper.prototype.getSrc = function () {
                return this.imgEl.src;
            };
            ImgHelper.prototype.getNaturalWidth = function () {
                return this.imgEl.naturalWidth;
            };
            return ImgHelper;
        })(dom.ElementHelper);
        dom.ImgHelper = ImgHelper;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementBuilder = (function () {
            function ElementBuilder() {
            }
            ElementBuilder.prototype.setGenerateId = function (value) {
                this.generateId = value;
                return this;
            };
            ElementBuilder.prototype.setClassName = function (name) {
                if (name) {
                    name = name.trim().split(/\s+/).filter(function (elem, index, arr) {
                        return arr.indexOf(elem) === index;
                    }).join(" ");
                }
                this.className = name;
                return this;
            };
            ElementBuilder.prototype.setParentElement = function (element) {
                this.parentElement = element;
                return this;
            };
            return ElementBuilder;
        })();
        dom.ElementBuilder = ElementBuilder;
        var ElementFromElementBuilder = (function (_super) {
            __extends(ElementFromElementBuilder, _super);
            function ElementFromElementBuilder() {
                _super.apply(this, arguments);
            }
            ElementFromElementBuilder.prototype.setElement = function (element) {
                this.element = element;
                return this;
            };
            return ElementFromElementBuilder;
        })(ElementBuilder);
        dom.ElementFromElementBuilder = ElementFromElementBuilder;
        var ElementFromHelperBuilder = (function (_super) {
            __extends(ElementFromHelperBuilder, _super);
            function ElementFromHelperBuilder() {
                _super.apply(this, arguments);
            }
            ElementFromHelperBuilder.prototype.setHelper = function (helper) {
                this.helper = helper;
                return this;
            };
            ElementFromHelperBuilder.prototype.setLoadExistingChildren = function (value) {
                this.loadExistingChildren = value;
                return this;
            };
            return ElementFromHelperBuilder;
        })(ElementBuilder);
        dom.ElementFromHelperBuilder = ElementFromHelperBuilder;
        var NewElementBuilder = (function (_super) {
            __extends(NewElementBuilder, _super);
            function NewElementBuilder() {
                _super.apply(this, arguments);
            }
            NewElementBuilder.prototype.setTagName = function (name) {
                api.util.assert(!api.util.StringHelper.isEmpty(name), 'Tag name cannot be empty');
                this.tagName = name;
                return this;
            };
            NewElementBuilder.prototype.setHelper = function (helper) {
                this.helper = helper;
                return this;
            };
            return NewElementBuilder;
        })(ElementBuilder);
        dom.NewElementBuilder = NewElementBuilder;
        var Element = (function () {
            function Element(builder) {
                var _this = this;
                this.addedListeners = [];
                this.removedListeners = [];
                this.renderedListeners = [];
                this.shownListeners = [];
                this.hiddenListeners = [];
                this.resizedListeners = [];
                this.children = [];
                this.rendered = false;
                if (api.ObjectHelper.iFrameSafeInstanceOf(builder, ElementFromElementBuilder)) {
                    var fromElementBuilder = builder;
                    var sourceElement = fromElementBuilder.element;
                    if (sourceElement) {
                        this.parentElement = fromElementBuilder.parentElement ? fromElementBuilder.parentElement : sourceElement.parentElement;
                        if (this.parentElement) {
                            this.parentElement.replaceChildElement(this, sourceElement);
                        }
                        this.children = sourceElement.children;
                        this.el = sourceElement.el;
                    }
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(builder, ElementFromHelperBuilder)) {
                    var fromHelperBuilder = builder;
                    this.el = fromHelperBuilder.helper;
                    if (fromHelperBuilder.loadExistingChildren) {
                        this.loadExistingChildren();
                    }
                    if (fromHelperBuilder.parentElement) {
                        this.parentElement = fromHelperBuilder.parentElement;
                    }
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(builder, NewElementBuilder)) {
                    var newElementBuilder = builder;
                    if (!newElementBuilder.tagName) {
                        throw new Error("tagName cannot be null");
                    }
                    if (newElementBuilder.helper) {
                        this.el = newElementBuilder.helper;
                    }
                    else {
                        this.el = dom.ElementHelper.fromName(newElementBuilder.tagName);
                    }
                    if (newElementBuilder.parentElement) {
                        this.parentElement = newElementBuilder.parentElement;
                    }
                }
                else {
                    throw new Error("Unsupported builder: " + api.ClassHelper.getClassName(builder));
                }
                if (this.parentElement && this.el.getHTMLElement().parentElement) {
                    if (!(this.parentElement.getHTMLElement() === this.el.getHTMLElement().parentElement)) {
                        throw new Error("Illegal state: HTMLElement in parent Element is not the as the HTMLElement parent to this HTMLElement");
                    }
                }
                var distance = api.ClassHelper.distanceTo(this, Element);
                if (builder.generateId || distance > 1) {
                    var id = dom.ElementRegistry.registerElement(this);
                    this.setId(id);
                }
                if (builder.className) {
                    this.setClass(builder.className);
                }
                this.onRemoved(function (event) {
                    if (_this.getId()) {
                        dom.ElementRegistry.unregisterElement(_this);
                    }
                });
            }
            Element.prototype.replaceChildElement = function (replacementChild, existingChild) {
                var index = this.children.indexOf(existingChild);
                this.children[index] = replacementChild;
            };
            Element.prototype.loadExistingChildren = function () {
                var children = this.getHTMLElement().children;
                for (var i = 0; i < children.length; i++) {
                    var child = children[i];
                    var childAsElement = Element.fromHtmlElement(child, true, this);
                    this.children.push(childAsElement);
                }
                return this;
            };
            Element.prototype.init = function () {
                this.children.forEach(function (child) {
                    child.init();
                });
                if (!this.isRendered()) {
                    this.render(false);
                }
                if (this.isVisible()) {
                    this.notifyShown();
                }
            };
            Element.prototype.render = function (deep) {
                if (deep === void 0) { deep = true; }
                if (deep) {
                    this.children.forEach(function (child) {
                        child.render();
                    });
                }
                this.rendered = this.doRender();
                this.notifyRendered();
            };
            Element.prototype.isRendered = function () {
                return this.rendered;
            };
            Element.prototype.doRender = function () {
                return true;
            };
            Element.prototype.show = function () {
                wemjq(this.el.getHTMLElement()).show();
                this.notifyShown(this);
            };
            Element.prototype.hide = function () {
                wemjq(this.el.getHTMLElement()).hide();
                this.notifyHidden(this);
            };
            Element.prototype.setVisible = function (value) {
                if (value) {
                    this.show();
                }
                else {
                    this.hide();
                }
            };
            Element.prototype.isVisible = function () {
                return this.el.isVisible();
            };
            Element.prototype.setClass = function (className) {
                api.util.assert(!api.util.StringHelper.isEmpty(className), 'Class name cannot be empty');
                this.el.setClass(className);
                return this;
            };
            Element.prototype.addClass = function (className) {
                api.util.assert(!api.util.StringHelper.isEmpty(className), 'Class name cannot be empty');
                this.el.addClass(className);
                return this;
            };
            Element.prototype.toggleClass = function (className, condition) {
                if (condition == false || condition == undefined && this.hasClass(className)) {
                    this.removeClass(className);
                }
                else {
                    this.addClass(className);
                }
                return this;
            };
            Element.prototype.hasClass = function (className) {
                return this.el.hasClass(className);
            };
            Element.prototype.removeClass = function (className) {
                api.util.assert(!api.util.StringHelper.isEmpty(className), 'Class name cannot be empty');
                this.el.removeClass(className);
                return this;
            };
            Element.prototype.getClass = function () {
                return this.el.getClass();
            };
            Element.prototype.getId = function () {
                return this.el.getId();
            };
            Element.prototype.setId = function (value) {
                this.el.setId(value);
                return this;
            };
            Element.prototype.getEl = function () {
                return this.el;
            };
            Element.prototype.traverse = function (handler) {
                this.getChildren().forEach(function (el) {
                    handler(el);
                    el.traverse(handler);
                });
            };
            Element.prototype.setDraggable = function (value) {
                this.getEl().setAttribute("draggable", value.toString());
            };
            Element.prototype.isDraggable = function () {
                return this.getEl().getAttribute('draggable') == 'true';
            };
            Element.prototype.setContentEditable = function (flag) {
                this.getEl().setAttribute('contenteditable', flag ? 'true' : 'false');
                return this;
            };
            Element.prototype.isContentEditable = function () {
                return this.getEl().getAttribute('contenteditable') == 'true';
            };
            Element.prototype.giveFocus = function () {
                if (!this.isVisible()) {
                    return false;
                }
                if (this.el.isDisabled()) {
                    return false;
                }
                this.el.focus();
                var gotFocus = document.activeElement == this.el.getHTMLElement();
                if (!gotFocus) {
                    console.log("Element.giveFocus(): Failed to give focus to Element: class = " + api.ClassHelper.getClassName(this) + ", id = " + this.getId());
                }
                return gotFocus;
            };
            Element.prototype.giveBlur = function () {
                if (!this.isVisible()) {
                    return false;
                }
                if (this.el.isDisabled()) {
                    return false;
                }
                this.el.blur();
                var gotBlur = document.activeElement != this.el.getHTMLElement();
                if (!gotBlur) {
                    console.log("Element.giveBlur(): Failed to give blur to Element: class = " + api.ClassHelper.getClassName(this) + ", id = " + this.getId());
                }
                return gotBlur;
            };
            Element.prototype.getHTMLElement = function () {
                return this.el.getHTMLElement();
            };
            Element.prototype.insertChild = function (child, index) {
                api.util.assertNotNull(child, 'Child cannot be null');
                this.el.insertChild(child.getEl().getHTMLElement(), index);
                this.insertChildElement(child, this, index);
                return this;
            };
            Element.prototype.appendChild = function (child) {
                return this.insertChild(child, this.children.length);
            };
            Element.prototype.appendChildren = function () {
                var _this = this;
                var children = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    children[_i - 0] = arguments[_i];
                }
                children.forEach(function (child) {
                    _this.appendChild(child);
                });
                return this;
            };
            Element.prototype.prependChild = function (child) {
                return this.insertChild(child, 0);
            };
            Element.prototype.insertAfterEl = function (existing) {
                api.util.assertNotNull(existing, 'Existing element cannot be null');
                var existingIndex = existing.getSiblingIndex();
                this.el.insertAfterEl(existing.el);
                return this.insertChildElement(this, existing.parentElement, existingIndex + 1);
            };
            Element.prototype.insertBeforeEl = function (existing) {
                api.util.assertNotNull(existing, 'Existing element cannot be null');
                var existingIndex = existing.getSiblingIndex();
                this.el.insertBeforeEl(existing.el);
                return this.insertChildElement(this, existing.getParentElement(), existingIndex);
            };
            Element.prototype.hasChild = function (child) {
                return this.children.indexOf(child) > -1;
            };
            Element.prototype.removeChild = function (child) {
                api.util.assertNotNull(child, "Child element to remove cannot be null");
                child.getEl().remove();
                this.removeChildElement(child);
                return this;
            };
            Element.prototype.removeChildren = function () {
                this.children.slice(0).forEach(function (child) {
                    child.remove();
                });
                this.el.setInnerHtml('');
                return this;
            };
            Element.prototype.insertChildElement = function (child, parent, index) {
                api.util.assertNotNull(child, 'Child element to insert cannot be null');
                api.util.assertNotNull(parent, 'Parent element cannot be null');
                parent.registerChildElement(child, index);
                if (parent.isRendered()) {
                    child.init();
                }
                child.notifyAdded();
                return this;
            };
            Element.prototype.removeChildElement = function (child) {
                api.util.assertNotNull(child, 'Child element to insert cannot be null');
                this.unregisterChildElement(child);
                child.notifyRemoved();
                return this;
            };
            Element.prototype.registerChildElement = function (child, index) {
                if (!(child.getHTMLElement().parentElement === this.getHTMLElement())) {
                    throw new Error("Given child must be a child of this Element in DOM before it can be registered");
                }
                if (!index) {
                    index = child.el.getSiblingIndex();
                }
                this.children.splice(index, 0, child);
                child.parentElement = this;
            };
            Element.prototype.unregisterChildElement = function (child) {
                var childIndex = this.children.indexOf(child);
                if (childIndex < 0) {
                    throw new Error("Child element to remove not found");
                }
                this.children.splice(childIndex, 1);
                child.parentElement = null;
                return childIndex;
            };
            Element.prototype.contains = function (element) {
                return this.getEl().contains(element.getHTMLElement());
            };
            Element.prototype.remove = function () {
                if (this.parentElement) {
                    this.parentElement.removeChild(this);
                }
                else {
                    this.getEl().remove();
                    this.notifyRemoved();
                }
                return this;
            };
            Element.prototype.replaceWith = function (replacement) {
                api.util.assertNotNull(replacement, 'replacement element cannot be null');
                replacement.el.insertAfterEl(this.el);
                replacement.notifyAdded();
                var parent = this.parentElement;
                var index = parent.unregisterChildElement(this);
                parent.registerChildElement(replacement, index);
                if (parent.isRendered()) {
                    replacement.init();
                }
                this.getEl().remove();
                this.notifyRemoved(this);
            };
            Element.prototype.wrapWithElement = function (wrapperElement) {
                api.util.assertNotNull(wrapperElement, 'wrapperElement cannot be null');
                var parent = this.parentElement;
                if (!parent) {
                    return;
                }
                var childPos = parent.children.indexOf(this);
                parent.removeChild(this);
                wrapperElement.appendChild(this);
                parent.el.appendChild(wrapperElement.getEl().getHTMLElement());
                parent.insertChildElement(this, wrapperElement, childPos);
            };
            Element.prototype.getParentElement = function () {
                return this.parentElement;
            };
            Element.prototype.getChildren = function () {
                return this.children;
            };
            Element.prototype.getLastChild = function () {
                return this.children[this.children.length - 1];
            };
            Element.prototype.getFirstChild = function () {
                return this.children[0];
            };
            Element.prototype.getNextElement = function () {
                var nextSiblingHtmlElement = this.getHTMLElement().nextElementSibling;
                if (!nextSiblingHtmlElement) {
                    return null;
                }
                return Element.fromHtmlElement(nextSiblingHtmlElement);
            };
            Element.prototype.getPreviousElement = function () {
                var previousSiblingHtmlElement = this.getHTMLElement().previousElementSibling;
                if (!previousSiblingHtmlElement) {
                    return null;
                }
                return Element.fromHtmlElement(previousSiblingHtmlElement);
            };
            Element.prototype.getSiblingIndex = function () {
                var indexFromDOM = this.el.getSiblingIndex();
                if (this.parentElement) {
                    var indexFromElement = this.parentElement.children.indexOf(this);
                    api.util.assertState(indexFromElement == indexFromDOM, "index of Element in parentElement.children" + " [" + indexFromElement + "] does not correspond with" + " the actual index [" + indexFromDOM + "] of the HTMLElement in DOM");
                }
                return indexFromDOM;
            };
            Element.prototype.toString = function () {
                return wemjq('<div>').append(wemjq(this.getHTMLElement()).clone()).html();
            };
            Element.prototype.getHtml = function () {
                return this.getEl().getInnerHtml();
            };
            Element.prototype.setHtml = function (value) {
                this.getEl().setInnerHtml(value);
                return this;
            };
            Element.prototype.onMouseEnter = function (handler) {
                var _this = this;
                if (typeof this.getHTMLElement().onmouseenter != "undefined") {
                    this.getEl().addEventListener('mouseenter', handler);
                }
                else {
                    this.getEl().addEventListener('mouseover', function (e) {
                        if (!_this.getEl().contains((e.relatedTarget || e.fromElement))) {
                            handler(e);
                        }
                    });
                }
            };
            Element.prototype.onMouseLeave = function (handler) {
                var _this = this;
                if (typeof this.getHTMLElement().onmouseleave != "undefined") {
                    this.getEl().addEventListener('mouseleave', handler);
                }
                else {
                    this.getEl().addEventListener('mouseout', function (e) {
                        if (!_this.getEl().contains((e.relatedTarget || e.toElement))) {
                            handler(e);
                        }
                    });
                }
            };
            Element.prototype.onMouseOver = function (listener) {
                this.getEl().addEventListener('mouseover', listener);
            };
            Element.prototype.unMouseOver = function (listener) {
                this.getEl().removeEventListener("mouseover", listener);
            };
            Element.prototype.onMouseOut = function (listener) {
                this.getEl().addEventListener('mouseout', listener);
            };
            Element.prototype.unMouseOut = function (listener) {
                this.getEl().removeEventListener("mouseout", listener);
            };
            Element.prototype.onAdded = function (listener) {
                this.addedListeners.push(listener);
            };
            Element.prototype.unAdded = function (listener) {
                this.addedListeners = this.addedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyAdded = function () {
                var addedEvent = new dom.ElementAddedEvent(this);
                this.addedListeners.forEach(function (listener) {
                    listener(addedEvent);
                });
            };
            Element.prototype.onRemoved = function (listener) {
                this.removedListeners.push(listener);
            };
            Element.prototype.unRemoved = function (listener) {
                this.removedListeners = this.removedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyRemoved = function (target) {
                var removedEvent = new dom.ElementRemovedEvent(this, target);
                this.removedListeners.forEach(function (listener) {
                    listener(removedEvent);
                });
                this.children.forEach(function (child) {
                    child.notifyRemoved(removedEvent.getTarget());
                });
            };
            Element.prototype.onRendered = function (listener) {
                this.renderedListeners.push(listener);
            };
            Element.prototype.unRendered = function (listener) {
                this.renderedListeners = this.renderedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyRendered = function () {
                var renderedEvent = new dom.ElementRenderedEvent(this);
                this.renderedListeners.forEach(function (listener) {
                    listener(renderedEvent);
                });
            };
            Element.prototype.onShown = function (listener) {
                this.shownListeners.push(listener);
            };
            Element.prototype.unShown = function (listener) {
                this.shownListeners = this.shownListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyShown = function (target) {
                var shownEvent = new dom.ElementShownEvent(this, target);
                this.shownListeners.forEach(function (listener) {
                    listener(shownEvent);
                });
                this.children.forEach(function (child) {
                    if (child.isVisible()) {
                        child.notifyShown(shownEvent.getTarget());
                    }
                });
            };
            Element.prototype.onHidden = function (listener) {
                this.hiddenListeners.push(listener);
            };
            Element.prototype.unHidden = function (listener) {
                this.hiddenListeners = this.hiddenListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyHidden = function (target) {
                var hiddenEvent = new dom.ElementHiddenEvent(this, target);
                this.hiddenListeners.forEach(function (listener) {
                    listener(hiddenEvent);
                });
                this.children.forEach(function (child) {
                    child.notifyHidden(hiddenEvent.getTarget());
                });
            };
            Element.prototype.onScrolled = function (listener) {
                this.getEl().addEventListener("mousewheel", listener);
                this.getEl().addEventListener("DOMMouseScroll", listener);
            };
            Element.prototype.onClicked = function (listener) {
                this.getEl().addEventListener("click", listener);
            };
            Element.prototype.unClicked = function (listener) {
                this.getEl().removeEventListener("click", listener);
            };
            Element.prototype.onDblClicked = function (listener) {
                this.getEl().addEventListener("dblclick", listener);
            };
            Element.prototype.unDblClicked = function (listener) {
                this.getEl().removeEventListener("dblclick", listener);
            };
            Element.prototype.onContextMenu = function (listener) {
                this.getEl().addEventListener("contextmenu", listener);
            };
            Element.prototype.unContextMenu = function (listener) {
                this.getEl().removeEventListener("contextmenu", listener);
            };
            Element.prototype.onMouseDown = function (listener) {
                this.getEl().addEventListener("mousedown", listener);
            };
            Element.prototype.unMouseDown = function (listener) {
                this.getEl().removeEventListener("mousedown", listener);
            };
            Element.prototype.onMouseUp = function (listener) {
                this.getEl().addEventListener("mouseup", listener);
            };
            Element.prototype.unMouseUp = function (listener) {
                this.getEl().removeEventListener("mouseup", listener);
            };
            Element.prototype.onMouseMove = function (listener) {
                this.getEl().addEventListener("mousemove", listener);
            };
            Element.prototype.unMouseMove = function (listener) {
                this.getEl().removeEventListener("mousemove", listener);
            };
            Element.prototype.onMouseWheel = function (listener) {
                var eventName = (/Firefox/i.test(navigator.userAgent)) ? "DOMMouseScroll" : "mousewheel";
                this.getEl().addEventListener(eventName, listener);
            };
            Element.prototype.unMouseWheel = function (listener) {
                var eventName = (/Firefox/i.test(navigator.userAgent)) ? "DOMMouseScroll" : "mousewheel";
                this.getEl().removeEventListener(eventName, listener);
            };
            Element.prototype.onTouchStart = function (listener) {
                this.getEl().addEventListener("touchstart", listener);
            };
            Element.prototype.unTouchStart = function (listener) {
                this.getEl().removeEventListener("touchstart", listener);
            };
            Element.prototype.onKeyUp = function (listener) {
                this.getEl().addEventListener("keyup", listener);
            };
            Element.prototype.unKeyUp = function (listener) {
                this.getEl().removeEventListener("keyup", listener);
            };
            Element.prototype.onKeyDown = function (listener) {
                this.getEl().addEventListener("keydown", listener);
            };
            Element.prototype.unKeyDown = function (listener) {
                this.getEl().removeEventListener("keydown", listener);
            };
            Element.prototype.onKeyPressed = function (listener) {
                this.getEl().addEventListener("keypress", listener);
            };
            Element.prototype.unKeyPressed = function (listener) {
                this.getEl().removeEventListener("keypress", listener);
            };
            Element.prototype.onFocus = function (listener) {
                this.getEl().addEventListener("focus", listener);
            };
            Element.prototype.unFocus = function (listener) {
                this.getEl().removeEventListener("focus", listener);
            };
            Element.prototype.onBlur = function (listener) {
                this.getEl().addEventListener("blur", listener);
            };
            Element.prototype.unBlur = function (listener) {
                this.getEl().removeEventListener("blur", listener);
            };
            Element.prototype.onScroll = function (listener) {
                this.getEl().addEventListener("scroll", listener);
            };
            Element.prototype.unScroll = function (listener) {
                this.getEl().removeEventListener("scroll", listener);
            };
            Element.prototype.onDrag = function (listener) {
                this.getEl().addEventListener("drag", listener);
            };
            Element.prototype.unDrag = function (listener) {
                this.getEl().removeEventListener("drag", listener);
            };
            Element.prototype.onDragStart = function (listener) {
                this.getEl().addEventListener("dragstart", listener);
            };
            Element.prototype.unDragStart = function (listener) {
                this.getEl().removeEventListener("dragstart", listener);
            };
            Element.prototype.onDragEnter = function (listener) {
                this.getEl().addEventListener("dragenter", listener);
            };
            Element.prototype.unDragEnter = function (listener) {
                this.getEl().removeEventListener("dragenter", listener);
            };
            Element.prototype.onDragOver = function (listener) {
                this.getEl().addEventListener("dragover", listener);
            };
            Element.prototype.unDragOver = function (listener) {
                this.getEl().removeEventListener("dragover", listener);
            };
            Element.prototype.onDragLeave = function (listener) {
                this.getEl().addEventListener("dragleave", listener);
            };
            Element.prototype.unDragLeave = function (listener) {
                this.getEl().removeEventListener("dragleave", listener);
            };
            Element.prototype.onDrop = function (listener) {
                this.getEl().addEventListener("drop", listener);
            };
            Element.prototype.unDrop = function (listener) {
                this.getEl().removeEventListener("drop", listener);
            };
            Element.prototype.onDragEnd = function (listener) {
                this.getEl().addEventListener("dragend", listener);
            };
            Element.prototype.unDragEnd = function (listener) {
                this.getEl().removeEventListener("dragend", listener);
            };
            Element.fromHtmlElement = function (element, loadExistingChildren, parent) {
                if (loadExistingChildren === void 0) { loadExistingChildren = false; }
                return new Element(new ElementFromHelperBuilder().setHelper(new dom.ElementHelper(element)).setLoadExistingChildren(loadExistingChildren).setParentElement(parent));
            };
            Element.fromString = function (s, setLoadExistingChildren) {
                if (setLoadExistingChildren === void 0) { setLoadExistingChildren = true; }
                var elementAsJQ = wemjq(s);
                var elementASHtmlElement = elementAsJQ.get(0);
                return !!elementASHtmlElement ? new Element(new ElementFromHelperBuilder().setHelper(new dom.ElementHelper(elementASHtmlElement)).setLoadExistingChildren(setLoadExistingChildren).setParentElement(Element.fromString(elementASHtmlElement.parentElement))) : null;
            };
            Element.elementsFromRequest = function (s, setLoadExistingChildren) {
                if (setLoadExistingChildren === void 0) { setLoadExistingChildren = true; }
                var elementAsJQ = wemjq(s);
                var elements = [];
                elementAsJQ.each(function (index, elem) {
                    var e = wemjq(elem);
                    elements.push(new Element(new ElementFromHelperBuilder().setHelper(new dom.ElementHelper(e.get(0))).setLoadExistingChildren(setLoadExistingChildren)));
                });
                return elements;
            };
            Element.debug = true;
            return Element;
        })();
        dom.Element = Element;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementRegistry = (function () {
            function ElementRegistry() {
            }
            ElementRegistry.registerElement = function (el) {
                var fullName, id = el.getId();
                if (!id) {
                    id = fullName = api.ClassHelper.getFullName(el);
                }
                else {
                    fullName = id;
                }
                var count = ElementRegistry.counters[fullName];
                if (count >= 0) {
                    id += '-' + (++count);
                }
                ElementRegistry.counters[fullName] = count || 0;
                ElementRegistry.elements[id] = el;
                return id;
            };
            ElementRegistry.unregisterElement = function (el) {
                if (el) {
                    delete ElementRegistry.elements[el.getId()];
                }
            };
            ElementRegistry.getElementById = function (id) {
                return ElementRegistry.elements[id];
            };
            ElementRegistry.counters = {};
            ElementRegistry.elements = {};
            return ElementRegistry;
        })();
        dom.ElementRegistry = ElementRegistry;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementEvent = (function (_super) {
            __extends(ElementEvent, _super);
            function ElementEvent(name, element, target) {
                _super.call(this, name);
                this.element = element;
                this.target = target || element;
            }
            ElementEvent.prototype.getElement = function () {
                return this.element;
            };
            ElementEvent.prototype.getTarget = function () {
                return this.target;
            };
            return ElementEvent;
        })(api.event.Event);
        dom.ElementEvent = ElementEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementAddedEvent = (function (_super) {
            __extends(ElementAddedEvent, _super);
            function ElementAddedEvent(element, target) {
                _super.call(this, "added", element, target);
            }
            return ElementAddedEvent;
        })(dom.ElementEvent);
        dom.ElementAddedEvent = ElementAddedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementRenderedEvent = (function (_super) {
            __extends(ElementRenderedEvent, _super);
            function ElementRenderedEvent(element, target) {
                _super.call(this, "rendered", element, target);
            }
            return ElementRenderedEvent;
        })(dom.ElementEvent);
        dom.ElementRenderedEvent = ElementRenderedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementShownEvent = (function (_super) {
            __extends(ElementShownEvent, _super);
            function ElementShownEvent(element, target) {
                _super.call(this, "shown", element, target);
            }
            return ElementShownEvent;
        })(dom.ElementEvent);
        dom.ElementShownEvent = ElementShownEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementHiddenEvent = (function (_super) {
            __extends(ElementHiddenEvent, _super);
            function ElementHiddenEvent(element, target) {
                _super.call(this, "hidden", element, target);
            }
            return ElementHiddenEvent;
        })(dom.ElementEvent);
        dom.ElementHiddenEvent = ElementHiddenEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementRemovedEvent = (function (_super) {
            __extends(ElementRemovedEvent, _super);
            function ElementRemovedEvent(element, target) {
                _super.call(this, "removed", element, target);
            }
            return ElementRemovedEvent;
        })(dom.ElementEvent);
        dom.ElementRemovedEvent = ElementRemovedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementResizedEvent = (function (_super) {
            __extends(ElementResizedEvent, _super);
            function ElementResizedEvent(newWidth, newHeight, element, target) {
                _super.call(this, "resized", element, target);
                this.newWidth = newWidth;
                this.newHeight = newHeight;
            }
            ElementResizedEvent.prototype.getNewWidth = function () {
                return this.newWidth;
            };
            ElementResizedEvent.prototype.getNewHeight = function () {
                return this.newHeight;
            };
            return ElementResizedEvent;
        })(dom.ElementEvent);
        dom.ElementResizedEvent = ElementResizedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var AEl = (function (_super) {
            __extends(AEl, _super);
            function AEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("a").setClassName(className));
                this.setUrl('#');
            }
            AEl.prototype.setUrl = function (value, target) {
                this.getEl().setAttribute('href', value);
                if (target) {
                    this.getEl().setAttribute('target', target);
                }
                return this;
            };
            return AEl;
        })(dom.Element);
        dom.AEl = AEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var BrEl = (function (_super) {
            __extends(BrEl, _super);
            function BrEl() {
                _super.call(this, new dom.NewElementBuilder().setTagName("br"));
            }
            return BrEl;
        })(dom.Element);
        dom.BrEl = BrEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var IEl = (function (_super) {
            __extends(IEl, _super);
            function IEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("i").setClassName(className));
            }
            return IEl;
        })(dom.Element);
        dom.IEl = IEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var Body = (function (_super) {
            __extends(Body, _super);
            function Body(loadExistingChildren, body) {
                if (loadExistingChildren === void 0) { loadExistingChildren = false; }
                if (!body) {
                    body = document.body;
                }
                var html = dom.Element.fromHtmlElement(body.parentElement);
                _super.call(this, new dom.ElementFromHelperBuilder().setHelper(new dom.ElementHelper(body)).setLoadExistingChildren(loadExistingChildren).setParentElement(html));
                html.appendChild(this);
                this.init();
                this.childrenLoaded = loadExistingChildren;
            }
            Body.get = function () {
                if (!Body.instance) {
                    Body.instance = new Body();
                }
                return Body.instance;
            };
            Body.prototype.isChildrenLoaded = function () {
                return this.childrenLoaded;
            };
            Body.prototype.loadExistingChildren = function () {
                if (!this.isChildrenLoaded()) {
                    _super.prototype.loadExistingChildren.call(this);
                    this.childrenLoaded = true;
                }
                return this;
            };
            return Body;
        })(dom.Element);
        dom.Body = Body;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var DivEl = (function (_super) {
            __extends(DivEl, _super);
            function DivEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("div").setClassName(className));
            }
            return DivEl;
        })(dom.Element);
        dom.DivEl = DivEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H1El = (function (_super) {
            __extends(H1El, _super);
            function H1El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h1").setClassName(className));
            }
            return H1El;
        })(dom.Element);
        dom.H1El = H1El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H2El = (function (_super) {
            __extends(H2El, _super);
            function H2El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h2").setClassName(className));
            }
            return H2El;
        })(dom.Element);
        dom.H2El = H2El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H3El = (function (_super) {
            __extends(H3El, _super);
            function H3El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h3").setClassName(className));
            }
            return H3El;
        })(dom.Element);
        dom.H3El = H3El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H4El = (function (_super) {
            __extends(H4El, _super);
            function H4El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h4").setClassName(className));
            }
            return H4El;
        })(dom.Element);
        dom.H4El = H4El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H5El = (function (_super) {
            __extends(H5El, _super);
            function H5El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h5").setClassName(className));
            }
            return H5El;
        })(dom.Element);
        dom.H5El = H5El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H6El = (function (_super) {
            __extends(H6El, _super);
            function H6El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h6").setClassName(className));
            }
            return H6El;
        })(dom.Element);
        dom.H6El = H6El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var UlEl = (function (_super) {
            __extends(UlEl, _super);
            function UlEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("ul").setClassName(className));
            }
            return UlEl;
        })(dom.Element);
        dom.UlEl = UlEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var LiEl = (function (_super) {
            __extends(LiEl, _super);
            function LiEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("li").setClassName(className));
            }
            return LiEl;
        })(dom.Element);
        dom.LiEl = LiEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var EmEl = (function (_super) {
            __extends(EmEl, _super);
            function EmEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("em").setClassName(className));
            }
            return EmEl;
        })(dom.Element);
        dom.EmEl = EmEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ImgEl = (function (_super) {
            __extends(ImgEl, _super);
            function ImgEl(src, className) {
                var _this = this;
                _super.call(this, new dom.NewElementBuilder().setTagName("img").setHelper(dom.ImgHelper.create()).setClassName(className));
                this.getEl().setSrc(src ? src : ImgEl.PLACEHOLDER);
                this.onLoaded(function (event) {
                    _this.loaded = true;
                });
            }
            ImgEl.prototype.refresh = function () {
                this.setSrc(this.getSrc());
            };
            ImgEl.prototype.getSrc = function () {
                return this.getEl().getSrc();
            };
            ImgEl.prototype.setSrc = function (source) {
                this.loaded = false;
                this.getEl().setSrc(source);
            };
            ImgEl.prototype.getEl = function () {
                return _super.prototype.getEl.call(this);
            };
            ImgEl.prototype.onLoaded = function (listener) {
                this.getEl().addEventListener("load", listener);
            };
            ImgEl.prototype.unLoaded = function (listener) {
                this.getEl().removeEventListener("load", listener);
            };
            ImgEl.prototype.isLoaded = function () {
                return this.loaded;
            };
            ImgEl.PLACEHOLDER = "data:image/gif;base64,R0lGODlhAQABAIAAAP///////yH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==";
            return ImgEl;
        })(dom.Element);
        dom.ImgEl = ImgEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var SpanEl = (function (_super) {
            __extends(SpanEl, _super);
            function SpanEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("span").setClassName(className));
            }
            return SpanEl;
        })(dom.Element);
        dom.SpanEl = SpanEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FormItemEl = (function (_super) {
            __extends(FormItemEl, _super);
            function FormItemEl(tagName, className) {
                _super.call(this, new dom.NewElementBuilder().setTagName(tagName).setClassName(className));
                this.validityChangedListeners = [];
            }
            FormItemEl.prototype.getName = function () {
                return this.getEl().getAttribute("name");
            };
            FormItemEl.prototype.setName = function (name) {
                this.getEl().setAttribute("name", name);
                return this;
            };
            FormItemEl.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            FormItemEl.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            FormItemEl.prototype.notifyValidityChanged = function (valid) {
                var _this = this;
                this.validityChangedListeners.forEach(function (listener) {
                    listener.call(_this, new api.ValidityChangedEvent(valid));
                });
            };
            return FormItemEl;
        })(dom.Element);
        dom.FormItemEl = FormItemEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ButtonEl = (function (_super) {
            __extends(ButtonEl, _super);
            function ButtonEl(className) {
                _super.call(this, "button", className);
            }
            return ButtonEl;
        })(dom.FormItemEl);
        dom.ButtonEl = ButtonEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var PEl = (function (_super) {
            __extends(PEl, _super);
            function PEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("p").setClassName(className));
            }
            return PEl;
        })(dom.Element);
        dom.PEl = PEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FormInputEl = (function (_super) {
            __extends(FormInputEl, _super);
            function FormInputEl(tagName, className) {
                _super.call(this, tagName, className);
                this.addClass('form-input');
            }
            FormInputEl.prototype.getValue = function () {
                return this.getEl().getValue();
            };
            FormInputEl.prototype.setValue = function (value) {
                this.getEl().setValue(value);
                return this;
            };
            FormInputEl.prototype.onChange = function (listener) {
                this.getEl().addEventListener("change", listener);
            };
            FormInputEl.prototype.unChange = function (listener) {
                this.getEl().removeEventListener("change", listener);
            };
            FormInputEl.prototype.onInput = function (listener) {
                this.getEl().addEventListener("input", listener);
            };
            FormInputEl.prototype.unInput = function (listener) {
                this.getEl().removeEventListener("input", listener);
            };
            return FormInputEl;
        })(dom.FormItemEl);
        dom.FormInputEl = FormInputEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var CompositeFormInputEl = (function (_super) {
            __extends(CompositeFormInputEl, _super);
            function CompositeFormInputEl(wrappedInput) {
                var _this = this;
                var additionalElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    additionalElements[_i - 1] = arguments[_i];
                }
                _super.call(this, "div");
                this.additionalElements = [];
                this.addClass("composite-input");
                this.wrappedInput = wrappedInput;
                this.appendChild(this.wrappedInput);
                if (additionalElements) {
                    additionalElements.forEach(function (element) {
                        _this.addAdditionalElement(element);
                    });
                }
            }
            CompositeFormInputEl.prototype.getValue = function () {
                return this.wrappedInput.getValue();
            };
            CompositeFormInputEl.prototype.getName = function () {
                return this.wrappedInput.getName();
            };
            CompositeFormInputEl.prototype.setValue = function (value) {
                this.wrappedInput.setValue(value);
                return this;
            };
            CompositeFormInputEl.prototype.addAdditionalElement = function (element) {
                this.appendChild(element);
                this.additionalElements.push(element);
            };
            return CompositeFormInputEl;
        })(api.dom.FormInputEl);
        dom.CompositeFormInputEl = CompositeFormInputEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var InputEl = (function (_super) {
            __extends(InputEl, _super);
            function InputEl(className, type) {
                _super.call(this, "input", className);
                this.setType(type || 'text');
            }
            InputEl.prototype.getValue = function () {
                return this.getEl().getValue();
            };
            InputEl.prototype.setValue = function (value) {
                this.getEl().setValue(value);
                return this;
            };
            InputEl.prototype.getName = function () {
                return this.getEl().getAttribute('name');
            };
            InputEl.prototype.setName = function (value) {
                this.getEl().setAttribute('name', value);
                return this;
            };
            InputEl.prototype.getType = function () {
                return this.getEl().getAttribute('type');
            };
            InputEl.prototype.setType = function (type) {
                this.getEl().setAttribute('type', type);
                return this;
            };
            InputEl.prototype.setPlaceholder = function (value) {
                this.getEl().setAttribute('placeholder', value);
                return this;
            };
            InputEl.prototype.getPlaceholder = function () {
                return this.getEl().getAttribute('placeholder');
            };
            InputEl.prototype.getPattern = function () {
                return this.getEl().getAttribute('pattern');
            };
            InputEl.prototype.setPattern = function (pattern) {
                this.getEl().setAttribute('pattern', pattern);
                return this;
            };
            InputEl.prototype.reset = function () {
                this.getEl().setValue("");
            };
            InputEl.prototype.isValid = function () {
                var validity = this.getHTMLElement().validity;
                return validity && validity.valid;
            };
            InputEl.prototype.validate = function () {
                return this.getHTMLElement().checkValidity();
            };
            InputEl.prototype.setRequired = function (required) {
                if (required) {
                    this.getEl().setAttribute('required', 'required');
                }
                else {
                    this.getEl().removeAttribute('required');
                }
                return this;
            };
            InputEl.prototype.isRequired = function () {
                return this.getEl().hasAttribute('required');
            };
            InputEl.prototype.setReadOnly = function (readOnly) {
                if (readOnly) {
                    this.getEl().setAttribute('readonly', 'readonly');
                }
                else {
                    this.getEl().removeAttribute('readonly');
                }
                return this;
            };
            InputEl.prototype.isReadOnly = function () {
                return this.getEl().hasAttribute('readonly');
            };
            return InputEl;
        })(dom.FormInputEl);
        dom.InputEl = InputEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var LabelEl = (function (_super) {
            __extends(LabelEl, _super);
            function LabelEl(value, forElement, className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("label").setClassName(className));
                this.setValue(value);
                if (forElement) {
                    this.getEl().setAttribute("for", forElement.getId());
                }
            }
            LabelEl.prototype.setValue = function (value) {
                this.getEl().setInnerHtml(value);
            };
            LabelEl.prototype.getValue = function () {
                return this.getEl().getInnerHtml();
            };
            return LabelEl;
        })(dom.Element);
        dom.LabelEl = LabelEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var SelectEl = (function (_super) {
            __extends(SelectEl, _super);
            function SelectEl(className) {
                _super.call(this, "select", className);
            }
            SelectEl.prototype.setName = function (value) {
                this.getEl().setAttribute('name', value);
                return this;
            };
            return SelectEl;
        })(dom.FormInputEl);
        dom.SelectEl = SelectEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var OptionEl = (function (_super) {
            __extends(OptionEl, _super);
            function OptionEl(value, displayName) {
                _super.call(this, new dom.NewElementBuilder().setTagName("option"));
                this.getEl().setValue(value);
                this.getEl().setInnerHtml(displayName);
            }
            return OptionEl;
        })(dom.Element);
        dom.OptionEl = OptionEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var IFrameEl = (function (_super) {
            __extends(IFrameEl, _super);
            function IFrameEl(className) {
                var _this = this;
                _super.call(this, new dom.NewElementBuilder().setTagName("iframe").setClassName(className));
                this.loaded = false;
                this.onLoaded(function (event) {
                    _this.loaded = true;
                });
            }
            IFrameEl.prototype.setSrc = function (src) {
                this.getEl().setAttribute("src", src);
                return this;
            };
            IFrameEl.prototype.isLoaded = function () {
                return this.loaded;
            };
            IFrameEl.prototype.postMessage = function (data, targetOrigin) {
                if (targetOrigin === void 0) { targetOrigin = "*"; }
                var thisIFrameElement = this.getHTMLElement();
                thisIFrameElement.contentWindow.postMessage(data, targetOrigin);
            };
            IFrameEl.prototype.onLoaded = function (listener) {
                this.getEl().addEventListener("load", listener);
            };
            IFrameEl.prototype.unLoaded = function (listener) {
                this.getEl().removeEventListener("load", listener);
            };
            return IFrameEl;
        })(api.dom.Element);
        dom.IFrameEl = IFrameEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FieldsetEl = (function (_super) {
            __extends(FieldsetEl, _super);
            function FieldsetEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("fieldset").setClassName(className));
            }
            return FieldsetEl;
        })(dom.Element);
        dom.FieldsetEl = FieldsetEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var LegendEl = (function (_super) {
            __extends(LegendEl, _super);
            function LegendEl(legend, className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("legend").setClassName(className));
                this.getEl().setInnerHtml(legend);
            }
            return LegendEl;
        })(dom.Element);
        dom.LegendEl = LegendEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FormEl = (function (_super) {
            __extends(FormEl, _super);
            function FormEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("form").setClassName(className));
            }
            FormEl.prototype.preventSubmit = function () {
                this.onSubmit(function (event) {
                    event.preventDefault();
                });
            };
            FormEl.prototype.onSubmit = function (listener) {
                this.getEl().addEventListener("submit", listener);
            };
            FormEl.prototype.unSubmit = function (listener) {
                this.getEl().removeEventListener("submit", listener);
            };
            FormEl.moveFocusToNextFocusable = function (input, focusableSelector) {
                var focusableElements = document.querySelectorAll(focusableSelector ? focusableSelector : "input, button, select");
                var index = FormEl.getIndexOfInput(focusableElements, input);
                if (index < 0) {
                    return;
                }
                for (var i = index + 1; i < focusableElements.length; i++) {
                    var nextFocusable = api.dom.Element.fromHtmlElement(focusableElements.item(i));
                    if (nextFocusable.getEl().getTabIndex() && nextFocusable.getEl().getTabIndex() < 0) {
                        continue;
                    }
                    else {
                        nextFocusable.giveFocus();
                        return;
                    }
                }
            };
            FormEl.moveFocusToPrevFocusable = function (input, focusableSelector) {
                var focusableElements = document.querySelectorAll(focusableSelector ? focusableSelector : "input, button, select");
                var index = FormEl.getIndexOfInput(focusableElements, input);
                do {
                    index = index - 1;
                    if (0 <= index) {
                        var nextFocusable = api.dom.Element.fromHtmlElement(focusableElements.item(index));
                    }
                } while (nextFocusable.getEl().getTabIndex() && nextFocusable.getEl().getTabIndex() < 0);
                nextFocusable.giveFocus();
                return;
            };
            FormEl.getIndexOfInput = function (elements, el) {
                var index = -1;
                var inputHTMLElement = el.getHTMLElement();
                for (var i = 0; i < elements.length; i++) {
                    if (inputHTMLElement == elements.item(i)) {
                        index = i;
                        break;
                    }
                }
                return index;
            };
            return FormEl;
        })(dom.Element);
        dom.FormEl = FormEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var AsideEl = (function (_super) {
            __extends(AsideEl, _super);
            function AsideEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("aside").setClassName(className));
            }
            return AsideEl;
        })(dom.Element);
        dom.AsideEl = AsideEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var SectionEl = (function (_super) {
            __extends(SectionEl, _super);
            function SectionEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("section").setClassName(className));
            }
            return SectionEl;
        })(dom.Element);
        dom.SectionEl = SectionEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ArticleEl = (function (_super) {
            __extends(ArticleEl, _super);
            function ArticleEl(className, contentEditable) {
                _super.call(this, new dom.NewElementBuilder().setTagName("article").setClassName(className));
                this.setContentEditable(contentEditable);
            }
            return ArticleEl;
        })(dom.Element);
        dom.ArticleEl = ArticleEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        (function (Type) {
            Type[Type["INFO"] = 0] = "INFO";
            Type[Type["ERROR"] = 1] = "ERROR";
            Type[Type["WARNING"] = 2] = "WARNING";
            Type[Type["ACTION"] = 3] = "ACTION";
            Type[Type["SUCCESS"] = 4] = "SUCCESS";
        })(notify.Type || (notify.Type = {}));
        var Type = notify.Type;
        var Action = (function () {
            function Action(name, handler) {
                this.name = name;
                this.handler = handler;
            }
            Action.prototype.getName = function () {
                return this.name;
            };
            Action.prototype.getHandler = function () {
                return this.handler;
            };
            return Action;
        })();
        notify.Action = Action;
        var Message = (function () {
            function Message(type, text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                this.type = type;
                this.text = text;
                this.actions = [];
                this.autoHide = autoHide;
            }
            Message.prototype.getType = function () {
                return this.type;
            };
            Message.prototype.getText = function () {
                return this.text;
            };
            Message.prototype.getActions = function () {
                return this.actions;
            };
            Message.prototype.getAutoHide = function () {
                return this.autoHide;
            };
            Message.prototype.addAction = function (name, handler) {
                this.actions.push(new Action(name, handler));
            };
            Message.newSuccess = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(4 /* SUCCESS */, text, autoHide);
            };
            Message.newInfo = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(0 /* INFO */, text, autoHide);
            };
            Message.newError = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(1 /* ERROR */, text, autoHide);
            };
            Message.newWarning = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(2 /* WARNING */, text, autoHide);
            };
            Message.newAction = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(3 /* ACTION */, text, autoHide);
            };
            return Message;
        })();
        notify.Message = Message;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        function showSuccess(message) {
            return notify.NotifyManager.get().showSuccess(message);
        }
        notify.showSuccess = showSuccess;
        function showFeedback(message) {
            return notify.NotifyManager.get().showFeedback(message);
        }
        notify.showFeedback = showFeedback;
        function showError(message, autoHide) {
            if (autoHide === void 0) { autoHide = true; }
            return notify.NotifyManager.get().showError(message, autoHide);
        }
        notify.showError = showError;
        function showWarning(message) {
            return notify.NotifyManager.get().showWarning(message);
        }
        notify.showWarning = showWarning;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotifyOpts = (function () {
            function NotifyOpts() {
            }
            NotifyOpts.prototype.addListeners = function (message) {
                this.listeners = [];
                var actions = message.getActions();
                for (var i = 0; i < actions.length; i++) {
                    this.listeners.push(actions[i].getHandler());
                }
            };
            NotifyOpts.prototype.createHtmlMessage = function (message) {
                var actions = message.getActions();
                this.message = '<span>' + message.getText() + '</span>';
                if (actions.length > 0) {
                    var linkHtml = '<span style="float: right; margin-left: 30px;">';
                    for (var i = 0; i < actions.length; i++) {
                        if ((i > 0) && (i == (actions.length - 1))) {
                            linkHtml += ' or ';
                        }
                        else if (i > 0) {
                            linkHtml += ', ';
                        }
                        linkHtml += '<a href="#" class="notify.action_"' + i + '">';
                        linkHtml += actions[i].getName() + "</a>";
                    }
                    linkHtml += '</span>';
                    this.message = linkHtml + this.message;
                }
            };
            NotifyOpts.buildOpts = function (message) {
                var opts = new NotifyOpts();
                opts.autoHide = message.getAutoHide();
                if (message.getType() == 1 /* ERROR */) {
                    opts.type = 'error';
                }
                else if (message.getType() == 2 /* WARNING */) {
                    opts.type = 'warning';
                }
                else if (message.getType() == 3 /* ACTION */) {
                    opts.type = 'action';
                }
                else if (message.getType() == 4 /* SUCCESS */) {
                    opts.type = 'success';
                }
                opts.createHtmlMessage(message);
                opts.addListeners(message);
                return opts;
            };
            return NotifyOpts;
        })();
        notify.NotifyOpts = NotifyOpts;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotifyManager = (function () {
            function NotifyManager() {
                this.space = 3;
                this.lifetime = 5000;
                this.slideDuration = 1000;
                this.timers = {};
                this.registry = {};
                this.el = new notify.NotificationContainer();
                api.dom.Body.get().appendChild(this.el);
                this.el.getEl().setBottomPx(0);
            }
            NotifyManager.prototype.showFeedback = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var feedback = notify.Message.newInfo(message, autoHide);
                return this.notify(feedback);
            };
            NotifyManager.prototype.showSuccess = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var feedback = notify.Message.newSuccess(message, autoHide);
                return this.notify(feedback);
            };
            NotifyManager.prototype.showError = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var error = notify.Message.newError(message, autoHide);
                return this.notify(error);
            };
            NotifyManager.prototype.showWarning = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var warning = notify.Message.newWarning(message, autoHide);
                return this.notify(warning);
            };
            NotifyManager.prototype.notify = function (message) {
                var opts = notify.NotifyOpts.buildOpts(message);
                return this.doNotify(opts);
            };
            NotifyManager.prototype.hide = function (messageId) {
                if (this.registry[messageId]) {
                    this.remove(this.registry[messageId]);
                    delete this.registry[messageId];
                }
            };
            NotifyManager.prototype.doNotify = function (opts) {
                var _this = this;
                var notificationEl = this.renderNotification(opts);
                this.registry[notificationEl.getEl().getId()] = notificationEl;
                this.setListeners(notificationEl, opts);
                wemjq(notificationEl.getHTMLElement()).animate({
                    height: 'toggle'
                }, this.slideDuration, function () {
                    if (opts.autoHide) {
                        _this.timers[notificationEl.getEl().getId()] = {
                            remainingTime: _this.lifetime
                        };
                        _this.startTimer(notificationEl);
                    }
                });
                return notificationEl.getEl().getId();
            };
            NotifyManager.prototype.setListeners = function (el, opts) {
                var _this = this;
                el.onClicked(function () {
                    _this.remove(el);
                    return false;
                });
                el.onMouseEnter(function () {
                    _this.stopTimer(el);
                });
                el.onMouseLeave(function () {
                    _this.startTimer(el);
                });
                if (opts.listeners) {
                    opts.listeners.forEach(function (listener) {
                        el.onClicked(listener);
                    });
                }
            };
            NotifyManager.prototype.remove = function (el) {
                var _this = this;
                if (!el) {
                    return;
                }
                wemjq(el.getHTMLElement()).animate({
                    height: 'hide'
                }, this.slideDuration, 'linear', function () {
                    _this.el.getWrapper().removeChild(el);
                });
                delete this.timers[el.getEl().getId()];
            };
            NotifyManager.prototype.startTimer = function (el) {
                var _this = this;
                var timer = this.timers[el.getEl().getId()];
                if (!timer) {
                    return;
                }
                timer.id = setTimeout(function () {
                    _this.remove(el);
                }, timer.remainingTime);
                timer.startTime = Date.now();
            };
            NotifyManager.prototype.stopTimer = function (el) {
                var timer = this.timers[el.getEl().getId()];
                if (!timer || !timer.id) {
                    return;
                }
                clearTimeout(timer.id);
                timer.id = null;
                timer.remainingTime -= Date.now() - timer.startTime;
            };
            NotifyManager.prototype.renderNotification = function (opts) {
                var style = {};
                var notifyDiv = new notify.NotificationMessage(opts.message);
                this.el.getWrapper().appendChild(notifyDiv);
                notifyDiv.hide();
                if (opts.type) {
                    notifyDiv.addClass(opts.type);
                }
                return notifyDiv;
            };
            NotifyManager.get = function () {
                if (!NotifyManager.instance) {
                    NotifyManager.instance = new NotifyManager();
                }
                return NotifyManager.instance;
            };
            return NotifyManager;
        })();
        notify.NotifyManager = NotifyManager;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotificationMessage = (function (_super) {
            __extends(NotificationMessage, _super);
            function NotificationMessage(message) {
                _super.call(this, "notification");
                this.notificationInner = new api.dom.DivEl("notification-inner");
                var notificationRemove = new api.dom.AEl("notification-remove");
                notificationRemove.setHtml("X");
                notificationRemove.setUrl("#");
                var notificationContent = new api.dom.DivEl("notification-content");
                notificationContent.getEl().setInnerHtml(message);
                this.notificationInner.appendChild(notificationRemove).appendChild(notificationContent);
                this.appendChild(this.notificationInner);
            }
            return NotificationMessage;
        })(api.dom.DivEl);
        notify.NotificationMessage = NotificationMessage;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotificationContainer = (function (_super) {
            __extends(NotificationContainer, _super);
            function NotificationContainer() {
                _super.call(this, "notification-container");
                this.wrapper = new api.dom.DivEl("notification-wrapper");
                this.appendChild(this.wrapper);
            }
            NotificationContainer.prototype.getWrapper = function () {
                return this.wrapper;
            };
            return NotificationContainer;
        })(api.dom.DivEl);
        notify.NotificationContainer = NotificationContainer;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var Path = (function () {
            function Path(elements, elementDivider, absolute) {
                this.elementDivider = elementDivider != null ? elementDivider : Path.DEFAULT_ELEMENT_DIVIDER;
                this.absolute = absolute == undefined ? true : absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? this.elementDivider : "") + this.elements.join(this.elementDivider);
            }
            Path.fromString = function (s, elementDivider) {
                if (elementDivider == null) {
                    elementDivider = Path.DEFAULT_ELEMENT_DIVIDER;
                }
                var absolute = s.charAt(0) == elementDivider;
                var elements = s.split(elementDivider);
                return new Path(Path.removeEmptyElements(elements), elementDivider, absolute);
            };
            Path.fromParent = function (parent) {
                var childElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    childElements[_i - 1] = arguments[_i];
                }
                var elements = parent.elements.slice(0);
                childElements.forEach(function (element) {
                    elements.push(element);
                });
                return new Path(elements, parent.elementDivider, parent.isAbsolute());
            };
            Path.removeEmptyElements = function (elements) {
                var filteredElements = [];
                elements.forEach(function (element) {
                    if (element.length > 0) {
                        filteredElements.push(element);
                    }
                });
                return filteredElements;
            };
            Path.prototype.getElements = function () {
                return this.elements;
            };
            Path.prototype.getElement = function (index) {
                return this.elements[index];
            };
            Path.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            Path.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElemements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElemements.push(element);
                    }
                });
                return new Path(parentElemements);
            };
            Path.prototype.toString = function () {
                return this.refString;
            };
            Path.prototype.isAbsolute = function () {
                return this.absolute;
            };
            Path.DEFAULT_ELEMENT_DIVIDER = "/";
            return Path;
        })();
        rest.Path = Path;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var JsonRequest = (function () {
            function JsonRequest() {
                this.method = "GET";
                this.timeoutMillis = 10000;
            }
            JsonRequest.prototype.setPath = function (value) {
                this.path = value;
                return this;
            };
            JsonRequest.prototype.setMethod = function (value) {
                this.method = value;
                return this;
            };
            JsonRequest.prototype.setParams = function (params) {
                this.params = params;
                return this;
            };
            JsonRequest.prototype.setTimeout = function (timeoutMillis) {
                if (timeoutMillis) {
                    this.timeoutMillis = timeoutMillis;
                }
                return this;
            };
            JsonRequest.prototype.send = function () {
                var deferred = wemQ.defer();
                var request = new XMLHttpRequest();
                request.onreadystatechange = function () {
                    if (request.readyState === 4) {
                        if (request.status === 204) {
                            deferred.resolve(new rest.JsonResponse(null));
                        }
                        else if (request.status >= 200 && request.status < 300) {
                            deferred.resolve(new rest.JsonResponse(request.response));
                        }
                        else if (request.status === 403) {
                            deferred.reject(new api.AccessDeniedException('Access denied'));
                        }
                        else {
                            try {
                                var errorJson = request.response ? JSON.parse(request.response) : null;
                            }
                            catch (error) {
                                deferred.reject(error);
                            }
                            deferred.reject(new rest.RequestError(request.status, request.statusText, errorJson ? errorJson.message : ""));
                        }
                    }
                };
                if ("POST" === this.method.toUpperCase()) {
                    this.preparePOSTRequest(request);
                    var paramString = JSON.stringify(this.params);
                    request.send(paramString);
                }
                else {
                    var request = this.prepareGETRequest(request);
                    request.send();
                }
                return deferred.promise;
            };
            JsonRequest.prototype.prepareGETRequest = function (request) {
                var paramString = api.util.UriHelper.encodeUrlParams(this.params);
                request.open(this.method, api.util.UriHelper.getUri(this.path.toString()) + '?' + paramString, true);
                request.timeout = this.timeoutMillis;
                request.setRequestHeader("Accept", "application/json");
                return request;
            };
            JsonRequest.prototype.preparePOSTRequest = function (request) {
                request.open(this.method, api.util.UriHelper.getUri(this.path.toString()), true);
                request.timeout = this.timeoutMillis;
                request.setRequestHeader("Accept", "application/json");
                request.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            };
            return JsonRequest;
        })();
        rest.JsonRequest = JsonRequest;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var Response = (function () {
            function Response() {
            }
            return Response;
        })();
        rest.Response = Response;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var JsonResponse = (function (_super) {
            __extends(JsonResponse, _super);
            function JsonResponse(json) {
                _super.call(this);
                this.json = JSON.parse(json);
            }
            JsonResponse.prototype.isBlank = function () {
                return !this.json;
            };
            JsonResponse.prototype.getJson = function () {
                return this.json;
            };
            JsonResponse.prototype.hasResult = function () {
                if (this.json == undefined || this.json == null) {
                    return false;
                }
                return true;
            };
            JsonResponse.prototype.getResult = function () {
                if (!this.hasResult()) {
                    return null;
                }
                if (this.json.result) {
                    return this.json.result;
                }
                else {
                    return this.json;
                }
            };
            return JsonResponse;
        })(api.rest.Response);
        rest.JsonResponse = JsonResponse;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var ExceptionType = api.ExceptionType;
        var RequestError = (function (_super) {
            __extends(RequestError, _super);
            function RequestError(statusCode, statusText, errorMsg) {
                var notifyMsg = (statusCode > 0) ? "HTTP Status " + statusCode + " - " + statusText + ": " + errorMsg : "Unable to connect to server";
                var type = (statusCode >= 400 && statusCode < 500) ? 2 /* WARNING */ : 1 /* ERROR */;
                _super.call(this, notifyMsg, type);
            }
            return RequestError;
        })(api.Exception);
        rest.RequestError = RequestError;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var ResourceRequest = (function () {
            function ResourceRequest() {
                this.method = "GET";
                this.restPath = rest.Path.fromString("admin/rest");
            }
            ResourceRequest.prototype.setMethod = function (value) {
                this.method = value;
            };
            ResourceRequest.prototype.getRestPath = function () {
                return this.restPath;
            };
            ResourceRequest.prototype.getRequestPath = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ResourceRequest.prototype.getParams = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ResourceRequest.prototype.setTimeout = function (timeoutMillis) {
                this.timeoutMillis = timeoutMillis;
            };
            ResourceRequest.prototype.validate = function () {
            };
            ResourceRequest.prototype.send = function () {
                this.validate();
                var jsonRequest = new rest.JsonRequest().setMethod(this.method).setParams(this.getParams()).setPath(this.getRequestPath()).setTimeout(this.timeoutMillis);
                return jsonRequest.send();
            };
            ResourceRequest.prototype.sendAndParse = function () {
                throw new Error("sendAndParse method was not implemented");
            };
            return ResourceRequest;
        })();
        rest.ResourceRequest = ResourceRequest;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        (function (Expand) {
            Expand[Expand["NONE"] = 0] = "NONE";
            Expand[Expand["SUMMARY"] = 1] = "SUMMARY";
            Expand[Expand["FULL"] = 2] = "FULL";
        })(rest.Expand || (rest.Expand = {}));
        var Expand = rest.Expand;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var icon;
    (function (icon) {
        var IconUrlResolver = (function () {
            function IconUrlResolver() {
            }
            IconUrlResolver.prototype.appendParam = function (paramName, paramValue, url) {
                var questionIndex = url.indexOf("?");
                if (questionIndex == -1) {
                    url += "?" + paramName + "=" + paramValue;
                    return url;
                }
                else if (url.charAt(url.length - 1) == '?') {
                    url += paramName + "=" + paramValue;
                    return url;
                }
                else {
                    url += "&" + paramName + "=" + paramValue;
                    return url;
                }
            };
            return IconUrlResolver;
        })();
        icon.IconUrlResolver = IconUrlResolver;
    })(icon = api.icon || (api.icon = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        (function (PrincipalType) {
            PrincipalType[PrincipalType["USER"] = 0] = "USER";
            PrincipalType[PrincipalType["GROUP"] = 1] = "GROUP";
            PrincipalType[PrincipalType["ROLE"] = 2] = "ROLE";
        })(security.PrincipalType || (security.PrincipalType = {}));
        var PrincipalType = security.PrincipalType;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStoreKey = (function () {
            function UserStoreKey(id) {
                api.util.assert(!api.util.StringHelper.isBlank(id), "UserStoreKey id cannot be null or empty");
                this.id = id;
            }
            UserStoreKey.prototype.isSystem = function () {
                return this.id === UserStoreKey.SYSTEM.id;
            };
            UserStoreKey.prototype.toString = function () {
                return this.id;
            };
            UserStoreKey.prototype.getId = function () {
                return this.id;
            };
            UserStoreKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStoreKey)) {
                    return false;
                }
                var other = o;
                return this.id === other.id;
            };
            UserStoreKey.SYSTEM = new UserStoreKey('system');
            return UserStoreKey;
        })();
        security.UserStoreKey = UserStoreKey;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalKey = (function () {
            function PrincipalKey(userStore, type, principalId) {
                api.util.assert((type === 2 /* ROLE */) || (!!userStore), "Principal user store cannot be null");
                api.util.assertNotNull(type, "Principal type cannot be null");
                api.util.assert(!api.util.StringHelper.isBlank(principalId), "Principal id cannot be null or empty");
                this.userStore = userStore;
                this.type = type;
                this.principalId = principalId;
                if (type === 2 /* ROLE */) {
                    this.refString = security.PrincipalType[type].toLowerCase() + PrincipalKey.SEPARATOR + principalId;
                }
                else {
                    this.refString = security.PrincipalType[type].toLowerCase() + PrincipalKey.SEPARATOR + userStore.toString() + PrincipalKey.SEPARATOR + principalId;
                }
            }
            PrincipalKey.fromString = function (str) {
                if (str === PrincipalKey.ANONYMOUS_PRINCIPAL.refString) {
                    return PrincipalKey.ANONYMOUS_PRINCIPAL;
                }
                var sepIndex = str.indexOf(PrincipalKey.SEPARATOR);
                if (sepIndex === -1) {
                    throw new Error("Not a valid principal key [" + str + "]");
                }
                var sepIndex2 = str.indexOf(PrincipalKey.SEPARATOR, sepIndex + 1);
                var typeStr = str.substring(0, sepIndex);
                var type = security.PrincipalType[typeStr.toUpperCase()];
                if (type === 2 /* ROLE */) {
                    var principalId = str.substring(sepIndex + 1, str.length) || '';
                    return new PrincipalKey(null, type, principalId);
                }
                else {
                    if (sepIndex2 === -1) {
                        throw new Error("Not a valid principal key [" + str + "]");
                    }
                    var userStore = str.substring(sepIndex + 1, sepIndex2) || '';
                    var principalId = str.substring(sepIndex2 + 1, str.length);
                    return new PrincipalKey(new security.UserStoreKey(userStore), type, principalId);
                }
            };
            PrincipalKey.prototype.getUserStore = function () {
                return this.userStore;
            };
            PrincipalKey.prototype.getType = function () {
                return this.type;
            };
            PrincipalKey.prototype.getId = function () {
                return this.principalId;
            };
            PrincipalKey.prototype.isUser = function () {
                return this.type === 0 /* USER */;
            };
            PrincipalKey.prototype.isGroup = function () {
                return this.type === 1 /* GROUP */;
            };
            PrincipalKey.prototype.isRole = function () {
                return this.type === 2 /* ROLE */;
            };
            PrincipalKey.prototype.isAnonymous = function () {
                return this.refString === PrincipalKey.ANONYMOUS_PRINCIPAL.refString;
            };
            PrincipalKey.prototype.toString = function () {
                return this.refString;
            };
            PrincipalKey.prototype.toPath = function (toParent) {
                if (toParent === void 0) { toParent = false; }
                var path = this.isRole() ? "/roles/" : api.util.StringHelper.format("/{0}/{1}/", this.getUserStore().toString(), security.PrincipalType[this.getType()].toLowerCase().replace(/(group|user)/g, "$&s"));
                if (!toParent) {
                    path += this.getId();
                }
                return path;
            };
            PrincipalKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PrincipalKey)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            PrincipalKey.ofAnonymous = function () {
                return PrincipalKey.ANONYMOUS_PRINCIPAL;
            };
            PrincipalKey.ofUser = function (userStore, userId) {
                return new PrincipalKey(userStore, 0 /* USER */, userId);
            };
            PrincipalKey.ofGroup = function (userStore, groupId) {
                return new PrincipalKey(userStore, 1 /* GROUP */, groupId);
            };
            PrincipalKey.ofRole = function (roleId) {
                return new PrincipalKey(security.UserStoreKey.SYSTEM, 2 /* ROLE */, roleId);
            };
            PrincipalKey.SEPARATOR = ":";
            PrincipalKey.ANONYMOUS_PRINCIPAL = new PrincipalKey(security.UserStoreKey.SYSTEM, 0 /* USER */, "anonymous");
            return PrincipalKey;
        })();
        security.PrincipalKey = PrincipalKey;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var RoleKeys = (function () {
            function RoleKeys() {
            }
            RoleKeys.EVERYONE = security.PrincipalKey.ofRole('system.everyone');
            RoleKeys.ADMIN = security.PrincipalKey.ofRole('system.admin');
            return RoleKeys;
        })();
        security.RoleKeys = RoleKeys;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Principal = (function () {
            function Principal(principalKey, displayName, modifiedTime) {
                this.key = principalKey;
                this.displayName = displayName || "";
                this.type = principalKey.getType();
                this.modifiedTime = modifiedTime;
            }
            Principal.fromJson = function (json) {
                var key = security.PrincipalKey.fromString(json.key);
                var date = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                return new Principal(key, json.displayName, date);
            };
            Principal.fromPrincipal = function (principal) {
                return new Principal(principal.getKey(), principal.getDisplayName(), principal.getModifiedTime());
            };
            Principal.prototype.toJson = function () {
                return {
                    displayName: this.displayName,
                    key: this.key.toString()
                };
            };
            Principal.prototype.getKey = function () {
                return this.key;
            };
            Principal.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Principal.prototype.getType = function () {
                return this.type;
            };
            Principal.prototype.getTypeName = function () {
                switch (this.type) {
                    case 1 /* GROUP */:
                        return "Group";
                    case 0 /* USER */:
                        return "User";
                    case 2 /* ROLE */:
                        return "Role";
                    default:
                        return "";
                }
            };
            Principal.prototype.isUser = function () {
                return this.type === 0 /* USER */;
            };
            Principal.prototype.isGroup = function () {
                return this.type === 1 /* GROUP */;
            };
            Principal.prototype.isRole = function () {
                return this.type === 2 /* ROLE */;
            };
            Principal.prototype.asUser = function () {
                return (this instanceof api.security.User) ? this : null;
            };
            Principal.prototype.asGroup = function () {
                return (this instanceof api.security.Group) ? this : null;
            };
            Principal.prototype.asRole = function () {
                return (this instanceof api.security.Role) ? this : null;
            };
            Principal.prototype.getModifiedTime = function () {
                return this.modifiedTime;
            };
            Principal.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Principal)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.key, other.key)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.modifiedTime, other.modifiedTime)) {
                    return false;
                }
                return true;
            };
            Principal.prototype.clone = function () {
                return new Principal(this.key, this.displayName, this.modifiedTime);
            };
            return Principal;
        })();
        security.Principal = Principal;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var User = (function (_super) {
            __extends(User, _super);
            function User(builder) {
                _super.call(this, builder.key, builder.displayName, builder.modifiedTime);
                api.util.assert(builder.key.isUser(), 'Expected PrincipalKey of type User');
                this.email = builder.email || "";
                this.login = builder.login || "";
                this.loginDisabled = builder.loginDisabled || false;
                this.memberships = builder.memberships || [];
            }
            User.prototype.getEmail = function () {
                return this.email;
            };
            User.prototype.getLogin = function () {
                return this.login;
            };
            User.prototype.isDisabled = function () {
                return this.loginDisabled;
            };
            User.prototype.getMemberships = function () {
                return this.memberships;
            };
            User.prototype.setMemberships = function (memberships) {
                this.memberships = memberships;
            };
            User.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, User)) {
                    return false;
                }
                var other = o;
                return _super.prototype.equals.call(this, o) && this.loginDisabled === other.isDisabled() && this.email === other.getEmail() && this.login === other.getLogin() && api.ObjectHelper.arrayEquals(this.memberships, other.getMemberships());
            };
            User.prototype.clone = function () {
                return this.newBuilder().build();
            };
            User.prototype.newBuilder = function () {
                return new UserBuilder(this);
            };
            User.create = function () {
                return new UserBuilder();
            };
            User.fromJson = function (json) {
                return new UserBuilder().fromJson(json).build();
            };
            return User;
        })(security.Principal);
        security.User = User;
        var UserBuilder = (function () {
            function UserBuilder(source) {
                if (source) {
                    this.key = source.getKey();
                    this.displayName = source.getDisplayName();
                    this.email = source.getEmail();
                    this.login = source.getLogin();
                    this.loginDisabled = source.isDisabled();
                    this.modifiedTime = source.getModifiedTime();
                    this.memberships = source.getMemberships().slice(0);
                }
                else {
                    this.memberships = [];
                }
            }
            UserBuilder.prototype.fromJson = function (json) {
                this.key = security.PrincipalKey.fromString(json.key);
                this.displayName = json.displayName;
                this.email = json.email;
                this.login = json.login;
                this.loginDisabled = json.loginDisabled;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                if (json.memberships) {
                    this.memberships = json.memberships.map(function (principalJson) { return security.Principal.fromJson(principalJson); });
                }
                return this;
            };
            UserBuilder.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UserBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UserBuilder.prototype.setEmail = function (value) {
                this.email = value;
                return this;
            };
            UserBuilder.prototype.setLogin = function (value) {
                this.login = value;
                return this;
            };
            UserBuilder.prototype.setDisabled = function (value) {
                this.loginDisabled = value;
                return this;
            };
            UserBuilder.prototype.setMemberships = function (memberships) {
                this.memberships = memberships || [];
                return this;
            };
            UserBuilder.prototype.build = function () {
                return new User(this);
            };
            return UserBuilder;
        })();
        security.UserBuilder = UserBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Group = (function (_super) {
            __extends(Group, _super);
            function Group(builder) {
                _super.call(this, builder.key, builder.displayName, builder.modifiedTime);
                api.util.assert(builder.key.isGroup(), 'Expected PrincipalKey of type Group');
                this.members = builder.members || [];
            }
            Group.prototype.getMembers = function () {
                return this.members;
            };
            Group.prototype.setMembers = function (members) {
                this.members = members || [];
            };
            Group.prototype.addMember = function (member) {
                this.members.push(member);
            };
            Group.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Group)) {
                    return false;
                }
                var other = o;
                return _super.prototype.equals.call(this, o) && api.ObjectHelper.arrayEquals(this.members, other.getMembers());
            };
            Group.prototype.clone = function () {
                return this.newBuilder().build();
            };
            Group.prototype.newBuilder = function () {
                return new GroupBuilder(this);
            };
            Group.create = function () {
                return new GroupBuilder();
            };
            Group.fromJson = function (json) {
                return new GroupBuilder().fromJson(json).build();
            };
            return Group;
        })(security.Principal);
        security.Group = Group;
        var GroupBuilder = (function () {
            function GroupBuilder(source) {
                if (source) {
                    this.key = source.getKey();
                    this.displayName = source.getDisplayName();
                    this.modifiedTime = source.getModifiedTime();
                    this.members = source.getMembers().slice(0);
                }
                else {
                    this.members = [];
                }
            }
            GroupBuilder.prototype.fromJson = function (json) {
                this.key = security.PrincipalKey.fromString(json.key);
                this.displayName = json.displayName;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                if (json.members) {
                    this.members = json.members.map(function (memberStr) { return security.PrincipalKey.fromString(memberStr); });
                }
                return this;
            };
            GroupBuilder.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            GroupBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            GroupBuilder.prototype.setMembers = function (members) {
                this.members = members || [];
                return this;
            };
            GroupBuilder.prototype.build = function () {
                return new Group(this);
            };
            return GroupBuilder;
        })();
        security.GroupBuilder = GroupBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Role = (function (_super) {
            __extends(Role, _super);
            function Role(builder) {
                _super.call(this, builder.key, builder.displayName, builder.modifiedTime);
                api.util.assert(builder.key.isRole(), 'Expected PrincipalKey of type Role');
                this.members = builder.members || [];
            }
            Role.prototype.getMembers = function () {
                return this.members;
            };
            Role.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Role)) {
                    return false;
                }
                var other = o;
                return _super.prototype.equals.call(this, o) && api.ObjectHelper.arrayEquals(this.members, other.getMembers());
            };
            Role.prototype.clone = function () {
                return this.newBuilder().build();
            };
            Role.prototype.newBuilder = function () {
                return new RoleBuilder(this);
            };
            Role.create = function () {
                return new RoleBuilder();
            };
            Role.fromJson = function (json) {
                return new RoleBuilder().fromJson(json).build();
            };
            return Role;
        })(security.Principal);
        security.Role = Role;
        var RoleBuilder = (function () {
            function RoleBuilder(source) {
                if (source) {
                    this.key = source.getKey();
                    this.displayName = source.getDisplayName();
                    this.modifiedTime = source.getModifiedTime();
                    this.members = source.getMembers().slice(0);
                }
                else {
                    this.members = [];
                }
            }
            RoleBuilder.prototype.fromJson = function (json) {
                this.key = security.PrincipalKey.fromString(json.key);
                this.displayName = json.displayName;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                if (json.members) {
                    this.members = json.members.map(function (memberStr) { return security.PrincipalKey.fromString(memberStr); });
                }
                return this;
            };
            RoleBuilder.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            RoleBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            RoleBuilder.prototype.setMembers = function (members) {
                this.members = members || [];
                return this;
            };
            RoleBuilder.prototype.build = function () {
                return new Role(this);
            };
            return RoleBuilder;
        })();
        security.RoleBuilder = RoleBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStore = (function () {
            function UserStore(builder) {
                this.displayName = builder.displayName;
                this.key = builder.key;
                this.permissions = builder.permissions || new api.security.acl.UserStoreAccessControlList();
            }
            UserStore.prototype.getDisplayName = function () {
                return this.displayName;
            };
            UserStore.prototype.getKey = function () {
                return this.key;
            };
            UserStore.prototype.getPermissions = function () {
                return this.permissions;
            };
            UserStore.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStore)) {
                    return false;
                }
                var other = o;
                return this.key.equals(other.key) && this.displayName === other.displayName && this.permissions.equals(other.permissions);
            };
            UserStore.prototype.clone = function () {
                return UserStore.create().setDisplayName(this.displayName).setKey(this.key.toString()).setPermissions(this.permissions).build();
            };
            UserStore.create = function () {
                return new UserStoreBuilder();
            };
            UserStore.fromJson = function (json) {
                return new UserStoreBuilder().fromJson(json).build();
            };
            return UserStore;
        })();
        security.UserStore = UserStore;
        var UserStoreBuilder = (function () {
            function UserStoreBuilder() {
            }
            UserStoreBuilder.prototype.fromJson = function (json) {
                this.key = new security.UserStoreKey(json.key);
                this.displayName = json.displayName;
                this.permissions = json.permissions ? api.security.acl.UserStoreAccessControlList.fromJson(json.permissions) : null;
                return this;
            };
            UserStoreBuilder.prototype.setKey = function (key) {
                this.key = new security.UserStoreKey(key);
                return this;
            };
            UserStoreBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UserStoreBuilder.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            UserStoreBuilder.prototype.build = function () {
                return new UserStore(this);
            };
            return UserStoreBuilder;
        })();
        security.UserStoreBuilder = UserStoreBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStoreListResult = (function () {
            function UserStoreListResult() {
            }
            return UserStoreListResult;
        })();
        security.UserStoreListResult = UserStoreListResult;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Role = api.security.Role;
        var Group = api.security.Group;
        var User = api.security.User;
        var SecurityResourceRequest = (function (_super) {
            __extends(SecurityResourceRequest, _super);
            function SecurityResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "security");
            }
            SecurityResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            SecurityResourceRequest.prototype.fromJsonToPrincipal = function (json) {
                var pKey = security.PrincipalKey.fromString(json.key);
                if (pKey.isRole()) {
                    return Role.fromJson(json);
                }
                else if (pKey.isGroup()) {
                    return Group.fromJson(json);
                }
                else if (pKey.isUser()) {
                    return User.fromJson(json);
                }
            };
            return SecurityResourceRequest;
        })(api.rest.ResourceRequest);
        security.SecurityResourceRequest = SecurityResourceRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var ListUserStoresRequest = (function (_super) {
            __extends(ListUserStoresRequest, _super);
            function ListUserStoresRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            ListUserStoresRequest.prototype.getParams = function () {
                return {};
            };
            ListUserStoresRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/list');
            };
            ListUserStoresRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.getResult().userStores.map(function (userStoreJson) {
                        return _this.fromJsonToUserStore(userStoreJson);
                    });
                });
            };
            ListUserStoresRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return ListUserStoresRequest;
        })(security.SecurityResourceRequest);
        security.ListUserStoresRequest = ListUserStoresRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var GetUserStoreByKeyRequest = (function (_super) {
            __extends(GetUserStoreByKeyRequest, _super);
            function GetUserStoreByKeyRequest(userStoreKey) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.userStoreKey = userStoreKey;
            }
            GetUserStoreByKeyRequest.prototype.getParams = function () {
                return {
                    key: this.userStoreKey.toString()
                };
            };
            GetUserStoreByKeyRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore');
            };
            GetUserStoreByKeyRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            GetUserStoreByKeyRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return GetUserStoreByKeyRequest;
        })(security.SecurityResourceRequest);
        security.GetUserStoreByKeyRequest = GetUserStoreByKeyRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalListJson = (function () {
            function PrincipalListJson() {
            }
            return PrincipalListJson;
        })();
        security.PrincipalListJson = PrincipalListJson;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var FindPrincipalsRequest = (function (_super) {
            __extends(FindPrincipalsRequest, _super);
            function FindPrincipalsRequest() {
                _super.call(this);
            }
            FindPrincipalsRequest.prototype.getParams = function () {
                return {
                    "types": this.enumToStrings(this.allowedTypes).join(','),
                    "query": this.searchQuery,
                    "userStoreKey": this.userStoreKey ? this.userStoreKey.toString() : undefined
                };
            };
            FindPrincipalsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
            };
            FindPrincipalsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    var principals = response.getResult().principals.map(function (principalJson) {
                        return _this.fromJsonToPrincipal(principalJson);
                    });
                    if (_this.filterPredicate) {
                        principals = principals.filter(_this.filterPredicate);
                    }
                    return principals;
                });
            };
            FindPrincipalsRequest.prototype.enumToStrings = function (types) {
                return types.map(function (type) {
                    return security.PrincipalType[type].toUpperCase();
                });
            };
            FindPrincipalsRequest.prototype.setUserStoreKey = function (key) {
                this.userStoreKey = key;
                return this;
            };
            FindPrincipalsRequest.prototype.setAllowedTypes = function (types) {
                this.allowedTypes = types;
                return this;
            };
            FindPrincipalsRequest.prototype.setSearchQuery = function (query) {
                this.searchQuery = query;
                return this;
            };
            FindPrincipalsRequest.prototype.setResultFilter = function (filterPredicate) {
                this.filterPredicate = filterPredicate;
            };
            return FindPrincipalsRequest;
        })(api.security.SecurityResourceRequest);
        security.FindPrincipalsRequest = FindPrincipalsRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var GetPrincipalByKeyRequest = (function (_super) {
            __extends(GetPrincipalByKeyRequest, _super);
            function GetPrincipalByKeyRequest(principalKey) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.principalKey = principalKey;
                this.includeMemberships = false;
            }
            GetPrincipalByKeyRequest.prototype.includeUserMemberships = function (includeMemberships) {
                this.includeMemberships = includeMemberships;
                return this;
            };
            GetPrincipalByKeyRequest.prototype.getParams = function () {
                return {
                    memberships: this.includeMemberships && this.principalKey.isUser()
                };
            };
            GetPrincipalByKeyRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', this.principalKey.toString());
            };
            GetPrincipalByKeyRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToPrincipal(response.getResult());
                });
            };
            return GetPrincipalByKeyRequest;
        })(security.SecurityResourceRequest);
        security.GetPrincipalByKeyRequest = GetPrincipalByKeyRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CheckEmailAvailabilityRequest = (function (_super) {
            __extends(CheckEmailAvailabilityRequest, _super);
            function CheckEmailAvailabilityRequest(email) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.email = email;
            }
            CheckEmailAvailabilityRequest.prototype.setUserStoreKey = function (key) {
                this.userStoreKey = key;
                return this;
            };
            CheckEmailAvailabilityRequest.prototype.getParams = function () {
                return {
                    email: this.email,
                    userStoreKey: this.userStoreKey ? this.userStoreKey.toString() : undefined
                };
            };
            CheckEmailAvailabilityRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'emailAvailable');
            };
            CheckEmailAvailabilityRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return response.getResult().available;
                });
            };
            return CheckEmailAvailabilityRequest;
        })(security.SecurityResourceRequest);
        security.CheckEmailAvailabilityRequest = CheckEmailAvailabilityRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var SetUserPasswordRequest = (function (_super) {
            __extends(SetUserPasswordRequest, _super);
            function SetUserPasswordRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            SetUserPasswordRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            SetUserPasswordRequest.prototype.setPassword = function (password) {
                this.password = password;
                return this;
            };
            SetUserPasswordRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    password: this.password
                };
            };
            SetUserPasswordRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'setPassword');
            };
            SetUserPasswordRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.User.fromJson(response.getResult());
                });
            };
            return SetUserPasswordRequest;
        })(security.SecurityResourceRequest);
        security.SetUserPasswordRequest = SetUserPasswordRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateUserRequest = (function (_super) {
            __extends(CreateUserRequest, _super);
            function CreateUserRequest() {
                _super.call(this);
                this.memberships = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateUserRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            CreateUserRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateUserRequest.prototype.setEmail = function (email) {
                this.email = email;
                return this;
            };
            CreateUserRequest.prototype.setLogin = function (login) {
                this.login = login;
                return this;
            };
            CreateUserRequest.prototype.setPassword = function (password) {
                this.password = password;
                return this;
            };
            CreateUserRequest.prototype.setMemberships = function (memberships) {
                this.memberships = memberships.slice(0);
                return this;
            };
            CreateUserRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    email: this.email,
                    login: this.login,
                    password: this.password,
                    memberships: this.memberships.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            CreateUserRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'createUser');
            };
            CreateUserRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.User.fromJson(response.getResult());
                });
            };
            return CreateUserRequest;
        })(security.SecurityResourceRequest);
        security.CreateUserRequest = CreateUserRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateGroupRequest = (function (_super) {
            __extends(CreateGroupRequest, _super);
            function CreateGroupRequest() {
                _super.call(this);
                this.members = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateGroupRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            CreateGroupRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateGroupRequest.prototype.setMembers = function (members) {
                this.members = members.slice(0);
                return this;
            };
            CreateGroupRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    members: this.members.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            CreateGroupRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'createGroup');
            };
            CreateGroupRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Group.fromJson(response.getResult());
                });
            };
            return CreateGroupRequest;
        })(security.SecurityResourceRequest);
        security.CreateGroupRequest = CreateGroupRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateRoleRequest = (function (_super) {
            __extends(CreateRoleRequest, _super);
            function CreateRoleRequest() {
                _super.call(this);
                this.members = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateRoleRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            CreateRoleRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateRoleRequest.prototype.setMembers = function (members) {
                this.members = members.slice(0);
                return this;
            };
            CreateRoleRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    members: this.members.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            CreateRoleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'createRole');
            };
            CreateRoleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Role.fromJson(response.getResult());
                });
            };
            return CreateRoleRequest;
        })(security.SecurityResourceRequest);
        security.CreateRoleRequest = CreateRoleRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateUserRequest = (function (_super) {
            __extends(UpdateUserRequest, _super);
            function UpdateUserRequest() {
                _super.call(this);
                this.membershipsToAdd = [];
                this.membershipsToRemove = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateUserRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UpdateUserRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateUserRequest.prototype.setEmail = function (email) {
                this.email = email;
                return this;
            };
            UpdateUserRequest.prototype.setLogin = function (login) {
                this.login = login;
                return this;
            };
            UpdateUserRequest.prototype.addMemberships = function (memberships) {
                this.membershipsToAdd = memberships.slice(0);
                return this;
            };
            UpdateUserRequest.prototype.removeMemberships = function (memberships) {
                this.membershipsToRemove = memberships.slice(0);
                return this;
            };
            UpdateUserRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    email: this.email,
                    login: this.login,
                    addMemberships: this.membershipsToAdd.map(function (memberKey) { return memberKey.toString(); }),
                    removeMemberships: this.membershipsToRemove.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            UpdateUserRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'updateUser');
            };
            UpdateUserRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.User.fromJson(response.getResult());
                });
            };
            return UpdateUserRequest;
        })(security.SecurityResourceRequest);
        security.UpdateUserRequest = UpdateUserRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateGroupRequest = (function (_super) {
            __extends(UpdateGroupRequest, _super);
            function UpdateGroupRequest() {
                _super.call(this);
                this.membersToAdd = [];
                this.membersToRemove = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateGroupRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UpdateGroupRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateGroupRequest.prototype.addMembers = function (members) {
                this.membersToAdd = members.slice(0);
                return this;
            };
            UpdateGroupRequest.prototype.removeMembers = function (members) {
                this.membersToRemove = members.slice(0);
                return this;
            };
            UpdateGroupRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    addMembers: this.membersToAdd.map(function (memberKey) { return memberKey.toString(); }),
                    removeMembers: this.membersToRemove.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            UpdateGroupRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'updateGroup');
            };
            UpdateGroupRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Group.fromJson(response.getResult());
                });
            };
            return UpdateGroupRequest;
        })(security.SecurityResourceRequest);
        security.UpdateGroupRequest = UpdateGroupRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateRoleRequest = (function (_super) {
            __extends(UpdateRoleRequest, _super);
            function UpdateRoleRequest() {
                _super.call(this);
                this.membersToAdd = [];
                this.membersToRemove = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateRoleRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UpdateRoleRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateRoleRequest.prototype.addMembers = function (members) {
                this.membersToAdd = members.slice(0);
                return this;
            };
            UpdateRoleRequest.prototype.removeMembers = function (members) {
                this.membersToRemove = members.slice(0);
                return this;
            };
            UpdateRoleRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    addMembers: this.membersToAdd.map(function (memberKey) { return memberKey.toString(); }),
                    removeMembers: this.membersToRemove.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            UpdateRoleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'updateRole');
            };
            UpdateRoleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Role.fromJson(response.getResult());
                });
            };
            return UpdateRoleRequest;
        })(security.SecurityResourceRequest);
        security.UpdateRoleRequest = UpdateRoleRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var DeletePrincipalResult = (function () {
            function DeletePrincipalResult() {
            }
            DeletePrincipalResult.prototype.getPrincipalKey = function () {
                return this.principalKey;
            };
            DeletePrincipalResult.prototype.isDeleted = function () {
                return this.deleted;
            };
            DeletePrincipalResult.prototype.getReason = function () {
                return this.reason;
            };
            DeletePrincipalResult.fromJson = function (json) {
                var result = new DeletePrincipalResult();
                result.principalKey = security.PrincipalKey.fromString(json.principalKey);
                result.deleted = json.deleted;
                result.reason = json.reason;
                return result;
            };
            return DeletePrincipalResult;
        })();
        security.DeletePrincipalResult = DeletePrincipalResult;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var DeletePrincipalRequest = (function (_super) {
            __extends(DeletePrincipalRequest, _super);
            function DeletePrincipalRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            DeletePrincipalRequest.prototype.setKeys = function (keys) {
                this.keys = keys.slice(0);
                return this;
            };
            DeletePrincipalRequest.prototype.getParams = function () {
                return {
                    keys: this.keys.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            DeletePrincipalRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'delete');
            };
            DeletePrincipalRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return response.getResult().results.map(function (resultJson) { return security.DeletePrincipalResult.fromJson(resultJson); });
                });
            };
            return DeletePrincipalRequest;
        })(security.SecurityResourceRequest);
        security.DeletePrincipalRequest = DeletePrincipalRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalLoader = (function (_super) {
            __extends(PrincipalLoader, _super);
            function PrincipalLoader() {
                this.findRequest = new security.FindPrincipalsRequest();
                this.skipPrincipalKeys = {};
                this.setAllowedTypes([1 /* GROUP */, 0 /* USER */, 2 /* ROLE */]);
                _super.call(this, this.findRequest);
            }
            PrincipalLoader.prototype.setUserStoreKey = function (key) {
                this.findRequest.setUserStoreKey(key);
                return this;
            };
            PrincipalLoader.prototype.setAllowedTypes = function (principalTypes) {
                this.findRequest.setAllowedTypes(principalTypes);
                return this;
            };
            PrincipalLoader.prototype.search = function (searchString) {
                this.findRequest.setSearchQuery(searchString);
                return this.load();
            };
            PrincipalLoader.prototype.skipPrincipals = function (principalKeys) {
                var _this = this;
                this.skipPrincipalKeys = {};
                principalKeys.forEach(function (principalKey) {
                    _this.skipPrincipalKeys[principalKey.toString()] = principalKey;
                });
                this.findRequest.setResultFilter(function (principal) { return !_this.skipPrincipalKeys[principal.getKey().toString()]; });
                return this;
            };
            return PrincipalLoader;
        })(api.util.loader.BaseLoader);
        security.PrincipalLoader = PrincipalLoader;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalNamedEvent = (function (_super) {
            __extends(PrincipalNamedEvent, _super);
            function PrincipalNamedEvent(wizard, principal) {
                _super.call(this);
                this.wizard = wizard;
                this.principal = principal;
            }
            PrincipalNamedEvent.prototype.getWizard = function () {
                return this.wizard;
            };
            PrincipalNamedEvent.prototype.getPrincipal = function () {
                return this.principal;
            };
            PrincipalNamedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            PrincipalNamedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return PrincipalNamedEvent;
        })(api.event.Event);
        security.PrincipalNamedEvent = PrincipalNamedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateUserStoreRequest = (function (_super) {
            __extends(CreateUserStoreRequest, _super);
            function CreateUserStoreRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateUserStoreRequest.prototype.getParams = function () {
                return {
                    key: this.userStoreKey.toString(),
                    displayName: this.displayName,
                    permissions: this.permissions ? this.permissions.toJson() : []
                };
            };
            CreateUserStoreRequest.prototype.setKey = function (userStoreKey) {
                this.userStoreKey = userStoreKey;
                return this;
            };
            CreateUserStoreRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateUserStoreRequest.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            CreateUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/create');
            };
            CreateUserStoreRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            CreateUserStoreRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return CreateUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.CreateUserStoreRequest = CreateUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateUserStoreRequest = (function (_super) {
            __extends(UpdateUserStoreRequest, _super);
            function UpdateUserStoreRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateUserStoreRequest.prototype.getParams = function () {
                return {
                    key: this.userStoreKey.toString(),
                    displayName: this.displayName,
                    permissions: this.permissions ? this.permissions.toJson() : undefined
                };
            };
            UpdateUserStoreRequest.prototype.setKey = function (userStoreKey) {
                this.userStoreKey = userStoreKey;
                return this;
            };
            UpdateUserStoreRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateUserStoreRequest.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            UpdateUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/update');
            };
            UpdateUserStoreRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            UpdateUserStoreRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return UpdateUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.UpdateUserStoreRequest = UpdateUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStoreNamedEvent = (function (_super) {
            __extends(UserStoreNamedEvent, _super);
            function UserStoreNamedEvent(wizard, userStore) {
                _super.call(this);
                this.wizard = wizard;
                this.userStore = userStore;
            }
            UserStoreNamedEvent.prototype.getWizard = function () {
                return this.wizard;
            };
            UserStoreNamedEvent.prototype.getUserStore = function () {
                return this.userStore;
            };
            UserStoreNamedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserStoreNamedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserStoreNamedEvent;
        })(api.event.Event);
        security.UserStoreNamedEvent = UserStoreNamedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserItemCreatedEvent = (function (_super) {
            __extends(UserItemCreatedEvent, _super);
            function UserItemCreatedEvent(principal, userStore, parentOfSameType) {
                _super.call(this);
                this.principal = principal;
                this.userStore = userStore;
                this.parentOfSameType = parentOfSameType;
            }
            UserItemCreatedEvent.prototype.getPrincipal = function () {
                return this.principal;
            };
            UserItemCreatedEvent.prototype.getUserStore = function () {
                return this.userStore;
            };
            UserItemCreatedEvent.prototype.isParentOfSameType = function () {
                return this.parentOfSameType;
            };
            UserItemCreatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserItemCreatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserItemCreatedEvent;
        })(api.event.Event);
        security.UserItemCreatedEvent = UserItemCreatedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserItemUpdatedEvent = (function (_super) {
            __extends(UserItemUpdatedEvent, _super);
            function UserItemUpdatedEvent(principal, userStore) {
                _super.call(this);
                this.principal = principal;
                this.userStore = userStore;
            }
            UserItemUpdatedEvent.prototype.getPrincipal = function () {
                return this.principal;
            };
            UserItemUpdatedEvent.prototype.getUserStore = function () {
                return this.userStore;
            };
            UserItemUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserItemUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserItemUpdatedEvent;
        })(api.event.Event);
        security.UserItemUpdatedEvent = UserItemUpdatedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserItemDeletedEvent = (function (_super) {
            __extends(UserItemDeletedEvent, _super);
            function UserItemDeletedEvent(principals) {
                _super.call(this);
                this.principals = principals;
            }
            UserItemDeletedEvent.prototype.getPrincipals = function () {
                return this.principals;
            };
            UserItemDeletedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserItemDeletedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserItemDeletedEvent;
        })(api.event.Event);
        security.UserItemDeletedEvent = UserItemDeletedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            (function (Permission) {
                Permission[Permission["READ"] = 0] = "READ";
                Permission[Permission["CREATE"] = 1] = "CREATE";
                Permission[Permission["MODIFY"] = 2] = "MODIFY";
                Permission[Permission["DELETE"] = 3] = "DELETE";
                Permission[Permission["PUBLISH"] = 4] = "PUBLISH";
                Permission[Permission["READ_PERMISSIONS"] = 5] = "READ_PERMISSIONS";
                Permission[Permission["WRITE_PERMISSIONS"] = 6] = "WRITE_PERMISSIONS";
            })(acl.Permission || (acl.Permission = {}));
            var Permission = acl.Permission;
            (function (PermissionState) {
                PermissionState[PermissionState["ALLOW"] = 0] = "ALLOW";
                PermissionState[PermissionState["DENY"] = 1] = "DENY";
                PermissionState[PermissionState["INHERIT"] = 2] = "INHERIT";
            })(acl.PermissionState || (acl.PermissionState = {}));
            var PermissionState = acl.PermissionState;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var ArrayHelper = api.util.ArrayHelper;
            var Principal = api.security.Principal;
            var AccessControlEntry = (function () {
                function AccessControlEntry(principal) {
                    this.principal = api.util.assertNotNull(principal);
                    this.allowedPermissions = [];
                    this.deniedPermissions = [];
                }
                AccessControlEntry.prototype.getPrincipal = function () {
                    return this.principal;
                };
                AccessControlEntry.prototype.getPrincipalKey = function () {
                    return this.principal.getKey();
                };
                AccessControlEntry.prototype.getPrincipalDisplayName = function () {
                    return this.principal.getDisplayName();
                };
                AccessControlEntry.prototype.getPrincipalTypeName = function () {
                    return this.principal.getTypeName();
                };
                AccessControlEntry.prototype.getAllowedPermissions = function () {
                    return this.allowedPermissions;
                };
                AccessControlEntry.prototype.getDeniedPermissions = function () {
                    return this.deniedPermissions;
                };
                AccessControlEntry.prototype.setAllowedPermissions = function (permissions) {
                    this.allowedPermissions = permissions;
                };
                AccessControlEntry.prototype.setDeniedPermissions = function (permissions) {
                    this.deniedPermissions = permissions;
                };
                AccessControlEntry.prototype.isAllowed = function (permission) {
                    return (this.allowedPermissions.indexOf(permission) > -1) && (this.deniedPermissions.indexOf(permission) === -1);
                };
                AccessControlEntry.prototype.isDenied = function (permission) {
                    return !this.isAllowed(permission);
                };
                AccessControlEntry.prototype.isSet = function (permission) {
                    return (this.allowedPermissions.indexOf(permission) > -1) || (this.deniedPermissions.indexOf(permission) > -1);
                };
                AccessControlEntry.prototype.allow = function (permission) {
                    ArrayHelper.addUnique(permission, this.allowedPermissions);
                    ArrayHelper.removeValue(permission, this.deniedPermissions);
                    return this;
                };
                AccessControlEntry.prototype.deny = function (permission) {
                    ArrayHelper.addUnique(permission, this.deniedPermissions);
                    ArrayHelper.removeValue(permission, this.allowedPermissions);
                    return this;
                };
                AccessControlEntry.prototype.remove = function (permission) {
                    ArrayHelper.removeValue(permission, this.allowedPermissions);
                    ArrayHelper.removeValue(permission, this.deniedPermissions);
                    return this;
                };
                AccessControlEntry.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, AccessControlEntry)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.getPrincipalKey(), other.getPrincipalKey())) {
                        return false;
                    }
                    if (!api.ObjectHelper.anyArrayEquals(this.allowedPermissions, other.allowedPermissions)) {
                        return false;
                    }
                    if (!api.ObjectHelper.anyArrayEquals(this.deniedPermissions, other.deniedPermissions)) {
                        return false;
                    }
                    return true;
                };
                AccessControlEntry.prototype.toString = function () {
                    var _this = this;
                    var values = '';
                    AccessControlEntry.ALL_PERMISSIONS.forEach(function (permission) {
                        if (_this.isSet(permission)) {
                            if (values !== '') {
                                values += ', ';
                            }
                            values += _this.isAllowed(permission) ? '+' : '-';
                            values += acl.Permission[permission].toUpperCase();
                        }
                    });
                    return this.getPrincipalKey().toString() + '[' + values + ']';
                };
                AccessControlEntry.prototype.clone = function () {
                    var ace = new AccessControlEntry(this.principal.clone());
                    ace.allowedPermissions = this.allowedPermissions.slice(0);
                    ace.deniedPermissions = this.deniedPermissions.slice(0);
                    return ace;
                };
                AccessControlEntry.prototype.toJson = function () {
                    return {
                        "principal": this.principal.toJson(),
                        "allow": this.allowedPermissions.map(function (perm) { return acl.Permission[perm]; }),
                        "deny": this.deniedPermissions.map(function (perm) { return acl.Permission[perm]; })
                    };
                };
                AccessControlEntry.fromJson = function (json) {
                    var ace = new AccessControlEntry(Principal.fromJson(json.principal));
                    var allow = json.allow.map(function (permStr) { return acl.Permission[permStr.toUpperCase()]; });
                    var deny = json.deny.map(function (permStr) { return acl.Permission[permStr.toUpperCase()]; });
                    ace.setAllowedPermissions(allow);
                    ace.setDeniedPermissions(deny);
                    return ace;
                };
                AccessControlEntry.ALL_PERMISSIONS = [0 /* READ */, 1 /* CREATE */, 2 /* MODIFY */, 3 /* DELETE */, 4 /* PUBLISH */, 5 /* READ_PERMISSIONS */, 6 /* WRITE_PERMISSIONS */];
                return AccessControlEntry;
            })();
            acl.AccessControlEntry = AccessControlEntry;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var PrincipalType = api.security.PrincipalType;
            var FindAccessControlEntriesRequest = (function (_super) {
                __extends(FindAccessControlEntriesRequest, _super);
                function FindAccessControlEntriesRequest() {
                    _super.call(this);
                }
                FindAccessControlEntriesRequest.prototype.getParams = function () {
                    return {
                        "types": this.enumToStrings(this.allowedTypes),
                        "query": this.searchQuery,
                        "userStoreKey": this.userStoreKey ? this.userStoreKey.toString() : undefined
                    };
                };
                FindAccessControlEntriesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
                };
                FindAccessControlEntriesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().principals.map(function (principalJson) {
                            return new acl.AccessControlEntry(_this.fromJsonToPrincipal(principalJson));
                        });
                    });
                };
                FindAccessControlEntriesRequest.prototype.enumToStrings = function (types) {
                    return types.map(function (type) {
                        return PrincipalType[type].toUpperCase();
                    });
                };
                FindAccessControlEntriesRequest.prototype.setUserStoreKey = function (key) {
                    this.userStoreKey = key;
                    return this;
                };
                FindAccessControlEntriesRequest.prototype.setAllowedTypes = function (types) {
                    this.allowedTypes = types;
                    return this;
                };
                FindAccessControlEntriesRequest.prototype.setSearchQuery = function (query) {
                    this.searchQuery = query;
                    return this;
                };
                return FindAccessControlEntriesRequest;
            })(api.security.SecurityResourceRequest);
            acl.FindAccessControlEntriesRequest = FindAccessControlEntriesRequest;
            var AccessControlEntryLoader = (function (_super) {
                __extends(AccessControlEntryLoader, _super);
                function AccessControlEntryLoader() {
                    this.findRequest = new FindAccessControlEntriesRequest();
                    this.setAllowedTypes([1 /* GROUP */, 0 /* USER */, 2 /* ROLE */]);
                    _super.call(this, this.findRequest);
                }
                AccessControlEntryLoader.prototype.setUserStoreKey = function (key) {
                    this.findRequest.setUserStoreKey(key);
                    return this;
                };
                AccessControlEntryLoader.prototype.setAllowedTypes = function (principalTypes) {
                    this.findRequest.setAllowedTypes(principalTypes);
                    return this;
                };
                AccessControlEntryLoader.prototype.search = function (searchString) {
                    this.findRequest.setSearchQuery(searchString);
                    return this.load();
                };
                return AccessControlEntryLoader;
            })(api.util.loader.BaseLoader);
            acl.AccessControlEntryLoader = AccessControlEntryLoader;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (_acl) {
            var AccessControlList = (function () {
                function AccessControlList(entries) {
                    this.entries = {};
                    if (entries) {
                        this.addAll(entries);
                    }
                }
                AccessControlList.prototype.getEntries = function () {
                    var values = [];
                    for (var key in this.entries) {
                        if (this.entries.hasOwnProperty(key)) {
                            values.push(this.entries[key]);
                        }
                    }
                    return values;
                };
                AccessControlList.prototype.getEntry = function (principalKey) {
                    return this.entries[principalKey.toString()];
                };
                AccessControlList.prototype.add = function (entry) {
                    this.entries[entry.getPrincipalKey().toString()] = entry;
                };
                AccessControlList.prototype.addAll = function (entries) {
                    var _this = this;
                    entries.forEach(function (entry) {
                        _this.entries[entry.getPrincipalKey().toString()] = entry;
                    });
                };
                AccessControlList.prototype.contains = function (principalKey) {
                    return this.entries.hasOwnProperty(principalKey.toString());
                };
                AccessControlList.prototype.remove = function (principalKey) {
                    delete this.entries[principalKey.toString()];
                };
                AccessControlList.prototype.toJson = function () {
                    var acl = [];
                    this.getEntries().forEach(function (entry) {
                        var entryJson = entry.toJson();
                        acl.push(entryJson);
                    });
                    return acl;
                };
                AccessControlList.prototype.toString = function () {
                    return '[' + this.getEntries().map(function (ace) { return ace.toString(); }).join(', ') + ']';
                };
                AccessControlList.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, AccessControlList)) {
                        return false;
                    }
                    var other = o;
                    return api.ObjectHelper.arrayEquals(this.getEntries().sort(), other.getEntries().sort());
                };
                AccessControlList.prototype.clone = function () {
                    var entries = [];
                    for (var key in this.entries) {
                        if (this.entries.hasOwnProperty(key)) {
                            entries.push(this.entries[key].clone());
                        }
                    }
                    return new AccessControlList(entries);
                };
                AccessControlList.fromJson = function (json) {
                    var acl = new AccessControlList();
                    json.forEach(function (entryJson) {
                        var entry = _acl.AccessControlEntry.fromJson(entryJson);
                        acl.add(entry);
                    });
                    return acl;
                };
                return AccessControlList;
            })();
            _acl.AccessControlList = AccessControlList;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            (function (UserStoreAccess) {
                UserStoreAccess[UserStoreAccess["READ"] = 0] = "READ";
                UserStoreAccess[UserStoreAccess["CREATE_USERS"] = 1] = "CREATE_USERS";
                UserStoreAccess[UserStoreAccess["WRITE_USERS"] = 2] = "WRITE_USERS";
                UserStoreAccess[UserStoreAccess["USER_STORE_MANAGER"] = 3] = "USER_STORE_MANAGER";
                UserStoreAccess[UserStoreAccess["ADMINISTRATOR"] = 4] = "ADMINISTRATOR";
            })(acl.UserStoreAccess || (acl.UserStoreAccess = {}));
            var UserStoreAccess = acl.UserStoreAccess;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var UserStoreAccessControlEntry = (function () {
                function UserStoreAccessControlEntry(principal, access) {
                    api.util.assertNotNull(principal, "principal not set");
                    this.principal = principal;
                    this.access = access;
                }
                UserStoreAccessControlEntry.prototype.getPrincipal = function () {
                    return this.principal;
                };
                UserStoreAccessControlEntry.prototype.getAccess = function () {
                    return this.access;
                };
                UserStoreAccessControlEntry.prototype.setAccess = function (value) {
                    this.access = acl.UserStoreAccess[value];
                    return this;
                };
                UserStoreAccessControlEntry.prototype.getPrincipalKey = function () {
                    return this.principal.getKey();
                };
                UserStoreAccessControlEntry.prototype.getPrincipalDisplayName = function () {
                    return this.principal.getDisplayName();
                };
                UserStoreAccessControlEntry.prototype.getPrincipalTypeName = function () {
                    return this.principal.getTypeName();
                };
                UserStoreAccessControlEntry.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStoreAccessControlEntry)) {
                        return false;
                    }
                    var other = o;
                    return this.principal.equals(other.getPrincipal()) && this.access === other.access;
                };
                UserStoreAccessControlEntry.prototype.getId = function () {
                    return this.principal.getKey().toString();
                };
                UserStoreAccessControlEntry.prototype.toString = function () {
                    return this.principal.getKey().toString() + '[' + acl.UserStoreAccess[this.access] + ']';
                };
                UserStoreAccessControlEntry.prototype.toJson = function () {
                    return {
                        "principal": this.principal.toJson(),
                        "access": acl.UserStoreAccess[this.access]
                    };
                };
                UserStoreAccessControlEntry.fromJson = function (json) {
                    return new UserStoreAccessControlEntry(security.Principal.fromJson(json.principal), acl.UserStoreAccess[json.access.toUpperCase()]);
                };
                return UserStoreAccessControlEntry;
            })();
            acl.UserStoreAccessControlEntry = UserStoreAccessControlEntry;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (_acl) {
            var UserStoreAccessControlList = (function () {
                function UserStoreAccessControlList(entries) {
                    this.entries = {};
                    if (entries) {
                        this.addAll(entries);
                    }
                }
                UserStoreAccessControlList.prototype.getEntries = function () {
                    var values = [];
                    for (var key in this.entries) {
                        if (this.entries.hasOwnProperty(key)) {
                            values.push(this.entries[key]);
                        }
                    }
                    return values;
                };
                UserStoreAccessControlList.prototype.getEntry = function (principalKey) {
                    return this.entries[principalKey.toString()];
                };
                UserStoreAccessControlList.prototype.add = function (entry) {
                    this.entries[entry.getPrincipal().getKey().toString()] = entry;
                };
                UserStoreAccessControlList.prototype.addAll = function (entries) {
                    var _this = this;
                    entries.forEach(function (entry) {
                        _this.entries[entry.getPrincipal().getKey().toString()] = entry;
                    });
                };
                UserStoreAccessControlList.prototype.contains = function (principalKey) {
                    return this.entries.hasOwnProperty(principalKey.toString());
                };
                UserStoreAccessControlList.prototype.remove = function (principalKey) {
                    delete this.entries[principalKey.toString()];
                };
                UserStoreAccessControlList.prototype.toJson = function () {
                    var acl = [];
                    this.getEntries().forEach(function (entry) {
                        var entryJson = entry.toJson();
                        acl.push(entryJson);
                    });
                    return acl;
                };
                UserStoreAccessControlList.prototype.toString = function () {
                    return '[' + this.getEntries().map(function (ace) { return ace.toString(); }).join(', ') + ']';
                };
                UserStoreAccessControlList.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStoreAccessControlList)) {
                        return false;
                    }
                    var other = o;
                    return api.ObjectHelper.arrayEquals(this.getEntries().sort(), other.getEntries().sort());
                };
                UserStoreAccessControlList.fromJson = function (json) {
                    var acl = new UserStoreAccessControlList();
                    json.forEach(function (entryJson) {
                        var entry = _acl.UserStoreAccessControlEntry.fromJson(entryJson);
                        acl.add(entry);
                    });
                    return acl;
                };
                return UserStoreAccessControlList;
            })();
            _acl.UserStoreAccessControlList = UserStoreAccessControlList;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var PrincipalType = api.security.PrincipalType;
            var FindUserStoreAccessControlEntriesRequest = (function (_super) {
                __extends(FindUserStoreAccessControlEntriesRequest, _super);
                function FindUserStoreAccessControlEntriesRequest() {
                    _super.call(this);
                }
                FindUserStoreAccessControlEntriesRequest.prototype.getParams = function () {
                    return {
                        "query": this.searchQuery
                    };
                };
                FindUserStoreAccessControlEntriesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
                };
                FindUserStoreAccessControlEntriesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().principals.map(function (principalJson) {
                            return new acl.UserStoreAccessControlEntry(_this.fromJsonToPrincipal(principalJson));
                        });
                    });
                };
                FindUserStoreAccessControlEntriesRequest.prototype.enumToStrings = function (types) {
                    return types.map(function (type) {
                        return PrincipalType[type].toUpperCase();
                    });
                };
                FindUserStoreAccessControlEntriesRequest.prototype.setUserStoreKey = function (key) {
                    this.userStoreKey = key;
                    return this;
                };
                FindUserStoreAccessControlEntriesRequest.prototype.setAllowedTypes = function (types) {
                    this.allowedTypes = types;
                    return this;
                };
                FindUserStoreAccessControlEntriesRequest.prototype.setSearchQuery = function (query) {
                    this.searchQuery = query;
                    return this;
                };
                return FindUserStoreAccessControlEntriesRequest;
            })(api.security.SecurityResourceRequest);
            acl.FindUserStoreAccessControlEntriesRequest = FindUserStoreAccessControlEntriesRequest;
            var UserStoreAccessControlEntryLoader = (function (_super) {
                __extends(UserStoreAccessControlEntryLoader, _super);
                function UserStoreAccessControlEntryLoader() {
                    this.findRequest = new FindUserStoreAccessControlEntriesRequest();
                    this.setAllowedTypes([1 /* GROUP */, 0 /* USER */, 2 /* ROLE */]);
                    _super.call(this, this.findRequest);
                }
                UserStoreAccessControlEntryLoader.prototype.setUserStoreKey = function (key) {
                    this.findRequest.setUserStoreKey(key);
                    return this;
                };
                UserStoreAccessControlEntryLoader.prototype.setAllowedTypes = function (principalTypes) {
                    this.findRequest.setAllowedTypes(principalTypes);
                    return this;
                };
                UserStoreAccessControlEntryLoader.prototype.search = function (searchString) {
                    this.findRequest.setSearchQuery(searchString);
                    return this.load();
                };
                return UserStoreAccessControlEntryLoader;
            })(api.util.loader.BaseLoader);
            acl.UserStoreAccessControlEntryLoader = UserStoreAccessControlEntryLoader;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LoginCredentials = (function () {
                function LoginCredentials() {
                    this.rememberMe = false;
                }
                LoginCredentials.prototype.getUser = function () {
                    return this.user;
                };
                LoginCredentials.prototype.getPassword = function () {
                    return this.password;
                };
                LoginCredentials.prototype.isRememberMe = function () {
                    return this.rememberMe;
                };
                LoginCredentials.prototype.setUser = function (user) {
                    this.user = user;
                    return this;
                };
                LoginCredentials.prototype.setPassword = function (pwd) {
                    this.password = pwd;
                    return this;
                };
                LoginCredentials.prototype.setRememberMe = function (value) {
                    this.rememberMe = value;
                    return this;
                };
                return LoginCredentials;
            })();
            auth.LoginCredentials = LoginCredentials;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LoginResult = (function () {
                function LoginResult(json) {
                    this.authenticated = json.authenticated;
                    if (json.user) {
                        this.user = api.security.User.fromJson(json.user);
                    }
                    this.applications = json.applications || [];
                    this.principals = json.principals ? json.principals.map(function (principal) { return api.security.PrincipalKey.fromString(principal); }) : [];
                    this.message = json.message;
                }
                LoginResult.prototype.isAuthenticated = function () {
                    return this.authenticated;
                };
                LoginResult.prototype.getUser = function () {
                    return this.user;
                };
                LoginResult.prototype.getApplications = function () {
                    return this.applications;
                };
                LoginResult.prototype.getPrincipals = function () {
                    return this.principals;
                };
                LoginResult.prototype.getMessage = function () {
                    return this.message;
                };
                return LoginResult;
            })();
            auth.LoginResult = LoginResult;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var AuthResourceRequest = (function (_super) {
                __extends(AuthResourceRequest, _super);
                function AuthResourceRequest() {
                    _super.call(this);
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "auth");
                }
                AuthResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                return AuthResourceRequest;
            })(api.rest.ResourceRequest);
            auth.AuthResourceRequest = AuthResourceRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LoginRequest = (function (_super) {
                __extends(LoginRequest, _super);
                function LoginRequest(loginCredentials) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                    this.loginCredentials = loginCredentials;
                }
                LoginRequest.prototype.getParams = function () {
                    return {
                        user: this.loginCredentials.getUser(),
                        password: this.loginCredentials.getPassword(),
                        rememberMe: this.loginCredentials.isRememberMe()
                    };
                };
                LoginRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'login');
                };
                LoginRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return new auth.LoginResult(response.getResult());
                    });
                };
                return LoginRequest;
            })(auth.AuthResourceRequest);
            auth.LoginRequest = LoginRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LogoutRequest = (function (_super) {
                __extends(LogoutRequest, _super);
                function LogoutRequest() {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                }
                LogoutRequest.prototype.getParams = function () {
                    return {};
                };
                LogoutRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'logout');
                };
                LogoutRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return;
                    });
                };
                return LogoutRequest;
            })(auth.AuthResourceRequest);
            auth.LogoutRequest = LogoutRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var IsAuthenticatedRequest = (function (_super) {
                __extends(IsAuthenticatedRequest, _super);
                function IsAuthenticatedRequest() {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                }
                IsAuthenticatedRequest.prototype.getParams = function () {
                    return {};
                };
                IsAuthenticatedRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'authenticated');
                };
                IsAuthenticatedRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return new auth.LoginResult(response.getResult());
                    });
                };
                return IsAuthenticatedRequest;
            })(auth.AuthResourceRequest);
            auth.IsAuthenticatedRequest = IsAuthenticatedRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var locale;
    (function (locale) {
        var Locale = (function () {
            function Locale() {
            }
            Locale.prototype.equals = function (other) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(other, Locale)) {
                    return false;
                }
                var o = other;
                return this.tag == o.tag && this.displayName == o.displayName && this.language == o.language && this.displayLanguage == o.displayLanguage && this.variant == o.variant && this.displayVariant == o.displayVariant && this.country == o.displayCountry && this.displayCountry == o.displayCountry;
            };
            Locale.fromJson = function (json) {
                var l = new Locale();
                l.tag = json.tag;
                l.displayName = json.displayName;
                l.country = json.country;
                l.displayCountry = json.displayCountry;
                l.variant = json.variant;
                l.displayVariant = json.displayVariant;
                l.language = json.language;
                l.displayLanguage = json.displayLanguage;
                return l;
            };
            Locale.prototype.getTag = function () {
                return this.tag;
            };
            Locale.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Locale.prototype.getLanguage = function () {
                return this.language;
            };
            Locale.prototype.getDisplayLanguage = function () {
                return this.displayLanguage;
            };
            Locale.prototype.getVariant = function () {
                return this.variant;
            };
            Locale.prototype.getDisplayVariant = function () {
                return this.displayVariant;
            };
            Locale.prototype.getCountry = function () {
                return this.country;
            };
            Locale.prototype.getDisplayCountry = function () {
                return this.displayCountry;
            };
            return Locale;
        })();
        locale.Locale = Locale;
    })(locale = api.locale || (api.locale = {}));
})(api || (api = {}));
var api;
(function (api) {
    var locale;
    (function (locale) {
        var GetLocalesRequest = (function (_super) {
            __extends(GetLocalesRequest, _super);
            function GetLocalesRequest() {
                _super.call(this);
            }
            GetLocalesRequest.prototype.getParams = function () {
                return {
                    "query": this.searchQuery
                };
            };
            GetLocalesRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), 'content', 'locales');
            };
            GetLocalesRequest.prototype.setSearchQuery = function (query) {
                this.searchQuery = query;
                return this;
            };
            GetLocalesRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.getResult().locales.map(function (localeJson) {
                        return locale.Locale.fromJson(localeJson);
                    }).sort(_this.sortFunction);
                });
            };
            GetLocalesRequest.prototype.sortFunction = function (a, b) {
                return a.getDisplayName().localeCompare(b.getDisplayName());
            };
            return GetLocalesRequest;
        })(api.rest.ResourceRequest);
        locale.GetLocalesRequest = GetLocalesRequest;
    })(locale = api.locale || (api.locale = {}));
})(api || (api = {}));
var api;
(function (api) {
    var locale;
    (function (locale) {
        var LocaleLoader = (function (_super) {
            __extends(LocaleLoader, _super);
            function LocaleLoader() {
                this.getLocalesRequest = new locale.GetLocalesRequest();
                _super.call(this, this.getLocalesRequest);
            }
            LocaleLoader.prototype.search = function (searchString) {
                this.getLocalesRequest.setSearchQuery(searchString);
                return this.load();
            };
            LocaleLoader.prototype.load = function () {
                var _this = this;
                this.notifyLoadingData();
                return this.sendRequest().then(function (locales) {
                    _this.notifyLoadedData(locales);
                    if (_this.preservedSearchString) {
                        _this.search(_this.preservedSearchString);
                        _this.preservedSearchString = null;
                    }
                    return locales;
                });
            };
            return LocaleLoader;
        })(api.util.loader.BaseLoader);
        locale.LocaleLoader = LocaleLoader;
    })(locale = api.locale || (api.locale = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveRange = (function () {
                function ResponsiveRange(minRange, maxRange, rangeClass) {
                    this.minRange = minRange;
                    this.maxRange = maxRange || 0;
                    this.rangeClass = rangeClass || ("_" + minRange + "-" + maxRange);
                }
                ResponsiveRange.prototype.getMinimumRange = function () {
                    return this.minRange;
                };
                ResponsiveRange.prototype.getMaximumRange = function () {
                    return this.maxRange;
                };
                ResponsiveRange.prototype.getRangeClass = function () {
                    return this.rangeClass;
                };
                ResponsiveRange.prototype.isFit = function (size) {
                    return (this.minRange <= size) && (size <= this.maxRange);
                };
                ResponsiveRange.prototype.isFitOrSmaller = function (size) {
                    return size <= this.maxRange;
                };
                ResponsiveRange.prototype.isFitOrBigger = function (size) {
                    return size > this.minRange;
                };
                return ResponsiveRange;
            })();
            responsive.ResponsiveRange = ResponsiveRange;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveRanges = (function () {
                function ResponsiveRanges() {
                }
                ResponsiveRanges._0_240 = new responsive.ResponsiveRange(0, 240);
                ResponsiveRanges._240_360 = new responsive.ResponsiveRange(240, 360);
                ResponsiveRanges._360_540 = new responsive.ResponsiveRange(360, 540);
                ResponsiveRanges._540_720 = new responsive.ResponsiveRange(540, 720);
                ResponsiveRanges._720_960 = new responsive.ResponsiveRange(720, 960);
                ResponsiveRanges._960_1200 = new responsive.ResponsiveRange(960, 1200);
                ResponsiveRanges._1200_1380 = new responsive.ResponsiveRange(1200, 1380);
                ResponsiveRanges._1380_1620 = new responsive.ResponsiveRange(1380, 1620);
                ResponsiveRanges._1620_1920 = new responsive.ResponsiveRange(1620, 1920);
                ResponsiveRanges._1920_UP = new responsive.ResponsiveRange(1920, Infinity);
                return ResponsiveRanges;
            })();
            responsive.ResponsiveRanges = ResponsiveRanges;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveItem = (function () {
                function ResponsiveItem(element, handler) {
                    if (handler === void 0) { handler = (function (item) {
                    }); }
                    this.element = element;
                    this.rangeValue = this.element.getEl().getWidthWithBorder();
                    this.oldRangeValue = this.rangeValue;
                    this.handle = handler;
                    this.fitToRange();
                    this.element.getEl().addClass(this.rangeSize.getRangeClass());
                }
                ResponsiveItem.prototype.fitToRange = function () {
                    for (var key in responsive.ResponsiveRanges) {
                        var range = responsive.ResponsiveRanges[key];
                        if (range && (api.ObjectHelper.iFrameSafeInstanceOf(range, responsive.ResponsiveRange)) && range.isFit(this.rangeValue)) {
                            this.rangeSize = range;
                            break;
                        }
                    }
                };
                ResponsiveItem.prototype.getElement = function () {
                    return this.element;
                };
                ResponsiveItem.prototype.update = function () {
                    var newRangeValue = this.element.getEl().getWidthWithBorder();
                    if (newRangeValue !== this.rangeValue) {
                        this.oldRangeValue = this.rangeValue;
                        this.oldRangeSize = this.rangeSize;
                        this.rangeValue = newRangeValue;
                        this.element.getEl().removeClass(this.rangeSize.getRangeClass());
                        this.fitToRange();
                        this.element.getEl().addClass(this.rangeSize.getRangeClass());
                    }
                    this.handle(this);
                };
                ResponsiveItem.prototype.isRangeSizeChanged = function () {
                    return this.rangeSize !== this.oldRangeSize;
                };
                ResponsiveItem.prototype.setHandler = function (handler) {
                    if (handler === void 0) { handler = (function (item) {
                    }); }
                    this.handle = handler;
                };
                ResponsiveItem.prototype.getRangeValue = function () {
                    return this.rangeValue;
                };
                ResponsiveItem.prototype.getOldRangeValue = function () {
                    return this.oldRangeValue;
                };
                ResponsiveItem.prototype.getRangeSize = function () {
                    return this.rangeSize;
                };
                ResponsiveItem.prototype.getOldRangeSize = function () {
                    return this.oldRangeSize;
                };
                ResponsiveItem.prototype.isInRange = function (range) {
                    return range.isFit(this.rangeValue);
                };
                ResponsiveItem.prototype.isInRangeOrSmaller = function (range) {
                    return range.isFitOrSmaller(this.rangeValue);
                };
                ResponsiveItem.prototype.isInRangeOrBigger = function (range) {
                    return range.isFitOrBigger(this.rangeValue);
                };
                return ResponsiveItem;
            })();
            responsive.ResponsiveItem = ResponsiveItem;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveListener = (function () {
                function ResponsiveListener(item, listener) {
                    this.item = item;
                    this.listener = listener;
                }
                ResponsiveListener.prototype.getItem = function () {
                    return this.item;
                };
                ResponsiveListener.prototype.getListener = function () {
                    return this.listener;
                };
                return ResponsiveListener;
            })();
            responsive.ResponsiveListener = ResponsiveListener;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveManager = (function () {
                function ResponsiveManager() {
                }
                ResponsiveManager.onAvailableSizeChanged = function (el, handler) {
                    if (handler === void 0) { handler = function (item) {
                    }; }
                    var responsiveItem = new responsive.ResponsiveItem(el, handler), listener = function () {
                        responsiveItem.update();
                    }, responsiveListener = new responsive.ResponsiveListener(responsiveItem, listener);
                    ResponsiveManager.responsiveListeners.push(responsiveListener);
                    ResponsiveManager.window.getHTMLElement().addEventListener('availablesizechange', listener);
                    ResponsiveManager.window.onResized(listener);
                    return responsiveItem;
                };
                ResponsiveManager.unAvailableSizeChanged = function (el) {
                    ResponsiveManager.responsiveListeners = ResponsiveManager.responsiveListeners.filter(function (curr) {
                        if (curr.getItem().getElement() === el) {
                            ResponsiveManager.window.getHTMLElement().removeEventListener('availablesizechange', curr.getListener());
                            ResponsiveManager.window.unResized(curr.getListener());
                            return false;
                        }
                        else {
                            return true;
                        }
                    });
                };
                ResponsiveManager.fireResizeEvent = function () {
                    var customEvent = document.createEvent('Event');
                    customEvent.initEvent('availablesizechange', false, true);
                    ResponsiveManager.window.getHTMLElement().dispatchEvent(customEvent);
                };
                ResponsiveManager.getWindow = function () {
                    return ResponsiveManager.window;
                };
                ResponsiveManager.window = api.dom.WindowDOM.get();
                ResponsiveManager.responsiveListeners = [];
                return ResponsiveManager;
            })();
            responsive.ResponsiveManager = ResponsiveManager;
            ResponsiveManager.onAvailableSizeChanged(api.dom.Body.get());
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ValueChangedEvent = (function () {
            function ValueChangedEvent(oldValue, newValue) {
                this.oldValue = oldValue;
                this.newValue = newValue;
            }
            ValueChangedEvent.prototype.getOldValue = function () {
                return this.oldValue;
            };
            ValueChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            return ValueChangedEvent;
        })();
        ui.ValueChangedEvent = ValueChangedEvent;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var KeyHelper = (function () {
            function KeyHelper() {
            }
            KeyHelper.isNumber = function (event) {
                return event.keyCode >= 48 && event.keyCode <= 57;
            };
            KeyHelper.isDash = function (event) {
                return event.keyCode == 189;
            };
            KeyHelper.isDel = function (event) {
                return event.keyCode == 46;
            };
            KeyHelper.isBackspace = function (event) {
                return event.keyCode == 8;
            };
            KeyHelper.isColon = function (event) {
                return event.keyCode == 186;
            };
            KeyHelper.isComma = function (event) {
                return event.keyCode == 188;
            };
            KeyHelper.isDot = function (event) {
                return event.keyCode == 190;
            };
            return KeyHelper;
        })();
        ui.KeyHelper = KeyHelper;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        (function (KeyBindingAction) {
            KeyBindingAction[KeyBindingAction["KEYDOWN"] = 0] = "KEYDOWN";
            KeyBindingAction[KeyBindingAction["KEYUP"] = 1] = "KEYUP";
            KeyBindingAction[KeyBindingAction["KEYPRESS"] = 2] = "KEYPRESS";
        })(ui.KeyBindingAction || (ui.KeyBindingAction = {}));
        var KeyBindingAction = ui.KeyBindingAction;
        var KeyBinding = (function () {
            function KeyBinding(combination, callback, action, global) {
                this.combination = combination;
                this.callback = callback;
                this.action = action;
                this.global = global;
            }
            KeyBinding.prototype.setCallback = function (func) {
                this.callback = func;
                return this;
            };
            KeyBinding.prototype.setAction = function (value) {
                this.action = value;
                return this;
            };
            KeyBinding.prototype.setGlobal = function (global) {
                this.global = global;
                return this;
            };
            KeyBinding.prototype.getCombination = function () {
                return this.combination;
            };
            KeyBinding.prototype.getCallback = function () {
                return this.callback;
            };
            KeyBinding.prototype.getAction = function () {
                return this.action;
            };
            KeyBinding.prototype.isGlobal = function () {
                return this.global;
            };
            KeyBinding.newKeyBinding = function (combination) {
                return new KeyBinding(combination);
            };
            KeyBinding.createMultiple = function (callback) {
                var combinations = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    combinations[_i - 1] = arguments[_i];
                }
                var bindings = [];
                combinations.forEach(function (combination) {
                    bindings.push(new KeyBinding(combination, callback));
                });
                return bindings;
            };
            return KeyBinding;
        })();
        ui.KeyBinding = KeyBinding;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var KeyBindings = (function () {
            function KeyBindings() {
                this.activeBindings = {};
                this.shelves = [];
                this.helpKeyPressedListeners = [];
                KeyBindings.instanceCount++;
                this.instance = KeyBindings.instanceCount;
                if (KeyBindings.debug) {
                    console.log("KeyBindings constructed instance #" + this.instance);
                }
                this.initializeHelpKey();
            }
            KeyBindings.get = function () {
                return KeyBindings.INSTANCE;
            };
            KeyBindings.prototype.bindKeys = function (bindings) {
                var _this = this;
                var logMessage = "Binded keys: [";
                bindings.forEach(function (binding) {
                    _this.bindKey(binding);
                    logMessage += "'" + binding.getCombination() + "' ,";
                });
                logMessage += "]";
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].bindKeys(): " + logMessage);
                }
            };
            KeyBindings.prototype.bindKey = function (binding) {
                if (binding.isGlobal()) {
                    Mousetrap.bindGlobal(binding.getCombination(), binding.getCallback(), binding.getAction() ? ui.KeyBindingAction[binding.getAction()].toLowerCase() : "");
                }
                else {
                    Mousetrap.bind(binding.getCombination(), binding.getCallback(), binding.getAction() ? ui.KeyBindingAction[binding.getAction()].toLowerCase() : "");
                }
                var bindingKey = binding.getAction() ? binding.getCombination() + "-" + binding.getAction() : binding.getCombination();
                this.activeBindings[bindingKey] = binding;
            };
            KeyBindings.prototype.unbindKeys = function (bindings) {
                var _this = this;
                var logMessage = "Binded keys: [";
                bindings.forEach(function (binding) {
                    _this.unbindKey(binding);
                    logMessage += "'" + binding.getCombination() + "' ,";
                });
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].unbindKeys(): " + logMessage);
                }
            };
            KeyBindings.prototype.unbindKey = function (binding) {
                Mousetrap.unbind(binding.getCombination());
                delete this.activeBindings[binding.getCombination()];
            };
            KeyBindings.prototype.trigger = function (combination, action) {
                Mousetrap.trigger(combination, action);
            };
            KeyBindings.prototype.reset = function () {
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].reset()");
                }
                Mousetrap.reset();
                this.activeBindings = {};
                this.shelves = [];
            };
            KeyBindings.prototype.getActiveBindings = function () {
                var _this = this;
                return Object.keys(this.activeBindings).map(function (key) {
                    return _this.activeBindings[key];
                });
            };
            KeyBindings.prototype.shelveBindings = function () {
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].shelveBindings(): ");
                }
                Mousetrap.reset();
                this.shelves.push(this.activeBindings);
                this.activeBindings = {};
                this.notifyHelpKeyPressed(null);
            };
            KeyBindings.prototype.unshelveBindings = function () {
                var previousMousetraps = this.shelves.pop();
                if (previousMousetraps == undefined) {
                    if (KeyBindings.debug) {
                        console.log("KeyBindings[#" + this.instance + "].unshelveBindings(): nothing to unshelve");
                    }
                    return;
                }
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].unshelveBindings(): unshelving... ");
                }
                this.activeBindings = {};
                Mousetrap.reset();
                for (var key in previousMousetraps) {
                    var mousetrap = previousMousetraps[key];
                    this.bindKey(mousetrap);
                }
            };
            KeyBindings.prototype.isActive = function (keyBinding) {
                for (var key in this.activeBindings) {
                    if (this.activeBindings[key] == keyBinding) {
                        return true;
                    }
                }
                return false;
            };
            KeyBindings.prototype.initializeHelpKey = function () {
                var _this = this;
                this.bindKey(new api.ui.KeyBinding("f2", function (e) {
                    _this.notifyHelpKeyPressed(e);
                }).setGlobal(false).setAction(0 /* KEYDOWN */));
                this.bindKey(new api.ui.KeyBinding("f2", function (e) {
                    _this.notifyHelpKeyPressed(e);
                }).setGlobal(false).setAction(1 /* KEYUP */));
            };
            KeyBindings.prototype.onHelpKeyPressed = function (listener) {
                this.helpKeyPressedListeners.push(listener);
            };
            KeyBindings.prototype.unHelpKeyPressed = function (listener) {
                this.helpKeyPressedListeners = this.helpKeyPressedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            KeyBindings.prototype.notifyHelpKeyPressed = function (e) {
                var _this = this;
                this.helpKeyPressedListeners.forEach(function (listener) {
                    listener.call(_this, e);
                });
            };
            KeyBindings.instanceCount = 0;
            KeyBindings.INSTANCE = new KeyBindings();
            KeyBindings.debug = false;
            return KeyBindings;
        })();
        ui.KeyBindings = KeyBindings;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Mnemonic = (function () {
            function Mnemonic(value) {
                this.value = value;
            }
            Mnemonic.prototype.getValue = function () {
                return this.value;
            };
            Mnemonic.prototype.toKeyBinding = function (callback) {
                return new ui.KeyBinding("alt+" + this.getValue(), callback);
            };
            Mnemonic.prototype.underlineMnemonic = function (text) {
                var mStart = text.indexOf(this.value);
                if (mStart == -1) {
                    mStart = text.indexOf(this.value.toLowerCase());
                    if (mStart == -1) {
                        mStart = text.indexOf(this.value.toUpperCase());
                    }
                }
                var result = "";
                if (mStart > 0) {
                    result = text.substr(0, mStart);
                }
                result += "<u>" + text.charAt(mStart) + "</u>";
                if (mStart < text.length - 1) {
                    result += text.substr(mStart + 1, text.length);
                }
                return result;
            };
            return Mnemonic;
        })();
        ui.Mnemonic = Mnemonic;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        (function (IconSize) {
            IconSize[IconSize["SMALL"] = 0] = "SMALL";
            IconSize[IconSize["MEDIUM"] = 1] = "MEDIUM";
            IconSize[IconSize["LARGE"] = 2] = "LARGE";
        })(ui.IconSize || (ui.IconSize = {}));
        var IconSize = ui.IconSize;
        var Icon = (function (_super) {
            __extends(Icon, _super);
            function Icon(iconClass, size) {
                _super.call(this, iconClass);
                this.addClass("wem-icon");
                if (size == 0 /* SMALL */) {
                    this.addClass("icon-small");
                }
                else if (size == 1 /* MEDIUM */) {
                    this.addClass("icon-medium");
                }
                else if (size == 2 /* LARGE */) {
                    this.addClass("icon-large");
                }
            }
            return Icon;
        })(api.dom.IEl);
        ui.Icon = Icon;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Action = (function () {
            function Action(label, shortcut, global) {
                var _this = this;
                this.enabled = true;
                this.visible = true;
                this.executionListeners = [];
                this.propertyChangedListeners = [];
                this.label = label;
                if (shortcut) {
                    this.shortcut = new ui.KeyBinding(shortcut).setGlobal(global).setCallback(function (e, combo) {
                        if (e.preventDefault) {
                            e.preventDefault();
                        }
                        else {
                            e.returnValue = false;
                        }
                        _this.execute();
                        return false;
                    });
                }
            }
            Action.prototype.getLabel = function () {
                return this.label;
            };
            Action.prototype.setLabel = function (value) {
                if (value !== this.label) {
                    this.label = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.isEnabled = function () {
                return this.enabled;
            };
            Action.prototype.setEnabled = function (value) {
                if (value !== this.enabled) {
                    this.enabled = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.isVisible = function () {
                return this.visible;
            };
            Action.prototype.setVisible = function (value) {
                if (value !== this.visible) {
                    this.visible = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.getIconClass = function () {
                return this.iconClass;
            };
            Action.prototype.setIconClass = function (value) {
                if (value !== this.iconClass) {
                    this.iconClass = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.notifyPropertyChanged = function () {
                for (var i in this.propertyChangedListeners) {
                    this.propertyChangedListeners[i](this);
                }
            };
            Action.prototype.hasShortcut = function () {
                return this.shortcut != null;
            };
            Action.prototype.getShortcut = function () {
                return this.shortcut;
            };
            Action.prototype.setMnemonic = function (value) {
                this.mnemonic = new ui.Mnemonic(value);
            };
            Action.prototype.hasMnemonic = function () {
                return this.mnemonic != null;
            };
            Action.prototype.getMnemonic = function () {
                return this.mnemonic;
            };
            Action.prototype.execute = function () {
                if (this.enabled) {
                    for (var i in this.executionListeners) {
                        this.executionListeners[i](this);
                    }
                }
            };
            Action.prototype.onExecuted = function (listener) {
                this.executionListeners.push(listener);
                return this;
            };
            Action.prototype.unExecuted = function (listener) {
                this.executionListeners = this.executionListeners.filter(function (curr) {
                    return curr != listener;
                });
                return this;
            };
            Action.prototype.onPropertyChanged = function (listener) {
                this.propertyChangedListeners.push(listener);
            };
            Action.prototype.getKeyBindings = function () {
                var _this = this;
                var bindings = [];
                if (this.hasShortcut()) {
                    bindings.push(this.getShortcut());
                }
                if (this.hasMnemonic()) {
                    bindings.push(this.getMnemonic().toKeyBinding(function () {
                        _this.execute();
                    }));
                }
                return bindings;
            };
            Action.getKeyBindings = function (actions) {
                var bindings = [];
                actions.forEach(function (action) {
                    action.getKeyBindings().forEach(function (keyBinding) {
                        bindings.push(keyBinding);
                    });
                });
                return bindings;
            };
            return Action;
        })();
        ui.Action = Action;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Viewer = (function (_super) {
            __extends(Viewer, _super);
            function Viewer(className) {
                _super.call(this, new api.dom.NewElementBuilder().setTagName("div").setClassName('viewer ' + (className || '')).setGenerateId(false));
            }
            Viewer.prototype.setObject = function (object) {
                this.object = object;
            };
            Viewer.prototype.getObject = function () {
                return this.object;
            };
            Viewer.prototype.getPreferredHeight = function () {
                throw new Error("Must be implemented by inheritors");
            };
            return Viewer;
        })(api.dom.Element);
        ui.Viewer = Viewer;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Tooltip = (function () {
            function Tooltip(target, text, showDelay, hideTimeout) {
                var _this = this;
                if (showDelay === void 0) { showDelay = 0; }
                if (hideTimeout === void 0) { hideTimeout = 0; }
                this.targetEl = target;
                this.text = text;
                this.showDelay = showDelay;
                this.hideTimeout = hideTimeout;
                this.side = Tooltip.SIDE_BOTTOM;
                this.overListener = function (event) {
                    _this.startShowDelay();
                };
                this.outListener = function (event) {
                    _this.startHideTimeout();
                };
                this.moveListener = function (event) {
                    if (_this.tooltipEl && _this.tooltipEl.isVisible()) {
                        _this.positionAtMouse(event);
                    }
                };
                this.setTrigger(Tooltip.TRIGGER_HOVER);
                this.setMode(Tooltip.MODE_STATIC);
                Tooltip.instances.push(this);
            }
            Tooltip.prototype.show = function () {
                this.stopTimeout();
                if (!this.tooltipEl) {
                    this.tooltipEl = new api.dom.DivEl("tooltip " + this.side);
                    if (this.contentEl) {
                        this.tooltipEl.appendChild(this.contentEl);
                    }
                    else {
                        this.tooltipEl.getEl().setInnerHtml(this.text);
                    }
                    var appendTo;
                    if (this.mode == Tooltip.MODE_STATIC) {
                        appendTo = this.targetEl.getParentElement() || this.targetEl;
                    }
                    else {
                        appendTo = api.dom.Body.get();
                    }
                    appendTo.appendChild(this.tooltipEl);
                    if (!Tooltip.multipleAllowed) {
                        this.hideOtherInstances();
                    }
                    this.tooltipEl.show();
                    if (this.mode == Tooltip.MODE_STATIC) {
                        this.positionByTarget();
                    }
                }
            };
            Tooltip.prototype.hide = function () {
                this.stopTimeout();
                if (this.tooltipEl) {
                    this.tooltipEl.remove();
                    this.tooltipEl = null;
                }
            };
            Tooltip.prototype.isVisible = function () {
                return this.tooltipEl && this.tooltipEl.isVisible();
            };
            Tooltip.prototype.showAfter = function (ms) {
                this.startShowDelay(ms);
                return this;
            };
            Tooltip.prototype.showFor = function (ms) {
                this.show();
                this.startHideTimeout(ms);
                return this;
            };
            Tooltip.prototype.setText = function (text) {
                this.text = text;
                this.contentEl = undefined;
                return this;
            };
            Tooltip.prototype.getText = function () {
                return this.text;
            };
            Tooltip.prototype.setContent = function (content) {
                this.contentEl = content;
                this.text = undefined;
                return this;
            };
            Tooltip.prototype.getContent = function () {
                return this.contentEl;
            };
            Tooltip.prototype.setHideTimeout = function (timeout) {
                this.hideTimeout = timeout;
                return this;
            };
            Tooltip.prototype.getHideTimeout = function () {
                return this.hideTimeout;
            };
            Tooltip.prototype.setShowDelay = function (delay) {
                this.showDelay = delay;
                return this;
            };
            Tooltip.prototype.getShowDelay = function () {
                return this.showDelay;
            };
            Tooltip.prototype.setTrigger = function (trigger) {
                if (trigger == this.trigger) {
                    return this;
                }
                this.targetEl.getEl().removeEventListener(this.getEventName(true), this.overListener).removeEventListener(this.getEventName(false), this.outListener);
                this.trigger = trigger;
                if (trigger != Tooltip.TRIGGER_NONE) {
                    this.targetEl.getEl().addEventListener(this.getEventName(true), this.overListener).addEventListener(this.getEventName(false), this.outListener);
                }
                return this;
            };
            Tooltip.prototype.getTrigger = function () {
                return this.trigger;
            };
            Tooltip.prototype.setSide = function (side) {
                this.side = side;
                return this;
            };
            Tooltip.prototype.getSide = function () {
                return this.side;
            };
            Tooltip.prototype.setMode = function (mode) {
                if (mode == this.mode) {
                    return this;
                }
                else if (mode == Tooltip.MODE_STATIC) {
                    api.dom.Body.get().unMouseMove(this.moveListener);
                }
                else if (mode == Tooltip.MODE_FOLLOW) {
                    api.dom.Body.get().onMouseMove(this.moveListener);
                }
                this.mode = mode;
                return this;
            };
            Tooltip.prototype.getMode = function () {
                return this.mode;
            };
            Tooltip.prototype.positionAtMouse = function (event) {
                var left, top, x = event.clientX, y = event.clientY, el = this.tooltipEl.getEl(), windowEl = api.dom.WindowDOM.get().getHTMLElement(), elProps = {
                    height: el.getHeightWithMargin(),
                    width: el.getWidthWithMargin(),
                    scrollLeft: this.mode == Tooltip.MODE_FOLLOW ? windowEl.scrollX : 0,
                    scrollTop: this.mode == Tooltip.MODE_FOLLOW ? windowEl.scrollY : 0
                };
                switch (this.side) {
                    case Tooltip.SIDE_TOP:
                        left = x - elProps.width / 2 + elProps.scrollLeft;
                        top = y - elProps.height + elProps.scrollTop;
                        break;
                    case Tooltip.SIDE_BOTTOM:
                        left = x - elProps.width / 2 + elProps.scrollLeft;
                        top = y + elProps.scrollTop;
                        break;
                    case Tooltip.SIDE_LEFT:
                        left = x - elProps.width + elProps.scrollLeft;
                        top = y - elProps.height / 2 + elProps.scrollTop;
                        break;
                    case Tooltip.SIDE_RIGHT:
                        left = x + elProps.scrollLeft;
                        top = y - elProps.height / 2 + elProps.scrollTop;
                        break;
                }
                this.tooltipEl.getEl().setLeftPx(left);
                this.tooltipEl.getEl().setTopPx(top);
            };
            Tooltip.prototype.positionByTarget = function () {
                var targetEl = this.targetEl.getHTMLElement(), targetOffset = this.targetEl.getEl().getOffset(), el = this.tooltipEl.getEl(), elProps = {
                    left: el.getMarginLeft() || 0,
                    top: el.getMarginTop() || 0,
                    height: el.getHeight(),
                    width: el.getWidth()
                };
                var offsetLeft, offsetTop;
                switch (this.side) {
                    case Tooltip.SIDE_TOP:
                        offsetLeft = targetOffset.left + (targetEl.offsetWidth - elProps.width) / 2 + elProps.left;
                        offsetTop = targetOffset.top - elProps.height + elProps.top;
                        break;
                    case Tooltip.SIDE_BOTTOM:
                        offsetLeft = targetOffset.left + (targetEl.offsetWidth - elProps.width) / 2 + elProps.left;
                        offsetTop = targetOffset.top + targetEl.offsetHeight + elProps.top;
                        break;
                    case Tooltip.SIDE_LEFT:
                        offsetLeft = targetOffset.left - elProps.width + elProps.left;
                        offsetTop = targetOffset.top + (targetEl.offsetHeight - elProps.height) / 2 + elProps.top;
                        break;
                    case Tooltip.SIDE_RIGHT:
                        offsetLeft = targetOffset.left + targetEl.offsetWidth + elProps.left;
                        offsetTop = targetOffset.top + (targetEl.offsetHeight - elProps.height) / 2 + elProps.top;
                        break;
                }
                if (offsetLeft < 0) {
                    offsetLeft = 0;
                }
                if (offsetTop < 0) {
                    offsetTop = 0;
                }
                el.setOffset({
                    left: offsetLeft,
                    top: offsetTop
                });
            };
            Tooltip.prototype.startHideTimeout = function (ms) {
                var _this = this;
                this.stopTimeout();
                var t = ms || this.hideTimeout;
                if (t > 0) {
                    this.timeoutTimer = setTimeout(function () {
                        _this.hide();
                    }, t);
                }
                else {
                    this.hide();
                }
            };
            Tooltip.prototype.startShowDelay = function (ms) {
                var _this = this;
                this.stopTimeout();
                var t = ms || this.showDelay;
                if (t > 0) {
                    if (this.trigger == Tooltip.TRIGGER_HOVER) {
                    }
                    this.timeoutTimer = setTimeout(function () {
                        _this.show();
                    }, t);
                }
                else {
                    this.show();
                }
            };
            Tooltip.prototype.stopTimeout = function () {
                if (this.timeoutTimer) {
                    clearTimeout(this.timeoutTimer);
                    this.timeoutTimer = undefined;
                }
            };
            Tooltip.prototype.hideOnMouseOut = function () {
                var tooltip = this;
                var mouseMoveListener = function (event) {
                    var tooltipTargetHtmlElement = tooltip.targetEl.getHTMLElement();
                    for (var element = event.target; element; element = element.parentNode) {
                        if (element == tooltipTargetHtmlElement) {
                            return;
                        }
                    }
                    tooltip.startHideTimeout();
                    api.dom.Body.get().unMouseMove(mouseMoveListener);
                };
                api.dom.Body.get().onMouseMove(mouseMoveListener);
            };
            Tooltip.prototype.getEventName = function (enter) {
                switch (this.trigger) {
                    case Tooltip.TRIGGER_FOCUS:
                        return enter ? "focus" : "blur";
                    case Tooltip.TRIGGER_HOVER:
                    default:
                        return enter ? "mouseenter" : "mouseleave";
                        break;
                }
            };
            Tooltip.prototype.hideOtherInstances = function () {
                var _this = this;
                Tooltip.instances.forEach(function (tooltip) {
                    if (tooltip != _this && tooltip.isVisible()) {
                        tooltip.hide();
                    }
                });
            };
            Tooltip.allowMultipleInstances = function (allow) {
                Tooltip.multipleAllowed = allow;
            };
            Tooltip.isMultipleInstancesAllowed = function () {
                return Tooltip.multipleAllowed;
            };
            Tooltip.SIDE_TOP = "top";
            Tooltip.SIDE_RIGHT = "right";
            Tooltip.SIDE_BOTTOM = "bottom";
            Tooltip.SIDE_LEFT = "left";
            Tooltip.TRIGGER_HOVER = "hover";
            Tooltip.TRIGGER_FOCUS = "focus";
            Tooltip.TRIGGER_NONE = "none";
            Tooltip.MODE_STATIC = "static";
            Tooltip.MODE_FOLLOW = "follow";
            Tooltip.multipleAllowed = true;
            Tooltip.instances = [];
            return Tooltip;
        })();
        ui.Tooltip = Tooltip;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ProgressBar = (function (_super) {
            __extends(ProgressBar, _super);
            function ProgressBar(value) {
                _super.call(this, "progress-bar");
                this.value = value || 0;
                var progress = this.progress = new api.dom.DivEl("progress-indicator");
                this.getEl().appendChild(progress.getHTMLElement());
                this.setValue(this.value);
            }
            ProgressBar.prototype.setValue = function (value) {
                var normalizedValue = value > 0 ? this.normalizeValue(value) : 0;
                this.progress.getEl().setWidth(normalizedValue * 100 + "%");
                this.value = normalizedValue;
            };
            ProgressBar.prototype.getValue = function () {
                return this.value;
            };
            ProgressBar.prototype.normalizeValue = function (value) {
                var integralLength = Math.ceil(Math.log(value) / Math.log(10));
                var maxValue = Math.pow(10, integralLength);
                return value / maxValue;
            };
            return ProgressBar;
        })(api.dom.DivEl);
        ui.ProgressBar = ProgressBar;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ToggleSlide = (function (_super) {
            __extends(ToggleSlide, _super);
            function ToggleSlide(actions) {
                var _this = this;
                _super.call(this, 'toggle-slide');
                this.enabled = true;
                this.animationDuration = 300;
                this.disabledClass = "disabled";
                this.actions = actions;
                actions.turnOnAction.onPropertyChanged(function (action) {
                    _this.setEnabled(action.isEnabled());
                    _this.setVisible(action.isVisible());
                });
                if (!actions.turnOnAction.isEnabled()) {
                    this.setEnabled(false);
                }
                this.createMarkup();
                this.calculateStyles();
                this.setupAnimation();
                var turnOn = actions.turnOnAction.isEnabled();
                if (turnOn) {
                    this.slideOn();
                }
                else {
                    this.slideOff();
                }
                this.isOn = turnOn;
                this.onClicked(function (event) {
                    if (_this.enabled) {
                        _this.toggle();
                    }
                });
            }
            ToggleSlide.prototype.toggle = function () {
                this.isOn ? this.turnOff() : this.turnOn();
            };
            ToggleSlide.prototype.turnOn = function () {
                this.slideOn();
                this.isOn = true;
                this.actions.turnOnAction.execute();
            };
            ToggleSlide.prototype.slideOn = function () {
                if (this.slideLeft.isRunning()) {
                    this.slideLeft.stop();
                }
                this.slideRight.start();
            };
            ToggleSlide.prototype.turnOff = function () {
                this.slideOff();
                this.isOn = false;
                this.actions.turnOffAction.execute();
            };
            ToggleSlide.prototype.slideOff = function () {
                if (this.slideRight.isRunning()) {
                    this.slideRight.stop();
                }
                this.slideLeft.start();
            };
            ToggleSlide.prototype.isTurnedOn = function () {
                return this.isOn;
            };
            ToggleSlide.prototype.setEnabled = function (enabled) {
                if (enabled != this.enabled) {
                    if (!enabled) {
                        this.addClass(this.disabledClass);
                    }
                    else {
                        this.removeClass(this.disabledClass);
                    }
                    this.enabled = enabled;
                }
            };
            ToggleSlide.prototype.isEnabled = function () {
                return this.enabled;
            };
            ToggleSlide.prototype.createMarkup = function () {
                this.slider = new api.dom.DivEl('slider');
                this.holder = new api.dom.DivEl('holder');
                this.onLabel = new api.dom.DivEl('on');
                this.offLabel = new api.dom.DivEl('off');
                this.appendChild(this.slider);
                this.appendChild(this.holder);
                this.holder.appendChild(this.onLabel);
                this.holder.appendChild(this.offLabel);
                this.onLabel.getEl().setInnerHtml(this.actions.turnOnAction.getLabel());
                this.offLabel.getEl().setInnerHtml(this.actions.turnOffAction.getLabel());
            };
            ToggleSlide.prototype.calculateStyles = function () {
                var sliderEl = this.slider.getEl(), onLabelEl = this.onLabel.getEl(), offLabelEl = this.offLabel.getEl();
                api.dom.Body.get().appendChild(this);
                var labelWidth = Math.max(onLabelEl.getWidth(), offLabelEl.getWidth());
                sliderEl.setWidth((labelWidth + 4) + 'px');
                this.sliderOffset = labelWidth - 4;
                onLabelEl.setWidth(labelWidth + 'px');
                offLabelEl.setWidth(labelWidth + 'px');
            };
            ToggleSlide.prototype.setupAnimation = function () {
                var _this = this;
                this.slideLeft = new api.util.Animation(this.animationDuration);
                this.slideLeft.onStep(function (progress) {
                    _this.slider.getEl().setLeft(_this.sliderOffset * (1 - progress) + 'px');
                });
                this.slideRight = new api.util.Animation(this.animationDuration);
                this.slideRight.onStep(function (progress) {
                    _this.slider.getEl().setLeft(_this.sliderOffset * progress + 'px');
                });
            };
            return ToggleSlide;
        })(api.dom.DivEl);
        ui.ToggleSlide = ToggleSlide;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var NavigatorEvent = (function () {
            function NavigatorEvent(tab) {
                this.tab = tab;
            }
            NavigatorEvent.prototype.getItem = function () {
                return this.tab;
            };
            return NavigatorEvent;
        })();
        ui.NavigatorEvent = NavigatorEvent;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Dropdown = (function (_super) {
            __extends(Dropdown, _super);
            function Dropdown(name) {
                var _this = this;
                _super.call(this);
                this.oldValue = "";
                this.valueChangedListeners = [];
                this.getEl().setAttribute("name", name);
                this.onChange(function (event) {
                    _this.notifyValueChanged(_this.oldValue, _this.getValue());
                    _this.oldValue = _this.getValue();
                });
            }
            Dropdown.prototype.addOption = function (value, displayName) {
                var option = new DropdownOption(value, displayName);
                this.appendChild(option);
            };
            Dropdown.prototype.setValue = function (value) {
                _super.prototype.setValue.call(this, value);
                this.notifyValueChanged(this.oldValue, value);
                this.oldValue = value;
                return this;
            };
            Dropdown.prototype.onValueChanged = function (listener) {
                this.valueChangedListeners.push(listener);
            };
            Dropdown.prototype.unValueChanged = function (listener) {
                this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            Dropdown.prototype.notifyValueChanged = function (oldValue, newValue) {
                var _this = this;
                this.valueChangedListeners.forEach(function (listener) {
                    listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                });
            };
            return Dropdown;
        })(api.dom.SelectEl);
        ui.Dropdown = Dropdown;
        var DropdownOption = (function (_super) {
            __extends(DropdownOption, _super);
            function DropdownOption(value, displayName) {
                _super.call(this, value, displayName);
            }
            return DropdownOption;
        })(api.dom.OptionEl);
        ui.DropdownOption = DropdownOption;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var RadioGroup = (function (_super) {
            __extends(RadioGroup, _super);
            function RadioGroup(name, orientation) {
                _super.call(this, "div", "radio-group");
                this.options = [];
                this.oldValue = "";
                this.valueChangedListeners = [];
                this.name = name;
                if (RadioGroup.ORIENTATION_VERTICAL == orientation) {
                    this.addClass("vertical");
                }
            }
            RadioGroup.prototype.addOption = function (value, label, checked) {
                var _this = this;
                var radio = new RadioButton(label, value, this.name, checked);
                this.options.push(radio);
                this.appendChild(radio);
                radio.onClicked(function (event) {
                    _this.notifyValueChanged(_this.oldValue, _this.getValue());
                    _this.oldValue = _this.getValue();
                });
            };
            RadioGroup.prototype.setValue = function (value) {
                var option;
                for (var i = 0; i < this.options.length; i++) {
                    option = this.options[i];
                    option.setChecked(option.getValue() == value);
                }
                return this;
            };
            RadioGroup.prototype.getValue = function () {
                var option;
                for (var i = 0; i < this.options.length; i++) {
                    option = this.options[i];
                    if (option.isChecked()) {
                        return option.getValue();
                    }
                }
                return undefined;
            };
            RadioGroup.prototype.getName = function () {
                return this.name;
            };
            RadioGroup.prototype.onValueChanged = function (listener) {
                this.valueChangedListeners.push(listener);
            };
            RadioGroup.prototype.unValueChanged = function (listener) {
                this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            RadioGroup.prototype.notifyValueChanged = function (oldValue, newValue) {
                var _this = this;
                this.valueChangedListeners.forEach(function (listener) {
                    listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                });
            };
            RadioGroup.ORIENTATION_VERTICAL = "vertical";
            RadioGroup.ORIENTATION_HORIZONTAL = "horizontal";
            return RadioGroup;
        })(api.dom.FormInputEl);
        ui.RadioGroup = RadioGroup;
        var RadioButton = (function (_super) {
            __extends(RadioButton, _super);
            function RadioButton(label, value, name, checked) {
                _super.call(this, "span", "radio-button");
                this.radio = new api.dom.InputEl();
                this.radio.getEl().setAttribute('type', 'radio');
                this.radio.setName(name).setValue(value);
                this.appendChild(this.radio);
                this.label = new api.dom.LabelEl(label, this.radio);
                this.appendChild(this.label);
                this.setChecked(checked);
            }
            RadioButton.prototype.setValue = function (value) {
                this.radio.setValue(value);
                return this;
            };
            RadioButton.prototype.getValue = function () {
                return this.radio.getValue();
            };
            RadioButton.prototype.getName = function () {
                return this.radio.getName();
            };
            RadioButton.prototype.isChecked = function () {
                return this.radio.getHTMLElement()['checked'];
            };
            RadioButton.prototype.setChecked = function (checked) {
                if (checked === void 0) { checked = false; }
                this.radio.getHTMLElement()['checked'] = checked;
                return this;
            };
            return RadioButton;
        })(api.dom.FormInputEl);
        ui.RadioButton = RadioButton;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Checkbox = (function (_super) {
            __extends(Checkbox, _super);
            function Checkbox(text) {
                var _this = this;
                _super.call(this, "div", "checkbox");
                this.oldValue = false;
                this.valueChangedListeners = [];
                this.checkbox = new api.dom.Element(new api.dom.NewElementBuilder().setTagName('input').setGenerateId(true));
                this.checkbox.getEl().setAttribute('type', 'checkbox');
                this.appendChild(this.checkbox);
                this.label = new api.dom.LabelEl(text, this.checkbox);
                this.appendChild(this.label);
                wemjq(this.checkbox.getHTMLElement()).change(function () {
                    var newValue = _this.isChecked();
                    _this.notifyValueChanged(_this.oldValue, newValue);
                    _this.oldValue = newValue;
                });
            }
            Checkbox.prototype.setChecked = function (newValue, suppressEvent) {
                this.checkbox.getHTMLElement()["checked"] = newValue;
                if (!suppressEvent) {
                    this.notifyValueChanged(this.oldValue, newValue);
                }
                this.oldValue = newValue;
                return this;
            };
            Checkbox.prototype.toggleChecked = function () {
                this.setChecked(!this.isChecked());
            };
            Checkbox.prototype.isChecked = function () {
                return this.checkbox.getHTMLElement()["checked"];
            };
            Checkbox.prototype.setValue = function (value) {
                throw new Error("CheckboxInput does not support method setValue, use setChecked instead");
            };
            Checkbox.prototype.getValue = function () {
                throw new Error("CheckboxInput does not support method setValue, use isChecked instead");
            };
            Checkbox.prototype.giveFocus = function () {
                return this.checkbox.giveFocus();
            };
            Checkbox.prototype.giveBlur = function () {
                return this.checkbox.giveBlur();
            };
            Checkbox.prototype.setName = function (value) {
                this.checkbox.setName(value);
                return this;
            };
            Checkbox.prototype.setLabel = function (text) {
                this.label.setValue(text);
                return this;
            };
            Checkbox.prototype.getLabel = function () {
                return this.label.getValue();
            };
            Checkbox.prototype.setPlaceholder = function (value) {
                this.checkbox.getEl().setAttribute('placeholder', value);
                return this;
            };
            Checkbox.prototype.getPlaceholder = function () {
                return this.checkbox.getEl().getAttribute('placeholder');
            };
            Checkbox.prototype.onValueChanged = function (listener) {
                this.valueChangedListeners.push(listener);
            };
            Checkbox.prototype.unValueChanged = function (listener) {
                this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            Checkbox.prototype.notifyValueChanged = function (oldValue, newValue) {
                var _this = this;
                this.valueChangedListeners.forEach(function (listener) {
                    listener.call(_this, new ui.ValueChangedEvent(oldValue.toString(), newValue.toString()));
                });
            };
            Checkbox.prototype.onFocus = function (listener) {
                this.checkbox.onFocus(listener);
            };
            Checkbox.prototype.unFocus = function (listener) {
                this.checkbox.unFocus(listener);
            };
            Checkbox.prototype.onBlur = function (listener) {
                this.checkbox.onBlur(listener);
            };
            Checkbox.prototype.unBlur = function (listener) {
                this.checkbox.unBlur(listener);
            };
            return Checkbox;
        })(api.dom.FormInputEl);
        ui.Checkbox = Checkbox;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var DragHelper = (function () {
            function DragHelper() {
            }
            DragHelper.getHtml = function () {
                return DragHelper.html;
            };
            DragHelper.setDropAllowed = function (isAllowed) {
                var helper = document.getElementById('drag-helper');
                if (!helper) {
                    console.warn('There is no drag helper to change its state.');
                    return;
                }
                var helperEl = new api.dom.ElementHelper(helper);
                if (isAllowed) {
                    helperEl.removeClass("drop-not-allowed").addClass("drop-allowed");
                }
                else {
                    helperEl.removeClass("drop-allowed").addClass("drop-not-allowed");
                }
            };
            DragHelper.html = '<div id="drag-helper" class="drop-allowed" style="width: 48px; height: 48px; position: absolute;"></div>';
            return DragHelper;
        })();
        ui.DragHelper = DragHelper;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var NamesAndIconViewer = (function (_super) {
            __extends(NamesAndIconViewer, _super);
            function NamesAndIconViewer(className, size) {
                if (size === void 0) { size = 2 /* small */; }
                _super.call(this, className);
                this.namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(size).build();
            }
            NamesAndIconViewer.prototype.setObject = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                _super.prototype.setObject.call(this, object);
                var displayName = this.resolveDisplayName(object) || this.normalizeDisplayName(this.resolveUnnamedDisplayName(object)), subName = this.resolveSubName(object, relativePath) || NamesAndIconViewer.EMPTY_SUB_NAME, subTitle = this.resolveSubTitle(object), iconClass = this.resolveIconClass(object), iconUrl = this.resolveIconUrl(object);
                this.namesAndIconView.setMainName(displayName).setSubName(subName, subTitle).setIconClass(iconClass);
                if (!!iconUrl) {
                    this.namesAndIconView.setIconUrl(iconUrl);
                }
                this.render();
            };
            NamesAndIconViewer.prototype.normalizeDisplayName = function (displayName) {
                if (api.util.StringHelper.isEmpty(displayName)) {
                    return NamesAndIconViewer.EMPTY_DISPLAY_NAME;
                }
                else {
                    displayName = api.util.StringHelper.capitalizeAll(displayName.replace(/-/g, " ").trim());
                    return "<Unnamed " + displayName + ">";
                }
            };
            NamesAndIconViewer.prototype.resolveDisplayName = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveUnnamedDisplayName = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return "";
            };
            NamesAndIconViewer.prototype.resolveSubTitle = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveIconClass = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveIconUrl = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.getPreferredHeight = function () {
                return 50;
            };
            NamesAndIconViewer.prototype.doRender = function () {
                this.removeChildren();
                this.appendChild(this.namesAndIconView);
                return true;
            };
            NamesAndIconViewer.EMPTY_DISPLAY_NAME = "<Display Name>";
            NamesAndIconViewer.EMPTY_SUB_NAME = "<unnamed>";
            return NamesAndIconViewer;
        })(api.ui.Viewer);
        ui.NamesAndIconViewer = NamesAndIconViewer;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var Button = (function (_super) {
                __extends(Button, _super);
                function Button(label) {
                    _super.call(this, "button");
                    this.labelEl = new api.dom.SpanEl();
                    if (label) {
                        this.labelEl.getEl().setInnerHtml(label);
                    }
                    this.appendChild(this.labelEl);
                }
                Button.prototype.setEnabled = function (value) {
                    this.getEl().setDisabled(!value);
                };
                Button.prototype.isEnabled = function () {
                    return !this.getEl().isDisabled();
                };
                Button.prototype.setActive = function (value) {
                    if (value) {
                        this.addClass("active");
                    }
                    else {
                        this.removeClass("active");
                    }
                };
                Button.prototype.isActive = function () {
                    return this.hasClass("active");
                };
                Button.prototype.setLabel = function (label) {
                    this.labelEl.getEl().setInnerHtml(label);
                };
                Button.prototype.getLabel = function () {
                    return this.labelEl.getEl().getInnerHtml();
                };
                return Button;
            })(api.dom.ButtonEl);
            button.Button = Button;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var ActionButton = (function (_super) {
                __extends(ActionButton, _super);
                function ActionButton(action, showTooltip) {
                    var _this = this;
                    if (showTooltip === void 0) { showTooltip = true; }
                    this.action = action;
                    var label;
                    if (this.action.hasMnemonic()) {
                        label = this.action.getMnemonic().underlineMnemonic(this.action.getLabel());
                    }
                    else {
                        label = this.action.getLabel();
                    }
                    _super.call(this, label);
                    this.addClass("action-button");
                    this.setEnabled(this.action.isEnabled());
                    if (this.action.getIconClass()) {
                        this.addClass(action.getIconClass());
                    }
                    if (this.action.hasShortcut() && showTooltip) {
                        this.tooltip = new ui.Tooltip(this, this.action.getShortcut().getCombination(), 1000);
                        api.ui.KeyBindings.get().onHelpKeyPressed(function (e) {
                            if (_this.action.isEnabled() && api.ui.KeyBindings.get().isActive(_this.action.getShortcut())) {
                                if (ui.KeyBindingAction[0 /* KEYDOWN */].toLowerCase() == e.type) {
                                    _this.tooltip.show();
                                    return;
                                }
                            }
                            _this.tooltip.hide();
                        });
                    }
                    this.onClicked(function (event) {
                        _this.action.execute();
                    });
                    this.action.onPropertyChanged(function (action) {
                        _this.setEnabled(action.isEnabled());
                        _this.setVisible(action.isVisible());
                    });
                }
                ActionButton.prototype.getTooltip = function () {
                    return this.tooltip;
                };
                return ActionButton;
            })(api.ui.button.Button);
            button.ActionButton = ActionButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var CycleButton = (function (_super) {
                __extends(CycleButton, _super);
                function CycleButton() {
                    var _this = this;
                    var actions = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        actions[_i - 0] = arguments[_i];
                    }
                    _super.call(this, "button cycle-button");
                    this.actionList = actions;
                    this.actionList.forEach(function (action, i) {
                        var el = _this.addAction(action, i);
                        if (i == 0) {
                            _this.setActive(el);
                        }
                    });
                    var dropDownButton = new api.dom.DivEl("drop-down-button");
                    dropDownButton.onClicked(function (event) {
                        if (_this.hasClass("expanded")) {
                            _this.removeClass("expanded");
                        }
                        else {
                            _this.addClass("expanded");
                        }
                    });
                    this.appendChild(dropDownButton);
                }
                CycleButton.prototype.disableAction = function (action) {
                    if (action.isEnabled()) {
                        action.setEnabled(false);
                        this.getElementFromIndex(this.getIndexFromAction(action)).addClass("hidden");
                    }
                };
                CycleButton.prototype.enableAction = function (action) {
                    action.setEnabled(true);
                    this.getElementFromIndex(this.getIndexFromAction(action)).removeClass("hidden");
                };
                CycleButton.prototype.setCurrentAction = function (action) {
                    this.doAction(this.getIndexFromAction(action));
                };
                CycleButton.prototype.addAction = function (action, index) {
                    var _this = this;
                    var cycleActionEl = new api.dom.DivEl("action");
                    cycleActionEl.getEl().setInnerHtml(action.getLabel());
                    cycleActionEl.getEl().setData("action", index + "");
                    cycleActionEl.onClicked(function () {
                        _this.removeClass("expanded");
                        if (cycleActionEl.hasClass("active")) {
                            _this.doAction(_this.nextActionIndex(index));
                        }
                        else {
                            _this.doAction(index);
                        }
                    });
                    this.appendChild(cycleActionEl);
                    return cycleActionEl;
                };
                CycleButton.prototype.nextActionIndex = function (index) {
                    index = index + 1;
                    if (index < this.actionList.length) {
                        if (this.actionList[index].isEnabled()) {
                            return index;
                        }
                        else {
                            return this.nextActionIndex(index);
                        }
                    }
                    else {
                        return 0;
                    }
                };
                CycleButton.prototype.doAction = function (index) {
                    while (!this.actionList[index].isEnabled()) {
                        index++;
                    }
                    this.setActive(this.getElementFromIndex(index));
                    this.actionList[index].execute();
                };
                CycleButton.prototype.getIndexFromAction = function (action) {
                    var index = -1;
                    this.actionList.forEach(function (a, i) {
                        if (a == action) {
                            index = i;
                        }
                    });
                    return index;
                };
                CycleButton.prototype.getElementFromIndex = function (index) {
                    var element = null;
                    this.getChildren().forEach(function (actionEl) {
                        if (parseInt(actionEl.getEl().getData("action")) == index) {
                            element = actionEl;
                        }
                    });
                    return element;
                };
                CycleButton.prototype.setActive = function (listItem) {
                    this.getChildren().forEach(function (child) {
                        child.removeClass("active");
                    });
                    listItem.addClass("active");
                    this.removeChild(listItem);
                    this.prependChild(listItem);
                };
                return CycleButton;
            })(api.dom.DivEl);
            button.CycleButton = CycleButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var CloseButton = (function (_super) {
                __extends(CloseButton, _super);
                function CloseButton(className) {
                    _super.call(this);
                    this.addClass('close-button icon-medium icon-close2');
                    if (className) {
                        this.addClass(className);
                    }
                }
                return CloseButton;
            })(api.ui.button.Button);
            button.CloseButton = CloseButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var TextInput = (function (_super) {
                __extends(TextInput, _super);
                function TextInput(className, size) {
                    var _this = this;
                    _super.call(this, className);
                    this.allowedKeyCodes = [8, 9, 46, 39, 37];
                    this.oldValue = "";
                    this.valueChangedListeners = [];
                    this.getEl().setAttribute('type', 'text');
                    this.addClass('text-input');
                    if (size) {
                        this.addClass(size);
                    }
                    this.onKeyPressed(function (event) {
                        if (!_this.stripCharsRe) {
                            return;
                        }
                        var symbol = String.fromCharCode(event.charCode);
                        if (_this.containsForbiddenChars(symbol)) {
                            if (!_this.keyCodeAllowed(event.keyCode)) {
                                event.preventDefault();
                                return false;
                            }
                        }
                    });
                    this.onInput(function (event) {
                        _this.notifyValueChanged(_this.oldValue, _this.getValue());
                        _this.oldValue = _this.getValue();
                    });
                }
                TextInput.large = function (className) {
                    return new TextInput(className, TextInput.LARGE);
                };
                TextInput.middle = function (className) {
                    return new TextInput(className, TextInput.MIDDLE);
                };
                TextInput.prototype.setValue = function (value) {
                    var oldValue = this.getValue();
                    var newValue = this.removeForbiddenChars(value);
                    _super.prototype.setValue.call(this, value);
                    if (oldValue != newValue) {
                        _super.prototype.setValue.call(this, newValue);
                        this.notifyValueChanged(oldValue, newValue);
                        this.oldValue = newValue;
                    }
                    return this;
                };
                TextInput.prototype.setName = function (value) {
                    _super.prototype.setName.call(this, value);
                    return this;
                };
                TextInput.prototype.setPlaceholder = function (value) {
                    _super.prototype.setPlaceholder.call(this, value);
                    return this;
                };
                TextInput.prototype.setForbiddenCharsRe = function (re) {
                    this.stripCharsRe = re;
                    return this;
                };
                TextInput.prototype.selectText = function (from, to) {
                    var htmlEl = this.getHTMLElement();
                    if (!from) {
                        htmlEl.select();
                    }
                    else if (!to) {
                        to = this.getValue().length;
                    }
                    if (htmlEl.createTextRange) {
                        var selRange = htmlEl.createTextRange();
                        selRange.collapse(true);
                        selRange.moveStart('character', from);
                        selRange.moveEnd('character', to);
                        selRange.select();
                    }
                    else if (htmlEl.setSelectionRange) {
                        htmlEl.setSelectionRange(from, to);
                    }
                    else if (htmlEl.selectionStart) {
                        htmlEl.selectionStart = from;
                        htmlEl.selectionEnd = to;
                    }
                    htmlEl.focus();
                };
                TextInput.prototype.moveCaretTo = function (pos) {
                    this.selectText(pos, pos);
                };
                TextInput.prototype.onValueChanged = function (listener) {
                    this.valueChangedListeners.push(listener);
                };
                TextInput.prototype.unValueChanged = function (listener) {
                    this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TextInput.prototype.updateValidationStatusOnUserInput = function (isValid) {
                    if (isValid) {
                        this.removeClass("invalid");
                        if (!api.util.StringHelper.isEmpty(this.getValue())) {
                            this.addClass("valid");
                        }
                        else {
                            this.removeClass("valid");
                        }
                    }
                    else {
                        this.removeClass("valid");
                        this.addClass("invalid");
                    }
                };
                TextInput.prototype.notifyValueChanged = function (oldValue, newValue) {
                    var _this = this;
                    this.valueChangedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                    });
                };
                TextInput.prototype.removeForbiddenChars = function (rawValue) {
                    return this.stripCharsRe ? (rawValue || '').replace(this.stripCharsRe, '') : rawValue;
                };
                TextInput.prototype.containsForbiddenChars = function (value) {
                    var forbidden = new RegExp(this.stripCharsRe);
                    return forbidden.test(value);
                };
                TextInput.prototype.keyCodeAllowed = function (keyCode) {
                    for (var i = 0; i < this.allowedKeyCodes.length; i++) {
                        if (keyCode == this.allowedKeyCodes[i]) {
                            return true;
                        }
                    }
                    return false;
                };
                TextInput.MIDDLE = 'middle';
                TextInput.LARGE = 'large';
                return TextInput;
            })(api.dom.InputEl);
            text.TextInput = TextInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var AutosizeTextInput = (function (_super) {
                __extends(AutosizeTextInput, _super);
                function AutosizeTextInput(className, size) {
                    var _this = this;
                    if (size === void 0) { size = text.TextInput.MIDDLE; }
                    _super.call(this, className, size);
                    this.addClass('autosize');
                    this.clone = new api.dom.DivEl().addClass('autosize-clone').addClass(this.getEl().getAttribute('class'));
                    this.attendant = new api.dom.DivEl().addClass('autosize-attendant');
                    this.attendant.appendChild(this.clone);
                    this.onShown(function (event) { return _this.updateSize(); });
                    this.onValueChanged(function (event) { return _this.updateSize(); });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.updateSize(); }, this);
                }
                AutosizeTextInput.large = function (className) {
                    return new AutosizeTextInput(className, text.TextInput.LARGE);
                };
                AutosizeTextInput.middle = function (className) {
                    return new AutosizeTextInput(className, text.TextInput.MIDDLE);
                };
                AutosizeTextInput.prototype.updateSize = function () {
                    var inputEl = this.getEl(), cloneEl = this.clone.getEl();
                    cloneEl.setFontSize(inputEl.getFontSize()).setPaddingLeft(inputEl.getPaddingLeft() + 'px').setPaddingRight(inputEl.getPaddingRight() + 'px');
                    this.attendant.insertAfterEl(this);
                    cloneEl.setInnerHtml(this.getValue());
                    inputEl.setWidthPx(Math.min(cloneEl.getWidthWithBorder(), this.attendant.getEl().getWidth()));
                    this.attendant.remove();
                };
                return AutosizeTextInput;
            })(text.TextInput);
            text.AutosizeTextInput = AutosizeTextInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var PasswordInput = (function (_super) {
                __extends(PasswordInput, _super);
                function PasswordInput(className) {
                    _super.call(this, className, "password");
                    this.addClass('password-input');
                }
                return PasswordInput;
            })(api.dom.InputEl);
            text.PasswordInput = PasswordInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (_text) {
            var TextArea = (function (_super) {
                __extends(TextArea, _super);
                function TextArea(name) {
                    var _this = this;
                    _super.call(this, "textarea", "text-area");
                    this.valueChangedListeners = [];
                    this.oldValue = "";
                    this.getEl().setAttribute("name", name);
                    this.onInput(function (event) {
                        _this.notifyValueChanged(_this.oldValue, _this.getValue());
                        _this.oldValue = _this.getValue();
                    });
                    this.clone = new api.dom.DivEl('autosize-clone').addClass(this.getEl().getAttribute('class'));
                    this.attendant = new api.dom.DivEl('autosize-attendant');
                    this.attendant.appendChild(this.clone);
                    this.onAdded(function (event) {
                        _this.attendant.insertAfterEl(_this);
                    });
                    this.onShown(function (event) { return _this.updateSize(); });
                    this.onValueChanged(function (event) { return _this.updateSize(); });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.updateSize(); }, this);
                }
                TextArea.prototype.setValue = function (text) {
                    if (this.oldValue != text) {
                        _super.prototype.setValue.call(this, text);
                        this.notifyValueChanged(this.oldValue, text);
                        this.oldValue = text;
                    }
                    return this;
                };
                TextArea.prototype.setRows = function (rows) {
                    this.getEl().setAttribute("rows", rows.toString());
                };
                TextArea.prototype.setColumns = function (columns) {
                    this.getEl().setAttribute("cols", columns.toString());
                };
                TextArea.prototype.updateSize = function () {
                    if (this.isRendered()) {
                        this.clone.getEl().setInnerHtml(this.getValue() + " ");
                        this.getEl().setHeightPx(this.clone.getEl().getHeightWithBorder());
                    }
                };
                TextArea.prototype.onValueChanged = function (listener) {
                    this.valueChangedListeners.push(listener);
                };
                TextArea.prototype.unValueChanged = function (listener) {
                    this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                TextArea.prototype.notifyValueChanged = function (oldValue, newValue) {
                    var _this = this;
                    this.valueChangedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                    });
                };
                return TextArea;
            })(api.dom.FormInputEl);
            _text.TextArea = TextArea;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var CodeAreaBuilder = (function () {
                function CodeAreaBuilder() {
                }
                CodeAreaBuilder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                CodeAreaBuilder.prototype.setMode = function (value) {
                    this.mode = value;
                    return this;
                };
                CodeAreaBuilder.prototype.setLineNumbers = function (value) {
                    this.lineNumbers = value;
                    return this;
                };
                CodeAreaBuilder.prototype.build = function () {
                    return new CodeArea(this);
                };
                return CodeAreaBuilder;
            })();
            text.CodeAreaBuilder = CodeAreaBuilder;
            var CodeArea = (function (_super) {
                __extends(CodeArea, _super);
                function CodeArea(builder) {
                    var _this = this;
                    this.textArea = new text.TextArea(builder.name);
                    _super.call(this, this.textArea);
                    this.mode = builder.mode;
                    this.options = {
                        lineNumbers: builder.lineNumbers
                    };
                    CodeMirror.modeURL = api.util.UriHelper.getUri('admin/common/lib/codemirror/mode/%N.js');
                    this.onAdded(function (event) {
                        _this.codeMirror = CodeMirror.fromTextArea(_this.textArea.getHTMLElement(), _this.options);
                        _this.codeMirror.setSize("540px", "350px");
                        _this.codeMirror.setOption("mode", _this.mode);
                        CodeMirror.autoLoadMode(_this.codeMirror, _this.mode);
                        _this.codeMirror.refresh();
                    });
                    this.onShown(function (event) {
                        _this.codeMirror.refresh();
                    });
                }
                CodeArea.prototype.setValue = function (value) {
                    this.codeMirror.setValue(value);
                    return this;
                };
                CodeArea.prototype.getValue = function () {
                    return this.codeMirror.getValue();
                };
                return CodeArea;
            })(api.dom.CompositeFormInputEl);
            text.CodeArea = CodeArea;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var StringHelper = api.util.StringHelper;
            var NumberHelper = api.util.NumberHelper;
            var ArrayHelper = api.util.ArrayHelper;
            var CharType;
            (function (CharType) {
                CharType[CharType["SPECIAL"] = 0] = "SPECIAL";
                CharType[CharType["DIGIT"] = 1] = "DIGIT";
                CharType[CharType["UPPERCASE"] = 2] = "UPPERCASE";
                CharType[CharType["LOWERCASE"] = 3] = "LOWERCASE";
            })(CharType || (CharType = {}));
            var PasswordGenerator = (function (_super) {
                __extends(PasswordGenerator, _super);
                function PasswordGenerator() {
                    var _this = this;
                    _super.call(this, "div", "password-generator");
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.SPECIAL_CHARS = '!@#$%^&*()_+{}:"<>?|[];\',./`~';
                    this.LOWERCASE_CHARS = 'abcdefghijklmnopqrstuvwxyz';
                    this.UPPERCASE_CHARS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
                    this.DIGIT_CHARS = '0123456789';
                    var inputWrapper = new api.dom.DivEl('input-wrapper');
                    this.appendChild(inputWrapper);
                    var strengthMeter = new api.dom.DivEl('strength-meter');
                    inputWrapper.appendChild(strengthMeter);
                    this.input = new text.PasswordInput();
                    this.initFocusEvents(this.input);
                    this.input.onInput(function (event) {
                        _this.assessComplexity(_this.input.getValue());
                        _this.notifyValidityChanged(_this.input.isValid());
                    });
                    inputWrapper.appendChild(this.input);
                    this.showLink = new api.dom.AEl('show-link');
                    this.initFocusEvents(this.showLink);
                    this.showLink.onClicked(function (event) {
                        var unlocked = _this.hasClass('unlocked');
                        _this.toggleClass('unlocked', !unlocked);
                        _this.input.setType(unlocked ? 'password' : 'text');
                    });
                    this.appendChild(this.showLink);
                    this.generateLink = new api.dom.AEl();
                    this.generateLink.setHtml('Generate');
                    this.initFocusEvents(this.generateLink);
                    this.generateLink.onClicked(function (event) {
                        _this.generatePassword();
                        _this.assessComplexity(_this.input.getValue());
                        _this.notifyValidityChanged(_this.input.isValid());
                    });
                    this.appendChild(this.generateLink);
                }
                PasswordGenerator.prototype.getValue = function () {
                    return this.input.getValue();
                };
                PasswordGenerator.prototype.setValue = function (value) {
                    this.input.setValue(value);
                    this.assessComplexity(value);
                    return this;
                };
                PasswordGenerator.prototype.getName = function () {
                    return this.input.getName();
                };
                PasswordGenerator.prototype.setName = function (value) {
                    this.input.setName(value);
                    return this;
                };
                PasswordGenerator.prototype.setPlaceholder = function (value) {
                    this.input.setPlaceholder(value);
                    return this;
                };
                PasswordGenerator.prototype.getPlaceholder = function () {
                    return this.input.getPlaceholder();
                };
                PasswordGenerator.prototype.assessComplexity = function (value) {
                    if (this.complexity) {
                        this.removeClass(this.complexity);
                        this.complexity = undefined;
                    }
                    if (this.isExtreme(value)) {
                        this.complexity = 'extreme';
                    }
                    else if (this.isStrong(value)) {
                        this.complexity = 'strong';
                    }
                    else if (this.isGood(value)) {
                        this.complexity = 'good';
                    }
                    else if (this.isWeak(value)) {
                        this.complexity = 'weak';
                    }
                    if (this.complexity) {
                        this.addClass(this.complexity);
                    }
                };
                PasswordGenerator.prototype.generatePassword = function () {
                    var length = NumberHelper.randomBetween(14, 16), maxSpecials = NumberHelper.randomBetween(1, 3), specials = 0, maxDigits = NumberHelper.randomBetween(2, 4), digits = 0, maxUppercase = NumberHelper.randomBetween(2, 4), uppercase = 0, maxLowercase = length - maxSpecials - maxDigits - maxUppercase, lowercase = 0;
                    var result = "";
                    var types = [0 /* SPECIAL */, 1 /* DIGIT */, 2 /* UPPERCASE */, 3 /* LOWERCASE */];
                    for (var i = 0; i < length; i++) {
                        var type = types[NumberHelper.randomBetween(0, types.length - 1)];
                        switch (type) {
                            case 0 /* SPECIAL */:
                                if (specials < maxSpecials) {
                                    result += this.SPECIAL_CHARS.charAt(NumberHelper.randomBetween(0, this.SPECIAL_CHARS.length - 1));
                                    specials++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(0 /* SPECIAL */, types);
                                }
                                break;
                            case 1 /* DIGIT */:
                                if (digits < maxDigits) {
                                    result += this.DIGIT_CHARS.charAt(NumberHelper.randomBetween(0, this.DIGIT_CHARS.length - 1));
                                    digits++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(1 /* DIGIT */, types);
                                }
                                break;
                            case 2 /* UPPERCASE */:
                                if (uppercase < maxUppercase) {
                                    result += this.UPPERCASE_CHARS.charAt(NumberHelper.randomBetween(0, this.UPPERCASE_CHARS.length - 1));
                                    uppercase++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(2 /* UPPERCASE */, types);
                                }
                                break;
                            case 3 /* LOWERCASE */:
                                if (lowercase < maxLowercase) {
                                    result += this.LOWERCASE_CHARS.charAt(NumberHelper.randomBetween(0, this.LOWERCASE_CHARS.length - 1));
                                    lowercase++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(3 /* LOWERCASE */, types);
                                }
                                break;
                        }
                    }
                    console.log('Generated ' + length + ' chars password (SP:' + maxSpecials + ', DG:' + maxDigits + ',UC:' + maxUppercase + ', LC:' + maxLowercase + ') = ' + result);
                    this.input.setValue(result);
                };
                PasswordGenerator.prototype.isWeak = function (value) {
                    return !StringHelper.isBlank(value) && (value.length < 8 || StringHelper.isLowerCase(value) || StringHelper.isUpperCase(value));
                };
                PasswordGenerator.prototype.isGood = function (value) {
                    return !StringHelper.isBlank(value) && (value.length >= 8 || (value.length >= 6 && StringHelper.isMixedCase(value) && this.containsNonAlphabetChars(value)));
                };
                PasswordGenerator.prototype.isStrong = function (value) {
                    return !StringHelper.isBlank(value) && value.length >= 10 && StringHelper.isMixedCase(value) && this.containsNonAlphabetChars(value);
                };
                PasswordGenerator.prototype.isExtreme = function (value) {
                    return !StringHelper.isBlank(value) && value.length >= 14 && StringHelper.isMixedCase(value) && this.containsDigits(value) && this.containsSpecialChars(value);
                };
                PasswordGenerator.prototype.containsDigits = function (value) {
                    return /\d/.test(value);
                };
                PasswordGenerator.prototype.containsSpecialChars = function (value) {
                    return /[^a-z0-9\s]/i.test(value);
                };
                PasswordGenerator.prototype.containsNonAlphabetChars = function (value) {
                    return /[^a-z\s]/i.test(value);
                };
                PasswordGenerator.prototype.initFocusEvents = function (el) {
                    var _this = this;
                    el.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    el.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                };
                PasswordGenerator.prototype.onInput = function (listener) {
                    this.input.onInput(listener);
                };
                PasswordGenerator.prototype.unInput = function (listener) {
                    this.input.unInput(listener);
                };
                PasswordGenerator.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                PasswordGenerator.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                PasswordGenerator.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                PasswordGenerator.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                PasswordGenerator.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                PasswordGenerator.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return PasswordGenerator;
            })(api.dom.FormInputEl);
            text.PasswordGenerator = PasswordGenerator;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var InputEl = api.dom.InputEl;
            var StringHelper = api.util.StringHelper;
            var CheckEmailAvailabilityRequest = api.security.CheckEmailAvailabilityRequest;
            var EmailInput = (function (_super) {
                __extends(EmailInput, _super);
                function EmailInput() {
                    var _this = this;
                    _super.call(this, "div", "email-input");
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.input = new InputEl(undefined, 'email');
                    this.input.setPattern("^[a-zA-Z0-9\_\-\.]+@[a-zA-Z0-9\-]+(?:\.[a-zA-Z0-9\-]{2,})*(?:\.[a-zA-Z]{2,4})$");
                    this.input.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    this.input.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    this.input.onInput(function (event) {
                        if (_this.checkTimeout) {
                            clearTimeout(_this.checkTimeout);
                        }
                        _this.checkTimeout = setTimeout(function (email) { return _this.checkAvailability(email); }, 500, _this.input.getValue());
                    });
                    this.appendChild(this.input);
                }
                EmailInput.prototype.getInput = function () {
                    return this.input;
                };
                EmailInput.prototype.getValue = function () {
                    return this.input.getValue();
                };
                EmailInput.prototype.setValue = function (value) {
                    this.input.setValue(value);
                    this.checkAvailability(value);
                    return this;
                };
                EmailInput.prototype.getOriginEmail = function () {
                    return this.originEmail;
                };
                EmailInput.prototype.setOriginEmail = function (value) {
                    this.originEmail = value;
                    return this;
                };
                EmailInput.prototype.setUserStoreKey = function (userStoreKey) {
                    this.userStoreKey = userStoreKey;
                    return this;
                };
                EmailInput.prototype.getName = function () {
                    return this.input.getName();
                };
                EmailInput.prototype.setName = function (value) {
                    this.input.setName(value);
                    return this;
                };
                EmailInput.prototype.isAvailable = function () {
                    return this.hasClass('available');
                };
                EmailInput.prototype.checkAvailability = function (email) {
                    var _this = this;
                    var status;
                    var isValid = this.input.isValid();
                    this.toggleClass('invalid', !isValid);
                    if (!StringHelper.isEmpty(email) && isValid) {
                        status = 'checking';
                        new CheckEmailAvailabilityRequest(email).setUserStoreKey(this.userStoreKey).sendAndParse().then(function (available) {
                            var availability = available || email === _this.originEmail;
                            _this.updateStatus(availability ? 'available' : 'notavailable');
                            _this.notifyValidityChanged(isValid && availability);
                        }).fail(function (reason) {
                            _this.notifyValidityChanged(false);
                            _this.updateStatus('error');
                        }).done();
                    }
                    else {
                        this.notifyValidityChanged(isValid);
                    }
                    this.updateStatus(status);
                };
                EmailInput.prototype.updateStatus = function (status) {
                    if (!!this.status) {
                        this.removeClass(this.status);
                    }
                    if (!StringHelper.isEmpty(status)) {
                        this.status = status;
                        this.addClass(this.status);
                    }
                };
                EmailInput.prototype.isValid = function () {
                    return this.input.isValid();
                };
                EmailInput.prototype.validate = function () {
                    return this.input.validate();
                };
                EmailInput.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                EmailInput.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                EmailInput.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                EmailInput.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                EmailInput.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                EmailInput.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return EmailInput;
            })(api.dom.FormInputEl);
            text.EmailInput = EmailInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var InputEl = api.dom.InputEl;
            var FileInput = (function (_super) {
                __extends(FileInput, _super);
                function FileInput(className) {
                    var _this = this;
                    _super.call(this, "div", "file-input");
                    if (className) {
                        this.addClass(className);
                    }
                    this.textInput = new InputEl("text");
                    this.mediaUploader = new api.content.MediaUploader({
                        operation: 0 /* create */,
                        name: 'file-input-uploader',
                        allowDrop: false,
                        showButtons: false,
                        showResult: false,
                        allowMultiSelection: true,
                        deferred: true
                    });
                    this.mediaUploader.onUploadStarted(function (event) {
                        var names = event.getUploadItems().map(function (uploadItem) {
                            return uploadItem.getName();
                        });
                        _this.textInput.setValue(names.join(', '));
                    });
                    this.appendChildren(this.textInput, this.mediaUploader);
                }
                FileInput.prototype.setUploaderParams = function (params) {
                    this.mediaUploader.setParams(params);
                    return this;
                };
                FileInput.prototype.getUploaderParams = function () {
                    return this.mediaUploader.getParams();
                };
                FileInput.prototype.setPlaceholder = function (placeholder) {
                    this.textInput.setPlaceholder(placeholder);
                    return this;
                };
                FileInput.prototype.getPlaceholder = function () {
                    return this.textInput.getPlaceholder();
                };
                FileInput.prototype.getValue = function () {
                    return this.textInput.getValue();
                };
                FileInput.prototype.setValue = function (value) {
                    this.textInput.setValue(value);
                    return this;
                };
                FileInput.prototype.onChange = function (listener) {
                    this.textInput.onChange(listener);
                };
                FileInput.prototype.unChange = function (listener) {
                    this.textInput.unChange(listener);
                };
                FileInput.prototype.onInput = function (listener) {
                    this.textInput.onInput(listener);
                };
                FileInput.prototype.unInput = function (listener) {
                    this.textInput.unInput(listener);
                };
                FileInput.prototype.giveFocus = function () {
                    return this.textInput.giveFocus();
                };
                FileInput.prototype.giveBlur = function () {
                    return this.textInput.giveBlur();
                };
                FileInput.prototype.reset = function () {
                    this.textInput.reset();
                    this.mediaUploader.reset();
                    return this;
                };
                FileInput.prototype.stop = function () {
                    this.mediaUploader.stop();
                    return this;
                };
                FileInput.prototype.onUploadStarted = function (listener) {
                    this.mediaUploader.onUploadStarted(listener);
                };
                FileInput.prototype.unUploadStarted = function (listener) {
                    this.mediaUploader.unUploadStarted(listener);
                };
                FileInput.prototype.onUploadProgress = function (listener) {
                    this.mediaUploader.onUploadProgress(listener);
                };
                FileInput.prototype.unUploadProgress = function (listener) {
                    this.mediaUploader.unUploadProgress(listener);
                };
                FileInput.prototype.onFileUploaded = function (listener) {
                    this.mediaUploader.onFileUploaded(listener);
                };
                FileInput.prototype.unFileUploaded = function (listener) {
                    this.mediaUploader.unFileUploaded(listener);
                };
                FileInput.prototype.onUploadCompleted = function (listener) {
                    this.mediaUploader.onUploadCompleted(listener);
                };
                FileInput.prototype.unUploadCompleted = function (listener) {
                    this.mediaUploader.unUploadCompleted(listener);
                };
                FileInput.prototype.onUploadReset = function (listener) {
                    this.mediaUploader.onUploadReset(listener);
                };
                FileInput.prototype.unUploadReset = function (listener) {
                    this.mediaUploader.unUploadReset(listener);
                };
                FileInput.prototype.onUploadFailed = function (listener) {
                    this.mediaUploader.onUploadFailed(listener);
                };
                FileInput.prototype.unUploadFailed = function (listener) {
                    this.mediaUploader.unUploadFailed(listener);
                };
                return FileInput;
            })(api.dom.FormInputEl);
            text.FileInput = FileInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabItem = (function (_super) {
                __extends(TabItem, _super);
                function TabItem(builder, classes) {
                    var _this = this;
                    _super.call(this, "tab-item" + (!classes ? "" : " " + classes));
                    this.active = false;
                    this.labelChangedListeners = [];
                    this.closedListeners = [];
                    this.selectedListeners = [];
                    this.labelEl = new api.dom.SpanEl('label');
                    this.appendChild(this.labelEl);
                    this.setLabel(builder.label, builder.markUnnamed);
                    this.markInvalid(builder.markInvalid);
                    this.closeAction = builder.closeAction;
                    if (builder.closeButtonEnabled) {
                        this.createRemoveButton();
                    }
                    this.onClicked(function (event) {
                        _this.notifySelectedListeners();
                    });
                }
                TabItem.prototype.createRemoveButton = function () {
                    if (this.closeAction && !this.removeButton) {
                        this.removeButton = new api.ui.button.ActionButton(this.closeAction);
                        this.removeButton.getTooltip().setSide(api.ui.Tooltip.SIDE_LEFT);
                        this.removeButton.onClicked(function (event) {
                            event.stopPropagation();
                            event.preventDefault();
                        });
                        this.prependChild(this.removeButton);
                    }
                };
                TabItem.prototype.setIndex = function (value) {
                    this.index = value;
                };
                TabItem.prototype.getIndex = function () {
                    return this.index;
                };
                TabItem.prototype.setLabel = function (newValue, markUnnamed) {
                    if (markUnnamed === void 0) { markUnnamed = false; }
                    if (this.label == newValue) {
                        return;
                    }
                    var oldValue = this.label;
                    this.label = newValue;
                    this.labelEl.getEl().setText(newValue);
                    this.labelEl.getEl().setAttribute('title', newValue);
                    this.labelEl.toggleClass("unnamed", markUnnamed);
                    this.notifyLabelChangedListeners(newValue, oldValue);
                };
                TabItem.prototype.markInvalid = function (markInvalid) {
                    if (markInvalid === void 0) { markInvalid = false; }
                    this.toggleClass("invalid", markInvalid);
                };
                TabItem.prototype.getLabel = function () {
                    return this.label;
                };
                TabItem.prototype.setActive = function (value) {
                    this.active = value;
                    this.toggleClass("active", value);
                };
                TabItem.prototype.isActive = function () {
                    return this.active;
                };
                TabItem.prototype.getCloseAction = function () {
                    return this.closeAction;
                };
                TabItem.prototype.setCloseAction = function (closeAction) {
                    this.closeAction = closeAction;
                };
                TabItem.prototype.onLabelChanged = function (listener) {
                    this.labelChangedListeners.push(listener);
                };
                TabItem.prototype.onSelected = function (listener) {
                    this.selectedListeners.push(listener);
                };
                TabItem.prototype.onClosed = function (listener) {
                    if (this.closeAction) {
                        throw new Error("Failed to set 'on closed' listener. Close action is already setted.");
                    }
                    else {
                        this.closedListeners.push(listener);
                    }
                };
                TabItem.prototype.unLabelChanged = function (listener) {
                    this.labelChangedListeners = this.labelChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabItem.prototype.unSelected = function (listener) {
                    this.selectedListeners = this.selectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabItem.prototype.unClosed = function (listener) {
                    this.closedListeners = this.closedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabItem.prototype.notifyLabelChangedListeners = function (newValue, oldValue) {
                    var _this = this;
                    this.labelChangedListeners.forEach(function (listener) {
                        listener.call(_this, new tab.TabItemLabelChangedEvent(_this, oldValue, newValue));
                    });
                };
                TabItem.prototype.notifySelectedListeners = function () {
                    var _this = this;
                    this.selectedListeners.forEach(function (listener) {
                        listener.call(_this, new tab.TabItemSelectedEvent(_this));
                    });
                };
                TabItem.prototype.notifyClosedListeners = function () {
                    var _this = this;
                    this.closedListeners.forEach(function (listener) {
                        listener.call(_this, new tab.TabItemClosedEvent(_this));
                    });
                };
                return TabItem;
            })(api.dom.LiEl);
            tab.TabItem = TabItem;
            var TabItemBuilder = (function () {
                function TabItemBuilder() {
                }
                TabItemBuilder.prototype.setLabel = function (label) {
                    this.label = label;
                    return this;
                };
                TabItemBuilder.prototype.setCloseAction = function (closeAction) {
                    this.closeAction = closeAction;
                    return this;
                };
                TabItemBuilder.prototype.setCloseButtonEnabled = function (enabled) {
                    this.closeButtonEnabled = enabled;
                    return this;
                };
                TabItemBuilder.prototype.setMarkUnnamed = function (markUnnamed) {
                    this.markUnnamed = markUnnamed;
                    return this;
                };
                TabItemBuilder.prototype.setMarkInvalid = function (markInvalid) {
                    this.markInvalid = markInvalid;
                    return this;
                };
                TabItemBuilder.prototype.build = function () {
                    return new TabItem(this);
                };
                return TabItemBuilder;
            })();
            tab.TabItemBuilder = TabItemBuilder;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabBar = (function (_super) {
                __extends(TabBar, _super);
                function TabBar(classes) {
                    _super.call(this, "tab-bar" + (!classes ? "" : " " + classes));
                    this.tabs = [];
                    this.selectedIndex = -1;
                    this.navigationItemAddedListeners = [];
                    this.navigationItemSelectedListeners = [];
                }
                TabBar.prototype.insertNavigationItem = function (tab, index, silent) {
                    var _this = this;
                    this.tabs.splice(index, 0, tab);
                    tab.setIndex(index);
                    this.tabs.slice(index + 1).forEach(function (tab) {
                        tab.setIndex(tab.getIndex() + 1);
                    });
                    this.insertChild(tab, index);
                    tab.onSelected(function (event) {
                        _this.selectNavigationItem(event.getTab().getIndex());
                    });
                    if (!silent) {
                        this.notifyTabAddedListeners(tab);
                    }
                };
                TabBar.prototype.addNavigationItem = function (tab, silent) {
                    this.insertNavigationItem(tab, this.tabs.length, silent);
                };
                TabBar.prototype.removeNavigationItem = function (tab) {
                    var tabIndex = tab.getIndex();
                    this.tabs.splice(tabIndex, 1);
                    for (var i = tabIndex; i < this.tabs.length; i++) {
                        this.tabs[i].setIndex(i);
                    }
                    if (this.isEmpty()) {
                        this.selectedIndex = -1;
                    }
                    else if ((this.getSize() - 1) < this.selectedIndex) {
                        this.selectedIndex = this.getSize() - 1;
                    }
                    else if (tabIndex < this.selectedIndex) {
                        this.selectedIndex--;
                    }
                    tab.remove();
                };
                TabBar.prototype.selectNavigationItem = function (index, silent) {
                    if (index < 0 || index >= this.getSize() || this.selectedIndex == index) {
                        return;
                    }
                    this.deselectNavigationItem();
                    this.selectedIndex = index;
                    var selectedTab = this.getSelectedNavigationItem();
                    selectedTab.setActive(true);
                    if (!silent) {
                        this.notifyTabShownListeners(selectedTab);
                    }
                };
                TabBar.prototype.deselectNavigationItem = function () {
                    if (this.selectedIndex != -1) {
                        this.getSelectedNavigationItem().setActive(false);
                    }
                    this.selectedIndex = -1;
                };
                TabBar.prototype.getNavigationItem = function (index) {
                    return this.tabs[index];
                };
                TabBar.prototype.getSelectedNavigationItem = function () {
                    return this.tabs[this.selectedIndex];
                };
                TabBar.prototype.getSelectedIndex = function () {
                    return this.selectedIndex;
                };
                TabBar.prototype.getSize = function () {
                    return this.tabs.length;
                };
                TabBar.prototype.isEmpty = function () {
                    return this.tabs.length === 0;
                };
                TabBar.prototype.getNavigationItems = function () {
                    return this.tabs;
                };
                TabBar.prototype.onNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners.push(listener);
                };
                TabBar.prototype.onNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners.push(listener);
                };
                TabBar.prototype.onNavigationItemDeselected = function (listener) {
                };
                TabBar.prototype.unNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners = this.navigationItemAddedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabBar.prototype.unNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners = this.navigationItemSelectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabBar.prototype.unNavigationItemDeselected = function (listener) {
                };
                TabBar.prototype.notifyTabAddedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemAddedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabBar.prototype.notifyTabShownListeners = function (tab) {
                    var _this = this;
                    this.navigationItemSelectedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                return TabBar;
            })(api.dom.UlEl);
            _tab.TabBar = TabBar;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabBarItem = (function (_super) {
                __extends(TabBarItem, _super);
                function TabBarItem(builder) {
                    _super.call(this, builder, "tab-bar-item");
                }
                return TabBarItem;
            })(tab.TabItem);
            tab.TabBarItem = TabBarItem;
            var TabBarItemBuilder = (function (_super) {
                __extends(TabBarItemBuilder, _super);
                function TabBarItemBuilder() {
                    _super.apply(this, arguments);
                }
                TabBarItemBuilder.prototype.build = function () {
                    return new TabBarItem(this);
                };
                return TabBarItemBuilder;
            })(tab.TabItemBuilder);
            tab.TabBarItemBuilder = TabBarItemBuilder;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabMenu = (function (_super) {
                __extends(TabMenu, _super);
                function TabMenu(className) {
                    var _this = this;
                    _super.call(this, "tab-menu" + (className ? " " + className : ""));
                    this.menuVisible = false;
                    this.tabs = [];
                    this.hideOnItemClick = true;
                    this.navigationItemAddedListeners = [];
                    this.navigationItemRemovedListeners = [];
                    this.navigationItemSelectedListeners = [];
                    this.navigationItemDeselectedListeners = [];
                    this.enabled = true;
                    this.tabMenuButton = this.createTabMenuButton();
                    this.tabMenuButton.hide();
                    this.tabMenuButton.addClass("tab-menu-button");
                    this.tabMenuButton.onClicked(function (event) {
                        if (_this.enabled) {
                            _this.toggleMenu();
                        }
                    });
                    this.appendChild(this.tabMenuButton);
                    this.menuEl = new api.dom.UlEl("menu");
                    this.appendChild(this.menuEl);
                    api.dom.Body.get().onClicked(function (event) {
                        if (!_this.getEl().contains(event.target)) {
                            _this.hideMenu();
                        }
                    });
                    this.onClicked(function (e) {
                        if (_this.enabled) {
                            e.preventDefault();
                            e.stopPropagation();
                            new _tab.HideTabMenuEvent(_this).fire();
                        }
                    });
                }
                TabMenu.prototype.setEnabled = function (enabled) {
                    this.enabled = enabled;
                    this.toggleClass('disabled', !enabled);
                    return this;
                };
                TabMenu.prototype.createTabMenuButton = function () {
                    return new _tab.TabMenuButton();
                };
                TabMenu.prototype.setButtonLabel = function (value) {
                    this.tabMenuButton.setLabel(value);
                    return this;
                };
                TabMenu.prototype.setButtonClass = function (cls) {
                    this.tabMenuButton.addClass(cls);
                    return this;
                };
                TabMenu.prototype.setHideOnItemClick = function (hide) {
                    this.hideOnItemClick = hide;
                    return this;
                };
                TabMenu.prototype.getTabMenuButtonEl = function () {
                    return this.tabMenuButton;
                };
                TabMenu.prototype.getMenuEl = function () {
                    return this.menuEl;
                };
                TabMenu.prototype.toggleMenu = function () {
                    if (!this.menuVisible) {
                        this.showMenu();
                    }
                    else {
                        this.hideMenu();
                    }
                };
                TabMenu.prototype.hideMenu = function () {
                    this.menuEl.hide();
                    this.menuVisible = false;
                    this.removeClass('expanded');
                };
                TabMenu.prototype.showMenu = function () {
                    this.menuEl.show();
                    this.menuVisible = true;
                    this.addClass('expanded');
                };
                TabMenu.prototype.isMenuVisible = function () {
                    return this.menuVisible;
                };
                TabMenu.prototype.insertNavigationItem = function (tab, index) {
                    this.tabs.splice(index, 0, tab);
                    tab.setIndex(index);
                    this.tabs.slice(index + 1).forEach(function (tab) {
                        tab.setIndex(tab.getIndex() + 1);
                    });
                    if (tab.isVisibleInMenu()) {
                        this.menuEl.insertChild(tab, index);
                        this.tabMenuButton.show();
                    }
                    this.initializeNewItemEvents(tab);
                };
                TabMenu.prototype.addNavigationItem = function (tab) {
                    this.insertNavigationItem(tab, this.tabs.length);
                };
                TabMenu.prototype.prependNavigationItem = function (tab) {
                    this.tabs.unshift(tab);
                    this.tabs.forEach(function (curTab) {
                        curTab.setIndex(curTab.getIndex() + 1);
                    });
                    tab.setIndex(0);
                    if (tab.isVisibleInMenu()) {
                        this.menuEl.prependChild(tab);
                        this.tabMenuButton.show();
                    }
                    this.initializeNewItemEvents(tab);
                };
                TabMenu.prototype.initializeNewItemEvents = function (tab) {
                    var _this = this;
                    tab.onSelected(function (event) {
                        _this.selectNavigationItem(event.getTab().getIndex());
                        if (_this.hideOnItemClick) {
                            _this.hideMenu();
                        }
                    });
                    tab.onLabelChanged(function (event) {
                        _this.setButtonLabel(event.getNewValue());
                    });
                    this.notifyTabAddedListeners(tab);
                };
                TabMenu.prototype.isEmpty = function () {
                    return this.tabs.length == 0;
                };
                TabMenu.prototype.getSize = function () {
                    return this.tabs.length;
                };
                TabMenu.prototype.countVisible = function () {
                    var size = 0;
                    this.tabs.forEach(function (tab) {
                        if (tab.isVisibleInMenu()) {
                            size++;
                        }
                    });
                    return size;
                };
                TabMenu.prototype.getSelectedNavigationItem = function () {
                    return this.tabs[this.selectedTab];
                };
                TabMenu.prototype.getSelectedIndex = function () {
                    return this.selectedTab;
                };
                TabMenu.prototype.getNavigationItem = function (tabIndex) {
                    return this.tabs[tabIndex];
                };
                TabMenu.prototype.getNavigationItems = function () {
                    return this.tabs;
                };
                TabMenu.prototype.removeNavigationItem = function (tab) {
                    tab.remove();
                    this.tabs.splice(tab.getIndex(), 1);
                    if (this.isEmpty()) {
                        this.selectedTab = -1;
                    }
                    else if (tab.getIndex() < this.selectedTab) {
                        this.selectedTab--;
                    }
                    else if (tab.getIndex() > this.getSize() - 1) {
                        this.selectedTab = this.getSize() - 1;
                    }
                    for (var i = tab.getIndex(); i < this.tabs.length; i++) {
                        this.tabs[i].setIndex(i);
                    }
                    if (this.countVisible() == 0) {
                        this.setButtonLabel("");
                        this.tabMenuButton.hide();
                        this.hideMenu();
                    }
                    else {
                        var newTab = this.getSelectedNavigationItem();
                        if (newTab) {
                            this.setButtonLabel(newTab.getLabel());
                            this.updateActiveTab(newTab.getIndex());
                            this.selectNavigationItem(newTab.getIndex());
                        }
                    }
                    this.notifyTabRemovedListeners(tab);
                };
                TabMenu.prototype.removeNavigationItems = function () {
                    var _this = this;
                    this.tabs.forEach(function (tab) {
                        tab.remove();
                        _this.notifyTabRemovedListeners(tab);
                    });
                    if (this.countVisible() == 0) {
                        this.setButtonLabel("");
                        this.tabMenuButton.hide();
                        this.hideMenu();
                    }
                    this.selectedTab = -1;
                    this.tabs = [];
                };
                TabMenu.prototype.updateActiveTab = function (tabIndex) {
                    this.tabs.forEach(function (tab, index) {
                        var activate = (tabIndex == index);
                        tab.setActive(activate);
                    });
                };
                TabMenu.prototype.selectNavigationItem = function (tabIndex) {
                    if (tabIndex < 0 || tabIndex >= this.getSize() || this.selectedTab == tabIndex) {
                        return;
                    }
                    this.selectedTab = tabIndex;
                    var selectedTab = this.getNavigationItem(tabIndex);
                    this.setButtonLabel(selectedTab.getLabel());
                    this.updateActiveTab(tabIndex);
                    this.notifyTabSelectedListeners(selectedTab);
                };
                TabMenu.prototype.deselectNavigationItem = function () {
                    var selectedTab = this.getSelectedNavigationItem();
                    this.selectedTab = -1;
                    this.updateActiveTab(this.selectedTab);
                    this.notifyTabDeselectedListeners(selectedTab);
                };
                TabMenu.prototype.onNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners.push(listener);
                };
                TabMenu.prototype.onNavigationItemRemoved = function (listener) {
                    this.navigationItemRemovedListeners.push(listener);
                };
                TabMenu.prototype.onNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners.push(listener);
                };
                TabMenu.prototype.onNavigationItemDeselected = function (listener) {
                    this.navigationItemDeselectedListeners.push(listener);
                };
                TabMenu.prototype.unNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners = this.navigationItemAddedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.unNavigationItemRemoved = function (listener) {
                    this.navigationItemRemovedListeners = this.navigationItemRemovedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.unNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners = this.navigationItemSelectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.unNavigationItemDeselected = function (listener) {
                    this.navigationItemDeselectedListeners = this.navigationItemDeselectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.notifyTabAddedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemAddedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabMenu.prototype.notifyTabRemovedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemRemovedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabMenu.prototype.notifyTabSelectedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemSelectedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabMenu.prototype.notifyTabDeselectedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemDeselectedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                return TabMenu;
            })(api.dom.DivEl);
            _tab.TabMenu = TabMenu;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabMenuButton = (function (_super) {
                __extends(TabMenuButton, _super);
                function TabMenuButton() {
                    _super.call(this, "tab-menu-button icon-arrow-down2");
                    this.labelEl = new api.dom.SpanEl('label');
                    this.appendChild(this.labelEl);
                }
                TabMenuButton.prototype.setLabel = function (value) {
                    this.labelEl.getEl().setInnerHtml(value);
                    this.labelEl.getEl().setAttribute('title', value);
                };
                return TabMenuButton;
            })(api.dom.DivEl);
            tab.TabMenuButton = TabMenuButton;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabMenuItem = (function (_super) {
                __extends(TabMenuItem, _super);
                function TabMenuItem(builder) {
                    _super.call(this, builder.setCloseButtonEnabled(true), "tab-menu-item");
                    this.visibleInMenu = true;
                }
                TabMenuItem.prototype.isVisibleInMenu = function () {
                    return this.visibleInMenu;
                };
                TabMenuItem.prototype.setVisibleInMenu = function (value) {
                    this.visibleInMenu = value;
                    _super.prototype.setVisible.call(this, value);
                };
                return TabMenuItem;
            })(tab.TabItem);
            tab.TabMenuItem = TabMenuItem;
            var TabMenuItemBuilder = (function (_super) {
                __extends(TabMenuItemBuilder, _super);
                function TabMenuItemBuilder() {
                    _super.apply(this, arguments);
                }
                TabMenuItemBuilder.prototype.build = function () {
                    return new TabMenuItem(this);
                };
                return TabMenuItemBuilder;
            })(tab.TabItemBuilder);
            tab.TabMenuItemBuilder = TabMenuItemBuilder;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemEvent = (function () {
                function TabItemEvent(tab) {
                    this.tab = tab;
                }
                TabItemEvent.prototype.getTab = function () {
                    return this.tab;
                };
                return TabItemEvent;
            })();
            _tab.TabItemEvent = TabItemEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemClosedEvent = (function (_super) {
                __extends(TabItemClosedEvent, _super);
                function TabItemClosedEvent(tab) {
                    _super.call(this, tab);
                }
                return TabItemClosedEvent;
            })(_tab.TabItemEvent);
            _tab.TabItemClosedEvent = TabItemClosedEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemLabelChangedEvent = (function (_super) {
                __extends(TabItemLabelChangedEvent, _super);
                function TabItemLabelChangedEvent(tab, oldValue, newValue) {
                    _super.call(this, tab);
                    this.oldValue = oldValue;
                    this.newValue = newValue;
                }
                TabItemLabelChangedEvent.prototype.getOldValue = function () {
                    return this.oldValue;
                };
                TabItemLabelChangedEvent.prototype.getNewValue = function () {
                    return this.newValue;
                };
                return TabItemLabelChangedEvent;
            })(_tab.TabItemEvent);
            _tab.TabItemLabelChangedEvent = TabItemLabelChangedEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemSelectedEvent = (function (_super) {
                __extends(TabItemSelectedEvent, _super);
                function TabItemSelectedEvent(tab) {
                    _super.call(this, tab);
                }
                return TabItemSelectedEvent;
            })(_tab.TabItemEvent);
            _tab.TabItemSelectedEvent = TabItemSelectedEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var HideTabMenuEvent = (function (_super) {
                __extends(HideTabMenuEvent, _super);
                function HideTabMenuEvent(tabMenu) {
                    _super.call(this);
                    this.tabMenu = tabMenu;
                }
                HideTabMenuEvent.prototype.getTabMenu = function () {
                    return this.tabMenu;
                };
                HideTabMenuEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                HideTabMenuEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return HideTabMenuEvent;
            })(api.event.Event);
            tab.HideTabMenuEvent = HideTabMenuEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var UploadItem = (function () {
                function UploadItem(file) {
                    this.failedListeners = [];
                    this.uploadListeners = [];
                    this.progressListeners = [];
                    this.file = file;
                }
                UploadItem.prototype.getId = function () {
                    return this.file.id;
                };
                UploadItem.prototype.setId = function (id) {
                    this.file.id = id;
                    return this;
                };
                UploadItem.prototype.getModel = function () {
                    return this.model;
                };
                UploadItem.prototype.setModel = function (model) {
                    this.model = model;
                    if (model) {
                        this.notifyUploaded(model);
                    }
                    else {
                        this.notifyFailed();
                    }
                    return this;
                };
                UploadItem.prototype.getName = function () {
                    return this.file.name;
                };
                UploadItem.prototype.setName = function (name) {
                    this.file.name = name;
                    return this;
                };
                UploadItem.prototype.getMimeType = function () {
                    return this.file.type;
                };
                UploadItem.prototype.setMimeType = function (type) {
                    this.file.type = type;
                    return this;
                };
                UploadItem.prototype.getSize = function () {
                    return this.file.size;
                };
                UploadItem.prototype.setSize = function (size) {
                    this.file.size = size;
                    return this;
                };
                UploadItem.prototype.getProgress = function () {
                    return this.file.percent;
                };
                UploadItem.prototype.setProgress = function (progress) {
                    this.file.percent = progress;
                    this.notifyProgress(progress);
                    return this;
                };
                UploadItem.prototype.getStatus = function () {
                    return this.file.status;
                };
                UploadItem.prototype.setStatus = function (status) {
                    this.file.status = status;
                    return this;
                };
                UploadItem.prototype.isUploaded = function () {
                    return !!this.model;
                };
                UploadItem.prototype.onProgress = function (listener) {
                    this.progressListeners.push(listener);
                };
                UploadItem.prototype.unProgress = function (listener) {
                    this.progressListeners = this.progressListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                UploadItem.prototype.notifyProgress = function (progress) {
                    this.progressListeners.forEach(function (listener) {
                        listener(progress);
                    });
                };
                UploadItem.prototype.onUploaded = function (listener) {
                    this.uploadListeners.push(listener);
                };
                UploadItem.prototype.unUploaded = function (listener) {
                    this.uploadListeners = this.uploadListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                UploadItem.prototype.notifyUploaded = function (model) {
                    this.uploadListeners.forEach(function (listener) {
                        listener(model);
                    });
                };
                UploadItem.prototype.onFailed = function (listener) {
                    this.failedListeners.push(listener);
                };
                UploadItem.prototype.unFailed = function (listener) {
                    this.failedListeners = this.failedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                UploadItem.prototype.notifyFailed = function () {
                    this.failedListeners.forEach(function (listener) {
                        listener();
                    });
                };
                return UploadItem;
            })();
            uploader.UploadItem = UploadItem;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadStartedEvent = (function () {
                function FileUploadStartedEvent(uploadItems) {
                    this.uploadItems = uploadItems;
                }
                FileUploadStartedEvent.prototype.getUploadItems = function () {
                    return this.uploadItems;
                };
                return FileUploadStartedEvent;
            })();
            uploader.FileUploadStartedEvent = FileUploadStartedEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadProgressEvent = (function () {
                function FileUploadProgressEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                FileUploadProgressEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return FileUploadProgressEvent;
            })();
            uploader.FileUploadProgressEvent = FileUploadProgressEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadedEvent = (function () {
                function FileUploadedEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                FileUploadedEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return FileUploadedEvent;
            })();
            uploader.FileUploadedEvent = FileUploadedEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadCompleteEvent = (function () {
                function FileUploadCompleteEvent(uploadItems) {
                    this.uploadItems = uploadItems;
                }
                FileUploadCompleteEvent.prototype.getUploadItems = function () {
                    return this.uploadItems;
                };
                return FileUploadCompleteEvent;
            })();
            uploader.FileUploadCompleteEvent = FileUploadCompleteEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadFailedEvent = (function () {
                function FileUploadFailedEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                FileUploadFailedEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return FileUploadFailedEvent;
            })();
            uploader.FileUploadFailedEvent = FileUploadFailedEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (_uploader) {
            var Button = api.ui.button.Button;
            var CloseButton = api.ui.button.CloseButton;
            var PluploadStatus = (function () {
                function PluploadStatus() {
                }
                PluploadStatus.QUEUED = plupload.QUEUED;
                PluploadStatus.UPLOADING = plupload.UPLOADING;
                PluploadStatus.FAILED = plupload.FAILED;
                PluploadStatus.DONE = plupload.DONE;
                return PluploadStatus;
            })();
            _uploader.PluploadStatus = PluploadStatus;
            var Uploader = (function (_super) {
                __extends(Uploader, _super);
                function Uploader(config) {
                    var _this = this;
                    _super.call(this, "div", "uploader");
                    this.uploadedItems = [];
                    this.uploadStartedListeners = [];
                    this.uploadProgressListeners = [];
                    this.fileUploadedListeners = [];
                    this.uploadCompleteListeners = [];
                    this.uploadFailedListeners = [];
                    this.uploadResetListeners = [];
                    this.initConfig(config);
                    if (config.showInput) {
                        this.input = api.ui.text.TextInput.middle();
                        this.input.setPlaceholder("Paste URL to image here");
                        this.appendChild(this.input);
                    }
                    if (config.value) {
                        this.value = config.value;
                    }
                    this.dropzoneContainer = new api.dom.DivEl('dropzone-container');
                    this.dropzone = new api.dom.AEl("dropzone");
                    this.dropzone.setId('uploader-dropzone-' + new Date().getTime());
                    this.dropzone.getEl().setTabIndex(-1);
                    this.getEl().setTabIndex(0);
                    this.dropzoneContainer.appendChild(this.dropzone);
                    this.appendChild(this.dropzoneContainer);
                    this.progress = new api.ui.ProgressBar();
                    this.appendChild(this.progress);
                    this.resultContainer = new api.dom.DivEl('result-container');
                    this.appendChild(this.resultContainer);
                    this.cancelBtn = new Button("Cancel");
                    this.cancelBtn.setVisible(this.config.showButtons);
                    this.cancelBtn.onClicked(function (event) {
                        _this.stop();
                        _this.reset();
                    });
                    this.appendChild(this.cancelBtn);
                    this.resetBtn = new CloseButton();
                    this.resetBtn.setVisible(this.config.showButtons);
                    this.resetBtn.onClicked(function (event) {
                        _this.reset();
                    });
                    this.appendChild(this.resetBtn);
                    this.onKeyPressed(function (event) {
                        if (event.keyCode == 13) {
                            wemjq(_this.dropzone.getEl().getHTMLElement()).simulate("click");
                        }
                    });
                    var resetHandler = function () {
                        _this.reset();
                        return false;
                    };
                    ui.KeyBindings.get().bindKeys([
                        new ui.KeyBinding('del', resetHandler),
                        new ui.KeyBinding('backspace', resetHandler)
                    ]);
                    if (this.config.disabled) {
                        if (Uploader.debug) {
                            console.log('Skipping init, because of config.disabled = true', this);
                        }
                    }
                    else {
                        if (this.config.deferred) {
                            this.onShown(function (event) { return _this.initHandler.call(_this, event); });
                        }
                        else {
                            this.onRendered(function (event) { return _this.initHandler.call(_this, event); });
                        }
                    }
                    this.onRemoved(function (event) { return _this.destroyHandler.call(_this, event); });
                }
                Uploader.prototype.initHandler = function () {
                    if (Uploader.debug) {
                        console.log('Initing uploader', this);
                    }
                    if (!this.uploader && this.config.url) {
                        this.uploader = this.initUploader(this.dropzone.getId());
                        if (this.value) {
                            this.setValue(this.value);
                        }
                        else {
                            this.setDropzoneVisible();
                        }
                    }
                };
                Uploader.prototype.destroyHandler = function () {
                    if (Uploader.debug) {
                        console.log('Destroying uploader', this);
                    }
                    if (this.uploader) {
                        this.uploader.destroy();
                        this.uploader = null;
                    }
                };
                Uploader.prototype.initConfig = function (config) {
                    this.config = config;
                    if (this.config.showResult == undefined) {
                        this.config.showResult = true;
                    }
                    if (this.config.allowMultiSelection == undefined) {
                        this.config.allowMultiSelection = false;
                    }
                    if (this.config.showButtons == undefined) {
                        this.config.showButtons = true;
                    }
                    if (this.config.maximumOccurrences == undefined) {
                        this.config.maximumOccurrences = 0;
                    }
                    if (this.config.allowBrowse == undefined) {
                        this.config.allowBrowse = true;
                    }
                    if (this.config.allowDrop == undefined) {
                        this.config.allowDrop = true;
                    }
                    if (this.config.dropzoneAlwaysVisible == undefined) {
                        this.config.dropzoneAlwaysVisible = false;
                    }
                    if (this.config.resultAlwaysVisisble == undefined) {
                        this.config.resultAlwaysVisisble = false;
                    }
                    if (this.config.allowTypes == undefined) {
                        this.config.allowTypes = [];
                    }
                    if (this.config.deferred == undefined) {
                        this.config.deferred = false;
                    }
                    if (this.config.disabled == undefined) {
                        this.config.disabled = false;
                    }
                };
                Uploader.prototype.getName = function () {
                    return this.config.name;
                };
                Uploader.prototype.getValue = function () {
                    return this.value;
                };
                Uploader.prototype.setValue = function (value) {
                    var _this = this;
                    if (Uploader.debug) {
                        console.log('Setting uploader value', value, this);
                    }
                    this.value = value;
                    if (value && this.config.showResult) {
                        this.setResultVisible();
                    }
                    else {
                        this.setDropzoneVisible();
                    }
                    var results = this.getResultContainer().removeChildren();
                    this.parseValues(value).forEach(function (val) {
                        if (val) {
                            results.appendChild(_this.createResultItem(val));
                        }
                    });
                    return this;
                };
                Uploader.prototype.parseValues = function (jsonString) {
                    try {
                        var o = JSON.parse(jsonString);
                        if (o && typeof o === "object" && o.length) {
                            return o;
                        }
                    }
                    catch (e) {
                    }
                    return [jsonString];
                };
                Uploader.prototype.createResultItem = function (value) {
                    throw new Error('Should be overridden by inheritors');
                };
                Uploader.prototype.setMaximumOccurrences = function (value) {
                    this.config.maximumOccurrences = value;
                    return this;
                };
                Uploader.prototype.stop = function () {
                    if (this.uploader) {
                        this.uploader.stop();
                    }
                    return this;
                };
                Uploader.prototype.reset = function () {
                    this.setValue(null);
                    this.notifyUploadReset();
                    return this;
                };
                Uploader.prototype.setDropzoneVisible = function (visible) {
                    if (visible === void 0) { visible = true; }
                    if (!visible && this.config.dropzoneAlwaysVisible) {
                        return;
                    }
                    if (visible) {
                        this.setProgressVisible(false);
                        this.setResultVisible(false);
                    }
                    if (this.input) {
                        this.input.setVisible(visible);
                    }
                    this.dropzoneContainer.setVisible(visible);
                };
                Uploader.prototype.setProgressVisible = function (visible) {
                    if (visible === void 0) { visible = true; }
                    if (visible) {
                        this.setDropzoneVisible(false);
                        this.setResultVisible(false);
                    }
                    this.progress.setVisible(visible);
                    this.cancelBtn.setVisible(visible && this.config.showButtons);
                };
                Uploader.prototype.setResultVisible = function (visible) {
                    if (visible === void 0) { visible = true; }
                    if (!visible && this.config.resultAlwaysVisisble) {
                        return;
                    }
                    if (visible) {
                        this.setDropzoneVisible(false);
                        this.setProgressVisible(false);
                    }
                    this.resultContainer.setVisible(visible);
                    this.resetBtn.setVisible(visible && this.config.showButtons);
                };
                Uploader.prototype.createModel = function (serverResponse) {
                    throw new Error('Should be overridden by inheritors');
                };
                Uploader.prototype.getModelValue = function (item) {
                    throw new Error('Should be overridden by inheritors');
                };
                Uploader.prototype.setParams = function (params) {
                    if (this.uploader) {
                        this.uploader.setOption('multipart_params', params);
                    }
                    else {
                        this.config.params = params;
                    }
                    return this;
                };
                Uploader.prototype.setEnabled = function (enabled) {
                    var _this = this;
                    if (!enabled) {
                        this.dropzone.getEl().setAttribute('disabled', 'true');
                    }
                    else {
                        this.dropzone.getEl().removeAttribute('disabled');
                    }
                    if (!enabled && this.uploader) {
                        if (Uploader.debug) {
                            console.log('Disabling uploader', this);
                        }
                        this.destroyHandler();
                    }
                    else if (enabled && !this.uploader) {
                        if (Uploader.debug) {
                            console.log('Enabling uploader', this);
                        }
                        if (this.config.deferred) {
                            if (this.isVisible()) {
                                this.initHandler();
                            }
                            else {
                                if (Uploader.debug) {
                                    console.log('Deferring enabling uploader until it\' shown', this);
                                }
                                this.onShown(function (event) { return _this.initHandler.call(_this, event); });
                            }
                        }
                        else {
                            if (this.isRendered()) {
                                this.initHandler();
                            }
                            else {
                                if (Uploader.debug) {
                                    console.log('Deferring enabling uploader until it\' rendered', this);
                                }
                                this.onRendered(function (event) { return _this.initHandler.call(_this, event); });
                            }
                        }
                    }
                    return this;
                };
                Uploader.prototype.isEnabled = function () {
                    return this.uploader != null;
                };
                Uploader.prototype.getParams = function () {
                    return this.uploader ? this.uploader.getOption('multipart_params') : this.config.params;
                };
                Uploader.prototype.findUploadItemById = function (id) {
                    for (var i = 0; i < this.uploadedItems.length; i++) {
                        var uploadItem = this.uploadedItems[i];
                        if (uploadItem.getId() == id) {
                            return uploadItem;
                        }
                    }
                    return null;
                };
                Uploader.prototype.isFolder = function (file) {
                    return file.size % 4096 == 0 && api.util.StringHelper.isEmpty(file.type);
                };
                Uploader.prototype.validateFiles = function (up, files) {
                    for (var i = 0; i < files.length; i++) {
                        var file = files[i];
                        if (this.isFolder(file)) {
                            if (Uploader.debug) {
                                console.log('Removing folder [' + file.name + '] because it can\'t be uploaded', this);
                            }
                            files.splice(i, 1);
                            up.splice(i, 1);
                            api.notify.NotifyManager.get().showWarning('Folder [' + file.name + '] upload is not supported yet');
                        }
                    }
                    if (this.config.maximumOccurrences > 0 && files.length > this.config.maximumOccurrences) {
                        if (Uploader.debug) {
                            console.log('Max ' + this.config.maximumOccurrences + ' files allowed, removing the rest', this);
                        }
                        files.splice(this.config.maximumOccurrences);
                        up.splice(this.config.maximumOccurrences);
                        api.notify.NotifyManager.get().showWarning('Max ' + this.config.maximumOccurrences + ' files are allowed');
                    }
                    return files.length > 0;
                };
                Uploader.prototype.initUploader = function (elId) {
                    var _this = this;
                    if (!plupload) {
                        throw new Error("Uploader: plupload not found, check if it is included in page.");
                    }
                    var uploader = new plupload.Uploader({
                        multipart_params: this.config.params,
                        runtimes: 'html5,flash,silverlight,html4',
                        multi_selection: this.config.allowMultiSelection,
                        browse_button: this.config.allowBrowse ? elId : undefined,
                        url: this.config.url,
                        multipart: true,
                        drop_element: this.config.allowDrop ? elId : undefined,
                        flash_swf_url: api.util.UriHelper.getAdminUri('common/js/lib/plupload/js/Moxie.swf'),
                        silverlight_xap_url: api.util.UriHelper.getAdminUri('common/js/lib/plupload/js/Moxie.xap'),
                        filters: {
                            mime_types: this.config.allowTypes
                        }
                    });
                    uploader.bind('Init', function (up, params) {
                        if (Uploader.debug) {
                            console.log('uploader init', up, params);
                        }
                        _this.disableInputFocus();
                    }, this);
                    uploader.bind('FilesAdded', function (up, files) {
                        if (Uploader.debug) {
                            console.log('uploader files added', up, files);
                        }
                        if (!_this.validateFiles(up, files)) {
                            if (Uploader.debug) {
                                console.log('no valid files for upload found', up, files);
                            }
                            return;
                        }
                        files.forEach(function (file) {
                            _this.uploadedItems.push(new _uploader.UploadItem(file));
                        });
                        _this.setProgressVisible();
                        setTimeout(function () {
                            up.start();
                        }, 1);
                        _this.notifyFileUploadStarted(_this.uploadedItems);
                    }, this);
                    uploader.bind('QueueChanged', function (up) {
                        if (Uploader.debug) {
                            console.log('uploader queue changed', up);
                        }
                    }, this);
                    uploader.bind('UploadFile', function (up, file) {
                        if (Uploader.debug) {
                            console.log('uploader upload file', up, file);
                        }
                    }, this);
                    uploader.bind('UploadProgress', function (up, file) {
                        if (Uploader.debug) {
                            console.log('uploader upload progress', up, file);
                        }
                        _this.progress.setValue(file.percent);
                        var uploadItem = _this.findUploadItemById(file.id);
                        if (uploadItem) {
                            uploadItem.setProgress(file.percent);
                            _this.notifyFileUploadProgress(uploadItem);
                        }
                    }, this);
                    uploader.bind('FileUploaded', function (up, file, response) {
                        if (Uploader.debug) {
                            console.log('uploader file uploaded', up, file, response);
                        }
                        if (response && response.status === 200) {
                            try {
                                var uploadItem = _this.findUploadItemById(file.id);
                                if (uploadItem) {
                                    var model = _this.createModel(JSON.parse(response.response));
                                    uploadItem.setModel(model);
                                    _this.notifyFileUploaded(uploadItem);
                                }
                            }
                            catch (e) {
                                console.warn("Failed to parse the response", response, e);
                            }
                        }
                    }, this);
                    uploader.bind('Error', function (up, response) {
                        if (Uploader.debug) {
                            console.log('uploader error', up, response);
                        }
                        try {
                            var responseObj = JSON.parse(response.response);
                            var error = new api.rest.RequestError(responseObj.status, responseObj.cause, responseObj.message);
                            api.DefaultErrorHandler.handle(error);
                        }
                        catch (e) {
                            console.warn("Failed to parse the response", response, e);
                            var files = up.files.map(function (file) { return file.name; });
                            api.notify.NotifyManager.get().showError("File(s) [" + files.join(', ') + "] were not uploaded");
                        }
                        var uploadItem = _this.findUploadItemById(response.file.id);
                        if (uploadItem) {
                            uploadItem.setModel(null);
                            _this.notifyUploadFailed(uploadItem);
                        }
                    }, this);
                    uploader.bind('UploadComplete', function (up, files) {
                        if (Uploader.debug) {
                            console.log('uploader upload complete', up, files);
                        }
                        var values = [];
                        _this.uploadedItems.forEach(function (item) {
                            if (item.getStatus() == PluploadStatus.DONE) {
                                values.push(_this.getModelValue(item.getModel()));
                            }
                        });
                        if (values.length > 0) {
                            _this.setValue(JSON.stringify(values));
                            _this.notifyUploadCompleted(_this.uploadedItems);
                        }
                        _this.uploadedItems.length = 0;
                        _this.uploader.splice();
                    }, this);
                    uploader.init();
                    return uploader;
                };
                Uploader.prototype.disableInputFocus = function () {
                    var focusableElements = this.getDropzoneContainer().getHTMLElement().getElementsByTagName("input");
                    for (var i = 0; i < focusableElements.length; i++) {
                        var el = focusableElements.item(i);
                        el.tabIndex = -1;
                    }
                };
                Uploader.prototype.getResultContainer = function () {
                    return this.resultContainer;
                };
                Uploader.prototype.getDropzoneContainer = function () {
                    return this.dropzoneContainer;
                };
                Uploader.prototype.onUploadStarted = function (listener) {
                    this.uploadStartedListeners.push(listener);
                };
                Uploader.prototype.unUploadStarted = function (listener) {
                    this.uploadStartedListeners = this.uploadStartedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadProgress = function (listener) {
                    this.uploadProgressListeners.push(listener);
                };
                Uploader.prototype.unUploadProgress = function (listener) {
                    this.uploadProgressListeners = this.uploadProgressListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onFileUploaded = function (listener) {
                    this.fileUploadedListeners.push(listener);
                };
                Uploader.prototype.unFileUploaded = function (listener) {
                    this.fileUploadedListeners = this.fileUploadedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadCompleted = function (listener) {
                    this.uploadCompleteListeners.push(listener);
                };
                Uploader.prototype.unUploadCompleted = function (listener) {
                    this.uploadCompleteListeners = this.uploadCompleteListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadReset = function (listener) {
                    this.uploadResetListeners.push(listener);
                };
                Uploader.prototype.unUploadReset = function (listener) {
                    this.uploadResetListeners = this.uploadResetListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadFailed = function (listener) {
                    this.uploadFailedListeners.push(listener);
                };
                Uploader.prototype.unUploadFailed = function (listener) {
                    this.uploadFailedListeners = this.uploadFailedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.notifyFileUploadStarted = function (uploadItems) {
                    this.uploadStartedListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadStartedEvent(uploadItems));
                    });
                };
                Uploader.prototype.notifyFileUploadProgress = function (uploadItem) {
                    this.uploadProgressListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadProgressEvent(uploadItem));
                    });
                };
                Uploader.prototype.notifyFileUploaded = function (uploadItem) {
                    var _this = this;
                    this.fileUploadedListeners.forEach(function (listener) {
                        listener.call(_this, new _uploader.FileUploadedEvent(uploadItem));
                    });
                };
                Uploader.prototype.notifyUploadCompleted = function (uploadItems) {
                    this.uploadCompleteListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadCompleteEvent(uploadItems));
                    });
                };
                Uploader.prototype.notifyUploadReset = function () {
                    var _this = this;
                    this.uploadResetListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                Uploader.prototype.notifyUploadFailed = function (uploadItem) {
                    this.uploadFailedListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadFailedEvent(uploadItem));
                    });
                };
                Uploader.debug = false;
                return Uploader;
            })(api.dom.FormInputEl);
            _uploader.Uploader = Uploader;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (panel) {
            var Panel = (function (_super) {
                __extends(Panel, _super);
                function Panel(className) {
                    var _this = this;
                    _super.call(this, "panel" + (className ? " " + className : ""));
                    this.doOffset = true;
                    this.onShown(function (event) {
                        if (_this.doOffset) {
                            _this.calculateOffset();
                        }
                    });
                }
                Panel.prototype.setDoOffset = function (value) {
                    this.doOffset = value;
                    if (value && this.isRendered()) {
                        this.calculateOffset();
                    }
                };
                Panel.prototype.calculateOffset = function () {
                    var previous = this.getEl().getPrevious();
                    var top = previous ? (previous.getOffsetTopRelativeToParent() + previous.getHeightWithMargin()) : 0;
                    this.getEl().setTopPx(top);
                };
                return Panel;
            })(api.dom.DivEl);
            panel.Panel = Panel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var PanelShownEvent = (function () {
                function PanelShownEvent(panel, index, previousPanel) {
                    this.panel = panel;
                    this.index = index;
                    this.previousPanel = previousPanel;
                }
                PanelShownEvent.prototype.getPanel = function () {
                    return this.panel;
                };
                PanelShownEvent.prototype.getIndex = function () {
                    return this.index;
                };
                PanelShownEvent.prototype.getPreviousPanel = function () {
                    return this.previousPanel;
                };
                return PanelShownEvent;
            })();
            _panel.PanelShownEvent = PanelShownEvent;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var DeckPanel = (function (_super) {
                __extends(DeckPanel, _super);
                function DeckPanel(className) {
                    _super.call(this, "deck-panel" + (className ? " " + className : ""));
                    this.panels = [];
                    this.panelShown = null;
                    this.panelShownListeners = [];
                }
                DeckPanel.prototype.isEmpty = function () {
                    return this.panels.length == 0;
                };
                DeckPanel.prototype.getSize = function () {
                    return this.panels.length;
                };
                DeckPanel.prototype.addPanel = function (panel) {
                    panel.hide();
                    panel.setDoOffset(false);
                    this.appendChild(panel);
                    return this.panels.push(panel) - 1;
                };
                DeckPanel.prototype.getPanel = function (index) {
                    return this.panels[index];
                };
                DeckPanel.prototype.getLastPanel = function () {
                    return this.getPanel(this.getSize() - 1) || null;
                };
                DeckPanel.prototype.getPanelShown = function () {
                    return this.panelShown;
                };
                DeckPanel.prototype.getPanelShownIndex = function () {
                    return this.getPanelIndex(this.panelShown);
                };
                DeckPanel.prototype.getPanelIndex = function (panel) {
                    var size = this.getSize();
                    for (var i = 0; i < size; i++) {
                        if (this.panels[i] === panel) {
                            return i;
                        }
                    }
                    return -1;
                };
                DeckPanel.prototype.removePanelByIndex = function (index, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var panelToRemove = this.getPanel(index);
                    return this.removePanel(panelToRemove, checkCanRemovePanel) ? panelToRemove : null;
                };
                DeckPanel.prototype.removePanel = function (panelToRemove, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var index = this.getPanelIndex(panelToRemove);
                    if (index < 0 || checkCanRemovePanel && !this.canRemovePanel(panelToRemove)) {
                        return -1;
                    }
                    panelToRemove.remove();
                    this.panels.splice(index, 1);
                    if (this.isEmpty()) {
                        this.panelShown = null;
                    }
                    else if (panelToRemove == this.getPanelShown()) {
                        this.showPanelByIndex(Math.min(index, this.getSize() - 1));
                    }
                    return index;
                };
                DeckPanel.prototype.canRemovePanel = function (panel) {
                    return true;
                };
                DeckPanel.prototype.showPanel = function (panel) {
                    var index = this.getPanelIndex(panel);
                    if (index > -1) {
                        this.showPanelByIndex(index);
                    }
                };
                DeckPanel.prototype.showPanelByIndex = function (index) {
                    var previousPanel = this.getPanelShown();
                    var panelToShow = this.getPanel(index);
                    if (panelToShow == null) {
                        return;
                    }
                    if (this.panelShown != null) {
                        this.panelShown.hide();
                    }
                    panelToShow.show();
                    this.panelShown = panelToShow;
                    this.notifyPanelShown(panelToShow, index, previousPanel);
                };
                DeckPanel.prototype.onPanelShown = function (listener) {
                    this.panelShownListeners.push(listener);
                };
                DeckPanel.prototype.unPanelShown = function (listener) {
                    this.panelShownListeners = this.panelShownListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                DeckPanel.prototype.notifyPanelShown = function (panel, panelIndex, previousPanel) {
                    var _this = this;
                    this.panelShownListeners.forEach(function (listener) {
                        listener.call(_this, new _panel.PanelShownEvent(panel, panelIndex, previousPanel));
                    });
                };
                return DeckPanel;
            })(_panel.Panel);
            _panel.DeckPanel = DeckPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var PanelStrip = (function (_super) {
                __extends(PanelStrip, _super);
                function PanelStrip(scrollable, className) {
                    var _this = this;
                    _super.call(this, "panel-strip" + (className ? " " + className : ""));
                    this.panels = [];
                    this.headers = [];
                    this.offset = 0;
                    this.panelShown = null;
                    this.panelShownListeners = [];
                    if (scrollable) {
                        this.scrollable = scrollable;
                        this.scrollable.addClass('panel-strip-scrollable');
                    }
                    else {
                        this.scrollable = this;
                    }
                    this.onShown(function () {
                        _this.updateLastPanelHeight();
                    });
                }
                PanelStrip.prototype.countExistingChildren = function (index) {
                    var count = Math.min(this.panels.length, index);
                    for (var i = 0; i < Math.min(this.headers.length, index); i++) {
                        if (this.headers[i]) {
                            count++;
                        }
                    }
                    return count;
                };
                PanelStrip.prototype.insertPanel = function (panel, index, header) {
                    panel.setDoOffset(false);
                    var previousChildrenIndex = this.countExistingChildren(index);
                    if (header) {
                        var headerEl = new api.dom.H2El("panel-strip-panel-header");
                        headerEl.getEl().setInnerHtml(header);
                        this.insertChild(headerEl, previousChildrenIndex);
                    }
                    this.panels.splice(index, 0, panel);
                    this.headers.splice(index, 0, headerEl);
                    if (header) {
                        this.insertChild(panel, previousChildrenIndex + 1);
                    }
                    else {
                        this.insertChild(panel, previousChildrenIndex);
                    }
                    if (this.isVisible()) {
                        this.updateLastPanelHeight();
                    }
                    return index;
                };
                PanelStrip.prototype.getPanels = function () {
                    return this.panels;
                };
                PanelStrip.prototype.getScrollable = function () {
                    return this.scrollable;
                };
                PanelStrip.prototype.setScrollOffset = function (offset) {
                    this.offset = offset;
                    return this;
                };
                PanelStrip.prototype.getScrollOffset = function () {
                    return this.offset;
                };
                PanelStrip.prototype.updateLastPanelHeight = function () {
                    if (this.getSize() == 0) {
                        return;
                    }
                    if (this.getSize() > 1) {
                        var beforeLastEl = this.getPanel(this.getSize() - 2).getEl();
                        var originalHeight = beforeLastEl.getData("originalHeight");
                        if (originalHeight) {
                            beforeLastEl.setHeight(originalHeight);
                        }
                    }
                    var lastEl = this.getPanel(this.getSize() - 1).getEl();
                    if (!lastEl.getData("originalHeight")) {
                        lastEl.setData("originalHeight", lastEl.getHTMLElement().style.height || "auto");
                    }
                    lastEl.setHeightPx(this.scrollable.getEl().getHeight());
                };
                PanelStrip.prototype.removePanel = function (panelToRemove, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var index = this.getPanelIndex(panelToRemove);
                    if (index < 0 || checkCanRemovePanel && !this.canRemovePanel(panelToRemove)) {
                        return -1;
                    }
                    this.removeChild(panelToRemove);
                    this.removeChild(this.getHeader(index));
                    this.panels.splice(index, 1);
                    this.headers.splice(index, 1);
                    if (this.isEmpty()) {
                        this.panelShown = null;
                    }
                    else if (panelToRemove == this.getPanelShown()) {
                        this.showPanelByIndex(Math.min(index, this.getSize() - 1));
                    }
                    if (this.isVisible() && index == this.getSize() && !this.isEmpty()) {
                        this.updateLastPanelHeight();
                    }
                    return index;
                };
                PanelStrip.prototype.canRemovePanel = function (panel) {
                    return true;
                };
                PanelStrip.prototype.isEmpty = function () {
                    return this.panels.length == 0;
                };
                PanelStrip.prototype.getSize = function () {
                    return this.panels.length;
                };
                PanelStrip.prototype.getPanel = function (index) {
                    return this.panels[index];
                };
                PanelStrip.prototype.getHeader = function (index) {
                    return this.headers[index];
                };
                PanelStrip.prototype.getPanelShown = function () {
                    return this.panelShown;
                };
                PanelStrip.prototype.getPanelShownIndex = function () {
                    return this.getPanelIndex(this.panelShown);
                };
                PanelStrip.prototype.getPanelIndex = function (panel) {
                    var size = this.getSize();
                    for (var i = 0; i < size; i++) {
                        if (this.panels[i] === panel) {
                            return i;
                        }
                    }
                    return -1;
                };
                PanelStrip.prototype.showPanel = function (panel) {
                    var index = this.getPanelIndex(panel);
                    if (index > -1) {
                        this.showPanelByIndex(index);
                    }
                };
                PanelStrip.prototype.showPanelByIndex = function (index) {
                    var _this = this;
                    var panelToShow = this.getPanel(index);
                    if (panelToShow == null) {
                        return;
                    }
                    wemjq(this.scrollable.getHTMLElement()).animate({
                        scrollTop: index == 0 ? 0 : this.scrollable.getHTMLElement().scrollTop - this.offset + panelToShow.getEl().getOffsetToParent().top - this.headers[index].getEl().getHeightWithBorder()
                    }, {
                        duration: 500,
                        complete: function () {
                            _this.notifyPanelShown(panelToShow, index, _this.getPanelShown());
                            _this.panelShown = panelToShow;
                        }
                    });
                };
                PanelStrip.prototype.onPanelShown = function (listener) {
                    this.panelShownListeners.push(listener);
                };
                PanelStrip.prototype.unPanelShown = function (listener) {
                    this.panelShownListeners = this.panelShownListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                PanelStrip.prototype.notifyPanelShown = function (panel, panelIndex, previousPanel) {
                    var _this = this;
                    this.panelShownListeners.forEach(function (listener) {
                        listener.call(_this, new _panel.PanelShownEvent(panel, panelIndex, previousPanel));
                    });
                };
                return PanelStrip;
            })(_panel.Panel);
            _panel.PanelStrip = PanelStrip;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (panel) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            (function (SplitPanelAlignment) {
                SplitPanelAlignment[SplitPanelAlignment["HORIZONTAL"] = 0] = "HORIZONTAL";
                SplitPanelAlignment[SplitPanelAlignment["VERTICAL"] = 1] = "VERTICAL";
            })(panel.SplitPanelAlignment || (panel.SplitPanelAlignment = {}));
            var SplitPanelAlignment = panel.SplitPanelAlignment;
            (function (SplitPanelUnit) {
                SplitPanelUnit[SplitPanelUnit["PIXEL"] = 0] = "PIXEL";
                SplitPanelUnit[SplitPanelUnit["PERCENT"] = 1] = "PERCENT";
            })(panel.SplitPanelUnit || (panel.SplitPanelUnit = {}));
            var SplitPanelUnit = panel.SplitPanelUnit;
            var SplitPanelBuilder = (function () {
                function SplitPanelBuilder(firstPanel, secondPanel) {
                    this.firstPanelSize = 50;
                    this.firstPanelMinSize = 0;
                    this.firstPanelUnit = 1 /* PERCENT */;
                    this.secondPanelSize = 50;
                    this.secondPanelMinSize = 0;
                    this.secondPanelUnit = 1 /* PERCENT */;
                    this.alignment = 0 /* HORIZONTAL */;
                    this.splitterThickness = 5;
                    this.firstPanelIsDecidingPanel = true;
                    this.firstPanel = firstPanel;
                    this.secondPanel = secondPanel;
                }
                SplitPanelBuilder.prototype.build = function () {
                    return new SplitPanel(this);
                };
                SplitPanelBuilder.prototype.setFirstPanelSize = function (size, unit) {
                    this.firstPanelSize = size;
                    this.firstPanelUnit = unit;
                    this.firstPanelIsDecidingPanel = true;
                    return this;
                };
                SplitPanelBuilder.prototype.setFirstPanelMinSize = function (size, unit) {
                    this.firstPanelMinSize = size;
                    this.firstPanelUnit = unit;
                    this.firstPanelIsDecidingPanel = false;
                    return this;
                };
                SplitPanelBuilder.prototype.setSecondPanelSize = function (size, unit) {
                    this.secondPanelSize = size;
                    this.secondPanelUnit = unit;
                    return this;
                };
                SplitPanelBuilder.prototype.setSecondPanelMinSize = function (size, unit) {
                    this.secondPanelMinSize = size;
                    this.secondPanelUnit = unit;
                    return this;
                };
                SplitPanelBuilder.prototype.setAlignment = function (alignment) {
                    this.alignment = alignment;
                    return this;
                };
                SplitPanelBuilder.prototype.setAlignmentTreshold = function (treshold) {
                    this.alignmentTreshold = treshold;
                    return this;
                };
                SplitPanelBuilder.prototype.setSplitterThickness = function (thickness) {
                    this.splitterThickness = thickness;
                    return this;
                };
                SplitPanelBuilder.prototype.getFirstPanel = function () {
                    return this.firstPanel;
                };
                SplitPanelBuilder.prototype.getFirstPanelMinSize = function () {
                    return this.firstPanelMinSize;
                };
                SplitPanelBuilder.prototype.getSecondPanel = function () {
                    return this.secondPanel;
                };
                SplitPanelBuilder.prototype.getSecondPanelMinSize = function () {
                    return this.secondPanelMinSize;
                };
                SplitPanelBuilder.prototype.getFirstPanelSize = function () {
                    return this.firstPanelSize;
                };
                SplitPanelBuilder.prototype.getSecondPanelSize = function () {
                    return this.secondPanelSize;
                };
                SplitPanelBuilder.prototype.getAlignment = function () {
                    return this.alignment;
                };
                SplitPanelBuilder.prototype.getAlignmentTreshold = function () {
                    return this.alignmentTreshold;
                };
                SplitPanelBuilder.prototype.getSplitterThickness = function () {
                    return this.splitterThickness;
                };
                SplitPanelBuilder.prototype.getFirstPanelUnit = function () {
                    return this.firstPanelUnit;
                };
                SplitPanelBuilder.prototype.getSecondPanelUnit = function () {
                    return this.secondPanelUnit;
                };
                SplitPanelBuilder.prototype.isFirstPanelDecidingPanel = function () {
                    return this.firstPanelIsDecidingPanel;
                };
                return SplitPanelBuilder;
            })();
            panel.SplitPanelBuilder = SplitPanelBuilder;
            var SplitPanel = (function (_super) {
                __extends(SplitPanel, _super);
                function SplitPanel(builder) {
                    var _this = this;
                    _super.call(this, "split-panel");
                    this.firstPanelSize = -1;
                    this.secondPanelSize = -1;
                    this.firstPanel = builder.getFirstPanel();
                    this.firstPanelMinSize = builder.getFirstPanelMinSize();
                    this.firstPanelUnit = builder.getFirstPanelUnit();
                    this.secondPanel = builder.getSecondPanel();
                    this.secondPanelMinSize = builder.getSecondPanelMinSize();
                    this.secondPanelUnit = builder.getSecondPanelUnit();
                    if (builder.isFirstPanelDecidingPanel()) {
                        this.setFirstPanelSize(builder.getFirstPanelSize(), this.firstPanelUnit);
                    }
                    else {
                        this.setSecondPanelSize(builder.getSecondPanelSize(), this.secondPanelUnit);
                    }
                    this.alignment = builder.getAlignment();
                    this.alignmentTreshold = builder.getAlignmentTreshold();
                    this.splitterThickness = builder.getSplitterThickness();
                    this.splitter = new api.dom.DivEl("splitter");
                    this.firstPanel.setDoOffset(false);
                    this.secondPanel.setDoOffset(false);
                    this.appendChild(this.firstPanel);
                    this.appendChild(this.splitter);
                    this.appendChild(this.secondPanel);
                    this.ghostDragger = new api.dom.DivEl("ghost-dragger");
                    this.mask = new api.ui.mask.DragMask(this);
                    this.appendChild(this.mask);
                    var initialPos = 0;
                    this.dragListener = function (e) {
                        if (_this.isHorizontal()) {
                            if (_this.splitterWithinBoundaries(initialPos - e.clientY)) {
                                _this.splitterPosition = e.clientY;
                                _this.ghostDragger.getEl().setTopPx(e.clientY - _this.getEl().getOffsetTop());
                            }
                        }
                        else {
                            if (_this.splitterWithinBoundaries(initialPos - e.clientX)) {
                                _this.splitterPosition = e.clientX;
                                _this.ghostDragger.getEl().setLeftPx(e.clientX - _this.getEl().getOffsetLeft());
                            }
                        }
                    };
                    this.splitter.onMouseDown(function (e) {
                        e.preventDefault();
                        if (_this.isHorizontal()) {
                            initialPos = e.clientY;
                        }
                        else {
                            initialPos = e.clientX;
                        }
                        _this.ghostDragger.insertBeforeEl(_this.splitter);
                        _this.startDrag();
                    });
                    this.onMouseUp(function (e) {
                        if (_this.ghostDragger.getHTMLElement().parentNode) {
                            _this.stopDrag(e);
                            _this.removeChild(_this.ghostDragger);
                        }
                    });
                    if (this.alignmentTreshold) {
                        api.dom.WindowDOM.get().onResized(function (event) { return _this.updateAlignment(); }, this);
                    }
                    this.onShown(function (event) {
                        var splitPanelSize = _this.isHorizontal() ? _this.getEl().getHeight() : _this.getEl().getWidth();
                        api.util.assert(_this.firstPanelMinSize + _this.secondPanelMinSize <= splitPanelSize, "warning: total sum of first and second panel minimum sizes exceed total split panel size");
                        _this.updateAlignment();
                    });
                    ResponsiveManager.onAvailableSizeChanged(this);
                    ResponsiveManager.onAvailableSizeChanged(this.firstPanel);
                    ResponsiveManager.onAvailableSizeChanged(this.secondPanel);
                    this.onRemoved(function (event) {
                        ResponsiveManager.unAvailableSizeChanged(_this);
                        ResponsiveManager.unAvailableSizeChanged(_this.firstPanel);
                        ResponsiveManager.unAvailableSizeChanged(_this.secondPanel);
                    });
                }
                SplitPanel.prototype.startDrag = function () {
                    this.mask.show();
                    this.addClass("dragging");
                    this.onMouseMove(this.dragListener);
                    if (this.isHorizontal()) {
                        this.ghostDragger.getEl().setTopPx(this.splitter.getEl().getOffsetTopRelativeToParent()).setLeft(null);
                    }
                    else {
                        this.ghostDragger.getEl().setLeftPx(this.splitter.getEl().getOffsetLeftRelativeToParent()).setTop(null);
                    }
                };
                SplitPanel.prototype.stopDrag = function (e) {
                    this.mask.hide();
                    this.removeClass("dragging");
                    this.unMouseMove(this.dragListener);
                    var splitPanelEl = this.getEl();
                    var dragOffset = this.isHorizontal() ? this.splitterPosition - splitPanelEl.getOffsetTop() : this.splitterPosition - splitPanelEl.getOffsetLeft();
                    var splitPanelSize = this.isHorizontal() ? splitPanelEl.getHeightWithBorder() : splitPanelEl.getWidthWithBorder();
                    if (this.firstPanelUnit == 1 /* PERCENT */) {
                        this.firstPanelSize = (dragOffset / splitPanelSize) * 100;
                    }
                    else {
                        this.firstPanelSize = dragOffset;
                    }
                    this.distribute();
                };
                SplitPanel.prototype.splitterWithinBoundaries = function (offset) {
                    var firstPanelSize = this.isHorizontal() ? this.firstPanel.getEl().getHeight() : this.firstPanel.getEl().getWidth();
                    var secondPanelSize = this.isHorizontal() ? this.secondPanel.getEl().getHeight() : this.secondPanel.getEl().getWidth();
                    var newFirstPanelWidth = firstPanelSize - offset;
                    var newSecondPanelWidth = secondPanelSize + offset;
                    return (newFirstPanelWidth >= this.firstPanelMinSize) && (newSecondPanelWidth >= this.secondPanelMinSize);
                };
                SplitPanel.prototype.updateAlignment = function () {
                    var splitPanelWidth = this.getEl().getWidthWithMargin();
                    if (splitPanelWidth > this.alignmentTreshold && this.isHorizontal()) {
                        this.alignment = 1 /* VERTICAL */;
                    }
                    else if (splitPanelWidth < this.alignmentTreshold && !this.isHorizontal()) {
                        this.alignment = 0 /* HORIZONTAL */;
                    }
                    if (this.isHorizontal()) {
                        this.removeClass("vertical");
                        this.addClass("horizontal");
                        this.firstPanel.getEl().setWidth(null);
                        this.secondPanel.getEl().setWidth(null);
                        this.splitter.getEl().setHeightPx(this.splitterThickness).setWidth(null).setLeft(null);
                    }
                    else {
                        this.addClass("vertical");
                        this.removeClass("horizontal");
                        this.firstPanel.getEl().setHeight(null);
                        this.secondPanel.getEl().setHeight(null);
                        this.splitter.getEl().setWidthPx(this.splitterThickness).setHeight(null);
                    }
                    this.distribute();
                };
                SplitPanel.prototype.setFirstPanelSize = function (size, unit) {
                    this.firstPanelSize = size;
                    this.secondPanelSize = -1;
                    if (unit) {
                        this.firstPanelUnit = unit;
                    }
                };
                SplitPanel.prototype.setSecondPanelSize = function (size, unit) {
                    this.secondPanelSize = size;
                    this.firstPanelSize = -1;
                    if (unit) {
                        this.secondPanelUnit = unit;
                    }
                };
                SplitPanel.prototype.distribute = function () {
                    if (this.isHorizontal()) {
                        this.firstPanel.getEl().setHeight(this.getPanelSizeString(1)).setWidth(null);
                        this.secondPanel.getEl().setHeight(this.getPanelSizeString(2)).setWidth(null);
                        ResponsiveManager.fireResizeEvent();
                    }
                    else {
                        this.firstPanel.getEl().setWidth(this.getPanelSizeString(1)).setHeight(null);
                        this.secondPanel.getEl().setWidth(this.getPanelSizeString(2)).setHeight(null);
                        ResponsiveManager.fireResizeEvent();
                        if (this.firstPanelUnit == 1 /* PERCENT */) {
                            var positionInPercentage = (this.firstPanelSize != -1) ? this.firstPanelSize : 100 - this.secondPanelSize;
                            this.splitter.getEl().setLeft("calc(" + positionInPercentage + "% - " + (this.splitterThickness / 2) + "px)");
                        }
                        else {
                            this.splitter.getEl().setLeft(this.getPanelSizeString(1));
                        }
                    }
                };
                SplitPanel.prototype.isHorizontal = function () {
                    return this.alignment == 0 /* HORIZONTAL */;
                };
                SplitPanel.prototype.getPanelSizeString = function (panelNumber) {
                    api.util.assert((panelNumber == 1 || panelNumber == 2), "Panel number must be 1 or 2");
                    var size = (panelNumber == 1) ? this.firstPanelSize : this.secondPanelSize;
                    var otherPanelSize = (panelNumber == 1) ? this.secondPanelSize : this.firstPanelSize;
                    var unit = (panelNumber == 1) ? this.firstPanelUnit : this.secondPanelUnit;
                    var otherPanelUnit = (panelNumber == 1) ? this.secondPanelUnit : this.firstPanelUnit;
                    var result;
                    if (size != -1) {
                        if (unit == 0 /* PIXEL */) {
                            result = size - (this.splitterThickness / 2) + "px";
                        }
                        else {
                            result = "calc(" + size + "%" + " - " + (this.splitterThickness / 2) + "px)";
                        }
                    }
                    else {
                        if (otherPanelUnit == 0 /* PIXEL */) {
                            result = "calc(100% - " + (otherPanelSize + (this.splitterThickness / 2)) + "px)";
                        }
                        else {
                            result = "calc(" + (100 - otherPanelSize) + "%" + " - " + (this.splitterThickness / 2) + "px)";
                        }
                    }
                    return result;
                };
                SplitPanel.prototype.showPanel = function (panelNumber) {
                    api.util.assert((panelNumber == 1 || panelNumber == 2), "Panel number must be 1 or 2");
                    if (!this.isPanelHidden(panelNumber)) {
                        return;
                    }
                    this.splitterThickness = this.previousSplitterThickness;
                    this.splitter.show();
                    if (panelNumber == 1) {
                        this.firstPanelSize = this.hiddenPanelPreviousSize;
                        this.firstPanel.show();
                    }
                    else {
                        this.secondPanelSize = this.hiddenPanelPreviousSize;
                        this.secondPanel.show();
                    }
                    this.hiddenPanel = -1;
                    this.distribute();
                };
                SplitPanel.prototype.hidePanel = function (panelNumber) {
                    api.util.assert((panelNumber == 1 || panelNumber == 2), "Panel number must be 1 or 2");
                    if (this.isPanelHidden(panelNumber)) {
                        return;
                    }
                    this.previousSplitterThickness = this.splitterThickness;
                    this.splitterThickness = 0;
                    this.splitter.hide();
                    this.hiddenPanel = panelNumber;
                    if (panelNumber == 1) {
                        this.hiddenPanelPreviousSize = this.firstPanelSize;
                        this.firstPanelSize = 0;
                        this.firstPanel.hide();
                    }
                    else {
                        this.hiddenPanelPreviousSize = this.secondPanelSize;
                        this.secondPanelSize = 0;
                        this.secondPanel.hide();
                    }
                    this.distribute();
                };
                SplitPanel.prototype.isPanelHidden = function (panelNumber) {
                    return this.hiddenPanel == panelNumber;
                };
                SplitPanel.prototype.getUnitString = function (panelNumber) {
                    api.util.assert((panelNumber == 1 || panelNumber == 2), "Panel number must be 1 or 2");
                    var unit = (panelNumber == 1) ? this.firstPanelUnit : this.secondPanelUnit;
                    if (unit == 0 /* PIXEL */) {
                        return "px";
                    }
                    else {
                        return "%";
                    }
                };
                return SplitPanel;
            })(panel.Panel);
            panel.SplitPanel = SplitPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var NavigatedDeckPanel = (function (_super) {
                __extends(NavigatedDeckPanel, _super);
                function NavigatedDeckPanel(navigator, className) {
                    var _this = this;
                    _super.call(this, className);
                    this.navigator = navigator;
                    navigator.onNavigationItemSelected(function (event) {
                        _this.showPanelByIndex(event.getItem().getIndex());
                    });
                }
                NavigatedDeckPanel.prototype.getSelectedNavigationItem = function () {
                    return this.navigator.getSelectedNavigationItem();
                };
                NavigatedDeckPanel.prototype.addNavigablePanel = function (item, panel, select) {
                    this.navigator.addNavigationItem(item);
                    var index = this.addPanel(panel);
                    if (select) {
                        this.selectPanelByIndex(index);
                    }
                    return index;
                };
                NavigatedDeckPanel.prototype.selectPanel = function (item) {
                    this.selectPanelByIndex(item.getIndex());
                };
                NavigatedDeckPanel.prototype.selectPanelByIndex = function (index) {
                    this.navigator.selectNavigationItem(index);
                };
                NavigatedDeckPanel.prototype.removeNavigablePanel = function (panel, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var index = this.removePanel(panel, checkCanRemovePanel);
                    if (index > -1) {
                        var navigationItem = this.navigator.getNavigationItem(index);
                        this.navigator.removeNavigationItem(navigationItem);
                    }
                    return index;
                };
                NavigatedDeckPanel.prototype.getNavigator = function () {
                    return this.navigator;
                };
                return NavigatedDeckPanel;
            })(_panel.DeckPanel);
            _panel.NavigatedDeckPanel = NavigatedDeckPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var NavigatedPanelStrip = (function (_super) {
                __extends(NavigatedPanelStrip, _super);
                function NavigatedPanelStrip(navigator, scrollable, className) {
                    var _this = this;
                    _super.call(this, scrollable, className);
                    this.scrollIndex = -1;
                    this.focusIndex = -1;
                    this.focusVisible = false;
                    this.navigator = navigator;
                    var listenToScroll = true;
                    navigator.onNavigationItemSelected(function (event) {
                        listenToScroll = false;
                        _this.showPanelByIndex(event.getItem().getIndex());
                    });
                    this.onPanelShown(function (event) {
                        listenToScroll = true;
                    });
                    this.getScrollable().onScroll(function (event) {
                        if (listenToScroll) {
                            _this.updateScrolledNavigationItem();
                        }
                    });
                }
                NavigatedPanelStrip.prototype.updateScrolledNavigationItem = function () {
                    var scrollTop = this.getScrollable().getHTMLElement().scrollTop;
                    var focusVisible = this.isFocusedPanelVisible(scrollTop);
                    var scrollIndex = this.getScrolledPanelIndex(scrollTop);
                    if (this.scrollIndex != scrollIndex || this.focusVisible != focusVisible) {
                        if (focusVisible) {
                            this.navigator.selectNavigationItem(this.focusIndex, true);
                        }
                        else {
                            this.navigator.selectNavigationItem(scrollIndex, true);
                        }
                        this.focusVisible = focusVisible;
                        this.scrollIndex = scrollIndex;
                    }
                };
                NavigatedPanelStrip.prototype.isFocusedPanelVisible = function (scrollTop) {
                    if (this.focusIndex < 0) {
                        return false;
                    }
                    var totalHeight = this.getScrollable().getEl().getHeight(), headerHeight = this.getFocusedHeaderHeight(this.focusIndex), panelEl = this.getPanel(this.focusIndex).getEl(), panelTop = panelEl.getOffsetToParent().top - this.getScrollOffset() - headerHeight, panelBottom = panelTop + panelEl.getHeight() - headerHeight;
                    return panelEl.isVisible() && ((panelTop <= 0 && panelBottom > 0) || (panelTop <= totalHeight && panelBottom > totalHeight));
                };
                NavigatedPanelStrip.prototype.getScrolledPanelIndex = function (scrollTop) {
                    var panelEl, panelTop, panelBottom;
                    if (scrollTop == 0) {
                        return 0;
                    }
                    for (var i = 0; i < this.getSize(); i++) {
                        panelEl = this.getPanel(i).getEl();
                        var headerHeight = this.getFocusedHeaderHeight(i);
                        if (panelEl.isVisible()) {
                            panelTop = scrollTop + panelEl.getOffsetToParent().top - this.getScrollOffset() - headerHeight;
                            panelBottom = panelTop + panelEl.getHeight();
                            if (scrollTop >= panelTop && scrollTop < panelBottom) {
                                return i;
                            }
                        }
                    }
                    return -1;
                };
                NavigatedPanelStrip.prototype.getSelectedNavigationItem = function () {
                    return this.navigator.getSelectedNavigationItem();
                };
                NavigatedPanelStrip.prototype.insertNavigablePanel = function (item, panel, header, index, select) {
                    var _this = this;
                    this.navigator.insertNavigationItem(item, index);
                    var panelHeader = select ? null : header;
                    _super.prototype.insertPanel.call(this, panel, index, panelHeader);
                    panel.onFocus(function (event) {
                        _this.navigator.selectNavigationItem(item.getIndex(), true);
                        _this.focusIndex = item.getIndex();
                    });
                    panel.onBlur(function (event) {
                        _this.focusIndex = -1;
                        _this.updateScrolledNavigationItem();
                    });
                    if (select) {
                        this.selectPanel(item);
                    }
                    return index;
                };
                NavigatedPanelStrip.prototype.addNavigablePanel = function (item, panel, header, select) {
                    return this.insertNavigablePanel(item, panel, header, this.getPanels().length, select);
                };
                NavigatedPanelStrip.prototype.selectPanel = function (item) {
                    this.selectPanelByIndex(item.getIndex());
                };
                NavigatedPanelStrip.prototype.selectPanelByIndex = function (index) {
                    this.navigator.selectNavigationItem(index);
                };
                NavigatedPanelStrip.prototype.removeNavigablePanel = function (panel, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var removedPanelIndex = _super.prototype.removePanel.call(this, panel, checkCanRemovePanel);
                    if (removedPanelIndex > -1) {
                        var navigationItem = this.navigator.getNavigationItem(removedPanelIndex);
                        this.navigator.removeNavigationItem(navigationItem);
                    }
                    return removedPanelIndex;
                };
                NavigatedPanelStrip.prototype.getFocusedHeaderHeight = function (curStrip) {
                    return this.getHeader(curStrip + 1) ? this.getHeader(curStrip + 1).getEl().getHeightWithBorder() : this.getHeader(curStrip).getEl().getHeightWithBorder();
                };
                return NavigatedPanelStrip;
            })(_panel.PanelStrip);
            _panel.NavigatedPanelStrip = NavigatedPanelStrip;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var TabBar = api.ui.tab.TabBar;
            var DockedPanel = (function (_super) {
                __extends(DockedPanel, _super);
                function DockedPanel() {
                    _super.call(this, "docked-panel");
                    this.items = [];
                    this.navigator = new TabBar();
                    this.deck = new _panel.NavigatedDeckPanel(this.navigator);
                    this.appendChild(this.navigator);
                    this.appendChild(this.deck);
                    this.setDoOffset(false);
                }
                DockedPanel.prototype.addItem = function (label, panel, hidden) {
                    var item = new api.ui.tab.TabBarItemBuilder().setLabel(label).build();
                    this.addItemArray(item);
                    this.deck.addNavigablePanel(item, panel, this.items.length == 1);
                    return this.deck.getPanelIndex(panel);
                };
                DockedPanel.prototype.selectPanel = function (panel) {
                    this.deck.selectPanelByIndex(this.deck.getPanelIndex(panel));
                };
                DockedPanel.prototype.getNavigator = function () {
                    return this.navigator;
                };
                DockedPanel.prototype.getDeck = function () {
                    return this.deck;
                };
                DockedPanel.prototype.addItemArray = function (item) {
                    this.items.push(item);
                };
                return DockedPanel;
            })(_panel.Panel);
            _panel.DockedPanel = DockedPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var dialog;
        (function (dialog) {
            var DialogButton = (function (_super) {
                __extends(DialogButton, _super);
                function DialogButton(action) {
                    _super.call(this, action, false);
                    this.addClass("dialog-button");
                }
                return DialogButton;
            })(api.ui.button.ActionButton);
            dialog.DialogButton = DialogButton;
        })(dialog = ui.dialog || (ui.dialog = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var dialog;
        (function (dialog) {
            var ModalDialog = (function (_super) {
                __extends(ModalDialog, _super);
                function ModalDialog(config) {
                    var _this = this;
                    _super.call(this, "modal-dialog");
                    this.actions = [];
                    this.config = config;
                    var wrapper = new api.dom.DivEl("modal-dialog-content-wrapper");
                    this.appendChild(wrapper);
                    this.createDefaultCancelAction();
                    this.cancelButton = this.createCancelButton();
                    wrapper.appendChild(this.cancelButton);
                    this.title = this.config.title;
                    wrapper.appendChild(this.title);
                    this.contentPanel = new ModalDialogContentPanel();
                    wrapper.appendChild(this.contentPanel);
                    var push = new api.dom.DivEl("modal-dialog-content-push");
                    wrapper.appendChild(push);
                    var footer = new api.dom.DivEl("modal-dialog-footer");
                    this.appendChild(footer);
                    this.buttonRow = new ModalDialogButtonRow();
                    footer.appendChild(this.buttonRow);
                    this.mouseClickListener = function (event) {
                        if (_this.isVisible()) {
                            for (var element = event.target; element; element = element.parentNode) {
                                if (element == _this.getHTMLElement()) {
                                    return;
                                }
                            }
                            if (_this.cancelAction) {
                                _this.cancelAction.execute();
                            }
                        }
                    };
                    api.dom.Body.get().onMouseDown(this.mouseClickListener);
                    this.onRemoved(function () { return api.dom.Body.get().unMouseDown(_this.mouseClickListener); });
                    this.onAdded(function () { return api.dom.Body.get().onMouseDown(_this.mouseClickListener); });
                    this.onShown(function () { return api.ui.responsive.ResponsiveManager.fireResizeEvent(); });
                    this.responsiveItem = api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        if (_this.isVisible()) {
                            _this.centerMyself();
                        }
                    });
                }
                ModalDialog.prototype.createDefaultCancelAction = function () {
                    var _this = this;
                    var cancelAction = new api.ui.Action("Cancel", "esc");
                    cancelAction.setIconClass("cancel-button-top");
                    cancelAction.setLabel("");
                    cancelAction.onExecuted(function () {
                        _this.close();
                    });
                    this.cancelAction = cancelAction;
                    this.actions.push(cancelAction);
                };
                ModalDialog.prototype.getCancelAction = function () {
                    return this.cancelAction;
                };
                ModalDialog.prototype.addCancelButtonToBottom = function () {
                    var _this = this;
                    var cancelAction = new api.ui.Action("Cancel", "esc");
                    cancelAction.setIconClass("cancel-button-bottom");
                    cancelAction.onExecuted(function () {
                        _this.close();
                    });
                    this.addAction(cancelAction);
                };
                ModalDialog.prototype.setTitle = function (value) {
                    this.title.setTitle(value);
                };
                ModalDialog.prototype.appendChildToContentPanel = function (child) {
                    this.contentPanel.appendChild(child);
                };
                ModalDialog.prototype.appendChildToTitle = function (child) {
                    this.title.appendChild(child);
                };
                ModalDialog.prototype.removeChildFromContentPanel = function (child) {
                    this.contentPanel.removeChild(child);
                };
                ModalDialog.prototype.addAction = function (action, useDefault, prepend) {
                    this.actions.push(action);
                    return this.buttonRow.addAction(action, useDefault, prepend);
                };
                ModalDialog.prototype.show = function () {
                    _super.prototype.show.call(this);
                    this.centerMyself();
                    this.buttonRow.focusDefaultAction();
                };
                ModalDialog.prototype.centerMyself = function () {
                    var el = this.getEl();
                    el.setMarginTop("-" + (el.getHeightWithBorder() / 2) + "px");
                    if (this.responsiveItem.isInRangeOrBigger(api.ui.responsive.ResponsiveRanges._540_720)) {
                        el.setMarginLeft("-" + (el.getWidthWithBorder() / 2) + "px");
                        el.addClass("centered_horizontally");
                    }
                    else {
                        el.setMarginLeft("0px");
                        el.removeClass("centered_horizontally");
                    }
                };
                ModalDialog.prototype.createCancelButton = function () {
                    return new dialog.DialogButton(this.getCancelAction());
                };
                ModalDialog.prototype.hide = function () {
                    _super.prototype.hide.call(this);
                };
                ModalDialog.prototype.close = function () {
                    api.ui.mask.BodyMask.get().hide();
                    this.hide();
                    api.ui.KeyBindings.get().unshelveBindings();
                };
                ModalDialog.prototype.open = function () {
                    api.ui.mask.BodyMask.get().show();
                    api.ui.KeyBindings.get().shelveBindings();
                    this.show();
                    api.ui.KeyBindings.get().bindKeys(api.ui.Action.getKeyBindings(this.actions));
                };
                return ModalDialog;
            })(api.dom.DivEl);
            dialog.ModalDialog = ModalDialog;
            var ModalDialogHeader = (function (_super) {
                __extends(ModalDialogHeader, _super);
                function ModalDialogHeader(title) {
                    _super.call(this, "dialog-header");
                    this.titleEl = new api.dom.H2El('title');
                    this.titleEl.setHtml(title);
                    this.appendChild(this.titleEl);
                }
                ModalDialogHeader.prototype.setTitle = function (value) {
                    this.titleEl.setHtml(value);
                };
                return ModalDialogHeader;
            })(api.dom.DivEl);
            dialog.ModalDialogHeader = ModalDialogHeader;
            var ModalDialogContentPanel = (function (_super) {
                __extends(ModalDialogContentPanel, _super);
                function ModalDialogContentPanel() {
                    _super.call(this, "dialog-content");
                }
                return ModalDialogContentPanel;
            })(api.dom.DivEl);
            dialog.ModalDialogContentPanel = ModalDialogContentPanel;
            var ModalDialogButtonRow = (function (_super) {
                __extends(ModalDialogButtonRow, _super);
                function ModalDialogButtonRow() {
                    _super.call(this, "dialog-buttons");
                    this.buttonContainer = new api.dom.DivEl('button-container');
                    this.appendChild(this.buttonContainer);
                }
                ModalDialogButtonRow.prototype.addAction = function (action, useDefault, prepend) {
                    var button = new dialog.DialogButton(action);
                    if (useDefault) {
                        this.defaultButton = button;
                    }
                    if (prepend) {
                        this.buttonContainer.prependChild(button);
                    }
                    else {
                        this.buttonContainer.appendChild(button);
                    }
                    action.onPropertyChanged(function () {
                        button.setLabel(action.getLabel());
                        button.setEnabled(action.isEnabled());
                    });
                    return button;
                };
                ModalDialogButtonRow.prototype.focusDefaultAction = function () {
                    if (this.defaultButton) {
                        this.defaultButton.giveFocus();
                    }
                };
                return ModalDialogButtonRow;
            })(api.dom.DivEl);
            dialog.ModalDialogButtonRow = ModalDialogButtonRow;
        })(dialog = ui.dialog || (ui.dialog = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var dialog;
        (function (dialog) {
            var ConfirmationDialog = (function (_super) {
                __extends(ConfirmationDialog, _super);
                function ConfirmationDialog() {
                    var _this = this;
                    _super.call(this, {
                        title: new api.ui.dialog.ModalDialogHeader("Confirmation")
                    });
                    this.addClass("confirmation-dialog");
                    this.questionEl = new api.dom.DivEl("question");
                    this.appendChildToContentPanel(this.questionEl);
                    this.noAction = new api.ui.Action("No", "esc");
                    this.noAction.onExecuted(function (action) {
                        _this.close();
                        if (_this.noCallback) {
                            _this.noCallback();
                        }
                    });
                    this.yesAction = new api.ui.Action("Yes", "enter");
                    this.yesAction.onExecuted(function (action) {
                        _this.close();
                        if (_this.yesCallback) {
                            _this.yesCallback();
                        }
                    });
                    this.addAction(this.yesAction, true);
                    this.addAction(this.noAction);
                    api.dom.Body.get().appendChild(this);
                }
                ConfirmationDialog.get = function () {
                    if (!ConfirmationDialog.instance) {
                        ConfirmationDialog.instance = new ConfirmationDialog();
                    }
                    return ConfirmationDialog.instance;
                };
                ConfirmationDialog.prototype.setQuestion = function (question) {
                    this.questionEl.getEl().setInnerHtml(question);
                    return this;
                };
                ConfirmationDialog.prototype.setYesCallback = function (callback) {
                    this.yesCallback = callback;
                    return this;
                };
                ConfirmationDialog.prototype.setNoCallback = function (callback) {
                    this.noCallback = callback;
                    return this;
                };
                return ConfirmationDialog;
            })(dialog.ModalDialog);
            dialog.ConfirmationDialog = ConfirmationDialog;
        })(dialog = ui.dialog || (ui.dialog = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var Fieldset = (function (_super) {
                __extends(Fieldset, _super);
                function Fieldset(legend) {
                    _super.call(this);
                    this.items = [];
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.validityChangedListeners = [];
                    if (legend) {
                        this.legend = new api.dom.LegendEl(legend);
                        this.appendChild(this.legend);
                    }
                }
                Fieldset.prototype.add = function (formItem) {
                    var _this = this;
                    formItem.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    formItem.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    formItem.onValidityChanged(function (event) {
                        _this.notifyValidityChanged(event.isValid());
                    });
                    this.items.push(formItem);
                    this.appendChild(formItem);
                };
                Fieldset.prototype.removeItem = function (formItem) {
                    for (var i = 0; i < this.items.length; i++) {
                        if (this.items[i] === formItem) {
                            this.items.splice(i, 1);
                            this.removeChild(formItem);
                        }
                    }
                };
                Fieldset.prototype.validate = function (validationResult, markInvalid) {
                    this.items.forEach(function (item) {
                        item.validate(validationResult, markInvalid);
                    });
                };
                Fieldset.prototype.setFieldsetData = function (data) {
                    var input, inputValue;
                    this.items.forEach(function (item) {
                        input = item.getInput();
                        inputValue = data[input.getName()];
                        if (inputValue) {
                            input.setValue(inputValue);
                        }
                    });
                };
                Fieldset.prototype.getFieldsetData = function () {
                    var input, data = {};
                    this.items.forEach(function (item) {
                        input = item.getInput();
                        data[input.getName()] = input.getValue();
                    });
                    return data;
                };
                Fieldset.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                Fieldset.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Fieldset.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                Fieldset.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Fieldset.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Fieldset.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Fieldset.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                Fieldset.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                Fieldset.prototype.notifyValidityChanged = function (valid) {
                    var _this = this;
                    this.validityChangedListeners.forEach(function (listener) {
                        listener.call(_this, new api.ValidityChangedEvent(valid));
                    });
                };
                return Fieldset;
            })(api.dom.FieldsetEl);
            form.Fieldset = Fieldset;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var Form = (function (_super) {
                __extends(Form, _super);
                function Form(className) {
                    _super.call(this, className);
                    this.fieldsets = [];
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.validityChangedListeners = [];
                    this.formEl = new api.dom.FormEl("form");
                    this.formEl.preventSubmit();
                    this.appendChild(this.formEl);
                }
                Form.prototype.add = function (fieldset) {
                    var _this = this;
                    fieldset.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    fieldset.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    fieldset.onValidityChanged(function (event) {
                        _this.notifyValidityChanged(event.isValid());
                    });
                    this.fieldsets.push(fieldset);
                    this.formEl.appendChild(fieldset);
                    return this;
                };
                Form.prototype.validate = function (markInvalid) {
                    var validationResult = new form.ValidationResult();
                    this.fieldsets.forEach(function (fieldset) {
                        fieldset.validate(validationResult, markInvalid);
                    });
                    return validationResult;
                };
                Form.prototype.setFormData = function (data) {
                    this.fieldsets.forEach(function (fieldset) {
                        fieldset.setFieldsetData(data);
                    });
                };
                Form.prototype.getFormData = function () {
                    var data = {};
                    var fieldsetData;
                    this.fieldsets.forEach(function (fieldset) {
                        fieldsetData = fieldset.getFieldsetData();
                        for (var property in fieldsetData) {
                            if (fieldsetData.hasOwnProperty(property)) {
                                data[property] = fieldsetData[property];
                            }
                        }
                    });
                    return data;
                };
                Form.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                Form.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Form.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                Form.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Form.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Form.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Form.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                Form.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                Form.prototype.notifyValidityChanged = function (valid) {
                    var _this = this;
                    this.validityChangedListeners.forEach(function (listener) {
                        listener.call(_this, new api.ValidityChangedEvent(valid));
                    });
                };
                return Form;
            })(api.dom.DivEl);
            form.Form = Form;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var FormItem = (function (_super) {
                __extends(FormItem, _super);
                function FormItem(builder) {
                    var _this = this;
                    _super.call(this, "form-item");
                    this.invalidClass = "invalid";
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.error = new api.dom.SpanEl("error");
                    this.appendChild(this.error);
                    this.input = builder.getInput();
                    this.input.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    this.input.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    if (builder.getLabel()) {
                        this.label = new api.dom.LabelEl(builder.getLabel(), this.input);
                        if (form.Validators.required == builder.getValidator()) {
                            this.label.setClass("required");
                        }
                        this.appendChild(this.label);
                    }
                    this.appendChild(this.input);
                    if (builder.getValidator()) {
                        this.validator = builder.getValidator();
                    }
                }
                FormItem.prototype.getLabel = function () {
                    return this.label;
                };
                FormItem.prototype.getInput = function () {
                    return this.input;
                };
                FormItem.prototype.getValidator = function () {
                    return this.validator;
                };
                FormItem.prototype.validate = function (validationResult, markInvalid) {
                    if (this.validator) {
                        var validationMessage = this.validator(this.input);
                        if (validationMessage) {
                            validationResult.addError(new form.ValidationError(this, validationMessage));
                        }
                        if (markInvalid) {
                            if (validationMessage) {
                                this.addClass(this.invalidClass);
                            }
                            else {
                                this.removeClass(this.invalidClass);
                            }
                            this.error.setHtml(validationMessage || "");
                        }
                    }
                };
                FormItem.prototype.onValidityChanged = function (listener) {
                    this.input.onValidityChanged(listener);
                };
                FormItem.prototype.unValidityChanged = function (listener) {
                    this.input.unValidityChanged(listener);
                };
                FormItem.prototype.notifyValidityChanged = function (valid) {
                    this.input.notifyValidityChanged(valid);
                };
                FormItem.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                FormItem.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                FormItem.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                FormItem.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                FormItem.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                FormItem.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return FormItem;
            })(api.dom.DivEl);
            form.FormItem = FormItem;
            var FormItemBuilder = (function () {
                function FormItemBuilder(input) {
                    if (!input) {
                        throw new Error("Input can't be null.");
                    }
                    this.input = input;
                }
                FormItemBuilder.prototype.build = function () {
                    return new FormItem(this);
                };
                FormItemBuilder.prototype.getInput = function () {
                    return this.input;
                };
                FormItemBuilder.prototype.setLabel = function (label) {
                    this.label = label;
                    return this;
                };
                FormItemBuilder.prototype.getLabel = function () {
                    return this.label;
                };
                FormItemBuilder.prototype.setValidator = function (validator) {
                    this.validator = validator;
                    return this;
                };
                FormItemBuilder.prototype.getValidator = function () {
                    return this.validator;
                };
                return FormItemBuilder;
            })();
            form.FormItemBuilder = FormItemBuilder;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var Validators = (function () {
                function Validators() {
                }
                Validators.required = function (input) {
                    var value = input.getValue();
                    return !value || value.length == 0 ? "Required field" : undefined;
                };
                return Validators;
            })();
            form.Validators = Validators;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var ValidationResult = (function () {
                function ValidationResult() {
                    this.valid = true;
                    this.errors = [];
                }
                ValidationResult.prototype.addError = function (error) {
                    this.errors.push(error);
                    if (this.valid) {
                        this.valid = false;
                    }
                };
                ValidationResult.prototype.isValid = function () {
                    return this.valid;
                };
                ValidationResult.prototype.getErrors = function () {
                    return this.errors;
                };
                return ValidationResult;
            })();
            form.ValidationResult = ValidationResult;
            var ValidationError = (function () {
                function ValidationError(formItem, message) {
                    this.formItem = formItem;
                    this.message = message;
                }
                ValidationError.prototype.getFormItem = function () {
                    return this.formItem;
                };
                ValidationError.prototype.getMessage = function () {
                    return this.message;
                };
                return ValidationError;
            })();
            form.ValidationError = ValidationError;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var Grid = (function (_super) {
                __extends(Grid, _super);
                function Grid(dataView, columns, options) {
                    var _this = this;
                    _super.call(this, "grid");
                    this.defaultHeight = 400;
                    this.defaultWidth = 800;
                    this.defaultAutoRenderGridOnDataChanges = true;
                    options = new grid.GridOptionsBuilder(options).build();
                    if (options.isHideColumnHeaders()) {
                        this.addClass("no-header");
                    }
                    this.checkboxSelectorPlugin = null;
                    this.checkableRows = options.isCheckableRows() || false;
                    this.dragAndDrop = options.isDragAndDrop() || false;
                    if (this.checkableRows) {
                        this.checkboxSelectorPlugin = new Slick.CheckboxSelectColumn({
                            cssClass: "slick-cell-checkboxsel",
                            width: 40
                        });
                        columns.unshift(this.checkboxSelectorPlugin.getColumnDefinition());
                    }
                    if (this.dragAndDrop) {
                        this.rowManagerPlugin = new Slick.RowMoveManager({
                            cancelEditOnDrag: true
                        });
                    }
                    this.getEl().setHeight((options.getHeight() || this.defaultHeight) + "px");
                    this.getEl().setWidth((options.getWidth() || this.defaultWidth) + "px");
                    this.dataView = dataView;
                    this.slickGrid = new Slick.Grid(this.getHTMLElement(), dataView.slick(), columns, options);
                    if (options.isAutoRenderGridOnDataChanges() || (options.isAutoRenderGridOnDataChanges() == undefined && this.defaultAutoRenderGridOnDataChanges)) {
                        this.autoRenderGridOnDataChanges(this.dataView);
                    }
                    if (this.checkboxSelectorPlugin != null) {
                        this.slickGrid.registerPlugin(this.checkboxSelectorPlugin);
                    }
                    if (this.rowManagerPlugin != null) {
                        this.slickGrid.registerPlugin(this.rowManagerPlugin);
                    }
                    this.onRemoved(function (event) {
                        ResponsiveManager.unAvailableSizeChanged(_this);
                    });
                    this.dataView.setItems([], options.getDataIdProperty());
                }
                Grid.prototype.autoRenderGridOnDataChanges = function (dataView) {
                    var _this = this;
                    dataView.onRowCountChanged(function (eventData, args) {
                        _this.updateRowCount();
                        _this.render();
                    });
                    dataView.onRowsChanged(function (eventData, args) {
                        _this.invalidateRows(args.rows);
                        _this.render();
                    });
                };
                Grid.prototype.setSelectionModel = function (selectionModel) {
                    this.slickGrid.setSelectionModel(selectionModel);
                };
                Grid.prototype.getDataView = function () {
                    return this.dataView;
                };
                Grid.prototype.setColumns = function (columns) {
                    if (this.checkboxSelectorPlugin) {
                        columns.unshift(this.checkboxSelectorPlugin.getColumnDefinition());
                    }
                    this.slickGrid.setColumns(columns);
                };
                Grid.prototype.getColumns = function () {
                    return this.slickGrid.getColumns();
                };
                Grid.prototype.setFilter = function (f) {
                    this.dataView.setFilter(f);
                };
                Grid.prototype.setOptions = function (options) {
                    this.slickGrid.setOptions(options);
                };
                Grid.prototype.getOptions = function () {
                    return this.slickGrid.getOptions();
                };
                Grid.prototype.getCheckboxSelectorPlugin = function () {
                    return this.checkboxSelectorPlugin;
                };
                Grid.prototype.registerPlugin = function (plugin) {
                    this.slickGrid.registerPlugin(plugin);
                };
                Grid.prototype.unregisterPlugin = function (plugin) {
                    this.slickGrid.unregisterPlugin(plugin);
                };
                Grid.prototype.render = function () {
                    this.slickGrid.render();
                    _super.prototype.render.call(this);
                };
                Grid.prototype.renderGrid = function () {
                    this.slickGrid.render();
                };
                Grid.prototype.resizeCanvas = function () {
                    this.slickGrid.resizeCanvas();
                };
                Grid.prototype.updateRowCount = function () {
                    this.slickGrid.updateRowCount();
                };
                Grid.prototype.invalidateRows = function (rows) {
                    this.slickGrid.invalidateRows(rows);
                };
                Grid.prototype.invalidate = function () {
                    this.slickGrid.invalidate();
                };
                Grid.prototype.syncGridSelection = function (preserveHidden) {
                    this.dataView.syncGridSelection(this.slickGrid, preserveHidden);
                };
                Grid.prototype.focus = function () {
                    this.slickGrid.focus();
                };
                Grid.prototype.setOnClick = function (callback) {
                    this.slickGrid.onClick.subscribe(function (event, data) {
                        event.stopPropagation();
                        callback(event, data);
                    });
                };
                Grid.prototype.setOnKeyDown = function (callback) {
                    this.slickGrid.onKeyDown.subscribe(function (event) {
                        event.stopPropagation();
                        callback(event);
                    });
                };
                Grid.prototype.getSelectedRows = function () {
                    return this.slickGrid.getSelectedRows();
                };
                Grid.prototype.getSelectedRowItems = function () {
                    var _this = this;
                    var rowItems = [];
                    var rows = this.getSelectedRows();
                    rows.forEach(function (rowIndex) {
                        rowItems.push(_this.dataView.getItem(rowIndex));
                    });
                    return rowItems;
                };
                Grid.prototype.setSelectedRows = function (rows) {
                    this.slickGrid.setSelectedRows(rows);
                };
                Grid.prototype.selectRow = function (row) {
                    var rows = this.getSelectedRows();
                    if (rows.length > 1 || (rows.length < 2 && rows.indexOf(row) < 0)) {
                        this.slickGrid.setSelectedRows([row]);
                    }
                };
                Grid.prototype.addSelectedRow = function (row) {
                    var rows = this.getSelectedRows();
                    if (rows.indexOf(row) < 0) {
                        rows.push(row);
                        this.setSelectedRows(rows);
                    }
                };
                Grid.prototype.toggleRow = function (row) {
                    var rows = this.getSelectedRows(), index = rows.indexOf(row);
                    if (index < 0) {
                        rows.push(row);
                        rows.sort(function (a, b) {
                            return a - b;
                        });
                    }
                    else {
                        rows.splice(index, 1);
                    }
                    this.slickGrid.setSelectedRows(rows);
                    return index;
                };
                Grid.prototype.isRowSelected = function (row) {
                    var rows = this.getSelectedRows(), index = rows.indexOf(row);
                    return index >= 0;
                };
                Grid.prototype.selectAll = function () {
                    var rows = [];
                    for (var i = 0; i < this.slickGrid.getDataLength(); i++) {
                        rows.push(i);
                    }
                    this.setSelectedRows(rows);
                };
                Grid.prototype.clearSelection = function () {
                    this.setSelectedRows([]);
                };
                Grid.prototype.isAllSelected = function () {
                    return this.slickGrid.getDataLength() === this.getSelectedRows().length;
                };
                Grid.prototype.resetActiveCell = function () {
                    if (this.slickGrid.getActiveCell()) {
                        this.slickGrid.resetActiveCell();
                    }
                };
                Grid.prototype.getCellFromEvent = function (e) {
                    return this.slickGrid.getCellFromEvent(e);
                };
                Grid.prototype.moveSelectedUp = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        var row = selected.length >= 1 ? selected[0] - 1 : -1;
                        if (selected.length === 1) {
                            if (row >= 0) {
                                this.selectRow(row);
                                return row;
                            }
                            else {
                                this.clearSelection();
                                return 0;
                            }
                        }
                        else if (selected.length > 1) {
                            row = Math.max(row, 0);
                            this.selectRow(row);
                            return row;
                        }
                    }
                    return -1;
                };
                Grid.prototype.moveSelectedDown = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        var row = selected.length >= 1 ? Math.min(selected[selected.length - 1] + 1, this.slickGrid.getDataLength() - 1) : 0;
                        this.selectRow(row);
                        return row;
                    }
                    return -1;
                };
                Grid.prototype.addSelectedUp = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        if (selected.length > 0 && (selected[0] - 1) >= 0) {
                            var row = selected[0] - 1;
                            selected.push(row);
                            selected = selected.sort(function (a, b) {
                                return a - b;
                            });
                            this.setSelectedRows(selected);
                            return row;
                        }
                    }
                    return -1;
                };
                Grid.prototype.addSelectedDown = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        if (selected.length > 0 && (selected[selected.length - 1] + 1) < this.slickGrid.getDataLength()) {
                            var row = selected[selected.length - 1] + 1;
                            selected.push(row);
                            this.setSelectedRows(selected);
                            return row;
                        }
                        else if (selected.length === 0) {
                            this.moveSelectedDown();
                            return 0;
                        }
                    }
                    return -1;
                };
                Grid.prototype.navigateUp = function () {
                    this.slickGrid.navigateUp();
                };
                Grid.prototype.navigateDown = function () {
                    this.slickGrid.navigateDown();
                };
                Grid.prototype.getActiveCell = function () {
                    return this.slickGrid.getActiveCell();
                };
                Grid.prototype.setActiveCell = function (row, cell) {
                    this.slickGrid.setActiveCell(row, cell);
                };
                Grid.prototype.setCellCssStyles = function (key, hash) {
                    this.slickGrid.setCellCssStyles(key, hash);
                };
                Grid.prototype.getCellCssStyles = function (key) {
                    return this.slickGrid.getCellCssStyles(key);
                };
                Grid.prototype.getCanvasNode = function () {
                    return this.slickGrid.getCanvasNode();
                };
                Grid.prototype.getGridPosition = function () {
                    return this.slickGrid.getGridPosition();
                };
                Grid.prototype.getRenderedRange = function (viewportTop, viewportLeft) {
                    return this.slickGrid.getRenderedRange(viewportTop, viewportLeft);
                };
                Grid.prototype.getViewport = function (viewportTop, viewportLeft) {
                    return this.slickGrid.getViewport(viewportTop, viewportLeft);
                };
                Grid.prototype.subscribeOnSelectedRowsChanged = function (callback) {
                    this.slickGrid.onSelectedRowsChanged.subscribe(callback);
                };
                Grid.prototype.subscribeOnClick = function (callback) {
                    this.slickGrid.onClick.subscribe(callback);
                };
                Grid.prototype.subscribeOnDblClick = function (callback) {
                    this.slickGrid.onDblClick.subscribe(callback);
                };
                Grid.prototype.subscribeOnContextMenu = function (callback) {
                    this.slickGrid.onContextMenu.subscribe(callback);
                };
                Grid.prototype.subscribeOnDrag = function (callback) {
                    this.slickGrid.onDrag.subscribe(callback);
                };
                Grid.prototype.subscribeOnDragEnd = function (callback) {
                    this.slickGrid.onDragEnd.subscribe(callback);
                };
                Grid.prototype.subscribeBeforeMoveRows = function (callback) {
                    if (this.rowManagerPlugin) {
                        this.rowManagerPlugin.onBeforeMoveRows.subscribe(callback);
                    }
                };
                Grid.prototype.subscribeMoveRows = function (callback) {
                    if (this.rowManagerPlugin) {
                        this.rowManagerPlugin.onMoveRows.subscribe(callback);
                    }
                };
                return Grid;
            })(api.dom.DivEl);
            grid.Grid = Grid;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var GridColumnBuilder = (function () {
                function GridColumnBuilder(source) {
                    if (source) {
                        this.asyncPostRender = source.getAsyncPostRender();
                        this.behavior = source.getBehavior();
                        this.cannotTriggerInsert = source.isCannotTriggerInsert();
                        this.cssClass = source.getCssClass();
                        this.defaultSortAsc = source.isDefaultSortAsc();
                        this.editor = source.getEditor();
                        this.field = source.getField();
                        this.focusable = source.isFocusable();
                        this.formatter = source.getFormatter();
                        this.headerCssClass = source.getHeaderCssClass();
                        this.id = source.getId();
                        this.maxWidth = source.getMaxWidth();
                        this.minWidth = source.getMinWidth();
                        this.name = source.getName();
                        this.rerenderOnResize = source.isRerenderOnResize();
                        this.resizable = source.isResizable();
                        this.selectable = source.isSelectable();
                        this.sortable = source.isSortable();
                        this.toolTip = source.getToolTip();
                        this.width = source.getWidth();
                    }
                }
                GridColumnBuilder.prototype.setAsyncPostRender = function (asyncPostRender) {
                    this.asyncPostRender = asyncPostRender;
                    return this;
                };
                GridColumnBuilder.prototype.setBehavior = function (behavior) {
                    this.behavior = behavior;
                    return this;
                };
                GridColumnBuilder.prototype.setCannotTriggerInsert = function (cannotTriggerInsert) {
                    this.cannotTriggerInsert = cannotTriggerInsert;
                    return this;
                };
                GridColumnBuilder.prototype.setCssClass = function (cssClass) {
                    this.cssClass = cssClass;
                    return this;
                };
                GridColumnBuilder.prototype.setDefaultSortAsc = function (defaultSortAsc) {
                    this.defaultSortAsc = defaultSortAsc;
                    return this;
                };
                GridColumnBuilder.prototype.setEditor = function (editor) {
                    this.editor = editor;
                    return this;
                };
                GridColumnBuilder.prototype.setField = function (field) {
                    this.field = field;
                    return this;
                };
                GridColumnBuilder.prototype.setFocusable = function (focusable) {
                    this.focusable = focusable;
                    return this;
                };
                GridColumnBuilder.prototype.setFormatter = function (formatter) {
                    this.formatter = formatter;
                    return this;
                };
                GridColumnBuilder.prototype.setHeaderCssClass = function (headerCssClass) {
                    this.headerCssClass = headerCssClass;
                    return this;
                };
                GridColumnBuilder.prototype.setId = function (id) {
                    this.id = id;
                    return this;
                };
                GridColumnBuilder.prototype.setMaxWidth = function (maxWidth) {
                    this.maxWidth = maxWidth;
                    return this;
                };
                GridColumnBuilder.prototype.setMinWidth = function (minWidth) {
                    this.minWidth = minWidth;
                    return this;
                };
                GridColumnBuilder.prototype.setName = function (name) {
                    this.name = name;
                    return this;
                };
                GridColumnBuilder.prototype.setRerenderOnResize = function (rerenderOnResize) {
                    this.rerenderOnResize = rerenderOnResize;
                    return this;
                };
                GridColumnBuilder.prototype.setResizable = function (resizable) {
                    this.resizable = resizable;
                    return this;
                };
                GridColumnBuilder.prototype.setSelectable = function (selectable) {
                    this.selectable = selectable;
                    return this;
                };
                GridColumnBuilder.prototype.setSortable = function (sortable) {
                    this.sortable = sortable;
                    return this;
                };
                GridColumnBuilder.prototype.setToolTip = function (toolTip) {
                    this.toolTip = toolTip;
                    return this;
                };
                GridColumnBuilder.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                GridColumnBuilder.prototype.build = function () {
                    return new GridColumn(this);
                };
                return GridColumnBuilder;
            })();
            grid.GridColumnBuilder = GridColumnBuilder;
            var GridColumn = (function () {
                function GridColumn(builder) {
                    this.asyncPostRender = builder.asyncPostRender;
                    this.behavior = builder.behavior;
                    this.cannotTriggerInsert = builder.cannotTriggerInsert;
                    this.cssClass = builder.cssClass;
                    this.defaultSortAsc = builder.defaultSortAsc;
                    this.editor = builder.editor;
                    this.field = builder.field;
                    this.focusable = builder.focusable;
                    this.formatter = builder.formatter;
                    this.headerCssClass = builder.headerCssClass;
                    this.id = builder.id;
                    this.maxWidth = builder.maxWidth;
                    this.minWidth = builder.minWidth;
                    this.name = builder.name;
                    this.rerenderOnResize = builder.rerenderOnResize;
                    this.resizable = builder.resizable;
                    this.selectable = builder.selectable;
                    this.sortable = builder.sortable;
                    this.toolTip = builder.toolTip;
                    this.width = builder.width;
                }
                GridColumn.prototype.getAsyncPostRender = function () {
                    return this.asyncPostRender;
                };
                GridColumn.prototype.getBehavior = function () {
                    return this.behavior;
                };
                GridColumn.prototype.isCannotTriggerInsert = function () {
                    return this.cannotTriggerInsert;
                };
                GridColumn.prototype.getCssClass = function () {
                    return this.cssClass;
                };
                GridColumn.prototype.isDefaultSortAsc = function () {
                    return this.defaultSortAsc;
                };
                GridColumn.prototype.getEditor = function () {
                    return this.editor;
                };
                GridColumn.prototype.getField = function () {
                    return this.field;
                };
                GridColumn.prototype.isFocusable = function () {
                    return this.focusable;
                };
                GridColumn.prototype.getFormatter = function () {
                    return this.formatter;
                };
                GridColumn.prototype.getHeaderCssClass = function () {
                    return this.headerCssClass;
                };
                GridColumn.prototype.getId = function () {
                    return this.id;
                };
                GridColumn.prototype.getMaxWidth = function () {
                    return this.maxWidth;
                };
                GridColumn.prototype.getMinWidth = function () {
                    return this.minWidth;
                };
                GridColumn.prototype.getName = function () {
                    return this.name;
                };
                GridColumn.prototype.isRerenderOnResize = function () {
                    return this.rerenderOnResize;
                };
                GridColumn.prototype.isResizable = function () {
                    return this.resizable;
                };
                GridColumn.prototype.isSelectable = function () {
                    return this.selectable;
                };
                GridColumn.prototype.isSortable = function () {
                    return this.sortable;
                };
                GridColumn.prototype.getToolTip = function () {
                    return this.toolTip;
                };
                GridColumn.prototype.getWidth = function () {
                    return this.width;
                };
                GridColumn.prototype.setAsyncPostRender = function (asyncPostRender) {
                    this.asyncPostRender = asyncPostRender;
                    return this;
                };
                GridColumn.prototype.setBehavior = function (behavior) {
                    this.behavior = behavior;
                    return this;
                };
                GridColumn.prototype.setCannotTriggerInsert = function (cannotTriggerInsert) {
                    this.cannotTriggerInsert = cannotTriggerInsert;
                    return this;
                };
                GridColumn.prototype.setCssClass = function (cssClass) {
                    this.cssClass = cssClass;
                    return this;
                };
                GridColumn.prototype.setDefaultSortAsc = function (defaultSortAsc) {
                    this.defaultSortAsc = defaultSortAsc;
                    return this;
                };
                GridColumn.prototype.setEditor = function (editor) {
                    this.editor = editor;
                    return this;
                };
                GridColumn.prototype.setField = function (field) {
                    this.field = field;
                    return this;
                };
                GridColumn.prototype.setFocusable = function (focusable) {
                    this.focusable = focusable;
                    return this;
                };
                GridColumn.prototype.setFormatter = function (formatter) {
                    this.formatter = formatter;
                    return this;
                };
                GridColumn.prototype.setHeaderCssClass = function (headerCssClass) {
                    this.headerCssClass = headerCssClass;
                    return this;
                };
                GridColumn.prototype.setId = function (id) {
                    this.id = id;
                    return this;
                };
                GridColumn.prototype.setMaxWidth = function (maxWidth) {
                    this.maxWidth = maxWidth;
                    return this;
                };
                GridColumn.prototype.setMinWidth = function (minWidth) {
                    this.minWidth = minWidth;
                    return this;
                };
                GridColumn.prototype.setName = function (name) {
                    this.name = name;
                    return this;
                };
                GridColumn.prototype.setRerenderOnResize = function (rerenderOnResize) {
                    this.rerenderOnResize = rerenderOnResize;
                    return this;
                };
                GridColumn.prototype.setResizable = function (resizable) {
                    this.resizable = resizable;
                    return this;
                };
                GridColumn.prototype.setSelectable = function (selectable) {
                    this.selectable = selectable;
                    return this;
                };
                GridColumn.prototype.setSortable = function (sortable) {
                    this.sortable = sortable;
                    return this;
                };
                GridColumn.prototype.setToolTip = function (toolTip) {
                    this.toolTip = toolTip;
                    return this;
                };
                GridColumn.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                return GridColumn;
            })();
            grid.GridColumn = GridColumn;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var GridOptionsBuilder = (function () {
                function GridOptionsBuilder(source) {
                    if (source) {
                        this.asyncEditorLoading = source.isAsyncEditorLoading();
                        this.asyncEditorLoadDelay = source.getAsyncEditorLoadDelay();
                        this.asyncPostRenderDelay = source.getAsyncPostRenderDelay();
                        this.autoEdit = source.isAutoEdit();
                        this.autoHeight = source.isAutoHeight();
                        this.cellFlashingCssClass = source.getCellFlashingCssClass();
                        this.cellHighlightCssClass = source.getCellHighlightCssClass();
                        this.dataItemColumnValueExtractor = source.getDataItemColumnValueExtractor();
                        this.defaultColumnWidth = source.getDefaultColumnWidth();
                        this.defaultFormatter = source.getDefaultFormatter();
                        this.editable = source.isEditable();
                        this.editCommandHandler = source.getEditCommandHandler();
                        this.editorFactory = source.getEditorFactory();
                        this.editorLock = source.getEditorLock();
                        this.enableAddRow = source.isEnableAddRow();
                        this.enableAsyncPostRender = source.isEnableAsyncPostRender();
                        this.enableCellRangeSelection = source.getEnableCellRangeSelection();
                        this.enableCellNavigation = source.isEnableCellNavigation();
                        this.enableColumnReorder = source.isEnableColumnReorder();
                        this.enableRowReordering = source.getEnableRowReordering();
                        this.enableTextSelectionOnCells = source.isEnableTextSelectionOnCells();
                        this.explicitInitialization = source.isExplicitInitialization();
                        this.forceFitColumns = source.isForceFitColumns();
                        this.forceSyncScrolling = source.isForceSyncScrolling();
                        this.formatterFactory = source.getFormatterFactory();
                        this.fullWidthRows = source.isFullWidthRows();
                        this.headerRowHeight = source.getHeaderRowHeight();
                        this.leaveSpaceForNewRows = source.isLeaveSpaceForNewRows();
                        this.multiColumnSort = source.isMultiColumnSort();
                        this.multiSelect = source.isMultiSelect();
                        this.rowHeight = source.getRowHeight();
                        this.selectedCellCssClass = source.getSelectedCellCssClass();
                        this.showHeaderRow = source.getShowHeaderRow();
                        this.syncColumnCellResize = source.isSyncColumnCellResize();
                        this.topPanelHeight = source.getTopPanelHeight();
                        this.hideColumnHeaders = source.isHideColumnHeaders();
                        this.width = source.getWidth();
                        this.height = source.getHeight();
                        this.dataIdProperty = source.getDataIdProperty();
                        this.autoRenderGridOnDataChanges = source.isAutoRenderGridOnDataChanges();
                        this.checkableRows = source.isCheckableRows();
                        this.disabledMultipleSelection = source.isMultipleSelectionDisabled();
                        this.dragAndDrop = source.isDragAndDrop();
                    }
                }
                GridOptionsBuilder.prototype.setAsyncEditorLoading = function (asyncEditorLoading) {
                    this.asyncEditorLoading = asyncEditorLoading;
                    return this;
                };
                GridOptionsBuilder.prototype.setAsyncEditorLoadDelay = function (asyncEditorLoadDelay) {
                    this.asyncEditorLoadDelay = asyncEditorLoadDelay;
                    return this;
                };
                GridOptionsBuilder.prototype.setAsyncPostRenderDelay = function (asyncPostRenderDelay) {
                    this.asyncPostRenderDelay = asyncPostRenderDelay;
                    return this;
                };
                GridOptionsBuilder.prototype.setAutoEdit = function (autoEdit) {
                    this.autoEdit = autoEdit;
                    return this;
                };
                GridOptionsBuilder.prototype.setAutoHeight = function (autoHeight) {
                    this.autoHeight = autoHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setCellFlashingCssClass = function (cellFlashingCssClass) {
                    this.cellFlashingCssClass = cellFlashingCssClass;
                    return this;
                };
                GridOptionsBuilder.prototype.setCellHighlightCssClass = function (cellHighlightCssClass) {
                    this.cellHighlightCssClass = cellHighlightCssClass;
                    return this;
                };
                GridOptionsBuilder.prototype.setDataItemColumnValueExtractor = function (dataItemColumnValueExtractor) {
                    this.dataItemColumnValueExtractor = dataItemColumnValueExtractor;
                    return this;
                };
                GridOptionsBuilder.prototype.setDefaultColumnWidth = function (defaultColumnWidth) {
                    this.defaultColumnWidth = defaultColumnWidth;
                    return this;
                };
                GridOptionsBuilder.prototype.setDefaultFormatter = function (defaultFormatter) {
                    this.defaultFormatter = defaultFormatter;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditable = function (editable) {
                    this.editable = editable;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditCommandHandler = function (editCommandHandler) {
                    this.editCommandHandler = editCommandHandler;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditorFactory = function (editorFactory) {
                    this.editorFactory = editorFactory;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditorLock = function (editorLock) {
                    this.editorLock = editorLock;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableAddRow = function (enableAddRow) {
                    this.enableAddRow = enableAddRow;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableAsyncPostRender = function (enableAsyncPostRender) {
                    this.enableAsyncPostRender = enableAsyncPostRender;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableCellRangeSelection = function (enableCellRangeSelection) {
                    this.enableCellRangeSelection = enableCellRangeSelection;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableCellNavigation = function (enableCellNavigation) {
                    this.enableCellNavigation = enableCellNavigation;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableColumnReorder = function (enableColumnReorder) {
                    this.enableColumnReorder = enableColumnReorder;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableRowReordering = function (enableRowReordering) {
                    this.enableRowReordering = enableRowReordering;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableTextSelectionOnCells = function (enableTextSelectionOnCells) {
                    this.enableTextSelectionOnCells = enableTextSelectionOnCells;
                    return this;
                };
                GridOptionsBuilder.prototype.setExplicitInitialization = function (explicitInitialization) {
                    this.explicitInitialization = explicitInitialization;
                    return this;
                };
                GridOptionsBuilder.prototype.setForceFitColumns = function (forceFitColumns) {
                    this.forceFitColumns = forceFitColumns;
                    return this;
                };
                GridOptionsBuilder.prototype.setForceSyncScrolling = function (forceSyncScrolling) {
                    this.forceSyncScrolling = forceSyncScrolling;
                    return this;
                };
                GridOptionsBuilder.prototype.setFormatterFactory = function (formatterFactory) {
                    this.formatterFactory = formatterFactory;
                    return this;
                };
                GridOptionsBuilder.prototype.setFullWidthRows = function (fullWidthRows) {
                    this.fullWidthRows = fullWidthRows;
                    return this;
                };
                GridOptionsBuilder.prototype.setHeaderRowHeight = function (headerRowHeight) {
                    this.headerRowHeight = headerRowHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setLeaveSpaceForNewRows = function (leaveSpaceForNewRows) {
                    this.leaveSpaceForNewRows = leaveSpaceForNewRows;
                    return this;
                };
                GridOptionsBuilder.prototype.setMultiColumnSort = function (multiColumnSort) {
                    this.multiColumnSort = multiColumnSort;
                    return this;
                };
                GridOptionsBuilder.prototype.setMultiSelect = function (multiSelect) {
                    this.multiSelect = multiSelect;
                    return this;
                };
                GridOptionsBuilder.prototype.setRowHeight = function (rowHeight) {
                    this.rowHeight = rowHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setSelectedCellCssClass = function (selectedCellCssClass) {
                    this.selectedCellCssClass = selectedCellCssClass;
                    return this;
                };
                GridOptionsBuilder.prototype.setShowHeaderRow = function (showHeaderRow) {
                    this.showHeaderRow = showHeaderRow;
                    return this;
                };
                GridOptionsBuilder.prototype.setSyncColumnCellResize = function (syncColumnCellResize) {
                    this.syncColumnCellResize = syncColumnCellResize;
                    return this;
                };
                GridOptionsBuilder.prototype.setTopPanelHeight = function (topPanelHeight) {
                    this.topPanelHeight = topPanelHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setHideColumnHeaders = function (hideColumnHeaders) {
                    this.hideColumnHeaders = hideColumnHeaders;
                    return this;
                };
                GridOptionsBuilder.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                GridOptionsBuilder.prototype.setHeight = function (height) {
                    this.height = height;
                    return this;
                };
                GridOptionsBuilder.prototype.setDataIdProperty = function (dataIdProperty) {
                    this.dataIdProperty = dataIdProperty;
                    return this;
                };
                GridOptionsBuilder.prototype.setAutoRenderGridOnDataChanges = function (autoRenderGridOnDataChanges) {
                    this.autoRenderGridOnDataChanges = autoRenderGridOnDataChanges;
                    return this;
                };
                GridOptionsBuilder.prototype.setCheckableRows = function (checkableRows) {
                    this.checkableRows = checkableRows;
                    return this;
                };
                GridOptionsBuilder.prototype.disableMultipleSelection = function (disabledMultipleSelection) {
                    this.disabledMultipleSelection = disabledMultipleSelection;
                    return this;
                };
                GridOptionsBuilder.prototype.setDragAndDrop = function (dragAndDrop) {
                    this.dragAndDrop = dragAndDrop;
                    return this;
                };
                GridOptionsBuilder.prototype.build = function () {
                    return new GridOptions(this);
                };
                return GridOptionsBuilder;
            })();
            grid.GridOptionsBuilder = GridOptionsBuilder;
            var GridOptions = (function () {
                function GridOptions(builder) {
                    this.asyncEditorLoading = builder.asyncEditorLoading;
                    this.asyncEditorLoadDelay = builder.asyncEditorLoadDelay;
                    this.asyncPostRenderDelay = builder.asyncPostRenderDelay;
                    this.autoEdit = builder.autoEdit;
                    this.autoHeight = builder.autoHeight;
                    this.cellFlashingCssClass = builder.cellFlashingCssClass;
                    this.cellHighlightCssClass = builder.cellHighlightCssClass;
                    this.dataItemColumnValueExtractor = builder.dataItemColumnValueExtractor;
                    this.defaultColumnWidth = builder.defaultColumnWidth;
                    this.defaultFormatter = builder.defaultFormatter;
                    this.editable = builder.editable;
                    this.editCommandHandler = builder.editCommandHandler;
                    this.editorFactory = builder.editorFactory;
                    this.editorLock = builder.editorLock;
                    this.enableAddRow = builder.enableAddRow;
                    this.enableAsyncPostRender = builder.enableAsyncPostRender;
                    this.enableCellRangeSelection = builder.enableCellRangeSelection;
                    this.enableCellNavigation = builder.enableCellNavigation;
                    this.enableColumnReorder = builder.enableColumnReorder;
                    this.enableRowReordering = builder.enableRowReordering;
                    this.enableTextSelectionOnCells = builder.enableTextSelectionOnCells;
                    this.explicitInitialization = builder.explicitInitialization;
                    this.forceFitColumns = builder.forceFitColumns;
                    this.forceSyncScrolling = builder.forceSyncScrolling;
                    this.formatterFactory = builder.formatterFactory;
                    this.fullWidthRows = builder.fullWidthRows;
                    this.headerRowHeight = builder.headerRowHeight;
                    this.leaveSpaceForNewRows = builder.leaveSpaceForNewRows;
                    this.multiColumnSort = builder.multiColumnSort;
                    this.multiSelect = builder.multiSelect;
                    this.rowHeight = builder.rowHeight;
                    this.selectedCellCssClass = builder.selectedCellCssClass;
                    this.showHeaderRow = builder.showHeaderRow;
                    this.syncColumnCellResize = builder.syncColumnCellResize;
                    this.topPanelHeight = builder.topPanelHeight;
                    this.hideColumnHeaders = builder.hideColumnHeaders;
                    this.width = builder.width;
                    this.height = builder.height;
                    this.dataIdProperty = builder.dataIdProperty;
                    this.autoRenderGridOnDataChanges = builder.autoRenderGridOnDataChanges;
                    this.checkableRows = builder.checkableRows;
                    this.disabledMultipleSelection = builder.disabledMultipleSelection;
                    this.dragAndDrop = builder.dragAndDrop;
                }
                GridOptions.prototype.isAsyncEditorLoading = function () {
                    return this.asyncEditorLoading;
                };
                GridOptions.prototype.getAsyncEditorLoadDelay = function () {
                    return this.asyncEditorLoadDelay;
                };
                GridOptions.prototype.getAsyncPostRenderDelay = function () {
                    return this.asyncPostRenderDelay;
                };
                GridOptions.prototype.isAutoEdit = function () {
                    return this.autoEdit;
                };
                GridOptions.prototype.isAutoHeight = function () {
                    return this.autoHeight;
                };
                GridOptions.prototype.getCellFlashingCssClass = function () {
                    return this.cellFlashingCssClass;
                };
                GridOptions.prototype.getCellHighlightCssClass = function () {
                    return this.cellHighlightCssClass;
                };
                GridOptions.prototype.getDataItemColumnValueExtractor = function () {
                    return this.dataItemColumnValueExtractor;
                };
                GridOptions.prototype.getDefaultColumnWidth = function () {
                    return this.defaultColumnWidth;
                };
                GridOptions.prototype.getDefaultFormatter = function () {
                    return this.defaultFormatter;
                };
                GridOptions.prototype.isEditable = function () {
                    return this.editable;
                };
                GridOptions.prototype.getEditCommandHandler = function () {
                    return this.editCommandHandler;
                };
                GridOptions.prototype.getEditorFactory = function () {
                    return this.editorFactory;
                };
                GridOptions.prototype.getEditorLock = function () {
                    return this.editorLock;
                };
                GridOptions.prototype.isEnableAddRow = function () {
                    return this.enableAddRow;
                };
                GridOptions.prototype.isEnableAsyncPostRender = function () {
                    return this.enableAsyncPostRender;
                };
                GridOptions.prototype.getEnableCellRangeSelection = function () {
                    return this.enableCellRangeSelection;
                };
                GridOptions.prototype.isEnableCellNavigation = function () {
                    return this.enableCellNavigation;
                };
                GridOptions.prototype.isEnableColumnReorder = function () {
                    return this.enableColumnReorder;
                };
                GridOptions.prototype.getEnableRowReordering = function () {
                    return this.enableRowReordering;
                };
                GridOptions.prototype.isEnableTextSelectionOnCells = function () {
                    return this.enableTextSelectionOnCells;
                };
                GridOptions.prototype.isExplicitInitialization = function () {
                    return this.explicitInitialization;
                };
                GridOptions.prototype.isForceFitColumns = function () {
                    return this.forceFitColumns;
                };
                GridOptions.prototype.isForceSyncScrolling = function () {
                    return this.forceSyncScrolling;
                };
                GridOptions.prototype.getFormatterFactory = function () {
                    return this.formatterFactory;
                };
                GridOptions.prototype.isFullWidthRows = function () {
                    return this.fullWidthRows;
                };
                GridOptions.prototype.getHeaderRowHeight = function () {
                    return this.headerRowHeight;
                };
                GridOptions.prototype.isLeaveSpaceForNewRows = function () {
                    return this.leaveSpaceForNewRows;
                };
                GridOptions.prototype.isMultiColumnSort = function () {
                    return this.multiColumnSort;
                };
                GridOptions.prototype.isMultiSelect = function () {
                    return this.multiSelect;
                };
                GridOptions.prototype.getRowHeight = function () {
                    return this.rowHeight;
                };
                GridOptions.prototype.getSelectedCellCssClass = function () {
                    return this.selectedCellCssClass;
                };
                GridOptions.prototype.getShowHeaderRow = function () {
                    return this.showHeaderRow;
                };
                GridOptions.prototype.isSyncColumnCellResize = function () {
                    return this.syncColumnCellResize;
                };
                GridOptions.prototype.getTopPanelHeight = function () {
                    return this.topPanelHeight;
                };
                GridOptions.prototype.isHideColumnHeaders = function () {
                    return this.hideColumnHeaders;
                };
                GridOptions.prototype.getWidth = function () {
                    return this.width;
                };
                GridOptions.prototype.getHeight = function () {
                    return this.height;
                };
                GridOptions.prototype.getDataIdProperty = function () {
                    return this.dataIdProperty;
                };
                GridOptions.prototype.isAutoRenderGridOnDataChanges = function () {
                    return this.autoRenderGridOnDataChanges;
                };
                GridOptions.prototype.isCheckableRows = function () {
                    return this.checkableRows;
                };
                GridOptions.prototype.isMultipleSelectionDisabled = function () {
                    return this.disabledMultipleSelection;
                };
                GridOptions.prototype.isDragAndDrop = function () {
                    return this.dragAndDrop;
                };
                GridOptions.prototype.setAsyncEditorLoading = function (asyncEditorLoading) {
                    this.asyncEditorLoading = asyncEditorLoading;
                    return this;
                };
                GridOptions.prototype.setAsyncEditorLoadDelay = function (asyncEditorLoadDelay) {
                    this.asyncEditorLoadDelay = asyncEditorLoadDelay;
                    return this;
                };
                GridOptions.prototype.setAsyncPostRenderDelay = function (asyncPostRenderDelay) {
                    this.asyncPostRenderDelay = asyncPostRenderDelay;
                    return this;
                };
                GridOptions.prototype.setAutoEdit = function (autoEdit) {
                    this.autoEdit = autoEdit;
                    return this;
                };
                GridOptions.prototype.setAutoHeight = function (autoHeight) {
                    this.autoHeight = autoHeight;
                    return this;
                };
                GridOptions.prototype.setCellFlashingCssClass = function (cellFlashingCssClass) {
                    this.cellFlashingCssClass = cellFlashingCssClass;
                    return this;
                };
                GridOptions.prototype.setCellHighlightCssClass = function (cellHighlightCssClass) {
                    this.cellHighlightCssClass = cellHighlightCssClass;
                    return this;
                };
                GridOptions.prototype.setDataItemColumnValueExtractor = function (dataItemColumnValueExtractor) {
                    this.dataItemColumnValueExtractor = dataItemColumnValueExtractor;
                    return this;
                };
                GridOptions.prototype.setDefaultColumnWidth = function (defaultColumnWidth) {
                    this.defaultColumnWidth = defaultColumnWidth;
                    return this;
                };
                GridOptions.prototype.setDefaultFormatter = function (defaultFormatter) {
                    this.defaultFormatter = defaultFormatter;
                    return this;
                };
                GridOptions.prototype.setEditable = function (editable) {
                    this.editable = editable;
                    return this;
                };
                GridOptions.prototype.setEditCommandHandler = function (editCommandHandler) {
                    this.editCommandHandler = editCommandHandler;
                    return this;
                };
                GridOptions.prototype.setEditorFactory = function (editorFactory) {
                    this.editorFactory = editorFactory;
                    return this;
                };
                GridOptions.prototype.setEditorLock = function (editorLock) {
                    this.editorLock = editorLock;
                    return this;
                };
                GridOptions.prototype.setEnableAddRow = function (enableAddRow) {
                    this.enableAddRow = enableAddRow;
                    return this;
                };
                GridOptions.prototype.setEnableAsyncPostRender = function (enableAsyncPostRender) {
                    this.enableAsyncPostRender = enableAsyncPostRender;
                    return this;
                };
                GridOptions.prototype.setEnableCellRangeSelection = function (enableCellRangeSelection) {
                    this.enableCellRangeSelection = enableCellRangeSelection;
                    return this;
                };
                GridOptions.prototype.setEnableCellNavigation = function (enableCellNavigation) {
                    this.enableCellNavigation = enableCellNavigation;
                    return this;
                };
                GridOptions.prototype.setEnableColumnReorder = function (enableColumnReorder) {
                    this.enableColumnReorder = enableColumnReorder;
                    return this;
                };
                GridOptions.prototype.setEnableRowReordering = function (enableRowReordering) {
                    this.enableRowReordering = enableRowReordering;
                    return this;
                };
                GridOptions.prototype.setEnableTextSelectionOnCells = function (enableTextSelectionOnCells) {
                    this.enableTextSelectionOnCells = enableTextSelectionOnCells;
                    return this;
                };
                GridOptions.prototype.setExplicitInitialization = function (explicitInitialization) {
                    this.explicitInitialization = explicitInitialization;
                    return this;
                };
                GridOptions.prototype.setForceFitColumns = function (forceFitColumns) {
                    this.forceFitColumns = forceFitColumns;
                    return this;
                };
                GridOptions.prototype.setForceSyncScrolling = function (forceSyncScrolling) {
                    this.forceSyncScrolling = forceSyncScrolling;
                    return this;
                };
                GridOptions.prototype.setFormatterFactory = function (formatterFactory) {
                    this.formatterFactory = formatterFactory;
                    return this;
                };
                GridOptions.prototype.setFullWidthRows = function (fullWidthRows) {
                    this.fullWidthRows = fullWidthRows;
                    return this;
                };
                GridOptions.prototype.setHeaderRowHeight = function (headerRowHeight) {
                    this.headerRowHeight = headerRowHeight;
                    return this;
                };
                GridOptions.prototype.setLeaveSpaceForNewRows = function (leaveSpaceForNewRows) {
                    this.leaveSpaceForNewRows = leaveSpaceForNewRows;
                    return this;
                };
                GridOptions.prototype.setMultiColumnSort = function (multiColumnSort) {
                    this.multiColumnSort = multiColumnSort;
                    return this;
                };
                GridOptions.prototype.setMultiSelect = function (multiSelect) {
                    this.multiSelect = multiSelect;
                    return this;
                };
                GridOptions.prototype.setRowHeight = function (rowHeight) {
                    this.rowHeight = rowHeight;
                    return this;
                };
                GridOptions.prototype.setSelectedCellCssClass = function (selectedCellCssClass) {
                    this.selectedCellCssClass = selectedCellCssClass;
                    return this;
                };
                GridOptions.prototype.setShowHeaderRow = function (showHeaderRow) {
                    this.showHeaderRow = showHeaderRow;
                    return this;
                };
                GridOptions.prototype.setSyncColumnCellResize = function (syncColumnCellResize) {
                    this.syncColumnCellResize = syncColumnCellResize;
                    return this;
                };
                GridOptions.prototype.setTopPanelHeight = function (topPanelHeight) {
                    this.topPanelHeight = topPanelHeight;
                    return this;
                };
                GridOptions.prototype.setHideColumnHeaders = function (hideColumnHeaders) {
                    this.hideColumnHeaders = hideColumnHeaders;
                    return this;
                };
                GridOptions.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                GridOptions.prototype.setHeight = function (height) {
                    this.height = height;
                    return this;
                };
                GridOptions.prototype.setDataIdProperty = function (dataIdProperty) {
                    this.dataIdProperty = dataIdProperty;
                    return this;
                };
                GridOptions.prototype.setAutoRenderGridOnDataChanges = function (autoRenderGridOnDataChanges) {
                    this.autoRenderGridOnDataChanges = autoRenderGridOnDataChanges;
                    return this;
                };
                GridOptions.prototype.setCheckableRows = function (checkableRows) {
                    this.checkableRows = checkableRows;
                    return this;
                };
                GridOptions.prototype.disableMultipleSelection = function (disabledMultipleSelection) {
                    this.disabledMultipleSelection = disabledMultipleSelection;
                    return this;
                };
                GridOptions.prototype.setDragAndDrop = function (dragAndDrop) {
                    this.dragAndDrop = dragAndDrop;
                    return this;
                };
                return GridOptions;
            })();
            grid.GridOptions = GridOptions;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (_grid) {
            var GridOnClickDataBuilder = (function () {
                function GridOnClickDataBuilder(source) {
                    if (source) {
                        this.row = source.getRow();
                        this.cell = source.getCell();
                        this.grid = source.getGrid();
                    }
                }
                GridOnClickDataBuilder.prototype.setRow = function (row) {
                    this.row = row;
                    return this;
                };
                GridOnClickDataBuilder.prototype.setCell = function (cell) {
                    this.cell = cell;
                    return this;
                };
                GridOnClickDataBuilder.prototype.setGrid = function (grid) {
                    this.grid = grid;
                    return this;
                };
                GridOnClickDataBuilder.prototype.build = function () {
                    return new GridOnClickData(this);
                };
                return GridOnClickDataBuilder;
            })();
            _grid.GridOnClickDataBuilder = GridOnClickDataBuilder;
            var GridOnClickData = (function () {
                function GridOnClickData(builder) {
                    this.row = builder.row;
                    this.cell = builder.cell;
                    this.grid = builder.grid;
                }
                GridOnClickData.prototype.getRow = function () {
                    return this.row;
                };
                GridOnClickData.prototype.getCell = function () {
                    return this.cell;
                };
                GridOnClickData.prototype.getGrid = function () {
                    return this.grid;
                };
                return GridOnClickData;
            })();
            _grid.GridOnClickData = GridOnClickData;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (_grid) {
            var DataView = (function () {
                function DataView() {
                    this.slickDataView = new Slick.Data.DataView({ inlineFilters: true });
                }
                DataView.prototype.slick = function () {
                    return this.slickDataView;
                };
                DataView.prototype.setFilter = function (f) {
                    this.slickDataView.setFilter(f);
                };
                DataView.prototype.setFilterArgs = function (args) {
                    this.slickDataView.setFilterArgs(args);
                };
                DataView.prototype.beginUpdate = function () {
                    this.slickDataView.beginUpdate();
                };
                DataView.prototype.endUpdate = function () {
                    this.slickDataView.endUpdate();
                };
                DataView.prototype.refresh = function () {
                    this.slickDataView.refresh();
                };
                DataView.prototype.setItems = function (items, objectIdProperty) {
                    this.slickDataView.setItems(items, objectIdProperty);
                };
                DataView.prototype.addItem = function (item) {
                    this.slickDataView.addItem(item);
                };
                DataView.prototype.insertItem = function (insertBefore, item) {
                    this.slickDataView.insertItem(insertBefore, item);
                };
                DataView.prototype.updateItem = function (id, item) {
                    this.slickDataView.updateItem(id, item);
                };
                DataView.prototype.deleteItem = function (id) {
                    this.slickDataView.deleteItem(id);
                };
                DataView.prototype.syncGridSelection = function (grid, preserveHidden) {
                    this.slickDataView.syncGridSelection(grid, preserveHidden);
                };
                DataView.prototype.getItem = function (index) {
                    return this.slickDataView.getItem(index);
                };
                DataView.prototype.getItems = function () {
                    return this.slickDataView.getItems();
                };
                DataView.prototype.getItemById = function (id) {
                    return this.slickDataView.getItemById(id);
                };
                DataView.prototype.getLength = function () {
                    return this.slickDataView.getLength();
                };
                DataView.prototype.getRowById = function (id) {
                    return this.slickDataView.getRowById(id);
                };
                DataView.prototype.onRowsChanged = function (callback) {
                    this.slickDataView.onRowsChanged.subscribe(callback);
                };
                DataView.prototype.onRowCountChanged = function (listener) {
                    this.slickDataView.onRowCountChanged.subscribe(listener);
                };
                return DataView;
            })();
            _grid.DataView = DataView;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var CollapsingItem = (function () {
                function CollapsingItem(actualItem) {
                    this.actualItem = actualItem;
                }
                return CollapsingItem;
            })();
            grid.CollapsingItem = CollapsingItem;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var MenuItem = (function (_super) {
                __extends(MenuItem, _super);
                function MenuItem(action) {
                    var _this = this;
                    _super.call(this, "menu-item");
                    this.action = action;
                    this.getEl().setInnerHtml(this.action.getLabel());
                    this.onClicked(function (event) {
                        if (action.isEnabled()) {
                            _this.action.execute();
                        }
                    });
                    this.setEnabled(action.isEnabled());
                    action.onPropertyChanged(function (action) {
                        _this.setEnabled(action.isEnabled());
                        _this.setVisible(action.isVisible());
                    });
                }
                MenuItem.prototype.getAction = function () {
                    return this.action;
                };
                MenuItem.prototype.setEnabled = function (value) {
                    var el = this.getEl();
                    el.setDisabled(!value);
                    if (value) {
                        el.removeClass("disabled");
                    }
                    else {
                        el.addClass("disabled");
                    }
                };
                return MenuItem;
            })(api.dom.LiEl);
            menu.MenuItem = MenuItem;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (_menu) {
            var ContextMenu = (function (_super) {
                __extends(ContextMenu, _super);
                function ContextMenu(actions, appendToBody) {
                    var _this = this;
                    if (appendToBody === void 0) { appendToBody = true; }
                    _super.call(this, "context-menu");
                    this.menuItems = [];
                    this.hideOnItemClick = true;
                    this.itemClickListeners = [];
                    if (actions) {
                        for (var i = 0; i < actions.length; i++) {
                            this.addAction(actions[i]);
                        }
                    }
                    if (appendToBody) {
                        api.dom.Body.get().appendChild(this);
                        api.dom.Body.get().onClicked(function (event) { return _this.hideMenuOnOutsideClick(event); });
                    }
                    this.onClicked(function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                    });
                }
                ContextMenu.prototype.addAction = function (action) {
                    var menuItem = this.createMenuItem(action);
                    this.appendChild(menuItem);
                    return this;
                };
                ContextMenu.prototype.addActions = function (actions) {
                    var _this = this;
                    actions.forEach(function (action) {
                        _this.addAction(action);
                    });
                    return this;
                };
                ContextMenu.prototype.removeAction = function (action) {
                    var menuItem = this.getMenuItem(action);
                    if (menuItem) {
                        this.removeMenuItem(menuItem);
                        this.removeChild(menuItem);
                    }
                    return this;
                };
                ContextMenu.prototype.removeActions = function (actions) {
                    var _this = this;
                    actions.forEach(function (action) {
                        _this.removeAction(action);
                    });
                    return this;
                };
                ContextMenu.prototype.setActions = function (actions) {
                    this.menuItems.length = 0;
                    this.removeChildren();
                    this.addActions(actions);
                    return this;
                };
                ContextMenu.prototype.setHideOnItemClick = function (hide) {
                    this.hideOnItemClick = hide;
                    return this;
                };
                ContextMenu.prototype.showAt = function (x, y) {
                    ContextMenu.prototype.doMoveTo(this, x, y);
                    this.show();
                };
                ContextMenu.prototype.moveBy = function (dx, dy) {
                    var offset = this.getEl().getOffsetToParent();
                    ContextMenu.prototype.doMoveTo(this, offset.left + dx, offset.top + dy);
                };
                ContextMenu.prototype.doMoveTo = function (menu, x, y) {
                    menu.getEl().setLeftPx(x).setTopPx(y);
                };
                ContextMenu.prototype.onItemClicked = function (listener) {
                    this.itemClickListeners.push(listener);
                };
                ContextMenu.prototype.unItemClicked = function (listener) {
                    this.itemClickListeners = this.itemClickListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                ContextMenu.prototype.notifyItemClicked = function (item) {
                    this.itemClickListeners.forEach(function (listener) {
                        listener(item);
                    });
                };
                ContextMenu.prototype.createMenuItem = function (action) {
                    var _this = this;
                    var menuItem = new _menu.MenuItem(action);
                    menuItem.onClicked(function (event) {
                        _this.notifyItemClicked(menuItem);
                        if (_this.hideOnItemClick) {
                            _this.hide();
                        }
                        event.preventDefault();
                        event.stopPropagation();
                    });
                    this.menuItems.push(menuItem);
                    return menuItem;
                };
                ContextMenu.prototype.removeMenuItem = function (menuItem) {
                    this.menuItems = this.menuItems.filter(function (item) {
                        return item != menuItem;
                    });
                };
                ContextMenu.prototype.getMenuItem = function (action) {
                    for (var i = 0; i < this.menuItems.length; i++) {
                        var menuItem = this.menuItems[i];
                        if (menuItem.getAction() == action) {
                            return menuItem;
                        }
                    }
                    return undefined;
                };
                ContextMenu.prototype.hideMenuOnOutsideClick = function (evt) {
                    if (!this.getEl().contains(evt.target)) {
                        this.hide();
                    }
                };
                return ContextMenu;
            })(api.dom.UlEl);
            _menu.ContextMenu = ContextMenu;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var ActionMenuItem = (function (_super) {
                __extends(ActionMenuItem, _super);
                function ActionMenuItem(action) {
                    var _this = this;
                    _super.call(this, "action");
                    this.action = action;
                    this.getEl().setInnerHtml(this.action.getLabel());
                    this.action.onPropertyChanged(function () {
                        if (_this.action.isEnabled()) {
                            _this.show();
                        }
                        else if (!_this.action.isEnabled()) {
                            _this.hide();
                        }
                    });
                    this.onClicked(function () {
                        _this.action.execute();
                    });
                }
                return ActionMenuItem;
            })(api.dom.LiEl);
            menu.ActionMenuItem = ActionMenuItem;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var ActionMenu = (function (_super) {
                __extends(ActionMenu, _super);
                function ActionMenu(label) {
                    var _this = this;
                    var actions = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        actions[_i - 1] = arguments[_i];
                    }
                    _super.call(this, "action-menu");
                    this.labelEl = new api.dom.DivEl("drop-down-button icon-arrow-down");
                    this.labelEl.setHtml(label);
                    this.appendChild(this.labelEl);
                    this.actionListEl = new api.dom.UlEl();
                    this.appendChild(this.actionListEl);
                    if (actions.length > 0) {
                        actions.forEach(function (action) {
                            _this.addAction(action);
                        });
                    }
                    this.labelEl.onClicked(function (event) {
                        if (_this.hasClass("expanded")) {
                            _this.removeClass("expanded");
                        }
                        else {
                            _this.addClass("expanded");
                        }
                    });
                }
                ActionMenu.prototype.setLabel = function (label) {
                    this.labelEl.getEl().setInnerHtml(label);
                };
                ActionMenu.prototype.addAction = function (action) {
                    var _this = this;
                    var actionMenuItem = new menu.ActionMenuItem(action);
                    this.actionListEl.appendChild(actionMenuItem);
                    actionMenuItem.onClicked(function () {
                        _this.removeClass("expanded");
                    });
                };
                return ActionMenu;
            })(api.dom.DivEl);
            menu.ActionMenu = ActionMenu;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var toolbar;
        (function (toolbar) {
            var Toolbar = (function (_super) {
                __extends(Toolbar, _super);
                function Toolbar() {
                    var _this = this;
                    _super.call(this, "toolbar");
                    this.actions = [];
                    this.fold = new toolbar.FoldButton();
                    this.fold.hide();
                    this.appendChild(this.fold);
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.foldOrExpand(); }, this);
                    this.onShown(function (event) { return _this.foldOrExpand(); });
                }
                Toolbar.prototype.addAction = function (action) {
                    this.actions.push(action);
                    this.addElement(new api.ui.button.ActionButton(action));
                };
                Toolbar.prototype.addActions = function (actions) {
                    var _this = this;
                    this.actions = this.actions.concat(actions);
                    actions.forEach(function (action) {
                        _this.addElement(new api.ui.button.ActionButton(action));
                    });
                };
                Toolbar.prototype.removeActions = function () {
                    var _this = this;
                    this.actions.forEach(function (action) {
                        _this.getChildren().forEach(function (element) {
                            if (api.ObjectHelper.iFrameSafeInstanceOf(element, api.ui.button.ActionButton)) {
                                if (action.getLabel() == element.getLabel()) {
                                    _this.removeChild(element);
                                }
                            }
                        });
                    });
                    this.actions = [];
                };
                Toolbar.prototype.getActions = function () {
                    return this.actions;
                };
                Toolbar.prototype.addElement = function (element) {
                    if (this.hasGreedySpacer) {
                        element.addClass('pull-right');
                        element.insertAfterEl(this.fold);
                    }
                    else {
                        element.insertBeforeEl(this.fold);
                    }
                };
                Toolbar.prototype.addGreedySpacer = function () {
                    this.hasGreedySpacer = true;
                };
                Toolbar.prototype.foldOrExpand = function () {
                    if (!this.isVisible()) {
                        return;
                    }
                    var toolbarWidth = this.getEl().getWidth();
                    if (toolbarWidth < this.getVisibleButtonsWidth()) {
                        while (toolbarWidth <= this.getVisibleButtonsWidth() && this.getNextFoldableButton()) {
                            var buttonToHide = this.getNextFoldableButton();
                            var buttonWidth = buttonToHide.getEl().getWidthWithBorder();
                            this.removeChild(buttonToHide);
                            this.fold.push(buttonToHide, buttonWidth);
                            if (!this.fold.isVisible()) {
                                this.fold.show();
                            }
                        }
                    }
                    else {
                        while (!this.fold.isEmpty() && (this.getVisibleButtonsWidth(this.fold.getButtonsCount() > 1) + this.fold.getNextButtonWidth() < toolbarWidth)) {
                            var buttonToShow = this.fold.pop();
                            buttonToShow.insertBeforeEl(this.fold);
                            if (this.fold.isEmpty()) {
                                this.fold.hide();
                            }
                        }
                    }
                    this.fold.setLabel(this.getFoldableButtons().length == 0 ? 'Actions' : 'More');
                };
                Toolbar.prototype.getVisibleButtonsWidth = function (includeFold) {
                    var _this = this;
                    if (includeFold === void 0) { includeFold = true; }
                    return this.getChildren().reduce(function (totalWidth, element) {
                        return totalWidth + (element.isVisible() && (includeFold || element != _this.fold) ? element.getEl().getWidthWithMargin() : 0);
                    }, 0);
                };
                Toolbar.prototype.getNextFoldableButton = function () {
                    return this.getChildren()[this.getChildren().indexOf(this.fold) - 1];
                };
                Toolbar.prototype.getFoldableButtons = function () {
                    return this.getChildren().slice(0, this.getChildren().indexOf(this.fold));
                };
                return Toolbar;
            })(api.dom.DivEl);
            toolbar.Toolbar = Toolbar;
        })(toolbar = ui.toolbar || (ui.toolbar = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var toolbar;
        (function (toolbar) {
            var FoldButton = (function (_super) {
                __extends(FoldButton, _super);
                function FoldButton() {
                    _super.call(this, 'button fold-button');
                    this.widthCache = [];
                    this.span = new api.dom.SpanEl();
                    this.span.setHtml("More");
                    this.appendChild(this.span);
                    this.dropdown = new api.dom.DivEl('dropdown');
                    this.appendChild(this.dropdown);
                }
                FoldButton.prototype.push = function (element, width) {
                    this.dropdown.prependChild(element);
                    this.widthCache.unshift(width);
                };
                FoldButton.prototype.pop = function () {
                    var top = this.dropdown.getFirstChild();
                    this.dropdown.removeChild(top);
                    this.widthCache.shift();
                    return top;
                };
                FoldButton.prototype.setLabel = function (label) {
                    this.span.setHtml(label);
                };
                FoldButton.prototype.getNextButtonWidth = function () {
                    return this.widthCache[0];
                };
                FoldButton.prototype.getButtonsCount = function () {
                    return this.dropdown.getChildren().length;
                };
                FoldButton.prototype.isEmpty = function () {
                    return this.dropdown.getChildren().length == 0;
                };
                return FoldButton;
            })(api.dom.DivEl);
            toolbar.FoldButton = FoldButton;
        })(toolbar = ui.toolbar || (ui.toolbar = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var Mask = (function (_super) {
                __extends(Mask, _super);
                function Mask(itemToMask) {
                    var _this = this;
                    _super.call(this, "mask");
                    this.masked = itemToMask;
                    if (this.masked) {
                        this.masked.onHidden(function (event) {
                            _this.hide();
                        });
                        this.masked.onRemoved(function (event) {
                            _this.remove();
                        });
                        api.dom.WindowDOM.get().onResized(function (event) {
                            if (_this.isVisible()) {
                                _this.positionOver(_this.masked);
                            }
                        }, this);
                    }
                    api.dom.Body.get().appendChild(this);
                }
                Mask.prototype.show = function () {
                    _super.prototype.show.call(this);
                    if (this.masked) {
                        this.positionOver(this.masked);
                    }
                };
                Mask.prototype.positionOver = function (masked) {
                    var maskedEl = masked.getEl(), maskEl = this.getEl(), maskedOffset, isMaskedPositioned = maskedEl.getPosition() != 'static', maskedDimensions = {
                        width: maskedEl.getWidthWithBorder() + "px",
                        height: maskedEl.getHeightWithBorder() + "px"
                    };
                    if (masked.contains(this) && isMaskedPositioned) {
                        maskedOffset = {
                            top: 0,
                            left: 0
                        };
                        if (maskedEl.getPosition() == 'absolute') {
                            maskedDimensions = {
                                width: '100%',
                                height: '100%'
                            };
                        }
                    }
                    else {
                        var maskedParent = maskedEl.getOffsetParent(), maskParent = maskEl.getOffsetParent();
                        maskedOffset = maskedEl.getOffsetToParent();
                        if (maskedParent != maskParent) {
                            var maskedParentOffset = new api.dom.ElementHelper(maskedParent).getOffset(), maskParentOffset = new api.dom.ElementHelper(maskParent).getOffset();
                            maskedOffset.left = maskedOffset.left + (maskedParentOffset.left - maskParentOffset.left);
                            maskedOffset.top = maskedOffset.top + (maskedParentOffset.top - maskParentOffset.top);
                        }
                        if (!isMaskedPositioned) {
                            maskedOffset.top += maskedEl.getMarginTop();
                            maskedOffset.left += maskedEl.getMarginLeft();
                        }
                    }
                    this.getEl().setTopPx(maskedOffset.top).setLeftPx(maskedOffset.left).setWidth(maskedDimensions.width).setHeight(maskedDimensions.height);
                };
                Mask.prototype.getMasked = function () {
                    return this.masked;
                };
                return Mask;
            })(api.dom.DivEl);
            mask.Mask = Mask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var BodyMask = (function (_super) {
                __extends(BodyMask, _super);
                function BodyMask() {
                    _super.call(this);
                    this.addClass("body-mask");
                }
                BodyMask.get = function () {
                    if (!BodyMask.instance) {
                        BodyMask.instance = new BodyMask();
                    }
                    return BodyMask.instance;
                };
                return BodyMask;
            })(mask.Mask);
            mask.BodyMask = BodyMask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var LoadMask = (function (_super) {
                __extends(LoadMask, _super);
                function LoadMask(el) {
                    _super.call(this, el);
                    this.addClass("load-mask");
                    this.splash = new api.dom.DivEl("mask-splash");
                    this.spinner = new api.dom.DivEl("spinner");
                    this.splash.appendChild(this.spinner);
                    this.appendChild(this.splash);
                }
                LoadMask.prototype.show = function () {
                    _super.prototype.show.call(this);
                    this.splash.show();
                    this.centerSplash();
                };
                LoadMask.prototype.hide = function () {
                    this.splash.hide();
                    _super.prototype.hide.call(this);
                };
                LoadMask.prototype.setText = function (text) {
                    if (!text) {
                        if (this.text) {
                            this.text.hide();
                        }
                    }
                    else {
                        if (!this.text) {
                            this.text = new api.dom.SpanEl("text");
                            this.splash.appendChild(this.text);
                        }
                        this.text.getEl().setInnerHtml(text);
                    }
                };
                LoadMask.prototype.getText = function () {
                    return this.text.getEl().getInnerHtml();
                };
                LoadMask.prototype.centerSplash = function () {
                    var loaderEl = this.splash.getEl();
                    loaderEl.setMarginLeft("-" + loaderEl.getWidthWithBorder() / 2 + "px");
                    loaderEl.setMarginTop("-" + loaderEl.getHeightWithBorder() / 2 + "px");
                };
                return LoadMask;
            })(mask.Mask);
            mask.LoadMask = LoadMask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var DragMask = (function (_super) {
                __extends(DragMask, _super);
                function DragMask(itemToMask) {
                    _super.call(this, itemToMask);
                    this.addClass("drag-mask");
                }
                return DragMask;
            })(mask.Mask);
            mask.DragMask = DragMask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var OptionFilterInput = (function (_super) {
                __extends(OptionFilterInput, _super);
                function OptionFilterInput() {
                    _super.call(this, "option-filter-input");
                    this.setPlaceholder("Type to search...");
                }
                OptionFilterInput.prototype.openForTypingAndFocus = function () {
                    this.openForTyping();
                    this.getHTMLElement().focus();
                };
                OptionFilterInput.prototype.openForTyping = function () {
                    this.setPlaceholder("Type to search...");
                    this.getEl().setDisabled(false);
                };
                return OptionFilterInput;
            })(api.ui.text.TextInput);
            selector.OptionFilterInput = OptionFilterInput;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var OptionFilterInputValueChangedEvent = (function () {
                function OptionFilterInputValueChangedEvent(oldValue, newValue, grid) {
                    this.oldValue = oldValue;
                    this.newValue = newValue;
                    this.grid = grid;
                }
                OptionFilterInputValueChangedEvent.prototype.getOldValue = function () {
                    return this.oldValue;
                };
                OptionFilterInputValueChangedEvent.prototype.getNewValue = function () {
                    return this.newValue;
                };
                OptionFilterInputValueChangedEvent.prototype.getGrid = function () {
                    return this.grid;
                };
                return OptionFilterInputValueChangedEvent;
            })();
            selector.OptionFilterInputValueChangedEvent = OptionFilterInputValueChangedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var OptionSelectedEvent = (function () {
                function OptionSelectedEvent(option) {
                    this.option = option;
                }
                OptionSelectedEvent.prototype.getOption = function () {
                    return this.option;
                };
                return OptionSelectedEvent;
            })();
            selector.OptionSelectedEvent = OptionSelectedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownHandle = (function (_super) {
                __extends(DropdownHandle, _super);
                function DropdownHandle() {
                    _super.call(this, "dropdown-handle");
                    this.setEnabled(true);
                    this.removeClass('down');
                }
                DropdownHandle.prototype.setEnabled = function (value) {
                    if (value) {
                        this.removeClass('disabled');
                    }
                    else {
                        this.addClass('disabled');
                    }
                };
                DropdownHandle.prototype.down = function () {
                    this.addClass('down');
                };
                DropdownHandle.prototype.up = function () {
                    this.removeClass('down');
                };
                return DropdownHandle;
            })(api.dom.DivEl);
            selector.DropdownHandle = DropdownHandle;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownGridRowSelectedEvent = (function () {
                function DropdownGridRowSelectedEvent(row) {
                    this.row = row;
                }
                DropdownGridRowSelectedEvent.prototype.getRow = function () {
                    return this.row;
                };
                return DropdownGridRowSelectedEvent;
            })();
            selector.DropdownGridRowSelectedEvent = DropdownGridRowSelectedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownGridMultipleSelectionEvent = (function () {
                function DropdownGridMultipleSelectionEvent(rows) {
                    this.rows = rows;
                }
                DropdownGridMultipleSelectionEvent.prototype.getRows = function () {
                    return this.rows;
                };
                return DropdownGridMultipleSelectionEvent;
            })();
            selector.DropdownGridMultipleSelectionEvent = DropdownGridMultipleSelectionEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DefaultOptionDisplayValueViewer = (function (_super) {
                __extends(DefaultOptionDisplayValueViewer, _super);
                function DefaultOptionDisplayValueViewer() {
                    _super.call(this);
                }
                DefaultOptionDisplayValueViewer.prototype.setObject = function (object) {
                    _super.prototype.setObject.call(this, object);
                    this.getEl().setInnerHtml(object.toString());
                };
                DefaultOptionDisplayValueViewer.prototype.getPreferredHeight = function () {
                    return 34;
                };
                return DefaultOptionDisplayValueViewer;
            })(api.ui.Viewer);
            selector.DefaultOptionDisplayValueViewer = DefaultOptionDisplayValueViewer;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownGrid = (function () {
                function DropdownGrid(config) {
                    var _this = this;
                    this.rowSelectionListeners = [];
                    this.multipleSelectionListeners = [];
                    this.maxHeight = config.maxHeight || 200;
                    this.optionDisplayValueViewer = config.optionDisplayValueViewer ? new config.optionDisplayValueViewer['constructor']() : new selector.DefaultOptionDisplayValueViewer();
                    this.filter = config.filter;
                    this.dataIdProperty = config.dataIdProperty || "value";
                    this.maxHeight = config.maxHeight;
                    this.width = config.width;
                    this.multipleSelections = config.multipleSelections || false;
                    var columnFormatter = function (row, cell, value, columnDef, dataContext) {
                        _this.optionDisplayValueViewer.setObject(value);
                        return _this.optionDisplayValueViewer.toString();
                    };
                    var columns = [
                        new api.ui.grid.GridColumnBuilder().setId("option").setName("Options").setField("displayValue").setFormatter(columnFormatter).build()
                    ];
                    var options = new api.ui.grid.GridOptionsBuilder().setWidth(this.width).setHeight(this.maxHeight).setHideColumnHeaders(true).setEnableColumnReorder(false).setFullWidthRows(true).setForceFitColumns(true).setRowHeight(this.optionDisplayValueViewer.getPreferredHeight()).setCheckableRows(this.multipleSelections).setMultiSelect(this.multipleSelections).setDataIdProperty(this.dataIdProperty).build();
                    this.gridData = new api.ui.grid.DataView();
                    if (this.filter) {
                        this.gridData.setFilter(this.filter);
                    }
                    this.grid = new api.ui.grid.Grid(this.gridData, columns, options);
                    this.grid.addClass("options-container");
                    this.grid.getEl().setPosition("absolute");
                    this.grid.hide();
                    this.grid.setSelectionModel(new Slick.RowSelectionModel({ selectActiveRow: false }));
                    this.grid.subscribeOnClick(function (e, args) {
                        _this.notifyRowSelection(args.row);
                        e.preventDefault();
                        e.stopPropagation();
                        return false;
                    });
                    this.grid.subscribeOnSelectedRowsChanged(function (e, args) {
                        _this.notifyMultipleSelection(args.rows);
                    });
                    this.gridData.onRowsChanged(function (e, args) {
                    });
                    this.gridData.onRowCountChanged(function (e, args) {
                    });
                }
                DropdownGrid.prototype.getElement = function () {
                    return this.grid;
                };
                DropdownGrid.prototype.renderGrid = function () {
                    this.grid.render();
                };
                DropdownGrid.prototype.isVisible = function () {
                    return this.grid.isVisible();
                };
                DropdownGrid.prototype.show = function () {
                    this.grid.show();
                };
                DropdownGrid.prototype.hide = function () {
                    this.grid.hide();
                };
                DropdownGrid.prototype.setOptions = function (options) {
                    this.gridData.setItems(options, this.dataIdProperty);
                };
                DropdownGrid.prototype.removeAllOptions = function () {
                    this.gridData.setItems([]);
                };
                DropdownGrid.prototype.addOption = function (option) {
                    this.gridData.addItem(option);
                };
                DropdownGrid.prototype.hasOptions = function () {
                    return this.gridData.getLength() > 0;
                };
                DropdownGrid.prototype.getSelectedOptionCount = function () {
                    return this.grid.getSelectedRows().length;
                };
                DropdownGrid.prototype.getOptionCount = function () {
                    return this.gridData.getLength();
                };
                DropdownGrid.prototype.getOptions = function () {
                    return this.gridData.getItems();
                };
                DropdownGrid.prototype.getOptionByValue = function (value) {
                    return this.gridData.getItemById(value);
                };
                DropdownGrid.prototype.getOptionByRow = function (rowIndex) {
                    return this.gridData.getItem(rowIndex);
                };
                DropdownGrid.prototype.setFilterArgs = function (args) {
                    this.gridData.setFilterArgs(args);
                    this.gridData.refresh();
                };
                DropdownGrid.prototype.setTopPx = function (value) {
                    this.grid.getEl().setTopPx(value);
                };
                DropdownGrid.prototype.setWidthPx = function (value) {
                    this.grid.getEl().setWidthPx(value);
                };
                DropdownGrid.prototype.adjustGridHeight = function () {
                    var gridEl = this.grid.getEl();
                    var rowsHeight = this.getOptionCount() * this.optionDisplayValueViewer.getPreferredHeight();
                    if (rowsHeight < this.maxHeight) {
                        var borderWidth = gridEl.getBorderTopWidth() + gridEl.getBorderBottomWidth();
                        gridEl.setHeightPx(rowsHeight + borderWidth);
                        this.grid.getOptions().setAutoHeight(true);
                    }
                    else if (gridEl.getHeight() < this.maxHeight) {
                        gridEl.setHeightPx(this.maxHeight);
                        this.grid.getOptions().setAutoHeight(false);
                    }
                    this.grid.resizeCanvas();
                };
                DropdownGrid.prototype.markSelections = function (selectedOptions, ignoreEmpty) {
                    var _this = this;
                    if (ignoreEmpty === void 0) { ignoreEmpty = false; }
                    var stylesHash = {};
                    var rows = [];
                    selectedOptions.forEach(function (selectedOption) {
                        var row = _this.gridData.getRowById(selectedOption.value);
                        rows.push(row);
                        stylesHash[row] = { option: "selected" };
                    });
                    this.grid.setCellCssStyles("selected", stylesHash);
                    if (!(rows.length === 0 && ignoreEmpty)) {
                        this.grid.setSelectedRows(rows);
                    }
                };
                DropdownGrid.prototype.hasActiveRow = function () {
                    var activeCell = this.grid.getActiveCell();
                    if (activeCell) {
                        return true;
                    }
                    else {
                        return false;
                    }
                };
                DropdownGrid.prototype.getActiveRow = function () {
                    var activeCell = this.grid.getActiveCell();
                    if (activeCell) {
                        return activeCell.row;
                    }
                    else {
                        return -1;
                    }
                };
                DropdownGrid.prototype.nagivateToFirstRow = function () {
                    this.grid.setActiveCell(0, 0);
                };
                DropdownGrid.prototype.navigateToFirstRowIfNotActive = function () {
                    if (!this.grid.getActiveCell()) {
                        this.grid.setActiveCell(0, 0);
                    }
                };
                DropdownGrid.prototype.navigateToNextRow = function () {
                    this.grid.navigateDown();
                };
                DropdownGrid.prototype.navigateToPreviousRow = function () {
                    this.grid.navigateUp();
                };
                DropdownGrid.prototype.resetActiveSelection = function () {
                    if (this.grid.getActiveCell()) {
                        this.grid.resetActiveCell();
                    }
                };
                DropdownGrid.prototype.toggleRowSelection = function (row, isMaximumReached) {
                    if (isMaximumReached === void 0) { isMaximumReached = false; }
                    var rows = this.grid.getSelectedRows();
                    var oldRows = rows.join();
                    var index = rows.indexOf(row);
                    if (index >= 0) {
                        rows.splice(index, 1);
                    }
                    else if (!isMaximumReached) {
                        rows.push(row);
                    }
                    if (oldRows !== rows.join()) {
                        this.grid.setSelectedRows(rows);
                    }
                };
                DropdownGrid.prototype.onRowSelection = function (listener) {
                    this.rowSelectionListeners.push(listener);
                };
                DropdownGrid.prototype.unRowSelection = function (listener) {
                    this.rowSelectionListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                DropdownGrid.prototype.onMultipleSelection = function (listener) {
                    this.multipleSelectionListeners.push(listener);
                };
                DropdownGrid.prototype.unMultipleSelection = function (listener) {
                    this.multipleSelectionListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                DropdownGrid.prototype.applyMultipleSelection = function () {
                    var rows = this.grid.getSelectedRows();
                    for (var key in rows) {
                        this.notifyRowSelection(rows[key]);
                    }
                };
                DropdownGrid.prototype.notifyRowSelection = function (rowSelected) {
                    var event = new selector.DropdownGridRowSelectedEvent(rowSelected);
                    this.rowSelectionListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                DropdownGrid.prototype.notifyMultipleSelection = function (rowsSelected) {
                    var event = new selector.DropdownGridMultipleSelectionEvent(rowsSelected);
                    this.multipleSelectionListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return DropdownGrid;
            })();
            selector.DropdownGrid = DropdownGrid;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var list;
            (function (list) {
                var ListBox = (function (_super) {
                    __extends(ListBox, _super);
                    function ListBox(className) {
                        _super.call(this, className);
                        this.items = [];
                        this.itemViews = {};
                        this.itemsAddedListeners = [];
                        this.itemsRemovedListeners = [];
                    }
                    ListBox.prototype.setItems = function (items) {
                        if (this.items.length > 0) {
                            this.notifyItemsRemoved(this.items);
                        }
                        this.items = items;
                        if (items.length > 0) {
                            this.layoutList(items);
                            this.notifyItemsAdded(items);
                        }
                    };
                    ListBox.prototype.getItems = function () {
                        return this.items;
                    };
                    ListBox.prototype.getItem = function (id) {
                        for (var i = 0; i < this.items.length; i++) {
                            var item = this.items[i];
                            if (this.getItemId(item) == id) {
                                return item;
                            }
                        }
                        return undefined;
                    };
                    ListBox.prototype.clearItems = function () {
                        if (this.items.length > 0) {
                            this.notifyItemsRemoved(this.items);
                        }
                        this.items.length = 0;
                        this.layoutList(this.items);
                    };
                    ListBox.prototype.addItem = function () {
                        var _this = this;
                        var items = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            items[_i - 0] = arguments[_i];
                        }
                        this.items = this.items.concat(items);
                        items.forEach(function (item) {
                            _this.addItemView(item);
                        });
                        if (items.length > 0) {
                            this.notifyItemsAdded(items);
                        }
                    };
                    ListBox.prototype.removeItem = function () {
                        var _this = this;
                        var items = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            items[_i - 0] = arguments[_i];
                        }
                        var itemsRemoved = [];
                        this.items = this.items.filter(function (item) {
                            for (var i = 0; i < items.length; i++) {
                                if (_this.getItemId(item) == _this.getItemId(items[i])) {
                                    _this.removeItemView(item);
                                    itemsRemoved.push(item);
                                    return false;
                                }
                            }
                            return true;
                        });
                        if (itemsRemoved.length > 0) {
                            this.notifyItemsRemoved(itemsRemoved);
                        }
                    };
                    ListBox.prototype.getItemCount = function () {
                        return this.items.length;
                    };
                    ListBox.prototype.createItemView = function (item) {
                        throw new Error("You must override createListItem to create views for list items");
                    };
                    ListBox.prototype.getItemId = function (item) {
                        throw new Error("You must override getItemId to find item views by items");
                    };
                    ListBox.prototype.getItemView = function (item) {
                        return this.itemViews[this.getItemId(item)];
                    };
                    ListBox.prototype.getItemViews = function () {
                        var _this = this;
                        return this.getItems().map(function (item) { return _this.getItemView(item); });
                    };
                    ListBox.prototype.refreshList = function () {
                        this.layoutList(this.items);
                    };
                    ListBox.prototype.layoutList = function (items) {
                        this.removeChildren();
                        for (var i = 0; i < items.length; i++) {
                            this.addItemView(items[i]);
                        }
                    };
                    ListBox.prototype.removeItemView = function (item) {
                        var itemView = this.itemViews[this.getItemId(item)];
                        if (itemView) {
                            this.removeChild(itemView);
                        }
                    };
                    ListBox.prototype.addItemView = function (item) {
                        var itemView = this.createItemView(item);
                        this.itemViews[this.getItemId(item)] = itemView;
                        this.appendChild(itemView);
                    };
                    ListBox.prototype.onItemsAdded = function (listener) {
                        this.itemsAddedListeners.push(listener);
                    };
                    ListBox.prototype.unItemsAdded = function (listener) {
                        this.itemsAddedListeners = this.itemsAddedListeners.filter(function (current) {
                            return current !== listener;
                        });
                    };
                    ListBox.prototype.notifyItemsAdded = function (items) {
                        this.itemsAddedListeners.forEach(function (listener) {
                            listener(items);
                        });
                    };
                    ListBox.prototype.onItemsRemoved = function (listener) {
                        this.itemsRemovedListeners.push(listener);
                    };
                    ListBox.prototype.unItemsRemoved = function (listener) {
                        this.itemsRemovedListeners = this.itemsRemovedListeners.filter(function (current) {
                            return current !== listener;
                        });
                    };
                    ListBox.prototype.notifyItemsRemoved = function (items) {
                        this.itemsRemovedListeners.forEach(function (listener) {
                            listener(items);
                        });
                    };
                    return ListBox;
                })(api.dom.UlEl);
                list.ListBox = ListBox;
            })(list = selector.list || (selector.list = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var ComboBoxOptionFilterInput = (function (_super) {
                    __extends(ComboBoxOptionFilterInput, _super);
                    function ComboBoxOptionFilterInput() {
                        _super.call(this);
                    }
                    ComboBoxOptionFilterInput.prototype.setMaximumReached = function () {
                        this.setPlaceholder("Maximum reached");
                        this.getEl().setDisabled(true);
                    };
                    ComboBoxOptionFilterInput.prototype.getWidth = function () {
                        return this.getEl().getWidth();
                    };
                    return ComboBoxOptionFilterInput;
                })(api.ui.selector.OptionFilterInput);
                combobox.ComboBoxOptionFilterInput = ComboBoxOptionFilterInput;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var DropdownGrid = api.ui.selector.DropdownGrid;
                var ComboBoxDropdown = (function () {
                    function ComboBoxDropdown(config) {
                        this.emptyDropdown = new api.dom.DivEl("empty-options");
                        this.emptyDropdown.getEl().setInnerHtml("No matching items");
                        this.emptyDropdown.hide();
                        this.dropdownGridConfig = {
                            maxHeight: config.maxHeight,
                            width: Math.max(config.width, 240),
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: config.filter,
                            dataIdProperty: config.dataIdProperty,
                            multipleSelections: config.multipleSelections
                        };
                        this.dropdownGrid = new DropdownGrid(this.dropdownGridConfig);
                    }
                    ComboBoxDropdown.prototype.getGrid = function () {
                        return this.dropdownGrid;
                    };
                    ComboBoxDropdown.prototype.renderDropdownGrid = function () {
                        this.dropdownGrid.renderGrid();
                    };
                    ComboBoxDropdown.prototype.getEmptyDropdown = function () {
                        return this.emptyDropdown;
                    };
                    ComboBoxDropdown.prototype.isDropdownShown = function () {
                        return this.emptyDropdown.isVisible() || this.dropdownGrid.isVisible();
                    };
                    ComboBoxDropdown.prototype.setOptions = function (options, selectedOptions) {
                        if (selectedOptions === void 0) { selectedOptions = []; }
                        this.dropdownGrid.setOptions(options);
                        if (this.isDropdownShown()) {
                            this.showDropdown(selectedOptions);
                        }
                    };
                    ComboBoxDropdown.prototype.removeAllOptions = function () {
                        this.dropdownGrid.removeAllOptions();
                    };
                    ComboBoxDropdown.prototype.addOption = function (option) {
                        this.dropdownGrid.addOption(option);
                    };
                    ComboBoxDropdown.prototype.hasOptions = function () {
                        return this.dropdownGrid.hasOptions();
                    };
                    ComboBoxDropdown.prototype.getSelectedOptionCount = function () {
                        return this.dropdownGrid.getSelectedOptionCount();
                    };
                    ComboBoxDropdown.prototype.getOptionCount = function () {
                        return this.dropdownGrid.getOptionCount();
                    };
                    ComboBoxDropdown.prototype.getOptions = function () {
                        return this.dropdownGrid.getOptions();
                    };
                    ComboBoxDropdown.prototype.getOptionByValue = function (value) {
                        return this.dropdownGrid.getOptionByValue(value);
                    };
                    ComboBoxDropdown.prototype.getOptionByRow = function (rowIndex) {
                        return this.dropdownGrid.getOptionByRow(rowIndex);
                    };
                    ComboBoxDropdown.prototype.setFilterArgs = function (args) {
                        this.dropdownGrid.setFilterArgs(args);
                    };
                    ComboBoxDropdown.prototype.showDropdown = function (selectedOptions) {
                        if (this.hasOptions()) {
                            this.emptyDropdown.hide();
                            this.dropdownGrid.show();
                            this.dropdownGrid.adjustGridHeight();
                            this.dropdownGrid.markSelections(selectedOptions);
                        }
                        else {
                            this.dropdownGrid.hide();
                            this.emptyDropdown.getEl().setInnerHtml("No matching items");
                            this.emptyDropdown.show();
                        }
                    };
                    ComboBoxDropdown.prototype.hideDropdown = function () {
                        this.emptyDropdown.hide();
                        this.dropdownGrid.hide();
                    };
                    ComboBoxDropdown.prototype.setEmptyDropdownText = function (label) {
                        if (this.isDropdownShown()) {
                            this.dropdownGrid.hide();
                            this.emptyDropdown.getEl().setInnerHtml(label);
                            this.emptyDropdown.show();
                        }
                    };
                    ComboBoxDropdown.prototype.setTopPx = function (value) {
                        this.dropdownGrid.setTopPx(value);
                        this.emptyDropdown.getEl().setTopPx(value);
                    };
                    ComboBoxDropdown.prototype.setWidth = function (value) {
                        this.dropdownGrid.setWidthPx(value);
                    };
                    ComboBoxDropdown.prototype.hasActiveRow = function () {
                        return this.dropdownGrid.hasActiveRow();
                    };
                    ComboBoxDropdown.prototype.getActiveRow = function () {
                        return this.dropdownGrid.getActiveRow();
                    };
                    ComboBoxDropdown.prototype.nagivateToFirstRow = function () {
                        this.dropdownGrid.nagivateToFirstRow();
                    };
                    ComboBoxDropdown.prototype.navigateToFirstRowIfNotActive = function () {
                        this.dropdownGrid.navigateToFirstRowIfNotActive();
                    };
                    ComboBoxDropdown.prototype.navigateToNextRow = function () {
                        this.dropdownGrid.navigateToNextRow();
                    };
                    ComboBoxDropdown.prototype.navigateToPreviousRow = function () {
                        this.dropdownGrid.navigateToPreviousRow();
                    };
                    ComboBoxDropdown.prototype.toggleRowSelection = function (row, isMaximumReached) {
                        if (isMaximumReached === void 0) { isMaximumReached = false; }
                        this.dropdownGrid.toggleRowSelection(row, isMaximumReached);
                    };
                    ComboBoxDropdown.prototype.resetActiveSelection = function () {
                        this.dropdownGrid.resetActiveSelection();
                    };
                    ComboBoxDropdown.prototype.applyMultipleSelection = function () {
                        this.dropdownGrid.applyMultipleSelection();
                    };
                    ComboBoxDropdown.prototype.markSelections = function (selectedOptions, ignoreEmpty) {
                        if (ignoreEmpty === void 0) { ignoreEmpty = false; }
                        this.dropdownGrid.markSelections(selectedOptions, ignoreEmpty);
                    };
                    ComboBoxDropdown.prototype.onRowSelection = function (listener) {
                        this.dropdownGrid.onRowSelection(listener);
                    };
                    ComboBoxDropdown.prototype.unRowSelection = function (listener) {
                        this.dropdownGrid.unRowSelection(listener);
                    };
                    ComboBoxDropdown.prototype.onMultipleSelection = function (listener) {
                        this.dropdownGrid.onMultipleSelection(listener);
                    };
                    ComboBoxDropdown.prototype.unMultipleSelection = function (listener) {
                        this.dropdownGrid.unMultipleSelection(listener);
                    };
                    return ComboBoxDropdown;
                })();
                combobox.ComboBoxDropdown = ComboBoxDropdown;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (_combobox) {
                var OptionSelectedEvent = api.ui.selector.OptionSelectedEvent;
                var OptionFilterInputValueChangedEvent = api.ui.selector.OptionFilterInputValueChangedEvent;
                var DropdownHandle = api.ui.selector.DropdownHandle;
                var DelayedFunctionCall = api.util.DelayedFunctionCall;
                var Button = api.ui.button.Button;
                var ComboBox = (function (_super) {
                    __extends(ComboBox, _super);
                    function ComboBox(name, config) {
                        var _this = this;
                        _super.call(this, "div", "combobox");
                        this.setNextInputFocusWhenMaxReached = true;
                        this.ignoreNextFocus = false;
                        this.minWidth = -1;
                        this.optionSelectedListeners = [];
                        this.optionFilterInputValueChangedListeners = [];
                        this.active = false;
                        this.getEl().setAttribute("name", name);
                        this.hideComboBoxWhenMaxReached = config.hideComboBoxWhenMaxReached;
                        if (config.setNextInputFocusWhenMaxReached !== undefined) {
                            this.setNextInputFocusWhenMaxReached = config.setNextInputFocusWhenMaxReached;
                        }
                        if (config.selectedOptionsView != null) {
                            this.selectedOptionsView = config.selectedOptionsView;
                            this.selectedOptionsView.setMaximumOccurrences(config.maximumOccurrences != null ? config.maximumOccurrences : 0);
                        }
                        if (config.iconUrl) {
                            this.icon = new api.dom.ImgEl(config.iconUrl, "input-icon");
                            this.appendChild(this.icon);
                        }
                        if (config.minWidth) {
                            this.minWidth = config.minWidth;
                        }
                        this.input = new _combobox.ComboBoxOptionFilterInput();
                        this.appendChild(this.input);
                        this.delayedInputValueChangedHandling = config.delayedInputValueChangedHandling || 0;
                        this.delayedHandleInputValueChangedFnCall = new DelayedFunctionCall(this.handleInputValueChanged, this, this.delayedInputValueChangedHandling);
                        this.dropdownHandle = new DropdownHandle();
                        this.appendChild(this.dropdownHandle);
                        if (this.selectedOptionsView && (config.maximumOccurrences != 1)) {
                            this.applySelectionsButton = new Button("Apply");
                            this.applySelectionsButton.addClass('small apply-button');
                            this.applySelectionsButton.hide();
                            this.appendChild(this.applySelectionsButton);
                        }
                        this.comboBoxDropdown = new _combobox.ComboBoxDropdown({
                            maxHeight: 200,
                            width: this.input.getWidth(),
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: config.filter,
                            dataIdProperty: config.dataIdProperty,
                            multipleSelections: (this.selectedOptionsView && (config.maximumOccurrences != 1))
                        });
                        this.appendChild(this.comboBoxDropdown.getEmptyDropdown());
                        this.appendChild(this.comboBoxDropdown.getGrid().getElement());
                        this.setupListeners();
                        this.onRendered(function (event) {
                            _this.doUpdateDropdownTopPositionAndWidth();
                        });
                    }
                    ComboBox.prototype.doUpdateDropdownTopPositionAndWidth = function () {
                        var inputEl = this.input.getEl();
                        this.comboBoxDropdown.setTopPx(inputEl.getHeightWithBorder() - inputEl.getBorderBottomWidth());
                        this.comboBoxDropdown.setWidth(Math.max(inputEl.getWidthWithBorder(), this.minWidth));
                    };
                    ComboBox.prototype.giveFocus = function () {
                        return this.input.giveFocus();
                    };
                    ComboBox.prototype.giveInputFocus = function () {
                        this.input.setReadOnly(false);
                        this.input.giveFocus();
                    };
                    ComboBox.prototype.isDropdownShown = function () {
                        return this.comboBoxDropdown.isDropdownShown();
                    };
                    ComboBox.prototype.showDropdown = function () {
                        this.doUpdateDropdownTopPositionAndWidth();
                        this.comboBoxDropdown.showDropdown(this.getSelectedOptions());
                        this.dropdownHandle.down();
                        this.comboBoxDropdown.renderDropdownGrid();
                        this.input.setReadOnly(true);
                        this.setOnBlurListener();
                    };
                    ComboBox.prototype.setEmptyDropdownText = function (label) {
                        this.comboBoxDropdown.setEmptyDropdownText(label);
                    };
                    ComboBox.prototype.hideDropdown = function () {
                        this.dropdownHandle.up();
                        this.comboBoxDropdown.hideDropdown();
                        if (this.applySelectionsButton) {
                            this.applySelectionsButton.hide();
                        }
                        this.input.setReadOnly(false);
                    };
                    ComboBox.prototype.setOptions = function (options) {
                        this.comboBoxDropdown.setOptions(options, this.getSelectedOptions());
                    };
                    ComboBox.prototype.addOption = function (option) {
                        this.comboBoxDropdown.addOption(option);
                    };
                    ComboBox.prototype.setIgnoreNextFocus = function (value) {
                        this.ignoreNextFocus = value;
                        return this;
                    };
                    ComboBox.prototype.isIgnoreNextFocus = function () {
                        return this.ignoreNextFocus;
                    };
                    ComboBox.prototype.loadOptionsAfterShowDropdown = function () {
                        return api.util.PromiseHelper.newResolvedVoidPromise();
                    };
                    ComboBox.prototype.hasOptions = function () {
                        return this.comboBoxDropdown.hasOptions();
                    };
                    ComboBox.prototype.getOptionCount = function () {
                        return this.comboBoxDropdown.getOptionCount();
                    };
                    ComboBox.prototype.getOptions = function () {
                        return this.comboBoxDropdown.getOptions();
                    };
                    ComboBox.prototype.getOptionByValue = function (value) {
                        return this.comboBoxDropdown.getOptionByValue(value);
                    };
                    ComboBox.prototype.getOptionByRow = function (rowIndex) {
                        return this.comboBoxDropdown.getOptionByRow(rowIndex);
                    };
                    ComboBox.prototype.setFilterArgs = function (args) {
                        this.comboBoxDropdown.setFilterArgs(args);
                    };
                    ComboBox.prototype.setValue = function (value) {
                        var option = this.getOptionByValue(value);
                        if (option != null) {
                            this.selectOption(option);
                        }
                        return this;
                    };
                    ComboBox.prototype.setValues = function (values) {
                        var _this = this;
                        values.forEach(function (value) {
                            _this.setValue(value);
                        });
                    };
                    ComboBox.prototype.handleRowSelected = function (index) {
                        var option = this.getOptionByRow(index);
                        if (option != null) {
                            if (!this.isOptionSelected(option)) {
                                this.selectOption(option);
                            }
                            else {
                                this.deselectOption(option);
                            }
                        }
                    };
                    ComboBox.prototype.isSelectionChanged = function () {
                        var _this = this;
                        var optionsMap = this.getDisplayedOptions().map(function (x) {
                            return x.value;
                        }).join();
                        var selectedOptions = this.getSelectedOptions();
                        var filteredOption = [], gridOptions = [];
                        for (var k in selectedOptions) {
                            if (optionsMap.search(selectedOptions[k].value) >= 0) {
                                filteredOption.push(selectedOptions[k].value);
                            }
                        }
                        this.comboBoxDropdown.getGrid().getElement().getSelectedRows().forEach(function (row) {
                            gridOptions.push(_this.comboBoxDropdown.getGrid().getOptionByRow(row).value);
                        });
                        return (filteredOption.length !== gridOptions.length) || (filteredOption.sort().join() !== gridOptions.sort().join());
                    };
                    ComboBox.prototype.selectRowOrApplySelection = function (index) {
                        if (this.applySelectionsButton && this.applySelectionsButton.isVisible()) {
                            this.clearSelection(true);
                            this.comboBoxDropdown.applyMultipleSelection();
                            this.hideDropdown();
                        }
                        else {
                            this.handleRowSelected(index);
                            this.input.setValue("");
                        }
                    };
                    ComboBox.prototype.selectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        api.util.assertNotNull(option, "option cannot be null");
                        if (this.isOptionSelected(option)) {
                            return;
                        }
                        var added = this.selectedOptionsView.addOption(option);
                        if (!added) {
                            return;
                        }
                        this.comboBoxDropdown.markSelections(this.getSelectedOptions());
                        this.hideDropdown();
                        this.addClass("followed-by-options");
                        if (this.maximumOccurrencesReached()) {
                            this.input.setMaximumReached();
                            if (this.setNextInputFocusWhenMaxReached && !this.ignoreNextFocus) {
                                api.dom.FormEl.moveFocusToNextFocusable(this.input);
                            }
                            this.dropdownHandle.setEnabled(false);
                        }
                        if (!silent) {
                            this.notifyOptionSelected(option);
                        }
                        if (this.maximumOccurrencesReached() && this.hideComboBoxWhenMaxReached) {
                            this.hide();
                        }
                        this.ignoreNextFocus = false;
                    };
                    ComboBox.prototype.isOptionSelected = function (option) {
                        return this.selectedOptionsView.isSelected(option);
                    };
                    ComboBox.prototype.deselectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        api.util.assertNotNull(option, "option cannot be null");
                        if (!this.isOptionSelected(option)) {
                            return;
                        }
                        this.selectedOptionsView.removeOption(option, silent);
                        this.comboBoxDropdown.markSelections(this.getSelectedOptions());
                        this.hideDropdown();
                        this.input.openForTypingAndFocus();
                        this.dropdownHandle.setEnabled(true);
                        if (!this.maximumOccurrencesReached() && this.hideComboBoxWhenMaxReached) {
                            this.show();
                        }
                    };
                    ComboBox.prototype.clearSelection = function (ignoreEmpty, giveInputFocus, forceClear) {
                        var _this = this;
                        if (ignoreEmpty === void 0) { ignoreEmpty = false; }
                        if (giveInputFocus === void 0) { giveInputFocus = true; }
                        if (forceClear === void 0) { forceClear = false; }
                        var optionsMap = this.getDisplayedOptions().map(function (x) { return x.value; }).join();
                        var selectedOptions = this.getSelectedOptions();
                        selectedOptions.forEach(function (option) {
                            if (forceClear) {
                                _this.selectedOptionsView.removeOption(option, true);
                            }
                            else {
                                var filteredOption = optionsMap.search(option.value) >= 0 ? option : undefined;
                                if (filteredOption) {
                                    _this.selectedOptionsView.removeOption(option, true);
                                }
                            }
                        });
                        this.comboBoxDropdown.markSelections([], ignoreEmpty);
                        this.input.setValue("");
                        if (giveInputFocus) {
                            this.input.openForTypingAndFocus();
                        }
                        else {
                            this.input.openForTyping();
                        }
                        this.dropdownHandle.setEnabled(true);
                        if (this.hideComboBoxWhenMaxReached) {
                            this.show();
                        }
                    };
                    ComboBox.prototype.getSelectedOptions = function () {
                        if (this.selectedOptionsView) {
                            return this.selectedOptionsView.getSelectedOptions().map(function (selectedOption) {
                                return selectedOption.getOption();
                            });
                        }
                        else {
                            throw new Error("Not supported yet");
                        }
                    };
                    ComboBox.prototype.getDisplayedOptions = function () {
                        var displayedOptions = [];
                        for (var row = 0; row < this.comboBoxDropdown.getOptionCount(); row++) {
                            var option = this.getOptionByRow(row);
                            if (option) {
                                displayedOptions.push(option);
                            }
                        }
                        return displayedOptions;
                    };
                    ComboBox.prototype.getValue = function () {
                        if (this.selectedOptionsView) {
                            return this.getSelectedOptions().map(function (item) { return item.value; }).join(';');
                        }
                        else {
                            throw new Error("Not supported yet");
                        }
                    };
                    ComboBox.prototype.countSelectedOptions = function () {
                        if (this.selectedOptionsView) {
                            return this.selectedOptionsView.count();
                        }
                        else {
                            throw new Error("Not supported yet");
                        }
                    };
                    ComboBox.prototype.maximumOccurrencesReached = function () {
                        api.util.assert(this.selectedOptionsView != null, "No point of calling maximumOccurrencesReached when no multiple selections are enabled");
                        return this.selectedOptionsView.maximumOccurrencesReached();
                    };
                    ComboBox.prototype.maximumSelectionsReached = function () {
                        if (this.selectedOptionsView && this.selectedOptionsView.getMaximumOccurrences() !== 0) {
                            var totalSelected = this.comboBoxDropdown.getSelectedOptionCount();
                            var optionsMap = this.getDisplayedOptions().map(function (x) { return x.value; }).join();
                            totalSelected += this.getSelectedOptions().filter(function (option) { return (optionsMap.search(option.value) < 0); }).length;
                            return this.selectedOptionsView.getMaximumOccurrences() <= totalSelected;
                        }
                        else {
                            return false;
                        }
                    };
                    ComboBox.prototype.setInputIconUrl = function (iconUrl) {
                        if (!this.icon) {
                            this.icon = new api.dom.ImgEl();
                            this.icon.addClass("input-icon");
                            this.icon.insertBeforeEl(this.input);
                        }
                        this.icon.getEl().setSrc(iconUrl);
                    };
                    ComboBox.prototype.setupListeners = function () {
                        var _this = this;
                        this.onClicked(function (event) {
                            _this.setOnBlurListener();
                        });
                        this.input.onClicked(function (event) {
                            _this.input.setReadOnly(false);
                        });
                        this.comboBoxDropdown.onRowSelection(function (event) {
                            _this.handleRowSelected(event.getRow());
                        });
                        this.dropdownHandle.onClicked(function (event) {
                            event.preventDefault();
                            event.stopPropagation();
                            if (!_this.maximumOccurrencesReached()) {
                                if (_this.isDropdownShown()) {
                                    _this.hideDropdown();
                                    _this.giveInputFocus();
                                }
                                else {
                                    _this.showDropdown();
                                    _this.loadOptionsAfterShowDropdown().then(function () {
                                        _this.giveInputFocus();
                                    }).catch(function (reason) {
                                        api.DefaultErrorHandler.handle(reason);
                                    }).done();
                                }
                            }
                        });
                        if (this.applySelectionsButton) {
                            this.applySelectionsButton.onClicked(function (event) {
                                _this.clearSelection(true);
                                _this.comboBoxDropdown.applyMultipleSelection();
                                _this.hideDropdown();
                            });
                            this.comboBoxDropdown.onMultipleSelection(this.handleMultipleSelectionChanged.bind(this));
                        }
                        this.input.onValueChanged(function (event) {
                            _this.preservedInputValueChangedEvent = event;
                            if (_this.delayedInputValueChangedHandling == 0) {
                                _this.handleInputValueChanged();
                            }
                            else {
                                _this.setEmptyDropdownText("Just keep on typing...");
                                _this.delayedHandleInputValueChangedFnCall.delayCall();
                            }
                        });
                        this.input.onDblClicked(function (event) {
                            event.preventDefault();
                            event.stopPropagation();
                            if (!_this.isDropdownShown()) {
                                _this.showDropdown();
                                _this.loadOptionsAfterShowDropdown().then(function () {
                                    _this.comboBoxDropdown.navigateToFirstRowIfNotActive();
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                                _this.input.setReadOnly(false);
                            }
                        });
                        this.onKeyDown(this.handleKeyDown.bind(this));
                        if (this.selectedOptionsView) {
                            this.selectedOptionsView.onOptionDeselected(function (removedOption) {
                                _this.handleSelectedOptionRemoved(removedOption);
                            });
                        }
                    };
                    ComboBox.prototype.handleInputValueChanged = function () {
                        if (this.preservedInputValueChangedEvent) {
                            this.notifyOptionFilterInputValueChanged(this.preservedInputValueChangedEvent.getOldValue(), this.preservedInputValueChangedEvent.getNewValue());
                            this.comboBoxDropdown.resetActiveSelection();
                            this.input.setReadOnly(false);
                        }
                    };
                    ComboBox.prototype.handleKeyDown = function (event) {
                        var _this = this;
                        if (event.which == 9) {
                            this.hideDropdown();
                            return;
                        }
                        else if (event.which == 16 || event.which == 17 || event.which == 18 || event.which == 91) {
                            return;
                        }
                        if (!this.isDropdownShown()) {
                            if (event.which === 40) {
                                this.showDropdown();
                                this.loadOptionsAfterShowDropdown().then(function () {
                                    _this.comboBoxDropdown.navigateToFirstRowIfNotActive();
                                    _this.input.setReadOnly(true);
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                            }
                            else {
                                this.showDropdown();
                                this.input.setReadOnly(false);
                            }
                            return;
                        }
                        switch (event.which) {
                            case 38:
                                if (this.comboBoxDropdown.hasActiveRow()) {
                                    if (this.comboBoxDropdown.getActiveRow() === 0) {
                                        this.comboBoxDropdown.resetActiveSelection();
                                        this.input.setReadOnly(false);
                                        this.input.giveFocus();
                                    }
                                    else {
                                        this.comboBoxDropdown.navigateToPreviousRow();
                                        this.input.setReadOnly(true);
                                    }
                                }
                                event.stopPropagation();
                                event.preventDefault();
                                break;
                            case 40:
                                if (this.comboBoxDropdown.hasActiveRow()) {
                                    this.comboBoxDropdown.navigateToNextRow();
                                }
                                else {
                                    this.comboBoxDropdown.nagivateToFirstRow();
                                }
                                this.input.setReadOnly(true);
                                event.stopPropagation();
                                event.preventDefault();
                                break;
                            case 13:
                                this.selectRowOrApplySelection(this.comboBoxDropdown.getActiveRow());
                                event.stopPropagation();
                                event.preventDefault();
                                break;
                            case 32:
                                if (this.input.isReadOnly() && this.applySelectionsButton) {
                                    this.comboBoxDropdown.toggleRowSelection(this.comboBoxDropdown.getActiveRow(), this.maximumSelectionsReached());
                                    event.stopPropagation();
                                    event.preventDefault();
                                }
                                break;
                            case 8:
                                if (this.input.isReadOnly()) {
                                    event.stopPropagation();
                                    event.preventDefault();
                                }
                                break;
                            case 27:
                                this.hideDropdown();
                                break;
                        }
                        this.input.giveFocus();
                    };
                    ComboBox.prototype.handleSelectedOptionRemoved = function (removedSelectedOption) {
                        this.comboBoxDropdown.markSelections(this.getSelectedOptions());
                        this.dropdownHandle.setEnabled(true);
                        if (this.hideComboBoxWhenMaxReached) {
                            this.show();
                        }
                        if (this.countSelectedOptions() == 0) {
                            this.removeClass("followed-by-options");
                        }
                        this.input.openForTypingAndFocus();
                    };
                    ComboBox.prototype.handleMultipleSelectionChanged = function (event) {
                        this.input.giveFocus();
                        if (this.isSelectionChanged()) {
                            this.applySelectionsButton.show();
                        }
                        else {
                            this.applySelectionsButton.hide();
                        }
                    };
                    ComboBox.prototype.setOnBlurListener = function () {
                        var combobox = this;
                        var hideDropdownOnBlur = function (event) {
                            var comboboxHtmlElement = combobox.getHTMLElement();
                            for (var element = event.target; element; element = element.parentNode) {
                                if (element == comboboxHtmlElement) {
                                    return;
                                }
                            }
                            combobox.hideDropdown();
                            combobox.active = false;
                            api.dom.Body.get().getEl().removeEventListener('click', hideDropdownOnBlur);
                        };
                        if (!this.active) {
                            this.active = true;
                            api.dom.Body.get().onClicked(hideDropdownOnBlur);
                        }
                    };
                    ComboBox.prototype.onOptionSelected = function (listener) {
                        this.optionSelectedListeners.push(listener);
                    };
                    ComboBox.prototype.unOptionSelected = function (listener) {
                        this.optionSelectedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    ComboBox.prototype.notifyOptionSelected = function (item) {
                        var event = new OptionSelectedEvent(item);
                        this.optionSelectedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    ComboBox.prototype.onOptionFilterInputValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.push(listener);
                    };
                    ComboBox.prototype.unOptionFilterInputValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    ComboBox.prototype.notifyOptionFilterInputValueChanged = function (oldValue, newValue) {
                        var event = new OptionFilterInputValueChangedEvent(oldValue, newValue, this.comboBoxDropdown.getGrid().getElement());
                        this.optionFilterInputValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    ComboBox.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionsView.onOptionDeselected(listener);
                    };
                    ComboBox.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionsView.unOptionDeselected(listener);
                    };
                    ComboBox.prototype.onFocus = function (listener) {
                        this.input.onFocus(listener);
                    };
                    ComboBox.prototype.unFocus = function (listener) {
                        this.input.unFocus(listener);
                    };
                    ComboBox.prototype.onBlur = function (listener) {
                        this.input.onBlur(listener);
                    };
                    ComboBox.prototype.unBlur = function (listener) {
                        this.input.unBlur(listener);
                    };
                    return ComboBox;
                })(api.dom.FormInputEl);
                _combobox.ComboBox = ComboBox;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var BaseSelectedOptionView = (function (_super) {
                    __extends(BaseSelectedOptionView, _super);
                    function BaseSelectedOptionView(option) {
                        _super.call(this, "selected-option");
                        this.selectedOptionToBeRemovedListeners = [];
                        this.layout();
                        this.setOption(option);
                    }
                    BaseSelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        if (this.optionValueEl) {
                            this.optionValueEl.getEl().setInnerHtml(this.option.displayValue.toString());
                        }
                    };
                    BaseSelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    BaseSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var removeButtonEl = new api.dom.AEl("remove");
                        this.optionValueEl = new api.dom.DivEl('option-value');
                        removeButtonEl.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButtonEl);
                        this.appendChild(this.optionValueEl);
                    };
                    BaseSelectedOptionView.prototype.notifySelectedOptionRemoveRequested = function () {
                        this.selectedOptionToBeRemovedListeners.forEach(function (listener) {
                            listener();
                        });
                    };
                    BaseSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                        this.selectedOptionToBeRemovedListeners.push(listener);
                    };
                    BaseSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                        this.selectedOptionToBeRemovedListeners = this.selectedOptionToBeRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return BaseSelectedOptionView;
                })(api.dom.DivEl);
                combobox.BaseSelectedOptionView = BaseSelectedOptionView;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var BaseSelectedOptionsView = (function (_super) {
                    __extends(BaseSelectedOptionsView, _super);
                    function BaseSelectedOptionsView(className) {
                        _super.call(this, "selected-options" + (className ? " " + className : ""));
                        this.list = [];
                        this.selectedOptionRemovedListeners = [];
                    }
                    BaseSelectedOptionsView.prototype.setMaximumOccurrences = function (value) {
                        this.maximumOccurrences = value;
                    };
                    BaseSelectedOptionsView.prototype.getMaximumOccurrences = function () {
                        return this.maximumOccurrences;
                    };
                    BaseSelectedOptionsView.prototype.createSelectedOption = function (option) {
                        return new combobox.SelectedOption(new combobox.BaseSelectedOptionView(option), this.count());
                    };
                    BaseSelectedOptionsView.prototype.addOption = function (option) {
                        var _this = this;
                        if (this.isSelected(option) || this.maximumOccurrencesReached()) {
                            return false;
                        }
                        var selectedOption = this.createSelectedOption(option);
                        selectedOption.getOptionView().onSelectedOptionRemoveRequest(function () {
                            _this.removeOption(option);
                        });
                        this.list.push(selectedOption);
                        this.appendChild(selectedOption.getOptionView());
                        return true;
                    };
                    BaseSelectedOptionsView.prototype.removeOption = function (optionToRemove, silent) {
                        if (silent === void 0) { silent = false; }
                        api.util.assertNotNull(optionToRemove, "optionToRemove cannot be null");
                        var selectedOption = this.getByOption(optionToRemove);
                        api.util.assertNotNull(selectedOption, "Did not find any selected option to remove from option: " + optionToRemove.value);
                        selectedOption.getOptionView().remove();
                        this.list = this.list.filter(function (option) {
                            return option.getOption().value != selectedOption.getOption().value;
                        });
                        if (selectedOption.getIndex() < this.list.length) {
                            for (var i = selectedOption.getIndex(); i < this.list.length; i++) {
                                this.list[i].setIndex(i);
                            }
                        }
                        if (!silent) {
                            this.notifySelectedOptionRemoved(selectedOption);
                        }
                    };
                    BaseSelectedOptionsView.prototype.count = function () {
                        return this.list.length;
                    };
                    BaseSelectedOptionsView.prototype.getSelectedOptions = function () {
                        return this.list;
                    };
                    BaseSelectedOptionsView.prototype.getByIndex = function (index) {
                        return this.list[index];
                    };
                    BaseSelectedOptionsView.prototype.getByOption = function (option) {
                        return this.getById(option.value);
                    };
                    BaseSelectedOptionsView.prototype.getById = function (id) {
                        return this.list.filter(function (selectedOption) {
                            return selectedOption.getOption().value == id;
                        })[0];
                    };
                    BaseSelectedOptionsView.prototype.isSelected = function (option) {
                        return this.getByOption(option) != null;
                    };
                    BaseSelectedOptionsView.prototype.maximumOccurrencesReached = function () {
                        if (this.maximumOccurrences == 0) {
                            return false;
                        }
                        return this.count() >= this.maximumOccurrences;
                    };
                    BaseSelectedOptionsView.prototype.moveOccurrence = function (formIndex, toIndex) {
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.list);
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.getChildren());
                        this.list.forEach(function (selectedOption, index) { return selectedOption.setIndex(index); });
                    };
                    BaseSelectedOptionsView.prototype.notifySelectedOptionRemoved = function (removed) {
                        this.selectedOptionRemovedListeners.forEach(function (listener) {
                            listener(removed);
                        });
                    };
                    BaseSelectedOptionsView.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners.push(listener);
                    };
                    BaseSelectedOptionsView.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners = this.selectedOptionRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return BaseSelectedOptionsView;
                })(api.dom.DivEl);
                combobox.BaseSelectedOptionsView = BaseSelectedOptionsView;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var SelectedOption = (function () {
                    function SelectedOption(optionView, index) {
                        api.util.assertNotNull(optionView, "optionView cannot be null");
                        this.optionView = optionView;
                        this.index = index;
                    }
                    SelectedOption.prototype.getOption = function () {
                        return this.optionView.getOption();
                    };
                    SelectedOption.prototype.getOptionView = function () {
                        return this.optionView;
                    };
                    SelectedOption.prototype.getIndex = function () {
                        return this.index;
                    };
                    SelectedOption.prototype.setIndex = function (value) {
                        this.index = value;
                    };
                    return SelectedOption;
                })();
                combobox.SelectedOption = SelectedOption;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var RichComboBox = (function (_super) {
                    __extends(RichComboBox, _super);
                    function RichComboBox(builder) {
                        var _this = this;
                        this.loadedListeners = [];
                        this.loadingListeners = [];
                        this.comboBoxName = builder.comboBoxName;
                        this.identifierMethod = builder.identifierMethod;
                        this.comboBoxView = new api.dom.DivEl();
                        this.delayedInputValueChangedHandling = builder.delayedInputValueChangedHandling;
                        this.selectedOptionsView = builder.selectedOptionsView;
                        this.selectedOptionsView.hide();
                        this.maximumOccurrences = builder.maximumOccurrences;
                        this.minWidth = builder.minWidth;
                        this.optionDisplayValueViewer = builder.optionDisplayValueViewer;
                        this.setNextInputFocusWhenMaxReached = builder.nextInputFocusWhenMaxReached;
                        var comboBoxConfig = {
                            maximumOccurrences: this.maximumOccurrences,
                            selectedOptionsView: this.selectedOptionsView,
                            optionDisplayValueViewer: this.optionDisplayValueViewer,
                            hideComboBoxWhenMaxReached: true,
                            setNextInputFocusWhenMaxReached: this.setNextInputFocusWhenMaxReached,
                            delayedInputValueChangedHandling: this.delayedInputValueChangedHandling,
                            minWidth: this.minWidth
                        };
                        this.loader = builder.loader;
                        this.comboBox = new RichComboBoxComboBox(name, comboBoxConfig, this.loader);
                        this.setupLoader();
                        this.comboBox.onClicked(function (event) {
                            _this.comboBox.giveFocus();
                        });
                        _super.call(this, this.comboBox, this.selectedOptionsView);
                        this.addClass('rich-combobox');
                    }
                    RichComboBox.prototype.setIgnoreNextFocus = function (value) {
                        if (value === void 0) { value = true; }
                        this.comboBox.setIgnoreNextFocus(value);
                        return this;
                    };
                    RichComboBox.prototype.isIgnoreNextFocus = function () {
                        return this.comboBox.isIgnoreNextFocus();
                    };
                    RichComboBox.prototype.getSelectedDisplayValues = function () {
                        return this.comboBox.getSelectedOptions().map(function (option) {
                            return option.displayValue;
                        });
                    };
                    RichComboBox.prototype.getSelectedValues = function () {
                        return this.comboBox.getSelectedOptions().map(function (option) {
                            return option.value;
                        });
                    };
                    RichComboBox.prototype.getDisplayValues = function () {
                        return this.comboBox.getOptions().map(function (option) {
                            return option.displayValue;
                        });
                    };
                    RichComboBox.prototype.getValues = function () {
                        return this.comboBox.getOptions().map(function (option) {
                            return option.value;
                        });
                    };
                    RichComboBox.prototype.getSelectedOptions = function () {
                        return this.selectedOptionsView.getSelectedOptions();
                    };
                    RichComboBox.prototype.getSelectedOption = function (option) {
                        return this.selectedOptionsView.getByOption(option);
                    };
                    RichComboBox.prototype.isOptionSelected = function (option) {
                        return this.comboBox.isOptionSelected(option);
                    };
                    RichComboBox.prototype.maximumOccurrencesReached = function () {
                        return this.comboBox.maximumOccurrencesReached();
                    };
                    RichComboBox.prototype.getComboBox = function () {
                        return this.comboBox;
                    };
                    RichComboBox.prototype.addOption = function (option) {
                        this.comboBox.addOption(option);
                    };
                    RichComboBox.prototype.selectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        this.comboBox.selectOption(option, silent);
                    };
                    RichComboBox.prototype.hasOptions = function () {
                        return this.comboBox.hasOptions();
                    };
                    RichComboBox.prototype.getOptionCount = function () {
                        return this.comboBox.getOptionCount();
                    };
                    RichComboBox.prototype.getOptions = function () {
                        return this.comboBox.getOptions();
                    };
                    RichComboBox.prototype.getOptionByValue = function (value) {
                        return this.comboBox.getOptionByValue(value);
                    };
                    RichComboBox.prototype.getOptionByRow = function (rowIndex) {
                        return this.comboBox.getOptionByRow(rowIndex);
                    };
                    RichComboBox.prototype.countSelected = function () {
                        return this.comboBox.countSelectedOptions();
                    };
                    RichComboBox.prototype.select = function (value) {
                        this.comboBox.selectOption(this.createOption(value));
                    };
                    RichComboBox.prototype.selectByOption = function (option) {
                        this.comboBox.selectOption(option);
                    };
                    RichComboBox.prototype.deselect = function (value) {
                        this.comboBox.deselectOption(this.createOption(value));
                    };
                    RichComboBox.prototype.clearSelection = function (forceClear) {
                        if (forceClear === void 0) { forceClear = false; }
                        this.comboBox.clearSelection(false, true, forceClear);
                    };
                    RichComboBox.prototype.isSelected = function (value) {
                        var selectedValues = this.getSelectedValues();
                        var valueToFind = this.getDisplayValueId(value);
                        for (var i = 0; i < selectedValues.length; i++) {
                            if (selectedValues[i] == valueToFind) {
                                return true;
                            }
                        }
                        return false;
                    };
                    RichComboBox.prototype.getDisplayValueId = function (value) {
                        var val = value[this.identifierMethod]();
                        return typeof val == 'object' && val['toString'] ? val.toString() : val;
                    };
                    RichComboBox.prototype.createOption = function (value) {
                        return {
                            value: this.getDisplayValueId(value),
                            displayValue: value
                        };
                    };
                    RichComboBox.prototype.setupLoader = function () {
                        var _this = this;
                        this.comboBox.onOptionFilterInputValueChanged(function (event) {
                            _this.loader.search(event.getNewValue()).then(function (result) {
                                if (!_this.comboBox.isDropdownShown()) {
                                    _this.comboBox.showDropdown();
                                    _this.comboBox.giveInputFocus();
                                }
                                return result;
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).done();
                        });
                        this.comboBox.onOptionSelected(function (event) {
                            _this.selectedOptionsView.show();
                        });
                        this.loader.onLoadingData(function (event) {
                            _this.comboBox.setEmptyDropdownText("Searching...");
                            _this.notifyLoading();
                        });
                        this.loader.onLoadedData(function (event) {
                            var options = _this.createOptions(event.getData());
                            _this.comboBox.setOptions(options);
                            _this.notifyLoaded(event.getData());
                        });
                    };
                    RichComboBox.prototype.createOptions = function (items) {
                        var _this = this;
                        var options = [];
                        items.forEach(function (itemInst) {
                            options.push(_this.createOption(itemInst));
                        });
                        return options;
                    };
                    RichComboBox.prototype.getLoader = function () {
                        return this.loader;
                    };
                    RichComboBox.prototype.setInputIconUrl = function (url) {
                        this.comboBox.setInputIconUrl(url);
                    };
                    RichComboBox.prototype.onOptionDeselected = function (listener) {
                        this.comboBox.onOptionDeselected(listener);
                    };
                    RichComboBox.prototype.unOptionDeselected = function (listener) {
                        this.comboBox.unOptionDeselected(listener);
                    };
                    RichComboBox.prototype.onOptionSelected = function (listener) {
                        this.comboBox.onOptionSelected(listener);
                    };
                    RichComboBox.prototype.unOptionSelected = function (listener) {
                        this.comboBox.unOptionSelected(listener);
                    };
                    RichComboBox.prototype.notifyLoading = function () {
                        this.loadingListeners.forEach(function (listener) {
                            listener();
                        });
                    };
                    RichComboBox.prototype.onLoading = function (listener) {
                        this.loadingListeners.push(listener);
                    };
                    RichComboBox.prototype.unLoading = function (listener) {
                        var index = this.loadedListeners.indexOf(listener);
                        this.loadedListeners.splice(index, 1);
                    };
                    RichComboBox.prototype.onLoaded = function (listener) {
                        this.loadedListeners.push(listener);
                    };
                    RichComboBox.prototype.unLoaded = function (listenerToBeRemoved) {
                        var index = this.loadedListeners.indexOf(listenerToBeRemoved);
                        this.loadedListeners.splice(index, 1);
                    };
                    RichComboBox.prototype.notifyLoaded = function (items) {
                        this.loadedListeners.forEach(function (listener) {
                            listener(items);
                        });
                    };
                    RichComboBox.prototype.giveFocus = function () {
                        return this.comboBox.giveFocus();
                    };
                    RichComboBox.prototype.onFocus = function (listener) {
                        this.comboBox.onFocus(listener);
                    };
                    RichComboBox.prototype.unFocus = function (listener) {
                        this.comboBox.unFocus(listener);
                    };
                    RichComboBox.prototype.onBlur = function (listener) {
                        this.comboBox.onBlur(listener);
                    };
                    RichComboBox.prototype.unBlur = function (listener) {
                        this.comboBox.unBlur(listener);
                    };
                    RichComboBox.prototype.setValue = function (value) {
                        var _this = this;
                        if (this.loader.isLoaded()) {
                            _super.prototype.setValue.call(this, value);
                        }
                        else {
                            var singleLoadListener = function (event) {
                                _super.prototype.setValue.call(_this, value);
                                _this.loader.unLoadedData(singleLoadListener);
                            };
                            this.loader.onLoadedData(singleLoadListener);
                            if (this.loader.isNotStarted()) {
                                this.loader.load();
                            }
                        }
                        return this;
                    };
                    return RichComboBox;
                })(api.dom.CompositeFormInputEl);
                combobox.RichComboBox = RichComboBox;
                var RichComboBoxComboBox = (function (_super) {
                    __extends(RichComboBoxComboBox, _super);
                    function RichComboBoxComboBox(name, config, loader) {
                        _super.call(this, name, config);
                        this.loader = loader;
                    }
                    RichComboBoxComboBox.prototype.loadOptionsAfterShowDropdown = function () {
                        var deferred = wemQ.defer();
                        this.loader.load().then(function () {
                            deferred.resolve(null);
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                        return deferred.promise;
                    };
                    return RichComboBoxComboBox;
                })(combobox.ComboBox);
                var RichComboBoxBuilder = (function () {
                    function RichComboBoxBuilder() {
                        this.identifierMethod = "getId";
                        this.maximumOccurrences = 0;
                        this.nextInputFocusWhenMaxReached = true;
                    }
                    RichComboBoxBuilder.prototype.setComboBoxName = function (comboBoxName) {
                        this.comboBoxName = comboBoxName;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setIdentifierMethod = function (identifierMethod) {
                        this.identifierMethod = identifierMethod;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setLoader = function (loader) {
                        this.loader = loader;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setSelectedOptionsView = function (selectedOptionsView) {
                        this.selectedOptionsView = selectedOptionsView;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setMaximumOccurrences = function (maximumOccurrences) {
                        this.maximumOccurrences = maximumOccurrences;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setOptionDisplayValueViewer = function (value) {
                        this.optionDisplayValueViewer = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setDelayedInputValueChangedHandling = function (value) {
                        this.delayedInputValueChangedHandling = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setNextInputFocusWhenMaxReached = function (value) {
                        this.nextInputFocusWhenMaxReached = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setMinWidth = function (value) {
                        this.minWidth = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.build = function () {
                        return new RichComboBox(this);
                    };
                    return RichComboBoxBuilder;
                })();
                combobox.RichComboBoxBuilder = RichComboBoxBuilder;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var RichSelectedOptionView = (function (_super) {
                    __extends(RichSelectedOptionView, _super);
                    function RichSelectedOptionView(option, size) {
                        if (size === void 0) { size = 2 /* small */; }
                        this.optionDisplayValue = option.displayValue;
                        this.size = size;
                        _super.call(this, option);
                    }
                    RichSelectedOptionView.prototype.resolveIconUrl = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.resolveTitle = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.resolveSubTitle = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.resolveIconClass = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(this.size).build();
                        namesAndIconView.setMainName(this.resolveTitle(this.optionDisplayValue)).setSubName(this.resolveSubTitle(this.optionDisplayValue));
                        var url = this.resolveIconUrl(this.optionDisplayValue);
                        if (!api.util.StringHelper.isBlank(url)) {
                            namesAndIconView.setIconUrl(this.resolveIconUrl(this.optionDisplayValue) + '?crop=false');
                        }
                        else {
                            namesAndIconView.setIconClass(this.resolveIconClass(this.optionDisplayValue));
                        }
                        var removeButton = new api.dom.AEl("remove");
                        removeButton.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButton);
                        this.appendChild(namesAndIconView);
                    };
                    return RichSelectedOptionView;
                })(api.ui.selector.combobox.BaseSelectedOptionView);
                combobox.RichSelectedOptionView = RichSelectedOptionView;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var DropdownGrid = api.ui.selector.DropdownGrid;
                var DropdownDropdown = (function () {
                    function DropdownDropdown(config) {
                        this.dropdownGrid = new DropdownGrid({
                            maxHeight: config.maxHeight,
                            width: config.width,
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: config.filter,
                            dataIdProperty: config.dataIdProperty
                        });
                    }
                    DropdownDropdown.prototype.getGrid = function () {
                        return this.dropdownGrid;
                    };
                    DropdownDropdown.prototype.renderDropdownGrid = function () {
                        this.dropdownGrid.renderGrid();
                    };
                    DropdownDropdown.prototype.isDropdownShown = function () {
                        return this.dropdownGrid.isVisible();
                    };
                    DropdownDropdown.prototype.setOptions = function (options) {
                        this.dropdownGrid.setOptions(options);
                        if (this.dropdownGrid.isVisible()) {
                            this.showDropdown(null);
                        }
                    };
                    DropdownDropdown.prototype.removeAllOptions = function () {
                        this.dropdownGrid.removeAllOptions();
                    };
                    DropdownDropdown.prototype.addOption = function (option) {
                        this.dropdownGrid.addOption(option);
                    };
                    DropdownDropdown.prototype.hasOptions = function () {
                        return this.dropdownGrid.hasOptions();
                    };
                    DropdownDropdown.prototype.getOptionCount = function () {
                        return this.dropdownGrid.getOptionCount();
                    };
                    DropdownDropdown.prototype.getOptions = function () {
                        return this.dropdownGrid.getOptions();
                    };
                    DropdownDropdown.prototype.getOptionByValue = function (value) {
                        return this.dropdownGrid.getOptionByValue(value);
                    };
                    DropdownDropdown.prototype.getOptionByRow = function (rowIndex) {
                        return this.dropdownGrid.getOptionByRow(rowIndex);
                    };
                    DropdownDropdown.prototype.setFilterArgs = function (args) {
                        this.dropdownGrid.setFilterArgs(args);
                    };
                    DropdownDropdown.prototype.showDropdown = function (selectedOption) {
                        if (this.hasOptions()) {
                            this.dropdownGrid.show();
                            this.dropdownGrid.adjustGridHeight();
                            if (selectedOption) {
                                this.dropdownGrid.markSelections([selectedOption]);
                            }
                        }
                        else {
                            this.dropdownGrid.hide();
                        }
                    };
                    DropdownDropdown.prototype.hideDropdown = function () {
                        this.dropdownGrid.hide();
                    };
                    DropdownDropdown.prototype.setTopPx = function (value) {
                        this.dropdownGrid.setTopPx(value);
                    };
                    DropdownDropdown.prototype.setWidth = function (value) {
                        this.dropdownGrid.setWidthPx(value);
                    };
                    DropdownDropdown.prototype.hasActiveRow = function () {
                        return this.dropdownGrid.hasActiveRow();
                    };
                    DropdownDropdown.prototype.getActiveRow = function () {
                        return this.dropdownGrid.getActiveRow();
                    };
                    DropdownDropdown.prototype.nagivateToFirstRow = function () {
                        this.dropdownGrid.nagivateToFirstRow();
                    };
                    DropdownDropdown.prototype.navigateToFirstRowIfNotActive = function () {
                        this.dropdownGrid.navigateToFirstRowIfNotActive();
                    };
                    DropdownDropdown.prototype.navigateToNextRow = function () {
                        this.dropdownGrid.navigateToNextRow();
                    };
                    DropdownDropdown.prototype.navigateToPreviousRow = function () {
                        this.dropdownGrid.navigateToPreviousRow();
                    };
                    DropdownDropdown.prototype.markSelections = function (selectedOptions) {
                        this.dropdownGrid.markSelections(selectedOptions);
                    };
                    DropdownDropdown.prototype.onRowSelection = function (listener) {
                        this.dropdownGrid.onRowSelection(listener);
                    };
                    DropdownDropdown.prototype.unRowSelection = function (listener) {
                        this.dropdownGrid.unRowSelection(listener);
                    };
                    return DropdownDropdown;
                })();
                dropdown.DropdownDropdown = DropdownDropdown;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var DropdownOptionFilterInput = (function (_super) {
                    __extends(DropdownOptionFilterInput, _super);
                    function DropdownOptionFilterInput() {
                        _super.call(this);
                    }
                    return DropdownOptionFilterInput;
                })(api.ui.selector.OptionFilterInput);
                dropdown.DropdownOptionFilterInput = DropdownOptionFilterInput;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var DropdownHandle = api.ui.selector.DropdownHandle;
                var SelectedOptionView = (function (_super) {
                    __extends(SelectedOptionView, _super);
                    function SelectedOptionView(objectViewer) {
                        var _this = this;
                        _super.call(this, "selected-option");
                        this.openDropdownListeners = [];
                        this.objectViewer = objectViewer;
                        this.dropdownHandle = new DropdownHandle();
                        this.optionValueEl = new api.dom.DivEl('option-value');
                        this.appendChild(this.optionValueEl);
                        this.optionValueEl.appendChild(this.objectViewer);
                        this.appendChild(this.dropdownHandle);
                        this.dropdownHandle.onClicked(function () {
                            _this.notifyOpenDropdown();
                        });
                        this.onClicked(function (event) {
                            if (document.selection && document.selection.empty) {
                                document.selection.empty();
                            }
                            else if (window.getSelection) {
                                var sel = window.getSelection();
                                sel.removeAllRanges();
                            }
                            _this.notifyOpenDropdown();
                        });
                        this.onKeyPressed(function (event) {
                            if (event.which == 32 || event.which == 13) {
                                _this.notifyOpenDropdown();
                            }
                        });
                    }
                    SelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        this.objectViewer.setObject(option.displayValue);
                    };
                    SelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    SelectedOptionView.prototype.notifyOpenDropdown = function () {
                        this.openDropdownListeners.forEach(function (listener) {
                            listener();
                        });
                    };
                    SelectedOptionView.prototype.onOpenDropdown = function (listener) {
                        this.openDropdownListeners.push(listener);
                    };
                    SelectedOptionView.prototype.unOpenDropdown = function (listener) {
                        this.openDropdownListeners = this.openDropdownListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return SelectedOptionView;
                })(api.dom.ButtonEl);
                dropdown.SelectedOptionView = SelectedOptionView;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var OptionSelectedEvent = api.ui.selector.OptionSelectedEvent;
                var OptionFilterInputValueChangedEvent = api.ui.selector.OptionFilterInputValueChangedEvent;
                var DropdownHandle = api.ui.selector.DropdownHandle;
                var DefaultOptionDisplayValueViewer = api.ui.selector.DefaultOptionDisplayValueViewer;
                var Dropdown = (function (_super) {
                    __extends(Dropdown, _super);
                    function Dropdown(name, config) {
                        var _this = this;
                        _super.call(this, "div", "dropdown");
                        this.optionSelectedListeners = [];
                        this.optionFilterInputValueChangedListeners = [];
                        this.active = false;
                        this.getEl().setAttribute("name", name);
                        this.optionDisplayValueViewer = config.optionDisplayValueViewer || new DefaultOptionDisplayValueViewer();
                        if (config.iconUrl) {
                            this.icon = new api.dom.ImgEl(config.iconUrl, "input-icon");
                            this.appendChild(this.icon);
                        }
                        this.input = new dropdown.DropdownOptionFilterInput();
                        this.appendChild(this.input);
                        this.dropdownHandle = new DropdownHandle();
                        this.appendChild(this.dropdownHandle);
                        var filter = this.defaultFilter;
                        if (config.filter) {
                            filter = config.filter;
                        }
                        this.dropdownDropdown = new dropdown.DropdownDropdown({
                            maxHeight: 200,
                            width: this.input.getEl().getWidth(),
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: filter,
                            dataIdProperty: config.dataIdProperty
                        });
                        if (filter) {
                            this.dropdownDropdown.setFilterArgs({ searchString: "" });
                        }
                        this.dropdownDropdown.onRowSelection(function (event) {
                            _this.selectRow(event.getRow());
                        });
                        this.appendChild(this.dropdownDropdown.getGrid().getElement());
                        this.selectedOptionView = new dropdown.SelectedOptionView(this.optionDisplayValueViewer);
                        this.selectedOptionView.hide();
                        this.appendChild(this.selectedOptionView);
                        this.selectedOptionView.onOpenDropdown(function () {
                            _this.showDropdown();
                            _this.dropdownDropdown.navigateToFirstRowIfNotActive();
                            _this.input.giveFocus();
                        });
                        this.setupListeners();
                        this.onRendered(function (event) {
                            _this.doUpdateDropdownTopPositionAndWidth();
                        });
                    }
                    Dropdown.prototype.defaultFilter = function (option, args) {
                        if (!args.searchString || api.util.StringHelper.isEmpty(args.searchString)) {
                            return true;
                        }
                        var lowerCasedSearchString = args.searchString.toLowerCase();
                        if (option.value.toLowerCase().indexOf(lowerCasedSearchString) > -1) {
                            return true;
                        }
                        var displayVaueAsString = option.displayValue.toString();
                        if (displayVaueAsString.toLowerCase().indexOf(lowerCasedSearchString) > -1) {
                            return true;
                        }
                        var indices = option.indices;
                        if (indices && indices.length > 0) {
                            for (var i = 0; i < indices.length; i++) {
                                var index = indices[i];
                                if (index) {
                                    if (index.toLocaleLowerCase().indexOf(lowerCasedSearchString) > -1) {
                                        return true;
                                    }
                                }
                            }
                        }
                        return false;
                    };
                    Dropdown.prototype.doUpdateDropdownTopPositionAndWidth = function () {
                        var inputEl = this.input.getEl();
                        this.dropdownDropdown.setTopPx(inputEl.getHeightWithBorder() - inputEl.getBorderBottomWidth());
                        this.dropdownDropdown.setWidth(inputEl.getWidthWithBorder());
                    };
                    Dropdown.prototype.giveFocus = function () {
                        return this.input.giveFocus();
                    };
                    Dropdown.prototype.isDropdownShown = function () {
                        return this.dropdownDropdown.isDropdownShown();
                    };
                    Dropdown.prototype.showDropdown = function () {
                        this.selectedOptionView.hide();
                        this.input.show();
                        this.dropdownHandle.show();
                        this.doUpdateDropdownTopPositionAndWidth();
                        this.dropdownDropdown.showDropdown(this.getSelectedOption());
                        this.dropdownHandle.down();
                        this.dropdownDropdown.renderDropdownGrid();
                    };
                    Dropdown.prototype.hideDropdown = function () {
                        if (this.selectedOptionView.getOption()) {
                            this.input.hide();
                            this.dropdownHandle.hide();
                            this.selectedOptionView.show();
                        }
                        else {
                            this.selectedOptionView.hide();
                            this.input.show();
                            this.dropdownHandle.up();
                            this.dropdownHandle.show();
                        }
                        this.dropdownDropdown.hideDropdown();
                    };
                    Dropdown.prototype.setOptions = function (options) {
                        this.dropdownDropdown.setOptions(options);
                    };
                    Dropdown.prototype.removeAllOptions = function () {
                        this.dropdownDropdown.removeAllOptions();
                    };
                    Dropdown.prototype.addOption = function (option) {
                        this.dropdownDropdown.addOption(option);
                    };
                    Dropdown.prototype.hasOptions = function () {
                        return this.dropdownDropdown.hasOptions();
                    };
                    Dropdown.prototype.getOptionCount = function () {
                        return this.dropdownDropdown.getOptionCount();
                    };
                    Dropdown.prototype.getOptions = function () {
                        return this.dropdownDropdown.getOptions();
                    };
                    Dropdown.prototype.getOptionByValue = function (value) {
                        return this.dropdownDropdown.getOptionByValue(value);
                    };
                    Dropdown.prototype.getOptionByRow = function (rowIndex) {
                        return this.dropdownDropdown.getOptionByRow(rowIndex);
                    };
                    Dropdown.prototype.setValue = function (value) {
                        var option = this.getOptionByValue(value);
                        if (option != null) {
                            this.selectOption(option);
                        }
                        return this;
                    };
                    Dropdown.prototype.selectRow = function (index) {
                        var option = this.getOptionByRow(index);
                        if (option != null) {
                            this.selectOption(option);
                            api.dom.FormEl.moveFocusToNextFocusable(this.input);
                        }
                    };
                    Dropdown.prototype.selectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        this.dropdownDropdown.markSelections([option]);
                        if (!silent) {
                            this.notifyOptionSelected(option);
                        }
                        this.selectedOptionView.setOption(option);
                        this.hideDropdown();
                    };
                    Dropdown.prototype.getSelectedOption = function () {
                        return this.selectedOptionView.getOption();
                    };
                    Dropdown.prototype.getValue = function () {
                        var selectedOption = this.getSelectedOption();
                        if (!selectedOption) {
                            return null;
                        }
                        return selectedOption.value;
                    };
                    Dropdown.prototype.setInputIconUrl = function (iconUrl) {
                        if (!this.icon) {
                            this.icon = new api.dom.ImgEl();
                            this.icon.addClass("input-icon");
                            this.icon.insertBeforeEl(this.input);
                        }
                        this.icon.getEl().setSrc(iconUrl);
                    };
                    Dropdown.prototype.setupListeners = function () {
                        var _this = this;
                        this.onClicked(function () {
                            _this.setOnBlurListener();
                        });
                        this.dropdownHandle.onClicked(function (event) {
                            _this.dropdownDropdown.navigateToFirstRowIfNotActive();
                            if (_this.isDropdownShown()) {
                                _this.hideDropdown();
                                _this.giveFocus();
                            }
                            else {
                                _this.showDropdown();
                                _this.giveFocus();
                            }
                        });
                        this.input.onValueChanged(function (event) {
                            _this.notifyOptionFilterInputValueChanged(event.getOldValue(), event.getNewValue());
                            _this.dropdownDropdown.setFilterArgs({ searchString: event.getNewValue() });
                            _this.showDropdown();
                            _this.dropdownDropdown.nagivateToFirstRow();
                        });
                        this.input.onDblClicked(function (event) {
                            _this.dropdownDropdown.navigateToFirstRowIfNotActive();
                            if (!_this.isDropdownShown()) {
                                _this.showDropdown();
                            }
                        });
                        this.input.onKeyDown(function (event) {
                            if (event.which == 9) {
                                _this.hideDropdown();
                                return;
                            }
                            else if (event.which == 16 || event.which == 17 || event.which == 18) {
                                return;
                            }
                            _this.dropdownDropdown.navigateToFirstRowIfNotActive();
                            if (!_this.isDropdownShown()) {
                                _this.showDropdown();
                                return;
                            }
                            if (event.which == 38) {
                                _this.dropdownDropdown.navigateToPreviousRow();
                            }
                            else if (event.which == 40) {
                                _this.dropdownDropdown.navigateToNextRow();
                            }
                            else if (event.which == 13) {
                                _this.selectRow(_this.dropdownDropdown.getActiveRow());
                                _this.input.getEl().setValue("");
                                event.preventDefault();
                                event.stopPropagation();
                            }
                            else if (event.which == 27) {
                                _this.hideDropdown();
                            }
                            _this.input.getHTMLElement().focus();
                        });
                    };
                    Dropdown.prototype.setOnBlurListener = function () {
                        var combobox = this;
                        var hideDropdownOnBlur = function (event) {
                            var comboboxHtmlElement = combobox.getHTMLElement();
                            for (var element = event.target; element; element = element.parentNode) {
                                if (element == comboboxHtmlElement) {
                                    return;
                                }
                            }
                            combobox.hideDropdown();
                            combobox.active = false;
                            api.dom.Body.get().getEl().removeEventListener('click', hideDropdownOnBlur);
                        };
                        if (!this.active) {
                            this.active = true;
                            api.dom.Body.get().onClicked(hideDropdownOnBlur);
                        }
                    };
                    Dropdown.prototype.onOptionSelected = function (listener) {
                        this.optionSelectedListeners.push(listener);
                    };
                    Dropdown.prototype.unOptionSelected = function (listener) {
                        this.optionSelectedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    Dropdown.prototype.notifyOptionSelected = function (item) {
                        var event = new OptionSelectedEvent(item);
                        this.optionSelectedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Dropdown.prototype.onValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.push(listener);
                    };
                    Dropdown.prototype.unValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    Dropdown.prototype.notifyOptionFilterInputValueChanged = function (oldValue, newValue) {
                        var event = new OptionFilterInputValueChangedEvent(oldValue, newValue, this.dropdownDropdown.getGrid().getElement());
                        this.optionFilterInputValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    return Dropdown;
                })(api.dom.FormInputEl);
                dropdown.Dropdown = Dropdown;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagBuilder = (function () {
                function TagBuilder() {
                }
                TagBuilder.prototype.setValue = function (value) {
                    this.value = value;
                    return this;
                };
                TagBuilder.prototype.setRemovable = function (value) {
                    this.removable = value;
                    return this;
                };
                TagBuilder.prototype.build = function () {
                    return new Tag(this);
                };
                return TagBuilder;
            })();
            tags.TagBuilder = TagBuilder;
            var Tag = (function (_super) {
                __extends(Tag, _super);
                function Tag(builder) {
                    var _this = this;
                    _super.call(this, "tag");
                    this.tagRemoveListeners = [];
                    this.value = builder.value;
                    this.valueHolderEl = new api.dom.SpanEl();
                    this.valueHolderEl.setHtml(this.value);
                    this.appendChild(this.valueHolderEl);
                    this.removable = builder.removable;
                    if (this.removable) {
                        this.removeButtonEl = new api.dom.AEl("remove-button");
                        this.appendChild(this.removeButtonEl);
                        this.removeButtonEl.onClicked(function () {
                            _this.notifyTagRemoved();
                        });
                    }
                }
                Tag.prototype.getValue = function () {
                    return this.value;
                };
                Tag.prototype.onTagRemove = function (listener) {
                    this.tagRemoveListeners.push(listener);
                };
                Tag.prototype.unTagRemoved = function (listener) {
                    this.tagRemoveListeners.push(listener);
                };
                Tag.prototype.notifyTagRemoved = function () {
                    this.tagRemoveListeners.forEach(function (listener) {
                        listener();
                    });
                };
                return Tag;
            })(api.dom.LiEl);
            tags.Tag = Tag;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (_tags) {
            var TagSuggestions = (function (_super) {
                __extends(TagSuggestions, _super);
                function TagSuggestions() {
                    var _this = this;
                    _super.call(this, 'tag-suggestions');
                    this.selectedIndex = null;
                    this.selectedListeners = [];
                    this.onMouseMove(function (event) {
                        var htmlEl = event.target;
                        if (htmlEl.tagName == 'LI') {
                            _this.notifySelected(htmlEl.innerText || htmlEl.textContent);
                        }
                    });
                }
                TagSuggestions.prototype.setTags = function (values) {
                    var _this = this;
                    this.removeChildren();
                    values.forEach(function (value) {
                        _this.appendChild(new api.dom.LiEl().setHtml(value));
                    });
                    this.selectedIndex = null;
                };
                TagSuggestions.prototype.moveDown = function () {
                    var nextIndex;
                    if (this.selectedIndex == null) {
                        nextIndex = 0;
                    }
                    else if (this.selectedIndex == this.getChildren().length - 1) {
                        nextIndex = null;
                    }
                    else {
                        nextIndex = this.selectedIndex + 1;
                    }
                    this.select(nextIndex);
                };
                TagSuggestions.prototype.moveUp = function () {
                    var nextIndex;
                    if (this.selectedIndex == null) {
                        nextIndex = this.getChildren().length - 1;
                    }
                    else if (this.selectedIndex == 0) {
                        nextIndex = null;
                    }
                    else {
                        nextIndex = this.selectedIndex - 1;
                    }
                    this.select(nextIndex);
                };
                TagSuggestions.prototype.select = function (index) {
                    var tags = this.getChildren();
                    var tag = tags[this.selectedIndex];
                    if (tag) {
                        tag.removeClass('selected');
                    }
                    this.selectedIndex = index;
                    tag = tags[this.selectedIndex];
                    if (tag) {
                        tag.addClass('selected');
                        this.notifySelected(tag.getEl().getText());
                    }
                    else {
                        this.notifySelected(null);
                    }
                };
                TagSuggestions.prototype.onSelected = function (listener) {
                    this.selectedListeners.push(listener);
                };
                TagSuggestions.prototype.unSelected = function (listener) {
                    this.selectedListeners.push(listener);
                };
                TagSuggestions.prototype.notifySelected = function (value) {
                    this.selectedListeners.forEach(function (listener) { return listener(value); });
                };
                return TagSuggestions;
            })(api.dom.UlEl);
            _tags.TagSuggestions = TagSuggestions;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagAddedEvent = (function () {
                function TagAddedEvent(value) {
                    this.value = value;
                }
                TagAddedEvent.prototype.getValue = function () {
                    return this.value;
                };
                return TagAddedEvent;
            })();
            tags.TagAddedEvent = TagAddedEvent;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagRemovedEvent = (function () {
                function TagRemovedEvent(value, index) {
                    this.value = value;
                    this.index = index;
                }
                TagRemovedEvent.prototype.getValue = function () {
                    return this.value;
                };
                TagRemovedEvent.prototype.getIndex = function () {
                    return this.index;
                };
                return TagRemovedEvent;
            })();
            tags.TagRemovedEvent = TagRemovedEvent;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagsBuilder = (function () {
                function TagsBuilder() {
                    this.tags = [];
                    this.maxTags = 0;
                }
                TagsBuilder.prototype.setTagSuggester = function (value) {
                    this.tagSuggester = value;
                    return this;
                };
                TagsBuilder.prototype.addTag = function (value) {
                    this.tags.push(value);
                    return this;
                };
                TagsBuilder.prototype.setMaxTags = function (value) {
                    this.maxTags = value;
                    return this;
                };
                TagsBuilder.prototype.build = function () {
                    return new Tags(this);
                };
                return TagsBuilder;
            })();
            tags.TagsBuilder = TagsBuilder;
            var Tags = (function (_super) {
                __extends(Tags, _super);
                function Tags(builder) {
                    var _this = this;
                    _super.call(this, "tags");
                    this.tags = [];
                    this.tagAddedListeners = [];
                    this.tagRemovedListeners = [];
                    this.tagSuggester = builder.tagSuggester;
                    builder.tags.forEach(function (value) {
                        _this.doAddTag(value);
                    });
                    this.maxTags = builder.maxTags;
                    this.textInput = new api.ui.text.TextInput();
                    this.appendChild(this.textInput);
                    this.tagSuggestions = new tags.TagSuggestions();
                    this.tagSuggestions.hide();
                    this.appendChild(this.tagSuggestions);
                    this.textInput.onKeyDown(function (event) {
                        if (event.keyCode == 188 || event.keyCode == 13) {
                            _this.handleWordCompleted();
                            event.preventDefault();
                        }
                        else if (event.keyCode == 8) {
                            if (!_this.textInput.getValue() && _this.countTags() > 0) {
                                _this.removeTag(_this.tags[_this.countTags() - 1]);
                            }
                        }
                        else if (event.keyCode == 38) {
                            if (_this.tagSuggestions.isVisible()) {
                                _this.tagSuggestions.moveUp();
                                event.preventDefault();
                            }
                        }
                        else if (event.keyCode == 40) {
                            if (_this.tagSuggestions.isVisible()) {
                                _this.tagSuggestions.moveDown();
                                event.preventDefault();
                            }
                        }
                    });
                    this.tagSuggestions.onSelected(function (value) {
                        _this.textInput.getEl().setValue(value || _this.preservedValue);
                        if (value) {
                            _this.textInput.selectText(_this.preservedValue.length, value.length);
                        }
                    });
                    this.textInput.onBlur(function (event) {
                        _this.handleWordCompleted();
                        _this.textInput.getEl().setValue('').setWidthPx(1);
                    });
                    this.textInput.onFocus(function (event) {
                        _this.textInput.getEl().setWidth('');
                    });
                    this.textInput.onValueChanged(function (event) {
                        var searchString = event.getNewValue();
                        _this.tagSuggester.suggest(searchString).then(function (values) {
                            var existingValues = _this.getTags().concat(searchString);
                            values = values.filter(function (value) { return (existingValues.indexOf(value) < 0); });
                            if (values.length == 0) {
                                _this.tagSuggestions.hide();
                            }
                            else {
                                _this.tagSuggestions.setTags(values);
                                _this.tagSuggestions.getEl().setTopPx(_this.textInput.getEl().getOffsetToParent().top + _this.textInput.getEl().getHeightWithMargin()).setLeftPx(_this.textInput.getEl().getOffsetToParent().left);
                                _this.tagSuggestions.show();
                                _this.preservedValue = searchString;
                            }
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                            return [];
                        }).done();
                    });
                    this.onClicked(function () {
                        _this.textInput.getEl().setWidth('');
                        _this.textInput.giveFocus();
                    });
                }
                Tags.prototype.handleWordCompleted = function () {
                    var inputValue = this.textInput.getValue();
                    var word = inputValue.trim();
                    var tag = this.doAddTag(word);
                    if (tag) {
                        this.notifyTagAdded(new tags.TagAddedEvent(tag.getValue()));
                        this.textInput.setValue("");
                        if (this.isMaxTagsReached()) {
                            this.textInput.hide();
                        }
                    }
                };
                Tags.prototype.clearTags = function () {
                    this.tags.forEach(function (tag) { return tag.remove(); });
                    this.tags = [];
                };
                Tags.prototype.addTag = function (value) {
                    var tag = this.doAddTag(value);
                    if (tag) {
                        if (this.isMaxTagsReached()) {
                            this.textInput.hide();
                        }
                    }
                };
                Tags.prototype.doAddTag = function (value) {
                    var _this = this;
                    if (this.hasTag(value) || !value) {
                        return null;
                    }
                    var tag = new tags.TagBuilder().setValue(value).setRemovable(true).build();
                    this.tags.push(tag);
                    tag.insertBeforeEl(this.textInput);
                    tag.onTagRemove(function () { return _this.removeTag(tag); });
                    return tag;
                };
                Tags.prototype.removeTag = function (tag) {
                    var index = this.tags.indexOf(tag);
                    if (index >= 0) {
                        tag.remove();
                        this.tags.splice(index, 1);
                        if (!this.textInput.isVisible() && !this.isMaxTagsReached()) {
                            this.textInput.setVisible(true);
                        }
                        this.textInput.giveFocus();
                        this.notifyTagRemoved(new tags.TagRemovedEvent(tag.getValue(), index));
                    }
                };
                Tags.prototype.hasTag = function (value) {
                    var match = false;
                    this.tags.forEach(function (tag) {
                        if (value == tag.getValue()) {
                            match = true;
                        }
                    });
                    return match;
                };
                Tags.prototype.countTags = function () {
                    return this.tags.length;
                };
                Tags.prototype.getTags = function () {
                    return this.tags.map(function (tag) { return tag.getValue(); });
                };
                Tags.prototype.isMaxTagsReached = function () {
                    if (this.maxTags == 0) {
                        return false;
                    }
                    return this.countTags() >= this.maxTags;
                };
                Tags.prototype.onTagAdded = function (listener) {
                    this.tagAddedListeners.push(listener);
                };
                Tags.prototype.unTagAdded = function (listener) {
                    this.tagAddedListeners.push(listener);
                };
                Tags.prototype.notifyTagAdded = function (event) {
                    this.tagAddedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Tags.prototype.onTagRemoved = function (listener) {
                    this.tagRemovedListeners.push(listener);
                };
                Tags.prototype.unTagRemoved = function (listener) {
                    this.tagRemovedListeners.push(listener);
                };
                Tags.prototype.giveFocus = function () {
                    if (this.isMaxTagsReached()) {
                        return this.tags[0].giveFocus();
                    }
                    else {
                        this.textInput.getEl().setWidth('');
                        return this.textInput.giveFocus();
                    }
                };
                Tags.prototype.notifyTagRemoved = function (event) {
                    this.tagRemovedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return Tags;
            })(api.dom.UlEl);
            tags.Tags = Tags;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var geo;
        (function (geo) {
            var GeoPoint = (function (_super) {
                __extends(GeoPoint, _super);
                function GeoPoint() {
                    var _this = this;
                    _super.call(this, "geo-point");
                    this.validUserInput = true;
                    this.geoLocationInput = new api.ui.text.TextInput();
                    this.geoLocationInput.getEl().setAttribute("title", "geo location").addClass("geo-point-input");
                    this.layoutItems();
                    this.geoLocationInput.onKeyUp(function (event) {
                        var typedGeoPoint = _this.geoLocationInput.getValue();
                        if (api.util.StringHelper.isEmpty(typedGeoPoint)) {
                            _this.validUserInput = true;
                        }
                        else {
                            if (api.util.GeoPoint.isValidString(typedGeoPoint)) {
                                _this.validUserInput = true;
                            }
                            else {
                                _this.validUserInput = false;
                            }
                        }
                        _this.updateInputStyling();
                    });
                }
                GeoPoint.prototype.layoutItems = function () {
                    this.removeChildren();
                    this.appendChild(this.geoLocationInput);
                    this.geoLocationInput.setPlaceholder(_i18n('geo location'));
                    return this;
                };
                GeoPoint.prototype.onValueChanged = function (listener) {
                    this.geoLocationInput.onValueChanged(listener);
                };
                GeoPoint.prototype.setGeoPoint = function (value) {
                    this.geoLocationInput.setValue("" + value.getLatitude() + "," + value.getLongitude());
                    return this;
                };
                GeoPoint.prototype.getGeoPoint = function () {
                    if (api.util.StringHelper.isEmpty(this.geoLocationInput.getValue())) {
                        return null;
                    }
                    return api.util.GeoPoint.fromString(this.geoLocationInput.getValue());
                };
                GeoPoint.prototype.giveFocus = function () {
                    return this.geoLocationInput.giveFocus();
                };
                GeoPoint.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                GeoPoint.prototype.updateInputStyling = function () {
                    this.geoLocationInput.updateValidationStatusOnUserInput(this.validUserInput);
                };
                return GeoPoint;
            })(api.dom.DivEl);
            geo.GeoPoint = GeoPoint;
        })(geo = ui.geo || (ui.geo = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DayOfWeek = (function () {
                function DayOfWeek(numberCode, oneLetterName, shortName, fullName) {
                    this.numberCode = numberCode;
                    this.oneLetterName = oneLetterName;
                    this.shortName = shortName;
                    this.fullName = fullName;
                }
                DayOfWeek.prototype.getNumberCode = function () {
                    return this.numberCode;
                };
                DayOfWeek.prototype.getOneLetterName = function () {
                    return this.oneLetterName;
                };
                DayOfWeek.prototype.getShortName = function () {
                    return this.shortName;
                };
                DayOfWeek.prototype.getFullName = function () {
                    return this.fullName;
                };
                DayOfWeek.prototype.getPrevioius = function () {
                    return this.previous;
                };
                DayOfWeek.prototype.getNext = function () {
                    return this.next;
                };
                DayOfWeek.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DayOfWeek)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.numberEquals(this.numberCode, other.numberCode)) {
                        return false;
                    }
                    return true;
                };
                return DayOfWeek;
            })();
            time.DayOfWeek = DayOfWeek;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DaysOfWeek = (function () {
                function DaysOfWeek() {
                }
                DaysOfWeek.getByNumberCode = function (value) {
                    var match = null;
                    DaysOfWeek.ALL.forEach(function (dayOfWeek) {
                        if (dayOfWeek.getNumberCode() == value) {
                            match = dayOfWeek;
                        }
                    });
                    return match;
                };
                DaysOfWeek.MONDAY = new time.DayOfWeek(1, "M", "Mon", "Monday");
                DaysOfWeek.TUESDAY = new time.DayOfWeek(2, "T", "Tue", "Tuesday");
                DaysOfWeek.WEDNESDAY = new time.DayOfWeek(3, "W", "Wed", "Wednesday");
                DaysOfWeek.THURSDAY = new time.DayOfWeek(4, "T", "Thu", "Thursday");
                DaysOfWeek.FRIDAY = new time.DayOfWeek(5, "F", "Fri", "Friday");
                DaysOfWeek.SATURDAY = new time.DayOfWeek(6, "S", "Sat", "Saturday");
                DaysOfWeek.SUNDAY = new time.DayOfWeek(0, "S", "Sun", "Sunday");
                DaysOfWeek.ALL = [DaysOfWeek.MONDAY, DaysOfWeek.TUESDAY, DaysOfWeek.WEDNESDAY, DaysOfWeek.THURSDAY, DaysOfWeek.FRIDAY, DaysOfWeek.SATURDAY, DaysOfWeek.SUNDAY];
                return DaysOfWeek;
            })();
            time.DaysOfWeek = DaysOfWeek;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarDayBuilder = (function () {
                function CalendarDayBuilder() {
                }
                CalendarDayBuilder.prototype.setDate = function (value) {
                    this.date = value;
                    return this;
                };
                CalendarDayBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                CalendarDayBuilder.prototype.setPreviousDay = function (value) {
                    this.previousDay = value;
                    return this;
                };
                CalendarDayBuilder.prototype.setNextDay = function (value) {
                    this.nextDay = value;
                    return this;
                };
                CalendarDayBuilder.prototype.build = function () {
                    return new CalendarDay(this);
                };
                return CalendarDayBuilder;
            })();
            time.CalendarDayBuilder = CalendarDayBuilder;
            var CalendarDay = (function (_super) {
                __extends(CalendarDay, _super);
                function CalendarDay(builder) {
                    var _this = this;
                    _super.call(this, "calendar-day");
                    this.selectedDay = false;
                    this.calendarDayClickedListeners = [];
                    this.date = builder.date;
                    this.month = builder.month;
                    this.previousDay = builder.previousDay;
                    if (this.previousDay) {
                        this.previousDay.nextDay = this;
                    }
                    this.nextDay = builder.nextDay;
                    if (this.nextDay) {
                        this.nextDay.previousDay = this;
                    }
                    if (new Date().toDateString() == builder.date.toDateString()) {
                        this.addClass('today');
                    }
                    this.dayOfWeek = time.DaysOfWeek.getByNumberCode(this.date.getDay());
                    this.setHtml("" + this.date.getDate());
                    if (this.isBeforeMonth()) {
                        this.addClass("before-month");
                    }
                    if (this.isAfterMonth()) {
                        this.addClass("after-month");
                    }
                    this.onClicked(function (event) {
                        _this.notifyCalendarDayClicked();
                    });
                }
                CalendarDay.prototype.getDate = function () {
                    return this.date;
                };
                CalendarDay.prototype.getDayOfMonth = function () {
                    return this.date.getDate();
                };
                CalendarDay.prototype.setSelectedDay = function (value) {
                    this.selectedDay = value;
                    this.refreshSelectedDay();
                };
                CalendarDay.prototype.refreshSelectedDay = function () {
                    if (this.selectedDay && !this.hasClass("selected-day")) {
                        this.addClass("selected-day");
                    }
                    else if (!this.selectedDay && this.hasClass("selected-day")) {
                        this.removeClass("selected-day");
                    }
                };
                CalendarDay.prototype.isInMonth = function () {
                    return !this.isBeforeMonth() || !this.isAfterMonth();
                };
                CalendarDay.prototype.isBeforeMonth = function () {
                    return this.date.getMonth() < this.month || (this.month == 0 && this.date.getMonth() == 11);
                };
                CalendarDay.prototype.isAfterMonth = function () {
                    return this.date.getMonth() > this.month || (this.month == 11 && this.date.getMonth() == 0);
                };
                CalendarDay.prototype.isLastDayOfMonth = function (month) {
                    var lastDateOfMonth = api.util.DateHelper.newUTCDate(this.date.getFullYear(), month + 1, 0);
                    return month == this.date.getMonth() && this.date.getDate() == lastDateOfMonth.getDate();
                };
                CalendarDay.prototype.getDayOfWeek = function () {
                    return this.dayOfWeek;
                };
                CalendarDay.prototype.getPrevious = function () {
                    if (this.previousDay) {
                        return this.previousDay;
                    }
                    else {
                        var prevDate = api.util.DateHelper.newUTCDate(this.date.getFullYear(), this.date.getMonth(), this.date.getDate() - 1);
                        this.previousDay = new CalendarDayBuilder().setDate(prevDate).setMonth(this.month).setNextDay(this).build();
                        return this.previousDay;
                    }
                };
                CalendarDay.prototype.getNext = function () {
                    if (this.nextDay) {
                        return this.nextDay;
                    }
                    else {
                        var nextDate = api.util.DateHelper.newUTCDate(this.date.getFullYear(), this.date.getMonth(), this.date.getDate() + 1);
                        this.nextDay = new CalendarDayBuilder().setDate(nextDate).setMonth(this.month).setPreviousDay(this).build();
                        return this.nextDay;
                    }
                };
                CalendarDay.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, CalendarDay)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.dateEquals(this.date, other.date)) {
                        return false;
                    }
                    if (!api.ObjectHelper.numberEquals(this.month, other.month)) {
                        return false;
                    }
                    return true;
                };
                CalendarDay.prototype.onCalendarDayClicked = function (listener) {
                    this.calendarDayClickedListeners.push(listener);
                };
                CalendarDay.prototype.unCalendarDayClicked = function (listener) {
                    this.calendarDayClickedListeners = this.calendarDayClickedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                CalendarDay.prototype.notifyCalendarDayClicked = function () {
                    var event = new time.CalendarDayClickedEvent(this);
                    this.calendarDayClickedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return CalendarDay;
            })(api.dom.LiEl);
            time.CalendarDay = CalendarDay;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarDayClickedEvent = (function () {
                function CalendarDayClickedEvent(calendarDay) {
                    this.calendarDay = calendarDay;
                }
                CalendarDayClickedEvent.prototype.getCalendarDay = function () {
                    return this.calendarDay;
                };
                return CalendarDayClickedEvent;
            })();
            time.CalendarDayClickedEvent = CalendarDayClickedEvent;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarWeekBuilder = (function () {
                function CalendarWeekBuilder() {
                    this.calendarDays = [];
                }
                CalendarWeekBuilder.prototype.addDay = function (value) {
                    this.calendarDays.push(value);
                    return this;
                };
                CalendarWeekBuilder.prototype.build = function () {
                    return new CalendarWeek(this);
                };
                return CalendarWeekBuilder;
            })();
            time.CalendarWeekBuilder = CalendarWeekBuilder;
            var CalendarWeek = (function (_super) {
                __extends(CalendarWeek, _super);
                function CalendarWeek(builder) {
                    var _this = this;
                    this.calendarDays = builder.calendarDays;
                    _super.call(this, "calendar-week");
                    this.calendarDays.forEach(function (day) {
                        _this.appendChild(day);
                    });
                }
                CalendarWeek.prototype.hasLastDayOfMonth = function (month) {
                    var match = false;
                    this.calendarDays.forEach(function (day) {
                        if (day.isLastDayOfMonth(month)) {
                            match = true;
                        }
                    });
                    return match;
                };
                CalendarWeek.prototype.getNextWeeksFirstDay = function () {
                    return this.calendarDays[6].getNext();
                };
                return CalendarWeek;
            })(api.dom.UlEl);
            time.CalendarWeek = CalendarWeek;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var SelectedDateChangedEvent = (function () {
                function SelectedDateChangedEvent(selectedDate) {
                    this.date = selectedDate;
                }
                SelectedDateChangedEvent.prototype.getDate = function () {
                    return this.date;
                };
                return SelectedDateChangedEvent;
            })();
            time.SelectedDateChangedEvent = SelectedDateChangedEvent;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarBuilder = (function () {
                function CalendarBuilder() {
                    this.interactive = false;
                }
                CalendarBuilder.prototype.setYear = function (value) {
                    this.year = value;
                    return this;
                };
                CalendarBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                CalendarBuilder.prototype.setSelectedDate = function (value) {
                    this.selectedDate = value;
                    return this;
                };
                CalendarBuilder.prototype.setStartingDayOfWeek = function (value) {
                    this.startingDayOfWeek = value;
                    return this;
                };
                CalendarBuilder.prototype.setInteractive = function (value) {
                    this.interactive = value;
                    return this;
                };
                CalendarBuilder.prototype.build = function () {
                    return new Calendar(this);
                };
                return CalendarBuilder;
            })();
            time.CalendarBuilder = CalendarBuilder;
            var Calendar = (function (_super) {
                __extends(Calendar, _super);
                function Calendar(builder) {
                    _super.call(this, "calendar");
                    this.selectedDateChangedListeners = [];
                    this.shownMonthChangedListeners = [];
                    var now = new Date();
                    this.year = builder.year || now.getUTCFullYear();
                    this.month = builder.month != undefined ? builder.month : now.getUTCMonth();
                    this.selectedDate = builder.selectedDate;
                    this.startingDayOfWeek = builder.startingDayOfWeek || time.DaysOfWeek.MONDAY;
                    this.interactive = builder.interactive;
                    this.renderMonth();
                }
                Calendar.prototype.selectDate = function (value) {
                    if (value) {
                        this.year = value.getUTCFullYear();
                        this.month = value.getUTCMonth();
                        this.selectedDate = value;
                        this.removeChildren();
                        if (api.util.DateHelper.isInvalidDate(value)) {
                            var spanEl = new api.dom.SpanEl().setHtml("Invalid date");
                            this.appendChild(spanEl);
                        }
                        else {
                            this.renderMonth();
                        }
                    }
                    else {
                        this.selectedDate = null;
                        var now = new Date();
                        this.year = now.getUTCFullYear();
                        this.month = now.getUTCMonth();
                    }
                };
                Calendar.prototype.nextMonth = function () {
                    this.month++;
                    if (this.month > 11) {
                        this.month = 0;
                        this.year++;
                    }
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.previousMonth = function () {
                    this.month--;
                    if (this.month < 0) {
                        this.month = 11;
                        this.year--;
                    }
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.nextYear = function () {
                    this.year++;
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.previousYear = function () {
                    this.year--;
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.renderMonth = function () {
                    var _this = this;
                    this.calendarDays = this.resolveDaysInMonth();
                    var firstDay = this.resolveFirstDayOfCalendar();
                    this.weeks = this.createCalendarWeeks(firstDay);
                    this.weeks.forEach(function (week) {
                        _this.appendChild(week);
                    });
                    this.notifyShownMonthChanged(this.month, this.year);
                };
                Calendar.prototype.resolveDaysInMonth = function () {
                    var calendarDays = [];
                    var daysInMonth = api.util.DateHelper.newUTCDate(this.year, this.month, 0).getDate();
                    var previousDay = null;
                    for (var i = 1; i <= daysInMonth; i++) {
                        var calendarDay = this.createCalendarDay(i, previousDay);
                        calendarDays.push(calendarDay);
                        previousDay = calendarDay;
                    }
                    return calendarDays;
                };
                Calendar.prototype.resolveFirstDayOfCalendar = function () {
                    var firstDay = null;
                    if (this.startingDayOfWeek.equals(this.calendarDays[0].getDayOfWeek())) {
                        firstDay = this.calendarDays[0];
                    }
                    else {
                        var previousDay = this.calendarDays[0].getPrevious();
                        while (!previousDay.getDayOfWeek().equals(this.startingDayOfWeek)) {
                            previousDay = previousDay.getPrevious();
                        }
                        firstDay = previousDay;
                    }
                    return firstDay;
                };
                Calendar.prototype.createCalendarWeeks = function (firstDay) {
                    var weeks = [];
                    var currWeek = this.createCalendarWeek(firstDay);
                    weeks.push(currWeek);
                    while (!currWeek.hasLastDayOfMonth(this.month)) {
                        var newWeek = this.createCalendarWeek(currWeek.getNextWeeksFirstDay());
                        weeks.push(newWeek);
                        currWeek = newWeek;
                    }
                    return weeks;
                };
                Calendar.prototype.createCalendarWeek = function (firstDayOfWeek) {
                    var weekBuilder = new time.CalendarWeekBuilder();
                    weekBuilder.addDay(firstDayOfWeek);
                    var nextDay = firstDayOfWeek.getNext();
                    for (var i = 0; i < 6; i++) {
                        weekBuilder.addDay(nextDay);
                        nextDay = nextDay.getNext();
                    }
                    return weekBuilder.build();
                };
                Calendar.prototype.createCalendarDay = function (dayOfMonth, previousDay) {
                    var _this = this;
                    var date = api.util.DateHelper.newUTCDate(this.year, this.month, dayOfMonth);
                    var calendarDay = new time.CalendarDayBuilder().setDate(date).setMonth(this.month).setPreviousDay(previousDay).build();
                    if (calendarDay.isInMonth()) {
                        if (this.interactive) {
                            calendarDay.onCalendarDayClicked(function (event) { return _this.handleCalendarDayClicked(event); });
                        }
                        if (this.selectedDate && this.selectedDate.toDateString() == date.toDateString()) {
                            calendarDay.setSelectedDay(true);
                        }
                    }
                    return calendarDay;
                };
                Calendar.prototype.handleCalendarDayClicked = function (event) {
                    this.calendarDays.forEach(function (calendarDay) {
                        calendarDay.setSelectedDay(event.getCalendarDay().equals(calendarDay));
                    });
                    this.selectedDate = event.getCalendarDay().getDate();
                    this.notifySelectedDateChanged(this.selectedDate);
                };
                Calendar.prototype.getSelectedDate = function () {
                    return this.selectedDate;
                };
                Calendar.prototype.getMonth = function () {
                    return this.month;
                };
                Calendar.prototype.getYear = function () {
                    return this.year;
                };
                Calendar.prototype.getCalendarDays = function () {
                    return this.calendarDays;
                };
                Calendar.prototype.onSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners.push(listener);
                };
                Calendar.prototype.unSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners = this.selectedDateChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Calendar.prototype.notifySelectedDateChanged = function (date) {
                    var event = new time.SelectedDateChangedEvent(date);
                    this.selectedDateChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Calendar.prototype.onShownMonthChanged = function (listener) {
                    this.shownMonthChangedListeners.push(listener);
                };
                Calendar.prototype.unShownMonthChanged = function (listener) {
                    this.shownMonthChangedListeners = this.shownMonthChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Calendar.prototype.notifyShownMonthChanged = function (month, year) {
                    this.shownMonthChangedListeners.forEach(function (listener) {
                        listener(month, year);
                    });
                };
                return Calendar;
            })(api.dom.DivEl);
            time.Calendar = Calendar;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DatePickerBuilder = (function () {
                function DatePickerBuilder() {
                    this.startingDayOfWeek = time.DaysOfWeek.MONDAY;
                    this.closeOnSelect = true;
                    this.closeOnOutsideClick = true;
                }
                DatePickerBuilder.prototype.setYear = function (value) {
                    this.year = value;
                    return this;
                };
                DatePickerBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                DatePickerBuilder.prototype.setSelectedDate = function (value) {
                    this.selectedDate = value;
                    return this;
                };
                DatePickerBuilder.prototype.setCalendar = function (value) {
                    this.calendar = value;
                    return this;
                };
                DatePickerBuilder.prototype.setStartingDayOfWeek = function (value) {
                    this.startingDayOfWeek = value;
                    return this;
                };
                DatePickerBuilder.prototype.setCloseOnSelect = function (value) {
                    this.closeOnSelect = value;
                    return this;
                };
                DatePickerBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DatePickerBuilder.prototype.build = function () {
                    return new DatePicker(this);
                };
                return DatePickerBuilder;
            })();
            time.DatePickerBuilder = DatePickerBuilder;
            var DatePicker = (function (_super) {
                __extends(DatePicker, _super);
                function DatePicker(builder) {
                    var _this = this;
                    _super.call(this, 'date-picker');
                    this.selectedDateChangedListeners = [];
                    this.validUserInput = true;
                    this.input = api.ui.text.TextInput.middle();
                    this.input.onClicked(function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                        _this.popup.show();
                    });
                    var wrapper = new api.dom.DivEl('wrapper');
                    wrapper.appendChild(this.input);
                    this.calendar = builder.calendar || new time.CalendarBuilder().setSelectedDate(builder.selectedDate).setMonth(builder.month).setYear(builder.year).setInteractive(true).build();
                    var popupBuilder = new time.DatePickerPopupBuilder().setCalendar(this.calendar).setCloseOnOutsideClick(builder.closeOnOutsideClick);
                    this.popup = popupBuilder.build();
                    wrapper.appendChild(this.popup);
                    this.popupTrigger = new api.ui.button.Button();
                    this.popupTrigger.addClass('icon-calendar4');
                    wrapper.appendChild(this.popupTrigger);
                    this.appendChild(wrapper);
                    this.popupTrigger.onClicked(function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                        if (_this.popup.isVisible()) {
                            _this.popup.hide();
                        }
                        else {
                            _this.popup.show();
                        }
                    });
                    if (builder.selectedDate) {
                        this.input.setValue(this.formatDate(builder.selectedDate));
                        this.selectedDate = builder.selectedDate;
                    }
                    this.popup.onSelectedDateChanged(function (e) {
                        if (builder.closeOnSelect) {
                            _this.popup.hide();
                        }
                        _this.selectedDate = e.getDate();
                        _this.input.setValue(_this.formatDate(e.getDate()));
                        _this.notifySelectedDateChanged(e);
                        _this.validUserInput = true;
                        _this.updateInputStyling();
                    });
                    this.input.onKeyUp(function (event) {
                        var typedDate = _this.input.getValue();
                        if (api.util.StringHelper.isEmpty(typedDate)) {
                            _this.calendar.selectDate(null);
                            _this.selectedDate = null;
                            _this.validUserInput = true;
                            _this.popup.hide();
                            _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(null));
                        }
                        else {
                            var date = api.util.DateHelper.parseUTCDate(typedDate);
                            if (date) {
                                var correctlyTyped = api.util.DateHelper.formatUTCDate(date) == typedDate;
                                if (correctlyTyped) {
                                    _this.selectedDate = date;
                                    _this.validUserInput = true;
                                    _this.calendar.selectDate(date);
                                    _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(date));
                                    if (!_this.popup.isVisible()) {
                                        _this.popup.show();
                                    }
                                }
                                else {
                                    _this.selectedDate = null;
                                    _this.validUserInput = false;
                                    _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(null));
                                }
                            }
                            else {
                                _this.selectedDate = null;
                                _this.validUserInput = false;
                                _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(null));
                            }
                        }
                        _this.updateInputStyling();
                    });
                }
                DatePicker.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                DatePicker.prototype.getSelectedDate = function () {
                    return this.selectedDate;
                };
                DatePicker.prototype.onSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners.push(listener);
                };
                DatePicker.prototype.unSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners = this.selectedDateChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                DatePicker.prototype.notifySelectedDateChanged = function (event) {
                    this.selectedDateChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                DatePicker.prototype.formatDate = function (date) {
                    return api.util.DateHelper.formatUTCDate(date);
                };
                DatePicker.prototype.updateInputStyling = function () {
                    this.input.updateValidationStatusOnUserInput(this.validUserInput);
                };
                return DatePicker;
            })(api.dom.DivEl);
            time.DatePicker = DatePicker;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DatePickerPopupBuilder = (function () {
                function DatePickerPopupBuilder() {
                    this.closeOnOutsideClick = true;
                }
                DatePickerPopupBuilder.prototype.setCalendar = function (value) {
                    this.calendar = value;
                    return this;
                };
                DatePickerPopupBuilder.prototype.getCalendar = function () {
                    return this.calendar;
                };
                DatePickerPopupBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DatePickerPopupBuilder.prototype.isCloseOnOutsideClick = function () {
                    return this.closeOnOutsideClick;
                };
                DatePickerPopupBuilder.prototype.build = function () {
                    return new DatePickerPopup(this);
                };
                return DatePickerPopupBuilder;
            })();
            time.DatePickerPopupBuilder = DatePickerPopupBuilder;
            var DatePickerPopup = (function (_super) {
                __extends(DatePickerPopup, _super);
                function DatePickerPopup(builder) {
                    var _this = this;
                    _super.call(this, 'date-picker-dialog');
                    var yearContainer = new api.dom.H2El('year-container');
                    this.appendChild(yearContainer);
                    this.prevYear = new api.dom.AEl('prev');
                    this.prevYear.onClicked(function (e) {
                        _this.calendar.previousYear();
                    });
                    yearContainer.appendChild(this.prevYear);
                    this.year = new api.dom.SpanEl();
                    yearContainer.appendChild(this.year);
                    this.nextYear = new api.dom.AEl('next');
                    this.nextYear.onClicked(function (e) {
                        _this.calendar.nextYear();
                    });
                    yearContainer.appendChild(this.nextYear);
                    var monthContainer = new api.dom.H5El('month-container');
                    this.appendChild(monthContainer);
                    this.prevMonth = new api.dom.AEl('prev');
                    this.prevMonth.onClicked(function (e) {
                        _this.calendar.previousMonth();
                    });
                    monthContainer.appendChild(this.prevMonth);
                    this.month = new api.dom.SpanEl();
                    monthContainer.appendChild(this.month);
                    this.nextMonth = new api.dom.AEl('next');
                    this.nextMonth.onClicked(function (e) {
                        _this.calendar.nextMonth();
                    });
                    monthContainer.appendChild(this.nextMonth);
                    this.calendar = builder.getCalendar() || new time.CalendarBuilder().build();
                    this.year.setHtml(this.calendar.getYear().toString());
                    this.month.setHtml(time.MonthsOfYear.getByNumberCode(this.calendar.getMonth()).getFullName());
                    this.calendar.onShownMonthChanged(function (month, year) {
                        _this.month.setHtml(time.MonthsOfYear.getByNumberCode(month).getFullName());
                        _this.year.setHtml(year.toString());
                    });
                    this.appendChild(this.calendar);
                    if (builder.isCloseOnOutsideClick()) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                DatePickerPopup.prototype.getSelectedDate = function () {
                    return this.calendar.getSelectedDate();
                };
                DatePickerPopup.prototype.getCalendar = function () {
                    return this.calendar;
                };
                DatePickerPopup.prototype.onSelectedDateChanged = function (listener) {
                    this.calendar.onSelectedDateChanged(listener);
                };
                DatePickerPopup.prototype.unSelectedDateChanged = function (listener) {
                    this.calendar.unSelectedDateChanged(listener);
                };
                DatePickerPopup.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.hide();
                    }
                };
                return DatePickerPopup;
            })(api.dom.DivEl);
            time.DatePickerPopup = DatePickerPopup;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var MonthOfYear = (function () {
                function MonthOfYear(numberCode, oneLetterName, shortName, fullName) {
                    this.numberCode = numberCode;
                    this.oneLetterName = oneLetterName;
                    this.shortName = shortName;
                    this.fullName = fullName;
                }
                MonthOfYear.prototype.getNumberCode = function () {
                    return this.numberCode;
                };
                MonthOfYear.prototype.getOneLetterName = function () {
                    return this.oneLetterName;
                };
                MonthOfYear.prototype.getShortName = function () {
                    return this.shortName;
                };
                MonthOfYear.prototype.getFullName = function () {
                    return this.fullName;
                };
                MonthOfYear.prototype.getPrevioius = function () {
                    return this.previous;
                };
                MonthOfYear.prototype.getNext = function () {
                    return this.next;
                };
                MonthOfYear.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, MonthOfYear)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.numberEquals(this.numberCode, other.numberCode)) {
                        return false;
                    }
                    return true;
                };
                return MonthOfYear;
            })();
            time.MonthOfYear = MonthOfYear;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var MonthsOfYear = (function () {
                function MonthsOfYear() {
                }
                MonthsOfYear.getByNumberCode = function (code) {
                    return MonthsOfYear.monthsByCode[code];
                };
                MonthsOfYear.JANUARY = new time.MonthOfYear(0, "J", "Jan", "January");
                MonthsOfYear.FEBRUARY = new time.MonthOfYear(1, "F", "Feb", "February");
                MonthsOfYear.MARCH = new time.MonthOfYear(2, "M", "Mar", "March");
                MonthsOfYear.APRIL = new time.MonthOfYear(3, "A", "Apr", "April");
                MonthsOfYear.MAY = new time.MonthOfYear(4, "M", "May", "May");
                MonthsOfYear.JUNE = new time.MonthOfYear(5, "J", "Jun", "June");
                MonthsOfYear.JULY = new time.MonthOfYear(6, "J", "Jul", "July");
                MonthsOfYear.AUGUST = new time.MonthOfYear(7, "A", "Aug", "August");
                MonthsOfYear.SEPTEMBER = new time.MonthOfYear(8, "S", "Sep", "September");
                MonthsOfYear.OCTOBER = new time.MonthOfYear(9, "O", "Oct", "October");
                MonthsOfYear.NOVEMBER = new time.MonthOfYear(10, "N", "Nov", "November");
                MonthsOfYear.DECEMBER = new time.MonthOfYear(11, "D", "Dec", "December");
                MonthsOfYear.monthsByCode = {
                    0: MonthsOfYear.JANUARY,
                    1: MonthsOfYear.FEBRUARY,
                    2: MonthsOfYear.MARCH,
                    3: MonthsOfYear.APRIL,
                    4: MonthsOfYear.MAY,
                    5: MonthsOfYear.JUNE,
                    6: MonthsOfYear.JULY,
                    7: MonthsOfYear.AUGUST,
                    8: MonthsOfYear.SEPTEMBER,
                    9: MonthsOfYear.OCTOBER,
                    10: MonthsOfYear.NOVEMBER,
                    11: MonthsOfYear.DECEMBER
                };
                return MonthsOfYear;
            })();
            time.MonthsOfYear = MonthsOfYear;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var TimePickerBuilder = (function () {
                function TimePickerBuilder() {
                    this.closeOnOutsideClick = true;
                }
                TimePickerBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                TimePickerBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                TimePickerBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                TimePickerBuilder.prototype.build = function () {
                    return new TimePicker(this);
                };
                return TimePickerBuilder;
            })();
            time.TimePickerBuilder = TimePickerBuilder;
            var TimePicker = (function (_super) {
                __extends(TimePicker, _super);
                function TimePicker(builder) {
                    var _this = this;
                    _super.call(this, 'time-picker');
                    this.validUserInput = true;
                    this.input = api.ui.text.TextInput.middle();
                    this.input.onClicked(function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                        _this.popup.show();
                    });
                    this.input.onKeyUp(function (event) {
                        var typedTime = _this.input.getValue();
                        if (api.util.StringHelper.isEmpty(typedTime)) {
                            _this.popup.setSelectedTime(null, null);
                            _this.validUserInput = true;
                            if (_this.popup.isVisible()) {
                                _this.popup.hide();
                            }
                        }
                        else {
                            var parsedTime = typedTime.match(/^[0-2][0-9]:[0-5][0-9]$/);
                            if (parsedTime && parsedTime.length == 1) {
                                var splitTime = parsedTime[0].split(':');
                                _this.popup.setSelectedTime(parseInt(splitTime[0]), parseInt(splitTime[1]));
                                _this.validUserInput = true;
                                if (!_this.popup.isVisible()) {
                                    _this.popup.show();
                                }
                            }
                            else {
                                _this.validUserInput = false;
                                _this.popup.setSelectedTime(null, null);
                            }
                        }
                        _this.updateInputStyling();
                    });
                    var wrapper = new api.dom.DivEl('wrapper');
                    wrapper.appendChild(this.input);
                    var popupBuilder = new time.TimePickerPopupBuilder().setHours(builder.hours).setMinutes(builder.minutes).setCloseOnOutsideClick(builder.closeOnOutsideClick);
                    this.popup = popupBuilder.build();
                    wrapper.appendChild(this.popup);
                    this.popupTrigger = new api.ui.button.Button();
                    this.popupTrigger.addClass('icon-clock');
                    wrapper.appendChild(this.popupTrigger);
                    this.appendChild(wrapper);
                    this.popupTrigger.onClicked(function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                        if (_this.popup.isVisible()) {
                            _this.popup.hide();
                        }
                        else {
                            _this.popup.show();
                        }
                    });
                    if (builder.hours || builder.minutes) {
                        this.input.setValue(this.formatTime(builder.hours, builder.minutes));
                    }
                    this.popup.onSelectedTimeChanged(function (hours, minutes) {
                        if (hours != null && minutes != null) {
                            _this.input.setValue(_this.formatTime(hours, minutes));
                            _this.validUserInput = true;
                        }
                        _this.updateInputStyling();
                    });
                }
                TimePicker.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                TimePicker.prototype.getSelectedTime = function () {
                    return this.popup.getSelectedTime();
                };
                TimePicker.prototype.onSelectedTimeChanged = function (listener) {
                    this.popup.onSelectedTimeChanged(listener);
                };
                TimePicker.prototype.unSelectedTimeChanged = function (listener) {
                    this.popup.unSelectedTimeChanged(listener);
                };
                TimePicker.prototype.formatTime = function (hours, minutes) {
                    return this.padNumber(hours, 2) + ':' + this.padNumber(minutes, 2);
                };
                TimePicker.prototype.padNumber = function (value, pad) {
                    return Array(pad - String(value).length + 1).join('0') + value;
                };
                TimePicker.prototype.updateInputStyling = function () {
                    this.input.updateValidationStatusOnUserInput(this.validUserInput);
                };
                TimePicker.prototype.giveFocus = function () {
                    return this.input.giveFocus();
                };
                return TimePicker;
            })(api.dom.DivEl);
            time.TimePicker = TimePicker;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var TimePickerPopupBuilder = (function () {
                function TimePickerPopupBuilder() {
                    this.closeOnOutsideClick = true;
                }
                TimePickerPopupBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.getHours = function () {
                    return this.hours;
                };
                TimePickerPopupBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.getMinutes = function () {
                    return this.minutes;
                };
                TimePickerPopupBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.isCloseOnOutsideClick = function () {
                    return this.closeOnOutsideClick;
                };
                TimePickerPopupBuilder.prototype.build = function () {
                    return new TimePickerPopup(this);
                };
                return TimePickerPopupBuilder;
            })();
            time.TimePickerPopupBuilder = TimePickerPopupBuilder;
            var TimePickerPopup = (function (_super) {
                __extends(TimePickerPopup, _super);
                function TimePickerPopup(builder) {
                    var _this = this;
                    _super.call(this, 'time-picker-dialog');
                    this.timeChangedListeners = [];
                    var hourContainer = new api.dom.LiEl();
                    this.appendChild(hourContainer);
                    this.nextHour = new api.dom.AEl('next');
                    this.nextHour.onMouseDown(function (e) {
                        _this.addHour(+1);
                        _this.startInterval(_this.addHour, 1);
                    });
                    api.dom.Body.get().onMouseUp(function (e) {
                        _this.stopInterval();
                    });
                    this.nextHour.appendChild(new api.dom.SpanEl());
                    hourContainer.appendChild(this.nextHour);
                    this.hour = new api.dom.SpanEl();
                    hourContainer.appendChild(this.hour);
                    this.prevHour = new api.dom.AEl('prev');
                    this.prevHour.onMouseDown(function (e) {
                        _this.addHour(-1);
                        _this.startInterval(_this.addHour, -1);
                    });
                    this.prevHour.appendChild(new api.dom.SpanEl());
                    hourContainer.appendChild(this.prevHour);
                    this.appendChild(new api.dom.LiEl('colon'));
                    var minuteContainer = new api.dom.LiEl();
                    this.appendChild(minuteContainer);
                    this.nextMinute = new api.dom.AEl('next');
                    this.nextMinute.onMouseDown(function (e) {
                        _this.addMinute(+1);
                        _this.startInterval(_this.addMinute, 1);
                    });
                    this.nextMinute.appendChild(new api.dom.SpanEl());
                    minuteContainer.appendChild(this.nextMinute);
                    this.minute = new api.dom.SpanEl();
                    minuteContainer.appendChild(this.minute);
                    this.prevMinute = new api.dom.AEl('prev');
                    this.prevMinute.onMouseDown(function (e) {
                        _this.addMinute(-1);
                        _this.startInterval(_this.addMinute, -1);
                    });
                    this.prevMinute.appendChild(new api.dom.SpanEl());
                    minuteContainer.appendChild(this.prevMinute);
                    this.selectedHour = builder.getHours() || null;
                    this.selectedMinute = builder.getMinutes() || null;
                    this.hour.setHtml(this.padNumber(this.selectedHour || 0, 2));
                    this.minute.setHtml(this.padNumber(this.selectedMinute || 0, 2));
                    if (builder.isCloseOnOutsideClick()) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                TimePickerPopup.prototype.getSelectedTime = function () {
                    return this.selectedHour != null && this.selectedMinute != null ? {
                        hour: this.selectedHour,
                        minute: this.selectedMinute
                    } : null;
                };
                TimePickerPopup.prototype.onSelectedTimeChanged = function (listener) {
                    this.timeChangedListeners.push(listener);
                };
                TimePickerPopup.prototype.unSelectedTimeChanged = function (listener) {
                    this.timeChangedListeners = this.timeChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                TimePickerPopup.prototype.startInterval = function (fn) {
                    var _this = this;
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    var times = 0;
                    var delay = 400;
                    var intervalFn = function () {
                        fn.apply(_this, args);
                        if (++times % 5 == 0 && delay > 50) {
                            _this.stopInterval();
                            delay /= 2;
                            _this.interval = setInterval(intervalFn, delay);
                        }
                    };
                    this.interval = setInterval(intervalFn, delay);
                };
                TimePickerPopup.prototype.stopInterval = function () {
                    clearInterval(this.interval);
                };
                TimePickerPopup.prototype.addHour = function (add, silent) {
                    this.selectedHour += add;
                    if (this.selectedHour < 0) {
                        this.selectedHour += 24;
                    }
                    else if (this.selectedHour > 23) {
                        this.selectedHour -= 24;
                    }
                    this.setSelectedTime(this.selectedHour, this.selectedMinute || 0, silent);
                };
                TimePickerPopup.prototype.addMinute = function (add, silent) {
                    this.selectedMinute += add;
                    if (this.selectedMinute < 0) {
                        this.selectedMinute += 60;
                        this.addHour(-1, true);
                    }
                    else if (this.selectedMinute > 59) {
                        this.selectedMinute -= 60;
                        this.addHour(1, true);
                    }
                    this.setSelectedTime(this.selectedHour || 0, this.selectedMinute, silent);
                };
                TimePickerPopup.prototype.notifyTimeChanged = function (hours, minutes) {
                    this.timeChangedListeners.forEach(function (listener) {
                        listener(hours, minutes);
                    });
                };
                TimePickerPopup.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.hide();
                    }
                };
                TimePickerPopup.prototype.setSelectedTime = function (hours, minutes, silent) {
                    if (hours == null || hours < 0 || hours > 23 || minutes == null || minutes < 0 || minutes > 59) {
                        this.selectedHour = null;
                        this.selectedMinute = null;
                    }
                    else {
                        this.selectedHour = hours;
                        this.selectedMinute = minutes;
                    }
                    this.hour.setHtml(this.padNumber(this.selectedHour || 0, 2));
                    this.minute.setHtml(this.padNumber(this.selectedMinute || 0, 2));
                    if (!silent) {
                        this.notifyTimeChanged(this.selectedHour, this.selectedMinute);
                    }
                };
                TimePickerPopup.prototype.padNumber = function (value, pad) {
                    return Array(pad - String(value).length + 1).join('0') + value;
                };
                return TimePickerPopup;
            })(api.dom.UlEl);
            time.TimePickerPopup = TimePickerPopup;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DateTimePickerBuilder = (function () {
                function DateTimePickerBuilder() {
                    this.startingDayOfWeek = time.DaysOfWeek.MONDAY;
                    this.closeOnSelect = false;
                    this.closeOnOutsideClick = true;
                }
                DateTimePickerBuilder.prototype.setYear = function (value) {
                    this.year = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setSelectedDate = function (value) {
                    this.selectedDate = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setStartingDayOfWeek = function (value) {
                    this.startingDayOfWeek = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setCloseOnSelect = function (value) {
                    this.closeOnSelect = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.build = function () {
                    return new DateTimePicker(this);
                };
                return DateTimePickerBuilder;
            })();
            time.DateTimePickerBuilder = DateTimePickerBuilder;
            var DateTimePicker = (function (_super) {
                __extends(DateTimePicker, _super);
                function DateTimePicker(builder) {
                    var _this = this;
                    _super.call(this, 'date-time-picker');
                    this.selectedDateTimeChangedListeners = [];
                    this.validUserInput = true;
                    this.input = api.ui.text.TextInput.middle();
                    this.input.onClicked(function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                        _this.popup.show();
                    });
                    var wrapper = new api.dom.DivEl('wrapper');
                    wrapper.appendChild(this.input);
                    this.calendar = new time.CalendarBuilder().setSelectedDate(builder.selectedDate).setMonth(builder.month).setYear(builder.year).setInteractive(true).build();
                    var popupBuilder = new time.DateTimePickerPopupBuilder().setHours(builder.hours).setMinutes(builder.minutes).setCalendar(this.calendar).setCloseOnOutsideClick(builder.closeOnOutsideClick);
                    this.popup = new time.DateTimePickerPopup(popupBuilder);
                    wrapper.appendChild(this.popup);
                    this.popupTrigger = new api.ui.button.Button();
                    this.popupTrigger.addClass('icon-calendar4');
                    wrapper.appendChild(this.popupTrigger);
                    this.appendChild(wrapper);
                    this.popupTrigger.onClicked(function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                        if (_this.popup.isVisible()) {
                            _this.popup.hide();
                        }
                        else {
                            _this.popup.show();
                        }
                    });
                    if (builder.selectedDate) {
                        this.input.setValue(this.formatDate(builder.selectedDate));
                        this.selectedDate = builder.selectedDate;
                    }
                    if (builder.hours || builder.minutes) {
                        var value = this.input.getValue() || "";
                        this.setTime(builder.hours, builder.minutes);
                        this.input.setValue(value + " " + this.formatTime(builder.hours, builder.minutes));
                    }
                    this.popup.onSelectedDateChanged(function (e) {
                        if (builder.closeOnSelect) {
                            _this.popup.hide();
                        }
                        _this.setDate(e.getDate());
                        _this.input.setValue(_this.formatDate(_this.selectedDate) + " " + _this.formatTime(_this.selectedDate.getUTCHours(), _this.selectedDate.getUTCMinutes()));
                        _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(_this.selectedDate));
                        _this.validUserInput = true;
                        _this.updateInputStyling();
                    });
                    this.popup.onSelectedTimeChanged(function (hours, minutes) {
                        _this.setTime(hours, minutes);
                        _this.input.setValue(_this.formatDate(_this.selectedDate) + " " + _this.formatTime(hours, minutes));
                        _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(_this.selectedDate));
                        _this.validUserInput = true;
                        _this.updateInputStyling();
                    });
                    this.input.onKeyUp(function (event) {
                        var typedDateTime = _this.input.getValue();
                        if (api.util.StringHelper.isEmpty(typedDateTime)) {
                            _this.calendar.selectDate(null);
                            _this.selectedDate = null;
                            _this.validUserInput = true;
                            _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(null));
                            _this.popup.hide();
                        }
                        else {
                            var date = api.util.DateHelper.parseUTCDateTime(typedDateTime);
                            var dateLength = date.getUTCFullYear().toString().length + 12;
                            if (date && date.toString() != "Invalid Date" && typedDateTime.length == dateLength) {
                                _this.selectedDate = date;
                                _this.validUserInput = true;
                                _this.calendar.selectDate(date);
                                _this.popup.setSelectedTime(date.getUTCHours(), date.getUTCMinutes());
                                _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(date));
                                if (!_this.popup.isVisible()) {
                                    _this.popup.show();
                                }
                            }
                            else {
                                _this.selectedDate = null;
                                _this.validUserInput = false;
                                _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(null));
                            }
                        }
                        _this.updateInputStyling();
                    });
                }
                DateTimePicker.prototype.setTime = function (hours, minutes) {
                    if (!this.selectedDate) {
                        var today = new Date();
                        this.selectedDate = api.util.DateHelper.newUTCDate(today.getUTCFullYear(), today.getUTCMonth(), today.getUTCDate());
                    }
                    this.selectedDate.setUTCHours(hours);
                    this.selectedDate.setUTCMinutes(minutes);
                };
                DateTimePicker.prototype.setDate = function (date) {
                    var hours = this.selectedDate ? this.selectedDate.getUTCHours() : 0, minutes = this.selectedDate ? this.selectedDate.getUTCMinutes() : 0;
                    this.selectedDate = date;
                    if (hours || minutes) {
                        this.setTime(hours, minutes);
                    }
                };
                DateTimePicker.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                DateTimePicker.prototype.getSelectedDateTime = function () {
                    return this.selectedDate;
                };
                DateTimePicker.prototype.onSelectedDateTimeChanged = function (listener) {
                    this.selectedDateTimeChangedListeners.push(listener);
                };
                DateTimePicker.prototype.unSelectedDateTimeChanged = function (listener) {
                    this.selectedDateTimeChangedListeners = this.selectedDateTimeChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                DateTimePicker.prototype.notifySelectedDateTimeChanged = function (event) {
                    this.selectedDateTimeChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                DateTimePicker.prototype.formatDate = function (date) {
                    return api.util.DateHelper.formatUTCDate(date);
                };
                DateTimePicker.prototype.formatTime = function (hours, minutes) {
                    return this.padNumber(hours, 2) + ':' + this.padNumber(minutes, 2);
                };
                DateTimePicker.prototype.padNumber = function (value, pad) {
                    return Array(pad - String(value).length + 1).join('0') + value;
                };
                DateTimePicker.prototype.updateInputStyling = function () {
                    this.input.updateValidationStatusOnUserInput(this.validUserInput);
                };
                DateTimePicker.prototype.giveFocus = function () {
                    return this.input.giveFocus();
                };
                return DateTimePicker;
            })(api.dom.DivEl);
            time.DateTimePicker = DateTimePicker;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (_time) {
            var DateTimePickerPopupBuilder = (function () {
                function DateTimePickerPopupBuilder() {
                    this.closeOnOutsideClick = true;
                }
                DateTimePickerPopupBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.getHours = function () {
                    return this.hours;
                };
                DateTimePickerPopupBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.getMinutes = function () {
                    return this.minutes;
                };
                DateTimePickerPopupBuilder.prototype.setCalendar = function (value) {
                    this.calendar = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.getCalendar = function () {
                    return this.calendar;
                };
                DateTimePickerPopupBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.isCloseOnOutsideClick = function () {
                    return this.closeOnOutsideClick;
                };
                DateTimePickerPopupBuilder.prototype.build = function () {
                    return new DateTimePickerPopup(this);
                };
                return DateTimePickerPopupBuilder;
            })();
            _time.DateTimePickerPopupBuilder = DateTimePickerPopupBuilder;
            var DateTimePickerPopup = (function (_super) {
                __extends(DateTimePickerPopup, _super);
                function DateTimePickerPopup(builder) {
                    var _this = this;
                    _super.call(this, 'date-time-dialog');
                    var closeOnOutsideClick = builder.isCloseOnOutsideClick();
                    builder.setCloseOnOutsideClick(false);
                    this.datePickerPopup = new _time.DatePickerPopupBuilder().setCalendar(builder.getCalendar()).setCloseOnOutsideClick(false).build();
                    this.timePickerPopup = new _time.TimePickerPopupBuilder().setHours(builder.getHours()).setCloseOnOutsideClick(false).setMinutes(builder.getMinutes()).build();
                    this.appendChildren(this.datePickerPopup, this.timePickerPopup);
                    if (closeOnOutsideClick) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                DateTimePickerPopup.prototype.getSelectedDate = function () {
                    return this.datePickerPopup.getCalendar().getSelectedDate();
                };
                DateTimePickerPopup.prototype.onSelectedDateChanged = function (listener) {
                    this.datePickerPopup.onSelectedDateChanged(listener);
                };
                DateTimePickerPopup.prototype.unSelectedDateChanged = function (listener) {
                    this.datePickerPopup.unSelectedDateChanged(listener);
                };
                DateTimePickerPopup.prototype.getSelectedTime = function () {
                    return this.timePickerPopup.getSelectedTime();
                };
                DateTimePickerPopup.prototype.onSelectedTimeChanged = function (listener) {
                    this.timePickerPopup.onSelectedTimeChanged(listener);
                };
                DateTimePickerPopup.prototype.unSelectedTimeChanged = function (listener) {
                    this.timePickerPopup.unSelectedTimeChanged(listener);
                };
                DateTimePickerPopup.prototype.getSelectedDateTime = function () {
                    var date = this.getSelectedDate();
                    var time = this.getSelectedTime();
                    if (!date || !time) {
                        return null;
                    }
                    return api.util.DateHelper.newUTCDateTime(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), time.hour, time.minute);
                };
                DateTimePickerPopup.prototype.setSelectedTime = function (hours, minutes, silent) {
                    this.timePickerPopup.setSelectedTime(hours, minutes, silent);
                };
                DateTimePickerPopup.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.hide();
                    }
                };
                return DateTimePickerPopup;
            })(api.dom.DivEl);
            _time.DateTimePickerPopup = DateTimePickerPopup;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var LocalTime = (function (_super) {
                __extends(LocalTime, _super);
                function LocalTime(timeBuilder) {
                    _super.call(this, "date-picker");
                    if (!timeBuilder) {
                        timeBuilder = new api.ui.time.TimePickerBuilder();
                    }
                    this.timePicker = timeBuilder.build();
                    this.layoutItems();
                }
                LocalTime.prototype.layoutItems = function () {
                    this.removeChildren();
                    this.appendChild(this.timePicker);
                    return this;
                };
                LocalTime.prototype.onTimeChanged = function (listener) {
                    this.timePicker.onSelectedTimeChanged(listener);
                };
                LocalTime.prototype.setTime = function (value) {
                    var localTime = value.getAdjustedTime();
                    this.timePicker = new api.ui.time.TimePickerBuilder().setHours(localTime.hour).setMinutes(localTime.minute).build();
                    this.layoutItems();
                    return this;
                };
                LocalTime.prototype.giveFocus = function () {
                    return this.timePicker.giveFocus();
                };
                LocalTime.prototype.getTimePicker = function () {
                    return this.timePicker;
                };
                return LocalTime;
            })(api.dom.DivEl);
            time.LocalTime = LocalTime;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var actions;
            (function (actions) {
                var Action = api.ui.Action;
                var SelectAllAction = (function (_super) {
                    __extends(SelectAllAction, _super);
                    function SelectAllAction(grid) {
                        var allCount = grid.getDataView().getLength();
                        _super.call(this, "Select All (" + allCount + ")");
                        this.setEnabled(true);
                        this.onExecuted(function () {
                            grid.selectAll();
                        });
                    }
                    return SelectAllAction;
                })(Action);
                actions.SelectAllAction = SelectAllAction;
            })(actions = treegrid.actions || (treegrid.actions = {}));
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var actions;
            (function (actions) {
                var Action = api.ui.Action;
                var ClearSelectionAction = (function (_super) {
                    __extends(ClearSelectionAction, _super);
                    function ClearSelectionAction(grid) {
                        var selectionCount = grid.getSelectedRows().length;
                        _super.call(this, "Clear Selection (" + selectionCount + ")");
                        this.setEnabled(true);
                        this.onExecuted(function () {
                            grid.clearSelection();
                        });
                    }
                    return ClearSelectionAction;
                })(Action);
                actions.ClearSelectionAction = ClearSelectionAction;
            })(actions = treegrid.actions || (treegrid.actions = {}));
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var actions;
            (function (actions) {
                var TreeGridToolbarActions = (function () {
                    function TreeGridToolbarActions(grid) {
                        this.actions = [];
                        this.SELECT_ALL = new actions.SelectAllAction(grid);
                        this.CLEAR_SELECTION = new actions.ClearSelectionAction(grid);
                        this.actions.push(this.SELECT_ALL, this.CLEAR_SELECTION);
                    }
                    TreeGridToolbarActions.prototype.getAllActions = function () {
                        return this.actions;
                    };
                    TreeGridToolbarActions.prototype.updateActionsEnabledState = function (browseItems) {
                        var deferred = wemQ.defer();
                        deferred.resolve(browseItems);
                        return deferred.promise;
                    };
                    return TreeGridToolbarActions;
                })();
                actions.TreeGridToolbarActions = TreeGridToolbarActions;
            })(actions = treegrid.actions || (treegrid.actions = {}));
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var Element = api.dom.Element;
            var ElementHelper = api.dom.ElementHelper;
            var Grid = api.ui.grid.Grid;
            var DataView = api.ui.grid.DataView;
            var KeyBinding = api.ui.KeyBinding;
            var KeyBindings = api.ui.KeyBindings;
            var TreeGridToolbarActions = api.ui.treegrid.actions.TreeGridToolbarActions;
            var TreeGrid = (function (_super) {
                __extends(TreeGrid, _super);
                function TreeGrid(builder) {
                    var _this = this;
                    _super.call(this, builder.getClasses());
                    this.columns = [];
                    this.loadedListeners = [];
                    this.contextMenuListeners = [];
                    this.selectionChangeListeners = [];
                    this.dataChangeListeners = [];
                    this.root = new treegrid.TreeRoot();
                    this.gridData = new DataView();
                    this.gridData.setFilter(function (node) {
                        return node.isVisible();
                    });
                    this.columns = this.updateColumnsFormatter(builder.getColumns());
                    this.gridOptions = builder.getOptions();
                    this.gridOptions.setForceFitColumns(true);
                    this.gridOptions.setAutoHeight(false);
                    this.grid = new Grid(this.gridData, this.columns, this.gridOptions);
                    this.grid.setSelectionModel(new Slick.RowSelectionModel({
                        selectActiveRow: false
                    }));
                    var selectorPlugin = this.grid.getCheckboxSelectorPlugin();
                    if (selectorPlugin) {
                        this.grid.unregisterPlugin(this.grid.getCheckboxSelectorPlugin());
                    }
                    this.grid.syncGridSelection(false);
                    this.actions = new TreeGridToolbarActions(this.grid);
                    this.onClicked(function () {
                        _this.grid.focus();
                    });
                    if (builder.getContextMenu()) {
                        this.contextMenu = builder.getContextMenu();
                        this.grid.subscribeOnContextMenu(function (event) {
                            event.preventDefault();
                            _this.active = false;
                            var cell = _this.grid.getCellFromEvent(event);
                            _this.grid.selectRow(cell.row);
                            _this.contextMenu.showAt(event.pageX, event.pageY);
                            _this.notifyContextMenuShown(event.pageX, event.pageY);
                            _this.active = true;
                        });
                    }
                    this.grid.subscribeOnClick(function (event, data) {
                        if (_this.active) {
                            _this.active = false;
                            var elem = new ElementHelper(event.target);
                            if (elem.hasClass("expand")) {
                                elem.removeClass("expand").addClass("collapse");
                                var node = _this.gridData.getItem(data.row);
                                _this.expandNode(node);
                            }
                            else if (elem.hasClass("collapse")) {
                                _this.active = false;
                                elem.removeClass("collapse").addClass("expand");
                                var node = _this.gridData.getItem(data.row);
                                _this.collapseNode(node);
                            }
                            else if (data.cell === 0) {
                                _this.active = true;
                                if (elem.getAttribute("type") === "checkbox") {
                                    _this.grid.toggleRow(data.row);
                                }
                            }
                            else {
                                _this.active = true;
                                _this.root.clearStashedSelection();
                                _this.grid.selectRow(data.row);
                            }
                        }
                        if (_this.contextMenu) {
                            _this.contextMenu.hide();
                        }
                        event.stopPropagation();
                    });
                    if (builder.isShowToolbar()) {
                        this.toolbar = new treegrid.TreeGridToolbar(this.actions, this);
                        this.appendChild(this.toolbar);
                        this.removeClass("no-toolbar");
                    }
                    else {
                        this.addClass("no-toolbar");
                    }
                    this.appendChild(this.grid);
                    if (builder.isAutoLoad()) {
                        this.reload();
                    }
                    if (builder.isPartialLoadEnabled()) {
                        this.loadBufferSize = builder.getLoadBufferSize();
                        var interval;
                        this.onShown(function () {
                            if (interval) {
                                clearInterval(interval);
                            }
                            interval = setInterval(_this.postLoad.bind(_this), 200);
                        });
                    }
                    var keyBindings = [];
                    this.onShown(function () {
                        _this.grid.resizeCanvas();
                        if (builder.isHotkeysEnabled()) {
                            if (!_this.gridOptions.isMultipleSelectionDisabled()) {
                                keyBindings = [
                                    new KeyBinding('shift+up', function (event) {
                                        if (_this.active) {
                                            _this.scrollToRow(_this.grid.addSelectedUp());
                                        }
                                        event.preventDefault();
                                        event.stopImmediatePropagation();
                                    }),
                                    new KeyBinding('shift+down', function (event) {
                                        if (_this.active) {
                                            _this.scrollToRow(_this.grid.addSelectedDown());
                                        }
                                        event.preventDefault();
                                        event.stopImmediatePropagation();
                                    })
                                ];
                            }
                            keyBindings = keyBindings.concat([
                                new KeyBinding('up', function () {
                                    if (_this.active) {
                                        _this.scrollToRow(_this.grid.moveSelectedUp());
                                    }
                                }),
                                new KeyBinding('down', function () {
                                    if (_this.active) {
                                        _this.scrollToRow(_this.grid.moveSelectedDown());
                                    }
                                }),
                                new KeyBinding('left', function () {
                                    var selected = _this.grid.getSelectedRows();
                                    if (selected.length === 1) {
                                        var node = _this.gridData.getItem(selected[0]);
                                        if (node && _this.active) {
                                            if (node.isExpanded()) {
                                                _this.active = false;
                                                _this.collapseNode(node);
                                            }
                                            else if (node.getParent() !== _this.root.getCurrentRoot()) {
                                                node = node.getParent();
                                                _this.active = false;
                                                var row = _this.gridData.getRowById(node.getId());
                                                _this.grid.selectRow(row);
                                                _this.collapseNode(node);
                                            }
                                        }
                                    }
                                }),
                                new KeyBinding('right', function () {
                                    var selected = _this.grid.getSelectedRows();
                                    if (selected.length === 1) {
                                        var node = _this.gridData.getItem(selected[0]);
                                        if (node && _this.hasChildren(node.getData()) && !node.isExpanded() && _this.active) {
                                            _this.active = false;
                                            _this.resetAndRender();
                                            _this.expandNode(node);
                                        }
                                    }
                                }),
                                new KeyBinding('space', function () {
                                    _this.deselectAll();
                                })
                            ]);
                            KeyBindings.get().bindKeys(keyBindings);
                        }
                    });
                    this.onRendered(function () {
                        _this.grid.resizeCanvas();
                    });
                    this.onRemoved(function () {
                        if (builder.isHotkeysEnabled()) {
                            KeyBindings.get().unbindKeys(keyBindings);
                        }
                        if (builder.isPartialLoadEnabled() && interval) {
                            clearInterval(interval);
                        }
                    });
                    this.grid.subscribeOnSelectedRowsChanged(function (event, rows) {
                        _this.notifySelectionChanged(event, rows.rows);
                    });
                    if (this.toolbar) {
                        this.gridData.onRowCountChanged(function () {
                            _this.toolbar.refresh();
                        });
                        this.onSelectionChanged(function () {
                            _this.toolbar.refresh();
                        });
                    }
                }
                TreeGrid.prototype.updateColumnsFormatter = function (columns) {
                    var _this = this;
                    if (columns.length > 0) {
                        var formatter = columns[0].getFormatter();
                        var toggleFormatter = function (row, cell, value, columnDef, node) {
                            var toggleSpan = new api.dom.SpanEl("toggle icon");
                            if (_this.hasChildren(node.getData())) {
                                var toggleClass = node.isExpanded() ? "collapse" : "expand";
                                toggleSpan.addClass(toggleClass);
                            }
                            toggleSpan.getEl().setMarginLeft(16 * (node.calcLevel() - 1) + "px");
                            return toggleSpan.toString() + formatter(row, cell, value, columnDef, node);
                        };
                        columns[0].setFormatter(toggleFormatter);
                    }
                    return columns;
                };
                TreeGrid.prototype.getGrid = function () {
                    return this.grid;
                };
                TreeGrid.prototype.getOptions = function () {
                    return this.gridOptions;
                };
                TreeGrid.prototype.getColumns = function () {
                    return this.grid.getColumns();
                };
                TreeGrid.prototype.getContextMenu = function () {
                    return this.contextMenu;
                };
                TreeGrid.prototype.getRoot = function () {
                    return this.root;
                };
                TreeGrid.prototype.isActive = function () {
                    return this.active;
                };
                TreeGrid.prototype.setActive = function (active) {
                    if (active === void 0) { active = true; }
                    this.active = active;
                };
                TreeGrid.prototype.hasToolbar = function () {
                    return !!this.toolbar;
                };
                TreeGrid.prototype.scrollToRow = function (row) {
                    var gridClasses = (" " + this.grid.getEl().getClass()).replace(/\s/g, ".");
                    var canvas = Element.fromString(".tree-grid " + gridClasses + " .grid-canvas", false);
                    var viewport = Element.fromString(".tree-grid " + gridClasses + " .slick-viewport", false);
                    var scrollEl = viewport.getEl();
                    if (row > -1 && this.grid.getSelectedRows().length > 0) {
                        if (scrollEl.getScrollTop() > row * 45) {
                            scrollEl.setScrollTop(row * 45);
                        }
                        else if (scrollEl.getScrollTop() + scrollEl.getHeight() < (row + 1) * 45) {
                            scrollEl.setScrollTop((row + 1) * 45 - scrollEl.getHeight());
                        }
                    }
                };
                TreeGrid.prototype.loadEmptyNode = function (node) {
                    var _this = this;
                    if (!this.getDataId(node.getData())) {
                        this.fetchChildren(node.getParent()).then(function (dataList) {
                            var oldChildren = node.getParent().getChildren();
                            if (oldChildren[oldChildren.length - 1].getDataId() === "") {
                                oldChildren.pop();
                            }
                            var fetchedChildren = _this.dataToTreeNodes(dataList, node.getParent());
                            var newChildren = oldChildren.concat(fetchedChildren.slice(oldChildren.length));
                            node.getParent().setChildren(newChildren);
                            _this.initData(_this.root.getCurrentRoot().treeToList());
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).finally(function () {
                        }).done(function () { return _this.notifyLoaded(); });
                    }
                };
                TreeGrid.prototype.postLoad = function () {
                    var iFrame = api.app.Application.getApplication().getAppFrame(), disabled = !iFrame.isVisible() || !this.isVisible() || !this.isActive();
                    if (disabled)
                        return;
                    var viewportRange = this.grid.getViewport(), lastIndex = this.gridData.getItems().length - 1, firstVisible = viewportRange.top, lastVisible = Math.min(viewportRange.bottom, lastIndex), from = firstVisible, to = Math.min(lastVisible + this.loadBufferSize, lastIndex);
                    for (var i = from; i <= to; i++) {
                        if (!!this.gridData.getItem(i) && this.gridData.getItem(i).getDataId() === "") {
                            this.loadEmptyNode(this.gridData.getItem(i));
                            break;
                        }
                    }
                };
                TreeGrid.prototype.hasChildren = function (data) {
                    return false;
                };
                TreeGrid.prototype.getDataId = function (data) {
                    throw new Error("Must be implemented by inheritors");
                };
                TreeGrid.prototype.fetch = function (node) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                TreeGrid.prototype.fetchChildren = function (parentNode) {
                    var deferred = wemQ.defer();
                    deferred.resolve([]);
                    return deferred.promise;
                };
                TreeGrid.prototype.fetchRoot = function () {
                    return this.fetchChildren();
                };
                TreeGrid.prototype.fetchData = function (parentNode) {
                    return parentNode ? this.fetchChildren(parentNode) : this.fetchRoot();
                };
                TreeGrid.prototype.dataToTreeNode = function (data, parent) {
                    return new treegrid.TreeNodeBuilder().setData(data, this.getDataId(data)).setParent(parent).build();
                };
                TreeGrid.prototype.dataToTreeNodes = function (dataArray, parent) {
                    var _this = this;
                    var nodes = [];
                    dataArray.forEach(function (data) {
                        nodes.push(_this.dataToTreeNode(data, parent));
                    });
                    return nodes;
                };
                TreeGrid.prototype.filter = function (dataList) {
                    this.active = false;
                    this.root.setFiltered(true);
                    this.root.getCurrentRoot().setChildren(this.dataToTreeNodes(dataList, this.root.getCurrentRoot()));
                    this.initData(this.root.getCurrentRoot().treeToList());
                    this.resetAndRender();
                    this.active = true;
                };
                TreeGrid.prototype.resetFilter = function () {
                    this.active = false;
                    if (this.root.isFiltered()) {
                        this.root.setFiltered(false);
                        this.initData(this.root.getCurrentRoot().treeToList());
                        this.resetAndRender();
                        this.active = true;
                        this.notifyLoaded();
                    }
                    else {
                        this.reload();
                    }
                    this.stash = null;
                };
                TreeGrid.prototype.selectAll = function () {
                    this.grid.selectAll();
                };
                TreeGrid.prototype.deselectAll = function () {
                    this.grid.clearSelection();
                };
                TreeGrid.prototype.deselectNode = function (dataId) {
                    var oldSelected = this.grid.getSelectedRows(), newSelected = [];
                    for (var i = 0; i < oldSelected.length; i++) {
                        if (dataId !== this.gridData.getItem(oldSelected[i]).getDataId()) {
                            newSelected.push(oldSelected[i]);
                        }
                    }
                    this.root.removeSelection(dataId);
                    this.triggerSelectionChangedListeners();
                    if (oldSelected.length !== newSelected.length) {
                        this.grid.setSelectedRows(newSelected);
                    }
                };
                TreeGrid.prototype.getSelectedNodes = function () {
                    return this.grid.getSelectedRowItems();
                };
                TreeGrid.prototype.getSelectedDataList = function () {
                    return this.root.getFullSelection().map(function (node) {
                        return node.getData();
                    });
                };
                TreeGrid.prototype.reload = function (parentNodeData) {
                    var _this = this;
                    this.root.resetCurrentRoot(parentNodeData);
                    this.initData([]);
                    this.fetchData().then(function (dataList) {
                        _this.root.getCurrentRoot().setChildren(_this.dataToTreeNodes(dataList, _this.root.getCurrentRoot()));
                        _this.initData(_this.root.getCurrentRoot().treeToList());
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).finally(function () {
                        _this.resetAndRender();
                        _this.active = true;
                    }).done(function () { return _this.notifyLoaded(); });
                };
                TreeGrid.prototype.refreshNode = function (node) {
                    var root = this.root.getCurrentRoot();
                    this.active = false;
                    node = node || root;
                    node.regenerateIds();
                    root.setExpanded(true);
                    this.initData(root.treeToList());
                    this.resetAndRender();
                    this.active = true;
                    this.notifyLoaded();
                };
                TreeGrid.prototype.refresh = function () {
                    var root = this.root.getCurrentRoot();
                    this.active = false;
                    this.grid.invalidate();
                    root.setExpanded(true);
                    this.initData(root.treeToList());
                    this.resetAndRender();
                    this.active = true;
                    this.notifyLoaded();
                };
                TreeGrid.prototype.updateNode = function (data, oldDataId) {
                    var _this = this;
                    var dataId = oldDataId || this.getDataId(data), nodesToUpdate = [], nodeToUpdate = this.root.getCurrentRoot().findNode(dataId), stashedNodeToUpdate;
                    if (!nodeToUpdate) {
                        throw new Error("TreeNode to update not found: " + dataId);
                    }
                    nodesToUpdate.push(nodeToUpdate);
                    if (!!this.stash) {
                        stashedNodeToUpdate = this.stash.findNode(dataId);
                        this.root.getCurrentRoot().getChildren().forEach(function (topNode) {
                            var match = topNode.findNode(dataId);
                            if (!!match) {
                                nodesToUpdate.push(match);
                            }
                        });
                    }
                    this.fetch(nodesToUpdate[0]).then(function (data) {
                        nodesToUpdate.forEach(function (node) {
                            node.setData(data);
                            node.setDataId(_this.getDataId(data));
                            node.clearViewers();
                            if (node.isVisible()) {
                                var selected = _this.grid.isRowSelected(_this.gridData.getRowById(node.getId()));
                                _this.gridData.updateItem(node.getId(), node);
                                if (selected) {
                                    _this.grid.addSelectedRow(_this.gridData.getRowById(node.getId()));
                                }
                            }
                        });
                        if (!!stashedNodeToUpdate) {
                            stashedNodeToUpdate.setData(data);
                            stashedNodeToUpdate.clearViewers();
                        }
                        _this.notifyDataChanged(new treegrid.DataChangedEvent(nodesToUpdate, treegrid.DataChangedEvent.UPDATED));
                        _this.initData(_this.root.getCurrentRoot().treeToList());
                        _this.root.updateSelection(dataId, data);
                        _this.triggerSelectionChangedListeners();
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    });
                };
                TreeGrid.prototype.deleteNode = function (data, stashedParentNode) {
                    var _this = this;
                    if (!stashedParentNode && this.stash) {
                        this.deleteNode(data, this.stash);
                    }
                    var root = stashedParentNode || this.root.getCurrentRoot(), dataId = this.getDataId(data), node;
                    while (node = root.findNode(dataId)) {
                        if (node.hasChildren()) {
                            node.getChildren().forEach(function (child) {
                                _this.deleteNode(child.getData());
                            });
                        }
                        if (this.gridData.getItemById(node.getId())) {
                            this.gridData.deleteItem(node.getId());
                        }
                        var parent = node.getParent();
                        if (node && parent) {
                            parent.removeChild(node);
                            parent.setMaxChildren(parent.getMaxChildren() - 1);
                            this.notifyDataChanged(new treegrid.DataChangedEvent([node], treegrid.DataChangedEvent.DELETED));
                        }
                    }
                    this.root.removeSelection(dataId);
                };
                TreeGrid.prototype.appendNode = function (data, nextToSelection, prepend, stashedParentNode) {
                    var _this = this;
                    if (prepend === void 0) { prepend = true; }
                    var deferred = wemQ.defer();
                    if (!stashedParentNode && this.stash) {
                        this.appendNode(data, nextToSelection, prepend, this.stash);
                    }
                    var root = stashedParentNode || this.root.getCurrentRoot();
                    var parentNode;
                    if (this.getSelectedNodes() && this.getSelectedNodes().length == 1) {
                        parentNode = root.findNode(this.getSelectedNodes()[0].getDataId());
                        if (nextToSelection) {
                            parentNode = parentNode.getParent() || this.root.getCurrentRoot();
                        }
                    }
                    else {
                        parentNode = root;
                    }
                    var isRootParentNode = (parentNode == root);
                    if (!parentNode.hasChildren() && !isRootParentNode) {
                        this.fetchData(parentNode).then(function (dataList) {
                            if (parentNode.hasChildren()) {
                                _this.doAppendNodeToParentWithChildren(parentNode, data, root, prepend, stashedParentNode, isRootParentNode);
                            }
                            else {
                                parentNode.setChildren(_this.dataToTreeNodes(dataList, parentNode));
                                _this.initData(root.treeToList());
                                var node = root.findNode(_this.getDataId(data));
                                if (!node) {
                                    parentNode.addChild(_this.dataToTreeNode(data, root), prepend);
                                    node = root.findNode(_this.getDataId(data));
                                }
                                if (node) {
                                    if (!stashedParentNode) {
                                        _this.gridData.setItems(root.treeToList());
                                    }
                                    _this.notifyDataChanged(new treegrid.DataChangedEvent([node], treegrid.DataChangedEvent.ADDED));
                                    if (parentNode != root) {
                                        _this.refreshNodeData(parentNode).then(function (node) {
                                            if (!stashedParentNode) {
                                                _this.updateSelectedNode(node);
                                            }
                                        });
                                    }
                                }
                            }
                            deferred.resolve(null);
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                            deferred.reject(reason);
                        });
                    }
                    else {
                        this.doAppendNodeToParentWithChildren(parentNode, data, root, prepend, stashedParentNode, isRootParentNode);
                        deferred.resolve(null);
                    }
                    return deferred.promise;
                };
                TreeGrid.prototype.doAppendNodeToParentWithChildren = function (parentNode, data, root, prepend, stashedParentNode, isRootParentNode) {
                    parentNode.addChild(this.dataToTreeNode(data, root), prepend);
                    var node = root.findNode(this.getDataId(data));
                    if (node) {
                        if (!stashedParentNode) {
                            this.gridData.setItems(root.treeToList());
                        }
                        if (isRootParentNode) {
                            this.sortNodeChildren(parentNode);
                        }
                        else {
                            if (!stashedParentNode) {
                                this.updateSelectedNode(parentNode);
                            }
                        }
                    }
                };
                TreeGrid.prototype.deleteNodes = function (dataList) {
                    var _this = this;
                    var root = this.root.getCurrentRoot(), updated = [], deleted = [];
                    dataList.forEach(function (data) {
                        var node = root.findNode(_this.getDataId(data));
                        if (node && node.getParent()) {
                            var parent = node.getParent();
                            _this.deleteNode(node.getData());
                            updated.push(parent);
                            deleted.push(node);
                            updated.filter(function (el) {
                                return el.getDataId() !== node.getDataId();
                            });
                        }
                    });
                    root.treeToList().forEach(function (child) {
                        _this.refreshNodeData(child);
                    });
                    if (this.stash) {
                        this.stash.treeToList().forEach(function (child) {
                            _this.refreshNodeData(child);
                        });
                    }
                    this.notifyDataChanged(new treegrid.DataChangedEvent(deleted, treegrid.DataChangedEvent.DELETED));
                };
                TreeGrid.prototype.initData = function (nodes) {
                    var selection = [], selectionIds = this.root.getFullSelection().map(function (el) {
                        return el.getDataId();
                    });
                    this.gridData.setItems(nodes, "id");
                    this.notifyDataChanged(new treegrid.DataChangedEvent(nodes, treegrid.DataChangedEvent.ADDED));
                    selectionIds.forEach(function (selectionId) {
                        nodes.forEach(function (node, index) {
                            if (node.getDataId() === selectionId) {
                                selection.push(index);
                            }
                        });
                    });
                    this.grid.setSelectedRows(selection);
                };
                TreeGrid.prototype.expandNode = function (node) {
                    var _this = this;
                    node = node || this.root.getCurrentRoot();
                    if (node) {
                        node.setExpanded(true);
                        if (node.hasChildren()) {
                            this.initData(this.root.getCurrentRoot().treeToList());
                            this.updateExpanded();
                        }
                        else {
                            this.fetchData(node).then(function (dataList) {
                                node.setChildren(_this.dataToTreeNodes(dataList, node));
                                _this.initData(_this.root.getCurrentRoot().treeToList());
                                _this.updateExpanded();
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).finally(function () {
                            }).done(function () { return _this.notifyLoaded(); });
                        }
                    }
                };
                TreeGrid.prototype.updateExpanded = function () {
                    this.resetAndRender();
                    this.active = true;
                };
                TreeGrid.prototype.updateSelectedNode = function (node) {
                    this.getGrid().clearSelection();
                    this.refreshNode(node);
                    var row = this.gridData.getRowById(node.getId());
                    this.grid.selectRow(row);
                };
                TreeGrid.prototype.collapseNode = function (node) {
                    node.setExpanded(false);
                    this.root.stashSelection();
                    this.gridData.refresh();
                    this.resetAndRender();
                    this.triggerSelectionChangedListeners();
                    this.active = true;
                };
                TreeGrid.prototype.notifyLoaded = function () {
                    var _this = this;
                    this.loadedListeners.forEach(function (listener) {
                        listener(_this);
                    });
                };
                TreeGrid.prototype.onLoaded = function (listener) {
                    this.loadedListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unLoaded = function (listener) {
                    this.loadedListeners = this.loadedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.notifySelectionChanged = function (event, rows) {
                    var _this = this;
                    var currentSelection = [];
                    if (rows) {
                        rows.forEach(function (rowIndex) {
                            currentSelection.push(_this.gridData.getItem(rowIndex));
                        });
                    }
                    this.root.setCurrentSelection(currentSelection);
                    this.triggerSelectionChangedListeners();
                };
                TreeGrid.prototype.triggerSelectionChangedListeners = function () {
                    for (var i in this.selectionChangeListeners) {
                        this.selectionChangeListeners[i](this.root.getCurrentSelection(), this.root.getFullSelection());
                    }
                };
                TreeGrid.prototype.onSelectionChanged = function (listener) {
                    this.selectionChangeListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unSelectionChanged = function (listener) {
                    this.selectionChangeListeners = this.selectionChangeListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.notifyContextMenuShown = function (x, y) {
                    var showContextMenuEvent = new treegrid.ContextMenuShownEvent(x, y);
                    this.contextMenuListeners.forEach(function (listener) {
                        listener(showContextMenuEvent);
                    });
                };
                TreeGrid.prototype.onContextMenuShown = function (listener) {
                    this.contextMenuListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unContextMenuShown = function (listener) {
                    this.contextMenuListeners = this.contextMenuListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.notifyDataChanged = function (event) {
                    this.dataChangeListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                TreeGrid.prototype.onDataChanged = function (listener) {
                    this.dataChangeListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unDataChanged = function (listener) {
                    this.dataChangeListeners = this.dataChangeListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.isFiltered = function () {
                    return !!this.stash;
                };
                TreeGrid.prototype.resetAndRender = function () {
                    this.grid.invalidate();
                    this.grid.renderGrid();
                };
                TreeGrid.prototype.initAndRender = function () {
                    this.initData(this.getRoot().getCurrentRoot().treeToList());
                    this.resetAndRender();
                };
                TreeGrid.prototype.refreshNodeData = function (parentNode) {
                    return null;
                };
                TreeGrid.prototype.sortNodeChildren = function (node) {
                };
                return TreeGrid;
            })(api.ui.panel.Panel);
            treegrid.TreeGrid = TreeGrid;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeNode = (function () {
                function TreeNode(builder) {
                    this.viewersByName = {};
                    this.id = Math.random().toString(36).substring(2);
                    this.dataId = builder.getDataId();
                    this.data = builder.getData();
                    this.parent = builder.getParent();
                    this.setChildren(builder.getChildren());
                    this.maxChildren = builder.getMaxChildren();
                    this.expanded = builder.isExpanded();
                    this.selected = builder.isSelected();
                    this.pinned = builder.isPinned();
                    if (this.pinned) {
                        this.pinToRoot();
                    }
                }
                TreeNode.prototype.getId = function () {
                    return this.id;
                };
                TreeNode.prototype.hasData = function () {
                    return !!this.data;
                };
                TreeNode.prototype.getDataId = function () {
                    return this.dataId;
                };
                TreeNode.prototype.isExpanded = function () {
                    return this.expanded;
                };
                TreeNode.prototype.setExpanded = function (expanded) {
                    if (expanded === void 0) { expanded = true; }
                    this.expanded = expanded;
                };
                TreeNode.prototype.isSelected = function () {
                    return this.selected;
                };
                TreeNode.prototype.setSelected = function (selected) {
                    if (selected === void 0) { selected = true; }
                    this.selected = selected;
                };
                TreeNode.prototype.isPinned = function () {
                    return this.pinned;
                };
                TreeNode.prototype.getMaxChildren = function () {
                    return this.maxChildren;
                };
                TreeNode.prototype.setMaxChildren = function (maxChildren) {
                    this.maxChildren = maxChildren;
                };
                TreeNode.prototype.getData = function () {
                    return this.data;
                };
                TreeNode.prototype.setData = function (data) {
                    this.data = data;
                };
                TreeNode.prototype.setDataId = function (dataId) {
                    this.dataId = dataId;
                };
                TreeNode.prototype.setViewer = function (name, viewer) {
                    this.viewersByName[name] = viewer;
                };
                TreeNode.prototype.clearViewers = function () {
                    this.viewersByName = {};
                };
                TreeNode.prototype.getViewer = function (name) {
                    return this.viewersByName[name];
                };
                TreeNode.prototype.getParent = function () {
                    return this.parent;
                };
                TreeNode.prototype.setParent = function (parent) {
                    this.parent = parent;
                    if (this.pinned) {
                        this.pinToRoot();
                    }
                };
                TreeNode.prototype.getRoot = function () {
                    var root = this, parent = this.getParent();
                    while (parent) {
                        root = parent;
                        parent = parent.getParent();
                    }
                    return root;
                };
                TreeNode.prototype.getChildren = function () {
                    return this.children;
                };
                TreeNode.prototype.setChildren = function (children) {
                    var _this = this;
                    this.children = children;
                    this.children.forEach(function (child) {
                        child.setParent(_this);
                    });
                };
                TreeNode.prototype.hasChildren = function () {
                    return this.children.length > 0;
                };
                TreeNode.prototype.regenerateId = function () {
                    this.id = Math.random().toString(36).substring(2);
                };
                TreeNode.prototype.regenerateIds = function () {
                    this.regenerateId();
                    this.children.forEach(function (elem) {
                        elem.regenerateIds();
                    });
                };
                TreeNode.prototype.addChild = function (child, isToBegin) {
                    this.children = this.children || [];
                    if (isToBegin) {
                        this.children.unshift(child);
                    }
                    else {
                        this.children.push(child);
                    }
                    this.clearViewers();
                    child.setParent(this);
                };
                TreeNode.prototype.removeChild = function (child) {
                    var children = [];
                    for (var i = 0; i < this.children.length; i++) {
                        if (this.children[i].getId() !== child.getId()) {
                            children.push(this.children[i]);
                        }
                    }
                    this.children = children;
                    if (this.children.length === 0) {
                        this.expanded = false;
                    }
                };
                TreeNode.prototype.remove = function () {
                    if (this.parent) {
                        this.parent.removeChild(this);
                        this.parent.clearViewers();
                        this.parent = null;
                    }
                };
                TreeNode.prototype.isVisible = function () {
                    var visible = true;
                    var parent = this.parent;
                    while (parent && visible) {
                        visible = parent.isExpanded();
                        parent = parent.getParent();
                    }
                    return visible;
                };
                TreeNode.prototype.treeToList = function (empty, expanded, selected) {
                    if (empty === void 0) { empty = false; }
                    if (expanded === void 0) { expanded = true; }
                    if (selected === void 0) { selected = false; }
                    var list = [];
                    if (this.selected === true || selected === false) {
                        if (this.getData() || empty === true) {
                            list.push(this);
                        }
                    }
                    if (this.expanded === true || expanded === false) {
                        this.children.forEach(function (child) {
                            list = list.concat(child.treeToList(empty, expanded, selected));
                        });
                    }
                    return list;
                };
                TreeNode.prototype.findNode = function (dataId) {
                    if (this.hasData() && this.getDataId() === dataId) {
                        return this;
                    }
                    for (var i = 0; i < this.children.length; i++) {
                        var child = this.children[i].findNode(dataId);
                        if (child) {
                            return child;
                        }
                    }
                    return null;
                };
                TreeNode.prototype.calcLevel = function () {
                    var parent = this.parent, lvl = 0;
                    while (parent) {
                        parent = parent.getParent();
                        lvl++;
                    }
                    return lvl;
                };
                TreeNode.prototype.pinToRoot = function () {
                    if (this.calcLevel() > 1 && this.data && this.parent) {
                        var duplicated = false;
                        var relatives = this.getRoot().getChildren();
                        for (var i = 0; i < relatives.length; i++) {
                            if (relatives[i].getData() && relatives[i].getDataId() === this.getDataId()) {
                                duplicated = true;
                                break;
                            }
                        }
                        if (!duplicated) {
                            if (this.pinned) {
                                this.parent.removeChild(this);
                                this.getRoot().addChild(this);
                            }
                            else {
                                new treegrid.TreeNodeBuilder(this).setPinned(true).build();
                            }
                        }
                    }
                };
                return TreeNode;
            })();
            treegrid.TreeNode = TreeNode;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeRoot = (function () {
                function TreeRoot() {
                    this.defaultRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    this.filteredRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    this.filtered = false;
                    this.currentSelection = [];
                    this.stashedSelection = [];
                }
                TreeRoot.prototype.getDefaultRoot = function () {
                    return this.defaultRoot;
                };
                TreeRoot.prototype.resetDefaultRoot = function (rootData) {
                    this.defaultRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    if (rootData) {
                        this.defaultRoot.setData(rootData);
                    }
                };
                TreeRoot.prototype.getFilteredRoot = function () {
                    return this.filteredRoot;
                };
                TreeRoot.prototype.resetFilteredRoot = function (rootData) {
                    this.filteredRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    if (rootData) {
                        this.filteredRoot.setData(rootData);
                    }
                };
                TreeRoot.prototype.resetCurrentRoot = function (rootData) {
                    if (this.isFiltered()) {
                        this.resetFilteredRoot(rootData);
                    }
                    else {
                        this.resetDefaultRoot(rootData);
                    }
                };
                TreeRoot.prototype.getCurrentRoot = function () {
                    return this.filtered ? this.filteredRoot : this.defaultRoot;
                };
                TreeRoot.prototype.isFiltered = function () {
                    return this.filtered;
                };
                TreeRoot.prototype.setFiltered = function (filtered) {
                    if (filtered === void 0) { filtered = true; }
                    if (filtered) {
                        this.filteredRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                        this.stashSelection();
                    }
                    else if (this.filtered && !filtered) {
                        this.stashSelection();
                    }
                    this.filtered = filtered;
                };
                TreeRoot.prototype.getCurrentSelection = function () {
                    return this.currentSelection;
                };
                TreeRoot.prototype.setCurrentSelection = function (selection) {
                    this.currentSelection = selection;
                    this.cleanStashedSelection();
                };
                TreeRoot.prototype.getStashedSelection = function () {
                    return this.stashedSelection;
                };
                TreeRoot.prototype.stashSelection = function () {
                    this.stashedSelection = this.stashedSelection.concat(this.currentSelection);
                    this.currentSelection = [];
                    this.cleanStashedSelection();
                };
                TreeRoot.prototype.getFullSelection = function (uniqueOnly) {
                    if (uniqueOnly === void 0) { uniqueOnly = true; }
                    var fullSelection = this.currentSelection.concat(this.stashedSelection);
                    if (uniqueOnly) {
                        var fullIds = fullSelection.map(function (el) {
                            return el.getDataId();
                        });
                        fullSelection = fullSelection.filter(function (value, index, self) {
                            return fullIds.indexOf(value.getDataId()) === index;
                        });
                    }
                    return fullSelection;
                };
                TreeRoot.prototype.cleanStashedSelection = function () {
                    var currentIds = this.currentSelection.map(function (el) {
                        return el.getDataId();
                    }), stashedIds = this.stashedSelection.map(function (el) {
                        return el.getDataId();
                    });
                    this.stashedSelection = this.stashedSelection.filter(function (value, index, self) {
                        return (currentIds.indexOf(value.getDataId()) < 0) && (stashedIds.indexOf(value.getDataId()) === index);
                    });
                };
                TreeRoot.prototype.clearStashedSelection = function () {
                    this.stashedSelection = [];
                };
                TreeRoot.prototype.removeSelection = function (dataId) {
                    this.currentSelection = this.currentSelection.filter(function (el) {
                        return el.getDataId() !== dataId;
                    });
                    this.stashedSelection = this.stashedSelection.filter(function (el) {
                        return el.getDataId() !== dataId;
                    });
                };
                TreeRoot.prototype.updateSelection = function (dataId, data) {
                    this.currentSelection.forEach(function (el) {
                        if (el.getDataId() === dataId) {
                            el.setData(data);
                        }
                    });
                    this.stashedSelection.forEach(function (el) {
                        if (el.getDataId() === dataId) {
                            el.setData(data);
                        }
                    });
                };
                return TreeRoot;
            })();
            treegrid.TreeRoot = TreeRoot;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeGridToolbar = (function (_super) {
                __extends(TreeGridToolbar, _super);
                function TreeGridToolbar(actions, treeGrid) {
                    _super.call(this);
                    this.addActions(actions.getAllActions());
                    this.treeGrid = treeGrid;
                }
                TreeGridToolbar.prototype.refresh = function () {
                    this.removeActions();
                    var actions = new api.ui.treegrid.actions.TreeGridToolbarActions(this.treeGrid.getGrid()).getAllActions();
                    this.addActions(actions);
                };
                return TreeGridToolbar;
            })(api.ui.toolbar.Toolbar);
            treegrid.TreeGridToolbar = TreeGridToolbar;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var DateTimeFormatter = (function () {
                function DateTimeFormatter() {
                }
                DateTimeFormatter.format = function (row, cell, value, columnDef, item) {
                    return DateTimeFormatter.createHtml(value);
                };
                DateTimeFormatter.formatNoTimestamp = function (row, cell, value, columnDef, item) {
                    return DateTimeFormatter.createHtmlNoTimestamp(value);
                };
                DateTimeFormatter.createHtml = function (date) {
                    if (!!date) {
                        var s = DateTimeFormatter.zeroPad(date.getFullYear(), 4) + "-" + DateTimeFormatter.zeroPad(date.getMonth() + 1, 2) + "-" + DateTimeFormatter.zeroPad(date.getDate(), 2) + " " + DateTimeFormatter.zeroPad(date.getHours(), 2) + ":" + DateTimeFormatter.zeroPad(date.getMinutes(), 2) + ":" + DateTimeFormatter.zeroPad(date.getSeconds(), 2);
                        return s;
                    }
                    return "";
                };
                DateTimeFormatter.createHtmlNoTimestamp = function (date) {
                    if (!!date) {
                        var s = DateTimeFormatter.zeroPad(date.getFullYear(), 4) + "-" + DateTimeFormatter.zeroPad(date.getMonth() + 1, 2) + "-" + DateTimeFormatter.zeroPad(date.getDate(), 2);
                        return s;
                    }
                    return "";
                };
                DateTimeFormatter.zeroPad = function (n, width) {
                    var nWidth = n.toString().length;
                    if (nWidth >= width) {
                        return "" + n;
                    }
                    var neededZeroes = width - nWidth;
                    var s = "";
                    for (var i = 0; i < neededZeroes; i++) {
                        s += "0";
                    }
                    return s + n;
                };
                return DateTimeFormatter;
            })();
            treegrid.DateTimeFormatter = DateTimeFormatter;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
            var GridOptionsBuilder = api.ui.grid.GridOptionsBuilder;
            var TreeGridBuilder = (function () {
                function TreeGridBuilder(grid) {
                    this.showToolbar = true;
                    this.columns = [];
                    this.classes = "";
                    this.autoLoad = true;
                    this.hotkeysEnabled = true;
                    this.partialLoadEnabled = false;
                    this.loadBufferSize = 0;
                    if (grid) {
                        this.showToolbar = grid.hasToolbar();
                        this.contextMenu = grid.getContextMenu();
                        this.classes = grid.getEl().getClass();
                        this.copyOptions(grid.getOptions()).copyColumns(grid.getColumns());
                    }
                    else {
                        this.options = this.buildDefaultOptions();
                        this.columns = this.buildDefaultColumns();
                    }
                    this.classes = "tree-grid " + this.classes;
                }
                TreeGridBuilder.prototype.nodeExtractor = function (node, column) {
                    var names = column.field.split('.');
                    var val = node["data"][names[0]];
                    for (var i = 1; i < names.length; i++) {
                        if (val && typeof val == 'object' && names[i] in val) {
                            val = val[names[i]];
                        }
                        else {
                            val = '';
                        }
                    }
                    return val;
                };
                TreeGridBuilder.prototype.buildDefaultOptions = function () {
                    return new GridOptionsBuilder().setDataItemColumnValueExtractor(this.nodeExtractor).setEditable(false).setEnableAsyncPostRender(true).setAutoRenderGridOnDataChanges(true).setEnableCellNavigation(false).setEnableColumnReorder(false).setForceFitColumns(true).setHideColumnHeaders(true).setCheckableRows(true).setRowHeight(45).setAutoHeight(true).build();
                };
                TreeGridBuilder.prototype.buildDefaultColumns = function () {
                    return [];
                };
                TreeGridBuilder.prototype.copyOptions = function (options) {
                    this.options = new GridOptionsBuilder().build();
                    return this;
                };
                TreeGridBuilder.prototype.copyColumns = function (columns) {
                    var _this = this;
                    this.columns = [];
                    columns.forEach(function (column) {
                        _this.columns.push(new GridColumnBuilder(column).build());
                    });
                    return this;
                };
                TreeGridBuilder.prototype.isShowToolbar = function () {
                    return this.showToolbar;
                };
                TreeGridBuilder.prototype.getContextMenu = function () {
                    return this.contextMenu;
                };
                TreeGridBuilder.prototype.getOptions = function () {
                    return this.options;
                };
                TreeGridBuilder.prototype.getColumns = function () {
                    return this.columns;
                };
                TreeGridBuilder.prototype.getClasses = function () {
                    return this.classes;
                };
                TreeGridBuilder.prototype.setShowToolbar = function (showToolbar) {
                    this.showToolbar = showToolbar;
                    return this;
                };
                TreeGridBuilder.prototype.setShowContextMenu = function (contextMenu) {
                    this.contextMenu = contextMenu;
                    return this;
                };
                TreeGridBuilder.prototype.setOptions = function (options) {
                    this.options = options;
                    return this;
                };
                TreeGridBuilder.prototype.setColumns = function (columns) {
                    this.columns = columns;
                    return this;
                };
                TreeGridBuilder.prototype.setClasses = function (classes) {
                    this.classes = classes;
                    return this;
                };
                TreeGridBuilder.prototype.prependClasses = function (classes) {
                    this.classes = classes + " " + this.classes;
                    return this;
                };
                TreeGridBuilder.prototype.setAutoLoad = function (autoLoad) {
                    this.autoLoad = autoLoad;
                    return this;
                };
                TreeGridBuilder.prototype.isAutoLoad = function () {
                    return this.autoLoad;
                };
                TreeGridBuilder.prototype.setCheckableRows = function (checkable) {
                    this.options.setCheckableRows(checkable);
                    return this;
                };
                TreeGridBuilder.prototype.isCheckableRows = function () {
                    return this.options.isCheckableRows();
                };
                TreeGridBuilder.prototype.setDragAndDrop = function (dragAndDrop) {
                    this.options.setDragAndDrop(dragAndDrop);
                    return this;
                };
                TreeGridBuilder.prototype.isDragAndDrop = function () {
                    return this.options.isDragAndDrop();
                };
                TreeGridBuilder.prototype.setSelectedCellCssClass = function (selectedCellCss) {
                    this.options.setSelectedCellCssClass(selectedCellCss);
                    return this;
                };
                TreeGridBuilder.prototype.getSelectedCellCssClass = function () {
                    return this.options.getSelectedCellCssClass();
                };
                TreeGridBuilder.prototype.disableMultipleSelection = function (disableMultipleSelection) {
                    this.options.disableMultipleSelection(disableMultipleSelection);
                    return this;
                };
                TreeGridBuilder.prototype.isMultipleSelectionDisabled = function () {
                    return this.options.isMultipleSelectionDisabled();
                };
                TreeGridBuilder.prototype.setHotkeysEnabled = function (enabled) {
                    this.hotkeysEnabled = enabled;
                    return this;
                };
                TreeGridBuilder.prototype.isHotkeysEnabled = function () {
                    return this.hotkeysEnabled;
                };
                TreeGridBuilder.prototype.setPartialLoadEnabled = function (enabled) {
                    this.partialLoadEnabled = enabled;
                    return this;
                };
                TreeGridBuilder.prototype.isPartialLoadEnabled = function () {
                    return this.partialLoadEnabled;
                };
                TreeGridBuilder.prototype.setLoadBufferSize = function (loadBufferSize) {
                    this.loadBufferSize = loadBufferSize;
                    return this;
                };
                TreeGridBuilder.prototype.getLoadBufferSize = function () {
                    return this.loadBufferSize;
                };
                TreeGridBuilder.prototype.build = function () {
                    return new treegrid.TreeGrid(this);
                };
                return TreeGridBuilder;
            })();
            treegrid.TreeGridBuilder = TreeGridBuilder;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeNodeBuilder = (function () {
                function TreeNodeBuilder(node) {
                    if (node) {
                        this.data = node.getData();
                        this.parent = node.getParent();
                        this.children = node.getChildren() || [];
                        this.maxChildren = node.getMaxChildren();
                    }
                    else {
                        this.children = [];
                        this.maxChildren = 0;
                    }
                    this.pinned = false;
                    this.expanded = false;
                    this.selected = false;
                }
                TreeNodeBuilder.prototype.isExpanded = function () {
                    return this.expanded;
                };
                TreeNodeBuilder.prototype.setExpanded = function (expanded) {
                    if (expanded === void 0) { expanded = true; }
                    this.expanded = expanded;
                    return this;
                };
                TreeNodeBuilder.prototype.isSelected = function () {
                    return this.selected;
                };
                TreeNodeBuilder.prototype.setSelected = function (selected) {
                    if (selected === void 0) { selected = true; }
                    this.selected = selected;
                    return this;
                };
                TreeNodeBuilder.prototype.isPinned = function () {
                    return this.pinned;
                };
                TreeNodeBuilder.prototype.setPinned = function (pinned) {
                    if (pinned === void 0) { pinned = true; }
                    this.pinned = pinned;
                    return this;
                };
                TreeNodeBuilder.prototype.getMaxChildren = function () {
                    return this.maxChildren;
                };
                TreeNodeBuilder.prototype.setMaxChildren = function (maxChildren) {
                    this.maxChildren = maxChildren;
                };
                TreeNodeBuilder.prototype.getData = function () {
                    return this.data;
                };
                TreeNodeBuilder.prototype.getDataId = function () {
                    return this.dataId;
                };
                TreeNodeBuilder.prototype.setData = function (data, dataId) {
                    this.data = data;
                    this.dataId = dataId;
                    return this;
                };
                TreeNodeBuilder.prototype.getParent = function () {
                    return this.parent;
                };
                TreeNodeBuilder.prototype.setParent = function (parent) {
                    this.parent = parent;
                    return this;
                };
                TreeNodeBuilder.prototype.getChildren = function () {
                    return this.children;
                };
                TreeNodeBuilder.prototype.setChildren = function (children) {
                    this.children = children;
                    return this;
                };
                TreeNodeBuilder.prototype.build = function () {
                    return new treegrid.TreeNode(this);
                };
                return TreeNodeBuilder;
            })();
            treegrid.TreeNodeBuilder = TreeNodeBuilder;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeGridContextMenu = (function (_super) {
                __extends(TreeGridContextMenu, _super);
                function TreeGridContextMenu(actions) {
                    _super.call(this);
                    this.actions = actions;
                    this.addActions(actions.getAllActions());
                }
                TreeGridContextMenu.prototype.getActions = function () {
                    return this.actions;
                };
                return TreeGridContextMenu;
            })(api.ui.menu.ContextMenu);
            treegrid.TreeGridContextMenu = TreeGridContextMenu;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var ContextMenuShownEvent = (function () {
                function ContextMenuShownEvent(x, y) {
                    this.x = x;
                    this.y = y;
                }
                ContextMenuShownEvent.prototype.getX = function () {
                    return this.x;
                };
                ContextMenuShownEvent.prototype.getY = function () {
                    return this.y;
                };
                return ContextMenuShownEvent;
            })();
            treegrid.ContextMenuShownEvent = ContextMenuShownEvent;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var DataChangedEvent = (function () {
                function DataChangedEvent(treeNode, action) {
                    this.treeNodes = treeNode;
                    this.type = action;
                }
                DataChangedEvent.prototype.getTreeNodes = function () {
                    return this.treeNodes;
                };
                DataChangedEvent.prototype.getType = function () {
                    return this.type;
                };
                DataChangedEvent.ADDED = 'added';
                DataChangedEvent.UPDATED = 'updated';
                DataChangedEvent.DELETED = 'deleted';
                return DataChangedEvent;
            })();
            treegrid.DataChangedEvent = DataChangedEvent;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var image;
        (function (_image) {
            var ImageCanvas = (function (_super) {
                __extends(ImageCanvas, _super);
                function ImageCanvas(image) {
                    var _this = this;
                    _super.call(this, 'image-canvas');
                    this.zoom = { x: -1, y: -1, factor: 1, previous: 1 };
                    this.pan = { x: 0, y: 0, overrideZoom: false };
                    this.zoomChangeListeners = [];
                    this.panChangeListeners = [];
                    this.image = image;
                    api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        console.group('on size changed');
                        if (_this.enabled) {
                            _this.recalculateHeight(true, false);
                            _this.renderCanvas();
                        }
                        console.groupEnd();
                    });
                    image.onLoaded(function (event) {
                        console.group('on image loaded');
                        if (_this.enabled) {
                            _this.recalculateHeight(false, true);
                            _this.renderCanvas();
                        }
                        console.groupEnd();
                    });
                    var prev;
                    image.onMouseDown(function (event) {
                        if (_this.enabled) {
                            event.preventDefault();
                            event.stopPropagation();
                            prev = { x: event.clientX, y: event.clientY };
                            _this.addClass('dragging');
                        }
                    });
                    image.onMouseMove(function (event) {
                        if (_this.enabled && prev && _this.zoom.factor > 1) {
                            _this.setPan(_this.pan.x + (event.clientX - prev.x) / _this.canvasWidth, _this.pan.y + (event.clientY - prev.y) / _this.canvasWidth);
                            prev = { x: event.clientX, y: event.clientY };
                        }
                    });
                    image.onMouseWheel(function (event) {
                        if (_this.enabled) {
                            var delta = (event['wheelDelta'] || -event.detail) > 0 ? 0.05 : -0.05;
                            var offset = _this.getEl().getOffset();
                            _this.setZoom(_this.getZoom() + delta, true, (event.clientX - offset.left) / _this.canvasWidth, (event.clientY - offset.top) / _this.canvasWidth);
                        }
                    });
                    var stopDrag = function (event) {
                        prev = undefined;
                        _this.removeClass('dragging');
                    };
                    image.onMouseLeave(stopDrag);
                    image.onMouseUp(stopDrag);
                }
                ImageCanvas.prototype.setPan = function (x, y, override) {
                    if (override === void 0) { override = true; }
                    this.pan.x = x;
                    this.pan.y = y;
                    if (override) {
                        this.pan.overrideZoom = true;
                    }
                    if (!this.enabled) {
                        return;
                    }
                    console.group('setPan');
                    console.debug('x = ' + x, 'y = ' + y);
                    this.renderCanvas();
                    if (this.pan.x == x && this.pan.y == y) {
                        this.notifyPanChanged(this.pan.x, this.pan.y);
                    }
                    console.groupEnd();
                };
                ImageCanvas.prototype.getPan = function () {
                    return {
                        x: this.pan.x,
                        y: this.pan.y
                    };
                };
                ImageCanvas.prototype.setZoom = function (value, override, x, y) {
                    if (override === void 0) { override = true; }
                    if (this.zoom.factor != value) {
                        this.zoom.previous = this.zoom.factor;
                        this.zoom.factor = +value.toFixed(3);
                    }
                    if (x) {
                        this.zoom.x = x;
                    }
                    if (y) {
                        this.zoom.y = y;
                    }
                    if (override) {
                        this.pan.overrideZoom = false;
                    }
                    if (!this.enabled) {
                        return;
                    }
                    console.group('setZoom');
                    console.debug('zoom = ' + value, 'x = ', x, 'y = ', y);
                    if (this.zoom.factor > 1) {
                        if (this.zoom.x < 0 && this.zoom.y < 0) {
                            this.zoom.x = 0.5;
                            this.zoom.y = 0.5 * this.canvasHeight / this.canvasWidth;
                            console.debug('setting default zoom center to (' + this.zoom.x + ', ' + this.zoom.y + ')');
                        }
                        if (!this.hasClass('draggable')) {
                            this.addClass('draggable');
                        }
                    }
                    else if (this.zoom.factor == 1) {
                        this.pan.overrideZoom = false;
                        if (this.hasClass('draggable')) {
                            this.removeClass('draggable');
                        }
                    }
                    this.renderCanvas();
                    if (this.zoom.factor == value) {
                        this.notifyZoomChanged(this.zoom.factor);
                    }
                    console.groupEnd();
                };
                ImageCanvas.prototype.getZoom = function () {
                    return this.zoom.factor;
                };
                ImageCanvas.prototype.setEnabled = function (isEnabled) {
                    this.enabled = isEnabled;
                    if (isEnabled) {
                        this.image.replaceWith(this);
                        this.appendChild(this.image);
                        this.enableCanvas();
                    }
                    else {
                        this.removeChild(this.image);
                        this.replaceWith(this.image);
                        this.disableCanvas();
                    }
                };
                ImageCanvas.prototype.isEnabled = function () {
                    return this.enabled;
                };
                ImageCanvas.prototype.renderCanvas = function () {
                    if (this.suspendRender || !this.enabled) {
                        return;
                    }
                    console.group('renderCanvas');
                    console.debug('zoom', this.zoom, '\npan', this.pan);
                    var imgEl = this.image.getEl();
                    console.debug('canvas (' + this.canvasWidth + ', ' + this.canvasHeight + ')');
                    var oldZoomFactor = this.zoom.previous, zoomFactor = Math.min(Math.max(this.zoom.factor, 1), 10);
                    console.debug('old zoom factor = ' + oldZoomFactor + ', \nnew zoom factor after restraining = ' + zoomFactor);
                    imgEl.setWidthPx(this.canvasWidth * zoomFactor);
                    var imgHeight = imgEl.getHeight(), imgWidth = imgEl.getWidth();
                    console.debug('image (' + imgWidth + ', ' + imgHeight + ')');
                    var panX, panY;
                    if (!this.pan.overrideZoom) {
                        var zoomWidthPanFactor = (this.zoom.x - this.pan.x) / oldZoomFactor;
                        var zoomHeightPanFactor = (this.zoom.y - this.pan.y) / oldZoomFactor;
                        panX = this.zoom.x - zoomFactor * zoomWidthPanFactor;
                        panY = this.zoom.y - zoomFactor * zoomHeightPanFactor;
                        console.debug('based on zoom center, pan = (' + panX + ', ' + panY + ')');
                    }
                    else {
                        panX = this.pan.x;
                        panY = this.pan.y;
                    }
                    var rightLimit = (this.canvasWidth - imgWidth) / this.canvasWidth;
                    if (imgWidth > this.canvasWidth) {
                        panX = Math.max(Math.min(panX, 0), rightLimit);
                    }
                    else {
                        panX = Math.min(Math.max(panX, 0), rightLimit);
                    }
                    var bottomLimit = (this.canvasHeight - imgHeight) / this.canvasWidth;
                    if (imgHeight > this.canvasHeight) {
                        panY = Math.max(Math.min(panY, 0), bottomLimit);
                    }
                    else {
                        panY = Math.min(Math.max(panY, 0), bottomLimit);
                    }
                    console.debug('after restraining to (0, 0, ' + rightLimit + ', ' + bottomLimit + '), \npan = (' + panX, ', ' + panY + ')');
                    imgEl.setMarginLeft(panX * 100 + '%').setMarginTop(panY * 100 + '%');
                    if (panX != this.pan.x || panY != this.pan.y) {
                        this.pan.x = panX;
                        this.pan.y = panY;
                        this.notifyPanChanged(panX, panY);
                    }
                    if (zoomFactor != this.zoom.factor) {
                        this.zoom.factor = zoomFactor;
                        this.notifyZoomChanged(zoomFactor);
                    }
                    console.groupEnd();
                };
                ImageCanvas.prototype.disableCanvas = function () {
                    this.image.getEl().setWidth('').setMarginLeft('').setMarginTop('');
                };
                ImageCanvas.prototype.enableCanvas = function () {
                    this.recalculateHeight(true, true);
                    this.suspendRender = true;
                    this.setZoom(this.zoom.factor, false);
                    this.setPan(this.pan.x, this.pan.y, false);
                    this.suspendRender = false;
                    this.renderCanvas();
                };
                ImageCanvas.prototype.recalculateHeight = function (updateWidth, updateRatio) {
                    if (updateWidth) {
                        this.canvasWidth = this.getEl().getWidth();
                        console.debug('new width = ' + this.canvasWidth);
                    }
                    if (updateRatio) {
                        this.imageRatio = +(this.image.getEl().getWidth() / this.image.getEl().getHeight()).toFixed(3);
                        console.debug('new ratio = ' + this.imageRatio);
                    }
                    this.canvasHeight = +(this.canvasWidth / this.imageRatio).toFixed(3);
                    console.debug('new height = ' + this.canvasHeight);
                    this.getEl().setHeightPx(this.canvasHeight);
                };
                ImageCanvas.prototype.onZoomChanged = function (listener) {
                    this.zoomChangeListeners.push(listener);
                };
                ImageCanvas.prototype.unZoomChanged = function (listener) {
                    this.zoomChangeListeners = this.zoomChangeListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageCanvas.prototype.notifyZoomChanged = function (zoom) {
                    console.debug('notifyZoomChanged', zoom);
                    this.zoomChangeListeners.forEach(function (listener) {
                        listener(zoom);
                    });
                };
                ImageCanvas.prototype.onPanChanged = function (listener) {
                    this.panChangeListeners.push(listener);
                };
                ImageCanvas.prototype.unPanChanged = function (listener) {
                    this.panChangeListeners = this.panChangeListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageCanvas.prototype.notifyPanChanged = function (x, y) {
                    console.debug('notifyPanChanged', x, y);
                    this.panChangeListeners.forEach(function (listener) {
                        listener(x, y);
                    });
                };
                return ImageCanvas;
            })(api.dom.DivEl);
            _image.ImageCanvas = ImageCanvas;
        })(image = ui.image || (ui.image = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var PrincipalType = api.security.PrincipalType;
            var PrincipalViewer = (function (_super) {
                __extends(PrincipalViewer, _super);
                function PrincipalViewer() {
                    _super.call(this);
                    this.removeClickedListeners = [];
                }
                PrincipalViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                PrincipalViewer.prototype.resolveUnnamedDisplayName = function (object) {
                    return object.getTypeName();
                };
                PrincipalViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getKey().toPath();
                };
                PrincipalViewer.prototype.resolveIconClass = function (object) {
                    switch (object.getKey().getType()) {
                        case 0 /* USER */:
                            return "icon-user";
                        case 1 /* GROUP */:
                            return "icon-users";
                        case 2 /* ROLE */:
                            return "icon-shield";
                    }
                    return "";
                };
                PrincipalViewer.prototype.onRemoveClicked = function (listener) {
                    this.removeClickedListeners.push(listener);
                };
                PrincipalViewer.prototype.unRemoveClicked = function (listener) {
                    this.removeClickedListeners = this.removeClickedListeners.filter(function (current) {
                        return current !== listener;
                    });
                };
                PrincipalViewer.prototype.notifyRemoveClicked = function (event) {
                    this.removeClickedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return PrincipalViewer;
            })(api.ui.NamesAndIconViewer);
            security.PrincipalViewer = PrincipalViewer;
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var PrincipalLoader = api.security.PrincipalLoader;
            var PrincipalComboBox = (function (_super) {
                __extends(PrincipalComboBox, _super);
                function PrincipalComboBox(loader, max) {
                    var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(max || 0).setComboBoxName("principalSelector").setIdentifierMethod("getKey").setLoader(loader || new PrincipalLoader()).setSelectedOptionsView(new PrincipalSelectedOptionsView()).setOptionDisplayValueViewer(new security.PrincipalViewer()).setDelayedInputValueChangedHandling(500);
                    _super.call(this, builder);
                }
                return PrincipalComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            security.PrincipalComboBox = PrincipalComboBox;
            var PrincipalSelectedOptionView = (function (_super) {
                __extends(PrincipalSelectedOptionView, _super);
                function PrincipalSelectedOptionView(option) {
                    var _this = this;
                    _super.call(this);
                    this.setOption(option);
                    this.setClass("principal-selected-option-view");
                    var removeButton = new api.dom.AEl("icon-close");
                    removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                    this.appendChild(removeButton);
                }
                PrincipalSelectedOptionView.prototype.setOption = function (option) {
                    this.option = option;
                    this.setObject(option.displayValue);
                };
                PrincipalSelectedOptionView.prototype.getOption = function () {
                    return this.option;
                };
                PrincipalSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                    this.onRemoveClicked(listener);
                };
                PrincipalSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                    this.unRemoveClicked(listener);
                };
                return PrincipalSelectedOptionView;
            })(security.PrincipalViewer);
            var PrincipalSelectedOptionsView = (function (_super) {
                __extends(PrincipalSelectedOptionsView, _super);
                function PrincipalSelectedOptionsView() {
                    _super.call(this, "principal-selected-options-view");
                }
                PrincipalSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    var optionView = new PrincipalSelectedOptionView(option);
                    return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
                };
                return PrincipalSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
                (function (Access) {
                    Access[Access["READ"] = 0] = "READ";
                    Access[Access["WRITE"] = 1] = "WRITE";
                    Access[Access["PUBLISH"] = 2] = "PUBLISH";
                    Access[Access["FULL"] = 3] = "FULL";
                    Access[Access["CUSTOM"] = 4] = "CUSTOM";
                })(acl.Access || (acl.Access = {}));
                var Access = acl.Access;
                var AccessSelector = (function (_super) {
                    __extends(AccessSelector, _super);
                    function AccessSelector() {
                        var _this = this;
                        _super.call(this, "access-selector");
                        this.valueChangedListeners = [];
                        AccessSelector.OPTIONS.forEach(function (option, index) {
                            var menuItem = new TabMenuItemBuilder().setLabel(option.name).build();
                            _this.addNavigationItem(menuItem);
                        });
                        this.onNavigationItemSelected(function (event) {
                            var item = event.getItem();
                            _this.setValue(AccessSelector.OPTIONS[item.getIndex()].value);
                        });
                    }
                    AccessSelector.prototype.getValue = function () {
                        return this.value;
                    };
                    AccessSelector.prototype.setValue = function (value, silent) {
                        var option = this.findOptionByValue(value);
                        if (option) {
                            this.selectNavigationItem(AccessSelector.OPTIONS.indexOf(option));
                            if (!silent) {
                                this.notifyValueChanged(new api.ui.ValueChangedEvent(Access[this.value], Access[value]));
                            }
                            this.value = value;
                        }
                        return this;
                    };
                    AccessSelector.prototype.findOptionByValue = function (value) {
                        for (var i = 0; i < AccessSelector.OPTIONS.length; i++) {
                            var option = AccessSelector.OPTIONS[i];
                            if (option.value == value) {
                                return option;
                            }
                        }
                        return undefined;
                    };
                    AccessSelector.prototype.showMenu = function () {
                        var menu = this.getMenuEl(), button = this.getTabMenuButtonEl(), entry = menu.getParentElement().getParentElement(), list = entry.getParentElement(), offset = entry.getEl().getOffsetTopRelativeToParent() - (list.getEl().getOffsetTopRelativeToParent() + list.getEl().getPaddingTop() + list.getEl().getScrollTop()), height = menu.getEl().getHeightWithoutPadding() - button.getEl().getHeight() + 2;
                        if (offset > height) {
                            menu.addClass("upward");
                        }
                        else {
                            menu.removeClass("upward");
                        }
                        _super.prototype.showMenu.call(this);
                    };
                    AccessSelector.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    AccessSelector.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    AccessSelector.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    AccessSelector.OPTIONS = [
                        { value: 0 /* READ */, name: 'Can Read' },
                        { value: 1 /* WRITE */, name: 'Can Write' },
                        { value: 2 /* PUBLISH */, name: 'Can Publish' },
                        { value: 3 /* FULL */, name: 'Full Access' },
                        { value: 4 /* CUSTOM */, name: 'Custom...' }
                    ];
                    return AccessSelector;
                })(api.ui.tab.TabMenu);
                acl.AccessSelector = AccessSelector;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var Permission = api.security.acl.Permission;
                var PermissionState = api.security.acl.PermissionState;
                var PermissionSelector = (function (_super) {
                    __extends(PermissionSelector, _super);
                    function PermissionSelector() {
                        var _this = this;
                        _super.call(this, 'permission-selector');
                        this.toggles = [];
                        this.valueChangedListeners = [];
                        this.enabled = true;
                        PermissionSelector.OPTIONS.forEach(function (option) {
                            var toggle = new PermissionToggle(option);
                            toggle.setEnabled(_this.enabled);
                            toggle.onValueChanged(function (event) {
                                var newValue = _this.getValue();
                                _this.notifyValueChanged(new api.ui.ValueChangedEvent(JSON.stringify(_this.oldValue), JSON.stringify(newValue)));
                                _this.oldValue = newValue;
                            });
                            _this.toggles.push(toggle);
                            _this.appendChild(toggle);
                        });
                    }
                    PermissionSelector.prototype.setEnabled = function (enabled) {
                        if (enabled != this.enabled) {
                            this.toggleClass('disabled', !enabled);
                            this.toggles.forEach(function (toggle) {
                                toggle.setEnabled(enabled);
                            });
                            this.enabled = enabled;
                        }
                        return this;
                    };
                    PermissionSelector.prototype.isEnabled = function () {
                        return this.enabled;
                    };
                    PermissionSelector.prototype.getValue = function () {
                        var values = {
                            allow: [],
                            deny: []
                        };
                        this.toggles.forEach(function (toggle) {
                            switch (toggle.getState()) {
                                case 0 /* ALLOW */:
                                    values.allow.push(toggle.getValue());
                                    break;
                                case 1 /* DENY */:
                                    values.deny.push(toggle.getValue());
                                    break;
                            }
                        });
                        return values;
                    };
                    PermissionSelector.prototype.setValue = function (newValue, silent) {
                        this.toggles.forEach(function (toggle) {
                            var value = toggle.getValue();
                            var state;
                            if (newValue.allow.indexOf(value) >= 0) {
                                state = 0 /* ALLOW */;
                            }
                            else if (newValue.deny.indexOf(value) >= 0) {
                                state = 1 /* DENY */;
                            }
                            else {
                                state = 2 /* INHERIT */;
                            }
                            toggle.setState(state, true);
                        });
                        if (!silent) {
                            this.notifyValueChanged(new api.ui.ValueChangedEvent(JSON.stringify(this.oldValue), JSON.stringify(newValue)));
                        }
                        this.oldValue = newValue;
                        return this;
                    };
                    PermissionSelector.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    PermissionSelector.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    PermissionSelector.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    PermissionSelector.OPTIONS = [
                        { name: 'Read', value: 0 /* READ */ },
                        { name: 'Create', value: 1 /* CREATE */ },
                        { name: 'Modify', value: 2 /* MODIFY */ },
                        { name: 'Delete', value: 3 /* DELETE */ },
                        { name: 'Publish', value: 4 /* PUBLISH */ },
                        { name: 'Read Permissions', value: 5 /* READ_PERMISSIONS */ },
                        { name: 'Write Permissions', value: 6 /* WRITE_PERMISSIONS */ }
                    ];
                    return PermissionSelector;
                })(api.dom.DivEl);
                acl.PermissionSelector = PermissionSelector;
                var PermissionToggle = (function (_super) {
                    __extends(PermissionToggle, _super);
                    function PermissionToggle(option, state) {
                        var _this = this;
                        if (state === void 0) { state = 2 /* INHERIT */; }
                        _super.call(this, 'permission-toggle ' + state);
                        this.valueChangedListeners = [];
                        this.originalStateIndex = -1;
                        this.stateIndex = -1;
                        this.enabled = true;
                        this.setHtml(option.name);
                        this.value = option.value;
                        if (state) {
                            this.setState(state, true);
                        }
                        this.onClicked(function (event) {
                            if (_this.enabled) {
                                var newIndex = (_this.stateIndex + 1) % PermissionToggle.STATES.length;
                                _this.setState(PermissionToggle.STATES[newIndex]);
                                event.preventDefault();
                                event.stopPropagation();
                            }
                        });
                    }
                    PermissionToggle.prototype.setEnabled = function (enabled) {
                        this.enabled = enabled;
                        this.toggleClass('disabled', !enabled);
                        return this;
                    };
                    PermissionToggle.prototype.isEnabled = function () {
                        return this.enabled;
                    };
                    PermissionToggle.prototype.getValue = function () {
                        return this.value;
                    };
                    PermissionToggle.prototype.getState = function () {
                        return PermissionToggle.STATES[this.stateIndex];
                    };
                    PermissionToggle.prototype.setState = function (newState, silent) {
                        var newStateIndex = PermissionToggle.STATES.indexOf(newState);
                        if (newStateIndex != this.stateIndex) {
                            if (this.originalStateIndex < 0) {
                                this.originalStateIndex = newStateIndex;
                            }
                            this.toggleClass('dirty', this.originalStateIndex >= 0 && this.originalStateIndex != newStateIndex);
                            var oldState = this.getState();
                            if (oldState != undefined) {
                                this.removeClass(PermissionState[oldState].toLowerCase());
                            }
                            this.addClass(PermissionState[newState].toLowerCase());
                            this.stateIndex = newStateIndex;
                            if (!silent) {
                                this.notifyValueChanged(new api.ui.ValueChangedEvent(PermissionState[oldState], PermissionState[newState]));
                            }
                        }
                        return this;
                    };
                    PermissionToggle.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    PermissionToggle.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    PermissionToggle.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    PermissionToggle.STATES = [0 /* ALLOW */, 1 /* DENY */, 2 /* INHERIT */];
                    return PermissionToggle;
                })(api.dom.AEl);
                acl.PermissionToggle = PermissionToggle;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var Principal = api.security.Principal;
                var Permission = api.security.acl.Permission;
                var AccessControlEntry = api.security.acl.AccessControlEntry;
                var AccessControlEntryView = (function (_super) {
                    __extends(AccessControlEntryView, _super);
                    function AccessControlEntryView(ace) {
                        var _this = this;
                        _super.call(this);
                        this.valueChangedListeners = [];
                        this.editable = true;
                        this.setClass('access-control-entry');
                        this.ace = ace;
                        this.accessSelector = new acl.AccessSelector();
                        this.removeButton = new api.dom.AEl("icon-close");
                        this.removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                        this.permissionSelector = new acl.PermissionSelector();
                        this.permissionSelector.onValueChanged(function (event) {
                            _this.toggleClass("dirty", event.getNewValue() != JSON.stringify({
                                allow: _this.ace.getAllowedPermissions().sort(),
                                deny: _this.ace.getDeniedPermissions().sort()
                            }));
                            _this.notifyValueChanged(_this.getAccessControlEntry());
                        });
                        this.permissionSelector.setValue({ allow: ace.getAllowedPermissions(), deny: ace.getDeniedPermissions() }, true);
                        this.accessSelector.onValueChanged(function (event) {
                            if (acl.Access[event.getNewValue()] == 4 /* CUSTOM */) {
                                _this.permissionSelector.show();
                            }
                            else {
                                if (acl.Access[event.getOldValue()] == 4 /* CUSTOM */) {
                                    _this.permissionSelector.hide();
                                }
                                _this.permissionSelector.setValue(_this.getPermissionsValueFromAccess(acl.Access[event.getNewValue()]));
                            }
                        });
                        this.setAccessControlEntry(this.ace, true);
                    }
                    AccessControlEntryView.prototype.doRender = function () {
                        _super.prototype.doRender.call(this);
                        this.appendChild(this.accessSelector);
                        this.appendChild(this.removeButton);
                        this.appendChild(this.permissionSelector);
                        return true;
                    };
                    AccessControlEntryView.prototype.getPermissionSelector = function () {
                        return this.permissionSelector;
                    };
                    AccessControlEntryView.prototype.getValueChangedListeners = function () {
                        return this.valueChangedListeners;
                    };
                    AccessControlEntryView.prototype.setEditable = function (editable) {
                        if (editable != this.editable) {
                            this.permissionSelector.setEnabled(editable);
                            this.accessSelector.setEnabled(editable);
                            this.editable = editable;
                        }
                    };
                    AccessControlEntryView.prototype.isEditable = function () {
                        return this.editable;
                    };
                    AccessControlEntryView.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    AccessControlEntryView.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    AccessControlEntryView.prototype.notifyValueChanged = function (item) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    AccessControlEntryView.prototype.setAccessControlEntry = function (ace, silent) {
                        this.ace = ace;
                        var principal = new Principal(ace.getPrincipalKey(), ace.getPrincipalDisplayName());
                        this.setObject(principal);
                        var permissions = {
                            allow: ace.getAllowedPermissions().sort(),
                            deny: ace.getDeniedPermissions().sort()
                        };
                        this.accessSelector.setValue(this.getAccessValueFromPermissions(permissions), silent);
                    };
                    AccessControlEntryView.prototype.getAccessControlEntry = function () {
                        var permissions = this.permissionSelector.getValue();
                        var ace = new AccessControlEntry(this.ace.getPrincipal());
                        ace.setAllowedPermissions(permissions.allow);
                        ace.setDeniedPermissions(permissions.deny);
                        return ace;
                    };
                    AccessControlEntryView.prototype.getPermissionsValueFromAccess = function (access) {
                        var permissions = {
                            allow: [],
                            deny: []
                        };
                        switch (access) {
                            case 3 /* FULL */:
                                permissions.allow.push(5 /* READ_PERMISSIONS */);
                                permissions.allow.push(6 /* WRITE_PERMISSIONS */);
                            case 2 /* PUBLISH */:
                                permissions.allow.push(4 /* PUBLISH */);
                            case 1 /* WRITE */:
                                permissions.allow.push(1 /* CREATE */);
                                permissions.allow.push(2 /* MODIFY */);
                                permissions.allow.push(3 /* DELETE */);
                            case 0 /* READ */:
                                permissions.allow.push(0 /* READ */);
                                break;
                        }
                        permissions.allow.sort();
                        permissions.deny.sort();
                        return permissions;
                    };
                    AccessControlEntryView.prototype.getAccessValueFromPermissions = function (permissions) {
                        if (permissions.deny.length == 0) {
                            if (this.isFullAccess(permissions.allow)) {
                                return 3 /* FULL */;
                            }
                            else if (this.isCanPublish(permissions.allow)) {
                                return 2 /* PUBLISH */;
                            }
                            else if (this.isCanWrite(permissions.allow)) {
                                return 1 /* WRITE */;
                            }
                            else if (this.isCanRead(permissions.allow)) {
                                return 0 /* READ */;
                            }
                        }
                        return 4 /* CUSTOM */;
                    };
                    AccessControlEntryView.prototype.isCanRead = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.length === 1;
                    };
                    AccessControlEntryView.prototype.isCanWrite = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.indexOf(1 /* CREATE */) >= 0 && allowed.indexOf(2 /* MODIFY */) >= 0 && allowed.indexOf(3 /* DELETE */) >= 0 && allowed.length === 4;
                    };
                    AccessControlEntryView.prototype.isCanPublish = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.indexOf(1 /* CREATE */) >= 0 && allowed.indexOf(2 /* MODIFY */) >= 0 && allowed.indexOf(3 /* DELETE */) >= 0 && allowed.indexOf(4 /* PUBLISH */) >= 0 && allowed.length === 5;
                    };
                    AccessControlEntryView.prototype.isFullAccess = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.indexOf(1 /* CREATE */) >= 0 && allowed.indexOf(2 /* MODIFY */) >= 0 && allowed.indexOf(3 /* DELETE */) >= 0 && allowed.indexOf(4 /* PUBLISH */) >= 0 && allowed.indexOf(5 /* READ_PERMISSIONS */) >= 0 && allowed.indexOf(6 /* WRITE_PERMISSIONS */) >= 0 && allowed.length === 7;
                    };
                    return AccessControlEntryView;
                })(api.ui.security.PrincipalViewer);
                acl.AccessControlEntryView = AccessControlEntryView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var AccessControlListView = (function (_super) {
                    __extends(AccessControlListView, _super);
                    function AccessControlListView(className) {
                        _super.call(this, 'access-control-list' + (className ? " " + className : ""));
                        this.itemValueChangedListeners = [];
                        this.itemsEditable = true;
                    }
                    AccessControlListView.prototype.createItemView = function (entry) {
                        var _this = this;
                        var itemView = new acl.AccessControlEntryView(entry);
                        itemView.setEditable(this.itemsEditable);
                        itemView.onRemoveClicked(function () {
                            _this.removeItem(entry);
                        });
                        itemView.onValueChanged(function (item) {
                            _this.notifyItemValueChanged(item);
                        });
                        return itemView;
                    };
                    AccessControlListView.prototype.getItemId = function (item) {
                        return item.getPrincipalKey().toString();
                    };
                    AccessControlListView.prototype.onItemValueChanged = function (listener) {
                        this.itemValueChangedListeners.push(listener);
                    };
                    AccessControlListView.prototype.unItemValueChanged = function (listener) {
                        this.itemValueChangedListeners = this.itemValueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    AccessControlListView.prototype.notifyItemValueChanged = function (item) {
                        this.itemValueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    AccessControlListView.prototype.setItemsEditable = function (editable) {
                        if (this.itemsEditable != editable) {
                            this.itemsEditable = editable;
                            this.refreshList();
                        }
                        return this;
                    };
                    AccessControlListView.prototype.isItemsEditable = function () {
                        return this.itemsEditable;
                    };
                    return AccessControlListView;
                })(api.ui.selector.list.ListBox);
                acl.AccessControlListView = AccessControlListView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var PrincipalType = api.security.PrincipalType;
                var AccessControlEntryViewer = (function (_super) {
                    __extends(AccessControlEntryViewer, _super);
                    function AccessControlEntryViewer() {
                        _super.call(this);
                    }
                    AccessControlEntryViewer.prototype.resolveDisplayName = function (object) {
                        return object.getPrincipalDisplayName();
                    };
                    AccessControlEntryViewer.prototype.resolveUnnamedDisplayName = function (object) {
                        return object.getPrincipalTypeName();
                    };
                    AccessControlEntryViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getPrincipalKey().toPath();
                    };
                    AccessControlEntryViewer.prototype.resolveIconClass = function (object) {
                        switch (object.getPrincipalKey().getType()) {
                            case 0 /* USER */:
                                return "icon-user";
                            case 1 /* GROUP */:
                                return "icon-users";
                            case 2 /* ROLE */:
                                return "icon-shield";
                        }
                        return "";
                    };
                    return AccessControlEntryViewer;
                })(api.ui.NamesAndIconViewer);
                acl.AccessControlEntryViewer = AccessControlEntryViewer;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var Permission = api.security.acl.Permission;
                var AccessControlEntryLoader = api.security.acl.AccessControlEntryLoader;
                var AccessControlComboBox = (function (_super) {
                    __extends(AccessControlComboBox, _super);
                    function AccessControlComboBox() {
                        this.aceSelectedOptionsView = new ACESelectedOptionsView();
                        var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(0).setComboBoxName("principalSelector").setIdentifierMethod("getPrincipalKey").setLoader(new AccessControlEntryLoader()).setSelectedOptionsView(this.aceSelectedOptionsView).setOptionDisplayValueViewer(new acl.AccessControlEntryViewer()).setDelayedInputValueChangedHandling(500);
                        _super.call(this, builder);
                    }
                    AccessControlComboBox.prototype.setEditable = function (editable) {
                        this.getSelectedOptions().forEach(function (option) {
                            option.getOptionView().setEditable(editable);
                        });
                    };
                    AccessControlComboBox.prototype.onOptionValueChanged = function (listener) {
                        this.aceSelectedOptionsView.onItemValueChanged(listener);
                    };
                    AccessControlComboBox.prototype.unItemValueChanged = function (listener) {
                        this.aceSelectedOptionsView.unItemValueChanged(listener);
                    };
                    return AccessControlComboBox;
                })(api.ui.selector.combobox.RichComboBox);
                acl.AccessControlComboBox = AccessControlComboBox;
                var ACESelectedOptionView = (function (_super) {
                    __extends(ACESelectedOptionView, _super);
                    function ACESelectedOptionView(option) {
                        var ace = option.displayValue;
                        if (ace.getAllowedPermissions().length == 0 && ace.getDeniedPermissions().length == 0) {
                            ace.allow(0 /* READ */);
                        }
                        _super.call(this, ace);
                        this.option = option;
                    }
                    ACESelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        this.setAccessControlEntry(option.displayValue);
                    };
                    ACESelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    ACESelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                        this.onRemoveClicked(listener);
                    };
                    ACESelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                        this.unRemoveClicked(listener);
                    };
                    return ACESelectedOptionView;
                })(acl.AccessControlEntryView);
                var ACESelectedOptionsView = (function (_super) {
                    __extends(ACESelectedOptionsView, _super);
                    function ACESelectedOptionsView(className) {
                        _super.call(this, className);
                        this.list = [];
                        this.selectedOptionRemovedListeners = [];
                    }
                    ACESelectedOptionsView.prototype.setMaximumOccurrences = function (value) {
                        this.maximumOccurrences = value;
                    };
                    ACESelectedOptionsView.prototype.getMaximumOccurrences = function () {
                        return this.maximumOccurrences;
                    };
                    ACESelectedOptionsView.prototype.createSelectedOption = function (option) {
                        throw new Error('Not supported, use createItemView instead');
                    };
                    ACESelectedOptionsView.prototype.createItemView = function (entry) {
                        var _this = this;
                        var option = {
                            displayValue: entry,
                            value: this.getItemId(entry)
                        };
                        var itemView = new ACESelectedOptionView(option);
                        itemView.onValueChanged(function (item) {
                            var selectedOption = _this.getById(item.getPrincipalKey().toString());
                            if (selectedOption) {
                                selectedOption.getOption().displayValue = item;
                            }
                            _this.notifyItemValueChanged(item);
                        });
                        var selectedOption = new SelectedOption(itemView, this.list.length);
                        itemView.onSelectedOptionRemoveRequest(function () {
                            _this.removeOption(option, false);
                        });
                        this.list.push(selectedOption);
                        return itemView;
                    };
                    ACESelectedOptionsView.prototype.addOption = function (option) {
                        this.addItem(option.displayValue);
                        return true;
                    };
                    ACESelectedOptionsView.prototype.removeOption = function (optionToRemove, silent) {
                        api.util.assertNotNull(optionToRemove, "optionToRemove cannot be null");
                        var selectedOption = this.getByOption(optionToRemove);
                        api.util.assertNotNull(selectedOption, "Did not find any selected option to remove from option: " + optionToRemove.value);
                        this.removeItem(optionToRemove.displayValue);
                        this.list = this.list.filter(function (option) {
                            return option.getOption().value != selectedOption.getOption().value;
                        });
                        if (selectedOption.getIndex() < this.list.length) {
                            for (var i = selectedOption.getIndex(); i < this.list.length; i++) {
                                this.list[i].setIndex(i);
                            }
                        }
                        if (!silent) {
                            this.notifySelectedOptionRemoved(selectedOption);
                        }
                    };
                    ACESelectedOptionsView.prototype.count = function () {
                        return this.list.length;
                    };
                    ACESelectedOptionsView.prototype.getSelectedOptions = function () {
                        return this.list;
                    };
                    ACESelectedOptionsView.prototype.getByIndex = function (index) {
                        return this.list[index];
                    };
                    ACESelectedOptionsView.prototype.getByOption = function (option) {
                        return this.getById(option.value);
                    };
                    ACESelectedOptionsView.prototype.getById = function (id) {
                        return this.list.filter(function (selectedOption) {
                            return selectedOption.getOption().value == id;
                        })[0];
                    };
                    ACESelectedOptionsView.prototype.isSelected = function (option) {
                        return this.getByOption(option) != null;
                    };
                    ACESelectedOptionsView.prototype.maximumOccurrencesReached = function () {
                        if (this.maximumOccurrences == 0) {
                            return false;
                        }
                        return this.count() >= this.maximumOccurrences;
                    };
                    ACESelectedOptionsView.prototype.moveOccurrence = function (formIndex, toIndex) {
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.list);
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.getChildren());
                        this.list.forEach(function (selectedOption, index) { return selectedOption.setIndex(index); });
                    };
                    ACESelectedOptionsView.prototype.notifySelectedOptionRemoved = function (removed) {
                        this.selectedOptionRemovedListeners.forEach(function (listener) {
                            listener(removed);
                        });
                    };
                    ACESelectedOptionsView.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners.push(listener);
                    };
                    ACESelectedOptionsView.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners = this.selectedOptionRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return ACESelectedOptionsView;
                })(acl.AccessControlListView);
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
                var UserStoreAccess = api.security.acl.UserStoreAccess;
                var UserStoreAccessSelector = (function (_super) {
                    __extends(UserStoreAccessSelector, _super);
                    function UserStoreAccessSelector() {
                        var _this = this;
                        _super.call(this, "access-selector");
                        this.valueChangedListeners = [];
                        UserStoreAccessSelector.OPTIONS.forEach(function (option, index) {
                            var menuItem = new TabMenuItemBuilder().setLabel(option.name).build();
                            _this.addNavigationItem(menuItem);
                        });
                        this.onNavigationItemSelected(function (event) {
                            var item = event.getItem();
                            _this.setValue(UserStoreAccessSelector.OPTIONS[item.getIndex()].value);
                        });
                    }
                    UserStoreAccessSelector.prototype.getValue = function () {
                        return this.value;
                    };
                    UserStoreAccessSelector.prototype.setValue = function (value, silent) {
                        var option = this.findOptionByValue(value);
                        if (option) {
                            this.selectNavigationItem(UserStoreAccessSelector.OPTIONS.indexOf(option));
                            if (!silent) {
                                this.notifyValueChanged(new api.ui.ValueChangedEvent(UserStoreAccess[this.value], UserStoreAccess[value]));
                            }
                            this.value = value;
                        }
                        return this;
                    };
                    UserStoreAccessSelector.prototype.findOptionByValue = function (value) {
                        for (var i = 0; i < UserStoreAccessSelector.OPTIONS.length; i++) {
                            var option = UserStoreAccessSelector.OPTIONS[i];
                            if (option.value == value) {
                                return option;
                            }
                        }
                        return undefined;
                    };
                    UserStoreAccessSelector.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    UserStoreAccessSelector.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    UserStoreAccessSelector.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    UserStoreAccessSelector.OPTIONS = [
                        { value: 0 /* READ */, name: 'Read' },
                        { value: 1 /* CREATE_USERS */, name: 'Create Users' },
                        { value: 2 /* WRITE_USERS */, name: 'Write Users' },
                        { value: 3 /* USER_STORE_MANAGER */, name: 'User Store Manager' },
                        { value: 4 /* ADMINISTRATOR */, name: 'Administrator' }
                    ];
                    return UserStoreAccessSelector;
                })(api.ui.tab.TabMenu);
                acl.UserStoreAccessSelector = UserStoreAccessSelector;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var Principal = api.security.Principal;
                var UserStoreAccessControlEntry = api.security.acl.UserStoreAccessControlEntry;
                var UserStoreAccess = api.security.acl.UserStoreAccess;
                var UserStoreAccessControlEntryView = (function (_super) {
                    __extends(UserStoreAccessControlEntryView, _super);
                    function UserStoreAccessControlEntryView(ace) {
                        var _this = this;
                        _super.call(this);
                        this.valueChangedListeners = [];
                        this.editable = true;
                        this.setClass('userstore-access-control-entry');
                        this.ace = ace;
                        if (!this.ace.getAccess()) {
                            this.ace.setAccess(UserStoreAccess[1 /* CREATE_USERS */]);
                        }
                        this.accessSelector = new acl.UserStoreAccessSelector();
                        this.appendChild(this.accessSelector);
                        this.accessSelector.onValueChanged(function (event) {
                            _this.ace.setAccess(event.getNewValue());
                        });
                        var removeButton = new api.dom.AEl("icon-close");
                        removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                        this.appendChild(removeButton);
                        this.setUserStoreAccessControlEntry(this.ace, true);
                    }
                    UserStoreAccessControlEntryView.prototype.getValueChangedListeners = function () {
                        return this.valueChangedListeners;
                    };
                    UserStoreAccessControlEntryView.prototype.setEditable = function (editable) {
                        if (editable != this.editable) {
                            this.accessSelector.setEnabled(editable);
                            this.editable = editable;
                        }
                    };
                    UserStoreAccessControlEntryView.prototype.isEditable = function () {
                        return this.editable;
                    };
                    UserStoreAccessControlEntryView.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    UserStoreAccessControlEntryView.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    UserStoreAccessControlEntryView.prototype.notifyValueChanged = function (item) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    UserStoreAccessControlEntryView.prototype.setUserStoreAccessControlEntry = function (ace, silent) {
                        this.ace = ace;
                        var principal = new Principal(ace.getPrincipal().getKey(), ace.getPrincipal().getDisplayName(), ace.getPrincipal().getModifiedTime());
                        this.setObject(principal);
                        this.accessSelector.setValue(ace.getAccess(), silent);
                    };
                    UserStoreAccessControlEntryView.prototype.getUserStoreAccessControlEntry = function () {
                        var ace = new UserStoreAccessControlEntry(this.ace.getPrincipal(), this.ace.getAccess());
                        return ace;
                    };
                    return UserStoreAccessControlEntryView;
                })(api.ui.security.PrincipalViewer);
                acl.UserStoreAccessControlEntryView = UserStoreAccessControlEntryView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var UserStoreAccessControlListView = (function (_super) {
                    __extends(UserStoreAccessControlListView, _super);
                    function UserStoreAccessControlListView(className) {
                        _super.call(this, 'access-control-list' + (className ? " " + className : ""));
                        this.itemValueChangedListeners = [];
                        this.itemsEditable = true;
                    }
                    UserStoreAccessControlListView.prototype.createItemView = function (entry) {
                        var _this = this;
                        var itemView = new acl.UserStoreAccessControlEntryView(entry);
                        itemView.setEditable(this.itemsEditable);
                        itemView.onRemoveClicked(function () {
                            _this.removeItem(entry);
                        });
                        itemView.onValueChanged(function (item) {
                            _this.notifyItemValueChanged(item);
                        });
                        return itemView;
                    };
                    UserStoreAccessControlListView.prototype.getItemId = function (item) {
                        return item.getPrincipal().getKey().toString();
                    };
                    UserStoreAccessControlListView.prototype.onItemValueChanged = function (listener) {
                        this.itemValueChangedListeners.push(listener);
                    };
                    UserStoreAccessControlListView.prototype.unItemValueChanged = function (listener) {
                        this.itemValueChangedListeners = this.itemValueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    UserStoreAccessControlListView.prototype.notifyItemValueChanged = function (item) {
                        this.itemValueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    UserStoreAccessControlListView.prototype.setItemsEditable = function (editable) {
                        if (this.itemsEditable != editable) {
                            this.itemsEditable = editable;
                            this.refreshList();
                        }
                        return this;
                    };
                    UserStoreAccessControlListView.prototype.isItemsEditable = function () {
                        return this.itemsEditable;
                    };
                    return UserStoreAccessControlListView;
                })(api.ui.selector.list.ListBox);
                acl.UserStoreAccessControlListView = UserStoreAccessControlListView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var UserStoreAccessControlEntryViewer = (function (_super) {
                    __extends(UserStoreAccessControlEntryViewer, _super);
                    function UserStoreAccessControlEntryViewer() {
                        _super.call(this);
                    }
                    UserStoreAccessControlEntryViewer.prototype.resolveDisplayName = function (object) {
                        return object.getPrincipalDisplayName();
                    };
                    UserStoreAccessControlEntryViewer.prototype.resolveUnnamedDisplayName = function (object) {
                        return object.getPrincipalTypeName();
                    };
                    UserStoreAccessControlEntryViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getPrincipalKey().toPath();
                    };
                    UserStoreAccessControlEntryViewer.prototype.resolveIconClass = function (object) {
                        return "icon-users";
                    };
                    return UserStoreAccessControlEntryViewer;
                })(api.ui.NamesAndIconViewer);
                acl.UserStoreAccessControlEntryViewer = UserStoreAccessControlEntryViewer;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var UserStoreAccessControlEntryLoader = api.security.acl.UserStoreAccessControlEntryLoader;
                var UserStoreAccessControlComboBox = (function (_super) {
                    __extends(UserStoreAccessControlComboBox, _super);
                    function UserStoreAccessControlComboBox() {
                        this.aceSelectedOptionsView = new UserStoreACESelectedOptionsView();
                        var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(0).setComboBoxName("principalSelector").setLoader(new UserStoreAccessControlEntryLoader()).setSelectedOptionsView(this.aceSelectedOptionsView).setOptionDisplayValueViewer(new acl.UserStoreAccessControlEntryViewer()).setDelayedInputValueChangedHandling(500);
                        _super.call(this, builder);
                    }
                    UserStoreAccessControlComboBox.prototype.setEditable = function (editable) {
                        this.getSelectedOptions().forEach(function (option) {
                            option.getOptionView().setEditable(editable);
                        });
                    };
                    UserStoreAccessControlComboBox.prototype.onOptionValueChanged = function (listener) {
                        this.aceSelectedOptionsView.onItemValueChanged(listener);
                    };
                    UserStoreAccessControlComboBox.prototype.unItemValueChanged = function (listener) {
                        this.aceSelectedOptionsView.unItemValueChanged(listener);
                    };
                    return UserStoreAccessControlComboBox;
                })(api.ui.selector.combobox.RichComboBox);
                acl.UserStoreAccessControlComboBox = UserStoreAccessControlComboBox;
                var UserStoreACESelectedOptionView = (function (_super) {
                    __extends(UserStoreACESelectedOptionView, _super);
                    function UserStoreACESelectedOptionView(option) {
                        var ace = option.displayValue;
                        _super.call(this, ace);
                        this.option = option;
                    }
                    UserStoreACESelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        this.setUserStoreAccessControlEntry(option.displayValue);
                    };
                    UserStoreACESelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    UserStoreACESelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                        this.onRemoveClicked(listener);
                    };
                    UserStoreACESelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                        this.unRemoveClicked(listener);
                    };
                    return UserStoreACESelectedOptionView;
                })(acl.UserStoreAccessControlEntryView);
                var UserStoreACESelectedOptionsView = (function (_super) {
                    __extends(UserStoreACESelectedOptionsView, _super);
                    function UserStoreACESelectedOptionsView(className) {
                        _super.call(this, className);
                        this.list = [];
                        this.selectedOptionRemovedListeners = [];
                    }
                    UserStoreACESelectedOptionsView.prototype.setMaximumOccurrences = function (value) {
                        this.maximumOccurrences = value;
                    };
                    UserStoreACESelectedOptionsView.prototype.getMaximumOccurrences = function () {
                        return this.maximumOccurrences;
                    };
                    UserStoreACESelectedOptionsView.prototype.createSelectedOption = function (option) {
                        throw new Error('Not supported, use createItemView instead');
                    };
                    UserStoreACESelectedOptionsView.prototype.createItemView = function (entry) {
                        var _this = this;
                        var option = {
                            displayValue: entry,
                            value: this.getItemId(entry)
                        };
                        var itemView = new UserStoreACESelectedOptionView(option);
                        itemView.onValueChanged(function (item) {
                            var selectedOption = _this.getById(item.getPrincipal().getKey().toString());
                            if (selectedOption) {
                                selectedOption.getOption().displayValue = item;
                            }
                            _this.notifyItemValueChanged(item);
                        });
                        var selectedOption = new SelectedOption(itemView, this.list.length);
                        itemView.onSelectedOptionRemoveRequest(function () {
                            _this.removeOption(option, false);
                        });
                        this.list.push(selectedOption);
                        return itemView;
                    };
                    UserStoreACESelectedOptionsView.prototype.addOption = function (option) {
                        this.addItem(option.displayValue);
                        return true;
                    };
                    UserStoreACESelectedOptionsView.prototype.removeOption = function (optionToRemove, silent) {
                        api.util.assertNotNull(optionToRemove, "optionToRemove cannot be null");
                        var selectedOption = this.getByOption(optionToRemove);
                        api.util.assertNotNull(selectedOption, "Did not find any selected option to remove from option: " + optionToRemove.value);
                        this.removeItem(optionToRemove.displayValue);
                        this.list = this.list.filter(function (option) {
                            return option.getOption().value != selectedOption.getOption().value;
                        });
                        if (selectedOption.getIndex() < this.list.length) {
                            for (var i = selectedOption.getIndex(); i < this.list.length; i++) {
                                this.list[i].setIndex(i);
                            }
                        }
                        if (!silent) {
                            this.notifySelectedOptionRemoved(selectedOption);
                        }
                    };
                    UserStoreACESelectedOptionsView.prototype.count = function () {
                        return this.list.length;
                    };
                    UserStoreACESelectedOptionsView.prototype.getSelectedOptions = function () {
                        return this.list;
                    };
                    UserStoreACESelectedOptionsView.prototype.getByIndex = function (index) {
                        return this.list[index];
                    };
                    UserStoreACESelectedOptionsView.prototype.getByOption = function (option) {
                        return this.getById(option.value);
                    };
                    UserStoreACESelectedOptionsView.prototype.getById = function (id) {
                        return this.list.filter(function (selectedOption) {
                            return selectedOption.getOption().value == id;
                        })[0];
                    };
                    UserStoreACESelectedOptionsView.prototype.isSelected = function (option) {
                        return this.getByOption(option) != null;
                    };
                    UserStoreACESelectedOptionsView.prototype.maximumOccurrencesReached = function () {
                        if (this.maximumOccurrences == 0) {
                            return false;
                        }
                        return this.count() >= this.maximumOccurrences;
                    };
                    UserStoreACESelectedOptionsView.prototype.moveOccurrence = function (formIndex, toIndex) {
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.list);
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.getChildren());
                        this.list.forEach(function (selectedOption, index) { return selectedOption.setIndex(index); });
                    };
                    UserStoreACESelectedOptionsView.prototype.notifySelectedOptionRemoved = function (removed) {
                        this.selectedOptionRemovedListeners.forEach(function (listener) {
                            listener(removed);
                        });
                    };
                    UserStoreACESelectedOptionsView.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners.push(listener);
                    };
                    UserStoreACESelectedOptionsView.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners = this.selectedOptionRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return UserStoreACESelectedOptionsView;
                })(acl.UserStoreAccessControlListView);
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var locale;
        (function (_locale) {
            var LocaleViewer = (function (_super) {
                __extends(LocaleViewer, _super);
                function LocaleViewer() {
                    _super.call(this);
                    this.removeClickedListeners = [];
                    this.displayNamePattern = '{0} ({1})';
                    this.namesView = new api.app.NamesView();
                    this.appendChild(this.namesView);
                }
                LocaleViewer.prototype.setObject = function (locale) {
                    _super.prototype.setObject.call(this, locale);
                    this.namesView.setMainName(api.util.StringHelper.format(this.displayNamePattern, locale.getDisplayName(), locale.getTag()));
                };
                LocaleViewer.prototype.getPreferredHeight = function () {
                    return 30;
                };
                LocaleViewer.prototype.onRemoveClicked = function (listener) {
                    this.removeClickedListeners.push(listener);
                };
                LocaleViewer.prototype.unRemoveClicked = function (listener) {
                    this.removeClickedListeners = this.removeClickedListeners.filter(function (current) {
                        return current !== listener;
                    });
                };
                LocaleViewer.prototype.notifyRemoveClicked = function (event) {
                    this.removeClickedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return LocaleViewer;
            })(api.ui.Viewer);
            _locale.LocaleViewer = LocaleViewer;
        })(locale = ui.locale || (ui.locale = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var locale;
        (function (locale) {
            var LocaleLoader = api.locale.LocaleLoader;
            var LocaleComboBox = (function (_super) {
                __extends(LocaleComboBox, _super);
                function LocaleComboBox(max) {
                    var _this = this;
                    var localeSelectedOptionsView = new LocaleSelectedOptionsView();
                    localeSelectedOptionsView.onOptionDeselected(function () {
                        _this.clearSelection();
                    });
                    var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(max || 0).setComboBoxName("localeSelector").setIdentifierMethod("getTag").setLoader(new LocaleLoader()).setSelectedOptionsView(localeSelectedOptionsView).setOptionDisplayValueViewer(new locale.LocaleViewer()).setDelayedInputValueChangedHandling(500);
                    _super.call(this, builder);
                }
                LocaleComboBox.prototype.clearSelection = function () {
                    this.getLoader().search("");
                    _super.prototype.clearSelection.call(this);
                };
                return LocaleComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            locale.LocaleComboBox = LocaleComboBox;
            var LocaleSelectedOptionView = (function (_super) {
                __extends(LocaleSelectedOptionView, _super);
                function LocaleSelectedOptionView(option) {
                    var _this = this;
                    _super.call(this);
                    this.setOption(option);
                    this.setClass("locale-selected-option-view");
                    var removeButton = new api.dom.AEl("icon-close");
                    removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                    this.appendChild(removeButton);
                }
                LocaleSelectedOptionView.prototype.setOption = function (option) {
                    this.option = option;
                    this.setObject(option.displayValue);
                };
                LocaleSelectedOptionView.prototype.getOption = function () {
                    return this.option;
                };
                LocaleSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                    this.onRemoveClicked(listener);
                };
                LocaleSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                    this.unRemoveClicked(listener);
                };
                return LocaleSelectedOptionView;
            })(locale.LocaleViewer);
            var LocaleSelectedOptionsView = (function (_super) {
                __extends(LocaleSelectedOptionsView, _super);
                function LocaleSelectedOptionsView() {
                    _super.call(this, "locale-selected-options-view");
                }
                LocaleSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    var optionView = new LocaleSelectedOptionView(option);
                    return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
                };
                return LocaleSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
        })(locale = ui.locale || (ui.locale = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueType = (function () {
            function ValueType(name) {
                this.name = name;
            }
            ValueType.prototype.toString = function () {
                return this.name;
            };
            ValueType.prototype.valueToString = function (value) {
                return String(value.getObject());
            };
            ValueType.prototype.valueToBoolean = function (value) {
                return value.getString() == "true";
            };
            ValueType.prototype.valueToNumber = function (value) {
                return Number(value.getObject());
            };
            ValueType.prototype.isValid = function (value) {
                return true;
            };
            ValueType.prototype.isConvertible = function (value) {
                return true;
            };
            ValueType.prototype.newValue = function (value) {
                return new data.Value(value, this);
            };
            ValueType.prototype.newNullValue = function () {
                return new data.Value(null, this);
            };
            ValueType.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ValueType)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                return true;
            };
            ValueType.prototype.valueEquals = function (a, b) {
                throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
            };
            ValueType.prototype.toJsonValue = function (value) {
                return value.getObject();
            };
            ValueType.prototype.fromJsonValue = function (jsonValue) {
                if (jsonValue) {
                    return this.newValue(jsonValue.toString());
                }
                else if ("" == jsonValue) {
                    return this.newValue(jsonValue);
                }
                else {
                    return this.newNullValue();
                }
            };
            return ValueType;
        })();
        data.ValueType = ValueType;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (_data) {
        var ValueTypePropertySet = (function (_super) {
            __extends(ValueTypePropertySet, _super);
            function ValueTypePropertySet() {
                _super.call(this, "PropertySet");
            }
            ValueTypePropertySet.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, _data.PropertySet)) {
                    return false;
                }
                return true;
            };
            ValueTypePropertySet.prototype.isConvertible = function (value) {
                return false;
            };
            ValueTypePropertySet.prototype.newValue = function (value) {
                throw new Error("A value of type Data cannot be created from a string");
            };
            ValueTypePropertySet.prototype.toJsonValue = function (value) {
                if (value.isNull()) {
                    return null;
                }
                var data = value.getObject();
                return data.toJson();
            };
            ValueTypePropertySet.prototype.fromJsonValue = function (propertyArrayJsonArray) {
                throw new Error("Method not supported!");
            };
            ValueTypePropertySet.prototype.valueToString = function (value) {
                throw new Error("A value of type Data cannot be made into a string");
            };
            ValueTypePropertySet.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypePropertySet;
        })(_data.ValueType);
        _data.ValueTypePropertySet = ValueTypePropertySet;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeString = (function (_super) {
            __extends(ValueTypeString, _super);
            function ValueTypeString() {
                _super.call(this, "String");
            }
            ValueTypeString.prototype.isValid = function (value) {
                return typeof value === 'string';
            };
            ValueTypeString.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.stringEquals(a, b);
            };
            return ValueTypeString;
        })(data.ValueType);
        data.ValueTypeString = ValueTypeString;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeHtmlPart = (function (_super) {
            __extends(ValueTypeHtmlPart, _super);
            function ValueTypeHtmlPart() {
                _super.call(this, "HtmlPart");
            }
            ValueTypeHtmlPart.prototype.isValid = function (value) {
                return typeof value === 'string';
            };
            ValueTypeHtmlPart.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.stringEquals(a, b);
            };
            return ValueTypeHtmlPart;
        })(data.ValueType);
        data.ValueTypeHtmlPart = ValueTypeHtmlPart;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeXml = (function (_super) {
            __extends(ValueTypeXml, _super);
            function ValueTypeXml() {
                _super.call(this, "Xml");
            }
            ValueTypeXml.prototype.isValid = function (value) {
                return typeof value === 'string';
            };
            ValueTypeXml.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.stringEquals(a, b);
            };
            return ValueTypeXml;
        })(data.ValueType);
        data.ValueTypeXml = ValueTypeXml;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeBoolean = (function (_super) {
            __extends(ValueTypeBoolean, _super);
            function ValueTypeBoolean() {
                _super.call(this, "Boolean");
            }
            ValueTypeBoolean.prototype.isValid = function (value) {
                return typeof value === 'boolean';
            };
            ValueTypeBoolean.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (!(value == "true" || value == "false")) {
                    return false;
                }
                var convertedValue = Boolean(value);
                return this.isValid(convertedValue);
            };
            ValueTypeBoolean.prototype.newValue = function (value) {
                if (!this.isConvertible(value)) {
                    return new data.Value(null, this);
                }
                return new data.Value(this.convertFromString(value), this);
            };
            ValueTypeBoolean.prototype.fromJsonValue = function (jsonValue) {
                return new data.Value(jsonValue, this);
            };
            ValueTypeBoolean.prototype.convertFromString = function (value) {
                if (value == "true") {
                    return true;
                }
                else if (value == "false") {
                    return false;
                }
                else {
                    throw new Error("given string cannot be converted to a Boolean Value: " + value);
                }
            };
            ValueTypeBoolean.prototype.valueToString = function (value) {
                return value.getObject().toString();
            };
            ValueTypeBoolean.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.booleanEquals(a, b);
            };
            ValueTypeBoolean.prototype.newFalse = function () {
                return new data.Value(false, this);
            };
            ValueTypeBoolean.prototype.newTrue = function () {
                return new data.Value(true, this);
            };
            return ValueTypeBoolean;
        })(data.ValueType);
        data.ValueTypeBoolean = ValueTypeBoolean;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLong = (function (_super) {
            __extends(ValueTypeLong, _super);
            function ValueTypeLong() {
                _super.call(this, "Long");
            }
            ValueTypeLong.prototype.isValid = function (value) {
                return api.util.NumberHelper.isWholeNumber(value);
            };
            ValueTypeLong.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                var convertedValue = Number(value);
                return this.isValid(convertedValue);
            };
            ValueTypeLong.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(this.convertFromString(value), this);
            };
            ValueTypeLong.prototype.fromJsonValue = function (jsonValue) {
                return new data.Value(jsonValue, this);
            };
            ValueTypeLong.prototype.convertFromString = function (value) {
                return Number(value);
            };
            ValueTypeLong.prototype.valueToString = function (value) {
                return value.getObject().toString();
            };
            ValueTypeLong.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.numberEquals(a, b);
            };
            return ValueTypeLong;
        })(data.ValueType);
        data.ValueTypeLong = ValueTypeLong;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeDouble = (function (_super) {
            __extends(ValueTypeDouble, _super);
            function ValueTypeDouble() {
                _super.call(this, "Double");
            }
            ValueTypeDouble.prototype.isValid = function (value) {
                return api.util.NumberHelper.isNumber(value);
            };
            ValueTypeDouble.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                var convertedValue = Number(value);
                return this.isValid(convertedValue);
            };
            ValueTypeDouble.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(this.convertFromString(value), this);
            };
            ValueTypeDouble.prototype.fromJsonValue = function (jsonValue) {
                return new data.Value(jsonValue, this);
            };
            ValueTypeDouble.prototype.convertFromString = function (value) {
                return Number(value);
            };
            ValueTypeDouble.prototype.valueToString = function (value) {
                return value.getObject().toString();
            };
            ValueTypeDouble.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.numberEquals(a, b);
            };
            return ValueTypeDouble;
        })(data.ValueType);
        data.ValueTypeDouble = ValueTypeDouble;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLocalDate = (function (_super) {
            __extends(ValueTypeLocalDate, _super);
            function ValueTypeLocalDate() {
                _super.call(this, "LocalDate");
            }
            ValueTypeLocalDate.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, Date)) {
                    return false;
                }
                var valueAsDate = value;
                return !api.util.DateHelper.isInvalidDate(valueAsDate);
            };
            ValueTypeLocalDate.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (value.length != 10) {
                    return false;
                }
                if (!(value.charAt(4) == '-' && value.charAt(7) == '-')) {
                    return false;
                }
                return this.isValid(new Date(value));
            };
            ValueTypeLocalDate.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                var date = api.util.DateHelper.parseUTCDate(value);
                return new data.Value(date, this);
            };
            ValueTypeLocalDate.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : api.util.DateHelper.formatUTCDate(value.getLocalDate());
            };
            ValueTypeLocalDate.prototype.valueToString = function (value) {
                return api.util.DateHelper.formatUTCDate(value.getObject());
            };
            ValueTypeLocalDate.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.dateEquals(a, b);
            };
            return ValueTypeLocalDate;
        })(data.ValueType);
        data.ValueTypeLocalDate = ValueTypeLocalDate;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLocalDateTime = (function (_super) {
            __extends(ValueTypeLocalDateTime, _super);
            function ValueTypeLocalDateTime() {
                _super.call(this, "LocalDateTime");
            }
            ValueTypeLocalDateTime.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, Date)) {
                    return false;
                }
                var valueAsDate = value;
                return !api.util.DateHelper.isInvalidDate(valueAsDate);
            };
            ValueTypeLocalDateTime.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (value.length != 19) {
                    return false;
                }
                var valueAsDate = api.util.DateHelper.parseUTCDateTime(value);
                return this.isValid(valueAsDate);
            };
            ValueTypeLocalDateTime.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                var date = api.util.DateHelper.parseUTCDateTime(value);
                return new data.Value(date, this);
            };
            ValueTypeLocalDateTime.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : api.util.DateHelper.formatUTCDateTime(value.getLocalDateTime());
            };
            ValueTypeLocalDateTime.prototype.valueToString = function (value) {
                return api.util.DateHelper.formatUTCDateTime(value.getObject());
            };
            ValueTypeLocalDateTime.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.dateEquals(a, b);
            };
            return ValueTypeLocalDateTime;
        })(data.ValueType);
        data.ValueTypeLocalDateTime = ValueTypeLocalDateTime;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLocalTime = (function (_super) {
            __extends(ValueTypeLocalTime, _super);
            function ValueTypeLocalTime() {
                _super.call(this, "LocalTime");
            }
            ValueTypeLocalTime.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.LocalTime)) {
                    return false;
                }
                return true;
            };
            ValueTypeLocalTime.prototype.isConvertible = function (value) {
                var asString = value;
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return api.util.LocalTime.isValidString(value);
            };
            ValueTypeLocalTime.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(api.util.LocalTime.fromString(value), this);
            };
            ValueTypeLocalTime.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getLocalTime().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeLocalTime.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            ValueTypeLocalTime.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getLocalTime().toString();
            };
            return ValueTypeLocalTime;
        })(data.ValueType);
        data.ValueTypeLocalTime = ValueTypeLocalTime;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeDateTime = (function (_super) {
            __extends(ValueTypeDateTime, _super);
            function ValueTypeDateTime() {
                _super.call(this, "DateTime");
            }
            ValueTypeDateTime.prototype.isValid = function (value) {
                return typeof value === 'string';
            };
            ValueTypeDateTime.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.stringEquals(a, b);
            };
            return ValueTypeDateTime;
        })(data.ValueType);
        data.ValueTypeDateTime = ValueTypeDateTime;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeGeoPoint = (function (_super) {
            __extends(ValueTypeGeoPoint, _super);
            function ValueTypeGeoPoint() {
                _super.call(this, "GeoPoint");
            }
            ValueTypeGeoPoint.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.GeoPoint)) {
                    return false;
                }
                return true;
            };
            ValueTypeGeoPoint.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return api.util.GeoPoint.isValidString(value);
            };
            ValueTypeGeoPoint.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(api.util.GeoPoint.fromString(value), this);
            };
            ValueTypeGeoPoint.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getGeoPoint().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeGeoPoint.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getGeoPoint().toString();
            };
            ValueTypeGeoPoint.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeGeoPoint;
        })(data.ValueType);
        data.ValueTypeGeoPoint = ValueTypeGeoPoint;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeBinaryReference = (function (_super) {
            __extends(ValueTypeBinaryReference, _super);
            function ValueTypeBinaryReference() {
                _super.call(this, "BinaryReference");
            }
            ValueTypeBinaryReference.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.BinaryReference)) {
                    return false;
                }
                return true;
            };
            ValueTypeBinaryReference.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return true;
            };
            ValueTypeBinaryReference.prototype.newValue = function (value) {
                if (this.isConvertible(value)) {
                    return new data.Value(new api.util.BinaryReference(value), this);
                }
                else {
                    return this.newNullValue();
                }
            };
            ValueTypeBinaryReference.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getBinaryReference().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeBinaryReference.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getBinaryReference().toString();
            };
            ValueTypeBinaryReference.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeBinaryReference;
        })(data.ValueType);
        data.ValueTypeBinaryReference = ValueTypeBinaryReference;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeReference = (function (_super) {
            __extends(ValueTypeReference, _super);
            function ValueTypeReference() {
                _super.call(this, "Reference");
            }
            ValueTypeReference.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.Reference)) {
                    return false;
                }
                return true;
            };
            ValueTypeReference.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return true;
            };
            ValueTypeReference.prototype.newValue = function (value) {
                if (this.isConvertible(value)) {
                    return new data.Value(new api.util.Reference(value), this);
                }
                else {
                    return this.newNullValue();
                }
            };
            ValueTypeReference.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getReference().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeReference.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getReference().toString();
            };
            ValueTypeReference.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeReference;
        })(data.ValueType);
        data.ValueTypeReference = ValueTypeReference;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLink = (function (_super) {
            __extends(ValueTypeLink, _super);
            function ValueTypeLink() {
                _super.call(this, "Link");
            }
            ValueTypeLink.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.Link)) {
                    return false;
                }
                return true;
            };
            ValueTypeLink.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return true;
            };
            ValueTypeLink.prototype.newValue = function (value) {
                if (this.isConvertible(value)) {
                    return new data.Value(new api.util.Link(value), this);
                }
                else {
                    return this.newNullValue();
                }
            };
            ValueTypeLink.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getLink().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeLink.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getLink().toString();
            };
            ValueTypeLink.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeLink;
        })(data.ValueType);
        data.ValueTypeLink = ValueTypeLink;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypes = (function () {
            function ValueTypes() {
            }
            ValueTypes.fromName = function (name) {
                for (var i = 0; i < ValueTypes.ALL.length; i++) {
                    var type = ValueTypes.ALL[i];
                    if (type.toString() == name) {
                        return type;
                    }
                }
                throw ("Unknown ValueType: " + name);
            };
            ValueTypes.DATA = new data.ValueTypePropertySet();
            ValueTypes.STRING = new data.ValueTypeString();
            ValueTypes.HTML_PART = new data.ValueTypeHtmlPart();
            ValueTypes.XML = new data.ValueTypeXml();
            ValueTypes.LOCAL_DATE = new data.ValueTypeLocalDate();
            ValueTypes.LOCAL_TIME = new data.ValueTypeLocalTime();
            ValueTypes.LOCAL_DATE_TIME = new data.ValueTypeLocalDateTime();
            ValueTypes.DATE_TIME = new data.ValueTypeDateTime();
            ValueTypes.LONG = new data.ValueTypeLong();
            ValueTypes.BOOLEAN = new data.ValueTypeBoolean();
            ValueTypes.DOUBLE = new data.ValueTypeDouble();
            ValueTypes.GEO_POINT = new data.ValueTypeGeoPoint();
            ValueTypes.REFERENCE = new data.ValueTypeReference();
            ValueTypes.BINARY_REFERENCE = new data.ValueTypeBinaryReference();
            ValueTypes.ALL = [
                ValueTypes.DATA,
                ValueTypes.STRING,
                ValueTypes.HTML_PART,
                ValueTypes.XML,
                ValueTypes.LOCAL_DATE,
                ValueTypes.LOCAL_TIME,
                ValueTypes.LOCAL_DATE_TIME,
                ValueTypes.DATE_TIME,
                ValueTypes.LONG,
                ValueTypes.BOOLEAN,
                ValueTypes.DOUBLE,
                ValueTypes.GEO_POINT,
                ValueTypes.REFERENCE,
                ValueTypes.BINARY_REFERENCE
            ];
            return ValueTypes;
        })();
        data.ValueTypes = ValueTypes;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var Value = (function () {
            function Value(value, type) {
                this.value = null;
                this.value = value;
                this.type = type;
                if (value) {
                    var isValid = this.type.isValid(value);
                    if (isValid == undefined) {
                        throw new Error(api.ClassHelper.getClassName(this.type) + ".isValid() did not return any value: " + isValid);
                    }
                    if (isValid == false) {
                        throw new Error("Invalid value for type " + type.toString() + ": " + value);
                    }
                }
            }
            Value.prototype.getType = function () {
                return this.type;
            };
            Value.prototype.isNotNull = function () {
                return !this.isNull();
            };
            Value.prototype.isNull = function () {
                return this.value == null || this.value == undefined;
            };
            Value.prototype.getObject = function () {
                return this.value;
            };
            Value.prototype.getString = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToString(this);
            };
            Value.prototype.isPropertySet = function () {
                return data.ValueTypes.DATA.toString() == this.type.toString();
            };
            Value.prototype.getPropertySet = function () {
                if (this.isNull()) {
                    return null;
                }
                api.util.assert(api.ObjectHelper.iFrameSafeInstanceOf(this.value, data.PropertySet), "Expected value to be a PropertySet: " + api.ClassHelper.getClassName(this.value));
                return this.value;
            };
            Value.prototype.getBoolean = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToBoolean(this);
            };
            Value.prototype.getLong = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToNumber(this);
            };
            Value.prototype.getDouble = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToNumber(this);
            };
            Value.prototype.getDateTime = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLocalDate = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLocalDateTime = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLocalTime = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getGeoPoint = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getBinaryReference = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getReference = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLink = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Value)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                return this.type.valueEquals(this.value, other.value);
            };
            Value.prototype.clone = function () {
                return new Value(this.value, this.type);
            };
            return Value;
        })();
        data.Value = Value;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyId = (function () {
            function PropertyId(value) {
                this.value = value;
            }
            PropertyId.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyId)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            PropertyId.prototype.toString = function () {
                return this.value;
            };
            return PropertyId;
        })();
        data.PropertyId = PropertyId;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyPath = (function () {
            function PropertyPath(elements, absolute) {
                if (absolute === void 0) { absolute = true; }
                this.absolute = absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.getName().length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? PropertyPath.ELEMENT_DIVIDER : "") + this.elements.join(PropertyPath.ELEMENT_DIVIDER);
            }
            PropertyPath.fromString = function (s) {
                var absolute = s.charAt(0) == PropertyPath.ELEMENT_DIVIDER;
                var dataPathElements = s.split(PropertyPath.ELEMENT_DIVIDER).filter(function (element) { return !!element; }).map(function (element) { return PropertyPathElement.fromString(element); });
                return new PropertyPath(dataPathElements, absolute);
            };
            PropertyPath.fromParent = function (parent) {
                var childElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    childElements[_i - 1] = arguments[_i];
                }
                var elements = parent.elements.slice(0).concat(childElements);
                return new PropertyPath(elements, parent.isAbsolute());
            };
            PropertyPath.fromPathElement = function (element) {
                return new PropertyPath([element], true);
            };
            PropertyPath.prototype.removeFirstPathElement = function () {
                api.util.assert(this.elements.length > 1, "Cannot create new path without first path element when path does not contain more than one element");
                return new PropertyPath(this.elements.slice(1), this.absolute);
            };
            PropertyPath.prototype.elementCount = function () {
                return this.getElements().length;
            };
            PropertyPath.prototype.getElements = function () {
                return this.elements;
            };
            PropertyPath.prototype.getElement = function (index) {
                return this.elements[index];
            };
            PropertyPath.prototype.getFirstElement = function () {
                return this.elements[0];
            };
            PropertyPath.prototype.getLastElement = function () {
                return this.elements[this.elements.length - 1];
            };
            PropertyPath.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            PropertyPath.prototype.getParentPath = function () {
                if (this.elements.length < 1) {
                    return null;
                }
                return new PropertyPath(this.elements.slice(0, -1));
            };
            PropertyPath.prototype.toString = function () {
                return this.refString;
            };
            PropertyPath.prototype.isAbsolute = function () {
                return this.absolute;
            };
            PropertyPath.prototype.asRelative = function () {
                return new PropertyPath(this.elements, false);
            };
            PropertyPath.prototype.isRoot = function () {
                return this.elementCount() == 0;
            };
            PropertyPath.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyPath)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            PropertyPath.ELEMENT_DIVIDER = ".";
            PropertyPath.ROOT = new PropertyPath([], true);
            return PropertyPath;
        })();
        data.PropertyPath = PropertyPath;
        var PropertyPathElement = (function () {
            function PropertyPathElement(name, index) {
                this.name = name;
                this.index = index;
            }
            PropertyPathElement.prototype.getName = function () {
                return this.name;
            };
            PropertyPathElement.prototype.getIndex = function () {
                return this.index;
            };
            PropertyPathElement.prototype.toString = function () {
                if (this.index == 0) {
                    return this.name;
                }
                else {
                    return this.name + "[" + this.index + "]";
                }
            };
            PropertyPathElement.fromString = function (str) {
                if (str.indexOf("[") == -1) {
                    return new PropertyPathElement(str, 0);
                }
                var name = str.substring(0, str.indexOf("["));
                var index = parseInt(str.substring(str.indexOf("[") + 1, str.indexOf("]")));
                return new PropertyPathElement(name, index);
            };
            return PropertyPathElement;
        })();
        data.PropertyPathElement = PropertyPathElement;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyEvent = (function () {
            function PropertyEvent(type, property) {
                this.type = type;
                this.property = property;
            }
            PropertyEvent.prototype.getType = function () {
                return this.type;
            };
            PropertyEvent.prototype.getProperty = function () {
                return this.property;
            };
            PropertyEvent.prototype.getPath = function () {
                return this.property.getPath();
            };
            PropertyEvent.prototype.toString = function () {
                return this.getPath().toString();
            };
            return PropertyEvent;
        })();
        data.PropertyEvent = PropertyEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyIndexChangedEvent = (function (_super) {
            __extends(PropertyIndexChangedEvent, _super);
            function PropertyIndexChangedEvent(property, previousIndex, newIndex) {
                _super.call(this, 1 /* INDEX_CHANGED */, property);
                this.previousIndex = previousIndex;
                this.newIndex = newIndex;
            }
            PropertyIndexChangedEvent.prototype.getPreviousIndex = function () {
                return this.previousIndex;
            };
            PropertyIndexChangedEvent.prototype.getNewIndex = function () {
                return this.newIndex;
            };
            PropertyIndexChangedEvent.prototype.toString = function () {
                return "[" + this.previousIndex + "] -> [" + this.newIndex + "]";
            };
            return PropertyIndexChangedEvent;
        })(data.PropertyEvent);
        data.PropertyIndexChangedEvent = PropertyIndexChangedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyValueChangedEvent = (function (_super) {
            __extends(PropertyValueChangedEvent, _super);
            function PropertyValueChangedEvent(property, previousValue, newValue) {
                _super.call(this, 2 /* VALUE_CHANGED */, property);
                this.previousValue = previousValue;
                this.newValue = newValue;
            }
            PropertyValueChangedEvent.prototype.getPreviousValue = function () {
                return this.previousValue;
            };
            PropertyValueChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            PropertyValueChangedEvent.prototype.toString = function () {
                return "[" + (this.previousValue ? this.previousValue.getObject() : null) + "] -> [" + (this.newValue ? this.newValue.getObject() : null) + "]";
            };
            return PropertyValueChangedEvent;
        })(data.PropertyEvent);
        data.PropertyValueChangedEvent = PropertyValueChangedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        (function (PropertyEventType) {
            PropertyEventType[PropertyEventType["ADDED"] = 0] = "ADDED";
            PropertyEventType[PropertyEventType["INDEX_CHANGED"] = 1] = "INDEX_CHANGED";
            PropertyEventType[PropertyEventType["VALUE_CHANGED"] = 2] = "VALUE_CHANGED";
            PropertyEventType[PropertyEventType["REMOVED"] = 3] = "REMOVED";
        })(data.PropertyEventType || (data.PropertyEventType = {}));
        var PropertyEventType = data.PropertyEventType;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyAddedEvent = (function (_super) {
            __extends(PropertyAddedEvent, _super);
            function PropertyAddedEvent(property) {
                _super.call(this, 0 /* ADDED */, property);
            }
            PropertyAddedEvent.prototype.toString = function () {
                var value = this.getProperty().getValue();
                return "" + this.getPath().toString() + " = " + (value.getObject() ? value.getObject().toString() : null);
            };
            return PropertyAddedEvent;
        })(data.PropertyEvent);
        data.PropertyAddedEvent = PropertyAddedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyRemovedEvent = (function (_super) {
            __extends(PropertyRemovedEvent, _super);
            function PropertyRemovedEvent(property) {
                _super.call(this, 3 /* REMOVED */, property);
            }
            PropertyRemovedEvent.prototype.toString = function () {
                var value = this.getProperty().getValue();
                return "" + this.getPath().toString() + " = " + (value.getObject() ? value.getObject().toString() : null);
            };
            return PropertyRemovedEvent;
        })(data.PropertyEvent);
        data.PropertyRemovedEvent = PropertyRemovedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var Property = (function () {
            function Property(builder) {
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
                api.util.assertNotNull(builder.array, "array of a Property cannot be null");
                api.util.assertNotNull(builder.name, "name of a Property cannot be null");
                api.util.assertNotNull(builder.index, "index of a Property cannot be null");
                api.util.assertNotNull(builder.value, "value of a Property cannot be null");
                this.array = builder.array;
                this.parent = builder.array.getParent();
                this.name = builder.name;
                this.index = builder.index;
                this.value = builder.value;
                this.id = builder.id;
                if (this.value.getType().equals(data.ValueTypes.DATA) && this.value.isNotNull()) {
                    var valuePropertySet = this.value.getPropertySet();
                    valuePropertySet.setContainerProperty(this);
                }
            }
            Property.prototype.setId = function (id) {
                api.util.assert(!(this.id && id), "The id of an Property cannot be changed.");
                this.id = id;
            };
            Property.prototype.setIndex = function (newIndex) {
                var oldIndex = newIndex;
                this.index = newIndex;
                if (oldIndex != newIndex) {
                    this.notifyPropertyIndexChangedEvent(oldIndex, newIndex);
                }
            };
            Property.prototype.setValue = function (value) {
                api.util.assertNotNull(value, "value of a Property cannot be null");
                var oldValue = this.value;
                this.value = value;
                if (this.value.getType().equals(data.ValueTypes.DATA) && this.value.isNotNull()) {
                    var propertySet = this.value.getPropertySet();
                    propertySet.setContainerProperty(this);
                    this.array.registerPropertySetListeners(propertySet);
                }
                if (oldValue.getType().equals(data.ValueTypes.DATA) && oldValue.isNotNull()) {
                    var removedPropertySet = oldValue.getPropertySet();
                    removedPropertySet.setContainerProperty(null);
                    this.array.unregisterPropertySetListeners(removedPropertySet);
                }
                if (!value.equals(oldValue)) {
                    this.notifyPropertyValueChangedEvent(oldValue, value);
                }
            };
            Property.prototype.detach = function () {
                this.array = null;
                this.parent = null;
                this.id = null;
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
            };
            Property.prototype.getId = function () {
                return this.id;
            };
            Property.prototype.getParent = function () {
                return this.parent;
            };
            Property.prototype.hasParentProperty = function () {
                return !!this.getParentProperty();
            };
            Property.prototype.getParentProperty = function () {
                return this.parent.getProperty();
            };
            Property.prototype.getPath = function () {
                if (this.hasParentProperty()) {
                    return data.PropertyPath.fromParent(this.getParentProperty().getPath(), new data.PropertyPathElement(this.name, this.index));
                }
                else {
                    return data.PropertyPath.fromPathElement(new data.PropertyPathElement(this.name, this.index));
                }
            };
            Property.prototype.getName = function () {
                return this.name;
            };
            Property.prototype.getIndex = function () {
                return this.index;
            };
            Property.prototype.getType = function () {
                return this.value.getType();
            };
            Property.prototype.getValue = function () {
                return this.value;
            };
            Property.prototype.hasNullValue = function () {
                return this.value.isNull();
            };
            Property.prototype.hasNonNullValue = function () {
                return !this.value.isNull();
            };
            Property.prototype.getPropertySet = function () {
                return this.value.getPropertySet();
            };
            Property.prototype.getString = function () {
                return this.value.getString();
            };
            Property.prototype.getLong = function () {
                return this.value.getLong();
            };
            Property.prototype.getDouble = function () {
                return this.value.getDouble();
            };
            Property.prototype.getBoolean = function () {
                return this.value.getBoolean();
            };
            Property.prototype.getDateTime = function () {
                return this.value.getDateTime();
            };
            Property.prototype.getLocalDate = function () {
                return this.value.getLocalDate();
            };
            Property.prototype.getLocalDateTime = function () {
                return this.value.getLocalDateTime();
            };
            Property.prototype.getLocalTime = function () {
                return this.value.getLocalTime();
            };
            Property.prototype.getGeoPoint = function () {
                return this.value.getGeoPoint();
            };
            Property.prototype.getReference = function () {
                return this.value.getReference();
            };
            Property.prototype.getBinaryReference = function () {
                return this.value.getBinaryReference();
            };
            Property.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Property)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.id, other.id)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.index, other.index)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            Property.prototype.copy = function (destinationPropertyArray, generateNewPropertyId) {
                if (generateNewPropertyId === void 0) { generateNewPropertyId = false; }
                var destinationTree = destinationPropertyArray.getTree();
                var id = null;
                if (generateNewPropertyId) {
                    id = destinationTree ? destinationTree.getNextId() : null;
                }
                else {
                    id = this.id;
                }
                var value;
                if (this.value.isPropertySet() && this.value.isNotNull()) {
                    var copiedPropertySet = this.value.getPropertySet().copy(destinationTree, generateNewPropertyId);
                    value = new data.Value(copiedPropertySet, data.ValueTypes.DATA);
                }
                else {
                    value = this.value;
                }
                var copy = Property.create().setId(id).setName(this.name).setValue(value).setIndex(this.index).setArray(destinationPropertyArray).build();
                return copy;
            };
            Property.prototype.onPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners.push(listener);
            };
            Property.prototype.unPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners = this.propertyIndexChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            Property.prototype.notifyPropertyIndexChangedEvent = function (previousIndex, newIndex) {
                var event = new data.PropertyIndexChangedEvent(this, previousIndex, newIndex);
                if (Property.debug) {
                    console.debug("Property[" + this.getPath().toString() + "].notifyPropertyIndexChangedEvent: " + event.toString());
                }
                this.propertyIndexChangedListeners.forEach(function (listener) { return listener(event); });
            };
            Property.prototype.onPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners.push(listener);
            };
            Property.prototype.unPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            Property.prototype.notifyPropertyValueChangedEvent = function (previousValue, newValue) {
                var event = new data.PropertyValueChangedEvent(this, previousValue, newValue);
                if (Property.debug) {
                    console.debug("Property[" + this.getPath().toString() + "].notifyPropertyValueChangedEvent: " + event.toString());
                }
                this.propertyValueChangedListeners.forEach(function (listener) { return listener(event); });
            };
            Property.checkName = function (name) {
                if (name == null) {
                    throw new Error("Property name cannot be null");
                }
                if (api.util.StringHelper.isBlank(name)) {
                    throw new Error("Property name cannot be blank");
                }
                if (name.indexOf(".") >= 0) {
                    throw new Error("Property name cannot contain .");
                }
                if (name.indexOf("[") >= 0 || name.indexOf("]") >= 0) {
                    throw new Error("Property name cannot contain [ or ]");
                }
            };
            Property.create = function () {
                return new PropertyBuilder();
            };
            Property.debug = false;
            return Property;
        })();
        data.Property = Property;
        var PropertyBuilder = (function () {
            function PropertyBuilder() {
            }
            PropertyBuilder.prototype.setArray = function (value) {
                this.array = value;
                return this;
            };
            PropertyBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            PropertyBuilder.prototype.setIndex = function (value) {
                this.index = value;
                return this;
            };
            PropertyBuilder.prototype.setValue = function (value) {
                this.value = value;
                return this;
            };
            PropertyBuilder.prototype.setId = function (value) {
                this.id = value;
                return this;
            };
            PropertyBuilder.prototype.build = function () {
                return new Property(this);
            };
            return PropertyBuilder;
        })();
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyVisitor = (function () {
            function PropertyVisitor() {
            }
            PropertyVisitor.prototype.restrictType = function (value) {
                this.valueType = value;
                return this;
            };
            PropertyVisitor.prototype.traverse = function (propertySet) {
                var _this = this;
                propertySet.forEach(function (property, index) {
                    if (_this.valueType == null || _this.valueType == property.getType()) {
                        _this.visit(property);
                    }
                    if (property.getType().equals(data.ValueTypes.DATA)) {
                        if (property.hasNonNullValue()) {
                            _this.traverse(property.getPropertySet());
                        }
                    }
                });
            };
            PropertyVisitor.prototype.visit = function (property) {
                throw new Error("Must be implemented by inheritor");
            };
            return PropertyVisitor;
        })();
        data.PropertyVisitor = PropertyVisitor;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyTreeComparator = (function () {
            function PropertyTreeComparator() {
            }
            PropertyTreeComparator.prototype.compareTree = function (treeA, treeB) {
                if (!treeA || !treeB) {
                    return;
                }
                this.compareSet(treeA.getRoot(), treeB.getRoot());
            };
            PropertyTreeComparator.prototype.compareSet = function (setA, setB) {
                var _this = this;
                setA.forEach(function (propertyA, index) {
                    var propertyB = setB.getProperty(propertyA.getName(), index);
                    if (!propertyB) {
                        console.log("Property A[" + propertyA.getPath().toString() + "] does not exist in B[" + setB.getPropertyPath().toString() + "]");
                    }
                    if (!propertyA.equals(propertyB)) {
                        console.log("Property A[" + propertyA.getPath().toString() + "] is not equal B[" + propertyB.getPath().toString() + "]");
                    }
                    if (propertyA.getValue().isPropertySet()) {
                        var childSetA = propertyA.getPropertySet();
                        var childSetB = propertyB.getPropertySet();
                        _this.compareSet(childSetA, childSetB);
                    }
                });
                setB.forEach(function (propertyB, index) {
                    var propertyA = setA.getProperty(propertyB.getName(), index);
                    if (!propertyA) {
                        console.log("Property B[" + propertyB.getPath().toString() + "] does not exist in A[" + setA.getPropertyPath().toString() + "]");
                    }
                });
            };
            return PropertyTreeComparator;
        })();
        data.PropertyTreeComparator = PropertyTreeComparator;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyArray = (function () {
            function PropertyArray(builder) {
                var _this = this;
                this.propertyAddedListeners = [];
                this.propertyRemovedListeners = [];
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
                this.tree = builder.parent.getTree();
                this.parent = builder.parent;
                this.name = builder.name;
                this.type = builder.type;
                this.array = [];
                this.propertyAddedEventHandler = function (event) {
                    _this.forwardPropertyAddedEvent(event);
                };
                this.propertyRemovedEventHandler = function (event) {
                    _this.forwardPropertyRemovedEvent(event);
                };
                this.propertyIndexChangedEventHandler = function (event) {
                    _this.forwardPropertyIndexChangedEvent(event);
                };
                this.propertyValueChangedEventHandler = function (event) {
                    _this.forwardPropertyValueChangedEvent(event);
                };
            }
            PropertyArray.prototype.forEach = function (callBack) {
                this.array.forEach(function (property, index) {
                    callBack(property, index);
                });
            };
            PropertyArray.prototype.getTree = function () {
                return this.tree;
            };
            PropertyArray.prototype.getParent = function () {
                return this.parent;
            };
            PropertyArray.prototype.getParentPropertyPath = function () {
                if (this.parent.getProperty() == null) {
                    return data.PropertyPath.ROOT;
                }
                return this.parent.getProperty().getPath();
            };
            PropertyArray.prototype.getName = function () {
                return this.name;
            };
            PropertyArray.prototype.getType = function () {
                return this.type;
            };
            PropertyArray.prototype.checkType = function (type) {
                if (!this.type.equals(type)) {
                    throw new Error("This PropertyArray expects only properties with value of type '" + this.type + "', got: " + type);
                }
            };
            PropertyArray.prototype.checkIndex = function (index) {
                if (!(index <= this.array.length)) {
                    throw new Error("Index out of bounds: index: " + index + ", size: " + this.array.length);
                }
            };
            PropertyArray.prototype.newSet = function () {
                return this.parent.newSet();
            };
            PropertyArray.prototype.addProperty = function (property) {
                api.util.assert(property.getName() == this.name, "Expected name of added Property to be [" + this.name + "], got: " + property.getName());
                api.util.assert(property.getType().equals(this.getType()), "Expected type of added Property to be [" + this.type.toString() + "], got: " + property.getType().toString());
                api.util.assert(property.getIndex() == this.array.length, "Expected index of added Property to be [" + this.array.length + "], got: " + property.getIndex());
                this.array.push(property);
                this.registerPropertyListeners(property);
            };
            PropertyArray.prototype.add = function (value) {
                this.checkType(value.getType());
                var property = data.Property.create().setArray(this).setName(this.name).setIndex(this.array.length).setValue(value).setId(this.tree ? this.tree.getNextId() : null).build();
                this.array.push(property);
                if (this.tree) {
                    this.tree.registerProperty(property);
                    if (this.type.equals(data.ValueTypes.DATA) && value.isNotNull()) {
                        var addedPropertySet = value.getPropertySet();
                        if (addedPropertySet && addedPropertySet.isDetached()) {
                            addedPropertySet.attachToTree(this.tree);
                        }
                    }
                }
                this.notifyPropertyAdded(property);
                this.registerPropertyListeners(property);
                return property;
            };
            PropertyArray.prototype.addSet = function () {
                var newSet = this.parent.newSet();
                this.add(new data.Value(newSet, data.ValueTypes.DATA));
                return newSet;
            };
            PropertyArray.prototype.set = function (index, value) {
                this.checkType(value.getType());
                this.checkIndex(index);
                var property;
                if (this.get(index) != null) {
                    property = this.array[index];
                    property.setValue(value);
                }
                else {
                    property = data.Property.create().setArray(this).setName(this.name).setIndex(this.array.length).setValue(value).setId(this.tree ? this.tree.getNextId() : null).build();
                    this.array[index] = property;
                    if (this.tree) {
                        this.tree.registerProperty(property);
                    }
                    this.notifyPropertyAdded(property);
                    this.registerPropertyListeners(property);
                }
                return property;
            };
            PropertyArray.prototype.move = function (index, destinationIndex) {
                var toBeMoved = this.array[index];
                api.util.ArrayHelper.moveElement(index, destinationIndex, this.array);
                toBeMoved.setIndex(destinationIndex);
                this.forEach(function (property, index) {
                    property.setIndex(index);
                });
            };
            PropertyArray.prototype.remove = function (index) {
                var propertyToRemove = this.get(index);
                if (!propertyToRemove) {
                    throw new Error("Property not found: " + data.PropertyPath.fromParent(this.getParentPropertyPath(), new data.PropertyPathElement(name, index)));
                }
                this.array.splice(index, 1);
                this.forEach(function (property, index) {
                    property.setIndex(index);
                });
                if (this.tree) {
                    this.tree.unregisterProperty(propertyToRemove.getId());
                }
                this.notifyPropertyRemoved(propertyToRemove);
                this.unregisterPropertyListeners(propertyToRemove);
                propertyToRemove.detach();
            };
            PropertyArray.prototype.exists = function (index) {
                return !!this.get(index);
            };
            PropertyArray.prototype.get = function (index) {
                if (index >= this.array.length) {
                    return null;
                }
                return this.array[index];
            };
            PropertyArray.prototype.getValue = function (index) {
                var property = this.get(index);
                return !property ? null : property.getValue();
            };
            PropertyArray.prototype.getSet = function (index) {
                var property = this.get(index);
                return !property ? null : property.getPropertySet();
            };
            PropertyArray.prototype.getSize = function () {
                return this.array.length;
            };
            PropertyArray.prototype.getProperties = function () {
                return this.array.slice(0);
            };
            PropertyArray.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyArray)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                if (!api.ObjectHelper.arrayEquals(this.array, other.array)) {
                    return false;
                }
                return true;
            };
            PropertyArray.prototype.copy = function (destinationPropertySet, generateNewPropertyIds) {
                if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                var copy = PropertyArray.create().setName(this.name).setType(this.type).setParent(destinationPropertySet).build();
                this.array.forEach(function (sourceProperty) {
                    copy.addProperty(sourceProperty.copy(copy, generateNewPropertyIds));
                });
                return copy;
            };
            PropertyArray.prototype.registerPropertyListeners = function (property) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].registerPropertyListeners: " + property.getPath().toString());
                }
                if (this.type.equals(data.ValueTypes.DATA) && property.hasNonNullValue()) {
                    this.registerPropertySetListeners(property.getPropertySet());
                }
                property.onPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                property.onPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertyArray.prototype.unregisterPropertyListeners = function (property) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].unregisterPropertyListeners: " + property.getPath().toString());
                }
                property.unPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                property.unPropertyValueChanged(this.propertyValueChangedEventHandler);
                if (property.hasNonNullValue() && property.getType().equals(data.ValueTypes.DATA)) {
                    var propertySet = property.getPropertySet();
                    this.unregisterPropertySetListeners(propertySet);
                }
            };
            PropertyArray.prototype.registerPropertySetListeners = function (propertySet) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].registerPropertySetListeners: " + propertySet.getPropertyPath().toString());
                }
                propertySet.onPropertyAdded(this.propertyAddedEventHandler);
                propertySet.onPropertyRemoved(this.propertyRemovedEventHandler);
                propertySet.onPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                propertySet.onPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertyArray.prototype.unregisterPropertySetListeners = function (propertySet) {
                propertySet.unPropertyAdded(this.propertyAddedEventHandler);
                propertySet.unPropertyRemoved(this.propertyRemovedEventHandler);
                propertySet.unPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                propertySet.unPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertyArray.prototype.onPropertyAdded = function (listener) {
                this.propertyAddedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyAdded = function (listener) {
                this.propertyAddedListeners = this.propertyAddedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.notifyPropertyAdded = function (property) {
                var event = new data.PropertyAddedEvent(property);
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].notifyPropertyAdded: " + event.toString());
                }
                this.propertyAddedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.forwardPropertyAddedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyAddedEvent: " + event.toString());
                }
                this.propertyAddedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.onPropertyRemoved = function (listener) {
                this.propertyRemovedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyRemoved = function (listener) {
                this.propertyRemovedListeners = this.propertyRemovedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.notifyPropertyRemoved = function (property) {
                var event = new data.PropertyRemovedEvent(property);
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].notifyPropertyRemoved: " + event.toString());
                }
                this.propertyRemovedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.forwardPropertyRemovedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyRemovedEvent: " + event.toString());
                }
                this.propertyRemovedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.onPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners = this.propertyIndexChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.forwardPropertyIndexChangedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyIndexChangedEvent: " + event.toString());
                }
                this.propertyIndexChangedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.onPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.forwardPropertyValueChangedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyValueChangedEvent: : " + event.toString());
                }
                this.propertyValueChangedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.create = function () {
                return new PropertyArrayBuilder();
            };
            PropertyArray.prototype.toJson = function () {
                var _this = this;
                var valuesJson = [];
                this.array.forEach(function (property) {
                    if (_this.type.equals(data.ValueTypes.DATA)) {
                        var valueSetJson = property.hasNullValue() ? null : property.getPropertySet().toJson();
                        valuesJson.push({
                            id: property.getId().toString(),
                            set: valueSetJson
                        });
                    }
                    else {
                        var valueJson = _this.type.toJsonValue(property.getValue());
                        valuesJson.push({
                            id: property.getId().toString(),
                            v: valueJson
                        });
                    }
                });
                return {
                    name: this.name,
                    type: this.type.toString(),
                    values: valuesJson
                };
            };
            PropertyArray.prototype.idForDebug = function () {
                return this.getParentPropertyPath().toString() + "." + this.getName();
            };
            PropertyArray.fromJson = function (json, parentPropertySet, tree) {
                var type = data.ValueTypes.fromName(json.type);
                var array = new PropertyArrayBuilder().setName(json.name).setType(type).setParent(parentPropertySet).build();
                json.values.forEach(function (valueAndPropertyIdJson, index) {
                    var value;
                    if (type.equals(data.ValueTypes.DATA)) {
                        var valueAsPropertySet = tree.newPropertySet();
                        var propertyArrayJsonArray = valueAndPropertyIdJson.set;
                        propertyArrayJsonArray.forEach(function (propertyArrayJson) {
                            valueAsPropertySet.addPropertyArray(PropertyArray.fromJson(propertyArrayJson, valueAsPropertySet, tree));
                        });
                        value = new data.Value(valueAsPropertySet, data.ValueTypes.DATA);
                    }
                    else {
                        value = type.fromJsonValue(valueAndPropertyIdJson.v);
                    }
                    var property = data.Property.create().setId(new data.PropertyId(valueAndPropertyIdJson.id)).setArray(array).setName(json.name).setIndex(index).setValue(value).build();
                    array.addProperty(property);
                });
                return array;
            };
            PropertyArray.debug = false;
            return PropertyArray;
        })();
        data.PropertyArray = PropertyArray;
        var PropertyArrayBuilder = (function () {
            function PropertyArrayBuilder() {
            }
            PropertyArrayBuilder.prototype.setParent = function (value) {
                this.parent = value;
                return this;
            };
            PropertyArrayBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            PropertyArrayBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            PropertyArrayBuilder.prototype.build = function () {
                return new PropertyArray(this);
            };
            return PropertyArrayBuilder;
        })();
        data.PropertyArrayBuilder = PropertyArrayBuilder;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertySet = (function () {
            function PropertySet(tree) {
                var _this = this;
                this.tree = null;
                this.property = null;
                this.propertyArrayByName = {};
                this._ifNotNull = false;
                this.changedListeners = [];
                this.propertyAddedListeners = [];
                this.propertyRemovedListeners = [];
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
                this.tree = tree;
                this.propertyAddedEventHandler = function (event) {
                    _this.forwardPropertyAddedEvent(event);
                };
                this.propertyRemovedEventHandler = function (event) {
                    _this.forwardPropertyRemovedEvent(event);
                };
                this.propertyIndexChangedEventHandler = function (event) {
                    _this.forwardPropertyIndexChangedEvent(event);
                };
                this.propertyValueChangedEventHandler = function (event) {
                    _this.forwardPropertyValueChangedEvent(event);
                };
            }
            PropertySet.prototype.setContainerProperty = function (value) {
                this.property = value;
            };
            PropertySet.prototype.isDetached = function () {
                return !this.tree;
            };
            PropertySet.prototype.getTree = function () {
                return this.tree;
            };
            PropertySet.prototype.attachToTree = function (tree) {
                var _this = this;
                this.tree = tree;
                this.forEach(function (property) {
                    property.setId(_this.tree.getNextId());
                    _this.tree.registerProperty(property);
                    if (property.hasNonNullValue() && property.getType().equals(data.ValueTypes.DATA)) {
                        property.getPropertySet().attachToTree(tree);
                    }
                });
            };
            PropertySet.prototype.ifNotNull = function () {
                this._ifNotNull = true;
                return this;
            };
            PropertySet.prototype.addPropertyArray = function (array) {
                api.util.assertState(this.tree === array.getTree(), "Added PropertyArray must be attached to the same PropertyTree as this PropertySet");
                api.util.assert(this == array.getParent(), "propertyArray must have this PropertySet as parent");
                this.propertyArrayByName[array.getName()] = array;
                this.registerPropertyArrayListeners(array);
            };
            PropertySet.prototype.addProperty = function (name, value) {
                if (this._ifNotNull && value.isNull()) {
                    this._ifNotNull = false;
                    return null;
                }
                var array = this.getOrCreatePropertyArray(name, value.getType());
                var property = array.add(value);
                return property;
            };
            PropertySet.prototype.setPropertyByPath = function (path, value) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(path, data.PropertyPath)) {
                    return this.doSetProperty(path, value);
                }
                else {
                    return this.doSetProperty(data.PropertyPath.fromString(path.toString()), value);
                }
            };
            PropertySet.prototype.doSetProperty = function (path, value) {
                var firstPathElement = path.getFirstElement();
                if (path.elementCount() > 1) {
                    var propertySet = this.getOrCreateSet(firstPathElement.getName(), firstPathElement.getIndex());
                    return propertySet.setPropertyByPath(path.removeFirstPathElement(), value);
                }
                else {
                    return this.setProperty(firstPathElement.getName(), firstPathElement.getIndex(), value);
                }
            };
            PropertySet.prototype.getOrCreateSet = function (name, index) {
                var existingProperty = this.getProperty(name, index);
                if (!existingProperty) {
                    var newSet = this.tree ? new PropertySet(this.tree) : new PropertySet();
                    this.setProperty(name, index, new data.Value(newSet, data.ValueTypes.DATA));
                    return newSet;
                }
                else {
                    return existingProperty.getPropertySet();
                }
            };
            PropertySet.prototype.setProperty = function (name, index, value) {
                var array = this.getOrCreatePropertyArray(name, value.getType());
                return array.set(index, value);
            };
            PropertySet.prototype.getOrCreatePropertyArray = function (name, type) {
                var array = this.propertyArrayByName[name];
                if (!array) {
                    array = data.PropertyArray.create().setParent(this).setName(name).setType(type).build();
                    this.propertyArrayByName[name] = array;
                    this.registerPropertyArrayListeners(array);
                }
                return array;
            };
            PropertySet.prototype.removeProperty = function (name, index) {
                var array = this.propertyArrayByName[name];
                if (array) {
                    array.remove(index);
                }
            };
            PropertySet.prototype.getSize = function () {
                var size = 0;
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    size += propertyArray.getSize();
                });
                return size;
            };
            PropertySet.prototype.countProperties = function (name) {
                var array = this.propertyArrayByName[name];
                if (!array) {
                    return 0;
                }
                return array.getSize();
            };
            PropertySet.prototype.getPropertyPath = function () {
                return !this.property ? data.PropertyPath.ROOT : this.property.getPath();
            };
            PropertySet.prototype.getProperty = function (identifier, index) {
                if (identifier == undefined && index == undefined) {
                    return this.property;
                }
                else if (index != undefined) {
                    data.Property.checkName(identifier);
                    var array = this.propertyArrayByName[identifier];
                    if (!array) {
                        return null;
                    }
                    return array.get(index);
                }
                else {
                    return this.getPropertyByPath(identifier);
                }
            };
            PropertySet.prototype.getPropertyByPath = function (path) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(path, data.PropertyPath)) {
                    return this.doGetPropertyByPath(path);
                }
                else {
                    return this.doGetPropertyByPath(data.PropertyPath.fromString(path.toString()));
                }
            };
            PropertySet.prototype.doGetPropertyByPath = function (path) {
                var firstElement = path.getFirstElement();
                if (path.elementCount() > 1) {
                    var property = this.getProperty(firstElement.getName(), firstElement.getIndex());
                    if (!property) {
                        return null;
                    }
                    var propertySet = property.getPropertySet();
                    return propertySet.getPropertyByPath(path.removeFirstPathElement());
                }
                else {
                    return this.getProperty(firstElement.getName(), firstElement.getIndex());
                }
            };
            PropertySet.prototype.getPropertyArray = function (name) {
                return this.propertyArrayByName[name];
            };
            PropertySet.prototype.forEach = function (callback) {
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    propertyArray.forEach(function (property, index) {
                        callback(property, index);
                    });
                });
            };
            PropertySet.prototype.forEachProperty = function (propertyName, callback) {
                var array = this.getPropertyArray(propertyName);
                if (array) {
                    array.forEach(callback);
                }
            };
            PropertySet.prototype.isNotNull = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                if (property == null) {
                    return false;
                }
                return !property.hasNullValue();
            };
            PropertySet.prototype.isNull = function (identifier, index) {
                return !this.isNotNull(identifier, index);
            };
            PropertySet.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertySet)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.mapEquals(this.propertyArrayByName, other.propertyArrayByName)) {
                    return false;
                }
                return true;
            };
            PropertySet.prototype.toTree = function (idProvider) {
                if (!this.tree && !idProvider) {
                    throw new Error("The PropertySet must be attached to a PropertyTree or a idProvider must be given before when this method is invoked");
                }
                if (idProvider) {
                    return new data.PropertyTree(idProvider, this);
                }
                else {
                    return new data.PropertyTree(this.tree.getIdProvider(), this);
                }
            };
            PropertySet.prototype.copy = function (destinationTree, generateNewPropertyIds) {
                if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                var copy = new PropertySet(destinationTree);
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, sourcePropertyArray) {
                    var propertyArrayCopy = sourcePropertyArray.copy(copy, generateNewPropertyIds);
                    copy.addPropertyArray(propertyArrayCopy);
                });
                return copy;
            };
            PropertySet.prototype.toJson = function () {
                var jsonArray = [];
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    jsonArray.push(propertyArray.toJson());
                });
                return jsonArray;
            };
            PropertySet.prototype.registerPropertyArrayListeners = function (array) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].registerPropertyArrayListeners: " + array.getName());
                }
                array.onPropertyAdded(this.propertyAddedEventHandler);
                array.onPropertyRemoved(this.propertyRemovedEventHandler);
                array.onPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                array.onPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertySet.prototype.unregisterPropertyArrayListeners = function (array) {
                array.unPropertyAdded(this.propertyAddedEventHandler);
                array.unPropertyRemoved(this.propertyRemovedEventHandler);
                array.unPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                array.unPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertySet.prototype.onChanged = function (listener) {
                this.changedListeners.push(listener);
            };
            PropertySet.prototype.unChanged = function (listener) {
                this.changedListeners = this.changedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.notifyChangedListeners = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].notifyChangedListeners: " + event.toString());
                }
                this.changedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertySet.prototype.onPropertyAdded = function (listener) {
                this.propertyAddedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyAdded = function (listener) {
                this.propertyAddedListeners = this.propertyAddedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyAddedEvent = function (event) {
                this.propertyAddedListeners.forEach(function (listener) { return listener(event); });
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyAddedEvent: " + event.toString());
                }
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.onPropertyRemoved = function (listener) {
                this.propertyRemovedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyRemoved = function (listener) {
                this.propertyRemovedListeners = this.propertyRemovedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyRemovedEvent = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyRemovedEvent: " + event.toString());
                }
                this.propertyRemovedListeners.forEach(function (listener) { return listener(event); });
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.onPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners = this.propertyIndexChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyIndexChangedEvent = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyIndexChangedEvent: " + event.toString());
                }
                this.propertyIndexChangedListeners.forEach(function (listener) { return listener(event); });
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.onPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyValueChangedEvent = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyValueChangedEvent: " + event.toString());
                }
                this.propertyValueChangedListeners.forEach(function (listener) { return listener(event); });
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.newSet = function () {
                if (!this.tree) {
                    throw new Error("The PropertySet must be attached to a PropertyTree before this method can be invoked. Use PropertySet constructor with no arguments instead.");
                }
                return this.tree.newPropertySet();
            };
            PropertySet.prototype.addPropertySet = function (name, value) {
                if (!value) {
                    if (!this.tree) {
                        throw new Error("The PropertySet must be attached to a PropertyTree before this method can be invoked. Use PropertySet constructor with no arguments instead.");
                    }
                    value = this.tree.newPropertySet();
                }
                this.addProperty(name, new data.Value(value, data.ValueTypes.DATA));
                return value;
            };
            PropertySet.prototype.setPropertySet = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.DATA));
            };
            PropertySet.prototype.setPropertySetByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.DATA));
            };
            PropertySet.prototype.getPropertySet = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getPropertySet();
            };
            PropertySet.prototype.getPropertySets = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getPropertySet());
                });
                return values;
            };
            PropertySet.prototype.addString = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.STRING));
            };
            PropertySet.prototype.addStrings = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addString(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setString = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.STRING));
            };
            PropertySet.prototype.setStringByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.STRING));
            };
            PropertySet.prototype.getString = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getString();
            };
            PropertySet.prototype.getStrings = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getString());
                });
                return values;
            };
            PropertySet.prototype.addLong = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LONG));
            };
            PropertySet.prototype.addLongs = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLong(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLong = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LONG));
            };
            PropertySet.prototype.setLongByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LONG));
            };
            PropertySet.prototype.getLong = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLong();
            };
            PropertySet.prototype.getLongs = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLong());
                });
                return values;
            };
            PropertySet.prototype.addDouble = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.DOUBLE));
            };
            PropertySet.prototype.addDoubles = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addDouble(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setDouble = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.DOUBLE));
            };
            PropertySet.prototype.setDoubleByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.DOUBLE));
            };
            PropertySet.prototype.getDouble = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDouble();
            };
            PropertySet.prototype.getDoubles = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDouble());
                });
                return values;
            };
            PropertySet.prototype.addBoolean = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.BOOLEAN));
            };
            PropertySet.prototype.addBooleans = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addBoolean(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setBoolean = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.BOOLEAN));
            };
            PropertySet.prototype.setBooleanByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.BOOLEAN));
            };
            PropertySet.prototype.getBoolean = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBoolean();
            };
            PropertySet.prototype.getBooleans = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBoolean());
                });
                return values;
            };
            PropertySet.prototype.addReference = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.REFERENCE));
            };
            PropertySet.prototype.addReferences = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addReference(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setReference = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.REFERENCE));
            };
            PropertySet.prototype.setReferenceByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.REFERENCE));
            };
            PropertySet.prototype.getReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getReference();
            };
            PropertySet.prototype.getReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getReference());
                });
                return values;
            };
            PropertySet.prototype.addBinaryReference = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.BINARY_REFERENCE));
            };
            PropertySet.prototype.addBinaryReferences = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addBinaryReference(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setBinaryReference = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.BINARY_REFERENCE));
            };
            PropertySet.prototype.setBinaryReferenceByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.BINARY_REFERENCE));
            };
            PropertySet.prototype.getBinaryReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBinaryReference();
            };
            PropertySet.prototype.getBinaryReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBinaryReference());
                });
                return values;
            };
            PropertySet.prototype.addGeoPoint = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.GEO_POINT));
            };
            PropertySet.prototype.addGeoPoints = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addGeoPoint(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setGeoPoint = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.GEO_POINT));
            };
            PropertySet.prototype.setGeoPointByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.GEO_POINT));
            };
            PropertySet.prototype.getGeoPoint = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getGeoPoint();
            };
            PropertySet.prototype.getGeoPoints = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getGeoPoint());
                });
                return values;
            };
            PropertySet.prototype.addLocalDate = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LOCAL_DATE));
            };
            PropertySet.prototype.addLocalDates = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLocalDate(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLocalDate = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LOCAL_DATE));
            };
            PropertySet.prototype.setLocalDateByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LOCAL_DATE));
            };
            PropertySet.prototype.getLocalDate = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDate();
            };
            PropertySet.prototype.getLocalDates = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDate());
                });
                return values;
            };
            PropertySet.prototype.addLocalDateTime = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LOCAL_DATE_TIME));
            };
            PropertySet.prototype.addLocalDateTimes = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLocalDateTime(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLocalDateTime = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LOCAL_DATE_TIME));
            };
            PropertySet.prototype.setLocalDateTimeByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LOCAL_DATE_TIME));
            };
            PropertySet.prototype.getLocalDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDateTime();
            };
            PropertySet.prototype.getLocalDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDateTime());
                });
                return values;
            };
            PropertySet.prototype.addLocalTime = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LOCAL_TIME));
            };
            PropertySet.prototype.addLocalTimes = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLocalTime(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLocalTime = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LOCAL_TIME));
            };
            PropertySet.prototype.setLocalTimeByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LOCAL_TIME));
            };
            PropertySet.prototype.getLocalTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalTime();
            };
            PropertySet.prototype.getLocalTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalTime());
                });
                return values;
            };
            PropertySet.prototype.addDateTime = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.DATE_TIME));
            };
            PropertySet.prototype.addDateTimes = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addDateTime(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setDateTime = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.DATE_TIME));
            };
            PropertySet.prototype.setDateTimeByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.DATE_TIME));
            };
            PropertySet.prototype.getDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDateTime();
            };
            PropertySet.prototype.getDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDateTime());
                });
                return values;
            };
            PropertySet.debug = false;
            return PropertySet;
        })();
        data.PropertySet = PropertySet;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var DefaultPropertyIdProvider = (function () {
            function DefaultPropertyIdProvider(seed) {
                this.nextId = 1;
                this.seed = seed;
            }
            DefaultPropertyIdProvider.prototype.getNextId = function () {
                var seedPrefix = (!this.seed ? "" : this.seed + "-");
                return new data.PropertyId(seedPrefix + (this.nextId++));
            };
            return DefaultPropertyIdProvider;
        })();
        data.DefaultPropertyIdProvider = DefaultPropertyIdProvider;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyRegistrar = (function (_super) {
            __extends(PropertyRegistrar, _super);
            function PropertyRegistrar(tree) {
                this.tree = tree;
                _super.call(this);
            }
            PropertyRegistrar.prototype.visit = function (property) {
                this.tree.registerProperty(property);
            };
            return PropertyRegistrar;
        })(data.PropertyVisitor);
        var PropertyTree = (function () {
            function PropertyTree(idProvider, sourceRoot, generateNewPropertyIds) {
                this.propertyById = {};
                if (sourceRoot) {
                    this.idProvider = idProvider;
                    this.root = sourceRoot.copy(this, generateNewPropertyIds);
                    var propertyRegistrar = new PropertyRegistrar(this);
                    propertyRegistrar.traverse(this.root);
                }
                else {
                    if (!idProvider) {
                        this.idProvider = new data.DefaultPropertyIdProvider();
                    }
                    else {
                        this.idProvider = idProvider;
                    }
                    this.root = new data.PropertySet(this);
                }
            }
            PropertyTree.prototype.getIdProvider = function () {
                return this.idProvider;
            };
            PropertyTree.prototype.getNextId = function () {
                return this.idProvider.getNextId();
            };
            PropertyTree.prototype.getRoot = function () {
                return this.root;
            };
            PropertyTree.prototype.registerProperty = function (property) {
                api.util.assertNotNull(property.getId(), "Cannot register a Property without id");
                var existing = this.propertyById[property.getId().toString()];
                if (existing) {
                    throw new Error("Property with id [" + property.getId().toString() + "] already registered: " + property.getPath().toString());
                }
                this.propertyById[property.getId().toString()] = property;
            };
            PropertyTree.prototype.unregisterProperty = function (id) {
                delete this.propertyById[id.toString()];
            };
            PropertyTree.prototype.getTotalSize = function () {
                var size = 0, key;
                for (key in this.propertyById) {
                    if (this.propertyById.hasOwnProperty(key)) {
                        size++;
                    }
                }
                return size;
            };
            PropertyTree.prototype.addProperty = function (name, value) {
                return this.root.addProperty(name, value);
            };
            PropertyTree.prototype.setPropertyByPath = function (path, value) {
                return this.root.setPropertyByPath(path, value);
            };
            PropertyTree.prototype.setProperty = function (name, index, value) {
                return this.root.setProperty(name, index, value);
            };
            PropertyTree.prototype.removeProperty = function (name, index) {
                this.root.removeProperty(name, index);
            };
            PropertyTree.prototype.getPropertyById = function (id) {
                return this.propertyById[id.toString()];
            };
            PropertyTree.prototype.getProperty = function (identifier, index) {
                return this.root.getProperty(identifier, index);
            };
            PropertyTree.prototype.getPropertyArray = function (name) {
                return this.root.getPropertyArray(name);
            };
            PropertyTree.prototype.forEachProperty = function (name, callback) {
                this.root.forEachProperty(name, callback);
            };
            PropertyTree.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyTree)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.root, other.root)) {
                    return false;
                }
                return true;
            };
            PropertyTree.prototype.copy = function (generateNewPropertyIds) {
                if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                return new PropertyTree(this.getIdProvider(), this.getRoot(), generateNewPropertyIds);
            };
            PropertyTree.prototype.toJson = function () {
                return this.getRoot().toJson();
            };
            PropertyTree.prototype.onChanged = function (listener) {
                this.root.onChanged(listener);
            };
            PropertyTree.prototype.unChanged = function (listener) {
                this.root.unChanged(listener);
            };
            PropertyTree.prototype.onPropertyAdded = function (listener) {
                this.root.onPropertyAdded(listener);
            };
            PropertyTree.prototype.unPropertyAdded = function (listener) {
                this.root.unPropertyAdded(listener);
            };
            PropertyTree.prototype.onPropertyRemoved = function (listener) {
                this.root.onPropertyRemoved(listener);
            };
            PropertyTree.prototype.unPropertyRemoved = function (listener) {
                this.root.unPropertyRemoved(listener);
            };
            PropertyTree.prototype.onPropertyIndexChanged = function (listener) {
                this.root.onPropertyIndexChanged(listener);
            };
            PropertyTree.prototype.unPropertyIndexChanged = function (listener) {
                this.root.unPropertyIndexChanged(listener);
            };
            PropertyTree.prototype.onPropertyValueChanged = function (listener) {
                this.root.onPropertyValueChanged(listener);
            };
            PropertyTree.prototype.unPropertyValueChanged = function (listener) {
                this.root.unPropertyValueChanged(listener);
            };
            PropertyTree.fromJson = function (json, idProvider) {
                api.util.assertNotNull(json, "a json is required");
                var tree = new PropertyTree(idProvider);
                json.forEach(function (propertyArrayJson) {
                    tree.root.addPropertyArray(data.PropertyArray.fromJson(propertyArrayJson, tree.root, tree));
                });
                return tree;
            };
            PropertyTree.prototype.newPropertySet = function () {
                return new data.PropertySet(this);
            };
            PropertyTree.prototype.addPropertySet = function (name, value) {
                return this.root.addPropertySet(name, value);
            };
            PropertyTree.prototype.setPropertySet = function (name, index, value) {
                return this.root.setPropertySet(name, index, value);
            };
            PropertyTree.prototype.setPropertySetByPath = function (path, value) {
                return this.root.setPropertySetByPath(path, value);
            };
            PropertyTree.prototype.getPropertySet = function (identifier, index) {
                return this.root.getPropertySet(identifier, index);
            };
            PropertyTree.prototype.getPropertySets = function (name) {
                return this.root.getPropertySets(name);
            };
            PropertyTree.prototype.addString = function (name, value) {
                return this.root.addString(name, value);
            };
            PropertyTree.prototype.addStrings = function (name, values) {
                return this.root.addStrings(name, values);
            };
            PropertyTree.prototype.setString = function (name, index, value) {
                return this.root.setString(name, index, value);
            };
            PropertyTree.prototype.setStringByPath = function (path, value) {
                return this.root.setStringByPath(path, value);
            };
            PropertyTree.prototype.getString = function (identifier, index) {
                return this.root.getString(identifier, index);
            };
            PropertyTree.prototype.getStrings = function (name) {
                return this.root.getStrings(name);
            };
            PropertyTree.prototype.addLong = function (name, value) {
                return this.root.addLong(name, value);
            };
            PropertyTree.prototype.addLongs = function (name, values) {
                return this.root.addLongs(name, values);
            };
            PropertyTree.prototype.setLong = function (name, index, value) {
                return this.root.setLong(name, index, value);
            };
            PropertyTree.prototype.setLongByPath = function (path, value) {
                return this.root.setLongByPath(path, value);
            };
            PropertyTree.prototype.getLong = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLong();
            };
            PropertyTree.prototype.getLongs = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLong());
                });
                return values;
            };
            PropertyTree.prototype.addDouble = function (name, value) {
                return this.root.addDouble(name, value);
            };
            PropertyTree.prototype.addDoubles = function (name, values) {
                return this.root.addDoubles(name, values);
            };
            PropertyTree.prototype.setDouble = function (name, index, value) {
                return this.root.setDouble(name, index, value);
            };
            PropertyTree.prototype.setDoubleByPath = function (path, value) {
                return this.root.setDoubleByPath(path, value);
            };
            PropertyTree.prototype.getDouble = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDouble();
            };
            PropertyTree.prototype.getDoubles = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDouble());
                });
                return values;
            };
            PropertyTree.prototype.addBoolean = function (name, value) {
                return this.root.addBoolean(name, value);
            };
            PropertyTree.prototype.addBooleans = function (name, values) {
                return this.root.addBooleans(name, values);
            };
            PropertyTree.prototype.setBoolean = function (name, index, value) {
                return this.root.setBoolean(name, index, value);
            };
            PropertyTree.prototype.setBooleanByPath = function (path, value) {
                return this.root.setBooleanByPath(path, value);
            };
            PropertyTree.prototype.getBoolean = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBoolean();
            };
            PropertyTree.prototype.getBooleans = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBoolean());
                });
                return values;
            };
            PropertyTree.prototype.addReference = function (name, value) {
                return this.root.addReference(name, value);
            };
            PropertyTree.prototype.addReferences = function (name, values) {
                return this.root.addReferences(name, values);
            };
            PropertyTree.prototype.setReference = function (name, index, value) {
                return this.root.setReference(name, index, value);
            };
            PropertyTree.prototype.setReferenceByPath = function (path, value) {
                return this.root.setReferenceByPath(path, value);
            };
            PropertyTree.prototype.getReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getReference();
            };
            PropertyTree.prototype.getReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getReference());
                });
                return values;
            };
            PropertyTree.prototype.addBinaryReference = function (name, value) {
                return this.root.addBinaryReference(name, value);
            };
            PropertyTree.prototype.addBinaryReferences = function (name, values) {
                return this.root.addBinaryReferences(name, values);
            };
            PropertyTree.prototype.setBinaryReference = function (name, index, value) {
                return this.root.setBinaryReference(name, index, value);
            };
            PropertyTree.prototype.setBinaryReferenceByPath = function (path, value) {
                return this.root.setBinaryReferenceByPath(path, value);
            };
            PropertyTree.prototype.getBinaryReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBinaryReference();
            };
            PropertyTree.prototype.getBinaryReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBinaryReference());
                });
                return values;
            };
            PropertyTree.prototype.addGeoPoint = function (name, value) {
                return this.root.addGeoPoint(name, value);
            };
            PropertyTree.prototype.addGeoPoints = function (name, values) {
                return this.root.addGeoPoints(name, values);
            };
            PropertyTree.prototype.setGeoPoint = function (name, index, value) {
                return this.root.setGeoPoint(name, index, value);
            };
            PropertyTree.prototype.setGeoPointByPath = function (path, value) {
                return this.root.setGeoPointByPath(path, value);
            };
            PropertyTree.prototype.getGeoPoint = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getGeoPoint();
            };
            PropertyTree.prototype.getGeoPoints = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getGeoPoint());
                });
                return values;
            };
            PropertyTree.prototype.addLocalDate = function (name, value) {
                return this.root.addLocalDate(name, value);
            };
            PropertyTree.prototype.addLocalDates = function (name, values) {
                return this.root.addLocalDates(name, values);
            };
            PropertyTree.prototype.setLocalDate = function (name, index, value) {
                return this.root.setLocalDate(name, index, value);
            };
            PropertyTree.prototype.setLocalDateByPath = function (path, value) {
                return this.root.setLocalDateByPath(path, value);
            };
            PropertyTree.prototype.getLocalDate = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDate();
            };
            PropertyTree.prototype.getLocalDates = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDate());
                });
                return values;
            };
            PropertyTree.prototype.addLocalDateTime = function (name, value) {
                return this.root.addLocalDateTime(name, value);
            };
            PropertyTree.prototype.addLocalDateTimes = function (name, values) {
                return this.root.addLocalDateTimes(name, values);
            };
            PropertyTree.prototype.setLocalDateTime = function (name, index, value) {
                return this.root.setLocalDateTime(name, index, value);
            };
            PropertyTree.prototype.setLocalDateTimeByPath = function (path, value) {
                return this.root.setLocalDateTimeByPath(path, value);
            };
            PropertyTree.prototype.getLocalDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDateTime();
            };
            PropertyTree.prototype.getLocalDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDateTime());
                });
                return values;
            };
            PropertyTree.prototype.addLocalTime = function (name, value) {
                return this.root.addLocalTime(name, value);
            };
            PropertyTree.prototype.addLocalTimes = function (name, values) {
                return this.root.addLocalTimes(name, values);
            };
            PropertyTree.prototype.setLocalTime = function (name, index, value) {
                return this.root.setLocalTime(name, index, value);
            };
            PropertyTree.prototype.setLocalTimeByPath = function (path, value) {
                return this.root.setLocalTimeByPath(path, value);
            };
            PropertyTree.prototype.getLocalTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalTime();
            };
            PropertyTree.prototype.getLocalTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalTime());
                });
                return values;
            };
            PropertyTree.prototype.addDateTime = function (name, value) {
                return this.root.addDateTime(name, value);
            };
            PropertyTree.prototype.addDateTimes = function (name, values) {
                return this.root.addDateTimes(name, values);
            };
            PropertyTree.prototype.setDateTime = function (name, index, value) {
                return this.root.setDateTime(name, index, value);
            };
            PropertyTree.prototype.setDateTimeByPath = function (path, value) {
                return this.root.setDateTimeByPath(path, value);
            };
            PropertyTree.prototype.getDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDateTime();
            };
            PropertyTree.prototype.getDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDateTime());
                });
                return values;
            };
            return PropertyTree;
        })();
        data.PropertyTree = PropertyTree;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FormItemJson = (function () {
                function FormItemJson() {
                }
                return FormItemJson;
            })();
            json.FormItemJson = FormItemJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var OccurrencesJson = (function () {
                function OccurrencesJson() {
                }
                return OccurrencesJson;
            })();
            json.OccurrencesJson = OccurrencesJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var LayoutJson = (function (_super) {
                __extends(LayoutJson, _super);
                function LayoutJson() {
                    _super.apply(this, arguments);
                }
                return LayoutJson;
            })(json.FormItemJson);
            json.LayoutJson = LayoutJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FieldSetJson = (function (_super) {
                __extends(FieldSetJson, _super);
                function FieldSetJson() {
                    _super.apply(this, arguments);
                }
                return FieldSetJson;
            })(json.LayoutJson);
            json.FieldSetJson = FieldSetJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FormItemSetJson = (function (_super) {
                __extends(FormItemSetJson, _super);
                function FormItemSetJson() {
                    _super.apply(this, arguments);
                }
                return FormItemSetJson;
            })(json.FormItemJson);
            json.FormItemSetJson = FormItemSetJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var InputTypeJson = (function () {
                function InputTypeJson() {
                }
                return InputTypeJson;
            })();
            json.InputTypeJson = InputTypeJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var InputJson = (function (_super) {
                __extends(InputJson, _super);
                function InputJson() {
                    _super.apply(this, arguments);
                }
                return InputJson;
            })(json.FormItemJson);
            json.InputJson = InputJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FormJson = (function () {
                function FormJson() {
                }
                return FormJson;
            })();
            json.FormJson = FormJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemPath = (function () {
            function FormItemPath(elements, elementDivider, absolute) {
                this.elementDivider = elementDivider != null ? elementDivider : FormItemPath.DEFAULT_ELEMENT_DIVIDER;
                this.absolute = absolute == undefined ? true : absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.getName().length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? this.elementDivider : "") + this.elements.join(this.elementDivider);
            }
            FormItemPath.fromString = function (s, elementDivider) {
                if (elementDivider == null) {
                    elementDivider = FormItemPath.DEFAULT_ELEMENT_DIVIDER;
                }
                var absolute = s.charAt(0) == elementDivider;
                var elements = s.split(elementDivider);
                elements = FormItemPath.removeEmptyElements(elements);
                var pathElements = [];
                elements.forEach(function (s) {
                    pathElements.push(FormItemPathElement.fromString(s));
                });
                return new FormItemPath(pathElements, elementDivider, absolute);
            };
            FormItemPath.fromParent = function (parent) {
                var childElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    childElements[_i - 1] = arguments[_i];
                }
                var elements = parent.elements.slice(0);
                childElements.forEach(function (element) {
                    elements.push(element);
                });
                return new FormItemPath(elements, parent.elementDivider, parent.isAbsolute());
            };
            FormItemPath.removeEmptyElements = function (elements) {
                var filteredElements = [];
                elements.forEach(function (element) {
                    if (element.length > 0) {
                        filteredElements.push(element);
                    }
                });
                return filteredElements;
            };
            FormItemPath.prototype.newWithoutFirstElement = function () {
                var arr = this.elements;
                arr.shift();
                return new FormItemPath(arr);
            };
            FormItemPath.prototype.elementCount = function () {
                return this.getElements().length;
            };
            FormItemPath.prototype.getElements = function () {
                return this.elements;
            };
            FormItemPath.prototype.getElement = function (index) {
                return this.elements[index];
            };
            FormItemPath.prototype.getFirstElement = function () {
                return this.elements[0];
            };
            FormItemPath.prototype.getLastElement = function () {
                return this.elements[this.elements.length - 1];
            };
            FormItemPath.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            FormItemPath.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElemements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElemements.push(element);
                    }
                });
                return new FormItemPath(parentElemements);
            };
            FormItemPath.prototype.toString = function () {
                return this.refString;
            };
            FormItemPath.prototype.isAbsolute = function () {
                return this.absolute;
            };
            FormItemPath.DEFAULT_ELEMENT_DIVIDER = ".";
            FormItemPath.ROOT = new FormItemPath([], FormItemPath.DEFAULT_ELEMENT_DIVIDER, true);
            return FormItemPath;
        })();
        form.FormItemPath = FormItemPath;
        var FormItemPathElement = (function () {
            function FormItemPathElement(name) {
                this.name = name;
            }
            FormItemPathElement.prototype.getName = function () {
                return this.name;
            };
            FormItemPathElement.prototype.toString = function () {
                return this.name;
            };
            FormItemPathElement.fromString = function (str) {
                return new FormItemPathElement(str);
            };
            return FormItemPathElement;
        })();
        form.FormItemPathElement = FormItemPathElement;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItem = (function () {
            function FormItem(name) {
                this.name = name;
            }
            FormItem.prototype.setParent = function (parent) {
                if (!(api.ObjectHelper.iFrameSafeInstanceOf(parent, form.FormItemSet) || api.ObjectHelper.iFrameSafeInstanceOf(parent, form.FieldSet))) {
                    throw new Error("A parent FormItem must either be a FormItemSet or a FieldSet");
                }
                this.parent = parent;
            };
            FormItem.prototype.getName = function () {
                return this.name;
            };
            FormItem.prototype.getPath = function () {
                return this.resolvePath();
            };
            FormItem.prototype.resolvePath = function () {
                return form.FormItemPath.fromParent(this.resolveParentPath(), form.FormItemPathElement.fromString(this.name));
            };
            FormItem.prototype.resolveParentPath = function () {
                if (this.parent == null) {
                    return form.FormItemPath.ROOT;
                }
                else {
                    return this.parent.getPath();
                }
            };
            FormItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FormItem)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                return true;
            };
            FormItem.prototype.toFormItemJson = function () {
                if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.Input)) {
                    return this.toInputJson();
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.FormItemSet)) {
                    return this.toFormItemSetJson();
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.Layout)) {
                    return this.toLayoutJson();
                }
                else {
                    throw new Error("Unsupported FormItem: " + this);
                }
            };
            FormItem.formItemsToJson = function (formItems) {
                var formItemArray = [];
                formItems.forEach(function (formItem) {
                    formItemArray.push(formItem.toFormItemJson());
                });
                return formItemArray;
            };
            return FormItem;
        })();
        form.FormItem = FormItem;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrencesBuilder = (function () {
            function OccurrencesBuilder() {
            }
            OccurrencesBuilder.prototype.setMinimum = function (value) {
                this.minimum = value;
                return this;
            };
            OccurrencesBuilder.prototype.setMaximum = function (value) {
                this.maximum = value;
                return this;
            };
            OccurrencesBuilder.prototype.fromJson = function (json) {
                this.minimum = json.minimum;
                this.maximum = json.maximum;
            };
            OccurrencesBuilder.prototype.build = function () {
                return new Occurrences(this);
            };
            return OccurrencesBuilder;
        })();
        form.OccurrencesBuilder = OccurrencesBuilder;
        var Occurrences = (function () {
            function Occurrences(builder) {
                this.minimum = builder.minimum;
                this.maximum = builder.maximum;
            }
            Occurrences.fromJson = function (json) {
                var builder = new OccurrencesBuilder();
                builder.fromJson(json);
                return builder.build();
            };
            Occurrences.prototype.getMaximum = function () {
                return this.maximum;
            };
            Occurrences.prototype.getMinimum = function () {
                return this.minimum;
            };
            Occurrences.prototype.required = function () {
                return this.minimum > 0;
            };
            Occurrences.prototype.multiple = function () {
                return this.maximum > 1 || this.maximum == 0;
            };
            Occurrences.prototype.minimumReached = function (occurrenceCount) {
                return occurrenceCount >= this.minimum;
            };
            Occurrences.prototype.minimumBreached = function (occurrenceCount) {
                return (this.minimum == 0) ? false : (occurrenceCount < this.minimum);
            };
            Occurrences.prototype.maximumReached = function (occurrenceCount) {
                if (this.maximum == 0) {
                    return false;
                }
                return occurrenceCount >= this.maximum;
            };
            Occurrences.prototype.maximumBreached = function (occurrenceCount) {
                if (this.maximum == 0) {
                    return false;
                }
                return occurrenceCount > this.maximum;
            };
            Occurrences.prototype.toJson = function () {
                return {
                    maximum: this.getMaximum(),
                    minimum: this.getMinimum()
                };
            };
            Occurrences.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Occurrences)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.minimum, other.minimum)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.maximum, other.maximum)) {
                    return false;
                }
                return true;
            };
            return Occurrences;
        })();
        form.Occurrences = Occurrences;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var InputTypeName = (function () {
            function InputTypeName(name, custom) {
                this.name = name;
                this.custom = custom;
                if (this.custom) {
                    this.refString = InputTypeName.CUSTOM_PREFIX + name;
                }
                else {
                    this.refString = name;
                }
            }
            InputTypeName.parseInputTypeName = function (str) {
                if (str.substr(0, InputTypeName.CUSTOM_PREFIX.length) == InputTypeName.CUSTOM_PREFIX) {
                    return new InputTypeName(str.substr(InputTypeName.CUSTOM_PREFIX.length, str.length), true);
                }
                else {
                    return new InputTypeName(str, false);
                }
            };
            InputTypeName.prototype.getName = function () {
                return this.name;
            };
            InputTypeName.prototype.isBuiltIn = function () {
                return !this.custom;
            };
            InputTypeName.prototype.toString = function () {
                return this.refString;
            };
            InputTypeName.prototype.toJson = function () {
                return {
                    name: this.toString()
                };
            };
            InputTypeName.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, InputTypeName)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.booleanEquals(this.custom, other.custom)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                return true;
            };
            InputTypeName.CUSTOM_PREFIX = "custom:";
            return InputTypeName;
        })();
        form.InputTypeName = InputTypeName;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var InputBuilder = (function () {
            function InputBuilder() {
                this.immutable = false;
                this.indexed = true;
            }
            InputBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            InputBuilder.prototype.setInputType = function (value) {
                this.inputType = value;
                return this;
            };
            InputBuilder.prototype.setLabel = function (value) {
                this.label = value;
                return this;
            };
            InputBuilder.prototype.setImmutable = function (value) {
                this.immutable = value;
                return this;
            };
            InputBuilder.prototype.setOccurrences = function (value) {
                this.occurrences = value;
                return this;
            };
            InputBuilder.prototype.setIndexed = function (value) {
                this.indexed = value;
                return this;
            };
            InputBuilder.prototype.setCustomText = function (value) {
                this.customText = value;
                return this;
            };
            InputBuilder.prototype.setValidationRegex = function (value) {
                this.validationRegex = value;
                return this;
            };
            InputBuilder.prototype.setHelpText = function (value) {
                this.helpText = value;
                return this;
            };
            InputBuilder.prototype.setInputTypeConfig = function (value) {
                this.inputTypeConfig = value;
                return this;
            };
            InputBuilder.prototype.fromJson = function (json) {
                this.name = json.name;
                this.inputType = form.InputTypeName.parseInputTypeName(json.inputType.name);
                this.label = json.label;
                this.immutable = json.immutable;
                this.occurrences = form.Occurrences.fromJson(json.occurrences);
                this.indexed = json.indexed;
                this.customText = json.customText;
                this.validationRegex = json.validationRegexp;
                this.helpText = json.helpText;
                this.inputTypeConfig = json.config;
                return this;
            };
            InputBuilder.prototype.build = function () {
                return new Input(this);
            };
            return InputBuilder;
        })();
        form.InputBuilder = InputBuilder;
        var Input = (function (_super) {
            __extends(Input, _super);
            function Input(builder) {
                _super.call(this, builder.name);
                this.inputType = builder.inputType;
                this.inputTypeConfig = builder.inputTypeConfig;
                this.label = builder.label;
                this.immutable = builder.immutable;
                this.occurrences = builder.occurrences;
                this.indexed = builder.indexed;
                this.customText = builder.customText;
                this.validationRegex = builder.validationRegex;
                this.helpText = builder.helpText;
            }
            Input.fromJson = function (json) {
                var builder = new InputBuilder();
                builder.fromJson(json);
                return builder.build();
            };
            Input.prototype.getInputType = function () {
                return this.inputType;
            };
            Input.prototype.getLabel = function () {
                return this.label;
            };
            Input.prototype.isImmutable = function () {
                return this.immutable;
            };
            Input.prototype.getOccurrences = function () {
                return this.occurrences;
            };
            Input.prototype.isIndexed = function () {
                return this.indexed;
            };
            Input.prototype.getCustomText = function () {
                return this.customText;
            };
            Input.prototype.getValidationRegex = function () {
                return this.validationRegex;
            };
            Input.prototype.getHelpText = function () {
                return this.helpText;
            };
            Input.prototype.getInputTypeConfig = function () {
                return this.inputTypeConfig;
            };
            Input.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Input)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.inputType, other.inputType)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.immutable, other.immutable)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.occurrences, other.occurrences)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.indexed, other.indexed)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.customText, other.customText)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.validationRegex, other.validationRegex)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.helpText, other.helpText)) {
                    return false;
                }
                if (!api.ObjectHelper.anyEquals(this.inputTypeConfig, other.inputTypeConfig)) {
                    return false;
                }
                return true;
            };
            Input.prototype.toInputJson = function () {
                return { Input: {
                    name: this.getName(),
                    customText: this.getCustomText(),
                    helpText: this.getHelpText(),
                    immutable: this.isImmutable(),
                    indexed: this.isIndexed(),
                    label: this.getLabel(),
                    occurrences: this.getOccurrences().toJson(),
                    validationRegexp: this.getValidationRegex(),
                    inputType: this.getInputType().toJson(),
                    config: this.getInputTypeConfig()
                } };
            };
            return Input;
        })(form.FormItem);
        form.Input = Input;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var Layout = (function (_super) {
            __extends(Layout, _super);
            function Layout(name) {
                _super.call(this, name);
            }
            Layout.prototype.toLayoutJson = function () {
                if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.FieldSet)) {
                    return this.toFieldSetJson();
                }
                else {
                    throw new Error("Unsupported Layout: " + this);
                }
            };
            Layout.prototype.equals = function (o) {
                if (!(api.ObjectHelper.iFrameSafeInstanceOf(o, Layout))) {
                    return false;
                }
                return _super.prototype.equals.call(this, o);
            };
            return Layout;
        })(form.FormItem);
        form.Layout = Layout;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FieldSet = (function (_super) {
            __extends(FieldSet, _super);
            function FieldSet(fieldSetJson) {
                var _this = this;
                _super.call(this, fieldSetJson.name);
                this.formItems = [];
                this.label = fieldSetJson.label;
                if (fieldSetJson.items != null) {
                    fieldSetJson.items.forEach(function (formItemJson) {
                        _this.addFormItem(form.FormItemFactory.createFormItem(formItemJson));
                    });
                }
            }
            FieldSet.prototype.addFormItem = function (formItem) {
                this.formItems.push(formItem);
            };
            FieldSet.prototype.getLabel = function () {
                return this.label;
            };
            FieldSet.prototype.getFormItems = function () {
                return this.formItems;
            };
            FieldSet.prototype.toFieldSetJson = function () {
                return { FieldSet: {
                    name: this.getName(),
                    items: form.FormItem.formItemsToJson(this.getFormItems()),
                    label: this.getLabel()
                } };
            };
            FieldSet.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FieldSet)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                    return false;
                }
                if (!api.ObjectHelper.arrayEquals(this.formItems, other.formItems)) {
                    return false;
                }
                return true;
            };
            return FieldSet;
        })(form.Layout);
        form.FieldSet = FieldSet;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSet = (function (_super) {
            __extends(FormItemSet, _super);
            function FormItemSet(formItemSetJson) {
                var _this = this;
                _super.call(this, formItemSetJson.name);
                this.formItems = [];
                this.formItemByName = {};
                this.label = formItemSetJson.label;
                this.immutable = formItemSetJson.immutable;
                this.occurrences = form.Occurrences.fromJson(formItemSetJson.occurrences);
                this.customText = formItemSetJson.customText;
                this.helpText = formItemSetJson.helpText;
                if (formItemSetJson.items != null) {
                    formItemSetJson.items.forEach(function (formItemJson) {
                        _this.addFormItem(form.FormItemFactory.createFormItem(formItemJson));
                    });
                }
            }
            FormItemSet.prototype.addFormItem = function (formItem) {
                if (this.formItemByName[name]) {
                    throw new Error("FormItem already added: " + name);
                }
                formItem.setParent(this);
                this.formItemByName[formItem.getName()] = formItem;
                this.formItems.push(formItem);
            };
            FormItemSet.prototype.getFormItems = function () {
                return this.formItems;
            };
            FormItemSet.prototype.getFormItemByName = function (name) {
                return this.formItemByName[name];
            };
            FormItemSet.prototype.getInputByName = function (name) {
                return this.formItemByName[name];
            };
            FormItemSet.prototype.getLabel = function () {
                return this.label;
            };
            FormItemSet.prototype.isImmutable = function () {
                return this.immutable;
            };
            FormItemSet.prototype.getCustomText = function () {
                return this.customText;
            };
            FormItemSet.prototype.getHelpText = function () {
                return this.helpText;
            };
            FormItemSet.prototype.getOccurrences = function () {
                return this.occurrences;
            };
            FormItemSet.prototype.toFormItemSetJson = function () {
                return { FormItemSet: {
                    name: this.getName(),
                    customText: this.getCustomText(),
                    helpText: this.getHelpText(),
                    immutable: this.isImmutable(),
                    items: form.FormItem.formItemsToJson(this.getFormItems()),
                    label: this.getLabel(),
                    occurrences: this.getOccurrences().toJson(),
                } };
            };
            FormItemSet.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FormItemSet)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.immutable, other.immutable)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.occurrences, other.occurrences)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.customText, other.customText)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.helpText, other.helpText)) {
                    return false;
                }
                if (!api.ObjectHelper.arrayEquals(this.formItems, other.formItems)) {
                    return false;
                }
                return true;
            };
            return FormItemSet;
        })(form.FormItem);
        form.FormItemSet = FormItemSet;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormValidityChangedEvent = (function () {
            function FormValidityChangedEvent(recording) {
                this.recording = recording;
            }
            FormValidityChangedEvent.prototype.isValid = function () {
                return this.recording.isValid();
            };
            FormValidityChangedEvent.prototype.getRecording = function () {
                return this.recording;
            };
            return FormValidityChangedEvent;
        })();
        form.FormValidityChangedEvent = FormValidityChangedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormBuilder = (function () {
            function FormBuilder() {
                this.formItems = [];
            }
            FormBuilder.prototype.addFormItem = function (formItem) {
                this.formItems.push(formItem);
                return this;
            };
            FormBuilder.prototype.addFormItems = function (formItems) {
                var _this = this;
                formItems.forEach(function (formItem) {
                    _this.addFormItem(formItem);
                });
                return this;
            };
            FormBuilder.prototype.fromJson = function (json) {
                var _this = this;
                json.formItems.forEach(function (formItemJson) {
                    _this.addFormItem(form.FormItemFactory.createFormItem(formItemJson));
                });
                return this;
            };
            FormBuilder.prototype.build = function () {
                return new Form(this);
            };
            return FormBuilder;
        })();
        form.FormBuilder = FormBuilder;
        var Form = (function () {
            function Form(builder) {
                var _this = this;
                this.formItems = [];
                this.formItemByName = {};
                builder.formItems.forEach(function (formItem) {
                    _this.addFormItem(formItem);
                });
            }
            Form.fromJson = function (json) {
                var builder = new FormBuilder();
                builder.fromJson(json);
                return builder.build();
            };
            Form.prototype.addFormItem = function (formItem) {
                if (this.formItemByName[name]) {
                    throw new Error("FormItem already added: " + name);
                }
                this.formItemByName[formItem.getName()] = formItem;
                this.formItems.push(formItem);
            };
            Form.prototype.getFormItems = function () {
                return this.formItems;
            };
            Form.prototype.getFormItemByName = function (name) {
                return this.formItemByName[name];
            };
            Form.prototype.getInputByName = function (name) {
                return this.formItemByName[name];
            };
            Form.prototype.toJson = function () {
                return {
                    formItems: form.FormItem.formItemsToJson(this.getFormItems())
                };
            };
            Form.prototype.equals = function (o) {
                if (!(api.ObjectHelper.iFrameSafeInstanceOf(o, Form))) {
                    return false;
                }
                var other = o;
                if (this.formItems.length != other.formItems.length) {
                    return false;
                }
                for (var i = 0; i < this.formItems.length; i++) {
                    if (!this.formItems[i].equals(other.formItems[i])) {
                        return false;
                    }
                }
                return true;
            };
            return Form;
        })();
        form.Form = Form;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemFactory = (function () {
            function FormItemFactory() {
            }
            FormItemFactory.createForm = function (formJson) {
                return form.Form.fromJson(formJson);
            };
            FormItemFactory.createFormItem = function (formItemTypeWrapperJson) {
                if (formItemTypeWrapperJson.Input) {
                    return FormItemFactory.createInput(formItemTypeWrapperJson.Input);
                }
                else if (formItemTypeWrapperJson.FormItemSet) {
                    return FormItemFactory.createFormItemSet(formItemTypeWrapperJson.FormItemSet);
                }
                else if (formItemTypeWrapperJson.FieldSet) {
                    return FormItemFactory.createFieldSetLayout(formItemTypeWrapperJson.FieldSet);
                }
                console.log("Unknown FormItem type: ", formItemTypeWrapperJson);
                throw new Error("Unknown FormItem");
            };
            FormItemFactory.createInput = function (inputJson) {
                return form.Input.fromJson(inputJson);
            };
            FormItemFactory.createFormItemSet = function (formItemSetJson) {
                return new form.FormItemSet(formItemSetJson);
            };
            FormItemFactory.createFieldSetLayout = function (fieldSetJson) {
                return new form.FieldSet(fieldSetJson);
            };
            return FormItemFactory;
        })();
        form.FormItemFactory = FormItemFactory;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var ValidationRecordingPath = (function () {
            function ValidationRecordingPath(parentPropertySet, dataName, min, max) {
                this.parentDataSet = parentPropertySet != null ? parentPropertySet.asRelative() : null;
                this.dataName = dataName;
                this.refString = this.resolveRefString();
                this.min = min;
                this.max = max;
            }
            ValidationRecordingPath.prototype.resolveRefString = function () {
                var s = "";
                if (this.parentDataSet && !this.parentDataSet.isRoot()) {
                    s += this.parentDataSet.toString();
                    if (this.dataName) {
                        s += "." + this.dataName;
                    }
                }
                else {
                    if (this.dataName) {
                        s += this.dataName;
                    }
                }
                return s;
            };
            ValidationRecordingPath.prototype.getParentDataSet = function () {
                return this.parentDataSet;
            };
            ValidationRecordingPath.prototype.getDataName = function () {
                return this.dataName;
            };
            ValidationRecordingPath.prototype.getMin = function () {
                return this.min;
            };
            ValidationRecordingPath.prototype.getMax = function () {
                return this.max;
            };
            ValidationRecordingPath.prototype.toString = function () {
                return this.refString;
            };
            return ValidationRecordingPath;
        })();
        form.ValidationRecordingPath = ValidationRecordingPath;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var RecordingValidityChangedEvent = (function () {
            function RecordingValidityChangedEvent(recording, origin) {
                this.recording = recording;
                this.origin = origin;
            }
            RecordingValidityChangedEvent.prototype.getOrigin = function () {
                return this.origin;
            };
            RecordingValidityChangedEvent.prototype.isValid = function () {
                return this.recording.isValid();
            };
            RecordingValidityChangedEvent.prototype.getRecording = function () {
                return this.recording;
            };
            return RecordingValidityChangedEvent;
        })();
        form.RecordingValidityChangedEvent = RecordingValidityChangedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var ValidationRecording = (function () {
            function ValidationRecording() {
                this.breaksMinimumOccurrencesArray = [];
                this.breaksMaximumOccurrencesArray = [];
            }
            ValidationRecording.prototype.breaksMinimumOccurrences = function (path) {
                if (!this.exists(path, this.breaksMinimumOccurrencesArray)) {
                    this.breaksMinimumOccurrencesArray.push(path);
                }
            };
            ValidationRecording.prototype.breaksMaximumOccurrences = function (path) {
                if (!this.exists(path, this.breaksMaximumOccurrencesArray)) {
                    this.breaksMaximumOccurrencesArray.push(path);
                }
            };
            ValidationRecording.prototype.isValid = function () {
                return this.breaksMinimumOccurrencesArray.length == 0 && this.breaksMaximumOccurrencesArray.length == 0;
            };
            ValidationRecording.prototype.isMinimumOccurrencesValid = function () {
                return this.breaksMinimumOccurrencesArray.length == 0;
            };
            ValidationRecording.prototype.isMaximumOccurrencesValid = function () {
                return this.breaksMaximumOccurrencesArray.length == 0;
            };
            ValidationRecording.prototype.flatten = function (recording) {
                var _this = this;
                recording.breaksMinimumOccurrencesArray.forEach(function (path) {
                    _this.breaksMinimumOccurrences(path);
                });
                recording.breaksMaximumOccurrencesArray.forEach(function (path) {
                    _this.breaksMaximumOccurrences(path);
                });
            };
            ValidationRecording.prototype.subtract = function (recording) {
                var _this = this;
                this.breaksMinimumOccurrencesArray = this.breaksMinimumOccurrencesArray.filter(function (path) {
                    return !_this.exists(path, recording.breaksMinimumOccurrencesArray);
                });
                this.breaksMaximumOccurrencesArray = this.breaksMaximumOccurrencesArray.filter(function (path) {
                    return !_this.exists(path, recording.breaksMaximumOccurrencesArray);
                });
            };
            ValidationRecording.prototype.removeByPath = function (path) {
                this.removeUnreachedMinimumOccurrencesByPath(path);
                this.removeBreachedMaximumOccurrencesByPath(path);
            };
            ValidationRecording.prototype.removeUnreachedMinimumOccurrencesByPath = function (path) {
                for (var i = this.breaksMinimumOccurrencesArray.length - 1; i >= 0; i--) {
                    if (this.breaksMinimumOccurrencesArray[i].toString().indexOf(path.toString()) == 0) {
                        this.breaksMinimumOccurrencesArray.splice(i, 1);
                    }
                }
            };
            ValidationRecording.prototype.removeBreachedMaximumOccurrencesByPath = function (path) {
                for (var i = this.breaksMaximumOccurrencesArray.length - 1; i >= 0; i--) {
                    if (this.breaksMaximumOccurrencesArray[i].toString().indexOf(path.toString()) == 0) {
                        this.breaksMaximumOccurrencesArray.splice(i, 1);
                    }
                }
            };
            ValidationRecording.prototype.equals = function (other) {
                if (this.breaksMinimumOccurrencesArray.length != other.breaksMinimumOccurrencesArray.length) {
                    return false;
                }
                else if (this.breaksMaximumOccurrencesArray.length != other.breaksMaximumOccurrencesArray.length) {
                    return false;
                }
                for (var i = 0; i < this.breaksMinimumOccurrencesArray.length; i++) {
                    if (this.breaksMinimumOccurrencesArray[i].toString() != other.breaksMinimumOccurrencesArray[i].toString()) {
                        return false;
                    }
                }
                for (var i = 0; i < this.breaksMaximumOccurrencesArray.length; i++) {
                    if (this.breaksMaximumOccurrencesArray[i].toString() != other.breaksMaximumOccurrencesArray[i].toString()) {
                        return false;
                    }
                }
                return true;
            };
            ValidationRecording.prototype.validityChanged = function (previous) {
                return previous == undefined || previous == null || !previous.equals(this);
            };
            ValidationRecording.prototype.exists = function (path, array) {
                for (var i = 0; i < array.length; i++) {
                    if (array[i].toString() == path.toString()) {
                        return true;
                    }
                }
                return false;
            };
            return ValidationRecording;
        })();
        form.ValidationRecording = ValidationRecording;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormContext = (function () {
            function FormContext(builder) {
                this.showEmptyFormItemSetOccurrences = builder.showEmptyFormItemSetOccurrences;
            }
            FormContext.prototype.getShowEmptyFormItemSetOccurrences = function () {
                return this.showEmptyFormItemSetOccurrences;
            };
            FormContext.create = function () {
                return new FormContextBuilder();
            };
            FormContext.prototype.createInputTypeViewContext = function (inputTypeConfig, parentPropertyPath, input) {
                return {
                    formContext: this,
                    input: input,
                    inputConfig: inputTypeConfig,
                    parentDataPath: parentPropertyPath
                };
            };
            return FormContext;
        })();
        form.FormContext = FormContext;
        var FormContextBuilder = (function () {
            function FormContextBuilder() {
            }
            FormContextBuilder.prototype.setShowEmptyFormItemSetOccurrences = function (value) {
                this.showEmptyFormItemSetOccurrences = value;
                return this;
            };
            FormContextBuilder.prototype.build = function () {
                return new FormContext(this);
            };
            return FormContextBuilder;
        })();
        form.FormContextBuilder = FormContextBuilder;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemOccurrenceView = (function (_super) {
            __extends(FormItemOccurrenceView, _super);
            function FormItemOccurrenceView(className, formItemOccurrence) {
                _super.call(this, className);
                this.removeButtonClickedListeners = [];
                this.formItemOccurrence = formItemOccurrence;
            }
            FormItemOccurrenceView.prototype.getDataPath = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrenceView.prototype.layout = function () {
                return wemQ(null);
            };
            FormItemOccurrenceView.prototype.hasValidUserInput = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrenceView.prototype.onRemoveButtonClicked = function (listener) {
                this.removeButtonClickedListeners.push(listener);
            };
            FormItemOccurrenceView.prototype.unRemoveButtonClicked = function (listener) {
                this.removeButtonClickedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            FormItemOccurrenceView.prototype.notifyRemoveButtonClicked = function () {
                var _this = this;
                this.removeButtonClickedListeners.forEach(function (listener) {
                    listener.call(_this, new form.RemoveButtonClickedEvent(_this, _this.formItemOccurrence.getIndex()));
                });
            };
            FormItemOccurrenceView.prototype.getIndex = function () {
                return this.formItemOccurrence.getIndex();
            };
            FormItemOccurrenceView.prototype.refresh = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrenceView.prototype.giveFocus = function () {
                return false;
            };
            return FormItemOccurrenceView;
        })(api.dom.DivEl);
        form.FormItemOccurrenceView = FormItemOccurrenceView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemOccurrences = (function () {
            function FormItemOccurrences(config) {
                this.occurrences = [];
                this.occurrenceViews = [];
                this.occurrenceAddedListeners = [];
                this.occurrenceRenderedListeners = [];
                this.occurrenceRemovedListeners = [];
                this.focusListeners = [];
                this.blurListeners = [];
                this.formItem = config.formItem;
                this.propertyArray = config.propertyArray;
                this.occurrenceViewContainer = config.occurrenceViewContainer;
                this.allowedOccurrences = config.allowedOccurrences;
            }
            FormItemOccurrences.prototype.getAllowedOccurrences = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrences.prototype.onOccurrenceRendered = function (listener) {
                this.occurrenceRenderedListeners.push(listener);
            };
            FormItemOccurrences.prototype.unOccurrenceRendered = function (listener) {
                this.occurrenceRenderedListeners = this.occurrenceRenderedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormItemOccurrences.prototype.notifyOccurrenceRendered = function (occurrence, occurrenceView) {
                var _this = this;
                this.occurrenceRenderedListeners.forEach(function (listener) {
                    listener.call(_this, new form.OccurrenceRenderedEvent(occurrence, occurrenceView));
                });
            };
            FormItemOccurrences.prototype.onOccurrenceAdded = function (listener) {
                this.occurrenceAddedListeners.push(listener);
            };
            FormItemOccurrences.prototype.unOccurrenceAdded = function (listener) {
                this.occurrenceAddedListeners = this.occurrenceAddedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormItemOccurrences.prototype.notifyOccurrenceAdded = function (occurrence, occurrenceView) {
                var _this = this;
                this.occurrenceAddedListeners.forEach(function (listener) {
                    listener.call(_this, new form.OccurrenceAddedEvent(occurrence, occurrenceView));
                });
            };
            FormItemOccurrences.prototype.onOccurrenceRemoved = function (listener) {
                this.occurrenceRemovedListeners.push(listener);
            };
            FormItemOccurrences.prototype.unOccurrenceRemoved = function (listener) {
                this.occurrenceRemovedListeners = this.occurrenceRemovedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormItemOccurrences.prototype.notifyOccurrenceRemoved = function (occurrence, occurrenceView) {
                var _this = this;
                this.occurrenceRemovedListeners.forEach(function (listener) {
                    listener.call(_this, new form.OccurrenceRemovedEvent(occurrence, occurrenceView));
                });
            };
            FormItemOccurrences.prototype.constructOccurrencesForNoData = function () {
                if (this.getAllowedOccurrences().getMinimum() > 0) {
                    for (var i = 0; i < this.getAllowedOccurrences().getMinimum(); i++) {
                        this.addOccurrence(this.createNewOccurrence(this, i));
                    }
                }
                else {
                    this.addOccurrence(this.createNewOccurrence(this, 0));
                }
            };
            FormItemOccurrences.prototype.addOccurrence = function (occurrence) {
                this.occurrences.push(occurrence);
            };
            FormItemOccurrences.prototype.getFormItem = function () {
                return this.formItem;
            };
            FormItemOccurrences.prototype.maximumOccurrencesReached = function () {
                return this.allowedOccurrences.maximumReached(this.countOccurrences());
            };
            FormItemOccurrences.prototype.layout = function () {
                var _this = this;
                var layoutPromises = [];
                this.occurrences.forEach(function (occurrence) {
                    var occurrenceView = _this.createNewOccurrenceView(occurrence);
                    _this.occurrenceViewContainer.appendChild(occurrenceView);
                    var layoutPromise = occurrenceView.layout();
                    layoutPromises.push(layoutPromise);
                    layoutPromise.then(function () {
                        occurrenceView.onFocus(function (event) { return _this.notifyFocused(event); });
                        occurrenceView.onBlur(function (event) { return _this.notifyBlurred(event); });
                        _this.occurrenceViews.push(occurrenceView);
                    });
                });
                return wemQ.all(layoutPromises).spread(function () { return wemQ(null); });
            };
            FormItemOccurrences.prototype.createNewOccurrenceView = function (occurrence) {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrences.prototype.createNewOccurrence = function (formItemOccurrences, insertAtIndex) {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrences.prototype.doRemoveOccurrence = function (occurrenceViewToRemove, indexToRemove) {
                if (!this.canRemove()) {
                    return;
                }
                occurrenceViewToRemove.remove();
                this.occurrenceViews = this.occurrenceViews.filter(function (curr) {
                    return curr != occurrenceViewToRemove;
                });
                var occurrenceToRemove = this.occurrences[indexToRemove];
                this.occurrences = this.occurrences.filter(function (curr) {
                    return curr.getIndex() != indexToRemove;
                });
                this.resetOccurrenceIndexes();
                this.refreshOccurrenceViews();
                this.propertyArray.remove(indexToRemove);
                this.notifyOccurrenceRemoved(occurrenceToRemove, occurrenceViewToRemove);
            };
            FormItemOccurrences.prototype.createAndAddOccurrence = function () {
                var _this = this;
                var insertAtIndex = this.countOccurrences();
                var occurrence = this.createNewOccurrence(this, insertAtIndex);
                this.doAddOccurrence(occurrence);
                var occurrenceView = this.occurrenceViews[occurrence.getIndex()];
                occurrenceView.onFocus(function (event) { return _this.notifyFocused(event); });
                occurrenceView.onBlur(function (event) { return _this.notifyBlurred(event); });
                if (occurrenceView) {
                    occurrenceView.giveFocus();
                }
            };
            FormItemOccurrences.prototype.onFocus = function (listener) {
                this.focusListeners.push(listener);
            };
            FormItemOccurrences.prototype.unFocus = function (listener) {
                this.focusListeners = this.focusListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            FormItemOccurrences.prototype.onBlur = function (listener) {
                this.blurListeners.push(listener);
            };
            FormItemOccurrences.prototype.unBlur = function (listener) {
                this.blurListeners = this.blurListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            FormItemOccurrences.prototype.notifyFocused = function (event) {
                this.focusListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemOccurrences.prototype.notifyBlurred = function (event) {
                this.blurListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemOccurrences.prototype.doAddOccurrence = function (occurrence) {
                var _this = this;
                if (this.allowedOccurrences.maximumReached(this.countOccurrences())) {
                    return;
                }
                var occurrenceView = this.createNewOccurrenceView(occurrence);
                var insertAtIndex = occurrence.getIndex();
                this.occurrences.splice(insertAtIndex, 0, occurrence);
                var occurrenceViewBefore = this.getOccurrenceViewElementBefore(insertAtIndex);
                if (occurrenceViewBefore != null) {
                    occurrenceView.insertAfterEl(occurrenceViewBefore);
                }
                else {
                    this.occurrenceViewContainer.appendChild(occurrenceView);
                }
                occurrenceView.layout().then(function () {
                    _this.notifyOccurrenceRendered(occurrence, occurrenceView);
                }).done();
                this.occurrenceViews.splice(insertAtIndex, 0, occurrenceView);
                this.resetOccurrenceIndexes();
                this.refreshOccurrenceViews();
                this.notifyOccurrenceAdded(occurrence, occurrenceView);
            };
            FormItemOccurrences.prototype.resetOccurrenceIndexes = function () {
                this.occurrences.forEach(function (currOccurrence, index) {
                    currOccurrence.setIndex(index);
                });
            };
            FormItemOccurrences.prototype.refreshOccurrenceViews = function () {
                this.occurrenceViews.forEach(function (currOccurrenceView) {
                    currOccurrenceView.refresh();
                });
            };
            FormItemOccurrences.prototype.getOccurrenceViewElementBefore = function (index) {
                if (index < 1) {
                    return null;
                }
                return this.occurrenceViews.filter(function (occurrenceView) {
                    return occurrenceView.getIndex() == index - 1;
                })[0];
            };
            FormItemOccurrences.prototype.countOccurrences = function () {
                return this.occurrences.length;
            };
            FormItemOccurrences.prototype.moveOccurrence = function (fromIndex, toIndex) {
                api.util.ArrayHelper.moveElement(fromIndex, toIndex, this.occurrences);
                this.occurrences.forEach(function (occurrence, index) {
                    occurrence.setIndex(index);
                });
                api.util.ArrayHelper.moveElement(fromIndex, toIndex, this.occurrenceViews);
                this.propertyArray.move(fromIndex, toIndex);
            };
            FormItemOccurrences.prototype.getOccurrences = function () {
                return this.occurrences;
            };
            FormItemOccurrences.prototype.canRemove = function () {
                return this.occurrences.length > Math.max(0, this.allowedOccurrences.getMinimum());
            };
            FormItemOccurrences.prototype.getOccurrenceViews = function () {
                return this.occurrenceViews;
            };
            return FormItemOccurrences;
        })();
        form.FormItemOccurrences = FormItemOccurrences;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemOccurrence = (function () {
            function FormItemOccurrence(occurrences, index, allowedOccurrences) {
                this.occurrences = occurrences;
                this.formItem = occurrences.getFormItem();
                this.allowedOccurrences = allowedOccurrences;
                this.index = index;
            }
            FormItemOccurrence.prototype.setIndex = function (value) {
                this.index = value;
            };
            FormItemOccurrence.prototype.getIndex = function () {
                return this.index;
            };
            FormItemOccurrence.prototype.showRemoveButton = function () {
                return this.moreThanRequiredOccurrences();
            };
            FormItemOccurrence.prototype.showAddButton = function () {
                if (!this.isLastOccurrence()) {
                    return false;
                }
                return this.lessOccurrencesThanMaximumAllowed();
            };
            FormItemOccurrence.prototype.isMultiple = function () {
                return this.allowedOccurrences.multiple();
            };
            FormItemOccurrence.prototype.oneAndOnly = function () {
                return this.index == 0 && this.occurrences.countOccurrences() == 1;
            };
            FormItemOccurrence.prototype.moreThanRequiredOccurrences = function () {
                return this.occurrences.countOccurrences() > this.allowedOccurrences.getMinimum();
            };
            FormItemOccurrence.prototype.lessOccurrencesThanMaximumAllowed = function () {
                return !this.allowedOccurrences.maximumReached(this.occurrences.countOccurrences());
            };
            FormItemOccurrence.prototype.isLastOccurrence = function () {
                return this.index == this.occurrences.countOccurrences() - 1;
            };
            return FormItemOccurrence;
        })();
        form.FormItemOccurrence = FormItemOccurrence;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemView = (function (_super) {
            __extends(FormItemView, _super);
            function FormItemView(config) {
                _super.call(this, config.className);
                this.editContentRequestListeners = [];
                api.util.assertNotNull(config.context, "context cannot be null");
                api.util.assertNotNull(config.formItem, "formItem cannot be null");
                this.context = config.context;
                this.formItem = config.formItem;
                this.parent = config.parent;
            }
            FormItemView.prototype.broadcastFormSizeChanged = function () {
                throw new Error("Must be implemented by inheritors");
            };
            FormItemView.prototype.getContext = function () {
                return this.context;
            };
            FormItemView.prototype.getFormItem = function () {
                return this.formItem;
            };
            FormItemView.prototype.getParent = function () {
                return this.parent;
            };
            FormItemView.prototype.displayValidationErrors = function (value) {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemView.prototype.hasValidUserInput = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                return new form.ValidationRecording();
            };
            FormItemView.prototype.giveFocus = function () {
                return false;
            };
            FormItemView.prototype.onEditContentRequest = function (listener) {
                this.editContentRequestListeners.push(listener);
            };
            FormItemView.prototype.unEditContentRequest = function (listener) {
                this.editContentRequestListeners = this.editContentRequestListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            FormItemView.prototype.notifyEditContentRequested = function (content) {
                this.editContentRequestListeners.forEach(function (listener) {
                    listener(content);
                });
            };
            FormItemView.prototype.onValidityChanged = function (listener) {
            };
            FormItemView.prototype.unValidityChanged = function (listener) {
            };
            return FormItemView;
        })(api.dom.DivEl);
        form.FormItemView = FormItemView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemLayer = (function () {
            function FormItemLayer() {
                this.formItemViews = [];
            }
            FormItemLayer.prototype.setFormContext = function (context) {
                this.context = context;
                return this;
            };
            FormItemLayer.prototype.setFormItems = function (formItems) {
                this.formItems = formItems;
                return this;
            };
            FormItemLayer.prototype.setParentElement = function (parentEl) {
                this.parentEl = parentEl;
                return this;
            };
            FormItemLayer.prototype.setParent = function (value) {
                this.parent = value;
                return this;
            };
            FormItemLayer.prototype.layout = function (propertySet) {
                var _this = this;
                this.formItemViews = [];
                return this.doLayoutPropertySet(propertySet).then(function () {
                    return wemQ(_this.formItemViews);
                });
            };
            FormItemLayer.prototype.doLayoutPropertySet = function (propertySet) {
                var _this = this;
                var layoutPromises = [];
                this.formItems.forEach(function (formItem) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(formItem, form.FormItemSet)) {
                        var formItemSet = formItem;
                        var propertyArray = propertySet.getPropertyArray(formItemSet.getName());
                        if (propertyArray && propertyArray.getSize() == 0) {
                            _this.context = form.FormContext.create().setShowEmptyFormItemSetOccurrences(false).build();
                        }
                        var formItemSetView = new form.FormItemSetView({
                            context: _this.context,
                            formItemSet: formItemSet,
                            parent: _this.parent,
                            parentDataSet: propertySet
                        });
                        _this.parentEl.appendChild(formItemSetView);
                        _this.formItemViews.push(formItemSetView);
                        layoutPromises.push(formItemSetView.layout());
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(formItem, form.FieldSet)) {
                        var fieldSet = formItem;
                        var fieldSetView = new form.FieldSetView({
                            context: _this.context,
                            fieldSet: fieldSet,
                            parent: _this.parent,
                            dataSet: propertySet
                        });
                        _this.parentEl.appendChild(fieldSetView);
                        _this.formItemViews.push(fieldSetView);
                        layoutPromises.push(fieldSetView.layout());
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(formItem, form.Input)) {
                        var input = formItem;
                        var inputView = new form.InputView({
                            context: _this.context,
                            input: input,
                            parent: _this.parent,
                            parentDataSet: propertySet
                        });
                        _this.parentEl.appendChild(inputView);
                        _this.formItemViews.push(inputView);
                        layoutPromises.push(inputView.layout());
                    }
                });
                return wemQ.all(layoutPromises).spread(function () {
                    return wemQ(null);
                });
            };
            return FormItemLayer;
        })();
        form.FormItemLayer = FormItemLayer;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var InputLabel = (function (_super) {
            __extends(InputLabel, _super);
            function InputLabel(input) {
                _super.call(this, "input-label");
                this.input = input;
                var wrapper = new api.dom.DivEl("wrapper");
                var label = new api.dom.DivEl("label");
                label.getEl().setInnerHtml(input.getLabel());
                label.getEl().setAttribute('title', input.getLabel());
                wrapper.getEl().appendChild(label.getHTMLElement());
                if (input.getOccurrences().required()) {
                    wrapper.addClass("required");
                }
                this.getEl().appendChild(wrapper.getHTMLElement());
            }
            return InputLabel;
        })(api.dom.DivEl);
        form.InputLabel = InputLabel;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var PropertyArray = api.data.PropertyArray;
        var InputView = (function (_super) {
            __extends(InputView, _super);
            function InputView(config) {
                _super.call(this, {
                    className: "input-view",
                    context: config.context,
                    formItem: config.input,
                    parent: config.parent
                });
                this.validityChangedListeners = [];
                api.util.assertNotNull(config.parentDataSet, "parentDataSet not expected to be null");
                api.util.assertNotNull(config.input, "input not expected to be null");
                this.input = config.input;
                this.parentPropertySet = config.parentDataSet;
            }
            InputView.prototype.layout = function () {
                var _this = this;
                if (this.input.getLabel()) {
                    var label = new form.InputLabel(this.input);
                    this.appendChild(label);
                }
                else {
                    this.addClass("no-label");
                }
                this.inputTypeView = this.createInputTypeView();
                this.inputTypeView.onEditContentRequest(function (content) {
                    _this.notifyEditContentRequested(content);
                });
                this.propertyArray = this.parentPropertySet.getPropertyArray(this.input.getName());
                if (!this.propertyArray) {
                    this.propertyArray = PropertyArray.create().setType(this.inputTypeView.getValueType()).setName(this.input.getName()).setParent(this.parentPropertySet).build();
                    this.parentPropertySet.addPropertyArray(this.propertyArray);
                    var initialValue = this.inputTypeView.newInitialValue();
                    if (initialValue) {
                        this.propertyArray.add(initialValue);
                    }
                }
                var inputTypeViewLayoutPromise = this.inputTypeView.layout(this.input, this.propertyArray);
                inputTypeViewLayoutPromise.then(function () {
                    _this.appendChild(_this.inputTypeView.getElement());
                    if (!_this.inputTypeView.isManagingAdd()) {
                        var inputTypeViewNotManagingAdd = _this.inputTypeView;
                        inputTypeViewNotManagingAdd.onOccurrenceAdded(function () {
                            _this.refresh();
                        });
                        inputTypeViewNotManagingAdd.onOccurrenceRemoved(function (event) {
                            _this.refresh();
                            if (api.ObjectHelper.iFrameSafeInstanceOf(event.getOccurrenceView(), api.form.inputtype.support.InputOccurrenceView)) {
                                _this.validate(false);
                            }
                        });
                        _this.addButton = new api.ui.button.Button("Add");
                        _this.addButton.addClass("small");
                        _this.addButton.onClicked(function (event) {
                            inputTypeViewNotManagingAdd.createAndAddOccurrence();
                        });
                        _this.bottomButtonRow = new api.dom.DivEl("bottom-button-row");
                        _this.appendChild(_this.bottomButtonRow);
                        _this.bottomButtonRow.appendChild(_this.addButton);
                    }
                    _this.validationViewer = new form.ValidationRecordingViewer();
                    _this.appendChild(_this.validationViewer);
                    _this.inputTypeView.onValidityChanged(function (event) {
                        _this.handleInputValidationRecording(event.getRecording(), false);
                    });
                    _this.refresh();
                });
                return inputTypeViewLayoutPromise;
            };
            InputView.prototype.createInputTypeView = function () {
                var inputType = this.input.getInputType();
                var inputTypeViewContext = this.getContext().createInputTypeViewContext(this.input.getInputTypeConfig(), this.parentPropertySet.getPropertyPath(), this.input);
                if (form.inputtype.InputTypeManager.isRegistered(inputType.getName())) {
                    return form.inputtype.InputTypeManager.createView(inputType.getName(), inputTypeViewContext);
                }
                else {
                    console.warn("Input type [" + inputType.getName() + "] needs to be registered first.");
                    return form.inputtype.InputTypeManager.createView("NoInputTypeFound", inputTypeViewContext);
                }
            };
            InputView.prototype.broadcastFormSizeChanged = function () {
                if (this.isVisible()) {
                    this.inputTypeView.availableSizeChanged();
                }
            };
            InputView.prototype.refresh = function () {
                if (!this.inputTypeView.isManagingAdd()) {
                    var inputTypeViewNotManagingAdd = this.inputTypeView;
                    this.addButton.setVisible(!inputTypeViewNotManagingAdd.maximumOccurrencesReached());
                }
                this.validate(false);
            };
            InputView.prototype.resolveValidationRecordingPath = function () {
                return new form.ValidationRecordingPath(this.propertyArray.getParentPropertyPath(), this.input.getName(), this.input.getOccurrences().getMinimum(), this.input.getOccurrences().getMaximum());
            };
            InputView.prototype.displayValidationErrors = function (value) {
                this.inputTypeView.displayValidationErrors(value);
            };
            InputView.prototype.hasValidUserInput = function () {
                return this.inputTypeView.hasValidUserInput();
            };
            InputView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var inputRecording = this.inputTypeView.validate(silent);
                return this.handleInputValidationRecording(inputRecording, silent);
            };
            InputView.prototype.handleInputValidationRecording = function (inputRecording, silent) {
                if (silent === void 0) { silent = true; }
                var recording = new form.ValidationRecording();
                var validationRecordingPath = this.resolveValidationRecordingPath();
                if (inputRecording.isMinimumOccurrenesBreached()) {
                    recording.breaksMinimumOccurrences(validationRecordingPath);
                }
                if (inputRecording.isMaximumOccurrenesBreached()) {
                    recording.breaksMaximumOccurrences(validationRecordingPath);
                }
                if (!silent && recording.validityChanged(this.previousValidityRecording)) {
                    this.notifyFormValidityChanged(new form.RecordingValidityChangedEvent(recording, validationRecordingPath));
                }
                this.renderValidationErrors(recording);
                this.previousValidityRecording = recording;
                return recording;
            };
            InputView.prototype.giveFocus = function () {
                return this.inputTypeView.giveFocus();
            };
            InputView.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            InputView.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners.filter(function (currentListener) {
                    return listener == currentListener;
                });
            };
            InputView.prototype.notifyFormValidityChanged = function (event) {
                this.validityChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            InputView.prototype.renderValidationErrors = function (recording) {
                if (recording.isValid()) {
                    this.removeClass("invalid");
                    this.addClass("valid");
                }
                else {
                    this.removeClass("valid");
                    this.addClass("invalid");
                }
                this.validationViewer.setObject(recording);
                if (recording.isValid()) {
                }
            };
            InputView.prototype.onFocus = function (listener) {
                this.inputTypeView.onFocus(listener);
            };
            InputView.prototype.unFocus = function (listener) {
                this.inputTypeView.unFocus(listener);
            };
            InputView.prototype.onBlur = function (listener) {
                this.inputTypeView.onBlur(listener);
            };
            InputView.prototype.unBlur = function (listener) {
                this.inputTypeView.unBlur(listener);
            };
            return InputView;
        })(api.form.FormItemView);
        form.InputView = InputView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (_form) {
        var FormView = (function (_super) {
            __extends(FormView, _super);
            function FormView(context, form, data) {
                _super.call(this, "form-view");
                this.formItemViews = [];
                this.formValidityChangedListeners = [];
                this.focusListeners = [];
                this.blurListeners = [];
                this.context = context;
                this.form = form;
                this.data = data;
            }
            FormView.prototype.layout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var formItems = this.form.getFormItems();
                var layoutPromise = new _form.FormItemLayer().setFormContext(this.context).setFormItems(formItems).setParentElement(this).layout(this.data);
                layoutPromise.then(function (formItemViews) {
                    _this.formItemViews = formItemViews;
                    api.util.assert(_this.formItemViews.length == formItems.length, "Not all FormItemView-s was created. Expected " + formItems.length + ", was: " + formItemViews.length);
                    deferred.resolve(null);
                    _this.formItemViews.forEach(function (formItemView) {
                        formItemView.onFocus(function (event) {
                            _this.notifyFocused(event);
                        });
                        formItemView.onBlur(function (event) {
                            _this.notifyBlurred(event);
                        });
                        formItemView.onValidityChanged(function (event) {
                            if (!_this.previousValidationRecording) {
                                _this.previousValidationRecording = event.getRecording();
                                _this.notifyValidityChanged(new _form.FormValidityChangedEvent(_this.previousValidationRecording));
                            }
                            else {
                                var previousValidState = _this.previousValidationRecording.isValid();
                                if (event.isValid()) {
                                    _this.previousValidationRecording.removeByPath(event.getOrigin());
                                }
                                else {
                                    _this.previousValidationRecording.flatten(event.getRecording());
                                }
                                if (previousValidState != _this.previousValidationRecording.isValid()) {
                                    _this.notifyValidityChanged(new _form.FormValidityChangedEvent(_this.previousValidationRecording));
                                }
                            }
                        });
                    });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.checkSizeChanges(); }, _this);
                    _this.onShown(function () { return _this.checkSizeChanges(); });
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                return deferred.promise;
            };
            FormView.prototype.checkSizeChanges = function () {
                if (this.isVisible() && this.isSizeChanged()) {
                    this.preserveCurrentSize();
                    this.broadcastFormSizeChanged();
                }
            };
            FormView.prototype.preserveCurrentSize = function () {
                this.width = this.getEl().getWidth();
            };
            FormView.prototype.isSizeChanged = function () {
                return this.width != this.getEl().getWidth();
            };
            FormView.prototype.broadcastFormSizeChanged = function () {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.broadcastFormSizeChanged();
                });
            };
            FormView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemViews.forEach(function (formItemView) {
                    if (!formItemView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FormView.prototype.validate = function (silent) {
                var recording = new _form.ValidationRecording();
                this.formItemViews.forEach(function (formItemView) {
                    recording.flatten(formItemView.validate(silent));
                });
                if (!silent && recording.validityChanged(this.previousValidationRecording)) {
                    this.notifyValidityChanged(new _form.FormValidityChangedEvent(recording));
                }
                this.previousValidationRecording = recording;
                return recording;
            };
            FormView.prototype.isValid = function () {
                if (!this.previousValidationRecording) {
                    this.previousValidationRecording = this.validate(true);
                }
                return this.previousValidationRecording.isValid();
            };
            FormView.prototype.displayValidationErrors = function (value) {
                if (value) {
                    this.addClass("display-validation-errors");
                }
                else {
                    this.removeClass("display-validation-errors");
                }
                for (var i = 0; i < this.formItemViews.length; i++) {
                    this.formItemViews[i].displayValidationErrors(value);
                }
            };
            FormView.prototype.getData = function () {
                return this.data;
            };
            FormView.prototype.giveFocus = function () {
                var focusGiven = false;
                if (this.formItemViews.length > 0) {
                    for (var i = 0; i < this.formItemViews.length; i++) {
                        if (this.formItemViews[i].giveFocus()) {
                            focusGiven = true;
                            break;
                        }
                    }
                }
                return focusGiven;
            };
            FormView.prototype.onEditContentRequest = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onEditContentRequest(listener);
                });
            };
            FormView.prototype.unEditContentRequest = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unEditContentRequest(listener);
                });
            };
            FormView.prototype.onValidityChanged = function (listener) {
                this.formValidityChangedListeners.push(listener);
            };
            FormView.prototype.unValidityChanged = function (listener) {
                this.formValidityChangedListeners = this.formValidityChangedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormView.prototype.notifyValidityChanged = function (event) {
                var _this = this;
                this.formValidityChangedListeners.forEach(function (listener) {
                    listener.call(_this, event);
                });
            };
            FormView.prototype.notifyEditContentRequested = function (content) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.notifyEditContentRequested(content);
                });
            };
            FormView.prototype.onFocus = function (listener) {
                this.focusListeners.push(listener);
            };
            FormView.prototype.unFocus = function (listener) {
                this.focusListeners = this.focusListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            FormView.prototype.onBlur = function (listener) {
                this.blurListeners.push(listener);
            };
            FormView.prototype.unBlur = function (listener) {
                this.blurListeners = this.blurListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            FormView.prototype.notifyFocused = function (event) {
                this.focusListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormView.prototype.notifyBlurred = function (event) {
                this.blurListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            return FormView;
        })(api.dom.DivEl);
        _form.FormView = FormView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetLabel = (function (_super) {
            __extends(FormItemSetLabel, _super);
            function FormItemSetLabel(formItemSet) {
                _super.call(this, "form-item-set-label");
                this.formItemSet = formItemSet;
                var nodes = [];
                var dragHandle = new api.dom.SpanEl("drag-handle");
                dragHandle.setHtml(":::");
                nodes.push(dragHandle.getHTMLElement());
                nodes.push(document.createTextNode(formItemSet.getLabel()));
                if (formItemSet.getOccurrences().required()) {
                    nodes.push(document.createTextNode(" "));
                    var requiredMarker = new api.dom.SpanEl("required");
                    nodes.push(requiredMarker.getHTMLElement());
                }
                nodes.push(document.createTextNode(":"));
                this.getEl().appendChildren(nodes);
            }
            return FormItemSetLabel;
        })(api.dom.DivEl);
        form.FormItemSetLabel = FormItemSetLabel;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetOccurrence = (function (_super) {
            __extends(FormItemSetOccurrence, _super);
            function FormItemSetOccurrence(formItemSetOccurrences, index) {
                _super.call(this, formItemSetOccurrences, index, formItemSetOccurrences.getFormItemSet().getOccurrences());
            }
            return FormItemSetOccurrence;
        })(form.FormItemOccurrence);
        form.FormItemSetOccurrence = FormItemSetOccurrence;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetOccurrences = (function (_super) {
            __extends(FormItemSetOccurrences, _super);
            function FormItemSetOccurrences(config) {
                this.occurrencesCollapsed = false;
                _super.call(this, {
                    formItem: config.formItemSet,
                    propertyArray: config.propertyArray,
                    occurrenceViewContainer: config.occurrenceViewContainer,
                    allowedOccurrences: config.formItemSet.getOccurrences()
                });
                this.context = config.context;
                this.formItemSet = config.formItemSet;
                this.parent = config.parent;
                var dataSetCount = this.propertyArray.getSize();
                if (dataSetCount > 0) {
                    this.constructOccurrencesForData();
                }
                else {
                    this.constructOccurrencesForNoData();
                }
            }
            FormItemSetOccurrences.prototype.getFormItemSet = function () {
                return this.formItemSet;
            };
            FormItemSetOccurrences.prototype.getAllowedOccurrences = function () {
                return this.formItemSet.getOccurrences();
            };
            FormItemSetOccurrences.prototype.constructOccurrencesForNoData = function () {
                var minimumOccurrences = this.getAllowedOccurrences().getMinimum();
                if (minimumOccurrences > 0) {
                    for (var i = 0; i < minimumOccurrences; i++) {
                        this.addOccurrence(this.createNewOccurrence(this, i));
                    }
                }
                else {
                    if (this.context.getShowEmptyFormItemSetOccurrences()) {
                        this.addOccurrence(this.createNewOccurrence(this, 0));
                    }
                }
            };
            FormItemSetOccurrences.prototype.constructOccurrencesForData = function () {
                var _this = this;
                this.propertyArray.forEach(function (property, index) {
                    _this.addOccurrence(new form.FormItemSetOccurrence(_this, index));
                });
                if (this.countOccurrences() < this.formItemSet.getOccurrences().getMinimum()) {
                    for (var index = this.countOccurrences(); index < this.formItemSet.getOccurrences().getMinimum(); index++) {
                        this.addOccurrence(new form.FormItemSetOccurrence(this, index));
                    }
                }
            };
            FormItemSetOccurrences.prototype.createNewOccurrence = function (formItemOccurrences, insertAtIndex) {
                return new form.FormItemSetOccurrence(formItemOccurrences, insertAtIndex);
            };
            FormItemSetOccurrences.prototype.createNewOccurrenceView = function (occurrence) {
                var _this = this;
                var dataSet = this.propertyArray.getSet(occurrence.getIndex());
                if (!dataSet) {
                    dataSet = this.propertyArray.addSet();
                }
                var newOccurrenceView = new form.FormItemSetOccurrenceView({
                    context: this.context,
                    formItemSetOccurrence: occurrence,
                    formItemSet: this.formItemSet,
                    parent: this.parent,
                    dataSet: dataSet
                });
                newOccurrenceView.onRemoveButtonClicked(function (event) {
                    _this.doRemoveOccurrence(event.getView(), event.getIndex());
                });
                return newOccurrenceView;
            };
            FormItemSetOccurrences.prototype.showOccurrences = function (show) {
                var views = this.getOccurrenceViews();
                this.occurrencesCollapsed = !show;
                views.forEach(function (formItemSetOccurrenceView) {
                    formItemSetOccurrenceView.showContainer(show);
                });
            };
            FormItemSetOccurrences.prototype.isCollapsed = function () {
                return this.occurrencesCollapsed;
            };
            FormItemSetOccurrences.prototype.moveOccurrence = function (index, destinationIndex) {
                _super.prototype.moveOccurrence.call(this, index, destinationIndex);
            };
            return FormItemSetOccurrences;
        })(form.FormItemOccurrences);
        form.FormItemSetOccurrences = FormItemSetOccurrences;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetOccurrenceView = (function (_super) {
            __extends(FormItemSetOccurrenceView, _super);
            function FormItemSetOccurrenceView(config) {
                _super.call(this, "form-item-set-occurrence-view", config.formItemSetOccurrence);
                this.formItemViews = [];
                this.validityChangedListeners = [];
                this.context = config.context;
                this.formItemSetOccurrence = config.formItemSetOccurrence;
                this.formItemSet = config.formItemSet;
                this.parent = config.parent;
                this.constructedWithData = config.dataSet != null;
                this.propertySet = config.dataSet;
            }
            FormItemSetOccurrenceView.prototype.getDataPath = function () {
                return this.propertySet.getProperty().getPath();
            };
            FormItemSetOccurrenceView.prototype.layout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                this.removeButton = new api.dom.AEl("remove-button");
                this.appendChild(this.removeButton);
                this.removeButton.onClicked(function (event) {
                    _this.notifyRemoveButtonClicked();
                });
                this.label = new form.FormItemSetLabel(this.formItemSet);
                this.appendChild(this.label);
                this.formItemSetOccurrencesContainer = new api.dom.DivEl("form-item-set-occurrences-container");
                this.appendChild(this.formItemSetOccurrencesContainer);
                var layoutPromise = new form.FormItemLayer().setFormContext(this.context).setFormItems(this.formItemSet.getFormItems()).setParentElement(this.formItemSetOccurrencesContainer).setParent(this).layout(this.propertySet);
                layoutPromise.then(function (formItemViews) {
                    _this.formItemViews = formItemViews;
                    _this.validate(true);
                    _this.formItemViews.forEach(function (formItemView) {
                        formItemView.onValidityChanged(function (event) {
                            var previousValidState = _this.previousValidationRecording.isValid();
                            if (event.isValid()) {
                                _this.previousValidationRecording.removeByPath(event.getOrigin());
                            }
                            else {
                                _this.previousValidationRecording.flatten(event.getRecording());
                            }
                            if (previousValidState != _this.previousValidationRecording.isValid()) {
                                _this.notifyValidityChanged(new form.RecordingValidityChangedEvent(_this.previousValidationRecording, _this.resolveValidationRecordingPath()));
                            }
                        });
                    });
                    _this.refresh();
                    deferred.resolve(null);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                return deferred.promise;
            };
            FormItemSetOccurrenceView.prototype.getFormItemViews = function () {
                return this.formItemViews;
            };
            FormItemSetOccurrenceView.prototype.giveFocus = function () {
                var focusGiven = false;
                this.getFormItemViews().forEach(function (formItemView) {
                    if (!focusGiven && formItemView.giveFocus()) {
                        focusGiven = true;
                    }
                });
                return focusGiven;
            };
            FormItemSetOccurrenceView.prototype.refresh = function () {
                if (!this.formItemSetOccurrence.oneAndOnly()) {
                    this.label.addClass("drag-control");
                }
                else {
                    this.label.removeClass("drag-control");
                }
                this.removeButton.setVisible(this.formItemSetOccurrence.showRemoveButton());
            };
            FormItemSetOccurrenceView.prototype.showContainer = function (show) {
                if (show) {
                    this.formItemSetOccurrencesContainer.show();
                }
                else {
                    this.formItemSetOccurrencesContainer.hide();
                }
            };
            FormItemSetOccurrenceView.prototype.resolveValidationRecordingPath = function () {
                return new form.ValidationRecordingPath(this.getDataPath(), null);
            };
            FormItemSetOccurrenceView.prototype.getLastValidationRecording = function () {
                return this.previousValidationRecording;
            };
            FormItemSetOccurrenceView.prototype.displayValidationErrors = function (value) {
                this.formItemViews.forEach(function (view) {
                    view.displayValidationErrors(value);
                });
            };
            FormItemSetOccurrenceView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemViews.forEach(function (formItemView) {
                    if (!formItemView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FormItemSetOccurrenceView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var allRecordings = new form.ValidationRecording();
                this.formItemViews.forEach(function (formItemView) {
                    var currRecording = formItemView.validate(silent);
                    allRecordings.flatten(currRecording);
                });
                if (!silent) {
                    if (allRecordings.validityChanged(this.previousValidationRecording)) {
                        this.notifyValidityChanged(new form.RecordingValidityChangedEvent(allRecordings, this.resolveValidationRecordingPath()));
                    }
                }
                this.previousValidationRecording = allRecordings;
                return allRecordings;
            };
            FormItemSetOccurrenceView.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            FormItemSetOccurrenceView.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners.filter(function (currentListener) {
                    return listener == currentListener;
                });
            };
            FormItemSetOccurrenceView.prototype.notifyValidityChanged = function (event) {
                this.validityChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemSetOccurrenceView.prototype.onFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onFocus(listener);
                });
            };
            FormItemSetOccurrenceView.prototype.unFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unFocus(listener);
                });
            };
            FormItemSetOccurrenceView.prototype.onBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onBlur(listener);
                });
            };
            FormItemSetOccurrenceView.prototype.unBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unBlur(listener);
                });
            };
            return FormItemSetOccurrenceView;
        })(form.FormItemOccurrenceView);
        form.FormItemSetOccurrenceView = FormItemSetOccurrenceView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var PropertyArray = api.data.PropertyArray;
        var ValueTypes = api.data.ValueTypes;
        var FormItemSetView = (function (_super) {
            __extends(FormItemSetView, _super);
            function FormItemSetView(config) {
                _super.call(this, {
                    className: "form-item-set-view",
                    context: config.context,
                    formItem: config.formItemSet,
                    parent: config.parent
                });
                this.validityChangedListeners = [];
                this.parentDataSet = config.parentDataSet;
                this.formItemSet = config.formItemSet;
                this.addClass(this.formItemSet.getPath().getElements().length % 2 ? "even" : "odd");
            }
            FormItemSetView.prototype.layout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                this.occurrenceViewsContainer = new api.dom.DivEl("occurrence-views-container");
                wemjq(this.occurrenceViewsContainer.getHTMLElement()).sortable({
                    revert: false,
                    containment: this.getHTMLElement(),
                    cursor: 'move',
                    cursorAt: { left: 14, top: 14 },
                    distance: 20,
                    tolerance: 'pointer',
                    handle: '.drag-control',
                    placeholder: 'form-item-set-drop-target-placeholder',
                    helper: function (event, helper) { return api.ui.DragHelper.getHtml(); },
                    start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                    update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                });
                this.appendChild(this.occurrenceViewsContainer);
                var propertyArray = this.parentDataSet.getPropertyArray(this.formItemSet.getName());
                if (!propertyArray) {
                    propertyArray = PropertyArray.create().setType(ValueTypes.DATA).setName(this.formItemSet.getName()).setParent(this.parentDataSet).build();
                    this.parentDataSet.addPropertyArray(propertyArray);
                }
                this.formItemSetOccurrences = new form.FormItemSetOccurrences({
                    context: this.getContext(),
                    occurrenceViewContainer: this.occurrenceViewsContainer,
                    formItemSet: this.formItemSet,
                    parent: this.getParent(),
                    propertyArray: propertyArray
                });
                var layoutPromise = this.formItemSetOccurrences.layout();
                layoutPromise.then(function () {
                    _this.formItemSetOccurrences.onOccurrenceRendered(function () {
                        _this.validate(false);
                    });
                    _this.formItemSetOccurrences.onOccurrenceAdded(function (event) {
                        _this.refresh();
                        wemjq(_this.occurrenceViewsContainer.getHTMLElement()).sortable("refresh");
                        if (api.ObjectHelper.iFrameSafeInstanceOf(event.getOccurrenceView(), form.FormItemSetOccurrenceView)) {
                            var addedFormItemSetOccurrenceView = event.getOccurrenceView();
                            addedFormItemSetOccurrenceView.onValidityChanged(function (event) {
                                _this.handleFormItemSetOccurrenceViewValidityChanged(event);
                            });
                        }
                    });
                    _this.formItemSetOccurrences.onOccurrenceRemoved(function (event) {
                        _this.refresh();
                        if (api.ObjectHelper.iFrameSafeInstanceOf(event.getOccurrenceView(), form.FormItemSetOccurrenceView)) {
                            _this.validate(false);
                        }
                    });
                    _this.formItemSetOccurrences.getOccurrenceViews().forEach(function (formItemSetOccurrenceView) {
                        formItemSetOccurrenceView.onValidityChanged(function (event) {
                            _this.handleFormItemSetOccurrenceViewValidityChanged(event);
                        });
                    });
                    _this.bottomButtonRow = new api.dom.DivEl("bottom-button-row");
                    _this.appendChild(_this.bottomButtonRow);
                    _this.addButton = new api.ui.button.Button("Add " + _this.formItemSet.getLabel());
                    _this.addButton.addClass("small");
                    _this.addButton.onClicked(function (event) {
                        _this.formItemSetOccurrences.createAndAddOccurrence();
                        if (_this.formItemSetOccurrences.isCollapsed()) {
                            _this.collapseButton.getHTMLElement().click();
                        }
                    });
                    _this.collapseButton = new api.dom.AEl("collapse-button");
                    _this.collapseButton.setHtml("Collapse");
                    _this.collapseButton.onClicked(function (event) {
                        if (_this.formItemSetOccurrences.isCollapsed()) {
                            _this.collapseButton.setHtml("Collapse");
                            _this.formItemSetOccurrences.showOccurrences(true);
                        }
                        else {
                            _this.collapseButton.setHtml("Expand");
                            _this.formItemSetOccurrences.showOccurrences(false);
                        }
                    });
                    _this.bottomButtonRow.appendChild(_this.addButton);
                    _this.bottomButtonRow.appendChild(_this.collapseButton);
                    _this.validationViewer = new api.form.ValidationRecordingViewer();
                    _this.appendChild(_this.validationViewer);
                    _this.refresh();
                    _this.validate(true);
                    deferred.resolve(null);
                });
                return deferred.promise;
            };
            FormItemSetView.prototype.handleFormItemSetOccurrenceViewValidityChanged = function (event) {
                if (!this.previousValidationRecording) {
                    return;
                }
                var previousValidState = this.previousValidationRecording.isValid();
                if (event.isValid()) {
                    this.previousValidationRecording.removeByPath(event.getOrigin());
                }
                else {
                    this.previousValidationRecording.flatten(event.getRecording());
                }
                var validationRecordingPath = this.resolveValidationRecordingPath();
                var occurrenceViews = this.formItemSetOccurrences.getOccurrenceViews();
                var occurrenceRecording = new form.ValidationRecording();
                var numberOfValids = 0;
                occurrenceViews.forEach(function (occurrenceView) {
                    var recordingForOccurrence = occurrenceView.getLastValidationRecording();
                    if (recordingForOccurrence && recordingForOccurrence.isValid()) {
                        numberOfValids++;
                    }
                });
                if (numberOfValids < this.formItemSet.getOccurrences().getMinimum()) {
                    this.previousValidationRecording.breaksMinimumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMinimumOccurrences(validationRecordingPath);
                }
                else {
                    this.previousValidationRecording.removeUnreachedMinimumOccurrencesByPath(validationRecordingPath);
                }
                if (this.formItemSet.getOccurrences().maximumBreached(numberOfValids)) {
                    this.previousValidationRecording.breaksMaximumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMaximumOccurrences(validationRecordingPath);
                }
                else {
                    this.previousValidationRecording.removeBreachedMaximumOccurrencesByPath(validationRecordingPath);
                }
                this.renderValidationErrors(occurrenceRecording);
                if (previousValidState != this.previousValidationRecording.isValid()) {
                    this.notifyValidityChanged(new form.RecordingValidityChangedEvent(this.previousValidationRecording, this.resolveValidationRecordingPath()));
                }
                else if (previousValidState != event.isValid()) {
                    this.notifyValidityChanged(event);
                }
            };
            FormItemSetView.prototype.broadcastFormSizeChanged = function () {
                this.formItemSetOccurrences.getOccurrenceViews().forEach(function (occurrenceView) {
                    occurrenceView.getFormItemViews().forEach(function (formItemView) {
                        formItemView.broadcastFormSizeChanged();
                    });
                });
            };
            FormItemSetView.prototype.refresh = function () {
                this.collapseButton.setVisible(this.formItemSetOccurrences.getOccurrences().length > 0);
                this.addButton.setVisible(!this.formItemSetOccurrences.maximumOccurrencesReached());
            };
            FormItemSetView.prototype.getFormItemSetOccurrenceView = function (index) {
                return this.formItemSetOccurrences.getOccurrenceViews()[index];
            };
            FormItemSetView.prototype.resolveValidationRecordingPath = function () {
                return new form.ValidationRecordingPath(this.parentDataSet.getPropertyPath(), this.formItemSet.getName(), this.formItemSet.getOccurrences().getMinimum(), this.formItemSet.getOccurrences().getMaximum());
            };
            FormItemSetView.prototype.displayValidationErrors = function (value) {
                this.formItemSetOccurrences.getOccurrenceViews().forEach(function (view) {
                    view.displayValidationErrors(value);
                });
            };
            FormItemSetView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemSetOccurrences.getOccurrenceViews().forEach(function (formItemOccurrenceView) {
                    if (!formItemOccurrenceView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FormItemSetView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var validationRecordingPath = this.resolveValidationRecordingPath();
                var wholeRecording = new form.ValidationRecording();
                var occurrenceViews = this.formItemSetOccurrences.getOccurrenceViews();
                var occurrenceRecording = new form.ValidationRecording();
                var numberOfValids = 0;
                occurrenceViews.forEach(function (occurrenceView) {
                    var recordingForOccurrence = occurrenceView.validate(silent);
                    if (recordingForOccurrence.isValid()) {
                        numberOfValids++;
                    }
                    wholeRecording.flatten(recordingForOccurrence);
                });
                if (numberOfValids < this.formItemSet.getOccurrences().getMinimum()) {
                    wholeRecording.breaksMinimumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMinimumOccurrences(validationRecordingPath);
                }
                if (this.formItemSet.getOccurrences().maximumBreached(numberOfValids)) {
                    wholeRecording.breaksMaximumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMaximumOccurrences(validationRecordingPath);
                }
                if (!silent && wholeRecording.validityChanged(this.previousValidationRecording)) {
                    this.notifyValidityChanged(new form.RecordingValidityChangedEvent(wholeRecording, validationRecordingPath));
                }
                this.renderValidationErrors(occurrenceRecording);
                this.previousValidationRecording = wholeRecording;
                return wholeRecording;
            };
            FormItemSetView.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            FormItemSetView.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners.filter(function (currentListener) {
                    return listener == currentListener;
                });
            };
            FormItemSetView.prototype.notifyValidityChanged = function (event) {
                this.validityChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemSetView.prototype.renderValidationErrors = function (recording) {
                if (recording.isValid()) {
                    this.removeClass("invalid");
                    this.addClass("valid");
                }
                else {
                    this.removeClass("valid");
                    this.addClass("invalid");
                }
                this.validationViewer.setObject(recording);
            };
            FormItemSetView.prototype.giveFocus = function () {
                var focusGiven = false;
                if (this.formItemSetOccurrences.getOccurrenceViews().length > 0) {
                    var views = this.formItemSetOccurrences.getOccurrenceViews();
                    for (var i = 0; i < views.length; i++) {
                        if (views[i].giveFocus()) {
                            focusGiven = true;
                            break;
                        }
                    }
                }
                return focusGiven;
            };
            FormItemSetView.prototype.handleDnDStart = function (event, ui) {
                var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                api.util.assert(draggedElement.hasClass("form-item-set-occurrence-view"));
                this.draggingIndex = draggedElement.getSiblingIndex();
                ui.placeholder.html("Drop form item set here");
            };
            FormItemSetView.prototype.handleDnDUpdate = function (event, ui) {
                if (this.draggingIndex >= 0) {
                    var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                    api.util.assert(draggedElement.hasClass("form-item-set-occurrence-view"));
                    var draggedToIndex = draggedElement.getSiblingIndex();
                    this.formItemSetOccurrences.moveOccurrence(this.draggingIndex, draggedToIndex);
                }
                this.draggingIndex = -1;
            };
            FormItemSetView.prototype.onFocus = function (listener) {
                this.formItemSetOccurrences.onFocus(listener);
            };
            FormItemSetView.prototype.unFocus = function (listener) {
                this.formItemSetOccurrences.unFocus(listener);
            };
            FormItemSetView.prototype.onBlur = function (listener) {
                this.formItemSetOccurrences.onBlur(listener);
            };
            FormItemSetView.prototype.unBlur = function (listener) {
                this.formItemSetOccurrences.unBlur(listener);
            };
            return FormItemSetView;
        })(form.FormItemView);
        form.FormItemSetView = FormItemSetView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var LayoutView = (function (_super) {
            __extends(LayoutView, _super);
            function LayoutView(config) {
                _super.call(this, {
                    className: config.className,
                    context: config.context,
                    formItem: config.layout,
                    parent: config.parent
                });
                this._layout = config.layout;
            }
            LayoutView.prototype.layout = function () {
                throw new Error("Must be implemented by inheritors");
            };
            return LayoutView;
        })(form.FormItemView);
        form.LayoutView = LayoutView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FieldSetLabel = (function (_super) {
            __extends(FieldSetLabel, _super);
            function FieldSetLabel(fieldSet) {
                _super.call(this, "field-set-label");
                this.fieldSet = fieldSet;
                this.getEl().setInnerHtml(this.fieldSet.getLabel());
                this.getEl().setAttribute("title", this.fieldSet.getLabel());
            }
            return FieldSetLabel;
        })(api.dom.DivEl);
        form.FieldSetLabel = FieldSetLabel;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FieldSetView = (function (_super) {
            __extends(FieldSetView, _super);
            function FieldSetView(config) {
                _super.call(this, {
                    context: config.context,
                    layout: config.fieldSet,
                    parent: config.parent,
                    className: "field-set-view"
                });
                this.formItemViews = [];
                this.fieldSet = config.fieldSet;
                this.propertySet = config.dataSet;
            }
            FieldSetView.prototype.broadcastFormSizeChanged = function () {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.broadcastFormSizeChanged();
                });
            };
            FieldSetView.prototype.layout = function () {
                return this.doLayout();
            };
            FieldSetView.prototype.doLayout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var label = new form.FieldSetLabel(this.fieldSet);
                this.appendChild(label);
                var wrappingDiv = new api.dom.DivEl("field-set-container");
                this.appendChild(wrappingDiv);
                var layoutPromise = new form.FormItemLayer().setFormContext(this.getContext()).setFormItems(this.fieldSet.getFormItems()).setParentElement(wrappingDiv).setParent(this.getParent()).layout(this.propertySet);
                layoutPromise.then(function (formItemViews) {
                    _this.formItemViews = formItemViews;
                    deferred.resolve(null);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                return deferred.promise;
            };
            FieldSetView.prototype.onEditContentRequest = function (listener) {
                _super.prototype.onEditContentRequest.call(this, listener);
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onEditContentRequest(listener);
                });
            };
            FieldSetView.prototype.unEditContentRequest = function (listener) {
                _super.prototype.unEditContentRequest.call(this, listener);
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unEditContentRequest(listener);
                });
            };
            FieldSetView.prototype.giveFocus = function () {
                var focusGiven = false;
                if (this.formItemViews.length > 0) {
                    for (var i = 0; i < this.formItemViews.length; i++) {
                        if (this.formItemViews[i].giveFocus()) {
                            focusGiven = true;
                            break;
                        }
                    }
                }
                return focusGiven;
            };
            FieldSetView.prototype.displayValidationErrors = function (value) {
                this.formItemViews.forEach(function (view) {
                    view.displayValidationErrors(value);
                });
            };
            FieldSetView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemViews.forEach(function (formItemView) {
                    if (!formItemView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FieldSetView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var recording = new form.ValidationRecording();
                this.formItemViews.forEach(function (formItemView) {
                    recording.flatten(formItemView.validate(silent));
                });
                return recording;
            };
            FieldSetView.prototype.onValidityChanged = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onValidityChanged(listener);
                });
            };
            FieldSetView.prototype.unValidityChanged = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unValidityChanged(listener);
                });
            };
            FieldSetView.prototype.onFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onFocus(listener);
                });
            };
            FieldSetView.prototype.unFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unFocus(listener);
                });
            };
            FieldSetView.prototype.onBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onBlur(listener);
                });
            };
            FieldSetView.prototype.unBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unBlur(listener);
                });
            };
            return FieldSetView;
        })(form.LayoutView);
        form.FieldSetView = FieldSetView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var RemoveButtonClickedEvent = (function () {
            function RemoveButtonClickedEvent(view, index) {
                this.view = view;
                this.index = index;
            }
            RemoveButtonClickedEvent.prototype.getView = function () {
                return this.view;
            };
            RemoveButtonClickedEvent.prototype.getIndex = function () {
                return this.index;
            };
            return RemoveButtonClickedEvent;
        })();
        form.RemoveButtonClickedEvent = RemoveButtonClickedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrenceRemovedEvent = (function () {
            function OccurrenceRemovedEvent(occurrence, occurrenceView) {
                this.occurrence = occurrence;
                this.occurrenceView = occurrenceView;
            }
            OccurrenceRemovedEvent.prototype.getOccurrence = function () {
                return this.occurrence;
            };
            OccurrenceRemovedEvent.prototype.getOccurrenceView = function () {
                return this.occurrenceView;
            };
            return OccurrenceRemovedEvent;
        })();
        form.OccurrenceRemovedEvent = OccurrenceRemovedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrenceAddedEvent = (function () {
            function OccurrenceAddedEvent(occurrence, occurrenceView) {
                this.occurrence = occurrence;
                this.occurrenceView = occurrenceView;
            }
            OccurrenceAddedEvent.prototype.getOccurrence = function () {
                return this.occurrence;
            };
            OccurrenceAddedEvent.prototype.getOccurrenceView = function () {
                return this.occurrenceView;
            };
            return OccurrenceAddedEvent;
        })();
        form.OccurrenceAddedEvent = OccurrenceAddedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrenceRenderedEvent = (function () {
            function OccurrenceRenderedEvent(occurrence, occurrenceView) {
                this.occurrence = occurrence;
                this.occurrenceView = occurrenceView;
            }
            OccurrenceRenderedEvent.prototype.getOccurrence = function () {
                return this.occurrence;
            };
            OccurrenceRenderedEvent.prototype.getOccurrenceView = function () {
                return this.occurrenceView;
            };
            return OccurrenceRenderedEvent;
        })();
        form.OccurrenceRenderedEvent = OccurrenceRenderedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var ValidationRecordingViewer = (function (_super) {
            __extends(ValidationRecordingViewer, _super);
            function ValidationRecordingViewer() {
                _super.call(this, 'validation-viewer');
                this.minText = "Min {0} occurrence{1} required";
                this.maxTextPlural = "Max {0} occurrence{1} are allowed";
                this.maxTextSingle = "Max {0} occurrence{1} is allowed";
                this.list = new api.dom.UlEl();
                this.appendChild(this.list);
            }
            ValidationRecordingViewer.prototype.setObject = function (recording) {
                var _this = this;
                this.list.removeChildren();
                recording.breaksMinimumOccurrencesArray.forEach(function (path) {
                    _this.list.appendChild(_this.createItemView(path, true));
                });
                recording.breaksMaximumOccurrencesArray.forEach(function (path) {
                    _this.list.appendChild(_this.createItemView(path, false));
                });
            };
            ValidationRecordingViewer.prototype.createItemView = function (path, breaksMin) {
                var showPlural = breaksMin && path.getMin() > 1 || !breaksMin && path.getMax() > 1;
                var text = api.util.StringHelper.format(breaksMin ? this.minText : this.resolveMaxText(path), breaksMin ? path.getMin() : path.getMax(), showPlural ? 's' : '');
                return new api.dom.LiEl().setHtml(text);
            };
            ValidationRecordingViewer.prototype.resolveMaxText = function (path) {
                return path.getMax() > 1 ? this.maxTextPlural : this.maxTextSingle;
            };
            return ValidationRecordingViewer;
        })(api.ui.Viewer);
        form.ValidationRecordingViewer = ValidationRecordingViewer;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var ValueAddedEvent = (function () {
                function ValueAddedEvent(value) {
                    this.value = value;
                }
                ValueAddedEvent.prototype.getValue = function () {
                    return this.value;
                };
                return ValueAddedEvent;
            })();
            inputtype.ValueAddedEvent = ValueAddedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var ValueChangedEvent = (function () {
                function ValueChangedEvent(newValue, arrayIndex) {
                    this.newValue = newValue;
                    this.arrayIndex = arrayIndex;
                }
                ValueChangedEvent.prototype.getNewValue = function () {
                    return this.newValue;
                };
                ValueChangedEvent.prototype.getArrayIndex = function () {
                    return this.arrayIndex;
                };
                return ValueChangedEvent;
            })();
            inputtype.ValueChangedEvent = ValueChangedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var ValueRemovedEvent = (function () {
                function ValueRemovedEvent(arrayIndex) {
                    this.arrayIndex = arrayIndex;
                }
                ValueRemovedEvent.prototype.getArrayIndex = function () {
                    return this.arrayIndex;
                };
                return ValueRemovedEvent;
            })();
            inputtype.ValueRemovedEvent = ValueRemovedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var InputValidationRecording = (function () {
                function InputValidationRecording() {
                    this.breaksMinimumOccurrences = false;
                    this.breaksMaximumOccurrences = false;
                }
                InputValidationRecording.prototype.isValid = function () {
                    return !this.breaksMaximumOccurrences && !this.breaksMinimumOccurrences;
                };
                InputValidationRecording.prototype.setBreaksMinimumOccurrences = function (value) {
                    this.breaksMinimumOccurrences = value;
                };
                InputValidationRecording.prototype.setBreaksMaximumOccurrences = function (value) {
                    this.breaksMaximumOccurrences = value;
                };
                InputValidationRecording.prototype.isMinimumOccurrenesBreached = function () {
                    return this.breaksMinimumOccurrences;
                };
                InputValidationRecording.prototype.isMaximumOccurrenesBreached = function () {
                    return this.breaksMaximumOccurrences;
                };
                InputValidationRecording.prototype.equals = function (that) {
                    if (this.breaksMinimumOccurrences != that.breaksMinimumOccurrences) {
                        return false;
                    }
                    if (this.breaksMaximumOccurrences != that.breaksMaximumOccurrences) {
                        return false;
                    }
                    return true;
                };
                InputValidationRecording.prototype.validityChanged = function (other) {
                    return other == undefined || other == null || !other.equals(this);
                };
                return InputValidationRecording;
            })();
            inputtype.InputValidationRecording = InputValidationRecording;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var InputValidityChangedEvent = (function () {
                function InputValidityChangedEvent(recording, inputName) {
                    this.recording = recording;
                    this.inputName = inputName;
                }
                InputValidityChangedEvent.prototype.getInputName = function () {
                    return this.inputName;
                };
                InputValidityChangedEvent.prototype.isValid = function () {
                    return this.recording.isValid();
                };
                InputValidityChangedEvent.prototype.getRecording = function () {
                    return this.recording;
                };
                return InputValidityChangedEvent;
            })();
            inputtype.InputValidityChangedEvent = InputValidityChangedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var InputTypeManager = (function () {
                function InputTypeManager() {
                }
                InputTypeManager.isRegistered = function (inputTypeClassName) {
                    return InputTypeManager.inputTypes[inputTypeClassName] != undefined;
                };
                InputTypeManager.register = function (inputTypeClass) {
                    if (!InputTypeManager.isRegistered(inputTypeClass.getName())) {
                        InputTypeManager.inputTypes[inputTypeClass.getName()] = inputTypeClass;
                    }
                    else {
                        throw new Error('Input type [' + inputTypeClass.getName() + '] is already registered, unregister it first.');
                    }
                };
                InputTypeManager.unregister = function (inputTypeName) {
                    if (InputTypeManager.isRegistered(inputTypeName)) {
                        InputTypeManager.inputTypes[inputTypeName] = undefined;
                        console.log('Unregistered input type [' + inputTypeName + "]");
                    }
                    else {
                        throw new Error('Input type [' + inputTypeName + '] is not registered.');
                    }
                };
                InputTypeManager.createView = function (inputTypeClassName, context) {
                    if (InputTypeManager.isRegistered(inputTypeClassName)) {
                        var inputTypeClass = InputTypeManager.inputTypes[inputTypeClassName];
                        return inputTypeClass.newInstance(context);
                    }
                    else {
                        throw new Error("Input type [" + inputTypeClassName + "] need to be registered first.");
                    }
                };
                InputTypeManager.inputTypes = {};
                return InputTypeManager;
            })();
            inputtype.InputTypeManager = InputTypeManager;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var wem;
(function (wem) {
    wem.inputTypes = api.form.inputtype.InputTypeManager;
})(wem || (wem = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var InputOccurrence = (function (_super) {
                    __extends(InputOccurrence, _super);
                    function InputOccurrence(inputOccurrences, index) {
                        _super.call(this, inputOccurrences, index, inputOccurrences.getInput().getOccurrences());
                    }
                    return InputOccurrence;
                })(api.form.FormItemOccurrence);
                support.InputOccurrence = InputOccurrence;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var InputOccurrencesBuilder = (function () {
                    function InputOccurrencesBuilder() {
                    }
                    InputOccurrencesBuilder.prototype.setBaseInputTypeView = function (value) {
                        this.baseInputTypeView = value;
                        return this;
                    };
                    InputOccurrencesBuilder.prototype.setInput = function (value) {
                        this.input = value;
                        return this;
                    };
                    InputOccurrencesBuilder.prototype.setPropertyArray = function (value) {
                        this.propertyArray = value;
                        return this;
                    };
                    InputOccurrencesBuilder.prototype.build = function () {
                        return new InputOccurrences(this);
                    };
                    return InputOccurrencesBuilder;
                })();
                support.InputOccurrencesBuilder = InputOccurrencesBuilder;
                var InputOccurrences = (function (_super) {
                    __extends(InputOccurrences, _super);
                    function InputOccurrences(config) {
                        _super.call(this, {
                            formItem: config.input,
                            propertyArray: config.propertyArray,
                            occurrenceViewContainer: config.baseInputTypeView,
                            allowedOccurrences: config.input.getOccurrences()
                        });
                        this.baseInputTypeView = config.baseInputTypeView;
                        this.input = config.input;
                        if (this.propertyArray.getSize() > 0) {
                            this.constructOccurrencesForData();
                        }
                        else {
                            this.constructOccurrencesForNoData();
                        }
                    }
                    InputOccurrences.prototype.hasValidUserInput = function () {
                        var result = true;
                        this.getOccurrenceViews().forEach(function (formItemOccurrenceView) {
                            if (!formItemOccurrenceView.hasValidUserInput()) {
                                result = false;
                            }
                        });
                        return result;
                    };
                    InputOccurrences.prototype.moveOccurrence = function (fromIndex, toIndex) {
                        _super.prototype.moveOccurrence.call(this, fromIndex, toIndex);
                    };
                    InputOccurrences.prototype.getInput = function () {
                        return this.input;
                    };
                    InputOccurrences.prototype.getAllowedOccurrences = function () {
                        return this.input.getOccurrences();
                    };
                    InputOccurrences.prototype.constructOccurrencesForData = function () {
                        var _this = this;
                        this.propertyArray.forEach(function (property, index) {
                            _this.addOccurrence(new support.InputOccurrence(_this, index));
                        });
                        if (this.countOccurrences() < this.input.getOccurrences().getMinimum()) {
                            for (var index = this.countOccurrences(); index < this.input.getOccurrences().getMinimum(); index++) {
                                this.addOccurrence(this.createNewOccurrence(this, index));
                            }
                        }
                    };
                    InputOccurrences.prototype.createNewOccurrence = function (formItemOccurrences, insertAtIndex) {
                        return new support.InputOccurrence(formItemOccurrences, insertAtIndex);
                    };
                    InputOccurrences.prototype.createNewOccurrenceView = function (occurrence) {
                        var newInitialValue = this.baseInputTypeView.newInitialValue();
                        api.util.assertNotNull(newInitialValue, "InputTypeView-s extending BaseInputTypeNotManagingAdd must must return a Value from newInitialValue");
                        var property = this.propertyArray.get(occurrence.getIndex());
                        if (!property) {
                            property = this.propertyArray.add(newInitialValue);
                        }
                        var inputOccurrenceView = new support.InputOccurrenceView(occurrence, this.baseInputTypeView, property);
                        var inputOccurrences = this;
                        inputOccurrenceView.onRemoveButtonClicked(function (event) {
                            inputOccurrences.doRemoveOccurrence(event.getView(), event.getIndex());
                        });
                        return inputOccurrenceView;
                    };
                    InputOccurrences.prototype.giveFocus = function () {
                        var focusGiven = false;
                        var occurrenceViews = this.getOccurrenceViews();
                        if (occurrenceViews.length > 0) {
                            for (var i = 0; i < occurrenceViews.length; i++) {
                                if (occurrenceViews[i].giveFocus()) {
                                    focusGiven = true;
                                    break;
                                }
                            }
                        }
                        return focusGiven;
                    };
                    InputOccurrences.create = function () {
                        return new InputOccurrencesBuilder();
                    };
                    return InputOccurrences;
                })(api.form.FormItemOccurrences);
                support.InputOccurrences = InputOccurrences;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var InputOccurrenceView = (function (_super) {
                    __extends(InputOccurrenceView, _super);
                    function InputOccurrenceView(inputOccurrence, baseInputTypeView, property) {
                        var _this = this;
                        _super.call(this, "input-occurrence-view", inputOccurrence);
                        this.property = property;
                        this.inputTypeView = baseInputTypeView;
                        this.inputElement = this.inputTypeView.createInputOccurrenceElement(inputOccurrence.getIndex(), property);
                        this.requiredContractBroken = this.inputTypeView.valueBreaksRequiredContract(property != null ? property.getValue() : null);
                        var propertyValueChangedHandler = function (event) {
                            var newStateOfRequiredContractBroken = _this.inputTypeView.valueBreaksRequiredContract(event.getNewValue());
                            if (_this.requiredContractBroken != newStateOfRequiredContractBroken) {
                                _this.requiredContractBroken = newStateOfRequiredContractBroken;
                                _this.inputTypeView.notifyRequiredContractBroken(newStateOfRequiredContractBroken, inputOccurrence.getIndex());
                            }
                        };
                        property.onPropertyValueChanged(propertyValueChangedHandler);
                        this.inputOccurrence = inputOccurrence;
                        this.dragControl = new api.dom.DivEl("drag-control");
                        this.appendChild(this.dragControl);
                        this.removeButtonEl = new api.dom.AEl("remove-button");
                        this.appendChild(this.removeButtonEl);
                        this.removeButtonEl.onClicked(function (event) {
                            _this.notifyRemoveButtonClicked();
                        });
                        var inputWrapper = new api.dom.DivEl("input-wrapper");
                        this.appendChild(inputWrapper);
                        inputWrapper.appendChild(this.inputElement);
                        this.refresh();
                    }
                    InputOccurrenceView.prototype.refresh = function () {
                        if (this.inputOccurrence.oneAndOnly()) {
                            this.addClass("single-occurrence").removeClass("multiple-occurrence");
                        }
                        else {
                            this.addClass("multiple-occurrence").removeClass("single-occurrence");
                        }
                    };
                    InputOccurrenceView.prototype.getDataPath = function () {
                        return this.property.getPath();
                    };
                    InputOccurrenceView.prototype.getIndex = function () {
                        return this.inputOccurrence.getIndex();
                    };
                    InputOccurrenceView.prototype.getInputElement = function () {
                        return this.inputElement;
                    };
                    InputOccurrenceView.prototype.hasValidUserInput = function () {
                        return this.inputTypeView.hasInputElementValidUserInput(this.inputElement);
                    };
                    InputOccurrenceView.prototype.giveFocus = function () {
                        return this.inputElement.giveFocus();
                    };
                    InputOccurrenceView.prototype.onFocus = function (listener) {
                        this.inputElement.onFocus(listener);
                    };
                    InputOccurrenceView.prototype.unFocus = function (listener) {
                        this.inputElement.unFocus(listener);
                    };
                    InputOccurrenceView.prototype.onBlur = function (listener) {
                        this.inputElement.onBlur(listener);
                    };
                    InputOccurrenceView.prototype.unBlur = function (listener) {
                        this.inputElement.unBlur(listener);
                    };
                    return InputOccurrenceView;
                })(api.form.FormItemOccurrenceView);
                support.InputOccurrenceView = InputOccurrenceView;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var BaseInputTypeNotManagingAdd = (function (_super) {
                    __extends(BaseInputTypeNotManagingAdd, _super);
                    function BaseInputTypeNotManagingAdd(context, className) {
                        var _this = this;
                        _super.call(this, "input-type-view" + (className ? " " + className : ""));
                        this.inputValidityChangedListeners = [];
                        api.util.assertNotNull(context, "context cannot be null");
                        this.context = context;
                        wemjq(this.getHTMLElement()).sortable({
                            axis: "y",
                            containment: 'parent',
                            handle: '.drag-control',
                            tolerance: 'pointer',
                            start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                            update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                        });
                    }
                    BaseInputTypeNotManagingAdd.prototype.handleDnDStart = function (event, ui) {
                        var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                        this.draggingIndex = draggedElement.getSiblingIndex();
                        ui.placeholder.html("Drop form item set here");
                    };
                    BaseInputTypeNotManagingAdd.prototype.handleDnDUpdate = function (event, ui) {
                        if (this.draggingIndex >= 0) {
                            var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                            var draggedToIndex = draggedElement.getSiblingIndex();
                            this.inputOccurrences.moveOccurrence(this.draggingIndex, draggedToIndex);
                        }
                        this.draggingIndex = -1;
                    };
                    BaseInputTypeNotManagingAdd.prototype.availableSizeChanged = function () {
                    };
                    BaseInputTypeNotManagingAdd.prototype.getContext = function () {
                        return this.context;
                    };
                    BaseInputTypeNotManagingAdd.prototype.getElement = function () {
                        return this;
                    };
                    BaseInputTypeNotManagingAdd.prototype.isManagingAdd = function () {
                        return false;
                    };
                    BaseInputTypeNotManagingAdd.prototype.onOccurrenceAdded = function (listener) {
                        this.inputOccurrences.onOccurrenceAdded(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.onOccurrenceRemoved = function (listener) {
                        this.inputOccurrences.onOccurrenceRemoved(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unOccurrenceAdded = function (listener) {
                        this.inputOccurrences.unOccurrenceAdded(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unOccurrenceRemoved = function (listener) {
                        this.inputOccurrences.unOccurrenceRemoved(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.onValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.push(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    BaseInputTypeNotManagingAdd.prototype.notifyValidityChanged = function (event) {
                        this.inputValidityChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    BaseInputTypeNotManagingAdd.prototype.maximumOccurrencesReached = function () {
                        return this.inputOccurrences.maximumOccurrencesReached();
                    };
                    BaseInputTypeNotManagingAdd.prototype.createAndAddOccurrence = function () {
                        this.inputOccurrences.createAndAddOccurrence();
                    };
                    BaseInputTypeNotManagingAdd.prototype.layout = function (input, propertyArray) {
                        var _this = this;
                        this.input = input;
                        this.propertyArray = propertyArray;
                        this.inputOccurrences = support.InputOccurrences.create().setBaseInputTypeView(this).setInput(this.input).setPropertyArray(propertyArray).build();
                        this.onAdded(function (event) {
                            _this.onOccurrenceAdded(function () {
                                wemjq(_this.getHTMLElement()).sortable("refresh");
                            });
                        });
                        return this.inputOccurrences.layout().then(function () {
                            wemjq(_this.getHTMLElement()).sortable("refresh");
                        });
                    };
                    BaseInputTypeNotManagingAdd.prototype.hasValidUserInput = function () {
                        return this.inputOccurrences.hasValidUserInput();
                    };
                    BaseInputTypeNotManagingAdd.prototype.hasInputElementValidUserInput = function (inputElement) {
                        throw new Error("Must be implemented by inheritor");
                    };
                    BaseInputTypeNotManagingAdd.prototype.onFocus = function (listener) {
                        this.inputOccurrences.onFocus(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unFocus = function (listener) {
                        this.inputOccurrences.unFocus(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.onBlur = function (listener) {
                        this.inputOccurrences.onBlur(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unBlur = function (listener) {
                        this.inputOccurrences.unBlur(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.displayValidationErrors = function (value) {
                    };
                    BaseInputTypeNotManagingAdd.prototype.validate = function (silent) {
                        var _this = this;
                        if (silent === void 0) { silent = true; }
                        var recording = new api.form.inputtype.InputValidationRecording();
                        var numberOfValids = 0;
                        this.inputOccurrences.getOccurrenceViews().forEach(function (occurrenceView) {
                            var valueFromPropertyArray = _this.propertyArray.getValue(occurrenceView.getIndex());
                            if (valueFromPropertyArray) {
                                var breaksRequiredContract = _this.valueBreaksRequiredContract(valueFromPropertyArray);
                                if (!breaksRequiredContract) {
                                    numberOfValids++;
                                }
                            }
                        });
                        if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                            recording.setBreaksMinimumOccurrences(true);
                        }
                        if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                            recording.setBreaksMaximumOccurrences(true);
                        }
                        if (!silent) {
                            if (recording.validityChanged(this.previousValidationRecording)) {
                                this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                            }
                        }
                        this.previousValidationRecording = recording;
                        return recording;
                    };
                    BaseInputTypeNotManagingAdd.prototype.notifyRequiredContractBroken = function (state, index) {
                        this.validate(false);
                    };
                    BaseInputTypeNotManagingAdd.prototype.getInput = function () {
                        return this.input;
                    };
                    BaseInputTypeNotManagingAdd.prototype.valueBreaksRequiredContract = function (value) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.createInputOccurrenceElement = function (index, property) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.getValueType = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.newInitialValue = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.giveFocus = function () {
                        if (this.inputOccurrences) {
                            return this.inputOccurrences.giveFocus();
                        }
                        return false;
                    };
                    BaseInputTypeNotManagingAdd.prototype.onEditContentRequest = function (listener) {
                    };
                    BaseInputTypeNotManagingAdd.prototype.unEditContentRequest = function (listener) {
                    };
                    return BaseInputTypeNotManagingAdd;
                })(api.dom.DivEl);
                support.BaseInputTypeNotManagingAdd = BaseInputTypeNotManagingAdd;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var BaseInputTypeManagingAdd = (function (_super) {
                    __extends(BaseInputTypeManagingAdd, _super);
                    function BaseInputTypeManagingAdd(className) {
                        _super.call(this, "input-type-view" + (className ? " " + className : ""));
                        this.inputValidityChangedListeners = [];
                    }
                    BaseInputTypeManagingAdd.prototype.availableSizeChanged = function () {
                    };
                    BaseInputTypeManagingAdd.prototype.getElement = function () {
                        return this;
                    };
                    BaseInputTypeManagingAdd.prototype.isManagingAdd = function () {
                        return true;
                    };
                    BaseInputTypeManagingAdd.prototype.getValueType = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.newInitialValue = function () {
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.layout = function (input, propertyArray) {
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.hasValidUserInput = function () {
                        return true;
                    };
                    BaseInputTypeManagingAdd.prototype.displayValidationErrors = function (value) {
                    };
                    BaseInputTypeManagingAdd.prototype.validate = function (silent) {
                        if (silent === void 0) { silent = true; }
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.onValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.push(listener);
                    };
                    BaseInputTypeManagingAdd.prototype.unValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    BaseInputTypeManagingAdd.prototype.notifyValidityChanged = function (event) {
                        this.inputValidityChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    BaseInputTypeManagingAdd.prototype.giveFocus = function () {
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.onEditContentRequest = function (listener) {
                    };
                    BaseInputTypeManagingAdd.prototype.unEditContentRequest = function (listener) {
                    };
                    return BaseInputTypeManagingAdd;
                })(api.dom.DivEl);
                support.BaseInputTypeManagingAdd = BaseInputTypeManagingAdd;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var BaseInputTypeSingleOccurrence = (function (_super) {
                    __extends(BaseInputTypeSingleOccurrence, _super);
                    function BaseInputTypeSingleOccurrence(CONTEXT, className) {
                        _super.call(this, "input-type-view" + (className ? " " + className : ""));
                        api.util.assertNotNull(CONTEXT, "CONTEXT cannt be null");
                        this.context = CONTEXT;
                    }
                    BaseInputTypeSingleOccurrence.prototype.availableSizeChanged = function () {
                    };
                    BaseInputTypeSingleOccurrence.prototype.getContext = function () {
                        return this.context;
                    };
                    BaseInputTypeSingleOccurrence.prototype.getElement = function () {
                        return this;
                    };
                    BaseInputTypeSingleOccurrence.prototype.isManagingAdd = function () {
                        return true;
                    };
                    BaseInputTypeSingleOccurrence.prototype.layout = function (input, propertyArray) {
                        this.layoutProperty(input, propertyArray.get(0));
                        return wemQ(null);
                    };
                    BaseInputTypeSingleOccurrence.prototype.layoutProperty = function (input, property) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.getValueType = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.newInitialValue = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.displayValidationErrors = function (value) {
                    };
                    BaseInputTypeSingleOccurrence.prototype.hasValidUserInput = function () {
                        return true;
                    };
                    BaseInputTypeSingleOccurrence.prototype.validate = function (silent) {
                        if (silent === void 0) { silent = true; }
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.onValidityChanged = function (listener) {
                    };
                    BaseInputTypeSingleOccurrence.prototype.unValidityChanged = function (listener) {
                    };
                    BaseInputTypeSingleOccurrence.prototype.onFocus = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.unFocus = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.onBlur = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.unBlur = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.onEditContentRequest = function (listener) {
                    };
                    BaseInputTypeSingleOccurrence.prototype.unEditContentRequest = function (listener) {
                    };
                    return BaseInputTypeSingleOccurrence;
                })(api.dom.DivEl);
                support.BaseInputTypeSingleOccurrence = BaseInputTypeSingleOccurrence;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var ValueTypes = api.data.ValueTypes;
                var NoInputTypeFoundView = (function (_super) {
                    __extends(NoInputTypeFoundView, _super);
                    function NoInputTypeFoundView(context) {
                        _super.call(this, context);
                    }
                    NoInputTypeFoundView.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    NoInputTypeFoundView.prototype.newInitialValue = function () {
                        return ValueTypes.STRING.newValue("");
                    };
                    NoInputTypeFoundView.prototype.layout = function (input, property) {
                        var divEl = new api.dom.DivEl();
                        divEl.getEl().setInnerHtml("Warning: no input type found: " + input.getInputType().toString());
                        return _super.prototype.layout.call(this, input, property);
                    };
                    NoInputTypeFoundView.prototype.createInputOccurrenceElement = function (index, property) {
                        var inputEl = api.ui.text.TextInput.middle();
                        inputEl.setName(this.getInput().getName());
                        if (property != null) {
                            inputEl.setValue(property.getString());
                        }
                        inputEl.onValueChanged(function (event) {
                            property.setValue(ValueTypes.STRING.newValue(event.getNewValue()));
                        });
                        return inputEl;
                    };
                    NoInputTypeFoundView.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    NoInputTypeFoundView.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    return NoInputTypeFoundView;
                })(support.BaseInputTypeNotManagingAdd);
                support.NoInputTypeFoundView = NoInputTypeFoundView;
                api.form.inputtype.InputTypeManager.register(new api.Class("NoInputTypeFound", NoInputTypeFoundView));
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var ValueChangedEvent = (function () {
                    function ValueChangedEvent(newValue) {
                        api.util.assertNotNull(newValue, "sending ValueChangedEvent-s for null values is not allowed");
                        this.newValue = newValue;
                    }
                    ValueChangedEvent.prototype.getNewValue = function () {
                        return this.newValue;
                    };
                    return ValueChangedEvent;
                })();
                support.ValueChangedEvent = ValueChangedEvent;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var combobox;
            (function (combobox) {
                var ComboBoxDisplayValueViewer = (function (_super) {
                    __extends(ComboBoxDisplayValueViewer, _super);
                    function ComboBoxDisplayValueViewer() {
                        _super.call(this);
                    }
                    ComboBoxDisplayValueViewer.prototype.setObject = function (value) {
                        _super.prototype.setObject.call(this, value);
                        this.getEl().setInnerHtml(value);
                    };
                    ComboBoxDisplayValueViewer.prototype.getPreferredHeight = function () {
                        return 34;
                    };
                    return ComboBoxDisplayValueViewer;
                })(api.ui.Viewer);
                combobox.ComboBoxDisplayValueViewer = ComboBoxDisplayValueViewer;
            })(combobox = inputtype.combobox || (inputtype.combobox = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var combobox;
            (function (combobox) {
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var ComboBox = (function (_super) {
                    __extends(ComboBox, _super);
                    function ComboBox(context) {
                        _super.call(this, "combo-box");
                        this.addClass("input-type-view");
                        this.context = context;
                        this.comboBoxConfig = context.inputConfig;
                    }
                    ComboBox.prototype.availableSizeChanged = function () {
                    };
                    ComboBox.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    ComboBox.prototype.newInitialValue = function () {
                        return null;
                    };
                    ComboBox.prototype.layout = function (input, propertyArray) {
                        var _this = this;
                        this.input = input;
                        this.propertyArray = propertyArray;
                        this.selectedOptionsView = new api.ui.selector.combobox.BaseSelectedOptionsView();
                        this.comboBox = this.createComboBox(input);
                        this.comboBoxConfig.options.forEach(function (option) {
                            _this.comboBox.addOption({ value: option.value, displayValue: option.label });
                        });
                        var valueArray = [];
                        this.propertyArray.forEach(function (property) {
                            valueArray.push(property.getString());
                        });
                        this.comboBox.setValues(valueArray);
                        this.appendChild(this.comboBox);
                        this.appendChild(this.selectedOptionsView);
                        return wemQ(null);
                    };
                    ComboBox.prototype.createComboBox = function (input) {
                        var _this = this;
                        var comboBox = new api.ui.selector.combobox.ComboBox(name, {
                            filter: this.comboboxFilter,
                            selectedOptionsView: this.selectedOptionsView,
                            maximumOccurrences: input.getOccurrences().getMaximum(),
                            optionDisplayValueViewer: new combobox.ComboBoxDisplayValueViewer(),
                            hideComboBoxWhenMaxReached: true
                        });
                        comboBox.onOptionFilterInputValueChanged(function (event) {
                            _this.comboBox.setFilterArgs({ searchString: event.getNewValue() });
                        });
                        comboBox.onOptionSelected(function (event) {
                            var value = new Value(event.getOption().value, ValueTypes.STRING);
                            if (comboBox.countSelectedOptions() == 1) {
                                _this.propertyArray.set(0, value);
                            }
                            else {
                                _this.propertyArray.add(value);
                            }
                            _this.validate(false);
                        });
                        comboBox.onOptionDeselected(function (removed) {
                            _this.propertyArray.remove(removed.getIndex());
                            _this.validate(false);
                        });
                        return comboBox;
                    };
                    ComboBox.prototype.giveFocus = function () {
                        if (this.comboBox.maximumOccurrencesReached()) {
                            return false;
                        }
                        return this.comboBox.giveFocus();
                    };
                    ComboBox.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || !this.isExistingValue(value.getString());
                    };
                    ComboBox.prototype.isExistingValue = function (value) {
                        return this.comboBoxConfig.options.some(function (option) {
                            return option.value == value;
                        });
                    };
                    ComboBox.prototype.comboboxFilter = function (item, args) {
                        return !(args && args.searchString && item.displayValue.toUpperCase().indexOf(args.searchString.toUpperCase()) == -1);
                    };
                    ComboBox.prototype.validate = function (silent) {
                        if (silent === void 0) { silent = true; }
                        var recording = new api.form.inputtype.InputValidationRecording();
                        var numberOfValids = this.comboBox.countSelectedOptions();
                        if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                            recording.setBreaksMinimumOccurrences(true);
                        }
                        if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                            recording.setBreaksMaximumOccurrences(true);
                        }
                        if (!silent && recording.validityChanged(this.previousValidationRecording)) {
                            this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                        }
                        this.previousValidationRecording = recording;
                        return recording;
                    };
                    ComboBox.prototype.onFocus = function (listener) {
                        this.comboBox.onFocus(listener);
                    };
                    ComboBox.prototype.unFocus = function (listener) {
                        this.comboBox.unFocus(listener);
                    };
                    ComboBox.prototype.onBlur = function (listener) {
                        this.comboBox.onBlur(listener);
                    };
                    ComboBox.prototype.unBlur = function (listener) {
                        this.comboBox.unBlur(listener);
                    };
                    return ComboBox;
                })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                combobox.ComboBox = ComboBox;
                api.form.inputtype.InputTypeManager.register(new api.Class("ComboBox", ComboBox));
            })(combobox = inputtype.combobox || (inputtype.combobox = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var singleselector;
            (function (singleselector) {
                var Dropdown = api.ui.selector.dropdown.Dropdown;
                var ComboBox = api.ui.selector.combobox.ComboBox;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                var SingleSelector = (function (_super) {
                    __extends(SingleSelector, _super);
                    function SingleSelector(config) {
                        _super.call(this, config, "single-selector");
                        this.type = config && config.inputConfig.selectorType && config.inputConfig.selectorType.toUpperCase();
                        if (!(SingleSelector.TYPE_RADIO == this.type || SingleSelector.TYPE_COMBOBOX == this.type || SingleSelector.TYPE_DROPDOWN == this.type)) {
                            throw new Error("Unsupported type of SingleSelector: " + this.type);
                        }
                    }
                    SingleSelector.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    SingleSelector.prototype.newInitialValue = function () {
                        return ValueTypes.STRING.newNullValue();
                    };
                    SingleSelector.prototype.createInputOccurrenceElement = function (index, property) {
                        var name = this.getInput().getName() + "-" + index;
                        if (SingleSelector.TYPE_RADIO == this.type) {
                            return this.createRadioElement(name, property);
                        }
                        else if (SingleSelector.TYPE_COMBOBOX == this.type) {
                            return this.createComboBoxElement(name, property);
                        }
                        else if (SingleSelector.TYPE_DROPDOWN == this.type) {
                            return this.createDropdownElement(name, property);
                        }
                        return null;
                    };
                    SingleSelector.prototype.createComboBoxElement = function (name, property) {
                        var _this = this;
                        var comboAndSelectedOptionsWrapper = new api.dom.DivEl();
                        var selectedOptionsView = new BaseSelectedOptionsView();
                        var comboBox = new ComboBox(name, {
                            filter: this.comboboxFilter,
                            selectedOptionsView: selectedOptionsView,
                            maximumOccurrences: 1,
                            hideComboBoxWhenMaxReached: true
                        });
                        var inputConfig = this.getContext().inputConfig;
                        if (inputConfig) {
                            var option;
                            for (var i = 0; i < inputConfig.options.length; i++) {
                                option = inputConfig.options[i];
                                comboBox.addOption({ value: option.value, displayValue: option.label });
                            }
                        }
                        if (property.hasNonNullValue()) {
                            comboBox.setValue(property.getString());
                        }
                        comboBox.onOptionFilterInputValueChanged(function (event) {
                            event.getGrid().getDataView().setFilterArgs({ searchString: event.getNewValue() });
                            event.getGrid().getDataView().refresh();
                        });
                        comboBox.onOptionSelected(function (event) {
                            property.setValue(_this.newValue(event.getOption().value));
                        });
                        comboAndSelectedOptionsWrapper.appendChild(comboBox);
                        comboAndSelectedOptionsWrapper.appendChild(selectedOptionsView);
                        return comboAndSelectedOptionsWrapper;
                    };
                    SingleSelector.prototype.createDropdownElement = function (name, property) {
                        var _this = this;
                        var dropdown = new Dropdown(name, {});
                        var inputEl = new api.ui.Dropdown(name);
                        var inputConfig = this.getContext().inputConfig;
                        if (inputConfig) {
                            for (var i = 0; i < inputConfig.options.length; i++) {
                                var option = inputConfig.options[i];
                                inputEl.addOption(option.value, option.label);
                                var option2 = { value: option.value, displayValue: option.label };
                                dropdown.addOption(option2);
                            }
                        }
                        if (property.hasNonNullValue()) {
                            inputEl.setValue(property.getString());
                            dropdown.setValue(property.getString());
                        }
                        dropdown.onOptionSelected(function (event) {
                            property.setValue(_this.newValue(event.getOption().value));
                        });
                        return dropdown;
                    };
                    SingleSelector.prototype.createRadioElement = function (name, property) {
                        var _this = this;
                        var radioGroup = new api.ui.RadioGroup(name);
                        var inputConfig = this.getContext().inputConfig;
                        if (inputConfig) {
                            for (var i = 0; i < inputConfig.options.length; i++) {
                                var option = inputConfig.options[i];
                                radioGroup.addOption(option.value, option.label);
                            }
                        }
                        if (property.hasNonNullValue()) {
                            radioGroup.setValue(property.getString());
                        }
                        radioGroup.onValueChanged(function (event) {
                            property.setValue(_this.newValue(event.getNewValue()));
                        });
                        return radioGroup;
                    };
                    SingleSelector.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    SingleSelector.prototype.comboboxFilter = function (item, args) {
                        return !(args && args.searchString && item.displayValue.toUpperCase().indexOf(args.searchString.toUpperCase()) == -1);
                    };
                    SingleSelector.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || !this.isExistingValue(value.getString());
                    };
                    SingleSelector.prototype.isExistingValue = function (value) {
                        var options = this.getContext().inputConfig.options || [];
                        return options.some(function (option) {
                            return option.value == value;
                        });
                    };
                    SingleSelector.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    SingleSelector.TYPE_DROPDOWN = "DROPDOWN";
                    SingleSelector.TYPE_RADIO = "RADIO";
                    SingleSelector.TYPE_COMBOBOX = "COMBOBOX";
                    return SingleSelector;
                })(api.form.inputtype.support.BaseInputTypeNotManagingAdd);
                singleselector.SingleSelector = SingleSelector;
                api.form.inputtype.InputTypeManager.register(new api.Class("SingleSelector", SingleSelector));
            })(singleselector = inputtype.singleselector || (inputtype.singleselector = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var support = api.form.inputtype.support;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var TextLine = (function (_super) {
                    __extends(TextLine, _super);
                    function TextLine(config) {
                        _super.call(this, config);
                    }
                    TextLine.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    TextLine.prototype.newInitialValue = function () {
                        return new Value("", ValueTypes.STRING);
                    };
                    TextLine.prototype.createInputOccurrenceElement = function (index, property) {
                        var _this = this;
                        var inputEl = api.ui.text.TextInput.middle();
                        if (property.hasNonNullValue()) {
                            inputEl.setName(this.getInput().getName() + "-" + property.getIndex());
                            inputEl.setValue(property.getString());
                        }
                        else {
                            inputEl.setName(this.getInput().getName());
                        }
                        inputEl.onValueChanged(function (event) {
                            property.setValue(_this.newValue(event.getNewValue()));
                        });
                        return inputEl;
                    };
                    TextLine.prototype.availableSizeChanged = function () {
                    };
                    TextLine.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    TextLine.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    TextLine.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    TextLine.getName = function () {
                        return new api.form.InputTypeName("TextLine", false);
                    };
                    return TextLine;
                })(support.BaseInputTypeNotManagingAdd);
                text.TextLine = TextLine;
                api.form.inputtype.InputTypeManager.register(new api.Class(TextLine.getName().getName(), TextLine));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var support = api.form.inputtype.support;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var TextArea = (function (_super) {
                    __extends(TextArea, _super);
                    function TextArea(config) {
                        _super.call(this, config);
                    }
                    TextArea.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    TextArea.prototype.newInitialValue = function () {
                        return new Value("", ValueTypes.STRING);
                    };
                    TextArea.prototype.createInputOccurrenceElement = function (index, property) {
                        var _this = this;
                        var inputEl = new api.ui.text.TextArea(this.getInput().getName() + "-" + index);
                        if (property.hasNonNullValue()) {
                            inputEl.setValue(property.getString());
                        }
                        inputEl.onValueChanged(function (event) {
                            property.setValue(_this.newValue(event.getNewValue()));
                        });
                        return inputEl;
                    };
                    TextArea.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    TextArea.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    TextArea.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    TextArea.getName = function () {
                        return new api.form.InputTypeName("TextArea", false);
                    };
                    return TextArea;
                })(support.BaseInputTypeNotManagingAdd);
                text.TextArea = TextArea;
                api.form.inputtype.InputTypeManager.register(new api.Class(TextArea.getName().getName(), TextArea));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var support = api.form.inputtype.support;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var HtmlArea = (function (_super) {
                    __extends(HtmlArea, _super);
                    function HtmlArea(config) {
                        _super.call(this, config);
                    }
                    HtmlArea.prototype.getValueType = function () {
                        return ValueTypes.HTML_PART;
                    };
                    HtmlArea.prototype.newInitialValue = function () {
                        return ValueTypes.HTML_PART.newValue("");
                    };
                    HtmlArea.prototype.createInputOccurrenceElement = function (index, property) {
                        var _this = this;
                        var textAreaEl = new api.ui.text.TextArea(this.getInput().getName() + "-" + index);
                        if (property.hasNonNullValue()) {
                            textAreaEl.setValue(property.getString());
                        }
                        textAreaEl.onValueChanged(function (event) {
                            var value = _this.newValue(event.getNewValue());
                            property.setValue(value);
                        });
                        return textAreaEl;
                    };
                    HtmlArea.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.HTML_PART);
                    };
                    HtmlArea.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.HTML_PART) || api.util.StringHelper.isBlank(value.getString());
                    };
                    HtmlArea.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    return HtmlArea;
                })(support.BaseInputTypeNotManagingAdd);
                text.HtmlArea = HtmlArea;
                api.form.inputtype.InputTypeManager.register(new api.Class("HtmlArea", HtmlArea));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var checkbox;
                (function (checkbox) {
                    var ValueTypes = api.data.ValueTypes;
                    var BaseInputTypeSingleOccurrence = api.form.inputtype.support.BaseInputTypeSingleOccurrence;
                    var Checkbox = (function (_super) {
                        __extends(Checkbox, _super);
                        function Checkbox(config) {
                            _super.call(this, config);
                            this.checkbox = new api.ui.Checkbox();
                        }
                        Checkbox.prototype.getValueType = function () {
                            return ValueTypes.BOOLEAN;
                        };
                        Checkbox.prototype.newInitialValue = function () {
                            return ValueTypes.BOOLEAN.newFalse();
                        };
                        Checkbox.prototype.layoutProperty = function (input, property) {
                            if (property.hasNonNullValue()) {
                                this.checkbox.setChecked(property.getBoolean());
                            }
                            else {
                                this.checkbox.setChecked(false);
                            }
                            this.appendChild(this.checkbox);
                            this.checkbox.onValueChanged(function (event) {
                                var newValue = ValueTypes.BOOLEAN.newValue(event.getNewValue());
                                if (newValue) {
                                    property.setValue(newValue);
                                }
                            });
                            return wemQ(null);
                        };
                        Checkbox.prototype.giveFocus = function () {
                            return this.checkbox.giveFocus();
                        };
                        Checkbox.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            return new api.form.inputtype.InputValidationRecording();
                        };
                        Checkbox.prototype.onFocus = function (listener) {
                            this.checkbox.onFocus(listener);
                        };
                        Checkbox.prototype.unFocus = function (listener) {
                            this.checkbox.unFocus(listener);
                        };
                        Checkbox.prototype.onBlur = function (listener) {
                            this.checkbox.onBlur(listener);
                        };
                        Checkbox.prototype.unBlur = function (listener) {
                            this.checkbox.unBlur(listener);
                        };
                        return Checkbox;
                    })(BaseInputTypeSingleOccurrence);
                    checkbox.Checkbox = Checkbox;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Checkbox", Checkbox));
                })(checkbox = inputtype.checkbox || (inputtype.checkbox = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var long;
                (function (long) {
                    var BaseInputTypeNotManagingAdd = api.form.inputtype.support.BaseInputTypeNotManagingAdd;
                    var ValueTypes = api.data.ValueTypes;
                    var Long = (function (_super) {
                        __extends(Long, _super);
                        function Long(config) {
                            _super.call(this, config);
                        }
                        Long.prototype.getValueType = function () {
                            return ValueTypes.LONG;
                        };
                        Long.prototype.newInitialValue = function () {
                            return ValueTypes.LONG.newNullValue();
                        };
                        Long.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var inputEl = api.ui.text.TextInput.middle();
                            inputEl.setName(this.getInput().getName() + "-" + property.getIndex());
                            inputEl.setValue(!property.hasNullValue() ? property.getString() : "");
                            inputEl.onValueChanged(function (event) {
                                var value = ValueTypes.LONG.newValue(event.getNewValue());
                                property.setValue(value);
                                inputEl.updateValidationStatusOnUserInput(_this.isValid(event.getNewValue()));
                            });
                            return inputEl;
                        };
                        Long.prototype.availableSizeChanged = function () {
                        };
                        Long.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LONG);
                        };
                        Long.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var value = inputElement;
                            return this.isValid(value.getValue());
                        };
                        Long.prototype.isValid = function (value) {
                            var validUserInput = true;
                            if (api.util.StringHelper.isEmpty(value)) {
                                validUserInput = true;
                            }
                            else {
                                if (api.util.NumberHelper.isWholeNumber(+value)) {
                                    validUserInput = true;
                                }
                                else {
                                    validUserInput = false;
                                }
                            }
                            return validUserInput;
                        };
                        return Long;
                    })(BaseInputTypeNotManagingAdd);
                    long.Long = Long;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Long", Long));
                })(long = inputtype.long || (inputtype.long = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var double;
                (function (double) {
                    var BaseInputTypeNotManagingAdd = api.form.inputtype.support.BaseInputTypeNotManagingAdd;
                    var ValueTypes = api.data.ValueTypes;
                    var Double = (function (_super) {
                        __extends(Double, _super);
                        function Double(config) {
                            _super.call(this, config);
                        }
                        Double.prototype.getValueType = function () {
                            return ValueTypes.DOUBLE;
                        };
                        Double.prototype.newInitialValue = function () {
                            return ValueTypes.DOUBLE.newNullValue();
                        };
                        Double.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var inputEl = api.ui.text.TextInput.middle();
                            inputEl.setName(this.getInput().getName() + "-" + property.getIndex());
                            inputEl.setValue(!property.hasNullValue() ? property.getString() : "");
                            inputEl.onValueChanged(function (event) {
                                var value = ValueTypes.DOUBLE.newValue(event.getNewValue());
                                property.setValue(value);
                                inputEl.updateValidationStatusOnUserInput(_this.isValid(event.getNewValue()));
                            });
                            return inputEl;
                        };
                        Double.prototype.availableSizeChanged = function () {
                        };
                        Double.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.DOUBLE);
                        };
                        Double.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var value = inputElement;
                            return this.isValid(value.getValue());
                        };
                        Double.prototype.isValid = function (value) {
                            var validUserInput = true;
                            if (api.util.StringHelper.isEmpty(value)) {
                                validUserInput = true;
                            }
                            else {
                                if (api.util.NumberHelper.isNumber(+value)) {
                                    validUserInput = true;
                                }
                                else {
                                    validUserInput = false;
                                }
                            }
                            return validUserInput;
                        };
                        return Double;
                    })(BaseInputTypeNotManagingAdd);
                    double.Double = Double;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Double", Double));
                })(double = inputtype.double || (inputtype.double = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var time;
                (function (time) {
                    var support = api.form.inputtype.support;
                    var ValueTypes = api.data.ValueTypes;
                    var Time = (function (_super) {
                        __extends(Time, _super);
                        function Time(config) {
                            _super.call(this, config);
                        }
                        Time.prototype.getValueType = function () {
                            return ValueTypes.LOCAL_TIME;
                        };
                        Time.prototype.newInitialValue = function () {
                            return ValueTypes.LOCAL_TIME.newNullValue();
                        };
                        Time.prototype.createInputOccurrenceElement = function (index, property) {
                            var localTimeEl = new api.ui.time.LocalTime();
                            var timeValue = property.getLocalTime();
                            if (timeValue) {
                                localTimeEl.setTime(timeValue);
                            }
                            localTimeEl.onTimeChanged(function (hours, minutes) {
                                var utcTime = api.util.DateHelper.parseUTCTime(hours + ":" + minutes);
                                var changedValue = ValueTypes.LOCAL_TIME.newValue(utcTime);
                                property.setValue(changedValue);
                            });
                            return localTimeEl;
                        };
                        Time.prototype.availableSizeChanged = function () {
                        };
                        Time.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LOCAL_TIME);
                        };
                        Time.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var localTime = inputElement;
                            var timePicker = localTime.getTimePicker();
                            return timePicker.hasValidUserInput();
                        };
                        return Time;
                    })(support.BaseInputTypeNotManagingAdd);
                    time.Time = Time;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Time", Time));
                })(time = inputtype.time || (inputtype.time = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var time;
                (function (time) {
                    var support = api.form.inputtype.support;
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var Date = (function (_super) {
                        __extends(Date, _super);
                        function Date(config) {
                            _super.call(this, config);
                        }
                        Date.prototype.getValueType = function () {
                            return ValueTypes.LOCAL_DATE;
                        };
                        Date.prototype.newInitialValue = function () {
                            return ValueTypes.LOCAL_DATE.newNullValue();
                        };
                        Date.prototype.createInputOccurrenceElement = function (index, property) {
                            var datePickerBuilder = new api.ui.time.DatePickerBuilder();
                            if (!property.hasNullValue()) {
                                var date = property.getLocalDate();
                                datePickerBuilder.setSelectedDate(date).setYear(date.getUTCFullYear()).setMonth(date.getUTCMonth());
                            }
                            var datePicker = datePickerBuilder.build();
                            datePicker.onSelectedDateChanged(function (event) {
                                var changedValue = new Value(event.getDate(), ValueTypes.LOCAL_DATE);
                                property.setValue(changedValue);
                            });
                            return datePicker;
                        };
                        Date.prototype.availableSizeChanged = function () {
                        };
                        Date.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LOCAL_DATE);
                        };
                        Date.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var datePicker = inputElement;
                            return datePicker.hasValidUserInput();
                        };
                        return Date;
                    })(support.BaseInputTypeNotManagingAdd);
                    time.Date = Date;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Date", Date));
                })(time = inputtype.time || (inputtype.time = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var time;
                (function (time) {
                    var support = api.form.inputtype.support;
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var DateTime = (function (_super) {
                        __extends(DateTime, _super);
                        function DateTime(config) {
                            _super.call(this, config);
                        }
                        DateTime.prototype.getValueType = function () {
                            return ValueTypes.LOCAL_DATE_TIME;
                        };
                        DateTime.prototype.newInitialValue = function () {
                            return ValueTypes.LOCAL_DATE_TIME.newNullValue();
                        };
                        DateTime.prototype.createInputOccurrenceElement = function (index, property) {
                            var dateTimeBuilder = new api.ui.time.DateTimePickerBuilder();
                            if (property.hasNonNullValue()) {
                                var date = property.getLocalDateTime();
                                dateTimeBuilder.setYear(date.getUTCFullYear()).setMonth(date.getUTCMonth()).setSelectedDate(date).setHours(date.getUTCHours()).setMinutes(date.getUTCMinutes());
                            }
                            var dateTimePicker = new api.ui.time.DateTimePicker(dateTimeBuilder);
                            dateTimePicker.onSelectedDateTimeChanged(function (event) {
                                var newValue = new Value(event.getDate(), ValueTypes.LOCAL_DATE_TIME);
                                property.setValue(newValue);
                            });
                            return dateTimePicker;
                        };
                        DateTime.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var dateTimePicker = inputElement;
                            return dateTimePicker.hasValidUserInput();
                        };
                        DateTime.prototype.availableSizeChanged = function () {
                        };
                        DateTime.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LOCAL_DATE_TIME);
                        };
                        return DateTime;
                    })(support.BaseInputTypeNotManagingAdd);
                    time.DateTime = DateTime;
                    api.form.inputtype.InputTypeManager.register(new api.Class("DateTime", DateTime));
                })(time = inputtype.time || (inputtype.time = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var item;
    (function (item) {
        var BaseItem = (function () {
            function BaseItem(builder) {
                this.id = builder.id;
                this.createdTime = builder.createdTime;
                this.modifiedTime = builder.modifiedTime;
                this.deletable = builder.deletable;
                this.editable = builder.editable;
            }
            BaseItem.prototype.getId = function () {
                return this.id;
            };
            BaseItem.prototype.getCreatedTime = function () {
                return this.createdTime;
            };
            BaseItem.prototype.getModifiedTime = function () {
                return this.modifiedTime;
            };
            BaseItem.prototype.isDeletable = function () {
                return this.deletable;
            };
            BaseItem.prototype.isEditable = function () {
                return this.editable;
            };
            BaseItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, BaseItem)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id, other.id)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.createdTime, other.createdTime)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.modifiedTime, other.modifiedTime)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.deletable, other.deletable)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.editable, other.editable)) {
                    return false;
                }
                return true;
            };
            return BaseItem;
        })();
        item.BaseItem = BaseItem;
        var BaseItemBuilder = (function () {
            function BaseItemBuilder(source) {
                if (source) {
                    this.id = source.getId();
                    this.createdTime = source.getCreatedTime();
                    this.modifiedTime = source.getModifiedTime();
                    this.deletable = source.isDeletable();
                    this.editable = source.isEditable();
                }
            }
            BaseItemBuilder.prototype.fromBaseItemJson = function (json, idProperty) {
                if (idProperty === void 0) { idProperty = 'id'; }
                this.id = json[idProperty];
                this.createdTime = json.createdTime ? new Date(json.createdTime) : null;
                this.modifiedTime = json.modifiedTime ? new Date(json.modifiedTime) : null;
                this.deletable = json.deletable;
                this.editable = json.editable;
                return this;
            };
            BaseItemBuilder.prototype.build = function () {
                return new BaseItem(this);
            };
            return BaseItemBuilder;
        })();
        item.BaseItemBuilder = BaseItemBuilder;
    })(item = api.item || (api.item = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var CompareExpr = (function () {
                function CompareExpr(field, operator, values) {
                    this.values = [];
                    this.field = field;
                    this.operator = operator;
                    this.values = values;
                }
                CompareExpr.prototype.getField = function () {
                    return this.field;
                };
                CompareExpr.prototype.getOperator = function () {
                    return this.operator;
                };
                CompareExpr.prototype.getFirstValue = function () {
                    return this.values.length > 0 ? this.values[0] : null;
                };
                CompareExpr.prototype.getValues = function () {
                    return this.values;
                };
                CompareExpr.prototype.toString = function () {
                    var result = this.field.toString();
                    result = result.concat(" ");
                    result = result.concat(this.operatorAsString());
                    result = result.concat(" ");
                    if (this.allowMultipleValues()) {
                        result = result.concat("(");
                        var sub = [];
                        this.values.forEach(function (expr) {
                            sub.push(expr.toString());
                        });
                        result = result.concat(sub.join(", "));
                        result = result.concat(")");
                    }
                    else {
                        result = result.concat(this.getFirstValue().toString());
                    }
                    return result;
                };
                CompareExpr.eq = function (field, value) {
                    return CompareExpr.create(field, 0 /* EQ */, [value]);
                };
                CompareExpr.neq = function (field, value) {
                    return CompareExpr.create(field, 1 /* NEQ */, [value]);
                };
                CompareExpr.gt = function (field, value) {
                    return CompareExpr.create(field, 2 /* GT */, [value]);
                };
                CompareExpr.gte = function (field, value) {
                    return CompareExpr.create(field, 3 /* GTE */, [value]);
                };
                CompareExpr.lt = function (field, value) {
                    return CompareExpr.create(field, 4 /* LT */, [value]);
                };
                CompareExpr.lte = function (field, value) {
                    return CompareExpr.create(field, 5 /* LTE */, [value]);
                };
                CompareExpr.like = function (field, value) {
                    return CompareExpr.create(field, 6 /* LIKE */, [value]);
                };
                CompareExpr.notLike = function (field, value) {
                    return CompareExpr.create(field, 7 /* NOT_LIKE */, [value]);
                };
                CompareExpr.In = function (field, values) {
                    return CompareExpr.create(field, 8 /* IN */, values);
                };
                CompareExpr.notIn = function (field, values) {
                    return CompareExpr.create(field, 9 /* NOT_IN */, values);
                };
                CompareExpr.create = function (field, operator, values) {
                    return new CompareExpr(field, operator, values);
                };
                CompareExpr.prototype.operatorAsString = function () {
                    switch (this.operator) {
                        case 0 /* EQ */:
                            return "=";
                        case 1 /* NEQ */:
                            return "!=";
                        case 2 /* GT */:
                            return ">";
                        case 3 /* GTE */:
                            return ">=";
                        case 4 /* LT */:
                            return "<";
                        case 5 /* LTE */:
                            return "<=";
                        case 6 /* LIKE */:
                            return "LIKE";
                        case 7 /* NOT_LIKE */:
                            return "NOT LIKE";
                        case 8 /* IN */:
                            return "IN";
                        case 9 /* NOT_IN */:
                            return "NOT IN";
                        default:
                            return "";
                    }
                };
                CompareExpr.prototype.allowMultipleValues = function () {
                    return this.operator == 8 /* IN */ || this.operator == 9 /* NOT_IN */;
                };
                return CompareExpr;
            })();
            _expr.CompareExpr = CompareExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var OrderExpr = (function () {
                function OrderExpr(direction) {
                    this.direction = direction;
                }
                OrderExpr.prototype.getDirection = function () {
                    return this.direction;
                };
                OrderExpr.prototype.directionAsString = function () {
                    switch (this.direction) {
                        case 0 /* ASC */:
                            return "ASC";
                        case 1 /* DESC */:
                            return "DESC";
                        default:
                            return "";
                    }
                };
                return OrderExpr;
            })();
            expr.OrderExpr = OrderExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var QueryExpr = (function () {
                function QueryExpr(constraint, orderList) {
                    this.orderList = [];
                    this.constraint = constraint;
                    if (orderList) {
                        this.orderList = orderList;
                    }
                }
                QueryExpr.prototype.getConstraint = function () {
                    return this.constraint;
                };
                QueryExpr.prototype.getOrderList = function () {
                    return this.orderList;
                };
                QueryExpr.prototype.toString = function () {
                    var result = "";
                    if (this.constraint != null) {
                        result = result.concat(this.constraint.toString());
                    }
                    if (this.orderList.length > 0) {
                        result = result.concat(" ORDER BY ");
                        var sub = [];
                        this.orderList.forEach(function (expr) {
                            sub.push(expr.toString());
                        });
                        result = result.concat(sub.join(", "));
                    }
                    return result;
                };
                return QueryExpr;
            })();
            _expr.QueryExpr = QueryExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var ValueExpr = (function () {
                function ValueExpr(value) {
                    this.value = value;
                }
                ValueExpr.prototype.getValue = function () {
                    return this.value;
                };
                ValueExpr.stringValue = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.STRING));
                };
                ValueExpr.prototype.toString = function () {
                    var type = this.value.getType();
                    if (type == api.data.ValueTypes.DOUBLE) {
                        return this.value.getString();
                    }
                    if (type == api.data.ValueTypes.DATE_TIME) {
                        return this.typecastFunction("dateTime", this.value.getString());
                    }
                    if (type == api.data.ValueTypes.GEO_POINT) {
                        return this.typecastFunction("geoPoint", this.value.getString());
                    }
                    return this.quoteString(this.value.getString());
                };
                ValueExpr.prototype.typecastFunction = function (name, argument) {
                    return name + "(" + this.quoteString(argument) + ")";
                };
                ValueExpr.prototype.quoteString = function (value) {
                    if (value.indexOf("'") > -1) {
                        return "\"" + value + "\"";
                    }
                    else {
                        return "'" + value + "'";
                    }
                };
                ValueExpr.string = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.STRING));
                };
                ValueExpr.number = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.DOUBLE));
                };
                ValueExpr.dateTime = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.DATE_TIME));
                };
                ValueExpr.geoPoint = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.GEO_POINT));
                };
                return ValueExpr;
            })();
            expr.ValueExpr = ValueExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var FunctionExpr = (function () {
                function FunctionExpr(name, arguments) {
                    this.arguments = [];
                    this.name = name;
                    this.arguments = arguments;
                }
                FunctionExpr.prototype.getName = function () {
                    return this.name;
                };
                FunctionExpr.prototype.getArguments = function () {
                    return this.arguments;
                };
                FunctionExpr.prototype.toString = function () {
                    var result = this.name;
                    result = result.concat("(");
                    var sub = [];
                    this.arguments.forEach(function (expr) {
                        sub.push(expr.toString());
                    });
                    result = result.concat(sub.join(", "));
                    result = result.concat(")");
                    return result;
                };
                return FunctionExpr;
            })();
            _expr.FunctionExpr = FunctionExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var FieldExpr = (function () {
                function FieldExpr(name) {
                    this.name = name;
                }
                FieldExpr.prototype.getName = function () {
                    return this.name;
                };
                FieldExpr.prototype.toString = function () {
                    return this.name;
                };
                return FieldExpr;
            })();
            expr.FieldExpr = FieldExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var DynamicConstraintExpr = (function () {
                function DynamicConstraintExpr(func) {
                    this.func = func;
                }
                DynamicConstraintExpr.prototype.getFunction = function () {
                    return this.func;
                };
                DynamicConstraintExpr.prototype.toString = function () {
                    return this.func.toString();
                };
                return DynamicConstraintExpr;
            })();
            expr.DynamicConstraintExpr = DynamicConstraintExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var DynamicOrderExpr = (function (_super) {
                __extends(DynamicOrderExpr, _super);
                function DynamicOrderExpr(func, direction) {
                    _super.call(this, direction);
                    this.func = func;
                }
                DynamicOrderExpr.prototype.getFunction = function () {
                    return this.func;
                };
                DynamicOrderExpr.prototype.toString = function () {
                    return this.func.toString() + " " + _super.prototype.directionAsString.call(this);
                };
                return DynamicOrderExpr;
            })(expr.OrderExpr);
            expr.DynamicOrderExpr = DynamicOrderExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var FieldOrderExpr = (function (_super) {
                __extends(FieldOrderExpr, _super);
                function FieldOrderExpr(field, direction) {
                    _super.call(this, direction);
                    this.field = field;
                }
                FieldOrderExpr.prototype.getField = function () {
                    return this.field;
                };
                FieldOrderExpr.prototype.toString = function () {
                    return this.field.toString() + " " + _super.prototype.directionAsString.call(this);
                };
                return FieldOrderExpr;
            })(expr.OrderExpr);
            expr.FieldOrderExpr = FieldOrderExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var LogicalExpr = (function () {
                function LogicalExpr(left, operator, right) {
                    this.left = left;
                    this.right = right;
                    this.operator = operator;
                }
                LogicalExpr.prototype.getLeft = function () {
                    return this.left;
                };
                LogicalExpr.prototype.getRight = function () {
                    return this.right;
                };
                LogicalExpr.prototype.getOperator = function () {
                    return this.operator;
                };
                LogicalExpr.prototype.toString = function () {
                    return "(" + this.left.toString() + " " + this.operatorAsString() + " " + this.right.toString() + ")";
                };
                LogicalExpr.and = function (left, right) {
                    return new LogicalExpr(left, 0 /* AND */, right);
                };
                LogicalExpr.or = function (left, right) {
                    return new LogicalExpr(left, 1 /* OR */, right);
                };
                LogicalExpr.prototype.operatorAsString = function () {
                    switch (this.operator) {
                        case 0 /* AND */:
                            return "AND";
                        case 1 /* OR */:
                            return "OR";
                        default:
                            return "";
                    }
                };
                return LogicalExpr;
            })();
            expr.LogicalExpr = LogicalExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var NotExpr = (function () {
                function NotExpr(expr) {
                    this.expr = expr;
                }
                NotExpr.prototype.getExpression = function () {
                    return this.expr;
                };
                NotExpr.prototype.toString = function () {
                    return "NOT (" + this.expr.toString() + ")";
                };
                return NotExpr;
            })();
            _expr.NotExpr = NotExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            (function (CompareOperator) {
                CompareOperator[CompareOperator["EQ"] = 0] = "EQ";
                CompareOperator[CompareOperator["NEQ"] = 1] = "NEQ";
                CompareOperator[CompareOperator["GT"] = 2] = "GT";
                CompareOperator[CompareOperator["GTE"] = 3] = "GTE";
                CompareOperator[CompareOperator["LT"] = 4] = "LT";
                CompareOperator[CompareOperator["LTE"] = 5] = "LTE";
                CompareOperator[CompareOperator["LIKE"] = 6] = "LIKE";
                CompareOperator[CompareOperator["NOT_LIKE"] = 7] = "NOT_LIKE";
                CompareOperator[CompareOperator["IN"] = 8] = "IN";
                CompareOperator[CompareOperator["NOT_IN"] = 9] = "NOT_IN";
            })(expr.CompareOperator || (expr.CompareOperator = {}));
            var CompareOperator = expr.CompareOperator;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            (function (LogicalOperator) {
                LogicalOperator[LogicalOperator["AND"] = 0] = "AND";
                LogicalOperator[LogicalOperator["OR"] = 1] = "OR";
            })(expr.LogicalOperator || (expr.LogicalOperator = {}));
            var LogicalOperator = expr.LogicalOperator;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            (function (OrderDirection) {
                OrderDirection[OrderDirection["ASC"] = 0] = "ASC";
                OrderDirection[OrderDirection["DESC"] = 1] = "DESC";
            })(expr.OrderDirection || (expr.OrderDirection = {}));
            var OrderDirection = expr.OrderDirection;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var SearchInputValues = (function () {
            function SearchInputValues() {
            }
            SearchInputValues.prototype.setAggregationSelections = function (aggregationSelections) {
                this.aggregationSelections = aggregationSelections;
            };
            SearchInputValues.prototype.setTextSearchFieldValue = function (textSearchFieldValue) {
                this.textSearchFieldValue = textSearchFieldValue;
            };
            SearchInputValues.prototype.getTextSearchFieldValue = function () {
                return this.textSearchFieldValue;
            };
            SearchInputValues.prototype.getSelectedValuesForAggregationName = function (name) {
                for (var i = 0; i < this.aggregationSelections.length; i++) {
                    if (this.aggregationSelections[i].getName() == name) {
                        return this.aggregationSelections[i].getSelectedBuckets();
                    }
                }
                return null;
            };
            return SearchInputValues;
        })();
        query.SearchInputValues = SearchInputValues;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var ValueExpr = api.query.expr.ValueExpr;
        var FunctionExpr = api.query.expr.FunctionExpr;
        var DynamicConstraintExpr = api.query.expr.DynamicConstraintExpr;
        var LogicalExpr = api.query.expr.LogicalExpr;
        var LogicalOperator = api.query.expr.LogicalOperator;
        var FulltextSearchExpression = (function () {
            function FulltextSearchExpression() {
            }
            FulltextSearchExpression.create = function (searchString, queryFields) {
                if (searchString == null) {
                    return null;
                }
                var arguments = [];
                arguments.push(ValueExpr.stringValue(queryFields.toString()));
                arguments.push(ValueExpr.stringValue(searchString));
                arguments.push(ValueExpr.stringValue("AND"));
                var fulltextExp = new FunctionExpr("fulltext", arguments);
                var fulltextDynamicExpr = new DynamicConstraintExpr(fulltextExp);
                var nGramExpr = new FunctionExpr("ngram", arguments);
                var nGramDynamicExpr = new DynamicConstraintExpr(nGramExpr);
                var booleanExpr = new LogicalExpr(fulltextDynamicExpr, 1 /* OR */, nGramDynamicExpr);
                return booleanExpr;
            };
            return FulltextSearchExpression;
        })();
        query.FulltextSearchExpression = FulltextSearchExpression;
        var FulltextSearchExpressionBuilder = (function () {
            function FulltextSearchExpressionBuilder() {
                this.queryFields = new query.QueryFields();
            }
            FulltextSearchExpressionBuilder.prototype.addField = function (queryField) {
                this.queryFields.add(queryField);
                return this;
            };
            FulltextSearchExpressionBuilder.prototype.setSearchString = function (searchString) {
                this.searchString = searchString;
                return this;
            };
            FulltextSearchExpressionBuilder.prototype.build = function () {
                return FulltextSearchExpression.create(this.searchString, this.queryFields);
            };
            return FulltextSearchExpressionBuilder;
        })();
        query.FulltextSearchExpressionBuilder = FulltextSearchExpressionBuilder;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var QueryField = (function () {
            function QueryField(name, weight) {
                this.name = name;
                if (weight) {
                    this.weight = weight;
                }
            }
            QueryField.prototype.toString = function () {
                if (this.weight) {
                    return this.name + QueryField.WEIGHT_SEPARATOR + this.weight;
                }
                else {
                    return this.name;
                }
            };
            QueryField.DISPLAY_NAME = "displayName";
            QueryField.NAME = "_name";
            QueryField.ALL = "_alltext";
            QueryField.MODIFIED_TIME = "modifiedTime";
            QueryField.MANUAL_ORDER_VALUE = "_manualOrderValue";
            QueryField.WEIGHT_SEPARATOR = "^";
            QueryField.CONTENT_TYPE = "type";
            return QueryField;
        })();
        query.QueryField = QueryField;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var QueryFields = (function () {
            function QueryFields() {
                this.queryFields = [];
            }
            QueryFields.prototype.add = function (queryField) {
                this.queryFields.push(queryField);
            };
            QueryFields.prototype.toString = function () {
                if (this.queryFields) {
                    return this.queryFields.join();
                }
            };
            return QueryFields;
        })();
        query.QueryFields = QueryFields;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var AggregationQuery = (function () {
                function AggregationQuery(name) {
                    this.name = name;
                }
                AggregationQuery.prototype.toJson = function () {
                    throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                };
                AggregationQuery.prototype.toAggregationQueryJson = function () {
                    return {
                        "name": this.getName()
                    };
                };
                AggregationQuery.prototype.getName = function () {
                    return this.name;
                };
                return AggregationQuery;
            })();
            aggregation.AggregationQuery = AggregationQuery;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var TermsAggregationQuery = (function (_super) {
                __extends(TermsAggregationQuery, _super);
                function TermsAggregationQuery(name) {
                    _super.call(this, name);
                    this.size = TermsAggregationQuery.TERM_DEFAULT_SIZE;
                    this.orderByDirection = TermsAggregationOrderDirection.ASC;
                    this.orderByType = TermsAggregationOrderType.TERM;
                }
                TermsAggregationQuery.prototype.toJson = function () {
                    var json = _super.prototype.toAggregationQueryJson.call(this);
                    json.fieldName = this.getFieldName();
                    json.size = this.getSize();
                    json.orderByDirection = this.orderByDirection;
                    json.orderByType = this.orderByType;
                    return {
                        TermsAggregationQuery: json
                    };
                };
                TermsAggregationQuery.prototype.setFieldName = function (fieldName) {
                    this.fieldName = fieldName;
                };
                TermsAggregationQuery.prototype.getFieldName = function () {
                    return this.fieldName;
                };
                TermsAggregationQuery.prototype.setSize = function (size) {
                    this.size = size;
                };
                TermsAggregationQuery.prototype.getSize = function () {
                    return this.size;
                };
                TermsAggregationQuery.TERM_DEFAULT_SIZE = 10;
                return TermsAggregationQuery;
            })(aggregation.AggregationQuery);
            aggregation.TermsAggregationQuery = TermsAggregationQuery;
            var TermsAggregationOrderDirection = (function () {
                function TermsAggregationOrderDirection() {
                }
                TermsAggregationOrderDirection.ASC = "ASC";
                TermsAggregationOrderDirection.DESC = "DESC";
                return TermsAggregationOrderDirection;
            })();
            aggregation.TermsAggregationOrderDirection = TermsAggregationOrderDirection;
            var TermsAggregationOrderType = (function () {
                function TermsAggregationOrderType() {
                }
                TermsAggregationOrderType.DOC_COUNT = "DOC_COUNT";
                TermsAggregationOrderType.TERM = "TERM";
                return TermsAggregationOrderType;
            })();
            aggregation.TermsAggregationOrderType = TermsAggregationOrderType;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var DateRangeAggregationQuery = (function (_super) {
                __extends(DateRangeAggregationQuery, _super);
                function DateRangeAggregationQuery(name) {
                    _super.call(this, name);
                    this.ranges = [];
                }
                DateRangeAggregationQuery.prototype.setFieldName = function (fieldName) {
                    this.fieldName = fieldName;
                };
                DateRangeAggregationQuery.prototype.getFieldName = function () {
                    return this.fieldName;
                };
                DateRangeAggregationQuery.prototype.addRange = function (range) {
                    this.ranges.push(range);
                };
                DateRangeAggregationQuery.prototype.toJson = function () {
                    var json = _super.prototype.toAggregationQueryJson.call(this);
                    json.fieldName = this.getFieldName();
                    json.ranges = [];
                    this.ranges.forEach(function (range) {
                        json.ranges.push(range.toJson());
                    });
                    return {
                        DateRangeAggregationQuery: json
                    };
                };
                return DateRangeAggregationQuery;
            })(api.query.aggregation.AggregationQuery);
            aggregation.DateRangeAggregationQuery = DateRangeAggregationQuery;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var Range = (function () {
                function Range(key) {
                    this.key = key;
                }
                Range.prototype.setKey = function (key) {
                    this.key = key;
                };
                Range.prototype.getKey = function () {
                    return this.key;
                };
                Range.prototype.toRangeJson = function () {
                    if (this.getKey() != null) {
                        return {
                            "key": this.getKey()
                        };
                    }
                    else {
                        return {};
                    }
                };
                return Range;
            })();
            aggregation.Range = Range;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var DateRange = (function (_super) {
                __extends(DateRange, _super);
                function DateRange(from, to, key) {
                    _super.call(this, key);
                    this.from = from;
                    this.to = to;
                }
                DateRange.prototype.setTo = function (to) {
                    this.to = to;
                };
                DateRange.prototype.setFrom = function (from) {
                    this.from = from;
                };
                DateRange.prototype.setToDate = function (to) {
                    this.to = to.toISOString();
                };
                DateRange.prototype.setFromDate = function (from) {
                    this.from = from.toISOString();
                };
                DateRange.prototype.toJson = function () {
                    var json = _super.prototype.toRangeJson.call(this);
                    json.from = this.from;
                    json.to = this.to;
                    return json;
                };
                return DateRange;
            })(aggregation.Range);
            aggregation.DateRange = DateRange;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var filter;
        (function (filter) {
            var Filter = (function () {
                function Filter() {
                }
                Filter.prototype.toJson = function () {
                    throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                };
                return Filter;
            })();
            filter.Filter = Filter;
        })(filter = query.filter || (query.filter = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var filter;
        (function (_filter) {
            var BooleanFilter = (function (_super) {
                __extends(BooleanFilter, _super);
                function BooleanFilter() {
                    _super.apply(this, arguments);
                    this.must = [];
                    this.mustNot = [];
                    this.should = [];
                }
                BooleanFilter.prototype.addMust = function (must) {
                    this.must.push(must);
                };
                BooleanFilter.prototype.addMustNot = function (mustNot) {
                    this.mustNot.push(mustNot);
                };
                BooleanFilter.prototype.addShould = function (should) {
                    this.should.push(should);
                };
                BooleanFilter.prototype.toJson = function () {
                    var json = {
                        must: this.toJsonWrapperElements(this.must),
                        mustNot: this.toJsonWrapperElements(this.mustNot),
                        should: this.toJsonWrapperElements(this.should)
                    };
                    return {
                        BooleanFilter: json
                    };
                };
                BooleanFilter.prototype.toJsonWrapperElements = function (filters) {
                    var wrapperJsons = [];
                    filters.forEach(function (filter) {
                        var filterTypeWrapperJson = filter.toJson();
                        wrapperJsons.push(filterTypeWrapperJson);
                    });
                    return wrapperJsons;
                };
                return BooleanFilter;
            })(_filter.Filter);
            _filter.BooleanFilter = BooleanFilter;
        })(filter = query.filter || (query.filter = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var filter;
        (function (filter) {
            var RangeFilter = (function (_super) {
                __extends(RangeFilter, _super);
                function RangeFilter(fieldName, from, to) {
                    _super.call(this);
                    this.fieldName = fieldName;
                    this.from = from;
                    this.to = to;
                }
                RangeFilter.prototype.toJson = function () {
                    var json = {
                        fieldName: this.fieldName,
                        from: this.from != null ? this.from.getString() : null,
                        to: this.to != null ? this.to.getString() : null
                    };
                    return {
                        RangeFilter: json
                    };
                };
                return RangeFilter;
            })(api.query.filter.Filter);
            filter.RangeFilter = RangeFilter;
        })(filter = query.filter || (query.filter = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ResourcePath = (function (_super) {
            __extends(ResourcePath, _super);
            function ResourcePath(elements, absolute) {
                _super.call(this, elements, ResourcePath.ELEMENT_DIVIDER, absolute);
            }
            ResourcePath.fromString = function (s) {
                var absolute = s.charAt(0) == ResourcePath.ELEMENT_DIVIDER;
                var elements = s.split(ResourcePath.ELEMENT_DIVIDER);
                return new ResourcePath(api.util.BasePath.removeEmptyElements(elements), absolute);
            };
            ResourcePath.prototype.newInstance = function (elements, absolute) {
                return new ResourcePath(elements, absolute);
            };
            ResourcePath.ELEMENT_DIVIDER = "/";
            return ResourcePath;
        })(api.util.BasePath);
        module.ResourcePath = ResourcePath;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleKey = (function () {
            function ModuleKey(moduleName) {
                this.name = moduleName;
            }
            ModuleKey.fromString = function (moduleName) {
                return new ModuleKey(moduleName);
            };
            ModuleKey.prototype.getName = function () {
                return this.name;
            };
            ModuleKey.prototype.toString = function () {
                return this.name;
            };
            ModuleKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ModuleKey)) {
                    return false;
                }
                var other = o;
                return api.ObjectHelper.stringEquals(this.name, other.name);
            };
            ModuleKey.toStringArray = function (keys) {
                return keys.map(function (key) { return key.toString(); });
            };
            ModuleKey.fromModules = function (modules) {
                return modules.map(function (mod) { return mod.getModuleKey(); });
            };
            ModuleKey.BASE = ModuleKey.fromString('base');
            ModuleKey.PORTAL = ModuleKey.fromString('portal');
            ModuleKey.MEDIA = ModuleKey.fromString('media');
            return ModuleKey;
        })();
        module.ModuleKey = ModuleKey;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleResourceKey = (function () {
            function ModuleResourceKey(moduleKey, path) {
                this.moduleKey = moduleKey;
                this.path = path;
                this.refString = moduleKey.toString() + ModuleResourceKey.SEPARATOR + path.toString();
            }
            ModuleResourceKey.fromString = function (str) {
                var sepIndex = str.indexOf(ModuleResourceKey.SEPARATOR);
                if (sepIndex == -1) {
                    throw new Error("ModuleResourceKey must contain separator '" + ModuleResourceKey.SEPARATOR + "':" + str);
                }
                var moduleKey = str.substring(0, sepIndex);
                var path = str.substring(sepIndex + 1, str.length);
                return new ModuleResourceKey(module.ModuleKey.fromString(moduleKey), module.ResourcePath.fromString(path));
            };
            ModuleResourceKey.prototype.getModuleKey = function () {
                return this.moduleKey;
            };
            ModuleResourceKey.prototype.getPath = function () {
                return this.path;
            };
            ModuleResourceKey.prototype.toString = function () {
                return this.refString;
            };
            ModuleResourceKey.SEPARATOR = ":";
            return ModuleResourceKey;
        })();
        module.ModuleResourceKey = ModuleResourceKey;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var Module = (function (_super) {
            __extends(Module, _super);
            function Module(builder) {
                _super.call(this, builder);
                this.moduleDependencies = [];
                this.contentTypeDependencies = [];
                this.moduleKey = builder.moduleKey;
                this.displayName = builder.displayName;
                this.vendorName = builder.vendorName;
                this.vendorUrl = builder.vendorUrl;
                this.url = builder.url;
                this.state = builder.state;
                this.version = builder.version;
                this.config = builder.config;
                this.moduleDependencies = builder.moduleDependencies;
                this.contentTypeDependencies = builder.contentTypeDependencies;
                this.metaSteps = builder.metaSteps;
                this.minSystemVersion = builder.minSystemVersion;
                this.maxSystemVersion = builder.maxSystemVersion;
            }
            Module.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Module.prototype.getModuleKey = function () {
                return this.moduleKey;
            };
            Module.prototype.getVersion = function () {
                return this.version;
            };
            Module.prototype.getName = function () {
                return this.moduleKey.getName();
            };
            Module.prototype.getVendorName = function () {
                return this.vendorName;
            };
            Module.prototype.getVendorUrl = function () {
                return this.vendorUrl;
            };
            Module.prototype.getUrl = function () {
                return this.url;
            };
            Module.prototype.getState = function () {
                return this.state;
            };
            Module.prototype.hasChildren = function () {
                return false;
            };
            Module.prototype.getForm = function () {
                return this.config;
            };
            Module.prototype.getMinSystemVersion = function () {
                return this.minSystemVersion;
            };
            Module.prototype.getMaxSystemVersion = function () {
                return this.maxSystemVersion;
            };
            Module.prototype.getModuleDependencies = function () {
                return this.moduleDependencies;
            };
            Module.prototype.getContentTypeDependencies = function () {
                return this.contentTypeDependencies;
            };
            Module.prototype.getMetaSteps = function () {
                return this.metaSteps;
            };
            Module.fromJson = function (json) {
                return new ModuleBuilder().fromJson(json).build();
            };
            Module.fromJsonArray = function (jsonArray) {
                var array = [];
                jsonArray.forEach(function (json) {
                    array.push(Module.fromJson(json));
                });
                return array;
            };
            Module.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Module) || !_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                return this.moduleKey.equals(other.moduleKey) && this.displayName == other.displayName && this.vendorName == other.vendorName && this.vendorUrl == other.vendorUrl && this.url == other.url && this.state == other.state && this.version == other.version && api.ObjectHelper.arrayEquals(this.moduleDependencies, other.moduleDependencies) && api.ObjectHelper.arrayEquals(this.contentTypeDependencies, other.contentTypeDependencies) && api.ObjectHelper.equals(this.metaSteps, other.metaSteps) && this.minSystemVersion == other.minSystemVersion && this.maxSystemVersion == other.maxSystemVersion;
            };
            return Module;
        })(api.item.BaseItem);
        module.Module = Module;
        var ModuleBuilder = (function (_super) {
            __extends(ModuleBuilder, _super);
            function ModuleBuilder(source) {
                this.moduleDependencies = [];
                this.contentTypeDependencies = [];
                this.metaSteps;
                if (source) {
                    _super.call(this, source);
                    this.moduleKey = source.getModuleKey();
                    this.displayName = source.getDisplayName();
                    this.vendorName = source.getVendorName();
                    this.vendorUrl = source.getVendorUrl();
                    this.url = source.getUrl();
                    this.state = source.getState();
                    this.version = source.getVersion();
                    this.config = source.getForm();
                    this.moduleDependencies = source.getModuleDependencies();
                    this.contentTypeDependencies = source.getContentTypeDependencies();
                    this.metaSteps = source.getMetaSteps();
                    this.minSystemVersion = source.getMinSystemVersion();
                    this.maxSystemVersion = source.getMaxSystemVersion();
                }
            }
            ModuleBuilder.prototype.fromJson = function (json) {
                var _this = this;
                _super.prototype.fromBaseItemJson.call(this, json, 'key');
                this.moduleKey = module.ModuleKey.fromString(json.key);
                this.displayName = json.displayName;
                this.vendorName = json.vendorName;
                this.vendorUrl = json.vendorUrl;
                this.url = json.url;
                this.state = json.state;
                this.version = json.version;
                this.config = json.config != null ? api.form.Form.fromJson(json.config) : null;
                this.minSystemVersion = json.minSystemVersion;
                this.maxSystemVersion = json.maxSystemVersion;
                if (json.moduleDependencies != null) {
                    json.moduleDependencies.forEach(function (dependency) {
                        _this.moduleDependencies.push(api.module.ModuleKey.fromString(dependency));
                    });
                }
                if (json.contentTypeDependencies != null) {
                    json.contentTypeDependencies.forEach(function (dependency) {
                        _this.contentTypeDependencies.push(new api.schema.content.ContentTypeName(dependency));
                    });
                }
                if (json.metaSteps != null) {
                    this.metaSteps = api.schema.mixin.MixinNames.create().fromStrings(json.metaSteps).build();
                }
                return this;
            };
            ModuleBuilder.prototype.build = function () {
                return new Module(this);
            };
            return ModuleBuilder;
        })(api.item.BaseItemBuilder);
        module.ModuleBuilder = ModuleBuilder;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleResourceRequest = (function (_super) {
            __extends(ModuleResourceRequest, _super);
            function ModuleResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "module");
            }
            ModuleResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            ModuleResourceRequest.prototype.fromJsonToModule = function (json) {
                return module.Module.fromJson(json);
            };
            return ModuleResourceRequest;
        })(api.rest.ResourceRequest);
        module.ModuleResourceRequest = ModuleResourceRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var GetModuleRequest = (function (_super) {
            __extends(GetModuleRequest, _super);
            function GetModuleRequest(moduleKey, skipCache) {
                if (skipCache === void 0) { skipCache = false; }
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.moduleKey = moduleKey;
                this.skipCache = skipCache;
            }
            GetModuleRequest.prototype.getParams = function () {
                return {
                    moduleKey: this.moduleKey.toString()
                };
            };
            GetModuleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this));
            };
            GetModuleRequest.prototype.sendAndParse = function () {
                var _this = this;
                var cache = module.ModuleCache.get();
                var moduleObj = this.skipCache ? null : cache.getByKey(this.moduleKey);
                if (moduleObj) {
                    return wemQ(moduleObj);
                }
                else {
                    return this.send().then(function (response) {
                        moduleObj = _this.fromJsonToModule(response.getResult());
                        cache.put(moduleObj);
                        return moduleObj;
                    });
                }
            };
            return GetModuleRequest;
        })(module.ModuleResourceRequest);
        module.GetModuleRequest = GetModuleRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ListModulesRequest = (function (_super) {
            __extends(ListModulesRequest, _super);
            function ListModulesRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            ListModulesRequest.prototype.getParams = function () {
                return {};
            };
            ListModulesRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list");
            };
            ListModulesRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return module.Module.fromJsonArray(response.getResult().modules);
                });
            };
            return ListModulesRequest;
        })(module.ModuleResourceRequest);
        module.ListModulesRequest = ListModulesRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleListResult = (function () {
            function ModuleListResult() {
            }
            return ModuleListResult;
        })();
        module.ModuleListResult = ModuleListResult;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleLoader = (function (_super) {
            __extends(ModuleLoader, _super);
            function ModuleLoader(delay) {
                if (delay === void 0) { delay = 500; }
                _super.call(this, new module.ListModulesRequest());
            }
            ModuleLoader.prototype.search = function (searchString) {
                return this.load();
            };
            ModuleLoader.prototype.load = function () {
                var _this = this;
                this.notifyLoadingData();
                return this.sendRequest().then(function (modules) {
                    _this.notifyLoadedData(modules);
                    if (_this.preservedSearchString) {
                        _this.search(_this.preservedSearchString);
                        _this.preservedSearchString = null;
                    }
                    return modules;
                });
            };
            return ModuleLoader;
        })(api.util.loader.BaseLoader);
        module.ModuleLoader = ModuleLoader;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleLoaderListener = (function () {
            function ModuleLoaderListener() {
            }
            return ModuleLoaderListener;
        })();
        module.ModuleLoaderListener = ModuleLoaderListener;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleViewer = (function (_super) {
            __extends(ModuleViewer, _super);
            function ModuleViewer() {
                _super.call(this);
            }
            ModuleViewer.prototype.resolveDisplayName = function (object) {
                return object.getDisplayName();
            };
            ModuleViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return object.getName();
            };
            ModuleViewer.prototype.resolveIconClass = function (object) {
                return "icon-puzzle icon-large";
            };
            return ModuleViewer;
        })(api.ui.NamesAndIconViewer);
        module.ModuleViewer = ModuleViewer;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleComboBox = (function (_super) {
            __extends(ModuleComboBox, _super);
            function ModuleComboBox(maximumOccurrences) {
                var builder = new api.ui.selector.combobox.RichComboBoxBuilder();
                builder.setMaximumOccurrences(maximumOccurrences || 0).setComboBoxName("moduleSelector").setLoader(new api.module.ModuleLoader()).setSelectedOptionsView(new ModuleSelectedOptionsView()).setOptionDisplayValueViewer(new module.ModuleViewer()).setDelayedInputValueChangedHandling(500);
                _super.call(this, builder);
            }
            return ModuleComboBox;
        })(api.ui.selector.combobox.RichComboBox);
        module.ModuleComboBox = ModuleComboBox;
        var ModuleSelectedOptionsView = (function (_super) {
            __extends(ModuleSelectedOptionsView, _super);
            function ModuleSelectedOptionsView() {
                _super.apply(this, arguments);
            }
            ModuleSelectedOptionsView.prototype.createSelectedOption = function (option) {
                var optionView = new ModuleSelectedOptionView(option);
                return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
            };
            return ModuleSelectedOptionsView;
        })(api.ui.selector.combobox.BaseSelectedOptionsView);
        module.ModuleSelectedOptionsView = ModuleSelectedOptionsView;
        var ModuleSelectedOptionView = (function (_super) {
            __extends(ModuleSelectedOptionView, _super);
            function ModuleSelectedOptionView(option) {
                _super.call(this, option);
            }
            ModuleSelectedOptionView.prototype.resolveIconUrl = function (content) {
                return api.util.UriHelper.getAdminUri("common/images/icons/icoMoon/128x128/puzzle.png");
            };
            ModuleSelectedOptionView.prototype.resolveTitle = function (content) {
                return content.getDisplayName().toString();
            };
            ModuleSelectedOptionView.prototype.resolveSubTitle = function (content) {
                return content.getModuleKey().toString();
            };
            return ModuleSelectedOptionView;
        })(api.ui.selector.combobox.RichSelectedOptionView);
        module.ModuleSelectedOptionView = ModuleSelectedOptionView;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var StopModuleRequest = (function (_super) {
            __extends(StopModuleRequest, _super);
            function StopModuleRequest(moduleKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.moduleKeys = moduleKeys;
            }
            StopModuleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "stop");
            };
            StopModuleRequest.prototype.getParams = function () {
                return {
                    key: module.ModuleKey.toStringArray(this.moduleKeys)
                };
            };
            StopModuleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return StopModuleRequest;
        })(module.ModuleResourceRequest);
        module.StopModuleRequest = StopModuleRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var StartModuleRequest = (function (_super) {
            __extends(StartModuleRequest, _super);
            function StartModuleRequest(moduleKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.moduleKeys = moduleKeys;
            }
            StartModuleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "start");
            };
            StartModuleRequest.prototype.getParams = function () {
                return {
                    key: module.ModuleKey.toStringArray(this.moduleKeys)
                };
            };
            StartModuleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return StartModuleRequest;
        })(module.ModuleResourceRequest);
        module.StartModuleRequest = StartModuleRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var InstallModuleRequest = (function (_super) {
            __extends(InstallModuleRequest, _super);
            function InstallModuleRequest(moduleUrl) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.moduleUrl = moduleUrl;
            }
            InstallModuleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "install");
            };
            InstallModuleRequest.prototype.getParams = function () {
                return {
                    url: this.moduleUrl
                };
            };
            InstallModuleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return InstallModuleRequest;
        })(module.ModuleResourceRequest);
        module.InstallModuleRequest = InstallModuleRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var UninstallModuleRequest = (function (_super) {
            __extends(UninstallModuleRequest, _super);
            function UninstallModuleRequest(moduleKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.moduleKeys = moduleKeys;
            }
            UninstallModuleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "uninstall");
            };
            UninstallModuleRequest.prototype.getParams = function () {
                return {
                    key: module.ModuleKey.toStringArray(this.moduleKeys)
                };
            };
            UninstallModuleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return UninstallModuleRequest;
        })(module.ModuleResourceRequest);
        module.UninstallModuleRequest = UninstallModuleRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var UpdateModuleRequest = (function (_super) {
            __extends(UpdateModuleRequest, _super);
            function UpdateModuleRequest(moduleKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.moduleKeys = moduleKeys;
            }
            UpdateModuleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "update");
            };
            UpdateModuleRequest.prototype.getParams = function () {
                return {
                    key: module.ModuleKey.toStringArray(this.moduleKeys)
                };
            };
            UpdateModuleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return UpdateModuleRequest;
        })(module.ModuleResourceRequest);
        module.UpdateModuleRequest = UpdateModuleRequest;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleUpdatedEvent = (function (_super) {
            __extends(ModuleUpdatedEvent, _super);
            function ModuleUpdatedEvent(moduleKey, eventType) {
                _super.call(this);
                this.moduleKey = moduleKey;
                this.eventType = eventType;
            }
            ModuleUpdatedEvent.prototype.getModuleKey = function () {
                return this.moduleKey;
            };
            ModuleUpdatedEvent.prototype.getEventType = function () {
                return this.eventType;
            };
            ModuleUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ModuleUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ModuleUpdatedEvent.fromJson = function (json) {
                var moduleKey = api.module.ModuleKey.fromString(json.moduleKey);
                var eventType = json.eventType;
                return new ModuleUpdatedEvent(moduleKey, eventType);
            };
            return ModuleUpdatedEvent;
        })(api.event.Event);
        module.ModuleUpdatedEvent = ModuleUpdatedEvent;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleCache = (function (_super) {
            __extends(ModuleCache, _super);
            function ModuleCache() {
                var _this = this;
                _super.call(this);
                module.ModuleUpdatedEvent.on(function (event) {
                    console.log("ModuleCache on ModuleUpdatedEvent, deleting: " + event.getModuleKey().toString());
                    _this.deleteByKey(event.getModuleKey());
                });
            }
            ModuleCache.prototype.copy = function (object) {
                return new module.ModuleBuilder(object).build();
            };
            ModuleCache.prototype.getKeyFromObject = function (object) {
                return object.getModuleKey();
            };
            ModuleCache.prototype.getKeyAsString = function (key) {
                return key.toString();
            };
            ModuleCache.get = function () {
                if (!ModuleCache.instance) {
                    ModuleCache.instance = new ModuleCache();
                }
                return ModuleCache.instance;
            };
            return ModuleCache;
        })(api.cache.Cache);
        module.ModuleCache = ModuleCache;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleBasedName = (function () {
            function ModuleBasedName(moduleKey, localName) {
                this.moduleKey = moduleKey;
                this.localName = localName;
                this.refString = moduleKey.toString() ? moduleKey.toString() + ModuleBasedName.SEPARATOR + localName : localName;
            }
            ModuleBasedName.prototype.getLocalName = function () {
                return this.localName;
            };
            ModuleBasedName.prototype.getModuleKey = function () {
                return this.moduleKey;
            };
            ModuleBasedName.prototype.toString = function () {
                return this.refString;
            };
            ModuleBasedName.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, api.ClassHelper.getClass(this))) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            ModuleBasedName.SEPARATOR = ":";
            return ModuleBasedName;
        })();
        module.ModuleBasedName = ModuleBasedName;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleCaches = (function () {
            function ModuleCaches() {
                this.cacheByModuleKey = {};
            }
            ModuleCaches.prototype.put = function (key, cache) {
                this.cacheByModuleKey[key.toString()] = cache;
            };
            ModuleCaches.prototype.getByKey = function (key) {
                return this.cacheByModuleKey[key.toString()];
            };
            ModuleCaches.prototype.removeByKey = function (key) {
                delete this.cacheByModuleKey[key.toString()];
            };
            return ModuleCaches;
        })();
        module.ModuleCaches = ModuleCaches;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var module;
    (function (module) {
        var ModuleBasedCache = (function () {
            function ModuleBasedCache() {
                var _this = this;
                this.moduleCaches = new module.ModuleCaches();
                module.ModuleUpdatedEvent.on(function (event) {
                    if (event.getEventType() == "STARTED") {
                        console.log(api.ClassHelper.getClassName(_this) + " received ModuleUpdatedEvent STARTED, calling - loadByModule.. " + event.getModuleKey().toString());
                        _this.loadByModule(event.getModuleKey());
                    }
                    else if (event.getEventType() == "STOPPED") {
                        console.log(api.ClassHelper.getClassName(_this) + " received ModuleUpdatedEvent STOPPED - calling deleteByModuleKey.. " + event.getModuleKey().toString());
                        _this.deleteByModuleKey(event.getModuleKey());
                    }
                });
            }
            ModuleBasedCache.prototype.loadByModule = function (moduleKey) {
                throw new Error("Must be implemented by inheritor");
            };
            ModuleBasedCache.prototype.getByModule = function (moduleKey) {
                api.util.assertNotNull(moduleKey, "moduleKey not given");
                var cache = this.moduleCaches.getByKey(moduleKey);
                if (!cache) {
                    return null;
                }
                return cache.getAll();
            };
            ModuleBasedCache.prototype.getByKey = function (key, moduleKey) {
                api.util.assertNotNull(key, "key not given");
                var cache = this.moduleCaches.getByKey(moduleKey);
                if (!cache) {
                    return null;
                }
                return cache.getByKey(key);
            };
            ModuleBasedCache.prototype.put = function (object, moduleKey) {
                api.util.assertNotNull(object, "a object to cache must be given");
                var cache = this.moduleCaches.getByKey(moduleKey);
                if (!cache) {
                    cache = this.createModuleCache();
                    this.moduleCaches.put(moduleKey, cache);
                }
                cache.put(object);
            };
            ModuleBasedCache.prototype.createModuleCache = function () {
                throw new Error("Must be implemented by inheritor");
            };
            ModuleBasedCache.prototype.deleteByModuleKey = function (moduleKey) {
                this.moduleCaches.removeByKey(moduleKey);
            };
            return ModuleBasedCache;
        })();
        module.ModuleBasedCache = ModuleBasedCache;
    })(module = api.module || (api.module = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var Schema = (function (_super) {
            __extends(Schema, _super);
            function Schema(builder) {
                _super.call(this, builder);
                this.name = builder.name;
                this.displayName = builder.displayName;
                this.description = builder.description;
                this.iconUrl = builder.iconUrl;
            }
            Schema.prototype.getName = function () {
                return this.name;
            };
            Schema.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Schema.prototype.getDescription = function () {
                return this.description;
            };
            Schema.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            Schema.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Schema)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.iconUrl, other.iconUrl)) {
                    return false;
                }
                return true;
            };
            Schema.fromJson = function (json) {
                return new SchemaBuilder().fromSchemaJson(json).build();
            };
            return Schema;
        })(api.item.BaseItem);
        schema.Schema = Schema;
        var SchemaBuilder = (function (_super) {
            __extends(SchemaBuilder, _super);
            function SchemaBuilder(source) {
                if (source) {
                    _super.call(this, source);
                    this.name = source.getName();
                    this.displayName = source.getDisplayName();
                    this.description = source.getDescription();
                    this.iconUrl = source.getIconUrl();
                }
            }
            SchemaBuilder.prototype.fromSchemaJson = function (json) {
                _super.prototype.fromBaseItemJson.call(this, json, "name");
                this.name = json.name;
                this.displayName = json.displayName;
                this.description = json.description;
                this.iconUrl = json.iconUrl;
                return this;
            };
            SchemaBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            SchemaBuilder.prototype.build = function () {
                return new Schema(this);
            };
            return SchemaBuilder;
        })(api.item.BaseItemBuilder);
        schema.SchemaBuilder = SchemaBuilder;
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (_schema) {
        var SchemaIconUrlResolver = (function (_super) {
            __extends(SchemaIconUrlResolver, _super);
            function SchemaIconUrlResolver() {
                _super.apply(this, arguments);
            }
            SchemaIconUrlResolver.prototype.resolve = function (schema) {
                return schema.getIconUrl();
            };
            SchemaIconUrlResolver.getResourcePath = function () {
                return api.rest.Path.fromString("schema/icon");
            };
            return SchemaIconUrlResolver;
        })(api.icon.IconUrlResolver);
        _schema.SchemaIconUrlResolver = SchemaIconUrlResolver;
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var ModuleKey = api.module.ModuleKey;
            var MixinName = (function (_super) {
                __extends(MixinName, _super);
                function MixinName(name) {
                    api.util.assertNotNull(name, "Mixin name can't be null");
                    var parts = name.split(api.module.ModuleBasedName.SEPARATOR);
                    _super.call(this, ModuleKey.fromString(parts[0]), parts[1]);
                }
                MixinName.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, MixinName)) {
                        return false;
                    }
                    return _super.prototype.equals.call(this, o);
                };
                return MixinName;
            })(api.module.ModuleBasedName);
            mixin.MixinName = MixinName;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (_mixin) {
            var MixinNames = (function () {
                function MixinNames(array) {
                    var _this = this;
                    this.array = [];
                    array.forEach(function (mixinName) {
                        var duplicate = _this.array.some(function (possibleDuplicate) {
                            return mixinName.equals(possibleDuplicate);
                        });
                        if (!duplicate) {
                            _this.array.push(mixinName);
                        }
                        else {
                            throw Error("MixinNames do not allow duplicates, found: '" + mixinName.toString() + "'");
                        }
                    });
                }
                MixinNames.prototype.forEach = function (callback) {
                    this.array.forEach(function (mixinName, index) {
                        callback(mixinName, index);
                    });
                };
                MixinNames.prototype.contains = function (mixinName) {
                    var containName = this.array.some(function (curMixin) {
                        return curMixin.equals(mixinName);
                    });
                    return !!containName;
                };
                MixinNames.prototype.filter = function (callbackfn) {
                    return new MixinNames(this.array.filter(function (value, index) {
                        return callbackfn(value, index);
                    }));
                };
                MixinNames.prototype.map = function (callbackfn) {
                    return this.array.map(function (value, index) {
                        return callbackfn(value, index);
                    });
                };
                MixinNames.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, MixinNames)) {
                        return false;
                    }
                    var other = o;
                    return api.ObjectHelper.arrayEquals(this.array, other.array);
                };
                MixinNames.create = function () {
                    return new MixinNamesBuilder();
                };
                return MixinNames;
            })();
            _mixin.MixinNames = MixinNames;
            var MixinNamesBuilder = (function () {
                function MixinNamesBuilder() {
                    this.array = [];
                }
                MixinNamesBuilder.prototype.fromStrings = function (values) {
                    var _this = this;
                    if (!!values) {
                        values.forEach(function (value) {
                            _this.addMixinName(new _mixin.MixinName(value));
                        });
                    }
                    return this;
                };
                MixinNamesBuilder.prototype.fromMixins = function (mixins) {
                    var _this = this;
                    if (!!mixins) {
                        mixins.forEach(function (mixin) {
                            _this.addMixinName(mixin.getMixinName());
                        });
                    }
                    return this;
                };
                MixinNamesBuilder.prototype.addMixinName = function (value) {
                    this.array.push(value);
                    return this;
                };
                MixinNamesBuilder.prototype.build = function () {
                    return new MixinNames(this.array);
                };
                return MixinNamesBuilder;
            })();
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var Mixin = (function (_super) {
                __extends(Mixin, _super);
                function Mixin(builder) {
                    _super.call(this, builder);
                    this.formItems = builder.formItems;
                    this.schemaKey = builder.schemaKey;
                }
                Mixin.prototype.getMixinName = function () {
                    return new mixin.MixinName(this.getName());
                };
                Mixin.prototype.getFormItems = function () {
                    return this.formItems;
                };
                Mixin.prototype.getSchemaKey = function () {
                    return this.schemaKey;
                };
                Mixin.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Mixin)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.schemaKey, other.schemaKey)) {
                        return false;
                    }
                    if (!api.ObjectHelper.arrayEquals(this.formItems, other.formItems)) {
                        return false;
                    }
                    return true;
                };
                Mixin.prototype.toForm = function () {
                    return new api.form.FormBuilder().addFormItems(this.formItems).build();
                };
                Mixin.fromJson = function (json) {
                    return new MixinBuilder().fromMixinJson(json).build();
                };
                return Mixin;
            })(api.schema.Schema);
            mixin.Mixin = Mixin;
            var MixinBuilder = (function (_super) {
                __extends(MixinBuilder, _super);
                function MixinBuilder(source) {
                    _super.call(this, source);
                    if (source) {
                        this.schemaKey = source.getSchemaKey();
                        this.formItems = source.getFormItems();
                    }
                }
                MixinBuilder.prototype.fromMixinJson = function (mixinJson) {
                    var _this = this;
                    _super.prototype.fromSchemaJson.call(this, mixinJson);
                    this.formItems = [];
                    mixinJson.items.forEach(function (formItemJson) {
                        _this.formItems.push(api.form.FormItemFactory.createFormItem(formItemJson));
                    });
                    this.schemaKey = "mixin:" + this.name;
                    return this;
                };
                MixinBuilder.prototype.build = function () {
                    return new Mixin(this);
                };
                return MixinBuilder;
            })(api.schema.SchemaBuilder);
            mixin.MixinBuilder = MixinBuilder;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var MixinResourceRequest = (function (_super) {
                __extends(MixinResourceRequest, _super);
                function MixinResourceRequest() {
                    _super.call(this);
                    this.resourceUrl = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "schema/mixin");
                }
                MixinResourceRequest.prototype.getResourcePath = function () {
                    return this.resourceUrl;
                };
                MixinResourceRequest.prototype.fromJsonToMixin = function (json) {
                    return mixin.Mixin.fromJson(json);
                };
                return MixinResourceRequest;
            })(api.rest.ResourceRequest);
            mixin.MixinResourceRequest = MixinResourceRequest;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var GetMixinByQualifiedNameRequest = (function (_super) {
                __extends(GetMixinByQualifiedNameRequest, _super);
                function GetMixinByQualifiedNameRequest(name) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.name = name;
                }
                GetMixinByQualifiedNameRequest.prototype.getParams = function () {
                    return {
                        name: this.name.toString()
                    };
                };
                GetMixinByQualifiedNameRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetMixinByQualifiedNameRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return _this.fromJsonToMixin(response.getResult());
                    });
                };
                return GetMixinByQualifiedNameRequest;
            })(mixin.MixinResourceRequest);
            mixin.GetMixinByQualifiedNameRequest = GetMixinByQualifiedNameRequest;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var GetMixinsByModuleRequest = (function (_super) {
                __extends(GetMixinsByModuleRequest, _super);
                function GetMixinsByModuleRequest(moduleKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.moduleKey = moduleKey;
                }
                GetMixinsByModuleRequest.prototype.getParams = function () {
                    return {
                        moduleKey: this.moduleKey.toString()
                    };
                };
                GetMixinsByModuleRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byModule");
                };
                GetMixinsByModuleRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().mixins.map(function (mixinJson) {
                            return _this.fromJsonToMixin(mixinJson);
                        });
                    });
                };
                return GetMixinsByModuleRequest;
            })(mixin.MixinResourceRequest);
            mixin.GetMixinsByModuleRequest = GetMixinsByModuleRequest;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var InlineMixinJson = (function (_super) {
                __extends(InlineMixinJson, _super);
                function InlineMixinJson() {
                    _super.apply(this, arguments);
                }
                return InlineMixinJson;
            })(api.form.json.FormItemJson);
            mixin.InlineMixinJson = InlineMixinJson;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ModuleKey = api.module.ModuleKey;
            var ContentTypeName = (function (_super) {
                __extends(ContentTypeName, _super);
                function ContentTypeName(name) {
                    api.util.assertNotNull(name, "Content type name can't be null");
                    var parts = name.split(api.module.ModuleBasedName.SEPARATOR);
                    _super.call(this, ModuleKey.fromString(parts[0]), parts[1]);
                }
                ContentTypeName.from = function (moduleKey, localName) {
                    return new ContentTypeName(moduleKey.toString() + ":" + localName);
                };
                ContentTypeName.prototype.isSite = function () {
                    return ContentTypeName.SITE.equals(this);
                };
                ContentTypeName.prototype.isPageTemplate = function () {
                    return ContentTypeName.PAGE_TEMPLATE.equals(this);
                };
                ContentTypeName.prototype.isTemplateFolder = function () {
                    return ContentTypeName.TEMPLATE_FOLDER.equals(this);
                };
                ContentTypeName.prototype.isImage = function () {
                    return ContentTypeName.IMAGE.equals(this);
                };
                ContentTypeName.prototype.isMedia = function () {
                    return ContentTypeName.MEDIA.equals(this);
                };
                ContentTypeName.prototype.isDescendantOfMedia = function () {
                    return ContentTypeName.MEDIA_ARCHIVE.equals(this) || ContentTypeName.MEDIA_AUDIO.equals(this) || ContentTypeName.MEDIA_VIDEO.equals(this) || ContentTypeName.MEDIA_CODE.equals(this) || ContentTypeName.MEDIA_DATA.equals(this) || ContentTypeName.MEDIA_DOCUMENT.equals(this) || ContentTypeName.MEDIA_EXECUTABLE.equals(this) || ContentTypeName.MEDIA_IMAGE.equals(this) || ContentTypeName.MEDIA_SPREADSHEET.equals(this) || ContentTypeName.MEDIA_PRESENTATION.equals(this) || ContentTypeName.MEDIA_VECTOR.equals(this) || ContentTypeName.MEDIA_TEXT.equals(this) || ContentTypeName.MEDIA_UNKNOWN.equals(this);
                };
                ContentTypeName.UNSTRUCTURED = ContentTypeName.from(ModuleKey.BASE, "unstructured");
                ContentTypeName.FOLDER = ContentTypeName.from(ModuleKey.BASE, "folder");
                ContentTypeName.SHORTCUT = ContentTypeName.from(ModuleKey.BASE, "shortcut");
                ContentTypeName.MEDIA = ContentTypeName.from(ModuleKey.BASE, "media");
                ContentTypeName.MEDIA_TEXT = ContentTypeName.from(ModuleKey.MEDIA, "text");
                ContentTypeName.MEDIA_DATA = ContentTypeName.from(ModuleKey.MEDIA, "data");
                ContentTypeName.MEDIA_AUDIO = ContentTypeName.from(ModuleKey.MEDIA, "audio");
                ContentTypeName.MEDIA_VIDEO = ContentTypeName.from(ModuleKey.MEDIA, "video");
                ContentTypeName.MEDIA_IMAGE = ContentTypeName.from(ModuleKey.MEDIA, "image");
                ContentTypeName.MEDIA_VECTOR = ContentTypeName.from(ModuleKey.MEDIA, "vector");
                ContentTypeName.MEDIA_ARCHIVE = ContentTypeName.from(ModuleKey.MEDIA, "archive");
                ContentTypeName.MEDIA_DOCUMENT = ContentTypeName.from(ModuleKey.MEDIA, "document");
                ContentTypeName.MEDIA_SPREADSHEET = ContentTypeName.from(ModuleKey.MEDIA, "spreadsheet");
                ContentTypeName.MEDIA_PRESENTATION = ContentTypeName.from(ModuleKey.MEDIA, "presentation");
                ContentTypeName.MEDIA_CODE = ContentTypeName.from(ModuleKey.MEDIA, "code");
                ContentTypeName.MEDIA_EXECUTABLE = ContentTypeName.from(ModuleKey.MEDIA, "executable");
                ContentTypeName.MEDIA_UNKNOWN = ContentTypeName.from(ModuleKey.MEDIA, "unknown");
                ContentTypeName.SITE = ContentTypeName.from(ModuleKey.PORTAL, "site");
                ContentTypeName.PAGE_TEMPLATE = ContentTypeName.from(ModuleKey.PORTAL, "page-template");
                ContentTypeName.TEMPLATE_FOLDER = ContentTypeName.from(ModuleKey.PORTAL, "template-folder");
                ContentTypeName.IMAGE = ContentTypeName.from(ModuleKey.MEDIA, "image");
                return ContentTypeName;
            })(api.module.ModuleBasedName);
            content.ContentTypeName = ContentTypeName;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var MixinNames = api.schema.mixin.MixinNames;
            var ContentTypeSummary = (function (_super) {
                __extends(ContentTypeSummary, _super);
                function ContentTypeSummary(builder) {
                    _super.call(this, builder);
                    this.allowChildContent = builder.allowChildContent;
                    this.final = builder.final;
                    this.abstract = builder.abstract;
                    this.superType = builder.superType;
                    this.contentDisplayNameScript = builder.contentDisplayNameScript;
                    this.owner = builder.owner;
                    this.modifier = builder.modifier;
                    this.metadata = builder.metadata;
                }
                ContentTypeSummary.prototype.getContentTypeName = function () {
                    return new api.schema.content.ContentTypeName(this.getName());
                };
                ContentTypeSummary.prototype.isSite = function () {
                    return this.getContentTypeName().isSite();
                };
                ContentTypeSummary.prototype.isPageTemplate = function () {
                    return this.getContentTypeName().isPageTemplate();
                };
                ContentTypeSummary.prototype.isImage = function () {
                    return this.getContentTypeName().isImage();
                };
                ContentTypeSummary.prototype.isFinal = function () {
                    return this.final;
                };
                ContentTypeSummary.prototype.isAbstract = function () {
                    return this.abstract;
                };
                ContentTypeSummary.prototype.isAllowChildContent = function () {
                    return this.allowChildContent;
                };
                ContentTypeSummary.prototype.getSuperType = function () {
                    return this.superType;
                };
                ContentTypeSummary.prototype.hasContentDisplayNameScript = function () {
                    return !api.util.StringHelper.isBlank(this.contentDisplayNameScript);
                };
                ContentTypeSummary.prototype.getContentDisplayNameScript = function () {
                    return this.contentDisplayNameScript;
                };
                ContentTypeSummary.prototype.getOwner = function () {
                    return this.owner;
                };
                ContentTypeSummary.prototype.getModifier = function () {
                    return this.modifier;
                };
                ContentTypeSummary.prototype.getMetadata = function () {
                    return this.metadata;
                };
                ContentTypeSummary.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentTypeSummary)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.booleanEquals(this.allowChildContent, other.allowChildContent)) {
                        return false;
                    }
                    if (!api.ObjectHelper.booleanEquals(this.abstract, other.abstract)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.superType, other.superType)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.contentDisplayNameScript, other.contentDisplayNameScript)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.modifier, other.modifier)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.owner, other.owner)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.metadata, other.metadata)) {
                        return false;
                    }
                    return true;
                };
                ContentTypeSummary.fromJsonArray = function (jsonArray) {
                    var array = [];
                    jsonArray.forEach(function (summaryJson) {
                        array.push(ContentTypeSummary.fromJson(summaryJson));
                    });
                    return array;
                };
                ContentTypeSummary.fromJson = function (json) {
                    return new ContentTypeSummaryBuilder().fromContentTypeSummaryJson(json).build();
                };
                return ContentTypeSummary;
            })(api.schema.Schema);
            content.ContentTypeSummary = ContentTypeSummary;
            var ContentTypeSummaryBuilder = (function (_super) {
                __extends(ContentTypeSummaryBuilder, _super);
                function ContentTypeSummaryBuilder(source) {
                    if (source) {
                        _super.call(this, source);
                        this.allowChildContent = source.isAllowChildContent();
                        this.abstract = source.isAbstract();
                        this.final = source.isFinal();
                        this.superType = source.getSuperType();
                        this.contentDisplayNameScript = source.getContentDisplayNameScript();
                        this.modifier = source.getModifier();
                        this.owner = source.getOwner();
                        this.metadata = source.getMetadata();
                    }
                }
                ContentTypeSummaryBuilder.prototype.fromContentTypeSummaryJson = function (json) {
                    _super.prototype.fromSchemaJson.call(this, json);
                    this.allowChildContent = json.allowChildContent;
                    this.final = json.final;
                    this.abstract = json.abstract;
                    this.superType = json.superType ? new api.schema.content.ContentTypeName(json.superType) : null;
                    this.contentDisplayNameScript = json.contentDisplayNameScript;
                    this.owner = json.owner;
                    this.modifier = json.modifier;
                    this.metadata = MixinNames.create().fromStrings(json.metadata).build();
                    return this;
                };
                ContentTypeSummaryBuilder.prototype.build = function () {
                    return new ContentTypeSummary(this);
                };
                return ContentTypeSummaryBuilder;
            })(api.schema.SchemaBuilder);
            content.ContentTypeSummaryBuilder = ContentTypeSummaryBuilder;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentType = (function (_super) {
                __extends(ContentType, _super);
                function ContentType(builder) {
                    _super.call(this, builder);
                    this.form = builder.form;
                }
                ContentType.prototype.getForm = function () {
                    return this.form;
                };
                ContentType.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentType)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.form, other.form)) {
                        return false;
                    }
                    return true;
                };
                ContentType.fromJson = function (json) {
                    return new ContentTypeBuilder().fromContentTypeJson(json).build();
                };
                return ContentType;
            })(content.ContentTypeSummary);
            content.ContentType = ContentType;
            var ContentTypeBuilder = (function (_super) {
                __extends(ContentTypeBuilder, _super);
                function ContentTypeBuilder(source) {
                    if (source) {
                        _super.call(this, source);
                        this.form = source.getForm();
                    }
                }
                ContentTypeBuilder.prototype.fromContentTypeJson = function (json) {
                    _super.prototype.fromContentTypeSummaryJson.call(this, json);
                    this.form = api.form.FormItemFactory.createForm(json.form);
                    return this;
                };
                ContentTypeBuilder.prototype.setForm = function (value) {
                    this.form = value;
                    return this;
                };
                ContentTypeBuilder.prototype.build = function () {
                    return new ContentType(this);
                };
                return ContentTypeBuilder;
            })(content.ContentTypeSummaryBuilder);
            content.ContentTypeBuilder = ContentTypeBuilder;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeIconUrlResolver = (function (_super) {
                __extends(ContentTypeIconUrlResolver, _super);
                function ContentTypeIconUrlResolver() {
                    _super.apply(this, arguments);
                }
                ContentTypeIconUrlResolver.default = function () {
                    return api.util.UriHelper.getRestUri(api.rest.Path.fromParent(api.schema.SchemaIconUrlResolver.getResourcePath(), "base:structured").toString());
                };
                return ContentTypeIconUrlResolver;
            })(api.schema.SchemaIconUrlResolver);
            content.ContentTypeIconUrlResolver = ContentTypeIconUrlResolver;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeSummaryViewer = (function (_super) {
                __extends(ContentTypeSummaryViewer, _super);
                function ContentTypeSummaryViewer() {
                    _super.call(this);
                    this.contentTypeIconUrlResolver = new content.ContentTypeIconUrlResolver();
                }
                ContentTypeSummaryViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                ContentTypeSummaryViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getName();
                };
                ContentTypeSummaryViewer.prototype.resolveIconUrl = function (object) {
                    return this.contentTypeIconUrlResolver.resolve(object);
                };
                return ContentTypeSummaryViewer;
            })(api.ui.NamesAndIconViewer);
            content.ContentTypeSummaryViewer = ContentTypeSummaryViewer;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (_content) {
            var ContentTypeComboBox = (function (_super) {
                __extends(ContentTypeComboBox, _super);
                function ContentTypeComboBox(maximumOccurrences) {
                    if (maximumOccurrences === void 0) { maximumOccurrences = 0; }
                    var loader = new _content.ContentTypeSummaryLoader();
                    _super.call(this, new api.ui.selector.combobox.RichComboBoxBuilder().setLoader(loader).setSelectedOptionsView(new ContentTypeSelectedOptionsView()).setOptionDisplayValueViewer(new _content.ContentTypeSummaryViewer()).setMaximumOccurrences(maximumOccurrences));
                    loader.load();
                }
                return ContentTypeComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            _content.ContentTypeComboBox = ContentTypeComboBox;
            var ContentTypeSelectedOptionsView = (function (_super) {
                __extends(ContentTypeSelectedOptionsView, _super);
                function ContentTypeSelectedOptionsView() {
                    _super.apply(this, arguments);
                }
                ContentTypeSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    var optionView = new ContentTypeSelectedOptionView(option);
                    return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
                };
                return ContentTypeSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
            _content.ContentTypeSelectedOptionsView = ContentTypeSelectedOptionsView;
            var ContentTypeSelectedOptionView = (function (_super) {
                __extends(ContentTypeSelectedOptionView, _super);
                function ContentTypeSelectedOptionView(option) {
                    _super.call(this, option);
                }
                ContentTypeSelectedOptionView.prototype.resolveIconUrl = function (content) {
                    return content.getIconUrl();
                };
                ContentTypeSelectedOptionView.prototype.resolveTitle = function (content) {
                    return content.getDisplayName().toString();
                };
                ContentTypeSelectedOptionView.prototype.resolveSubTitle = function (content) {
                    return content.getName();
                };
                return ContentTypeSelectedOptionView;
            })(api.ui.selector.combobox.RichSelectedOptionView);
            _content.ContentTypeSelectedOptionView = ContentTypeSelectedOptionView;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeSummaryLoader = (function (_super) {
                __extends(ContentTypeSummaryLoader, _super);
                function ContentTypeSummaryLoader() {
                    _super.call(this, new content.GetAllContentTypesRequest());
                }
                ContentTypeSummaryLoader.prototype.filterFn = function (contentType) {
                    return contentType.getContentTypeName().toString().indexOf(this.getSearchString().toLowerCase()) != -1;
                };
                return ContentTypeSummaryLoader;
            })(api.util.loader.BaseLoader);
            content.ContentTypeSummaryLoader = ContentTypeSummaryLoader;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            (function (ContentStateEnum) {
                ContentStateEnum[ContentStateEnum["PENDING_DELETE"] = 0] = "PENDING_DELETE";
                ContentStateEnum[ContentStateEnum["DEFAULT"] = 1] = "DEFAULT";
            })(content.ContentStateEnum || (content.ContentStateEnum = {}));
            var ContentStateEnum = content.ContentStateEnum;
            var ContentState = (function () {
                function ContentState(state) {
                    if (state === void 0) { state = 1 /* DEFAULT */; }
                    this.state = state;
                }
                ContentState.prototype.getState = function () {
                    return this.state;
                };
                ContentState.prototype.getStateAsString = function () {
                    return ContentStateEnum[this.state];
                };
                ContentState.fromString = function (value) {
                    if (value === ContentStateEnum[0 /* PENDING_DELETE */]) {
                        return new ContentState(0 /* PENDING_DELETE */);
                    }
                    else {
                        return new ContentState(1 /* DEFAULT */);
                    }
                };
                ContentState.prototype.isDefault = function () {
                    return this.state === 1 /* DEFAULT */;
                };
                ContentState.prototype.isPendingDelete = function () {
                    return this.state === 0 /* PENDING_DELETE */;
                };
                return ContentState;
            })();
            content.ContentState = ContentState;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeResourceRequest = (function (_super) {
                __extends(ContentTypeResourceRequest, _super);
                function ContentTypeResourceRequest() {
                    _super.call(this);
                    this.resourceUrl = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "schema/content");
                }
                ContentTypeResourceRequest.prototype.getResourcePath = function () {
                    return this.resourceUrl;
                };
                ContentTypeResourceRequest.prototype.fromJsonToContentType = function (json) {
                    return content.ContentType.fromJson(json);
                };
                ContentTypeResourceRequest.prototype.fromJsonToContentTypeSummary = function (json) {
                    return content.ContentTypeSummary.fromJson(json);
                };
                return ContentTypeResourceRequest;
            })(api.rest.ResourceRequest);
            content.ContentTypeResourceRequest = ContentTypeResourceRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var GetAllContentTypesRequest = (function (_super) {
                __extends(GetAllContentTypesRequest, _super);
                function GetAllContentTypesRequest() {
                    _super.call(this);
                    this.inlineMixinsToFormItems = true;
                    _super.prototype.setMethod.call(this, "GET");
                }
                GetAllContentTypesRequest.prototype.getParams = function () {
                    return {
                        inlineMixinsToFormItems: this.inlineMixinsToFormItems
                    };
                };
                GetAllContentTypesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "all");
                };
                GetAllContentTypesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contentTypes.map(function (contentTypeJson) {
                            return _this.fromJsonToContentTypeSummary(contentTypeJson);
                        });
                    });
                };
                return GetAllContentTypesRequest;
            })(content.ContentTypeResourceRequest);
            content.GetAllContentTypesRequest = GetAllContentTypesRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var GetContentTypeByNameRequest = (function (_super) {
                __extends(GetContentTypeByNameRequest, _super);
                function GetContentTypeByNameRequest(name) {
                    _super.call(this);
                    this.inlineMixinsToFormItems = true;
                    _super.prototype.setMethod.call(this, "GET");
                    this.name = name;
                }
                GetContentTypeByNameRequest.prototype.getParams = function () {
                    return {
                        name: this.name.toString(),
                        inlineMixinsToFormItems: this.inlineMixinsToFormItems
                    };
                };
                GetContentTypeByNameRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetContentTypeByNameRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    var contentTypeCache = content.ContentTypeCache.get();
                    var contentType = contentTypeCache.getByKey(this.name);
                    if (contentType) {
                        return wemQ(contentType);
                    }
                    else {
                        return this.send().then(function (response) {
                            contentType = _this.fromJsonToContentType(response.getResult());
                            contentTypeCache.put(contentType);
                            return contentType;
                        });
                    }
                };
                return GetContentTypeByNameRequest;
            })(content.ContentTypeResourceRequest);
            content.GetContentTypeByNameRequest = GetContentTypeByNameRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var GetContentTypesByModuleRequest = (function (_super) {
                __extends(GetContentTypesByModuleRequest, _super);
                function GetContentTypesByModuleRequest(moduleKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.moduleKey = moduleKey;
                }
                GetContentTypesByModuleRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byModule");
                };
                GetContentTypesByModuleRequest.prototype.getParams = function () {
                    return {
                        moduleKey: this.moduleKey.toString()
                    };
                };
                GetContentTypesByModuleRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contentTypes.map(function (contentTypeJson) {
                            return _this.fromJsonToContentTypeSummary(contentTypeJson);
                        });
                    });
                };
                return GetContentTypesByModuleRequest;
            })(content.ContentTypeResourceRequest);
            content.GetContentTypesByModuleRequest = GetContentTypesByModuleRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeUpdatedEvent = (function (_super) {
                __extends(ContentTypeUpdatedEvent, _super);
                function ContentTypeUpdatedEvent(name, modifiedTime) {
                    _super.call(this);
                    this.contentTypeName = name;
                    this.modifiedTime = modifiedTime;
                }
                ContentTypeUpdatedEvent.prototype.getContentTypeName = function () {
                    return this.contentTypeName;
                };
                ContentTypeUpdatedEvent.prototype.getModifiedTime = function () {
                    return this.modifiedTime;
                };
                ContentTypeUpdatedEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentTypeUpdatedEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                ContentTypeUpdatedEvent.fromJson = function (json) {
                    var modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                    return new ContentTypeUpdatedEvent(new content.ContentTypeName(json.name), modifiedTime);
                };
                return ContentTypeUpdatedEvent;
            })(api.event.Event);
            content.ContentTypeUpdatedEvent = ContentTypeUpdatedEvent;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeDeletedEvent = (function (_super) {
                __extends(ContentTypeDeletedEvent, _super);
                function ContentTypeDeletedEvent(name) {
                    _super.call(this);
                    this.contentTypeName = name;
                }
                ContentTypeDeletedEvent.prototype.getContentTypeName = function () {
                    return this.contentTypeName;
                };
                ContentTypeDeletedEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentTypeDeletedEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                ContentTypeDeletedEvent.fromJson = function (json) {
                    return new ContentTypeDeletedEvent(new content.ContentTypeName(json.name));
                };
                return ContentTypeDeletedEvent;
            })(api.event.Event);
            content.ContentTypeDeletedEvent = ContentTypeDeletedEvent;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeCache = (function (_super) {
                __extends(ContentTypeCache, _super);
                function ContentTypeCache() {
                    var _this = this;
                    _super.call(this);
                    content.ContentTypeUpdatedEvent.on(function (event) {
                        console.log("ContentTypeCache on ContentTypeUpdatedEvent: " + event.getContentTypeName().toString());
                        var cachedObject = _this.getByKey(event.getContentTypeName());
                        if (cachedObject) {
                            var cachedModifiedTimeAsMillis = cachedObject.getModifiedTime().getTime();
                            var serverUpdateModifiedTimeAsMillis = event.getModifiedTime().getTime();
                            console.log("cachedObject.getModifiedTime().toUTCString(): " + cachedObject.getModifiedTime().toUTCString());
                            console.log("event.getModifiedTime().toUTCString():  " + event.getModifiedTime().toUTCString());
                            if (cachedModifiedTimeAsMillis != serverUpdateModifiedTimeAsMillis) {
                                console.log("  updated ContentType modified after cached ContentType - removing object from cache");
                                console.log("  cachedObject.getModifiedTime(): " + cachedObject.getModifiedTime());
                                console.log("  event: " + event.getModifiedTime());
                                _this.deleteByKey(event.getContentTypeName());
                            }
                            else {
                                console.log("  updated ContentType already in cache!");
                                _this.deleteByKey(event.getContentTypeName());
                            }
                        }
                    });
                    content.ContentTypeDeletedEvent.on(function (event) {
                        console.log("ContentTypeCache on ContentTypeDeletedEvent, deleting: " + event.getContentTypeName().toString());
                        _this.deleteByKey(event.getContentTypeName());
                    });
                }
                ContentTypeCache.prototype.copy = function (object) {
                    return new content.ContentTypeBuilder(object).build();
                };
                ContentTypeCache.prototype.getKeyFromObject = function (object) {
                    return new content.ContentTypeName(object.getName());
                };
                ContentTypeCache.prototype.getKeyAsString = function (key) {
                    return key.toString();
                };
                ContentTypeCache.get = function () {
                    if (!ContentTypeCache.instance) {
                        ContentTypeCache.instance = new ContentTypeCache();
                    }
                    return ContentTypeCache.instance;
                };
                return ContentTypeCache;
            })(api.cache.Cache);
            content.ContentTypeCache = ContentTypeCache;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var inputtype;
            (function (inputtype) {
                var InputValidationRecording = api.form.inputtype.InputValidationRecording;
                var InputValidityChangedEvent = api.form.inputtype.InputValidityChangedEvent;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var ContentTypeComboBox = api.schema.content.ContentTypeComboBox;
                var ContentTypeFilter = (function (_super) {
                    __extends(ContentTypeFilter, _super);
                    function ContentTypeFilter(context) {
                        _super.call(this, 'content-type-filter');
                    }
                    ContentTypeFilter.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    ContentTypeFilter.prototype.newInitialValue = function () {
                        return null;
                    };
                    ContentTypeFilter.prototype.layout = function (input, propertyArray) {
                        var _this = this;
                        this.layoutInProgress = true;
                        this.input = input;
                        this.propertyArray = propertyArray;
                        this.combobox = new ContentTypeComboBox(input.getOccurrences().getMaximum());
                        var selectProperties = function (contentTypeArray) {
                            propertyArray.forEach(function (property) {
                                var contentTypeName = property.getString();
                                _this.combobox.getComboBox().setValue(contentTypeName);
                            });
                            _this.layoutInProgress = false;
                            _this.combobox.unLoaded(selectProperties);
                        };
                        this.combobox.onLoaded(selectProperties);
                        this.combobox.onOptionSelected(function (event) {
                            if (_this.layoutInProgress) {
                                return;
                            }
                            var value = new Value(event.getOption().displayValue.getContentTypeName().toString(), ValueTypes.STRING);
                            if (_this.combobox.countSelected() == 1) {
                                _this.propertyArray.set(0, value);
                            }
                            else {
                                _this.propertyArray.add(value);
                            }
                            _this.validate(false);
                        });
                        this.combobox.onOptionDeselected(function (option) {
                            _this.propertyArray.remove(option.getIndex());
                            _this.validate(false);
                        });
                        this.appendChild(this.combobox);
                        return wemQ(null);
                    };
                    ContentTypeFilter.prototype.getValues = function () {
                        return this.combobox.getSelectedDisplayValues().map(function (contentType) {
                            return new Value(contentType.getContentTypeName().toString(), ValueTypes.STRING);
                        });
                    };
                    ContentTypeFilter.prototype.validate = function (silent) {
                        if (silent === void 0) { silent = true; }
                        var recording = new InputValidationRecording(), values = this.getValues(), occurrences = this.input.getOccurrences();
                        recording.setBreaksMinimumOccurrences(occurrences.minimumBreached(values.length));
                        recording.setBreaksMaximumOccurrences(occurrences.maximumBreached(values.length));
                        if (!silent && recording.validityChanged(this.validationRecording)) {
                            this.notifyValidityChanged(new InputValidityChangedEvent(recording, this.input.getName()));
                        }
                        this.validationRecording = recording;
                        return recording;
                    };
                    ContentTypeFilter.prototype.giveFocus = function () {
                        return this.combobox.maximumOccurrencesReached() ? false : this.combobox.giveFocus();
                    };
                    ContentTypeFilter.prototype.onFocus = function (listener) {
                        this.combobox.onFocus(listener);
                    };
                    ContentTypeFilter.prototype.unFocus = function (listener) {
                        this.combobox.unFocus(listener);
                    };
                    ContentTypeFilter.prototype.onBlur = function (listener) {
                        this.combobox.onBlur(listener);
                    };
                    ContentTypeFilter.prototype.unBlur = function (listener) {
                        this.combobox.unBlur(listener);
                    };
                    return ContentTypeFilter;
                })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                inputtype.ContentTypeFilter = ContentTypeFilter;
                api.form.inputtype.InputTypeManager.register(new api.Class("ContentTypeFilter", ContentTypeFilter));
            })(inputtype = content.inputtype || (content.inputtype = {}));
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var ModuleKey = api.module.ModuleKey;
            var RelationshipTypeName = (function (_super) {
                __extends(RelationshipTypeName, _super);
                function RelationshipTypeName(name) {
                    api.util.assertNotNull(name, "RelationshipType name can't be null");
                    var parts = name.split(api.module.ModuleBasedName.SEPARATOR);
                    _super.call(this, ModuleKey.fromString(parts[0]), parts[1]);
                }
                RelationshipTypeName.REFERENCE = new RelationshipTypeName("reference");
                return RelationshipTypeName;
            })(api.module.ModuleBasedName);
            relationshiptype.RelationshipTypeName = RelationshipTypeName;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var RelationshipType = (function (_super) {
                __extends(RelationshipType, _super);
                function RelationshipType(builder) {
                    _super.call(this, builder);
                    this.fromSemantic = builder.fromSemantic;
                    this.toSemantic = builder.toSemantic;
                    this.allowedFromTypes = builder.allowedFromTypes;
                    this.allowedToTypes = builder.allowedToTypes;
                }
                RelationshipType.prototype.getRelationshiptypeName = function () {
                    return new relationshiptype.RelationshipTypeName(this.getName());
                };
                RelationshipType.prototype.getFromSemantic = function () {
                    return this.fromSemantic;
                };
                RelationshipType.prototype.getToSemantic = function () {
                    return this.toSemantic;
                };
                RelationshipType.prototype.getAllowedFromTypes = function () {
                    return this.allowedFromTypes;
                };
                RelationshipType.prototype.getAllowedToTypes = function () {
                    return this.allowedToTypes;
                };
                RelationshipType.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, RelationshipType)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.fromSemantic, other.fromSemantic)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.toSemantic, other.toSemantic)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringArrayEquals(this.allowedFromTypes, other.allowedFromTypes)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringArrayEquals(this.allowedToTypes, other.allowedToTypes)) {
                        return false;
                    }
                    return true;
                };
                RelationshipType.fromJson = function (json) {
                    return new RelationshipTypeBuilder().fromRelationshipTypeJson(json).build();
                };
                return RelationshipType;
            })(api.schema.Schema);
            relationshiptype.RelationshipType = RelationshipType;
            var RelationshipTypeBuilder = (function (_super) {
                __extends(RelationshipTypeBuilder, _super);
                function RelationshipTypeBuilder(source) {
                    if (source) {
                        _super.call(this, source);
                        this.fromSemantic = source.getFromSemantic();
                        this.toSemantic = source.getToSemantic();
                        this.allowedFromTypes = source.getAllowedFromTypes();
                        this.allowedToTypes = source.getAllowedToTypes();
                    }
                }
                RelationshipTypeBuilder.prototype.fromRelationshipTypeJson = function (relationshipTypeJson) {
                    _super.prototype.fromSchemaJson.call(this, relationshipTypeJson);
                    this.fromSemantic = relationshipTypeJson.fromSemantic;
                    this.toSemantic = relationshipTypeJson.toSemantic;
                    this.allowedFromTypes = relationshipTypeJson.allowedFromTypes;
                    this.allowedToTypes = relationshipTypeJson.allowedToTypes;
                    return this;
                };
                RelationshipTypeBuilder.prototype.build = function () {
                    return new RelationshipType(this);
                };
                return RelationshipTypeBuilder;
            })(api.schema.SchemaBuilder);
            relationshiptype.RelationshipTypeBuilder = RelationshipTypeBuilder;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var RelationshipTypeResourceRequest = (function (_super) {
                __extends(RelationshipTypeResourceRequest, _super);
                function RelationshipTypeResourceRequest() {
                    _super.call(this);
                    this.resourceUrl = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "schema/relationship");
                }
                RelationshipTypeResourceRequest.prototype.getResourcePath = function () {
                    return this.resourceUrl;
                };
                RelationshipTypeResourceRequest.prototype.fromJsonToReleationshipType = function (json) {
                    return relationshiptype.RelationshipType.fromJson(json);
                };
                return RelationshipTypeResourceRequest;
            })(api.rest.ResourceRequest);
            relationshiptype.RelationshipTypeResourceRequest = RelationshipTypeResourceRequest;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var GetRelationshipTypeByNameRequest = (function (_super) {
                __extends(GetRelationshipTypeByNameRequest, _super);
                function GetRelationshipTypeByNameRequest(name) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.name = name;
                }
                GetRelationshipTypeByNameRequest.prototype.getParams = function () {
                    return {
                        name: this.name.toString()
                    };
                };
                GetRelationshipTypeByNameRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetRelationshipTypeByNameRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return _this.fromJsonToReleationshipType(response.getResult());
                    });
                };
                return GetRelationshipTypeByNameRequest;
            })(relationshiptype.RelationshipTypeResourceRequest);
            relationshiptype.GetRelationshipTypeByNameRequest = GetRelationshipTypeByNameRequest;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var GetRelationshipTypesByModuleRequest = (function (_super) {
                __extends(GetRelationshipTypesByModuleRequest, _super);
                function GetRelationshipTypesByModuleRequest(moduleKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.moduleKey = moduleKey;
                }
                GetRelationshipTypesByModuleRequest.prototype.getParams = function () {
                    return {
                        moduleKey: this.moduleKey.toString()
                    };
                };
                GetRelationshipTypesByModuleRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byModule");
                };
                GetRelationshipTypesByModuleRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().relationshipTypes.map(function (json) { return _this.fromJsonToReleationshipType(json); });
                    });
                };
                return GetRelationshipTypesByModuleRequest;
            })(relationshiptype.RelationshipTypeResourceRequest);
            relationshiptype.GetRelationshipTypesByModuleRequest = GetRelationshipTypesByModuleRequest;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var thumb;
    (function (thumb) {
        var Thumbnail = (function () {
            function Thumbnail(builder) {
                this.binaryReference = builder.binaryReference;
                this.mimeType = builder.mimeType;
                this.size = builder.size;
            }
            Thumbnail.prototype.getBinaryReference = function () {
                return this.binaryReference;
            };
            Thumbnail.prototype.getMimeType = function () {
                return this.mimeType;
            };
            Thumbnail.prototype.getSize = function () {
                return this.size;
            };
            Thumbnail.prototype.toJson = function () {
                return {
                    "binaryReference": this.getBinaryReference().toString(),
                    "mimeType": this.getMimeType(),
                    "size": this.getSize()
                };
            };
            Thumbnail.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Thumbnail)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.binaryReference, other.binaryReference)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.mimeType, other.mimeType)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.size, other.size)) {
                    return false;
                }
                return true;
            };
            Thumbnail.create = function () {
                return new ThumbnailBuilder();
            };
            return Thumbnail;
        })();
        thumb.Thumbnail = Thumbnail;
        var ThumbnailBuilder = (function () {
            function ThumbnailBuilder() {
            }
            ThumbnailBuilder.prototype.fromJson = function (json) {
                this.binaryReference = new api.util.BinaryReference(json.binaryReference);
                this.mimeType = json.mimeType;
                this.size = json.size;
                return this;
            };
            ThumbnailBuilder.prototype.setBinaryReference = function (value) {
                this.binaryReference = value;
                return this;
            };
            ThumbnailBuilder.prototype.setMimeType = function (value) {
                this.mimeType = value;
                return this;
            };
            ThumbnailBuilder.prototype.setSize = function (value) {
                this.size = value;
                return this;
            };
            ThumbnailBuilder.prototype.build = function () {
                return new Thumbnail(this);
            };
            return ThumbnailBuilder;
        })();
        thumb.ThumbnailBuilder = ThumbnailBuilder;
    })(thumb = api.thumb || (api.thumb = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentId = (function () {
            function ContentId(value) {
                if (!ContentId.isValidContentId(value)) {
                    throw new Error("Invalid content id: " + value);
                }
                this.value = value;
            }
            ContentId.prototype.toString = function () {
                return this.value;
            };
            ContentId.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentId)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            ContentId.isValidContentId = function (id) {
                return !api.util.StringHelper.isEmpty(id) && !api.util.StringHelper.isBlank(id);
            };
            ContentId.fromReference = function (reference) {
                return new ContentId(reference.getNodeId());
            };
            return ContentId;
        })();
        content.ContentId = ContentId;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentName = (function (_super) {
            __extends(ContentName, _super);
            function ContentName(name) {
                _super.call(this, name);
            }
            ContentName.prototype.isUnnamed = function () {
                return false;
            };
            ContentName.prototype.toUnnamed = function () {
                api.util.assert(api.ObjectHelper.iFrameSafeInstanceOf(this, content.ContentUnnamed), "this is not a ContentUnnamed");
                return this;
            };
            ContentName.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentName)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                return true;
            };
            ContentName.fromString = function (str) {
                api.util.assert(str != null, "name cannot be null");
                if (str.indexOf(ContentName.UNNAMED_PREFIX) == 0) {
                    return new content.ContentUnnamed(str);
                }
                else {
                    return new ContentName(str);
                }
            };
            ContentName.UNNAMED_PREFIX = "__unnamed__";
            return ContentName;
        })(api.Name);
        content.ContentName = ContentName;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentUnnamed = (function (_super) {
            __extends(ContentUnnamed, _super);
            function ContentUnnamed(name) {
                _super.call(this, name);
                api.util.assert(name.indexOf(content.ContentName.UNNAMED_PREFIX) == 0, "An UnnamedContent must start with [" + content.ContentName.UNNAMED_PREFIX + "]: " + name);
            }
            ContentUnnamed.prototype.isUnnamed = function () {
                return true;
            };
            ContentUnnamed.prototype.toString = function () {
                return "";
            };
            ContentUnnamed.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentUnnamed)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                return true;
            };
            ContentUnnamed.prototype.toStringIncludingHidden = function () {
                return _super.prototype.toString.call(this);
            };
            ContentUnnamed.newUnnamed = function () {
                return new ContentUnnamed(content.ContentName.UNNAMED_PREFIX);
            };
            ContentUnnamed.PRETTY_UNNAMED = "unnamed";
            return ContentUnnamed;
        })(content.ContentName);
        content.ContentUnnamed = ContentUnnamed;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentPath = (function () {
            function ContentPath(elements) {
                this.elements = elements;
                if (elements.length == 0) {
                    this.refString = ContentPath.ELEMENT_DIVIDER;
                }
                else {
                    this.refString = ContentPath.ELEMENT_DIVIDER + this.elements.join(ContentPath.ELEMENT_DIVIDER);
                }
            }
            ContentPath.fromParent = function (parent, name) {
                var elements = parent.elements;
                elements.push(name);
                return new ContentPath(elements);
            };
            ContentPath.fromString = function (path) {
                var elements;
                if (path.indexOf("/") == 0 && path.length > 1) {
                    path = path.substr(1);
                    elements = path.split(ContentPath.ELEMENT_DIVIDER);
                }
                else if (path == "/") {
                    elements = [];
                }
                return new ContentPath(elements);
            };
            ContentPath.prototype.getElements = function () {
                return this.elements;
            };
            ContentPath.prototype.getName = function () {
                return this.elements[this.elements.length - 1];
            };
            ContentPath.prototype.getLevel = function () {
                return this.elements.length;
            };
            ContentPath.prototype.hasParentContent = function () {
                return this.elements.length > 1;
            };
            ContentPath.prototype.getLastElement = function () {
                return (this.elements[this.elements.length - 1] || "");
            };
            ContentPath.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElements.push(element);
                    }
                });
                return new ContentPath(parentElements);
            };
            ContentPath.prototype.isRoot = function () {
                return this.equals(ContentPath.ROOT);
            };
            ContentPath.prototype.isNotRoot = function () {
                return !this.equals(ContentPath.ROOT);
            };
            ContentPath.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentPath)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            ContentPath.prototype.isDescendantOf = function (path) {
                return (this.refString.indexOf(path.toString()) === 0) && (this.getLevel() > path.getLevel());
            };
            ContentPath.prototype.isChildOf = function (path) {
                return (this.refString.indexOf(path.toString()) === 0) && (this.getLevel() === path.getLevel() + 1);
            };
            ContentPath.prototype.prettifyUnnamedPathElements = function () {
                var prettyElements = [];
                this.elements.forEach(function (element) {
                    if (content.ContentName.fromString(element).isUnnamed()) {
                        prettyElements.push("<" + content.ContentUnnamed.PRETTY_UNNAMED + ">");
                    }
                    else {
                        prettyElements.push(element);
                    }
                });
                return new ContentPath(prettyElements);
            };
            ContentPath.prototype.toString = function () {
                return this.refString;
            };
            ContentPath.ELEMENT_DIVIDER = "/";
            ContentPath.ROOT = ContentPath.fromString("/");
            return ContentPath;
        })();
        content.ContentPath = ContentPath;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentIdBaseItem = (function () {
            function ContentIdBaseItem(builder) {
                this.contentId = builder.contentId;
            }
            ContentIdBaseItem.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentIdBaseItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, content.ContentSummary)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.contentId, other.contentId)) {
                    return false;
                }
                return true;
            };
            ContentIdBaseItem.fromJson = function (json, propertyIdProvider) {
                return new ContentIdBaseItemBuilder().fromContentIdBaseItemJson(json).build();
            };
            ContentIdBaseItem.fromJsonArray = function (jsonArray, propertyIdProvider) {
                var array = [];
                jsonArray.forEach(function (json) {
                    array.push(ContentIdBaseItem.fromJson(json, propertyIdProvider));
                });
                return array;
            };
            return ContentIdBaseItem;
        })();
        content.ContentIdBaseItem = ContentIdBaseItem;
        var ContentIdBaseItemBuilder = (function () {
            function ContentIdBaseItemBuilder(source) {
                if (source) {
                    this.contentId = source.getContentId();
                }
            }
            ContentIdBaseItemBuilder.prototype.fromContentIdBaseItemJson = function (json) {
                this.contentId = new content.ContentId(json.id);
                return this;
            };
            ContentIdBaseItemBuilder.prototype.build = function () {
                return new ContentIdBaseItem(this);
            };
            return ContentIdBaseItemBuilder;
        })();
        content.ContentIdBaseItemBuilder = ContentIdBaseItemBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var Thumbnail = api.thumb.Thumbnail;
        var ContentState = api.schema.content.ContentState;
        var ContentSummary = (function (_super) {
            __extends(ContentSummary, _super);
            function ContentSummary(builder) {
                _super.call(this, builder);
                this.name = builder.name;
                this.displayName = builder.displayName;
                this.path = builder.path;
                this.root = builder.root;
                this.children = builder.children;
                this.type = builder.type;
                this.iconUrl = builder.iconUrl;
                this.thumbnail = builder.thumbnail;
                this.modifier = builder.modifier;
                this.owner = builder.owner;
                this.page = builder.page;
                this.valid = builder.valid;
                this.requireValid = builder.requireValid;
                this.id = builder.id;
                this.createdTime = builder.createdTime;
                this.modifiedTime = builder.modifiedTime;
                this.deletable = builder.deletable;
                this.editable = builder.editable;
                this.childOrder = builder.childOrder;
                this.language = builder.language;
                this.contentState = builder.contentState;
            }
            ContentSummary.prototype.getName = function () {
                return this.name;
            };
            ContentSummary.prototype.getDisplayName = function () {
                return this.displayName;
            };
            ContentSummary.prototype.hasParent = function () {
                return this.path.hasParentContent();
            };
            ContentSummary.prototype.getPath = function () {
                return this.path;
            };
            ContentSummary.prototype.isRoot = function () {
                return this.root;
            };
            ContentSummary.prototype.hasChildren = function () {
                return this.children;
            };
            ContentSummary.prototype.getType = function () {
                return this.type;
            };
            ContentSummary.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            ContentSummary.prototype.hasThumbnail = function () {
                return !!this.thumbnail;
            };
            ContentSummary.prototype.getThumbnail = function () {
                return this.thumbnail;
            };
            ContentSummary.prototype.getOwner = function () {
                return this.owner;
            };
            ContentSummary.prototype.getModifier = function () {
                return this.modifier;
            };
            ContentSummary.prototype.isSite = function () {
                return this.type.isSite();
            };
            ContentSummary.prototype.isPage = function () {
                return this.page;
            };
            ContentSummary.prototype.isPageTemplate = function () {
                return this.type.isPageTemplate();
            };
            ContentSummary.prototype.isImage = function () {
                return this.type.isImage();
            };
            ContentSummary.prototype.isValid = function () {
                return this.valid;
            };
            ContentSummary.prototype.isRequireValid = function () {
                return this.requireValid;
            };
            ContentSummary.prototype.getId = function () {
                return this.id;
            };
            ContentSummary.prototype.getCreatedTime = function () {
                return this.createdTime;
            };
            ContentSummary.prototype.getModifiedTime = function () {
                return this.modifiedTime;
            };
            ContentSummary.prototype.isDeletable = function () {
                return this.deletable;
            };
            ContentSummary.prototype.isEditable = function () {
                return this.editable;
            };
            ContentSummary.prototype.getChildOrder = function () {
                return this.childOrder;
            };
            ContentSummary.prototype.getLanguage = function () {
                return this.language;
            };
            ContentSummary.prototype.getContentState = function () {
                return this.contentState;
            };
            ContentSummary.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentSummary)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id, other.id)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.path, other.path)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.root, other.root)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.children, other.children)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.iconUrl, other.iconUrl)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.thumbnail, other.thumbnail)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.modifier, other.modifier)) {
                    return false;
                }
                if (!api.ObjectHelper.objectEquals(this.owner, other.owner)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.page, other.page)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.valid, other.valid)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.requireValid, other.requireValid)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.createdTime, other.createdTime)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.modifiedTime, other.modifiedTime)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.deletable, other.deletable)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.editable, other.editable)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.language, other.language)) {
                    return false;
                }
                if (!api.ObjectHelper.objectEquals(this.contentState, other.contentState)) {
                    return false;
                }
                return true;
            };
            ContentSummary.fromJson = function (json, propertyIdProvider) {
                return new ContentSummaryBuilder().fromContentSummaryJson(json).build();
            };
            ContentSummary.fromJsonArray = function (jsonArray, propertyIdProvider) {
                var array = [];
                jsonArray.forEach(function (json) {
                    array.push(ContentSummary.fromJson(json, propertyIdProvider));
                });
                return array;
            };
            return ContentSummary;
        })(content.ContentIdBaseItem);
        content.ContentSummary = ContentSummary;
        var ContentSummaryBuilder = (function (_super) {
            __extends(ContentSummaryBuilder, _super);
            function ContentSummaryBuilder(source) {
                _super.call(this, source);
                if (source) {
                    this.id = source.getId();
                    this.name = source.getName();
                    this.displayName = source.getDisplayName();
                    this.path = source.getPath();
                    this.root = source.isRoot();
                    this.children = source.hasChildren();
                    this.type = source.getType();
                    this.iconUrl = source.getIconUrl();
                    this.thumbnail = source.getThumbnail();
                    this.modifier = source.getModifier();
                    this.owner = source.getOwner();
                    this.page = source.isPage();
                    this.valid = source.isValid();
                    this.requireValid = source.isRequireValid();
                    this.createdTime = source.getCreatedTime();
                    this.modifiedTime = source.getModifiedTime();
                    this.deletable = source.isDeletable();
                    this.editable = source.isEditable();
                    this.childOrder = source.getChildOrder();
                    this.language = source.getLanguage();
                    this.contentState = source.getContentState();
                }
            }
            ContentSummaryBuilder.prototype.fromContentSummaryJson = function (json) {
                _super.prototype.fromContentIdBaseItemJson.call(this, json);
                this.name = content.ContentName.fromString(json.name);
                this.displayName = json.displayName;
                this.path = content.ContentPath.fromString(json.path);
                this.root = json.isRoot;
                this.children = json.hasChildren;
                this.type = new api.schema.content.ContentTypeName(json.type);
                this.iconUrl = json.iconUrl;
                this.thumbnail = json.thumbnail ? Thumbnail.create().fromJson(json.thumbnail).build() : null;
                this.modifier = json.modifier;
                this.owner = json.owner ? api.security.PrincipalKey.fromString(json.owner) : null;
                this.page = json.isPage;
                this.valid = json.isValid;
                this.requireValid = json.requireValid;
                this.language = json.language;
                this.id = json.id;
                this.createdTime = json.createdTime ? new Date(Date.parse(json.createdTime)) : null;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                this.deletable = json.deletable;
                this.editable = json.editable;
                this.childOrder = content.ChildOrder.fromJson(json.childOrder);
                this.contentState = ContentState.fromString(json.contentState);
                return this;
            };
            ContentSummaryBuilder.prototype.setId = function (value) {
                this.id = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setValid = function (value) {
                this.valid = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setRequireValid = function (value) {
                this.valid = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setPath = function (path) {
                this.path = path;
                return this;
            };
            ContentSummaryBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setDisplayName = function (value) {
                this.displayName = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setHasChildren = function (value) {
                this.children = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setDeletable = function (value) {
                this.deletable = value;
                return this;
            };
            ContentSummaryBuilder.prototype.build = function () {
                return new ContentSummary(this);
            };
            return ContentSummaryBuilder;
        })(content.ContentIdBaseItemBuilder);
        content.ContentSummaryBuilder = ContentSummaryBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryViewer = (function (_super) {
            __extends(ContentSummaryViewer, _super);
            function ContentSummaryViewer() {
                _super.call(this, "content-summary-viewer");
            }
            ContentSummaryViewer.prototype.resolveDisplayName = function (object) {
                var contentName = object.getName(), invalid = !object.isValid() || !object.getDisplayName() || contentName.isUnnamed(), pendingDelete = object.getContentState().isPendingDelete();
                this.toggleClass("invalid", invalid);
                this.toggleClass("pending-delete", pendingDelete);
                return object.getDisplayName();
            };
            ContentSummaryViewer.prototype.resolveUnnamedDisplayName = function (object) {
                return object.getType() ? object.getType().getLocalName() : "";
            };
            ContentSummaryViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                var contentName = object.getName();
                if (relativePath) {
                    return !contentName.isUnnamed() ? object.getName().toString() : api.ui.NamesAndIconViewer.EMPTY_SUB_NAME;
                }
                else {
                    return !contentName.isUnnamed() ? object.getPath().toString() : content.ContentPath.fromParent(object.getPath().getParentPath(), api.ui.NamesAndIconViewer.EMPTY_SUB_NAME).toString();
                }
            };
            ContentSummaryViewer.prototype.resolveSubTitle = function (object) {
                return object.getPath().toString();
            };
            ContentSummaryViewer.prototype.resolveIconUrl = function (object) {
                return new content.ContentIconUrlResolver().setContent(object).resolve();
            };
            return ContentSummaryViewer;
        })(api.ui.NamesAndIconViewer);
        content.ContentSummaryViewer = ContentSummaryViewer;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var QueryField = api.query.QueryField;
        var QueryExpr = api.query.expr.QueryExpr;
        var ContentSummaryRequest = (function (_super) {
            __extends(ContentSummaryRequest, _super);
            function ContentSummaryRequest() {
                _super.call(this);
                this.contentQuery = new content.query.ContentQuery();
                this.request = new content.ContentQueryRequest(this.contentQuery).setExpand(1 /* SUMMARY */);
            }
            ContentSummaryRequest.prototype.getRestPath = function () {
                return this.request.getRestPath();
            };
            ContentSummaryRequest.prototype.getRequestPath = function () {
                return this.request.getRequestPath();
            };
            ContentSummaryRequest.prototype.getParams = function () {
                return this.request.getParams();
            };
            ContentSummaryRequest.prototype.send = function () {
                return this.request.send();
            };
            ContentSummaryRequest.prototype.sendAndParse = function () {
                return this.request.sendAndParse().then(function (queryResult) {
                    return queryResult.getContents();
                });
            };
            ContentSummaryRequest.prototype.setAllowedContentTypes = function (contentTypes) {
                this.contentQuery.setContentTypeNames(this.createContentTypeNames(contentTypes));
            };
            ContentSummaryRequest.prototype.setSize = function (size) {
                this.contentQuery.setSize(size);
            };
            ContentSummaryRequest.prototype.setQueryExpr = function (searchString, orderList) {
                var fulltextExpression = new api.query.FulltextSearchExpressionBuilder().setSearchString(searchString).addField(new QueryField(QueryField.DISPLAY_NAME, 5)).addField(new QueryField(QueryField.NAME, 3)).addField(new QueryField(QueryField.ALL)).build();
                var queryExpr = new QueryExpr(fulltextExpression, orderList);
                this.contentQuery.setQueryExpr(queryExpr);
            };
            ContentSummaryRequest.prototype.createContentTypeNames = function (names) {
                return (names || []).map(function (name) { return new api.schema.content.ContentTypeName(name); });
            };
            return ContentSummaryRequest;
        })(api.rest.ResourceRequest);
        content.ContentSummaryRequest = ContentSummaryRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var FieldOrderExpr = api.query.expr.FieldOrderExpr;
        var OrderDirection = api.query.expr.OrderDirection;
        var FieldExpr = api.query.expr.FieldExpr;
        var ContentSummaryLoader = (function (_super) {
            __extends(ContentSummaryLoader, _super);
            function ContentSummaryLoader() {
                this.contentSummaryRequest = new content.ContentSummaryRequest();
                this.order = ContentSummaryLoader.ORDER_BY_MODIFIED_TIME_DESC;
                _super.call(this, this.contentSummaryRequest);
            }
            ContentSummaryLoader.prototype.setOrder = function (orderList) {
                this.order = orderList;
            };
            ContentSummaryLoader.prototype.setAllowedContentTypes = function (contentTypes) {
                this.contentSummaryRequest.setAllowedContentTypes(contentTypes);
            };
            ContentSummaryLoader.prototype.setSize = function (size) {
                this.contentSummaryRequest.setSize(size);
            };
            ContentSummaryLoader.prototype.search = function (searchString) {
                this.contentSummaryRequest.setQueryExpr(searchString, this.order);
                return this.load();
            };
            ContentSummaryLoader.prototype.sendRequest = function () {
                return this.contentSummaryRequest.sendAndParse();
            };
            ContentSummaryLoader.MODIFIED_TIME_DESC = new FieldOrderExpr(new FieldExpr("_modifiedTime"), 1 /* DESC */);
            ContentSummaryLoader.ORDER_BY_MODIFIED_TIME_DESC = [ContentSummaryLoader.MODIFIED_TIME_DESC];
            return ContentSummaryLoader;
        })(api.util.loader.BaseLoader);
        content.ContentSummaryLoader = ContentSummaryLoader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentIconUrlResolver = (function (_super) {
            __extends(ContentIconUrlResolver, _super);
            function ContentIconUrlResolver() {
                _super.apply(this, arguments);
            }
            ContentIconUrlResolver.prototype.setContent = function (value) {
                this.content = value;
                return this;
            };
            ContentIconUrlResolver.prototype.resolve = function () {
                var url = this.content.getIconUrl();
                if (!url) {
                    return null;
                }
                if (this.crop == undefined) {
                    this.crop = !this.content.isImage();
                }
                url = this.appendParam("crop", this.crop.toString(), url);
                return url;
            };
            ContentIconUrlResolver.default = function () {
                return api.util.UriHelper.getAdminUri("common/images/default_content.png");
            };
            return ContentIconUrlResolver;
        })(api.icon.IconUrlResolver);
        content.ContentIconUrlResolver = ContentIconUrlResolver;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentImageUrlResolver = (function (_super) {
            __extends(ContentImageUrlResolver, _super);
            function ContentImageUrlResolver() {
                _super.apply(this, arguments);
                this.size = "";
                this.ts = null;
            }
            ContentImageUrlResolver.prototype.setContentId = function (value) {
                this.contentId = value;
                return this;
            };
            ContentImageUrlResolver.prototype.setSize = function (value) {
                this.size = "" + value;
                return this;
            };
            ContentImageUrlResolver.prototype.setTimestamp = function (value) {
                this.ts = "" + value.getTime();
                return this;
            };
            ContentImageUrlResolver.prototype.resolve = function () {
                var url = "content/image/" + this.contentId.toString();
                if (this.size.length > 0) {
                    url = this.appendParam("size", this.size, url);
                }
                if (this.ts) {
                    url = this.appendParam("ts", this.ts, url);
                }
                return api.util.UriHelper.getRestUri(url);
            };
            return ContentImageUrlResolver;
        })(api.icon.IconUrlResolver);
        content.ContentImageUrlResolver = ContentImageUrlResolver;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentMetadata = (function () {
            function ContentMetadata(hits, totalHits) {
                this.hits = hits;
                this.totalHits = totalHits;
            }
            ContentMetadata.prototype.getHits = function () {
                return this.hits;
            };
            ContentMetadata.prototype.getTotalHits = function () {
                return this.totalHits;
            };
            ContentMetadata.prototype.setHits = function (hits) {
                this.hits = hits;
            };
            ContentMetadata.prototype.setTotalHits = function (totalHits) {
                this.totalHits = totalHits;
            };
            ContentMetadata.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentMetadata)) {
                    return false;
                }
                var other = o;
                if (this.hits !== other.hits || this.totalHits !== other.totalHits) {
                    return false;
                }
                return true;
            };
            return ContentMetadata;
        })();
        content.ContentMetadata = ContentMetadata;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var AccessControlList = api.security.acl.AccessControlList;
        var PropertyTree = api.data.PropertyTree;
        var Content = (function (_super) {
            __extends(Content, _super);
            function Content(builder) {
                _super.call(this, builder);
                this.metadata = [];
                api.util.assertNotNull(builder.data, "data is required for Content");
                this.data = builder.data;
                this.attachments = builder.attachments;
                this.metadata = builder.metadata || [];
                this.pageObj = builder.pageObj;
                this.permissions = builder.permissions || new AccessControlList();
                this.inheritPermissions = builder.inheritPermissions;
            }
            Content.prototype.getContentData = function () {
                return this.data;
            };
            Content.prototype.getAttachments = function () {
                return this.attachments;
            };
            Content.prototype.getMetadata = function (name) {
                return this.metadata.filter(function (item) { return item.getName().equals(name); })[0];
            };
            Content.prototype.getAllMetadata = function () {
                return this.metadata;
            };
            Content.prototype.getPageMode = function () {
                if (this.isPage()) {
                    if (this.getPage().hasTemplate()) {
                        return 1 /* FORCED_TEMPLATE */;
                    }
                    else {
                        return 2 /* FORCED_CONTROLLER */;
                    }
                }
                else {
                    return 0 /* AUTOMATIC */;
                }
            };
            Content.prototype.getPage = function () {
                return this.pageObj;
            };
            Content.prototype.getPermissions = function () {
                return this.permissions;
            };
            Content.prototype.isInheritPermissionsEnabled = function () {
                return this.inheritPermissions;
            };
            Content.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Content)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.data, other.data)) {
                    return false;
                }
                var comparator = new api.content.MetadataByMixinNameComparator();
                if (!api.ObjectHelper.arrayEquals(this.metadata.sort(comparator.compare), other.metadata.sort(comparator.compare))) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.pageObj, other.pageObj)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.permissions, other.permissions)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.attachments, other.attachments)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.inheritPermissions, other.inheritPermissions)) {
                    return false;
                }
                return true;
            };
            Content.prototype.clone = function () {
                return this.newBuilder().build();
            };
            Content.prototype.newBuilder = function () {
                return new ContentBuilder(this);
            };
            Content.fromJson = function (json, propertyIdProvider) {
                var type = new api.schema.content.ContentTypeName(json.type);
                if (type.isSite()) {
                    return new content.site.SiteBuilder().fromContentJson(json, propertyIdProvider).build();
                }
                else if (type.isPageTemplate()) {
                    return new content.page.PageTemplateBuilder().fromContentJson(json, propertyIdProvider).build();
                }
                return new ContentBuilder().fromContentJson(json, propertyIdProvider).build();
            };
            return Content;
        })(content.ContentSummary);
        content.Content = Content;
        var ContentBuilder = (function (_super) {
            __extends(ContentBuilder, _super);
            function ContentBuilder(source) {
                _super.call(this, source);
                this.inheritPermissions = true;
                if (source) {
                    this.data = source.getContentData() ? source.getContentData().copy() : null;
                    this.attachments = source.getAttachments();
                    this.metadata = source.getAllMetadata() ? source.getAllMetadata().map(function (metadata) { return metadata.clone(); }) : [];
                    this.pageObj = source.getPage() ? source.getPage().clone() : null;
                    this.permissions = source.getPermissions();
                    this.inheritPermissions = source.isInheritPermissionsEnabled();
                }
            }
            ContentBuilder.prototype.fromContentJson = function (json, propertyIdProvider) {
                var _this = this;
                _super.prototype.fromContentSummaryJson.call(this, json);
                this.data = PropertyTree.fromJson(json.data, propertyIdProvider);
                this.attachments = new api.content.attachment.AttachmentsBuilder().fromJson(json.attachments).build();
                this.metadata = [];
                json.meta.forEach(function (metadataJson) {
                    _this.metadata.push(content.Metadata.fromJson(metadataJson, propertyIdProvider));
                });
                if (this.page) {
                    this.pageObj = new api.content.page.PageBuilder().fromJson(json.page, propertyIdProvider).build();
                    this.page = true;
                }
                if (json.permissions) {
                    this.permissions = AccessControlList.fromJson(json.permissions);
                }
                if (typeof json.inheritPermissions !== "undefined") {
                    this.inheritPermissions = json.inheritPermissions;
                }
                return this;
            };
            ContentBuilder.prototype.setData = function (value) {
                this.data = value;
                return this;
            };
            ContentBuilder.prototype.setAttachments = function (value) {
                this.attachments = value;
                return this;
            };
            ContentBuilder.prototype.setPage = function (value) {
                this.pageObj = value;
                this.page = value ? true : false;
                return this;
            };
            ContentBuilder.prototype.setMetadata = function (metadata) {
                this.metadata = metadata;
                return this;
            };
            ContentBuilder.prototype.setPermissions = function (value) {
                this.permissions = value;
                return this;
            };
            ContentBuilder.prototype.setInheritPermissionsEnabled = function (value) {
                this.inheritPermissions = value;
                return this;
            };
            ContentBuilder.prototype.build = function () {
                return new Content(this);
            };
            return ContentBuilder;
        })(content.ContentSummaryBuilder);
        content.ContentBuilder = ContentBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentResponse = (function () {
            function ContentResponse(contents, metadata) {
                this.contents = contents;
                this.metadata = metadata;
            }
            ContentResponse.prototype.getContents = function () {
                return this.contents;
            };
            ContentResponse.prototype.getMetadata = function () {
                return this.metadata;
            };
            ContentResponse.prototype.setContents = function (contents) {
                this.contents = contents;
                return this;
            };
            ContentResponse.prototype.setMetadata = function (metadata) {
                this.metadata = metadata;
                return this;
            };
            return ContentResponse;
        })();
        content.ContentResponse = ContentResponse;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentNodeByDisplayNameComparator = (function () {
            function ContentNodeByDisplayNameComparator() {
            }
            ContentNodeByDisplayNameComparator.prototype.compare = function (a, b) {
                if (!a.getData().getContentSummary()) {
                    return 1;
                }
                else {
                    var firstName = a.getData().getContentSummary().getDisplayName();
                }
                if (!b.getData().getContentSummary()) {
                    return -1;
                }
                else {
                    var secondName = b.getData().getContentSummary().getDisplayName();
                }
                return firstName.localeCompare(secondName);
            };
            return ContentNodeByDisplayNameComparator;
        })();
        content.ContentNodeByDisplayNameComparator = ContentNodeByDisplayNameComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentNodeByModifiedTimeComparator = (function () {
            function ContentNodeByModifiedTimeComparator() {
            }
            ContentNodeByModifiedTimeComparator.prototype.compare = function (a, b) {
                var firstDate = !a.getData().getContentSummary() ? null : a.getData().getContentSummary().getModifiedTime(), secondDate = !b.getData().getContentSummary() ? null : b.getData().getContentSummary().getModifiedTime();
                return firstDate < secondDate ? 1 : (firstDate > secondDate) ? -1 : 0;
            };
            return ContentNodeByModifiedTimeComparator;
        })();
        content.ContentNodeByModifiedTimeComparator = ContentNodeByModifiedTimeComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentByPathComparator = (function () {
            function ContentByPathComparator() {
            }
            ContentByPathComparator.prototype.compare = function (a, b) {
                if (!a) {
                    return 1;
                }
                else {
                    var firstName = a.getPath().toString();
                }
                if (!b) {
                    return -1;
                }
                else {
                    var secondName = b.getPath().toString();
                }
                return firstName.localeCompare(secondName);
            };
            return ContentByPathComparator;
        })();
        content.ContentByPathComparator = ContentByPathComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var MoveContentSummaryLoader = (function (_super) {
            __extends(MoveContentSummaryLoader, _super);
            function MoveContentSummaryLoader() {
                this.contentByPathComparator = new _content.ContentByPathComparator();
                this.contentSummaryRequest = new _content.ContentSummaryRequest();
                _super.call(this, this.contentSummaryRequest);
            }
            MoveContentSummaryLoader.prototype.setSize = function (size) {
                this.contentSummaryRequest.setSize(size);
            };
            MoveContentSummaryLoader.prototype.setFilterContentPath = function (filterContentPath) {
                this.filterContentPath = filterContentPath;
            };
            MoveContentSummaryLoader.prototype.search = function (searchString) {
                this.contentSummaryRequest.setQueryExpr(searchString);
                return this.load();
            };
            MoveContentSummaryLoader.prototype.load = function () {
                var _this = this;
                this.notifyLoadingData();
                return this.sendRequest().then(function (contents) {
                    contents = _this.filterContent(contents);
                    if (contents && contents.length > 0) {
                        contents.sort(_this.contentByPathComparator.compare);
                        _this.notifyLoadedData(contents);
                    }
                    if (_this.preservedSearchString) {
                        _this.search(_this.preservedSearchString);
                        _this.preservedSearchString = null;
                    }
                    return contents;
                });
            };
            MoveContentSummaryLoader.prototype.filterContent = function (contents) {
                var _this = this;
                if (!!this.filterContentPath) {
                    return contents.filter(function (content) { return (!content.getPath().isDescendantOf(_this.filterContentPath)) && !_this.filterContentPath.isChildOf(content.getPath()) && !_this.filterContentPath.equals(content.getPath()); });
                }
                return [];
            };
            MoveContentSummaryLoader.prototype.sendRequest = function () {
                return this.contentSummaryRequest.sendAndParse();
            };
            return MoveContentSummaryLoader;
        })(api.util.loader.BaseLoader);
        _content.MoveContentSummaryLoader = MoveContentSummaryLoader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var MixinName = api.schema.mixin.MixinName;
        var PropertyTree = api.data.PropertyTree;
        var Metadata = (function () {
            function Metadata(name, data) {
                this.name = name;
                this.data = data;
            }
            Metadata.prototype.getName = function () {
                return this.name;
            };
            Metadata.prototype.getData = function () {
                return this.data;
            };
            Metadata.prototype.clone = function () {
                return new Metadata(this.name, this.data.copy());
            };
            Metadata.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Metadata)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.data, other.data)) {
                    return false;
                }
                return true;
            };
            Metadata.prototype.toJson = function () {
                return {
                    name: this.name.toString(),
                    data: this.data.toJson()
                };
            };
            Metadata.fromJson = function (metadataJson, propertyIdProvider) {
                return new Metadata(new MixinName(metadataJson.name), PropertyTree.fromJson(metadataJson.data, propertyIdProvider));
            };
            return Metadata;
        })();
        content.Metadata = Metadata;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var MetadataByMixinNameComparator = (function () {
            function MetadataByMixinNameComparator() {
            }
            MetadataByMixinNameComparator.prototype.compare = function (a, b) {
                if (!a.getName()) {
                    return 1;
                }
                else {
                    var firstName = a.getName().getLocalName();
                }
                if (!b.getName()) {
                    return -1;
                }
                else {
                    var secondName = b.getName().getLocalName();
                }
                return firstName.localeCompare(secondName);
            };
            return MetadataByMixinNameComparator;
        })();
        content.MetadataByMixinNameComparator = MetadataByMixinNameComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var TreeNodesOfContentPath = (function () {
            function TreeNodesOfContentPath(path, altPath) {
                this.path = path;
                this.altPath = altPath;
                this.nodes = [];
            }
            TreeNodesOfContentPath.prototype.getPath = function () {
                return this.path;
            };
            TreeNodesOfContentPath.prototype.getAltPath = function () {
                return this.altPath;
            };
            TreeNodesOfContentPath.prototype.getNodes = function () {
                return this.nodes;
            };
            TreeNodesOfContentPath.prototype.hasNodes = function () {
                return this.nodes.length > 0;
            };
            TreeNodesOfContentPath.prototype.getId = function () {
                return (this.hasNodes() && this.nodes[0].getData()) ? this.nodes[0].getData().getId() : "";
            };
            TreeNodesOfContentPath.prototype.updateNodeData = function (data) {
                this.nodes.forEach(function (node) {
                    node.setData(data);
                    node.clearViewers();
                });
            };
            return TreeNodesOfContentPath;
        })();
        content.TreeNodesOfContentPath = TreeNodesOfContentPath;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var TreeNodeParentOfContent = (function () {
            function TreeNodeParentOfContent(data, node) {
                this.data = data;
                this.node = node;
            }
            TreeNodeParentOfContent.prototype.getData = function () {
                return this.data;
            };
            TreeNodeParentOfContent.prototype.getNode = function () {
                return this.node;
            };
            return TreeNodeParentOfContent;
        })();
        content.TreeNodeParentOfContent = TreeNodeParentOfContent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (ContentServerChangeType) {
            ContentServerChangeType[ContentServerChangeType["UNKNOWN"] = 0] = "UNKNOWN";
            ContentServerChangeType[ContentServerChangeType["PUBLISH"] = 1] = "PUBLISH";
            ContentServerChangeType[ContentServerChangeType["DUPLICATE"] = 2] = "DUPLICATE";
            ContentServerChangeType[ContentServerChangeType["CREATE"] = 3] = "CREATE";
            ContentServerChangeType[ContentServerChangeType["UPDATE"] = 4] = "UPDATE";
            ContentServerChangeType[ContentServerChangeType["DELETE"] = 5] = "DELETE";
            ContentServerChangeType[ContentServerChangeType["PENDING"] = 6] = "PENDING";
            ContentServerChangeType[ContentServerChangeType["RENAME"] = 7] = "RENAME";
        })(content.ContentServerChangeType || (content.ContentServerChangeType = {}));
        var ContentServerChangeType = content.ContentServerChangeType;
        var ContentServerChange = (function () {
            function ContentServerChange(contentPaths, type) {
                this.contentPaths = contentPaths;
                this.type = type;
            }
            ContentServerChange.prototype.getContentPaths = function () {
                return this.contentPaths;
            };
            ContentServerChange.prototype.getChangeType = function () {
                return this.type;
            };
            ContentServerChange.prototype.toString = function () {
                return ContentServerChangeType[this.type] + ": <" + this.contentPaths.map(function (contentPath) { return contentPath.toString(); }).join(", ") + ">";
            };
            ContentServerChange.fromJson = function (json) {
                var contentEventType;
                switch (json.t) {
                    case 'P':
                        contentEventType = 1 /* PUBLISH */;
                        break;
                    case 'C':
                        contentEventType = 3 /* CREATE */;
                        break;
                    case 'U':
                        contentEventType = 4 /* UPDATE */;
                        break;
                    case 'X':
                        contentEventType = 5 /* DELETE */;
                        break;
                    case 'D':
                        contentEventType = 2 /* DUPLICATE */;
                        break;
                    case 'A':
                        contentEventType = 6 /* PENDING */;
                        break;
                    case 'R':
                        contentEventType = 7 /* RENAME */;
                        break;
                    default:
                        contentEventType = 0 /* UNKNOWN */;
                }
                var contentPaths = json.p.map(function (contentPath) { return api.content.ContentPath.fromString(contentPath); });
                return new ContentServerChange(contentPaths, contentEventType);
            };
            return ContentServerChange;
        })();
        content.ContentServerChange = ContentServerChange;
        var ContentServerEvent = (function (_super) {
            __extends(ContentServerEvent, _super);
            function ContentServerEvent(changes) {
                _super.call(this);
                this.changes = changes || [];
            }
            ContentServerEvent.prototype.getContentChanges = function () {
                return this.changes;
            };
            ContentServerEvent.prototype.toString = function () {
                return "ContentServerEvent: [" + this.changes.map(function (change) { return change.toString(); }).join(", ") + "]";
            };
            ContentServerEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentServerEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentServerEvent.fromJson = function (json) {
                var changes = json.changes.map(function (changeJson) { return ContentServerChange.fromJson(changeJson); });
                return new ContentServerEvent(changes);
            };
            return ContentServerEvent;
        })(api.event.Event);
        content.ContentServerEvent = ContentServerEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentChangeResult = (function () {
            function ContentChangeResult(type, result) {
                this.type = type;
                this.result = result;
            }
            ContentChangeResult.prototype.getResult = function () {
                return this.result;
            };
            ContentChangeResult.prototype.getChangeType = function () {
                return this.type;
            };
            return ContentChangeResult;
        })();
        content.ContentChangeResult = ContentChangeResult;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentResourceRequest = (function (_super) {
            __extends(ContentResourceRequest, _super);
            function ContentResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content");
                this.propertyIdProvider = api.Client.get().getPropertyIdProvider();
            }
            ContentResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            ContentResourceRequest.prototype.fromJsonToContent = function (json) {
                return content.Content.fromJson(json, this.propertyIdProvider);
            };
            ContentResourceRequest.prototype.fromJsonToContentArray = function (json) {
                var _this = this;
                var array = [];
                json.forEach(function (itemJson) {
                    array.push(_this.fromJsonToContent(itemJson));
                });
                return array;
            };
            ContentResourceRequest.prototype.fromJsonToContentSummary = function (json) {
                return content.ContentSummary.fromJson(json, this.propertyIdProvider);
            };
            ContentResourceRequest.prototype.fromJsonToContentSummaryArray = function (json) {
                var _this = this;
                var array = [];
                json.forEach(function (itemJson) {
                    array.push(_this.fromJsonToContentSummary(itemJson));
                });
                return array;
            };
            ContentResourceRequest.prototype.fromJsonToContentIdBaseItem = function (json) {
                return content.ContentIdBaseItem.fromJson(json, this.propertyIdProvider);
            };
            ContentResourceRequest.prototype.fromJsonToContentIdBaseItemArray = function (jsonArray) {
                return content.ContentIdBaseItem.fromJsonArray(jsonArray, this.propertyIdProvider);
            };
            ContentResourceRequest.EXPAND_NONE = 'none';
            ContentResourceRequest.EXPAND_SUMMARY = 'summary';
            ContentResourceRequest.EXPAND_FULL = 'full';
            return ContentResourceRequest;
        })(api.rest.ResourceRequest);
        content.ContentResourceRequest = ContentResourceRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ApplyContentPermissionsRequest = (function (_super) {
            __extends(ApplyContentPermissionsRequest, _super);
            function ApplyContentPermissionsRequest() {
                _super.call(this);
                this.inheritPermissions = true;
                this.overwriteChildPermissions = false;
                this.setMethod("POST");
            }
            ApplyContentPermissionsRequest.prototype.setId = function (id) {
                this.id = id;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.setInheritPermissions = function (inheritPermissions) {
                this.inheritPermissions = inheritPermissions;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.setOverwriteChildPermissions = function (overwriteChildPermissions) {
                this.overwriteChildPermissions = overwriteChildPermissions;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.getParams = function () {
                return {
                    contentId: this.id,
                    permissions: this.permissions ? this.permissions.toJson() : undefined,
                    inheritPermissions: this.inheritPermissions,
                    overwriteChildPermissions: this.overwriteChildPermissions
                };
            };
            ApplyContentPermissionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "applyPermissions");
            };
            ApplyContentPermissionsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return ApplyContentPermissionsRequest;
        })(content.ContentResourceRequest);
        content.ApplyContentPermissionsRequest = ApplyContentPermissionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentByIdRequest = (function (_super) {
            __extends(GetContentByIdRequest, _super);
            function GetContentByIdRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.id = id;
            }
            GetContentByIdRequest.prototype.setExpand = function (expand) {
                this.expand = expand;
                return this;
            };
            GetContentByIdRequest.prototype.getParams = function () {
                return {
                    id: this.id.toString(),
                    expand: this.expand
                };
            };
            GetContentByIdRequest.prototype.getRequestPath = function () {
                return _super.prototype.getResourcePath.call(this);
            };
            GetContentByIdRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return GetContentByIdRequest;
        })(content.ContentResourceRequest);
        content.GetContentByIdRequest = GetContentByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentSummaryByIdRequest = (function (_super) {
            __extends(GetContentSummaryByIdRequest, _super);
            function GetContentSummaryByIdRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.id = id;
                this.expand = content.ContentResourceRequest.EXPAND_SUMMARY;
            }
            GetContentSummaryByIdRequest.prototype.getParams = function () {
                return {
                    id: this.id.toString(),
                    expand: this.expand
                };
            };
            GetContentSummaryByIdRequest.prototype.getRequestPath = function () {
                return _super.prototype.getResourcePath.call(this);
            };
            GetContentSummaryByIdRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContentSummary(response.getResult());
                });
            };
            return GetContentSummaryByIdRequest;
        })(content.ContentResourceRequest);
        content.GetContentSummaryByIdRequest = GetContentSummaryByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentSummaryByIds = (function () {
            function GetContentSummaryByIds(ids) {
                this.ids = ids;
            }
            GetContentSummaryByIds.prototype.get = function () {
                var deferred = wemQ.defer();
                var allPromises = (this.ids || []).map(function (contentId) {
                    return new api.content.GetContentSummaryByIdRequest(contentId).sendAndParse();
                });
                var settledPromises = wemQ.allSettled(allPromises);
                settledPromises.spread(function () {
                    var contentResults = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        contentResults[_i - 0] = arguments[_i];
                    }
                    if (!contentResults) {
                        deferred.resolve([]);
                        return;
                    }
                    var contents = contentResults.filter(function (contentResult) { return contentResult.state === 'fulfilled'; }).map(function (contentResult) { return contentResult.value; });
                    deferred.resolve(contents);
                }).done();
                return deferred.promise;
            };
            return GetContentSummaryByIds;
        })();
        content.GetContentSummaryByIds = GetContentSummaryByIds;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentByPathRequest = (function (_super) {
            __extends(GetContentByPathRequest, _super);
            function GetContentByPathRequest(path) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.contentPath = path;
            }
            GetContentByPathRequest.prototype.getParams = function () {
                return {
                    path: this.contentPath.toString()
                };
            };
            GetContentByPathRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "bypath");
            };
            GetContentByPathRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return GetContentByPathRequest;
        })(content.ContentResourceRequest);
        content.GetContentByPathRequest = GetContentByPathRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var AccessControlList = api.security.acl.AccessControlList;
        var GetContentRootPermissionsRequest = (function (_super) {
            __extends(GetContentRootPermissionsRequest, _super);
            function GetContentRootPermissionsRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            GetContentRootPermissionsRequest.prototype.getParams = function () {
                return {};
            };
            GetContentRootPermissionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "rootPermissions");
            };
            GetContentRootPermissionsRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return AccessControlList.fromJson(response.getResult().permissions);
                });
            };
            return GetContentRootPermissionsRequest;
        })(content.ContentResourceRequest);
        content.GetContentRootPermissionsRequest = GetContentRootPermissionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CreateContentRequest = (function (_super) {
            __extends(CreateContentRequest, _super);
            function CreateContentRequest() {
                _super.call(this);
                this.meta = [];
                this.valid = false;
                this.requireValid = false;
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateContentRequest.prototype.setValid = function (value) {
                this.valid = value;
                return this;
            };
            CreateContentRequest.prototype.setRequireValid = function (value) {
                this.requireValid = value;
                return this;
            };
            CreateContentRequest.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            CreateContentRequest.prototype.setParent = function (value) {
                this.parent = value;
                return this;
            };
            CreateContentRequest.prototype.setContentType = function (value) {
                this.contentType = value;
                return this;
            };
            CreateContentRequest.prototype.setData = function (data) {
                this.data = data;
                return this;
            };
            CreateContentRequest.prototype.setMetadata = function (metadata) {
                this.meta = metadata;
                return this;
            };
            CreateContentRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateContentRequest.prototype.getParams = function () {
                return {
                    valid: this.valid,
                    requireValid: this.requireValid,
                    name: this.name.isUnnamed() ? this.name.toUnnamed().toStringIncludingHidden() : this.name.toString(),
                    parent: this.parent.toString(),
                    contentType: this.contentType.toString(),
                    data: this.data.toJson(),
                    meta: this.metadataToJson(),
                    displayName: this.displayName
                };
            };
            CreateContentRequest.prototype.metadataToJson = function () {
                return this.meta ? this.meta.map(function (metadata) { return metadata.toJson(); }) : null;
            };
            CreateContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "create");
            };
            CreateContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return CreateContentRequest;
        })(content.ContentResourceRequest);
        content.CreateContentRequest = CreateContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderContentRequest = (function (_super) {
            __extends(OrderContentRequest, _super);
            function OrderContentRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            OrderContentRequest.prototype.setContentId = function (value) {
                this.contentId = value;
                return this;
            };
            OrderContentRequest.prototype.setChildOrder = function (value) {
                this.childOrder = value;
                return this;
            };
            OrderContentRequest.prototype.getParams = function () {
                return this.contentToJson();
            };
            OrderContentRequest.prototype.contentToJson = function () {
                return content.ChildOrder.toSetChildOrderJson(this.contentId, this.childOrder);
            };
            OrderContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "setChildOrder");
            };
            OrderContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return OrderContentRequest;
        })(content.ContentResourceRequest);
        content.OrderContentRequest = OrderContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderChildContentRequest = (function (_super) {
            __extends(OrderChildContentRequest, _super);
            function OrderChildContentRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            OrderChildContentRequest.prototype.setContentMovements = function (value) {
                this.contentMovements = value;
                return this;
            };
            OrderChildContentRequest.prototype.getParams = function () {
                return this.contentMovements.toJson();
            };
            OrderChildContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "reorderChildren");
            };
            OrderChildContentRequest.prototype.sendAndParse = function () {
                return this.send();
            };
            return OrderChildContentRequest;
        })(content.ContentResourceRequest);
        content.OrderChildContentRequest = OrderChildContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var UpdateContentRequest = (function (_super) {
            __extends(UpdateContentRequest, _super);
            function UpdateContentRequest(id) {
                _super.call(this);
                this.id = id;
                this.valid = false;
                this.requireValid = false;
                this.setMethod("POST");
            }
            UpdateContentRequest.prototype.setThumbnail = function (thumbnail) {
                this.thumbnail = thumbnail;
            };
            UpdateContentRequest.prototype.setId = function (id) {
                this.id = id;
                return this;
            };
            UpdateContentRequest.prototype.setContentName = function (value) {
                this.name = value;
                return this;
            };
            UpdateContentRequest.prototype.setData = function (contentData) {
                this.data = contentData;
                return this;
            };
            UpdateContentRequest.prototype.setMetadata = function (metadata) {
                this.meta = metadata;
                return this;
            };
            UpdateContentRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateContentRequest.prototype.setValid = function (valid) {
                this.valid = valid;
                return this;
            };
            UpdateContentRequest.prototype.setRequireValid = function (requireValid) {
                this.requireValid = requireValid;
                return this;
            };
            UpdateContentRequest.prototype.setLanguage = function (language) {
                this.language = language;
                return this;
            };
            UpdateContentRequest.prototype.setOwner = function (owner) {
                this.owner = owner;
                return this;
            };
            UpdateContentRequest.prototype.getParams = function () {
                return {
                    contentId: this.id,
                    valid: this.valid,
                    requireValid: this.requireValid,
                    contentName: this.name.isUnnamed() ? this.name.toUnnamed().toStringIncludingHidden() : this.name.toString(),
                    data: this.data.toJson(),
                    meta: (this.meta || []).map(function (metadata) { return metadata.toJson(); }),
                    displayName: this.displayName,
                    thumbnail: this.thumbnail ? this.thumbnail.toJson() : undefined,
                    language: this.language,
                    owner: this.owner ? this.owner.toString() : undefined
                };
            };
            UpdateContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "update");
            };
            UpdateContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return UpdateContentRequest;
        })(content.ContentResourceRequest);
        content.UpdateContentRequest = UpdateContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DuplicateContentRequest = (function (_super) {
            __extends(DuplicateContentRequest, _super);
            function DuplicateContentRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.id = id;
            }
            DuplicateContentRequest.prototype.getParams = function () {
                return {
                    contentId: this.id.toString()
                };
            };
            DuplicateContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "duplicate");
            };
            DuplicateContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return DuplicateContentRequest;
        })(content.ContentResourceRequest);
        content.DuplicateContentRequest = DuplicateContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var MoveContentRequest = (function (_super) {
            __extends(MoveContentRequest, _super);
            function MoveContentRequest(id, parentPath) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.id = id;
                this.parentPath = parentPath;
            }
            MoveContentRequest.prototype.getParams = function () {
                return {
                    contentId: this.id.toString(),
                    parentContentPath: !!this.parentPath ? this.parentPath.toString() : ""
                };
            };
            MoveContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "move");
            };
            MoveContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return MoveContentRequest;
        })(content.ContentResourceRequest);
        content.MoveContentRequest = MoveContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ListContentByIdRequest = (function (_super) {
            __extends(ListContentByIdRequest, _super);
            function ListContentByIdRequest(parentId) {
                _super.call(this);
                this.expand = 1 /* SUMMARY */;
                _super.prototype.setMethod.call(this, "GET");
                this.parentId = parentId;
            }
            ListContentByIdRequest.prototype.setExpand = function (value) {
                this.expand = value;
                return this;
            };
            ListContentByIdRequest.prototype.setFrom = function (value) {
                this.from = value;
                return this;
            };
            ListContentByIdRequest.prototype.setSize = function (value) {
                this.size = value;
                return this;
            };
            ListContentByIdRequest.prototype.setOrder = function (value) {
                this.order = value;
                return this;
            };
            ListContentByIdRequest.prototype.getParams = function () {
                return {
                    parentId: this.parentId ? this.parentId.toString() : null,
                    expand: this.expand,
                    from: this.from,
                    size: this.size,
                    childOrder: !!this.order ? this.order.toString() : ""
                };
            };
            ListContentByIdRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list");
            };
            ListContentByIdRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return new content.ContentResponse(content.ContentSummary.fromJsonArray(response.getResult().contents, _this.propertyIdProvider), new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]));
                });
            };
            return ListContentByIdRequest;
        })(content.ContentResourceRequest);
        content.ListContentByIdRequest = ListContentByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ListContentByPathRequest = (function (_super) {
            __extends(ListContentByPathRequest, _super);
            function ListContentByPathRequest(parentPath) {
                _super.call(this);
                this.expand = 1 /* SUMMARY */;
                _super.prototype.setMethod.call(this, "GET");
                this.parentPath = parentPath;
            }
            ListContentByPathRequest.prototype.setExpand = function (value) {
                this.expand = value;
                return this;
            };
            ListContentByPathRequest.prototype.setFrom = function (value) {
                this.from = value;
                return this;
            };
            ListContentByPathRequest.prototype.setSize = function (value) {
                this.size = value;
                return this;
            };
            ListContentByPathRequest.prototype.getParams = function () {
                return {
                    parentPath: this.parentPath.toString(),
                    expand: this.expand,
                    from: this.from,
                    size: this.size
                };
            };
            ListContentByPathRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "bypath");
            };
            return ListContentByPathRequest;
        })(content.ContentResourceRequest);
        content.ListContentByPathRequest = ListContentByPathRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DeleteContentRequest = (function (_super) {
            __extends(DeleteContentRequest, _super);
            function DeleteContentRequest(contentPath) {
                _super.call(this);
                this.contentPaths = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentPath) {
                    this.addContentPath(contentPath);
                }
            }
            DeleteContentRequest.prototype.setContentPaths = function (contentPaths) {
                this.contentPaths = contentPaths;
                return this;
            };
            DeleteContentRequest.prototype.addContentPath = function (contentPath) {
                this.contentPaths.push(contentPath);
                return this;
            };
            DeleteContentRequest.prototype.getParams = function () {
                var fn = function (contentPath) {
                    return contentPath.toString();
                };
                return {
                    contentPaths: this.contentPaths.map(fn)
                };
            };
            DeleteContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "delete");
            };
            DeleteContentRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return content.DeleteContentResult.fromJson(response.getResult());
                });
            };
            return DeleteContentRequest;
        })(content.ContentResourceRequest);
        content.DeleteContentRequest = DeleteContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DeleteContentResult = (function () {
            function DeleteContentResult(success, failures) {
                this.deleteSuccess = success;
                this.deleteFailures = failures;
            }
            DeleteContentResult.prototype.getDeleted = function () {
                return this.deleteSuccess;
            };
            DeleteContentResult.prototype.getDeleteFailures = function () {
                return this.deleteFailures;
            };
            DeleteContentResult.fromJson = function (json) {
                var success = json.successes.map(function (success) { return content.ContentPath.fromString(success.path); });
                var failure = json.failures.map(function (success) { return new DeleteContentResultFailure(content.ContentPath.fromString(success.path), success.reason); });
                return new DeleteContentResult(success, failure);
            };
            return DeleteContentResult;
        })();
        content.DeleteContentResult = DeleteContentResult;
        var DeleteContentResultFailure = (function () {
            function DeleteContentResultFailure(path, reason) {
                this.path = path;
                this.reason = reason;
            }
            DeleteContentResultFailure.prototype.getPath = function () {
                return this.path;
            };
            DeleteContentResultFailure.prototype.getReason = function () {
                return this.reason;
            };
            return DeleteContentResultFailure;
        })();
        content.DeleteContentResultFailure = DeleteContentResultFailure;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var BatchContentRequest = (function (_super) {
            __extends(BatchContentRequest, _super);
            function BatchContentRequest(contentPath) {
                _super.call(this);
                this.contentPaths = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentPath) {
                    this.addContentPath(contentPath);
                }
            }
            BatchContentRequest.prototype.setContentPaths = function (contentPaths) {
                this.contentPaths = contentPaths;
                return this;
            };
            BatchContentRequest.prototype.addContentPath = function (contentPath) {
                this.contentPaths.push(contentPath);
                return this;
            };
            BatchContentRequest.prototype.getParams = function () {
                var fn = function (contentPath) {
                    return contentPath.toString();
                };
                return {
                    contentPaths: this.contentPaths.map(fn)
                };
            };
            BatchContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "batch");
            };
            BatchContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return new content.ContentResponse(content.ContentSummary.fromJsonArray(response.getResult().contents, _this.propertyIdProvider), new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]));
                });
            };
            return BatchContentRequest;
        })(content.ContentResourceRequest);
        content.BatchContentRequest = BatchContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentCreatedEvent = (function (_super) {
            __extends(ContentCreatedEvent, _super);
            function ContentCreatedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentCreatedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentCreatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentCreatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentCreatedEvent.fromJson = function (json) {
                return new ContentCreatedEvent(new content.ContentId(json.contentId));
            };
            return ContentCreatedEvent;
        })(api.event.Event);
        content.ContentCreatedEvent = ContentCreatedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentUpdatedEvent = (function (_super) {
            __extends(ContentUpdatedEvent, _super);
            function ContentUpdatedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentUpdatedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentUpdatedEvent.fromJson = function (json) {
                return new ContentUpdatedEvent(new content.ContentId(json.contentId));
            };
            return ContentUpdatedEvent;
        })(api.event.Event);
        content.ContentUpdatedEvent = ContentUpdatedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentPublishedEvent = (function (_super) {
            __extends(ContentPublishedEvent, _super);
            function ContentPublishedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentPublishedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentPublishedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentPublishedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentPublishedEvent.fromJson = function (json) {
                return new ContentPublishedEvent(new content.ContentId(json.contentId));
            };
            return ContentPublishedEvent;
        })(api.event.Event);
        content.ContentPublishedEvent = ContentPublishedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentDeletedEvent = (function (_super) {
            __extends(ContentDeletedEvent, _super);
            function ContentDeletedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentDeletedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentDeletedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentDeletedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentDeletedEvent.fromJson = function (json) {
                return new ContentDeletedEvent(new content.ContentId(json.contentId));
            };
            return ContentDeletedEvent;
        })(api.event.Event);
        content.ContentDeletedEvent = ContentDeletedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentChildOrderUpdatedEvent = (function (_super) {
            __extends(ContentChildOrderUpdatedEvent, _super);
            function ContentChildOrderUpdatedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentChildOrderUpdatedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentChildOrderUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentChildOrderUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentChildOrderUpdatedEvent;
        })(api.event.Event);
        content.ContentChildOrderUpdatedEvent = ContentChildOrderUpdatedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var SelectedOption = api.ui.selector.combobox.SelectedOption;
        var RichComboBox = api.ui.selector.combobox.RichComboBox;
        var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
        var ContentComboBox = (function (_super) {
            __extends(ContentComboBox, _super);
            function ContentComboBox(builder) {
                var loader = builder.loader ? builder.loader : new _content.ContentSummaryLoader();
                loader.setAllowedContentTypes(builder.allowedContentTypes);
                var richComboBoxBuilder = new RichComboBoxBuilder().setComboBoxName(builder.name ? builder.name : 'contentSelector').setLoader(loader).setSelectedOptionsView(new ContentSelectedOptionsView()).setMaximumOccurrences(builder.maximumOccurrences).setOptionDisplayValueViewer(new api.content.ContentSummaryViewer()).setDelayedInputValueChangedHandling(750).setMinWidth(builder.minWidth);
                _super.call(this, richComboBoxBuilder);
            }
            ContentComboBox.prototype.getContent = function (contentId) {
                var option = this.getOptionByValue(contentId.toString());
                if (option) {
                    return option.displayValue;
                }
                return null;
            };
            ContentComboBox.prototype.setContent = function (content) {
                this.clearSelection();
                if (content) {
                    var optionToSelect = this.getOptionByValue(content.getContentId().toString());
                    if (!optionToSelect) {
                        optionToSelect = {
                            value: content.getContentId().toString(),
                            displayValue: content
                        };
                        this.addOption(optionToSelect);
                    }
                    this.selectOption(optionToSelect);
                }
            };
            ContentComboBox.create = function () {
                return new ContentComboBoxBuilder();
            };
            return ContentComboBox;
        })(RichComboBox);
        _content.ContentComboBox = ContentComboBox;
        var ContentSelectedOptionsView = (function (_super) {
            __extends(ContentSelectedOptionsView, _super);
            function ContentSelectedOptionsView() {
                _super.apply(this, arguments);
            }
            ContentSelectedOptionsView.prototype.createSelectedOption = function (option) {
                var optionView = new ContentSelectedOptionView(option);
                return new SelectedOption(optionView, this.count());
            };
            return ContentSelectedOptionsView;
        })(api.ui.selector.combobox.BaseSelectedOptionsView);
        _content.ContentSelectedOptionsView = ContentSelectedOptionsView;
        var ContentSelectedOptionView = (function (_super) {
            __extends(ContentSelectedOptionView, _super);
            function ContentSelectedOptionView(option) {
                _super.call(this, option);
            }
            ContentSelectedOptionView.prototype.resolveIconUrl = function (content) {
                return content.getIconUrl();
            };
            ContentSelectedOptionView.prototype.resolveTitle = function (content) {
                return content.getDisplayName().toString();
            };
            ContentSelectedOptionView.prototype.resolveSubTitle = function (content) {
                return content.getPath().toString();
            };
            return ContentSelectedOptionView;
        })(api.ui.selector.combobox.RichSelectedOptionView);
        _content.ContentSelectedOptionView = ContentSelectedOptionView;
        var ContentComboBoxBuilder = (function () {
            function ContentComboBoxBuilder() {
                this.maximumOccurrences = 0;
            }
            ContentComboBoxBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            ContentComboBoxBuilder.prototype.setMaximumOccurrences = function (maximumOccurrences) {
                this.maximumOccurrences = maximumOccurrences;
                return this;
            };
            ContentComboBoxBuilder.prototype.setLoader = function (loader) {
                this.loader = loader;
                return this;
            };
            ContentComboBoxBuilder.prototype.setAllowedContentTypes = function (allowedTypes) {
                this.allowedContentTypes = allowedTypes;
                return this;
            };
            ContentComboBoxBuilder.prototype.setMinWidth = function (value) {
                this.minWidth = value;
                return this;
            };
            ContentComboBoxBuilder.prototype.build = function () {
                return new ContentComboBox(this);
            };
            return ContentComboBoxBuilder;
        })();
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentMovedEvent = (function (_super) {
            __extends(ContentMovedEvent, _super);
            function ContentMovedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentMovedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentMovedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentMovedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentMovedEvent;
        })(api.event.Event);
        content.ContentMovedEvent = ContentMovedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentMoveComboBox = (function (_super) {
            __extends(ContentMoveComboBox, _super);
            function ContentMoveComboBox() {
                this.contentLoader = new content.MoveContentSummaryLoader();
                this.contentLoader.setSize(-1);
                var richComboBoxBuilder = new api.ui.selector.combobox.RichComboBoxBuilder();
                richComboBoxBuilder.setMaximumOccurrences(1).setComboBoxName("contentSelector").setLoader(this.contentLoader).setSelectedOptionsView(new content.ContentSelectedOptionsView()).setOptionDisplayValueViewer(new api.content.ContentSummaryViewer()).setDelayedInputValueChangedHandling(500);
                _super.call(this, richComboBoxBuilder);
            }
            ContentMoveComboBox.prototype.setFilterContentPath = function (contentPath) {
                this.contentLoader.setFilterContentPath(contentPath);
            };
            return ContentMoveComboBox;
        })(api.ui.selector.combobox.RichComboBox);
        content.ContentMoveComboBox = ContentMoveComboBox;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentQueryRequest = (function (_super) {
            __extends(ContentQueryRequest, _super);
            function ContentQueryRequest(contentQuery) {
                _super.call(this);
                this.expand = 1 /* SUMMARY */;
                _super.prototype.setMethod.call(this, "POST");
                this.contentQuery = contentQuery;
            }
            ContentQueryRequest.prototype.setContentQuery = function (contentQuery) {
                this.contentQuery = contentQuery;
            };
            ContentQueryRequest.prototype.setExpand = function (expand) {
                this.expand = expand;
                return this;
            };
            ContentQueryRequest.prototype.getParams = function () {
                var queryExprAsString = this.contentQuery.getQueryExpr() ? this.contentQuery.getQueryExpr().toString() : "";
                return {
                    queryExpr: queryExprAsString,
                    from: this.contentQuery.getFrom(),
                    size: this.contentQuery.getSize(),
                    contentTypeNames: this.contentTypeNamesAsString(this.contentQuery.getContentTypes()),
                    expand: this.expandAsString(),
                    aggregationQueries: this.aggregationQueriesToJson(this.contentQuery.getAggregationQueries()),
                    queryFilters: this.queryFiltersToJson(this.contentQuery.getQueryFilters())
                };
            };
            ContentQueryRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    var responseResult = response.getResult();
                    var aggregations = api.aggregation.Aggregation.fromJsonArray(responseResult.aggregations);
                    var contentsAsJson = responseResult.contents;
                    var contentQueryResult;
                    var metadata = new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]);
                    if (_this.expand == 0 /* NONE */) {
                        var contentIdBaseItems = _this.fromJsonToContentIdBaseItemArray(contentsAsJson);
                        contentQueryResult = new content.ContentQueryResult(contentIdBaseItems, aggregations, contentsAsJson, metadata);
                    }
                    else if (_this.expand == 1 /* SUMMARY */) {
                        var contentSummaries = _this.fromJsonToContentSummaryArray(contentsAsJson);
                        contentQueryResult = new content.ContentQueryResult(contentSummaries, aggregations, contentsAsJson, metadata);
                    }
                    else {
                        var contents = _this.fromJsonToContentArray(contentsAsJson);
                        contentQueryResult = new content.ContentQueryResult(contents, aggregations, contentsAsJson, metadata);
                    }
                    return contentQueryResult;
                });
            };
            ContentQueryRequest.prototype.aggregationQueriesToJson = function (aggregationQueries) {
                var aggregationQueryJsons = [];
                if (aggregationQueries == null) {
                    return aggregationQueryJsons;
                }
                aggregationQueries.forEach(function (aggregationQuery) {
                    aggregationQueryJsons.push(aggregationQuery.toJson());
                });
                return aggregationQueryJsons;
            };
            ContentQueryRequest.prototype.queryFiltersToJson = function (queryFilters) {
                var queryFilterJsons = [];
                if (queryFilters == null || queryFilters.length == 0) {
                    return queryFilterJsons;
                }
                queryFilters.forEach(function (queryFilter) {
                    queryFilterJsons.push(queryFilter.toJson());
                });
                return queryFilterJsons;
            };
            ContentQueryRequest.prototype.expandAsString = function () {
                switch (this.expand) {
                    case 2 /* FULL */:
                        return "full";
                    case 1 /* SUMMARY */:
                        return "summary";
                    case 0 /* NONE */:
                        return "none";
                    default:
                        return "summary";
                }
            };
            ContentQueryRequest.prototype.contentTypeNamesAsString = function (names) {
                var result = [];
                names.forEach(function (name) {
                    result.push(name.toString());
                });
                return result;
            };
            ContentQueryRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "query");
            };
            return ContentQueryRequest;
        })(content.ContentResourceRequest);
        content.ContentQueryRequest = ContentQueryRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentQueryResult = (function () {
            function ContentQueryResult(contents, aggreations, contentsAsJson, metadata) {
                this.contents = contents;
                this.aggregations = aggreations;
                this.contentsAsJson = contentsAsJson;
                this.metadata = metadata;
            }
            ContentQueryResult.prototype.getContents = function () {
                return this.contents;
            };
            ContentQueryResult.prototype.getContentsAsJson = function () {
                return this.contentsAsJson;
            };
            ContentQueryResult.prototype.getAggregations = function () {
                return this.aggregations;
            };
            ContentQueryResult.prototype.getMetadata = function () {
                return this.metadata;
            };
            return ContentQueryResult;
        })();
        content.ContentQueryResult = ContentQueryResult;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var PublishContentRequest = (function (_super) {
            __extends(PublishContentRequest, _super);
            function PublishContentRequest(contentId) {
                _super.call(this);
                this.ids = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentId) {
                    this.addId(contentId);
                }
            }
            PublishContentRequest.prototype.setIds = function (contentIds) {
                this.ids = contentIds;
                return this;
            };
            PublishContentRequest.prototype.addId = function (contentId) {
                this.ids.push(contentId);
                return this;
            };
            PublishContentRequest.prototype.getParams = function () {
                return {
                    ids: this.ids.map(function (el) {
                        return el.toString();
                    })
                };
            };
            PublishContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "publish");
            };
            PublishContentRequest.feedback = function (jsonResponse) {
                var result = jsonResponse.getResult(), succeeded = result.successes.length, failed = result.failures.length, deleted = result.deleted.length, total = succeeded + failed + deleted;
                switch (total) {
                    case 0:
                        api.notify.showFeedback('Nothing to publish.');
                        break;
                    case 1:
                        if (succeeded === 1) {
                            api.notify.showSuccess('Content [' + result.successes[0].name + '] published!');
                        }
                        else if (failed === 1) {
                            api.notify.showError('Content [' + result.failures[0].path + '] failed, reason: ' + result.failures[0].reason);
                        }
                        else {
                            api.notify.showSuccess('Content [' + result.deleted[0] + '] deleted');
                        }
                        break;
                    default:
                        if (succeeded > 0) {
                            api.notify.showSuccess('[' + succeeded + '] contents published!');
                        }
                        if (deleted > 0) {
                            api.notify.showSuccess('[' + deleted + '] contents deleted ');
                        }
                        if (failed > 0) {
                            api.notify.showError('[' + failed + '] contents failed to publish!');
                        }
                }
            };
            return PublishContentRequest;
        })(content.ContentResourceRequest);
        content.PublishContentRequest = PublishContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CompareContentRequest = (function (_super) {
            __extends(CompareContentRequest, _super);
            function CompareContentRequest(ids) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.ids = ids;
            }
            CompareContentRequest.fromContentSummaries = function (contentSummaries) {
                var ids = [];
                contentSummaries.forEach(function (contentSummary) {
                    ids.push(contentSummary.getContentId().toString());
                });
                return new CompareContentRequest(ids);
            };
            CompareContentRequest.prototype.getParams = function () {
                return {
                    ids: this.ids
                };
            };
            CompareContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "compare");
            };
            CompareContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToCompareResults(response.getResult());
                });
            };
            CompareContentRequest.prototype.fromJsonToCompareResults = function (json) {
                return content.CompareContentResults.fromJson(json);
            };
            return CompareContentRequest;
        })(content.ContentResourceRequest);
        content.CompareContentRequest = CompareContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CompareContentResults = (function () {
            function CompareContentResults(compareContentResults) {
                this.compareContentResults = [];
                this.compareContentResults = compareContentResults;
            }
            CompareContentResults.prototype.get = function (contentId) {
                var compareContentResult = null;
                this.compareContentResults.forEach(function (result) {
                    if (result.getId() == contentId) {
                        compareContentResult = result;
                    }
                });
                return compareContentResult;
            };
            CompareContentResults.fromJson = function (json) {
                var list = [];
                json.compareContentResults.forEach(function (compareContentResult) {
                    list.push(content.CompareContentResult.fromJson(compareContentResult));
                });
                return new CompareContentResults(list);
            };
            return CompareContentResults;
        })();
        content.CompareContentResults = CompareContentResults;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CompareContentResult = (function () {
            function CompareContentResult(id, compareStatus) {
                this.compareStatus = compareStatus;
                this.id = id;
            }
            CompareContentResult.prototype.getId = function () {
                return this.id;
            };
            CompareContentResult.prototype.getCompareStatus = function () {
                return this.compareStatus;
            };
            CompareContentResult.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, CompareContentResult)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id.toString(), other.id.toString())) {
                    return false;
                }
                return true;
            };
            CompareContentResult.fromJson = function (json) {
                var status = content.CompareStatus[json.compareStatus];
                return new CompareContentResult(json.id, status);
            };
            return CompareContentResult;
        })();
        content.CompareContentResult = CompareContentResult;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (CompareStatus) {
            CompareStatus[CompareStatus["NEW"] = 0] = "NEW";
            CompareStatus[CompareStatus["NEW_TARGET"] = 1] = "NEW_TARGET";
            CompareStatus[CompareStatus["NEWER"] = 2] = "NEWER";
            CompareStatus[CompareStatus["OLDER"] = 3] = "OLDER";
            CompareStatus[CompareStatus["PENDING_DELETE"] = 4] = "PENDING_DELETE";
            CompareStatus[CompareStatus["PENDING_DELETE_TARGET"] = 5] = "PENDING_DELETE_TARGET";
            CompareStatus[CompareStatus["EQUAL"] = 6] = "EQUAL";
            CompareStatus[CompareStatus["MOVED"] = 7] = "MOVED";
            CompareStatus[CompareStatus["CONFLICT_PATH_EXISTS"] = 8] = "CONFLICT_PATH_EXISTS";
            CompareStatus[CompareStatus["CONFLICT_VERSION_BRANCH_DIVERGS"] = 9] = "CONFLICT_VERSION_BRANCH_DIVERGS";
            CompareStatus[CompareStatus["UNKNOWN"] = 10] = "UNKNOWN";
        })(content.CompareStatus || (content.CompareStatus = {}));
        var CompareStatus = content.CompareStatus;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (Branch) {
            Branch[Branch["DRAFT"] = 0] = "DRAFT";
            Branch[Branch["MASTER"] = 1] = "MASTER";
        })(content.Branch || (content.Branch = {}));
        var Branch = content.Branch;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryAndCompareStatus = (function () {
            function ContentSummaryAndCompareStatus() {
            }
            ContentSummaryAndCompareStatus.fromContentSummary = function (contentSummary) {
                return new ContentSummaryAndCompareStatus().setContentSummary(contentSummary);
            };
            ContentSummaryAndCompareStatus.fromContentAndCompareStatus = function (contentSummary, compareStatus) {
                return new ContentSummaryAndCompareStatus().setContentSummary(contentSummary).setCompareStatus(compareStatus);
            };
            ContentSummaryAndCompareStatus.fromUploadItem = function (item) {
                return new ContentSummaryAndCompareStatus().setUploadItem(item);
            };
            ContentSummaryAndCompareStatus.prototype.getContentSummary = function () {
                return this.contentSummary;
            };
            ContentSummaryAndCompareStatus.prototype.setContentSummary = function (contentSummary) {
                this.contentSummary = contentSummary;
                return this;
            };
            ContentSummaryAndCompareStatus.prototype.getCompareStatus = function () {
                return this.compareStatus;
            };
            ContentSummaryAndCompareStatus.prototype.setCompareStatus = function (status) {
                this.compareStatus = status;
                return this;
            };
            ContentSummaryAndCompareStatus.prototype.getUploadItem = function () {
                return this.uploadItem;
            };
            ContentSummaryAndCompareStatus.prototype.setUploadItem = function (item) {
                var _this = this;
                this.uploadItem = item;
                if (item.isUploaded()) {
                    this.contentSummary = item.getModel();
                }
                else {
                    item.onUploaded(function (contentSummary) {
                        _this.contentSummary = contentSummary;
                    });
                }
                return this;
            };
            ContentSummaryAndCompareStatus.prototype.getContentId = function () {
                return this.contentSummary ? this.contentSummary.getContentId() : null;
            };
            ContentSummaryAndCompareStatus.prototype.getId = function () {
                return (this.contentSummary && this.contentSummary.getId()) || (this.uploadItem && this.uploadItem.getId()) || "";
            };
            ContentSummaryAndCompareStatus.prototype.hasChildren = function () {
                return !!this.contentSummary ? this.contentSummary.hasChildren() : false;
            };
            return ContentSummaryAndCompareStatus;
        })();
        content.ContentSummaryAndCompareStatus = ContentSummaryAndCompareStatus;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ContentSummaryAndCompareStatusFetcher = (function () {
            function ContentSummaryAndCompareStatusFetcher() {
            }
            ContentSummaryAndCompareStatusFetcher.fetchChildren = function (parentContentId, from, size, childOrder) {
                if (from === void 0) { from = 0; }
                if (size === void 0) { size = -1; }
                var deferred = wemQ.defer();
                new _content.ListContentByIdRequest(parentContentId).setFrom(from).setSize(size).setOrder(childOrder).sendAndParse().then(function (response) {
                    _content.CompareContentRequest.fromContentSummaries(response.getContents()).sendAndParse().then(function (compareResults) {
                        var result = new _content.ContentResponse(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(response.getContents(), compareResults), response.getMetadata());
                        deferred.resolve(result);
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetch = function (contentId) {
                var deferred = wemQ.defer();
                new _content.GetContentByIdRequest(contentId).sendAndParse().then(function (content) {
                    _content.CompareContentRequest.fromContentSummaries([content]).sendAndParse().then(function (compareResults) {
                        deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus([content], compareResults)[0]);
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchByPaths = function (paths) {
                var deferred = wemQ.defer();
                new _content.BatchContentRequest().setContentPaths(paths).sendAndParse().then(function (response) {
                    var contentSummaries = response.getContents();
                    _content.CompareContentRequest.fromContentSummaries(contentSummaries).sendAndParse().then(function (compareResults) {
                        deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchStatus = function (contentSummaries) {
                var deferred = wemQ.defer();
                _content.CompareContentRequest.fromContentSummaries(contentSummaries).sendAndParse().then(function (compareResults) {
                    deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.updateCompareStatus = function (contentSummaries, compareResults) {
                var list = [];
                contentSummaries.forEach(function (contentSummary) {
                    var compareResult = compareResults.get(contentSummary.getId());
                    var newEntry = _content.ContentSummaryAndCompareStatus.fromContentAndCompareStatus(contentSummary, compareResult.getCompareStatus());
                    list.push(newEntry);
                });
                return list;
            };
            return ContentSummaryAndCompareStatusFetcher;
        })();
        _content.ContentSummaryAndCompareStatusFetcher = ContentSummaryAndCompareStatusFetcher;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryAndCompareStatusViewer = (function (_super) {
            __extends(ContentSummaryAndCompareStatusViewer, _super);
            function ContentSummaryAndCompareStatusViewer() {
                _super.call(this, "content-summary-and-compare-status-viewer");
            }
            ContentSummaryAndCompareStatusViewer.prototype.resolveDisplayName = function (object) {
                var contentSummary = object.getContentSummary(), uploadItem = object.getUploadItem();
                if (contentSummary) {
                    return contentSummary.getDisplayName();
                }
                else if (uploadItem) {
                    return uploadItem.getName();
                }
                return "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveUnnamedDisplayName = function (object) {
                var contentSummary = object.getContentSummary();
                return (contentSummary && contentSummary.getType()) ? contentSummary.getType().getLocalName() : "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                var contentSummary = object.getContentSummary(), uploadItem = object.getUploadItem();
                if (contentSummary) {
                    var contentName = contentSummary.getName(), invalid = !contentSummary.isValid() || !contentSummary.getDisplayName() || contentName.isUnnamed(), pendingDelete = contentSummary.getContentState().isPendingDelete();
                    this.toggleClass("invalid", invalid);
                    this.toggleClass("pending-delete", pendingDelete);
                    if (relativePath) {
                        return !contentName.isUnnamed() ? contentName.toString() : api.ui.NamesAndIconViewer.EMPTY_SUB_NAME;
                    }
                    else {
                        return !contentName.isUnnamed() ? contentSummary.getPath().toString() : content.ContentPath.fromParent(contentSummary.getPath().getParentPath(), api.ui.NamesAndIconViewer.EMPTY_SUB_NAME).toString();
                    }
                }
                else if (uploadItem) {
                    return uploadItem.getName();
                }
                return "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveSubTitle = function (object) {
                var contentSummary = object.getContentSummary();
                return !!contentSummary ? contentSummary.getPath().toString() : "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveIconClass = function (object) {
                return !!object.getUploadItem() ? "icon-file-upload2" : "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveIconUrl = function (object) {
                var contentSummary = object.getContentSummary();
                return !!contentSummary ? new content.ContentIconUrlResolver().setContent(contentSummary).resolve() : "";
            };
            return ContentSummaryAndCompareStatusViewer;
        })(api.ui.NamesAndIconViewer);
        content.ContentSummaryAndCompareStatusViewer = ContentSummaryAndCompareStatusViewer;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentVersion = (function () {
            function ContentVersion() {
            }
            ContentVersion.fromJson = function (contentVersionJson, workspaces) {
                console.log("WOrkspaces: " + workspaces);
                var contentVersion = new ContentVersion();
                contentVersion.modifier = contentVersionJson.modifier;
                contentVersion.displayName = contentVersionJson.displayName;
                contentVersion.modified = new Date(contentVersionJson.modified);
                contentVersion.comment = contentVersionJson.comment;
                contentVersion.id = contentVersionJson.id;
                contentVersion.workspaces = workspaces || [];
                return contentVersion;
            };
            return ContentVersion;
        })();
        content.ContentVersion = ContentVersion;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentVersionViewer = (function (_super) {
            __extends(ContentVersionViewer, _super);
            function ContentVersionViewer() {
                _super.call(this);
                this.namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).setAppendIcon(false).build();
                this.appendChild(this.namesAndIconView);
            }
            ContentVersionViewer.prototype.setObject = function (contentVersion, row) {
                _super.prototype.setObject.call(this, contentVersion);
                this.namesAndIconView.setMainName(api.ui.treegrid.DateTimeFormatter.createHtml(contentVersion.modified) + "        " + contentVersion.modifier + "        " + contentVersion.displayName).setSubName(contentVersion.comment);
            };
            return ContentVersionViewer;
        })(api.ui.Viewer);
        content.ContentVersionViewer = ContentVersionViewer;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetActiveContentVersionsRequest = (function (_super) {
            __extends(GetActiveContentVersionsRequest, _super);
            function GetActiveContentVersionsRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.id = id;
            }
            GetActiveContentVersionsRequest.prototype.getParams = function () {
                return {
                    id: this.id.toString()
                };
            };
            GetActiveContentVersionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'getActiveVersions');
            };
            GetActiveContentVersionsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContentVersions(response.getResult().activeContentVersions);
                });
            };
            GetActiveContentVersionsRequest.prototype.fromJsonToContentVersions = function (json) {
                var contentVersionJson;
                var contentVersion;
                var contentVersionsMap = {};
                json.forEach(function (activeContentVersion) {
                    console.log("ActiveContentVersion: ", activeContentVersion);
                    contentVersionJson = activeContentVersion.contentVersion;
                    console.log("contentVersionJson: ", activeContentVersion.contentVersion);
                    contentVersion = contentVersionsMap[contentVersionJson.id];
                    if (!contentVersion) {
                        contentVersion = content.ContentVersion.fromJson(contentVersionJson, [activeContentVersion.branch]);
                        contentVersionsMap[contentVersion.id] = contentVersion;
                    }
                    else {
                        contentVersion.workspaces.push(activeContentVersion.branch);
                    }
                });
                return Object.keys(contentVersionsMap).map(function (key) {
                    return contentVersionsMap[key];
                });
            };
            return GetActiveContentVersionsRequest;
        })(content.ContentResourceRequest);
        content.GetActiveContentVersionsRequest = GetActiveContentVersionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentVersionsRequest = (function (_super) {
            __extends(GetContentVersionsRequest, _super);
            function GetContentVersionsRequest(contentId) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.contentId = contentId;
            }
            GetContentVersionsRequest.prototype.setFrom = function (from) {
                this.from = from;
                return this;
            };
            GetContentVersionsRequest.prototype.setSize = function (size) {
                this.size = size;
                return this;
            };
            GetContentVersionsRequest.prototype.getParams = function () {
                return {
                    contentId: this.contentId.toString(),
                    from: this.from,
                    size: this.size
                };
            };
            GetContentVersionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'getVersions');
            };
            GetContentVersionsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContentVersions(response.getResult().contentVersions);
                });
            };
            GetContentVersionsRequest.prototype.fromJsonToContentVersions = function (json) {
                var contentVersions = [];
                json.forEach(function (contentVersionJson) {
                    contentVersions.push(content.ContentVersion.fromJson(contentVersionJson));
                });
                return contentVersions;
            };
            return GetContentVersionsRequest;
        })(content.ContentResourceRequest);
        content.GetContentVersionsRequest = GetContentVersionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetNearestSiteRequest = (function (_super) {
            __extends(GetNearestSiteRequest, _super);
            function GetNearestSiteRequest(contentId) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.contentId = contentId;
            }
            GetNearestSiteRequest.prototype.getParams = function () {
                return {
                    contentId: this.contentId.toString()
                };
            };
            GetNearestSiteRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "nearestSite");
            };
            GetNearestSiteRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                });
            };
            return GetNearestSiteRequest;
        })(content.ContentResourceRequest);
        content.GetNearestSiteRequest = GetNearestSiteRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ContentNamedEvent = (function (_super) {
            __extends(ContentNamedEvent, _super);
            function ContentNamedEvent(wizard, content) {
                _super.call(this);
                this.wizard = wizard;
                this.content = content;
            }
            ContentNamedEvent.prototype.getWizard = function () {
                return this.wizard;
            };
            ContentNamedEvent.prototype.getContent = function () {
                return this.content;
            };
            ContentNamedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentNamedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentNamedEvent;
        })(api.event.Event);
        _content.ContentNamedEvent = ContentNamedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderChildMovement = (function () {
            function OrderChildMovement(id, moveBeforeId) {
                this.contentId = id;
                this.moveBefore = moveBeforeId;
            }
            OrderChildMovement.prototype.getContentId = function () {
                return this.contentId;
            };
            OrderChildMovement.prototype.getMoveBefore = function () {
                return this.moveBefore;
            };
            OrderChildMovement.prototype.toJson = function () {
                return {
                    "contentId": this.contentId.toString(),
                    "moveBefore": !!this.moveBefore ? this.moveBefore.toString() : ""
                };
            };
            return OrderChildMovement;
        })();
        content.OrderChildMovement = OrderChildMovement;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderChildMovements = (function () {
            function OrderChildMovements() {
                this.reorderChildren = [];
            }
            OrderChildMovements.prototype.getReorderChildren = function () {
                return this.reorderChildren;
            };
            OrderChildMovements.prototype.addChildMovement = function (movement) {
                this.reorderChildren.push(movement);
            };
            OrderChildMovements.prototype.toJson = function () {
                return {
                    "reorderChildren": this.toArrayJson()
                };
            };
            OrderChildMovements.prototype.toArrayJson = function () {
                var result = [];
                this.reorderChildren.forEach(function (movement) {
                    result.push(movement.toJson());
                });
                return result;
            };
            return OrderChildMovements;
        })();
        content.OrderChildMovements = OrderChildMovements;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderExpr = (function () {
            function OrderExpr(builder) {
                this.direction = builder.direction;
            }
            OrderExpr.prototype.getDirection = function () {
                return this.direction;
            };
            OrderExpr.prototype.toJson = function () {
                throw new Error("Must be implemented by inheritors");
            };
            OrderExpr.prototype.toString = function () {
                throw new Error("Must be implemented by inheritors");
            };
            OrderExpr.toArrayJson = function (expressions) {
                var wrappers = [];
                expressions.forEach(function (expr) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(expr, content.FieldOrderExpr)) {
                        wrappers.push({ "FieldOrderExpr": expr.toJson() });
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(expr, content.DynamicOrderExpr)) {
                        wrappers.push({ "DynamicOrderExpr": expr.toJson() });
                    }
                });
                return wrappers;
            };
            OrderExpr.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, OrderExpr)) {
                    return false;
                }
                var other = o;
                if (this.direction.toLowerCase() != other.getDirection().toLowerCase()) {
                    return false;
                }
                return true;
            };
            return OrderExpr;
        })();
        content.OrderExpr = OrderExpr;
        var OrderExprBuilder = (function () {
            function OrderExprBuilder(json) {
                if (json) {
                    this.direction = json.direction;
                }
            }
            OrderExprBuilder.prototype.setDirection = function (value) {
                this.direction = value;
                return this;
            };
            OrderExprBuilder.prototype.build = function () {
                return new OrderExpr(this);
            };
            return OrderExprBuilder;
        })();
        content.OrderExprBuilder = OrderExprBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DynamicOrderExpr = (function (_super) {
            __extends(DynamicOrderExpr, _super);
            function DynamicOrderExpr(builder) {
                _super.call(this, builder);
                this.function = builder.function;
            }
            DynamicOrderExpr.prototype.getFunction = function () {
                return this.function;
            };
            DynamicOrderExpr.prototype.toString = function () {
                return this.function + " " + _super.prototype.getDirection.call(this);
            };
            DynamicOrderExpr.prototype.toJson = function () {
                return {
                    "function": this.function,
                    "direction": this.getDirection()
                };
            };
            return DynamicOrderExpr;
        })(content.OrderExpr);
        content.DynamicOrderExpr = DynamicOrderExpr;
        var DynamicOrderExprBuilder = (function (_super) {
            __extends(DynamicOrderExprBuilder, _super);
            function DynamicOrderExprBuilder(json) {
                _super.call(this, json);
                this.function = json.function;
            }
            DynamicOrderExprBuilder.prototype.setFunction = function (value) {
                this.function = value;
                return this;
            };
            DynamicOrderExprBuilder.prototype.build = function () {
                return new DynamicOrderExpr(this);
            };
            return DynamicOrderExprBuilder;
        })(content.OrderExprBuilder);
        content.DynamicOrderExprBuilder = DynamicOrderExprBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var FieldOrderExpr = (function (_super) {
            __extends(FieldOrderExpr, _super);
            function FieldOrderExpr(builder) {
                _super.call(this, builder);
                this.fieldName = builder.fieldName;
            }
            FieldOrderExpr.prototype.getFieldName = function () {
                return this.fieldName;
            };
            FieldOrderExpr.prototype.toJson = function () {
                return {
                    "fieldName": this.fieldName,
                    "direction": this.getDirection()
                };
            };
            FieldOrderExpr.prototype.toString = function () {
                return this.fieldName + " " + _super.prototype.getDirection.call(this);
            };
            FieldOrderExpr.prototype.equals = function (o) {
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FieldOrderExpr)) {
                    return false;
                }
                var other = o;
                if (this.fieldName.toLowerCase() != other.getFieldName().toLowerCase()) {
                    return false;
                }
                return true;
            };
            return FieldOrderExpr;
        })(content.OrderExpr);
        content.FieldOrderExpr = FieldOrderExpr;
        var FieldOrderExprBuilder = (function (_super) {
            __extends(FieldOrderExprBuilder, _super);
            function FieldOrderExprBuilder(json) {
                _super.call(this, json);
                if (json) {
                    this.fieldName = json.fieldName;
                }
            }
            FieldOrderExprBuilder.prototype.setFieldName = function (value) {
                this.fieldName = value;
                return this;
            };
            FieldOrderExprBuilder.prototype.build = function () {
                return new FieldOrderExpr(this);
            };
            return FieldOrderExprBuilder;
        })(content.OrderExprBuilder);
        content.FieldOrderExprBuilder = FieldOrderExprBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ChildOrder = (function () {
            function ChildOrder() {
                this.DEFAULT_ORDER_DIRECTION_VALUE = "DESC";
                this.orderExpressions = [];
            }
            ChildOrder.prototype.getOrderExpressions = function () {
                return this.orderExpressions;
            };
            ChildOrder.prototype.addOrderExpr = function (expr) {
                this.orderExpressions.push(expr);
            };
            ChildOrder.prototype.addOrderExpressions = function (expressions) {
                var _this = this;
                expressions.forEach(function (expr) {
                    _this.orderExpressions.push(expr);
                });
            };
            ChildOrder.fromJson = function (childOrderJson) {
                var childOrder = new ChildOrder();
                childOrderJson.orderExpressions.forEach(function (orderExprJson) {
                    if (orderExprJson.FieldOrderExpr) {
                        childOrder.orderExpressions.push(new content.FieldOrderExprBuilder(orderExprJson.FieldOrderExpr).build());
                    }
                    else if (orderExprJson.DynamicOrderExpr) {
                        childOrder.orderExpressions.push(new content.DynamicOrderExprBuilder(orderExprJson.DynamicOrderExpr).build());
                    }
                });
                return childOrder;
            };
            ChildOrder.prototype.isManual = function () {
                if (this.orderExpressions.length == 0) {
                    return false;
                }
                var order = this.orderExpressions[0];
                if (api.ObjectHelper.iFrameSafeInstanceOf(order, content.FieldOrderExpr)) {
                    return api.ObjectHelper.stringEquals(ChildOrder.MANUAL_ORDER_VALUE_KEY.toLowerCase(), order.getFieldName().toLowerCase());
                }
                return false;
            };
            ChildOrder.prototype.isDesc = function () {
                if (this.orderExpressions.length == 0) {
                    return this.DEFAULT_ORDER_DIRECTION_VALUE == ChildOrder.DESC_ORDER_DIRECTION_VALUE;
                }
                var order = this.orderExpressions[0];
                return api.ObjectHelper.stringEquals(ChildOrder.DESC_ORDER_DIRECTION_VALUE.toLowerCase(), order.getDirection().toLowerCase());
            };
            ChildOrder.prototype.isDefault = function () {
                var order = this.orderExpressions[0];
                if (api.ObjectHelper.iFrameSafeInstanceOf(order, content.FieldOrderExpr)) {
                    var fieldOrder = order;
                    if (api.ObjectHelper.stringEquals(this.DEFAULT_ORDER_DIRECTION_VALUE.toLowerCase(), fieldOrder.getDirection().toLowerCase()) && api.ObjectHelper.stringEquals(ChildOrder.DEFAULT_ORDER_FIELD_VALUE.toLowerCase(), fieldOrder.getFieldName().toLowerCase())) {
                        return true;
                    }
                }
                return false;
            };
            ChildOrder.prototype.toJson = function () {
                return {
                    "orderExpressions": content.OrderExpr.toArrayJson(this.getOrderExpressions())
                };
            };
            ChildOrder.prototype.toString = function () {
                var result = "";
                this.orderExpressions.forEach(function (expr) {
                    result = result.concat(" ", expr.toString());
                });
                return result;
            };
            ChildOrder.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ChildOrder)) {
                    return false;
                }
                var other = o;
                if (this.orderExpressions.length != other.getOrderExpressions().length) {
                    return false;
                }
                for (var count in this.orderExpressions) {
                    if (!this.orderExpressions[count].equals(other.getOrderExpressions()[count])) {
                        return false;
                    }
                }
                return true;
            };
            ChildOrder.toSetChildOrderJson = function (contentId, childOrder) {
                if (contentId && childOrder) {
                    return {
                        "childOrder": childOrder.toJson(),
                        "contentId": contentId.toString()
                    };
                }
            };
            ChildOrder.DEFAULT_ORDER_FIELD_VALUE = api.query.QueryField.MODIFIED_TIME;
            ChildOrder.ASC_ORDER_DIRECTION_VALUE = "ASC";
            ChildOrder.DESC_ORDER_DIRECTION_VALUE = "DESC";
            ChildOrder.MANUAL_ORDER_VALUE_KEY = api.query.QueryField.MANUAL_ORDER_VALUE;
            return ChildOrder;
        })();
        content.ChildOrder = ChildOrder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (MediaUploaderOperation) {
            MediaUploaderOperation[MediaUploaderOperation["create"] = 0] = "create";
            MediaUploaderOperation[MediaUploaderOperation["update"] = 1] = "update";
        })(content.MediaUploaderOperation || (content.MediaUploaderOperation = {}));
        var MediaUploaderOperation = content.MediaUploaderOperation;
        var MediaUploader = (function (_super) {
            __extends(MediaUploader, _super);
            function MediaUploader(config) {
                if (config.url == undefined) {
                    config.url = api.util.UriHelper.getRestUri("content/" + MediaUploaderOperation[config.operation] + "Media");
                }
                _super.call(this, config);
                this.addClass('media-uploader');
                this.propertyIdProvider = api.Client.get().getPropertyIdProvider();
            }
            MediaUploader.prototype.createModel = function (serverResponse) {
                if (serverResponse) {
                    return new api.content.ContentBuilder().fromContentJson(serverResponse, this.propertyIdProvider).build();
                }
                else {
                    return null;
                }
            };
            MediaUploader.prototype.getModelValue = function (item) {
                return item.getId();
            };
            MediaUploader.prototype.createResultItem = function (value) {
                var link = new api.dom.AEl().setUrl(api.util.UriHelper.getRestUri('content/media/' + value), "_blank");
                link.setHtml(value);
                return link;
            };
            return MediaUploader;
        })(api.ui.uploader.Uploader);
        content.MediaUploader = MediaUploader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ImageUploader = (function (_super) {
            __extends(ImageUploader, _super);
            function ImageUploader(config) {
                var _this = this;
                this.images = [];
                if (config.allowTypes == undefined) {
                    config.allowTypes = [
                        { title: 'Image files', extensions: 'jpg,gif,png' }
                    ];
                }
                _super.call(this, config);
                this.addClass('image-uploader');
                this.initialWidth = 0;
                this.onShown(function () {
                    if (_this.getEl().getWidth() == 0) {
                        _this.initialWidth = Math.max(_this.getParentElement().getEl().getWidth(), _this.initialWidth);
                        _this.getEl().setMaxWidthPx(_this.initialWidth);
                    }
                });
            }
            ImageUploader.prototype.createResultItem = function (value) {
                var _this = this;
                this.initialWidth = this.getParentElement().getEl().getWidth();
                var container = new api.dom.DivEl();
                var imgUrl = new content.ContentImageUrlResolver().setContentId(new api.content.ContentId(value)).setSize(this.initialWidth).setTimestamp(new Date()).resolve();
                var image = new api.dom.ImgEl(imgUrl);
                this.getEl().setMaxWidthPx(image.getEl().getNaturalWidth());
                image.onLoaded(function () {
                    _this.getEl().setMaxWidthPx(image.getEl().getNaturalWidth());
                });
                image.onRemoved(function () {
                    _this.getEl().setMaxWidthPx(_this.initialWidth);
                });
                image.onClicked(function (event) {
                    image.toggleClass('selected');
                    event.stopPropagation();
                    event.preventDefault();
                });
                container.appendChild(image);
                this.images.push(image);
                api.dom.Body.get().onClicked(function (event) {
                    _this.images.forEach(function (img) {
                        img.removeClass('selected');
                    });
                });
                return container;
            };
            return ImageUploader;
        })(content.MediaUploader);
        content.ImageUploader = ImageUploader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ThumbnailUploader = (function (_super) {
            __extends(ThumbnailUploader, _super);
            function ThumbnailUploader(config) {
                if (config.url == undefined) {
                    config.url = api.util.UriHelper.getRestUri("content/updateThumbnail");
                }
                if (config.showButtons == undefined) {
                    config.showButtons = false;
                }
                if (config.dropzoneAlwaysVisible == undefined) {
                    config.dropzoneAlwaysVisible = true;
                }
                if (config.resultAlwaysVisisble == undefined) {
                    config.resultAlwaysVisisble = true;
                }
                if (config.allowTypes == undefined) {
                    config.allowTypes = [
                        { title: 'Image files', extensions: 'jpg,gif,png' }
                    ];
                }
                if (config.allowMultiSelection == undefined) {
                    config.allowMultiSelection = false;
                }
                _super.call(this, config);
                this.addClass('thumbnail-uploader');
                this.propertyIdProvider = api.Client.get().getPropertyIdProvider();
                this.iconUrlResolver = new content.ContentIconUrlResolver();
            }
            ThumbnailUploader.prototype.createModel = function (serverResponse) {
                if (serverResponse) {
                    return new api.content.ContentBuilder().fromContentJson(serverResponse, this.propertyIdProvider).build();
                }
                else {
                    return null;
                }
            };
            ThumbnailUploader.prototype.getModelValue = function (item) {
                return this.iconUrlResolver.setContent(item).resolve();
            };
            ThumbnailUploader.prototype.createResultItem = function (value) {
                return new api.dom.ImgEl(value);
            };
            return ThumbnailUploader;
        })(api.ui.uploader.Uploader);
        content.ThumbnailUploader = ThumbnailUploader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var query;
        (function (query) {
            var ContentQuery = (function () {
                function ContentQuery() {
                    this.contentTypeNames = [];
                    this.aggregationQueries = [];
                    this.queryFilters = [];
                    this.from = 0;
                    this.size = 100;
                }
                ContentQuery.prototype.setQueryExpr = function (queryExpr) {
                    this.queryExpr = queryExpr;
                };
                ContentQuery.prototype.getQueryExpr = function () {
                    return this.queryExpr;
                };
                ContentQuery.prototype.setContentTypeNames = function (contentTypeNames) {
                    this.contentTypeNames = contentTypeNames;
                };
                ContentQuery.prototype.getContentTypes = function () {
                    return this.contentTypeNames;
                };
                ContentQuery.prototype.setFrom = function (from) {
                    this.from = from;
                };
                ContentQuery.prototype.getFrom = function () {
                    return this.from;
                };
                ContentQuery.prototype.setSize = function (size) {
                    this.size = size;
                };
                ContentQuery.prototype.getSize = function () {
                    return this.size;
                };
                ContentQuery.prototype.addAggregationQuery = function (aggregationQuery) {
                    this.aggregationQueries.push(aggregationQuery);
                };
                ContentQuery.prototype.getAggregationQueries = function () {
                    return this.aggregationQueries;
                };
                ContentQuery.prototype.addQueryFilter = function (queryFilter) {
                    this.queryFilters.push(queryFilter);
                };
                ContentQuery.prototype.getQueryFilters = function () {
                    return this.queryFilters;
                };
                return ContentQuery;
            })();
            query.ContentQuery = ContentQuery;
        })(query = content.query || (content.query = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var attachment;
        (function (attachment) {
            var AttachmentName = (function () {
                function AttachmentName(fileName) {
                    this.fileName = fileName;
                }
                AttachmentName.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, AttachmentName)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.fileName, other.fileName)) {
                        return false;
                    }
                    return true;
                };
                AttachmentName.prototype.toString = function () {
                    return this.fileName;
                };
                return AttachmentName;
            })();
            attachment.AttachmentName = AttachmentName;
        })(attachment = content.attachment || (content.attachment = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var attachment;
        (function (attachment) {
            var Attachment = (function () {
                function Attachment(builder) {
                    this.name = builder.name;
                    this.label = builder.label;
                    this.mimeType = builder.mimeType;
                    this.size = builder.size;
                }
                Attachment.prototype.getBinaryReference = function () {
                    return new api.util.BinaryReference(this.name.toString());
                };
                Attachment.prototype.getName = function () {
                    return this.name;
                };
                Attachment.prototype.getLabel = function () {
                    return this.label;
                };
                Attachment.prototype.getMimeType = function () {
                    return this.mimeType;
                };
                Attachment.prototype.getSize = function () {
                    return this.size;
                };
                Attachment.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Attachment)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.name, other.name)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.mimeType, other.mimeType)) {
                        return false;
                    }
                    if (!api.ObjectHelper.numberEquals(this.size, other.size)) {
                        return false;
                    }
                    return true;
                };
                Attachment.prototype.toJson = function () {
                    return {
                        "name": this.getName().toString(),
                        "label": this.getLabel(),
                        "mimeType": this.getMimeType(),
                        "size": this.getSize()
                    };
                };
                Attachment.create = function () {
                    return new AttachmentBuilder();
                };
                return Attachment;
            })();
            attachment.Attachment = Attachment;
            var AttachmentBuilder = (function () {
                function AttachmentBuilder() {
                }
                AttachmentBuilder.prototype.fromJson = function (json) {
                    this.setName(new attachment.AttachmentName(json.name)).setLabel(json.label).setSize(json.size).setMimeType(json.mimeType);
                    return this;
                };
                AttachmentBuilder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                AttachmentBuilder.prototype.setLabel = function (value) {
                    this.label = value;
                    return this;
                };
                AttachmentBuilder.prototype.setMimeType = function (value) {
                    this.mimeType = value;
                    return this;
                };
                AttachmentBuilder.prototype.setSize = function (value) {
                    this.size = value;
                    return this;
                };
                AttachmentBuilder.prototype.build = function () {
                    return new Attachment(this);
                };
                return AttachmentBuilder;
            })();
            attachment.AttachmentBuilder = AttachmentBuilder;
        })(attachment = content.attachment || (content.attachment = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var attachment;
        (function (_attachment) {
            var Attachments = (function () {
                function Attachments(builder) {
                    var _this = this;
                    this.attachmentByName = {};
                    this.attachmentByLabel = {};
                    this.attachments = [];
                    var count = 0;
                    builder.attachments.forEach(function (attachment) {
                        _this.attachmentByName[attachment.getName().toString()] = attachment;
                        _this.attachmentByLabel[attachment.getLabel()] = attachment;
                        count++;
                        _this.attachments.push(attachment);
                    });
                    this.size = count;
                }
                Attachments.prototype.forEach = function (callBack) {
                    this.attachments.forEach(function (attachment, index) {
                        callBack(attachment, index);
                    });
                };
                Attachments.prototype.getAttachmentByName = function (name) {
                    return this.attachmentByName[name];
                };
                Attachments.prototype.getAttachmentByLabel = function (label) {
                    return this.attachmentByLabel[name];
                };
                Attachments.prototype.getAttachment = function (index) {
                    return this.attachments[index];
                };
                Attachments.prototype.getSize = function () {
                    return this.size;
                };
                Attachments.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Attachments)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.arrayEquals(this.attachments, other.attachments)) {
                        return false;
                    }
                    return true;
                };
                return Attachments;
            })();
            _attachment.Attachments = Attachments;
            var AttachmentsBuilder = (function () {
                function AttachmentsBuilder() {
                    this.attachments = [];
                }
                AttachmentsBuilder.prototype.fromJson = function (jsons) {
                    var _this = this;
                    jsons.forEach(function (json) {
                        _this.attachments.push(new _attachment.AttachmentBuilder().fromJson(json).build());
                    });
                    return this;
                };
                AttachmentsBuilder.prototype.add = function (value) {
                    this.attachments.push(value);
                    return this;
                };
                AttachmentsBuilder.prototype.addAll = function (value) {
                    var _this = this;
                    value.forEach(function (attachment) {
                        _this.attachments.push(attachment);
                    });
                    return this;
                };
                AttachmentsBuilder.prototype.build = function () {
                    return new Attachments(this);
                };
                return AttachmentsBuilder;
            })();
            _attachment.AttachmentsBuilder = AttachmentsBuilder;
        })(attachment = content.attachment || (content.attachment = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var image;
        (function (image) {
            var PropertyTree = api.data.PropertyTree;
            var ImageContentDataFactory = (function () {
                function ImageContentDataFactory() {
                }
                ImageContentDataFactory.prototype.setImage = function (value) {
                    this.image = value;
                    return this;
                };
                ImageContentDataFactory.prototype.setMimeType = function (value) {
                    this.mimeType = value;
                    return this;
                };
                ImageContentDataFactory.prototype.create = function () {
                    var data = new PropertyTree();
                    data.addString("image", this.image.toString());
                    data.addString("mimeType", this.mimeType);
                    return data;
                };
                return ImageContentDataFactory;
            })();
            image.ImageContentDataFactory = ImageContentDataFactory;
        })(image = content.image || (content.image = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DescriptorName = (function () {
                function DescriptorName(name) {
                    if (name == null) {
                        throw new Error("name cannot be null");
                    }
                    this.name = name;
                }
                DescriptorName.prototype.toString = function () {
                    return this.name;
                };
                return DescriptorName;
            })();
            page.DescriptorName = DescriptorName;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var Descriptor = (function () {
                function Descriptor(builder) {
                    this.name = builder.name;
                    this.key = builder.key;
                    this.displayName = builder.displayName;
                    this.config = builder.config;
                }
                Descriptor.prototype.getKey = function () {
                    return this.key;
                };
                Descriptor.prototype.getName = function () {
                    return this.name;
                };
                Descriptor.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                Descriptor.prototype.getConfig = function () {
                    return this.config;
                };
                Descriptor.prototype.clone = function () {
                    throw new Error("Must be implemented in inheritor");
                };
                return Descriptor;
            })();
            page.Descriptor = Descriptor;
            var DescriptorBuilder = (function () {
                function DescriptorBuilder(source) {
                    if (source) {
                        this.key = source.getKey();
                        this.name = source.getName();
                        this.displayName = source.getDisplayName();
                        this.config = source.getConfig();
                    }
                }
                return DescriptorBuilder;
            })();
            page.DescriptorBuilder = DescriptorBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DescriptorKey = (function () {
                function DescriptorKey(moduleKey, name) {
                    this.moduleKey = moduleKey;
                    this.name = name;
                    this.refString = moduleKey.toString() + DescriptorKey.SEPARATOR + name.toString();
                }
                DescriptorKey.fromString = function (str) {
                    var sepIndex = str.indexOf(DescriptorKey.SEPARATOR);
                    if (sepIndex == -1) {
                        throw new Error("DescriptorKey must contain separator '" + DescriptorKey.SEPARATOR + "':" + str);
                    }
                    var moduleKey = str.substring(0, sepIndex);
                    var name = str.substring(sepIndex + 1, str.length);
                    return new DescriptorKey(api.module.ModuleKey.fromString(moduleKey), new page.DescriptorName(name));
                };
                DescriptorKey.prototype.getModuleKey = function () {
                    return this.moduleKey;
                };
                DescriptorKey.prototype.getName = function () {
                    return this.name;
                };
                DescriptorKey.prototype.toString = function () {
                    return this.refString;
                };
                DescriptorKey.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DescriptorKey)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                        return false;
                    }
                    return true;
                };
                DescriptorKey.SEPARATOR = ":";
                return DescriptorKey;
            })();
            page.DescriptorKey = DescriptorKey;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageDescriptor = (function (_super) {
                __extends(PageDescriptor, _super);
                function PageDescriptor(builder) {
                    _super.call(this, builder);
                    this.regions = builder.regions;
                }
                PageDescriptor.prototype.getRegions = function () {
                    return this.regions;
                };
                PageDescriptor.prototype.clone = function () {
                    return new PageDescriptorBuilder(this).build();
                };
                return PageDescriptor;
            })(page.Descriptor);
            page.PageDescriptor = PageDescriptor;
            var PageDescriptorBuilder = (function (_super) {
                __extends(PageDescriptorBuilder, _super);
                function PageDescriptorBuilder(source) {
                    this.regions = [];
                    if (source) {
                        _super.call(this, source);
                        this.regions = source.getRegions();
                    }
                }
                PageDescriptorBuilder.prototype.fromJson = function (json) {
                    this.setName(new page.DescriptorName(json.name));
                    this.setDisplayName(json.displayName);
                    this.setConfig(json.config != null ? api.form.Form.fromJson(json.config) : null);
                    this.setKey(page.DescriptorKey.fromString(json.key));
                    for (var i = 0; i < json.regions.length; i++) {
                        var region = new api.content.page.region.RegionDescriptorBuilder().fromJson(json.regions[i]).build();
                        this.regions.push(region);
                    }
                    return this;
                };
                PageDescriptorBuilder.prototype.setKey = function (key) {
                    this.key = key;
                    return this;
                };
                PageDescriptorBuilder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                PageDescriptorBuilder.prototype.setDisplayName = function (value) {
                    this.displayName = value;
                    return this;
                };
                PageDescriptorBuilder.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                PageDescriptorBuilder.prototype.addRegion = function (value) {
                    this.regions.push(value);
                    return this;
                };
                PageDescriptorBuilder.prototype.build = function () {
                    return new PageDescriptor(this);
                };
                return PageDescriptorBuilder;
            })(page.DescriptorBuilder);
            page.PageDescriptorBuilder = PageDescriptorBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var ModuleBasedCache = api.module.ModuleBasedCache;
            var PageDescriptorCache = (function (_super) {
                __extends(PageDescriptorCache, _super);
                function PageDescriptorCache() {
                    if (PageDescriptorCache.instance) {
                        throw new Error("Instantiation failed: Use PageDescriptorCache.get() instead!");
                    }
                    _super.call(this);
                }
                PageDescriptorCache.get = function () {
                    var w = api.dom.WindowDOM.get();
                    var topWindow = w.getTopParent().asWindow();
                    if (!topWindow.api.content.page.PageDescriptorCache.instance) {
                        topWindow.api.content.page.PageDescriptorCache.instance = new PageDescriptorCache();
                    }
                    return topWindow.api.content.page.PageDescriptorCache.instance;
                };
                PageDescriptorCache.prototype.loadByModule = function (moduleKey) {
                    new page.GetPageDescriptorsByModuleRequest(moduleKey).sendAndParse().catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                };
                PageDescriptorCache.prototype.put = function (descriptor) {
                    api.util.assertNotNull(descriptor, "a PageDescriptor must be given");
                    _super.prototype.put.call(this, descriptor, descriptor.getKey().getModuleKey());
                };
                PageDescriptorCache.prototype.getByKey = function (key) {
                    return _super.prototype.getByKey.call(this, key, key.getModuleKey());
                };
                PageDescriptorCache.prototype.createModuleCache = function () {
                    return new PageDescriptorModuleCache();
                };
                return PageDescriptorCache;
            })(ModuleBasedCache);
            page.PageDescriptorCache = PageDescriptorCache;
            var PageDescriptorModuleCache = (function (_super) {
                __extends(PageDescriptorModuleCache, _super);
                function PageDescriptorModuleCache() {
                    _super.apply(this, arguments);
                }
                PageDescriptorModuleCache.prototype.copy = function (object) {
                    return object.clone();
                };
                PageDescriptorModuleCache.prototype.getKeyFromObject = function (object) {
                    return object.getKey();
                };
                PageDescriptorModuleCache.prototype.getKeyAsString = function (key) {
                    return key.toString();
                };
                return PageDescriptorModuleCache;
            })(api.cache.Cache);
            page.PageDescriptorModuleCache = PageDescriptorModuleCache;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageDescriptorResourceRequest = (function (_super) {
                __extends(PageDescriptorResourceRequest, _super);
                function PageDescriptorResourceRequest() {
                    _super.call(this);
                    this.cache = page.PageDescriptorCache.get();
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "descriptor");
                }
                PageDescriptorResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                PageDescriptorResourceRequest.prototype.fromJsonToPageDescriptor = function (json) {
                    var pageDescriptor = new api.content.page.PageDescriptorBuilder().fromJson(json).build();
                    this.cache.put(pageDescriptor);
                    return pageDescriptor;
                };
                PageDescriptorResourceRequest.prototype.fromJsonToPageDescriptors = function (json) {
                    var _this = this;
                    return json.descriptors.map(function (descriptorJson) {
                        return _this.fromJsonToPageDescriptor(descriptorJson);
                    });
                };
                return PageDescriptorResourceRequest;
            })(api.rest.ResourceRequest);
            page.PageDescriptorResourceRequest = PageDescriptorResourceRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageDescriptorByKeyRequest = (function (_super) {
                __extends(GetPageDescriptorByKeyRequest, _super);
                function GetPageDescriptorByKeyRequest(key) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.key = key;
                }
                GetPageDescriptorByKeyRequest.prototype.getParams = function () {
                    return {
                        key: this.key.toString()
                    };
                };
                GetPageDescriptorByKeyRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetPageDescriptorByKeyRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    var pageDescriptor = this.cache.getByKey(this.key);
                    if (pageDescriptor) {
                        return wemQ(pageDescriptor);
                    }
                    else {
                        return this.send().then(function (response) {
                            pageDescriptor = _this.fromJsonToPageDescriptor(response.getResult());
                            return pageDescriptor;
                        });
                    }
                };
                return GetPageDescriptorByKeyRequest;
            })(page.PageDescriptorResourceRequest);
            page.GetPageDescriptorByKeyRequest = GetPageDescriptorByKeyRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageDescriptorsByModuleRequest = (function (_super) {
                __extends(GetPageDescriptorsByModuleRequest, _super);
                function GetPageDescriptorsByModuleRequest(moduleKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.moduleKey = moduleKey;
                }
                GetPageDescriptorsByModuleRequest.prototype.getParams = function () {
                    return {
                        moduleKey: this.moduleKey.toString()
                    };
                };
                GetPageDescriptorsByModuleRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "by_module");
                };
                GetPageDescriptorsByModuleRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    var cached = this.cache.getByModule(this.moduleKey);
                    if (cached) {
                        return wemQ(cached);
                    }
                    else {
                        return this.send().then(function (response) {
                            return _this.fromJsonToPageDescriptors(response.getResult());
                        });
                    }
                };
                return GetPageDescriptorsByModuleRequest;
            })(page.PageDescriptorResourceRequest);
            page.GetPageDescriptorsByModuleRequest = GetPageDescriptorsByModuleRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageDescriptorsByModulesRequest = (function (_super) {
                __extends(GetPageDescriptorsByModulesRequest, _super);
                function GetPageDescriptorsByModulesRequest(moduleKeys) {
                    _super.call(this);
                    this.moduleKeys = moduleKeys;
                }
                GetPageDescriptorsByModulesRequest.prototype.setModuleKeys = function (moduleKeys) {
                    this.moduleKeys = moduleKeys;
                };
                GetPageDescriptorsByModulesRequest.prototype.getParams = function () {
                    throw new Error("Unexpected call");
                };
                GetPageDescriptorsByModulesRequest.prototype.getRequestPath = function () {
                    throw new Error("Unexpected call");
                };
                GetPageDescriptorsByModulesRequest.prototype.sendAndParse = function () {
                    var promises = this.moduleKeys.map(function (moduleKey) { return new page.GetPageDescriptorsByModuleRequest(moduleKey).sendAndParse(); });
                    return wemQ.all(promises).then(function (results) {
                        var all = [];
                        results.forEach(function (descriptors) {
                            Array.prototype.push.apply(all, descriptors);
                        });
                        return all;
                    });
                };
                return GetPageDescriptorsByModulesRequest;
            })(page.PageDescriptorResourceRequest);
            page.GetPageDescriptorsByModulesRequest = GetPageDescriptorsByModulesRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var Dropdown = api.ui.selector.dropdown.Dropdown;
            var PageDescriptorDropdown = (function (_super) {
                __extends(PageDescriptorDropdown, _super);
                function PageDescriptorDropdown(name, config) {
                    var _this = this;
                    _super.call(this, name, {
                        optionDisplayValueViewer: new page.PageDescriptorViewer(),
                        dataIdProperty: 'value'
                    });
                    this.loadedDataListeners = [];
                    this.loader = config.loader;
                    this.loader.onLoadedData(function (event) {
                        event.getData().forEach(function (descriptor) {
                            var option = {
                                value: descriptor.getKey().toString(),
                                displayValue: descriptor,
                                indices: [descriptor.getDisplayName(), descriptor.getName().toString()]
                            };
                            _this.addOption(option);
                        });
                        _this.notifyLoadedData(event);
                    });
                }
                PageDescriptorDropdown.prototype.load = function () {
                    this.loader.load();
                };
                PageDescriptorDropdown.prototype.onLoadedData = function (listener) {
                    this.loadedDataListeners.push(listener);
                };
                PageDescriptorDropdown.prototype.unLoadedData = function (listener) {
                    this.loadedDataListeners = this.loadedDataListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                PageDescriptorDropdown.prototype.notifyLoadedData = function (event) {
                    var _this = this;
                    this.loadedDataListeners.forEach(function (listener) {
                        listener.call(_this, event);
                    });
                };
                return PageDescriptorDropdown;
            })(Dropdown);
            page.PageDescriptorDropdown = PageDescriptorDropdown;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageDescriptorViewer = (function (_super) {
                __extends(PageDescriptorViewer, _super);
                function PageDescriptorViewer() {
                    _super.call(this);
                }
                PageDescriptorViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                PageDescriptorViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getKey().toString();
                };
                PageDescriptorViewer.prototype.resolveIconClass = function (object) {
                    return "icon-file icon-large";
                };
                return PageDescriptorViewer;
            })(api.ui.NamesAndIconViewer);
            page.PageDescriptorViewer = PageDescriptorViewer;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateKey = (function (_super) {
                __extends(PageTemplateKey, _super);
                function PageTemplateKey(s) {
                    _super.call(this, s);
                }
                PageTemplateKey.fromContentId = function (id) {
                    return new PageTemplateKey(id.toString());
                };
                PageTemplateKey.fromString = function (s) {
                    return new PageTemplateKey(s);
                };
                PageTemplateKey.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PageTemplateKey)) {
                        return false;
                    }
                    var other = o;
                    return _super.prototype.equals.call(this, other);
                };
                return PageTemplateKey;
            })(api.content.ContentId);
            page.PageTemplateKey = PageTemplateKey;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateName = (function () {
                function PageTemplateName(name) {
                    if (name == null) {
                        throw new Error("name cannot be null");
                    }
                    this.name = name;
                }
                PageTemplateName.prototype.toString = function () {
                    return this.name;
                };
                return PageTemplateName;
            })();
            page.PageTemplateName = PageTemplateName;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var ContentTypeName = api.schema.content.ContentTypeName;
            var PageTemplate = (function (_super) {
                __extends(PageTemplate, _super);
                function PageTemplate(builder) {
                    var _this = this;
                    _super.call(this, builder);
                    this.canRender = [];
                    this.getContentData().forEachProperty("supports", function (property) {
                        _this.canRender.push(new ContentTypeName(property.getString()));
                    });
                }
                PageTemplate.prototype.getKey = function () {
                    return this.getContentId();
                };
                PageTemplate.prototype.getPageMode = function () {
                    if (this.isPage()) {
                        if (this.getPage().hasController()) {
                            return 2 /* FORCED_CONTROLLER */;
                        }
                        else {
                            throw new Error("Illegal state: A PageTemplate's Page must a controller set");
                        }
                    }
                    else {
                        return 3 /* NO_CONTROLLER */;
                    }
                };
                PageTemplate.prototype.getController = function () {
                    return this.getPage().getController();
                };
                PageTemplate.prototype.isCanRender = function (pattern) {
                    return this.getCanRender().some(function (name) {
                        return name.equals(pattern);
                    });
                };
                PageTemplate.prototype.getCanRender = function () {
                    return this.canRender;
                };
                PageTemplate.prototype.hasRegions = function () {
                    if (!this.isPage()) {
                        return false;
                    }
                    return this.getPage().hasRegions();
                };
                PageTemplate.prototype.getRegions = function () {
                    if (!this.isPage()) {
                        return null;
                    }
                    return this.getPage().getRegions();
                };
                PageTemplate.prototype.hasConfig = function () {
                    if (!this.isPage()) {
                        return false;
                    }
                    return this.getPage().hasConfig();
                };
                PageTemplate.prototype.getConfig = function () {
                    if (!this.isPage()) {
                        return null;
                    }
                    return this.getPage().getConfig();
                };
                PageTemplate.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PageTemplate)) {
                        return false;
                    }
                    return _super.prototype.equals.call(this, o);
                };
                PageTemplate.prototype.clone = function () {
                    return this.newBuilder().build();
                };
                PageTemplate.prototype.newBuilder = function () {
                    return new PageTemplateBuilder(this);
                };
                return PageTemplate;
            })(api.content.Content);
            page.PageTemplate = PageTemplate;
            var PageTemplateBuilder = (function (_super) {
                __extends(PageTemplateBuilder, _super);
                function PageTemplateBuilder(source) {
                    _super.call(this, source);
                }
                PageTemplateBuilder.prototype.fromContentJson = function (contentJson, propertyIdProvider) {
                    _super.prototype.fromContentJson.call(this, contentJson, propertyIdProvider);
                    return this;
                };
                PageTemplateBuilder.prototype.build = function () {
                    return new PageTemplate(this);
                };
                return PageTemplateBuilder;
            })(api.content.ContentBuilder);
            page.PageTemplateBuilder = PageTemplateBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateViewer = (function (_super) {
                __extends(PageTemplateViewer, _super);
                function PageTemplateViewer() {
                    _super.call(this);
                }
                PageTemplateViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                PageTemplateViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getController().toString();
                };
                PageTemplateViewer.prototype.resolveIconClass = function (object) {
                    return "icon-newspaper icon-large";
                };
                return PageTemplateViewer;
            })(api.ui.NamesAndIconViewer);
            page.PageTemplateViewer = PageTemplateViewer;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateComboBox = (function (_super) {
                __extends(PageTemplateComboBox, _super);
                function PageTemplateComboBox() {
                    _super.call(this, new api.ui.selector.combobox.RichComboBoxBuilder().setSelectedOptionsView(new PageTemplateSelectedOptionsView()).setIdentifierMethod("getKey").setMaximumOccurrences(1).setOptionDisplayValueViewer(new page.PageTemplateViewer));
                }
                return PageTemplateComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            page.PageTemplateComboBox = PageTemplateComboBox;
            var PageTemplateSelectedOptionsView = (function (_super) {
                __extends(PageTemplateSelectedOptionsView, _super);
                function PageTemplateSelectedOptionsView() {
                    _super.apply(this, arguments);
                }
                PageTemplateSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    return new api.ui.selector.combobox.SelectedOption(new PageTemplateSelectedOptionView(option), this.count());
                };
                return PageTemplateSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
            page.PageTemplateSelectedOptionsView = PageTemplateSelectedOptionsView;
            var PageTemplateSelectedOptionView = (function (_super) {
                __extends(PageTemplateSelectedOptionView, _super);
                function PageTemplateSelectedOptionView(option) {
                    this.pageTemplate = option.displayValue;
                    _super.call(this, option);
                    this.addClass("page-template-selected-option-view");
                }
                PageTemplateSelectedOptionView.prototype.layout = function () {
                    var _this = this;
                    var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                    namesAndIconView.setIconClass("icon-newspaper icon-large").setMainName(this.pageTemplate.getDisplayName()).setSubName(this.pageTemplate.getController().toString());
                    var removeButtonEl = new api.dom.AEl("remove");
                    removeButtonEl.onClicked(function (event) {
                        _this.notifySelectedOptionRemoveRequested();
                        event.stopPropagation();
                        event.preventDefault();
                        return false;
                    });
                    this.appendChild(removeButtonEl);
                    this.appendChild(namesAndIconView);
                };
                return PageTemplateSelectedOptionView;
            })(api.ui.selector.combobox.BaseSelectedOptionView);
            page.PageTemplateSelectedOptionView = PageTemplateSelectedOptionView;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateLoader = (function (_super) {
                __extends(PageTemplateLoader, _super);
                function PageTemplateLoader(request) {
                    _super.call(this, request);
                }
                PageTemplateLoader.prototype.filterFn = function (template) {
                    return template.getDisplayName().toString().indexOf(this.getSearchString().toLowerCase()) != -1;
                };
                return PageTemplateLoader;
            })(api.util.loader.BaseLoader);
            page.PageTemplateLoader = PageTemplateLoader;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateResourceRequest = (function (_super) {
                __extends(PageTemplateResourceRequest, _super);
                function PageTemplateResourceRequest() {
                    _super.call(this);
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "template");
                    this.propertyIdProvider = api.Client.get().getPropertyIdProvider();
                }
                PageTemplateResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                PageTemplateResourceRequest.prototype.fromJsonToContent = function (json) {
                    return api.content.Content.fromJson(json, this.propertyIdProvider);
                };
                return PageTemplateResourceRequest;
            })(api.rest.ResourceRequest);
            page.PageTemplateResourceRequest = PageTemplateResourceRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageTemplateByKeyRequest = (function (_super) {
                __extends(GetPageTemplateByKeyRequest, _super);
                function GetPageTemplateByKeyRequest(pageTemplateKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.pageTemplateKey = pageTemplateKey;
                }
                GetPageTemplateByKeyRequest.prototype.validate = function () {
                    api.util.assertNotNull(this.pageTemplateKey, "pageTemplateKey cannot be null");
                };
                GetPageTemplateByKeyRequest.prototype.getParams = function () {
                    return {
                        key: this.pageTemplateKey.toString()
                    };
                };
                GetPageTemplateByKeyRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetPageTemplateByKeyRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return _this.fromJsonToContent(response.getResult());
                    });
                };
                return GetPageTemplateByKeyRequest;
            })(page.PageTemplateResourceRequest);
            page.GetPageTemplateByKeyRequest = GetPageTemplateByKeyRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageTemplatesRequest = (function (_super) {
                __extends(GetPageTemplatesRequest, _super);
                function GetPageTemplatesRequest() {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                }
                GetPageTemplatesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list");
                };
                GetPageTemplatesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contents.map(function (contentJson) {
                            return _this.fromJsonToContent(contentJson);
                        });
                    });
                };
                return GetPageTemplatesRequest;
            })(page.PageTemplateResourceRequest);
            page.GetPageTemplatesRequest = GetPageTemplatesRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageTemplatesByCanRenderRequest = (function (_super) {
                __extends(GetPageTemplatesByCanRenderRequest, _super);
                function GetPageTemplatesByCanRenderRequest(site, contentTypeName) {
                    _super.call(this);
                    this.setMethod("GET");
                    this.site = site;
                    this.contentTypeName = contentTypeName;
                }
                GetPageTemplatesByCanRenderRequest.prototype.getParams = function () {
                    return {
                        siteId: this.site.toString(),
                        contentTypeName: this.contentTypeName.toString()
                    };
                };
                GetPageTemplatesByCanRenderRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "listByCanRender");
                };
                GetPageTemplatesByCanRenderRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contents.map(function (contentJson) {
                            return _this.fromJsonToContent(contentJson);
                        });
                    });
                };
                return GetPageTemplatesByCanRenderRequest;
            })(page.PageTemplateResourceRequest);
            page.GetPageTemplatesByCanRenderRequest = GetPageTemplatesByCanRenderRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetDefaultPageTemplateRequest = (function (_super) {
                __extends(GetDefaultPageTemplateRequest, _super);
                function GetDefaultPageTemplateRequest(site, contentName) {
                    _super.call(this);
                    this.setMethod("GET");
                    this.site = site;
                    this.contentTypeName = contentName;
                }
                GetDefaultPageTemplateRequest.prototype.getParams = function () {
                    return {
                        siteId: this.site.toString(),
                        contentTypeName: this.contentTypeName.toString()
                    };
                };
                GetDefaultPageTemplateRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "default");
                };
                GetDefaultPageTemplateRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        if (response.hasResult()) {
                            return _this.fromJsonToContent(response.getResult());
                        }
                        else {
                            return null;
                        }
                    });
                };
                return GetDefaultPageTemplateRequest;
            })(page.PageTemplateResourceRequest);
            page.GetDefaultPageTemplateRequest = GetDefaultPageTemplateRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var IsRenderableRequest = (function (_super) {
                __extends(IsRenderableRequest, _super);
                function IsRenderableRequest(contentId) {
                    _super.call(this);
                    this.setMethod("GET");
                    this.contentId = contentId;
                }
                IsRenderableRequest.prototype.setContentId = function (value) {
                    this.contentId = value;
                    return this;
                };
                IsRenderableRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString()
                    };
                };
                IsRenderableRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "isRenderable");
                };
                IsRenderableRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return response.getResult();
                    });
                };
                return IsRenderableRequest;
            })(page.PageTemplateResourceRequest);
            page.IsRenderableRequest = IsRenderableRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageResourceRequest = (function (_super) {
                __extends(PageResourceRequest, _super);
                function PageResourceRequest() {
                    _super.call(this);
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page");
                    this.propertyIdProvider = api.Client.get().getPropertyIdProvider();
                }
                PageResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                PageResourceRequest.prototype.fromJsonToContent = function (json) {
                    return api.content.Content.fromJson(json, this.propertyIdProvider);
                };
                return PageResourceRequest;
            })(api.rest.ResourceRequest);
            page.PageResourceRequest = PageResourceRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var CreatePageRequest = (function (_super) {
                __extends(CreatePageRequest, _super);
                function CreatePageRequest(contentId) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                    this.contentId = contentId;
                }
                CreatePageRequest.prototype.setController = function (controller) {
                    this.controller = controller;
                    return this;
                };
                CreatePageRequest.prototype.setPageTemplateKey = function (pageTemplateKey) {
                    this.template = pageTemplateKey;
                    return this;
                };
                CreatePageRequest.prototype.setConfig = function (config) {
                    this.config = config;
                    return this;
                };
                CreatePageRequest.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                CreatePageRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString(),
                        controller: this.controller ? this.controller.toString() : null,
                        template: this.template ? this.template.toString() : null,
                        config: this.config ? this.config.toJson() : null,
                        regions: this.regions != null ? this.regions.toJson() : null
                    };
                };
                CreatePageRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "create");
                };
                CreatePageRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                    });
                };
                return CreatePageRequest;
            })(page.PageResourceRequest);
            page.CreatePageRequest = CreatePageRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var UpdatePageRequest = (function (_super) {
                __extends(UpdatePageRequest, _super);
                function UpdatePageRequest(contentId) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                    this.contentId = contentId;
                }
                UpdatePageRequest.prototype.setController = function (controller) {
                    this.controller = controller;
                    return this;
                };
                UpdatePageRequest.prototype.setPageTemplateKey = function (pageTemplateKey) {
                    this.template = pageTemplateKey;
                    return this;
                };
                UpdatePageRequest.prototype.setConfig = function (config) {
                    this.config = config;
                    return this;
                };
                UpdatePageRequest.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                UpdatePageRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString(),
                        controller: this.controller ? this.controller.toString() : null,
                        template: this.template ? this.template.toString() : null,
                        config: this.config ? this.config.toJson() : null,
                        regions: this.regions != null ? this.regions.toJson() : null
                    };
                };
                UpdatePageRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "update");
                };
                UpdatePageRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                    });
                };
                return UpdatePageRequest;
            })(page.PageResourceRequest);
            page.UpdatePageRequest = UpdatePageRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DeletePageRequest = (function (_super) {
                __extends(DeletePageRequest, _super);
                function DeletePageRequest(contentId) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.contentId = contentId;
                }
                DeletePageRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString()
                    };
                };
                DeletePageRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "delete");
                };
                DeletePageRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                    });
                };
                return DeletePageRequest;
            })(page.PageResourceRequest);
            page.DeletePageRequest = DeletePageRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PropertyTree = api.data.PropertyTree;
            var Page = (function () {
                function Page(builder) {
                    this.controller = builder.controller;
                    this.template = builder.template;
                    this.regions = builder.regions;
                    this.config = builder.config;
                }
                Page.prototype.hasController = function () {
                    return !!this.controller;
                };
                Page.prototype.getController = function () {
                    return this.controller;
                };
                Page.prototype.hasTemplate = function () {
                    return !!this.template;
                };
                Page.prototype.getTemplate = function () {
                    return this.template;
                };
                Page.prototype.hasRegions = function () {
                    return this.regions != null;
                };
                Page.prototype.getRegions = function () {
                    return this.regions;
                };
                Page.prototype.hasConfig = function () {
                    return this.config != null;
                };
                Page.prototype.getConfig = function () {
                    return this.config;
                };
                Page.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Page)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.controller, other.controller)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.template, other.template)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.regions, other.regions)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.config, other.config)) {
                        return false;
                    }
                    return true;
                };
                Page.prototype.clone = function () {
                    return new PageBuilder(this).build();
                };
                return Page;
            })();
            page.Page = Page;
            var PageBuilder = (function () {
                function PageBuilder(source) {
                    if (source) {
                        this.controller = source.getController();
                        this.template = source.getTemplate();
                        this.regions = source.getRegions() ? source.getRegions().clone() : null;
                        this.config = source.getConfig() ? source.getConfig().copy() : null;
                    }
                }
                PageBuilder.prototype.fromJson = function (json, propertyIdProvider) {
                    this.setController(json.controller ? page.DescriptorKey.fromString(json.controller) : null);
                    this.setTemplate(json.template ? page.PageTemplateKey.fromString(json.template) : null);
                    this.setRegions(json.regions != null ? api.content.page.region.Regions.create().fromJson(json.regions, propertyIdProvider, null).build() : null);
                    this.setConfig(json.config != null ? PropertyTree.fromJson(json.config, propertyIdProvider) : null);
                    return this;
                };
                PageBuilder.prototype.setController = function (value) {
                    this.controller = value;
                    return this;
                };
                PageBuilder.prototype.setTemplate = function (value) {
                    this.template = value;
                    return this;
                };
                PageBuilder.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                PageBuilder.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                PageBuilder.prototype.build = function () {
                    return new Page(this);
                };
                return PageBuilder;
            })();
            page.PageBuilder = PageBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            (function (PageMode) {
                PageMode[PageMode["AUTOMATIC"] = 0] = "AUTOMATIC";
                PageMode[PageMode["FORCED_TEMPLATE"] = 1] = "FORCED_TEMPLATE";
                PageMode[PageMode["FORCED_CONTROLLER"] = 2] = "FORCED_CONTROLLER";
                PageMode[PageMode["NO_CONTROLLER"] = 3] = "NO_CONTROLLER";
            })(page.PageMode || (page.PageMode = {}));
            var PageMode = page.PageMode;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageModeChangedEvent = (function () {
                function PageModeChangedEvent(previousMode, newMode) {
                    this.previousMode = previousMode;
                    this.newMode = newMode;
                }
                PageModeChangedEvent.prototype.getPreviousMode = function () {
                    return this.previousMode;
                };
                PageModeChangedEvent.prototype.getNewMode = function () {
                    return this.newMode;
                };
                return PageModeChangedEvent;
            })();
            page.PageModeChangedEvent = PageModeChangedEvent;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PropertyTree = api.data.PropertyTree;
            var SetController = (function () {
                function SetController(eventSource) {
                    this.eventSource = eventSource;
                }
                SetController.prototype.setDescriptor = function (value) {
                    this.descriptor = value;
                    return this;
                };
                SetController.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                SetController.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                return SetController;
            })();
            page.SetController = SetController;
            var SetTemplate = (function () {
                function SetTemplate(eventSource) {
                    this.eventSource = eventSource;
                }
                SetTemplate.prototype.setTemplate = function (template, descriptor) {
                    this.template = template;
                    this.descriptor = descriptor;
                    return this;
                };
                SetTemplate.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                SetTemplate.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                return SetTemplate;
            })();
            page.SetTemplate = SetTemplate;
            var PageModel = (function () {
                function PageModel(liveEditModel, defaultTemplate, defaultTemplateDescriptor) {
                    var _this = this;
                    this.pageModeChangedListeners = [];
                    this.propertyChangedListeners = [];
                    this.componentPropertyChangedListeners = [];
                    this.ignorePropertyChanges = false;
                    this.liveEditModel = liveEditModel;
                    this.defaultTemplate = defaultTemplate;
                    this.defaultTemplateDescriptor = defaultTemplateDescriptor;
                    this.configPropertyChangedHandler = function (event) {
                        if (!_this.ignorePropertyChanges) {
                            if (_this.mode == 0 /* AUTOMATIC */) {
                                var setTemplate = new SetTemplate(_this).setTemplate(_this.defaultTemplate, _this.defaultTemplateDescriptor);
                                _this.setTemplate(setTemplate);
                            }
                        }
                    };
                    this.regionsChangedEventHandler = function (event) {
                        if (!_this.ignorePropertyChanges) {
                            if (_this.mode == 0 /* AUTOMATIC */) {
                                var setTemplate = new SetTemplate(_this).setTemplate(_this.defaultTemplate, _this.defaultTemplateDescriptor);
                                _this.setTemplate(setTemplate);
                            }
                        }
                    };
                    this.componentPropertyChangedEventHandler = function (event) {
                        if (!_this.isPageTemplate() && _this.getMode() == 0 /* AUTOMATIC */) {
                            _this.initializePageFromDefault(_this);
                        }
                        _this.componentPropertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                }
                PageModel.prototype.setIgnorePropertyChanges = function (value) {
                    this.ignorePropertyChanges = value;
                };
                PageModel.prototype.initializePageFromDefault = function (eventSource) {
                    var skip = false;
                    if (this.hasTemplate()) {
                        skip = true;
                    }
                    if (this.hasController()) {
                        skip = true;
                    }
                    if (!skip) {
                        var setTemplate = new SetTemplate(eventSource).setTemplate(this.defaultTemplate, this.defaultTemplateDescriptor);
                        this.setTemplate(setTemplate);
                    }
                };
                PageModel.prototype.reset = function (eventSource) {
                    if (this.liveEditModel.getContent().isPageTemplate()) {
                        var setController = new SetController(eventSource).setDescriptor(null).setConfig(new PropertyTree(api.Client.get().getPropertyIdProvider())).setRegions(api.content.page.region.Regions.create().build());
                        this.setController(setController);
                    }
                    else {
                        this.setAutomaticTemplate(eventSource);
                    }
                };
                PageModel.prototype.setMode = function (value) {
                    var oldValue = this.mode;
                    this.mode = value;
                    if (this.mode != oldValue) {
                        this.notifyPageModeChanged(oldValue, this.mode);
                    }
                };
                PageModel.prototype.setController = function (setController) {
                    var oldControllerKey = this.controller ? this.controller.getKey() : null;
                    this.controller = setController.descriptor;
                    if (setController.descriptor) {
                        this.setMode(2 /* FORCED_CONTROLLER */);
                    }
                    else {
                        this.setMode(3 /* NO_CONTROLLER */);
                    }
                    if (setController.config) {
                        this.setConfig(setController.config, setController.eventSource);
                    }
                    if (setController.regions) {
                        this.setRegions(setController.regions, setController.eventSource);
                    }
                    if (this.regions && setController.descriptor) {
                        this.regions.changeRegionsTo(setController.descriptor.getRegions());
                    }
                    var newControllerKey = this.controller ? this.controller.getKey() : null;
                    if (!api.ObjectHelper.equals(oldControllerKey, newControllerKey)) {
                        this.setIgnorePropertyChanges(true);
                        this.notifyPropertyChanged("controller", oldControllerKey, newControllerKey, setController.eventSource);
                        this.setIgnorePropertyChanges(false);
                    }
                    return this;
                };
                PageModel.prototype.setAutomaticTemplate = function (eventSource) {
                    var config = this.defaultTemplate.hasConfig() ? this.defaultTemplate.getConfig().copy() : new PropertyTree(api.Client.get().getPropertyIdProvider());
                    var regions = this.defaultTemplate.hasRegions() ? this.defaultTemplate.getRegions().clone() : api.content.page.region.Regions.create().build();
                    var setTemplate = new SetTemplate(eventSource).setTemplate(null, this.defaultTemplateDescriptor).setRegions(regions).setConfig(config);
                    this.setTemplate(setTemplate);
                    return this;
                };
                PageModel.prototype.setTemplate = function (setTemplate) {
                    var oldTemplateKey = this.template ? this.template.getKey() : null;
                    if (setTemplate.template) {
                        this.setMode(1 /* FORCED_TEMPLATE */);
                    }
                    else {
                        this.setMode(0 /* AUTOMATIC */);
                    }
                    this.template = setTemplate.template;
                    this.templateDescriptor = setTemplate.descriptor;
                    if (setTemplate.config) {
                        this.setConfig(setTemplate.config, setTemplate.eventSource);
                    }
                    if (setTemplate.regions) {
                        this.setRegions(setTemplate.regions, setTemplate.eventSource);
                    }
                    if (this.regions) {
                        this.regions.changeRegionsTo(setTemplate.descriptor.getRegions());
                    }
                    var newTemplateKey = this.template ? this.template.getKey() : null;
                    if (!api.ObjectHelper.equals(oldTemplateKey, newTemplateKey)) {
                        this.setIgnorePropertyChanges(true);
                        this.notifyPropertyChanged("template", oldTemplateKey, newTemplateKey, setTemplate.eventSource);
                        this.setIgnorePropertyChanges(false);
                    }
                    return this;
                };
                PageModel.prototype.setRegions = function (value, eventOrigin) {
                    var oldValue = this.regions;
                    if (oldValue) {
                        this.unregisterRegionsListeners(oldValue);
                    }
                    this.regions = value;
                    this.registerRegionsListeners(this.regions);
                    this.setIgnorePropertyChanges(true);
                    this.notifyPropertyChanged("regions", oldValue, value, eventOrigin);
                    this.setIgnorePropertyChanges(false);
                    return this;
                };
                PageModel.prototype.setConfig = function (value, eventOrigin) {
                    var oldValue = this.config;
                    if (oldValue) {
                        oldValue.unChanged(this.configPropertyChangedHandler);
                    }
                    this.config = value;
                    this.config.onChanged(this.configPropertyChangedHandler);
                    this.setIgnorePropertyChanges(true);
                    this.notifyPropertyChanged("config", oldValue, value, eventOrigin);
                    this.setIgnorePropertyChanges(false);
                    return this;
                };
                PageModel.prototype.getPage = function () {
                    if (this.mode == 0 /* AUTOMATIC */) {
                        return null;
                    }
                    else if (this.mode == 1 /* FORCED_TEMPLATE */) {
                        var regionsUnchanged = this.regions.equals(this.defaultTemplate.getRegions());
                        var regions = regionsUnchanged ? null : this.regions;
                        var configUnchanged = this.config.equals(this.defaultTemplate.getConfig());
                        var config = configUnchanged ? null : this.config;
                        return new page.PageBuilder().setTemplate(this.getTemplateKey()).setRegions(regions).setConfig(config).build();
                    }
                    else if (this.mode == 2 /* FORCED_CONTROLLER */) {
                        return new page.PageBuilder().setController(this.controller.getKey()).setRegions(this.regions).setConfig(this.config).build();
                    }
                    else if (this.mode == 3 /* NO_CONTROLLER */) {
                        return null;
                    }
                    else {
                        throw new Error("Page mode not supported: " + this.mode);
                    }
                };
                PageModel.prototype.getDefaultPageTemplate = function () {
                    return this.defaultTemplate;
                };
                PageModel.prototype.getDefaultPageTemplateController = function () {
                    return this.defaultTemplateDescriptor;
                };
                PageModel.prototype.getMode = function () {
                    return this.mode;
                };
                PageModel.prototype.isPageTemplate = function () {
                    return this.liveEditModel.getContent().isPageTemplate();
                };
                PageModel.prototype.hasController = function () {
                    return !!this.controller;
                };
                PageModel.prototype.getController = function () {
                    return this.controller;
                };
                PageModel.prototype.hasTemplate = function () {
                    return !!this.template;
                };
                PageModel.prototype.getTemplateKey = function () {
                    return this.template ? this.template.getKey() : null;
                };
                PageModel.prototype.getTemplate = function () {
                    return this.template;
                };
                PageModel.prototype.getTemplateDescriptor = function () {
                    return this.templateDescriptor;
                };
                PageModel.prototype.getRegions = function () {
                    return this.regions;
                };
                PageModel.prototype.getConfig = function () {
                    return this.config;
                };
                PageModel.prototype.registerRegionsListeners = function (regions) {
                    regions.onComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    regions.onChanged(this.regionsChangedEventHandler);
                };
                PageModel.prototype.unregisterRegionsListeners = function (regions) {
                    regions.unComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    regions.unChanged(this.regionsChangedEventHandler);
                };
                PageModel.prototype.onPageModeChanged = function (listener) {
                    this.pageModeChangedListeners.push(listener);
                };
                PageModel.prototype.unPageModeChanged = function (listener) {
                    this.pageModeChangedListeners = this.pageModeChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                PageModel.prototype.notifyPageModeChanged = function (oldValue, newValue) {
                    var event = new page.PageModeChangedEvent(oldValue, newValue);
                    this.pageModeChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                PageModel.prototype.onPropertyChanged = function (listener) {
                    this.propertyChangedListeners.push(listener);
                };
                PageModel.prototype.unPropertyChanged = function (listener) {
                    this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                PageModel.prototype.notifyPropertyChanged = function (property, oldValue, newValue, origin) {
                    var event = new api.PropertyChangedEvent(property, oldValue, newValue, origin);
                    this.propertyChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                PageModel.prototype.onComponentPropertyChangedEvent = function (listener) {
                    this.componentPropertyChangedListeners.push(listener);
                };
                PageModel.prototype.unComponentPropertyChangedEvent = function (listener) {
                    this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                return PageModel;
            })();
            page.PageModel = PageModel;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DescriptorByDisplayNameComparator = (function () {
                function DescriptorByDisplayNameComparator() {
                }
                DescriptorByDisplayNameComparator.prototype.compare = function (a, b) {
                    if (!a) {
                        return 1;
                    }
                    else {
                        var firstName = a.getDisplayName();
                    }
                    if (!b) {
                        return -1;
                    }
                    else {
                        var secondName = b.getDisplayName();
                    }
                    return firstName.localeCompare(secondName);
                };
                return DescriptorByDisplayNameComparator;
            })();
            page.DescriptorByDisplayNameComparator = DescriptorByDisplayNameComparator;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateByDisplayNameComparator = (function () {
                function PageTemplateByDisplayNameComparator() {
                }
                PageTemplateByDisplayNameComparator.prototype.compare = function (a, b) {
                    if (!a) {
                        return 1;
                    }
                    else {
                        var firstName = a.getDisplayName();
                    }
                    if (!b) {
                        return -1;
                    }
                    else {
                        var secondName = b.getDisplayName();
                    }
                    return firstName.localeCompare(secondName);
                };
                return PageTemplateByDisplayNameComparator;
            })();
            page.PageTemplateByDisplayNameComparator = PageTemplateByDisplayNameComparator;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentPath = (function () {
                    function ComponentPath(regionAndComponentList) {
                        var _this = this;
                        this.regionAndComponentList = regionAndComponentList;
                        this.refString = "";
                        this.regionAndComponentList.forEach(function (regionAndComponent, index) {
                            _this.refString += regionAndComponent.toString();
                            if (index < _this.regionAndComponentList.length - 1) {
                                _this.refString += ComponentPath.DIVIDER;
                            }
                        });
                    }
                    ComponentPath.prototype.numberOfLevels = function () {
                        return this.regionAndComponentList.length;
                    };
                    ComponentPath.prototype.getFirstLevel = function () {
                        return this.regionAndComponentList[0];
                    };
                    ComponentPath.prototype.getLastLevel = function () {
                        return this.regionAndComponentList[this.regionAndComponentList.length - 1];
                    };
                    ComponentPath.prototype.getLevels = function () {
                        return this.regionAndComponentList;
                    };
                    ComponentPath.prototype.getComponentIndex = function () {
                        return this.getLastLevel().getComponentIndex();
                    };
                    ComponentPath.prototype.getRegionPath = function () {
                        var _this = this;
                        var regionPathAsString = "";
                        this.regionAndComponentList.forEach(function (regionAndComponent, index) {
                            if (index == _this.regionAndComponentList.length - 1) {
                                regionPathAsString += regionAndComponent.getRegionName();
                            }
                            else {
                                regionPathAsString += regionAndComponent.toString();
                                regionPathAsString += "/";
                            }
                        });
                        return region.RegionPath.fromString(regionPathAsString);
                    };
                    ComponentPath.prototype.removeFirstLevel = function () {
                        if (this.numberOfLevels() <= 1) {
                            return null;
                        }
                        var newRegionAndComponentList = [];
                        for (var i = 1; i < this.regionAndComponentList.length; i++) {
                            newRegionAndComponentList.push(this.regionAndComponentList[i]);
                        }
                        return new ComponentPath(newRegionAndComponentList);
                    };
                    ComponentPath.prototype.toString = function () {
                        return this.refString;
                    };
                    ComponentPath.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ComponentPath)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                            return false;
                        }
                        return true;
                    };
                    ComponentPath.fromString = function (str) {
                        if (!str) {
                            return null;
                        }
                        var elements = api.util.StringHelper.removeEmptyStrings(str.split(ComponentPath.DIVIDER));
                        var regionAndComponentList = [];
                        for (var i = 0; i < elements.length - 1; i += 2) {
                            var regionName = elements[i];
                            var componentIndexAsString = elements[i + 1];
                            var regionAndComponent = new ComponentPathRegionAndComponent(regionName, parseInt(componentIndexAsString));
                            regionAndComponentList.push(regionAndComponent);
                        }
                        return new ComponentPath(regionAndComponentList);
                    };
                    ComponentPath.fromRegionPathAndComponentIndex = function (regionPath, componentIndex) {
                        api.util.assertNotNull(regionPath, "regionPath cannot be null");
                        api.util.assert(componentIndex >= 0, "componentIndex must be zero or more");
                        var regionAndComponentList = [];
                        if (regionPath.getParentComponentPath()) {
                            regionPath.getParentComponentPath().regionAndComponentList.forEach(function (regionAndComponent) {
                                regionAndComponentList.push(regionAndComponent);
                            });
                        }
                        regionAndComponentList.push(new ComponentPathRegionAndComponent(regionPath.getRegionName(), componentIndex));
                        return new ComponentPath(regionAndComponentList);
                    };
                    ComponentPath.DIVIDER = "/";
                    return ComponentPath;
                })();
                region.ComponentPath = ComponentPath;
                var ComponentPathRegionAndComponent = (function () {
                    function ComponentPathRegionAndComponent(regionName, componentIndex) {
                        this.regionName = regionName;
                        this.componentIndex = componentIndex;
                        this.refString = regionName + ComponentPathRegionAndComponent.DIVIDER + this.componentIndex;
                    }
                    ComponentPathRegionAndComponent.prototype.getRegionName = function () {
                        return this.regionName;
                    };
                    ComponentPathRegionAndComponent.prototype.getComponentIndex = function () {
                        return this.componentIndex;
                    };
                    ComponentPathRegionAndComponent.prototype.toString = function () {
                        return this.refString;
                    };
                    ComponentPathRegionAndComponent.DIVIDER = "/";
                    return ComponentPathRegionAndComponent;
                })();
                region.ComponentPathRegionAndComponent = ComponentPathRegionAndComponent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentName = (function () {
                    function ComponentName(value) {
                        api.util.assertNotNull(value, "ComponentName value cannot be null");
                        this.value = value;
                    }
                    ComponentName.prototype.hasCountPostfix = function () {
                        var countDelimiterIndex = this.value.lastIndexOf(ComponentName.COUNT_DELIMITER);
                        return countDelimiterIndex > 0 && countDelimiterIndex <= this.value.length - 2;
                    };
                    ComponentName.prototype.removeCountPostfix = function () {
                        if (!this.hasCountPostfix()) {
                            return this;
                        }
                        var nameWithoutCountPostfix = this.value.substring(0, this.value.lastIndexOf(ComponentName.COUNT_DELIMITER));
                        return new ComponentName(nameWithoutCountPostfix);
                    };
                    ComponentName.prototype.isDuplicateOf = function (other) {
                        if (this.value == other.value) {
                            return true;
                        }
                        if (!this.hasCountPostfix()) {
                            return false;
                        }
                        var nameWithoutCountPostfix = this.removeCountPostfix();
                        return nameWithoutCountPostfix.equals(other);
                    };
                    ComponentName.prototype.createDuplicate = function (count) {
                        var newValue = this.value + ComponentName.COUNT_DELIMITER + "" + count;
                        return new ComponentName(newValue);
                    };
                    ComponentName.prototype.toString = function () {
                        return this.value;
                    };
                    ComponentName.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ComponentName)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                            return false;
                        }
                        return true;
                    };
                    ComponentName.COUNT_DELIMITER = "-";
                    return ComponentName;
                })();
                region.ComponentName = ComponentName;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentType = (function () {
                    function ComponentType(shortName) {
                        ComponentType.shortNameToInstance[shortName] = this;
                        this.shortName = shortName;
                    }
                    ComponentType.prototype.getShortName = function () {
                        return this.shortName;
                    };
                    ComponentType.prototype.newComponentBuilder = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    ComponentType.byShortName = function (shortName) {
                        return ComponentType.shortNameToInstance[shortName];
                    };
                    ComponentType.shortNameToInstance = {};
                    return ComponentType;
                })();
                region.ComponentType = ComponentType;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentChangedEvent = (function () {
                    function ComponentChangedEvent(path) {
                        this.path = path;
                    }
                    ComponentChangedEvent.prototype.getPath = function () {
                        return this.path;
                    };
                    return ComponentChangedEvent;
                })();
                region.ComponentChangedEvent = ComponentChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentPropertyChangedEvent = (function (_super) {
                    __extends(ComponentPropertyChangedEvent, _super);
                    function ComponentPropertyChangedEvent(builder) {
                        _super.call(this, builder.component.getPath());
                        this.component = builder.component;
                        this.propertyName = builder.propertyName;
                    }
                    ComponentPropertyChangedEvent.prototype.getComponent = function () {
                        return this.component;
                    };
                    ComponentPropertyChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    ComponentPropertyChangedEvent.create = function () {
                        return new ComponentPropertyChangedEventBuilder();
                    };
                    return ComponentPropertyChangedEvent;
                })(region.ComponentChangedEvent);
                region.ComponentPropertyChangedEvent = ComponentPropertyChangedEvent;
                var ComponentPropertyChangedEventBuilder = (function () {
                    function ComponentPropertyChangedEventBuilder() {
                    }
                    ComponentPropertyChangedEventBuilder.prototype.setPropertyName = function (value) {
                        this.propertyName = value;
                        return this;
                    };
                    ComponentPropertyChangedEventBuilder.prototype.setComponent = function (value) {
                        this.component = value;
                        return this;
                    };
                    ComponentPropertyChangedEventBuilder.prototype.build = function () {
                        return new ComponentPropertyChangedEvent(this);
                    };
                    return ComponentPropertyChangedEventBuilder;
                })();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentResetEvent = (function (_super) {
                    __extends(ComponentResetEvent, _super);
                    function ComponentResetEvent(componentPath) {
                        _super.call(this, componentPath);
                    }
                    return ComponentResetEvent;
                })(region.ComponentChangedEvent);
                region.ComponentResetEvent = ComponentResetEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentPropertyValueChangedEvent = (function (_super) {
                    __extends(ComponentPropertyValueChangedEvent, _super);
                    function ComponentPropertyValueChangedEvent(path, propertyName) {
                        _super.call(this, path);
                        this.propertyName = propertyName;
                    }
                    ComponentPropertyValueChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    return ComponentPropertyValueChangedEvent;
                })(region.ComponentChangedEvent);
                region.ComponentPropertyValueChangedEvent = ComponentPropertyValueChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Component = (function () {
                    function Component(builder) {
                        this.index = -1;
                        this.changedListeners = [];
                        this.propertyChangedListeners = [];
                        this.propertyValueChangedListeners = [];
                        this.resetListeners = [];
                        this.name = builder.name;
                        this.index = builder.index;
                        this.parent = builder.parent;
                    }
                    Component.prototype.setParent = function (parent) {
                        this.parent = parent;
                    };
                    Component.prototype.setIndex = function (value) {
                        this.index = value;
                    };
                    Component.prototype.getIndex = function () {
                        return this.index;
                    };
                    Component.prototype.hasPath = function () {
                        return this.parent && this.index >= 0;
                    };
                    Component.prototype.getPath = function () {
                        return this.hasPath() ? region.ComponentPath.fromRegionPathAndComponentIndex(this.parent.getPath(), this.index) : null;
                    };
                    Component.prototype.getName = function () {
                        return this.name;
                    };
                    Component.prototype.setName = function (newValue) {
                        var oldValue = this.name;
                        this.name = newValue;
                        if (!api.ObjectHelper.equals(oldValue, newValue)) {
                            this.notifyPropertyChanged(Component.PROPERTY_NAME);
                        }
                    };
                    Component.prototype.doReset = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    Component.prototype.reset = function () {
                        this.doReset();
                        this.notifyResetEvent();
                    };
                    Component.prototype.isEmpty = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    Component.prototype.getParent = function () {
                        return this.parent;
                    };
                    Component.prototype.duplicate = function () {
                        var duplicateName = this.getName();
                        var duplicatedComponent = this.clone(true);
                        duplicatedComponent.setName(duplicateName);
                        return duplicatedComponent;
                    };
                    Component.prototype.remove = function () {
                        this.parent.removeComponent(this);
                    };
                    Component.prototype.toJson = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    Component.prototype.toString = function () {
                        return "Component[" + (this.name ? this.name.toString() : "") + "]";
                    };
                    Component.prototype.toComponentJson = function () {
                        return {
                            "name": this.name ? this.name.toString() : null
                        };
                    };
                    Component.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Component)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.equals(this.name, other.name)) {
                            return false;
                        }
                        return true;
                    };
                    Component.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        throw new Error("Must be implemented by inheritors");
                    };
                    Component.prototype.onChanged = function (listener) {
                        this.changedListeners.push(listener);
                    };
                    Component.prototype.unChanged = function (listener) {
                        this.changedListeners = this.changedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyChangedEvent = function (event) {
                        this.changedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Component.prototype.onReset = function (listener) {
                        this.resetListeners.push(listener);
                    };
                    Component.prototype.unReset = function (listener) {
                        this.resetListeners = this.resetListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyResetEvent = function () {
                        var event = new region.ComponentResetEvent(this.getPath());
                        this.resetListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Component.prototype.onPropertyChanged = function (listener) {
                        this.propertyChangedListeners.push(listener);
                    };
                    Component.prototype.unPropertyChanged = function (listener) {
                        this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyPropertyChanged = function (propertyName) {
                        var event = region.ComponentPropertyChangedEvent.create().setComponent(this).setPropertyName(propertyName).build();
                        this.propertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Component.prototype.forwardComponentPropertyChangedEvent = function (event) {
                        this.propertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Component.prototype.onPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners.push(listener);
                    };
                    Component.prototype.unPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyPropertyValueChanged = function (propertyName) {
                        var event = new region.ComponentPropertyValueChangedEvent(this.getPath(), propertyName);
                        this.propertyValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Component.PROPERTY_NAME = 'name';
                    return Component;
                })();
                region.Component = Component;
                var ComponentBuilder = (function () {
                    function ComponentBuilder(source) {
                        if (source) {
                            this.name = source.getName();
                            this.parent = source.getParent();
                            this.index = source.getIndex();
                        }
                    }
                    ComponentBuilder.prototype.setIndex = function (value) {
                        this.index = value;
                        return this;
                    };
                    ComponentBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    ComponentBuilder.prototype.setParent = function (value) {
                        this.parent = value;
                        return this;
                    };
                    ComponentBuilder.prototype.build = function () {
                        throw new Error("Must be implemented by inheritor");
                    };
                    return ComponentBuilder;
                })();
                region.ComponentBuilder = ComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var ComponentFactory = (function () {
                    function ComponentFactory() {
                    }
                    ComponentFactory.createFromJson = function (json, componentIndex, region, propertyIdProvider) {
                        if (json.PartComponent) {
                            return new _region.PartComponentBuilder().fromJson(json.PartComponent, region, propertyIdProvider).build();
                        }
                        else if (json.ImageComponent) {
                            return new _region.ImageComponentBuilder().fromJson(json.ImageComponent, region, propertyIdProvider).build();
                        }
                        else if (json.LayoutComponent) {
                            var layoutComponentBuilder = new _region.LayoutComponentBuilder();
                            layoutComponentBuilder.setIndex(componentIndex);
                            return layoutComponentBuilder.fromJson(json.LayoutComponent, region, propertyIdProvider);
                        }
                        else if (json.TextComponent) {
                            return new _region.TextComponentBuilder().fromJson(json.TextComponent, region).setIndex(componentIndex).build();
                        }
                        else {
                            throw new Error("Not a component that can be placed in a Region: " + json);
                        }
                    };
                    return ComponentFactory;
                })();
                _region.ComponentFactory = ComponentFactory;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PropertyTree = api.data.PropertyTree;
                var DescriptorBasedComponent = (function (_super) {
                    __extends(DescriptorBasedComponent, _super);
                    function DescriptorBasedComponent(builder) {
                        var _this = this;
                        _super.call(this, builder);
                        this.descriptor = builder.descriptor;
                        this.config = builder.config;
                        this.configChangedHandler = function (event) {
                            if (DescriptorBasedComponent.debug) {
                                console.debug("DescriptorBasedComponent[" + _this.getPath().toString() + "].config.onChanged: ", event);
                            }
                            if (!_this.disableEventForwarding) {
                                _this.notifyPropertyValueChanged(DescriptorBasedComponent.PROPERTY_CONFIG);
                            }
                        };
                        this.config.onChanged(this.configChangedHandler);
                    }
                    DescriptorBasedComponent.prototype.setDisableEventForwarding = function (value) {
                        this.disableEventForwarding = value;
                    };
                    DescriptorBasedComponent.prototype.hasDescriptor = function () {
                        return !!this.descriptor;
                    };
                    DescriptorBasedComponent.prototype.getDescriptor = function () {
                        return this.descriptor;
                    };
                    DescriptorBasedComponent.prototype.setDescriptor = function (descriptorKey, descriptor) {
                        var oldValue = this.descriptor;
                        this.descriptor = descriptorKey;
                        this.alignNameWithDescriptor(descriptor);
                        if (!api.ObjectHelper.equals(oldValue, descriptorKey)) {
                            this.notifyPropertyChanged(DescriptorBasedComponent.PROPERTY_DESCRIPTOR);
                        }
                        this.setConfig(new PropertyTree(this.config.getIdProvider()));
                    };
                    DescriptorBasedComponent.prototype.setConfig = function (config) {
                        var oldValue = this.config;
                        if (oldValue) {
                            this.config.unChanged(this.configChangedHandler);
                        }
                        this.config = config;
                        this.config.onChanged(this.configChangedHandler);
                        if (!api.ObjectHelper.equals(oldValue, config)) {
                            this.notifyPropertyChanged(DescriptorBasedComponent.PROPERTY_CONFIG);
                        }
                    };
                    DescriptorBasedComponent.prototype.alignNameWithDescriptor = function (descriptor) {
                        var newName = descriptor ? new region.ComponentName(descriptor.getDisplayName()) : null;
                        this.setName(newName);
                    };
                    DescriptorBasedComponent.prototype.getConfig = function () {
                        return this.config;
                    };
                    DescriptorBasedComponent.prototype.doReset = function () {
                        this.setDescriptor(null, null);
                    };
                    DescriptorBasedComponent.prototype.toComponentJson = function () {
                        return {
                            "name": this.getName() ? this.getName().toString() : null,
                            "descriptor": this.descriptor != null ? this.descriptor.toString() : null,
                            "config": this.config != null ? this.config.toJson() : null
                        };
                    };
                    DescriptorBasedComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DescriptorBasedComponent)) {
                            return false;
                        }
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.equals(this.descriptor, other.descriptor)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.config, other.config)) {
                            return false;
                        }
                        return true;
                    };
                    DescriptorBasedComponent.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        throw new Error("Must be implemented by inheritors");
                    };
                    DescriptorBasedComponent.debug = false;
                    DescriptorBasedComponent.PROPERTY_DESCRIPTOR = 'descriptor';
                    DescriptorBasedComponent.PROPERTY_CONFIG = 'config';
                    return DescriptorBasedComponent;
                })(region.Component);
                region.DescriptorBasedComponent = DescriptorBasedComponent;
                var DescriptorBasedComponentBuilder = (function (_super) {
                    __extends(DescriptorBasedComponentBuilder, _super);
                    function DescriptorBasedComponentBuilder(source, generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        _super.call(this, source);
                        if (source) {
                            this.descriptor = source.getDescriptor();
                            this.config = source.getConfig() ? source.getConfig().copy(generateNewPropertyIds) : null;
                        }
                        else {
                            this.config = new PropertyTree(api.Client.get().getPropertyIdProvider());
                        }
                    }
                    DescriptorBasedComponentBuilder.prototype.setDescriptor = function (value) {
                        this.descriptor = value;
                        return this;
                    };
                    DescriptorBasedComponentBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    DescriptorBasedComponentBuilder.prototype.build = function () {
                        throw new Error("Must be implemented by inheritor");
                    };
                    return DescriptorBasedComponentBuilder;
                })(region.ComponentBuilder);
                region.DescriptorBasedComponentBuilder = DescriptorBasedComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionsChangedEvent = (function () {
                    function RegionsChangedEvent() {
                    }
                    return RegionsChangedEvent;
                })();
                region.RegionsChangedEvent = RegionsChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionAddedEvent = (function (_super) {
                    __extends(RegionAddedEvent, _super);
                    function RegionAddedEvent(regionPath) {
                        _super.call(this);
                        this.regionPath = regionPath;
                    }
                    RegionAddedEvent.prototype.getRegionPath = function () {
                        return this.regionPath;
                    };
                    return RegionAddedEvent;
                })(region.RegionsChangedEvent);
                region.RegionAddedEvent = RegionAddedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionRemovedEvent = (function (_super) {
                    __extends(RegionRemovedEvent, _super);
                    function RegionRemovedEvent(regionPath) {
                        _super.call(this);
                        this.regionPath = regionPath;
                    }
                    RegionRemovedEvent.prototype.getRegionPath = function () {
                        return this.regionPath;
                    };
                    return RegionRemovedEvent;
                })(region.RegionsChangedEvent);
                region.RegionRemovedEvent = RegionRemovedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionChangedEvent = (function (_super) {
                    __extends(RegionChangedEvent, _super);
                    function RegionChangedEvent(regionPath) {
                        _super.call(this);
                        this.regionPath = regionPath;
                    }
                    RegionChangedEvent.prototype.getRegionPath = function () {
                        return this.regionPath;
                    };
                    return RegionChangedEvent;
                })(region.RegionsChangedEvent);
                region.RegionChangedEvent = RegionChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var Regions = (function () {
                    function Regions(builder) {
                        var _this = this;
                        this.regionByName = {};
                        this.changedListeners = [];
                        this.componentPropertyChangedListeners = [];
                        this.regionChangedListeners = [];
                        this.regionAddedListeners = [];
                        this.regionRemovedListeners = [];
                        this.regionChangedEventHandler = function (event) { return _this.notifyRegionChanged(event.getPath()); };
                        this.componentPropertyChangedEventHandler = function (event) { return _this.forwardComponentPropertyChangedEvent(event); };
                        builder.regions.forEach(function (region) {
                            if (_this.regionByName[region.getName()] != undefined) {
                                throw new Error("Regions must be unique by name, duplicate found: " + region.getName());
                            }
                            _this.addRegion(region);
                        });
                    }
                    Regions.prototype.mergeRegions = function (descriptorRegions, parent) {
                        return new _region.LayoutRegionsMerger().merge(this, descriptorRegions, parent);
                    };
                    Regions.prototype.addRegion = function (region) {
                        this.regionByName[region.getName()] = region;
                        this.notifyRegionAdded(region.getPath());
                        this.registerRegionListeners(region);
                    };
                    Regions.prototype.registerRegionListeners = function (region) {
                        region.onChanged(this.regionChangedEventHandler);
                        region.onComponentPropertyChangedEvent(this.componentPropertyChangedEventHandler);
                    };
                    Regions.prototype.unregisterRegionListeners = function (region) {
                        region.unChanged(this.regionChangedEventHandler);
                        region.unComponentPropertyChangedEvent(this.componentPropertyChangedEventHandler);
                    };
                    Regions.prototype.removeRegions = function (regions) {
                        var _this = this;
                        regions.forEach(function (region) {
                            delete _this.regionByName[region.getName()];
                            _this.notifyRegionRemoved(region.getPath());
                            _this.unregisterRegionListeners(region);
                        });
                    };
                    Regions.prototype.getRegions = function () {
                        var regions = [];
                        for (var i in this.regionByName) {
                            var region = this.regionByName[i];
                            regions.push(region);
                        }
                        return regions;
                    };
                    Regions.prototype.getRegionByName = function (name) {
                        return this.regionByName[name];
                    };
                    Regions.prototype.getComponent = function (path) {
                        var first = path.getFirstLevel();
                        var region = this.getRegionByName(first.getRegionName());
                        var component = region.getComponentByIndex(first.getComponentIndex());
                        if (path.numberOfLevels() == 1) {
                            return component;
                        }
                        else {
                            if (!api.ObjectHelper.iFrameSafeInstanceOf(component, _region.LayoutComponent)) {
                                throw new Error("Expected component to be a LayoutComponent: " + api.ClassHelper.getClassName(component));
                            }
                            var layoutComponent = component;
                            return layoutComponent.getComponent(path.removeFirstLevel());
                        }
                    };
                    Regions.prototype.changeRegionsTo = function (regionDescriptors) {
                        var _this = this;
                        var regionsToRemove = this.getRegions().filter(function (region, index) {
                            return !regionDescriptors.some(function (regionDescriptor) {
                                return regionDescriptor.getName() == region.getName();
                            });
                        });
                        this.removeRegions(regionsToRemove);
                        regionDescriptors.forEach(function (regionDescriptor) {
                            var region = _this.getRegionByName(regionDescriptor.getName());
                            if (!region) {
                                region = _region.Region.create().setName(regionDescriptor.getName()).build();
                                _this.addRegion(region);
                            }
                        });
                    };
                    Regions.prototype.toJson = function () {
                        var regionJsons = [];
                        this.getRegions().forEach(function (region) {
                            regionJsons.push(region.toJson());
                        });
                        return regionJsons;
                    };
                    Regions.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Regions)) {
                            return false;
                        }
                        var other = o;
                        var thisRegions = this.getRegions();
                        var otherRegions = other.getRegions();
                        if (!api.ObjectHelper.arrayEquals(thisRegions, otherRegions)) {
                            return false;
                        }
                        return true;
                    };
                    Regions.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new RegionsBuilder(this, generateNewPropertyIds).build();
                    };
                    Regions.prototype.onChanged = function (listener) {
                        this.changedListeners.push(listener);
                    };
                    Regions.prototype.unChanged = function (listener) {
                        this.changedListeners = this.changedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyChanged = function (event) {
                        if (Regions.debug) {
                            console.debug("Regions.notifyChanged");
                        }
                        this.changedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Regions.prototype.onComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners.push(listener);
                    };
                    Regions.prototype.unComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.forwardComponentPropertyChangedEvent = function (event) {
                        this.componentPropertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Regions.prototype.onRegionChanged = function (listener) {
                        this.regionChangedListeners.push(listener);
                    };
                    Regions.prototype.unRegionChanged = function (listener) {
                        this.regionChangedListeners = this.regionChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyRegionChanged = function (regionPath) {
                        var event = new _region.RegionChangedEvent(regionPath);
                        if (Regions.debug) {
                            console.debug("Regions.notifyRegionChanged: " + event.getRegionPath().toString());
                        }
                        this.regionChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChanged(event);
                    };
                    Regions.prototype.onRegionAdded = function (listener) {
                        this.regionAddedListeners.push(listener);
                    };
                    Regions.prototype.unRegionAdded = function (listener) {
                        this.regionAddedListeners = this.regionAddedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyRegionAdded = function (regionPath) {
                        var event = new _region.RegionAddedEvent(regionPath);
                        if (Regions.debug) {
                            console.debug("Regions.notifyRegionAdded: " + event.getRegionPath().toString());
                        }
                        this.regionAddedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChanged(event);
                    };
                    Regions.prototype.onRegionRemoved = function (listener) {
                        this.regionRemovedListeners.push(listener);
                    };
                    Regions.prototype.unRegionRemoved = function (listener) {
                        this.regionRemovedListeners = this.regionRemovedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyRegionRemoved = function (regionPath) {
                        var event = new _region.RegionRemovedEvent(regionPath);
                        if (Regions.debug) {
                            console.debug("Regions.notifyRegionRemoved: " + event.getRegionPath().toString());
                        }
                        this.regionRemovedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChanged(event);
                    };
                    Regions.create = function () {
                        return new RegionsBuilder();
                    };
                    Regions.debug = false;
                    return Regions;
                })();
                _region.Regions = Regions;
                var RegionsBuilder = (function () {
                    function RegionsBuilder(source, generateNewPropertyIds) {
                        var _this = this;
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        this.regions = [];
                        if (source) {
                            source.getRegions().forEach(function (region) {
                                _this.regions.push(region.clone(generateNewPropertyIds));
                            });
                        }
                    }
                    RegionsBuilder.prototype.fromJson = function (regionsJson, propertyIdProvider, parent) {
                        var _this = this;
                        regionsJson.forEach(function (regionJson) {
                            var region = _region.Region.create().setName(regionJson.name).setParent(parent).build();
                            regionJson.components.forEach(function (componentJson, componentIndex) {
                                var component = _region.ComponentFactory.createFromJson(componentJson, componentIndex, region, propertyIdProvider);
                                region.addComponent(component);
                            });
                            _this.addRegion(region);
                        });
                        return this;
                    };
                    RegionsBuilder.prototype.addRegion = function (value) {
                        this.regions.push(value);
                        return this;
                    };
                    RegionsBuilder.prototype.build = function () {
                        return new Regions(this);
                    };
                    return RegionsBuilder;
                })();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var FormBuilder = api.form.FormBuilder;
                var OccurrencesBuilder = api.form.OccurrencesBuilder;
                var TextArea = api.form.inputtype.text.TextArea;
                var PropertyTree = api.data.PropertyTree;
                var ImageComponent = (function (_super) {
                    __extends(ImageComponent, _super);
                    function ImageComponent(builder) {
                        var _this = this;
                        _super.call(this, builder);
                        this.image = builder.image;
                        this.config = builder.config;
                        this.configChangedHandler = function (event) {
                            if (ImageComponent.debug) {
                                console.debug("ImageComponent[" + _this.getPath().toString() + "].config.onChanged: ", event);
                            }
                            if (!_this.disableEventForwarding) {
                                _this.notifyPropertyValueChanged(ImageComponent.PROPERTY_CONFIG);
                            }
                        };
                        this.config.onChanged(this.configChangedHandler);
                        var formBuilder = new FormBuilder();
                        formBuilder.addFormItem(new api.form.InputBuilder().setName("caption").setInputType(TextArea.getName()).setLabel("Caption").setOccurrences(new OccurrencesBuilder().setMinimum(0).setMaximum(1).build()).build());
                        this.form = formBuilder.build();
                    }
                    ImageComponent.prototype.setDisableEventForwarding = function (value) {
                        this.disableEventForwarding = value;
                    };
                    ImageComponent.prototype.getImage = function () {
                        return this.image;
                    };
                    ImageComponent.prototype.getForm = function () {
                        return this.form;
                    };
                    ImageComponent.prototype.getConfig = function () {
                        return this.config;
                    };
                    ImageComponent.prototype.setImage = function (contentId, name) {
                        var oldValue = this.image;
                        this.image = contentId;
                        this.setName(name ? new _region.ComponentName(name) : null);
                        if (!api.ObjectHelper.equals(oldValue, contentId)) {
                            this.notifyPropertyChanged(ImageComponent.PROPERTY_IMAGE);
                        }
                    };
                    ImageComponent.prototype.hasImage = function () {
                        return !!this.image;
                    };
                    ImageComponent.prototype.doReset = function () {
                        this.setImage(null, null);
                    };
                    ImageComponent.prototype.isEmpty = function () {
                        return !this.image;
                    };
                    ImageComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        json.image = this.image != null ? this.image.toString() : null;
                        json.config = this.config != null ? this.config.toJson() : null;
                        return {
                            ImageComponent: json
                        };
                    };
                    ImageComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ImageComponent)) {
                            return false;
                        }
                        var other = o;
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.image, other.image)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.config, other.config)) {
                            return false;
                        }
                        return true;
                    };
                    ImageComponent.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new ImageComponentBuilder(this, generateNewPropertyIds).build();
                    };
                    ImageComponent.PROPERTY_IMAGE = 'image';
                    ImageComponent.PROPERTY_CONFIG = 'config';
                    ImageComponent.debug = false;
                    return ImageComponent;
                })(_region.Component);
                _region.ImageComponent = ImageComponent;
                var ImageComponentBuilder = (function (_super) {
                    __extends(ImageComponentBuilder, _super);
                    function ImageComponentBuilder(source, generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        _super.call(this, source);
                        if (source) {
                            this.image = source.getImage();
                            this.config = source.getConfig() ? source.getConfig().copy(generateNewPropertyIds) : null;
                        }
                        else {
                            this.config = new PropertyTree(api.Client.get().getPropertyIdProvider());
                        }
                    }
                    ImageComponentBuilder.prototype.setImage = function (value) {
                        this.image = value;
                        return this;
                    };
                    ImageComponentBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    ImageComponentBuilder.prototype.fromJson = function (json, region, propertyIdProvider) {
                        if (json.image) {
                            this.setImage(new api.content.ContentId(json.image));
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        if (json.config) {
                            this.setConfig(PropertyTree.fromJson(json.config, propertyIdProvider));
                        }
                        this.setParent(region);
                        return this;
                    };
                    ImageComponentBuilder.prototype.build = function () {
                        return new ImageComponent(this);
                    };
                    return ImageComponentBuilder;
                })(_region.ComponentBuilder);
                _region.ImageComponentBuilder = ImageComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ImageComponentType = (function (_super) {
                    __extends(ImageComponentType, _super);
                    function ImageComponentType() {
                        _super.call(this, "image");
                    }
                    ImageComponentType.prototype.newComponentBuilder = function () {
                        return new region.ImageComponentBuilder();
                    };
                    return ImageComponentType;
                })(region.ComponentType);
                region.ImageComponentType = ImageComponentType;
                new ImageComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var TextComponent = (function (_super) {
                    __extends(TextComponent, _super);
                    function TextComponent(builder) {
                        _super.call(this, builder);
                        if (builder) {
                            this.setText(builder.text, true);
                        }
                    }
                    TextComponent.prototype.getText = function () {
                        return this.text;
                    };
                    TextComponent.prototype.setText = function (value, silent) {
                        this.text = api.util.StringHelper.isBlank(value) ? undefined : value;
                        if (!silent) {
                            this.notifyPropertyChanged(TextComponent.PROPERTY_TEXT);
                        }
                    };
                    TextComponent.prototype.doReset = function () {
                        this.setText();
                    };
                    TextComponent.prototype.isEmpty = function () {
                        return api.util.StringHelper.isBlank(this.text);
                    };
                    TextComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        json.text = this.text != null ? this.text : null;
                        return {
                            TextComponent: json
                        };
                    };
                    TextComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, TextComponent)) {
                            return false;
                        }
                        var other = o;
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        if (!api.ObjectHelper.stringEquals(this.text, other.text)) {
                            return false;
                        }
                        return true;
                    };
                    TextComponent.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new TextComponentBuilder(this).build();
                    };
                    TextComponent.PROPERTY_TEXT = "text";
                    return TextComponent;
                })(_region.Component);
                _region.TextComponent = TextComponent;
                var TextComponentBuilder = (function (_super) {
                    __extends(TextComponentBuilder, _super);
                    function TextComponentBuilder(source) {
                        _super.call(this, source);
                        if (source) {
                            this.text = source.getText();
                        }
                    }
                    TextComponentBuilder.prototype.fromJson = function (json, region) {
                        if (json.text) {
                            this.setText(json.text);
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        this.setParent(region);
                        return this;
                    };
                    TextComponentBuilder.prototype.setText = function (value) {
                        this.text = value;
                        return this;
                    };
                    TextComponentBuilder.prototype.build = function () {
                        return new TextComponent(this);
                    };
                    return TextComponentBuilder;
                })(_region.ComponentBuilder);
                _region.TextComponentBuilder = TextComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var TextComponentType = (function (_super) {
                    __extends(TextComponentType, _super);
                    function TextComponentType() {
                        _super.call(this, "text");
                    }
                    TextComponentType.prototype.newComponentBuilder = function () {
                        return new region.TextComponentBuilder();
                    };
                    return TextComponentType;
                })(region.ComponentType);
                region.TextComponentType = TextComponentType;
                new TextComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptor = (function (_super) {
                    __extends(PartDescriptor, _super);
                    function PartDescriptor() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptor.prototype.clone = function () {
                        return new PartDescriptorBuilder(this).build();
                    };
                    return PartDescriptor;
                })(api.content.page.Descriptor);
                region.PartDescriptor = PartDescriptor;
                var PartDescriptorBuilder = (function (_super) {
                    __extends(PartDescriptorBuilder, _super);
                    function PartDescriptorBuilder(source) {
                        _super.call(this, source);
                    }
                    PartDescriptorBuilder.prototype.fromJson = function (json) {
                        this.setKey(api.content.page.DescriptorKey.fromString(json.key));
                        this.setName(new api.content.page.DescriptorName(json.name));
                        this.setDisplayName(json.displayName);
                        this.setConfig(json.config != null ? api.form.Form.fromJson(json.config) : null);
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setKey = function (value) {
                        this.key = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setDisplayName = function (value) {
                        this.displayName = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.build = function () {
                        return new PartDescriptor(this);
                    };
                    return PartDescriptorBuilder;
                })(api.content.page.DescriptorBuilder);
                region.PartDescriptorBuilder = PartDescriptorBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var PropertyTree = api.data.PropertyTree;
                var PartComponent = (function (_super) {
                    __extends(PartComponent, _super);
                    function PartComponent(builder) {
                        _super.call(this, builder);
                    }
                    PartComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        return {
                            PartComponent: json
                        };
                    };
                    PartComponent.prototype.isEmpty = function () {
                        return !this.hasDescriptor();
                    };
                    PartComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PartComponent)) {
                            return false;
                        }
                        return _super.prototype.equals.call(this, o);
                    };
                    PartComponent.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new PartComponentBuilder(this, generateNewPropertyIds).build();
                    };
                    return PartComponent;
                })(_region.DescriptorBasedComponent);
                _region.PartComponent = PartComponent;
                var PartComponentBuilder = (function (_super) {
                    __extends(PartComponentBuilder, _super);
                    function PartComponentBuilder(source, generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        _super.call(this, source, generateNewPropertyIds);
                    }
                    PartComponentBuilder.prototype.fromJson = function (json, region, propertyIdProvider) {
                        if (json.descriptor) {
                            this.setDescriptor(api.content.page.DescriptorKey.fromString(json.descriptor));
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        if (json.config) {
                            this.setConfig(PropertyTree.fromJson(json.config, propertyIdProvider));
                        }
                        this.setParent(region);
                        return this;
                    };
                    PartComponentBuilder.prototype.build = function () {
                        return new PartComponent(this);
                    };
                    return PartComponentBuilder;
                })(_region.DescriptorBasedComponentBuilder);
                _region.PartComponentBuilder = PartComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartComponentType = (function (_super) {
                    __extends(PartComponentType, _super);
                    function PartComponentType() {
                        _super.call(this, "part");
                    }
                    PartComponentType.prototype.newComponentBuilder = function () {
                        return new region.PartComponentBuilder();
                    };
                    return PartComponentType;
                })(region.ComponentType);
                region.PartComponentType = PartComponentType;
                new PartComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorViewer = (function (_super) {
                    __extends(PartDescriptorViewer, _super);
                    function PartDescriptorViewer() {
                        _super.call(this);
                    }
                    PartDescriptorViewer.prototype.resolveDisplayName = function (object) {
                        return object.getDisplayName();
                    };
                    PartDescriptorViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getKey().toString();
                    };
                    PartDescriptorViewer.prototype.resolveIconClass = function (object) {
                        return "icon-puzzle icon-large";
                    };
                    return PartDescriptorViewer;
                })(api.ui.NamesAndIconViewer);
                region.PartDescriptorViewer = PartDescriptorViewer;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RichComboBox = api.ui.selector.combobox.RichComboBox;
                var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var BaseSelectedOptionView = api.ui.selector.combobox.BaseSelectedOptionView;
                var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                var PartDescriptorComboBox = (function (_super) {
                    __extends(PartDescriptorComboBox, _super);
                    function PartDescriptorComboBox(loader) {
                        _super.call(this, new RichComboBoxBuilder().setIdentifierMethod("getKey").setOptionDisplayValueViewer(new region.PartDescriptorViewer()).setSelectedOptionsView(new PartDescriptorSelectedOptionsView()).setLoader(loader).setMaximumOccurrences(1).setNextInputFocusWhenMaxReached(false));
                    }
                    PartDescriptorComboBox.prototype.getDescriptor = function (descriptorKey) {
                        var option = this.getOptionByValue(descriptorKey.toString());
                        if (option) {
                            return option.displayValue;
                        }
                        return null;
                    };
                    PartDescriptorComboBox.prototype.setDescriptor = function (descriptor) {
                        this.clearSelection();
                        if (descriptor) {
                            var optionToSelect = this.getOptionByValue(descriptor.getKey().toString());
                            if (!optionToSelect) {
                                optionToSelect = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor
                                };
                                this.addOption(optionToSelect);
                            }
                            this.selectOption(optionToSelect);
                        }
                    };
                    return PartDescriptorComboBox;
                })(RichComboBox);
                region.PartDescriptorComboBox = PartDescriptorComboBox;
                var PartDescriptorSelectedOptionsView = (function (_super) {
                    __extends(PartDescriptorSelectedOptionsView, _super);
                    function PartDescriptorSelectedOptionsView() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                        return new SelectedOption(new PartDescriptorSelectedOptionView(option), this.count());
                    };
                    return PartDescriptorSelectedOptionsView;
                })(BaseSelectedOptionsView);
                region.PartDescriptorSelectedOptionsView = PartDescriptorSelectedOptionsView;
                var PartDescriptorSelectedOptionView = (function (_super) {
                    __extends(PartDescriptorSelectedOptionView, _super);
                    function PartDescriptorSelectedOptionView(option) {
                        this.descriptor = option.displayValue;
                        _super.call(this, option);
                        this.addClass("part-descriptor-selected-option-view");
                    }
                    PartDescriptorSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                        namesAndIconView.setIconClass("icon-puzzle icon-medium").setMainName(this.descriptor.getDisplayName()).setSubName(this.descriptor.getKey().toString());
                        var removeButtonEl = new api.dom.AEl("remove");
                        removeButtonEl.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButtonEl);
                        this.appendChild(namesAndIconView);
                    };
                    return PartDescriptorSelectedOptionView;
                })(BaseSelectedOptionView);
                region.PartDescriptorSelectedOptionView = PartDescriptorSelectedOptionView;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorLoader = (function (_super) {
                    __extends(PartDescriptorLoader, _super);
                    function PartDescriptorLoader(request) {
                        _super.call(this, request);
                    }
                    PartDescriptorLoader.prototype.filterFn = function (descriptor) {
                        return descriptor.getDisplayName().toString().toLowerCase().indexOf(this.getSearchString().toLowerCase()) != -1;
                    };
                    return PartDescriptorLoader;
                })(api.util.loader.BaseLoader);
                region.PartDescriptorLoader = PartDescriptorLoader;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorResourceRequest = (function (_super) {
                    __extends(PartDescriptorResourceRequest, _super);
                    function PartDescriptorResourceRequest() {
                        _super.call(this);
                        this.cache = region.PartDescriptorCache.get();
                        this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "part", "descriptor");
                    }
                    PartDescriptorResourceRequest.prototype.getResourcePath = function () {
                        return this.resourcePath;
                    };
                    PartDescriptorResourceRequest.prototype.fromJsonToPartDescriptor = function (json) {
                        var partDescriptor = new region.PartDescriptorBuilder().fromJson(json).build();
                        this.cache.put(partDescriptor);
                        return partDescriptor;
                    };
                    return PartDescriptorResourceRequest;
                })(api.rest.ResourceRequest);
                region.PartDescriptorResourceRequest = PartDescriptorResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorsResourceRequest = (function (_super) {
                    __extends(PartDescriptorsResourceRequest, _super);
                    function PartDescriptorsResourceRequest() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptorsResourceRequest.prototype.fromJsonToPartDescriptors = function (json) {
                        var _this = this;
                        var array = [];
                        json.descriptors.forEach(function (descriptorJson) {
                            array.push(_this.fromJsonToPartDescriptor(descriptorJson));
                        });
                        return array;
                    };
                    return PartDescriptorsResourceRequest;
                })(region.PartDescriptorResourceRequest);
                region.PartDescriptorsResourceRequest = PartDescriptorsResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetPartDescriptorsByModulesRequest = (function (_super) {
                    __extends(GetPartDescriptorsByModulesRequest, _super);
                    function GetPartDescriptorsByModulesRequest(moduleKeys) {
                        _super.call(this);
                        this.moduleKeys = moduleKeys;
                    }
                    GetPartDescriptorsByModulesRequest.prototype.setModuleKeys = function (moduleKeys) {
                        this.moduleKeys = moduleKeys;
                    };
                    GetPartDescriptorsByModulesRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorsByModulesRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorsByModulesRequest.prototype.sendAndParse = function () {
                        var promises = this.moduleKeys.map(function (moduleKey) { return new region.GetPartDescriptorsByModuleRequest(moduleKey).sendAndParse(); });
                        return wemQ.all(promises).then(function (results) {
                            var all = [];
                            results.forEach(function (result) {
                                Array.prototype.push.apply(all, result);
                            });
                            return all;
                        });
                    };
                    return GetPartDescriptorsByModulesRequest;
                })(region.PartDescriptorsResourceRequest);
                region.GetPartDescriptorsByModulesRequest = GetPartDescriptorsByModulesRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetPartDescriptorsByModuleRequest = (function (_super) {
                    __extends(GetPartDescriptorsByModuleRequest, _super);
                    function GetPartDescriptorsByModuleRequest(moduleKey) {
                        _super.call(this);
                        _super.prototype.setMethod.call(this, "GET");
                        this.moduleKey = moduleKey;
                    }
                    GetPartDescriptorsByModuleRequest.prototype.getParams = function () {
                        return {
                            moduleKey: this.moduleKey.toString()
                        };
                    };
                    GetPartDescriptorsByModuleRequest.prototype.getRequestPath = function () {
                        return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "by_module");
                    };
                    GetPartDescriptorsByModuleRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var cached = this.cache.getByModule(this.moduleKey);
                        if (cached) {
                            return wemQ(cached);
                        }
                        else {
                            return this.send().then(function (response) {
                                return _this.fromJsonToPartDescriptors(response.getResult());
                            });
                        }
                    };
                    return GetPartDescriptorsByModuleRequest;
                })(region.PartDescriptorsResourceRequest);
                region.GetPartDescriptorsByModuleRequest = GetPartDescriptorsByModuleRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetPartDescriptorByKeyRequest = (function (_super) {
                    __extends(GetPartDescriptorByKeyRequest, _super);
                    function GetPartDescriptorByKeyRequest(key) {
                        _super.call(this);
                        this.key = key;
                    }
                    GetPartDescriptorByKeyRequest.prototype.setKey = function (key) {
                        this.key = key;
                    };
                    GetPartDescriptorByKeyRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorByKeyRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorByKeyRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var deferred = wemQ.defer();
                        new region.GetPartDescriptorsByModuleRequest(this.key.getModuleKey()).sendAndParse().then(function (descriptors) {
                            descriptors.forEach(function (descriptor) {
                                if (_this.key.equals(descriptor.getKey())) {
                                    deferred.resolve(descriptor);
                                }
                            });
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                        return deferred.promise;
                    };
                    return GetPartDescriptorByKeyRequest;
                })(region.PartDescriptorResourceRequest);
                region.GetPartDescriptorByKeyRequest = GetPartDescriptorByKeyRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var DefaultPartDescriptorResolver = (function () {
                    function DefaultPartDescriptorResolver() {
                    }
                    DefaultPartDescriptorResolver.resolve = function (moduleKeys) {
                        return new region.GetPartDescriptorsByModulesRequest(moduleKeys).sendAndParse().then(function (descriptors) {
                            return descriptors[0];
                        });
                    };
                    return DefaultPartDescriptorResolver;
                })();
                region.DefaultPartDescriptorResolver = DefaultPartDescriptorResolver;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ModuleBasedCache = api.module.ModuleBasedCache;
                var PartDescriptorCache = (function (_super) {
                    __extends(PartDescriptorCache, _super);
                    function PartDescriptorCache() {
                        if (PartDescriptorCache.instance) {
                            throw new Error("Instantiation failed: Use PartDescriptorCache.get() instead!");
                        }
                        _super.call(this);
                    }
                    PartDescriptorCache.get = function () {
                        var w = api.dom.WindowDOM.get();
                        var topWindow = w.getTopParent().asWindow();
                        if (!topWindow.api.content.page.region.PartDescriptorCache.instance) {
                            topWindow.api.content.page.region.PartDescriptorCache.instance = new PartDescriptorCache();
                        }
                        return topWindow.api.content.page.region.PartDescriptorCache.instance;
                    };
                    PartDescriptorCache.prototype.loadByModule = function (moduleKey) {
                        new region.GetPartDescriptorsByModuleRequest(moduleKey).sendAndParse().catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    };
                    PartDescriptorCache.prototype.put = function (descriptor) {
                        api.util.assertNotNull(descriptor, "a PartDescriptor must be given");
                        _super.prototype.put.call(this, descriptor, descriptor.getKey().getModuleKey());
                    };
                    PartDescriptorCache.prototype.getByKey = function (key) {
                        return _super.prototype.getByKey.call(this, key, key.getModuleKey());
                    };
                    PartDescriptorCache.prototype.createModuleCache = function () {
                        return new PartDescriptorModuleCache();
                    };
                    return PartDescriptorCache;
                })(ModuleBasedCache);
                region.PartDescriptorCache = PartDescriptorCache;
                var PartDescriptorModuleCache = (function (_super) {
                    __extends(PartDescriptorModuleCache, _super);
                    function PartDescriptorModuleCache() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptorModuleCache.prototype.copy = function (object) {
                        return object.clone();
                    };
                    PartDescriptorModuleCache.prototype.getKeyFromObject = function (object) {
                        return object.getKey();
                    };
                    PartDescriptorModuleCache.prototype.getKeyAsString = function (key) {
                        return key.toString();
                    };
                    return PartDescriptorModuleCache;
                })(api.cache.Cache);
                region.PartDescriptorModuleCache = PartDescriptorModuleCache;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var LayoutDescriptor = (function (_super) {
                    __extends(LayoutDescriptor, _super);
                    function LayoutDescriptor(builder) {
                        _super.call(this, builder);
                        this.regions = builder.regions;
                    }
                    LayoutDescriptor.prototype.getRegions = function () {
                        return this.regions;
                    };
                    LayoutDescriptor.prototype.clone = function () {
                        return new LayoutDescriptorBuilder(this).build();
                    };
                    return LayoutDescriptor;
                })(api.content.page.Descriptor);
                _region.LayoutDescriptor = LayoutDescriptor;
                var LayoutDescriptorBuilder = (function (_super) {
                    __extends(LayoutDescriptorBuilder, _super);
                    function LayoutDescriptorBuilder(source) {
                        _super.call(this, source);
                        this.regions = [];
                        if (source) {
                            this.regions = source.getRegions();
                        }
                    }
                    LayoutDescriptorBuilder.prototype.fromJson = function (json) {
                        this.setKey(api.content.page.DescriptorKey.fromString(json.key));
                        this.setName(new api.content.page.DescriptorName(json.name));
                        this.setDisplayName(json.displayName);
                        this.setConfig(json.config != null ? api.form.Form.fromJson(json.config) : null);
                        for (var i = 0; i < json.regions.length; i++) {
                            var region = new _region.RegionDescriptorBuilder().fromJson(json.regions[i]).build();
                            this.regions.push(region);
                        }
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setKey = function (value) {
                        this.key = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setDisplayName = function (value) {
                        this.displayName = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.addRegion = function (value) {
                        this.regions.push(value);
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.build = function () {
                        return new LayoutDescriptor(this);
                    };
                    return LayoutDescriptorBuilder;
                })(api.content.page.DescriptorBuilder);
                _region.LayoutDescriptorBuilder = LayoutDescriptorBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutComponentType = (function (_super) {
                    __extends(LayoutComponentType, _super);
                    function LayoutComponentType() {
                        _super.call(this, "layout");
                    }
                    LayoutComponentType.prototype.newComponentBuilder = function () {
                        return new region.LayoutComponentBuilder();
                    };
                    return LayoutComponentType;
                })(region.ComponentType);
                region.LayoutComponentType = LayoutComponentType;
                new LayoutComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var PropertyTree = api.data.PropertyTree;
                var LayoutComponent = (function (_super) {
                    __extends(LayoutComponent, _super);
                    function LayoutComponent(builder) {
                        var _this = this;
                        _super.call(this, builder);
                        this.componentPropertyChangedListeners = [];
                        if (builder.regions) {
                            this.regions = builder.regions;
                            this.regions.getRegions().forEach(function (region) {
                                region.setParent(_this);
                            });
                        }
                        else {
                            this.regions = _region.Regions.create().build();
                        }
                        this.componentPropertyChangedEventHandler = function (event) { return _this.forwardComponentPropertyChangedEvent(event); };
                        this.regionsChangedEventHandler = function (event) {
                            if (LayoutComponent.debug) {
                                console.debug("LayoutComponent[" + _this.getPath().toString() + "].onChanged: ", event);
                            }
                            _this.notifyPropertyValueChanged("regions");
                        };
                        this.registerRegionsListeners(this.regions);
                    }
                    LayoutComponent.prototype.getComponent = function (path) {
                        return this.regions.getComponent(path);
                    };
                    LayoutComponent.prototype.getRegions = function () {
                        return this.regions;
                    };
                    LayoutComponent.prototype.setRegions = function (value) {
                        var oldValue = this.regions;
                        if (oldValue) {
                            this.unregisterRegionsListeners(oldValue);
                        }
                        this.regions = value;
                        this.registerRegionsListeners(this.regions);
                        if (!api.ObjectHelper.equals(oldValue, value)) {
                            if (LayoutComponent.debug) {
                                console.debug("LayoutComponent[" + this.getPath().toString() + "].regions reassigned: ", api.event);
                            }
                            this.notifyPropertyChanged("regions");
                        }
                    };
                    LayoutComponent.prototype.setDescriptor = function (descriptorKey, descriptor) {
                        _super.prototype.setDescriptor.call(this, descriptorKey, descriptor);
                        if (descriptor) {
                            this.addRegions(descriptor);
                        }
                    };
                    LayoutComponent.prototype.addRegions = function (layoutDescriptor) {
                        var sourceRegions = this.getRegions();
                        var mergedRegions = sourceRegions.mergeRegions(layoutDescriptor.getRegions(), this);
                        this.setRegions(mergedRegions);
                    };
                    LayoutComponent.prototype.isEmpty = function () {
                        return !this.hasDescriptor();
                    };
                    LayoutComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        json.regions = this.regions.toJson();
                        return {
                            LayoutComponent: json
                        };
                    };
                    LayoutComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, LayoutComponent)) {
                            return false;
                        }
                        var other = o;
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.regions, other.regions)) {
                            return false;
                        }
                        return true;
                    };
                    LayoutComponent.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new LayoutComponentBuilder(this, generateNewPropertyIds).build();
                    };
                    LayoutComponent.prototype.registerRegionsListeners = function (regions) {
                        regions.onChanged(this.regionsChangedEventHandler);
                        regions.onComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    };
                    LayoutComponent.prototype.unregisterRegionsListeners = function (regions) {
                        regions.unChanged(this.regionsChangedEventHandler);
                        regions.unComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    };
                    LayoutComponent.prototype.onComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners.push(listener);
                    };
                    LayoutComponent.prototype.unComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    LayoutComponent.debug = false;
                    return LayoutComponent;
                })(_region.DescriptorBasedComponent);
                _region.LayoutComponent = LayoutComponent;
                var LayoutComponentBuilder = (function (_super) {
                    __extends(LayoutComponentBuilder, _super);
                    function LayoutComponentBuilder(source, generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        _super.call(this, source, generateNewPropertyIds);
                        if (source) {
                            this.regions = source.getRegions().clone(generateNewPropertyIds);
                        }
                    }
                    LayoutComponentBuilder.prototype.fromJson = function (json, region, propertyIdProvider) {
                        if (json.descriptor) {
                            this.setDescriptor(api.content.page.DescriptorKey.fromString(json.descriptor));
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        if (json.config) {
                            this.setConfig(PropertyTree.fromJson(json.config, propertyIdProvider));
                        }
                        this.setParent(region);
                        var layoutComponent = this.build();
                        var layoutRegions = _region.Regions.create().fromJson(json.regions, propertyIdProvider, layoutComponent).build();
                        layoutComponent.setRegions(layoutRegions);
                        return layoutComponent;
                    };
                    LayoutComponentBuilder.prototype.setRegions = function (value) {
                        this.regions = value;
                        return this;
                    };
                    LayoutComponentBuilder.prototype.build = function () {
                        return new LayoutComponent(this);
                    };
                    return LayoutComponentBuilder;
                })(_region.DescriptorBasedComponentBuilder);
                _region.LayoutComponentBuilder = LayoutComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorViewer = (function (_super) {
                    __extends(LayoutDescriptorViewer, _super);
                    function LayoutDescriptorViewer() {
                        _super.call(this);
                    }
                    LayoutDescriptorViewer.prototype.resolveDisplayName = function (object) {
                        return object.getDisplayName();
                    };
                    LayoutDescriptorViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getKey().toString();
                    };
                    LayoutDescriptorViewer.prototype.resolveIconClass = function (object) {
                        return "icon-insert-template icon-large";
                    };
                    return LayoutDescriptorViewer;
                })(api.ui.NamesAndIconViewer);
                region.LayoutDescriptorViewer = LayoutDescriptorViewer;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RichComboBox = api.ui.selector.combobox.RichComboBox;
                var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var BaseSelectedOptionView = api.ui.selector.combobox.BaseSelectedOptionView;
                var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                var LayoutDescriptorComboBox = (function (_super) {
                    __extends(LayoutDescriptorComboBox, _super);
                    function LayoutDescriptorComboBox(loader) {
                        _super.call(this, new RichComboBoxBuilder().setIdentifierMethod("getKey").setOptionDisplayValueViewer(new region.LayoutDescriptorViewer()).setSelectedOptionsView(new LayoutDescriptorSelectedOptionsView()).setLoader(loader).setMaximumOccurrences(1).setNextInputFocusWhenMaxReached(false));
                    }
                    LayoutDescriptorComboBox.prototype.getDescriptor = function (descriptorKey) {
                        var option = this.getOptionByValue(descriptorKey.toString());
                        if (option) {
                            return option.displayValue;
                        }
                        return null;
                    };
                    LayoutDescriptorComboBox.prototype.setDescriptor = function (descriptor) {
                        this.clearSelection();
                        if (descriptor) {
                            var optionToSelect = this.getOptionByValue(descriptor.getKey().toString());
                            if (!optionToSelect) {
                                optionToSelect = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor
                                };
                                this.addOption(optionToSelect);
                            }
                            this.selectOption(optionToSelect);
                        }
                    };
                    return LayoutDescriptorComboBox;
                })(RichComboBox);
                region.LayoutDescriptorComboBox = LayoutDescriptorComboBox;
                var LayoutDescriptorSelectedOptionsView = (function (_super) {
                    __extends(LayoutDescriptorSelectedOptionsView, _super);
                    function LayoutDescriptorSelectedOptionsView() {
                        _super.apply(this, arguments);
                    }
                    LayoutDescriptorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                        return new SelectedOption(new LayoutDescriptorSelectedOptionView(option), this.count());
                    };
                    return LayoutDescriptorSelectedOptionsView;
                })(BaseSelectedOptionsView);
                region.LayoutDescriptorSelectedOptionsView = LayoutDescriptorSelectedOptionsView;
                var LayoutDescriptorSelectedOptionView = (function (_super) {
                    __extends(LayoutDescriptorSelectedOptionView, _super);
                    function LayoutDescriptorSelectedOptionView(option) {
                        this.descriptor = option.displayValue;
                        _super.call(this, option);
                        this.addClass("layout-descriptor-selected-option-view");
                    }
                    LayoutDescriptorSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                        namesAndIconView.setIconClass("icon-earth icon-medium").setMainName(this.descriptor.getDisplayName()).setSubName(this.descriptor.getKey().toString());
                        var removeButtonEl = new api.dom.AEl("remove");
                        removeButtonEl.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButtonEl);
                        this.appendChild(namesAndIconView);
                    };
                    return LayoutDescriptorSelectedOptionView;
                })(BaseSelectedOptionView);
                region.LayoutDescriptorSelectedOptionView = LayoutDescriptorSelectedOptionView;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorLoader = (function (_super) {
                    __extends(LayoutDescriptorLoader, _super);
                    function LayoutDescriptorLoader(request) {
                        _super.call(this, request);
                    }
                    LayoutDescriptorLoader.prototype.filterFn = function (descriptor) {
                        return descriptor.getDisplayName().toString().toLowerCase().indexOf(this.getSearchString().toLowerCase()) != -1;
                    };
                    return LayoutDescriptorLoader;
                })(api.util.loader.BaseLoader);
                region.LayoutDescriptorLoader = LayoutDescriptorLoader;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorResourceRequest = (function (_super) {
                    __extends(LayoutDescriptorResourceRequest, _super);
                    function LayoutDescriptorResourceRequest() {
                        _super.call(this);
                        this.cache = region.LayoutDescriptorCache.get();
                        this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "layout", "descriptor");
                    }
                    LayoutDescriptorResourceRequest.prototype.getResourcePath = function () {
                        return this.resourcePath;
                    };
                    LayoutDescriptorResourceRequest.prototype.fromJsonToLayoutDescriptor = function (json) {
                        var descriptor = new region.LayoutDescriptorBuilder().fromJson(json).build();
                        this.cache.put(descriptor);
                        return descriptor;
                    };
                    return LayoutDescriptorResourceRequest;
                })(api.rest.ResourceRequest);
                region.LayoutDescriptorResourceRequest = LayoutDescriptorResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorsResourceRequest = (function (_super) {
                    __extends(LayoutDescriptorsResourceRequest, _super);
                    function LayoutDescriptorsResourceRequest() {
                        _super.apply(this, arguments);
                    }
                    LayoutDescriptorsResourceRequest.prototype.fromJsonToLayoutDescriptors = function (json) {
                        var _this = this;
                        var array = [];
                        json.descriptors.forEach(function (descriptorJson) {
                            array.push(_this.fromJsonToLayoutDescriptor(descriptorJson));
                        });
                        return array;
                    };
                    return LayoutDescriptorsResourceRequest;
                })(region.LayoutDescriptorResourceRequest);
                region.LayoutDescriptorsResourceRequest = LayoutDescriptorsResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetLayoutDescriptorsByModuleRequest = (function (_super) {
                    __extends(GetLayoutDescriptorsByModuleRequest, _super);
                    function GetLayoutDescriptorsByModuleRequest(moduleKey) {
                        _super.call(this);
                        _super.prototype.setMethod.call(this, "GET");
                        this.moduleKey = moduleKey;
                    }
                    GetLayoutDescriptorsByModuleRequest.prototype.getParams = function () {
                        return {
                            moduleKey: this.moduleKey.toString()
                        };
                    };
                    GetLayoutDescriptorsByModuleRequest.prototype.getRequestPath = function () {
                        return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "by_module");
                    };
                    GetLayoutDescriptorsByModuleRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var cached = this.cache.getByModule(this.moduleKey);
                        if (cached) {
                            return wemQ(cached);
                        }
                        else {
                            return this.send().then(function (response) {
                                return _this.fromJsonToLayoutDescriptors(response.getResult());
                            });
                        }
                    };
                    return GetLayoutDescriptorsByModuleRequest;
                })(region.LayoutDescriptorsResourceRequest);
                region.GetLayoutDescriptorsByModuleRequest = GetLayoutDescriptorsByModuleRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetLayoutDescriptorsByModulesRequest = (function (_super) {
                    __extends(GetLayoutDescriptorsByModulesRequest, _super);
                    function GetLayoutDescriptorsByModulesRequest(moduleKeys) {
                        _super.call(this);
                        this.moduleKeys = moduleKeys;
                    }
                    GetLayoutDescriptorsByModulesRequest.prototype.setModuleKeys = function (moduleKeys) {
                        this.moduleKeys = moduleKeys;
                    };
                    GetLayoutDescriptorsByModulesRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorsByModulesRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorsByModulesRequest.prototype.sendAndParse = function () {
                        var promises = this.moduleKeys.map(function (moduleKey) { return new region.GetLayoutDescriptorsByModuleRequest(moduleKey).sendAndParse(); });
                        return wemQ.all(promises).then(function (results) {
                            var all = [];
                            results.forEach(function (result) {
                                Array.prototype.push.apply(all, result);
                            });
                            return all;
                        });
                    };
                    return GetLayoutDescriptorsByModulesRequest;
                })(region.LayoutDescriptorsResourceRequest);
                region.GetLayoutDescriptorsByModulesRequest = GetLayoutDescriptorsByModulesRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetLayoutDescriptorByKeyRequest = (function (_super) {
                    __extends(GetLayoutDescriptorByKeyRequest, _super);
                    function GetLayoutDescriptorByKeyRequest(key) {
                        _super.call(this);
                        this.key = key;
                    }
                    GetLayoutDescriptorByKeyRequest.prototype.setKey = function (key) {
                        this.key = key;
                    };
                    GetLayoutDescriptorByKeyRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorByKeyRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorByKeyRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var deferred = wemQ.defer();
                        new region.GetLayoutDescriptorsByModuleRequest(this.key.getModuleKey()).sendAndParse().then(function (descriptors) {
                            descriptors.forEach(function (descriptor) {
                                if (_this.key.equals(descriptor.getKey())) {
                                    deferred.resolve(descriptor);
                                }
                            });
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                        return deferred.promise;
                    };
                    return GetLayoutDescriptorByKeyRequest;
                })(region.LayoutDescriptorResourceRequest);
                region.GetLayoutDescriptorByKeyRequest = GetLayoutDescriptorByKeyRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var DefaultLayoutDescriptorResolver = (function () {
                    function DefaultLayoutDescriptorResolver() {
                    }
                    DefaultLayoutDescriptorResolver.resolve = function (moduleKeys) {
                        return new region.GetLayoutDescriptorsByModulesRequest(moduleKeys).sendAndParse().then(function (descriptors) {
                            return descriptors[0];
                        });
                    };
                    return DefaultLayoutDescriptorResolver;
                })();
                region.DefaultLayoutDescriptorResolver = DefaultLayoutDescriptorResolver;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Dropdown = api.ui.selector.dropdown.Dropdown;
                var LayoutDescriptorDropdown = (function (_super) {
                    __extends(LayoutDescriptorDropdown, _super);
                    function LayoutDescriptorDropdown(name, config) {
                        var _this = this;
                        _super.call(this, name, {
                            optionDisplayValueViewer: new region.LayoutDescriptorViewer(),
                            dataIdProperty: "value"
                        });
                        config.loader.onLoadedData(function (event) {
                            var descriptors = event.getData();
                            descriptors.forEach(function (descriptor) {
                                var indices = [];
                                indices.push(descriptor.getDisplayName());
                                indices.push(descriptor.getName().toString());
                                var option = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor,
                                    indices: indices
                                };
                                _this.addOption(option);
                            });
                        });
                    }
                    LayoutDescriptorDropdown.prototype.setDescriptor = function (key) {
                        var option = this.getOptionByValue(key.toString());
                        if (option) {
                            this.selectOption(option, true);
                        }
                    };
                    return LayoutDescriptorDropdown;
                })(Dropdown);
                region.LayoutDescriptorDropdown = LayoutDescriptorDropdown;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var LayoutRegionsMerger = (function () {
                    function LayoutRegionsMerger() {
                    }
                    LayoutRegionsMerger.prototype.merge = function (regions, layoutDescriptorRegions, parent) {
                        var _this = this;
                        this.layoutComponentRegions = regions.getRegions();
                        this.initLookupTables(layoutDescriptorRegions, parent);
                        this.mergeExistingRegions();
                        this.mergeMissingRegions(layoutDescriptorRegions);
                        var layoutRegionsBuilder = _region.Regions.create();
                        layoutDescriptorRegions.forEach(function (regionDescriptor) {
                            var layoutRegion = _this.targetRegionsByName[regionDescriptor.getName()];
                            layoutRegionsBuilder.addRegion(layoutRegion);
                        });
                        return layoutRegionsBuilder.build();
                    };
                    LayoutRegionsMerger.prototype.mergeExistingRegions = function () {
                        var _this = this;
                        this.layoutComponentRegions.forEach(function (region) {
                            var regionName = region.getName();
                            if (_this.targetRegionsByName[regionName]) {
                                var targetRegion = _this.targetRegionsByName[regionName];
                                var updatedRegion = _this.addComponents(region, targetRegion);
                                _this.targetRegionsByName[regionName] = updatedRegion;
                            }
                        });
                    };
                    LayoutRegionsMerger.prototype.mergeMissingRegions = function (layoutDescriptorRegions) {
                        var _this = this;
                        var lastRegionName = layoutDescriptorRegions[layoutDescriptorRegions.length - 1].getName();
                        this.layoutComponentRegions.forEach(function (region) {
                            var regionName = region.getName();
                            if (!_this.targetRegionsByName[regionName]) {
                                var sourceRegionPos = _this.sourceRegionsPositionByName[regionName];
                                var targetRegionName = _this.targetRegionsNameByPosition[sourceRegionPos] || lastRegionName;
                                var targetRegion = _this.targetRegionsByName[targetRegionName];
                                var updatedRegion = _this.addComponents(region, targetRegion);
                                _this.targetRegionsByName[targetRegionName] = updatedRegion;
                            }
                        });
                    };
                    LayoutRegionsMerger.prototype.initLookupTables = function (layoutDescriptorRegions, parent) {
                        var _this = this;
                        this.targetRegionsByName = {};
                        this.targetRegionsNameByPosition = {};
                        this.sourceRegionsPositionByName = {};
                        layoutDescriptorRegions.forEach(function (regionDescriptor, idx) {
                            var regionName = regionDescriptor.getName();
                            var layoutRegion = _region.Region.create().setName(regionName).setParent(parent).build();
                            _this.targetRegionsByName[regionName] = layoutRegion;
                            _this.targetRegionsNameByPosition[idx] = regionName;
                        });
                        this.layoutComponentRegions.forEach(function (region, idx) {
                            _this.sourceRegionsPositionByName[region.getName()] = idx;
                        });
                    };
                    LayoutRegionsMerger.prototype.addComponents = function (fromRegion, toRegion) {
                        if (fromRegion.getComponents().length === 0) {
                            return toRegion;
                        }
                        var result = _region.Region.create(toRegion);
                        fromRegion.getComponents().forEach(function (component) {
                            result.addComponent(component);
                        });
                        return result.build();
                    };
                    return LayoutRegionsMerger;
                })();
                _region.LayoutRegionsMerger = LayoutRegionsMerger;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ModuleBasedCache = api.module.ModuleBasedCache;
                var LayoutDescriptorCache = (function (_super) {
                    __extends(LayoutDescriptorCache, _super);
                    function LayoutDescriptorCache() {
                        if (LayoutDescriptorCache.instance) {
                            throw new Error("Instantiation failed: Use LayoutDescriptorCache.get() instead!");
                        }
                        _super.call(this);
                    }
                    LayoutDescriptorCache.get = function () {
                        var w = api.dom.WindowDOM.get();
                        var topWindow = w.getTopParent().asWindow();
                        if (!topWindow.api.content.page.region.LayoutDescriptorCache.instance) {
                            topWindow.api.content.page.region.LayoutDescriptorCache.instance = new LayoutDescriptorCache();
                        }
                        return topWindow.api.content.page.region.LayoutDescriptorCache.instance;
                    };
                    LayoutDescriptorCache.prototype.loadByModule = function (moduleKey) {
                        new region.GetLayoutDescriptorsByModuleRequest(moduleKey).sendAndParse().catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    };
                    LayoutDescriptorCache.prototype.put = function (descriptor) {
                        api.util.assertNotNull(descriptor, "a LayoutDescriptor must be given");
                        _super.prototype.put.call(this, descriptor, descriptor.getKey().getModuleKey());
                    };
                    LayoutDescriptorCache.prototype.getByKey = function (key) {
                        return _super.prototype.getByKey.call(this, key, key.getModuleKey());
                    };
                    LayoutDescriptorCache.prototype.createModuleCache = function () {
                        return new LayoutDescriptorModuleCache();
                    };
                    return LayoutDescriptorCache;
                })(ModuleBasedCache);
                region.LayoutDescriptorCache = LayoutDescriptorCache;
                var LayoutDescriptorModuleCache = (function (_super) {
                    __extends(LayoutDescriptorModuleCache, _super);
                    function LayoutDescriptorModuleCache() {
                        _super.apply(this, arguments);
                    }
                    LayoutDescriptorModuleCache.prototype.copy = function (object) {
                        return object.clone();
                    };
                    LayoutDescriptorModuleCache.prototype.getKeyFromObject = function (object) {
                        return object.getKey();
                    };
                    LayoutDescriptorModuleCache.prototype.getKeyAsString = function (key) {
                        return key.toString();
                    };
                    return LayoutDescriptorModuleCache;
                })(api.cache.Cache);
                region.LayoutDescriptorModuleCache = LayoutDescriptorModuleCache;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionPath = (function () {
                    function RegionPath(parentComponentPath, regionName) {
                        this.parentComponentPath = parentComponentPath;
                        this.regionName = regionName;
                        if (parentComponentPath != null) {
                            this.refString = parentComponentPath + "/" + regionName;
                        }
                        else {
                            this.refString = regionName;
                        }
                    }
                    RegionPath.prototype.hasParentComponentPath = function () {
                        return this.parentComponentPath != null;
                    };
                    RegionPath.prototype.getParentComponentPath = function () {
                        return this.parentComponentPath;
                    };
                    RegionPath.prototype.getRegionName = function () {
                        return this.regionName;
                    };
                    RegionPath.prototype.toString = function () {
                        return this.refString;
                    };
                    RegionPath.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, RegionPath)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                            return false;
                        }
                        return true;
                    };
                    RegionPath.fromString = function (str) {
                        var lastDivider = str.lastIndexOf(RegionPath.DIVIDER);
                        if (lastDivider == -1) {
                            return new RegionPath(null, str);
                        }
                        var regionNameStart = lastDivider + 1;
                        var regionName = str.substring(regionNameStart, str.length);
                        var componentPathAsString = str.substring(0, regionNameStart);
                        var parentPath = region.ComponentPath.fromString(componentPathAsString);
                        return new RegionPath(parentPath, regionName);
                    };
                    RegionPath.DIVIDER = "/";
                    return RegionPath;
                })();
                region.RegionPath = RegionPath;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionDescriptor = (function () {
                    function RegionDescriptor(builder) {
                        this.name = builder.name;
                    }
                    RegionDescriptor.prototype.getName = function () {
                        return this.name;
                    };
                    return RegionDescriptor;
                })();
                region.RegionDescriptor = RegionDescriptor;
                var RegionDescriptorBuilder = (function () {
                    function RegionDescriptorBuilder() {
                    }
                    RegionDescriptorBuilder.prototype.fromJson = function (json) {
                        this.name = json.name;
                        return this;
                    };
                    RegionDescriptorBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    RegionDescriptorBuilder.prototype.build = function () {
                        return new RegionDescriptor(this);
                    };
                    return RegionDescriptorBuilder;
                })();
                region.RegionDescriptorBuilder = RegionDescriptorBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var BaseRegionChangedEvent = (function () {
                    function BaseRegionChangedEvent(path) {
                        this.path = path;
                    }
                    BaseRegionChangedEvent.prototype.getPath = function () {
                        return this.path;
                    };
                    return BaseRegionChangedEvent;
                })();
                region.BaseRegionChangedEvent = BaseRegionChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionPropertyChangedEvent = (function (_super) {
                    __extends(RegionPropertyChangedEvent, _super);
                    function RegionPropertyChangedEvent(path, propertyName) {
                        _super.call(this, path);
                        this.propertyName = propertyName;
                    }
                    RegionPropertyChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    return RegionPropertyChangedEvent;
                })(region.BaseRegionChangedEvent);
                region.RegionPropertyChangedEvent = RegionPropertyChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionPropertyValueChangedEvent = (function (_super) {
                    __extends(RegionPropertyValueChangedEvent, _super);
                    function RegionPropertyValueChangedEvent(path, propertyName) {
                        _super.call(this, path);
                        this.propertyName = propertyName;
                    }
                    RegionPropertyValueChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    return RegionPropertyValueChangedEvent;
                })(region.BaseRegionChangedEvent);
                region.RegionPropertyValueChangedEvent = RegionPropertyValueChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentAddedEvent = (function (_super) {
                    __extends(ComponentAddedEvent, _super);
                    function ComponentAddedEvent(regionPath, componentPath) {
                        _super.call(this, regionPath);
                        this.componentPath = componentPath;
                    }
                    ComponentAddedEvent.prototype.getComponentPath = function () {
                        return this.componentPath;
                    };
                    return ComponentAddedEvent;
                })(region.BaseRegionChangedEvent);
                region.ComponentAddedEvent = ComponentAddedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentRemovedEvent = (function (_super) {
                    __extends(ComponentRemovedEvent, _super);
                    function ComponentRemovedEvent(regionPath, componentPath) {
                        _super.call(this, regionPath);
                        this.componentPath = componentPath;
                    }
                    ComponentRemovedEvent.prototype.getComponentPath = function () {
                        return this.componentPath;
                    };
                    return ComponentRemovedEvent;
                })(region.BaseRegionChangedEvent);
                region.ComponentRemovedEvent = ComponentRemovedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Region = (function () {
                    function Region(builder) {
                        var _this = this;
                        this.components = [];
                        this.changedListeners = [];
                        this.componentAddedListeners = [];
                        this.componentRemovedListeners = [];
                        this.componentPropertyChangedListeners = [];
                        this.propertyValueChangedListeners = [];
                        this.name = builder.name;
                        this.parent = builder.parent;
                        this.componentChangedEventHandler = function (event) {
                            if (Region.debug) {
                                console.debug(_this.toString() + ".handleComponentChanged: ", event);
                            }
                            _this.notifyRegionPropertyValueChanged("components");
                        };
                        this.componentPropertyChangedEventHandler = function (event) { return _this.forwardComponentPropertyChangedEvent(event); };
                        builder.components.forEach(function (component) {
                            _this.registerComponent(component);
                        });
                    }
                    Region.prototype.getName = function () {
                        return this.name;
                    };
                    Region.prototype.setParent = function (value) {
                        this.parent = value;
                    };
                    Region.prototype.getParent = function () {
                        return this.parent;
                    };
                    Region.prototype.getPath = function () {
                        var parentPath = null;
                        if (this.parent) {
                            parentPath = this.parent.getPath();
                        }
                        return new region.RegionPath(parentPath, this.name);
                    };
                    Region.prototype.isEmpty = function () {
                        return !this.components || this.components.length == 0;
                    };
                    Region.prototype.empty = function () {
                        if (Region.debug) {
                            console.debug(this.toString() + ".empty()", this.components);
                        }
                        while (this.components.length > 0) {
                            this.removeComponent(this.components[0]);
                        }
                    };
                    Region.prototype.addComponent = function (component, index) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".addComponent: " + component.toString());
                        }
                        this.registerComponent(component, index);
                        this.notifyComponentAdded(component.getPath());
                        return component;
                    };
                    Region.prototype.removeComponent = function (component) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".removeComponent: " + component.toString(), this.components);
                        }
                        var path = component.getPath();
                        this.unregisterComponent(component);
                        this.notifyComponentRemoved(path);
                        return component;
                    };
                    Region.prototype.getComponents = function () {
                        return this.components;
                    };
                    Region.prototype.getComponentByIndex = function (index) {
                        var component = this.components[index];
                        api.util.assertState(component.getIndex() == index, "Index of Component is not as expected. Expected [" + index + "], was: " + component.getIndex());
                        return component;
                    };
                    Region.prototype.toJson = function () {
                        var componentJsons = [];
                        this.components.forEach(function (component) {
                            componentJsons.push(component.toJson());
                        });
                        return {
                            name: this.name,
                            components: componentJsons
                        };
                    };
                    Region.prototype.toString = function () {
                        return "Region[" + this.getPath().toString() + "]";
                    };
                    Region.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Region)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                            return false;
                        }
                        if (!api.ObjectHelper.arrayEquals(this.components, other.components)) {
                            return false;
                        }
                        return true;
                    };
                    Region.prototype.clone = function (generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new RegionBuilder(this, generateNewPropertyIds).build();
                    };
                    Region.prototype.checkIllegalLayoutComponentWithinLayoutComponent = function (component, parent) {
                        if (!!parent && api.ObjectHelper.iFrameSafeInstanceOf(component, region.LayoutComponent)) {
                            throw new Error("Not allowed to have a LayoutComponent within a LayoutComponent: " + component.getPath().toString());
                        }
                    };
                    Region.prototype.refreshIndexes = function (start) {
                        for (var i = Math.min(0, start); i < this.components.length; i++) {
                            this.components[i].setIndex(i);
                        }
                    };
                    Region.prototype.registerComponent = function (component, index) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".registerComponent: " + component.toString() + " at " + component.getIndex());
                        }
                        this.checkIllegalLayoutComponentWithinLayoutComponent(component, this.parent);
                        if (index >= 0 && index < this.components.length) {
                            this.components.splice(index, 0, component);
                            this.refreshIndexes(index);
                        }
                        else {
                            this.components.push(component);
                            component.setIndex(this.components.length - 1);
                        }
                        component.setParent(this);
                        component.onChanged(this.componentChangedEventHandler);
                        component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                    };
                    Region.prototype.unregisterComponent = function (component) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".unregisterComponent: " + component.toString(), this.components);
                        }
                        component.unChanged(this.componentChangedEventHandler);
                        component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                        var index = component.getIndex();
                        if (index == -1) {
                            throw new Error(component.toString() + " to remove does not exist in " + this.toString());
                        }
                        this.components.splice(index, 1);
                        this.refreshIndexes(index);
                        component.setParent(null);
                        component.setIndex(-1);
                    };
                    Region.prototype.onChanged = function (listener) {
                        this.changedListeners.push(listener);
                    };
                    Region.prototype.unChanged = function (listener) {
                        this.changedListeners = this.changedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyChangedEvent = function (event) {
                        this.changedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Region.prototype.onComponentAdded = function (listener) {
                        this.componentAddedListeners.push(listener);
                    };
                    Region.prototype.unComponentAdded = function (listener) {
                        this.componentAddedListeners = this.componentAddedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyComponentAdded = function (componentPath) {
                        var event = new region.ComponentAddedEvent(this.getPath(), componentPath);
                        this.componentAddedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Region.prototype.onComponentRemoved = function (listener) {
                        this.componentRemovedListeners.push(listener);
                    };
                    Region.prototype.unComponentRemoved = function (listener) {
                        this.componentRemovedListeners = this.componentRemovedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyComponentRemoved = function (componentPath) {
                        var event = new region.ComponentRemovedEvent(this.getPath(), componentPath);
                        this.componentRemovedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Region.prototype.onComponentPropertyChangedEvent = function (listener) {
                        this.componentPropertyChangedListeners.push(listener);
                    };
                    Region.prototype.unComponentPropertyChangedEvent = function (listener) {
                        this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.forwardComponentPropertyChangedEvent = function (event) {
                        this.componentPropertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Region.prototype.onRegionPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners.push(listener);
                    };
                    Region.prototype.unRegionPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyRegionPropertyValueChanged = function (propertyName) {
                        var event = new region.RegionPropertyValueChangedEvent(this.getPath(), propertyName);
                        this.propertyValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Region.create = function (source, generateNewPropertyIds) {
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        return new RegionBuilder(source, generateNewPropertyIds);
                    };
                    Region.debug = false;
                    return Region;
                })();
                region.Region = Region;
                var RegionBuilder = (function () {
                    function RegionBuilder(source, generateNewPropertyIds) {
                        var _this = this;
                        if (generateNewPropertyIds === void 0) { generateNewPropertyIds = false; }
                        this.components = [];
                        if (source) {
                            this.name = source.getName();
                            this.parent = source.getParent();
                            source.getComponents().forEach(function (component) {
                                _this.components.push(component.clone(generateNewPropertyIds));
                            });
                        }
                    }
                    RegionBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    RegionBuilder.prototype.setParent = function (value) {
                        this.parent = value;
                        return this;
                    };
                    RegionBuilder.prototype.addComponent = function (value) {
                        this.components.push(value);
                        return this;
                    };
                    RegionBuilder.prototype.build = function () {
                        return new Region(this);
                    };
                    return RegionBuilder;
                })();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var inputtype;
            (function (inputtype) {
                var pagecontroller;
                (function (pagecontroller) {
                    var support = api.form.inputtype.support;
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var PageDescriptorDropdown = api.content.page.PageDescriptorDropdown;
                    var GetPageDescriptorsByModulesRequest = api.content.page.GetPageDescriptorsByModulesRequest;
                    var PageController = (function (_super) {
                        __extends(PageController, _super);
                        function PageController(context) {
                            _super.call(this, context);
                        }
                        PageController.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        PageController.prototype.newInitialValue = function () {
                            return ValueTypes.STRING.newNullValue();
                        };
                        PageController.prototype.createInputOccurrenceElement = function (index, property) {
                            var context = this.getContext(), moduleKeys = context.site.getModuleKeys(), request = new GetPageDescriptorsByModulesRequest(moduleKeys), loader = new api.util.loader.BaseLoader(request);
                            var dropdown = new PageDescriptorDropdown('page-controller[' + index + ']', {
                                loader: loader
                            });
                            loader.onLoadedData(function (event) {
                                dropdown.setValue(property.getString());
                            });
                            dropdown.onOptionSelected(function (event) {
                                var newValue = new Value(event.getOption().value, ValueTypes.STRING);
                                property.setValue(newValue);
                            });
                            return dropdown;
                        };
                        PageController.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.STRING);
                        };
                        PageController.getName = function () {
                            return new api.form.InputTypeName("PageController", false);
                        };
                        PageController.prototype.hasInputElementValidUserInput = function (inputElement) {
                            return true;
                        };
                        return PageController;
                    })(support.BaseInputTypeNotManagingAdd);
                    pagecontroller.PageController = PageController;
                    api.form.inputtype.InputTypeManager.register(new api.Class(PageController.getName().getName(), PageController));
                })(pagecontroller = inputtype.pagecontroller || (inputtype.pagecontroller = {}));
            })(inputtype = page.inputtype || (page.inputtype = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var PropertyTree = api.data.PropertyTree;
            var ModuleKey = api.module.ModuleKey;
            var ModuleConfig = (function () {
                function ModuleConfig(builder) {
                    this.moduleKey = builder.moduleKey;
                    this.config = builder.config;
                }
                ModuleConfig.prototype.getModuleKey = function () {
                    return this.moduleKey;
                };
                ModuleConfig.prototype.getConfig = function () {
                    return this.config;
                };
                ModuleConfig.prototype.toJson = function () {
                    return {
                        moduleKey: this.moduleKey.toString(),
                        config: this.config.toJson()
                    };
                };
                ModuleConfig.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ModuleConfig)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.moduleKey, other.moduleKey)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.config, other.config)) {
                        return false;
                    }
                    return true;
                };
                ModuleConfig.prototype.toPropertySet = function (parent) {
                    var moduleConfigSet = parent.addPropertySet("moduleConfig");
                    moduleConfigSet.addString("moduleKey", this.moduleKey.getName());
                    moduleConfigSet.addPropertySet("config", this.config.copy(parent.getTree()));
                    return moduleConfigSet;
                };
                ModuleConfig.prototype.clone = function () {
                    return new ModuleConfigBuilder(this).build();
                };
                ModuleConfig.create = function () {
                    return new ModuleConfigBuilder();
                };
                return ModuleConfig;
            })();
            site.ModuleConfig = ModuleConfig;
            var ModuleConfigBuilder = (function () {
                function ModuleConfigBuilder(source) {
                    if (source) {
                        this.moduleKey = source.getModuleKey();
                        if (source.getConfig()) {
                            var newTree = new PropertyTree(source.getConfig().getTree().getIdProvider(), source.getConfig());
                            this.config = newTree.getRoot();
                        }
                    }
                }
                ModuleConfigBuilder.prototype.fromData = function (propertySet) {
                    api.util.assertNotNull(propertySet, "data cannot be null");
                    var moduleKey = ModuleKey.fromString(propertySet.getString("moduleKey"));
                    var moduleConfig = propertySet.getPropertySet("config");
                    this.setModuleKey(moduleKey);
                    this.setConfig(moduleConfig);
                    return this;
                };
                ModuleConfigBuilder.prototype.setModuleKey = function (value) {
                    this.moduleKey = value;
                    return this;
                };
                ModuleConfigBuilder.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                ModuleConfigBuilder.prototype.build = function () {
                    return new ModuleConfig(this);
                };
                return ModuleConfigBuilder;
            })();
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var Site = (function (_super) {
                __extends(Site, _super);
                function Site(builder) {
                    _super.call(this, builder);
                }
                Site.prototype.isSite = function () {
                    return true;
                };
                Site.prototype.getDescription = function () {
                    return this.getContentData().getString("description");
                };
                Site.prototype.getModuleConfigs = function () {
                    var moduleConfigs = [];
                    this.getContentData().forEachProperty("moduleConfig", function (moduleProperty) {
                        var moduleConfigData = moduleProperty.getPropertySet();
                        if (moduleConfigData) {
                            var moduleConfig = site.ModuleConfig.create().fromData(moduleConfigData).build();
                            moduleConfigs.push(moduleConfig);
                        }
                    });
                    return moduleConfigs;
                };
                Site.prototype.getModuleKeys = function () {
                    return this.getModuleConfigs().map(function (config) { return config.getModuleKey(); });
                };
                Site.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Site)) {
                        return false;
                    }
                    return _super.prototype.equals.call(this, o);
                };
                Site.prototype.clone = function () {
                    return this.newBuilder().build();
                };
                Site.prototype.newBuilder = function () {
                    return new SiteBuilder(this);
                };
                return Site;
            })(api.content.Content);
            site.Site = Site;
            var SiteBuilder = (function (_super) {
                __extends(SiteBuilder, _super);
                function SiteBuilder(source) {
                    _super.call(this, source);
                }
                SiteBuilder.prototype.fromContentJson = function (contentJson, propertyIdProvider) {
                    _super.prototype.fromContentJson.call(this, contentJson, propertyIdProvider);
                    return this;
                };
                SiteBuilder.prototype.build = function () {
                    return new Site(this);
                };
                return SiteBuilder;
            })(api.content.ContentBuilder);
            site.SiteBuilder = SiteBuilder;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (_site) {
            var ModuleKey = api.module.ModuleKey;
            var SiteModel = (function () {
                function SiteModel(site) {
                    var _this = this;
                    this.moduleAddedListeners = [];
                    this.moduleRemovedListeners = [];
                    this.propertyChangedListeners = [];
                    this.site = site;
                    this.moduleConfigs = site.getModuleConfigs();
                    this.site.getContentData().onPropertyAdded(function (event) {
                        var property = event.getProperty();
                        if (property.getPath().toString().indexOf(".moduleConfig") == 0 && property.getName() == "config") {
                            var moduleConfig = api.content.site.ModuleConfig.create().fromData(property.getParent()).build();
                            if (!_this.moduleConfigs) {
                                _this.moduleConfigs = [];
                            }
                            _this.moduleConfigs.push(moduleConfig);
                            _this.notifyModuleAdded(moduleConfig);
                        }
                    });
                    this.site.getContentData().onPropertyRemoved(function (event) {
                        var property = event.getProperty();
                        if (property.getName() == "moduleConfig") {
                            var moduleKey = ModuleKey.fromString(property.getPropertySet().getString("moduleKey"));
                            _this.moduleConfigs = _this.moduleConfigs.filter(function (moduleConfig) { return !moduleConfig.getModuleKey().equals(moduleKey); });
                            _this.notifyModuleRemoved(moduleKey);
                        }
                    });
                }
                SiteModel.prototype.getSite = function () {
                    return this.site;
                };
                SiteModel.prototype.getSiteId = function () {
                    return this.site.getContentId();
                };
                SiteModel.prototype.getModuleKeys = function () {
                    return this.moduleConfigs.map(function (mc) { return mc.getModuleKey(); });
                };
                SiteModel.prototype.onPropertyChanged = function (listener) {
                    this.propertyChangedListeners.push(listener);
                };
                SiteModel.prototype.unPropertyChanged = function (listener) {
                    this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                SiteModel.prototype.notifyPropertyChanged = function (property, oldValue, newValue, source) {
                    var event = new api.PropertyChangedEvent(property, oldValue, newValue, source);
                    this.propertyChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                SiteModel.prototype.onModuleAdded = function (listener) {
                    this.moduleAddedListeners.push(listener);
                };
                SiteModel.prototype.unModuleAdded = function (listener) {
                    this.moduleAddedListeners = this.moduleAddedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                SiteModel.prototype.notifyModuleAdded = function (moduleConfig) {
                    var event = new _site.ModuleAddedEvent(moduleConfig);
                    this.moduleAddedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                SiteModel.prototype.onModuleRemoved = function (listener) {
                    this.moduleRemovedListeners.push(listener);
                };
                SiteModel.prototype.unModuleRemoved = function (listener) {
                    this.moduleRemovedListeners = this.moduleRemovedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                SiteModel.prototype.notifyModuleRemoved = function (moduleKey) {
                    var event = new _site.ModuleRemovedEvent(moduleKey);
                    this.moduleRemovedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                SiteModel.PROPERTY_NAME_MODULE_CONFIGS = "moduleConfigs";
                return SiteModel;
            })();
            _site.SiteModel = SiteModel;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var ModuleAddedEvent = (function () {
                function ModuleAddedEvent(moduleConfig) {
                    this.moduleConfig = moduleConfig;
                }
                ModuleAddedEvent.prototype.getModuleKey = function () {
                    return this.moduleConfig.getModuleKey();
                };
                ModuleAddedEvent.prototype.getModuleConfig = function () {
                    return this.moduleConfig;
                };
                return ModuleAddedEvent;
            })();
            site.ModuleAddedEvent = ModuleAddedEvent;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var ModuleRemovedEvent = (function () {
                function ModuleRemovedEvent(moduleKey) {
                    this.moduleKey = moduleKey;
                }
                ModuleRemovedEvent.prototype.getModuleKey = function () {
                    return this.moduleKey;
                };
                return ModuleRemovedEvent;
            })();
            site.ModuleRemovedEvent = ModuleRemovedEvent;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var moduleconfigurator;
                (function (moduleconfigurator) {
                    var ModuleSelectedEvent = (function () {
                        function ModuleSelectedEvent(selectedModule, formView) {
                            this.selectedModule = selectedModule;
                            this.formView = formView;
                        }
                        ModuleSelectedEvent.prototype.getSelectedModule = function () {
                            return this.selectedModule;
                        };
                        ModuleSelectedEvent.prototype.getFormView = function () {
                            return this.formView;
                        };
                        return ModuleSelectedEvent;
                    })();
                    moduleconfigurator.ModuleSelectedEvent = ModuleSelectedEvent;
                })(moduleconfigurator = inputtype.moduleconfigurator || (inputtype.moduleconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var moduleconfigurator;
                (function (moduleconfigurator) {
                    var ModuleDeselectedEvent = (function () {
                        function ModuleDeselectedEvent(selectedModule, formView) {
                            this.deselectedModule = selectedModule;
                            this.formView = formView;
                        }
                        ModuleDeselectedEvent.prototype.getDeselectedModule = function () {
                            return this.deselectedModule;
                        };
                        ModuleDeselectedEvent.prototype.getFormView = function () {
                            return this.formView;
                        };
                        return ModuleDeselectedEvent;
                    })();
                    moduleconfigurator.ModuleDeselectedEvent = ModuleDeselectedEvent;
                })(moduleconfigurator = inputtype.moduleconfigurator || (inputtype.moduleconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var moduleconfigurator;
                (function (moduleconfigurator) {
                    var FormView = api.form.FormView;
                    var ModuleView = (function (_super) {
                        __extends(ModuleView, _super);
                        function ModuleView(mod, moduleConfig, formContext) {
                            var _this = this;
                            _super.call(this, "module-view");
                            this.moduleConfigFormDisplayedListeners = [];
                            this.removeClickedListeners = [];
                            this.collapseClickedListeners = [];
                            this.module = mod;
                            this.moduleConfig = moduleConfig;
                            var header = new api.dom.DivEl('header');
                            var namesAndIconView = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(0 /* large */)).setMainName(this.module.getDisplayName()).setSubName(this.module.getName() + "-" + this.module.getVersion()).setIconClass("icon-xlarge icon-puzzle");
                            header.appendChild(namesAndIconView);
                            var removeButton = new api.dom.AEl("remove-button icon-close");
                            removeButton.onClicked(function (event) {
                                _this.notifyRemoveClicked(event);
                            });
                            header.appendChild(removeButton);
                            var collapseButton = new api.dom.AEl('collapse-button');
                            collapseButton.setHtml('Collapse');
                            collapseButton.onClicked(function (event) {
                                if (_this.formView.isVisible()) {
                                    _this.formView.hide();
                                    collapseButton.setHtml('Expand');
                                    _this.addClass('collapsed');
                                }
                                else {
                                    _this.formView.show();
                                    collapseButton.setHtml('Collapse');
                                    _this.removeClass('collapsed');
                                }
                                _this.notifyCollapseClicked(event);
                            });
                            header.appendChild(collapseButton);
                            this.appendChild(header);
                            this.formView = new FormView(formContext, this.module.getForm(), this.moduleConfig.getConfig());
                            this.formView.addClass("module-form");
                            this.appendChild(this.formView);
                            this.formView.layout().then(function () {
                                _this.notifyModuleConfigFormDisplayed(_this.module.getModuleKey());
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).done();
                        }
                        ModuleView.prototype.getModule = function () {
                            return this.module;
                        };
                        ModuleView.prototype.getModuleConfig = function () {
                            return this.moduleConfig;
                        };
                        ModuleView.prototype.getFormView = function () {
                            return this.formView;
                        };
                        ModuleView.prototype.onRemoveClicked = function (listener) {
                            this.removeClickedListeners.push(listener);
                        };
                        ModuleView.prototype.unRemoveClicked = function (listener) {
                            this.removeClickedListeners = this.removeClickedListeners.filter(function (curr) {
                                return listener != curr;
                            });
                        };
                        ModuleView.prototype.notifyRemoveClicked = function (event) {
                            this.removeClickedListeners.forEach(function (listener) {
                                listener(event);
                            });
                        };
                        ModuleView.prototype.onCollapseClicked = function (listener) {
                            this.collapseClickedListeners.push(listener);
                        };
                        ModuleView.prototype.unCollapseClicked = function (listener) {
                            this.collapseClickedListeners = this.collapseClickedListeners.filter(function (curr) {
                                return listener != curr;
                            });
                        };
                        ModuleView.prototype.notifyCollapseClicked = function (event) {
                            this.collapseClickedListeners.forEach(function (listener) {
                                listener(event);
                            });
                        };
                        ModuleView.prototype.onModuleConfigFormDisplayed = function (listener) {
                            this.moduleConfigFormDisplayedListeners.push(listener);
                        };
                        ModuleView.prototype.unModuleConfigFormDisplayed = function (listener) {
                            this.moduleConfigFormDisplayedListeners = this.moduleConfigFormDisplayedListeners.filter(function (curr) { return (curr != listener); });
                        };
                        ModuleView.prototype.notifyModuleConfigFormDisplayed = function (moduleKey) {
                            this.moduleConfigFormDisplayedListeners.forEach(function (listener) { return listener(moduleKey); });
                        };
                        return ModuleView;
                    })(api.dom.DivEl);
                    moduleconfigurator.ModuleView = ModuleView;
                })(moduleconfigurator = inputtype.moduleconfigurator || (inputtype.moduleconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var moduleconfigurator;
                (function (moduleconfigurator) {
                    var ModuleConfig = api.content.site.ModuleConfig;
                    var ModuleConfigProvider = (function () {
                        function ModuleConfigProvider(propertyArray) {
                            this.propertyArray = propertyArray;
                        }
                        ModuleConfigProvider.prototype.getConfig = function (moduleKey) {
                            var match = null;
                            this.propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    var moduleConfigAsSet = property.getPropertySet();
                                    var moduleConfig = ModuleConfig.create().fromData(moduleConfigAsSet).build();
                                    if (moduleConfig.getModuleKey().equals(moduleKey)) {
                                        match = moduleConfig;
                                    }
                                }
                            });
                            if (!match) {
                                var moduleConfigAsSet = this.propertyArray.addSet();
                                moduleConfigAsSet.addString("moduleKey", moduleKey.toString());
                                moduleConfigAsSet.addPropertySet("config");
                                var newModuleConfig = ModuleConfig.create().fromData(moduleConfigAsSet).build();
                                return newModuleConfig;
                            }
                            else {
                                return match;
                            }
                        };
                        return ModuleConfigProvider;
                    })();
                    moduleconfigurator.ModuleConfigProvider = ModuleConfigProvider;
                })(moduleconfigurator = inputtype.moduleconfigurator || (inputtype.moduleconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var moduleconfigurator;
                (function (moduleconfigurator) {
                    var ValueTypes = api.data.ValueTypes;
                    var ModuleConfig = api.content.site.ModuleConfig;
                    var GetModuleRequest = api.module.GetModuleRequest;
                    var ModuleConfigurator = (function (_super) {
                        __extends(ModuleConfigurator, _super);
                        function ModuleConfigurator(config) {
                            _super.call(this, "module-configurator");
                            this._displayValidationErrors = false;
                            this.context = config;
                            this.formContext = config.formContext;
                        }
                        ModuleConfigurator.prototype.getValueType = function () {
                            return ValueTypes.DATA;
                        };
                        ModuleConfigurator.prototype.newInitialValue = function () {
                            return null;
                        };
                        ModuleConfigurator.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            this.input = input;
                            this.propertyArray = propertyArray;
                            var moduleConfigProvider = new moduleconfigurator.ModuleConfigProvider(propertyArray);
                            this.comboBox = this.createComboBox(input, moduleConfigProvider);
                            this.appendChild(this.comboBox);
                            return this.doLoadModules(propertyArray).then(function () {
                                _this.comboBox.onOptionDeselected(function (removed) {
                                    _this.propertyArray.remove(removed.getIndex());
                                    _this.validate(false);
                                });
                                _this.comboBox.onOptionSelected(function (event) {
                                    var key = event.getOption().displayValue.getModuleKey();
                                    if (!key) {
                                        return;
                                    }
                                    var selectedOption = _this.comboBox.getSelectedOption(event.getOption());
                                    var selectedOptionView = selectedOption.getOptionView();
                                    var moduleConfig = selectedOptionView.getModuleConfig();
                                    _this.validate(false);
                                });
                            });
                        };
                        ModuleConfigurator.prototype.doLoadModules = function (propertyArray) {
                            var _this = this;
                            var deferred = wemQ.defer();
                            var moduleConfigFormsToDisplay = [];
                            if (propertyArray.getSize() == 0) {
                                deferred.resolve(null);
                            }
                            else {
                                propertyArray.forEach(function (property) {
                                    if (property.hasNonNullValue()) {
                                        var moduleConfig = ModuleConfig.create().fromData(property.getPropertySet()).build();
                                        moduleConfigFormsToDisplay.push(moduleConfig.getModuleKey().toString());
                                        new GetModuleRequest(moduleConfig.getModuleKey()).sendAndParse().then(function (requestedModule) {
                                            _this.comboBox.onModuleConfigFormDisplayed(function (moduleKey, formView) {
                                                var indexToRemove = moduleConfigFormsToDisplay.indexOf(moduleKey.toString());
                                                if (indexToRemove != -1) {
                                                    moduleConfigFormsToDisplay.splice(indexToRemove, 1);
                                                }
                                                if (moduleConfigFormsToDisplay.length == 0) {
                                                    deferred.resolve(null);
                                                }
                                                formView.onValidityChanged(function (event) {
                                                    _this.validate(false);
                                                });
                                            });
                                            _this.comboBox.select(requestedModule);
                                        });
                                    }
                                });
                            }
                            return deferred.promise;
                        };
                        ModuleConfigurator.prototype.createComboBox = function (input, moduleConfigProvider) {
                            return new moduleconfigurator.ModuleConfiguratorComboBox(input.getOccurrences().getMaximum() || 0, moduleConfigProvider, this.formContext);
                        };
                        ModuleConfigurator.prototype.availableSizeChanged = function () {
                        };
                        ModuleConfigurator.prototype.displayValidationErrors = function (value) {
                            this._displayValidationErrors = value;
                            this.comboBox.getSelectedOptionViews().forEach(function (view) {
                                view.getFormView().displayValidationErrors(value);
                            });
                        };
                        ModuleConfigurator.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            var recording = new api.form.inputtype.InputValidationRecording();
                            var numberOfValids = this.comboBox.countSelected();
                            if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                                recording.setBreaksMinimumOccurrences(true);
                            }
                            if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                                recording.setBreaksMaximumOccurrences(true);
                            }
                            this.comboBox.getSelectedOptionViews().forEach(function (view) {
                                var validationRecording = view.getFormView().validate(silent);
                                if (!validationRecording.isMinimumOccurrencesValid()) {
                                    recording.setBreaksMinimumOccurrences(true);
                                }
                                if (!validationRecording.isMaximumOccurrencesValid()) {
                                    recording.setBreaksMaximumOccurrences(true);
                                }
                            });
                            if (!silent) {
                                if (recording.validityChanged(this.previousValidationRecording)) {
                                    this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                                }
                            }
                            this.previousValidationRecording = recording;
                            return recording;
                        };
                        ModuleConfigurator.prototype.giveFocus = function () {
                            if (this.comboBox.maximumOccurrencesReached()) {
                                return false;
                            }
                            return this.comboBox.giveFocus();
                        };
                        return ModuleConfigurator;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    moduleconfigurator.ModuleConfigurator = ModuleConfigurator;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ModuleConfigurator", ModuleConfigurator));
                })(moduleconfigurator = inputtype.moduleconfigurator || (inputtype.moduleconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var moduleconfigurator;
                (function (moduleconfigurator) {
                    var ModuleViewer = api.module.ModuleViewer;
                    var ModuleLoader = api.module.ModuleLoader;
                    var SelectedOption = api.ui.selector.combobox.SelectedOption;
                    var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                    var ModuleConfiguratorComboBox = (function (_super) {
                        __extends(ModuleConfiguratorComboBox, _super);
                        function ModuleConfiguratorComboBox(maxOccurrences, moduleConfigProvider, formContext) {
                            this.moduleConfiguratorSelectedOptionsView = new ModuleConfiguratorSelectedOptionsView(moduleConfigProvider, formContext);
                            var builder = new api.ui.selector.combobox.RichComboBoxBuilder();
                            builder.setMaximumOccurrences(maxOccurrences).setIdentifierMethod('getModuleKey').setComboBoxName("moduleSelector").setLoader(new ModuleLoader()).setSelectedOptionsView(this.moduleConfiguratorSelectedOptionsView).setOptionDisplayValueViewer(new ModuleViewer()).setDelayedInputValueChangedHandling(500);
                            _super.call(this, builder);
                        }
                        ModuleConfiguratorComboBox.prototype.getSelectedOptionViews = function () {
                            var views = [];
                            this.getSelectedOptions().forEach(function (selectedOption) {
                                views.push(selectedOption.getOptionView());
                            });
                            return views;
                        };
                        ModuleConfiguratorComboBox.prototype.onModuleConfigFormDisplayed = function (listener) {
                            this.moduleConfiguratorSelectedOptionsView.onModuleConfigFormDisplayed(listener);
                        };
                        ModuleConfiguratorComboBox.prototype.unModuleConfigFormDisplayed = function (listener) {
                            this.moduleConfiguratorSelectedOptionsView.unModuleConfigFormDisplayed(listener);
                        };
                        return ModuleConfiguratorComboBox;
                    })(api.ui.selector.combobox.RichComboBox);
                    moduleconfigurator.ModuleConfiguratorComboBox = ModuleConfiguratorComboBox;
                    var ModuleConfiguratorSelectedOptionsView = (function (_super) {
                        __extends(ModuleConfiguratorSelectedOptionsView, _super);
                        function ModuleConfiguratorSelectedOptionsView(moduleConfigProvider, formContext) {
                            _super.call(this);
                            this.moduleConfigFormDisplayedListeners = [];
                            this.moduleConfigProvider = moduleConfigProvider;
                            this.formContext = formContext;
                        }
                        ModuleConfiguratorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                            var _this = this;
                            var moduleConfig = this.moduleConfigProvider.getConfig(option.displayValue.getModuleKey());
                            var optionView = new ModuleConfiguratorSelectedOptionView(option, moduleConfig, this.formContext);
                            optionView.onModuleConfigFormDisplayed(function (moduleKey) {
                                _this.notifyModuleConfigFormDisplayed(moduleKey, optionView.getFormView());
                            });
                            return new SelectedOption(optionView, this.count());
                        };
                        ModuleConfiguratorSelectedOptionsView.prototype.onModuleConfigFormDisplayed = function (listener) {
                            this.moduleConfigFormDisplayedListeners.push(listener);
                        };
                        ModuleConfiguratorSelectedOptionsView.prototype.unModuleConfigFormDisplayed = function (listener) {
                            this.moduleConfigFormDisplayedListeners = this.moduleConfigFormDisplayedListeners.filter(function (curr) { return (curr != listener); });
                        };
                        ModuleConfiguratorSelectedOptionsView.prototype.notifyModuleConfigFormDisplayed = function (moduleKey, formView) {
                            this.moduleConfigFormDisplayedListeners.forEach(function (listener) { return listener(moduleKey, formView); });
                        };
                        return ModuleConfiguratorSelectedOptionsView;
                    })(BaseSelectedOptionsView);
                    moduleconfigurator.ModuleConfiguratorSelectedOptionsView = ModuleConfiguratorSelectedOptionsView;
                    var ModuleConfiguratorSelectedOptionView = (function (_super) {
                        __extends(ModuleConfiguratorSelectedOptionView, _super);
                        function ModuleConfiguratorSelectedOptionView(option, moduleConfig, formContext) {
                            var _this = this;
                            this.selectedOptionToBeRemovedListeners = [];
                            this.option = option;
                            _super.call(this, option.displayValue, moduleConfig, formContext);
                            this.onRemoveClicked(function (event) {
                                _this.notifySelectedOptionRemoveRequested();
                            });
                        }
                        ModuleConfiguratorSelectedOptionView.prototype.setOption = function (option) {
                            this.option = option;
                        };
                        ModuleConfiguratorSelectedOptionView.prototype.getOption = function () {
                            return this.option;
                        };
                        ModuleConfiguratorSelectedOptionView.prototype.notifySelectedOptionRemoveRequested = function () {
                            this.selectedOptionToBeRemovedListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        ModuleConfiguratorSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                            this.selectedOptionToBeRemovedListeners.push(listener);
                        };
                        ModuleConfiguratorSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                            this.selectedOptionToBeRemovedListeners = this.selectedOptionToBeRemovedListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        return ModuleConfiguratorSelectedOptionView;
                    })(moduleconfigurator.ModuleView);
                    moduleconfigurator.ModuleConfiguratorSelectedOptionView = ModuleConfiguratorSelectedOptionView;
                })(moduleconfigurator = inputtype.moduleconfigurator || (inputtype.moduleconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var ContentFormContext = (function (_super) {
                __extends(ContentFormContext, _super);
                function ContentFormContext(builder) {
                    _super.call(this, builder);
                    this.site = builder.site;
                    this.parentContent = builder.parentContent;
                    this.persistedContent = builder.persistedContent;
                }
                ContentFormContext.prototype.getSite = function () {
                    return this.site;
                };
                ContentFormContext.prototype.getContentId = function () {
                    return this.persistedContent != null ? this.persistedContent.getContentId() : null;
                };
                ContentFormContext.prototype.getContentPath = function () {
                    return this.persistedContent != null ? this.persistedContent.getPath() : null;
                };
                ContentFormContext.prototype.getParentContentPath = function () {
                    if (this.parentContent == null) {
                        return api.content.ContentPath.ROOT;
                    }
                    return this.parentContent.getPath();
                };
                ContentFormContext.prototype.createInputTypeViewContext = function (inputTypeConfig, parentPropertyPath, input) {
                    return {
                        formContext: this,
                        input: input,
                        inputConfig: inputTypeConfig,
                        parentDataPath: parentPropertyPath,
                        site: this.getSite(),
                        contentId: this.getContentId(),
                        contentPath: this.getContentPath(),
                        parentContentPath: this.getParentContentPath()
                    };
                };
                ContentFormContext.create = function () {
                    return new ContentFormContextBuilder();
                };
                return ContentFormContext;
            })(api.form.FormContext);
            form.ContentFormContext = ContentFormContext;
            var ContentFormContextBuilder = (function (_super) {
                __extends(ContentFormContextBuilder, _super);
                function ContentFormContextBuilder() {
                    _super.apply(this, arguments);
                }
                ContentFormContextBuilder.prototype.setSite = function (value) {
                    this.site = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.setParentContent = function (value) {
                    this.parentContent = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.setPersistedContent = function (value) {
                    this.persistedContent = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.build = function () {
                    return new ContentFormContext(this);
                };
                return ContentFormContextBuilder;
            })(api.form.FormContextBuilder);
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var upload;
                (function (upload) {
                    var ValueTypes = api.data.ValueTypes;
                    var ImageUploader = (function (_super) {
                        __extends(ImageUploader, _super);
                        function ImageUploader(config) {
                            _super.call(this, config, "image");
                            var input = config.input;
                            this.imageUploader = new api.content.ImageUploader({
                                params: {
                                    content: config.contentId.toString()
                                },
                                operation: 1 /* update */,
                                name: input.getName(),
                                maximumOccurrences: 1
                            });
                            this.appendChild(this.imageUploader);
                        }
                        ImageUploader.prototype.getContext = function () {
                            return _super.prototype.getContext.call(this);
                        };
                        ImageUploader.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        ImageUploader.prototype.newInitialValue = function () {
                            return ValueTypes.STRING.newNullValue();
                        };
                        ImageUploader.prototype.layoutProperty = function (input, property) {
                            var _this = this;
                            if (property.hasNonNullValue()) {
                                new api.content.GetContentByIdRequest(this.getContext().contentId).sendAndParse().then(function (content) {
                                    _this.imageUploader.setValue(content.getId());
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                            }
                            this.imageUploader.onFileUploaded(function (event) {
                                property.setValue(ValueTypes.STRING.newValue(event.getUploadItem().getModel().getContentId().toString()));
                            });
                            this.imageUploader.onUploadReset(function () {
                                property.setValue(ValueTypes.STRING.newNullValue());
                            });
                            return wemQ(null);
                        };
                        ImageUploader.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            return new api.form.inputtype.InputValidationRecording();
                        };
                        ImageUploader.prototype.onFocus = function (listener) {
                            this.imageUploader.onFocus(listener);
                        };
                        ImageUploader.prototype.unFocus = function (listener) {
                            this.imageUploader.unFocus(listener);
                        };
                        ImageUploader.prototype.onBlur = function (listener) {
                            this.imageUploader.onBlur(listener);
                        };
                        ImageUploader.prototype.unBlur = function (listener) {
                            this.imageUploader.unBlur(listener);
                        };
                        return ImageUploader;
                    })(api.form.inputtype.support.BaseInputTypeSingleOccurrence);
                    upload.ImageUploader = ImageUploader;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ImageUploader", ImageUploader));
                })(upload = inputtype.upload || (inputtype.upload = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var upload;
                (function (upload) {
                    var ValueTypes = api.data.ValueTypes;
                    var FileUploader = (function (_super) {
                        __extends(FileUploader, _super);
                        function FileUploader(config) {
                            _super.call(this, config, "file");
                            var input = config.input;
                            this.fileUploader = new api.content.MediaUploader({
                                params: {
                                    content: config.contentId.toString()
                                },
                                operation: 1 /* update */,
                                name: input.getName(),
                                maximumOccurrences: 1
                            });
                            this.appendChild(this.fileUploader);
                        }
                        FileUploader.prototype.getContext = function () {
                            return _super.prototype.getContext.call(this);
                        };
                        FileUploader.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        FileUploader.prototype.newInitialValue = function () {
                            return ValueTypes.STRING.newNullValue();
                        };
                        FileUploader.prototype.layoutProperty = function (input, property) {
                            if (this.getContext().contentId) {
                                this.fileUploader.setValue(this.getContext().contentId.toString());
                            }
                            this.fileUploader.onFileUploaded(function (event) {
                                property.setValue(ValueTypes.STRING.newValue(event.getUploadItem().getName()));
                            });
                            this.fileUploader.onUploadReset(function () {
                                property.setValue(ValueTypes.STRING.newNullValue());
                            });
                            return wemQ(null);
                        };
                        FileUploader.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            return new api.form.inputtype.InputValidationRecording();
                        };
                        FileUploader.prototype.onFocus = function (listener) {
                            this.fileUploader.onFocus(listener);
                        };
                        FileUploader.prototype.unFocus = function (listener) {
                            this.fileUploader.unFocus(listener);
                        };
                        FileUploader.prototype.onBlur = function (listener) {
                            this.fileUploader.onBlur(listener);
                        };
                        FileUploader.prototype.unBlur = function (listener) {
                            this.fileUploader.unBlur(listener);
                        };
                        return FileUploader;
                    })(api.form.inputtype.support.BaseInputTypeSingleOccurrence);
                    upload.FileUploader = FileUploader;
                    api.form.inputtype.InputTypeManager.register(new api.Class("FileUploader", FileUploader));
                })(upload = inputtype.upload || (inputtype.upload = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var relationship;
                (function (relationship) {
                    var ContentSelectorLoader = (function (_super) {
                        __extends(ContentSelectorLoader, _super);
                        function ContentSelectorLoader() {
                            _super.apply(this, arguments);
                            this.postponedSearchString = null;
                        }
                        ContentSelectorLoader.prototype.setAllowedContentTypes = function (contentTypes) {
                            this.allowedContentTypes = contentTypes;
                            _super.prototype.setAllowedContentTypes.call(this, contentTypes);
                            if (this.postponedSearchString != null) {
                                this.search(this.postponedSearchString);
                            }
                        };
                        ContentSelectorLoader.prototype.search = function (searchString) {
                            return _super.prototype.search.call(this, searchString);
                        };
                        return ContentSelectorLoader;
                    })(api.content.ContentSummaryLoader);
                    relationship.ContentSelectorLoader = ContentSelectorLoader;
                })(relationship = inputtype.relationship || (inputtype.relationship = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var relationship;
                (function (relationship) {
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var GetRelationshipTypeByNameRequest = api.schema.relationshiptype.GetRelationshipTypeByNameRequest;
                    var ContentSelector = (function (_super) {
                        __extends(ContentSelector, _super);
                        function ContentSelector(config) {
                            _super.call(this, "relationship");
                            this.addClass("input-type-view");
                            this.config = config;
                            this.relationshipTypeName = config.inputConfig.relationshipType ? new api.schema.relationshiptype.RelationshipTypeName(config.inputConfig.relationshipType) : api.schema.relationshiptype.RelationshipTypeName.REFERENCE;
                        }
                        ContentSelector.prototype.availableSizeChanged = function () {
                            console.log("Relationship.availableSizeChanged(" + this.getEl().getWidth() + "x" + this.getEl().getWidth() + ")");
                        };
                        ContentSelector.prototype.getValueType = function () {
                            return ValueTypes.REFERENCE;
                        };
                        ContentSelector.prototype.newInitialValue = function () {
                            return null;
                        };
                        ContentSelector.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            this.input = input;
                            this.propertyArray = propertyArray;
                            var relationshipLoader = new relationship.ContentSelectorLoader();
                            this.contentComboBox = api.content.ContentComboBox.create().setName(input.getName()).setMaximumOccurrences(input.getOccurrences().getMaximum()).setLoader(relationshipLoader).build();
                            return new GetRelationshipTypeByNameRequest(this.relationshipTypeName).sendAndParse().then(function (relationshipType) {
                                _this.contentComboBox.setInputIconUrl(relationshipType.getIconUrl());
                                var inputAllowedContentTypes = _this.config.inputConfig.allowedContentTypes || [];
                                var relationshipAllowedContentTypes = relationshipType.getAllowedToTypes() || [];
                                var allowedContentTypes = inputAllowedContentTypes.length ? inputAllowedContentTypes : relationshipAllowedContentTypes;
                                relationshipLoader.setAllowedContentTypes(allowedContentTypes);
                                _this.appendChild(_this.contentComboBox);
                                return _this.doLoadContent(propertyArray).then(function (contents) {
                                    contents.forEach(function (content) {
                                        _this.contentComboBox.select(content);
                                    });
                                    _this.contentComboBox.onOptionSelected(function (event) {
                                        var reference = api.util.Reference.from(event.getOption().displayValue.getContentId());
                                        var value = new Value(reference, ValueTypes.REFERENCE);
                                        if (_this.contentComboBox.countSelected() == 1) {
                                            _this.propertyArray.set(0, value);
                                        }
                                        else {
                                            _this.propertyArray.add(value);
                                        }
                                        _this.validate(false);
                                    });
                                    _this.contentComboBox.onOptionDeselected(function (removed) {
                                        _this.propertyArray.remove(removed.getIndex());
                                        _this.validate(false);
                                    });
                                });
                            });
                        };
                        ContentSelector.prototype.doLoadContent = function (propertyArray) {
                            var contentIds = [];
                            propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    contentIds.push(_content.ContentId.fromReference(property.getReference()));
                                }
                            });
                            return new api.content.GetContentSummaryByIds(contentIds).get().then(function (result) {
                                return result;
                            });
                        };
                        ContentSelector.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            var recording = new api.form.inputtype.InputValidationRecording();
                            var numberOfValids = this.contentComboBox.countSelected();
                            if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                                recording.setBreaksMinimumOccurrences(true);
                            }
                            if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                                recording.setBreaksMaximumOccurrences(true);
                            }
                            if (!silent) {
                                if (recording.validityChanged(this.previousValidationRecording)) {
                                    this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                                }
                            }
                            this.previousValidationRecording = recording;
                            return recording;
                        };
                        ContentSelector.prototype.giveFocus = function () {
                            if (this.contentComboBox.maximumOccurrencesReached()) {
                                return false;
                            }
                            return this.contentComboBox.giveFocus();
                        };
                        ContentSelector.prototype.onFocus = function (listener) {
                            this.contentComboBox.onFocus(listener);
                        };
                        ContentSelector.prototype.unFocus = function (listener) {
                            this.contentComboBox.unFocus(listener);
                        };
                        ContentSelector.prototype.onBlur = function (listener) {
                            this.contentComboBox.onBlur(listener);
                        };
                        ContentSelector.prototype.unBlur = function (listener) {
                            this.contentComboBox.unBlur(listener);
                        };
                        return ContentSelector;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    relationship.ContentSelector = ContentSelector;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ContentSelector", ContentSelector));
                })(relationship = inputtype.relationship || (inputtype.relationship = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var LoadMask = api.ui.mask.LoadMask;
                    var ImageSelectorSelectedOptionView = (function (_super) {
                        __extends(ImageSelectorSelectedOptionView, _super);
                        function ImageSelectorSelectedOptionView(option) {
                            _super.call(this, option);
                            this.selectionChangeListeners = [];
                        }
                        ImageSelectorSelectedOptionView.prototype.setOption = function (option) {
                            _super.prototype.setOption.call(this, option);
                            var content = this.getOption().displayValue;
                            if (content.getContentSummary()) {
                                if (this.isVisible()) {
                                    this.showSpinner();
                                }
                                this.icon.setSrc(content.getImageUrl() + "?thumbnail=false&size=" + ImageSelectorSelectedOptionView.IMAGE_SIZE);
                                this.label.getEl().setInnerHtml(content.getLabel());
                            }
                            else {
                                this.showProgress();
                            }
                        };
                        ImageSelectorSelectedOptionView.prototype.setProgress = function (value) {
                            this.progress.setValue(value);
                            if (value == 100) {
                                this.showSpinner();
                            }
                        };
                        ImageSelectorSelectedOptionView.prototype.layout = function () {
                            var _this = this;
                            this.icon = new api.dom.ImgEl();
                            this.appendChild(this.icon);
                            this.label = new api.dom.DivEl("label");
                            this.appendChild(this.label);
                            this.check = new api.ui.Checkbox();
                            this.appendChild(this.check);
                            this.progress = new api.ui.ProgressBar();
                            this.appendChild(this.progress);
                            this.error = new api.dom.DivEl("error");
                            this.appendChild(this.error);
                            this.loadMask = new LoadMask(this);
                            this.appendChild(this.loadMask);
                            this.check.onClicked(function (event) {
                                _this.check.toggleChecked();
                                event.preventDefault();
                                event.stopPropagation();
                            });
                            this.check.onMouseDown(function (event) {
                                event.stopPropagation();
                                event.preventDefault();
                            });
                            this.check.onValueChanged(function (event) {
                                _this.notifyChecked(event.getNewValue() == 'true');
                            });
                            this.onShown(function () {
                                if (_this.getOption().displayValue.getContentSummary()) {
                                    if (_this.icon.isLoaded()) {
                                        _this.icon.refresh();
                                    }
                                    else {
                                        _this.showSpinner();
                                    }
                                }
                            });
                            this.icon.onLoaded(function (event) {
                                if (_this.getOption().displayValue.getContentSummary()) {
                                    _this.showResult();
                                }
                            });
                        };
                        ImageSelectorSelectedOptionView.prototype.showProgress = function () {
                            this.check.hide();
                            this.icon.getEl().setVisibility('hidden');
                            this.loadMask.hide();
                            this.progress.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.showSpinner = function () {
                            this.progress.hide();
                            this.check.hide();
                            this.icon.getEl().setVisibility('hidden');
                            this.loadMask.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.showResult = function () {
                            this.loadMask.hide();
                            this.icon.getEl().setVisibility('visible');
                            this.check.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.showError = function (text) {
                            this.progress.hide();
                            this.error.setHtml(text).show();
                            this.check.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.updateProportions = function (optionHeight) {
                            this.getEl().setHeightPx(optionHeight);
                            var iconHeight = this.icon.getEl().getHeightWithBorder();
                            var contentHeight = optionHeight - this.getEl().getBorderTopWidth() - this.getEl().getBorderBottomWidth();
                            if (iconHeight <= contentHeight && iconHeight !== 0) {
                                this.icon.getEl().setMarginTop((contentHeight - iconHeight) / 2 + 'px');
                            }
                            else {
                                this.icon.getEl().setMarginTop('0px');
                            }
                            this.centerVertically(this.progress, contentHeight);
                            this.centerVertically(this.error, contentHeight);
                        };
                        ImageSelectorSelectedOptionView.prototype.centerVertically = function (el, contentHeight) {
                            el.getEl().setMarginTop((contentHeight - this.progress.getEl().getHeight()) / 2 + 'px');
                        };
                        ImageSelectorSelectedOptionView.prototype.getIcon = function () {
                            return this.icon;
                        };
                        ImageSelectorSelectedOptionView.prototype.getCheckbox = function () {
                            return this.check;
                        };
                        ImageSelectorSelectedOptionView.prototype.toggleChecked = function () {
                            this.check.toggleChecked();
                        };
                        ImageSelectorSelectedOptionView.prototype.notifyChecked = function (checked) {
                            var _this = this;
                            this.selectionChangeListeners.forEach(function (listener) {
                                listener(_this, checked);
                            });
                        };
                        ImageSelectorSelectedOptionView.prototype.onChecked = function (listener) {
                            this.selectionChangeListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionView.prototype.unChecked = function (listener) {
                            this.selectionChangeListeners = this.selectionChangeListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelectorSelectedOptionView.IMAGE_SIZE = 270;
                        return ImageSelectorSelectedOptionView;
                    })(api.ui.selector.combobox.BaseSelectedOptionView);
                    image.ImageSelectorSelectedOptionView = ImageSelectorSelectedOptionView;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var SelectedOption = api.ui.selector.combobox.SelectedOption;
                    var ValueTypes = api.data.ValueTypes;
                    var ValueChangedEvent = api.form.inputtype.ValueChangedEvent;
                    var ImageSelectorSelectedOptionsView = (function (_super) {
                        __extends(ImageSelectorSelectedOptionsView, _super);
                        function ImageSelectorSelectedOptionsView() {
                            var _this = this;
                            _super.call(this);
                            this.numberOfOptionsPerRow = 3;
                            this.selection = [];
                            this.editSelectedOptionsListeners = [];
                            this.removeSelectedOptionsListeners = [];
                            this.valueChangedListeners = [];
                            this.dialog = new image.ImageSelectorDialog();
                            wemjq(this.getHTMLElement()).sortable({
                                containment: this.getHTMLElement(),
                                cursor: 'move',
                                tolerance: 'pointer',
                                start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                                update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                            });
                            this.toolbar = new image.SelectionToolbar();
                            this.toolbar.hide();
                            this.toolbar.onEditClicked(function () {
                                _this.notifyEditSelectedOptions(_this.selection);
                            });
                            this.toolbar.onRemoveClicked(function () {
                                _this.notifyRemoveSelectedOptions(_this.selection);
                                _this.selection.length = 0;
                                _this.updateSelectionToolbarLayout();
                            });
                            this.appendChild(this.toolbar);
                            this.onShown(function (event) {
                                _this.updateLayout();
                            });
                        }
                        ImageSelectorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                            return new SelectedOption(new image.ImageSelectorSelectedOptionView(option), this.count());
                        };
                        ImageSelectorSelectedOptionsView.prototype.addOption = function (option) {
                            this.dialog.remove();
                            var selectedOption = this.getByOption(option);
                            if (!selectedOption && !this.maximumOccurrencesReached()) {
                                this.addNewOption(option);
                                return true;
                            }
                            else if (selectedOption) {
                                var displayValue = selectedOption.getOption().displayValue;
                                if (displayValue.getContentSummary() == null && option.displayValue.getContentSummary() != null) {
                                    this.updateUplodedOption(option);
                                    return true;
                                }
                            }
                            return false;
                        };
                        ImageSelectorSelectedOptionsView.prototype.addNewOption = function (option) {
                            var _this = this;
                            var selectedOption = this.createSelectedOption(option);
                            this.getSelectedOptions().push(selectedOption);
                            var optionView = selectedOption.getOptionView();
                            optionView.updateProportions(this.calculateOptionHeight());
                            optionView.onClicked(function (event) {
                                if (_this.dialog.isVisible()) {
                                    _this.hideImageSelectorDialog();
                                    if (optionView.getOption().displayValue != _this.dialog.getContent()) {
                                        _this.showImageSelectorDialog(selectedOption);
                                        optionView.getCheckbox().giveFocus();
                                    }
                                }
                                else {
                                    _this.showImageSelectorDialog(selectedOption);
                                    optionView.getCheckbox().giveFocus();
                                }
                            });
                            optionView.getCheckbox().onKeyPressed(function (event) {
                                var checkbox = optionView.getCheckbox();
                                switch (event.which) {
                                    case 32:
                                        var isChecked = !checkbox.isChecked();
                                        checkbox.setChecked(isChecked, isChecked);
                                        break;
                                    case 8:
                                        checkbox.setChecked(false);
                                        _this.removeOptionViewAndRefocus(selectedOption);
                                        event.preventDefault();
                                        break;
                                    case 46:
                                        checkbox.setChecked(false);
                                        _this.removeOptionViewAndRefocus(selectedOption);
                                        break;
                                    case 13:
                                        _this.notifyEditSelectedOptions([selectedOption]);
                                        break;
                                }
                                event.stopPropagation();
                            });
                            optionView.getCheckbox().onFocus(function (event) { return _this.showImageSelectorDialog(selectedOption); });
                            optionView.getCheckbox().onBlur(function (event) { return _this.hideImageSelectorDialog(); });
                            optionView.onChecked(function (view, checked) {
                                if (checked) {
                                    _this.selection.push(selectedOption);
                                }
                                else {
                                    var index = _this.selection.indexOf(selectedOption);
                                    if (index > -1) {
                                        _this.selection.splice(index, 1);
                                    }
                                }
                                _this.updateSelectionToolbarLayout();
                            });
                            optionView.getIcon().onLoaded(function (event) {
                                optionView.updateProportions(_this.calculateOptionHeight());
                                wemjq(_this.getHTMLElement()).sortable("refresh");
                            });
                            optionView.insertBeforeEl(this.toolbar);
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateUplodedOption = function (option) {
                            var selectedOption = this.getByOption(option);
                            var content = option.displayValue.getContentSummary();
                            var newOption = {
                                value: content.getId(),
                                displayValue: image.ImageSelectorDisplayValue.fromContentSummary(content)
                            };
                            selectedOption.getOptionView().setOption(newOption);
                        };
                        ImageSelectorSelectedOptionsView.prototype.removeOptionViewAndRefocus = function (option) {
                            var index = this.isLast(option.getIndex()) ? (this.isFirst(option.getIndex()) ? -1 : option.getIndex() - 1) : option.getIndex();
                            this.notifyRemoveSelectedOptions([option]);
                            this.hideImageSelectorDialog();
                            if (index > -1) {
                                this.getByIndex(index).getOptionView().getCheckbox().giveFocus();
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.showImageSelectorDialog = function (option) {
                            if (this.dialog) {
                                this.dialog.remove();
                            }
                            var selectedOptions = this.getSelectedOptions();
                            for (var i = 0; i < selectedOptions.length; i++) {
                                var view = selectedOptions[i].getOptionView();
                                var passedSelectedOption = i >= option.getIndex();
                                if ((this.isLastInRow(i) || this.isLast(i)) && passedSelectedOption) {
                                    this.dialog.insertAfterEl(view);
                                    break;
                                }
                            }
                            if (this.activeOption) {
                                this.activeOption.getOptionView().removeClass("editing");
                            }
                            this.activeOption = option;
                            option.getOptionView().addClass("editing");
                            this.dialog.setContent(option.getOption().displayValue);
                            this.setOutsideClickListener();
                            this.updateDialogLayout(this.calculateOptionHeight());
                            wemjq(this.getHTMLElement()).sortable("disable");
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateLayout = function () {
                            var optionHeight = this.calculateOptionHeight();
                            this.getSelectedOptions().forEach(function (selectedOption) {
                                selectedOption.getOptionView().updateProportions(optionHeight);
                            });
                            if (this.dialog.isVisible()) {
                                this.updateDialogLayout(optionHeight);
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateDialogLayout = function (optionHeight) {
                            this.dialog.getEl().setMarginTop(((optionHeight / 3) - 20) + 'px');
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateSelectionToolbarLayout = function () {
                            var showToolbar = this.selection.length > 0;
                            this.toolbar.setVisible(showToolbar);
                            if (showToolbar) {
                                this.toolbar.setSelectionCount(this.selection.length);
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.calculateOptionHeight = function () {
                            var availableWidth = this.getEl().getWidthWithMargin();
                            return Math.floor(0.3 * availableWidth);
                        };
                        ImageSelectorSelectedOptionsView.prototype.hideImageSelectorDialog = function () {
                            this.dialog.remove();
                            this.activeOption.getOptionView().removeClass('editing first-in-row last-in-row');
                            wemjq(this.getHTMLElement()).sortable("enable");
                            api.dom.Body.get().unClicked(this.mouseClickListener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.setOutsideClickListener = function () {
                            var _this = this;
                            this.mouseClickListener = function (event) {
                                for (var element = event.target; element; element = element.parentNode) {
                                    if (element == _this.getHTMLElement()) {
                                        return;
                                    }
                                }
                                _this.hideImageSelectorDialog();
                            };
                            api.dom.Body.get().onClicked(this.mouseClickListener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.isFirstInRow = function (index) {
                            return index % this.numberOfOptionsPerRow == 0;
                        };
                        ImageSelectorSelectedOptionsView.prototype.isLastInRow = function (index) {
                            return index % this.numberOfOptionsPerRow == 2;
                        };
                        ImageSelectorSelectedOptionsView.prototype.isFirst = function (index) {
                            return index == 0;
                        };
                        ImageSelectorSelectedOptionsView.prototype.isLast = function (index) {
                            return index == this.getSelectedOptions().length - 1;
                        };
                        ImageSelectorSelectedOptionsView.prototype.handleDnDStart = function (event, ui) {
                            var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                            this.draggingIndex = draggedElement.getSiblingIndex();
                        };
                        ImageSelectorSelectedOptionsView.prototype.handleDnDUpdate = function (event, ui) {
                            if (this.draggingIndex >= 0) {
                                var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                                var draggedToIndex = draggedElement.getSiblingIndex();
                                this.handleMovedOccurrence(this.draggingIndex, draggedToIndex);
                            }
                            this.draggingIndex = -1;
                        };
                        ImageSelectorSelectedOptionsView.prototype.handleMovedOccurrence = function (fromIndex, toIndex) {
                            var _this = this;
                            this.moveOccurrence(fromIndex, toIndex);
                            this.getValues().forEach(function (value, index) {
                                if (Math.min(fromIndex, toIndex) <= index && index <= Math.max(fromIndex, toIndex)) {
                                    _this.notifyValueChanged(new ValueChangedEvent(value, index));
                                }
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.getValues = function () {
                            return this.getSelectedOptions().map(function (selectedOption) {
                                return ValueTypes.REFERENCE.newValue(selectedOption.getOption().value);
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.notifyRemoveSelectedOptions = function (option) {
                            this.removeSelectedOptionsListeners.forEach(function (listener) {
                                listener(option);
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.onRemoveSelectedOptions = function (listener) {
                            this.removeSelectedOptionsListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.unRemoveSelectedOptions = function (listener) {
                            this.removeSelectedOptionsListeners = this.removeSelectedOptionsListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.notifyEditSelectedOptions = function (option) {
                            this.editSelectedOptionsListeners.forEach(function (listener) {
                                listener(option);
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.onEditSelectedOptions = function (listener) {
                            this.editSelectedOptionsListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.unEditSelectedOptions = function (listener) {
                            this.editSelectedOptionsListeners = this.editSelectedOptionsListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.onValueChanged = function (listener) {
                            this.valueChangedListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.unValueChanged = function (listener) {
                            this.valueChangedListeners.filter(function (currentListener) {
                                return listener == currentListener;
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.notifyValueChanged = function (event) {
                            this.valueChangedListeners.forEach(function (listener) {
                                listener(event);
                            });
                        };
                        return ImageSelectorSelectedOptionsView;
                    })(api.ui.selector.combobox.BaseSelectedOptionsView);
                    image.ImageSelectorSelectedOptionsView = ImageSelectorSelectedOptionsView;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ImageSelectorDialog = (function (_super) {
                        __extends(ImageSelectorDialog, _super);
                        function ImageSelectorDialog() {
                            _super.call(this, "dialog");
                            this.nameEl = new api.dom.H1El();
                            this.appendChild(this.nameEl);
                            this.pathEl = new api.dom.PEl();
                            this.appendChild(this.pathEl);
                        }
                        ImageSelectorDialog.prototype.setContent = function (value) {
                            this.content = value;
                            this.refreshUI();
                        };
                        ImageSelectorDialog.prototype.getContent = function () {
                            return this.content;
                        };
                        ImageSelectorDialog.prototype.refreshUI = function () {
                            this.nameEl.getEl().setInnerHtml(this.content.getLabel());
                            this.pathEl.getEl().setInnerHtml(this.content.getPath());
                        };
                        return ImageSelectorDialog;
                    })(api.dom.DivEl);
                    image.ImageSelectorDialog = ImageSelectorDialog;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var ContentId = api.content.ContentId;
                    var ContentSummaryLoader = api.content.ContentSummaryLoader;
                    var ContentTypeName = api.schema.content.ContentTypeName;
                    var ComboBox = api.ui.selector.combobox.ComboBox;
                    var ResponsiveManager = api.ui.responsive.ResponsiveManager;
                    var RelationshipTypeName = api.schema.relationshiptype.RelationshipTypeName;
                    var ImageSelector = (function (_super) {
                        __extends(ImageSelector, _super);
                        function ImageSelector(config) {
                            var _this = this;
                            _super.call(this, "image-selector");
                            this.editContentRequestListeners = [];
                            this.addClass("input-type-view");
                            this.config = config;
                            this.contentRequestsAllowed = false;
                            if (config.inputConfig.relationshipType) {
                                this.relationshipTypeName = new RelationshipTypeName(config.inputConfig.relationshipType);
                            }
                            else {
                                this.relationshipTypeName = RelationshipTypeName.REFERENCE;
                            }
                            ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                                _this.availableSizeChanged();
                            });
                            this.onRemoved(function (event) {
                                _this.uploadDialog.remove();
                                ResponsiveManager.unAvailableSizeChanged(_this);
                            });
                        }
                        ImageSelector.prototype.availableSizeChanged = function () {
                            this.selectedOptionsView.updateLayout();
                        };
                        ImageSelector.prototype.getValueType = function () {
                            return ValueTypes.REFERENCE;
                        };
                        ImageSelector.prototype.newInitialValue = function () {
                            return null;
                        };
                        ImageSelector.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            this.layoutInProgress = true;
                            this.input = input;
                            this.propertyArray = propertyArray;
                            var comboboxWrapper = new api.dom.DivEl("combobox-wrapper");
                            this.comboBox = this.createComboBox(input);
                            comboboxWrapper.appendChild(this.comboBox);
                            this.comboBox.onHidden(function (event) {
                                _this.uploadButton.hide();
                            });
                            this.comboBox.onShown(function (event) {
                                _this.uploadButton.show();
                            });
                            this.uploadButton = new api.dom.ButtonEl();
                            comboboxWrapper.appendChild(this.uploadButton);
                            this.uploadButton.addClass("upload-button");
                            this.uploadButton.onClicked(function (event) {
                                var inputMaximum = input.getOccurrences().getMaximum();
                                var countSelected = _this.comboBox.countSelectedOptions();
                                var rest = -1;
                                if (inputMaximum == 0) {
                                    rest = 0;
                                }
                                else {
                                    rest = inputMaximum - countSelected;
                                    rest = (rest == 0) ? -1 : rest;
                                }
                                _this.uploadDialog.setMaximumOccurrences(rest);
                                _this.uploadDialog.open();
                            });
                            this.appendChild(comboboxWrapper);
                            this.appendChild(this.selectedOptionsView);
                            this.contentSummaryLoader = new ContentSummaryLoader();
                            this.contentSummaryLoader.onLoadingData(function (event) {
                                _this.comboBox.setEmptyDropdownText("Searching...");
                            });
                            this.contentSummaryLoader.onLoadedData(function (event) {
                                var options = _this.createOptions(event.getData());
                                _this.comboBox.setOptions(options);
                            });
                            return new api.schema.relationshiptype.GetRelationshipTypeByNameRequest(this.relationshipTypeName).sendAndParse().then(function (relationshipType) {
                                _this.comboBox.setInputIconUrl(relationshipType.getIconUrl());
                                var relationshipAllowedContentTypes = relationshipType.getAllowedToTypes() || [];
                                var allowedContentTypes = relationshipAllowedContentTypes.length ? relationshipAllowedContentTypes : [ContentTypeName.IMAGE.toString()];
                                _this.contentSummaryLoader.setAllowedContentTypes(allowedContentTypes);
                                _this.contentRequestsAllowed = true;
                                _this.loadOptions("");
                                if (_this.config.contentId) {
                                    _this.layoutUploadDialog();
                                }
                                var loadContentPromise = _this.doLoadContent(_this.propertyArray);
                                return loadContentPromise.then(function (contents) {
                                    contents.forEach(function (content) {
                                        _this.comboBox.selectOption({
                                            value: content.getId(),
                                            displayValue: image.ImageSelectorDisplayValue.fromContentSummary(content)
                                        });
                                    });
                                    _this.layoutInProgress = false;
                                });
                            });
                        };
                        ImageSelector.prototype.layoutUploadDialog = function () {
                            var _this = this;
                            this.uploadDialog = new image.ImageUploadDialog(this.config.contentId);
                            this.uploadDialog.onUploadStarted(function (event) {
                                _this.uploadDialog.close();
                                event.getUploadItems().forEach(function (uploadItem) {
                                    var value = image.ImageSelectorDisplayValue.fromUploadItem(uploadItem);
                                    var option = {
                                        value: value.getId(),
                                        displayValue: value
                                    };
                                    _this.comboBox.selectOption(option);
                                });
                            });
                            this.uploadDialog.onUploadProgress(function (event) {
                                var item = event.getUploadItem();
                                var selectedOption = _this.selectedOptionsView.getById(item.getId());
                                selectedOption.getOptionView().setProgress(item.getProgress());
                            });
                            this.uploadDialog.onImageUploaded(function (event) {
                                var item = event.getUploadItem();
                                var createdContent = item.getModel();
                                new api.content.ContentUpdatedEvent(_this.config.contentId).fire();
                                var selectedOption = _this.selectedOptionsView.getById(item.getId());
                                var option = selectedOption.getOption();
                                option.displayValue.setContentSummary(createdContent);
                                selectedOption.getOptionView().setOption(option);
                                _this.setContentIdProperty(createdContent.getContentId());
                                _this.validate(false);
                            });
                            this.uploadDialog.onUploadFailed(function (event) {
                                var item = event.getUploadItem();
                                var selectedOption = _this.selectedOptionsView.getById(item.getId());
                                selectedOption.getOptionView().showError("Upload failed");
                            });
                        };
                        ImageSelector.prototype.doLoadContent = function (propertyArray) {
                            var contentIds = [];
                            propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    contentIds.push(ContentId.fromReference(property.getReference()));
                                }
                            });
                            return new api.content.GetContentSummaryByIds(contentIds).get();
                        };
                        ImageSelector.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            var recording = new api.form.inputtype.InputValidationRecording();
                            var numberOfValids = this.comboBox.countSelectedOptions();
                            if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                                recording.setBreaksMinimumOccurrences(true);
                            }
                            if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                                recording.setBreaksMaximumOccurrences(true);
                            }
                            if (!silent) {
                                if (recording.validityChanged(this.previousValidationRecording)) {
                                    this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                                }
                            }
                            this.previousValidationRecording = recording;
                            return recording;
                        };
                        ImageSelector.prototype.giveFocus = function () {
                            if (this.comboBox.maximumOccurrencesReached()) {
                                return false;
                            }
                            return this.comboBox.giveFocus();
                        };
                        ImageSelector.prototype.createComboBox = function (input) {
                            var _this = this;
                            this.selectedOptionsView = new image.ImageSelectorSelectedOptionsView();
                            this.selectedOptionsView.onEditSelectedOptions(function (options) {
                                options.forEach(function (option) {
                                    _this.notifyEditContentRequested(option.getOption().displayValue.getContentSummary());
                                });
                            });
                            this.selectedOptionsView.onRemoveSelectedOptions(function (options) {
                                options.forEach(function (option) {
                                    _this.comboBox.deselectOption(option.getOption());
                                });
                                _this.validate(false);
                            });
                            this.selectedOptionsView.onValueChanged(function (event) {
                                _this.propertyArray.set(event.getArrayIndex(), event.getNewValue());
                                _this.validate(false);
                            });
                            var comboBoxConfig = {
                                hideComboBoxWhenMaxReached: true,
                                optionDisplayValueViewer: new image.ImageSelectorViewer(),
                                selectedOptionsView: this.selectedOptionsView,
                                maximumOccurrences: input.getOccurrences().getMaximum(),
                                delayedInputValueChangedHandling: 500
                            };
                            var comboBox = new ComboBox(input.getName(), comboBoxConfig);
                            this.loadOptions("");
                            comboBox.onOptionDeselected(function (removed) {
                                _this.propertyArray.remove(removed.getIndex());
                                _this.validate(false);
                            });
                            comboBox.onOptionFilterInputValueChanged(function (event) {
                                _this.loadOptions(event.getNewValue());
                            });
                            comboBox.onOptionSelected(function (event) {
                                if (!_this.layoutInProgress) {
                                    var contentId = event.getOption().displayValue.getContentId();
                                    if (!contentId) {
                                        return;
                                    }
                                    _this.setContentIdProperty(contentId);
                                }
                                _this.validate(false);
                            });
                            return comboBox;
                        };
                        ImageSelector.prototype.loadOptions = function (searchString) {
                            if (!this.contentRequestsAllowed || !this.comboBox) {
                                return;
                            }
                            this.contentSummaryLoader.search(searchString);
                        };
                        ImageSelector.prototype.createOptions = function (contents) {
                            return contents.map(function (content) {
                                return {
                                    value: content.getId(),
                                    displayValue: image.ImageSelectorDisplayValue.fromContentSummary(content)
                                };
                            });
                        };
                        ImageSelector.prototype.setContentIdProperty = function (contentId) {
                            var reference = api.util.Reference.from(contentId);
                            var value = new Value(reference, ValueTypes.REFERENCE);
                            if (this.comboBox.countSelectedOptions() == 1) {
                                this.propertyArray.set(0, value);
                            }
                            else {
                                this.propertyArray.add(value);
                            }
                        };
                        ImageSelector.prototype.onFocus = function (listener) {
                            this.comboBox.onFocus(listener);
                        };
                        ImageSelector.prototype.unFocus = function (listener) {
                            this.comboBox.unFocus(listener);
                        };
                        ImageSelector.prototype.onBlur = function (listener) {
                            this.comboBox.onBlur(listener);
                        };
                        ImageSelector.prototype.unBlur = function (listener) {
                            this.comboBox.unBlur(listener);
                        };
                        ImageSelector.prototype.onEditContentRequest = function (listener) {
                            this.editContentRequestListeners.push(listener);
                        };
                        ImageSelector.prototype.unEditContentRequest = function (listener) {
                            this.editContentRequestListeners = this.editContentRequestListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelector.prototype.notifyEditContentRequested = function (content) {
                            this.editContentRequestListeners.forEach(function (listener) {
                                listener(content);
                            });
                        };
                        return ImageSelector;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    image.ImageSelector = ImageSelector;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ImageSelector", ImageSelector));
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var UploadDialogCancelAction = (function (_super) {
                        __extends(UploadDialogCancelAction, _super);
                        function UploadDialogCancelAction() {
                            _super.call(this, "Cancel", "esc");
                        }
                        return UploadDialogCancelAction;
                    })(api.ui.Action);
                    image.UploadDialogCancelAction = UploadDialogCancelAction;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ImageUploadDialog = (function (_super) {
                        __extends(ImageUploadDialog, _super);
                        function ImageUploadDialog(parentContent) {
                            var _this = this;
                            api.util.assertNotNull(parentContent, "parentContent required");
                            _super.call(this, {
                                title: new api.ui.dialog.ModalDialogHeader("Image uploader")
                            });
                            this.getEl().addClass("upload-dialog");
                            var description = new api.dom.PEl();
                            description.getEl().setInnerHtml("Images uploaded will be added as children to this content, you may move them later if desired");
                            this.appendChildToContentPanel(description);
                            this.imageUploader = new api.content.ImageUploader({
                                params: {
                                    parent: parentContent.toString()
                                },
                                operation: 0 /* create */,
                                name: 'image-selector-upload-dialog',
                                showButtons: false,
                                showResult: false
                            });
                            this.imageUploader.onUploadCompleted(function () {
                                _this.close();
                            });
                            this.appendChildToContentPanel(this.imageUploader);
                            this.getCancelAction().onExecuted(function (action) {
                                _this.imageUploader.stop();
                                _this.imageUploader.reset();
                                _this.close();
                            });
                            api.dom.Body.get().appendChild(this);
                            this.addCancelButtonToBottom();
                        }
                        ImageUploadDialog.prototype.setMaximumOccurrences = function (value) {
                            this.imageUploader.setMaximumOccurrences(value);
                        };
                        ImageUploadDialog.prototype.open = function () {
                            this.imageUploader.reset();
                            _super.prototype.open.call(this);
                        };
                        ImageUploadDialog.prototype.onUploadStarted = function (listener) {
                            this.imageUploader.onUploadStarted(listener);
                        };
                        ImageUploadDialog.prototype.unUploadStarted = function (listener) {
                            this.imageUploader.unUploadStarted(listener);
                        };
                        ImageUploadDialog.prototype.onUploadProgress = function (listener) {
                            this.imageUploader.onUploadProgress(listener);
                        };
                        ImageUploadDialog.prototype.unUploadProgress = function (listener) {
                            this.imageUploader.unUploadProgress(listener);
                        };
                        ImageUploadDialog.prototype.onImageUploaded = function (listener) {
                            this.imageUploader.onFileUploaded(listener);
                        };
                        ImageUploadDialog.prototype.unImageUploaded = function (listener) {
                            this.imageUploader.unFileUploaded(listener);
                        };
                        ImageUploadDialog.prototype.onUploadCompleted = function (listener) {
                            this.imageUploader.onUploadCompleted(listener);
                        };
                        ImageUploadDialog.prototype.unUploadCompleted = function (listener) {
                            this.imageUploader.unUploadCompleted(listener);
                        };
                        ImageUploadDialog.prototype.onUploadReset = function (listener) {
                            this.imageUploader.onUploadReset(listener);
                        };
                        ImageUploadDialog.prototype.unUploadReset = function (listener) {
                            this.imageUploader.unUploadReset(listener);
                        };
                        ImageUploadDialog.prototype.onUploadFailed = function (listener) {
                            this.imageUploader.onUploadFailed(listener);
                        };
                        ImageUploadDialog.prototype.unUploadFailed = function (listener) {
                            this.imageUploader.unUploadFailed(listener);
                        };
                        return ImageUploadDialog;
                    })(api.ui.dialog.ModalDialog);
                    image.ImageUploadDialog = ImageUploadDialog;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var Button = api.ui.button.Button;
                    var SelectionToolbar = (function (_super) {
                        __extends(SelectionToolbar, _super);
                        function SelectionToolbar() {
                            var _this = this;
                            _super.call(this, "selection-toolbar");
                            this.editClickListeners = [];
                            this.removeClickListeners = [];
                            this.editButton = new Button("Edit");
                            this.editButton.addClass("large");
                            this.editButton.onClicked(function (event) {
                                _this.notifyEditClicked();
                            });
                            this.appendChild(this.editButton);
                            this.removeButton = new Button("Remove");
                            this.removeButton.addClass("large red");
                            this.removeButton.onClicked(function (event) {
                                _this.notifyRemoveClicked();
                            });
                            this.appendChild(this.removeButton);
                        }
                        SelectionToolbar.prototype.setSelectionCount = function (count) {
                            this.selectionCount = count;
                            this.refreshUI();
                        };
                        SelectionToolbar.prototype.refreshUI = function () {
                            this.editButton.setLabel("Edit" + (this.selectionCount > 0 ? " (" + this.selectionCount + ")" : ""));
                            this.removeButton.setLabel("Remove " + (this.selectionCount > 0 ? " (" + this.selectionCount + ")" : ""));
                        };
                        SelectionToolbar.prototype.notifyEditClicked = function () {
                            this.editClickListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        SelectionToolbar.prototype.onEditClicked = function (listener) {
                            this.editClickListeners.push(listener);
                        };
                        SelectionToolbar.prototype.unEditClicked = function (listener) {
                            this.editClickListeners = this.editClickListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        SelectionToolbar.prototype.notifyRemoveClicked = function () {
                            this.removeClickListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        SelectionToolbar.prototype.onRemoveClicked = function (listener) {
                            this.removeClickListeners.push(listener);
                        };
                        SelectionToolbar.prototype.unRemoveClicked = function (listener) {
                            this.removeClickListeners = this.removeClickListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        return SelectionToolbar;
                    })(api.dom.DivEl);
                    image.SelectionToolbar = SelectionToolbar;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var FocusChangedEvent = (function () {
                        function FocusChangedEvent(focused) {
                            this.focused = focused;
                        }
                        FocusChangedEvent.prototype.isFocused = function () {
                            return this.focused;
                        };
                        return FocusChangedEvent;
                    })();
                    image.FocusChangedEvent = FocusChangedEvent;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
                    var ImageSelectorDisplayValue = (function () {
                        function ImageSelectorDisplayValue() {
                        }
                        ImageSelectorDisplayValue.fromUploadItem = function (item) {
                            return new ImageSelectorDisplayValue().setUploadItem(item);
                        };
                        ImageSelectorDisplayValue.fromContentSummary = function (content) {
                            return new ImageSelectorDisplayValue().setContentSummary(content);
                        };
                        ImageSelectorDisplayValue.prototype.setUploadItem = function (item) {
                            this.uploadItem = item;
                            return this;
                        };
                        ImageSelectorDisplayValue.prototype.setContentSummary = function (contentSummary) {
                            this.content = contentSummary;
                            return this;
                        };
                        ImageSelectorDisplayValue.prototype.getUploadItem = function () {
                            return this.uploadItem;
                        };
                        ImageSelectorDisplayValue.prototype.getContentSummary = function () {
                            return this.content;
                        };
                        ImageSelectorDisplayValue.prototype.getId = function () {
                            return this.uploadItem ? this.uploadItem.getId() : this.content.getId();
                        };
                        ImageSelectorDisplayValue.prototype.getContentId = function () {
                            return this.content ? this.content.getContentId() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getImageUrl = function () {
                            return this.content ? new ContentIconUrlResolver().setContent(this.content).resolve() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getLabel = function () {
                            return this.content ? this.content.getName().toString() : this.uploadItem.getName();
                        };
                        ImageSelectorDisplayValue.prototype.getDisplayName = function () {
                            return this.content ? this.content.getDisplayName() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getTypeLocaleName = function () {
                            return (this.content && this.content.getType()) ? this.content.getType().getLocalName() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getPath = function () {
                            return this.content ? this.content.getPath().toString() : null;
                        };
                        return ImageSelectorDisplayValue;
                    })();
                    image.ImageSelectorDisplayValue = ImageSelectorDisplayValue;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ImageSelectorViewer = (function (_super) {
                        __extends(ImageSelectorViewer, _super);
                        function ImageSelectorViewer() {
                            _super.call(this);
                        }
                        ImageSelectorViewer.prototype.resolveDisplayName = function (object) {
                            return object.getDisplayName();
                        };
                        ImageSelectorViewer.prototype.resolveUnnamedDisplayName = function (object) {
                            return object.getTypeLocaleName();
                        };
                        ImageSelectorViewer.prototype.resolveSubName = function (object, relativePath) {
                            if (relativePath === void 0) { relativePath = false; }
                            return object.getPath();
                        };
                        ImageSelectorViewer.prototype.resolveIconUrl = function (object) {
                            return object.getImageUrl() + "?crop=false";
                        };
                        return ImageSelectorViewer;
                    })(api.ui.NamesAndIconViewer);
                    image.ImageSelectorViewer = ImageSelectorViewer;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var tag;
                (function (tag) {
                    var ContentQuery = api.content.query.ContentQuery;
                    var ContentQueryRequest = api.content.ContentQueryRequest;
                    var QueryExpr = api.query.expr.QueryExpr;
                    var ContentTagSuggesterBuilder = (function () {
                        function ContentTagSuggesterBuilder() {
                        }
                        ContentTagSuggesterBuilder.prototype.setDataPath = function (value) {
                            this.dataPath = value;
                            return this;
                        };
                        ContentTagSuggesterBuilder.prototype.build = function () {
                            return new ContentTagSuggester(this);
                        };
                        return ContentTagSuggesterBuilder;
                    })();
                    tag.ContentTagSuggesterBuilder = ContentTagSuggesterBuilder;
                    var ContentTagSuggester = (function () {
                        function ContentTagSuggester(builder) {
                            this.propertyPath = builder.dataPath;
                        }
                        ContentTagSuggester.prototype.suggest = function (value) {
                            var _this = this;
                            var fieldName = "data" + this.propertyPath.getParentPath().toString() + this.propertyPath.getLastElement().getName();
                            var fulltextExpression = new api.query.FulltextSearchExpressionBuilder().setSearchString(value).addField(new api.query.QueryField(fieldName)).build();
                            var queryExpr = new QueryExpr(fulltextExpression);
                            var query = new ContentQuery();
                            query.setSize(10);
                            query.setQueryExpr(queryExpr);
                            var queryRequest = new ContentQueryRequest(query);
                            queryRequest.setExpand(2 /* FULL */);
                            return queryRequest.sendAndParse().then(function (contentQueryResult) {
                                var suggestedTags = [];
                                contentQueryResult.getContents().forEach(function (content) {
                                    var propertySet = _this.propertyPath.getParentPath().isRoot() ? content.getContentData().getRoot() : content.getContentData().getPropertySet(_this.propertyPath);
                                    propertySet.forEachProperty(_this.propertyPath.getLastElement().getName(), function (property) {
                                        if (property.hasNonNullValue()) {
                                            var suggestedTag = property.getString();
                                            if (suggestedTag.indexOf(value) == 0 && suggestedTags.indexOf(suggestedTag) < 0) {
                                                suggestedTags.push(suggestedTag);
                                            }
                                        }
                                    });
                                });
                                return suggestedTags;
                            });
                        };
                        return ContentTagSuggester;
                    })();
                    tag.ContentTagSuggester = ContentTagSuggester;
                })(tag = inputtype.tag || (inputtype.tag = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var tag;
                (function (tag) {
                    var PropertyPath = api.data.PropertyPath;
                    var PropertyPathElement = api.data.PropertyPathElement;
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var Tag = (function (_super) {
                        __extends(Tag, _super);
                        function Tag(context) {
                            _super.call(this, "tag");
                            this.input = context.input;
                            this.addClass("input-type-view");
                            var dataPath = this.resolveDataPath(context);
                            this.tagSuggester = new tag.ContentTagSuggesterBuilder().setDataPath(dataPath).build();
                            var tagsBuilder = new api.ui.tags.TagsBuilder().setTagSuggester(this.tagSuggester).setMaxTags(context.input.getOccurrences().getMaximum());
                            this.tags = tagsBuilder.build();
                            this.appendChild(this.tags);
                        }
                        Tag.prototype.resolveDataPath = function (context) {
                            if (context.parentDataPath) {
                                return PropertyPath.fromParent(context.parentDataPath, PropertyPathElement.fromString(context.input.getName()));
                            }
                            else {
                                return new PropertyPath([PropertyPathElement.fromString(context.input.getName())], false);
                            }
                        };
                        Tag.prototype.availableSizeChanged = function () {
                        };
                        Tag.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        Tag.prototype.newInitialValue = function () {
                            return null;
                        };
                        Tag.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            this.propertyArray = propertyArray;
                            this.tags.clearTags();
                            propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    _this.tags.addTag(property.getString());
                                }
                            });
                            this.tags.onTagAdded(function (event) {
                                var value = new Value(event.getValue(), ValueTypes.STRING);
                                if (_this.tags.countTags() == 1) {
                                    _this.propertyArray.set(0, value);
                                }
                                else {
                                    _this.propertyArray.add(value);
                                }
                            });
                            this.tags.onTagRemoved(function (event) {
                                _this.propertyArray.remove(event.getIndex());
                            });
                            return wemQ(null);
                        };
                        Tag.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            var recording = new api.form.inputtype.InputValidationRecording();
                            var numberOfValids = this.tags.countTags();
                            if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                                recording.setBreaksMinimumOccurrences(true);
                            }
                            if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                                recording.setBreaksMaximumOccurrences(true);
                            }
                            if (!silent) {
                                if (recording.validityChanged(this.previousValidationRecording)) {
                                    this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                                }
                            }
                            this.previousValidationRecording = recording;
                            return recording;
                        };
                        Tag.prototype.giveFocus = function () {
                            return this.tags.giveFocus();
                        };
                        Tag.prototype.onFocus = function (listener) {
                            this.tags.onFocus(listener);
                        };
                        Tag.prototype.unFocus = function (listener) {
                            this.tags.unFocus(listener);
                        };
                        Tag.prototype.onBlur = function (listener) {
                            this.tags.onBlur(listener);
                        };
                        Tag.prototype.unBlur = function (listener) {
                            this.tags.unBlur(listener);
                        };
                        return Tag;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    tag.Tag = Tag;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Tag", Tag));
                })(tag = inputtype.tag || (inputtype.tag = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var geo;
                (function (geo) {
                    var support = api.form.inputtype.support;
                    var ValueTypes = api.data.ValueTypes;
                    var GeoPoint = (function (_super) {
                        __extends(GeoPoint, _super);
                        function GeoPoint(config) {
                            _super.call(this, config);
                        }
                        GeoPoint.prototype.getValueType = function () {
                            return ValueTypes.GEO_POINT;
                        };
                        GeoPoint.prototype.newInitialValue = function () {
                            return ValueTypes.GEO_POINT.newNullValue();
                        };
                        GeoPoint.prototype.createInputOccurrenceElement = function (index, property) {
                            var geoPoint = new api.ui.geo.GeoPoint();
                            if (property.hasNonNullValue()) {
                                var geoPointValue = property.getGeoPoint();
                                if (geoPointValue) {
                                    geoPoint.setGeoPoint(geoPointValue);
                                }
                            }
                            geoPoint.onValueChanged(function (event) {
                                var geoLocation = event.getNewValue();
                                var value = ValueTypes.GEO_POINT.newValue(geoLocation);
                                property.setValue(value);
                            });
                            return geoPoint;
                        };
                        GeoPoint.prototype.availableSizeChanged = function () {
                        };
                        GeoPoint.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.GEO_POINT);
                        };
                        GeoPoint.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var geoPoint = inputElement;
                            return geoPoint.hasValidUserInput();
                        };
                        return GeoPoint;
                    })(support.BaseInputTypeNotManagingAdd);
                    geo.GeoPoint = GeoPoint;
                    api.form.inputtype.InputTypeManager.register(new api.Class("GeoPoint", GeoPoint));
                })(geo = inputtype.geo || (inputtype.geo = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var Aggregation = (function () {
            function Aggregation(name) {
                this.name = name;
            }
            Aggregation.prototype.getName = function () {
                return this.name;
            };
            Aggregation.fromJsonArray = function (aggregationWrapperJsons) {
                var aggregations = [];
                aggregationWrapperJsons.forEach(function (aggregationJson) {
                    aggregations.push(aggregation.AggregationFactory.createFromJson(aggregationJson));
                });
                return aggregations;
            };
            return Aggregation;
        })();
        aggregation.Aggregation = Aggregation;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (_aggregation) {
        var AggregationView = (function (_super) {
            __extends(AggregationView, _super);
            function AggregationView(aggregation, parentGroupView) {
                _super.call(this, 'aggregation-view');
                this.bucketSelectionChangedListeners = [];
                this.displayNameMap = [];
                this.aggregation = aggregation;
                this.parentGroupView = parentGroupView;
            }
            AggregationView.prototype.setDisplayNamesMap = function (displayNameMap) {
                this.displayNameMap = displayNameMap;
                this.setDisplayNames();
            };
            AggregationView.prototype.setDisplayNames = function () {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.getDisplayNameForName = function (name) {
                return this.displayNameMap[name];
            };
            AggregationView.prototype.getAggregation = function () {
                return this.aggregation;
            };
            AggregationView.prototype.getParentGroupView = function () {
                return this.parentGroupView;
            };
            AggregationView.prototype.getName = function () {
                return this.aggregation.getName();
            };
            AggregationView.prototype.deselectFacet = function (supressEvent) {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.hasSelectedEntry = function () {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.getSelectedValues = function () {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.update = function (aggregation) {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.onBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners.push(listener);
            };
            AggregationView.prototype.unBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners = this.bucketSelectionChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            AggregationView.prototype.notifyBucketViewSelectionChanged = function (event) {
                this.bucketSelectionChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            AggregationView.createAggregationView = function (aggregation, parentGroupView) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(aggregation, api.aggregation.BucketAggregation)) {
                    return new api.aggregation.BucketAggregationView(aggregation, parentGroupView);
                }
                else {
                    throw Error("Creating AggregationView of this type of Aggregation is not supported: " + aggregation);
                }
            };
            return AggregationView;
        })(api.dom.DivEl);
        _aggregation.AggregationView = AggregationView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketAggregation = (function (_super) {
            __extends(BucketAggregation, _super);
            function BucketAggregation(name) {
                _super.call(this, name);
                this.buckets = [];
            }
            BucketAggregation.prototype.getBucketByName = function (name) {
                for (var i = 0; i < this.buckets.length; i++) {
                    var bucket = this.buckets[i];
                    if (bucket.getKey() == name) {
                        return bucket;
                    }
                }
                return null;
            };
            BucketAggregation.prototype.getBuckets = function () {
                return this.buckets;
            };
            BucketAggregation.prototype.addBucket = function (bucket) {
                this.buckets.push(bucket);
            };
            BucketAggregation.fromJson = function (json) {
                var bucketAggregation = new BucketAggregation(json.name);
                json.buckets.forEach(function (bucketWrapper) {
                    bucketAggregation.addBucket(api.aggregation.BucketFactory.createFromJson(bucketWrapper));
                });
                return bucketAggregation;
            };
            return BucketAggregation;
        })(aggregation.Aggregation);
        aggregation.BucketAggregation = BucketAggregation;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (_aggregation) {
        var BucketAggregationView = (function (_super) {
            __extends(BucketAggregationView, _super);
            function BucketAggregationView(bucketAggregation, parentGroupView) {
                var _this = this;
                _super.call(this, bucketAggregation, parentGroupView);
                this.bucketViews = [];
                this.bucketAggregation = bucketAggregation;
                this.showBucketView = false;
                this.bucketAggregation.getBuckets().forEach(function (bucket) {
                    _this.addBucket(new api.aggregation.BucketView(bucket, _this, false, _this.getDisplayNameForName(bucket.getKey())));
                    if (bucket.getDocCount() > 0) {
                        _this.showBucketView = true;
                    }
                });
                if (!this.showBucketView) {
                    this.hide();
                }
            }
            BucketAggregationView.prototype.setDisplayNames = function () {
                var _this = this;
                this.bucketViews.forEach(function (bucketView) {
                    bucketView.setDisplayName(_this.getDisplayNameForName(bucketView.getName()));
                });
            };
            BucketAggregationView.prototype.hasSelectedEntry = function () {
                var isSelected = false;
                this.bucketViews.forEach(function (bucketView) {
                    if (bucketView.isSelected()) {
                        isSelected = true;
                    }
                });
                return isSelected;
            };
            BucketAggregationView.prototype.addBucket = function (bucketView) {
                var _this = this;
                this.appendChild(bucketView);
                bucketView.onSelectionChanged(function (event) {
                    _this.notifyBucketViewSelectionChanged(event);
                });
                this.bucketViews.push(bucketView);
            };
            BucketAggregationView.prototype.getSelectedValues = function () {
                var selectedBuckets = [];
                this.bucketViews.forEach(function (bucketView) {
                    if (bucketView.isSelected()) {
                        selectedBuckets.push(bucketView.getBucket());
                    }
                });
                return selectedBuckets;
            };
            BucketAggregationView.prototype.deselectFacet = function (supressEvent) {
                this.bucketViews.forEach(function (bucketView) {
                    bucketView.deselect(supressEvent);
                });
            };
            BucketAggregationView.prototype.update = function (aggregation) {
                var _this = this;
                var selectedBucketNames = this.getSelectedBucketNames();
                this.bucketAggregation = aggregation;
                this.bucketViews = [];
                this.removeChildren();
                var anyBucketVisible = false;
                this.bucketAggregation.getBuckets().forEach(function (bucket) {
                    var wasSelected = (wemjq.inArray(bucket.getKey(), selectedBucketNames)) > -1;
                    var bucketView = new api.aggregation.BucketView(bucket, _this, wasSelected, _this.getDisplayNameForName(bucket.getKey()));
                    _this.addBucket(bucketView);
                    if (bucket.getDocCount() > 0 || wasSelected) {
                        anyBucketVisible = true;
                    }
                });
                this.showBucketView = anyBucketVisible;
                if (!this.showBucketView) {
                    this.hide();
                }
                else if (!this.isVisible()) {
                    this.show();
                }
            };
            BucketAggregationView.prototype.getSelectedBucketNames = function () {
                var selectedBucketNames = [];
                var selectedBuckets = this.getSelectedValues();
                selectedBuckets.forEach(function (bucket) {
                    selectedBucketNames.push(bucket.getKey());
                });
                return selectedBucketNames;
            };
            return BucketAggregationView;
        })(api.aggregation.AggregationView);
        _aggregation.BucketAggregationView = BucketAggregationView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var Bucket = (function () {
            function Bucket(key, docCount) {
                this.key = key;
                this.docCount = docCount;
            }
            Bucket.prototype.getKey = function () {
                return this.key;
            };
            Bucket.prototype.getDocCount = function () {
                return this.docCount;
            };
            Bucket.prototype.setKey = function (key) {
                this.key = key;
            };
            Bucket.prototype.setDocCount = function (docCount) {
                this.docCount = docCount;
            };
            Bucket.fromJson = function (json) {
                return new Bucket(json.key, json.docCount);
            };
            Bucket.prototype.getSelectionValue = function () {
                return "test";
            };
            return Bucket;
        })();
        aggregation.Bucket = Bucket;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketFactory = (function () {
            function BucketFactory() {
            }
            BucketFactory.createFromJson = function (json) {
                if (json.DateRangeBucket) {
                    return aggregation.DateRangeBucket.fromDateRangeJson(json.DateRangeBucket);
                }
                else if (json.BucketJson) {
                    return aggregation.Bucket.fromJson(json.BucketJson);
                }
                else {
                    throw new Error("Bucket-type not recognized: " + json);
                }
            };
            return BucketFactory;
        })();
        aggregation.BucketFactory = BucketFactory;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketView = (function (_super) {
            __extends(BucketView, _super);
            function BucketView(bucket, parentAggregationView, select, displayName) {
                var _this = this;
                _super.call(this, 'aggregation-bucket-view');
                this.selectionChangedListeners = [];
                this.bucket = bucket;
                this.parentAggregationView = parentAggregationView;
                this.displayName = displayName;
                this.checkbox = new api.ui.Checkbox(this.resolveLabelValue());
                if (select) {
                    this.checkbox.setChecked(true, true);
                }
                this.checkbox.onValueChanged(function (event) {
                    _this.notifySelectionChanged(eval(event.getOldValue()), eval(event.getNewValue()));
                });
                this.appendChild(this.checkbox);
                this.updateUI();
            }
            BucketView.prototype.resolveLabelValue = function () {
                if (this.displayName != null) {
                    return this.displayName + ' (' + this.bucket.getDocCount() + ')';
                }
                return this.bucket.getKey() + ' (' + this.bucket.getDocCount() + ')';
            };
            BucketView.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                this.updateLabel();
            };
            BucketView.prototype.updateLabel = function () {
                this.checkbox.setLabel(this.resolveLabelValue());
            };
            BucketView.prototype.getBucket = function () {
                return this.bucket;
            };
            BucketView.prototype.getName = function () {
                return this.bucket.getKey();
            };
            BucketView.prototype.update = function (bucket) {
                this.bucket = bucket;
                this.updateUI();
            };
            BucketView.prototype.isSelected = function () {
                return this.checkbox.isChecked();
            };
            BucketView.prototype.deselect = function (supressEvent) {
                this.checkbox.setChecked(false, supressEvent);
            };
            BucketView.prototype.updateUI = function () {
                this.updateLabel();
                if (this.bucket.getDocCount() > 0 || this.isSelected()) {
                    this.show();
                }
                else {
                    this.hide();
                }
            };
            BucketView.prototype.getParentAggregationView = function () {
                return this.parentAggregationView;
            };
            BucketView.prototype.notifySelectionChanged = function (oldValue, newValue) {
                var _this = this;
                this.selectionChangedListeners.forEach(function (listener) {
                    listener(new api.aggregation.BucketViewSelectionChangedEvent(oldValue, newValue, _this));
                });
            };
            BucketView.prototype.unSelectionChanged = function (listener) {
                this.selectionChangedListeners = this.selectionChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            BucketView.prototype.onSelectionChanged = function (listener) {
                this.selectionChangedListeners.push(listener);
            };
            return BucketView;
        })(api.dom.DivEl);
        aggregation.BucketView = BucketView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var DateRangeBucket = (function (_super) {
            __extends(DateRangeBucket, _super);
            function DateRangeBucket(key, docCount) {
                _super.call(this, key, docCount);
            }
            DateRangeBucket.prototype.getFrom = function () {
                return this.from;
            };
            DateRangeBucket.prototype.getTo = function () {
                return this.to;
            };
            DateRangeBucket.fromDateRangeJson = function (json) {
                var dateRangeBucket = new api.aggregation.DateRangeBucket(json.key, json.docCount);
                dateRangeBucket.from = json.from;
                dateRangeBucket.to = json.to;
                return dateRangeBucket;
            };
            return DateRangeBucket;
        })(aggregation.Bucket);
        aggregation.DateRangeBucket = DateRangeBucket;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationContainer = (function (_super) {
            __extends(AggregationContainer, _super);
            function AggregationContainer() {
                _super.call(this);
                this.aggregationGroupViews = [];
            }
            AggregationContainer.prototype.addAggregationGroupView = function (aggregationGroupView) {
                var _this = this;
                this.appendChild(aggregationGroupView);
                aggregationGroupView.onBucketViewSelectionChanged(function (event) {
                    if (event.getNewValue()) {
                        _this.lastSelectedGroupView = event.getBucketView().getParentAggregationView().getParentGroupView();
                    }
                });
                this.aggregationGroupViews.push(aggregationGroupView);
            };
            AggregationContainer.prototype.deselectAll = function (supressEvent) {
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    aggregationGroupView.deselectGroup(supressEvent);
                });
                this.lastSelectedGroupView = null;
            };
            AggregationContainer.prototype.hasSelectedBuckets = function () {
                var hasSelected = false;
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    if (aggregationGroupView.hasSelections()) {
                        hasSelected = true;
                    }
                });
                return hasSelected;
            };
            AggregationContainer.prototype.updateAggregations = function (aggregations, doUpdateAll) {
                var _this = this;
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    var matchingAggregations = aggregations.filter(function (current) {
                        return aggregationGroupView.handlesAggregation(current);
                    });
                    if (doUpdateAll || _this.isGroupUpdatable(aggregationGroupView)) {
                        aggregationGroupView.update(matchingAggregations);
                    }
                });
            };
            AggregationContainer.prototype.isGroupUpdatable = function (aggregationGroupView) {
                return aggregationGroupView != this.lastSelectedGroupView;
            };
            AggregationContainer.prototype.getSelectedValuesByAggregationName = function () {
                var aggregationSelections = [];
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    var selectedValuesByAggregationName = aggregationGroupView.getSelectedValuesByAggregationName();
                    aggregationSelections = aggregationSelections.concat(selectedValuesByAggregationName);
                });
                return aggregationSelections;
            };
            return AggregationContainer;
        })(api.dom.DivEl);
        aggregation.AggregationContainer = AggregationContainer;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationFactory = (function () {
            function AggregationFactory() {
            }
            AggregationFactory.createFromJson = function (json) {
                if (json.BucketAggregation) {
                    return api.aggregation.BucketAggregation.fromJson(json.BucketAggregation);
                }
                else {
                    throw new Error("Aggregation type not recognized: " + json);
                }
            };
            return AggregationFactory;
        })();
        aggregation.AggregationFactory = AggregationFactory;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (_aggregation) {
        var AggregationGroupView = (function (_super) {
            __extends(AggregationGroupView, _super);
            function AggregationGroupView(name, displayName) {
                _super.call(this, "aggregation-group-view");
                this.aggregationViews = [];
                this.titleEl = new api.dom.H2El();
                this.bucketSelectionChangedListeners = [];
                this.name = name;
                this.displayName = displayName;
                this.titleEl.getEl().setInnerHtml(this.displayName);
                this.appendChild(this.titleEl);
            }
            AggregationGroupView.prototype.addAggregationView = function (aggregationView) {
                var _this = this;
                this.appendChild(aggregationView);
                aggregationView.onBucketViewSelectionChanged(function (event) {
                    _this.notifyBucketViewSelectionChanged(event);
                });
                this.aggregationViews.push(aggregationView);
            };
            AggregationGroupView.prototype.initialize = function () {
            };
            AggregationGroupView.prototype.getAggregationViews = function () {
                return this.aggregationViews;
            };
            AggregationGroupView.prototype.getName = function () {
                return this.name;
            };
            AggregationGroupView.prototype.handlesAggregation = function (aggregation) {
                return aggregation.getName() == this.name;
            };
            AggregationGroupView.prototype.getSelectedValuesByAggregationName = function () {
                var aggregationSelections = [];
                this.aggregationViews.forEach(function (bucketAggregationView) {
                    var selectedBuckets = bucketAggregationView.getSelectedValues();
                    if (selectedBuckets != null) {
                        var aggregationSelection = new api.aggregation.AggregationSelection(bucketAggregationView.getName());
                        aggregationSelection.setValues(selectedBuckets);
                        aggregationSelections.push(aggregationSelection);
                    }
                });
                return aggregationSelections;
            };
            AggregationGroupView.prototype.hasSelections = function () {
                var hasSelections = false;
                for (var i = 0; i < this.aggregationViews.length; i++) {
                    if (this.aggregationViews[i].hasSelectedEntry()) {
                        hasSelections = true;
                        break;
                    }
                }
                return hasSelections;
            };
            AggregationGroupView.prototype.deselectGroup = function (supressEvent) {
                this.aggregationViews.forEach(function (aggregationView) {
                    aggregationView.deselectFacet(supressEvent);
                });
            };
            AggregationGroupView.prototype.onBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners.push(listener);
            };
            AggregationGroupView.prototype.unBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners = this.bucketSelectionChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            AggregationGroupView.prototype.notifyBucketViewSelectionChanged = function (event) {
                this.bucketSelectionChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            AggregationGroupView.prototype.update = function (aggregations) {
                var _this = this;
                aggregations.forEach(function (aggregation) {
                    var existingAggregationView = _this.getAggregationView(aggregation.getName());
                    if (existingAggregationView == null) {
                        _this.addAggregationView(api.aggregation.AggregationView.createAggregationView(aggregation, _this));
                    }
                    else {
                        if (api.ObjectHelper.iFrameSafeInstanceOf(existingAggregationView, api.aggregation.BucketAggregationView)) {
                            var bucketAggregationView = existingAggregationView;
                            bucketAggregationView.update(aggregation);
                        }
                    }
                });
            };
            AggregationGroupView.prototype.getAggregationView = function (name) {
                for (var i = 0; i < this.aggregationViews.length; i++) {
                    var aggregationView = this.aggregationViews[i];
                    if (aggregationView.getName() == name) {
                        return aggregationView;
                    }
                }
                return null;
            };
            return AggregationGroupView;
        })(api.dom.DivEl);
        _aggregation.AggregationGroupView = AggregationGroupView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var ContentTypeAggregationGroupView = (function (_super) {
            __extends(ContentTypeAggregationGroupView, _super);
            function ContentTypeAggregationGroupView() {
                _super.apply(this, arguments);
            }
            ContentTypeAggregationGroupView.prototype.initialize = function () {
                var _this = this;
                var displayNameMap = [];
                var mask = new api.ui.mask.LoadMask(this);
                this.appendChild(mask);
                this.onRendered(function (event) {
                    mask.show();
                });
                var request = new api.schema.content.GetAllContentTypesRequest();
                request.sendAndParse().done(function (contentTypes) {
                    contentTypes.forEach(function (contentType) {
                        displayNameMap[contentType.getName()] = contentType.getDisplayName();
                    });
                    _this.getAggregationViews().forEach(function (aggregationView) {
                        aggregationView.setDisplayNamesMap(displayNameMap);
                    });
                    mask.remove();
                });
            };
            return ContentTypeAggregationGroupView;
        })(aggregation.AggregationGroupView);
        aggregation.ContentTypeAggregationGroupView = ContentTypeAggregationGroupView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationEntryViewSelectionChangedEvent = (function () {
            function AggregationEntryViewSelectionChangedEvent(oldValue, newValue, bucketView) {
                this.oldValue = oldValue;
                this.newValue = newValue;
                this.bucketView = bucketView;
            }
            AggregationEntryViewSelectionChangedEvent.prototype.getOldValue = function () {
                return this.oldValue;
            };
            AggregationEntryViewSelectionChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            AggregationEntryViewSelectionChangedEvent.prototype.getBucketView = function () {
                return this.bucketView;
            };
            return AggregationEntryViewSelectionChangedEvent;
        })();
        aggregation.AggregationEntryViewSelectionChangedEvent = AggregationEntryViewSelectionChangedEvent;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketViewSelectionChangedEvent = (function () {
            function BucketViewSelectionChangedEvent(oldValue, newValue, bucketView) {
                this.oldValue = oldValue;
                this.newValue = newValue;
                this.bucketView = bucketView;
            }
            BucketViewSelectionChangedEvent.prototype.getOldValue = function () {
                return this.oldValue;
            };
            BucketViewSelectionChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            BucketViewSelectionChangedEvent.prototype.getBucketView = function () {
                return this.bucketView;
            };
            return BucketViewSelectionChangedEvent;
        })();
        aggregation.BucketViewSelectionChangedEvent = BucketViewSelectionChangedEvent;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationSelection = (function () {
            function AggregationSelection(name) {
                this.name = name;
            }
            AggregationSelection.prototype.setValues = function (selectedBuckets) {
                this.selectedBuckets = selectedBuckets;
            };
            AggregationSelection.prototype.getName = function () {
                return this.name;
            };
            AggregationSelection.prototype.getSelectedBuckets = function () {
                return this.selectedBuckets;
            };
            return AggregationSelection;
        })();
        aggregation.AggregationSelection = AggregationSelection;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var PrincipalAggregationGroupView = (function (_super) {
            __extends(PrincipalAggregationGroupView, _super);
            function PrincipalAggregationGroupView() {
                _super.apply(this, arguments);
            }
            PrincipalAggregationGroupView.prototype.initialize = function () {
                var displayNameMap = [];
                var mask = new api.ui.mask.LoadMask(this);
                this.appendChild(mask);
                this.onRendered(function (event) {
                    mask.show();
                });
                mask.remove();
            };
            return PrincipalAggregationGroupView;
        })(aggregation.AggregationGroupView);
        aggregation.PrincipalAggregationGroupView = PrincipalAggregationGroupView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rendering;
    (function (rendering) {
        (function (RenderingMode) {
            RenderingMode[RenderingMode["EDIT"] = 0] = "EDIT";
            RenderingMode[RenderingMode["PREVIEW"] = 1] = "PREVIEW";
            RenderingMode[RenderingMode["LIVE"] = 2] = "LIVE";
        })(rendering.RenderingMode || (rendering.RenderingMode = {}));
        var RenderingMode = rendering.RenderingMode;
    })(rendering = api.rendering || (api.rendering = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rendering;
    (function (rendering) {
        var UriHelper = (function () {
            function UriHelper() {
            }
            UriHelper.getPortalUri = function (path, renderingMode, workspace) {
                path = api.util.UriHelper.relativePath(path);
                var workspaceName;
                switch (workspace) {
                    case 0 /* DRAFT */:
                        workspaceName = "draft";
                        break;
                    case 1 /* MASTER */:
                        workspaceName = "master";
                        break;
                    default:
                        workspaceName = "draft";
                }
                switch (renderingMode) {
                    case 0 /* EDIT */:
                        return api.util.UriHelper.getPortalUri('edit/' + workspaceName + '/' + path);
                    case 2 /* LIVE */:
                        return api.util.UriHelper.getPortalUri('live/' + workspaceName + '/' + path);
                    case 1 /* PREVIEW */:
                        return api.util.UriHelper.getPortalUri('preview/' + workspaceName + '/' + path);
                }
            };
            UriHelper.getComponentUri = function (contentId, componentPath, renderingMode, workspace) {
                return UriHelper.getPortalUri(contentId + "/_/component/" + componentPath, renderingMode, workspace);
            };
            return UriHelper;
        })();
        rendering.UriHelper = UriHelper;
    })(rendering = api.rendering || (api.rendering = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ShowAppLauncherEvent = (function (_super) {
            __extends(ShowAppLauncherEvent, _super);
            function ShowAppLauncherEvent(application, sessionExpired) {
                _super.call(this);
                this.application = application;
                this.sessionExpired = !!sessionExpired;
            }
            ShowAppLauncherEvent.prototype.getApplication = function () {
                return this.application;
            };
            ShowAppLauncherEvent.prototype.isSessionExpired = function () {
                return this.sessionExpired;
            };
            ShowAppLauncherEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowAppLauncherEvent;
        })(api.event.Event);
        app.ShowAppLauncherEvent = ShowAppLauncherEvent;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ShowBrowsePanelEvent = (function (_super) {
            __extends(ShowBrowsePanelEvent, _super);
            function ShowBrowsePanelEvent() {
                _super.apply(this, arguments);
            }
            ShowBrowsePanelEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowBrowsePanelEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowBrowsePanelEvent;
        })(api.event.Event);
        app.ShowBrowsePanelEvent = ShowBrowsePanelEvent;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var NamesView = (function (_super) {
            __extends(NamesView, _super);
            function NamesView(addTitleAttribute) {
                if (addTitleAttribute === void 0) { addTitleAttribute = true; }
                _super.call(this, "names-view");
                this.addTitleAttribute = addTitleAttribute;
                this.mainNameEl = new api.dom.H6El("main-name");
                this.appendChild(this.mainNameEl);
                this.subNameEl = new api.dom.PEl("sub-name");
                this.appendChild(this.subNameEl);
            }
            NamesView.prototype.setMainName = function (value) {
                this.mainNameEl.getEl().setText(value);
                if (this.addTitleAttribute) {
                    this.mainNameEl.getEl().setAttribute("title", value);
                }
                return this;
            };
            NamesView.prototype.setSubName = function (value, title) {
                this.subNameEl.getEl().setText(value);
                if (this.addTitleAttribute) {
                    this.subNameEl.getEl().setAttribute("title", title || value);
                }
                return this;
            };
            return NamesView;
        })(api.dom.DivEl);
        app.NamesView = NamesView;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        (function (NamesAndIconViewSize) {
            NamesAndIconViewSize[NamesAndIconViewSize["large"] = 0] = "large";
            NamesAndIconViewSize[NamesAndIconViewSize["medium"] = 1] = "medium";
            NamesAndIconViewSize[NamesAndIconViewSize["small"] = 2] = "small";
        })(app.NamesAndIconViewSize || (app.NamesAndIconViewSize = {}));
        var NamesAndIconViewSize = app.NamesAndIconViewSize;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var NamesAndIconViewBuilder = (function () {
            function NamesAndIconViewBuilder() {
                this.addTitleAttribute = true;
                this.appendIcon = true;
            }
            NamesAndIconViewBuilder.prototype.setSize = function (size) {
                this.size = size;
                return this;
            };
            NamesAndIconViewBuilder.prototype.setAddTitleAttribute = function (addTitleAttribute) {
                this.addTitleAttribute = addTitleAttribute;
                return this;
            };
            NamesAndIconViewBuilder.prototype.setAppendIcon = function (appendIcon) {
                this.appendIcon = appendIcon;
                return this;
            };
            NamesAndIconViewBuilder.prototype.build = function () {
                return new NamesAndIconView(this);
            };
            return NamesAndIconViewBuilder;
        })();
        app.NamesAndIconViewBuilder = NamesAndIconViewBuilder;
        var NamesAndIconView = (function (_super) {
            __extends(NamesAndIconView, _super);
            function NamesAndIconView(builder) {
                _super.call(this, "names-and-icon-view");
                var sizeClassName = app.NamesAndIconViewSize[builder.size];
                if (builder.size) {
                    this.addClass(sizeClassName);
                }
                if (builder.appendIcon) {
                    this.wrapperDivEl = new api.dom.DivEl("wrapper");
                    this.appendChild(this.wrapperDivEl);
                    this.iconImageEl = new api.dom.ImgEl(null, "icon");
                    this.wrapperDivEl.appendChild(this.iconImageEl);
                    this.iconDivEl = new api.dom.DivEl("icon");
                    this.wrapperDivEl.appendChild(this.iconDivEl);
                    this.iconDivEl.hide();
                }
                this.namesView = new api.app.NamesView(builder.addTitleAttribute);
                this.appendChild(this.namesView);
                this.iconLabelEl = new api.dom.SpanEl("icon-label");
                this.iconLabelEl.hide();
                this.appendChild(this.iconLabelEl);
            }
            NamesAndIconView.prototype.setMainName = function (value) {
                this.namesView.setMainName(value);
                return this;
            };
            NamesAndIconView.prototype.setSubName = function (value, title) {
                this.namesView.setSubName(value, title);
                return this;
            };
            NamesAndIconView.prototype.setIconClass = function (value) {
                this.iconDivEl.setClass("icon " + value);
                this.iconDivEl.getEl().setDisplay('inline-block');
                this.iconImageEl.hide();
                return this;
            };
            NamesAndIconView.prototype.setIconUrl = function (value) {
                this.iconImageEl.setSrc(value);
                this.iconDivEl.hide();
                this.iconImageEl.show();
                return this;
            };
            NamesAndIconView.prototype.setDisplayIconLabel = function (display) {
                if (display) {
                    this.iconLabelEl.show();
                }
                else {
                    this.iconLabelEl.hide();
                }
                return this;
            };
            NamesAndIconView.prototype.getIconImageEl = function () {
                return this.iconImageEl;
            };
            NamesAndIconView.prototype.getWrapperDivEl = function () {
                return this.wrapperDivEl;
            };
            return NamesAndIconView;
        })(api.dom.DivEl);
        app.NamesAndIconView = NamesAndIconView;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var Application = (function () {
            function Application(id, name, shortName, icon, appFrame, iconImage) {
                if (appFrame === void 0) { appFrame = null; }
                if (iconImage === void 0) { iconImage = false; }
                this.loadedListeners = [];
                this.id = id;
                this.name = name;
                this.shortName = shortName;
                this.iconUrl = icon;
                this.fullSizeIcon = iconImage;
                this.openTabs = 0;
            }
            Application.getApplication = function () {
                return window.parent['getApplication'] ? window.parent['getApplication'](Application.getAppId()) : null;
            };
            Application.getAppId = function () {
                return window.frameElement ? new api.dom.ElementHelper(window.frameElement).getAttribute("data-wem-app-id") : null;
            };
            Application.prototype.isLoaded = function () {
                return this.loaded && !!this.appFrame && !!this.appFrame.getParentElement();
            };
            Application.prototype.getId = function () {
                return this.id;
            };
            Application.prototype.getName = function () {
                return this.name;
            };
            Application.prototype.getShortName = function () {
                return this.shortName;
            };
            Application.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            Application.prototype.getAppUrl = function () {
                return api.util.UriHelper.getUri('admin?app=' + this.id);
            };
            Application.prototype.getOpenTabs = function () {
                return this.openTabs;
            };
            Application.prototype.getAppFrame = function () {
                if (!this.appFrame) {
                    this.appFrame = new api.dom.IFrameEl();
                    this.appFrame.getEl().setHeight('100%').setWidth('100%').getHTMLElement().style.border = '0';
                    this.appFrame.setSrc(this.getAppUrl());
                    this.appFrame.getEl().setAttribute('data-wem-app-id', this.id);
                }
                return this.appFrame;
            };
            Application.prototype.getWindow = function () {
                return this.getAppFrame().getHTMLElement()["contentWindow"];
            };
            Application.prototype.isFullSizeIcon = function () {
                return this.fullSizeIcon;
            };
            Application.prototype.hide = function () {
                if (this.appFrame) {
                    this.appFrame.hide();
                }
            };
            Application.prototype.setOpenTabs = function (value) {
                this.openTabs = value;
                return this;
            };
            Application.prototype.setLoaded = function (value) {
                this.loaded = value;
                this.notifyLoaded();
                return this;
            };
            Application.prototype.setFullSizeIcon = function (value) {
                this.fullSizeIcon = value;
                return this;
            };
            Application.prototype.setPath = function (path) {
                this.path = path;
                return this;
            };
            Application.prototype.getPath = function () {
                return this.path;
            };
            Application.prototype.onLoaded = function (listener) {
                this.loadedListeners.push(listener);
            };
            Application.prototype.unLoaded = function (listener) {
                this.loadedListeners = this.loadedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Application.prototype.notifyLoaded = function () {
                this.loadedListeners.forEach(function (listener) {
                    listener();
                });
            };
            return Application;
        })();
        app.Application = Application;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        (function (AppLauncherEventType) {
            AppLauncherEventType[AppLauncherEventType["Show"] = 0] = "Show";
        })(app.AppLauncherEventType || (app.AppLauncherEventType = {}));
        var AppLauncherEventType = app.AppLauncherEventType;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var AppPanel = (function (_super) {
            __extends(AppPanel, _super);
            function AppPanel(tabNavigator) {
                _super.call(this, tabNavigator);
            }
            AppPanel.prototype.addBrowsePanel = function (browsePanel) {
                if (!this.browsePanel) {
                    var browseMenuItem = new api.app.bar.AppBarTabMenuItemBuilder().setLabel("<Select>").setTabId(new api.app.bar.AppBarTabId("hidden", "____home")).build();
                    browseMenuItem.setVisibleInMenu(false);
                    this.addNavigablePanel(browseMenuItem, browsePanel, true);
                    this.browsePanel = browsePanel;
                }
            };
            AppPanel.prototype.getBrowsePanel = function () {
                return this.browsePanel;
            };
            AppPanel.prototype.removeNavigablePanel = function (panel, checkCanRemovePanel) {
                if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                var index = _super.prototype.removeNavigablePanel.call(this, panel, checkCanRemovePanel);
                this.checkBrowsePanelNeedsToBeShown(index, panel);
                return index;
            };
            AppPanel.prototype.checkBrowsePanelNeedsToBeShown = function (index, panel) {
                if (panel == this.browsePanel && index > -1) {
                    this.browsePanel = undefined;
                }
                else if (this.getSize() == 0) {
                    new app.ShowBrowsePanelEvent().fire();
                }
            };
            AppPanel.prototype.getNavigator = function () {
                return _super.prototype.getNavigator.call(this);
            };
            return AppPanel;
        })(api.ui.panel.NavigatedDeckPanel);
        app.AppPanel = AppPanel;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var BrowseAndWizardBasedAppPanel = (function (_super) {
            __extends(BrowseAndWizardBasedAppPanel, _super);
            function BrowseAndWizardBasedAppPanel(config) {
                var _this = this;
                _super.call(this, config.appBar.getTabMenu());
                this.appBar = config.appBar;
                this.appBarTabMenu = config.appBar.getTabMenu();
                this.onPanelShown(function (event) {
                    if (event.getPanel() === _this.getBrowsePanel()) {
                        _this.getBrowsePanel().refreshFilter();
                    }
                    var previousActions = _this.resolveActions(event.getPreviousPanel());
                    api.ui.KeyBindings.get().unbindKeys(api.ui.Action.getKeyBindings(previousActions));
                    var nextActions = _this.resolveActions(event.getPanel());
                    _this.currentKeyBindings = api.ui.Action.getKeyBindings(nextActions);
                    api.ui.KeyBindings.get().bindKeys(_this.currentKeyBindings);
                });
            }
            BrowseAndWizardBasedAppPanel.prototype.addBrowsePanel = function (browsePanel) {
                _super.prototype.addBrowsePanel.call(this, browsePanel);
                this.currentKeyBindings = api.ui.Action.getKeyBindings(this.resolveActions(browsePanel));
                this.activateCurrentKeyBindings();
            };
            BrowseAndWizardBasedAppPanel.prototype.activateCurrentKeyBindings = function () {
                if (this.currentKeyBindings) {
                    api.ui.KeyBindings.get().bindKeys(this.currentKeyBindings);
                }
            };
            BrowseAndWizardBasedAppPanel.prototype.getAppBarTabMenu = function () {
                return this.appBarTabMenu;
            };
            BrowseAndWizardBasedAppPanel.prototype.addViewPanel = function (tabMenuItem, viewPanel) {
                var _this = this;
                _super.prototype.addNavigablePanel.call(this, tabMenuItem, viewPanel, true);
                viewPanel.onClosed(function (event) {
                    _this.removeNavigablePanel(event.getView(), false);
                });
            };
            BrowseAndWizardBasedAppPanel.prototype.addWizardPanel = function (tabMenuItem, wizardPanel) {
                var _this = this;
                _super.prototype.addNavigablePanel.call(this, tabMenuItem, wizardPanel, true);
                wizardPanel.onClosed(function (event) {
                    _this.removeNavigablePanel(event.getWizard(), false);
                });
            };
            BrowseAndWizardBasedAppPanel.prototype.canRemovePanel = function (panel) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.wizard.WizardPanel)) {
                    var wizardPanel = panel;
                    return wizardPanel.canClose();
                }
                return true;
            };
            BrowseAndWizardBasedAppPanel.prototype.resolveActions = function (panel) {
                var actions = [];
                actions = actions.concat(this.appBar.getActions());
                if (api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.wizard.WizardPanel) || api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.browse.BrowsePanel) || api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.view.ItemViewPanel)) {
                    var actionContainer = panel;
                    actions = actions.concat(actionContainer.getActions());
                }
                return actions;
            };
            return BrowseAndWizardBasedAppPanel;
        })(app.AppPanel);
        app.BrowseAndWizardBasedAppPanel = BrowseAndWizardBasedAppPanel;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var AppManager = (function () {
            function AppManager() {
                this.connectionLostListeners = [];
                this.connectionRestoredListeners = [];
                this.showLauncherListeners = [];
                api.app.AppManager._instance = this;
            }
            AppManager.prototype.onConnectionLost = function (listener) {
                this.connectionLostListeners.push(listener);
            };
            AppManager.prototype.onConnectionRestored = function (listener) {
                this.connectionRestoredListeners.push(listener);
            };
            AppManager.prototype.unConnectionLost = function (listener) {
                this.connectionLostListeners = this.connectionLostListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            AppManager.prototype.unConnectionRestored = function (listener) {
                this.connectionRestoredListeners = this.connectionRestoredListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            AppManager.prototype.notifyConnectionLost = function () {
                var _this = this;
                this.connectionLostListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            AppManager.prototype.notifyConnectionRestored = function () {
                var _this = this;
                this.connectionRestoredListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            AppManager.instance = function () {
                if (api.app.AppManager._instance) {
                    return api.app.AppManager._instance;
                }
                else if (window !== window.parent) {
                    var apiAppModule = window.parent.api.app;
                    if (apiAppModule && apiAppModule.AppManager) {
                        var parentAppManager = apiAppModule.AppManager._instance;
                        if (parentAppManager) {
                            api.app.AppManager._instance = parentAppManager;
                        }
                    }
                }
                return api.app.AppManager._instance;
            };
            AppManager._instance = null;
            return AppManager;
        })();
        app.AppManager = AppManager;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ServerEventsConnection = (function () {
            function ServerEventsConnection(reconnectIntervalSeconds) {
                if (reconnectIntervalSeconds === void 0) { reconnectIntervalSeconds = 5; }
                this.serverEventReceivedListeners = [];
                this.connectionLostListeners = [];
                this.connectionRestoredListeners = [];
                this.connected = false;
                this.keepConnected = false;
                this.ws = null;
                this.reconnectInterval = reconnectIntervalSeconds * 1000;
            }
            ServerEventsConnection.prototype.connect = function () {
                if (!WebSocket) {
                    console.warn('WebSockets not supported. Server events disabled.');
                    return;
                }
                var wsUrl = api.util.UriHelper.joinPath(this.getWebSocketUriPrefix(), 'admin', 'event');
                this.keepConnected = true;
                this.doConnect(wsUrl);
            };
            ServerEventsConnection.prototype.doConnect = function (wsUrl) {
                var _this = this;
                this.ws = new WebSocket(wsUrl, 'text');
                this.ws.addEventListener('close', function (ev) {
                    _this.disconnectTimeoutHandle = setTimeout(function () {
                        if (_this.connected) {
                            if (_this.keepConnected) {
                                _this.notifyConnectionLost();
                            }
                            _this.connected = !_this.connected;
                        }
                    }, _this.reconnectInterval + 1000);
                    if (_this.keepConnected) {
                        setTimeout(function () {
                            if (_this.keepConnected) {
                                _this.doConnect(wsUrl);
                            }
                        }, _this.reconnectInterval);
                    }
                });
                this.ws.addEventListener('error', function (ev) {
                });
                this.ws.addEventListener('message', function (remoteEvent) {
                    var jsonEvent = JSON.parse(remoteEvent.data);
                    _this.handleServerEvent(jsonEvent);
                });
                this.ws.addEventListener('open', function (event) {
                    clearInterval(_this.disconnectTimeoutHandle);
                    if (!_this.connected) {
                        _this.notifyConnectionRestored();
                        _this.connected = !_this.connected;
                    }
                });
            };
            ServerEventsConnection.prototype.disconnect = function () {
                this.keepConnected = false;
                if (this.ws) {
                    this.ws.close();
                }
            };
            ServerEventsConnection.prototype.isConnected = function () {
                return this.ws.readyState === WebSocket.OPEN;
            };
            ServerEventsConnection.prototype.handleServerEvent = function (serverEventJson) {
                var clientEvent = this.translateServerEvent(serverEventJson);
                if (clientEvent) {
                    this.notifyServerEvent(clientEvent);
                }
            };
            ServerEventsConnection.prototype.translateServerEvent = function (serverEventJson) {
                var eventType = serverEventJson.type;
                if (eventType === 'ContentChangeEvent') {
                    var contentChange = api.content.ContentServerEvent.fromJson(serverEventJson.event);
                    console.log('Server changes: ' + contentChange.toString());
                    return contentChange;
                }
                if (eventType === 'ModuleUpdatedEvent') {
                    return api.module.ModuleUpdatedEvent.fromJson(serverEventJson.event);
                }
                if (eventType === 'ContentTypeUpdatedEvent') {
                    return api.schema.content.ContentTypeUpdatedEvent.fromJson(serverEventJson.event);
                }
                else if (eventType === 'ContentTypeDeletedEvent') {
                    return api.schema.content.ContentTypeDeletedEvent.fromJson(serverEventJson.event);
                }
                return null;
            };
            ServerEventsConnection.prototype.getWebSocketUriPrefix = function () {
                var loc = window.location, newUri;
                if (loc.protocol === "https:") {
                    newUri = "wss:";
                }
                else {
                    newUri = "ws:";
                }
                newUri += "//" + loc.host;
                return newUri;
            };
            ServerEventsConnection.prototype.notifyServerEvent = function (serverEvent) {
                var _this = this;
                this.serverEventReceivedListeners.forEach(function (listener) {
                    listener.call(_this, serverEvent);
                });
            };
            ServerEventsConnection.prototype.onServerEvent = function (listener) {
                this.serverEventReceivedListeners.push(listener);
            };
            ServerEventsConnection.prototype.unServerEvent = function (listener) {
                this.serverEventReceivedListeners = this.serverEventReceivedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            ServerEventsConnection.prototype.notifyConnectionLost = function () {
                var _this = this;
                this.connectionLostListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            ServerEventsConnection.prototype.onConnectionLost = function (listener) {
                this.connectionLostListeners.push(listener);
            };
            ServerEventsConnection.prototype.unConnectionLost = function (listener) {
                this.connectionLostListeners = this.connectionLostListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            ServerEventsConnection.prototype.notifyConnectionRestored = function () {
                var _this = this;
                this.connectionRestoredListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            ServerEventsConnection.prototype.onConnectionRestored = function (listener) {
                this.connectionRestoredListeners.push(listener);
            };
            ServerEventsConnection.prototype.unConnectionRestored = function (listener) {
                this.connectionRestoredListeners = this.connectionRestoredListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            return ServerEventsConnection;
        })();
        app.ServerEventsConnection = ServerEventsConnection;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ServerEventsListener = (function () {
            function ServerEventsListener(applications) {
                var _this = this;
                this.applications = applications;
                this.serverEventsConnection = new api.app.ServerEventsConnection();
                this.serverEventsConnection.onServerEvent(function (event) { return _this.onServerEvent(event); });
            }
            ServerEventsListener.prototype.start = function () {
                this.serverEventsConnection.connect();
            };
            ServerEventsListener.prototype.stop = function () {
                this.serverEventsConnection.disconnect();
            };
            ServerEventsListener.prototype.onConnectionLost = function (listener) {
                this.serverEventsConnection.onConnectionLost(listener);
            };
            ServerEventsListener.prototype.unConnectionLost = function (listener) {
                this.serverEventsConnection.unConnectionLost(listener);
            };
            ServerEventsListener.prototype.onConnectionRestored = function (listener) {
                this.serverEventsConnection.onConnectionRestored(listener);
            };
            ServerEventsListener.prototype.unConnectionRestored = function (listener) {
                this.serverEventsConnection.unConnectionRestored(listener);
            };
            ServerEventsListener.prototype.onServerEvent = function (event) {
                this.applications.forEach(function (application) {
                    var appWindow = application.getWindow();
                    if (appWindow) {
                        event.fire(appWindow);
                    }
                });
            };
            return ServerEventsListener;
        })();
        app.ServerEventsListener = ServerEventsListener;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var ShowAppLauncherAction = (function (_super) {
                __extends(ShowAppLauncherAction, _super);
                function ShowAppLauncherAction(application) {
                    _super.call(this, 'Start', 'mod+esc', true);
                    this.onExecuted(function () {
                        new app.ShowAppLauncherEvent(application).fire(window.parent);
                        new app.ShowAppLauncherEvent(application).fire();
                    });
                }
                return ShowAppLauncherAction;
            })(api.ui.Action);
            bar.ShowAppLauncherAction = ShowAppLauncherAction;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var ShowBrowsePanelAction = (function (_super) {
                __extends(ShowBrowsePanelAction, _super);
                function ShowBrowsePanelAction() {
                    _super.call(this, 'Browse');
                    this.onExecuted(function () {
                        new api.app.ShowBrowsePanelEvent().fire();
                    });
                }
                return ShowBrowsePanelAction;
            })(api.ui.Action);
            bar.ShowBrowsePanelAction = ShowBrowsePanelAction;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarActions = (function () {
                function AppBarActions() {
                }
                AppBarActions.SHOW_BROWSE_PANEL = new bar.ShowBrowsePanelAction();
                return AppBarActions;
            })();
            bar.AppBarActions = AppBarActions;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (_app) {
        var bar;
        (function (bar) {
            var AppBar = (function (_super) {
                __extends(AppBar, _super);
                function AppBar(application) {
                    var _this = this;
                    _super.call(this, "appbar");
                    this.application = application;
                    this.tabMenu = new bar.AppBarTabMenu();
                    this.showAppLauncherAction = new bar.ShowAppLauncherAction(this.application);
                    this.launcherButton = new LauncherButton(this.showAppLauncherAction);
                    this.appendChild(this.launcherButton);
                    this.homeButton = new HomeButton(this.application, bar.AppBarActions.SHOW_BROWSE_PANEL);
                    this.appendChild(this.homeButton);
                    this.appendChild(this.tabMenu);
                    this.tabMenu.onNavigationItemAdded(function (event) {
                        _this.updateAppOpenTabs();
                    });
                    this.tabMenu.onNavigationItemRemoved(function (event) {
                        _this.updateAppOpenTabs();
                    });
                    api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        if (_this.tabMenu.countVisible() > 0) {
                            _this.addClass("tabs-present");
                        }
                        else {
                            _this.removeClass("tabs-present");
                        }
                    });
                    this.onRendered(function () {
                        api.ui.responsive.ResponsiveManager.fireResizeEvent();
                    });
                }
                AppBar.prototype.getActions = function () {
                    return [this.showAppLauncherAction];
                };
                AppBar.prototype.getTabMenu = function () {
                    return this.tabMenu;
                };
                AppBar.prototype.updateAppOpenTabs = function () {
                    this.application.setOpenTabs(this.tabMenu.countVisible());
                };
                return AppBar;
            })(api.dom.DivEl);
            bar.AppBar = AppBar;
            var LauncherButton = (function (_super) {
                __extends(LauncherButton, _super);
                function LauncherButton(action) {
                    _super.call(this, action, false);
                    this.addClass('launcher-button');
                }
                return LauncherButton;
            })(api.ui.button.ActionButton);
            bar.LauncherButton = LauncherButton;
            var HomeButton = (function (_super) {
                __extends(HomeButton, _super);
                function HomeButton(app, action) {
                    _super.call(this, app.getName());
                    this.addClass("home-button app-icon icon-" + app.getIconUrl());
                    this.onClicked(function (event) {
                        action.execute();
                    });
                }
                return HomeButton;
            })(api.ui.button.Button);
            bar.HomeButton = HomeButton;
        })(bar = _app.bar || (_app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarTabId = (function () {
                function AppBarTabId(mode, id) {
                    this.mode = mode;
                    this.id = id;
                }
                AppBarTabId.forNew = function (id) {
                    return new AppBarTabId("new", id);
                };
                AppBarTabId.forEdit = function (id) {
                    return new AppBarTabId("edit", id);
                };
                AppBarTabId.forView = function (id) {
                    return new AppBarTabId("view", id);
                };
                AppBarTabId.prototype.getId = function () {
                    return this.id;
                };
                AppBarTabId.prototype.changeToEditMode = function (id) {
                    this.mode = "edit";
                    this.id = id;
                };
                AppBarTabId.prototype.equals = function (other) {
                    return other.id == this.id && other.mode == this.mode;
                };
                AppBarTabId.prototype.toString = function () {
                    return this.mode + ":" + this.id;
                };
                return AppBarTabId;
            })();
            bar.AppBarTabId = AppBarTabId;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var AppBarTabMenu = (function (_super) {
                __extends(AppBarTabMenu, _super);
                function AppBarTabMenu() {
                    var _this = this;
                    _super.call(this, "appbar-tab-menu");
                    this.buttonLabelChangedListeners = [];
                    this.barEl = new api.dom.UlEl("bar");
                    this.prependChild(this.barEl);
                    this.onRendered(function () {
                        _this.updateTabMenuButtonVisibility();
                    });
                    ResponsiveManager.onAvailableSizeChanged(this, this.moveTabs.bind(this, 50));
                }
                AppBarTabMenu.prototype.updateTabMenuButtonVisibility = function () {
                    var menuTabsCount = this.getMenuEl().getChildren().length;
                    if (menuTabsCount === 0) {
                        this.appBarTabMenuButton.hide();
                        this.getMenuEl().hide();
                    }
                    else {
                        this.appBarTabMenuButton.show();
                    }
                    this.appBarTabMenuButton.setTabCount(menuTabsCount);
                };
                AppBarTabMenu.prototype.moveTabs = function (timeout) {
                    if (timeout === void 0) { timeout = 0; }
                    clearInterval(this.timeoutHandler);
                    if (timeout > 0) {
                        this.timeoutHandler = setTimeout(this.moveTabsHandler.bind(this), timeout);
                    }
                    else {
                        this.moveTabsHandler();
                    }
                };
                AppBarTabMenu.prototype.moveTabsHandler = function () {
                    var width = this.getEl().getWidth(), barWidth = this.barEl.getEl().getWidth(), exactTabs = AppBarTabMenu.MAX_WIDTH < width ? Math.ceil(barWidth / AppBarTabMenu.TAB_WIDTH) || 1 : 1, barTabs = this.barEl.getChildren(), menuTabs = this.getMenuEl().getChildren(), tabsInBar = barTabs.length, tabsInMenu = menuTabs.length;
                    while (!(exactTabs === tabsInBar || (exactTabs > tabsInBar && tabsInMenu === 0))) {
                        if (exactTabs > tabsInBar) {
                            if (tabsInMenu > 0) {
                                var tabEl = this.getMenuEl().getFirstChild();
                                this.getMenuEl().removeChild(tabEl);
                                this.barEl.appendChild(tabEl);
                                tabsInBar++;
                                tabsInMenu--;
                            }
                        }
                        else if (exactTabs < tabsInBar) {
                            if (tabsInBar > 0) {
                                var tabEl = this.barEl.getLastChild();
                                this.barEl.removeChild(tabEl);
                                this.getMenuEl().prependChild(tabEl);
                                tabsInBar--;
                                tabsInMenu++;
                            }
                        }
                    }
                    this.updateTabMenuButtonVisibility();
                };
                AppBarTabMenu.prototype.createTabMenuButton = function () {
                    this.appBarTabMenuButton = new bar.AppBarTabMenuButton();
                    return this.appBarTabMenuButton;
                };
                AppBarTabMenu.prototype.setButtonLabel = function (value) {
                    _super.prototype.setButtonLabel.call(this, value);
                    this.notifyButtonLabelChanged();
                    return this;
                };
                AppBarTabMenu.prototype.addNavigationItem = function (tab) {
                    _super.prototype.addNavigationItem.call(this, tab);
                    this.appBarTabMenuButton.setTabCount(this.countVisible());
                    this.appBarTabMenuButton.setEditing(tab.isEditing());
                    this.moveTabs(0);
                    this.makeTabFirst(tab);
                    ResponsiveManager.fireResizeEvent();
                };
                AppBarTabMenu.prototype.removeNavigationItem = function (tab) {
                    _super.prototype.removeNavigationItem.call(this, tab);
                    this.appBarTabMenuButton.setTabCount(this.countVisible());
                    var newSelectedTab = this.getSelectedNavigationItem();
                    if (newSelectedTab) {
                        this.appBarTabMenuButton.setEditing(newSelectedTab.isEditing());
                    }
                    this.moveTabs(0);
                    ResponsiveManager.fireResizeEvent();
                };
                AppBarTabMenu.prototype.getNavigationItemById = function (tabId) {
                    var items = this.getNavigationItems();
                    var item;
                    for (var i = 0; i < items.length; i++) {
                        item = items[i];
                        if (item.getTabId().equals(tabId)) {
                            return item;
                        }
                    }
                    return null;
                };
                AppBarTabMenu.prototype.getNavigationItemByIdValue = function (tabIdValue) {
                    var items = this.getNavigationItems();
                    var item;
                    for (var i = 0; i < items.length; i++) {
                        item = items[i];
                        if (item.getTabId().getId() === tabIdValue) {
                            return item;
                        }
                    }
                    return null;
                };
                AppBarTabMenu.prototype.selectNavigationItem = function (tabIndex) {
                    _super.prototype.selectNavigationItem.call(this, tabIndex);
                    var tab = this.getNavigationItem(tabIndex);
                    this.appBarTabMenuButton.setEditing(tab.isEditing());
                    this.hideMenu();
                    this.moveTabs();
                    this.makeSelectedTabFirst(tab);
                };
                AppBarTabMenu.prototype.makeSelectedTabFirst = function (tab) {
                    if (this.getMenuEl().hasChild(tab)) {
                        this.getMenuEl().removeChild(tab);
                        this.barEl.prependChild(tab);
                        this.moveTabs();
                    }
                };
                AppBarTabMenu.prototype.makeTabFirst = function (tab) {
                    var canBeMoved;
                    if (canBeMoved = this.getMenuEl().hasChild(tab)) {
                        this.getMenuEl().removeChild(tab);
                    }
                    else if (canBeMoved = this.barEl.hasChild(tab)) {
                        this.barEl.removeChild(tab);
                    }
                    if (canBeMoved) {
                        this.barEl.prependChild(tab);
                        this.moveTabs();
                    }
                };
                AppBarTabMenu.prototype.deselectNavigationItem = function () {
                    _super.prototype.deselectNavigationItem.call(this);
                    this.appBarTabMenuButton.setEditing(false);
                };
                AppBarTabMenu.prototype.onButtonLabelChanged = function (listener) {
                    this.buttonLabelChangedListeners.push(listener);
                };
                AppBarTabMenu.prototype.unButtonLabelChanged = function (listener) {
                    this.buttonLabelChangedListeners = this.buttonLabelChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                AppBarTabMenu.prototype.notifyButtonLabelChanged = function () {
                    var _this = this;
                    this.buttonLabelChangedListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                AppBarTabMenu.TAB_WIDTH = 190;
                AppBarTabMenu.MAX_WIDTH = 248;
                return AppBarTabMenu;
            })(api.ui.tab.TabMenu);
            bar.AppBarTabMenu = AppBarTabMenu;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarTabMenuButton = (function (_super) {
                __extends(AppBarTabMenuButton, _super);
                function AppBarTabMenuButton() {
                    _super.call(this);
                    this.tabCountEl = new AppBarTabCount();
                    this.prependChild(this.tabCountEl);
                }
                AppBarTabMenuButton.prototype.setTabCount = function (value) {
                    this.tabCountEl.setCount(value);
                };
                AppBarTabMenuButton.prototype.setEditing = function (editing) {
                    if (editing && !this.hasClass('editing')) {
                        this.addClass('editing');
                    }
                    else if (!editing && this.hasClass('editing')) {
                        this.removeClass('editing');
                    }
                };
                return AppBarTabMenuButton;
            })(api.ui.tab.TabMenuButton);
            bar.AppBarTabMenuButton = AppBarTabMenuButton;
            var AppBarTabCount = (function (_super) {
                __extends(AppBarTabCount, _super);
                function AppBarTabCount() {
                    _super.call(this, "tab-count");
                }
                AppBarTabCount.prototype.setCount = function (value) {
                    this.getEl().setInnerHtml(value > 0 ? "" + value : "");
                };
                return AppBarTabCount;
            })(api.dom.SpanEl);
            bar.AppBarTabCount = AppBarTabCount;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarTabMenuItem = (function (_super) {
                __extends(AppBarTabMenuItem, _super);
                function AppBarTabMenuItem(builder) {
                    _super.call(this, builder);
                    this.addClass("appbar-tab-menu-item");
                    this.editing = builder.editing;
                    this.tabId = builder.tabId;
                }
                AppBarTabMenuItem.prototype.isEditing = function () {
                    return this.editing;
                };
                AppBarTabMenuItem.prototype.getTabId = function () {
                    return this.tabId;
                };
                AppBarTabMenuItem.prototype.setTabId = function (tabId) {
                    this.tabId = tabId;
                };
                return AppBarTabMenuItem;
            })(api.ui.tab.TabMenuItem);
            bar.AppBarTabMenuItem = AppBarTabMenuItem;
            var AppBarTabMenuItemBuilder = (function (_super) {
                __extends(AppBarTabMenuItemBuilder, _super);
                function AppBarTabMenuItemBuilder() {
                    _super.call(this);
                    this.closeButtonEnabled = true;
                }
                AppBarTabMenuItemBuilder.prototype.setLabel = function (label) {
                    _super.prototype.setLabel.call(this, label);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setTabId = function (tabId) {
                    this.tabId = tabId;
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setEditing = function (editing) {
                    this.editing = editing;
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setCloseAction = function (closeAction) {
                    _super.prototype.setCloseAction.call(this, closeAction);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setMarkUnnamed = function (markUnnamed) {
                    _super.prototype.setMarkUnnamed.call(this, markUnnamed);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setMarkInvalid = function (markInvalid) {
                    _super.prototype.setMarkInvalid.call(this, markInvalid);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.build = function () {
                    return new AppBarTabMenuItem(this);
                };
                return AppBarTabMenuItemBuilder;
            })(api.ui.tab.TabMenuItemBuilder);
            bar.AppBarTabMenuItemBuilder = AppBarTabMenuItemBuilder;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
            var BrowsePanel = (function (_super) {
                __extends(BrowsePanel, _super);
                function BrowsePanel(params) {
                    var _this = this;
                    _super.call(this);
                    this.filterPanelRefreshNeeded = false;
                    this.filterPanelForcedShown = false;
                    this.browseToolbar = params.browseToolbar;
                    this.treeGrid = params.treeGrid;
                    this.browseItemPanel = params.browseItemPanel;
                    this.filterPanel = params.filterPanel;
                    this.browseItemPanel.onDeselected(function (event) {
                        _this.treeGrid.deselectNode(event.getBrowseItem().getId());
                    });
                    this.gridAndToolbarContainer = new api.ui.panel.Panel();
                    this.gridAndToolbarContainer.appendChild(this.browseToolbar);
                    var gridPanel = new api.ui.panel.Panel();
                    gridPanel.appendChild(this.treeGrid);
                    this.gridAndToolbarContainer.appendChild(gridPanel);
                    this.gridAndDetailSplitPanel = new api.ui.panel.SplitPanelBuilder(this.gridAndToolbarContainer, this.browseItemPanel).setAlignmentTreshold(BrowsePanel.SPLIT_PANEL_ALIGNMENT_TRESHOLD).build();
                    if (this.filterPanel) {
                        this.filterAndGridAndDetailSplitPanel = new api.ui.panel.SplitPanelBuilder(this.filterPanel, this.gridAndDetailSplitPanel).setFirstPanelSize(200, 0 /* PIXEL */).setAlignment(1 /* VERTICAL */).build();
                    }
                    else {
                        this.filterAndGridAndDetailSplitPanel = this.gridAndDetailSplitPanel;
                    }
                    this.treeGrid.onSelectionChanged(function (currentSelection, fullSelection) {
                        var browseItems = _this.treeNodesToBrowseItems(fullSelection);
                        _this.treeGrid.getContextMenu().getActions().updateActionsEnabledState(browseItems).then(function (updatedBrowseItems) {
                            _this.browseItemPanel.setItems(updatedBrowseItems);
                        });
                    });
                    this.onRendered(function (event) {
                        _this.appendChild(_this.filterAndGridAndDetailSplitPanel);
                    });
                    ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        if (item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                            if (_this.filterPanel && !_this.filterAndGridAndDetailSplitPanel.isPanelHidden(1) && !_this.filterPanelForcedShown) {
                                _this.filterAndGridAndDetailSplitPanel.hidePanel(1);
                            }
                            if (!_this.gridAndDetailSplitPanel.isPanelHidden(2)) {
                                _this.gridAndDetailSplitPanel.hidePanel(2);
                            }
                        }
                        else if (item.isInRangeOrBigger(ResponsiveRanges._540_720)) {
                            if (_this.filterPanel && _this.filterAndGridAndDetailSplitPanel.isPanelHidden(1)) {
                                _this.filterAndGridAndDetailSplitPanel.showPanel(1);
                            }
                            if (_this.gridAndDetailSplitPanel.isPanelHidden(2)) {
                                _this.gridAndDetailSplitPanel.showPanel(2);
                            }
                        }
                    });
                }
                BrowsePanel.prototype.getTreeGrid = function () {
                    return this.treeGrid;
                };
                BrowsePanel.prototype.getBrowseItemPanel = function () {
                    return this.browseItemPanel;
                };
                BrowsePanel.prototype.getActions = function () {
                    return this.browseToolbar.getActions();
                };
                BrowsePanel.prototype.treeNodesToBrowseItems = function (nodes) {
                    return [];
                };
                BrowsePanel.prototype.refreshFilter = function () {
                    if (this.isFilterPanelRefreshNeeded()) {
                        if (this.filterPanel) {
                            this.filterPanel.refresh();
                        }
                        this.filterPanelRefreshNeeded = false;
                    }
                };
                BrowsePanel.prototype.isFilterPanelRefreshNeeded = function () {
                    return this.filterPanelRefreshNeeded;
                };
                BrowsePanel.prototype.setFilterPanelRefreshNeeded = function (refreshNeeded) {
                    this.filterPanelRefreshNeeded = refreshNeeded;
                };
                BrowsePanel.prototype.toggleFilterPanel = function () {
                    this.filterPanelForcedShown = !this.filterPanelForcedShown;
                    !this.filterAndGridAndDetailSplitPanel.isPanelHidden(1) ? this.filterAndGridAndDetailSplitPanel.hidePanel(1) : this.filterAndGridAndDetailSplitPanel.showPanel(1);
                };
                BrowsePanel.SPLIT_PANEL_ALIGNMENT_TRESHOLD = 1180;
                return BrowsePanel;
            })(api.ui.panel.Panel);
            browse.BrowsePanel = BrowsePanel;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var BrowseItem = (function () {
                function BrowseItem(model) {
                    this.model = model;
                }
                BrowseItem.prototype.setId = function (value) {
                    this.id = value;
                    return this;
                };
                BrowseItem.prototype.setDisplayName = function (value) {
                    this.displayName = value;
                    return this;
                };
                BrowseItem.prototype.setPath = function (value) {
                    this.path = value;
                    return this;
                };
                BrowseItem.prototype.setIconUrl = function (value) {
                    this.iconUrl = value;
                    return this;
                };
                BrowseItem.prototype.setIconClass = function (iconClass) {
                    this.iconClass = iconClass;
                    return this;
                };
                BrowseItem.prototype.setRenderable = function (value) {
                    this.renderable = value;
                    return this;
                };
                BrowseItem.prototype.getIconClass = function () {
                    return this.iconClass;
                };
                BrowseItem.prototype.getModel = function () {
                    return this.model;
                };
                BrowseItem.prototype.getId = function () {
                    return this.id;
                };
                BrowseItem.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                BrowseItem.prototype.getPath = function () {
                    return this.path;
                };
                BrowseItem.prototype.getIconUrl = function () {
                    return this.iconUrl;
                };
                BrowseItem.prototype.isRenderable = function () {
                    return this.renderable;
                };
                BrowseItem.prototype.toViewItem = function () {
                    return new api.app.view.ViewItem(this.model).setIconUrl(this.iconUrl).setIconClass(this.iconClass).setDisplayName(this.displayName).setPath(this.path).setRenderable(this.isRenderable());
                };
                BrowseItem.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, BrowseItem)) {
                        return false;
                    }
                    var other = o;
                    return this.model.equals(other.model) && this.displayName == other.displayName && this.path == other.path && this.iconUrl == other.iconUrl && this.iconClass == other.iconClass;
                };
                return BrowseItem;
            })();
            browse.BrowseItem = BrowseItem;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var SelectionItem = (function (_super) {
                __extends(SelectionItem, _super);
                function SelectionItem(viewer, item, removeCallback) {
                    _super.call(this, "browse-selection-item");
                    this.viewer = viewer;
                    this.item = item;
                    this.initRemoveButton(removeCallback);
                }
                SelectionItem.prototype.initRemoveButton = function (callback) {
                    this.removeEl = new api.dom.DivEl("icon remove");
                    this.removeEl.onClicked(function (event) {
                        if (callback) {
                            callback();
                        }
                    });
                };
                SelectionItem.prototype.getBrowseItem = function () {
                    return this.item;
                };
                SelectionItem.prototype.updateViewer = function (viewer) {
                    this.viewer = viewer;
                    this.render();
                };
                SelectionItem.prototype.doRender = function () {
                    this.removeChildren();
                    this.appendChild(this.removeEl);
                    this.appendChild(this.viewer);
                    return true;
                };
                return SelectionItem;
            })(api.dom.DivEl);
            browse.SelectionItem = SelectionItem;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var BrowseItemsSelectionPanel = (function (_super) {
                __extends(BrowseItemsSelectionPanel, _super);
                function BrowseItemsSelectionPanel() {
                    _super.call(this, "items-selection-panel");
                    this.deselectedListeners = [];
                    this.items = [];
                    this.selectionItems = [];
                    this.messageForNoSelection = "You are wasting this space - select something!";
                    this.getEl().addClass('no-selection').setInnerHtml(this.messageForNoSelection);
                }
                BrowseItemsSelectionPanel.prototype.addItem = function (item) {
                    var _this = this;
                    var index = this.indexOf(item);
                    if (index >= 0) {
                        var currentItem = this.items[index];
                        if (!currentItem.equals(item)) {
                            this.items[index] = item;
                        }
                        return;
                    }
                    if (this.items.length === 0) {
                        this.removeClass('no-selection');
                        this.removeChildren();
                    }
                    var removeCallback = function () {
                        _this.removeItem(item);
                    };
                    var selectionItem = new browse.SelectionItem(this.createItemViewer(item), item, removeCallback);
                    this.appendChild(selectionItem);
                    this.selectionItems.push(selectionItem);
                    this.items.push(item);
                };
                BrowseItemsSelectionPanel.prototype.removeItem = function (item) {
                    var index = this.indexOf(item);
                    if (index < 0) {
                        return;
                    }
                    this.selectionItems[index].remove();
                    this.selectionItems.splice(index, 1);
                    this.items.splice(index, 1);
                    if (this.items.length === 0) {
                        this.getEl().addClass('no-selection').setInnerHtml(this.messageForNoSelection);
                    }
                    this.notifyDeselected(item);
                };
                BrowseItemsSelectionPanel.prototype.getItems = function () {
                    return this.items;
                };
                BrowseItemsSelectionPanel.prototype.setItems = function (items) {
                    var _this = this;
                    var itemsToRemove = this.items.filter(function (item) {
                        for (var i = 0; i < items.length; i++) {
                            if (item.getPath() == items[i].getPath()) {
                                return false;
                            }
                        }
                        return true;
                    });
                    itemsToRemove.forEach(function (item) {
                        _this.removeItem(item);
                    });
                    items.forEach(function (item) {
                        _this.addItem(item);
                    });
                };
                BrowseItemsSelectionPanel.prototype.createItemViewer = function (item) {
                    var viewer = new api.ui.Viewer();
                    viewer.setObject(item.getModel());
                    return viewer;
                };
                BrowseItemsSelectionPanel.prototype.updateItemViewers = function (items) {
                    var _this = this;
                    items.forEach(function (item) {
                        for (var i = 0; i < _this.selectionItems.length; i++) {
                            if (_this.selectionItems[i].getBrowseItem().getPath() === item.getPath()) {
                                _this.selectionItems[i].updateViewer(_this.createItemViewer(item));
                                console.log(_this.createItemViewer(item));
                                break;
                            }
                        }
                    });
                };
                BrowseItemsSelectionPanel.prototype.indexOf = function (item) {
                    for (var i = 0; i < this.items.length; i++) {
                        if (item.getPath() == this.items[i].getPath()) {
                            return i;
                        }
                    }
                    return -1;
                };
                BrowseItemsSelectionPanel.prototype.onDeselected = function (listener) {
                    this.deselectedListeners.push(listener);
                };
                BrowseItemsSelectionPanel.prototype.unDeselected = function (listener) {
                    this.deselectedListeners = this.deselectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                BrowseItemsSelectionPanel.prototype.notifyDeselected = function (item) {
                    var _this = this;
                    this.deselectedListeners.forEach(function (listener) {
                        listener.call(_this, new browse.ItemDeselectedEvent(item));
                    });
                };
                return BrowseItemsSelectionPanel;
            })(api.ui.panel.Panel);
            browse.BrowseItemsSelectionPanel = BrowseItemsSelectionPanel;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var BrowseItemPanel = (function (_super) {
                __extends(BrowseItemPanel, _super);
                function BrowseItemPanel() {
                    _super.call(this, "browse-item-panel");
                    this.itemsSelectionPanel = this.createItemSelectionPanel();
                    this.itemStatisticsPanel = this.createItemStatisticsPanel();
                    this.addPanel(this.itemsSelectionPanel);
                    this.addPanel(this.itemStatisticsPanel);
                    this.showPanelByIndex(0);
                }
                BrowseItemPanel.prototype.createItemSelectionPanel = function () {
                    return new browse.BrowseItemsSelectionPanel();
                };
                BrowseItemPanel.prototype.createItemStatisticsPanel = function () {
                    return new api.app.view.ItemStatisticsPanel();
                };
                BrowseItemPanel.prototype.setItems = function (items) {
                    this.itemsSelectionPanel.setItems(items);
                    this.updateDisplayedPanel();
                };
                BrowseItemPanel.prototype.getItems = function () {
                    return this.itemsSelectionPanel.getItems();
                };
                BrowseItemPanel.prototype.updateItemViewers = function (items) {
                    this.itemsSelectionPanel.updateItemViewers(items);
                };
                BrowseItemPanel.prototype.updateDisplayedPanel = function () {
                    var selectedItems = this.getItems();
                    if (selectedItems.length == 1) {
                        this.showPanelByIndex(1);
                        this.itemStatisticsPanel.setItem(selectedItems[0].toViewItem());
                    }
                    else {
                        this.showPanelByIndex(0);
                    }
                };
                BrowseItemPanel.prototype.onDeselected = function (listener) {
                    this.itemsSelectionPanel.onDeselected(listener);
                };
                BrowseItemPanel.prototype.unDeselected = function (listener) {
                    this.itemsSelectionPanel.unDeselected(listener);
                };
                return BrowseItemPanel;
            })(api.ui.panel.DeckPanel);
            browse.BrowseItemPanel = BrowseItemPanel;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var ItemDeselectedEvent = (function () {
                function ItemDeselectedEvent(browseItem) {
                    this.browseItem = browseItem;
                }
                ItemDeselectedEvent.prototype.getBrowseItem = function () {
                    return this.browseItem;
                };
                return ItemDeselectedEvent;
            })();
            browse.ItemDeselectedEvent = ItemDeselectedEvent;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var BrowseFilterPanel = (function (_super) {
                    __extends(BrowseFilterPanel, _super);
                    function BrowseFilterPanel(aggregations, groupViews) {
                        var _this = this;
                        _super.call(this);
                        this.searchListeners = [];
                        this.refreshListeners = [];
                        this.resetListeners = [];
                        this.addClass('filter-panel');
                        this.searchField = new filter.TextSearchField('Search');
                        this.searchField.onValueChanged(function () {
                            _this.search(_this.searchField);
                        });
                        this.clearFilter = new filter.ClearFilterButton();
                        this.clearFilter.onClicked(function (event) {
                            _this.reset();
                        });
                        this.aggregationContainer = new api.aggregation.AggregationContainer();
                        this.appendChild(this.aggregationContainer);
                        if (groupViews != null) {
                            groupViews.forEach(function (aggregationGroupView) {
                                aggregationGroupView.onBucketViewSelectionChanged(function (event) {
                                    _this.search(event.getBucketView());
                                });
                                _this.aggregationContainer.addAggregationGroupView(aggregationGroupView);
                            });
                        }
                        this.onRendered(function (event) {
                            _this.appendChild(_this.searchField);
                            _this.appendChild(_this.clearFilter);
                            _this.appendChild(_this.aggregationContainer);
                            api.ui.KeyBindings.get().bindKey(new api.ui.KeyBinding("/", function (e) {
                                setTimeout(_this.giveFocusToSearch.bind(_this), 100);
                            }).setGlobal(true));
                        });
                    }
                    BrowseFilterPanel.prototype.giveFocusToSearch = function () {
                        this.searchField.giveFocus();
                    };
                    BrowseFilterPanel.prototype.updateAggregations = function (aggregations, doUpdateAll) {
                        this.aggregationContainer.updateAggregations(aggregations, doUpdateAll);
                    };
                    BrowseFilterPanel.prototype.getSearchInputValues = function () {
                        var searchInputValues = new api.query.SearchInputValues();
                        searchInputValues.setAggregationSelections(this.aggregationContainer.getSelectedValuesByAggregationName());
                        searchInputValues.setTextSearchFieldValue(this.searchField.getEl().getValue());
                        return searchInputValues;
                    };
                    BrowseFilterPanel.prototype.hasFilterSet = function () {
                        return this.aggregationContainer.hasSelectedBuckets() || this.searchField.getHTMLElement()['value'].trim() != '';
                    };
                    BrowseFilterPanel.prototype.search = function (elementChanged) {
                        if (this.hasFilterSet()) {
                            this.clearFilter.show();
                        }
                        else {
                            this.clearFilter.hide();
                        }
                        var values = this.getSearchInputValues();
                        this.notifySearch(values, elementChanged);
                    };
                    BrowseFilterPanel.prototype.refresh = function () {
                        this.notifyRefresh();
                    };
                    BrowseFilterPanel.prototype.reset = function () {
                        this.searchField.clear(true);
                        this.aggregationContainer.deselectAll(true);
                        this.clearFilter.hide();
                        this.notifyReset();
                    };
                    BrowseFilterPanel.prototype.onSearch = function (listener) {
                        this.searchListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.onReset = function (listener) {
                        this.resetListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.onRefresh = function (listener) {
                        this.refreshListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.unSearch = function (listener) {
                        this.searchListeners = this.searchListeners.filter(function (currentListener) {
                            return currentListener != listener;
                        });
                    };
                    BrowseFilterPanel.prototype.unReset = function (listener) {
                        this.resetListeners = this.resetListeners.filter(function (currentListener) {
                            return currentListener != listener;
                        });
                    };
                    BrowseFilterPanel.prototype.notifySearch = function (searchInputValues, elementChanged) {
                        var _this = this;
                        this.searchListeners.forEach(function (listener) {
                            listener.call(_this, new filter.SearchEvent(searchInputValues, elementChanged));
                        });
                    };
                    BrowseFilterPanel.prototype.notifyRefresh = function () {
                        var _this = this;
                        this.refreshListeners.forEach(function (listener) {
                            listener.call(_this, new filter.RefreshEvent(_this.getSearchInputValues()));
                        });
                    };
                    BrowseFilterPanel.prototype.notifyReset = function () {
                        var _this = this;
                        this.resetListeners.forEach(function (listener) {
                            listener.call(_this);
                        });
                    };
                    return BrowseFilterPanel;
                })(api.ui.panel.Panel);
                filter.BrowseFilterPanel = BrowseFilterPanel;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var TextSearchField = (function (_super) {
                    __extends(TextSearchField, _super);
                    function TextSearchField(placeholder) {
                        var _this = this;
                        _super.call(this, 'text-search-field');
                        this.valueChangedListeners = [];
                        this.setPlaceholder(placeholder);
                        this.onKeyDown(function (event) {
                            if (event.which == 9) {
                            }
                            else if (event.which == 13) {
                                _this.notifyValueChanged();
                            }
                            else {
                                if (_this.timerId !== null) {
                                    window.clearTimeout(_this.timerId);
                                    _this.timerId = null;
                                }
                                _this.timerId = window.setTimeout(function () {
                                    _this.notifyValueChanged();
                                }, 500);
                            }
                        });
                    }
                    TextSearchField.prototype.clear = function (supressEvent) {
                        window.clearTimeout(this.timerId);
                        this.getHTMLElement()['value'] = '';
                        if (!supressEvent) {
                            this.notifyValueChanged();
                        }
                    };
                    TextSearchField.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    TextSearchField.prototype.notifyValueChanged = function () {
                        var currentValue = this.getHTMLElement()['value'];
                        if (currentValue == this.previousValue) {
                            return;
                        }
                        this.valueChangedListeners.forEach(function (listener) {
                            listener();
                        });
                        this.previousValue = currentValue;
                    };
                    return TextSearchField;
                })(api.dom.InputEl);
                filter.TextSearchField = TextSearchField;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var ClearFilterButton = (function (_super) {
                    __extends(ClearFilterButton, _super);
                    function ClearFilterButton() {
                        _super.call(this, 'clear-filter-button');
                        this.getEl().setInnerHtml('Clear filter');
                        this.getHTMLElement().setAttribute('href', 'javascript:;');
                        this.hide();
                    }
                    return ClearFilterButton;
                })(api.dom.AEl);
                filter.ClearFilterButton = ClearFilterButton;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var SearchEvent = (function () {
                    function SearchEvent(searchInputValues, elementChanged) {
                        this.searchInputValues = searchInputValues;
                        this.elementChanged = elementChanged;
                    }
                    SearchEvent.prototype.getSearchInputValues = function () {
                        return this.searchInputValues;
                    };
                    SearchEvent.prototype.getElementChanged = function () {
                        return this.elementChanged;
                    };
                    return SearchEvent;
                })();
                filter.SearchEvent = SearchEvent;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var RefreshEvent = (function (_super) {
                    __extends(RefreshEvent, _super);
                    function RefreshEvent() {
                        _super.apply(this, arguments);
                    }
                    return RefreshEvent;
                })(filter.SearchEvent);
                filter.RefreshEvent = RefreshEvent;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var remove;
        (function (remove) {
            var DeleteItem = (function () {
                function DeleteItem(iconUrl, displayName) {
                    this.iconUrl = iconUrl;
                    this.displayName = displayName;
                }
                DeleteItem.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                DeleteItem.prototype.getIconUrl = function () {
                    return this.iconUrl;
                };
                return DeleteItem;
            })();
            remove.DeleteItem = DeleteItem;
        })(remove = app.remove || (app.remove = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var remove;
        (function (remove) {
            var DeleteDialog = (function (_super) {
                __extends(DeleteDialog, _super);
                function DeleteDialog(modelName) {
                    _super.call(this, {
                        title: new api.ui.dialog.ModalDialogHeader("Delete " + modelName)
                    });
                    this.itemList = new DeleteDialogItemList();
                    this.modelName = modelName;
                    this.getEl().addClass("delete-dialog");
                    this.appendChildToContentPanel(this.itemList);
                }
                DeleteDialog.prototype.show = function () {
                    api.dom.Body.get().appendChild(this);
                    _super.prototype.show.call(this);
                };
                DeleteDialog.prototype.close = function () {
                    _super.prototype.close.call(this);
                    this.remove();
                };
                DeleteDialog.prototype.setDeleteAction = function (action) {
                    this.deleteAction = action;
                    this.addAction(action, true, true);
                };
                DeleteDialog.prototype.getDeleteAction = function () {
                    return this.deleteAction;
                };
                DeleteDialog.prototype.setDeleteViewers = function (deleteViewers) {
                    this.deleteViewers = deleteViewers;
                    this.itemList.clear();
                    if (deleteViewers.length > 1) {
                        this.setTitle("Delete " + this.modelName + "s");
                    }
                    else {
                        this.setTitle("Delete " + this.modelName);
                    }
                    for (var i in this.deleteViewers) {
                        var deleteViewer = this.deleteViewers[i];
                        this.itemList.appendChild(deleteViewer);
                    }
                };
                return DeleteDialog;
            })(api.ui.dialog.ModalDialog);
            remove.DeleteDialog = DeleteDialog;
            var DeleteDialogItemList = (function (_super) {
                __extends(DeleteDialogItemList, _super);
                function DeleteDialogItemList() {
                    _super.call(this);
                    this.getEl().addClass("item-list");
                }
                DeleteDialogItemList.prototype.clear = function () {
                    this.removeChildren();
                };
                return DeleteDialogItemList;
            })(api.dom.DivEl);
            remove.DeleteDialogItemList = DeleteDialogItemList;
        })(remove = app.remove || (app.remove = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var remove;
        (function (remove) {
            var CancelDeleteDialogAction = (function (_super) {
                __extends(CancelDeleteDialogAction, _super);
                function CancelDeleteDialogAction() {
                    _super.call(this, "Cancel", "esc");
                }
                return CancelDeleteDialogAction;
            })(api.ui.Action);
            remove.CancelDeleteDialogAction = CancelDeleteDialogAction;
        })(remove = app.remove || (app.remove = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ViewItem = (function () {
                function ViewItem(model) {
                    this.model = model;
                }
                ViewItem.prototype.setDisplayName = function (value) {
                    this.displayName = value;
                    return this;
                };
                ViewItem.prototype.setPath = function (value) {
                    this.path = value;
                    return this;
                };
                ViewItem.prototype.setPathName = function (value) {
                    this.pathName = value;
                    return this;
                };
                ViewItem.prototype.setIconUrl = function (value) {
                    this.iconUrl = value;
                    return this;
                };
                ViewItem.prototype.setIconClass = function (iconClass) {
                    this.iconClass = iconClass;
                    return this;
                };
                ViewItem.prototype.setIconSize = function (value) {
                    this.iconSize = value;
                    return this;
                };
                ViewItem.prototype.setRenderable = function (value) {
                    this.renderable = value;
                    return this;
                };
                ViewItem.prototype.getModel = function () {
                    return this.model;
                };
                ViewItem.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                ViewItem.prototype.getPath = function () {
                    return this.path;
                };
                ViewItem.prototype.getPathName = function () {
                    return this.pathName;
                };
                ViewItem.prototype.getIconUrl = function () {
                    return this.iconUrl;
                };
                ViewItem.prototype.getIconClass = function () {
                    return this.iconClass;
                };
                ViewItem.prototype.getIconSize = function () {
                    return this.iconSize;
                };
                ViewItem.prototype.isRenderable = function () {
                    return this.renderable;
                };
                ViewItem.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ViewItem)) {
                        return false;
                    }
                    var other = o;
                    return this.model.equals(other.getModel()) && this.displayName === other.getDisplayName() && this.path === other.getPath() && this.pathName === other.getPathName() && this.iconUrl === other.getIconUrl() && this.iconClass === other.getIconClass() && this.iconSize === other.getIconSize();
                };
                return ViewItem;
            })();
            view.ViewItem = ViewItem;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemStatisticsPanel = (function (_super) {
                __extends(ItemStatisticsPanel, _super);
                function ItemStatisticsPanel(className) {
                    _super.call(this, "item-statistics-panel" + (className ? " " + className : ""));
                    this.header = new view.ItemStatisticsHeader();
                    this.appendChild(this.header);
                }
                ItemStatisticsPanel.prototype.getHeader = function () {
                    return this.header;
                };
                ItemStatisticsPanel.prototype.setItem = function (item) {
                    this.browseItem = item;
                    this.header.setItem(item);
                };
                ItemStatisticsPanel.prototype.getItem = function () {
                    return this.browseItem;
                };
                return ItemStatisticsPanel;
            })(api.ui.panel.Panel);
            view.ItemStatisticsPanel = ItemStatisticsPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var MultiItemStatisticsPanel = (function (_super) {
                __extends(MultiItemStatisticsPanel, _super);
                function MultiItemStatisticsPanel(className) {
                    _super.call(this, className);
                    this.tabMenu = new api.ui.tab.TabMenu();
                    this.tabMenu.hide();
                    this.appendChild(this.tabMenu);
                    this.deckPanel = new api.ui.panel.NavigatedDeckPanel(this.tabMenu);
                    this.deckPanel.setDoOffset(false);
                    this.appendChild(this.deckPanel);
                }
                MultiItemStatisticsPanel.prototype.getTabMenu = function () {
                    return this.tabMenu;
                };
                MultiItemStatisticsPanel.prototype.getDeckPanel = function () {
                    return this.deckPanel;
                };
                MultiItemStatisticsPanel.prototype.addNavigablePanel = function (tab, panel, select) {
                    this.tabMenu.show();
                    this.deckPanel.addNavigablePanel(tab, panel, select);
                };
                MultiItemStatisticsPanel.prototype.showPanel = function (index) {
                    this.tabMenu.selectNavigationItem(index);
                };
                return MultiItemStatisticsPanel;
            })(view.ItemStatisticsPanel);
            view.MultiItemStatisticsPanel = MultiItemStatisticsPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemStatisticsHeader = (function (_super) {
                __extends(ItemStatisticsHeader, _super);
                function ItemStatisticsHeader() {
                    _super.call(this, "header");
                    this.headerTitleEl = new api.dom.H1El("title");
                    this.headerPathEl = new api.dom.H4El("path");
                    this.appendChild(this.headerTitleEl);
                    this.appendChild(this.headerPathEl);
                }
                ItemStatisticsHeader.prototype.setItem = function (item) {
                    this.browseItem = item;
                    if (this.iconEl) {
                        this.iconEl.remove();
                    }
                    if (this.iconDivEl) {
                        this.iconDivEl.remove();
                    }
                    var icon = null;
                    if (this.browseItem.getIconUrl()) {
                        var size = this.browseItem.getIconSize() || 64;
                        icon = api.util.loader.ImageLoader.get(this.browseItem.getIconUrl() + "?size=size", size, size);
                        this.iconEl = new api.dom.Element(new api.dom.NewElementBuilder().setTagName("img").setHelper(new api.dom.ImgHelper(icon)));
                        this.iconEl.addClass("icon");
                        this.prependChild(this.iconEl);
                    }
                    else {
                        this.iconDivEl = new api.dom.DivEl(this.browseItem.getIconClass());
                        this.iconDivEl.addClass("icon");
                        this.prependChild(this.iconDivEl);
                    }
                    if (this.browseItem.getDisplayName()) {
                        this.headerTitleEl.getEl().setInnerHtml(this.browseItem.getDisplayName()).setAttribute("title", this.browseItem.getDisplayName());
                    }
                    else {
                        this.headerTitleEl.getEl().setInnerHtml("").setAttribute("title", "");
                    }
                    this.headerPathEl.removeChildren();
                    if (this.browseItem.getPath()) {
                        var path = new api.dom.SpanEl("parent-path");
                        path.getEl().setInnerHtml(this.browseItem.getPath());
                        this.headerPathEl.appendChild(path);
                    }
                    if (this.browseItem.getPath()) {
                        var pathName = new api.dom.SpanEl("path-name");
                        pathName.getEl().setInnerHtml(this.browseItem.getPathName());
                        this.headerPathEl.appendChild(pathName);
                    }
                };
                return ItemStatisticsHeader;
            })(api.dom.DivEl);
            view.ItemStatisticsHeader = ItemStatisticsHeader;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemViewPanel = (function (_super) {
                __extends(ItemViewPanel, _super);
                function ItemViewPanel(toolbar, panel) {
                    _super.call(this, "item-view-panel");
                    this.closedListeners = [];
                    this.toolbar = toolbar;
                    this.panel = panel;
                    this.appendChild(this.toolbar);
                    this.appendChild(this.panel);
                }
                ItemViewPanel.prototype.getActions = function () {
                    return [];
                };
                ItemViewPanel.prototype.setItem = function (item) {
                    this.browseItem = item;
                };
                ItemViewPanel.prototype.getItem = function () {
                    return this.browseItem;
                };
                ItemViewPanel.prototype.close = function (checkCanClose) {
                    if (checkCanClose === void 0) { checkCanClose = false; }
                    if (!checkCanClose || this.canClose()) {
                        this.notifyClosed();
                    }
                };
                ItemViewPanel.prototype.canClose = function () {
                    return true;
                };
                ItemViewPanel.prototype.onClosed = function (listener) {
                    this.closedListeners.push(listener);
                };
                ItemViewPanel.prototype.unClosed = function (listener) {
                    this.closedListeners = this.closedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                ItemViewPanel.prototype.notifyClosed = function () {
                    var _this = this;
                    this.closedListeners.forEach(function (listener) {
                        listener.call(_this, new view.ItemViewClosedEvent(_this));
                    });
                };
                return ItemViewPanel;
            })(api.ui.panel.Panel);
            view.ItemViewPanel = ItemViewPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (_view) {
            var ItemViewClosedEvent = (function () {
                function ItemViewClosedEvent(view) {
                    this.view = view;
                }
                ItemViewClosedEvent.prototype.getView = function () {
                    return this.view;
                };
                return ItemViewClosedEvent;
            })();
            _view.ItemViewClosedEvent = ItemViewClosedEvent;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemPreviewPanel = (function (_super) {
                __extends(ItemPreviewPanel, _super);
                function ItemPreviewPanel(className) {
                    var _this = this;
                    _super.call(this, "item-preview-panel" + (className ? " " + className : ""));
                    this.mask = new api.ui.mask.LoadMask(this);
                    this.frame = new api.dom.IFrameEl();
                    this.frame.onLoaded(function (event) { return _this.mask.hide(); });
                    this.appendChild(this.frame);
                }
                return ItemPreviewPanel;
            })(api.ui.panel.Panel);
            view.ItemPreviewPanel = ItemPreviewPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemDataGroup = (function (_super) {
                __extends(ItemDataGroup, _super);
                function ItemDataGroup(title, className) {
                    _super.call(this, !!className ? className + " item-data-group" : "item-data-group");
                    this.header = new api.dom.H2El();
                    this.header.getEl().setInnerHtml(title);
                    this.appendChild(this.header);
                }
                ItemDataGroup.prototype.addDataList = function (header) {
                    var datas = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        datas[_i - 1] = arguments[_i];
                    }
                    this.addDataArray(header, datas);
                };
                ItemDataGroup.prototype.addDataArray = function (header, datas) {
                    var dataList = new api.dom.UlEl("data-list");
                    if (header) {
                        var headerElement = new api.dom.LiEl();
                        headerElement.addClass("list-header");
                        headerElement.getEl().setInnerHtml(header);
                        dataList.appendChild(headerElement);
                    }
                    datas.forEach(function (data) {
                        var dataElement = new api.dom.LiEl();
                        dataElement.getEl().setInnerHtml(data);
                        dataList.appendChild(dataElement);
                    });
                    this.appendChild(dataList);
                };
                return ItemDataGroup;
            })(api.dom.DivEl);
            view.ItemDataGroup = ItemDataGroup;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var SaveAction = (function (_super) {
                __extends(SaveAction, _super);
                function SaveAction(wizardPanel) {
                    var _this = this;
                    _super.call(this, "Save", "mod+s", true);
                    this.onExecuted(function () {
                        _this.setEnabled(false);
                        wizardPanel.saveChanges().catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).finally(function () { return _this.setEnabled(true); }).done();
                    });
                }
                return SaveAction;
            })(api.ui.Action);
            wizard.SaveAction = SaveAction;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var CloseAction = (function (_super) {
                __extends(CloseAction, _super);
                function CloseAction(wizardPanel, checkCanClose) {
                    if (checkCanClose === void 0) { checkCanClose = true; }
                    _super.call(this, "Close", "alt+w", true);
                    this.onExecuted(function () {
                        wizardPanel.close(checkCanClose);
                    });
                }
                return CloseAction;
            })(api.ui.Action);
            wizard.CloseAction = CloseAction;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var SaveAndCloseAction = (function (_super) {
                __extends(SaveAndCloseAction, _super);
                function SaveAndCloseAction(wizardPanel) {
                    _super.call(this, "SaveAndClose", "mod+enter", true);
                    this.onExecuted(function () {
                        new wizard.SaveAction(wizardPanel).execute();
                        new wizard.CloseAction(wizardPanel).execute();
                    });
                }
                return SaveAndCloseAction;
            })(api.ui.Action);
            wizard.SaveAndCloseAction = SaveAndCloseAction;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var FormIcon = (function (_super) {
                __extends(FormIcon, _super);
                function FormIcon(iconUrl, iconTitle, uploadUrl) {
                    var _this = this;
                    _super.call(this, "form-icon");
                    this.iconUrl = iconUrl;
                    this.iconTitle = iconTitle;
                    this.uploadUrl = uploadUrl;
                    this.uploadStartedListeners = [];
                    this.uploadFinishedListeners = [];
                    var el = this.getEl();
                    this.img = new api.dom.ImgEl(this.iconUrl);
                    el.appendChild(this.img.getHTMLElement());
                    if (this.uploadUrl) {
                        this.progress = new api.ui.ProgressBar();
                        el.appendChild(this.progress.getHTMLElement());
                    }
                    this.onRendered(function (event) {
                        if (!_this.uploader && _this.uploadUrl) {
                            _this.uploader = _this.initUploader(_this.getId());
                        }
                    });
                    this.onRemoved(function (event) {
                        if (_this.uploader) {
                            _this.uploader.destroy();
                        }
                    });
                }
                FormIcon.prototype.setSrc = function (src) {
                    this.img.setSrc(src);
                };
                FormIcon.prototype.initUploader = function (elId) {
                    var _this = this;
                    var uploader = new plupload.Uploader({
                        runtimes: 'gears,html5,flash,silverlight,browserplus',
                        multi_selection: false,
                        browse_button: elId,
                        url: this.uploadUrl,
                        multipart: true,
                        drop_element: elId,
                        flash_swf_url: api.util.UriHelper.getUri('common/js/fileupload/plupload/js/plupload.flash.swf'),
                        silverlight_xap_url: api.util.UriHelper.getUri('common/js/fileupload/plupload/js/plupload.silverlight.xap'),
                        filters: [
                            { title: 'Image files', extensions: 'jpg,gif,png' }
                        ]
                    });
                    uploader.bind('Init', function (up, params) {
                    });
                    uploader.bind('FilesAdded', function (up, files) {
                    });
                    uploader.bind('QueueChanged', function (up) {
                        if (up.files.length > 0) {
                            up.start();
                            _this.notifyUploadStarted();
                        }
                    });
                    uploader.bind('UploadFile', function (up, file) {
                        _this.progress.show();
                    });
                    uploader.bind('UploadProgress', function (up, file) {
                        _this.progress.setValue(file.percent);
                    });
                    uploader.bind('FileUploaded', function (up, file, response) {
                        console.log('uploader file uploaded', up, file, response);
                        var responseObj;
                        if (response && response.status === 200) {
                            responseObj = JSON.parse(response.response);
                            if (responseObj.items && responseObj.items.length > 0) {
                                var uploadedFile = responseObj.items[0];
                                var uploadItem = new api.ui.uploader.UploadItem(file);
                                _this.notifyUploadFinished(uploadItem);
                            }
                        }
                        _this.progress.hide();
                    });
                    uploader.bind('UploadComplete', function (up, files) {
                        up.total.reset();
                        var uploadedFiles = up.splice();
                    });
                    uploader.init();
                    return uploader;
                };
                FormIcon.prototype.onUploadStarted = function (listener) {
                    this.uploadStartedListeners.push(listener);
                };
                FormIcon.prototype.onUploadFinished = function (listener) {
                    this.uploadFinishedListeners.push(listener);
                };
                FormIcon.prototype.unUploadStarted = function (listener) {
                    this.uploadStartedListeners = this.uploadStartedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                FormIcon.prototype.unUploadFinished = function (listener) {
                    this.uploadFinishedListeners = this.uploadFinishedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                FormIcon.prototype.notifyUploadStarted = function () {
                    var _this = this;
                    this.uploadStartedListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                FormIcon.prototype.notifyUploadFinished = function (uploadItem) {
                    var _this = this;
                    this.uploadFinishedListeners.forEach(function (listener) {
                        listener.call(_this, new wizard.UploadFinishedEvent(uploadItem));
                    });
                };
                return FormIcon;
            })(api.dom.ButtonEl);
            wizard.FormIcon = FormIcon;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardHeader = (function (_super) {
                __extends(WizardHeader, _super);
                function WizardHeader() {
                    _super.call(this, "wizard-header");
                    this.propertyChangedListeners = [];
                }
                WizardHeader.prototype.onPropertyChanged = function (listener) {
                    this.propertyChangedListeners.push(listener);
                };
                WizardHeader.prototype.unPropertyChanged = function (listener) {
                    this.propertyChangedListeners = this.propertyChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                WizardHeader.prototype.notifyPropertyChanged = function (property, oldValue, newValue) {
                    var _this = this;
                    var event = new api.PropertyChangedEvent(property, oldValue, newValue);
                    this.propertyChangedListeners.forEach(function (listener) {
                        listener.call(_this, event);
                    });
                };
                WizardHeader.prototype.isValid = function () {
                    return true;
                };
                WizardHeader.prototype.giveFocus = function () {
                    return false;
                };
                return WizardHeader;
            })(api.dom.DivEl);
            wizard.WizardHeader = WizardHeader;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardHeaderWithDisplayNameAndNameBuilder = (function () {
                function WizardHeaderWithDisplayNameAndNameBuilder() {
                }
                WizardHeaderWithDisplayNameAndNameBuilder.prototype.setDisplayNameGenerator = function (value) {
                    this.displayNameGenerator = value;
                    return this;
                };
                WizardHeaderWithDisplayNameAndNameBuilder.prototype.build = function () {
                    return new WizardHeaderWithDisplayNameAndName(this);
                };
                return WizardHeaderWithDisplayNameAndNameBuilder;
            })();
            wizard.WizardHeaderWithDisplayNameAndNameBuilder = WizardHeaderWithDisplayNameAndNameBuilder;
            var WizardHeaderWithDisplayNameAndName = (function (_super) {
                __extends(WizardHeaderWithDisplayNameAndName, _super);
                function WizardHeaderWithDisplayNameAndName(builder) {
                    var _this = this;
                    _super.call(this);
                    this.forbiddenChars = /[\/\\]+/ig;
                    this.autoGenerateName = false;
                    this.autoGenerationEnabled = true;
                    this.ignoreGenerateStatusForName = true;
                    this.addClass("wizard-header-with-display-name-and-name");
                    this.displayNameGenerator = builder.displayNameGenerator;
                    this.displayNameProgrammaticallySet = this.displayNameGenerator != null;
                    this.displayNameEl = api.ui.text.AutosizeTextInput.large().setName('displayName');
                    this.displayNameEl.setPlaceholder("<Display Name>");
                    this.displayNameEl.onValueChanged(function (event) {
                        _this.notifyPropertyChanged("displayName", event.getOldValue(), event.getNewValue());
                    });
                    this.appendChild(this.displayNameEl);
                    this.pathEl = new api.dom.SpanEl('path');
                    this.pathEl.hide();
                    this.appendChild(this.pathEl);
                    this.nameElWrapper = new api.dom.DivEl("name-wrapper");
                    this.nameEl = api.ui.text.AutosizeTextInput.middle().setName('name').setForbiddenCharsRe(this.forbiddenChars);
                    this.nameEl.setPlaceholder("<name>");
                    this.nameEl.onValueChanged(function (event) {
                        _this.notifyPropertyChanged("name", event.getOldValue(), event.getNewValue());
                    });
                    this.nameElWrapper.appendChild(this.nameEl);
                    this.appendChild(this.nameElWrapper);
                    this.displayNameEl.onValueChanged(function (event) {
                        _this.displayNameEl.removeClass("generated");
                        var currentDisplayName = event.getNewValue() || "";
                        if (_this.displayNameGenerator && _this.displayNameGenerator.hasScript()) {
                            var generatedDisplayName = _this.displayNameGenerator.execute() || "";
                            _this.displayNameProgrammaticallySet = generatedDisplayName.toLowerCase() === currentDisplayName.toLowerCase() || generatedDisplayName.trim().toLowerCase() === currentDisplayName.toLowerCase() || api.util.StringHelper.isEmpty(currentDisplayName);
                            if (_this.displayNameProgrammaticallySet) {
                                _this.displayNameEl.addClass("generated");
                            }
                        }
                        _this.doAutoGenerateName(currentDisplayName);
                    });
                    this.nameEl.onValueChanged(function (event) {
                        var currentName = event.getNewValue() || "";
                        var displayName = _this.getDisplayName() || "";
                        _this.autoGenerateName = _this.checkAutoGenerateName(currentName, displayName);
                        _this.updateNameGeneratedStatus();
                    });
                    this.onShown(function (event) { return _this.updatePathAndNameWidth(); });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.updatePathAndNameWidth(); }, this);
                }
                WizardHeaderWithDisplayNameAndName.prototype.checkAutoGenerateName = function (name, displayName) {
                    return api.util.StringHelper.isEmpty(name) || displayName.toLowerCase() === name.toLowerCase() || name.toLowerCase() === this.generateName(displayName).toLowerCase();
                };
                WizardHeaderWithDisplayNameAndName.prototype.initNames = function (displayName, name, forceDisplayNameProgrammaticallySet) {
                    this.autoGenerateName = this.checkAutoGenerateName(name, displayName);
                    this.displayNameEl.setValue(displayName);
                    if (name != null) {
                        this.nameEl.setValue(name);
                    }
                    else {
                        this.nameEl.setValue(this.generateName(displayName));
                    }
                    if (this.displayNameGenerator && this.displayNameGenerator.hasScript()) {
                        if (!forceDisplayNameProgrammaticallySet) {
                            var generatedDisplayName = this.displayNameGenerator.execute();
                            this.displayNameProgrammaticallySet = generatedDisplayName == displayName;
                        }
                        else {
                            this.displayNameProgrammaticallySet = true;
                        }
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.doAutoGenerateName = function (value) {
                    if (this.autoGenerateName && this.autoGenerationEnabled) {
                        this.nameEl.setValue(this.generateName(value));
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.isAutoGenerationEnabled = function () {
                    return this.autoGenerationEnabled;
                };
                WizardHeaderWithDisplayNameAndName.prototype.setAutoGenerationEnabled = function (value) {
                    this.autoGenerationEnabled = value;
                };
                WizardHeaderWithDisplayNameAndName.prototype.setDisplayName = function (value) {
                    if (this.displayNameProgrammaticallySet) {
                        value = value.trim();
                        this.displayNameEl.setValue(value);
                        this.doAutoGenerateName(value);
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.setPath = function (value) {
                    this.pathEl.getEl().setText(value);
                    this.pathEl.getEl().setAttribute('title', value);
                    if (value) {
                        this.pathEl.show();
                    }
                    else {
                        this.pathEl.hide();
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.disableNameInput = function () {
                    this.nameEl.getEl().setAttribute("disabled", "disabled");
                };
                WizardHeaderWithDisplayNameAndName.prototype.getName = function () {
                    return this.nameEl.getValue();
                };
                WizardHeaderWithDisplayNameAndName.prototype.getDisplayName = function () {
                    return this.displayNameEl.getValue();
                };
                WizardHeaderWithDisplayNameAndName.prototype.giveFocus = function () {
                    return this.displayNameEl.giveFocus();
                };
                WizardHeaderWithDisplayNameAndName.prototype.generateName = function (value) {
                    return api.Name.ensureValidName(value);
                };
                WizardHeaderWithDisplayNameAndName.prototype.setIgnoreGenerateStatusForName = function (value) {
                    this.ignoreGenerateStatusForName = value;
                    this.updateNameGeneratedStatus();
                };
                WizardHeaderWithDisplayNameAndName.prototype.disableNameGeneration = function (value) {
                    var _this = this;
                    if (value) {
                        this.setAutoGenerationEnabled(false);
                        this.nameEl.getEl().setDisabled(true);
                        this.nameElWrapper.onClicked(function () {
                            _this.nameEl.getEl().setDisabled(false);
                            _this.nameEl.giveFocus();
                        });
                        this.nameEl.onBlur(function () {
                            _this.nameEl.getEl().setDisabled(true);
                        });
                    }
                    this.setIgnoreGenerateStatusForName(value);
                };
                WizardHeaderWithDisplayNameAndName.prototype.updateNameGeneratedStatus = function () {
                    if (this.autoGenerateName && !this.ignoreGenerateStatusForName) {
                        this.nameEl.addClass("generated");
                    }
                    else {
                        this.nameEl.removeClass("generated");
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.updatePathAndNameWidth = function () {
                    var pathEl = this.pathEl.getEl(), nameEl = this.nameEl.getEl(), headerWidth = this.getEl().getWidth(), pathWidth = pathEl.getWidthWithMargin(), nameWidth = nameEl.getWidthWithMargin(), nameMinWidth = nameEl.getMinWidth();
                    if (pathWidth + nameWidth > headerWidth) {
                        if (nameWidth > nameMinWidth) {
                            nameEl.setWidthPx(Math.max(nameMinWidth, headerWidth - pathWidth));
                        }
                        if (pathWidth + nameMinWidth > headerWidth) {
                            pathEl.setWidthPx(headerWidth - nameMinWidth - pathEl.getMarginLeft() - pathEl.getMarginRight());
                        }
                    }
                    else {
                        var pathClone = new api.dom.SpanEl('path');
                        pathClone.setHtml(pathEl.getInnerHtml());
                        pathClone.getEl().setHeight('0px');
                        pathClone.insertAfterEl(this.pathEl);
                        var pathCloneWidth = pathClone.getEl().getWidth();
                        if (pathEl.getWidth() < pathCloneWidth) {
                            pathEl.setWidthPx(Math.min(pathCloneWidth, headerWidth - nameWidth - pathEl.getMarginLeft() - pathEl.getMarginRight()));
                        }
                        pathClone.remove();
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.isValid = function () {
                    return !!this.displayNameEl.getValue() && !!this.nameEl.getValue();
                };
                return WizardHeaderWithDisplayNameAndName;
            })(wizard.WizardHeader);
            wizard.WizardHeaderWithDisplayNameAndName = WizardHeaderWithDisplayNameAndName;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardHeaderWithName = (function (_super) {
                __extends(WizardHeaderWithName, _super);
                function WizardHeaderWithName() {
                    var _this = this;
                    _super.call(this);
                    this.nameEl = api.ui.text.AutosizeTextInput.large().setName('name').setForbiddenCharsRe(/[^_a-z0-9\-]+/ig);
                    this.nameEl.onValueChanged(function (event) {
                        _this.notifyPropertyChanged("name", event.getOldValue(), event.getNewValue());
                    });
                    this.appendChild(this.nameEl);
                }
                WizardHeaderWithName.prototype.getName = function () {
                    return this.nameEl.getValue();
                };
                WizardHeaderWithName.prototype.setName = function (value) {
                    this.nameEl.setValue(value);
                };
                WizardHeaderWithName.prototype.giveFocus = function () {
                    return this.nameEl.giveFocus();
                };
                return WizardHeaderWithName;
            })(wizard.WizardHeader);
            wizard.WizardHeaderWithName = WizardHeaderWithName;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStepsPanel = (function (_super) {
                __extends(WizardStepsPanel, _super);
                function WizardStepsPanel(navigator, scrollable) {
                    _super.call(this, navigator, scrollable, "wizard-steps-panel");
                }
                return WizardStepsPanel;
            })(api.ui.panel.NavigatedPanelStrip);
            wizard.WizardStepsPanel = WizardStepsPanel;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStepNavigator = (function (_super) {
                __extends(WizardStepNavigator, _super);
                function WizardStepNavigator() {
                    _super.call(this, "wizard-step-navigator");
                }
                WizardStepNavigator.prototype.addNavigationItem = function (step) {
                    _super.prototype.addNavigationItem.call(this, step);
                    if (this.getSize() == 1) {
                        step.addClass("first");
                    }
                };
                WizardStepNavigator.prototype.nextStep = function () {
                    var nextIndex = Math.min(this.getSelectedIndex() + 1, this.getSize() - 1);
                    this.selectNavigationItem(nextIndex);
                };
                WizardStepNavigator.prototype.previousStep = function () {
                    var previousIndex = Math.max(this.getSelectedIndex() - 1, 0);
                    this.selectNavigationItem(previousIndex);
                };
                WizardStepNavigator.prototype.hasNext = function () {
                    return this.getSelectedIndex() < this.getSize() - 1;
                };
                WizardStepNavigator.prototype.hasPrevious = function () {
                    return this.getSelectedIndex() > 0;
                };
                return WizardStepNavigator;
            })(api.ui.tab.TabBar);
            wizard.WizardStepNavigator = WizardStepNavigator;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStep = (function () {
                function WizardStep(label, stepForm) {
                    this.tabBarItem = new api.ui.tab.TabBarItemBuilder().setLabel(label).build();
                    this.stepForm = stepForm;
                }
                WizardStep.prototype.getTabBarItem = function () {
                    return this.tabBarItem;
                };
                WizardStep.prototype.getStepForm = function () {
                    return this.stepForm;
                };
                return WizardStep;
            })();
            wizard.WizardStep = WizardStep;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStepValidityChangedEvent = (function () {
                function WizardStepValidityChangedEvent(isValid) {
                    this.valid = isValid;
                }
                WizardStepValidityChangedEvent.prototype.isValid = function () {
                    return this.valid;
                };
                return WizardStepValidityChangedEvent;
            })();
            wizard.WizardStepValidityChangedEvent = WizardStepValidityChangedEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var ValidationRecording = api.form.ValidationRecording;
            var WizardStepForm = (function (_super) {
                __extends(WizardStepForm, _super);
                function WizardStepForm(className) {
                    _super.call(this, className);
                    this.validityChangedListeners = [];
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.addClass("wizard-step-form");
                }
                WizardStepForm.prototype.validate = function (silent) {
                    return new ValidationRecording();
                };
                WizardStepForm.prototype.isValid = function () {
                    if (!this.previousValidation) {
                        this.previousValidation = this.validate(true);
                    }
                    return this.previousValidation.isValid();
                };
                WizardStepForm.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                WizardStepForm.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                WizardStepForm.prototype.notifyValidityChanged = function (event) {
                    this.validityChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                WizardStepForm.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                WizardStepForm.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                WizardStepForm.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                WizardStepForm.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                WizardStepForm.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                WizardStepForm.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return WizardStepForm;
            })(api.ui.panel.Panel);
            wizard.WizardStepForm = WizardStepForm;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var WizardPanel = (function (_super) {
                __extends(WizardPanel, _super);
                function WizardPanel(params, callback) {
                    var _this = this;
                    _super.call(this, "wizard-panel");
                    this.isChanged = true;
                    this.closedListeners = [];
                    this.splitPanelThreshold = 960;
                    this.validityManager = new wizard.WizardValidityManager();
                    this.tabId = params.tabId;
                    this.persistedItem = params.persistedItem;
                    this.header = params.header;
                    this.mainToolbar = params.mainToolbar;
                    this.stepToolbar = params.stepToolbar;
                    this.actions = params.actions;
                    this.validityManager.setHeader(this.header);
                    this.formPanel = new api.ui.panel.Panel("form-panel");
                    this.formPanel.onScroll(function () { return _this.updateStickyToolbar(); });
                    this.appendChild(this.mainToolbar);
                    if (params.split && params.livePanel) {
                        this.splitPanel = this.createSplitPanel(this.formPanel, params.livePanel);
                        this.appendChild(this.splitPanel);
                    }
                    else {
                        this.appendChild(this.formPanel);
                    }
                    var aboveStepPanels = new api.dom.DivEl();
                    aboveStepPanels.appendChild(params.formIcon);
                    aboveStepPanels.appendChild(this.header);
                    var container = new api.dom.DivEl("test-container");
                    this.stepNavigatorAndToolbarContainer = new api.dom.DivEl("wizard-step-navigator-and-toolbar");
                    this.stepNavigator = new wizard.WizardStepNavigator();
                    if (this.stepToolbar) {
                        this.stepNavigatorAndToolbarContainer.appendChild(this.stepToolbar);
                    }
                    this.stepNavigatorAndToolbarContainer.appendChild(this.stepNavigator);
                    aboveStepPanels.appendChild(this.stepNavigatorAndToolbarContainer);
                    this.stepsPanel = new wizard.WizardStepsPanel(this.stepNavigator, this.formPanel);
                    this.stepNavigatorAndToolbarContainer.onShown(function (event) {
                        _this.stepsPanel.setScrollOffset(event.getElement().getEl().getHeight());
                    });
                    ResponsiveManager.onAvailableSizeChanged(this.stepNavigatorAndToolbarContainer, function (item) {
                        if (_this.isVisible()) {
                            _this.updateStickyToolbar();
                            _this.stepsPanel.setScrollOffset(item.getElement().getEl().getHeight());
                        }
                    });
                    this.formPanel.appendChild(aboveStepPanels).appendChild(this.stepsPanel);
                    this.layingOutNew = this.persistedItem == null;
                    if (this.layingOutNew) {
                        this.preLayoutNew().then(function () { return _this.layoutNew(); }).then(function () { return _this.postLayoutNew(); }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).finally(function () { return callback(); }).done(function () { return _this.validityManager.notifyValidityChanged(_this.isValid()); });
                    }
                    else {
                        this.startLayoutPersistedItem(this.persistedItem).then(function () { return _this.postLayoutPersisted(_this.persistedItem); }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).finally(function () { return callback(); }).done(function () { return _this.validityManager.notifyValidityChanged(_this.isValid()); });
                    }
                    this.onRendered(function (event) {
                        _this.firstShow = true;
                    });
                    this.onShown(function (event) {
                        if (_this.firstShow) {
                            _this.firstShow = false;
                            _this.giveInitialFocus();
                        }
                        if (_this.lastFocusedElement) {
                            _this.lastFocusedElement.focus();
                        }
                    });
                }
                WizardPanel.prototype.updateStickyToolbar = function () {
                    var scrollTop = this.formPanel.getHTMLElement().scrollTop;
                    var wizardHeaderHeight = this.header.getEl().getHeightWithMargin() + this.header.getEl().getOffsetTopRelativeToParent();
                    if (scrollTop > wizardHeaderHeight) {
                        this.mainToolbar.removeClass("scroll-shadow");
                        var stepNavigatorEl = this.stepNavigatorAndToolbarContainer.getEl().addClass("scroll-stick");
                        if (!this.stepNavigatorPlaceholder) {
                            this.stepNavigatorPlaceholder = new api.dom.DivEl('toolbar-placeholder');
                            this.stepNavigatorPlaceholder.insertAfterEl(this.stepNavigatorAndToolbarContainer);
                            this.stepNavigatorPlaceholder.getEl().setWidthPx(stepNavigatorEl.getWidth()).setHeightPx(stepNavigatorEl.getHeight());
                        }
                    }
                    else if (scrollTop < wizardHeaderHeight) {
                        this.mainToolbar.addClass("scroll-shadow");
                        this.stepNavigatorAndToolbarContainer.removeClass("scroll-stick");
                        if (this.stepNavigatorPlaceholder) {
                            this.stepNavigatorPlaceholder.remove();
                            this.stepNavigatorPlaceholder = undefined;
                        }
                    }
                    if (scrollTop == 0) {
                        this.mainToolbar.removeClass("scroll-shadow");
                    }
                    this.stepNavigatorAndToolbarContainer.getEl().setWidthPx(this.formPanel.getEl().getWidth());
                };
                WizardPanel.prototype.giveInitialFocus = function () {
                    this.header.giveFocus();
                    this.startRememberFocus();
                };
                WizardPanel.prototype.startRememberFocus = function () {
                    var _this = this;
                    wemjq(this.getHTMLElement()).on("focus", "*", function (e) {
                        e.stopPropagation();
                        _this.lastFocusedElement = wemjq(e.target);
                    });
                };
                WizardPanel.prototype.getTabId = function () {
                    return this.tabId;
                };
                WizardPanel.prototype.setTabId = function (tabId) {
                    this.tabId = tabId;
                };
                WizardPanel.prototype.getHeader = function () {
                    return this.header;
                };
                WizardPanel.prototype.getIconUrl = function () {
                    return null;
                };
                WizardPanel.prototype.getActions = function () {
                    return this.actions.getActions();
                };
                WizardPanel.prototype.getSteps = function () {
                    return this.steps;
                };
                WizardPanel.prototype.setSteps = function (steps) {
                    var _this = this;
                    steps.forEach(function (step, index) {
                        _this.addStep(step, index === 0);
                    });
                    this.steps = steps;
                };
                WizardPanel.prototype.addStep = function (step, select) {
                    this.stepsPanel.addNavigablePanel(step.getTabBarItem(), step.getStepForm(), step.getTabBarItem().getLabel(), select);
                    this.validityManager.addItem(step);
                };
                WizardPanel.prototype.insertStepBefore = function (stepToInsert, beforeStep) {
                    var indexOfBeforeStep = this.steps.indexOf(beforeStep);
                    this.steps.splice(indexOfBeforeStep, 0, stepToInsert);
                    this.stepsPanel.insertNavigablePanel(stepToInsert.getTabBarItem(), stepToInsert.getStepForm(), stepToInsert.getTabBarItem().getLabel(), indexOfBeforeStep);
                    this.validityManager.addItem(stepToInsert);
                };
                WizardPanel.prototype.removeStepWithForm = function (form) {
                    var _this = this;
                    this.steps = this.steps.filter(function (step) {
                        var remove = (step.getStepForm() == form);
                        if (remove) {
                            _this.validityManager.removeItem(step);
                        }
                        return !remove;
                    });
                    return this.stepsPanel.removeNavigablePanel(form);
                };
                WizardPanel.prototype.preLayoutNew = function () {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.isLayingOutNew = function () {
                    return this.layingOutNew;
                };
                WizardPanel.prototype.layoutNew = function () {
                    var deferred = wemQ.defer();
                    this.actions.enableActionsForNew();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.postLayoutNew = function () {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.startLayoutPersistedItem = function (persistedItem) {
                    this.persistedItem = persistedItem;
                    this.actions.enableActionsForExisting(persistedItem);
                    return this.layoutPersistedItem(persistedItem);
                };
                WizardPanel.prototype.layoutPersistedItem = function (persistedItem) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.postLayoutPersisted = function (existing) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.getPersistedItem = function () {
                    return this.persistedItem;
                };
                WizardPanel.prototype.isItemPersisted = function () {
                    return this.persistedItem != null;
                };
                WizardPanel.prototype.setPersistedItem = function (newPersistedItem) {
                    this.persistedItem = newPersistedItem;
                };
                WizardPanel.prototype.hasUnsavedChanges = function () {
                    return this.isChanged;
                };
                WizardPanel.prototype.askUserForSaveChangesBeforeClosing = function () {
                    new wizard.SaveBeforeCloseDialog(this).open();
                };
                WizardPanel.prototype.saveChanges = function () {
                    var _this = this;
                    if (this.isItemPersisted()) {
                        return this.updatePersistedItem().then(function (persistedItem) {
                            _this.isChanged = false;
                            return _this.startLayoutPersistedItem(persistedItem).then(function () { return persistedItem; });
                        });
                    }
                    else {
                        return this.persistNewItem().then(function (persistedItem) {
                            _this.isChanged = false;
                            return _this.postPersistNewItem(persistedItem).then(function () { return _this.startLayoutPersistedItem(persistedItem); }).then(function () { return persistedItem; });
                        });
                    }
                };
                WizardPanel.prototype.persistNewItem = function () {
                    throw new Error("Must be overriden by inheritor");
                };
                WizardPanel.prototype.postPersistNewItem = function (persistedItem) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.updatePersistedItem = function () {
                    throw new Error("Must be overriden by inheritor");
                };
                WizardPanel.prototype.close = function (checkCanClose) {
                    if (checkCanClose === void 0) { checkCanClose = false; }
                    if (!checkCanClose || this.canClose()) {
                        this.notifyClosed();
                    }
                };
                WizardPanel.prototype.canClose = function () {
                    if (this.hasUnsavedChanges()) {
                        this.askUserForSaveChangesBeforeClosing();
                        return false;
                    }
                    else {
                        return true;
                    }
                };
                WizardPanel.prototype.onClosed = function (listener) {
                    this.closedListeners.push(listener);
                };
                WizardPanel.prototype.unClosed = function (listener) {
                    this.closedListeners = this.closedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                WizardPanel.prototype.getSplitPanel = function () {
                    return this.splitPanel;
                };
                WizardPanel.prototype.createSplitPanel = function (firstPanel, secondPanel) {
                    var splitPanel = new api.ui.panel.SplitPanelBuilder(firstPanel, secondPanel).setFirstPanelMinSize(280, 0 /* PIXEL */).setAlignment(1 /* VERTICAL */).build();
                    this.updateSplitPanel(splitPanel);
                    return splitPanel;
                };
                WizardPanel.prototype.updateSplitPanel = function (splitPanel) {
                    if (wemjq(window).width() > this.splitPanelThreshold) {
                        splitPanel.setFirstPanelSize(38, 1 /* PERCENT */);
                    }
                    splitPanel.distribute();
                };
                WizardPanel.prototype.notifyClosed = function () {
                    var _this = this;
                    this.closedListeners.forEach(function (listener) {
                        listener(new wizard.WizardClosedEvent(_this));
                    });
                };
                WizardPanel.prototype.onValidityChanged = function (listener) {
                    this.validityManager.onValidityChanged(listener);
                };
                WizardPanel.prototype.unValidityChanged = function (listener) {
                    this.validityManager.onValidityChanged(listener);
                };
                WizardPanel.prototype.notifyValidityChanged = function (valid) {
                    this.validityManager.notifyValidityChanged(valid);
                };
                WizardPanel.prototype.isValid = function () {
                    return this.validityManager.isAllValid();
                };
                return WizardPanel;
            })(api.ui.panel.Panel);
            wizard.WizardPanel = WizardPanel;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var SaveBeforeCloseDialog = (function (_super) {
                __extends(SaveBeforeCloseDialog, _super);
                function SaveBeforeCloseDialog(wizardPanel) {
                    var _this = this;
                    _super.call(this, {
                        idPrefix: "SaveBeforeCloseDialog",
                        title: new api.ui.dialog.ModalDialogHeader("Close wizard")
                    });
                    this.yesAction = new api.ui.Action('Yes');
                    this.noAction = new api.ui.Action('No');
                    this.wizardPanel = wizardPanel;
                    var message = new api.dom.H6El();
                    message.getEl().setInnerHtml("There are unsaved changes, do you want to save them before closing?");
                    this.appendChildToContentPanel(message);
                    this.yesAction.setMnemonic("y");
                    this.yesAction.onExecuted(function () {
                        _this.doSaveAndClose();
                    });
                    this.addAction(this.yesAction, true);
                    this.noAction.setMnemonic("n");
                    this.noAction.onExecuted(function () {
                        _this.doCloseWithoutSaveCheck();
                    });
                    this.addAction(this.noAction);
                    this.getCancelAction().setMnemonic("c");
                    this.getCancelAction().onExecuted(function () {
                        _this.close();
                    });
                }
                SaveBeforeCloseDialog.prototype.show = function () {
                    api.dom.Body.get().appendChild(this);
                    _super.prototype.show.call(this);
                };
                SaveBeforeCloseDialog.prototype.close = function () {
                    this.remove();
                    _super.prototype.close.call(this);
                };
                SaveBeforeCloseDialog.prototype.doSaveAndClose = function () {
                    var _this = this;
                    this.close();
                    this.wizardPanel.saveChanges().then(function () { return _this.wizardPanel.close(true); }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                };
                SaveBeforeCloseDialog.prototype.doCloseWithoutSaveCheck = function () {
                    this.close();
                    this.wizardPanel.close();
                };
                return SaveBeforeCloseDialog;
            })(api.ui.dialog.ModalDialog);
            wizard.SaveBeforeCloseDialog = SaveBeforeCloseDialog;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var UploadFinishedEvent = (function () {
                function UploadFinishedEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                UploadFinishedEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return UploadFinishedEvent;
            })();
            wizard.UploadFinishedEvent = UploadFinishedEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (_wizard) {
            var WizardClosedEvent = (function () {
                function WizardClosedEvent(wizard) {
                    this.wizard = wizard;
                }
                WizardClosedEvent.prototype.getWizard = function () {
                    return this.wizard;
                };
                return WizardClosedEvent;
            })();
            _wizard.WizardClosedEvent = WizardClosedEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardValidityManager = (function () {
                function WizardValidityManager() {
                    this.validityChangedListeners = [];
                    this.steps = [];
                    this.validityChangedListeners = [];
                }
                WizardValidityManager.prototype.clearItems = function () {
                    this.steps = [];
                };
                WizardValidityManager.prototype.getSteps = function () {
                    return this.steps;
                };
                WizardValidityManager.prototype.addItem = function (step) {
                    var _this = this;
                    this.steps.push(step);
                    step.getStepForm().onValidityChanged(function (event) {
                        _this.notifyValidityChanged(event.isValid());
                    });
                };
                WizardValidityManager.prototype.removeItem = function (step) {
                    var index = this.steps.indexOf(step);
                    if (index >= 0) {
                        this.steps.splice(index, 1);
                    }
                };
                WizardValidityManager.prototype.setHeader = function (header) {
                    var _this = this;
                    this.header = header;
                    this.header.onPropertyChanged(function (event) {
                        _this.notifyValidityChanged(_this.header.isValid());
                    });
                };
                WizardValidityManager.prototype.isAllValid = function () {
                    if (this.header && !this.header.isValid()) {
                        return false;
                    }
                    for (var i = 0; i < this.steps.length; i++) {
                        if (!this.steps[i].getStepForm().isValid()) {
                            return false;
                        }
                    }
                    return true;
                };
                WizardValidityManager.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                WizardValidityManager.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                WizardValidityManager.prototype.notifyValidityChanged = function (valid) {
                    var _this = this;
                    this.validityChangedListeners.forEach(function (listener) {
                        listener.call(_this, new api.ValidityChangedEvent(valid));
                    });
                };
                return WizardValidityManager;
            })();
            wizard.WizardValidityManager = WizardValidityManager;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var system;
    (function (system) {
        var StatusRequest = (function (_super) {
            __extends(StatusRequest, _super);
            function StatusRequest() {
                _super.apply(this, arguments);
            }
            StatusRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "status");
            };
            StatusRequest.prototype.getParams = function () {
                return {};
            };
            StatusRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return new system.StatusResult(response.getResult());
                });
            };
            return StatusRequest;
        })(api.rest.ResourceRequest);
        system.StatusRequest = StatusRequest;
    })(system = api.system || (api.system = {}));
})(api || (api = {}));
var api;
(function (api) {
    var system;
    (function (system) {
        var StatusResult = (function () {
            function StatusResult(json) {
                this.version = json.version;
                this.installation = json.installation;
                if (json.context) {
                    this.authenticated = json.context.authenticated;
                    this.principals = json.context.principals ? json.context.principals.map(function (principal) { return api.security.PrincipalKey.fromString(principal); }) : [];
                }
                else {
                    this.authenticated = false;
                    this.principals = [];
                }
            }
            StatusResult.prototype.isAuthenticated = function () {
                return this.authenticated;
            };
            StatusResult.prototype.getInstallation = function () {
                return this.installation;
            };
            StatusResult.prototype.getVersion = function () {
                return this.version;
            };
            StatusResult.prototype.getPrincipals = function () {
                return this.principals;
            };
            return StatusResult;
        })();
        system.StatusResult = StatusResult;
    })(system = api.system || (api.system = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PropertyTree = api.data.PropertyTree;
        var GetPageDescriptorByKeyRequest = api.content.page.GetPageDescriptorByKeyRequest;
        var SetTemplate = api.content.page.SetTemplate;
        var PageModel = api.content.page.PageModel;
        var SetController = api.content.page.SetController;
        var Regions = api.content.page.region.Regions;
        var PageMode = api.content.page.PageMode;
        var GetPageTemplateByKeyRequest = api.content.page.GetPageTemplateByKeyRequest;
        var LiveEditModel = (function () {
            function LiveEditModel(builder) {
                this.siteModel = builder.siteModel;
                this.parentContent = builder.parentContent;
                this.content = builder.content;
                this.formContext = builder.formContext;
            }
            LiveEditModel.prototype.init = function (defaultTemplate, defaultTemplateDescriptor) {
                var _this = this;
                return this.initPageModel(defaultTemplate, defaultTemplateDescriptor).then(function (pageModel) {
                    _this.pageModel = pageModel;
                });
            };
            LiveEditModel.prototype.initPageModel = function (defaultPageTemplate, defaultTemplateDescriptor) {
                var _this = this;
                var deferred = wemQ.defer();
                var pageModel = new PageModel(this, defaultPageTemplate, defaultTemplateDescriptor);
                var pageMode = this.content.getPageMode();
                var pageDescriptorPromise = null;
                var pageTemplatePromise = null;
                if (this.content.isPageTemplate()) {
                    var pageTemplate = this.content;
                    if (pageMode == 2 /* FORCED_CONTROLLER */) {
                        var pageDescriptorKey = pageTemplate.getController();
                        pageDescriptorPromise = this.loadPageDescriptor(pageDescriptorKey);
                        pageDescriptorPromise.then(function (pageDescriptor) {
                            var config = pageTemplate.hasConfig() ? pageTemplate.getPage().getConfig().copy() : new PropertyTree(api.Client.get().getPropertyIdProvider());
                            var regions = pageTemplate.hasRegions() ? pageTemplate.getRegions().clone() : Regions.create().build();
                            var setController = new SetController(_this).setDescriptor(pageDescriptor).setConfig(config).setRegions(regions);
                            pageModel.setController(setController);
                        });
                    }
                    else if (pageMode == 3 /* NO_CONTROLLER */) {
                        var config = pageTemplate.hasConfig() ? pageTemplate.getConfig().copy() : new PropertyTree(api.Client.get().getPropertyIdProvider());
                        var regions = pageTemplate.hasRegions() ? pageTemplate.getRegions().clone() : Regions.create().build();
                        var setController = new SetController(this).setDescriptor(null).setConfig(config).setRegions(regions);
                        pageModel.setController(setController);
                    }
                    else {
                        throw new Error("Unsupported PageMode for a PageTemplate: " + pageMode);
                    }
                }
                else {
                    if (pageMode == 1 /* FORCED_TEMPLATE */) {
                        var pageTemplateKey = this.content.getPage().getTemplate();
                        pageTemplatePromise = this.loadPageTemplate(pageTemplateKey);
                        pageTemplatePromise.then(function (pageTemplate) {
                            var pageDescriptorKey = pageTemplate.getController();
                            pageDescriptorPromise = _this.loadPageDescriptor(pageDescriptorKey);
                            pageDescriptorPromise.then(function (pageDescriptor) {
                                var config = _this.content.getPage().hasConfig() ? _this.content.getPage().getConfig().copy() : pageTemplate.getConfig().copy();
                                var regions = _this.content.getPage().hasRegions() ? _this.content.getPage().getRegions().clone() : pageTemplate.getRegions().clone();
                                var setTemplate = new SetTemplate(_this).setTemplate(pageTemplate, pageDescriptor).setRegions(regions).setConfig(config);
                                pageModel.setTemplate(setTemplate);
                            });
                        });
                    }
                    else if (pageMode == 0 /* AUTOMATIC */) {
                        pageModel.setAutomaticTemplate(this);
                    }
                    else {
                        throw new Error("Unsupported PageMode for a Content: " + pageMode);
                    }
                }
                var promises = [];
                if (pageDescriptorPromise) {
                    promises.push(pageDescriptorPromise);
                }
                if (pageTemplatePromise) {
                    promises.push(pageTemplatePromise);
                }
                if (promises.length > 0) {
                    wemQ.all(promises).then(function () {
                        deferred.resolve(pageModel);
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                }
                else {
                    deferred.resolve(pageModel);
                }
                return deferred.promise;
            };
            LiveEditModel.prototype.loadPageTemplate = function (key) {
                return new GetPageTemplateByKeyRequest(key).sendAndParse();
            };
            LiveEditModel.prototype.loadPageDescriptor = function (key) {
                return new GetPageDescriptorByKeyRequest(key).sendAndParse();
            };
            LiveEditModel.prototype.setContent = function (value) {
                this.content = value;
            };
            LiveEditModel.prototype.getParentContent = function () {
                return this.parentContent;
            };
            LiveEditModel.prototype.getFormContext = function () {
                return this.formContext;
            };
            LiveEditModel.prototype.getContent = function () {
                return this.content;
            };
            LiveEditModel.prototype.getSiteModel = function () {
                return this.siteModel;
            };
            LiveEditModel.prototype.getPageModel = function () {
                return this.pageModel;
            };
            LiveEditModel.create = function () {
                return new LiveEditModelBuilder();
            };
            return LiveEditModel;
        })();
        liveedit.LiveEditModel = LiveEditModel;
        var LiveEditModelBuilder = (function () {
            function LiveEditModelBuilder() {
            }
            LiveEditModelBuilder.prototype.setSiteModel = function (value) {
                this.siteModel = value;
                return this;
            };
            LiveEditModelBuilder.prototype.setParentContent = function (value) {
                this.parentContent = value;
                return this;
            };
            LiveEditModelBuilder.prototype.setContent = function (value) {
                this.content = value;
                return this;
            };
            LiveEditModelBuilder.prototype.setContentFormContext = function (value) {
                this.formContext = value;
                return this;
            };
            LiveEditModelBuilder.prototype.build = function () {
                return new LiveEditModel(this);
            };
            return LiveEditModelBuilder;
        })();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemType = (function () {
            function ItemType(shortName, config) {
                ItemType.shortNameToInstance[shortName] = this;
                this.shortName = shortName;
                this.config = new liveedit.ItemTypeConfig(config);
            }
            ItemType.prototype.getShortName = function () {
                return this.shortName;
            };
            ItemType.prototype.getConfig = function () {
                return this.config;
            };
            ItemType.prototype.isComponentType = function () {
                return false;
            };
            ItemType.prototype.toComponentType = function () {
                api.util.assert(this.isComponentType(), "Not support when ItemType is not a ComponentType");
                return api.content.page.region.ComponentType.byShortName(this.shortName);
            };
            ItemType.prototype.createView = function (config) {
                throw new Error("Must be implemented by inheritors");
            };
            ItemType.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ItemType)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.shortName, other.shortName)) {
                    return false;
                }
                return true;
            };
            ItemType.getDraggables = function () {
                var draggables = [];
                for (var shortName in ItemType.shortNameToInstance) {
                    var itemType = ItemType.shortNameToInstance[shortName];
                    if (itemType.getConfig().isDraggable()) {
                        draggables.push(itemType);
                    }
                }
                return draggables;
            };
            ItemType.byShortName = function (shortName) {
                return ItemType.shortNameToInstance[shortName];
            };
            ItemType.fromHTMLElement = function (element) {
                var typeAsString = element.getAttribute("data-" + ItemType.ATTRIBUTE_TYPE);
                return ItemType.byShortName(typeAsString);
            };
            ItemType.fromElement = function (element) {
                var typeAsString = element.getEl().getAttribute("data-" + ItemType.ATTRIBUTE_TYPE);
                return ItemType.byShortName(typeAsString);
            };
            ItemType.ATTRIBUTE_TYPE = "portal-component-type";
            ItemType.shortNameToInstance = {};
            return ItemType;
        })();
        liveedit.ItemType = ItemType;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var CreateItemViewConfig = (function () {
            function CreateItemViewConfig() {
                this.positionIndex = -1;
            }
            CreateItemViewConfig.prototype.setItemViewProducer = function (value) {
                this.itemViewProducer = value;
                return this;
            };
            CreateItemViewConfig.prototype.setParentView = function (value) {
                this.parentView = value;
                return this;
            };
            CreateItemViewConfig.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            CreateItemViewConfig.prototype.setData = function (value) {
                this.data = value;
                return this;
            };
            CreateItemViewConfig.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            CreateItemViewConfig.prototype.setPositionIndex = function (value) {
                this.positionIndex = value;
                return this;
            };
            return CreateItemViewConfig;
        })();
        liveedit.CreateItemViewConfig = CreateItemViewConfig;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemTypeConfig = (function () {
            function ItemTypeConfig(json) {
                this.typeName = json.typeName;
                this.draggable = json.draggable;
                this.cssSelector = json.cssSelector;
                this.iconCls = json.iconCls;
                this.cursor = json.cursor;
                this.highlighterStyle = json.highlighterStyle;
                this.contextMenuConfig = json.contextMenuConfig;
            }
            ItemTypeConfig.prototype.getName = function () {
                return this.typeName;
            };
            ItemTypeConfig.prototype.isDraggable = function () {
                return this.draggable;
            };
            ItemTypeConfig.prototype.getCssSelector = function () {
                return this.cssSelector;
            };
            ItemTypeConfig.prototype.getIconCls = function () {
                return this.iconCls;
            };
            ItemTypeConfig.prototype.getCursor = function () {
                return this.cursor;
            };
            ItemTypeConfig.prototype.getHighlighterStyle = function () {
                return this.highlighterStyle;
            };
            ItemTypeConfig.prototype.getContextMenuConfig = function () {
                return this.contextMenuConfig;
            };
            return ItemTypeConfig;
        })();
        liveedit.ItemTypeConfig = ItemTypeConfig;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionItemType = (function (_super) {
            __extends(RegionItemType, _super);
            function RegionItemType() {
                _super.call(this, "region", {
                    cssSelector: '[data-portal-component-type=region]',
                    draggable: false,
                    cursor: 'pointer',
                    iconCls: 'live-edit-font-icon-region',
                    highlighterStyle: {
                        stroke: 'rgba(20, 20, 20, 1)',
                        strokeDasharray: '',
                        fill: 'rgba(255, 255, 255, 0)'
                    },
                    contextMenuConfig: ['parent', 'clearRegion']
                });
            }
            RegionItemType.get = function () {
                return RegionItemType.INSTANCE;
            };
            RegionItemType.prototype.createView = function (config) {
                return new liveedit.RegionView(new liveedit.RegionViewBuilder().setParentView(config.parentView).setParentElement(config.parentElement).setRegion(config.data).setElement(config.element));
            };
            RegionItemType.INSTANCE = new RegionItemType();
            return RegionItemType;
        })(liveedit.ItemType);
        liveedit.RegionItemType = RegionItemType;
        RegionItemType.get();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var InitializeLiveEditEvent = (function (_super) {
            __extends(InitializeLiveEditEvent, _super);
            function InitializeLiveEditEvent(liveEditModel) {
                _super.call(this);
                this.liveEditModel = liveEditModel;
            }
            InitializeLiveEditEvent.prototype.getLiveEditModel = function () {
                return this.liveEditModel;
            };
            InitializeLiveEditEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            InitializeLiveEditEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return InitializeLiveEditEvent;
        })(api.event.Event);
        liveedit.InitializeLiveEditEvent = InitializeLiveEditEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentItemType = (function (_super) {
            __extends(ComponentItemType, _super);
            function ComponentItemType(shortName, config) {
                _super.call(this, shortName, config);
            }
            ComponentItemType.prototype.createView = function (config) {
                throw new Error("Must be implemented by inheritors");
            };
            return ComponentItemType;
        })(liveedit.ItemType);
        liveedit.ComponentItemType = ComponentItemType;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageItemType = (function (_super) {
            __extends(PageItemType, _super);
            function PageItemType() {
                _super.call(this, "page", {
                    cssSelector: '[data-portal-component-type=page]',
                    draggable: false,
                    cursor: 'pointer',
                    iconCls: 'live-edit-font-icon-page',
                    highlighterStyle: {
                        stroke: 'rgba(20, 20, 20, 1)',
                        strokeDasharray: '',
                        fill: 'rgba(255, 255, 255, 0)'
                    },
                    contextMenuConfig: ['reset']
                });
            }
            PageItemType.get = function () {
                return PageItemType.INSTANCE;
            };
            PageItemType.prototype.createView = function (config) {
                throw new Error("Not supported");
            };
            PageItemType.INSTANCE = new PageItemType();
            return PageItemType;
        })(liveedit.ItemType);
        liveedit.PageItemType = PageItemType;
        PageItemType.get();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ContentItemType = (function (_super) {
            __extends(ContentItemType, _super);
            function ContentItemType() {
                _super.call(this, "content", {
                    cssSelector: '[data-portal-component-type=content]',
                    draggable: false,
                    cursor: 'pointer',
                    iconCls: 'live-edit-font-icon-content',
                    highlighterStyle: {
                        stroke: '',
                        strokeDasharray: '',
                        fill: 'rgba(0, 108, 255, .25)'
                    },
                    contextMenuConfig: ['parent', 'opencontent', 'view']
                });
            }
            ContentItemType.get = function () {
                return ContentItemType.INSTANCE;
            };
            ContentItemType.prototype.createView = function (config) {
                return new liveedit.ContentView(new liveedit.ContentViewBuilder().setParentPartComponentView(config.parentView).setParentElement(config.parentElement).setElement(config.element));
            };
            ContentItemType.INSTANCE = new ContentItemType();
            return ContentItemType;
        })(liveedit.ItemType);
        liveedit.ContentItemType = ContentItemType;
        ContentItemType.get();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var DivEl = api.dom.DivEl;
        var Body = api.dom.Body;
        var Shader = (function () {
            function Shader() {
                var _this = this;
                this.clickListeners = [];
                this.unlockClickedListeners = [];
                this.mouseEnterListeners = [];
                this.mouseLeaveListeners = [];
                this.pageShader = new DivEl(Shader.CLS_NAME + " page");
                var centeredContainer = new api.dom.DivEl('centered');
                var unlockLink = new api.dom.AEl();
                unlockLink.setHtml('Unlock').onClicked(function (event) { return _this.handleUnlockClick(event); });
                centeredContainer.appendChild(unlockLink);
                this.pageShader.appendChild(centeredContainer);
                this.northShader = new DivEl(Shader.CLS_NAME + " north");
                this.eastShader = new DivEl(Shader.CLS_NAME + " east");
                this.southShader = new DivEl(Shader.CLS_NAME + " south");
                this.westShader = new DivEl(Shader.CLS_NAME + " west");
                this.shaders = [this.pageShader, this.northShader, this.eastShader, this.southShader, this.westShader];
                var body = Body.get();
                body.appendChildren.apply(body, this.shaders);
                this.shaders.forEach(function (shader) {
                    shader.onClicked(function (event) { return _this.handleClick(event); });
                    shader.onContextMenu(function (event) { return _this.handleClick(event); });
                    shader.onMouseEnter(function (event) { return _this.notifyMouseEntered(event); });
                    shader.onMouseLeave(function (event) { return _this.notifyMouseLeft(event); });
                });
            }
            Shader.get = function () {
                if (!Shader.INSTANCE) {
                    Shader.INSTANCE = new Shader();
                }
                return Shader.INSTANCE;
            };
            Shader.prototype.shadeItemView = function (itemView) {
                if (!itemView) {
                    this.hide();
                    return;
                }
                if (itemView.getType().equals(api.liveedit.PageItemType.get())) {
                    this.resizeToPage();
                }
                else {
                    this.resizeToItemView(itemView);
                }
            };
            Shader.prototype.hide = function () {
                this.shaders.forEach(function (shader) { return shader.hide(); });
            };
            Shader.prototype.onUnlockClicked = function (listener) {
                this.unlockClickedListeners.push(listener);
            };
            Shader.prototype.unUnlockClicked = function (listener) {
                this.unlockClickedListeners = this.unlockClickedListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyUnlockClicked = function (event) {
                this.unlockClickedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.onMouseEnter = function (listener) {
                this.mouseEnterListeners.push(listener);
            };
            Shader.prototype.unMouseEnter = function (listener) {
                this.mouseEnterListeners = this.mouseEnterListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyMouseEntered = function (event) {
                this.mouseEnterListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.onMouseLeave = function (listener) {
                this.mouseLeaveListeners.push(listener);
            };
            Shader.prototype.unMouseLeave = function (listener) {
                this.mouseLeaveListeners = this.mouseLeaveListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyMouseLeft = function (event) {
                this.mouseLeaveListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.onClicked = function (listener) {
                this.clickListeners.push(listener);
            };
            Shader.prototype.unClicked = function (listener) {
                this.clickListeners = this.clickListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyClicked = function (event) {
                this.clickListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.handleUnlockClick = function (event) {
                event.stopPropagation();
                event.preventDefault();
                this.notifyUnlockClicked(event);
            };
            Shader.prototype.handleClick = function (event) {
                event.stopPropagation();
                event.preventDefault();
                this.notifyClicked(event);
            };
            Shader.prototype.resizeToPage = function () {
                this.pageShader.getEl().setTopPx(0).setRightPx(0).setBottomPx(0).setLeftPx(0);
                this.pageShader.show();
            };
            Shader.prototype.resizeToItemView = function (itemView) {
                var win = api.dom.WindowDOM.get(), bodyEl = api.dom.Body.get().getEl(), documentWidth = Math.max(win.getWidth(), bodyEl.getWidth()), documentHeight = Math.max(win.getHeight(), bodyEl.getHeight());
                var dimensions = itemView.getElementDimensions(), x = dimensions.left, y = dimensions.top, w = dimensions.width, h = dimensions.height;
                this.northShader.getEl().setTopPx(0).setLeftPx(0).setWidthPx(documentWidth).setHeightPx(y);
                this.northShader.show();
                this.eastShader.getEl().setTopPx(y).setLeftPx(x + w).setWidthPx(documentWidth - (x + w)).setHeightPx(h);
                this.eastShader.show();
                this.southShader.getEl().setTopPx(y + h).setLeftPx(0).setWidthPx(documentWidth).setHeightPx(documentHeight - (y + h));
                this.southShader.show();
                this.westShader.getEl().setTopPx(y).setLeftPx(0).setWidthPx(x).setHeightPx(h);
                this.westShader.show();
            };
            Shader.CLS_NAME = 'shader';
            return Shader;
        })();
        liveedit.Shader = Shader;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Highlighter = (function (_super) {
            __extends(Highlighter, _super);
            function Highlighter() {
                var html = '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" class="live-edit-highlight-border" style="top:-5000px;left:-5000px">' + '    <rect width="150" height="150"/>' + '</svg>';
                _super.call(this, new api.dom.ElementFromHelperBuilder().setHelper(api.dom.Element.fromString(html).getEl()).setLoadExistingChildren(true));
                this.rectangle = this.getChildren()[0];
                this.appendChild(this.rectangle);
                api.dom.Body.get().appendChild(this);
            }
            Highlighter.get = function () {
                if (!Highlighter.INSTANCE) {
                    Highlighter.INSTANCE = new Highlighter();
                }
                return Highlighter.INSTANCE;
            };
            Highlighter.prototype.highlightItemView = function (itemView) {
                if (!itemView) {
                    this.hide();
                    return;
                }
                this.resizeToComponent(itemView);
                this.show();
            };
            Highlighter.prototype.resizeToComponent = function (itemView) {
                var itemDimensions = itemView.getElementDimensions();
                var w = Math.round(itemDimensions.width), h = Math.round(itemDimensions.height), top = Math.round(itemDimensions.top), left = Math.round(itemDimensions.left);
                this.getEl().setWidthPx(w).setHeightPx(h).setTopPx(top).setLeftPx(left);
                this.rectangle.getEl().setAttribute('width', w + '').setAttribute('height', h + '');
                var style = itemView.getType().getConfig().getHighlighterStyle();
                this.getEl().setStroke(style.stroke).setStrokeDasharray(style.strokeDasharray).setFill(style.fill);
            };
            return Highlighter;
        })(api.dom.Element);
        liveedit.Highlighter = Highlighter;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Body = api.dom.Body;
        var Cursor = (function () {
            function Cursor() {
                this.defaultBodyCursor = Body.get().getEl().getCursor();
            }
            Cursor.get = function () {
                if (!Cursor.INSTANCE) {
                    Cursor.INSTANCE = new Cursor();
                }
                return Cursor.INSTANCE;
            };
            Cursor.prototype.displayItemViewCursor = function (itemView) {
                if (!itemView) {
                    return;
                }
                Body.get().getEl().setCursor(itemView.getType().getConfig().getCursor());
            };
            Cursor.prototype.hide = function () {
                Body.get().getEl().setCursor('none');
            };
            Cursor.prototype.reset = function () {
                Body.get().getEl().setCursor(this.defaultBodyCursor || '');
            };
            return Cursor;
        })();
        liveedit.Cursor = Cursor;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewId = (function () {
            function ItemViewId(value) {
                api.util.assert(value >= 1, "An ItemViewId must be 1 or larger");
                this.value = value;
                this.refString = "" + value;
            }
            ItemViewId.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ItemViewId)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            ItemViewId.prototype.toNumber = function () {
                return this.value;
            };
            ItemViewId.prototype.toString = function () {
                return this.refString;
            };
            ItemViewId.fromString = function (s) {
                return new ItemViewId(+s);
            };
            ItemViewId.DATA_ATTRIBUTE = "live-edit-id";
            return ItemViewId;
        })();
        liveedit.ItemViewId = ItemViewId;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewIdProducer = (function () {
            function ItemViewIdProducer() {
                this.itemViewCounter = 0;
            }
            ItemViewIdProducer.prototype.next = function () {
                return new liveedit.ItemViewId(++this.itemViewCounter);
            };
            return ItemViewIdProducer;
        })();
        liveedit.ItemViewIdProducer = ItemViewIdProducer;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RepeatNextItemViewIdProducer = (function (_super) {
            __extends(RepeatNextItemViewIdProducer, _super);
            function RepeatNextItemViewIdProducer(idToRepeatNext, itemViewProducer) {
                _super.call(this);
                this.idToRepeatNext = idToRepeatNext;
                this.itemViewProducer = itemViewProducer;
                this.repeated = false;
            }
            RepeatNextItemViewIdProducer.prototype.next = function () {
                if (!this.repeated) {
                    this.repeated = true;
                    return this.idToRepeatNext;
                }
                else {
                    return this.itemViewProducer.next();
                }
            };
            return RepeatNextItemViewIdProducer;
        })(liveedit.ItemViewIdProducer);
        liveedit.RepeatNextItemViewIdProducer = RepeatNextItemViewIdProducer;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewBuilder = (function () {
            function ItemViewBuilder() {
            }
            ItemViewBuilder.prototype.setLiveEditModel = function (value) {
                this.liveEditModel = value;
                return this;
            };
            ItemViewBuilder.prototype.setItemViewIdProducer = function (value) {
                this.itemViewIdProducer = value;
                return this;
            };
            ItemViewBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            ItemViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            ItemViewBuilder.prototype.setPlaceholder = function (value) {
                this.placeholder = value;
                return this;
            };
            ItemViewBuilder.prototype.setTooltipViewer = function (value) {
                this.tooltipViewer = value;
                return this;
            };
            ItemViewBuilder.prototype.setParentView = function (value) {
                this.parentView = value;
                return this;
            };
            ItemViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            ItemViewBuilder.prototype.setContextMenuActions = function (actions) {
                this.contextMenuActions = actions;
                return this;
            };
            ItemViewBuilder.prototype.setContextMenuTitle = function (title) {
                this.contextMenuTitle = title;
                return this;
            };
            return ItemViewBuilder;
        })();
        liveedit.ItemViewBuilder = ItemViewBuilder;
        var ItemView = (function (_super) {
            __extends(ItemView, _super);
            function ItemView(builder) {
                api.util.assertNotNull(builder.type, "type cannot be null");
                this.type = builder.type;
                this.parentItemView = builder.parentView;
                this.liveEditModel = builder.liveEditModel ? builder.liveEditModel : builder.parentView.liveEditModel;
                this.itemViewIdProducer = builder.itemViewIdProducer;
                this.contextMenuActions = builder.contextMenuActions;
                this.contextMenuTitle = builder.contextMenuTitle;
                var props = null;
                if (builder.element) {
                    var elementFromElementBuilder = new api.dom.ElementFromElementBuilder();
                    elementFromElementBuilder.setElement(builder.element);
                    elementFromElementBuilder.setParentElement(builder.parentElement);
                    elementFromElementBuilder.setGenerateId(false);
                    props = elementFromElementBuilder;
                }
                else {
                    var newElementBuilder = new api.dom.NewElementBuilder();
                    newElementBuilder.setTagName("div");
                    newElementBuilder.setParentElement(builder.parentElement);
                    newElementBuilder.setGenerateId(false);
                    props = newElementBuilder;
                }
                _super.call(this, props);
                this.addClass("item-view");
                this.setDraggable(true);
                this.mouseOver = false;
                this.mouseOverViewListeners = [];
                this.mouseOutViewListeners = [];
                this.setItemId(builder.itemViewIdProducer.next());
                if (!builder.element) {
                    this.getEl().setData(liveedit.ItemType.ATTRIBUTE_TYPE, builder.type.getShortName());
                }
                if (builder.placeholder) {
                    this.placeholder = builder.placeholder;
                    this.appendChild(this.placeholder);
                }
                this.onMouseEnter(this.handleMouseEnter.bind(this));
                this.onMouseLeave(this.handleMouseLeave.bind(this));
                this.onClicked(this.handleClick.bind(this));
                this.onContextMenu(this.handleClick.bind(this));
                this.onTouchStart(this.handleClick.bind(this));
                this.onRemoved(this.invalidateContextMenu.bind(this));
                this.bindMouseListeners();
            }
            ItemView.prototype.bindMouseListeners = function () {
                var _this = this;
                api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (_this.isSelected()) {
                        _this.highlight();
                        _this.shade();
                    }
                });
                this.onMouseOverView(function () {
                    var hasSelectedView = _this.getPageView().hasSelectedView();
                    if (!hasSelectedView) {
                        _this.showTooltip();
                        _this.showCursor();
                        _this.highlight();
                    }
                });
                this.onMouseLeaveView(function () {
                    var hasSelectedView = _this.getPageView().hasSelectedView();
                    if (!hasSelectedView) {
                        _this.hideTooltip();
                        _this.resetCursor();
                        _this.unhighlight();
                    }
                });
                liveedit.Shader.get().onClicked(function (event) { return _this.handleShaderClick(event); });
            };
            ItemView.prototype.highlight = function () {
                liveedit.Highlighter.get().highlightItemView(this);
            };
            ItemView.prototype.unhighlight = function () {
                liveedit.Highlighter.get().hide();
            };
            ItemView.prototype.shade = function () {
                liveedit.Shader.get().shadeItemView(this);
            };
            ItemView.prototype.unshade = function () {
                liveedit.Shader.get().hide();
            };
            ItemView.prototype.showCursor = function () {
                liveedit.Cursor.get().displayItemViewCursor(this);
            };
            ItemView.prototype.resetCursor = function () {
                liveedit.Cursor.get().reset();
            };
            ItemView.prototype.getPageView = function () {
                var pageView = this;
                while (!liveedit.PageItemType.get().equals(pageView.getType())) {
                    pageView = pageView.parentItemView;
                }
                return pageView;
            };
            ItemView.prototype.remove = function () {
                if (this.contextMenu) {
                    this.contextMenu.remove();
                }
                if (this.loadMask) {
                    this.loadMask.remove();
                }
                this.unhighlight();
                this.unshade();
                _super.prototype.remove.call(this);
                return this;
            };
            ItemView.prototype.setDraggable = function (value) {
                _super.prototype.setDraggable.call(this, value);
                this.toggleClass('not-draggable', !value);
            };
            ItemView.prototype.scrollComponentIntoView = function () {
                var dimensions = this.getElementDimensions();
                var screenTopPosition = document.body.scrollTop != 0 ? document.body.scrollTop : document.documentElement.scrollTop;
                if (dimensions.top != undefined && dimensions.top - 10 < screenTopPosition) {
                    wemjq("html,body").animate({ scrollTop: dimensions.top - 10 }, 200);
                }
            };
            ItemView.prototype.handleMouseEnter = function (event) {
                if (ItemView.debug) {
                    console.group("mouse enter [" + this.getId() + "]");
                }
                if (this.mouseOver) {
                    if (ItemView.debug) {
                        console.log('mouseOver = true, returning.');
                        console.groupEnd();
                    }
                    return;
                }
                this.manageParentsMouseOver();
                this.mouseOver = true;
                this.notifyMouseOverView();
                if (ItemView.debug) {
                    console.groupEnd();
                }
            };
            ItemView.prototype.manageParentsMouseOver = function () {
                var parentsStack = [];
                for (var parent = this.parentItemView; parent; parent = parent.parentItemView) {
                    parentsStack.push(parent);
                    if (parent.mouseOver) {
                        break;
                    }
                }
                parentsStack.reverse().forEach(function (view) {
                    if (view.mouseOver) {
                        if (ItemView.debug) {
                            console.debug('parent.mouseOver = true, notifying mouse out [' + view.getId() + "]");
                        }
                        view.notifyMouseLeaveView();
                    }
                    else {
                        view.mouseOver = true;
                        if (ItemView.debug) {
                            console.debug('parent.mouseOver = false, setting to true [' + view.getId() + "]");
                        }
                        view.notifyMouseOverView();
                        view.notifyMouseLeaveView();
                    }
                });
            };
            ItemView.prototype.handleMouseLeave = function (event) {
                if (ItemView.debug) {
                    console.group("mouse leave [" + this.getId() + "]");
                }
                this.mouseOver = false;
                this.notifyMouseLeaveView();
                if (this.tooltip) {
                    this.tooltip.hide();
                }
                if (this.parentItemView) {
                    this.parentItemView.notifyMouseOverView();
                }
                if (ItemView.debug) {
                    console.groupEnd();
                }
            };
            ItemView.prototype.isEmpty = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ItemView.prototype.refreshEmptyState = function () {
                this.toggleClass('empty', this.isEmpty());
            };
            ItemView.prototype.handleClick = function (event) {
                event.stopPropagation();
                if (!this.isSelected()) {
                    this.deselectParent();
                    this.select(!this.isEmpty() ? { x: event.pageX, y: event.pageY } : null);
                }
                else {
                    this.deselect();
                }
            };
            ItemView.prototype.handleShaderClick = function (event) {
                if (this.isSelected()) {
                    this.deselect();
                }
            };
            ItemView.prototype.deselectParent = function () {
                for (var parent = this.parentItemView; parent; parent = parent.parentItemView) {
                    if (parent.isSelected()) {
                        parent.deselect();
                        return;
                    }
                }
            };
            ItemView.prototype.getItemViewIdProducer = function () {
                return this.itemViewIdProducer;
            };
            ItemView.prototype.setTooltipObject = function (object) {
                if (this.tooltipViewer) {
                    this.tooltipViewer.setObject(object);
                }
            };
            ItemView.prototype.showTooltip = function () {
                if (ItemView.debug) {
                    console.log('showing tooltip [' + this.getId() + "]");
                }
                if (this.tooltip) {
                    this.tooltip.show();
                }
            };
            ItemView.prototype.hideTooltip = function (hideParentTooltip) {
                if (hideParentTooltip === void 0) { hideParentTooltip = true; }
                if (ItemView.debug) {
                    console.log('hiding tooltip [' + this.getId() + "]");
                }
                if (this.tooltip) {
                    this.tooltip.hide();
                }
                if (hideParentTooltip && this.parentItemView) {
                    this.parentItemView.hideTooltip();
                }
            };
            ItemView.prototype.showContextMenu = function (clickPosition, menuPosition) {
                if (!this.contextMenu) {
                    this.contextMenu = new api.liveedit.ItemViewContextMenu(this.contextMenuTitle, this.contextMenuActions);
                }
                var dimensions = this.getElementDimensions();
                var x, y;
                if (clickPosition) {
                    x = clickPosition.x;
                    y = clickPosition.y;
                }
                else {
                    x = dimensions.left + dimensions.width / 2;
                    y = dimensions.top + (0 /* TOP */ == menuPosition ? 0 : dimensions.height);
                }
                this.contextMenu.showAt(x, y);
            };
            ItemView.prototype.hideContextMenu = function () {
                if (this.contextMenu) {
                    this.contextMenu.hide();
                }
            };
            ItemView.prototype.invalidateContextMenu = function () {
                if (this.contextMenu) {
                    this.contextMenu.remove();
                    this.contextMenu = null;
                }
            };
            ItemView.prototype.setItemId = function (value) {
                this.getEl().setAttribute("data-" + liveedit.ItemViewId.DATA_ATTRIBUTE, value.toString());
            };
            ItemView.prototype.getItemId = function () {
                var asString = this.getEl().getAttribute("data-" + liveedit.ItemViewId.DATA_ATTRIBUTE);
                if (!asString) {
                    return null;
                }
                return liveedit.ItemViewId.fromString(asString);
            };
            ItemView.parseItemId = function (element) {
                var attribute = element.getAttribute("data-" + liveedit.ItemViewId.DATA_ATTRIBUTE);
                if (api.util.StringHelper.isEmpty(attribute)) {
                    return null;
                }
                return liveedit.ItemViewId.fromString(attribute);
            };
            ItemView.prototype.getType = function () {
                return this.type;
            };
            ItemView.prototype.getParentItemView = function () {
                return this.parentItemView;
            };
            ItemView.prototype.setParentItemView = function (itemView) {
                this.parentItemView = itemView;
            };
            ItemView.prototype.isSelected = function () {
                return this.getEl().hasAttribute('data-live-edit-selected');
            };
            ItemView.prototype.select = function (clickPosition, menuPosition) {
                this.getEl().setData("live-edit-selected", "true");
                this.showContextMenu(clickPosition, menuPosition);
                this.hideTooltip();
                this.highlight();
                this.shade();
                this.showCursor();
                this.stopTextEditMode();
                if (this.isEmpty()) {
                    this.selectPlaceholder();
                }
                new liveedit.ItemViewSelectedEvent(this, clickPosition).fire();
            };
            ItemView.prototype.deselect = function (silent) {
                this.getEl().removeAttribute("data-live-edit-selected");
                this.hideContextMenu();
                this.unhighlight();
                this.unshade();
                if (this.isEmpty()) {
                    this.deselectPlaceholder();
                }
                if (!silent) {
                    new liveedit.ItemViewDeselectEvent(this).fire();
                }
            };
            ItemView.prototype.stopTextEditMode = function () {
                var pageView = this.getPageView();
                if (pageView.isTextEditMode()) {
                    pageView.setTextEditMode(false);
                }
            };
            ItemView.prototype.selectPlaceholder = function () {
                if (this.placeholder) {
                    this.placeholder.select();
                }
            };
            ItemView.prototype.deselectPlaceholder = function () {
                if (this.placeholder) {
                    this.placeholder.deselect();
                }
            };
            ItemView.prototype.showRenderingError = function (url, errorMessage) {
                if (this.placeholder) {
                    this.addClass("error");
                    this.placeholder.showRenderingError(url, errorMessage);
                }
            };
            ItemView.prototype.getName = function () {
                return '[No Name]';
            };
            ItemView.prototype.showLoadingSpinner = function () {
                if (!this.loadMask) {
                    this.loadMask = new api.ui.mask.LoadMask(this);
                    this.appendChild(this.loadMask);
                }
                this.loadMask.show();
            };
            ItemView.prototype.hideLoadingSpinner = function () {
                if (this.loadMask) {
                    this.loadMask.hide();
                }
            };
            ItemView.prototype.getElementDimensions = function () {
                var el = this.getEl(), offset = el.getOffset();
                return {
                    top: offset.top,
                    left: offset.left,
                    width: el.getWidthWithBorder(),
                    height: el.getHeightWithBorder()
                };
            };
            ItemView.prototype.toItemViewArray = function () {
                return [this];
            };
            ItemView.prototype.toString = function () {
                return this.getItemId().toNumber() + " : " + this.getType().getShortName();
            };
            ItemView.findParentItemViewAsHTMLElement = function (htmlElement) {
                var parentHTMLElement = htmlElement.parentElement;
                var parseItemId = ItemView.parseItemId(parentHTMLElement);
                while (parseItemId == null) {
                    parentHTMLElement = parentHTMLElement.parentElement;
                    parseItemId = ItemView.parseItemId(parentHTMLElement);
                }
                return parentHTMLElement;
            };
            ItemView.findPreviousItemView = function (htmlElement) {
                var element = new api.dom.ElementHelper(htmlElement);
                var previous = element.getPrevious();
                while (previous != null && !previous.hasAttribute("data-" + liveedit.ItemType.ATTRIBUTE_TYPE)) {
                    previous = previous.getPrevious();
                }
                return previous;
            };
            ItemView.prototype.onMouseOverView = function (listener) {
                this.mouseOverViewListeners.push(listener);
            };
            ItemView.prototype.unMouseOverView = function (listener) {
                this.mouseOverViewListeners = this.mouseOverViewListeners.filter(function (current) { return (current != listener); });
            };
            ItemView.prototype.notifyMouseOverView = function () {
                if (ItemView.debug) {
                    console.log("notifying mouse over [" + this.getId() + "]");
                }
                this.mouseOverViewListeners.forEach(function (listener) { return listener(); });
            };
            ItemView.prototype.onMouseLeaveView = function (listener) {
                this.mouseOutViewListeners.push(listener);
            };
            ItemView.prototype.unMouseLeaveView = function (listener) {
                this.mouseOutViewListeners = this.mouseOutViewListeners.filter(function (current) { return (current != listener); });
            };
            ItemView.prototype.notifyMouseLeaveView = function () {
                if (ItemView.debug) {
                    console.log("notifying mouse out [" + this.getId() + "]");
                }
                this.mouseOutViewListeners.forEach(function (listener) { return listener(); });
            };
            ItemView.debug = false;
            return ItemView;
        })(api.dom.Element);
        liveedit.ItemView = ItemView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewAddedEvent = (function () {
            function ItemViewAddedEvent(view) {
                this.view = view;
            }
            ItemViewAddedEvent.prototype.getView = function () {
                return this.view;
            };
            return ItemViewAddedEvent;
        })();
        liveedit.ItemViewAddedEvent = ItemViewAddedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewRemovedEvent = (function () {
            function ItemViewRemovedEvent(view) {
                this.view = view;
            }
            ItemViewRemovedEvent.prototype.getView = function () {
                return this.view;
            };
            return ItemViewRemovedEvent;
        })();
        liveedit.ItemViewRemovedEvent = ItemViewRemovedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewSelectedEvent = (function (_super) {
            __extends(ItemViewSelectedEvent, _super);
            function ItemViewSelectedEvent(itemView, position) {
                _super.call(this);
                this.pageItemView = itemView;
                this.position = position;
            }
            ItemViewSelectedEvent.prototype.getItemView = function () {
                return this.pageItemView;
            };
            ItemViewSelectedEvent.prototype.getPosition = function () {
                return this.position;
            };
            ItemViewSelectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ItemViewSelectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ItemViewSelectedEvent;
        })(api.event.Event);
        liveedit.ItemViewSelectedEvent = ItemViewSelectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewDeselectEvent = (function (_super) {
            __extends(ItemViewDeselectEvent, _super);
            function ItemViewDeselectEvent(itemView) {
                _super.call(this);
                this.itemView = itemView;
            }
            ItemViewDeselectEvent.prototype.getItemView = function () {
                return this.itemView;
            };
            ItemViewDeselectEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ItemViewDeselectEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ItemViewDeselectEvent;
        })(api.event.Event);
        liveedit.ItemViewDeselectEvent = ItemViewDeselectEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewContextMenuTitle = (function (_super) {
            __extends(ItemViewContextMenuTitle, _super);
            function ItemViewContextMenuTitle(name, icon) {
                _super.call(this, new api.app.NamesAndIconViewBuilder().setAddTitleAttribute(false));
                this.setMainName(name);
                this.setIconClass(icon);
            }
            return ItemViewContextMenuTitle;
        })(api.app.NamesAndIconView);
        liveedit.ItemViewContextMenuTitle = ItemViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Component = api.content.page.region.Component;
        var ComponentViewContextMenuTitle = (function (_super) {
            __extends(ComponentViewContextMenuTitle, _super);
            function ComponentViewContextMenuTitle(component, type) {
                var _this = this;
                var handler = function (event) {
                    if (event.getPropertyName() == Component.PROPERTY_NAME) {
                        _this.setMainName(component.getName() ? component.getName().toString() : "");
                    }
                };
                _super.call(this, component.getName() ? component.getName().toString() : "", type.getConfig().getIconCls());
                component.onPropertyChanged(handler);
                this.onRemoved(function () { return component.unPropertyChanged(handler); });
            }
            return ComponentViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.ComponentViewContextMenuTitle = ComponentViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ContentViewContextMenuTitle = (function (_super) {
            __extends(ContentViewContextMenuTitle, _super);
            function ContentViewContextMenuTitle(contentView) {
                _super.call(this, contentView.getName(), liveedit.ContentItemType.get().getConfig().getIconCls());
            }
            return ContentViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.ContentViewContextMenuTitle = ContentViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageViewContextMenuTitle = (function (_super) {
            __extends(PageViewContextMenuTitle, _super);
            function PageViewContextMenuTitle(content) {
                _super.call(this, content.getDisplayName(), liveedit.PageItemType.get().getConfig().getIconCls());
            }
            return PageViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.PageViewContextMenuTitle = PageViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionViewContextMenuTitle = (function (_super) {
            __extends(RegionViewContextMenuTitle, _super);
            function RegionViewContextMenuTitle(region) {
                _super.call(this, region.getName(), liveedit.RegionItemType.get().getConfig().getIconCls());
            }
            return RegionViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.RegionViewContextMenuTitle = RegionViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewContextMenu = (function (_super) {
            __extends(ItemViewContextMenu, _super);
            function ItemViewContextMenu(menuTitle, actions) {
                var _this = this;
                _super.call(this, 'item-view-context-menu bottom');
                this.title = menuTitle;
                if (this.title) {
                    var lastPosition;
                    var dragListener = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        var x = e.pageX, y = e.pageY;
                        _this.moveBy(x - lastPosition.x, y - lastPosition.y);
                        lastPosition = {
                            x: x,
                            y: y
                        };
                    };
                    var upListener = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        _this.stopDrag(dragListener, upListener);
                    };
                    this.title.onMouseDown(function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        lastPosition = {
                            x: e.pageX,
                            y: e.pageY
                        };
                        _this.startDrag(dragListener, upListener);
                    });
                    this.appendChild(this.title);
                }
                this.menu = new api.ui.menu.ContextMenu(actions, false).setHideOnItemClick(false);
                this.menu.onItemClicked(function (item) {
                    _this.hide();
                });
                this.appendChild(this.menu);
                this.onClicked(function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                });
                this.onHidden(function (e) {
                    _this.stopDrag(dragListener, upListener);
                });
                api.dom.Body.get().appendChild(this);
            }
            ItemViewContextMenu.prototype.showAt = function (x, y) {
                this.menu.showAt.call(this, x - this.getEl().getWidth() / 2, y);
            };
            ItemViewContextMenu.prototype.moveBy = function (dx, dy) {
                this.menu.moveBy.call(this, dx, dy);
            };
            ItemViewContextMenu.prototype.setActions = function (actions) {
                this.menu.setActions(actions);
            };
            ItemViewContextMenu.prototype.startDrag = function (dragListener, upListener) {
                api.dom.Body.get().onMouseMove(dragListener);
                api.dom.Body.get().onMouseUp(upListener);
            };
            ItemViewContextMenu.prototype.stopDrag = function (dragListener, upListener) {
                api.dom.Body.get().unMouseMove(dragListener);
                api.dom.Body.get().unMouseUp(upListener);
            };
            return ItemViewContextMenu;
        })(api.dom.DivEl);
        liveedit.ItemViewContextMenu = ItemViewContextMenu;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        (function (ItemViewContextMenuPosition) {
            ItemViewContextMenuPosition[ItemViewContextMenuPosition["TOP"] = 0] = "TOP";
            ItemViewContextMenuPosition[ItemViewContextMenuPosition["BOTTOM"] = 1] = "BOTTOM";
        })(liveedit.ItemViewContextMenuPosition || (liveedit.ItemViewContextMenuPosition = {}));
        var ItemViewContextMenuPosition = liveedit.ItemViewContextMenuPosition;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionViewBuilder = (function () {
            function RegionViewBuilder() {
            }
            RegionViewBuilder.prototype.setLiveEditModel = function (value) {
                this.liveEditModel = value;
                return this;
            };
            RegionViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            RegionViewBuilder.prototype.setParentView = function (value) {
                this.parentView = value;
                return this;
            };
            RegionViewBuilder.prototype.setRegion = function (value) {
                this.region = value;
                return this;
            };
            RegionViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            return RegionViewBuilder;
        })();
        liveedit.RegionViewBuilder = RegionViewBuilder;
        var RegionView = (function (_super) {
            __extends(RegionView, _super);
            function RegionView(builder) {
                var _this = this;
                this.componentViews = [];
                this.itemViewAddedListeners = [];
                this.itemViewRemovedListeners = [];
                this.parentView = builder.parentView;
                RegionView.debug = false;
                this.itemViewAddedListener = function (event) { return _this.notifyItemViewAdded(event.getView()); };
                this.itemViewRemovedListener = function (event) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(event.getView(), liveedit.ComponentView)) {
                        var removedComponentView = event.getView();
                        var childIndex = _this.getComponentViewIndex(removedComponentView);
                        if (childIndex > -1) {
                            _this.componentViews.splice(childIndex, 1);
                        }
                    }
                    _this.notifyItemViewRemoved(event.getView());
                };
                _super.call(this, new liveedit.ItemViewBuilder().setItemViewIdProducer(builder.parentView.getItemViewIdProducer()).setType(liveedit.RegionItemType.get()).setElement(builder.element).setPlaceholder(new liveedit.RegionPlaceholder(builder.region)).setTooltipViewer(new liveedit.RegionComponentViewer()).setParentElement(builder.parentElement).setParentView(builder.parentView).setContextMenuActions(this.createRegionContextMenuActions()).setContextMenuTitle(new liveedit.RegionViewContextMenuTitle(builder.region)));
                this.addClass('region-view');
                this.setRegion(builder.region);
                this.parseComponentViews();
            }
            RegionView.prototype.createRegionContextMenuActions = function () {
                var _this = this;
                var actions = [];
                actions.push(new api.ui.Action('Parent').onExecuted(function () {
                    var parentView = _this.getParentItemView();
                    if (parentView) {
                        _this.deselect();
                        parentView.select(null, 0 /* TOP */);
                        parentView.scrollComponentIntoView();
                    }
                }));
                actions.push(new api.ui.Action('Empty').onExecuted(function () {
                    _this.deselect();
                    _this.empty();
                }));
                return actions;
            };
            RegionView.prototype.getParentItemView = function () {
                return this.parentView;
            };
            RegionView.prototype.setParentItemView = function (itemView) {
                _super.prototype.setParentItemView.call(this, itemView);
                this.parentView = itemView;
            };
            RegionView.prototype.setRegion = function (region) {
                var _this = this;
                this.region = region;
                if (region) {
                    this.setTooltipObject(region);
                    region.onComponentAdded(function (event) {
                        if (RegionView.debug) {
                            console.log('RegionView.handleComponentAdded: ' + event.getComponentPath().toString());
                        }
                        _this.refreshEmptyState();
                    });
                    region.onComponentRemoved(function (event) {
                        if (RegionView.debug) {
                            console.log('RegionView.handleComponentRemoved: ' + event.getComponentPath().toString());
                        }
                        _this.refreshEmptyState();
                    });
                    var components = region.getComponents();
                    var componentViews = this.getComponentViews();
                    componentViews.forEach(function (view, index) {
                        view.setComponent(components[index]);
                    });
                }
                this.refreshEmptyState();
            };
            RegionView.prototype.getRegion = function () {
                return this.region;
            };
            RegionView.prototype.getRegionName = function () {
                return this.getRegionPath() ? this.getRegionPath().getRegionName() : null;
            };
            RegionView.prototype.getRegionPath = function () {
                return this.region ? this.region.getPath() : null;
            };
            RegionView.prototype.getName = function () {
                return this.getRegionName() ? this.getRegionName().toString() : "[No Name]";
            };
            RegionView.prototype.highlight = function () {
                if (!this.getPageView().isTextEditMode()) {
                    _super.prototype.highlight.call(this);
                }
            };
            RegionView.prototype.showCursor = function () {
                if (!this.getPageView().isTextEditMode()) {
                    _super.prototype.showCursor.call(this);
                }
            };
            RegionView.prototype.handleClick = function (event) {
                event.stopPropagation();
                var pageView = this.getPageView();
                if (pageView.isTextEditMode()) {
                    pageView.setTextEditMode(false);
                }
                else {
                    _super.prototype.handleClick.call(this, event);
                }
            };
            RegionView.prototype.select = function (clickPosition, menuPosition) {
                _super.prototype.select.call(this, clickPosition, menuPosition);
                new liveedit.RegionSelectedEvent(this).fire();
            };
            RegionView.prototype.toString = function () {
                return _super.prototype.toString.call(this) + " : " + this.getRegionPath().toString();
            };
            RegionView.prototype.registerComponentView = function (componentView, index) {
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + '].registerComponentView: ' + componentView.toString() + " at " + index);
                }
                if (index >= 0) {
                    this.componentViews.splice(index, 0, componentView);
                }
                else {
                    this.componentViews.push(componentView);
                }
                componentView.setParentItemView(this);
                componentView.onItemViewAdded(this.itemViewAddedListener);
                componentView.onItemViewRemoved(this.itemViewRemovedListener);
                this.notifyItemViewAdded(componentView);
            };
            RegionView.prototype.unregisterComponentView = function (componentView) {
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + '].unregisterComponentView: ' + componentView.toString());
                }
                var indexToRemove = this.getComponentViewIndex(componentView);
                if (indexToRemove >= 0) {
                    componentView.unItemViewAdded(this.itemViewAddedListener);
                    componentView.unItemViewRemoved(this.itemViewRemovedListener);
                    this.componentViews.splice(indexToRemove, 1);
                    componentView.setParentItemView(null);
                    this.notifyItemViewRemoved(componentView);
                }
                else {
                    throw new Error("Did not find ComponentView to remove: " + componentView.getItemId().toString());
                }
            };
            RegionView.prototype.addComponentView = function (componentView, index) {
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + ']addComponentView: ' + componentView.toString() + " at " + index);
                }
                if (componentView.getComponent()) {
                    this.region.addComponent(componentView.getComponent(), index);
                }
                this.insertChild(componentView, index);
                this.registerComponentView(componentView, index);
                new liveedit.ComponentAddedEvent(componentView).fire();
            };
            RegionView.prototype.removeComponentView = function (componentView) {
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + '].removeComponentView: ' + componentView.toString());
                }
                this.unregisterComponentView(componentView);
                this.removeChild(componentView);
                if (componentView.getComponent()) {
                    componentView.getComponent().remove();
                }
                new liveedit.ComponentRemovedEvent(componentView).fire();
            };
            RegionView.prototype.getComponentViews = function () {
                return this.componentViews;
            };
            RegionView.prototype.getComponentViewIndex = function (view) {
                return this.componentViews.indexOf(view);
            };
            RegionView.prototype.getComponentViewByIndex = function (index) {
                return this.componentViews[index];
            };
            RegionView.prototype.getComponentViewByPath = function (path) {
                var firstLevelOfPath = path.getFirstLevel();
                if (path.numberOfLevels() == 1) {
                    return this.componentViews[firstLevelOfPath.getComponentIndex()];
                }
                for (var i = 0; i < this.componentViews.length; i++) {
                    var componentView = this.componentViews[i];
                    if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, api.liveedit.layout.LayoutComponentView)) {
                        var layoutView = componentView;
                        var match = layoutView.getComponentViewByPath(path.removeFirstLevel());
                        if (match) {
                            return match;
                        }
                    }
                }
                return null;
            };
            RegionView.prototype.hasParentLayoutComponentView = function () {
                return api.ObjectHelper.iFrameSafeInstanceOf(this.parentView, api.liveedit.layout.LayoutComponentView);
            };
            RegionView.prototype.hasOnlyMovingComponentViews = function () {
                return this.componentViews.length > 0 && this.componentViews.every(function (view) {
                    return view.isMoving();
                });
            };
            RegionView.prototype.hasComponentDropzone = function () {
                var foundDropZone = false;
                var child = this.getHTMLElement().firstChild;
                while (child) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(child, HTMLElement)) {
                        var childHtmlElement = new api.dom.ElementHelper(child);
                        if (childHtmlElement.hasClass("drag-placeholder") || childHtmlElement.hasClass("live-edit-drag-placeholder-container")) {
                            if (childHtmlElement.getDisplay() != "none") {
                                foundDropZone = true;
                                break;
                            }
                        }
                    }
                    child = child.nextSibling;
                }
                return foundDropZone;
            };
            RegionView.prototype.isEmpty = function () {
                return !this.hasComponentDropzone() && (!this.region || this.region.isEmpty() || this.hasOnlyMovingComponentViews());
            };
            RegionView.prototype.empty = function () {
                if (RegionView.debug) {
                    console.debug("RegionView[" + this.toString() + "].empty()", this.componentViews);
                }
                while (this.componentViews.length > 0) {
                    this.removeComponentView(this.componentViews[0]);
                }
            };
            RegionView.prototype.toItemViewArray = function () {
                var array = [];
                array.push(this);
                this.componentViews.forEach(function (componentView) {
                    var itemViews = componentView.toItemViewArray();
                    array = array.concat(itemViews);
                });
                return array;
            };
            RegionView.prototype.onItemViewAdded = function (listener) {
                this.itemViewAddedListeners.push(listener);
            };
            RegionView.prototype.unItemViewAdded = function (listener) {
                this.itemViewAddedListeners = this.itemViewAddedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            RegionView.prototype.notifyItemViewAddedForAll = function (itemViews) {
                var _this = this;
                itemViews.forEach(function (itemView) {
                    _this.notifyItemViewAdded(itemView);
                });
            };
            RegionView.prototype.notifyItemViewAdded = function (itemView) {
                var event = new liveedit.ItemViewAddedEvent(itemView);
                this.itemViewAddedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            RegionView.prototype.onItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners.push(listener);
            };
            RegionView.prototype.unItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners = this.itemViewRemovedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            RegionView.prototype.notifyItemViewRemovedForAll = function (itemViews) {
                var _this = this;
                itemViews.forEach(function (itemView) {
                    _this.notifyItemViewRemoved(itemView);
                });
            };
            RegionView.prototype.notifyItemViewRemoved = function (itemView) {
                var event = new liveedit.ItemViewRemovedEvent(itemView);
                this.itemViewRemovedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            RegionView.isRegionViewFromHTMLElement = function (htmlElement) {
                var type = htmlElement.getAttribute("data-" + liveedit.ItemType.ATTRIBUTE_TYPE);
                if (api.util.StringHelper.isBlank(type)) {
                    return false;
                }
                return type == "region";
            };
            RegionView.prototype.parseComponentViews = function () {
                this.doParseComponentViews();
            };
            RegionView.prototype.doParseComponentViews = function (parentElement) {
                var _this = this;
                var children = parentElement ? parentElement.getChildren() : this.getChildren();
                var region = this.getRegion();
                var componentCount = 0;
                children.forEach(function (childElement) {
                    var itemType = liveedit.ItemType.fromElement(childElement);
                    if (itemType) {
                        api.util.assert(itemType.isComponentType(), "Expected ItemView beneath a Region to be a Component: " + itemType.getShortName());
                        var component = region.getComponentByIndex(componentCount++);
                        if (component) {
                            var componentView = itemType.createView(new liveedit.CreateItemViewConfig().setParentView(_this).setData(component).setElement(childElement).setParentElement(parentElement ? parentElement : _this));
                            _this.registerComponentView(componentView, componentCount);
                        }
                    }
                    else {
                        _this.doParseComponentViews(childElement);
                    }
                });
            };
            RegionView.prototype.handleDragEnter = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ItemView.handleDragEnter", event, this.getHTMLElement());
                }
            };
            RegionView.prototype.handleDragLeave = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ItemView.handleDragLeave", event, this.getHTMLElement());
                }
            };
            RegionView.prototype.handleDragOver = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("RegionView[" + this.toString() + "].handleDragOver: ", event.target, event.target);
                    event.preventDefault();
                }
            };
            RegionView.prototype.handleDrop = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("RegionView[" + this.toString() + "].handleDrop: ", event.target, this.getHTMLElement());
                    event.preventDefault();
                    var data = event.dataTransfer.getData("Text");
                }
            };
            return RegionView;
        })(liveedit.ItemView);
        liveedit.RegionView = RegionView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionComponentViewer = (function (_super) {
            __extends(RegionComponentViewer, _super);
            function RegionComponentViewer() {
                _super.call(this);
            }
            RegionComponentViewer.prototype.resolveDisplayName = function (object) {
                return object.getName().toString();
            };
            RegionComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return object.getPath().toString();
            };
            RegionComponentViewer.prototype.resolveIconClass = function (object) {
                return "live-edit-font-icon-region";
            };
            return RegionComponentViewer;
        })(api.ui.NamesAndIconViewer);
        liveedit.RegionComponentViewer = RegionComponentViewer;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewPlaceholder = (function (_super) {
            __extends(ItemViewPlaceholder, _super);
            function ItemViewPlaceholder() {
                _super.call(this, "item-placeholder");
            }
            ItemViewPlaceholder.prototype.showRenderingError = function (url, errorMessage) {
                if (errorMessage === void 0) { errorMessage = "Error rendering component"; }
                this.removeChildren();
                this.addClass("rendering-error");
                var errorTitle = new api.dom.PEl().setHtml(errorMessage);
                var urlAnchor = new api.dom.AEl().setUrl(url, "_blank").setHtml("Show more...");
                this.appendChildren(errorTitle, urlAnchor);
            };
            ItemViewPlaceholder.prototype.select = function () {
            };
            ItemViewPlaceholder.prototype.deselect = function () {
            };
            return ItemViewPlaceholder;
        })(api.dom.DivEl);
        liveedit.ItemViewPlaceholder = ItemViewPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var DragPlaceholderBuilder = (function () {
            function DragPlaceholderBuilder() {
                this.dropAllowed = true;
                this.text = "Drop {0} here";
            }
            DragPlaceholderBuilder.prototype.setItemType = function (value) {
                this.itemType = value;
                return this;
            };
            DragPlaceholderBuilder.prototype.setDropAllowed = function (value) {
                this.dropAllowed = value;
                return this;
            };
            DragPlaceholderBuilder.prototype.setText = function (value) {
                this.text = value;
                return this;
            };
            DragPlaceholderBuilder.prototype.setRegionView = function (value) {
                this.regionView = value;
                return this;
            };
            DragPlaceholderBuilder.prototype.setComponentView = function (value) {
                this.componentView = value;
                this.itemType = value.getType();
                return this;
            };
            DragPlaceholderBuilder.prototype.build = function () {
                return new DragPlaceholder(this);
            };
            return DragPlaceholderBuilder;
        })();
        liveedit.DragPlaceholderBuilder = DragPlaceholderBuilder;
        var DragPlaceholder = (function (_super) {
            __extends(DragPlaceholder, _super);
            function DragPlaceholder(builder) {
                _super.call(this);
                this.itemType = builder.itemType;
                this.dropAllowed = builder.dropAllowed;
                this.text = api.util.StringHelper.format(builder.text, api.util.StringHelper.capitalize(this.itemType.getShortName()));
                this.regionView = builder.regionView;
                this.componentView = builder.componentView;
                this.message = new api.dom.DivEl("message");
                if (builder.text) {
                    this.message.setHtml(this.text);
                }
                this.appendChild(this.message);
                this.addClass(this.itemType.getShortName().toLowerCase() + "-placeholder drag-placeholder");
                if (!this.dropAllowed) {
                    this.addClass("forbidden");
                }
            }
            return DragPlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.DragPlaceholder = DragPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionPlaceholder = (function (_super) {
            __extends(RegionPlaceholder, _super);
            function RegionPlaceholder(region) {
                _super.call(this);
                this.addClass("region-placeholder");
                this.region = region;
                var dragComponentsHereEl = new api.dom.PEl();
                dragComponentsHereEl.setHtml("Drop components here");
                var componentTypeInfoText = api.util.StringHelper.capitalize(liveedit.RegionItemType.get().getShortName()) + ': ' + region.getName();
                var typeInfoEl = new api.dom.PEl();
                typeInfoEl.setHtml(componentTypeInfoText);
                this.appendChildren(dragComponentsHereEl, typeInfoEl);
            }
            return RegionPlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.RegionPlaceholder = RegionPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentPlaceholder = (function (_super) {
            __extends(ComponentPlaceholder, _super);
            function ComponentPlaceholder() {
                _super.apply(this, arguments);
            }
            return ComponentPlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.ComponentPlaceholder = ComponentPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentViewBuilder = (function () {
            function ComponentViewBuilder() {
            }
            ComponentViewBuilder.prototype.setItemViewProducer = function (value) {
                this.itemViewProducer = value;
                return this;
            };
            ComponentViewBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            ComponentViewBuilder.prototype.setParentRegionView = function (value) {
                this.parentRegionView = value;
                return this;
            };
            ComponentViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            ComponentViewBuilder.prototype.setComponent = function (value) {
                this.component = value;
                return this;
            };
            ComponentViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            ComponentViewBuilder.prototype.setPositionIndex = function (value) {
                this.positionIndex = value;
                return this;
            };
            ComponentViewBuilder.prototype.setContextMenuActions = function (actions) {
                this.contextMenuActions = actions;
                return this;
            };
            ComponentViewBuilder.prototype.setPlaceholder = function (value) {
                this.placeholder = value;
                return this;
            };
            ComponentViewBuilder.prototype.setTooltipViewer = function (value) {
                this.tooltipViewer = value;
                return this;
            };
            return ComponentViewBuilder;
        })();
        liveedit.ComponentViewBuilder = ComponentViewBuilder;
        var ComponentView = (function (_super) {
            __extends(ComponentView, _super);
            function ComponentView(builder) {
                var _this = this;
                this.itemViewAddedListeners = [];
                this.itemViewRemovedListeners = [];
                this.moving = false;
                this.parentRegionView = builder.parentRegionView;
                _super.call(this, new liveedit.ItemViewBuilder().setItemViewIdProducer(builder.itemViewProducer ? builder.itemViewProducer : builder.parentRegionView.getItemViewIdProducer()).setPlaceholder(builder.placeholder).setTooltipViewer(builder.tooltipViewer).setType(builder.type).setElement(builder.element).setParentView(builder.parentRegionView).setParentElement(builder.parentElement).setContextMenuActions(this.createComponentContextMenuActions(builder.contextMenuActions)).setContextMenuTitle(new liveedit.ComponentViewContextMenuTitle(builder.component, builder.type)));
                this.propertyChangedListener = function () { return _this.refreshEmptyState(); };
                this.resetListener = function () {
                    _this.deselect();
                    var clone = _this.clone();
                    _this.replaceWith(clone);
                    clone.select();
                    new api.liveedit.ComponentResetEvent(clone).fire();
                };
                this.setComponent(builder.component);
            }
            ComponentView.prototype.registerComponentListeners = function (component) {
                component.onReset(this.resetListener);
                component.onPropertyChanged(this.propertyChangedListener);
            };
            ComponentView.prototype.unregisterComponentListeners = function (component) {
                component.unPropertyChanged(this.propertyChangedListener);
                component.unReset(this.resetListener);
            };
            ComponentView.prototype.createComponentContextMenuActions = function (actions) {
                var _this = this;
                var actions = actions || [];
                actions.push(new api.ui.Action("Parent").onExecuted(function () {
                    var parentView = _this.getParentItemView();
                    if (parentView) {
                        _this.deselect();
                        parentView.select(null, 0 /* TOP */);
                        parentView.scrollComponentIntoView();
                    }
                }));
                actions.push(new api.ui.Action("Empty").onExecuted(function () {
                    _this.component.reset();
                }));
                actions.push(new api.ui.Action("Remove").onExecuted(function () {
                    _this.deselect();
                    _this.remove();
                }));
                actions.push(new api.ui.Action("Duplicate").onExecuted(function () {
                    _this.deselect();
                    var duplicatedComponent = _this.getComponent().duplicate();
                    var duplicatedView = _this.duplicate(duplicatedComponent);
                    duplicatedView.select();
                    duplicatedView.showLoadingSpinner();
                    new liveedit.ComponentDuplicatedEvent(_this, duplicatedView).fire();
                }));
                return actions;
            };
            ComponentView.prototype.remove = function () {
                var parentView = this.getParentItemView();
                if (parentView) {
                    parentView.removeComponentView(this);
                }
                else {
                    _super.prototype.remove.call(this);
                }
                return this;
            };
            ComponentView.prototype.getType = function () {
                return _super.prototype.getType.call(this);
            };
            ComponentView.prototype.setComponent = function (component) {
                if (component) {
                    if (this.component) {
                        this.unregisterComponentListeners(this.component);
                    }
                    this.setTooltipObject(component);
                    this.registerComponentListeners(component);
                }
                this.component = component;
                this.refreshEmptyState();
            };
            ComponentView.prototype.getComponent = function () {
                return this.component;
            };
            ComponentView.prototype.hasComponentPath = function () {
                return this.component && this.component.hasPath();
            };
            ComponentView.prototype.getComponentPath = function () {
                return this.hasComponentPath() ? this.component.getPath() : null;
            };
            ComponentView.prototype.getName = function () {
                return this.component && this.component.getName() ? this.component.getName().toString() : null;
            };
            ComponentView.prototype.getParentItemView = function () {
                return this.parentRegionView;
            };
            ComponentView.prototype.setParentItemView = function (regionView) {
                _super.prototype.setParentItemView.call(this, regionView);
                this.parentRegionView = regionView;
            };
            ComponentView.prototype.setMoving = function (value) {
                this.moving = value;
            };
            ComponentView.prototype.isMoving = function () {
                return this.moving;
            };
            ComponentView.prototype.clone = function () {
                var index = this.getParentItemView().getComponentViewIndex(this);
                var clone = this.getType().createView(new liveedit.CreateItemViewConfig().setParentView(this.getParentItemView()).setParentElement(this.getParentElement()).setData(this.getComponent()).setPositionIndex(index));
                return clone;
            };
            ComponentView.prototype.duplicate = function (duplicate) {
                var parentView = this.getParentItemView();
                var index = parentView.getComponentViewIndex(this);
                var duplicateView = this.getType().createView(new liveedit.CreateItemViewConfig().setParentView(this.getParentItemView()).setParentElement(this.getParentElement()).setData(duplicate).setPositionIndex(index + 1));
                parentView.addComponentView(duplicateView, index + 1);
                return duplicateView;
            };
            ComponentView.prototype.toString = function () {
                var extra = "";
                if (this.hasComponentPath()) {
                    extra = " : " + this.getComponentPath().toString();
                }
                return _super.prototype.toString.call(this) + extra;
            };
            ComponentView.prototype.replaceWith = function (replacement) {
                if (ComponentView.debug) {
                    console.log('ComponentView.replaceWith', this, replacement);
                }
                _super.prototype.replaceWith.call(this, replacement);
                var index = this.getParentItemView().getComponentViewIndex(this);
                this.unregisterComponentListeners(this.component);
                var parentRegionView = this.parentRegionView;
                this.parentRegionView.unregisterComponentView(this);
                parentRegionView.registerComponentView(replacement, index);
            };
            ComponentView.prototype.moveToRegion = function (toRegionView, precedingComponentView) {
                if (ComponentView.debug) {
                    console.log('ComponentView.moveToRegion', this, this.parentRegionView, toRegionView);
                }
                this.moving = false;
                var indexInNewParent = 0;
                var precedingComponent = null;
                if (precedingComponentView) {
                    precedingComponent = precedingComponentView.getComponent();
                    indexInNewParent = precedingComponent.getIndex() + 1;
                }
                if (this.parentRegionView.getRegionPath().equals(toRegionView.getRegionPath()) && indexInNewParent == this.parentRegionView.getComponentViewIndex(this)) {
                    if (ComponentView.debug) {
                        console.debug('Dropped in the same region at the same index, no need to move', this.parentRegionView, toRegionView);
                    }
                    return;
                }
                this.remove();
                toRegionView.addComponentView(this, indexInNewParent);
            };
            ComponentView.prototype.onItemViewAdded = function (listener) {
                this.itemViewAddedListeners.push(listener);
            };
            ComponentView.prototype.unItemViewAdded = function (listener) {
                this.itemViewAddedListeners = this.itemViewAddedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            ComponentView.prototype.notifyItemViewAdded = function (view) {
                var event = new liveedit.ItemViewAddedEvent(view);
                this.itemViewAddedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            ComponentView.prototype.onItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners.push(listener);
            };
            ComponentView.prototype.unItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners = this.itemViewRemovedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            ComponentView.prototype.notifyItemViewRemoved = function (view) {
                var event = new liveedit.ItemViewRemovedEvent(view);
                this.itemViewRemovedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            ComponentView.findParentRegionViewHTMLElement = function (htmlElement) {
                var parentItemView = liveedit.ItemView.findParentItemViewAsHTMLElement(htmlElement);
                while (!liveedit.RegionView.isRegionViewFromHTMLElement(parentItemView)) {
                    parentItemView = liveedit.ItemView.findParentItemViewAsHTMLElement(parentItemView);
                }
                return parentItemView;
            };
            ComponentView.findPrecedingComponentItemViewId = function (htmlElement) {
                var previousItemView = liveedit.ItemView.findPreviousItemView(htmlElement);
                if (!previousItemView) {
                    return null;
                }
                var asString = previousItemView.getData(liveedit.ItemViewId.DATA_ATTRIBUTE);
                if (api.util.StringHelper.isEmpty(asString)) {
                    return null;
                }
                return liveedit.ItemViewId.fromString(asString);
            };
            ComponentView.prototype.handleDragStart2 = function (event) {
                if (event.target === this.getHTMLElement()) {
                    event.dataTransfer.effectAllowed = "move";
                    console.log("ComponentView[" + this.getItemId().toNumber() + "].handleDragStart", event, this.getHTMLElement());
                    this.hideTooltip();
                }
            };
            ComponentView.prototype.handleDrag = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ComponentView[" + this.getItemId().toNumber() + "].handleDrag", event, this.getHTMLElement());
                }
            };
            ComponentView.prototype.handleDragEnd = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ComponentView[" + this.getItemId().toNumber() + "].handleDragEnd", event, this.getHTMLElement());
                }
            };
            ComponentView.debug = false;
            return ComponentView;
        })(liveedit.ItemView);
        liveedit.ComponentView = ComponentView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var GetPageDescriptorsByModulesRequest = api.content.page.GetPageDescriptorsByModulesRequest;
        var SetController = api.content.page.SetController;
        var SiteModel = api.content.site.SiteModel;
        var PageDescriptorDropdown = api.content.page.PageDescriptorDropdown;
        var PagePlaceholder = (function (_super) {
            __extends(PagePlaceholder, _super);
            function PagePlaceholder(pageView) {
                var _this = this;
                _super.call(this);
                this.addClass("page-placeholder");
                this.pageView = pageView;
                var moduleKeys = pageView.liveEditModel.getSiteModel().getModuleKeys(), request = new GetPageDescriptorsByModulesRequest(moduleKeys), loader = new api.util.loader.BaseLoader(request);
                this.controllerDropdown = new PageDescriptorDropdown('page-controller', {
                    loader: loader
                });
                this.controllerDropdown.load();
                this.controllerDropdown.onOptionSelected(function (event) {
                    var setController = new SetController(_this).setDescriptor(event.getOption().displayValue);
                    _this.pageView.liveEditModel.getPageModel().setController(setController);
                });
                this.appendChild(this.controllerDropdown);
                this.controllerDropdown.onClicked(function (event) {
                    _this.select();
                    event.stopPropagation();
                });
                pageView.liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                    if (event.getPropertyName() == SiteModel.PROPERTY_NAME_MODULE_CONFIGS) {
                        request.setModuleKeys(pageView.liveEditModel.getSiteModel().getModuleKeys());
                        loader.load();
                    }
                });
            }
            PagePlaceholder.prototype.select = function () {
                this.controllerDropdown.giveFocus();
            };
            PagePlaceholder.prototype.deselect = function () {
            };
            return PagePlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.PagePlaceholder = PagePlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageMode = api.content.page.PageMode;
        var PageViewBuilder = (function () {
            function PageViewBuilder() {
            }
            PageViewBuilder.prototype.setLiveEditModel = function (value) {
                this.liveEditModel = value;
                return this;
            };
            PageViewBuilder.prototype.setItemViewProducer = function (value) {
                this.itemViewProducer = value;
                return this;
            };
            PageViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            PageViewBuilder.prototype.build = function () {
                return new PageView(this);
            };
            return PageViewBuilder;
        })();
        liveedit.PageViewBuilder = PageViewBuilder;
        var PageView = (function (_super) {
            __extends(PageView, _super);
            function PageView(builder) {
                var _this = this;
                this.liveEditModel = builder.liveEditModel;
                this.pageModel = builder.liveEditModel.getPageModel();
                this.pageModel.onPropertyChanged(function () { return _this.refreshEmptyState(); });
                this.regionViews = [];
                this.viewsById = {};
                this.itemViewAddedListeners = [];
                this.itemViewRemovedListeners = [];
                PageView.debug = true;
                var resetAction = new api.ui.Action('Reset');
                if (this.pageModel.getMode() == 0 /* AUTOMATIC */ || this.pageModel.getMode() == 3 /* NO_CONTROLLER */) {
                    resetAction.setEnabled(false);
                }
                resetAction.onExecuted(function () {
                    _this.pageModel.reset(_this);
                    _this.setLocked(true);
                });
                this.pageModel.onPageModeChanged(function (event) {
                    var resetEnabled = !(event.getNewMode() != 0 /* AUTOMATIC */ && event.getNewMode() != 3 /* NO_CONTROLLER */);
                    resetAction.setEnabled(resetEnabled);
                });
                this.unlockedScreenActions = [resetAction];
                this.itemViewAddedListener = function (event) {
                    event.getView().toItemViewArray().forEach(function (itemView) {
                        _this.registerItemView(itemView);
                    });
                    _this.exitTextEditModeIfNeeded();
                };
                this.itemViewRemovedListener = function (event) {
                    event.getView().toItemViewArray().forEach(function (itemView) {
                        _this.unregisterItemView(itemView);
                    });
                };
                _super.call(this, new liveedit.ItemViewBuilder().setLiveEditModel(builder.liveEditModel).setItemViewIdProducer(builder.itemViewProducer).setPlaceholder(new liveedit.PagePlaceholder(this)).setTooltipViewer(new api.content.ContentSummaryViewer()).setType(liveedit.PageItemType.get()).setElement(builder.element).setParentElement(builder.element.getParentElement()).setContextMenuActions(this.unlockedScreenActions).setContextMenuTitle(new liveedit.PageViewContextMenuTitle(builder.liveEditModel.getContent())));
                this.addClass('page-view');
                this.setTooltipObject(builder.liveEditModel.getContent());
                this.parseItemViews();
                this.appendChild(this.createTextModeToolbar());
                this.refreshEmptyState();
                if (!this.liveEditModel.getContent().isPageTemplate() && !this.isPageModified(this.pageModel)) {
                    this.setLocked(true);
                }
                this.listenToMouseEvents();
            }
            PageView.prototype.isPageModified = function (pageModel) {
                return pageModel.getDefaultPageTemplate().isPage() && !pageModel.getDefaultPageTemplate().getRegions().equals(pageModel.getRegions());
            };
            PageView.prototype.showTooltip = function () {
                if (!this.isTextEditMode() && !this.isLocked()) {
                    _super.prototype.showTooltip.call(this);
                }
            };
            PageView.prototype.highlight = function () {
                if (!this.isTextEditMode() && !this.isLocked()) {
                    _super.prototype.highlight.call(this);
                }
            };
            PageView.prototype.showCursor = function () {
                if (!this.isTextEditMode() && !this.isLocked()) {
                    _super.prototype.showCursor.call(this);
                }
            };
            PageView.prototype.shade = function () {
                if (!this.isEmpty()) {
                    _super.prototype.shade.call(this);
                }
            };
            PageView.prototype.showContextMenu = function (clickPosition, menuPosition) {
                if (!this.isLocked()) {
                    _super.prototype.showContextMenu.call(this, clickPosition, menuPosition);
                }
            };
            PageView.prototype.unshade = function () {
                if (!this.isLocked()) {
                    _super.prototype.unshade.call(this);
                }
            };
            PageView.prototype.listenToMouseEvents = function () {
                var _this = this;
                liveedit.Shader.get().onUnlockClicked(function (event) {
                    if (_this.isLocked()) {
                        _this.setLocked(false);
                    }
                });
            };
            PageView.prototype.select = function (clickPosition, menuPosition) {
                _super.prototype.select.call(this, clickPosition, menuPosition);
                new liveedit.PageSelectedEvent(this).fire();
            };
            PageView.prototype.handleShaderClick = function (event) {
                if (!this.isLocked() && this.isSelected()) {
                    this.deselect();
                }
            };
            PageView.prototype.handleClick = function (event) {
                event.stopPropagation();
                if (this.isTextEditMode()) {
                    this.setTextEditMode(false);
                }
                else {
                    _super.prototype.handleClick.call(this, event);
                }
            };
            PageView.prototype.isLocked = function () {
                return this.hasClass('locked');
            };
            PageView.prototype.setLocked = function (locked) {
                this.toggleClass('locked', locked);
                if (locked) {
                    this.shade();
                    new liveedit.PageLockedEvent(this).fire();
                }
                else {
                    this.unshade();
                    if (!this.pageModel.isPageTemplate() || this.pageModel.getMode() == 0 /* AUTOMATIC */) {
                        this.pageModel.initializePageFromDefault(this);
                    }
                }
            };
            PageView.prototype.createTextModeToolbar = function () {
                var _this = this;
                var toolbar = new api.dom.DivEl('text-edit-toolbar');
                var wrapper = new api.dom.DivEl('wrapper');
                wrapper.setHtml('Text Edit Mode');
                var closeButton = new api.ui.button.CloseButton('no-bg');
                closeButton.onClicked(function (event) {
                    event.stopPropagation();
                    event.preventDefault();
                    _this.setTextEditMode(false);
                });
                wrapper.appendChild(closeButton);
                toolbar.appendChild(wrapper);
                return toolbar;
            };
            PageView.prototype.isTextEditMode = function () {
                return this.hasClass('text-edit-mode');
            };
            PageView.prototype.setTextEditMode = function (flag) {
                this.toggleClass('text-edit-mode', flag);
                var textItemViews = this.getItemViewsByType(api.liveedit.text.TextItemType.get());
                var textView;
                textItemViews.forEach(function (view) {
                    textView = view;
                    if (textView.isEditMode() != flag) {
                        textView.setEditMode(flag);
                    }
                });
                if (flag) {
                    new liveedit.PageTextModeStartedEvent(this).fire();
                }
            };
            PageView.prototype.isEmpty = function () {
                return !this.pageModel || this.pageModel.getMode() == 3 /* NO_CONTROLLER */;
            };
            PageView.prototype.getName = function () {
                return this.liveEditModel.getContent() ? this.liveEditModel.getContent().getDisplayName() : "[No name]";
            };
            PageView.prototype.getParentItemView = function () {
                return null;
            };
            PageView.prototype.setParentItemView = function (itemView) {
                throw new Error("PageView is the topmost item view and cannot have a parent");
            };
            PageView.prototype.registerRegionView = function (regionView) {
                this.regionViews.push(regionView);
                regionView.onItemViewAdded(this.itemViewAddedListener);
                regionView.onItemViewRemoved(this.itemViewRemovedListener);
            };
            PageView.prototype.unregisterRegionView = function (regionView) {
                var index = this.regionViews.indexOf(regionView);
                if (index > -1) {
                    this.regionViews.splice(index, 1);
                    regionView.unItemViewAdded(this.itemViewAddedListener);
                    regionView.unItemViewRemoved(this.itemViewRemovedListener);
                }
            };
            PageView.prototype.getRegions = function () {
                return this.regionViews;
            };
            PageView.prototype.toItemViewArray = function () {
                var array = [];
                array.push(this);
                this.regionViews.forEach(function (regionView) {
                    var itemViews = regionView.toItemViewArray();
                    array = array.concat(itemViews);
                });
                return array;
            };
            PageView.prototype.hasSelectedView = function () {
                return !!this.getSelectedView();
            };
            PageView.prototype.getSelectedView = function () {
                for (var id in this.viewsById) {
                    if (this.viewsById.hasOwnProperty(id) && this.viewsById[id].isSelected()) {
                        return this.viewsById[id];
                    }
                }
                return null;
            };
            PageView.prototype.getItemViewById = function (id) {
                api.util.assertNotNull(id, "value cannot be null");
                return this.viewsById[id.toNumber()];
            };
            PageView.prototype.getItemViewsByType = function (type) {
                var views = [];
                for (var key in this.viewsById) {
                    if (this.viewsById.hasOwnProperty(key)) {
                        var view = this.viewsById[key];
                        if (type.equals(view.getType())) {
                            views.push(view);
                        }
                    }
                }
                return views;
            };
            PageView.prototype.getItemViewByElement = function (element) {
                api.util.assertNotNull(element, "element cannot be null");
                var itemId = liveedit.ItemView.parseItemId(element);
                if (!itemId) {
                    return null;
                }
                var itemView = this.getItemViewById(itemId);
                api.util.assertNotNull(itemView, "ItemView not found: " + itemId.toString());
                return itemView;
            };
            PageView.prototype.getRegionViewByElement = function (element) {
                api.util.assertNotNull(element, "element cannot be null");
                var itemId = liveedit.ItemView.parseItemId(element);
                if (!itemId) {
                    return null;
                }
                var itemView = this.getItemViewById(itemId);
                api.util.assertNotNull(itemView, "ItemView not found: " + itemId.toString());
                if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, liveedit.RegionView)) {
                    return itemView;
                }
                return null;
            };
            PageView.prototype.getComponentViewByElement = function (element) {
                api.util.assertNotNull(element, "element cannot be null");
                var itemId = liveedit.ItemView.parseItemId(element);
                if (!itemId) {
                    return null;
                }
                var itemView = this.getItemViewById(itemId);
                api.util.assertNotNull(itemView, "ItemView not found: " + itemId.toString());
                if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, liveedit.ComponentView)) {
                    return itemView;
                }
                return null;
            };
            PageView.prototype.getRegionViewByPath = function (path) {
                for (var i = 0; i < this.regionViews.length; i++) {
                    var regionView = this.regionViews[i];
                    if (path.hasParentComponentPath()) {
                        var componentView = this.getComponentViewByPath(path.getParentComponentPath());
                        if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, api.liveedit.layout.LayoutComponentView)) {
                            var layoutView = componentView;
                            layoutView.getRegionViewByName(path.getRegionName());
                        }
                    }
                    else {
                        if (path.getRegionName() == regionView.getRegionName()) {
                            return regionView;
                        }
                    }
                }
                return null;
            };
            PageView.prototype.getComponentViewByPath = function (path) {
                var firstLevelOfPath = path.getFirstLevel();
                for (var i = 0; i < this.regionViews.length; i++) {
                    var regionView = this.regionViews[i];
                    if (firstLevelOfPath.getRegionName() == regionView.getRegionName()) {
                        if (path.numberOfLevels() == 1) {
                            return regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                        }
                        else {
                            var layoutView = regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                            return layoutView.getComponentViewByPath(path.removeFirstLevel());
                        }
                    }
                }
                return null;
            };
            PageView.prototype.exitTextEditModeIfNeeded = function () {
                if (this.isTextEditMode()) {
                    this.setTextEditMode(false);
                }
            };
            PageView.prototype.registerItemView = function (view) {
                if (PageView.debug) {
                    console.debug("PageView.registerItemView: " + view.toString());
                }
                this.viewsById[view.getItemId().toNumber()] = view;
                this.notifyItemViewAdded(view);
            };
            PageView.prototype.unregisterItemView = function (view) {
                if (PageView.debug) {
                    console.debug("PageView.unregisterItemView: " + view.toString());
                }
                delete this.viewsById[view.getItemId().toNumber()];
                this.notifyItemViewRemoved(view);
            };
            PageView.prototype.parseItemViews = function () {
                var _this = this;
                this.doParseItemViews();
                this.toItemViewArray().forEach(function (itemView) {
                    _this.registerItemView(itemView);
                });
            };
            PageView.prototype.doParseItemViews = function (parentElement) {
                var _this = this;
                var pageRegions = this.liveEditModel.getPageModel().getRegions();
                if (!pageRegions) {
                    return;
                }
                var regions = pageRegions.getRegions();
                var children = parentElement ? parentElement.getChildren() : this.getChildren();
                var regionIndex = 0;
                children.forEach(function (childElement) {
                    var itemType = liveedit.ItemType.fromElement(childElement);
                    if (itemType) {
                        if (liveedit.RegionItemType.get().equals(itemType)) {
                            var region = regions[regionIndex++];
                            if (region) {
                                var regionView = new liveedit.RegionView(new liveedit.RegionViewBuilder().setLiveEditModel(_this.liveEditModel).setParentView(_this).setRegion(region).setElement(childElement));
                                _this.registerRegionView(regionView);
                            }
                        }
                        else {
                            _this.doParseItemViews(childElement);
                        }
                    }
                    else {
                        _this.doParseItemViews(childElement);
                    }
                });
            };
            PageView.prototype.onItemViewAdded = function (listener) {
                this.itemViewAddedListeners.push(listener);
            };
            PageView.prototype.unItemViewAdded = function (listener) {
                this.itemViewAddedListeners = this.itemViewAddedListeners.filter(function (current) { return (current != listener); });
            };
            PageView.prototype.notifyItemViewAdded = function (itemView) {
                var event = new liveedit.ItemViewAddedEvent(itemView);
                this.itemViewAddedListeners.forEach(function (listener) { return listener(event); });
            };
            PageView.prototype.onItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners.push(listener);
            };
            PageView.prototype.unItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners = this.itemViewRemovedListeners.filter(function (current) { return (current != listener); });
            };
            PageView.prototype.notifyItemViewRemoved = function (itemView) {
                var event = new liveedit.ItemViewRemovedEvent(itemView);
                this.itemViewRemovedListeners.forEach(function (listener) { return listener(event); });
            };
            return PageView;
        })(liveedit.ItemView);
        liveedit.PageView = PageView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ContentViewBuilder = (function () {
            function ContentViewBuilder() {
            }
            ContentViewBuilder.prototype.setParentPartComponentView = function (value) {
                this.parentPartComponentView = value;
                return this;
            };
            ContentViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            ContentViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            return ContentViewBuilder;
        })();
        liveedit.ContentViewBuilder = ContentViewBuilder;
        var ContentView = (function (_super) {
            __extends(ContentView, _super);
            function ContentView(builder) {
                _super.call(this, new liveedit.ItemViewBuilder().setItemViewIdProducer(builder.parentPartComponentView.getItemViewIdProducer()).setType(liveedit.ContentItemType.get()).setElement(builder.element).setParentElement(builder.parentElement).setParentView(builder.parentPartComponentView).setContextMenuActions(this.createContentContextMenuActions()).setContextMenuTitle(new liveedit.ContentViewContextMenuTitle(this)));
                this.parentPartComponentView = builder.parentPartComponentView;
            }
            ContentView.prototype.createContentContextMenuActions = function () {
                var _this = this;
                var actions = [];
                actions.push(new api.ui.Action('Parent').onExecuted(function () {
                    var parentView = _this.getParentItemView();
                    if (parentView) {
                        _this.deselect();
                        parentView.select(null, 0 /* TOP */);
                    }
                }));
                actions.push(new api.ui.Action('Insert').onExecuted(function () {
                }));
                actions.push(new api.ui.Action('View').onExecuted(function () {
                }));
                actions.push(new api.ui.Action('Edit').onExecuted(function () {
                }));
                return actions;
            };
            ContentView.prototype.isEmpty = function () {
                return false;
            };
            ContentView.prototype.getParentItemView = function () {
                return this.parentPartComponentView;
            };
            ContentView.prototype.setParentItemView = function (partView) {
                _super.prototype.setParentItemView.call(this, partView);
                this.parentPartComponentView = partView;
            };
            ContentView.prototype.select = function (clickPosition, menuPosition) {
                _super.prototype.select.call(this, clickPosition, menuPosition);
            };
            return ContentView;
        })(liveedit.ItemView);
        liveedit.ContentView = ContentView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageLockedEvent = (function (_super) {
            __extends(PageLockedEvent, _super);
            function PageLockedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageLockedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageLockedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageLockedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageLockedEvent;
        })(api.event.Event);
        liveedit.PageLockedEvent = PageLockedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageTextModeStartedEvent = (function (_super) {
            __extends(PageTextModeStartedEvent, _super);
            function PageTextModeStartedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageTextModeStartedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageTextModeStartedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageTextModeStartedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageTextModeStartedEvent;
        })(api.event.Event);
        liveedit.PageTextModeStartedEvent = PageTextModeStartedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ImageOpenUploadDialogEvent = (function (_super) {
            __extends(ImageOpenUploadDialogEvent, _super);
            function ImageOpenUploadDialogEvent(target) {
                _super.call(this);
                this.targetImagePlaceholder = target;
            }
            ImageOpenUploadDialogEvent.prototype.getTargetImagePlaceholder = function () {
                return this.targetImagePlaceholder;
            };
            ImageOpenUploadDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ImageOpenUploadDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ImageOpenUploadDialogEvent;
        })(api.event.Event);
        liveedit.ImageOpenUploadDialogEvent = ImageOpenUploadDialogEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ImageUploadedEvent = (function (_super) {
            __extends(ImageUploadedEvent, _super);
            function ImageUploadedEvent(uploadedItem, target) {
                _super.call(this);
                this.uploadedItem = uploadedItem;
                this.targetImagePlaceholder = target;
            }
            ImageUploadedEvent.prototype.getUploadedItem = function () {
                return this.uploadedItem;
            };
            ImageUploadedEvent.prototype.getTargetImagePlaceholder = function () {
                return this.targetImagePlaceholder;
            };
            ImageUploadedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ImageUploadedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ImageUploadedEvent;
        })(api.event.Event);
        liveedit.ImageUploadedEvent = ImageUploadedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var DraggingComponentViewStartedEvent = (function (_super) {
            __extends(DraggingComponentViewStartedEvent, _super);
            function DraggingComponentViewStartedEvent() {
                _super.apply(this, arguments);
            }
            DraggingComponentViewStartedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            DraggingComponentViewStartedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return DraggingComponentViewStartedEvent;
        })(Event);
        liveedit.DraggingComponentViewStartedEvent = DraggingComponentViewStartedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var DraggingComponentViewCompletedEvent = (function (_super) {
            __extends(DraggingComponentViewCompletedEvent, _super);
            function DraggingComponentViewCompletedEvent(componentView) {
                _super.call(this);
                api.util.assertNotNull(componentView, "componentView cannot be null");
                this.componentView = componentView;
            }
            DraggingComponentViewCompletedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            DraggingComponentViewCompletedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            DraggingComponentViewCompletedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return DraggingComponentViewCompletedEvent;
        })(Event);
        liveedit.DraggingComponentViewCompletedEvent = DraggingComponentViewCompletedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var DraggingComponentViewCanceledEvent = (function (_super) {
            __extends(DraggingComponentViewCanceledEvent, _super);
            function DraggingComponentViewCanceledEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            DraggingComponentViewCanceledEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            DraggingComponentViewCanceledEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            DraggingComponentViewCanceledEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return DraggingComponentViewCanceledEvent;
        })(Event);
        liveedit.DraggingComponentViewCanceledEvent = DraggingComponentViewCanceledEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ItemFromContextWindowDroppedEvent = (function (_super) {
            __extends(ItemFromContextWindowDroppedEvent, _super);
            function ItemFromContextWindowDroppedEvent() {
                _super.apply(this, arguments);
            }
            ItemFromContextWindowDroppedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ItemFromContextWindowDroppedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ItemFromContextWindowDroppedEvent;
        })(Event);
        liveedit.ItemFromContextWindowDroppedEvent = ItemFromContextWindowDroppedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageSelectedEvent = (function (_super) {
            __extends(PageSelectedEvent, _super);
            function PageSelectedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageSelectedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageSelectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageSelectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageSelectedEvent;
        })(api.event.Event);
        liveedit.PageSelectedEvent = PageSelectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionSelectedEvent = (function (_super) {
            __extends(RegionSelectedEvent, _super);
            function RegionSelectedEvent(regionView) {
                _super.call(this);
                this.pageItemView = regionView;
            }
            RegionSelectedEvent.prototype.getRegionView = function () {
                return this.pageItemView;
            };
            RegionSelectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            RegionSelectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return RegionSelectedEvent;
        })(api.event.Event);
        liveedit.RegionSelectedEvent = RegionSelectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentAddedEvent = (function (_super) {
            __extends(ComponentAddedEvent, _super);
            function ComponentAddedEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            ComponentAddedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentAddedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentAddedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentAddedEvent;
        })(api.event.Event);
        liveedit.ComponentAddedEvent = ComponentAddedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentRemovedEvent = (function (_super) {
            __extends(ComponentRemovedEvent, _super);
            function ComponentRemovedEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            ComponentRemovedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentRemovedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentRemovedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentRemovedEvent;
        })(api.event.Event);
        liveedit.ComponentRemovedEvent = ComponentRemovedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentResetEvent = (function (_super) {
            __extends(ComponentResetEvent, _super);
            function ComponentResetEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            ComponentResetEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentResetEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentResetEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentResetEvent;
        })(api.event.Event);
        liveedit.ComponentResetEvent = ComponentResetEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentDuplicatedEvent = (function (_super) {
            __extends(ComponentDuplicatedEvent, _super);
            function ComponentDuplicatedEvent(originalComponentView, duplicatedComponentView) {
                _super.call(this);
                this.originalComponentView = originalComponentView;
                this.duplicatedComponentView = duplicatedComponentView;
            }
            ComponentDuplicatedEvent.prototype.getOriginalComponentView = function () {
                return this.originalComponentView;
            };
            ComponentDuplicatedEvent.prototype.getDuplicatedComponentView = function () {
                return this.duplicatedComponentView;
            };
            ComponentDuplicatedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentDuplicatedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentDuplicatedEvent;
        })(api.event.Event);
        liveedit.ComponentDuplicatedEvent = ComponentDuplicatedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentLoadedEvent = (function (_super) {
            __extends(ComponentLoadedEvent, _super);
            function ComponentLoadedEvent(itemView) {
                _super.call(this);
                this.itemView = itemView;
            }
            ComponentLoadedEvent.prototype.getItemView = function () {
                return this.itemView;
            };
            ComponentLoadedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentLoadedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentLoadedEvent;
        })(Event);
        liveedit.ComponentLoadedEvent = ComponentLoadedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var LiveEditPageViewReadyEvent = (function (_super) {
            __extends(LiveEditPageViewReadyEvent, _super);
            function LiveEditPageViewReadyEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            LiveEditPageViewReadyEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            LiveEditPageViewReadyEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            LiveEditPageViewReadyEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return LiveEditPageViewReadyEvent;
        })(api.event.Event);
        liveedit.LiveEditPageViewReadyEvent = LiveEditPageViewReadyEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (image) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var ImageItemType = (function (_super) {
                __extends(ImageItemType, _super);
                function ImageItemType() {
                    _super.call(this, "image", {
                        cssSelector: '[data-portal-component-type=image]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-image',
                        highlighterStyle: {
                            stroke: 'rgba(68, 68, 68, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(255, 255, 255, 0)'
                        },
                        contextMenuConfig: ['parent', 'remove', 'clear', 'duplicate']
                    });
                }
                ImageItemType.get = function () {
                    return ImageItemType.INSTANCE;
                };
                ImageItemType.prototype.createView = function (config) {
                    return new image.ImageComponentView(new image.ImageComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setElement(config.element).setComponent(config.data).setPositionIndex(config.positionIndex));
                };
                ImageItemType.prototype.isComponentType = function () {
                    return true;
                };
                ImageItemType.INSTANCE = new ImageItemType();
                return ImageItemType;
            })(ComponentItemType);
            image.ImageItemType = ImageItemType;
            ImageItemType.get();
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (_image) {
            var ComponentView = api.liveedit.ComponentView;
            var ImageComponentViewBuilder = (function (_super) {
                __extends(ImageComponentViewBuilder, _super);
                function ImageComponentViewBuilder() {
                    _super.call(this);
                    this.setType(_image.ImageItemType.get());
                }
                return ImageComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            _image.ImageComponentViewBuilder = ImageComponentViewBuilder;
            var ImageComponentView = (function (_super) {
                __extends(ImageComponentView, _super);
                function ImageComponentView(builder) {
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.imageComponent = builder.component;
                    _super.call(this, builder.setPlaceholder(new _image.ImagePlaceholder(this)).setTooltipViewer(new _image.ImageComponentViewer()));
                    this.initializeImage();
                    this.addClass('image-view');
                }
                ImageComponentView.prototype.initializeImage = function () {
                    var _this = this;
                    var figureElChildren = this.getChildren();
                    for (var i = 0; i < figureElChildren.length; i++) {
                        var image = figureElChildren[i];
                        if (image.getHTMLElement().tagName.toUpperCase() == 'IMG') {
                            this.image = image;
                            this.image.getEl().addEventListener("load", function (event) {
                                if (_this.isSelected()) {
                                    _this.highlight();
                                    _this.shade();
                                }
                            });
                        }
                        return;
                    }
                };
                ImageComponentView.prototype.isEmpty = function () {
                    return !this.imageComponent || this.imageComponent.isEmpty();
                };
                return ImageComponentView;
            })(ComponentView);
            _image.ImageComponentView = ImageComponentView;
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (image) {
            var ImageUploadedEvent = api.liveedit.ImageUploadedEvent;
            var ImageOpenUploadDialogEvent = api.liveedit.ImageOpenUploadDialogEvent;
            var ContentTypeName = api.schema.content.ContentTypeName;
            var ImagePlaceholder = (function (_super) {
                __extends(ImagePlaceholder, _super);
                function ImagePlaceholder(imageView) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("image-placeholder");
                    this.imageComponentView = imageView;
                    var imageUploadHandler = function (event) {
                        if (event.getTargetImagePlaceholder() === _this) {
                            var createdImage = event.getUploadedItem();
                            var component = _this.imageComponentView.getComponent();
                            component.setImage(createdImage.getContentId(), createdImage.getDisplayName());
                        }
                    };
                    ImageUploadedEvent.on(imageUploadHandler);
                    this.onRemoved(function (event) { return ImageUploadedEvent.un(imageUploadHandler); });
                    this.comboboxWrapper = new api.dom.DivEl('rich-combobox-wrapper');
                    this.uploadButton = new api.dom.ButtonEl("button upload-button");
                    this.uploadButton.addClass("upload-button");
                    this.uploadButton.onClicked(function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        new ImageOpenUploadDialogEvent(_this).fire();
                    });
                    this.comboBox = api.content.ContentComboBox.create().setMaximumOccurrences(1).setAllowedContentTypes([ContentTypeName.IMAGE.toString()]).setLoader(new api.content.ContentSummaryLoader()).setMinWidth(270).build();
                    this.comboboxWrapper.appendChildren(this.comboBox, this.uploadButton);
                    this.appendChild(this.comboboxWrapper);
                    this.comboBox.onOptionSelected(function (event) {
                        var component = _this.imageComponentView.getComponent();
                        var imageContent = event.getOption().displayValue;
                        component.setImage(imageContent.getContentId(), imageContent.getDisplayName());
                        _this.uploadButton.hide();
                        _this.imageComponentView.showLoadingSpinner();
                    });
                }
                ImagePlaceholder.prototype.select = function () {
                    this.comboboxWrapper.show();
                    this.comboBox.giveFocus();
                };
                ImagePlaceholder.prototype.deselect = function () {
                    this.comboboxWrapper.hide();
                };
                return ImagePlaceholder;
            })(api.liveedit.ComponentPlaceholder);
            image.ImagePlaceholder = ImagePlaceholder;
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (image) {
            var ImageComponentViewer = (function (_super) {
                __extends(ImageComponentViewer, _super);
                function ImageComponentViewer() {
                    _super.call(this);
                }
                ImageComponentViewer.prototype.resolveDisplayName = function (object) {
                    return !!object.getName() ? object.getName().toString() : "";
                };
                ImageComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                ImageComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-image";
                };
                return ImageComponentViewer;
            })(api.ui.NamesAndIconViewer);
            image.ImageComponentViewer = ImageComponentViewer;
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var PartItemType = (function (_super) {
                __extends(PartItemType, _super);
                function PartItemType() {
                    _super.call(this, "part", {
                        cssSelector: '[data-portal-component-type=part]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-part',
                        highlighterStyle: {
                            stroke: 'rgba(68, 68, 68, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(255, 255, 255, 0)'
                        },
                        contextMenuConfig: ['parent', 'remove', 'clear', 'duplicate']
                    });
                }
                PartItemType.get = function () {
                    return PartItemType.INSTANCE;
                };
                PartItemType.prototype.isComponentType = function () {
                    return true;
                };
                PartItemType.prototype.createView = function (config) {
                    return new part.PartComponentView(new part.PartComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setComponent(config.data).setElement(config.element).setPositionIndex(config.positionIndex));
                };
                PartItemType.INSTANCE = new PartItemType();
                return PartItemType;
            })(ComponentItemType);
            part.PartItemType = PartItemType;
            PartItemType.get();
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var SiteModel = api.content.site.SiteModel;
            var PartDescriptorLoader = api.content.page.region.PartDescriptorLoader;
            var PartDescriptorComboBox = api.content.page.region.PartDescriptorComboBox;
            var GetPartDescriptorsByModulesRequest = api.content.page.region.GetPartDescriptorsByModulesRequest;
            var PartPlaceholder = (function (_super) {
                __extends(PartPlaceholder, _super);
                function PartPlaceholder(partView) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("part-placeholder");
                    this.partComponentView = partView;
                    var request = new GetPartDescriptorsByModulesRequest(partView.liveEditModel.getSiteModel().getModuleKeys());
                    var loader = new PartDescriptorLoader(request);
                    loader.setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                    this.comboBox = new PartDescriptorComboBox(loader);
                    loader.load();
                    this.appendChild(this.comboBox);
                    this.comboBox.onOptionSelected(function (event) {
                        _this.partComponentView.showLoadingSpinner();
                        var descriptor = event.getOption().displayValue;
                        var partComponent = _this.partComponentView.getComponent();
                        partComponent.setDescriptor(descriptor.getKey(), descriptor);
                    });
                    partView.liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                        if (event.getPropertyName() == SiteModel.PROPERTY_NAME_MODULE_CONFIGS) {
                            request.setModuleKeys(partView.liveEditModel.getSiteModel().getModuleKeys());
                            loader.load();
                        }
                    });
                }
                PartPlaceholder.prototype.select = function () {
                    this.comboBox.show();
                    this.comboBox.giveFocus();
                };
                PartPlaceholder.prototype.deselect = function () {
                    this.comboBox.hide();
                };
                return PartPlaceholder;
            })(liveedit.ComponentPlaceholder);
            part.PartPlaceholder = PartPlaceholder;
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var ComponentView = api.liveedit.ComponentView;
            var ContentView = api.liveedit.ContentView;
            var PartComponentViewBuilder = (function (_super) {
                __extends(PartComponentViewBuilder, _super);
                function PartComponentViewBuilder() {
                    _super.call(this);
                    this.setType(part.PartItemType.get());
                }
                return PartComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            part.PartComponentViewBuilder = PartComponentViewBuilder;
            var PartComponentView = (function (_super) {
                __extends(PartComponentView, _super);
                function PartComponentView(builder) {
                    this.contentViews = [];
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.partComponent = builder.component;
                    _super.call(this, builder.setTooltipViewer(new part.PartComponentViewer()).setPlaceholder(new part.PartPlaceholder(this)));
                    this.addClass('part-view');
                    this.parseContentViews(this);
                }
                PartComponentView.prototype.addContent = function (view) {
                    this.contentViews.push(view);
                };
                PartComponentView.prototype.getContentViews = function () {
                    return this.contentViews;
                };
                PartComponentView.prototype.isEmpty = function () {
                    return !this.partComponent || this.partComponent.isEmpty();
                };
                PartComponentView.prototype.toItemViewArray = function () {
                    var array = [];
                    array.push(this);
                    this.contentViews.forEach(function (contentView) {
                        array = array.concat(contentView.toItemViewArray());
                    });
                    return array;
                };
                PartComponentView.prototype.parseContentViews = function (parentElement) {
                    var _this = this;
                    var children = parentElement ? parentElement.getChildren() : this.getChildren();
                    children.forEach(function (childElement) {
                        var itemType = liveedit.ItemType.fromElement(childElement);
                        if (itemType) {
                            if (liveedit.ContentItemType.get().equals(itemType)) {
                                var contentView = new ContentView(new liveedit.ContentViewBuilder().setParentPartComponentView(_this).setParentElement(parentElement ? parentElement : _this).setElement(childElement));
                                _this.addContent(contentView);
                            }
                            else {
                                _this.parseContentViews(childElement);
                            }
                        }
                    });
                };
                return PartComponentView;
            })(ComponentView);
            part.PartComponentView = PartComponentView;
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var PartComponentViewer = (function (_super) {
                __extends(PartComponentViewer, _super);
                function PartComponentViewer() {
                    _super.call(this);
                }
                PartComponentViewer.prototype.resolveDisplayName = function (object) {
                    return !!object.getName() ? object.getName().toString() : "";
                };
                PartComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                PartComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-part";
                };
                return PartComponentViewer;
            })(api.ui.NamesAndIconViewer);
            part.PartComponentViewer = PartComponentViewer;
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var LayoutItemType = (function (_super) {
                __extends(LayoutItemType, _super);
                function LayoutItemType() {
                    _super.call(this, "layout", {
                        cssSelector: '[data-portal-component-type=layout]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-layout',
                        highlighterStyle: {
                            stroke: 'rgba(255, 165, 0, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(100, 12, 36, 0)'
                        },
                        contextMenuConfig: ['parent', 'remove', 'clear', 'duplicate']
                    });
                }
                LayoutItemType.get = function () {
                    return LayoutItemType.INSTANCE;
                };
                LayoutItemType.prototype.isComponentType = function () {
                    return true;
                };
                LayoutItemType.prototype.createView = function (config) {
                    return new layout.LayoutComponentView(new layout.LayoutComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setComponent(config.data).setElement(config.element).setPositionIndex(config.positionIndex));
                };
                LayoutItemType.INSTANCE = new LayoutItemType();
                return LayoutItemType;
            })(ComponentItemType);
            layout.LayoutItemType = LayoutItemType;
            LayoutItemType.get();
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var ComponentView = api.liveedit.ComponentView;
            var RegionView = api.liveedit.RegionView;
            var LayoutComponentViewBuilder = (function (_super) {
                __extends(LayoutComponentViewBuilder, _super);
                function LayoutComponentViewBuilder() {
                    _super.call(this);
                    this.setType(layout.LayoutItemType.get());
                }
                return LayoutComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            layout.LayoutComponentViewBuilder = LayoutComponentViewBuilder;
            var LayoutComponentView = (function (_super) {
                __extends(LayoutComponentView, _super);
                function LayoutComponentView(builder) {
                    var _this = this;
                    this.regionViews = [];
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.layoutComponent = builder.component;
                    LayoutComponentView.debug = false;
                    this.itemViewAddedListener = function (event) { return _this.notifyItemViewAdded(event.getView()); };
                    this.itemViewRemovedListener = function (event) { return _this.notifyItemViewRemoved(event.getView()); };
                    _super.call(this, builder.setTooltipViewer(new layout.LayoutComponentViewer()).setPlaceholder(new layout.LayoutPlaceholder(this)));
                    this.addClass('layout-view');
                    this.parseRegions();
                }
                LayoutComponentView.prototype.getRegionViewByName = function (name) {
                    for (var i = 0; i < this.regionViews.length; i++) {
                        var regionView = this.regionViews[i];
                        if (regionView.getRegionName() == name) {
                            return regionView;
                        }
                    }
                    return null;
                };
                LayoutComponentView.prototype.getComponentViewByPath = function (path) {
                    var firstLevelOfPath = path.getFirstLevel();
                    for (var i = 0; i < this.regionViews.length; i++) {
                        var regionView = this.regionViews[i];
                        if (firstLevelOfPath.getRegionName() == regionView.getRegionName()) {
                            if (path.numberOfLevels() == 1) {
                                return regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                            }
                            else {
                                var layoutView = regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                                return layoutView.getComponentViewByPath(path.removeFirstLevel());
                            }
                        }
                    }
                    return null;
                };
                LayoutComponentView.prototype.setComponent = function (layoutComponent) {
                    _super.prototype.setComponent.call(this, layoutComponent);
                    var regions = layoutComponent.getRegions().getRegions();
                    this.regionViews.forEach(function (regionView, index) {
                        var region = regions[index];
                        regionView.setRegion(region);
                    });
                };
                LayoutComponentView.prototype.getRegions = function () {
                    return this.regionViews;
                };
                LayoutComponentView.prototype.isEmpty = function () {
                    return !this.layoutComponent || this.layoutComponent.isEmpty();
                };
                LayoutComponentView.prototype.toItemViewArray = function () {
                    var array = [];
                    array.push(this);
                    this.regionViews.forEach(function (regionView) {
                        var itemsInRegion = regionView.toItemViewArray();
                        array = array.concat(itemsInRegion);
                    });
                    return array;
                };
                LayoutComponentView.prototype.parseRegions = function () {
                    return this.doParseRegions();
                };
                LayoutComponentView.prototype.doParseRegions = function (parentElement) {
                    var _this = this;
                    var layoutComponent = this.getComponent();
                    var layoutRegions = layoutComponent.getRegions();
                    if (!layoutRegions) {
                        return;
                    }
                    var regions = layoutRegions.getRegions();
                    var children = parentElement ? parentElement.getChildren() : this.getChildren();
                    var regionIndex = 0;
                    children.forEach(function (childElement) {
                        var itemType = liveedit.ItemType.fromElement(childElement);
                        if (itemType) {
                            if (liveedit.RegionItemType.get().equals(itemType)) {
                                var region = regions[regionIndex++];
                                if (region) {
                                    var regionView = new RegionView(new liveedit.RegionViewBuilder().setParentView(_this).setParentElement(parentElement ? parentElement : _this).setRegion(region).setElement(childElement));
                                    _this.registerRegionView(regionView);
                                }
                            }
                            else {
                                _this.doParseRegions(childElement);
                            }
                        }
                        else {
                            _this.doParseRegions(childElement);
                        }
                    });
                };
                LayoutComponentView.prototype.registerRegionView = function (regionView) {
                    if (LayoutComponentView.debug) {
                        console.log('LayoutComponentView.registerRegionView: ' + regionView.toString());
                    }
                    this.regionViews.push(regionView);
                    this.notifyItemViewAdded(regionView);
                    regionView.onItemViewAdded(this.itemViewAddedListener);
                    regionView.onItemViewRemoved(this.itemViewRemovedListener);
                };
                LayoutComponentView.prototype.unregisterRegionView = function (regionView) {
                    if (LayoutComponentView.debug) {
                        console.log('LayoutComponentView.unregisterRegionView: ' + regionView.toString(), this.regionViews);
                    }
                    var index = this.regionViews.indexOf(regionView);
                    if (index > -1) {
                        this.regionViews.splice(index, 1);
                        this.notifyItemViewRemoved(regionView);
                        regionView.unItemViewAdded(this.itemViewAddedListener);
                        regionView.unItemViewRemoved(this.itemViewRemovedListener);
                    }
                };
                return LayoutComponentView;
            })(ComponentView);
            layout.LayoutComponentView = LayoutComponentView;
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var SiteModel = api.content.site.SiteModel;
            var GetLayoutDescriptorsByModulesRequest = api.content.page.region.GetLayoutDescriptorsByModulesRequest;
            var LayoutDescriptorLoader = api.content.page.region.LayoutDescriptorLoader;
            var LayoutDescriptorComboBox = api.content.page.region.LayoutDescriptorComboBox;
            var LayoutPlaceholder = (function (_super) {
                __extends(LayoutPlaceholder, _super);
                function LayoutPlaceholder(layoutView) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("layout-placeholder");
                    this.layoutComponentView = layoutView;
                    var request = new GetLayoutDescriptorsByModulesRequest(layoutView.liveEditModel.getSiteModel().getModuleKeys());
                    var loader = new LayoutDescriptorLoader(request);
                    loader.setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                    this.comboBox = new LayoutDescriptorComboBox(loader);
                    loader.load();
                    this.appendChild(this.comboBox);
                    this.comboBox.onOptionSelected(function (event) {
                        _this.layoutComponentView.showLoadingSpinner();
                        var descriptor = event.getOption().displayValue;
                        var layoutComponent = _this.layoutComponentView.getComponent();
                        layoutComponent.setDescriptor(descriptor.getKey(), descriptor);
                    });
                    layoutView.liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                        if (event.getPropertyName() == SiteModel.PROPERTY_NAME_MODULE_CONFIGS) {
                            request.setModuleKeys(layoutView.liveEditModel.getSiteModel().getModuleKeys());
                            loader.load();
                        }
                    });
                }
                LayoutPlaceholder.prototype.select = function () {
                    this.comboBox.show();
                    this.comboBox.giveFocus();
                };
                LayoutPlaceholder.prototype.deselect = function () {
                    this.comboBox.hide();
                };
                return LayoutPlaceholder;
            })(liveedit.ComponentPlaceholder);
            layout.LayoutPlaceholder = LayoutPlaceholder;
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var LayoutComponentViewer = (function (_super) {
                __extends(LayoutComponentViewer, _super);
                function LayoutComponentViewer() {
                    _super.call(this);
                }
                LayoutComponentViewer.prototype.resolveDisplayName = function (object) {
                    return !!object.getName() ? object.getName().toString() : "";
                };
                LayoutComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                LayoutComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-layout";
                };
                return LayoutComponentViewer;
            })(api.ui.NamesAndIconViewer);
            layout.LayoutComponentViewer = LayoutComponentViewer;
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var TextItemType = (function (_super) {
                __extends(TextItemType, _super);
                function TextItemType() {
                    _super.call(this, "text", {
                        cssSelector: '[data-portal-component-type=text]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-text',
                        highlighterStyle: {
                            stroke: 'rgba(152, 201, 242, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(255, 255, 255, 0)'
                        },
                        contextMenuConfig: ['parent', 'edit', 'remove', 'clear', 'duplicate']
                    });
                }
                TextItemType.get = function () {
                    return TextItemType.INSTANCE;
                };
                TextItemType.prototype.createView = function (config) {
                    return new text.TextComponentView(new text.TextComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setComponent(config.data).setElement(config.element).setPositionIndex(config.positionIndex));
                };
                TextItemType.prototype.isComponentType = function () {
                    return true;
                };
                TextItemType.INSTANCE = new TextItemType();
                return TextItemType;
            })(ComponentItemType);
            text.TextItemType = TextItemType;
            TextItemType.get();
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var TextPlaceholder = (function (_super) {
                __extends(TextPlaceholder, _super);
                function TextPlaceholder(componentView) {
                    this.textView = componentView;
                    _super.call(this);
                    this.addClass("text-placeholder");
                }
                return TextPlaceholder;
            })(liveedit.ComponentPlaceholder);
            text.TextPlaceholder = TextPlaceholder;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var MediumHeadersDropdownExtension = (function () {
                function MediumHeadersDropdownExtension() {
                    this.button = new api.dom.ButtonEl('medium-editor-action');
                    this.button.onClicked(this.onClick.bind(this));
                    this.icon = new api.dom.IEl('icon-headers');
                    this.button.appendChild(this.icon);
                }
                MediumHeadersDropdownExtension.prototype.onClick = function (event) {
                };
                MediumHeadersDropdownExtension.prototype.getButton = function () {
                    return this.button.getHTMLElement();
                };
                MediumHeadersDropdownExtension.prototype.beforeCheckState = function () {
                    this.icon.removeClass('icon-headers');
                    this.icon.removeClass('icon-header1');
                    this.icon.removeClass('icon-header2');
                    this.icon.removeClass('icon-header3');
                    this.icon.removeClass('icon-header4');
                    this.button.removeClass('medium-editor-button-active');
                };
                MediumHeadersDropdownExtension.prototype.checkState = function (node) {
                    if (this.icon.getClass() !== '') {
                        return;
                    }
                    var tagName = node.tagName.toLowerCase();
                    if (tagName === 'h1') {
                        this.icon.setClass('icon-header1');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h2') {
                        this.icon.setClass('icon-header2');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h3') {
                        this.icon.setClass('icon-header3');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h4') {
                        this.icon.setClass('icon-header4');
                        this.button.addClass('medium-editor-button-active');
                    }
                };
                MediumHeadersDropdownExtension.prototype.afterCheckState = function () {
                    if (this.icon.getClass() === '') {
                        this.icon.setClass('icon-headers');
                    }
                };
                MediumHeadersDropdownExtension.prototype.setEditor = function (editor) {
                    this.editor = editor;
                };
                MediumHeadersDropdownExtension.prototype.onHideToolbar = function () {
                };
                MediumHeadersDropdownExtension.prototype.onShowToolbar = function () {
                    this.initDropdownButtons();
                };
                MediumHeadersDropdownExtension.prototype.initDropdownButtons = function () {
                    var mainButtonParent = this.button.getHTMLElement().parentElement;
                    if (!mainButtonParent) {
                        return;
                    }
                    if (this.dropdownButtons && mainButtonParent.contains(this.dropdownButtons.getHTMLElement())) {
                        return;
                    }
                    this.dropdownButtons = new api.dom.UlEl('medium-editor-toolbar-actions');
                    this.dropdownButtons.addClass('clearfix');
                    var header1Item = this.createHeaderMenuItem('h1', 'icon-header1');
                    var header2Item = this.createHeaderMenuItem('h2', 'icon-header2');
                    var header3Item = this.createHeaderMenuItem('h3', 'icon-header3');
                    var header4Item = this.createHeaderMenuItem('h4', 'icon-header4', true);
                    this.dropdownButtons.appendChildren(header1Item, header2Item, header3Item, header4Item);
                    this.button.appendChild(this.dropdownButtons);
                    mainButtonParent.appendChild(this.dropdownButtons.getHTMLElement());
                };
                MediumHeadersDropdownExtension.prototype.createHeaderMenuItem = function (elementTag, iconClass, lastButton) {
                    var _this = this;
                    if (lastButton === void 0) { lastButton = false; }
                    var menuItem = new api.dom.LiEl();
                    var button = new api.dom.ButtonEl('medium-editor-action');
                    if (lastButton) {
                        button.addClass('medium-editor-button-bottom');
                    }
                    var icon = new api.dom.IEl(iconClass);
                    button.appendChild(icon);
                    button.onClicked(function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        _this.editor.execAction('append-' + elementTag, event);
                        _this.icon.setClass(iconClass);
                        _this.button.addClass('medium-editor-button-active');
                    });
                    menuItem.appendChild(button);
                    return menuItem;
                };
                return MediumHeadersDropdownExtension;
            })();
            text.MediumHeadersDropdownExtension = MediumHeadersDropdownExtension;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (_text) {
            var ComponentView = api.liveedit.ComponentView;
            var TextComponentViewBuilder = (function (_super) {
                __extends(TextComponentViewBuilder, _super);
                function TextComponentViewBuilder() {
                    _super.call(this);
                    this.setType(_text.TextItemType.get());
                }
                return TextComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            _text.TextComponentViewBuilder = TextComponentViewBuilder;
            var TextComponentView = (function (_super) {
                __extends(TextComponentView, _super);
                function TextComponentView(builder) {
                    this.lastClicked = 0;
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.textComponent = builder.component;
                    _super.call(this, builder.setContextMenuActions(this.createTextContextMenuActions()).setPlaceholder(new _text.TextPlaceholder(this)).setTooltipViewer(new _text.TextComponentViewer()));
                    this.addClass('text-view');
                    this.initializeRootElement();
                    this.onKeyDown(this.handleKey.bind(this));
                    this.onKeyUp(this.handleKey.bind(this));
                }
                TextComponentView.prototype.showTooltip = function () {
                    if (!this.isEditMode()) {
                        _super.prototype.showTooltip.call(this);
                    }
                };
                TextComponentView.prototype.hideTooltip = function (hideParentTooltip) {
                    if (hideParentTooltip === void 0) { hideParentTooltip = true; }
                    if (!this.isEditMode()) {
                        _super.prototype.hideTooltip.call(this, hideParentTooltip);
                    }
                };
                TextComponentView.prototype.highlight = function () {
                    if (!this.isEditMode()) {
                        _super.prototype.highlight.call(this);
                    }
                };
                TextComponentView.prototype.unhighlight = function () {
                    if (!this.isEditMode()) {
                        _super.prototype.unhighlight.call(this);
                    }
                };
                TextComponentView.prototype.initializeRootElement = function () {
                    for (var i = 0; i < this.getChildren().length; i++) {
                        var child = this.getChildren()[i];
                        if (child.getEl().getTagName().toUpperCase() == 'SECTION') {
                            this.rootElement = child;
                            break;
                        }
                    }
                    if (!this.rootElement) {
                        this.rootElement = new api.dom.SectionEl();
                        this.prependChild(this.rootElement);
                    }
                };
                TextComponentView.prototype.processChanges = function () {
                    var text = this.rootElement.getHtml();
                    if (TextComponentView.debug) {
                        console.log('Processing editor contents: \n', text);
                    }
                    var contentWithoutTags = text.replace(/(<([^>]+)>)/ig, "").trim();
                    this.textComponent.setText(contentWithoutTags.length == 0 ? undefined : text);
                };
                TextComponentView.prototype.isEmpty = function () {
                    return !this.textComponent || this.textComponent.isEmpty();
                };
                TextComponentView.prototype.doHandleDbClick = function (event) {
                    if (this.isEditMode()) {
                        return;
                    }
                    this.startPageTextEditMode();
                    this.selectText();
                };
                TextComponentView.prototype.doHandleClick = function (event) {
                    if (this.isEditMode()) {
                        return;
                    }
                    _super.prototype.handleClick.call(this, event);
                };
                TextComponentView.prototype.handleClick = function (event) {
                    var _this = this;
                    if (TextComponentView.debug) {
                        console.group('Handling click [' + this.getId() + '] at ' + new Date().getTime());
                        console.log(event);
                    }
                    event.stopPropagation();
                    if (this.isEditMode()) {
                        if (TextComponentView.debug) {
                            console.log('Is in text edit mode, not handling click');
                            console.groupEnd();
                        }
                        return;
                    }
                    var timeSinceLastClick = new Date().getTime() - this.lastClicked;
                    if (timeSinceLastClick > TextComponentView.DBL_CLICK_TIMEOUT) {
                        this.singleClickTimer = setTimeout(function () {
                            if (TextComponentView.debug) {
                                console.log('no dblclick occured during ' + TextComponentView.DBL_CLICK_TIMEOUT + 'ms, notifying click', _this);
                                console.groupEnd();
                            }
                            _this.doHandleClick(event);
                        }, TextComponentView.DBL_CLICK_TIMEOUT);
                    }
                    else {
                        if (TextComponentView.debug) {
                            console.log('dblclick occured after ' + timeSinceLastClick + 'ms, notifying dbl click', this);
                            console.groupEnd();
                            console.groupEnd();
                        }
                        clearTimeout(this.singleClickTimer);
                        this.doHandleDbClick(event);
                    }
                    this.lastClicked = new Date().getTime();
                };
                TextComponentView.prototype.handleKey = function () {
                    this.processChanges();
                };
                TextComponentView.prototype.isEditMode = function () {
                    return this.hasClass('edit-mode');
                };
                TextComponentView.prototype.setEditMode = function (flag) {
                    if (!flag && this.editor) {
                        this.deselectText();
                        this.editor.deactivate();
                    }
                    this.toggleClass('edit-mode', flag);
                    this.setDraggable(!flag);
                    if (flag) {
                        this.hideTooltip();
                        if (!this.editor) {
                            this.editor = this.createEditor();
                        }
                        this.editor.activate();
                    }
                };
                TextComponentView.prototype.createEditor = function () {
                    var _this = this;
                    var headersExtension = new _text.MediumHeadersDropdownExtension();
                    var editor = new MediumEditor([this.rootElement.getHTMLElement()], {
                        buttons: ['bold', 'italic', 'underline', 'strikethrough', 'justifyLeft', 'justifyCenter', 'justifyRight', 'justifyFull', 'anchor', 'headers', 'orderedlist', 'unorderedlist', 'quote'],
                        buttonLabels: {
                            'bold': '<i class="icon-bold"></i>',
                            'italic': '<i class="icon-italic"></i>',
                            'underline': '<i class="icon-underline"></i>',
                            'strikethrough': '<i class="icon-strikethrough"></i>',
                            'justifyCenter': '<i class="icon-justify-center"></i>',
                            'justifyFull': '<i class="icon-justify-full"></i>',
                            'justifyLeft': '<i class="icon-justify-left"></i>',
                            'justifyRight': '<i class="icon-justify-right"></i>',
                            'anchor': '<i class="icon-anchor"></i>',
                            'header1': '<i class="icon-header1"></i>',
                            'header2': '<i class="icon-header2"></i>',
                            'orderedlist': '<i class="icon-ordered-list"></i>',
                            'unorderedlist': '<i class="icon-unordered-list"></i>',
                            'quote': '<i class="icon-quote"></i>'
                        },
                        cleanPastedHTML: true,
                        targetBlank: true,
                        disablePlaceholders: true,
                        firstHeader: 'h1',
                        secondHeader: 'h2',
                        extensions: {
                            'headers': headersExtension
                        }
                    });
                    var checkActiveButtons = editor.checkActiveButtons.bind(editor);
                    editor.checkActiveButtons = function () {
                        headersExtension.beforeCheckState();
                        checkActiveButtons();
                        headersExtension.afterCheckState();
                    };
                    headersExtension.setEditor(editor);
                    editor.onHideToolbar = function () {
                        _this.processChanges();
                        headersExtension.onHideToolbar();
                    };
                    editor.onShowToolbar = function () {
                        headersExtension.onShowToolbar();
                    };
                    return editor;
                };
                TextComponentView.prototype.selectText = function () {
                    var doc = document;
                    var text = this.rootElement.getHTMLElement();
                    if (window['getSelection']) {
                        var selection = window['getSelection']();
                        var rangeOther = doc.createRange();
                        rangeOther.selectNodeContents(text);
                        selection.removeAllRanges();
                        selection.addRange(rangeOther);
                    }
                    else if (doc.body['createTextRange']) {
                        var rangeIE = doc.body['createTextRange']();
                        rangeIE.moveToElementText(text);
                        rangeIE.select();
                    }
                    text.click();
                };
                TextComponentView.prototype.deselectText = function () {
                    if (window['getSelection']) {
                        window['getSelection']().removeAllRanges();
                    }
                    else if (document.selection) {
                        document.selection.empty();
                    }
                };
                TextComponentView.prototype.startPageTextEditMode = function () {
                    this.deselect();
                    var pageView = this.getPageView();
                    if (!pageView.isTextEditMode()) {
                        pageView.setTextEditMode(true);
                    }
                    this.giveFocus();
                };
                TextComponentView.prototype.giveFocus = function () {
                    if (!this.isEditMode()) {
                        return false;
                    }
                    return this.rootElement.giveFocus();
                };
                TextComponentView.prototype.createTextContextMenuActions = function () {
                    var _this = this;
                    var actions = [];
                    actions.push(new api.ui.Action('Edit').onExecuted(function () {
                        _this.startPageTextEditMode();
                    }));
                    return actions;
                };
                TextComponentView.debug = false;
                TextComponentView.DBL_CLICK_TIMEOUT = 250;
                return TextComponentView;
            })(ComponentView);
            _text.TextComponentView = TextComponentView;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var TextComponentViewer = (function (_super) {
                __extends(TextComponentViewer, _super);
                function TextComponentViewer() {
                    _super.call(this);
                }
                TextComponentViewer.prototype.resolveDisplayName = function (object) {
                    return object.getText();
                };
                TextComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                TextComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-text";
                };
                return TextComponentViewer;
            })(api.ui.NamesAndIconViewer);
            text.TextComponentViewer = TextComponentViewer;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
wemQ.longStackSupport = true;
var app;
(function (app) {
    var Router = (function () {
        function Router() {
        }
        Router.setHash = function (path) {
            if (window.parent["setHash"]) {
                window.parent["setHash"](api.app.Application.getAppId() + "/" + path);
            }
        };
        return Router;
    })();
    app.Router = Router;
})(app || (app = {}));
var app;
(function (app) {
    var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
    var ContentId = api.content.ContentId;
    var ContentUpdatedEvent = api.content.ContentUpdatedEvent;
    var AppBarTabId = api.app.bar.AppBarTabId;
    var AppBarTabMenuItemBuilder = api.app.bar.AppBarTabMenuItemBuilder;
    var ShowBrowsePanelEvent = api.app.ShowBrowsePanelEvent;
    var ContentAppPanel = (function (_super) {
        __extends(ContentAppPanel, _super);
        function ContentAppPanel(appBar, path) {
            _super.call(this, {
                appBar: appBar
            });
            this.mask = new api.ui.mask.LoadMask(this);
            this.handleGlobalEvents();
            this.route(path);
        }
        ContentAppPanel.prototype.addWizardPanel = function (tabMenuItem, wizardPanel) {
            var _this = this;
            _super.prototype.addWizardPanel.call(this, tabMenuItem, wizardPanel);
            wizardPanel.getHeader().onPropertyChanged(function (event) {
                if (event.getPropertyName() === "displayName") {
                    var contentType = wizardPanel.getContentType(), name = event.getNewValue() || "<Unnamed " + _this.convertName(contentType.getDisplayName()) + ">";
                    tabMenuItem.setLabel(name, !event.getNewValue());
                }
            });
            var contentWizardPanel = wizardPanel;
            tabMenuItem.markInvalid(!contentWizardPanel.getPersistedItem().isValid());
            contentWizardPanel.onValidityChanged(function (event) {
                tabMenuItem.markInvalid(!contentWizardPanel.isValid());
            });
        };
        ContentAppPanel.prototype.route = function (path) {
            var action = path ? path.getElement(0) : undefined;
            switch (action) {
                case 'edit':
                    var id = path.getElement(1);
                    if (id) {
                        new api.content.GetContentByIdRequest(new ContentId(id)).sendAndParse().done(function (content) {
                            new app.browse.EditContentEvent([content]).fire();
                        });
                    }
                    break;
                case 'view':
                    var id = path.getElement(1);
                    if (id) {
                        new api.content.GetContentByIdRequest(new ContentId(id)).sendAndParse().done(function (content) {
                            new app.browse.ViewContentEvent([content]).fire();
                        });
                    }
                    break;
                default:
                    new ShowBrowsePanelEvent().fire();
                    break;
            }
        };
        ContentAppPanel.prototype.handleGlobalEvents = function () {
            var _this = this;
            app.create.NewContentEvent.on(function (event) {
                _this.handleNewContent(event);
            });
            app.browse.ViewContentEvent.on(function (event) {
                _this.handleView(event);
            });
            app.browse.EditContentEvent.on(function (event) {
                _this.handleEdit(event);
            });
            ShowBrowsePanelEvent.on(function (event) {
                _this.handleBrowse(event);
            });
            ContentUpdatedEvent.on(function (event) {
                _this.handleUpdated(event);
            });
            app.browse.SortContentEvent.on(function (event) {
                _this.handleSort(event);
            });
            app.browse.MoveContentEvent.on(function (event) {
                _this.handleMove(event);
            });
            api.content.ContentDeletedEvent.on(function (event) {
                var item = _this.getNavigator().getNavigationItemByIdValue(event.getContentId().toString());
                if (item) {
                    item.getCloseAction().execute();
                }
            });
        };
        ContentAppPanel.prototype.handleUpdated = function (event) {
        };
        ContentAppPanel.prototype.handleBrowse = function (event) {
            var browsePanel = this.getBrowsePanel();
            if (!browsePanel) {
                this.addBrowsePanel(new app.browse.ContentBrowsePanel());
            }
            else {
                this.selectPanelByIndex(this.getPanelIndex(browsePanel));
            }
        };
        ContentAppPanel.prototype.handleNewContent = function (newContentEvent) {
            var _this = this;
            var contentTypeSummary = newContentEvent.getContentType();
            var tabId = AppBarTabId.forNew(contentTypeSummary.getName());
            var tabMenuItem = this.getAppBarTabMenu().getNavigationItemById(tabId);
            if (tabMenuItem != null) {
                this.selectPanel(tabMenuItem);
            }
            else {
                this.mask.show();
                var contentWizardPanelFactory = new app.wizard.ContentWizardPanelFactory().setAppBarTabId(tabId).setParentContent(newContentEvent.getParentContent()).setContentTypeName(contentTypeSummary.getContentTypeName());
                contentWizardPanelFactory.setCreateSite(newContentEvent.getContentType().isSite());
                contentWizardPanelFactory.createForNew().then(function (wizard) {
                    if (!wizard.getPersistedItem()) {
                        wizard.close();
                        return;
                    }
                    tabMenuItem = new AppBarTabMenuItemBuilder().setLabel("<New " + _this.convertName(contentTypeSummary.getDisplayName()) + ">").setTabId(tabId).setCloseAction(wizard.getCloseAction()).build();
                    wizard.onContentNamed(function (event) {
                        _this.handleContentNamedEvent(event);
                    });
                    _this.addWizardPanel(tabMenuItem, wizard);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).finally(function () {
                    _this.mask.hide();
                }).done();
            }
        };
        ContentAppPanel.prototype.handleEdit = function (event) {
            var _this = this;
            var contents = event.getModels();
            contents.forEach(function (content) {
                if (!content) {
                    return;
                }
                var closeViewPanelMenuItem = _this.resolveTabMenuItemForContentBeingViewed(content);
                var tabMenuItem = _this.resolveTabMenuItemForContentBeingEdited(content);
                if (tabMenuItem != null) {
                    _this.selectPanel(tabMenuItem);
                }
                else {
                    _this.mask.show();
                    var tabId = AppBarTabId.forEdit(content.getId());
                    new app.wizard.ContentWizardPanelFactory().setAppBarTabId(tabId).setContentToEdit(content.getContentId()).createForEdit().then(function (wizard) {
                        if (closeViewPanelMenuItem != null) {
                            _this.getAppBarTabMenu().deselectNavigationItem();
                            _this.getAppBarTabMenu().removeNavigationItem(closeViewPanelMenuItem);
                            _this.removePanelByIndex(closeViewPanelMenuItem.getIndex());
                        }
                        var contentType = wizard.getContentType(), name = content.getDisplayName() || "<Unnamed " + _this.convertName(contentType.getDisplayName()) + ">";
                        tabMenuItem = new AppBarTabMenuItemBuilder().setLabel(name).setMarkUnnamed(!content.getDisplayName()).setMarkInvalid(!content.isValid()).setTabId(tabId).setEditing(true).setCloseAction(wizard.getCloseAction()).build();
                        _this.addWizardPanel(tabMenuItem, wizard);
                        var viewTabId = AppBarTabId.forView(content.getId());
                        var viewTabMenuItem = _this.getAppBarTabMenu().getNavigationItemById(viewTabId);
                        if (viewTabMenuItem != null) {
                            _this.removePanelByIndex(viewTabMenuItem.getIndex());
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).finally(function () {
                        _this.mask.hide();
                    }).done();
                }
            });
        };
        ContentAppPanel.prototype.handleView = function (event) {
            var _this = this;
            var contents = event.getModels();
            contents.forEach(function (content) {
                if (!content) {
                    return;
                }
                var tabMenuItem = _this.resolveTabMenuItemForContentBeingEditedOrViewed(content);
                if (tabMenuItem) {
                    _this.selectPanel(tabMenuItem);
                }
                else {
                    var tabId = AppBarTabId.forView(content.getId());
                    var contentItemViewPanel = new app.view.ContentItemViewPanel();
                    tabMenuItem = new AppBarTabMenuItemBuilder().setLabel(content.getDisplayName()).setMarkInvalid(!content.isValid()).setTabId(tabId).setCloseAction(contentItemViewPanel.getCloseAction()).build();
                    if (!content.getDisplayName()) {
                        new api.schema.content.GetContentTypeByNameRequest(content.getType()).sendAndParse().then(function (contentType) {
                            tabMenuItem.setLabel("<Unnamed " + _this.convertName(contentType.getDisplayName()) + ">", true);
                        }).done();
                    }
                    var contentItem = new api.app.view.ViewItem(content).setDisplayName(content.getDisplayName()).setPath(content.getPath().toString()).setIconUrl(new ContentIconUrlResolver().setContent(content).resolve());
                    contentItemViewPanel.setItem(contentItem);
                    _this.addViewPanel(tabMenuItem, contentItemViewPanel);
                }
            });
        };
        ContentAppPanel.prototype.handleSort = function (event) {
            var contents = event.getModels();
            new app.browse.OpenSortDialogEvent(contents[0]).fire();
        };
        ContentAppPanel.prototype.handleMove = function (event) {
            var contents = event.getModels();
            new app.browse.OpenMoveDialogEvent(contents[0]).fire();
        };
        ContentAppPanel.prototype.handleContentNamedEvent = function (event) {
            var wizard = event.getWizard(), tabMenuItem = this.getAppBarTabMenu().getNavigationItemById(wizard.getTabId());
            var newTabId = AppBarTabId.forEdit(event.getContent().getId());
            tabMenuItem.setTabId(newTabId);
            wizard.setTabId(newTabId);
        };
        ContentAppPanel.prototype.resolveTabMenuItemForContentBeingEditedOrViewed = function (content) {
            var result = this.resolveTabMenuItemForContentBeingEdited(content);
            if (!result) {
                result = this.resolveTabMenuItemForContentBeingViewed(content);
            }
            return result;
        };
        ContentAppPanel.prototype.resolveTabMenuItemForContentBeingEdited = function (content) {
            if (!!content) {
                var tabId = this.getAppBarTabMenu().getNavigationItemById(AppBarTabId.forEdit(content.getId()));
                if (tabId) {
                    return tabId;
                }
            }
            return null;
        };
        ContentAppPanel.prototype.resolveTabMenuItemForContentBeingViewed = function (content) {
            if (!!content) {
                var tabId = this.getAppBarTabMenu().getNavigationItemById(AppBarTabId.forView(content.getId()));
                if (tabId) {
                    return tabId;
                }
            }
            return null;
        };
        ContentAppPanel.prototype.convertName = function (name) {
            return api.util.StringHelper.capitalizeAll(name.replace(/-/g, " ").trim());
        };
        return ContentAppPanel;
    })(api.app.BrowseAndWizardBasedAppPanel);
    app.ContentAppPanel = ContentAppPanel;
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentBrowseResetEvent = (function (_super) {
                __extends(ContentBrowseResetEvent, _super);
                function ContentBrowseResetEvent() {
                    _super.apply(this, arguments);
                }
                ContentBrowseResetEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentBrowseResetEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return ContentBrowseResetEvent;
            })(api.event.Event);
            filter.ContentBrowseResetEvent = ContentBrowseResetEvent;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentBrowseSearchEvent = (function (_super) {
                __extends(ContentBrowseSearchEvent, _super);
                function ContentBrowseSearchEvent(contentQueryResult, contentQuery) {
                    _super.call(this);
                    this.contentQueryResult = contentQueryResult;
                    this.contentQuery = contentQuery;
                }
                ContentBrowseSearchEvent.prototype.getContentQueryResult = function () {
                    return this.contentQueryResult;
                };
                ContentBrowseSearchEvent.prototype.getContentQuery = function () {
                    return this.contentQuery;
                };
                ContentBrowseSearchEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentBrowseSearchEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return ContentBrowseSearchEvent;
            })(api.event.Event);
            filter.ContentBrowseSearchEvent = ContentBrowseSearchEvent;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentBrowseRefreshEvent = (function (_super) {
                __extends(ContentBrowseRefreshEvent, _super);
                function ContentBrowseRefreshEvent() {
                    _super.apply(this, arguments);
                }
                ContentBrowseRefreshEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentBrowseRefreshEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return ContentBrowseRefreshEvent;
            })(api.event.Event);
            filter.ContentBrowseRefreshEvent = ContentBrowseRefreshEvent;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentQueryRequest = api.content.ContentQueryRequest;
            var ContentTypeName = api.schema.content.ContentTypeName;
            var AggregationGroupView = api.aggregation.AggregationGroupView;
            var ContentTypeAggregationGroupView = api.aggregation.ContentTypeAggregationGroupView;
            var ContentQuery = api.content.query.ContentQuery;
            var TermsAggregationQuery = api.query.aggregation.TermsAggregationQuery;
            var DateRangeAggregationQuery = api.query.aggregation.DateRangeAggregationQuery;
            var DateRange = api.query.aggregation.DateRange;
            var QueryExpr = api.query.expr.QueryExpr;
            var ValueExpr = api.query.expr.ValueExpr;
            var RefreshEvent = api.app.browse.filter.RefreshEvent;
            var QueryField = api.query.QueryField;
            var ContentBrowseFilterPanel = (function (_super) {
                __extends(ContentBrowseFilterPanel, _super);
                function ContentBrowseFilterPanel() {
                    var _this = this;
                    var contentTypeAggregation = new ContentTypeAggregationGroupView(ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME, ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_DISPLAY_NAME);
                    var lastModifiedAggregation = new AggregationGroupView(ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME, ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_DISPLAY_NAME);
                    _super.call(this, null, [contentTypeAggregation, lastModifiedAggregation]);
                    this.initAggregationGroupView([contentTypeAggregation, lastModifiedAggregation]);
                    this.onReset(function () {
                        _this.resetFacets();
                    });
                    this.onRefresh(this.searchFacets);
                    this.onSearch(this.searchFacets);
                }
                ContentBrowseFilterPanel.prototype.searchFacets = function (event) {
                    var _this = this;
                    var isClean = !this.hasFilterSet();
                    if (isClean) {
                        if (event instanceof RefreshEvent) {
                            this.resetFacets(true, true).then(function () {
                                new filter.ContentBrowseRefreshEvent().fire();
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).done();
                        }
                        else {
                            this.reset();
                        }
                        return;
                    }
                    var contentQuery = new ContentQuery();
                    this.appendFulltextSearch(event.getSearchInputValues(), contentQuery);
                    this.appendContentTypeFilter(event.getSearchInputValues(), contentQuery);
                    var lastModifiedFilter = this.appendLastModifiedQuery(event.getSearchInputValues());
                    if (lastModifiedFilter != null) {
                        contentQuery.addQueryFilter(lastModifiedFilter);
                    }
                    this.appendContentTypesAggregationQuery(contentQuery);
                    this.appendLastModifiedAggregationQuery(contentQuery);
                    new ContentQueryRequest(contentQuery).setExpand(1 /* SUMMARY */).sendAndParse().then(function (contentQueryResult) {
                        var serchStrValue = event.getSearchInputValues().getTextSearchFieldValue();
                        if (serchStrValue != null && serchStrValue.length > 0) {
                            contentQuery = new ContentQuery();
                            _this.appendFulltextSearch(event.getSearchInputValues(), contentQuery);
                            _this.appendContentTypesAggregationQuery(contentQuery);
                            _this.appendLastModifiedAggregationQuery(contentQuery);
                            return new ContentQueryRequest(contentQuery).setExpand(1 /* SUMMARY */).sendAndParse().then(function (filterSearchResult) {
                                _this.updateAggregations(filterSearchResult.getAggregations(), true);
                                new filter.ContentBrowseSearchEvent(contentQueryResult, contentQuery).fire();
                            });
                        }
                        else {
                            _this.resetFacets(true, true);
                            if (event instanceof RefreshEvent) {
                                new filter.ContentBrowseRefreshEvent().fire();
                            }
                            else {
                                new filter.ContentBrowseSearchEvent(contentQueryResult, contentQuery).fire();
                            }
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                };
                ContentBrowseFilterPanel.prototype.initAggregationGroupView = function (aggregationGroupView) {
                    var _this = this;
                    var contentQuery = this.buildAggregationsQuery(new QueryExpr(null));
                    new ContentQueryRequest(contentQuery).sendAndParse().then(function (contentQueryResult) {
                        _this.updateAggregations(contentQueryResult.getAggregations(), false);
                        aggregationGroupView.forEach(function (aggregationGroupView) {
                            aggregationGroupView.initialize();
                        });
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                };
                ContentBrowseFilterPanel.prototype.resetFacets = function (supressEvent, doResetAll) {
                    var _this = this;
                    var contentQuery = this.buildAggregationsQuery(new QueryExpr(null));
                    return new ContentQueryRequest(contentQuery).sendAndParse().then(function (contentQueryResult) {
                        _this.updateAggregations(contentQueryResult.getAggregations(), doResetAll);
                        if (!supressEvent) {
                            new filter.ContentBrowseResetEvent().fire();
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    });
                };
                ContentBrowseFilterPanel.prototype.buildAggregationsQuery = function (queryExpr) {
                    var contentQuery = new ContentQuery();
                    contentQuery.setQueryExpr(queryExpr);
                    contentQuery.setSize(0);
                    this.appendContentTypesAggregationQuery(contentQuery);
                    this.appendLastModifiedAggregationQuery(contentQuery);
                    return contentQuery;
                };
                ContentBrowseFilterPanel.prototype.appendFulltextSearch = function (searchInputValues, contentQuery) {
                    var searchString = searchInputValues.getTextSearchFieldValue();
                    var fulltextSearchExpression = new api.query.FulltextSearchExpressionBuilder().setSearchString(searchString).addField(new QueryField(QueryField.DISPLAY_NAME, 5)).addField(new QueryField(QueryField.NAME, 3)).addField(new QueryField(QueryField.ALL)).build();
                    var query = new QueryExpr(fulltextSearchExpression);
                    contentQuery.setQueryExpr(query);
                };
                ContentBrowseFilterPanel.prototype.appendContentTypeFilter = function (searchInputValues, contentQuery) {
                    var selectedBuckets = searchInputValues.getSelectedValuesForAggregationName(ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME);
                    var contentTypeNames = this.parseContentTypeNames(selectedBuckets);
                    contentQuery.setContentTypeNames(contentTypeNames);
                };
                ContentBrowseFilterPanel.prototype.appendLastModifiedQuery = function (searchInputValues) {
                    var lastModifiedSelectedBuckets = searchInputValues.getSelectedValuesForAggregationName(ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME);
                    if (lastModifiedSelectedBuckets == null || lastModifiedSelectedBuckets.length == 0) {
                        return null;
                    }
                    if (lastModifiedSelectedBuckets.length == 1) {
                        var dateRangeBucket = lastModifiedSelectedBuckets.pop();
                        return new api.query.filter.RangeFilter(QueryField.MODIFIED_TIME, ValueExpr.dateTime(dateRangeBucket.getFrom()).getValue(), null);
                    }
                    var booleanFilter = new api.query.filter.BooleanFilter();
                    lastModifiedSelectedBuckets.forEach(function (selectedBucket) {
                        var rangeFilter = new api.query.filter.RangeFilter(QueryField.MODIFIED_TIME, ValueExpr.dateTime(selectedBucket.getFrom()).getValue(), null);
                        booleanFilter.addShould(rangeFilter);
                    });
                    return booleanFilter;
                };
                ContentBrowseFilterPanel.prototype.parseContentTypeNames = function (buckets) {
                    var contentTypeNames = [];
                    if (buckets) {
                        for (var i = 0; i < buckets.length; i++) {
                            var bucket = buckets[i];
                            contentTypeNames.push(new ContentTypeName(bucket.getKey()));
                        }
                    }
                    return contentTypeNames;
                };
                ContentBrowseFilterPanel.prototype.appendContentTypesAggregationQuery = function (contentQuery) {
                    contentQuery.addAggregationQuery(this.createTermsAggregation((ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME), QueryField.CONTENT_TYPE, 15));
                };
                ContentBrowseFilterPanel.prototype.createTermsAggregation = function (name, fieldName, size) {
                    var termsAggregation = new TermsAggregationQuery(name);
                    termsAggregation.setFieldName(fieldName);
                    termsAggregation.setSize(size);
                    return termsAggregation;
                };
                ContentBrowseFilterPanel.prototype.appendLastModifiedAggregationQuery = function (contentQuery) {
                    var dateRangeAgg = new DateRangeAggregationQuery((ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME));
                    dateRangeAgg.setFieldName(QueryField.MODIFIED_TIME);
                    dateRangeAgg.addRange(new DateRange("now-1h", null, "< 1 hour"));
                    dateRangeAgg.addRange(new DateRange("now-1d", null, "< 1 day"));
                    dateRangeAgg.addRange(new DateRange("now-1w", null, "< 1 week"));
                    contentQuery.addAggregationQuery(dateRangeAgg);
                };
                ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME = "contentTypes";
                ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME = "lastModified";
                ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_DISPLAY_NAME = "Content Types";
                ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_DISPLAY_NAME = "Last Modified";
                return ContentBrowseFilterPanel;
            })(api.app.browse.filter.BrowseFilterPanel);
            filter.ContentBrowseFilterPanel = ContentBrowseFilterPanel;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var ShowNewContentDialogAction = (function (_super) {
                __extends(ShowNewContentDialogAction, _super);
                function ShowNewContentDialogAction(grid) {
                    _super.call(this, "New", "alt+n");
                    this.setEnabled(true);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.ShowNewContentDialogEvent(contentSummaries.length > 0 ? contentSummaries[0] : null).fire();
                    });
                }
                return ShowNewContentDialogAction;
            })(Action);
            action.ShowNewContentDialogAction = ShowNewContentDialogAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var RenderingMode = api.rendering.RenderingMode;
            var PreviewContentAction = (function (_super) {
                __extends(PreviewContentAction, _super);
                function PreviewContentAction(grid) {
                    var _this = this;
                    _super.call(this, "Preview", "");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        _this.showPreviewDialog(contentSummaries[0]);
                    });
                }
                PreviewContentAction.prototype.showPreviewDialog = function (content) {
                    window.open(api.rendering.UriHelper.getPortalUri(content.getPath().toString(), 1 /* PREVIEW */, 0 /* DRAFT */), 'preview').focus();
                };
                return PreviewContentAction;
            })(Action);
            action.PreviewContentAction = PreviewContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var EditContentAction = (function (_super) {
                __extends(EditContentAction, _super);
                function EditContentAction(grid) {
                    _super.call(this, "Edit", "mod+e");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.EditContentEvent(contentSummaries).fire();
                    });
                }
                return EditContentAction;
            })(Action);
            action.EditContentAction = EditContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var DeleteContentAction = (function (_super) {
                __extends(DeleteContentAction, _super);
                function DeleteContentAction(grid) {
                    _super.call(this, "Delete", "mod+del");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.ContentDeletePromptEvent(contentSummaries).fire();
                    });
                }
                return DeleteContentAction;
            })(Action);
            action.DeleteContentAction = DeleteContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var DuplicateContentAction = (function (_super) {
                __extends(DuplicateContentAction, _super);
                function DuplicateContentAction(grid) {
                    var _this = this;
                    _super.call(this, "Duplicate");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        grid.getSelectedDataList().forEach(function (elem) {
                            _this.duplicate(elem.getContentSummary());
                        });
                    });
                }
                DuplicateContentAction.prototype.duplicate = function (source) {
                    new api.content.DuplicateContentRequest(source.getContentId()).sendAndParse().then(function (content) {
                        api.notify.showFeedback('Content [' + source.getPath() + '] was duplicated!');
                    });
                };
                return DuplicateContentAction;
            })(Action);
            action.DuplicateContentAction = DuplicateContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var MoveContentAction = (function (_super) {
                __extends(MoveContentAction, _super);
                function MoveContentAction(grid) {
                    _super.call(this, "Move");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.MoveContentEvent(contentSummaries).fire();
                    });
                }
                return MoveContentAction;
            })(Action);
            action.MoveContentAction = MoveContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var SortContentAction = (function (_super) {
                __extends(SortContentAction, _super);
                function SortContentAction(grid) {
                    _super.call(this, "Sort");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new app.browse.SortContentEvent(contentSummaries).fire();
                    });
                }
                return SortContentAction;
            })(Action);
            action.SortContentAction = SortContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var PublishContentRequest = api.content.PublishContentRequest;
            var PublishContentAction = (function (_super) {
                __extends(PublishContentAction, _super);
                function PublishContentAction(grid) {
                    _super.call(this, "Publish");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        new PublishContentRequest().setIds(grid.getSelectedDataList().map(function (el) {
                            return new api.content.ContentId(el.getContentSummary().getId());
                        })).send().done(function (jsonResponse) {
                            PublishContentRequest.feedback(jsonResponse);
                        });
                    });
                }
                return PublishContentAction;
            })(Action);
            action.PublishContentAction = PublishContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var SaveSortedContentAction = (function (_super) {
                __extends(SaveSortedContentAction, _super);
                function SaveSortedContentAction(dialog) {
                    _super.call(this, "Save");
                    this.setEnabled(true);
                    this.onExecuted(function () {
                        new browse.SaveSortedContentEvent(dialog.getContent()).fire();
                    });
                }
                return SaveSortedContentAction;
            })(api.ui.Action);
            action.SaveSortedContentAction = SaveSortedContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var ToggleSearchPanelAction = (function (_super) {
                __extends(ToggleSearchPanelAction, _super);
                function ToggleSearchPanelAction() {
                    _super.call(this, "");
                    this.onExecuted(function () {
                        new browse.ToggleSearchPanelEvent().fire();
                    });
                    this.setIconClass("icon-search3");
                }
                return ToggleSearchPanelAction;
            })(Action);
            action.ToggleSearchPanelAction = ToggleSearchPanelAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var ContentTreeGridActions = (function () {
                function ContentTreeGridActions(grid) {
                    this.actions = [];
                    this.TOGGLE_SEARCH_PANEL = new action.ToggleSearchPanelAction();
                    this.SHOW_NEW_CONTENT_DIALOG_ACTION = new action.ShowNewContentDialogAction(grid);
                    this.PREVIEW_CONTENT = new action.PreviewContentAction(grid);
                    this.EDIT_CONTENT = new action.EditContentAction(grid);
                    this.DELETE_CONTENT = new action.DeleteContentAction(grid);
                    this.DUPLICATE_CONTENT = new action.DuplicateContentAction(grid);
                    this.MOVE_CONTENT = new action.MoveContentAction(grid);
                    this.SORT_CONTENT = new action.SortContentAction(grid);
                    this.PUBLISH_CONTENT = new action.PublishContentAction(grid);
                    this.actions.push(this.SHOW_NEW_CONTENT_DIALOG_ACTION, this.PREVIEW_CONTENT, this.EDIT_CONTENT, this.DELETE_CONTENT, this.DUPLICATE_CONTENT, this.MOVE_CONTENT, this.SORT_CONTENT, this.PUBLISH_CONTENT);
                }
                ContentTreeGridActions.prototype.getAllActions = function () {
                    return this.actions;
                };
                ContentTreeGridActions.prototype.updateActionsEnabledState = function (contentBrowseItems) {
                    var _this = this;
                    this.TOGGLE_SEARCH_PANEL.setVisible(false);
                    var contentSummaries = contentBrowseItems.map(function (elem) {
                        return elem.getModel();
                    });
                    var deferred = wemQ.defer();
                    switch (contentBrowseItems.length) {
                        case 0:
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(true);
                            this.EDIT_CONTENT.setEnabled(false);
                            this.DELETE_CONTENT.setEnabled(false);
                            this.DUPLICATE_CONTENT.setEnabled(false);
                            this.MOVE_CONTENT.setEnabled(false);
                            this.SORT_CONTENT.setEnabled(false);
                            this.PREVIEW_CONTENT.setEnabled(false);
                            this.PUBLISH_CONTENT.setEnabled(false);
                            deferred.resolve(contentBrowseItems);
                            break;
                        case 1:
                            var contentSummary = contentSummaries[0];
                            this.EDIT_CONTENT.setEnabled(!contentSummary ? false : contentSummary.isEditable());
                            this.DELETE_CONTENT.setEnabled(!contentSummary ? false : contentSummary.isDeletable());
                            this.DUPLICATE_CONTENT.setEnabled(true);
                            this.MOVE_CONTENT.setEnabled(true);
                            this.SORT_CONTENT.setEnabled(true);
                            this.PUBLISH_CONTENT.setEnabled(true);
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(false);
                            this.PREVIEW_CONTENT.setEnabled(false);
                            var parallelPromises = [
                                new api.content.page.IsRenderableRequest(contentSummary.getContentId()).sendAndParse().then(function (renderable) {
                                    _this.PREVIEW_CONTENT.setEnabled(renderable);
                                    contentBrowseItems[0].setRenderable(renderable);
                                }),
                                new api.schema.content.GetContentTypeByNameRequest(contentSummary.getType()).sendAndParse().then(function (contentType) {
                                    _this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(contentType && contentType.isAllowChildContent());
                                })
                            ];
                            wemQ.all(parallelPromises).spread(function () {
                                deferred.resolve(contentBrowseItems);
                                return wemQ(null);
                            });
                            break;
                        default:
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(false);
                            this.PREVIEW_CONTENT.setEnabled(false);
                            this.EDIT_CONTENT.setEnabled(this.anyEditable(contentSummaries));
                            this.DELETE_CONTENT.setEnabled(this.anyDeletable(contentSummaries));
                            this.DUPLICATE_CONTENT.setEnabled(false);
                            this.MOVE_CONTENT.setEnabled(false);
                            this.SORT_CONTENT.setEnabled(false);
                            this.PUBLISH_CONTENT.setEnabled(true);
                            deferred.resolve(contentBrowseItems);
                    }
                    return deferred.promise;
                };
                ContentTreeGridActions.prototype.anyEditable = function (contentSummaries) {
                    for (var i = 0; i < contentSummaries.length; i++) {
                        var content = contentSummaries[i];
                        if (!!content && content.isEditable()) {
                            return true;
                        }
                    }
                    return false;
                };
                ContentTreeGridActions.prototype.anyDeletable = function (contentSummaries) {
                    for (var i = 0; i < contentSummaries.length; i++) {
                        var content = contentSummaries[i];
                        if (!!content && content.isDeletable()) {
                            return true;
                        }
                    }
                    return false;
                };
                return ContentTreeGridActions;
            })();
            action.ContentTreeGridActions = ContentTreeGridActions;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var BaseContentModelEvent = (function (_super) {
            __extends(BaseContentModelEvent, _super);
            function BaseContentModelEvent(model) {
                this.model = model;
                _super.call(this);
            }
            BaseContentModelEvent.prototype.getModels = function () {
                return this.model;
            };
            return BaseContentModelEvent;
        })(api.event.Event);
        browse.BaseContentModelEvent = BaseContentModelEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var Event = api.event.Event;
        var CloseContentEvent = (function (_super) {
            __extends(CloseContentEvent, _super);
            function CloseContentEvent(panel, checkCanRemovePanel) {
                if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                _super.call(this);
                this.panel = panel;
                this.checkCanRemovePanel = checkCanRemovePanel;
            }
            CloseContentEvent.prototype.getPanel = function () {
                return this.panel;
            };
            CloseContentEvent.prototype.isCheckCanRemovePanel = function () {
                return this.checkCanRemovePanel;
            };
            CloseContentEvent.on = function (handler) {
                Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            CloseContentEvent.un = function (handler) {
                Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return CloseContentEvent;
        })(Event);
        browse.CloseContentEvent = CloseContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentDeletePromptEvent = (function (_super) {
            __extends(ContentDeletePromptEvent, _super);
            function ContentDeletePromptEvent() {
                _super.apply(this, arguments);
            }
            ContentDeletePromptEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentDeletePromptEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentDeletePromptEvent;
        })(browse.BaseContentModelEvent);
        browse.ContentDeletePromptEvent = ContentDeletePromptEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var EditContentEvent = (function (_super) {
            __extends(EditContentEvent, _super);
            function EditContentEvent() {
                _super.apply(this, arguments);
            }
            EditContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            EditContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return EditContentEvent;
        })(browse.BaseContentModelEvent);
        browse.EditContentEvent = EditContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var GridSelectionChangeEvent = (function (_super) {
            __extends(GridSelectionChangeEvent, _super);
            function GridSelectionChangeEvent() {
                _super.apply(this, arguments);
            }
            GridSelectionChangeEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            GridSelectionChangeEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return GridSelectionChangeEvent;
        })(browse.BaseContentModelEvent);
        browse.GridSelectionChangeEvent = GridSelectionChangeEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var MoveContentEvent = (function (_super) {
            __extends(MoveContentEvent, _super);
            function MoveContentEvent() {
                _super.apply(this, arguments);
            }
            MoveContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            MoveContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return MoveContentEvent;
        })(browse.BaseContentModelEvent);
        browse.MoveContentEvent = MoveContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowContextMenuEvent = (function (_super) {
            __extends(ShowContextMenuEvent, _super);
            function ShowContextMenuEvent(x, y) {
                this.x = x;
                this.y = y;
                _super.call(this);
            }
            ShowContextMenuEvent.prototype.getX = function () {
                return this.x;
            };
            ShowContextMenuEvent.prototype.getY = function () {
                return this.y;
            };
            ShowContextMenuEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowContextMenuEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowContextMenuEvent;
        })(api.event.Event);
        browse.ShowContextMenuEvent = ShowContextMenuEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowDetailsEvent = (function (_super) {
            __extends(ShowDetailsEvent, _super);
            function ShowDetailsEvent() {
                _super.apply(this, arguments);
            }
            ShowDetailsEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowDetailsEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowDetailsEvent;
        })(browse.BaseContentModelEvent);
        browse.ShowDetailsEvent = ShowDetailsEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowNewContentDialogEvent = (function (_super) {
            __extends(ShowNewContentDialogEvent, _super);
            function ShowNewContentDialogEvent(parentContent) {
                _super.call(this, [parentContent]);
                this.parentContent = parentContent;
            }
            ShowNewContentDialogEvent.prototype.getParentContent = function () {
                return this.parentContent;
            };
            ShowNewContentDialogEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowNewContentDialogEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowNewContentDialogEvent;
        })(browse.BaseContentModelEvent);
        browse.ShowNewContentDialogEvent = ShowNewContentDialogEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowNewContentGridEvent = (function (_super) {
            __extends(ShowNewContentGridEvent, _super);
            function ShowNewContentGridEvent() {
                _super.apply(this, arguments);
            }
            ShowNewContentGridEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowNewContentGridEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowNewContentGridEvent;
        })(api.event.Event);
        browse.ShowNewContentGridEvent = ShowNewContentGridEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowPreviewEvent = (function (_super) {
            __extends(ShowPreviewEvent, _super);
            function ShowPreviewEvent() {
                _super.apply(this, arguments);
            }
            ShowPreviewEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowPreviewEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowPreviewEvent;
        })(browse.BaseContentModelEvent);
        browse.ShowPreviewEvent = ShowPreviewEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ToggleSearchPanelEvent = (function (_super) {
            __extends(ToggleSearchPanelEvent, _super);
            function ToggleSearchPanelEvent() {
                _super.apply(this, arguments);
            }
            ToggleSearchPanelEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ToggleSearchPanelEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ToggleSearchPanelEvent;
        })(api.event.Event);
        browse.ToggleSearchPanelEvent = ToggleSearchPanelEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ViewContentEvent = (function (_super) {
            __extends(ViewContentEvent, _super);
            function ViewContentEvent() {
                _super.apply(this, arguments);
            }
            ViewContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ViewContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ViewContentEvent;
        })(browse.BaseContentModelEvent);
        browse.ViewContentEvent = ViewContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentBrowseToolbar = (function (_super) {
            __extends(ContentBrowseToolbar, _super);
            function ContentBrowseToolbar(actions) {
                _super.call(this);
                this.addActions(actions.getAllActions());
            }
            return ContentBrowseToolbar;
        })(api.ui.toolbar.Toolbar);
        browse.ContentBrowseToolbar = ContentBrowseToolbar;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentBrowseItemPanel = (function (_super) {
            __extends(ContentBrowseItemPanel, _super);
            function ContentBrowseItemPanel() {
                _super.apply(this, arguments);
            }
            ContentBrowseItemPanel.prototype.createItemSelectionPanel = function () {
                return new browse.ContentBrowseItemsSelectionPanel();
            };
            ContentBrowseItemPanel.prototype.createItemStatisticsPanel = function () {
                return new app.view.ContentItemStatisticsPanel();
            };
            return ContentBrowseItemPanel;
        })(api.app.browse.BrowseItemPanel);
        browse.ContentBrowseItemPanel = ContentBrowseItemPanel;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentSummaryViewer = api.content.ContentSummaryViewer;
        var ContentBrowseItemsSelectionPanel = (function (_super) {
            __extends(ContentBrowseItemsSelectionPanel, _super);
            function ContentBrowseItemsSelectionPanel() {
                _super.apply(this, arguments);
            }
            ContentBrowseItemsSelectionPanel.prototype.createItemViewer = function (item) {
                var viewer = new ContentSummaryViewer();
                viewer.setObject(item.getModel());
                return viewer;
            };
            return ContentBrowseItemsSelectionPanel;
        })(api.app.browse.BrowseItemsSelectionPanel);
        browse.ContentBrowseItemsSelectionPanel = ContentBrowseItemsSelectionPanel;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var BrowseItem = api.app.browse.BrowseItem;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var ResponsiveManager = api.ui.responsive.ResponsiveManager;
        var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
        var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
        var ContentServerEvent = api.content.ContentServerEvent;
        var ContentServerChangeType = api.content.ContentServerChangeType;
        var ContentChangeResult = api.content.ContentChangeResult;
        var ContentBrowsePanel = (function (_super) {
            __extends(ContentBrowsePanel, _super);
            function ContentBrowsePanel() {
                var _this = this;
                this.contentTreeGrid = new app.browse.ContentTreeGrid();
                this.contentBrowseItemPanel = components.detailPanel = new browse.ContentBrowseItemPanel();
                this.contentFilterPanel = new app.browse.filter.ContentBrowseFilterPanel();
                this.browseActions = this.contentTreeGrid.getContextMenu().getActions();
                this.toolbar = new browse.ContentBrowseToolbar(this.browseActions);
                this.contentTreeGridMask = new api.ui.mask.LoadMask(this.contentTreeGrid);
                _super.call(this, {
                    browseToolbar: this.toolbar,
                    treeGrid: this.contentTreeGrid,
                    browseItemPanel: this.contentBrowseItemPanel,
                    filterPanel: this.contentFilterPanel
                });
                var showMask = function () {
                    _this.contentTreeGridMask.show();
                };
                this.contentTreeGridMask.show();
                this.contentFilterPanel.onSearch(showMask);
                this.contentFilterPanel.onReset(showMask);
                this.contentFilterPanel.onRefresh(showMask);
                this.contentTreeGrid.onRendered(showMask);
                this.contentTreeGrid.onLoaded(function () {
                    _this.contentTreeGridMask.hide();
                });
                this.getTreeGrid().onDataChanged(function (event) {
                    if (event.getType() === 'updated') {
                        var browseItems = _this.treeNodesToBrowseItems(event.getTreeNodes());
                        _this.getBrowseItemPanel().updateItemViewers(browseItems);
                    }
                });
                this.onShown(function (event) {
                    app.Router.setHash("browse");
                });
                ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                        _this.browseActions.TOGGLE_SEARCH_PANEL.setVisible(true);
                    }
                    else if (item.isInRangeOrBigger(ResponsiveRanges._540_720)) {
                        _this.browseActions.TOGGLE_SEARCH_PANEL.setVisible(false);
                    }
                });
                this.handleGlobalEvents();
            }
            ContentBrowsePanel.prototype.treeNodesToBrowseItems = function (nodes) {
                var browseItems = [];
                nodes.forEach(function (node, index) {
                    for (var i = 0; i <= index; i++) {
                        if (nodes[i].getData().getId() === node.getData().getId()) {
                            break;
                        }
                    }
                    if (i === index) {
                        var content = node.getData().getContentSummary();
                        if (!!content) {
                            var item = new BrowseItem(content).setId(content.getId()).setDisplayName(content.getDisplayName()).setPath(content.getPath().toString()).setIconUrl(new ContentIconUrlResolver().setContent(content).resolve());
                            browseItems.push(item);
                        }
                    }
                });
                return browseItems;
            };
            ContentBrowsePanel.prototype.handleGlobalEvents = function () {
                var _this = this;
                api.content.ContentMovedEvent.on(function (event) {
                    _this.contentTreeGrid.reload();
                });
                api.content.ContentChildOrderUpdatedEvent.on(function (event) {
                    _this.handleChildOrderUpdated(event);
                });
                browse.ToggleSearchPanelEvent.on(function () {
                    _this.toggleFilterPanel();
                });
                app.create.NewMediaUploadEvent.on(function (event) {
                    _this.handleNewMediaUpload(event);
                });
                var handler = this.contentServerEventHandler.bind(this);
                ContentServerEvent.on(handler);
                this.onRemoved(function () {
                    ContentServerEvent.un(handler);
                });
            };
            ContentBrowsePanel.prototype.contentServerEventHandler = function (event) {
                var _this = this;
                var changes = event.getContentChanges();
                var deferred = wemQ.defer(), promise = deferred.promise;
                changes.forEach(function (change) {
                    switch (change.getChangeType()) {
                        case 3 /* CREATE */:
                            promise = _this.handleContentCreated(change, promise);
                            break;
                        case 4 /* UPDATE */:
                            promise = _this.handleContentUpdated(change, promise);
                            break;
                        case 7 /* RENAME */:
                            promise = _this.handleContentRenamed(change, promise);
                            break;
                        case 5 /* DELETE */:
                            promise = _this.handleContentDeleted(change, promise);
                            break;
                        case 6 /* PENDING */:
                            promise = _this.handleContentPending(change, promise);
                            break;
                        case 2 /* DUPLICATE */:
                            promise = _this.handleContentCreated(change, promise);
                            break;
                        case 1 /* PUBLISH */:
                            promise = _this.handleContentPublished(change, promise);
                            break;
                        case 0 /* UNKNOWN */:
                            break;
                        default:
                    }
                });
                deferred.resolve(null);
            };
            ContentBrowsePanel.prototype.handleContentCreated = function (change, promise) {
                var _this = this;
                promise = promise.then(function (result) {
                    var createResult = _this.contentTreeGrid.findByPaths(change.getContentPaths(), true);
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(createResult.map(function (el) {
                        return el.getAltPath();
                    })).then(function (data) {
                        var isFiltered = _this.contentTreeGrid.getRoot().isFiltered(), nodes = [];
                        data.forEach(function (el) {
                            for (var i = 0; i < createResult.length; i++) {
                                if (el.getContentSummary().getPath().isChildOf(createResult[i].getPath())) {
                                    if (result && result.getChangeType() === 7 /* RENAME */) {
                                        var premerged = result.getResult().map(function (el) {
                                            return el.getNodes();
                                        });
                                        nodes = nodes.concat.apply(nodes, premerged);
                                        nodes.forEach(function (node) {
                                            if (node.getDataId() === el.getId()) {
                                                node.setData(el);
                                                node.clearViewers();
                                                _this.contentTreeGrid.updatePathsInChildren(node);
                                            }
                                        });
                                    }
                                    else {
                                        nodes = nodes.concat(_this.contentTreeGrid.xAppendContentNodes(createResult[i].getNodes().map(function (node) {
                                            return new api.content.TreeNodeParentOfContent(el, node);
                                        }), !isFiltered));
                                    }
                                    break;
                                }
                            }
                        });
                        if (result && result.getChangeType() === 5 /* DELETE */) {
                            var ids = result.getResult().map(function (el) {
                                return el.getId();
                            });
                            nodes.forEach(function (node) {
                                var index = ids.indexOf(node.getDataId());
                                if (index >= 0) {
                                    _this.contentTreeGrid.xPopulateWithChildren(result.getResult()[index].getNodes()[0], node);
                                }
                            });
                        }
                        _this.contentTreeGrid.initAndRender();
                        isFiltered = true;
                        if (isFiltered) {
                            _this.setFilterPanelRefreshNeeded(true);
                            window.setTimeout(function () {
                                _this.refreshFilter();
                            }, 1000);
                        }
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentUpdated = function (change, promise) {
                var _this = this;
                promise = promise.then(function (result) {
                    var updateResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(updateResult.map(function (el) {
                        return el.getPath();
                    })).then(function (data) {
                        data.forEach(function (el) {
                            for (var i = 0; i < updateResult.length; i++) {
                                if (updateResult[i].getId() === el.getId()) {
                                    updateResult[i].updateNodeData(el);
                                    break;
                                }
                            }
                        });
                        _this.contentTreeGrid.resetAndRender();
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentRenamed = function (change, promise) {
                var _this = this;
                promise = promise.then(function (result) {
                    var renameResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return new ContentChangeResult(7 /* RENAME */, renameResult);
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentDeleted = function (change, promise) {
                var _this = this;
                promise = promise.then(function (result) {
                    if (result && result.getChangeType() === 7 /* RENAME */) {
                        return result;
                    }
                    var deleteResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    var nodes = deleteResult.map(function (el) {
                        return el.getNodes();
                    });
                    var merged = [];
                    merged = merged.concat.apply(merged, nodes);
                    merged.forEach(function (node) {
                        if (node.getData() && node.getData().getContentSummary()) {
                            new api.content.ContentDeletedEvent(node.getData().getContentSummary().getContentId()).fire();
                        }
                    });
                    _this.contentTreeGrid.xDeleteContentNodes(merged);
                    var isFiltered = _this.contentTreeGrid.getRoot().isFiltered();
                    isFiltered = true;
                    if (isFiltered) {
                        _this.setFilterPanelRefreshNeeded(true);
                        window.setTimeout(function () {
                            _this.refreshFilter();
                        }, 1000);
                    }
                    return new ContentChangeResult(5 /* DELETE */, deleteResult);
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentPending = function (change, promise) {
                var _this = this;
                promise = promise.then(function () {
                    var pendingResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(pendingResult.map(function (el) {
                        return (el.getNodes().length > 0 && el.getNodes()[0].getData()) ? el.getNodes()[0].getData().getContentSummary().getPath() : null;
                    }).filter(function (el) {
                        return el !== null;
                    })).then(function (data) {
                        data.forEach(function (el) {
                            for (var i = 0; i < pendingResult.length; i++) {
                                if (pendingResult[i].getId() === el.getId()) {
                                    pendingResult[i].updateNodeData(el);
                                    break;
                                }
                            }
                        });
                        _this.contentTreeGrid.resetAndRender();
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentPublished = function (change, promise) {
                var _this = this;
                promise = promise.then(function () {
                    var publishResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(publishResult.map(function (el) {
                        return el.getPath();
                    })).then(function (data) {
                        data.forEach(function (el) {
                            for (var i = 0; i < publishResult.length; i++) {
                                if (publishResult[i].getId() === el.getId()) {
                                    new api.content.ContentPublishedEvent(new api.content.ContentId(el.getId())).fire();
                                    publishResult[i].updateNodeData(el);
                                    break;
                                }
                            }
                        });
                        _this.contentTreeGrid.resetAndRender();
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleChildOrderUpdated = function (event) {
                this.contentTreeGrid.updateContentNode(event.getContentId());
                var updatedNode = this.contentTreeGrid.getRoot().getCurrentRoot().findNode(event.getContentId().toString());
                this.contentTreeGrid.sortNodeChildren(updatedNode);
            };
            ContentBrowsePanel.prototype.handleNewMediaUpload = function (event) {
                var _this = this;
                event.getUploadItems().forEach(function (item) {
                    _this.contentTreeGrid.appendUploadNode(item);
                });
            };
            return ContentBrowsePanel;
        })(api.app.browse.BrowsePanel);
        browse.ContentBrowsePanel = ContentBrowsePanel;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var DateTimeFormatter = api.ui.treegrid.DateTimeFormatter;
        var TreeGridContextMenu = api.ui.treegrid.TreeGridContextMenu;
        var ContentPath = api.content.ContentPath;
        var ContentSummaryBuilder = api.content.ContentSummaryBuilder;
        var ContentSummaryAndCompareStatusViewer = api.content.ContentSummaryAndCompareStatusViewer;
        var CompareContentRequest = api.content.CompareContentRequest;
        var ContentSummaryAndCompareStatus = api.content.ContentSummaryAndCompareStatus;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var TreeNodesOfContentPath = api.content.TreeNodesOfContentPath;
        var ContentBrowseSearchEvent = app.browse.filter.ContentBrowseSearchEvent;
        var ContentBrowseResetEvent = app.browse.filter.ContentBrowseResetEvent;
        var ContentBrowseRefreshEvent = app.browse.filter.ContentBrowseRefreshEvent;
        var ContentQueryRequest = api.content.ContentQueryRequest;
        var ContentTreeGridActions = app.browse.action.ContentTreeGridActions;
        var CompareStatus = api.content.CompareStatus;
        var ContentTreeGrid = (function (_super) {
            __extends(ContentTreeGrid, _super);
            function ContentTreeGrid() {
                var _this = this;
                var nameColumn = new GridColumnBuilder().setName("Name").setId("displayName").setField("contentSummary.displayName").setMinWidth(130).setFormatter(this.nameFormatter).build();
                var compareStatusColumn = new GridColumnBuilder().setName("CompareStatus").setId("compareStatus").setField("compareStatus").setFormatter(this.statusFormatter).setCssClass("status").setMinWidth(75).setMaxWidth(75).build();
                var modifiedTimeColumn = new GridColumnBuilder().setName("ModifiedTime").setId("modifiedTime").setField("contentSummary.modifiedTime").setCssClass("modified").setMinWidth(150).setMaxWidth(170).setFormatter(DateTimeFormatter.format).build();
                var orderColumn = new GridColumnBuilder().setName("Order").setId("order").setField("contentSummary.order").setCssClass("order").setMinWidth(80).setMaxWidth(80).setFormatter(this.orderFormatter).build();
                _super.call(this, new TreeGridBuilder().setColumns([
                    nameColumn,
                    orderColumn,
                    compareStatusColumn,
                    modifiedTimeColumn
                ]).setShowContextMenu(new TreeGridContextMenu(new ContentTreeGridActions(this))).setPartialLoadEnabled(true).setLoadBufferSize(20).prependClasses("content-tree-grid"));
                api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (item.isRangeSizeChanged()) {
                        if (item.isInRangeOrSmaller(api.ui.responsive.ResponsiveRanges._240_360)) {
                            _this.getGrid().setColumns([nameColumn, orderColumn]);
                        }
                        else if (item.isInRangeOrSmaller(api.ui.responsive.ResponsiveRanges._360_540)) {
                            _this.getGrid().setColumns([nameColumn, orderColumn, modifiedTimeColumn]);
                        }
                        else {
                            _this.getGrid().setColumns([nameColumn, orderColumn, compareStatusColumn, modifiedTimeColumn]);
                        }
                        if (item.isInRangeOrSmaller(api.ui.responsive.ResponsiveRanges._540_720)) {
                            modifiedTimeColumn.setMaxWidth(100);
                            modifiedTimeColumn.setFormatter(DateTimeFormatter.formatNoTimestamp);
                        }
                        else {
                            modifiedTimeColumn.setMaxWidth(170);
                            modifiedTimeColumn.setFormatter(DateTimeFormatter.format);
                        }
                    }
                    else {
                        _this.getGrid().resizeCanvas();
                    }
                });
                this.getGrid().subscribeOnDblClick(function (event, data) {
                    if (_this.isActive()) {
                        var node = _this.getGrid().getDataView().getItem(data.row);
                        if (!!_this.getDataId(node.getData())) {
                            new browse.EditContentEvent([node.getData().getContentSummary()]).fire();
                        }
                    }
                });
                ContentBrowseSearchEvent.on(function (event) {
                    var contentQueryResult = event.getContentQueryResult();
                    var contentSummaries = contentQueryResult.getContents(), compareRequest = CompareContentRequest.fromContentSummaries(contentSummaries);
                    _this.filterQuery = event.getContentQuery();
                    compareRequest.sendAndParse().then(function (compareResults) {
                        var contents = ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults);
                        var metadata = contentQueryResult.getMetadata();
                        if (metadata.getTotalHits() > metadata.getHits()) {
                            contents.push(new ContentSummaryAndCompareStatus());
                        }
                        _this.filter(contents);
                        _this.getRoot().getCurrentRoot().setMaxChildren(metadata.getTotalHits());
                        _this.notifyLoaded();
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                });
                ContentBrowseResetEvent.on(function (event) {
                    _this.resetFilter();
                });
                ContentBrowseRefreshEvent.on(function (event) {
                    _this.notifyLoaded();
                });
            }
            ContentTreeGrid.prototype.typeFormatter = function (row, cell, value, columnDef, node) {
                var wrapper = new api.dom.SpanEl();
                wrapper.getEl().setTitle(value);
                wrapper.getEl().setInnerHtml(value.toString().split(':')[1]);
                return wrapper.toString();
            };
            ContentTreeGrid.prototype.orderFormatter = function (row, cell, value, columnDef, node) {
                var wrapper = new api.dom.SpanEl();
                wrapper.getEl().setTitle(value);
                if (node.getData().getContentSummary()) {
                    var childOrder = node.getData().getContentSummary().getChildOrder();
                    var icon;
                    if (!childOrder.isDefault()) {
                        if (!childOrder.isManual()) {
                            if (childOrder.isDesc()) {
                                icon = new api.dom.DivEl("icon-arrow-up4");
                            }
                            else {
                                icon = new api.dom.DivEl("icon-arrow-down4");
                            }
                        }
                        else {
                            icon = new api.dom.DivEl("icon-menu3");
                        }
                        wrapper.getEl().setInnerHtml(icon.toString());
                    }
                }
                return wrapper.toString();
            };
            ContentTreeGrid.prototype.statusFormatter = function (row, cell, value, columnDef, node) {
                var data = node.getData(), status, statusEl = new api.dom.SpanEl();
                if (!!data.getContentSummary()) {
                    var compareStatus = CompareStatus[CompareStatus[value]];
                    switch (compareStatus) {
                        case 0 /* NEW */:
                            status = "New";
                            break;
                        case 2 /* NEWER */:
                            status = "Modified";
                            break;
                        case 3 /* OLDER */:
                            status = "Behind";
                            break;
                        case 10 /* UNKNOWN */:
                            status = "Unknown";
                            break;
                        case 4 /* PENDING_DELETE */:
                            status = "Pending delete";
                            break;
                        case 6 /* EQUAL */:
                            status = "Online";
                            break;
                        case 7 /* MOVED */:
                            status = "Moved";
                            break;
                        case 5 /* PENDING_DELETE_TARGET */:
                            status = "Deleted in prod";
                            break;
                        case 1 /* NEW_TARGET */:
                            status = "New in prod";
                            break;
                        case 8 /* CONFLICT_PATH_EXISTS */:
                            status = "Conflict";
                            break;
                        default:
                            status = "Unknown";
                    }
                    if (!!CompareStatus[value]) {
                        statusEl.addClass(CompareStatus[value].toLowerCase().replace("_", "-") || "unknown");
                    }
                    statusEl.getEl().setText(status);
                }
                else if (!!data.getUploadItem()) {
                    status = new api.ui.ProgressBar(data.getUploadItem().getProgress());
                    statusEl.appendChild(status);
                }
                return statusEl.toString();
            };
            ContentTreeGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                var data = node.getData();
                if (!!data.getContentSummary() || !!data.getUploadItem()) {
                    var viewer = node.getViewer("name");
                    if (!viewer) {
                        viewer = new ContentSummaryAndCompareStatusViewer();
                        node.setViewer("name", viewer);
                    }
                    viewer.setObject(node.getData(), node.calcLevel() > 1);
                    return viewer.toString();
                }
                else {
                    var content = new api.dom.DivEl("children-to-load"), parent = node.getParent();
                    content.setHtml((parent.getMaxChildren() - parent.getChildren().length + 1) + " children left to load.");
                    return content.toString();
                }
            };
            ContentTreeGrid.prototype.fetch = function (node) {
                return this.fetchById(node.getData().getContentId());
            };
            ContentTreeGrid.prototype.fetchById = function (id) {
                return ContentSummaryAndCompareStatusFetcher.fetch(id);
            };
            ContentTreeGrid.prototype.fetchChildren = function (parentNode) {
                var parentContentId = null;
                if (parentNode) {
                    parentContentId = parentNode.getData() ? parentNode.getData().getContentId() : parentContentId;
                }
                else {
                    parentNode = this.getRoot().getCurrentRoot();
                }
                var from = parentNode.getChildren().length;
                if (from > 0 && !parentNode.getChildren()[from - 1].getData().getContentSummary()) {
                    parentNode.getChildren().pop();
                    from--;
                }
                if (!this.isFiltered() || parentNode != this.getRoot().getCurrentRoot()) {
                    return ContentSummaryAndCompareStatusFetcher.fetchChildren(parentContentId, from, ContentTreeGrid.MAX_FETCH_SIZE).then(function (data) {
                        var contents = parentNode.getChildren().map(function (el) {
                            return el.getData();
                        }).slice(0, from).concat(data.getContents());
                        var meta = data.getMetadata();
                        parentNode.setMaxChildren(meta.getTotalHits());
                        if (from + meta.getHits() < meta.getTotalHits()) {
                            contents.push(new ContentSummaryAndCompareStatus());
                        }
                        return contents;
                    });
                }
                else {
                    this.filterQuery.setFrom(from);
                    this.filterQuery.setSize(ContentTreeGrid.MAX_FETCH_SIZE);
                    return new ContentQueryRequest(this.filterQuery).setExpand(1 /* SUMMARY */).sendAndParse().then(function (contentQueryResult) {
                        var contentSummaries = contentQueryResult.getContents();
                        var compareRequest = CompareContentRequest.fromContentSummaries(contentSummaries);
                        return compareRequest.sendAndParse().then(function (compareResults) {
                            var list = parentNode.getChildren().map(function (el) {
                                return el.getData();
                            }).slice(0, from).concat(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                            var meta = contentQueryResult.getMetadata();
                            if (from + meta.getHits() < meta.getTotalHits()) {
                                list.push(new ContentSummaryAndCompareStatus());
                            }
                            parentNode.setMaxChildren(meta.getTotalHits());
                            return list;
                        });
                    });
                }
            };
            ContentTreeGrid.prototype.hasChildren = function (data) {
                return data.hasChildren();
            };
            ContentTreeGrid.prototype.getDataId = function (data) {
                return data.getId();
            };
            ContentTreeGrid.prototype.deleteNodes = function (dataList) {
                var _this = this;
                var root = this.getRoot().getCurrentRoot(), node;
                dataList = dataList.filter(function (data) {
                    node = root.findNode(_this.getDataId(data));
                    if (node.getData().getCompareStatus() !== 0 /* NEW */) {
                        node.clearViewers();
                        return false;
                    }
                    return true;
                });
                _super.prototype.deleteNodes.call(this, dataList);
            };
            ContentTreeGrid.prototype.updateContentNode = function (contentId) {
                var root = this.getRoot().getCurrentRoot();
                var treeNode = root.findNode(contentId.toString());
                if (treeNode) {
                    var content = treeNode.getData();
                    this.updateNode(ContentSummaryAndCompareStatus.fromContentSummary(content.getContentSummary()));
                }
            };
            ContentTreeGrid.prototype.appendContentNode = function (contentId, nextToSelection) {
                var _this = this;
                this.fetchById(contentId).then(function (data) {
                    _this.appendNode(data, nextToSelection);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                });
            };
            ContentTreeGrid.prototype.appendUploadNode = function (item) {
                var _this = this;
                var data = ContentSummaryAndCompareStatus.fromUploadItem(item);
                var parent = this.getRoot().getCurrentSelection()[0];
                this.appendNode(data, false).then(function () {
                    if (parent) {
                        var parentData = parent.getData();
                        var contentSummary = new ContentSummaryBuilder(parentData.getContentSummary()).setHasChildren(true).build();
                        _this.updateNode(parentData.setContentSummary(contentSummary));
                        _this.expandNode(parent);
                    }
                }).done();
                item.onProgress(function (progress) {
                    _this.resetAndRender();
                });
                item.onUploaded(function (model) {
                    var nodeToRemove = _this.getRoot().getCurrentRoot().findNode(item.getId());
                    if (nodeToRemove) {
                        nodeToRemove.remove();
                        _this.resetAndRender();
                    }
                    api.notify.showFeedback(data.getContentSummary().getType().toString() + " \"" + item.getName() + "\" created successfully");
                });
                item.onFailed(function () {
                    _this.deleteNode(data);
                });
            };
            ContentTreeGrid.prototype.refreshNodeData = function (parentNode) {
                var _this = this;
                return ContentSummaryAndCompareStatusFetcher.fetch(parentNode.getData().getContentId()).then(function (content) {
                    parentNode.setData(content);
                    _this.refreshNode(parentNode);
                    return parentNode;
                });
            };
            ContentTreeGrid.prototype.sortNodeChildren = function (node) {
                var _this = this;
                var rootNode = this.getRoot().getCurrentRoot();
                if (node != rootNode) {
                    if (node.hasChildren()) {
                        node.setChildren([]);
                        node.setMaxChildren(0);
                        this.fetchChildren(node).then(function (dataList) {
                            var parentNode = _this.getRoot().getCurrentRoot().findNode(node.getDataId());
                            parentNode.setChildren(_this.dataToTreeNodes(dataList, parentNode));
                            var rootList = _this.getRoot().getCurrentRoot().treeToList();
                            _this.initData(rootList);
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    }
                }
            };
            ContentTreeGrid.prototype.findByPaths = function (paths, useParent) {
                if (useParent === void 0) { useParent = false; }
                var root = this.getRoot().getDefaultRoot().treeToList(false, false), filter = this.getRoot().getFilteredRoot().treeToList(false, false), all = root.concat(filter), result = [];
                for (var i = 0; i < paths.length; i++) {
                    var node = useParent ? new TreeNodesOfContentPath(paths[i].getParentPath(), paths[i]) : new TreeNodesOfContentPath(paths[i]);
                    if (useParent && node.getPath().isRoot()) {
                        node.getNodes().push(this.getRoot().getDefaultRoot());
                    }
                    else {
                        for (var j = 0; j < all.length; j++) {
                            var path = (all[j].getData() && all[j].getData().getContentSummary()) ? all[j].getData().getContentSummary().getPath() : null;
                            if (path && path.equals(node.getPath())) {
                                node.getNodes().push(all[j]);
                            }
                        }
                    }
                    if (node.hasNodes()) {
                        result.push(node);
                    }
                }
                return result;
            };
            ContentTreeGrid.prototype.xAppendContentNode = function (relationship, update) {
                if (update === void 0) { update = true; }
                var appendedNode = this.dataToTreeNode(relationship.getData(), relationship.getNode());
                relationship.getNode().addChild(appendedNode, true);
                var data = relationship.getNode().getData();
                if (data && relationship.getNode().hasChildren() && !data.getContentSummary().hasChildren()) {
                    data.setContentSummary(new ContentSummaryBuilder(data.getContentSummary()).setHasChildren(true).build());
                }
                relationship.getNode().clearViewers();
                if (update) {
                    this.initAndRender();
                }
                return appendedNode;
            };
            ContentTreeGrid.prototype.xAppendContentNodes = function (relationships, update) {
                var _this = this;
                if (update === void 0) { update = true; }
                var nodes = [];
                relationships.forEach(function (relationship) {
                    nodes.push(_this.xAppendContentNode(relationship, false));
                });
                if (update) {
                    this.initAndRender();
                }
                return nodes;
            };
            ContentTreeGrid.prototype.xDeleteContentNode = function (node, update) {
                if (update === void 0) { update = true; }
                var parentNode = node.getParent();
                node.remove();
                var data = !!parentNode ? parentNode.getData() : null;
                if (data && !parentNode.hasChildren() && data.getContentSummary().hasChildren()) {
                    data.setContentSummary(new ContentSummaryBuilder(data.getContentSummary()).setHasChildren(false).build());
                }
                if (update) {
                    this.initAndRender();
                }
            };
            ContentTreeGrid.prototype.xDeleteContentNodes = function (nodes, update) {
                var _this = this;
                if (update === void 0) { update = true; }
                nodes.forEach(function (node) {
                    _this.xDeleteContentNode(node, false);
                });
                if (update) {
                    this.initAndRender();
                }
            };
            ContentTreeGrid.prototype.xPopulateWithChildren = function (source, dest) {
                dest.setChildren(source.getChildren());
                dest.setExpanded(source.isExpanded());
                if (dest.getData() && dest.getData().getContentSummary()) {
                    dest.getData().setContentSummary(new ContentSummaryBuilder(dest.getData().getContentSummary()).setHasChildren(dest.hasChildren()).build());
                    this.updatePathsInChildren(dest);
                }
                dest.clearViewers();
            };
            ContentTreeGrid.prototype.updatePathsInChildren = function (node) {
                var _this = this;
                node.getChildren().forEach(function (child) {
                    var nodeSummary = node.getData() ? node.getData().getContentSummary() : null, childSummary = child.getData() ? child.getData().getContentSummary() : null;
                    if (nodeSummary && childSummary) {
                        var path = ContentPath.fromParent(nodeSummary.getPath(), childSummary.getPath().getName());
                        child.getData().setContentSummary(new ContentSummaryBuilder(childSummary).setPath(path).build());
                        child.clearViewers();
                        _this.updatePathsInChildren(child);
                    }
                });
            };
            ContentTreeGrid.MAX_FETCH_SIZE = 10;
            return ContentTreeGrid;
        })(TreeGrid);
        browse.ContentTreeGrid = ContentTreeGrid;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SortContentEvent = (function (_super) {
            __extends(SortContentEvent, _super);
            function SortContentEvent() {
                _super.apply(this, arguments);
            }
            SortContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            SortContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return SortContentEvent;
        })(browse.BaseContentModelEvent);
        browse.SortContentEvent = SortContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var OpenSortDialogEvent = (function (_super) {
            __extends(OpenSortDialogEvent, _super);
            function OpenSortDialogEvent(content) {
                _super.call(this);
                this.content = content;
            }
            OpenSortDialogEvent.prototype.getContent = function () {
                return this.content;
            };
            OpenSortDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            OpenSortDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return OpenSortDialogEvent;
        })(api.event.Event);
        browse.OpenSortDialogEvent = OpenSortDialogEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var OpenMoveDialogEvent = (function (_super) {
            __extends(OpenMoveDialogEvent, _super);
            function OpenMoveDialogEvent(content) {
                _super.call(this);
                this.content = content;
            }
            OpenMoveDialogEvent.prototype.getContent = function () {
                return this.content;
            };
            OpenMoveDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            OpenMoveDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return OpenMoveDialogEvent;
        })(api.event.Event);
        browse.OpenMoveDialogEvent = OpenMoveDialogEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SaveSortedContentAction = browse.action.SaveSortedContentAction;
        var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
        var SortContentDialog = (function (_super) {
            __extends(SortContentDialog, _super);
            function SortContentDialog() {
                var _this = this;
                _super.call(this, {
                    title: new api.ui.dialog.ModalDialogHeader("Default sorting")
                });
                var menu = new api.ui.tab.TabMenu();
                var tabMenuItem = new TabMenuItemBuilder().setLabel("(sorting type)").build();
                tabMenuItem.setActive(true);
                menu.addNavigationItem(tabMenuItem);
                menu.selectNavigationItem(0);
                menu.show();
                this.sortContentMenu = new browse.SortContentTabMenu();
                this.sortContentMenu.show();
                this.appendChildToTitle(this.sortContentMenu);
                this.sortContentMenu.onSortOrderChanged(function () {
                    var newOrder = _this.sortContentMenu.getSelectedNavigationItem().getChildOrder();
                    if (!_this.curChildOrder.equals(newOrder)) {
                        if (!newOrder.isManual()) {
                            _this.curChildOrder = newOrder;
                            _this.contentGrid.setChildOrder(_this.curChildOrder);
                            api.content.ContentSummaryAndCompareStatusFetcher.fetch(_this.parentContent.getContentId()).done(function (response) {
                                _this.contentGrid.reload(response);
                            });
                            _this.gridDragHandler.clearContentMovements();
                        }
                        else {
                            _this.prevChildOrder = _this.curChildOrder;
                            _this.curChildOrder = newOrder;
                            _this.contentGrid.setChildOrder(_this.curChildOrder);
                        }
                    }
                });
                this.getEl().addClass("sort-content-dialog");
                this.sortAction = new SaveSortedContentAction(this);
                this.addAction(this.sortAction);
                this.contentGrid = new app.browse.SortContentTreeGrid();
                this.contentGrid.getEl().addClass("sort-content-grid");
                this.contentGrid.onLoaded(function () {
                    _this.contentGrid.render(true);
                    if (_this.contentGrid.getContentId()) {
                        _this.open();
                    }
                });
                this.gridDragHandler = new browse.ContentGridDragHandler(this.contentGrid);
                this.gridDragHandler.onPositionChanged(function () {
                    _this.sortContentMenu.selectManualSortingItem();
                });
                var header = new api.dom.H6El();
                header.setHtml("Sort content by selecting default sort above, or drag and drop for manual sorting");
                this.appendChildToContentPanel(header);
                this.appendChildToContentPanel(this.contentGrid);
                this.sortAction.onExecuted(function () {
                    if (_this.curChildOrder.equals(_this.parentContent.getChildOrder()) && !_this.curChildOrder.isManual()) {
                        _this.close();
                    }
                    else {
                        if (_this.curChildOrder.isManual()) {
                            if (_this.prevChildOrder && !_this.prevChildOrder.isManual()) {
                                _this.setContentChildOrder(_this.prevChildOrder).done(function () {
                                    _this.setContentChildOrder(_this.curChildOrder).done(function () {
                                        _this.setManualReorder(_this.gridDragHandler.getContentMovements()).done(function () {
                                            new api.content.ContentChildOrderUpdatedEvent(_this.parentContent.getContentId()).fire();
                                            _this.close();
                                        });
                                    });
                                });
                            }
                            else {
                                _this.setManualReorder(_this.gridDragHandler.getContentMovements()).done(function () {
                                    new api.content.ContentChildOrderUpdatedEvent(_this.parentContent.getContentId()).fire();
                                    _this.close();
                                });
                            }
                        }
                        else {
                            _this.setContentChildOrder(_this.curChildOrder).done(function () {
                                new api.content.ContentChildOrderUpdatedEvent(_this.parentContent.getContentId()).fire();
                                _this.close();
                            });
                        }
                    }
                });
                browse.OpenSortDialogEvent.on(function (event) {
                    _this.parentContent = event.getContent();
                    _this.curChildOrder = _this.parentContent.getChildOrder();
                    _this.sortContentMenu.selectNavigationItemByOrder(_this.curChildOrder);
                    api.content.ContentSummaryAndCompareStatusFetcher.fetch(_this.parentContent.getContentId()).done(function (response) {
                        _this.contentGrid.reload(response);
                        if (!response.hasChildren()) {
                            _this.contentGrid.getEl().setAttribute("data-content", event.getContent().getPath().toString());
                            _this.contentGrid.addClass("no-content");
                        }
                        else {
                            _this.contentGrid.removeClass("no-content");
                            _this.contentGrid.getEl().removeAttribute("data-content");
                        }
                    });
                });
                this.addCancelButtonToBottom();
            }
            SortContentDialog.prototype.open = function () {
                if (!this.isOpen) {
                    if (this.contentGrid.getGrid().getDataView().getLength() > 0) {
                        this.contentGrid.getGrid().getEl().setHeightPx(45);
                    }
                    this.contentGrid.getGrid().resizeCanvas();
                    _super.prototype.open.call(this);
                    this.isOpen = true;
                }
            };
            SortContentDialog.prototype.show = function () {
                api.dom.Body.get().appendChild(this);
                _super.prototype.show.call(this);
            };
            SortContentDialog.prototype.close = function () {
                _super.prototype.close.call(this);
                this.remove();
                this.isOpen = false;
                this.contentGrid.setChildOrder(null);
                this.gridDragHandler.clearContentMovements();
            };
            SortContentDialog.prototype.getContent = function () {
                return this.parentContent;
            };
            SortContentDialog.prototype.setContentChildOrder = function (order) {
                return new api.content.OrderContentRequest().setContentId(this.parentContent.getContentId()).setChildOrder(order).sendAndParse();
            };
            SortContentDialog.prototype.setManualReorder = function (movements) {
                return new api.content.OrderChildContentRequest().setContentMovements(movements).sendAndParse();
            };
            return SortContentDialog;
        })(api.ui.dialog.ModalDialog);
        browse.SortContentDialog = SortContentDialog;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentPath = api.content.ContentPath;
        var MoveContentDialog = (function (_super) {
            __extends(MoveContentDialog, _super);
            function MoveContentDialog() {
                var _this = this;
                _super.call(this, {
                    title: new api.ui.dialog.ModalDialogHeader("Move item with children")
                });
                this.getEl().addClass("move-content-dialog");
                this.contentPath = new api.dom.H6El().addClass("content-path");
                var descMessage = new api.dom.H6El().addClass("desc-message").setHtml("Moves selected items with all children and current permissions to selected destination");
                this.appendChildToContentPanel(this.contentPath);
                this.appendChildToContentPanel(descMessage);
                this.contentComboBox = new api.content.ContentMoveComboBox();
                this.contentComboBox.addClass("content-selector");
                this.appendChildToContentPanel(this.contentComboBox);
                this.initializeActions();
                this.contentMoveMask = new api.ui.mask.LoadMask(this);
                this.appendChildToContentPanel(this.contentMoveMask);
                browse.OpenMoveDialogEvent.on(function (event) {
                    _this.movedContentSummary = event.getContent();
                    _this.contentComboBox.setFilterContentPath(_this.movedContentSummary.getPath());
                    _this.contentPath.setHtml(event.getContent().getPath().toString());
                    _this.open();
                });
                this.addCancelButtonToBottom();
            }
            MoveContentDialog.prototype.initializeActions = function () {
                var _this = this;
                this.addAction(new api.ui.Action("Move", "").onExecuted(function () {
                    _this.contentMoveMask.show();
                    var parentContent = _this.getParentContent();
                    _this.moveContent(parentContent);
                }));
            };
            MoveContentDialog.prototype.moveContent = function (parentContent) {
                var _this = this;
                var parentRoot = (!!parentContent) ? parentContent.getPath() : ContentPath.ROOT;
                new api.content.MoveContentRequest(this.movedContentSummary.getContentId(), parentRoot).sendAndParse().then(function (content) {
                    if (parentContent) {
                        _this.contentComboBox.deselect(parentContent);
                    }
                    _this.contentMoveMask.hide();
                    new api.content.ContentMovedEvent(content.getContentId()).fire();
                    api.notify.showFeedback('Content was moved!');
                    _this.close();
                }).catch(function (reason) {
                    api.notify.showWarning(reason.getMessage());
                    _this.close();
                    _this.contentComboBox.deselect(_this.getParentContent());
                }).done();
            };
            MoveContentDialog.prototype.getParentContent = function () {
                return this.contentComboBox.getSelectedDisplayValues()[0];
            };
            MoveContentDialog.prototype.open = function () {
                _super.prototype.open.call(this);
            };
            MoveContentDialog.prototype.show = function () {
                api.dom.Body.get().appendChild(this);
                _super.prototype.show.call(this);
            };
            MoveContentDialog.prototype.close = function () {
                _super.prototype.close.call(this);
            };
            return MoveContentDialog;
        })(api.ui.dialog.ModalDialog);
        browse.MoveContentDialog = MoveContentDialog;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var ContentSummaryViewer = api.content.ContentSummaryViewer;
        var ContentSummaryAndCompareStatus = api.content.ContentSummaryAndCompareStatus;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var CompareStatus = api.content.CompareStatus;
        var SortContentTreeGrid = (function (_super) {
            __extends(SortContentTreeGrid, _super);
            function SortContentTreeGrid() {
                var dragColumn = new GridColumnBuilder().setName("Drag").setId("drag").setMinWidth(50).setMaxWidth(50).setField("contentSummary.order").setCssClass("drag-icon").setFormatter(this.dragFormatter).setBehavior("selectAndMove").build();
                var nameColumn = new GridColumnBuilder().setName("Name").setId("displayName").setField("contentSummary.displayName").setMinWidth(130).setFormatter(this.nameFormatter).build();
                var compareStatusColumn = new GridColumnBuilder().setName("CompareStatus").setId("compareStatus").setField("compareContentResult.compareStatus").setFormatter(this.statusFormatter).setCssClass("status").setMinWidth(75).setMaxWidth(75).build();
                _super.call(this, new TreeGridBuilder().setColumns([
                    dragColumn,
                    nameColumn,
                    compareStatusColumn
                ]).setPartialLoadEnabled(true).setCheckableRows(false).setShowToolbar(false).setDragAndDrop(true).disableMultipleSelection(true).prependClasses("content-tree-grid").setSelectedCellCssClass("selected-sort-row selected"));
            }
            SortContentTreeGrid.prototype.statusFormatter = function (row, cell, value, columnDef, node) {
                if (!node.getData().getContentSummary()) {
                    return "";
                }
                var compareLabel = api.content.CompareStatus[value];
                var compareStatus = CompareStatus[compareLabel];
                switch (compareStatus) {
                    case 0 /* NEW */:
                        return "New";
                        break;
                    case 2 /* NEWER */:
                        return "Modified";
                        break;
                    case 3 /* OLDER */:
                        return "Behind";
                        break;
                    case 10 /* UNKNOWN */:
                        return "Unknown";
                        break;
                    case 4 /* PENDING_DELETE */:
                        return "Deleted";
                        break;
                    case 6 /* EQUAL */:
                        return "Online";
                        break;
                    default:
                        return "Unknown";
                }
            };
            SortContentTreeGrid.prototype.dragFormatter = function (row, cell, value, columnDef, node) {
                var wrapper = new api.dom.SpanEl();
                wrapper.getEl().setTitle(value);
                var icon = new api.dom.DivEl("icon-menu3 drag-icon");
                wrapper.getEl().setInnerHtml(icon.toString());
                return wrapper.toString();
            };
            SortContentTreeGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                if (!!node.getData().getContentSummary()) {
                    var viewer = node.getViewer("name");
                    if (!viewer) {
                        viewer = new ContentSummaryViewer();
                        viewer.setObject(node.getData().getContentSummary(), node.calcLevel() > 1);
                        node.setViewer("name", viewer);
                    }
                    return viewer.toString();
                }
                else {
                    var content = new api.dom.DivEl("children-to-load"), parent = node.getParent();
                    content.setHtml((parent.getMaxChildren() - parent.getChildren().length + 1) + " children left to load.");
                    return content.toString();
                }
            };
            SortContentTreeGrid.prototype.sortNodeChildren = function (node) {
                this.initData(this.getRoot().getCurrentRoot().treeToList());
            };
            SortContentTreeGrid.prototype.fetchChildren = function () {
                var parentContentId;
                var parentNode = this.getRoot().getCurrentRoot();
                if (parentNode.getData()) {
                    parentContentId = parentNode.getData().getContentSummary().getContentId();
                    this.contentId = parentContentId;
                    parentNode.setData(null);
                }
                else {
                    parentContentId = this.contentId;
                }
                var from = parentNode.getChildren().length;
                if (from > 0 && !parentNode.getChildren()[from - 1].getData().getContentSummary()) {
                    parentNode.getChildren().pop();
                    from--;
                }
                return ContentSummaryAndCompareStatusFetcher.fetchChildren(parentContentId, from, SortContentTreeGrid.MAX_FETCH_SIZE, this.curChildOrder).then(function (data) {
                    var contents = parentNode.getChildren().map(function (el) {
                        return el.getData();
                    }).slice(0, from).concat(data.getContents());
                    var meta = data.getMetadata();
                    parentNode.setMaxChildren(meta.getTotalHits());
                    if (from + meta.getHits() < meta.getTotalHits()) {
                        contents.push(new ContentSummaryAndCompareStatus());
                    }
                    return contents;
                });
            };
            SortContentTreeGrid.prototype.hasChildren = function (data) {
                return data.hasChildren();
            };
            SortContentTreeGrid.prototype.getDataId = function (data) {
                return data.getId();
            };
            SortContentTreeGrid.prototype.getContentId = function () {
                return this.contentId;
            };
            SortContentTreeGrid.prototype.getChildOrder = function () {
                return this.curChildOrder;
            };
            SortContentTreeGrid.prototype.setChildOrder = function (value) {
                this.curChildOrder = value;
            };
            SortContentTreeGrid.MAX_FETCH_SIZE = 30;
            return SortContentTreeGrid;
        })(TreeGrid);
        browse.SortContentTreeGrid = SortContentTreeGrid;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SortContentTabMenu = (function (_super) {
            __extends(SortContentTabMenu, _super);
            function SortContentTabMenu() {
                _super.call(this, "sort-tab-menu");
                this.sortOrderChangedListeners = [];
                this.navigationItems = new browse.SortContentTabMenuItems();
                this.addNavigationItems(this.navigationItems.getAllItems());
                this.selectNavigationItem(0);
            }
            SortContentTabMenu.prototype.selectNavigationItem = function (tabIndex) {
                _super.prototype.selectNavigationItem.call(this, tabIndex);
                this.notifySortOrderChanged();
            };
            SortContentTabMenu.prototype.getSelectedNavigationItem = function () {
                return _super.prototype.getSelectedNavigationItem.call(this);
            };
            SortContentTabMenu.prototype.getSortMenuNavigationItems = function () {
                return this.navigationItems;
            };
            SortContentTabMenu.prototype.addNavigationItems = function (items) {
                var _this = this;
                if (items) {
                    items.forEach(function (item) {
                        _this.addNavigationItem(item);
                    });
                }
            };
            SortContentTabMenu.prototype.selectNavigationItemByOrder = function (order) {
                var items = this.navigationItems.getAllItems();
                for (var key in items) {
                    if (items[key].getChildOrder().equals(order)) {
                        var item = items.splice(key, 1)[0];
                        this.removeNavigationItems();
                        this.addNavigationItem(item);
                        this.addNavigationItems(items);
                        this.selectNavigationItem(0);
                        break;
                    }
                }
            };
            SortContentTabMenu.prototype.selectManualSortingItem = function () {
                this.selectNavigationItemByOrder(this.navigationItems.SORT_MANUAL_ITEM.getChildOrder());
            };
            SortContentTabMenu.prototype.onSortOrderChanged = function (listener) {
                this.sortOrderChangedListeners.push(listener);
            };
            SortContentTabMenu.prototype.unSortOrderChanged = function (listener) {
                this.sortOrderChangedListeners = this.sortOrderChangedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            SortContentTabMenu.prototype.notifySortOrderChanged = function () {
                var _this = this;
                this.sortOrderChangedListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            return SortContentTabMenu;
        })(api.ui.tab.TabMenu);
        browse.SortContentTabMenu = SortContentTabMenu;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
        var SortContentTabMenuItem = (function (_super) {
            __extends(SortContentTabMenuItem, _super);
            function SortContentTabMenuItem(builder) {
                _super.call(this, new TabMenuItemBuilder().setLabel(builder.label));
                this.childOrder = builder.childOrder;
            }
            SortContentTabMenuItem.prototype.getChildOrder = function () {
                return this.childOrder;
            };
            return SortContentTabMenuItem;
        })(api.ui.tab.TabMenuItem);
        browse.SortContentTabMenuItem = SortContentTabMenuItem;
        var SortContentTabMenuItemBuilder = (function () {
            function SortContentTabMenuItemBuilder() {
            }
            SortContentTabMenuItemBuilder.prototype.setLabel = function (label) {
                this.label = label;
                return this;
            };
            SortContentTabMenuItemBuilder.prototype.setChildOrder = function (value) {
                this.childOrder = value;
                return this;
            };
            SortContentTabMenuItemBuilder.prototype.build = function () {
                return new SortContentTabMenuItem(this);
            };
            return SortContentTabMenuItemBuilder;
        })();
        browse.SortContentTabMenuItemBuilder = SortContentTabMenuItemBuilder;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ChildOrder = api.content.ChildOrder;
        var FieldOrderExprBuilder = api.content.FieldOrderExprBuilder;
        var SortContentTabMenuItems = (function () {
            function SortContentTabMenuItems() {
                this.items = [];
                var order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.DISPLAY_NAME).setDirection(ChildOrder.ASC_ORDER_DIRECTION_VALUE).build());
                this.SORT_ASC_DISPALAY_NAME_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("DisplayName - Ascending").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.DISPLAY_NAME).setDirection(ChildOrder.DESC_ORDER_DIRECTION_VALUE).build());
                this.SORT_DESC_DISPALAY_NAME_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("DisplayName - Descending").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.MODIFIED_TIME).setDirection(ChildOrder.ASC_ORDER_DIRECTION_VALUE).build());
                this.SORT_ASC_MODIFIED_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("Modified - Ascending").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.MODIFIED_TIME).setDirection(ChildOrder.DESC_ORDER_DIRECTION_VALUE).build());
                this.SORT_DESC_MODIFIED_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("Modified - Descending (default)").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.MANUAL_ORDER_VALUE).setDirection(ChildOrder.DESC_ORDER_DIRECTION_VALUE).build());
                this.SORT_MANUAL_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("Manually Sorted").setChildOrder(order).build();
                this.items.push(this.SORT_ASC_DISPALAY_NAME_ITEM, this.SORT_DESC_DISPALAY_NAME_ITEM, this.SORT_ASC_MODIFIED_ITEM, this.SORT_DESC_MODIFIED_ITEM, this.SORT_MANUAL_ITEM);
            }
            SortContentTabMenuItems.prototype.getAllItems = function () {
                return this.items.slice(0, this.items.length);
            };
            return SortContentTabMenuItems;
        })();
        browse.SortContentTabMenuItems = SortContentTabMenuItems;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SaveSortedContentEvent = (function (_super) {
            __extends(SaveSortedContentEvent, _super);
            function SaveSortedContentEvent(content) {
                this.content = content;
                _super.call(this);
            }
            SaveSortedContentEvent.prototype.getContent = function () {
                return this.content;
            };
            SaveSortedContentEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            SaveSortedContentEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return SaveSortedContentEvent;
        })(api.event.Event);
        browse.SaveSortedContentEvent = SaveSortedContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var OrderChildMovement = api.content.OrderChildMovement;
        var OrderChildMovements = api.content.OrderChildMovements;
        var Element = api.dom.Element;
        var ContentGridDragHandler = (function () {
            function ContentGridDragHandler(treeGrid) {
                this.positionChangedListeners = [];
                this.prev = 0;
                this.contentGrid = treeGrid;
                this.contentGrid.getGrid().subscribeOnDrag(this.handleDrag.bind(this));
                this.contentGrid.getGrid().subscribeOnDragEnd(this.handleDragEnd.bind(this));
                this.contentGrid.getGrid().subscribeBeforeMoveRows(this.handleBeforeMoveRows.bind(this));
                this.contentGrid.getGrid().subscribeMoveRows(this.handleMoveRows.bind(this));
                this.movements = new OrderChildMovements();
            }
            ContentGridDragHandler.prototype.getContentMovements = function () {
                return this.movements;
            };
            ContentGridDragHandler.prototype.clearContentMovements = function () {
                this.movements = new OrderChildMovements();
            };
            ContentGridDragHandler.prototype.handleDragStart = function () {
                var draggableClass = this.contentGrid.getOptions().getSelectedCellCssClass();
                draggableClass = (" " + draggableClass).replace(/\s/g, ".");
                var row = Element.fromString(draggableClass).getParentElement();
                this.draggableTop = row.getEl().getTopPx();
                this.draggableItem = Element.fromString(row.toString());
                row.getEl().setDisplay("none");
                this.rowHeight = row.getEl().getHeight();
                var proxyEl = Element.fromString(".slick-reorder-proxy").getEl();
                this.draggableItem.getEl().setTop(proxyEl.getTop()).setPosition("absolute");
                var gridClasses = (" " + this.contentGrid.getGrid().getEl().getClass()).replace(/\s/g, ".");
                wemjq(".tree-grid " + gridClasses + " .slick-viewport").append(this.draggableItem.getHTMLElement());
            };
            ContentGridDragHandler.prototype.handleDrag = function (event, data) {
                if (!this.draggableItem) {
                    this.handleDragStart();
                }
                var top = Element.fromString(".slick-reorder-proxy").getEl().getTopPx();
                this.draggableItem.getEl().setTopPx(top - this.rowHeight / 2).setZindex(2);
            };
            ContentGridDragHandler.prototype.handleDragEnd = function (event, data) {
                this.contentGrid.refresh();
                this.draggableItem.remove();
                this.draggableItem = null;
            };
            ContentGridDragHandler.prototype.handleBeforeMoveRows = function (event, data) {
                if (!this.draggableItem) {
                    this.handleDragStart();
                }
                var gridClasses = (" " + this.contentGrid.getGrid().getEl().getClass()).replace(/\s/g, "."), children = Element.elementsFromRequest(".tree-grid " + gridClasses + " .grid-canvas .slick-row", false);
                var draggableTop = !!this.draggableItem ? this.draggableItem.getEl().getTopPx() : 0;
                for (var key in children) {
                    var currentRowTop = children[key].getEl().getTopPx();
                    if (data.rows[0] < data.insertBefore) {
                        if (this.draggableTop < currentRowTop && currentRowTop - this.rowHeight / 2 <= draggableTop) {
                            children[key].getEl().setMarginTop("-" + this.rowHeight + "px");
                        }
                        else {
                            children[key].getEl().setMarginTop(null);
                        }
                    }
                    else if (data.rows[0] > data.insertBefore) {
                        if (this.draggableTop > currentRowTop && currentRowTop + this.rowHeight / 2 >= draggableTop) {
                            children[key].getEl().setMarginTop(this.rowHeight + "px");
                        }
                        else {
                            children[key].getEl().setMarginTop(null);
                        }
                    }
                }
                this.contentGrid.scrollToRow(data.insertBefore);
                return true;
            };
            ContentGridDragHandler.prototype.handleMoveRows = function (event, args) {
                var dataView = this.contentGrid.getGrid().getDataView();
                var draggableRow = args.rows[0];
                var rowDataId = dataView.getItem(draggableRow).getData().getContentId();
                var moveBeforeRowDataId = (dataView.getLength() <= args.insertBefore) ? null : dataView.getItem(args.insertBefore).getData().getContentId();
                var extractedRow, left, right;
                var insertBefore = args.insertBefore;
                left = dataView.slick().getItems().slice(0, insertBefore);
                right = dataView.slick().getItems().slice(insertBefore, dataView.slick().getItems().length);
                extractedRow = dataView.slick().getItem(draggableRow);
                if (draggableRow < insertBefore) {
                    left.splice(draggableRow, 1);
                }
                else {
                    right.splice(draggableRow - insertBefore, 1);
                }
                left.push(extractedRow);
                var data = left.concat(right);
                var selectedRow = left.length - 1;
                this.makeMovementInNodes(draggableRow, args.insertBefore, data);
                this.contentGrid.getGrid().setSelectedRows([selectedRow]);
                this.movements.addChildMovement(new OrderChildMovement(rowDataId, moveBeforeRowDataId));
                this.notifyPositionChanged();
            };
            ContentGridDragHandler.prototype.makeMovementInNodes = function (draggableRow, insertBefore, data) {
                var root = this.contentGrid.getRoot().getCurrentRoot();
                var rootChildren = root.getChildren();
                var item = rootChildren.slice(draggableRow, draggableRow + 1)[0];
                this.contentGrid.initData(data);
                rootChildren.splice(rootChildren.indexOf(item), 1);
                var insertPosition = (draggableRow > insertBefore) ? insertBefore : insertBefore - 1;
                rootChildren.splice(insertPosition, 0, item);
                root.setChildren(rootChildren);
            };
            ContentGridDragHandler.prototype.notifyPositionChanged = function () {
                var _this = this;
                this.positionChangedListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            ContentGridDragHandler.prototype.onPositionChanged = function (listener) {
                this.positionChangedListeners.push(listener);
            };
            ContentGridDragHandler.prototype.unPositionChanged = function (listener) {
                this.positionChangedListeners = this.positionChangedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            return ContentGridDragHandler;
        })();
        browse.ContentGridDragHandler = ContentGridDragHandler;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var remove;
    (function (remove) {
        var ContentDeleteDialog = (function (_super) {
            __extends(ContentDeleteDialog, _super);
            function ContentDeleteDialog() {
                var _this = this;
                _super.call(this, "Content");
                this.setDeleteAction(new remove.ContentDeleteDialogAction());
                this.getDeleteAction().onExecuted(function () {
                    var deleteRequest = new api.content.DeleteContentRequest();
                    for (var i = 0; i < _this.contentToDelete.length; i++) {
                        deleteRequest.addContentPath(_this.contentToDelete[i].getPath());
                    }
                    deleteRequest.sendAndParse().then(function (result) {
                        var paths = [], deletedContents = [];
                        for (var i = 0; i < result.getDeleted().length; i++) {
                            var path = result.getDeleted()[i];
                            paths.push(path);
                            _this.contentToDelete.forEach(function (content) {
                                if (content.getPath().equals(path)) {
                                    deletedContents.push(content);
                                }
                            });
                        }
                        _this.close();
                        if (result.getDeleted().length > 0) {
                            api.notify.showFeedback('Content [' + paths.map(function (path) { return path.toString(); }).join(', ') + '] deleted!');
                        }
                        else {
                            var reason = result.getDeleteFailures().length > 0 ? result.getDeleteFailures()[0].getReason() : '';
                            api.notify.showWarning('Content could not be deleted. ' + reason);
                        }
                    }).catch(function (reason) {
                        if (reason && reason.message) {
                            api.notify.showError(reason.message);
                        }
                        else {
                            api.notify.showError('Content could not be deleted.');
                        }
                    }).done();
                });
                this.addCancelButtonToBottom();
            }
            ContentDeleteDialog.prototype.setContentToDelete = function (contentModels) {
                this.contentToDelete = contentModels;
                var deleteViewers = [];
                for (var i in contentModels) {
                    var content = contentModels[i];
                    var deleteViewer = new api.content.ContentSummaryViewer();
                    deleteViewer.setObject(content);
                    deleteViewers.push(deleteViewer);
                }
                this.setDeleteViewers(deleteViewers);
            };
            return ContentDeleteDialog;
        })(api.app.remove.DeleteDialog);
        remove.ContentDeleteDialog = ContentDeleteDialog;
    })(remove = app.remove || (app.remove = {}));
})(app || (app = {}));
var app;
(function (app) {
    var remove;
    (function (remove) {
        var ContentDeleteDialogAction = (function (_super) {
            __extends(ContentDeleteDialogAction, _super);
            function ContentDeleteDialogAction() {
                _super.call(this, "Delete", "enter");
            }
            return ContentDeleteDialogAction;
        })(api.ui.Action);
        remove.ContentDeleteDialogAction = ContentDeleteDialogAction;
    })(remove = app.remove || (app.remove = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentEvent = (function (_super) {
            __extends(NewContentEvent, _super);
            function NewContentEvent(contentType, parentContent) {
                _super.call(this);
                this.contentType = contentType;
                this.parentContent = parentContent;
            }
            NewContentEvent.prototype.getContentType = function () {
                return this.contentType;
            };
            NewContentEvent.prototype.getParentContent = function () {
                return this.parentContent;
            };
            NewContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            NewContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return NewContentEvent;
        })(api.event.Event);
        create.NewContentEvent = NewContentEvent;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewMediaUploadEvent = (function (_super) {
            __extends(NewMediaUploadEvent, _super);
            function NewMediaUploadEvent(items, parentContent) {
                _super.call(this);
                this.uploadItems = items;
                this.parentContent = parentContent;
            }
            NewMediaUploadEvent.prototype.getUploadItems = function () {
                return this.uploadItems;
            };
            NewMediaUploadEvent.prototype.getParentContent = function () {
                return this.parentContent;
            };
            NewMediaUploadEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            NewMediaUploadEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return NewMediaUploadEvent;
        })(api.event.Event);
        create.NewMediaUploadEvent = NewMediaUploadEvent;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var GetAllContentTypesRequest = api.schema.content.GetAllContentTypesRequest;
        var GetNearestSiteRequest = api.content.GetNearestSiteRequest;
        var ContentPath = api.content.ContentPath;
        var ContentTypeName = api.schema.content.ContentTypeName;
        var NewContentDialog = (function (_super) {
            __extends(NewContentDialog, _super);
            function NewContentDialog() {
                var _this = this;
                this.contentDialogTitle = new NewContentDialogTitle("Create Content", "");
                _super.call(this, {
                    title: this.contentDialogTitle
                });
                this.addClass("new-content-dialog");
                var section = new api.dom.SectionEl().setClass("column");
                this.appendChildToContentPanel(section);
                this.fileInput = new api.ui.text.FileInput('large').setPlaceholder("Search for content types").setUploaderParams({
                    parent: ContentPath.ROOT.toString()
                });
                this.fileInput.onUploadStarted(function (event) {
                    _this.closeAndFireEventFromMediaUpload(event.getUploadItems());
                });
                this.contentList = new app.create.NewContentDialogList();
                section.appendChildren(this.fileInput, this.contentList);
                var aside = new api.dom.AsideEl("column");
                this.appendChildToContentPanel(aside);
                this.initMediaUploader();
                var recentTitle = new api.dom.H1El();
                recentTitle.setHtml('Recently Used');
                this.recentList = new create.RecentItemsList();
                aside.appendChildren(recentTitle, this.recentList);
                api.dom.Body.get().appendChild(this);
                this.contentListMask = new api.ui.mask.LoadMask(this.contentList);
                this.recentListMask = new api.ui.mask.LoadMask(this.recentList);
                this.contentList.appendChild(this.contentListMask);
                this.recentList.appendChild(this.recentListMask);
                this.listItems = [];
                this.fileInput.onInput(function (event) {
                    _this.filterList();
                });
                this.fileInput.onKeyUp(function (event) {
                    if (event.keyCode === 27) {
                        _this.getCancelAction().execute();
                    }
                });
                this.contentList.onSelected(function (event) {
                    _this.closeAndFireEventFromContentType(event.getItem());
                });
                this.recentList.onSelected(function (event) {
                    _this.closeAndFireEventFromContentType(event.getItem());
                });
            }
            NewContentDialog.prototype.initMediaUploader = function () {
                var _this = this;
                var uploaderContainer = new api.dom.DivEl('uploader-container');
                this.appendChild(uploaderContainer);
                var uploaderMask = new api.dom.DivEl('uploader-mask');
                uploaderContainer.appendChild(uploaderMask);
                this.mediaUploader = new api.content.MediaUploader({
                    operation: 0 /* create */,
                    params: {
                        parent: ContentPath.ROOT.toString()
                    },
                    name: 'new-content-uploader',
                    showButtons: false,
                    showResult: false,
                    allowMultiSelection: true,
                    deferred: true
                });
                uploaderContainer.appendChild(this.mediaUploader);
                this.mediaUploader.onUploadStarted(function (event) {
                    _this.closeAndFireEventFromMediaUpload(event.getUploadItems());
                });
                var dragOverEl;
                this.onDragEnter(function (event) {
                    var target = event.target;
                    if (!!dragOverEl || dragOverEl == _this.getHTMLElement()) {
                        uploaderContainer.show();
                    }
                    dragOverEl = target;
                });
                this.onDragLeave(function (event) {
                    var targetEl = event.target;
                    if (dragOverEl == targetEl) {
                        uploaderContainer.hide();
                    }
                });
                this.onDrop(function (event) {
                    uploaderContainer.hide();
                });
            };
            NewContentDialog.prototype.closeAndFireEventFromMediaUpload = function (items) {
                this.close();
                new create.NewMediaUploadEvent(items, this.parentContent).fire();
            };
            NewContentDialog.prototype.closeAndFireEventFromContentType = function (item) {
                this.close();
                new create.NewContentEvent(item.getContentType(), this.parentContent).fire();
            };
            NewContentDialog.prototype.filterList = function () {
                var inputValue = this.fileInput.getValue();
                var filteredItems = this.listItems.filter(function (item) {
                    return (!inputValue || (item.getDisplayName().indexOf(inputValue) != -1) || (item.getName().indexOf(inputValue) != -1));
                });
                this.contentList.setItems(filteredItems);
            };
            NewContentDialog.prototype.filterByParentContent = function (items, siteModuleKeys) {
                var typesAllowedEverywhere = {};
                [ContentTypeName.UNSTRUCTURED, ContentTypeName.FOLDER, ContentTypeName.SITE, ContentTypeName.SHORTCUT].forEach(function (contentTypeName) {
                    typesAllowedEverywhere[contentTypeName.toString()] = contentTypeName;
                });
                var siteModules = {};
                siteModuleKeys.forEach(function (moduleKey) {
                    siteModules[moduleKey.toString()] = moduleKey;
                });
                var parentContentIsTemplateFolder = this.parentContent && this.parentContent.getType().isTemplateFolder();
                var parentContentIsSite = this.parentContent && this.parentContent.getType().isSite();
                var parentContentIsPageTemplate = this.parentContent && this.parentContent.getType().isPageTemplate();
                return items.filter(function (item) {
                    var contentType = item.getContentType();
                    var contentTypeName = contentType.getContentTypeName();
                    if (contentType.isAbstract()) {
                        return false;
                    }
                    else if (parentContentIsPageTemplate) {
                        return false;
                    }
                    else if (contentTypeName.isTemplateFolder()) {
                        return parentContentIsSite;
                    }
                    else if (contentTypeName.isPageTemplate()) {
                        return parentContentIsTemplateFolder;
                    }
                    else if (parentContentIsTemplateFolder) {
                        return contentTypeName.isPageTemplate();
                    }
                    else if (typesAllowedEverywhere[contentTypeName.toString()]) {
                        return true;
                    }
                    else if (siteModules[contentTypeName.getModuleKey().toString()]) {
                        return true;
                    }
                    else {
                        return false;
                    }
                });
            };
            NewContentDialog.prototype.setParentContent = function (parent) {
                this.parentContent = parent;
                var params = {
                    parent: parent ? parent.getPath().toString() : api.content.ContentPath.ROOT.toString()
                };
                this.mediaUploader.setParams(params);
                this.fileInput.setUploaderParams(params);
            };
            NewContentDialog.prototype.open = function () {
                _super.prototype.open.call(this);
                var keyBindings = [
                    new api.ui.KeyBinding('up', function () {
                        api.dom.FormEl.moveFocusToPrevFocusable(api.dom.Element.fromHtmlElement(document.activeElement), "input,li");
                    }).setGlobal(true),
                    new api.ui.KeyBinding('down', function () {
                        api.dom.FormEl.moveFocusToNextFocusable(api.dom.Element.fromHtmlElement(document.activeElement), "input,li");
                    }).setGlobal(true)
                ];
                api.ui.KeyBindings.get().bindKeys(keyBindings);
            };
            NewContentDialog.prototype.show = function () {
                if (this.parentContent) {
                    this.contentDialogTitle.setPath(this.parentContent.getPath().toString());
                }
                else {
                    this.contentDialogTitle.setPath('');
                }
                _super.prototype.show.call(this);
                this.fileInput.reset().giveFocus();
                this.mediaUploader.reset();
                this.loadContentTypes();
            };
            NewContentDialog.prototype.hide = function () {
                _super.prototype.hide.call(this);
                this.mediaUploader.stop();
            };
            NewContentDialog.prototype.close = function () {
                this.fileInput.reset();
                _super.prototype.close.call(this);
            };
            NewContentDialog.prototype.loadContentTypes = function () {
                var _this = this;
                this.contentListMask.show();
                this.recentListMask.show();
                var requests = [];
                requests.push(new GetAllContentTypesRequest().sendAndParse());
                if (this.parentContent) {
                    requests.push(new GetNearestSiteRequest(this.parentContent.getContentId()).sendAndParse());
                }
                wemQ.all(requests).spread(function (contentTypes, parentSite) {
                    var listItems = _this.createListItems(contentTypes);
                    var siteModules = parentSite ? parentSite.getModuleKeys() : [];
                    _this.listItems = _this.filterByParentContent(listItems, siteModules);
                    if (_this.listItems.length > 0) {
                        _this.contentList.setItems(_this.listItems);
                        _this.recentList.setItems(_this.listItems);
                    }
                    else {
                        _this.contentList.clearItems();
                        _this.recentList.clearItems();
                    }
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).finally(function () {
                    _this.filterList();
                    _this.contentListMask.hide();
                    _this.recentListMask.hide();
                    api.ui.responsive.ResponsiveManager.fireResizeEvent();
                }).done();
            };
            NewContentDialog.prototype.createListItems = function (contentTypes) {
                var contentTypesByName = {};
                var items = [];
                contentTypes.forEach(function (contentType) {
                    var contentTypeName = contentType.getContentTypeName();
                    if (!contentTypeName.isMedia() && !contentTypeName.isDescendantOfMedia()) {
                        contentTypesByName[contentType.getName()] = contentType;
                        items.push(create.NewContentDialogListItem.fromContentType(contentType));
                    }
                });
                items.sort(this.compareListItems);
                return items;
            };
            NewContentDialog.prototype.compareListItems = function (item1, item2) {
                if (item1.getDisplayName().toLowerCase() > item2.getDisplayName().toLowerCase()) {
                    return 1;
                }
                else if (item1.getDisplayName().toLowerCase() < item2.getDisplayName().toLowerCase()) {
                    return -1;
                }
                else if (item1.getName() > item2.getName()) {
                    return 1;
                }
                else if (item1.getName() < item2.getName()) {
                    return -1;
                }
                else {
                    return 0;
                }
            };
            return NewContentDialog;
        })(api.ui.dialog.ModalDialog);
        create.NewContentDialog = NewContentDialog;
        var NewContentDialogTitle = (function (_super) {
            __extends(NewContentDialogTitle, _super);
            function NewContentDialogTitle(title, path) {
                _super.call(this, title);
                this.pathEl = new api.dom.PEl('path');
                this.pathEl.setHtml(path);
                this.appendChild(this.pathEl);
            }
            NewContentDialogTitle.prototype.setPath = function (path) {
                this.pathEl.setHtml(path).setVisible(!api.util.StringHelper.isBlank(path));
            };
            return NewContentDialogTitle;
        })(api.ui.dialog.ModalDialogHeader);
        create.NewContentDialogTitle = NewContentDialogTitle;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentDialogList = (function (_super) {
            __extends(NewContentDialogList, _super);
            function NewContentDialogList() {
                _super.call(this, 'content-types-list');
                this.selectedListeners = [];
            }
            NewContentDialogList.prototype.onSelected = function (listener) {
                this.selectedListeners.push(listener);
            };
            NewContentDialogList.prototype.unSelected = function (listener) {
                this.selectedListeners = this.selectedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            NewContentDialogList.prototype.notifySelected = function (listItem) {
                var _this = this;
                this.selectedListeners.forEach(function (listener) {
                    listener.call(_this, new create.NewContentDialogItemSelectedEvent(listItem));
                });
            };
            NewContentDialogList.prototype.createItemView = function (item) {
                var _this = this;
                var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                namesAndIconView.setIconUrl(item.getIconUrl()).setMainName(item.getDisplayName()).setSubName(item.getName()).setDisplayIconLabel(item.isSite());
                var itemEl = new api.dom.LiEl('content-types-list-item' + (item.isSite() ? ' site' : ''));
                itemEl.getEl().setTabIndex(0);
                itemEl.appendChild(namesAndIconView);
                itemEl.onClicked(function (event) { return _this.notifySelected(item); });
                itemEl.onKeyPressed(function (event) {
                    if (event.keyCode == 13) {
                        _this.notifySelected(item);
                    }
                });
                return itemEl;
            };
            NewContentDialogList.prototype.getItemId = function (item) {
                return item.getName();
            };
            return NewContentDialogList;
        })(api.ui.selector.list.ListBox);
        create.NewContentDialogList = NewContentDialogList;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentDialogListItem = (function () {
            function NewContentDialogListItem(contentType) {
                this.contentType = contentType;
                this.name = contentType.getName();
                this.displayName = contentType.getDisplayName();
                this.iconUrl = contentType.getIconUrl();
            }
            NewContentDialogListItem.fromContentType = function (contentType) {
                return new NewContentDialogListItem(contentType);
            };
            NewContentDialogListItem.prototype.getContentType = function () {
                return this.contentType;
            };
            NewContentDialogListItem.prototype.isSite = function () {
                return this.contentType.isSite();
            };
            NewContentDialogListItem.prototype.getName = function () {
                return this.name;
            };
            NewContentDialogListItem.prototype.getDisplayName = function () {
                return this.displayName;
            };
            NewContentDialogListItem.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            return NewContentDialogListItem;
        })();
        create.NewContentDialogListItem = NewContentDialogListItem;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentDialogItemSelectedEvent = (function () {
            function NewContentDialogItemSelectedEvent(item) {
                this.item = item;
            }
            NewContentDialogItemSelectedEvent.prototype.getItem = function () {
                return this.item;
            };
            return NewContentDialogItemSelectedEvent;
        })();
        create.NewContentDialogItemSelectedEvent = NewContentDialogItemSelectedEvent;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        create.NewContentEvent.on(function (event) {
            RecentItems.get().addItemName(event.getContentType());
        });
        var RecentItems = (function () {
            function RecentItems() {
                this.maximum = 7;
                this.cookieKey = 'app.browse.RecentItemsList';
                this.cookieExpire = 30;
                this.valueSeparator = '|';
            }
            RecentItems.get = function () {
                return RecentItems.INSTANCE;
            };
            RecentItems.prototype.addItemName = function (contentType) {
                var itemsNames = this.getRecentItemsNames();
                var name = contentType.getName();
                itemsNames = itemsNames.filter(function (storedName) { return storedName != name; });
                itemsNames.unshift(name);
                itemsNames = itemsNames.slice(0, this.maximum);
                api.util.CookieHelper.setCookie(this.cookieKey, itemsNames.join(this.valueSeparator), this.cookieExpire);
            };
            RecentItems.prototype.getRecentItemsNames = function () {
                var cookies = api.util.CookieHelper.getCookie(this.cookieKey);
                return cookies ? cookies.split(this.valueSeparator) : [];
            };
            RecentItems.INSTANCE = new RecentItems();
            return RecentItems;
        })();
        create.RecentItems = RecentItems;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var RecentItemsList = (function (_super) {
            __extends(RecentItemsList, _super);
            function RecentItemsList() {
                _super.call(this);
            }
            RecentItemsList.prototype.setItems = function (items) {
                var itemsByName = {};
                items.forEach(function (item) {
                    itemsByName[item.getName()] = item;
                });
                var recentItemsNames = create.RecentItems.get().getRecentItemsNames();
                var recentItems = [];
                recentItemsNames.forEach(function (name) {
                    if (itemsByName[name]) {
                        recentItems.push(itemsByName[name]);
                    }
                });
                _super.prototype.setItems.call(this, recentItems);
            };
            return RecentItemsList;
        })(create.NewContentDialogList);
        create.RecentItemsList = RecentItemsList;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var CloseAction = (function (_super) {
            __extends(CloseAction, _super);
            function CloseAction(itemViewPanel, checkCanRemovePanel) {
                if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                _super.call(this, "Close", "mod+alt+f4");
                this.onExecuted(function () {
                    itemViewPanel.close(checkCanRemovePanel);
                });
            }
            return CloseAction;
        })(api.ui.Action);
        view.CloseAction = CloseAction;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var DeleteAction = (function (_super) {
            __extends(DeleteAction, _super);
            function DeleteAction(itemViewPanel) {
                _super.call(this, "Delete", "mod+del");
                this.onExecuted(function () {
                    var contentToDelete = itemViewPanel.getItem().getModel();
                    api.ui.dialog.ConfirmationDialog.get().setQuestion("Are you sure you want to delete this content?").setNoCallback(null).setYesCallback(function () {
                        itemViewPanel.close();
                        new api.content.DeleteContentRequest().addContentPath(contentToDelete.getPath()).sendAndParse().then(function (result) {
                            if (result.getDeleted().length > 0) {
                                var path = result.getDeleted()[0].toString();
                                api.notify.showFeedback('Content [' + path + '] deleted!');
                            }
                            else {
                                var reason = result.getDeleteFailures().length > 0 ? result.getDeleteFailures()[0].getReason() : '';
                                api.notify.showWarning('Content could not be deleted. ' + reason);
                            }
                        }).catch(function (reason) {
                            if (reason && reason.message) {
                                api.notify.showError(reason.message);
                            }
                            else {
                                api.notify.showError('Content could not be deleted.');
                            }
                        }).done();
                    }).open();
                });
            }
            return DeleteAction;
        })(api.ui.Action);
        view.DeleteAction = DeleteAction;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var EditAction = (function (_super) {
            __extends(EditAction, _super);
            function EditAction(panel) {
                _super.call(this, "Edit");
                this.onExecuted(function () {
                    new app.browse.EditContentEvent([panel.getItem().getModel()]).fire();
                });
            }
            return EditAction;
        })(api.ui.Action);
        view.EditAction = EditAction;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemViewToolbar = (function (_super) {
            __extends(ContentItemViewToolbar, _super);
            function ContentItemViewToolbar(params) {
                _super.call(this);
                _super.prototype.addAction.call(this, params.editAction);
                _super.prototype.addAction.call(this, params.deleteAction);
            }
            return ContentItemViewToolbar;
        })(api.ui.toolbar.Toolbar);
        view.ContentItemViewToolbar = ContentItemViewToolbar;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var RenderingMode = api.rendering.RenderingMode;
        var ContentImageUrlResolver = api.content.ContentImageUrlResolver;
        var ContentItemPreviewPanel = (function (_super) {
            __extends(ContentItemPreviewPanel, _super);
            function ContentItemPreviewPanel() {
                var _this = this;
                _super.call(this, "content-item-preview-panel");
                this.image = new api.dom.ImgEl();
                this.image.onLoaded(function (event) {
                    _this.mask.hide();
                    var imgEl = _this.image.getEl();
                    var myEl = _this.getEl();
                    _this.centerImage(imgEl.getWidth(), imgEl.getHeight(), myEl.getWidth(), myEl.getHeight());
                });
                this.appendChild(this.image);
                api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (_this.hasClass("image-preview")) {
                        var imgEl = _this.image.getEl(), el = _this.getEl();
                        _this.centerImage(imgEl.getWidth(), imgEl.getHeight(), el.getWidth(), el.getHeight());
                    }
                });
                this.onShown(function (event) {
                    if (_this.item && _this.hasClass("image-preview")) {
                        _this.addImageSizeToUrl(_this.item);
                    }
                });
            }
            ContentItemPreviewPanel.prototype.centerImage = function (imgWidth, imgHeight, myWidth, myHeight) {
                var imgMarginTop = 0;
                if (imgHeight < myHeight) {
                    imgMarginTop = (myHeight - imgHeight) / 2;
                }
                this.image.getEl().setMarginTop(imgMarginTop + "px");
            };
            ContentItemPreviewPanel.prototype.addImageSizeToUrl = function (item) {
                var imgSize = Math.max(this.getEl().getWidth(), this.getEl().getHeight());
                var imgUrl = new ContentImageUrlResolver().setContentId(item.getModel().getContentId()).setSize(imgSize).resolve();
                this.image.setSrc(imgUrl);
            };
            ContentItemPreviewPanel.prototype.setItem = function (item) {
                if (!this.item || !item || !this.item.equals(item)) {
                    this.item = item;
                    if (item.getModel().getType().isImage()) {
                        this.getEl().removeClass("no-preview page-preview").addClass("image-preview");
                        if (this.isVisible()) {
                            this.addImageSizeToUrl(item);
                        }
                        if (!this.image.isLoaded()) {
                            this.mask.show();
                        }
                    }
                    else {
                        this.mask.show();
                        if (item.isRenderable()) {
                            this.getEl().removeClass("image-preview no-preview").addClass('page-preview');
                            this.frame.setSrc(api.rendering.UriHelper.getPortalUri(item.getPath(), 1 /* PREVIEW */, 0 /* DRAFT */));
                        }
                        else {
                            this.getEl().removeClass("image-preview page-preview").addClass('no-preview');
                            this.frame.setSrc("about:blank");
                            this.mask.hide();
                        }
                    }
                }
            };
            ContentItemPreviewPanel.prototype.getItem = function () {
                return this.item;
            };
            return ContentItemPreviewPanel;
        })(api.app.view.ItemPreviewPanel);
        view.ContentItemPreviewPanel = ContentItemPreviewPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentVersionViewer = api.content.ContentVersionViewer;
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var ContentVersionsTreeGrid = (function (_super) {
            __extends(ContentVersionsTreeGrid, _super);
            function ContentVersionsTreeGrid() {
                var descColumn = new GridColumnBuilder().setName("DisplayName").setField("displayName").setCssClass("description").setMinWidth(240).setFormatter(this.descriptionFormatter).build();
                var statusColumn = new GridColumnBuilder().setName("Status").setField("workspaces").setCssClass("status").setFormatter(this.statusFormatter).setMinWidth(80).build();
                _super.call(this, new TreeGridBuilder().setAutoLoad(false).setCheckableRows(false).setHotkeysEnabled(false).setSelectedCellCssClass("").setShowToolbar(false).setColumns([
                    descColumn,
                    statusColumn
                ]).prependClasses("content-versions-tree-grid"));
            }
            ContentVersionsTreeGrid.prototype.setContentId = function (contentId) {
                this.contentId = contentId;
                this.reload();
            };
            ContentVersionsTreeGrid.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentVersionsTreeGrid.prototype.getDataId = function (data) {
                return data.id;
            };
            ContentVersionsTreeGrid.prototype.descriptionFormatter = function (row, cell, value, columnDef, node) {
                var description = "";
                if (node.getData()) {
                    var viewer = node.getViewer("description");
                    if (!viewer) {
                        viewer = new ContentVersionViewer();
                        var length = node.getRoot().getChildren().length;
                        viewer.setObject(node.getData(), length - row);
                        node.setViewer("description", viewer);
                    }
                    description = viewer.toString();
                }
                return description;
            };
            ContentVersionsTreeGrid.prototype.statusFormatter = function (row, cell, value, columnDef, node) {
                var badges = [];
                value.forEach(function (workspace) {
                    badges.push(new api.dom.SpanEl('badge ' + workspace).setHtml(workspace).toString());
                });
                return badges.join("");
            };
            return ContentVersionsTreeGrid;
        })(api.ui.treegrid.TreeGrid);
        view.ContentVersionsTreeGrid = ContentVersionsTreeGrid;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var AllContentVersionsTreeGrid = (function (_super) {
            __extends(AllContentVersionsTreeGrid, _super);
            function AllContentVersionsTreeGrid() {
                _super.call(this);
            }
            AllContentVersionsTreeGrid.prototype.fetchChildren = function (parentNode) {
                var _this = this;
                if (this.contentId) {
                    var activePromise = new api.content.GetActiveContentVersionsRequest(this.contentId);
                    var allPromise = new api.content.GetContentVersionsRequest(this.contentId);
                    return wemQ.all([allPromise.sendAndParse(), activePromise.sendAndParse()]).spread(function (allVersions, activeVersions) { return _this.enrichWithWorkspaces(allVersions, activeVersions); });
                }
                else {
                    throw new Error("Required contentId not set for ActiveContentVersionsTreeGrid");
                }
            };
            AllContentVersionsTreeGrid.prototype.enrichWithWorkspaces = function (allVersions, activeVersions) {
                activeVersions.forEach(function (activeVersion) {
                    for (var i = 0; i < allVersions.length; i++) {
                        var allVersion = allVersions[i];
                        if (activeVersion.id == allVersion.id) {
                            allVersion.workspaces = activeVersion.workspaces;
                            break;
                        }
                    }
                });
                return allVersions;
            };
            return AllContentVersionsTreeGrid;
        })(view.ContentVersionsTreeGrid);
        view.AllContentVersionsTreeGrid = AllContentVersionsTreeGrid;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ActiveContentVersionsTreeGrid = (function (_super) {
            __extends(ActiveContentVersionsTreeGrid, _super);
            function ActiveContentVersionsTreeGrid() {
                _super.call(this);
            }
            ActiveContentVersionsTreeGrid.prototype.fetchChildren = function (parentNode) {
                if (this.contentId) {
                    return new api.content.GetActiveContentVersionsRequest(this.contentId).sendAndParse();
                }
                else {
                    throw new Error("Required contentId not set for ActiveContentVersionsTreeGrid");
                }
            };
            return ActiveContentVersionsTreeGrid;
        })(view.ContentVersionsTreeGrid);
        view.ActiveContentVersionsTreeGrid = ActiveContentVersionsTreeGrid;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemVersionsPanel = (function (_super) {
            __extends(ContentItemVersionsPanel, _super);
            function ContentItemVersionsPanel() {
                var _this = this;
                _super.call(this, "content-item-versions-panel");
                var navigator = new api.ui.tab.TabBar();
                this.deckPanel = new api.ui.panel.NavigatedDeckPanel(navigator);
                this.appendChild(navigator);
                this.appendChild(this.deckPanel);
                this.mask = new api.ui.mask.LoadMask(this);
                this.appendChild(this.mask);
                navigator.onNavigationItemSelected(function (event) {
                    _this.setItem(_this.item);
                });
                this.allGrid = new view.AllContentVersionsTreeGrid();
                this.allGrid.onLoaded(function () {
                    _this.mask.hide();
                });
                this.deckPanel.addNavigablePanel(new api.ui.tab.TabBarItemBuilder().setLabel('All Versions').build(), this.allGrid, true);
                this.activeGrid = new view.ActiveContentVersionsTreeGrid();
                this.activeGrid.onLoaded(function () {
                    _this.mask.hide();
                });
                this.deckPanel.addNavigablePanel(new api.ui.tab.TabBarItemBuilder().setLabel('Active Versions').build(), this.activeGrid);
            }
            ContentItemVersionsPanel.prototype.setItem = function (item) {
                this.item = item;
                if (this.item) {
                    var panel = this.deckPanel.getPanelShown();
                    if (panel.getContentId() != this.item.getModel().getContentId()) {
                        this.mask.show();
                        this.deckPanel.getPanelShown().setContentId(item.getModel().getContentId());
                    }
                }
            };
            ContentItemVersionsPanel.prototype.getItem = function () {
                return this.item;
            };
            return ContentItemVersionsPanel;
        })(api.ui.panel.Panel);
        view.ContentItemVersionsPanel = ContentItemVersionsPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
        var ContentItemStatisticsPanel = (function (_super) {
            __extends(ContentItemStatisticsPanel, _super);
            function ContentItemStatisticsPanel() {
                var _this = this;
                _super.call(this, "content-item-statistics-panel");
                this.previewPanel = new view.ContentItemPreviewPanel();
                this.addNavigablePanel(new TabMenuItemBuilder().setLabel("Preview").build(), this.previewPanel, true);
                this.versionsPanel = new view.ContentItemVersionsPanel();
                this.addNavigablePanel(new TabMenuItemBuilder().setLabel("Version History").build(), this.versionsPanel);
                this.getTabMenu().onNavigationItemSelected(function (event) {
                    _this.onTabSelected(event.getItem());
                });
            }
            ContentItemStatisticsPanel.prototype.setItem = function (item) {
                if (this.getItem() != item) {
                    _super.prototype.setItem.call(this, item);
                    switch (this.getTabMenu().getSelectedIndex()) {
                        case 0:
                            this.previewPanel.setItem(item);
                            break;
                        case 1:
                            this.versionsPanel.setItem(item);
                            break;
                    }
                }
            };
            ContentItemStatisticsPanel.prototype.onTabSelected = function (navigationItem) {
                var item = this.getItem();
                switch (navigationItem.getIndex()) {
                    case 0:
                        this.getHeader().hide();
                        if (this.previewPanel.getItem() != item) {
                            this.previewPanel.setItem(item);
                        }
                        break;
                    case 1:
                        this.getHeader().hide();
                        if (this.versionsPanel.getItem() != item) {
                            this.versionsPanel.setItem(item);
                        }
                        break;
                    default:
                        this.getHeader().show();
                }
            };
            return ContentItemStatisticsPanel;
        })(api.app.view.MultiItemStatisticsPanel);
        view.ContentItemStatisticsPanel = ContentItemStatisticsPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemViewPanel = (function (_super) {
            __extends(ContentItemViewPanel, _super);
            function ContentItemViewPanel() {
                var _this = this;
                this.deckPanel = new api.ui.panel.DeckPanel();
                this.editAction = new view.EditAction(this);
                this.deleteAction = new view.DeleteAction(this);
                this.closeAction = new view.CloseAction(this, true);
                this.actions = [this.editAction, this.deleteAction, this.closeAction];
                var toolbar = new view.ContentItemViewToolbar({
                    editAction: this.editAction,
                    deleteAction: this.deleteAction
                });
                _super.call(this, toolbar, this.deckPanel);
                this.statisticsPanel = new view.ContentItemStatisticsPanel();
                this.previewPanel = new view.ContentItemPreviewPanel();
                this.statisticsPanelIndex = this.deckPanel.addPanel(this.statisticsPanel);
                this.previewPanelIndex = this.deckPanel.addPanel(this.previewPanel);
                this.showPreview(false);
                app.browse.ShowPreviewEvent.on(function (event) {
                    _this.showPreview(true);
                });
                app.browse.ShowDetailsEvent.on(function (event) {
                    _this.showPreview(false);
                });
                this.onShown(function (event) {
                    if (_this.getItem()) {
                        app.Router.setHash("view/" + _this.getItem().getModel().getId());
                    }
                });
            }
            ContentItemViewPanel.prototype.setItem = function (item) {
                _super.prototype.setItem.call(this, item);
                this.statisticsPanel.setItem(item);
                this.previewPanel.setItem(item);
            };
            ContentItemViewPanel.prototype.showPreview = function (enabled) {
                this.previewMode = enabled;
                if (enabled) {
                    this.deckPanel.showPanelByIndex(this.previewPanelIndex);
                }
                else {
                    this.deckPanel.showPanelByIndex(this.statisticsPanelIndex);
                }
            };
            ContentItemViewPanel.prototype.getCloseAction = function () {
                return this.closeAction;
            };
            ContentItemViewPanel.prototype.getActions = function () {
                return this.actions;
            };
            return ContentItemViewPanel;
        })(api.app.view.ItemViewPanel);
        view.ContentItemViewPanel = ContentItemViewPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var action;
        (function (action) {
            var DeleteContentAction = (function (_super) {
                __extends(DeleteContentAction, _super);
                function DeleteContentAction(wizardPanel) {
                    _super.call(this, "Delete", "mod+del", true);
                    this.onExecuted(function () {
                        api.ui.dialog.ConfirmationDialog.get().setQuestion("Are you sure you want to delete this content?").setNoCallback(null).setYesCallback(function () {
                            wizardPanel.close();
                            new api.content.DeleteContentRequest().addContentPath(wizardPanel.getPersistedItem().getPath()).sendAndParse().then(function (result) {
                                if (result.getDeleted().length > 0) {
                                    var path = result.getDeleted()[0].toString();
                                    api.notify.showFeedback('Content [' + path + '] deleted!');
                                }
                                else {
                                    var reason = result.getDeleteFailures().length > 0 ? result.getDeleteFailures()[0].getReason() : '';
                                    api.notify.showWarning('Content could not be deleted. ' + reason);
                                }
                            }).catch(function (reason) {
                                if (reason && reason.message) {
                                    api.notify.showError(reason.message);
                                }
                                else {
                                    api.notify.showError('Content could not be deleted.');
                                }
                            }).done();
                        }).open();
                    });
                }
                return DeleteContentAction;
            })(api.ui.Action);
            action.DeleteContentAction = DeleteContentAction;
        })(action = wizard.action || (wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var action;
        (function (action) {
            var DuplicateContentAction = (function (_super) {
                __extends(DuplicateContentAction, _super);
                function DuplicateContentAction(wizardPanel) {
                    _super.call(this, "Duplicate");
                    this.onExecuted(function () {
                        var source = wizardPanel.getPersistedItem();
                        new api.content.DuplicateContentRequest(source.getContentId()).sendAndParse().then(function (content) {
                            api.notify.showFeedback('Content [' + source.getPath() + '] was duplicated!');
                        });
                    });
                }
                return DuplicateContentAction;
            })(api.ui.Action);
            action.DuplicateContentAction = DuplicateContentAction;
        })(action = wizard.action || (wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ContentId = api.content.ContentId;
            var PublishContentRequest = api.content.PublishContentRequest;
            var PublishAction = (function (_super) {
                __extends(PublishAction, _super);
                function PublishAction(wizard) {
                    var _this = this;
                    _super.call(this, "Publish");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        if (wizard.checkContentCanBePublished()) {
                            wizard.setRequireValid(true);
                            _this.setEnabled(false);
                            wizard.updatePersistedItem().then(function (content) {
                                if (content) {
                                    new PublishContentRequest(new ContentId(content.getId())).send().done(PublishContentRequest.feedback);
                                }
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).finally(function () { return _this.setEnabled(true); }).done();
                        }
                        else {
                            api.notify.showWarning('The content cannot be published yet. One or more form values are not valid.');
                        }
                    });
                }
                return PublishAction;
            })(api.ui.Action);
            action.PublishAction = PublishAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var RenderingMode = api.rendering.RenderingMode;
            var PreviewAction = (function (_super) {
                __extends(PreviewAction, _super);
                function PreviewAction(wizard) {
                    var _this = this;
                    _super.call(this, "Preview");
                    this.onExecuted(function () {
                        if (wizard.hasUnsavedChanges()) {
                            wizard.setRequireValid(true);
                            wizard.updatePersistedItem().then(_this.showPreviewDialog).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                        }
                        else {
                            _this.showPreviewDialog(wizard.getPersistedItem());
                        }
                    });
                }
                PreviewAction.prototype.showPreviewDialog = function (content) {
                    window.open(api.rendering.UriHelper.getPortalUri(content.getPath().toString(), 1 /* PREVIEW */, 0 /* DRAFT */), 'preview');
                };
                return PreviewAction;
            })(api.ui.Action);
            action.PreviewAction = PreviewAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ShowFormAction = (function (_super) {
                __extends(ShowFormAction, _super);
                function ShowFormAction(wizard) {
                    _super.call(this, "Form");
                    this.setEnabled(true);
                    this.onExecuted(function () {
                        new _wizard.ShowContentFormEvent().fire();
                        wizard.showForm();
                    });
                }
                return ShowFormAction;
            })(api.ui.Action);
            action.ShowFormAction = ShowFormAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ShowLiveEditAction = (function (_super) {
                __extends(ShowLiveEditAction, _super);
                function ShowLiveEditAction(wizard) {
                    _super.call(this, "Live");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        new _wizard.ShowLiveEditEvent().fire();
                        wizard.showLiveEdit();
                    });
                }
                return ShowLiveEditAction;
            })(api.ui.Action);
            action.ShowLiveEditAction = ShowLiveEditAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ShowSplitEditAction = (function (_super) {
                __extends(ShowSplitEditAction, _super);
                function ShowSplitEditAction(wizard) {
                    _super.call(this, "Split");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        new _wizard.ShowSplitEditEvent().fire();
                        wizard.showSplitEdit();
                    });
                }
                return ShowSplitEditAction;
            })(api.ui.Action);
            action.ShowSplitEditAction = ShowSplitEditAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var action;
        (function (action) {
            var ContentWizardActions = (function () {
                function ContentWizardActions(wizardPanel) {
                    this.save = new api.app.wizard.SaveAction(wizardPanel);
                    this.duplicate = new action.DuplicateContentAction(wizardPanel);
                    this.delete = new action.DeleteContentAction(wizardPanel);
                    this.close = new api.app.wizard.CloseAction(wizardPanel);
                    this.saveAndClose = new api.app.wizard.SaveAndCloseAction(wizardPanel);
                    this.publish = new action.PublishAction(wizardPanel);
                    this.preview = new action.PreviewAction(wizardPanel);
                    this.showLiveEditAction = new action.ShowLiveEditAction(wizardPanel);
                    this.showFormAction = new action.ShowFormAction(wizardPanel);
                    this.showSplitEditAction = new action.ShowSplitEditAction(wizardPanel);
                    this.actions = [
                        this.save,
                        this.duplicate,
                        this.delete,
                        this.close,
                        this.publish,
                        this.preview,
                        this.showLiveEditAction,
                        this.showFormAction,
                        this.showSplitEditAction,
                        this.saveAndClose
                    ];
                }
                ContentWizardActions.prototype.enableActionsForNew = function () {
                    this.save.setEnabled(true);
                    this.duplicate.setEnabled(false);
                    this.delete.setEnabled(false);
                    this.publish.setEnabled(true);
                };
                ContentWizardActions.prototype.enableActionsForExisting = function (existing) {
                    this.save.setEnabled(existing.isEditable());
                    this.duplicate.setEnabled(true);
                    this.delete.setEnabled(existing.isDeletable());
                    this.publish.setEnabled(true);
                };
                ContentWizardActions.prototype.getDeleteAction = function () {
                    return this.delete;
                };
                ContentWizardActions.prototype.getSaveAction = function () {
                    return this.save;
                };
                ContentWizardActions.prototype.getDuplicateAction = function () {
                    return this.duplicate;
                };
                ContentWizardActions.prototype.getCloseAction = function () {
                    return this.close;
                };
                ContentWizardActions.prototype.getPublishAction = function () {
                    return this.publish;
                };
                ContentWizardActions.prototype.getPreviewAction = function () {
                    return this.preview;
                };
                ContentWizardActions.prototype.getShowLiveEditAction = function () {
                    return this.showLiveEditAction;
                };
                ContentWizardActions.prototype.getShowFormAction = function () {
                    return this.showFormAction;
                };
                ContentWizardActions.prototype.getShowSplitEditAction = function () {
                    return this.showSplitEditAction;
                };
                ContentWizardActions.prototype.getActions = function () {
                    return this.actions;
                };
                return ContentWizardActions;
            })();
            action.ContentWizardActions = ContentWizardActions;
        })(action = wizard.action || (wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var Insertable = (function () {
                        function Insertable() {
                        }
                        Insertable.prototype.setName = function (value) {
                            this.name = value;
                            return this;
                        };
                        Insertable.prototype.getName = function () {
                            return this.name;
                        };
                        Insertable.prototype.setDescription = function (value) {
                            this.description = value;
                            return this;
                        };
                        Insertable.prototype.getDescription = function () {
                            return this.description;
                        };
                        Insertable.prototype.setDisplayName = function (value) {
                            this.displayName = value;
                            return this;
                        };
                        Insertable.prototype.getDisplayName = function () {
                            return this.displayName;
                        };
                        Insertable.prototype.setIconCls = function (value) {
                            this.iconCls = value;
                            return this;
                        };
                        Insertable.prototype.getIconCls = function () {
                            return this.iconCls;
                        };
                        return Insertable;
                    })();
                    insert.Insertable = Insertable;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var Insertables = (function () {
                        function Insertables() {
                        }
                        Insertables.IMAGE = new insert.Insertable().setName("image").setDisplayName("Image").setDescription("Upload or use existing images").setIconCls("image");
                        Insertables.PART = new insert.Insertable().setName("part").setDisplayName("Part").setDescription("Advanced components").setIconCls("part");
                        Insertables.LAYOUT = new insert.Insertable().setName("layout").setDisplayName("Layout").setDescription("Customize page layout").setIconCls("layout");
                        Insertables.TEXT = new insert.Insertable().setName("text").setDisplayName("Text").setDescription("Write directly on the page").setIconCls("text");
                        Insertables.ALL = [Insertables.IMAGE, Insertables.PART, Insertables.LAYOUT, Insertables.TEXT];
                        return Insertables;
                    })();
                    insert.Insertables = Insertables;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var InsertablesPanel = (function (_super) {
                        __extends(InsertablesPanel, _super);
                        function InsertablesPanel(config) {
                            var _this = this;
                            _super.call(this, "insertables-panel");
                            this.hideContextWindowRequestListeners = [];
                            this.liveEditPage = config.liveEditPage;
                            var topDescription = new api.dom.PEl();
                            topDescription.getEl().setInnerHtml('Drag and drop components into the page');
                            this.appendChild(topDescription);
                            this.insertablesDataView = new api.ui.grid.DataView();
                            this.insertablesGrid = new insert.InsertablesGrid(this.insertablesDataView, { draggableRows: true, rowClass: "comp" });
                            this.appendChild(this.insertablesGrid);
                            this.insertablesDataView.setItems(insert.Insertables.ALL, "name");
                            this.onRendered(function (event) {
                                _this.initComponentDraggables();
                            });
                            this.onShown(this.updateDraggables.bind(this));
                        }
                        InsertablesPanel.prototype.initComponentDraggables = function () {
                            var _this = this;
                            wemjq(this.liveEditPage.getIFrame().getHTMLElement()).droppable({
                                tolerance: 'pointer',
                                addClasses: false,
                                accept: '.comp',
                                scope: 'component',
                                over: function (event, ui) {
                                    _this.onDragOverIFrame(event, ui);
                                }
                            });
                            this.liveEditPage.onItemFromContextWindowDropped(function () {
                                _this.simulateMouseUpForDraggable();
                            });
                        };
                        InsertablesPanel.prototype.updateDraggables = function () {
                            var _this = this;
                            var components = wemjq('[data-context-window-draggable="true"]:not(.ui-draggable)');
                            components.draggable({
                                cursorAt: { left: -10, top: -15 },
                                appendTo: 'body',
                                cursor: 'move',
                                revert: 'true',
                                distance: 10,
                                scope: 'component',
                                helper: function () { return api.ui.DragHelper.getHtml(); },
                                start: function (event, ui) { return _this.handleDragStart(event, ui); }
                            });
                        };
                        InsertablesPanel.prototype.simulateMouseUpForDraggable = function () {
                            wemjq('[data-context-window-draggable="true"]').simulate('mouseup');
                        };
                        InsertablesPanel.prototype.handleDragStart = function (event, ui) {
                            this.liveEditPage.showDragMask();
                        };
                        InsertablesPanel.prototype.onDragOverIFrame = function (event, ui) {
                            this.liveEditPage.hideDragMask();
                            var liveEditJQuery = this.liveEditPage.getLiveEditJQuery();
                            var clonedDraggable = liveEditJQuery(ui.draggable.clone());
                            clonedDraggable.css({
                                'position': 'absolute',
                                'top': '-1000px'
                            });
                            liveEditJQuery('body').append(clonedDraggable);
                            ui.helper.hide(null);
                            this.liveEditPage.createDraggable(clonedDraggable);
                            clonedDraggable.simulate('mousedown');
                            this.notifyHideContextWindowRequest();
                        };
                        InsertablesPanel.prototype.onHideContextWindowRequest = function (listener) {
                            this.hideContextWindowRequestListeners.push(listener);
                        };
                        InsertablesPanel.prototype.unHideContextWindowRequest = function (listener) {
                            this.hideContextWindowRequestListeners = this.hideContextWindowRequestListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        InsertablesPanel.prototype.notifyHideContextWindowRequest = function () {
                            this.hideContextWindowRequestListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        return InsertablesPanel;
                    })(api.ui.panel.Panel);
                    insert.InsertablesPanel = InsertablesPanel;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var InsertablesGrid = (function (_super) {
                        __extends(InsertablesGrid, _super);
                        function InsertablesGrid(dataView, options) {
                            var _this = this;
                            if (options === void 0) { options = {}; }
                            _super.call(this, dataView, this.createColumns(), this.createOptions());
                            this.componentDataView = dataView;
                            this.componentGridOptions = options;
                            this.onRendered(function (event) {
                                if (_this.componentGridOptions.onClick) {
                                    _this.setOnClick(_this.componentGridOptions.onClick);
                                }
                            });
                        }
                        InsertablesGrid.prototype.createOptions = function () {
                            return new api.ui.grid.GridOptionsBuilder().setHideColumnHeaders(true).setRowHeight(50).setHeight(400).setWidth(320).build();
                        };
                        InsertablesGrid.prototype.createColumns = function () {
                            var _this = this;
                            return [new api.ui.grid.GridColumnBuilder().setName("component").setField("component").setId("component").setWidth(320).setCssClass("grid-row").setFormatter(function (row, cell, value, columnDef, dataContext) {
                                return _this.buildRow(row, cell, value, columnDef, dataContext).toString();
                            }).build()];
                        };
                        InsertablesGrid.prototype.buildRow = function (row, cell, value, columnDef, insertable) {
                            var rowEl = new api.dom.DivEl();
                            rowEl.getEl().setData('portal-component-type', insertable.getName());
                            if (this.componentGridOptions.draggableRows) {
                                rowEl.getEl().setData('context-window-draggable', 'true');
                            }
                            if (this.componentGridOptions.rowClass) {
                                rowEl.addClass(this.componentGridOptions.rowClass);
                            }
                            var icon = new api.dom.DivEl();
                            icon.setClass('live-edit-font-icon-' + insertable.getIconCls());
                            icon.addClass('icon');
                            var title = new api.dom.H5El();
                            title.getEl().setInnerHtml(insertable.getDisplayName());
                            var subtitle = new api.dom.H6El();
                            subtitle.getEl().setInnerHtml(insertable.getDescription());
                            rowEl.appendChild(icon);
                            rowEl.appendChild(title);
                            rowEl.appendChild(subtitle);
                            return rowEl;
                        };
                        return InsertablesGrid;
                    })(api.ui.grid.Grid);
                    insert.InsertablesGrid = InsertablesGrid;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var BaseInspectionPanel = (function (_super) {
                        __extends(BaseInspectionPanel, _super);
                        function BaseInspectionPanel() {
                            var _this = this;
                            _super.call(this, "inspection-panel");
                            this.onRendered(function (event) {
                                wemjq(_this.getHTMLElement()).slimScroll({
                                    height: '100%'
                                });
                            });
                        }
                        return BaseInspectionPanel;
                    })(api.ui.panel.Panel);
                    inspect.BaseInspectionPanel = BaseInspectionPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var NoSelectionInspectionPanel = (function (_super) {
                        __extends(NoSelectionInspectionPanel, _super);
                        function NoSelectionInspectionPanel() {
                            _super.call(this, "inspection-panel");
                            this.header = new api.app.NamesView().setMainName("No item selected");
                            this.appendChild(this.header);
                        }
                        return NoSelectionInspectionPanel;
                    })(api.ui.panel.Panel);
                    inspect.NoSelectionInspectionPanel = NoSelectionInspectionPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var ContentInspectionPanel = (function (_super) {
                        __extends(ContentInspectionPanel, _super);
                        function ContentInspectionPanel() {
                            _super.call(this);
                            this.namesAndIcon = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(1 /* medium */)).setIconClass("live-edit-font-icon-content icon-xlarge");
                            this.appendChild(this.namesAndIcon);
                        }
                        ContentInspectionPanel.prototype.setContent = function (content) {
                            this.content = content;
                            if (content) {
                                this.namesAndIcon.setMainName(content.getDisplayName());
                                this.namesAndIcon.setSubName(content.getPath().toString());
                            }
                            else {
                                this.namesAndIcon.setMainName("[No Content given]");
                                this.namesAndIcon.setSubName("");
                            }
                        };
                        return ContentInspectionPanel;
                    })(inspect.BaseInspectionPanel);
                    inspect.ContentInspectionPanel = ContentInspectionPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var InspectionsPanel = (function (_super) {
                        __extends(InspectionsPanel, _super);
                        function InspectionsPanel(config) {
                            _super.call(this, 'inspections-panel');
                            this.saveRequestListeners = [];
                            this.deck = new api.ui.panel.DeckPanel();
                            this.noSelectionPanel = new inspect.NoSelectionInspectionPanel();
                            this.imageInspectionPanel = config.imageInspectionPanel;
                            this.partInspectionPanel = config.partInspectionPanel;
                            this.layoutInspectionPanel = config.layoutInspectionPanel;
                            this.contentInspectionPanel = config.contentInspectionPanel;
                            this.pageInspectionPanel = config.pageInspectionPanel;
                            this.regionInspectionPanel = config.regionInspectionPanel;
                            this.deck.addPanel(this.imageInspectionPanel);
                            this.deck.addPanel(this.partInspectionPanel);
                            this.deck.addPanel(this.layoutInspectionPanel);
                            this.deck.addPanel(this.contentInspectionPanel);
                            this.deck.addPanel(this.regionInspectionPanel);
                            this.deck.addPanel(this.pageInspectionPanel);
                            this.deck.addPanel(this.noSelectionPanel);
                            this.deck.showPanel(this.pageInspectionPanel);
                            this.buttons = new api.dom.DivEl('button-bar');
                            var saveButton = new api.ui.button.ActionButton(config.saveAction);
                            this.buttons.appendChild(saveButton);
                            this.appendChildren(this.deck, this.buttons);
                        }
                        InspectionsPanel.prototype.showInspectionPanel = function (panel) {
                            this.deck.showPanel(panel);
                            var showButtons = !(api.ObjectHelper.iFrameSafeInstanceOf(panel, inspect.region.RegionInspectionPanel) || api.ObjectHelper.iFrameSafeInstanceOf(panel, inspect.NoSelectionInspectionPanel));
                            this.buttons.setVisible(showButtons);
                        };
                        InspectionsPanel.prototype.clearSelection = function () {
                            this.showInspectionPanel(this.noSelectionPanel);
                        };
                        return InspectionsPanel;
                    })(api.ui.panel.Panel);
                    inspect.InspectionsPanel = InspectionsPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateOption = (function () {
                            function PageTemplateOption(template, pageModel) {
                                this.template = template;
                                this.pageModel = pageModel;
                            }
                            PageTemplateOption.prototype.getPageTemplate = function () {
                                return this.template;
                            };
                            PageTemplateOption.prototype.getPageModel = function () {
                                return this.pageModel;
                            };
                            return PageTemplateOption;
                        })();
                        page.PageTemplateOption = PageTemplateOption;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateOptionViewer = (function (_super) {
                            __extends(PageTemplateOptionViewer, _super);
                            function PageTemplateOptionViewer() {
                                _super.call(this);
                            }
                            PageTemplateOptionViewer.prototype.resolveDisplayName = function (object) {
                                return !!object.getPageTemplate() ? object.getPageTemplate().getDisplayName() : "Automatic";
                            };
                            PageTemplateOptionViewer.prototype.resolveSubName = function (object, relativePath) {
                                if (relativePath === void 0) { relativePath = false; }
                                return !!object.getPageTemplate() ? object.getPageTemplate().getPath().toString() : "(" + object.getPageModel().getDefaultPageTemplate().getDisplayName().toString() + ")";
                            };
                            PageTemplateOptionViewer.prototype.resolveIconClass = function (object) {
                                return !!object.getPageTemplate() ? "icon-newspaper icon-large" : "icon-wand icon-large";
                            };
                            PageTemplateOptionViewer.prototype.getPreferredHeight = function () {
                                return 50;
                            };
                            return PageTemplateOptionViewer;
                        })(api.ui.NamesAndIconViewer);
                        page.PageTemplateOptionViewer = PageTemplateOptionViewer;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var GetPageTemplatesByCanRenderRequest = api.content.page.GetPageTemplatesByCanRenderRequest;
                        var PageTemplateLoader = api.content.page.PageTemplateLoader;
                        var PageTemplateOptions = (function () {
                            function PageTemplateOptions(siteId, contentType, pageModel) {
                                this.siteId = siteId;
                                this.contentType = contentType;
                                this.pageModel = pageModel;
                                this.defaultPageTemplateOption = { value: "__auto__", displayValue: new page.PageTemplateOption(null, pageModel) };
                            }
                            PageTemplateOptions.prototype.getDefault = function () {
                                return this.defaultPageTemplateOption;
                            };
                            PageTemplateOptions.prototype.getOptions = function () {
                                var _this = this;
                                var deferred = wemQ.defer();
                                var options = [];
                                options.push(this.defaultPageTemplateOption);
                                var loader = new PageTemplateLoader(new GetPageTemplatesByCanRenderRequest(this.siteId, this.contentType));
                                loader.setComparator(new api.content.page.PageTemplateByDisplayNameComparator());
                                loader.onLoadedData(function (event) {
                                    event.getData().forEach(function (pageTemplate) {
                                        var indices = [];
                                        indices.push(pageTemplate.getName().toString());
                                        indices.push(pageTemplate.getDisplayName());
                                        indices.push(pageTemplate.getController().toString());
                                        var option = {
                                            value: pageTemplate.getId().toString(),
                                            displayValue: new page.PageTemplateOption(pageTemplate, _this.pageModel),
                                            indices: indices
                                        };
                                        options.push(option);
                                    });
                                    deferred.resolve(options);
                                });
                                loader.load();
                                return deferred.promise;
                            };
                            return PageTemplateOptions;
                        })();
                        page.PageTemplateOptions = PageTemplateOptions;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var SiteModel = api.content.site.SiteModel;
                        var SetController = api.content.page.SetController;
                        var PageDescriptorDropdown = api.content.page.PageDescriptorDropdown;
                        var GetPageDescriptorsByModulesRequest = api.content.page.GetPageDescriptorsByModulesRequest;
                        var PageControllerSelector = (function (_super) {
                            __extends(PageControllerSelector, _super);
                            function PageControllerSelector() {
                                var _this = this;
                                this.getPageDescriptorsByModulesRequest = new GetPageDescriptorsByModulesRequest([]);
                                _super.call(this, 'page-controller', {
                                    loader: new api.util.loader.BaseLoader(this.getPageDescriptorsByModulesRequest).setComparator(new api.content.page.DescriptorByDisplayNameComparator())
                                });
                                this.onOptionSelected(function (event) {
                                    var pageDescriptor = event.getOption().displayValue;
                                    var setController = new SetController(_this).setDescriptor(pageDescriptor);
                                    _this.pageModel.setController(setController);
                                });
                            }
                            PageControllerSelector.prototype.setModel = function (model) {
                                var _this = this;
                                this.siteModel = model.getSiteModel();
                                this.pageModel = model.getPageModel();
                                this.getPageDescriptorsByModulesRequest.setModuleKeys(this.siteModel.getModuleKeys());
                                this.onLoadedData(function (event) {
                                    if (_this.pageModel.hasController()) {
                                        _this.selectController(_this.pageModel.getController().getKey());
                                    }
                                });
                                this.load();
                                this.siteModel.onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == SiteModel.PROPERTY_NAME_MODULE_CONFIGS) {
                                        _this.getPageDescriptorsByModulesRequest.setModuleKeys(_this.siteModel.getModuleKeys());
                                        _this.load();
                                    }
                                });
                                this.pageModel.onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == "controller" && _this !== event.getSource()) {
                                        var descriptorKey = event.getNewValue();
                                        if (descriptorKey) {
                                            _this.selectController(descriptorKey);
                                        }
                                    }
                                });
                            };
                            PageControllerSelector.prototype.selectController = function (descriptorKey) {
                                var optionToSelect = this.getOptionByValue(descriptorKey.toString());
                                if (optionToSelect) {
                                    this.selectOption(optionToSelect, true);
                                }
                            };
                            return PageControllerSelector;
                        })(PageDescriptorDropdown);
                        page.PageControllerSelector = PageControllerSelector;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var Dropdown = api.ui.selector.dropdown.Dropdown;
                        var PageTemplateSelector = (function (_super) {
                            __extends(PageTemplateSelector, _super);
                            function PageTemplateSelector() {
                                _super.call(this, "pageTemplate", {
                                    optionDisplayValueViewer: new page.PageTemplateOptionViewer()
                                });
                                this.selectionListeners = [];
                            }
                            PageTemplateSelector.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                this.pageModel = liveEditModel.getPageModel();
                                var pageTemplateOptions = new page.PageTemplateOptions(liveEditModel.getSiteModel().getSiteId(), liveEditModel.getContent().getType(), this.pageModel);
                                pageTemplateOptions.getOptions().then(function (options) {
                                    options.forEach(function (option) {
                                        _this.addOption(option);
                                    });
                                    if (_this.pageModel.hasTemplate()) {
                                        _this.selectTemplate(_this.pageModel.getTemplateKey());
                                    }
                                    else {
                                        _this.selectOption(pageTemplateOptions.getDefault(), true);
                                    }
                                    _this.onOptionSelected(function (event) {
                                        var pageTemplate = event.getOption().displayValue.getPageTemplate();
                                        _this.notifySelection(pageTemplate);
                                    });
                                    _this.pageModel.onPropertyChanged(function (event) {
                                        if (event.getPropertyName() == "template" && _this !== event.getSource()) {
                                            var pageTemplateKey = event.getNewValue();
                                            if (pageTemplateKey) {
                                                _this.selectTemplate(pageTemplateKey);
                                            }
                                            else {
                                                _this.selectOption(pageTemplateOptions.getDefault());
                                            }
                                        }
                                    });
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                            };
                            PageTemplateSelector.prototype.selectTemplate = function (template) {
                                var optionToSelect = this.getOptionByValue(template.toString());
                                if (optionToSelect) {
                                    this.selectOption(optionToSelect, true);
                                }
                            };
                            PageTemplateSelector.prototype.onSelection = function (listener) {
                                this.selectionListeners.push(listener);
                            };
                            PageTemplateSelector.prototype.unSelection = function (listener) {
                                this.selectionListeners.filter(function (currentListener) {
                                    return listener != currentListener;
                                });
                            };
                            PageTemplateSelector.prototype.notifySelection = function (item) {
                                this.selectionListeners.forEach(function (listener) {
                                    listener(item);
                                });
                            };
                            return PageTemplateSelector;
                        })(Dropdown);
                        page.PageTemplateSelector = PageTemplateSelector;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageControllerForm = (function (_super) {
                            __extends(PageControllerForm, _super);
                            function PageControllerForm(controllerSelector) {
                                _super.call(this, 'form-view');
                                this.controllerSelector = controllerSelector;
                                var fieldSet = new api.ui.form.Fieldset();
                                fieldSet.add(new api.ui.form.FormItemBuilder(controllerSelector).setLabel("Page Controller").build());
                                this.add(fieldSet);
                            }
                            PageControllerForm.prototype.getSelector = function () {
                                return this.controllerSelector;
                            };
                            return PageControllerForm;
                        })(api.ui.form.Form);
                        page.PageControllerForm = PageControllerForm;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateForm = (function (_super) {
                            __extends(PageTemplateForm, _super);
                            function PageTemplateForm(templateSelector) {
                                _super.call(this, 'form-view');
                                this.templateSelector = templateSelector;
                                var fieldSet = new api.ui.form.Fieldset();
                                fieldSet.add(new api.ui.form.FormItemBuilder(templateSelector).setLabel("Page template").build());
                                this.add(fieldSet);
                            }
                            PageTemplateForm.prototype.getSelector = function () {
                                return this.templateSelector;
                            };
                            return PageTemplateForm;
                        })(api.ui.form.Form);
                        page.PageTemplateForm = PageTemplateForm;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var FormContextBuilder = api.form.FormContextBuilder;
                        var FormView = api.form.FormView;
                        var SetTemplate = api.content.page.SetTemplate;
                        var PageMode = api.content.page.PageMode;
                        var GetPageDescriptorByKeyRequest = api.content.page.GetPageDescriptorByKeyRequest;
                        var PageInspectionPanel = (function (_super) {
                            __extends(PageInspectionPanel, _super);
                            function PageInspectionPanel() {
                                _super.call(this);
                            }
                            PageInspectionPanel.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                this.liveEditModel = liveEditModel;
                                this.pageModel = liveEditModel.getPageModel();
                                this.siteModel = liveEditModel.getSiteModel();
                                this.pageTemplateSelector = new page.PageTemplateSelector();
                                this.pageTemplateForm = new page.PageTemplateForm(this.pageTemplateSelector);
                                this.pageTemplateForm.hide();
                                this.appendChild(this.pageTemplateForm);
                                this.pageControllerSelector = new page.PageControllerSelector();
                                this.pageControllerForm = new page.PageControllerForm(this.pageControllerSelector);
                                this.pageControllerForm.hide();
                                this.appendChild(this.pageControllerForm);
                                if (this.pageModel.isPageTemplate()) {
                                    new PageTemplateInspectionHandler().setPageModel(this.pageModel).setPageInspectionPanel(this).setPageControllerForm(this.pageControllerForm).setPageTemplateForm(this.pageTemplateForm).setModel(liveEditModel);
                                }
                                else {
                                    new ContentInspectionHandler().setPageModel(this.pageModel).setPageInspectionPanel(this).setPageControllerForm(this.pageControllerForm).setPageTemplateForm(this.pageTemplateForm).setModel(liveEditModel);
                                }
                                this.pageTemplateForm.getSelector().onSelection(function (pageTemplate) {
                                    if (pageTemplate) {
                                        new GetPageDescriptorByKeyRequest(pageTemplate.getController()).sendAndParse().then(function (pageDescriptor) {
                                            var setTemplate = new SetTemplate(_this).setTemplate(pageTemplate, pageDescriptor);
                                            _this.pageModel.setTemplate(setTemplate);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                    else {
                                        _this.pageModel.setAutomaticTemplate(_this);
                                    }
                                });
                            };
                            return PageInspectionPanel;
                        })(app.wizard.page.contextwindow.inspect.BaseInspectionPanel);
                        page.PageInspectionPanel = PageInspectionPanel;
                        var BaseInspectionHandler = (function () {
                            function BaseInspectionHandler() {
                            }
                            BaseInspectionHandler.prototype.setPageModel = function (value) {
                                this.pageModel = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setPageInspectionPanel = function (value) {
                                this.pageInspectionPanel = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setPageControllerForm = function (value) {
                                this.pageControllerForm = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setPageTemplateForm = function (value) {
                                this.pageTemplateForm = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setModel = function (liveEditModel) {
                                throw new Error("Must be implemented by inheritors");
                            };
                            BaseInspectionHandler.prototype.refreshConfigForm = function (pageDescriptor, config) {
                                var _this = this;
                                if (this.configForm) {
                                    this.configForm.remove();
                                    this.configForm = null;
                                }
                                if (!pageDescriptor) {
                                    return;
                                }
                                this.configForm = new FormView(new FormContextBuilder().build(), pageDescriptor.getConfig(), config.getRoot());
                                this.pageInspectionPanel.appendChild(this.configForm);
                                this.pageModel.setIgnorePropertyChanges(true);
                                this.configForm.layout().catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).finally(function () {
                                    _this.pageModel.setIgnorePropertyChanges(false);
                                }).done();
                            };
                            return BaseInspectionHandler;
                        })();
                        var PageTemplateInspectionHandler = (function (_super) {
                            __extends(PageTemplateInspectionHandler, _super);
                            function PageTemplateInspectionHandler() {
                                _super.apply(this, arguments);
                            }
                            PageTemplateInspectionHandler.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                var pageModel = liveEditModel.getPageModel();
                                this.pageControllerForm.getSelector().setModel(liveEditModel);
                                this.pageControllerForm.show();
                                this.refreshConfigForm(pageModel.getController(), pageModel.getConfig());
                                pageModel.onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == "controller" && _this !== event.getSource()) {
                                        _this.pageControllerForm.show();
                                        _this.refreshConfigForm(pageModel.getController(), pageModel.getConfig());
                                    }
                                    else if (event.getPropertyName() == "config" && _this !== event.getSource()) {
                                        _this.pageControllerForm.show();
                                        var controller = pageModel.getController();
                                        if (controller) {
                                            _this.refreshConfigForm(controller, pageModel.getConfig());
                                        }
                                    }
                                });
                            };
                            return PageTemplateInspectionHandler;
                        })(BaseInspectionHandler);
                        var ContentInspectionHandler = (function (_super) {
                            __extends(ContentInspectionHandler, _super);
                            function ContentInspectionHandler() {
                                _super.apply(this, arguments);
                            }
                            ContentInspectionHandler.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                var pageModel = liveEditModel.getPageModel();
                                var pageMode = pageModel.getMode();
                                if (pageMode == 1 /* FORCED_TEMPLATE */) {
                                    this.pageTemplateForm.getSelector().setModel(liveEditModel);
                                    this.pageTemplateForm.show();
                                    this.showPageConfig(pageModel);
                                }
                                else if (pageMode == 0 /* AUTOMATIC */) {
                                    this.pageTemplateForm.getSelector().setModel(liveEditModel);
                                    this.pageTemplateForm.show();
                                    this.showDefaultPageTemplateConfig(pageModel);
                                }
                                else {
                                    throw new Error("Unsupported PageMode: " + pageMode);
                                }
                                pageModel.onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == "controller" && _this !== event.getSource()) {
                                        _this.pageControllerForm.show();
                                        var controller = pageModel.getController();
                                        if (controller) {
                                            _this.refreshConfigForm(controller, pageModel.getConfig());
                                        }
                                    }
                                    else if (event.getPropertyName() == "config" && _this !== event.getSource()) {
                                        _this.pageTemplateForm.show();
                                        if (pageModel.getMode() == 0 /* AUTOMATIC */) {
                                            _this.showDefaultPageTemplateConfig(pageModel);
                                        }
                                        else {
                                            _this.showPageConfig(pageModel);
                                        }
                                    }
                                });
                            };
                            ContentInspectionHandler.prototype.showPageConfig = function (pageModel) {
                                this.refreshConfigForm(pageModel.getTemplateDescriptor(), pageModel.getConfig());
                            };
                            ContentInspectionHandler.prototype.showDefaultPageTemplateConfig = function (pageModel) {
                                var controller = pageModel.getDefaultPageTemplateController();
                                var config = pageModel.getConfig();
                                this.refreshConfigForm(controller, config);
                            };
                            return ContentInspectionHandler;
                        })(BaseInspectionHandler);
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var ComponentInspectionPanel = (function (_super) {
                            __extends(ComponentInspectionPanel, _super);
                            function ComponentInspectionPanel(config) {
                                _super.call(this);
                            }
                            ComponentInspectionPanel.prototype.setModel = function (liveEditModel) {
                                this.liveEditModel = liveEditModel;
                                this.formContext = liveEditModel.getFormContext();
                            };
                            ComponentInspectionPanel.prototype.setComponent = function (component) {
                                this.component = component;
                            };
                            ComponentInspectionPanel.prototype.getComponentView = function () {
                                throw new Error("Must be implemented by inheritors");
                            };
                            ComponentInspectionPanel.prototype.getComponent = function () {
                                return this.component;
                            };
                            return ComponentInspectionPanel;
                        })(app.wizard.page.contextwindow.inspect.BaseInspectionPanel);
                        region.ComponentInspectionPanel = ComponentInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var FormView = api.form.FormView;
                        var DescriptorBasedComponentInspectionPanel = (function (_super) {
                            __extends(DescriptorBasedComponentInspectionPanel, _super);
                            function DescriptorBasedComponentInspectionPanel(config) {
                                _super.call(this, config);
                                this.formView = null;
                            }
                            DescriptorBasedComponentInspectionPanel.prototype.setupComponentForm = function (component, descriptor) {
                                if (this.formView) {
                                    this.removeChild(this.formView);
                                    this.formView = null;
                                }
                                if (!component || !descriptor) {
                                    return;
                                }
                                var form = descriptor.getConfig();
                                var config = component.getConfig();
                                this.formView = new FormView(this.formContext, form, config.getRoot());
                                this.appendChild(this.formView);
                                component.setDisableEventForwarding(true);
                                this.formView.layout().catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).finally(function () {
                                    component.setDisableEventForwarding(false);
                                }).done();
                            };
                            return DescriptorBasedComponentInspectionPanel;
                        })(region.ComponentInspectionPanel);
                        region.DescriptorBasedComponentInspectionPanel = DescriptorBasedComponentInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var ImageComponent = api.content.page.region.ImageComponent;
                        var GetContentSummaryByIdRequest = api.content.GetContentSummaryByIdRequest;
                        var ContentComboBox = api.content.ContentComboBox;
                        var ContentTypeName = api.schema.content.ContentTypeName;
                        var ImageInspectionPanel = (function (_super) {
                            __extends(ImageInspectionPanel, _super);
                            function ImageInspectionPanel() {
                                var _this = this;
                                _super.call(this, {
                                    iconClass: "live-edit-font-icon-image icon-xlarge"
                                });
                                this.handleSelectorEvents = true;
                                this.imageSelector = ContentComboBox.create().setMaximumOccurrences(1).setAllowedContentTypes([ContentTypeName.IMAGE.toString()]).setLoader(new api.content.ContentSummaryLoader()).build();
                                this.componentPropertyChangedEventHandler = function (event) {
                                    if (event.getPropertyName() == ImageComponent.PROPERTY_IMAGE) {
                                        if (!_this.imageComponent.hasImage()) {
                                            _this.setupComponentForm(_this.imageComponent);
                                            _this.imageSelector.setContent(null);
                                        }
                                    }
                                };
                                this.initSelectorListeners();
                                this.appendChild(this.imageSelector);
                            }
                            ImageInspectionPanel.prototype.setComponent = function (component) {
                                _super.prototype.setComponent.call(this, component);
                            };
                            ImageInspectionPanel.prototype.setImageComponent = function (imageView) {
                                var _this = this;
                                this.imageView = imageView;
                                if (this.imageComponent) {
                                    this.unregisterComponentListeners(this.imageComponent);
                                }
                                this.imageComponent = imageView.getComponent();
                                this.setComponent(this.imageComponent);
                                var contentId = this.imageComponent.getImage();
                                if (contentId) {
                                    var image = this.imageSelector.getContent(contentId);
                                    if (image) {
                                        this.setSelectorValue(image);
                                        this.setupComponentForm(this.imageComponent);
                                    }
                                    else {
                                        new GetContentSummaryByIdRequest(contentId).sendAndParse().then(function (image) {
                                            _this.setSelectorValue(image);
                                            _this.setupComponentForm(_this.imageComponent);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                }
                                else {
                                    this.setSelectorValue(null);
                                    this.setupComponentForm(this.imageComponent);
                                }
                                this.registerComponentListeners(this.imageComponent);
                            };
                            ImageInspectionPanel.prototype.registerComponentListeners = function (component) {
                                component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            ImageInspectionPanel.prototype.unregisterComponentListeners = function (component) {
                                component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            ImageInspectionPanel.prototype.setSelectorValue = function (image) {
                                this.handleSelectorEvents = false;
                                this.imageSelector.setContent(image);
                                this.handleSelectorEvents = true;
                            };
                            ImageInspectionPanel.prototype.setupComponentForm = function (imageComponent) {
                                if (this.formView) {
                                    this.removeChild(this.formView);
                                    this.formView = null;
                                }
                                var configData = imageComponent.getConfig();
                                var configForm = imageComponent.getForm();
                                this.formView = new api.form.FormView(this.formContext, configForm, configData.getRoot());
                                this.appendChild(this.formView);
                                imageComponent.setDisableEventForwarding(true);
                                this.formView.layout().catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).finally(function () {
                                    imageComponent.setDisableEventForwarding(false);
                                }).done();
                            };
                            ImageInspectionPanel.prototype.initSelectorListeners = function () {
                                var _this = this;
                                this.imageSelector.onOptionSelected(function (event) {
                                    if (_this.handleSelectorEvents) {
                                        var option = event.getOption();
                                        var imageContent = option.displayValue;
                                        _this.imageComponent.setImage(imageContent.getContentId(), imageContent.getDisplayName());
                                    }
                                });
                                this.imageSelector.onOptionDeselected(function (option) {
                                    if (_this.handleSelectorEvents) {
                                        _this.imageComponent.setImage(null, null);
                                    }
                                });
                            };
                            ImageInspectionPanel.prototype.getComponentView = function () {
                                return this.imageView;
                            };
                            return ImageInspectionPanel;
                        })(region.ComponentInspectionPanel);
                        region.ImageInspectionPanel = ImageInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var PartDescriptorComboBox = api.content.page.region.PartDescriptorComboBox;
                        var PartDescriptorLoader = api.content.page.region.PartDescriptorLoader;
                        var GetPartDescriptorsByModulesRequest = api.content.page.region.GetPartDescriptorsByModulesRequest;
                        var GetPartDescriptorByKeyRequest = api.content.page.region.GetPartDescriptorByKeyRequest;
                        var DescriptorBasedComponent = api.content.page.region.DescriptorBasedComponent;
                        var PartInspectionPanel = (function (_super) {
                            __extends(PartInspectionPanel, _super);
                            function PartInspectionPanel() {
                                _super.call(this, {
                                    iconClass: "live-edit-font-icon-part icon-xlarge"
                                });
                                this.handleSelectorEvents = true;
                            }
                            PartInspectionPanel.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                _super.prototype.setModel.call(this, liveEditModel);
                                var descriptorsRequest = new GetPartDescriptorsByModulesRequest(liveEditModel.getSiteModel().getModuleKeys());
                                var loader = new PartDescriptorLoader(descriptorsRequest);
                                loader.setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                                this.partSelector = new PartDescriptorComboBox(loader);
                                loader.load();
                                this.componentPropertyChangedEventHandler = function (event) {
                                    if (event.getPropertyName() == DescriptorBasedComponent.PROPERTY_DESCRIPTOR) {
                                        if (!_this.partComponent.hasDescriptor()) {
                                            _this.setupComponentForm(_this.partComponent, null);
                                            _this.partSelector.setDescriptor(null);
                                        }
                                    }
                                };
                                this.initSelectorListeners();
                                this.appendChild(this.partSelector);
                            };
                            PartInspectionPanel.prototype.setComponent = function (component, descriptor) {
                                _super.prototype.setComponent.call(this, component);
                                if (descriptor) {
                                    this.partSelector.setDescriptor(descriptor);
                                }
                            };
                            PartInspectionPanel.prototype.setSelectorValue = function (descriptor) {
                                this.handleSelectorEvents = false;
                                this.partSelector.setDescriptor(descriptor);
                                this.handleSelectorEvents = true;
                            };
                            PartInspectionPanel.prototype.registerComponentListeners = function (component) {
                                component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            PartInspectionPanel.prototype.unregisterComponentListeners = function (component) {
                                component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            PartInspectionPanel.prototype.setPartComponent = function (partView) {
                                var _this = this;
                                if (this.partComponent) {
                                    this.unregisterComponentListeners(this.partComponent);
                                }
                                this.partView = partView;
                                this.partComponent = partView.getComponent();
                                this.setComponent(this.partComponent);
                                var key = this.partComponent.getDescriptor();
                                if (key) {
                                    var descriptor = this.partSelector.getDescriptor(key);
                                    if (descriptor) {
                                        this.setSelectorValue(descriptor);
                                        this.setupComponentForm(this.partComponent, descriptor);
                                    }
                                    else {
                                        new GetPartDescriptorByKeyRequest(key).sendAndParse().then(function (descriptor) {
                                            _this.setSelectorValue(descriptor);
                                            _this.setupComponentForm(_this.partComponent, descriptor);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                }
                                else {
                                    this.setSelectorValue(null);
                                    this.setupComponentForm(this.partComponent, null);
                                }
                                this.registerComponentListeners(this.partComponent);
                            };
                            PartInspectionPanel.prototype.initSelectorListeners = function () {
                                var _this = this;
                                this.partSelector.onOptionSelected(function (event) {
                                    if (_this.handleSelectorEvents) {
                                        var option = event.getOption();
                                        var selectedDescriptorKey = option.displayValue.getKey();
                                        _this.partComponent.setDescriptor(selectedDescriptorKey, option.displayValue);
                                    }
                                });
                                this.partSelector.onOptionDeselected(function (option) {
                                    if (_this.handleSelectorEvents) {
                                        _this.partComponent.setDescriptor(null, null);
                                    }
                                });
                            };
                            return PartInspectionPanel;
                        })(region.DescriptorBasedComponentInspectionPanel);
                        region.PartInspectionPanel = PartInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var SiteModel = api.content.site.SiteModel;
                        var DescriptorByDisplayNameComparator = api.content.page.DescriptorByDisplayNameComparator;
                        var DescriptorBasedComponent = api.content.page.region.DescriptorBasedComponent;
                        var GetLayoutDescriptorByKeyRequest = api.content.page.region.GetLayoutDescriptorByKeyRequest;
                        var GetLayoutDescriptorsByModulesRequest = api.content.page.region.GetLayoutDescriptorsByModulesRequest;
                        var LayoutDescriptorLoader = api.content.page.region.LayoutDescriptorLoader;
                        var LayoutDescriptorComboBox = api.content.page.region.LayoutDescriptorComboBox;
                        var LayoutInspectionPanel = (function (_super) {
                            __extends(LayoutInspectionPanel, _super);
                            function LayoutInspectionPanel() {
                                _super.call(this, {
                                    iconClass: "live-edit-font-icon-layout icon-xlarge"
                                });
                                this.handleSelectorEvents = true;
                            }
                            LayoutInspectionPanel.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                _super.prototype.setModel.call(this, liveEditModel);
                                var descriptorsRequest = new GetLayoutDescriptorsByModulesRequest(liveEditModel.getSiteModel().getModuleKeys());
                                var loader = new LayoutDescriptorLoader(descriptorsRequest);
                                loader.setComparator(new DescriptorByDisplayNameComparator());
                                this.layoutSelector = new LayoutDescriptorComboBox(loader);
                                loader.load();
                                this.componentPropertyChangedEventHandler = function (event) {
                                    if (event.getPropertyName() == DescriptorBasedComponent.PROPERTY_DESCRIPTOR) {
                                        if (!_this.layoutComponent.hasDescriptor()) {
                                            _this.setupComponentForm(_this.layoutComponent, null);
                                            _this.layoutSelector.setDescriptor(null);
                                        }
                                    }
                                };
                                this.initSelectorListeners();
                                this.appendChild(this.layoutSelector);
                                liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == SiteModel.PROPERTY_NAME_MODULE_CONFIGS) {
                                        descriptorsRequest.setModuleKeys(liveEditModel.getSiteModel().getModuleKeys());
                                        loader.load();
                                    }
                                });
                            };
                            LayoutInspectionPanel.prototype.registerComponentListeners = function (component) {
                                component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            LayoutInspectionPanel.prototype.unregisterComponentListeners = function (component) {
                                component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            LayoutInspectionPanel.prototype.setComponent = function (component, descriptor) {
                                _super.prototype.setComponent.call(this, component);
                                if (descriptor) {
                                    this.layoutSelector.setDescriptor(descriptor);
                                }
                            };
                            LayoutInspectionPanel.prototype.setLayoutComponent = function (layoutView) {
                                var _this = this;
                                if (this.layoutComponent) {
                                    this.unregisterComponentListeners(this.layoutComponent);
                                }
                                this.layoutView = layoutView;
                                this.layoutComponent = layoutView.getComponent();
                                this.setComponent(this.layoutComponent);
                                var key = this.layoutComponent.getDescriptor();
                                if (key) {
                                    var descriptor = this.layoutSelector.getDescriptor(key);
                                    if (descriptor) {
                                        this.setSelectorValue(descriptor);
                                        this.setupComponentForm(this.layoutComponent, descriptor);
                                    }
                                    else {
                                        new GetLayoutDescriptorByKeyRequest(key).sendAndParse().then(function (descriptor) {
                                            _this.setSelectorValue(descriptor);
                                            _this.setupComponentForm(_this.layoutComponent, descriptor);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                }
                                else {
                                    this.setSelectorValue(null);
                                    this.setupComponentForm(this.layoutComponent, null);
                                }
                                this.registerComponentListeners(this.layoutComponent);
                            };
                            LayoutInspectionPanel.prototype.setSelectorValue = function (descriptor) {
                                this.handleSelectorEvents = false;
                                this.layoutSelector.setDescriptor(descriptor);
                                this.handleSelectorEvents = true;
                            };
                            LayoutInspectionPanel.prototype.initSelectorListeners = function () {
                                var _this = this;
                                this.layoutSelector.onOptionSelected(function (event) {
                                    if (_this.handleSelectorEvents) {
                                        var option = event.getOption();
                                        var selectedDescriptorKey = option.displayValue.getKey();
                                        _this.layoutComponent.setDescriptor(selectedDescriptorKey, option.displayValue);
                                    }
                                });
                                this.layoutSelector.onOptionDeselected(function (option) {
                                    if (_this.handleSelectorEvents) {
                                        _this.layoutComponent.setDescriptor(null, null);
                                    }
                                });
                            };
                            return LayoutInspectionPanel;
                        })(region.DescriptorBasedComponentInspectionPanel);
                        region.LayoutInspectionPanel = LayoutInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (_region) {
                        var RegionInspectionPanel = (function (_super) {
                            __extends(RegionInspectionPanel, _super);
                            function RegionInspectionPanel() {
                                _super.call(this);
                                this.namesAndIcon = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(1 /* medium */)).setIconClass("live-edit-font-icon-region icon-xlarge");
                                this.appendChild(this.namesAndIcon);
                            }
                            RegionInspectionPanel.prototype.setRegion = function (region) {
                                this.region = region;
                                if (region) {
                                    this.namesAndIcon.setMainName(region.getName());
                                    this.namesAndIcon.setSubName(region.getPath().toString());
                                }
                                else {
                                    this.namesAndIcon.setMainName("[No  Region given]");
                                    this.namesAndIcon.setSubName("");
                                }
                            };
                            return RegionInspectionPanel;
                        })(inspect.BaseInspectionPanel);
                        _region.RegionInspectionPanel = RegionInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var EmulatorGrid = (function (_super) {
                    __extends(EmulatorGrid, _super);
                    function EmulatorGrid(dataView) {
                        _super.call(this, dataView, this.createColumns(), this.createOptions());
                    }
                    EmulatorGrid.prototype.createOptions = function () {
                        return new api.ui.grid.GridOptionsBuilder().setHideColumnHeaders(true).setRowHeight(50).setHeight(400).setWidth(320).build();
                    };
                    EmulatorGrid.prototype.createColumns = function () {
                        var _this = this;
                        return [new api.ui.grid.GridColumnBuilder().setName("device").setField("device").setId("device").setWidth(320).setCssClass("grid-row").setFormatter(function (row, cell, value, columnDef, dataContext) {
                            return _this.buildRow(row, cell, value).toString();
                        }).build()];
                    };
                    EmulatorGrid.prototype.buildRow = function (row, cell, data) {
                        var rowEl = new api.dom.DivEl();
                        rowEl.getEl().setData('width', data.width);
                        rowEl.getEl().setData('height', data.height);
                        rowEl.getEl().setData('type', data.device_type);
                        var icon = new api.dom.DivEl();
                        icon.setClass('icon-' + data.device_type);
                        icon.addClass('icon');
                        var title = new api.dom.H5El();
                        title.getEl().setInnerHtml(data.name);
                        var subtitle = new api.dom.H6El();
                        subtitle.getEl().setInnerHtml(data.width + " &times; " + data.height);
                        rowEl.appendChild(icon);
                        rowEl.appendChild(title);
                        rowEl.appendChild(subtitle);
                        if (data.rotatable == true) {
                            var rotator = new api.dom.DivEl();
                            rotator.addClass('rotate');
                            rotator.addClass('icon-loop');
                            rowEl.appendChild(rotator);
                        }
                        return rowEl;
                    };
                    EmulatorGrid.toSlickData = function (data) {
                        var result = [];
                        var i = 1;
                        data["devices"].forEach(function (item, index) {
                            var tmp = {
                                "id": i,
                                "device": item
                            };
                            result.push(tmp);
                            i++;
                        });
                        return result;
                    };
                    return EmulatorGrid;
                })(api.ui.grid.Grid);
                contextwindow.EmulatorGrid = EmulatorGrid;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var EmulatorPanel = (function (_super) {
                    __extends(EmulatorPanel, _super);
                    function EmulatorPanel(config) {
                        var _this = this;
                        _super.call(this, "emulator-panel");
                        this.liveEditPage = config.liveEditPage;
                        var text = new api.dom.PEl();
                        text.getEl().setInnerHtml("Emulate different client's physical sizes");
                        this.appendChild(text);
                        this.dataView = new api.ui.grid.DataView();
                        this.grid = new contextwindow.EmulatorGrid(this.dataView);
                        this.appendChild(this.grid);
                        this.getData();
                        wemjq(this.getHTMLElement()).on("click", ".grid-row", function (event) {
                            var width = wemjq(event.currentTarget).children('div').data("width");
                            var height = wemjq(event.currentTarget).children('div').data("height");
                            var type = wemjq(event.currentTarget).children('div').data("device.type");
                            _this.liveEditPage.setWidth(width);
                            _this.liveEditPage.setHeight(height);
                        });
                        wemjq(this.getHTMLElement()).on("click", ".rotate", function (event) {
                            event.stopPropagation();
                            _this.liveEditPage.setHeightPx(_this.liveEditPage.getHeight());
                            _this.liveEditPage.setWidthPx(_this.liveEditPage.getWidth());
                        });
                    }
                    EmulatorPanel.prototype.getData = function () {
                        var _this = this;
                        wemjq.ajax({
                            url: api.util.UriHelper.getAdminUri("apps/content-manager/js/data/context-window/devices.json"),
                            success: function (data, textStatus, jqXHR) {
                                _this.dataView.setItems(contextwindow.EmulatorGrid.toSlickData(data));
                                _this.grid.setActiveCell(0, 0);
                            }
                        });
                    };
                    return EmulatorPanel;
                })(api.ui.panel.Panel);
                contextwindow.EmulatorPanel = EmulatorPanel;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var ResponsiveManager = api.ui.responsive.ResponsiveManager;
                var ContextWindow = (function (_super) {
                    __extends(ContextWindow, _super);
                    function ContextWindow(config) {
                        var _this = this;
                        _super.call(this);
                        this.minWidth = 280;
                        this.parentMinWidth = 15;
                        this.displayModeChangedListeners = [];
                        this.liveFormPanel = config.liveFormPanel;
                        this.inspectionsPanel = config.inspectionPanel;
                        this.emulatorPanel = config.emulatorPanel;
                        this.insertablesPanel = config.insertablesPanel;
                        this.addClass("context-window");
                        this.ghostDragger = new api.dom.DivEl("ghost-dragger");
                        this.splitter = new api.dom.DivEl("splitter");
                        ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                            _this.updateFrameSize();
                        });
                        ResponsiveManager.onAvailableSizeChanged(this.liveFormPanel, function (item) {
                            _this.updateFrameSize();
                        });
                        this.appendChild(this.splitter);
                        this.addItem("Insert", this.insertablesPanel);
                        this.addItem("Inspect", this.inspectionsPanel);
                        this.addItem("Emulator", this.emulatorPanel);
                        this.onRendered(function () { return _this.initializeResizable(); });
                        this.onRemoved(function (event) {
                            ResponsiveManager.unAvailableSizeChanged(_this);
                            ResponsiveManager.unAvailableSizeChanged(_this.liveFormPanel);
                        });
                    }
                    ContextWindow.prototype.initializeResizable = function () {
                        var _this = this;
                        var initialPos = 0;
                        var splitterPosition = 0;
                        var parent = this.getParentElement();
                        this.actualWidth = this.getEl().getWidth();
                        this.mask = new api.ui.mask.DragMask(parent);
                        var dragListener = function (e) {
                            if (_this.splitterWithinBoundaries(initialPos - e.clientX)) {
                                splitterPosition = e.clientX;
                                _this.ghostDragger.getEl().setLeftPx(e.clientX - _this.getEl().getOffsetLeft());
                            }
                        };
                        this.splitter.onMouseDown(function (e) {
                            e.preventDefault();
                            initialPos = e.clientX;
                            splitterPosition = e.clientX;
                            _this.startDrag(dragListener);
                        });
                        this.mask.onMouseUp(function (e) {
                            _this.actualWidth = _this.getEl().getWidth() + initialPos - splitterPosition;
                            _this.stopDrag(dragListener);
                            ResponsiveManager.fireResizeEvent();
                        });
                        app.wizard.ShowLiveEditEvent.on(function () {
                            _this.show();
                        });
                    };
                    ContextWindow.prototype.splitterWithinBoundaries = function (offset) {
                        var newWidth = this.actualWidth + offset;
                        return (newWidth >= this.minWidth) && (newWidth <= this.getParentElement().getEl().getWidth() - this.parentMinWidth);
                    };
                    ContextWindow.prototype.startDrag = function (dragListener) {
                        this.mask.show();
                        this.mask.onMouseMove(dragListener);
                        this.ghostDragger.insertBeforeEl(this.splitter);
                        this.ghostDragger.getEl().setLeftPx(this.splitter.getEl().getOffsetLeftRelativeToParent()).setTop(null);
                    };
                    ContextWindow.prototype.stopDrag = function (dragListener) {
                        this.getEl().setWidthPx(this.actualWidth);
                        this.mask.unMouseMove(dragListener);
                        this.mask.hide();
                        this.removeChild(this.ghostDragger);
                    };
                    ContextWindow.prototype.isShown = function () {
                        return this.shown;
                    };
                    ContextWindow.prototype.slideOut = function () {
                        this.getEl().addClass("hidden").setRight(-this.getEl().getWidthWithBorder() + "px");
                        this.shown = false;
                        this.updateFrameSize();
                    };
                    ContextWindow.prototype.slideIn = function () {
                        this.getEl().removeClass("hidden").setRight("0px");
                        this.shown = true;
                        this.updateFrameSize();
                    };
                    ContextWindow.prototype.showInspectionPanel = function (panel) {
                        this.inspectionsPanel.showInspectionPanel(panel);
                        this.selectPanel(this.inspectionsPanel);
                    };
                    ContextWindow.prototype.clearSelection = function () {
                        this.inspectionsPanel.clearSelection();
                        this.selectPanel(this.insertablesPanel);
                    };
                    ContextWindow.prototype.updateFrameSize = function () {
                        var contextWindowWidth = this.actualWidth || this.getEl().getWidth(), displayModeChanged = this.hasClass('floating') && !this.isFloating();
                        this.liveFormPanel.updateFrameContainerSize(!this.isFloating() && this.shown, contextWindowWidth);
                        this.isFloating() ? this.addClass("floating") : this.removeClass("floating");
                        if (displayModeChanged) {
                            this.notifyDisplayModeChanged();
                        }
                    };
                    ContextWindow.prototype.isFloating = function () {
                        var contextWindowWidth = this.actualWidth || this.getEl().getWidth();
                        var liveFormPanelWidth = this.liveFormPanel.getEl().getWidth();
                        return (liveFormPanelWidth < 1380) || ((liveFormPanelWidth - contextWindowWidth) < 960);
                    };
                    ContextWindow.prototype.notifyDisplayModeChanged = function () {
                        this.displayModeChangedListeners.forEach(function (listener) { return listener(); });
                    };
                    ContextWindow.prototype.onDisplayModeChanged = function (listener) {
                        this.displayModeChangedListeners.push(listener);
                    };
                    ContextWindow.prototype.unDisplayModeChanged = function (listener) {
                        this.displayModeChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    return ContextWindow;
                })(api.ui.panel.DockedPanel);
                contextwindow.ContextWindow = ContextWindow;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var ContextWindowToggler = (function (_super) {
                    __extends(ContextWindowToggler, _super);
                    function ContextWindowToggler() {
                        var _this = this;
                        _super.call(this, "");
                        this.addClass("icon-menu3 icon-medium toggler");
                        this.setActive(false);
                        wizard.ShowLiveEditEvent.on(function () {
                            _this.setEnabled(true);
                        });
                        wizard.ShowSplitEditEvent.on(function () {
                            _this.setEnabled(true);
                        });
                        wizard.ShowContentFormEvent.on(function () {
                            _this.setEnabled(false);
                        });
                    }
                    return ContextWindowToggler;
                })(api.ui.button.Button);
                contextwindow.ContextWindowToggler = ContextWindowToggler;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var ContextWindowController = (function () {
                    function ContextWindowController(contextWindow, contextWindowToggler) {
                        var _this = this;
                        this.contextWindow = contextWindow;
                        this.contextWindowToggler = contextWindowToggler;
                        this.contextWindowToggler.onClicked(function (event) {
                            var active = !_this.contextWindowToggler.isActive();
                            _this.contextWindowToggler.setActive(active);
                            if (active) {
                                _this.contextWindow.slideIn();
                            }
                            else {
                                _this.contextWindow.slideOut();
                            }
                        });
                        this.contextWindow.onShown(function () {
                            if (_this.contextWindow.isFloating()) {
                                _this.contextWindow.slideOut();
                                _this.contextWindowToggler.setActive(false);
                            }
                            else {
                                _this.contextWindow.slideIn();
                                _this.contextWindowToggler.setActive(true);
                            }
                        });
                        this.contextWindow.onDisplayModeChanged(function () {
                            if (!_this.contextWindow.isFloating() && !_this.contextWindowToggler.isActive() && _this.contextWindow.isShown()) {
                                _this.contextWindow.slideOut();
                            }
                        });
                    }
                    return ContextWindowController;
                })();
                contextwindow.ContextWindowController = ContextWindowController;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var DefaultModels = (function () {
                function DefaultModels(config) {
                    if (config.pageTemplate) {
                        api.util.assert(config.pageTemplate.getType().isPageTemplate(), "given pageTemplate is not a PageTemplate: " + config.pageTemplate.getType().toString());
                    }
                    this.pageTemplate = config.pageTemplate;
                    this.pageDescriptor = config.pageDescriptor;
                    this.partDescriptor = config.partDescriptor;
                    this.layoutDescriptor = config.layoutDescriptor;
                }
                DefaultModels.prototype.hasPageTemplate = function () {
                    return !!this.pageTemplate;
                };
                DefaultModels.prototype.getPageTemplate = function () {
                    return this.pageTemplate ? this.pageTemplate.clone() : null;
                };
                DefaultModels.prototype.getPageDescriptor = function () {
                    return this.pageDescriptor;
                };
                DefaultModels.prototype.getPartDescriptor = function () {
                    return this.partDescriptor;
                };
                DefaultModels.prototype.getLayoutDescriptor = function () {
                    return this.layoutDescriptor;
                };
                return DefaultModels;
            })();
            page.DefaultModels = DefaultModels;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var GetPageDescriptorByKeyRequest = api.content.page.GetPageDescriptorByKeyRequest;
            var GetDefaultPageTemplateRequest = api.content.page.GetDefaultPageTemplateRequest;
            var DefaultPartDescriptorResolver = api.content.page.region.DefaultPartDescriptorResolver;
            var DefaultLayoutDescriptorResolver = api.content.page.region.DefaultLayoutDescriptorResolver;
            var DefaultModelsFactory = (function () {
                function DefaultModelsFactory() {
                }
                DefaultModelsFactory.create = function (config) {
                    return new GetDefaultPageTemplateRequest(config.siteId, config.contentType).sendAndParse().then(function (defaultPageTemplate) {
                        var defaultPageTemplateDescriptorPromise;
                        if (defaultPageTemplate && defaultPageTemplate.isPage()) {
                            defaultPageTemplateDescriptorPromise = new GetPageDescriptorByKeyRequest(defaultPageTemplate.getController()).sendAndParse();
                        }
                        else if (defaultPageTemplate && !defaultPageTemplate.isPage()) {
                            defaultPageTemplate = null;
                        }
                        var defaultPartDescriptorPromise = DefaultPartDescriptorResolver.resolve(config.modules);
                        var defaultLayoutDescriptorPromise = DefaultLayoutDescriptorResolver.resolve(config.modules);
                        var allPromises = [
                            defaultPageTemplateDescriptorPromise,
                            defaultPartDescriptorPromise,
                            defaultLayoutDescriptorPromise
                        ];
                        return wemQ.all(allPromises).spread(function (defaultPageTemplateDescriptor, partDescriptor, layoutDescriptor) {
                            var defaultModelsConfig = {
                                pageTemplate: defaultPageTemplate,
                                pageDescriptor: defaultPageTemplateDescriptor,
                                partDescriptor: partDescriptor,
                                layoutDescriptor: layoutDescriptor
                            };
                            return new page.DefaultModels(defaultModelsConfig);
                        });
                    });
                };
                return DefaultModelsFactory;
            })();
            page.DefaultModelsFactory = DefaultModelsFactory;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var ImageUploadDialog = api.content.form.inputtype.image.ImageUploadDialog;
            var RenderingMode = api.rendering.RenderingMode;
            var Workspace = api.content.Branch;
            var ImageOpenUploadDialogEvent = api.liveedit.ImageOpenUploadDialogEvent;
            var ImageUploadedEvent = api.liveedit.ImageUploadedEvent;
            var LiveEditPageViewReadyEvent = api.liveedit.LiveEditPageViewReadyEvent;
            var DraggingComponentViewStartedEvent = api.liveedit.DraggingComponentViewStartedEvent;
            var DraggingComponentViewCompletedEvent = api.liveedit.DraggingComponentViewCompletedEvent;
            var DraggingComponentViewCanceledEvent = api.liveedit.DraggingComponentViewCanceledEvent;
            var ItemFromContextWindowDroppedEvent = api.liveedit.ItemFromContextWindowDroppedEvent;
            var PageSelectedEvent = api.liveedit.PageSelectedEvent;
            var PageLockedEvent = api.liveedit.PageLockedEvent;
            var PageTextModeStartedEvent = api.liveedit.PageTextModeStartedEvent;
            var RegionSelectedEvent = api.liveedit.RegionSelectedEvent;
            var ItemViewSelectedEvent = api.liveedit.ItemViewSelectedEvent;
            var ItemViewDeselectEvent = api.liveedit.ItemViewDeselectEvent;
            var ComponentAddedEvent = api.liveedit.ComponentAddedEvent;
            var ComponentRemovedEvent = api.liveedit.ComponentRemovedEvent;
            var ComponentDuplicatedEvent = api.liveedit.ComponentDuplicatedEvent;
            var ComponentLoadedEvent = api.liveedit.ComponentLoadedEvent;
            var RepeatNextItemViewIdProducer = api.liveedit.RepeatNextItemViewIdProducer;
            var CreateItemViewConfig = api.liveedit.CreateItemViewConfig;
            var LiveEditPageProxy = (function () {
                function LiveEditPageProxy() {
                    var _this = this;
                    this.loadedListeners = [];
                    this.draggingComponentViewStartedListeners = [];
                    this.draggingComponentViewCompletedListeners = [];
                    this.draggingComponentViewCanceledListeners = [];
                    this.itemFromContextWindowDroppedListeners = [];
                    this.pageSelectedListeners = [];
                    this.pageLockedListeners = [];
                    this.pageTextModeStartedListeners = [];
                    this.regionSelectedListeners = [];
                    this.itemViewSelectedListeners = [];
                    this.itemViewDeselectedListeners = [];
                    this.componentAddedListeners = [];
                    this.componentRemovedListeners = [];
                    this.componentDuplicatedListeners = [];
                    this.liveEditPageViewReadyListeners = [];
                    this.LIVE_EDIT_ERROR_PAGE_BODY_ID = "wem-error-page";
                    this.liveEditIFrame = new api.dom.IFrameEl("live-edit-frame");
                    console.log("LiveEditPageProxy.constructor ");
                    this.liveEditIFrame.onLoaded(function () { return _this.handleIFrameLoadedEvent(); });
                    this.loadMask = new api.ui.mask.LoadMask(this.liveEditIFrame);
                    this.dragMask = new api.ui.mask.DragMask(this.liveEditIFrame);
                    wizard.ShowContentFormEvent.on(function () {
                        if (_this.loadMask.isVisible()) {
                            _this.loadMask.hide();
                        }
                    });
                }
                LiveEditPageProxy.prototype.setModel = function (liveEditModel) {
                    this.liveEditModel = liveEditModel;
                };
                LiveEditPageProxy.prototype.setWidth = function (value) {
                    this.liveEditIFrame.getEl().setWidth(value);
                };
                LiveEditPageProxy.prototype.setWidthPx = function (value) {
                    this.liveEditIFrame.getEl().setWidthPx(value);
                };
                LiveEditPageProxy.prototype.setHeight = function (value) {
                    this.liveEditIFrame.getEl().setHeight(value);
                };
                LiveEditPageProxy.prototype.setHeightPx = function (value) {
                    this.liveEditIFrame.getEl().setHeightPx(value);
                };
                LiveEditPageProxy.prototype.getWidth = function () {
                    return this.liveEditIFrame.getEl().getWidth();
                };
                LiveEditPageProxy.prototype.getHeight = function () {
                    return this.liveEditIFrame.getEl().getHeight();
                };
                LiveEditPageProxy.prototype.getIFrame = function () {
                    return this.liveEditIFrame;
                };
                LiveEditPageProxy.prototype.getLoadMask = function () {
                    return this.loadMask;
                };
                LiveEditPageProxy.prototype.getLiveEditJQuery = function () {
                    return this.liveEditJQuery;
                };
                LiveEditPageProxy.prototype.createDraggable = function (item) {
                    this.liveEditWindow.LiveEdit.component.dragdropsort.DragDropSort.createJQueryUiDraggable(item);
                };
                LiveEditPageProxy.prototype.showDragMask = function () {
                    this.dragMask.show();
                };
                LiveEditPageProxy.prototype.hideDragMask = function () {
                    this.dragMask.hide();
                };
                LiveEditPageProxy.prototype.remove = function () {
                    this.dragMask.remove();
                    this.loadMask.remove();
                };
                LiveEditPageProxy.prototype.load = function () {
                    this.loadMask.show();
                    var contentId = this.liveEditModel.getContent().getContentId().toString();
                    var pageUrl = api.rendering.UriHelper.getPortalUri(contentId, 0 /* EDIT */, 0 /* DRAFT */);
                    console.log("LiveEditPageProxy.load pageUrl: " + pageUrl);
                    this.liveEditIFrame.setSrc(pageUrl);
                };
                LiveEditPageProxy.prototype.handleIFrameLoadedEvent = function () {
                    var liveEditWindow = this.liveEditIFrame.getHTMLElement()["contentWindow"];
                    if (liveEditWindow && liveEditWindow.wemjq) {
                        liveEditWindow.CONFIG = { baseUri: CONFIG.baseUri };
                        this.liveEditJQuery = liveEditWindow.wemjq;
                        if (this.liveEditIFrame != liveEditWindow) {
                            this.liveEditWindow = liveEditWindow;
                            this.listenToPage();
                        }
                        this.loadMask.hide();
                        new api.liveedit.InitializeLiveEditEvent(this.liveEditModel).fire(this.liveEditWindow);
                    }
                    else if (liveEditWindow.document.body.id == this.LIVE_EDIT_ERROR_PAGE_BODY_ID) {
                        this.loadMask.hide();
                    }
                    this.notifyLoaded();
                };
                LiveEditPageProxy.prototype.loadComponent = function (componentView, componentUrl) {
                    var _this = this;
                    var deferred = wemQ.defer();
                    api.util.assertNotNull(componentView, "componentView cannot be null");
                    api.util.assertNotNull(componentUrl, "componentUrl cannot be null");
                    wemjq.ajax({
                        url: componentUrl,
                        type: 'GET',
                        success: function (htmlAsString) {
                            var newElement = api.dom.Element.fromString(htmlAsString);
                            var repeatNextItemViewIdProducer = new RepeatNextItemViewIdProducer(componentView.getItemId(), componentView.getItemViewIdProducer());
                            var createViewConfig = new CreateItemViewConfig().setItemViewProducer(repeatNextItemViewIdProducer).setParentView(componentView.getParentItemView()).setData(componentView.getComponent()).setElement(newElement);
                            var newComponentView = componentView.getType().createView(createViewConfig);
                            componentView.replaceWith(newComponentView);
                            new ComponentLoadedEvent(newComponentView).fire(_this.liveEditWindow);
                            newComponentView.select();
                            deferred.resolve("");
                        },
                        error: function (jqXHR, textStatus, errorThrow) {
                            var responseHtml = wemjq.parseHTML(jqXHR.responseText);
                            var errorMessage = "";
                            responseHtml.forEach(function (el, i) {
                                if (el.tagName && el.tagName.toLowerCase() == "title") {
                                    errorMessage = el.innerHTML;
                                }
                            });
                            deferred.reject(errorMessage);
                        }
                    });
                    return deferred.promise;
                };
                LiveEditPageProxy.prototype.listenToPage = function () {
                    var _this = this;
                    ImageOpenUploadDialogEvent.on(function (openDialogEvent) {
                        var imageUploadDialog = new ImageUploadDialog(_this.liveEditModel.getContent().getContentId());
                        imageUploadDialog.onImageUploaded(function (event) {
                            new ImageUploadedEvent(event.getUploadItem().getModel(), openDialogEvent.getTargetImagePlaceholder()).fire(_this.liveEditWindow);
                            imageUploadDialog.close();
                            imageUploadDialog.remove();
                        });
                        imageUploadDialog.open();
                    }, this.liveEditWindow);
                    DraggingComponentViewStartedEvent.on(this.notifyDraggingComponentViewStarted.bind(this), this.liveEditWindow);
                    DraggingComponentViewCompletedEvent.on(this.notifyDraggingComponentViewCompleted.bind(this), this.liveEditWindow);
                    DraggingComponentViewCanceledEvent.on(this.notifyDraggingComponentViewCanceled.bind(this), this.liveEditWindow);
                    ItemFromContextWindowDroppedEvent.on(this.notifyItemFromContextWindowDropped.bind(this), this.liveEditWindow);
                    PageSelectedEvent.on(this.notifyPageSelected.bind(this), this.liveEditWindow);
                    PageLockedEvent.on(this.notifyPageLocked.bind(this), this.liveEditWindow);
                    PageTextModeStartedEvent.on(this.notifyPageTextModeStarted.bind(this), this.liveEditWindow);
                    RegionSelectedEvent.on(this.notifyRegionSelected.bind(this), this.liveEditWindow);
                    ItemViewSelectedEvent.on(this.notifyItemViewSelected.bind(this), this.liveEditWindow);
                    ItemViewDeselectEvent.on(this.notifyItemViewDeselected.bind(this), this.liveEditWindow);
                    ComponentAddedEvent.on(this.notifyComponentAdded.bind(this), this.liveEditWindow);
                    ComponentRemovedEvent.on(this.notifyComponentRemoved.bind(this), this.liveEditWindow);
                    ComponentDuplicatedEvent.on(this.notifyComponentDuplicated.bind(this), this.liveEditWindow);
                    LiveEditPageViewReadyEvent.on(this.notifyLiveEditPageViewReady.bind(this), this.liveEditWindow);
                };
                LiveEditPageProxy.prototype.onLoaded = function (listener) {
                    this.loadedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unLoaded = function (listener) {
                    this.loadedListeners = this.loadedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                LiveEditPageProxy.prototype.notifyLoaded = function () {
                    this.loadedListeners.forEach(function (listener) {
                        listener();
                    });
                };
                LiveEditPageProxy.prototype.onDraggingComponentViewStartedEvent = function (listener) {
                    this.draggingComponentViewStartedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unDraggingComponentViewStartedEvent = function (listener) {
                    this.draggingComponentViewStartedListeners = this.draggingComponentViewStartedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyDraggingComponentViewStarted = function (event) {
                    this.draggingComponentViewStartedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onDraggingComponentViewCompleted = function (listener) {
                    this.draggingComponentViewCompletedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unDraggingComponentViewCompleted = function (listener) {
                    this.draggingComponentViewCompletedListeners = this.draggingComponentViewCompletedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyDraggingComponentViewCompleted = function (event) {
                    this.draggingComponentViewCompletedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onDraggingComponentViewCanceled = function (listener) {
                    this.draggingComponentViewCanceledListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unDraggingComponentViewCanceled = function (listener) {
                    this.draggingComponentViewCanceledListeners = this.draggingComponentViewCanceledListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyDraggingComponentViewCanceled = function (event) {
                    this.draggingComponentViewCanceledListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onItemFromContextWindowDropped = function (listener) {
                    this.itemFromContextWindowDroppedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unItemFromContextWindowDropped = function (listener) {
                    this.itemFromContextWindowDroppedListeners = this.itemFromContextWindowDroppedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyItemFromContextWindowDropped = function (event) {
                    this.itemFromContextWindowDroppedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageSelected = function (listener) {
                    this.pageSelectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageSelected = function (listener) {
                    this.pageSelectedListeners = this.pageSelectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageSelected = function (event) {
                    this.pageSelectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageLocked = function (listener) {
                    this.pageLockedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageLocked = function (listener) {
                    this.pageLockedListeners = this.pageLockedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageLocked = function (event) {
                    this.pageLockedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageTextModeStarted = function (listener) {
                    this.pageTextModeStartedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageTextModeStarted = function (listener) {
                    this.pageTextModeStartedListeners = this.pageTextModeStartedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageTextModeStarted = function (event) {
                    this.pageTextModeStartedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onRegionSelected = function (listener) {
                    this.regionSelectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unRegionSelected = function (listener) {
                    this.regionSelectedListeners = this.regionSelectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyRegionSelected = function (event) {
                    this.regionSelectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onItemViewSelected = function (listener) {
                    this.itemViewSelectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unItemViewSelected = function (listener) {
                    this.itemViewSelectedListeners = this.itemViewSelectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyItemViewSelected = function (event) {
                    this.itemViewSelectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onItemViewDeselected = function (listener) {
                    this.itemViewDeselectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unItemViewDeselected = function (listener) {
                    this.itemViewDeselectedListeners = this.itemViewDeselectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyItemViewDeselected = function (event) {
                    this.itemViewDeselectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentAdded = function (listener) {
                    this.componentAddedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentAdded = function (listener) {
                    this.componentAddedListeners = this.componentAddedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentAdded = function (event) {
                    this.componentAddedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentRemoved = function (listener) {
                    this.componentRemovedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentRemoved = function (listener) {
                    this.componentRemovedListeners = this.componentRemovedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentRemoved = function (event) {
                    this.componentRemovedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentDuplicated = function (listener) {
                    this.componentDuplicatedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentDuplicated = function (listener) {
                    this.componentDuplicatedListeners = this.componentDuplicatedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentDuplicated = function (event) {
                    this.componentDuplicatedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onLiveEditPageViewReady = function (listener) {
                    this.liveEditPageViewReadyListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unLiveEditPageViewReady = function (listener) {
                    this.liveEditPageViewReadyListeners = this.liveEditPageViewReadyListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyLiveEditPageViewReady = function (event) {
                    this.liveEditPageViewReadyListeners.forEach(function (listener) { return listener(event); });
                };
                return LiveEditPageProxy;
            })();
            page.LiveEditPageProxy = LiveEditPageProxy;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var PageMode = api.content.page.PageMode;
            var ImageComponent = api.content.page.region.ImageComponent;
            var DescriptorBasedComponent = api.content.page.region.DescriptorBasedComponent;
            var InspectionsPanel = app.wizard.page.contextwindow.inspect.InspectionsPanel;
            var ContentInspectionPanel = app.wizard.page.contextwindow.inspect.ContentInspectionPanel;
            var PageInspectionPanel = app.wizard.page.contextwindow.inspect.page.PageInspectionPanel;
            var RegionInspectionPanel = app.wizard.page.contextwindow.inspect.region.RegionInspectionPanel;
            var ImageInspectionPanel = app.wizard.page.contextwindow.inspect.region.ImageInspectionPanel;
            var PartInspectionPanel = app.wizard.page.contextwindow.inspect.region.PartInspectionPanel;
            var LayoutInspectionPanel = app.wizard.page.contextwindow.inspect.region.LayoutInspectionPanel;
            var ContextWindow = app.wizard.page.contextwindow.ContextWindow;
            var EmulatorPanel = app.wizard.page.contextwindow.EmulatorPanel;
            var InsertablesPanel = app.wizard.page.contextwindow.insert.InsertablesPanel;
            var RenderingMode = api.rendering.RenderingMode;
            var ComponentView = api.liveedit.ComponentView;
            var PageView = api.liveedit.PageView;
            var ImageComponentView = api.liveedit.image.ImageComponentView;
            var PartComponentView = api.liveedit.part.PartComponentView;
            var LayoutComponentView = api.liveedit.layout.LayoutComponentView;
            var TextComponentView = api.liveedit.text.TextComponentView;
            var Panel = api.ui.panel.Panel;
            var LiveFormPanel = (function (_super) {
                __extends(LiveFormPanel, _super);
                function LiveFormPanel(config) {
                    var _this = this;
                    _super.call(this, "live-form-panel");
                    this.contentWizardPanel = config.contentWizardPanel;
                    this.defaultModels = config.defaultModels;
                    this.pageLoading = false;
                    this.pageSkipReload = false;
                    this.liveEditPageProxy = new page.LiveEditPageProxy();
                    this.contentInspectionPanel = new ContentInspectionPanel();
                    this.pageInspectionPanel = new PageInspectionPanel();
                    this.regionInspectionPanel = new RegionInspectionPanel();
                    this.imageInspectionPanel = new ImageInspectionPanel();
                    this.partInspectionPanel = new PartInspectionPanel();
                    this.layoutInspectionPanel = new LayoutInspectionPanel();
                    var saveAction = new api.ui.Action('Apply');
                    saveAction.onExecuted(function () {
                        var itemView = _this.pageView.getSelectedView();
                        if (itemView) {
                            if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, ComponentView)) {
                                _this.saveAndReloadOnlyComponent(itemView);
                            }
                            else if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, PageView)) {
                                _this.saveAndReloadPage();
                            }
                        }
                    });
                    this.inspectionsPanel = new InspectionsPanel({
                        contentInspectionPanel: this.contentInspectionPanel,
                        pageInspectionPanel: this.pageInspectionPanel,
                        regionInspectionPanel: this.regionInspectionPanel,
                        imageInspectionPanel: this.imageInspectionPanel,
                        partInspectionPanel: this.partInspectionPanel,
                        layoutInspectionPanel: this.layoutInspectionPanel,
                        saveAction: saveAction
                    });
                    this.emulatorPanel = new EmulatorPanel({
                        liveEditPage: this.liveEditPageProxy
                    });
                    this.insertablesPanel = new InsertablesPanel({
                        liveEditPage: this.liveEditPageProxy
                    });
                    this.frameContainer = new Panel("frame-container");
                    this.appendChild(this.frameContainer);
                    this.frameContainer.appendChild(this.liveEditPageProxy.getIFrame());
                    this.appendChild(this.liveEditPageProxy.getLoadMask());
                    this.contextWindow = new ContextWindow({
                        liveFormPanel: this,
                        inspectionPanel: this.inspectionsPanel,
                        emulatorPanel: this.emulatorPanel,
                        insertablesPanel: this.insertablesPanel
                    });
                    this.appendChild(this.contextWindow);
                    this.contextWindowController = new app.wizard.page.contextwindow.ContextWindowController(this.contextWindow, this.contentWizardPanel.getContextWindowToggler());
                    this.liveEditListen();
                }
                LiveFormPanel.prototype.remove = function () {
                    this.liveEditPageProxy.remove();
                    _super.prototype.remove.call(this);
                    return this;
                };
                LiveFormPanel.prototype.getPage = function () {
                    return this.pageModel.getPage();
                };
                LiveFormPanel.prototype.setModel = function (liveEditModel) {
                    var _this = this;
                    this.liveEditModel = liveEditModel;
                    this.content = liveEditModel.getContent();
                    this.pageModel = liveEditModel.getPageModel();
                    this.pageModel.setIgnorePropertyChanges(true);
                    this.liveEditPageProxy.setModel(liveEditModel);
                    this.imageInspectionPanel.setModel(liveEditModel);
                    this.pageInspectionPanel.setModel(liveEditModel);
                    this.partInspectionPanel.setModel(liveEditModel);
                    this.layoutInspectionPanel.setModel(liveEditModel);
                    this.imageInspectionPanel.setModel(liveEditModel);
                    this.pageModel.setIgnorePropertyChanges(false);
                    this.pageModel.onPropertyChanged(function (event) {
                        if (event.getPropertyName() == "controller" && _this !== event.getSource()) {
                            _this.saveAndReloadPage();
                        }
                        else if (event.getPropertyName() == "template" && _this !== event.getSource()) {
                            if (_this.pageModel.getMode() == 0 /* AUTOMATIC */) {
                                _this.saveAndReloadPage();
                            }
                            else {
                            }
                        }
                    });
                    this.pageModel.onComponentPropertyChangedEvent(function (event) {
                        if (api.ObjectHelper.iFrameSafeInstanceOf(event.getComponent(), DescriptorBasedComponent)) {
                            if (event.getPropertyName() == DescriptorBasedComponent.PROPERTY_DESCRIPTOR) {
                                var componentView = _this.pageView.getComponentViewByPath(event.getPath());
                                if (componentView) {
                                    if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, PartComponentView)) {
                                        var partView = componentView;
                                        var partComponent = partView.getComponent();
                                        if (partComponent.hasDescriptor()) {
                                            _this.saveAndReloadOnlyComponent(componentView);
                                        }
                                    }
                                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, LayoutComponentView)) {
                                        var layoutView = componentView;
                                        var layoutComponent = layoutView.getComponent();
                                        if (layoutComponent.hasDescriptor()) {
                                            _this.saveAndReloadOnlyComponent(componentView);
                                        }
                                    }
                                }
                                else {
                                    console.debug("ComponentView by path not found: " + event.getPath().toString());
                                }
                            }
                        }
                        else if (api.ObjectHelper.iFrameSafeInstanceOf(event.getComponent(), ImageComponent)) {
                            if (event.getPropertyName() == ImageComponent.PROPERTY_IMAGE && !event.getComponent().isEmpty()) {
                                var componentView = _this.pageView.getComponentViewByPath(event.getPath());
                                if (componentView) {
                                    _this.saveAndReloadOnlyComponent(componentView);
                                }
                            }
                        }
                    });
                };
                LiveFormPanel.prototype.loadPage = function () {
                    var _this = this;
                    if (this.pageSkipReload == false && !this.pageLoading) {
                        this.contextWindow.clearSelection();
                        this.pageLoading = true;
                        this.liveEditPageProxy.load();
                        this.liveEditPageProxy.onLoaded(function () {
                            _this.pageLoading = false;
                        });
                    }
                };
                LiveFormPanel.prototype.saveAndReloadPage = function () {
                    var _this = this;
                    this.pageSkipReload = true;
                    this.contentWizardPanel.saveChanges().then(function () {
                        _this.pageSkipReload = false;
                        _this.liveEditPageProxy.load();
                    }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                };
                LiveFormPanel.prototype.saveAndReloadOnlyComponent = function (componentView) {
                    var _this = this;
                    api.util.assertNotNull(componentView, "componentView cannot be null");
                    this.pageSkipReload = true;
                    var componentUrl = api.rendering.UriHelper.getComponentUri(this.content.getContentId().toString(), componentView.getComponentPath().toString(), 0 /* EDIT */, 0 /* DRAFT */);
                    this.contentWizardPanel.saveChanges().then(function () {
                        _this.pageSkipReload = false;
                        componentView.showLoadingSpinner();
                        return _this.liveEditPageProxy.loadComponent(componentView, componentUrl);
                    }).catch(function (errorMessage) {
                        api.DefaultErrorHandler.handle(errorMessage);
                        componentView.hideLoadingSpinner();
                        componentView.showRenderingError(componentUrl, errorMessage);
                    }).done();
                };
                LiveFormPanel.prototype.updateFrameContainerSize = function (contextWindowShown, contextWindowWidth) {
                    if (contextWindowShown && contextWindowWidth) {
                        this.frameContainer.getEl().setWidth("calc(100% - " + (contextWindowWidth - 1) + "px)");
                    }
                    else {
                        this.frameContainer.getEl().setWidth("100%");
                    }
                };
                LiveFormPanel.prototype.liveEditListen = function () {
                    var _this = this;
                    this.liveEditPageProxy.onPageLocked(function (event) {
                        _this.inspectPage();
                    });
                    this.liveEditPageProxy.onPageTextModeStarted(function (event) {
                        if (_this.contextWindow.isFloating() && _this.contextWindow.isShown()) {
                            _this.contextWindow.slideOut();
                        }
                    });
                    this.liveEditPageProxy.onLiveEditPageViewReady(function (event) {
                        _this.pageView = event.getPageView();
                    });
                    this.liveEditPageProxy.onPageSelected(function (event) {
                        _this.inspectPage();
                    });
                    this.liveEditPageProxy.onRegionSelected(function (event) {
                        _this.inspectRegion(event.getRegionView());
                    });
                    this.liveEditPageProxy.onItemViewSelected(function (event) {
                        var itemView = event.getItemView();
                        if (itemView.isEmpty() || api.ObjectHelper.iFrameSafeInstanceOf(itemView, TextComponentView)) {
                            if (_this.contextWindow.isFloating() && _this.contextWindow.isShown()) {
                                _this.contextWindow.slideOut();
                            }
                        }
                        else {
                            if (_this.contextWindow.isFloating() && !_this.contextWindow.isShown()) {
                                _this.contextWindow.slideIn();
                            }
                        }
                        if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, ComponentView)) {
                            _this.inspectComponent(itemView);
                        }
                    });
                    this.liveEditPageProxy.onItemViewDeselected(function (event) {
                        var toggler = _this.contentWizardPanel.getContextWindowToggler();
                        if (!toggler.isActive() && _this.contextWindow.isShown()) {
                            _this.contextWindow.slideOut();
                        }
                        else if (toggler.isActive() && !_this.contextWindow.isShown()) {
                            _this.contextWindow.slideIn();
                        }
                        _this.contextWindow.clearSelection();
                    });
                    this.liveEditPageProxy.onComponentAdded(function (event) {
                    });
                    this.liveEditPageProxy.onComponentRemoved(function (event) {
                        var toggler = _this.contentWizardPanel.getContextWindowToggler();
                        if ((_this.contextWindow.isFloating() || toggler.isActive()) && !_this.contextWindow.isShown()) {
                            _this.contextWindow.slideIn();
                        }
                        if (!_this.pageModel.isPageTemplate() && _this.pageModel.getMode() == 0 /* AUTOMATIC */) {
                            _this.pageModel.initializePageFromDefault(_this);
                        }
                        _this.contextWindow.clearSelection();
                    });
                    this.liveEditPageProxy.onDraggingComponentViewStartedEvent(function (event) {
                        if (_this.contextWindow.isFloating() && _this.contextWindow.isShown()) {
                            _this.contextWindow.slideOut();
                        }
                    });
                    this.liveEditPageProxy.onDraggingComponentViewCompleted(function (event) {
                        var componentView = event.getComponentView();
                        if (!componentView.isEmpty()) {
                            var toggler = _this.contentWizardPanel.getContextWindowToggler();
                            if (_this.contextWindow.isFloating() && !_this.contextWindow.isShown() && toggler.isActive()) {
                                _this.contextWindow.slideIn();
                            }
                            _this.inspectComponent(componentView);
                        }
                    });
                    this.liveEditPageProxy.onDraggingComponentViewCanceled(function (event) {
                        var toggler = _this.contentWizardPanel.getContextWindowToggler();
                        if (_this.contextWindow.isFloating() && !_this.contextWindow.isShown() && toggler.isActive()) {
                            _this.contextWindow.slideIn();
                        }
                    });
                    this.liveEditPageProxy.onComponentDuplicated(function (event) {
                        _this.saveAndReloadOnlyComponent(event.getDuplicatedComponentView());
                    });
                    this.insertablesPanel.onHideContextWindowRequest(function () {
                        if (_this.contextWindow.isFloating() && _this.contextWindow.isShown()) {
                            _this.contextWindow.slideOut();
                        }
                    });
                };
                LiveFormPanel.prototype.inspectContent = function (contentId) {
                    this.contextWindow.showInspectionPanel(this.contentInspectionPanel);
                };
                LiveFormPanel.prototype.inspectPage = function () {
                    this.contextWindow.showInspectionPanel(this.pageInspectionPanel);
                };
                LiveFormPanel.prototype.inspectRegion = function (regionView) {
                    var region = regionView.getRegion();
                    this.regionInspectionPanel.setRegion(region);
                    this.contextWindow.showInspectionPanel(this.regionInspectionPanel);
                };
                LiveFormPanel.prototype.inspectComponent = function (componentView) {
                    api.util.assertNotNull(componentView, "componentView cannot be null");
                    if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, ImageComponentView)) {
                        this.imageInspectionPanel.setImageComponent(componentView);
                        this.contextWindow.showInspectionPanel(this.imageInspectionPanel);
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, PartComponentView)) {
                        this.partInspectionPanel.setPartComponent(componentView);
                        this.contextWindow.showInspectionPanel(this.partInspectionPanel);
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, LayoutComponentView)) {
                        this.layoutInspectionPanel.setLayoutComponent(componentView);
                        this.contextWindow.showInspectionPanel(this.layoutInspectionPanel);
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, TextComponentView)) {
                    }
                    else {
                        throw new Error("ComponentView cannot be selected: " + api.ClassHelper.getClassName(componentView));
                    }
                };
                return LiveFormPanel;
            })(api.ui.panel.Panel);
            page.LiveFormPanel = LiveFormPanel;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ContentSettingsModel = (function () {
            function ContentSettingsModel(content) {
                this.propertyChangedListeners = [];
                this.language = content.getLanguage();
                this.owner = content.getOwner();
            }
            ContentSettingsModel.prototype.getOwner = function () {
                return this.owner;
            };
            ContentSettingsModel.prototype.setOwner = function (owner, silent) {
                if (!silent) {
                    this.notifyPropertyChanged(new api.PropertyChangedEvent(ContentSettingsModel.PROPERTY_OWNER, this.owner, owner));
                }
                this.owner = owner;
                return this;
            };
            ContentSettingsModel.prototype.getLanguage = function () {
                return this.language;
            };
            ContentSettingsModel.prototype.setLanguage = function (lang, silent) {
                if (!silent) {
                    this.notifyPropertyChanged(new api.PropertyChangedEvent(ContentSettingsModel.PROPERTY_LANG, this.language, lang));
                }
                this.language = lang;
                return this;
            };
            ContentSettingsModel.prototype.onPropertyChanged = function (listener) {
                this.propertyChangedListeners.push(listener);
            };
            ContentSettingsModel.prototype.unPropertyChanged = function (listener) {
                this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            ContentSettingsModel.prototype.notifyPropertyChanged = function (event) {
                this.propertyChangedListeners.forEach(function (listener) { return listener(event); });
            };
            ContentSettingsModel.prototype.equals = function (other) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(other, ContentSettingsModel)) {
                    return false;
                }
                else {
                    var otherModel = other;
                    return otherModel.owner == this.owner && otherModel.language == this.language;
                }
            };
            ContentSettingsModel.prototype.apply = function (builder) {
                builder.owner = this.owner;
                builder.language = this.language;
            };
            ContentSettingsModel.PROPERTY_OWNER = 'owner';
            ContentSettingsModel.PROPERTY_LANG = 'language';
            return ContentSettingsModel;
        })();
        wizard.ContentSettingsModel = ContentSettingsModel;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var FormView = api.form.FormView;
        var WizardStepValidityChangedEvent = api.app.wizard.WizardStepValidityChangedEvent;
        var ContentWizardStepForm = (function (_super) {
            __extends(ContentWizardStepForm, _super);
            function ContentWizardStepForm() {
                _super.call(this);
            }
            ContentWizardStepForm.prototype.layout = function (formContext, data, form) {
                var _this = this;
                this.formContext = formContext;
                this.form = form;
                this.data = data;
                return this.doLayout(form, data).then(function () {
                    if (form.getFormItems().length === 0) {
                        _this.hide();
                    }
                });
            };
            ContentWizardStepForm.prototype.doLayout = function (form, data) {
                var _this = this;
                this.formView = new FormView(this.formContext, form, data.getRoot());
                return this.formView.layout().then(function () {
                    _this.formView.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    _this.formView.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    _this.formView.onEditContentRequest(function (content) {
                        new app.browse.EditContentEvent([content]).fire();
                    });
                    _this.appendChild(_this.formView);
                    _this.formView.onValidityChanged(function (event) {
                        _this.notifyValidityChanged(new WizardStepValidityChangedEvent(event.isValid()));
                    });
                    var formViewValid = _this.formView.isValid();
                    _this.notifyValidityChanged(new WizardStepValidityChangedEvent(formViewValid));
                });
            };
            ContentWizardStepForm.prototype.validate = function (silent) {
                return this.formView.validate(silent);
            };
            ContentWizardStepForm.prototype.displayValidationErrors = function (display) {
                this.formView.displayValidationErrors(display);
            };
            ContentWizardStepForm.prototype.getForm = function () {
                return this.form;
            };
            ContentWizardStepForm.prototype.getFormView = function () {
                return this.formView;
            };
            ContentWizardStepForm.prototype.getData = function () {
                return this.data;
            };
            ContentWizardStepForm.prototype.giveFocus = function () {
                return this.formView.giveFocus();
            };
            return ContentWizardStepForm;
        })(api.app.wizard.WizardStepForm);
        wizard.ContentWizardStepForm = ContentWizardStepForm;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PrincipalType = api.security.PrincipalType;
        var PrincipalLoader = api.security.PrincipalLoader;
        var FormItemBuilder = api.ui.form.FormItemBuilder;
        var PrincipalComboBox = api.ui.security.PrincipalComboBox;
        var LocaleComboBox = api.ui.locale.LocaleComboBox;
        var SettingsWizardStepForm = (function (_super) {
            __extends(SettingsWizardStepForm, _super);
            function SettingsWizardStepForm() {
                var _this = this;
                _super.call(this, "settings-wizard-step-form");
                this.localeCombo = new LocaleComboBox(1);
                var localeFormItem = new FormItemBuilder(this.localeCombo).setLabel('Language').build();
                var loader = new PrincipalLoader().setAllowedTypes([0 /* USER */]);
                this.ownerCombo = new PrincipalComboBox(loader, 1);
                var ownerFormItem = new FormItemBuilder(this.ownerCombo).setLabel('Owner').build();
                var fieldSet = new api.ui.form.Fieldset();
                fieldSet.add(localeFormItem);
                fieldSet.add(ownerFormItem);
                var form = new api.ui.form.Form().add(fieldSet);
                this.appendChild(form);
                form.onFocus(function (event) {
                    _this.notifyFocused(event);
                });
                form.onBlur(function (event) {
                    _this.notifyBlurred(event);
                });
            }
            SettingsWizardStepForm.prototype.layout = function (content) {
                this.content = content;
            };
            SettingsWizardStepForm.prototype.setModel = function (model) {
                var _this = this;
                api.util.assertNotNull(model, "Model can't be null");
                if (model.getOwner()) {
                    this.ownerCombo.setIgnoreNextFocus().setValue(model.getOwner().toString());
                }
                if (model.getLanguage()) {
                    this.localeCombo.setIgnoreNextFocus().setValue(model.getLanguage());
                }
                var ownerListener = function () {
                    var principals = _this.ownerCombo.getSelectedDisplayValues();
                    model.setOwner(principals.length > 0 ? principals[0].getKey() : null, true);
                };
                this.ownerCombo.onOptionSelected(function (event) { return ownerListener(); });
                this.ownerCombo.onOptionDeselected(function (option) { return ownerListener(); });
                var localeListener = function () {
                    model.setLanguage(_this.localeCombo.getValue(), true);
                };
                this.localeCombo.onOptionSelected(function (event) { return localeListener(); });
                this.localeCombo.onOptionDeselected(function (option) { return localeListener(); });
                model.onPropertyChanged(function (event) {
                    switch (event.getPropertyName()) {
                        case wizard.ContentSettingsModel.PROPERTY_LANG:
                            _this.localeCombo.setValue(event.getNewValue());
                            break;
                        case wizard.ContentSettingsModel.PROPERTY_OWNER:
                            _this.ownerCombo.setValue(event.getNewValue());
                            break;
                    }
                });
                this.model = model;
            };
            SettingsWizardStepForm.prototype.getModel = function () {
                return this.model;
            };
            SettingsWizardStepForm.prototype.giveFocus = function () {
                return this.ownerCombo.giveFocus();
            };
            return SettingsWizardStepForm;
        })(api.app.wizard.WizardStepForm);
        wizard.SettingsWizardStepForm = SettingsWizardStepForm;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var AccessControlListView = api.ui.security.acl.AccessControlListView;
        var DivEl = api.dom.DivEl;
        var LabelEl = api.dom.LabelEl;
        var Button = api.ui.button.Button;
        var SecurityWizardStepForm = (function (_super) {
            __extends(SecurityWizardStepForm, _super);
            function SecurityWizardStepForm() {
                var _this = this;
                _super.call(this, "security-wizard-step-form");
                var label = new DivEl("input-label"), wrapper = new DivEl("wrapper required");
                this.label = new LabelEl("Permissions");
                wrapper.appendChild(this.label);
                label.appendChild(wrapper);
                this.inheritance = new DivEl();
                this.accessListView = new AccessControlListView();
                this.accessListView.setItemsEditable(false);
                this.editLink = new Button("Edit Permissions");
                this.editLink.addClass("edit-permissions");
                this.editLink.onFocus(function (event) {
                    _this.notifyFocused(event);
                });
                this.editLink.onBlur(function (event) {
                    _this.notifyBlurred(event);
                });
                var formView = new DivEl("form-view"), inputView = new DivEl("input-view valid"), inputTypeView = new DivEl("input-type-view"), inputOccurrenceView = new DivEl("input-occurrence-view single-occurrence"), inputWrapper = new DivEl("input-wrapper");
                inputWrapper.appendChildren(this.inheritance, this.accessListView, this.editLink);
                inputOccurrenceView.appendChild(inputWrapper);
                inputTypeView.appendChild(inputOccurrenceView);
                inputView.appendChildren(label, inputTypeView);
                formView.appendChild(inputView);
                this.appendChild(formView);
                this.editLink.onClicked(function () {
                    if (!!_this.content) {
                        new wizard.OpenEditPermissionsDialogEvent(_this.content).fire();
                    }
                });
                wizard.ContentPermissionsAppliedEvent.on(function (event) {
                    var content = event.getContent();
                    if (content.getId() === _this.content.getId()) {
                        _this.layout(content);
                    }
                });
            }
            SecurityWizardStepForm.prototype.layout = function (content) {
                var _this = this;
                this.accessListView.clearItems();
                content.getPermissions().getEntries().sort().forEach(function (entry) {
                    _this.accessListView.addItem(entry);
                    var entryView = _this.accessListView.getItemView(entry), selector = entryView.getPermissionSelector();
                    entryView.getValueChangedListeners().splice(0);
                    entryView.getPermissionSelector().hide();
                    entryView.onClicked(function () {
                        var isDisplayed = selector.getEl().getDisplay() !== "block";
                        _this.accessListView.getItemViews().forEach(function (itemView) {
                            itemView.getPermissionSelector().hide();
                        });
                        if (isDisplayed) {
                            selector.show();
                        }
                    });
                });
                var inheritsText = "";
                if (content.isInheritPermissionsEnabled() && content.isRoot() == false) {
                    inheritsText = "Inherits permissions from parent";
                    this.inheritance.addClass("inheritance");
                }
                else {
                    this.inheritance.removeClass("inheritance");
                }
                this.inheritance.setHtml(inheritsText);
                this.content = content;
            };
            SecurityWizardStepForm.prototype.giveFocus = function () {
                return this.accessListView.giveFocus();
            };
            return SecurityWizardStepForm;
        })(api.app.wizard.WizardStepForm);
        wizard.SecurityWizardStepForm = SecurityWizardStepForm;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var CycleButton = api.ui.button.CycleButton;
        var ContextWindowToggler = app.wizard.page.contextwindow.ContextWindowToggler;
        var ContentWizardToolbar = (function (_super) {
            __extends(ContentWizardToolbar, _super);
            function ContentWizardToolbar(params) {
                _super.call(this);
                _super.prototype.addAction.call(this, params.saveAction);
                _super.prototype.addAction.call(this, params.duplicateAction);
                _super.prototype.addAction.call(this, params.deleteAction);
                _super.prototype.addAction.call(this, params.publishAction);
                _super.prototype.addAction.call(this, params.previewAction);
                _super.prototype.addGreedySpacer.call(this);
                this.cycleViewModeButton = new CycleButton(params.showSplitEditAction, params.showLiveEditAction, params.showFormAction);
                _super.prototype.addElement.call(this, this.cycleViewModeButton);
                this.contextWindowToggler = new ContextWindowToggler();
                _super.prototype.addElement.call(this, this.contextWindowToggler);
            }
            ContentWizardToolbar.prototype.getCycleViewModeButton = function () {
                return this.cycleViewModeButton;
            };
            ContentWizardToolbar.prototype.getContextWindowToggler = function () {
                return this.contextWindowToggler;
            };
            return ContentWizardToolbar;
        })(api.ui.toolbar.Toolbar);
        wizard.ContentWizardToolbar = ContentWizardToolbar;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var DisplayNameScriptExecutor = (function () {
            function DisplayNameScriptExecutor() {
            }
            DisplayNameScriptExecutor.prototype.setFormView = function (value) {
                this.formView = value;
                return this;
            };
            DisplayNameScriptExecutor.prototype.setScript = function (value) {
                this.script = value;
                return this;
            };
            DisplayNameScriptExecutor.prototype.hasScript = function () {
                return !api.util.StringHelper.isBlank(this.script);
            };
            DisplayNameScriptExecutor.prototype.execute = function () {
                api.util.assertNotNull(this.formView, "formView not set");
                api.util.assertNotNull(this.script, "script not set");
                return this.safeEval(this.script, this.formView);
            };
            DisplayNameScriptExecutor.prototype.safeEval = function (script, formView) {
                function $() {
                    var paths = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        paths[_i - 0] = arguments[_i];
                    }
                    var strValues = [];
                    paths.forEach(function (path) {
                        var strValue = formView.getData().getString(path);
                        if (!api.util.StringHelper.isBlank(strValue)) {
                            strValues.push(strValue);
                        }
                    });
                    var strValue = "";
                    strValues.forEach(function (s, index) {
                        strValue += s;
                        if (index < strValues.length - 1) {
                            strValue += " ";
                        }
                    });
                    return strValue;
                }
                var result = '';
                try {
                    result = eval('var eval; var Function; var document; var location; ' + 'var window; var parent; var self; var top; ' + script);
                }
                catch (e) {
                    console.error('Cannot evaluate script [' + script + '].', e);
                }
                return result;
            };
            return DisplayNameScriptExecutor;
        })();
        wizard.DisplayNameScriptExecutor = DisplayNameScriptExecutor;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var PropertyTree = api.data.PropertyTree;
        var FormView = api.form.FormView;
        var ContentFormContext = api.content.form.ContentFormContext;
        var CompareStatus = api.content.CompareStatus;
        var ContentBuilder = api.content.ContentBuilder;
        var ContentName = api.content.ContentName;
        var CreateContentRequest = api.content.CreateContentRequest;
        var UpdateContentRequest = api.content.UpdateContentRequest;
        var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
        var Metadata = api.content.Metadata;
        var SiteModel = api.content.site.SiteModel;
        var LiveEditModel = api.liveedit.LiveEditModel;
        var IsRenderableRequest = api.content.page.IsRenderableRequest;
        var ConfirmationDialog = api.ui.dialog.ConfirmationDialog;
        var ResponsiveManager = api.ui.responsive.ResponsiveManager;
        var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
        var ThumbnailUploader = api.content.ThumbnailUploader;
        var WizardHeaderWithDisplayNameAndNameBuilder = api.app.wizard.WizardHeaderWithDisplayNameAndNameBuilder;
        var WizardStep = api.app.wizard.WizardStep;
        var MixinName = api.schema.mixin.MixinName;
        var MixinNames = api.schema.mixin.MixinNames;
        var GetMixinByQualifiedNameRequest = api.schema.mixin.GetMixinByQualifiedNameRequest;
        var ContentWizardPanel = (function (_super) {
            __extends(ContentWizardPanel, _super);
            function ContentWizardPanel(params, callback) {
                var _this = this;
                this.constructing = true;
                this.isContentFormValid = false;
                this.isSecurityWizardStepFormAllowed = false;
                this.requireValid = false;
                this.contentNamedListeners = [];
                this.parentContent = params.parentContent;
                this.defaultModels = params.defaultModels;
                this.site = params.site;
                this.contentType = params.contentType;
                this.displayNameScriptExecutor = new _wizard.DisplayNameScriptExecutor();
                this.contentWizardHeader = new WizardHeaderWithDisplayNameAndNameBuilder().setDisplayNameGenerator(this.displayNameScriptExecutor).build();
                this.thumbnailUploader = new ThumbnailUploader({
                    name: 'thumbnail-uploader',
                    disabled: params.contentType.isImage(),
                    deferred: true
                });
                this.wizardActions = new app.wizard.action.ContentWizardActions(this);
                this.previewAction = this.wizardActions.getPreviewAction();
                this.publishAction = this.wizardActions.getPublishAction();
                var mainToolbar = new _wizard.ContentWizardToolbar({
                    saveAction: this.wizardActions.getSaveAction(),
                    duplicateAction: this.wizardActions.getDuplicateAction(),
                    deleteAction: this.wizardActions.getDeleteAction(),
                    publishAction: this.wizardActions.getPublishAction(),
                    previewAction: this.wizardActions.getPreviewAction(),
                    showLiveEditAction: this.wizardActions.getShowLiveEditAction(),
                    showFormAction: this.wizardActions.getShowFormAction(),
                    showSplitEditAction: this.wizardActions.getShowSplitEditAction()
                });
                this.contextWindowToggler = mainToolbar.getContextWindowToggler();
                this.cycleViewModeButton = mainToolbar.getCycleViewModeButton();
                this.showLiveEditAction = this.wizardActions.getShowLiveEditAction();
                this.showSplitEditAction = this.wizardActions.getShowSplitEditAction();
                this.showLiveEditAction.setEnabled(false);
                if (this.parentContent) {
                    this.contentWizardHeader.setPath(this.parentContent.getPath().prettifyUnnamedPathElements().toString() + "/");
                }
                else {
                    this.contentWizardHeader.setPath("/");
                }
                this.createSite = params.createSite;
                if (this.createSite || (params.persistedContent && params.persistedContent.isSite())) {
                    this.thumbnailUploader.addClass("site");
                }
                this.contentWizardStepForm = new _wizard.ContentWizardStepForm();
                this.metadataStepFormByName = {};
                this.settingsWizardStepForm = new _wizard.SettingsWizardStepForm();
                this.securityWizardStepForm = new _wizard.SecurityWizardStepForm();
                _wizard.ContentPermissionsAppliedEvent.on(function (event) { return _this.contentPermissionsUpdated(event.getContent()); });
                var isSiteOrWithinSite = this.site || this.createSite;
                var hasPageTemplate = this.defaultModels && this.defaultModels.hasPageTemplate();
                var hasSiteAndPageTemplate = isSiteOrWithinSite && hasPageTemplate;
                var isPageTemplate = this.contentType.getContentTypeName().isPageTemplate();
                if (hasSiteAndPageTemplate || isPageTemplate) {
                    this.liveFormPanel = new _wizard.page.LiveFormPanel({
                        contentWizardPanel: this,
                        contentType: this.contentType.getContentTypeName(),
                        defaultModels: params.defaultModels
                    });
                }
                if (this.contentType.hasContentDisplayNameScript()) {
                    this.displayNameScriptExecutor.setScript(this.contentType.getContentDisplayNameScript());
                }
                _super.call(this, {
                    tabId: params.tabId,
                    persistedItem: params.persistedContent,
                    formIcon: this.thumbnailUploader,
                    mainToolbar: mainToolbar,
                    header: this.contentWizardHeader,
                    actions: this.wizardActions,
                    livePanel: this.liveFormPanel,
                    split: !!this.liveFormPanel
                }, function () {
                    _this.onValidityChanged(function (event) {
                        _this.isContentFormValid = _this.isValid();
                        _this.thumbnailUploader.toggleClass("invalid", !_this.isValid());
                    });
                    _this.addClass("content-wizard-panel");
                    if (_this.getSplitPanel()) {
                        _this.getSplitPanel().addClass("prerendered");
                    }
                    var responsiveItem = ResponsiveManager.onAvailableSizeChanged(_this, function (item) {
                        if (_this.isVisible()) {
                            _this.updateStickyToolbar();
                            if (item.isInRangeOrSmaller(ResponsiveRanges._720_960)) {
                                _this.cycleViewModeButton.disableAction(_this.wizardActions.getShowSplitEditAction());
                                if (_this.isSplitView()) {
                                    _this.cycleViewModeButton.setCurrentAction(_this.wizardActions.getShowFormAction());
                                }
                            }
                            else if (item.isInRangeOrBigger(ResponsiveRanges._960_1200)) {
                                _this.cycleViewModeButton.enableAction(_this.wizardActions.getShowSplitEditAction());
                            }
                        }
                    });
                    _this.onRemoved(function (event) {
                        ResponsiveManager.unAvailableSizeChanged(_this);
                    });
                    _this.onShown(function (event) {
                        if (_this.getPersistedItem()) {
                            app.Router.setHash("edit/" + _this.getPersistedItem().getId());
                        }
                        else {
                            app.Router.setHash("new/" + _this.contentType.getName());
                        }
                        _this.wizardActions.getShowSplitEditAction().execute();
                        responsiveItem.update();
                    });
                    _this.constructing = false;
                    callback(_this);
                });
            }
            ContentWizardPanel.prototype.getContentType = function () {
                return this.contentType;
            };
            ContentWizardPanel.prototype.giveInitialFocus = function () {
                console.log("ContentWizardPanel.giveInitialFocus");
                if (this.contentType.hasContentDisplayNameScript()) {
                    if (!this.contentWizardStepForm.giveFocus()) {
                        console.log("ContentWizardPanel.giveInitialFocus() WARNING: Failed to give focus to contentWizardStepForm");
                        this.contentWizardHeader.giveFocus();
                    }
                }
                else {
                    this.contentWizardHeader.giveFocus();
                }
                this.startRememberFocus();
            };
            ContentWizardPanel.prototype.createSteps = function () {
                var _this = this;
                var moduleKeys = this.site ? this.site.getModuleKeys() : [];
                var modulePromises = moduleKeys.map(function (key) { return new api.module.GetModuleRequest(key).sendAndParse(); });
                return new api.security.auth.IsAuthenticatedRequest().sendAndParse().then(function (loginResult) {
                    _this.checkSecurityWizardStepFormAllowed(loginResult);
                    return wemQ.all(modulePromises);
                }).then(function (modules) {
                    var metadataMixinPromises = [];
                    metadataMixinPromises = metadataMixinPromises.concat(_this.contentType.getMetadata().map(function (name) {
                        return new GetMixinByQualifiedNameRequest(name).sendAndParse();
                    }));
                    modules.forEach(function (mdl) {
                        metadataMixinPromises = metadataMixinPromises.concat(mdl.getMetaSteps().map(function (name) {
                            return new GetMixinByQualifiedNameRequest(name).sendAndParse();
                        }));
                    });
                    return wemQ.all(metadataMixinPromises);
                }).then(function (mixins) {
                    var steps = [];
                    _this.contentWizardStep = new WizardStep(_this.contentType.getDisplayName(), _this.contentWizardStepForm);
                    steps.push(_this.contentWizardStep);
                    mixins.forEach(function (mixin, index) {
                        if (!_this.metadataStepFormByName[mixin.getMixinName().toString()]) {
                            var stepForm = new _wizard.ContentWizardStepForm();
                            _this.metadataStepFormByName[mixin.getMixinName().toString()] = stepForm;
                            steps.splice(index + 1, 0, new WizardStep(mixin.getDisplayName(), stepForm));
                        }
                    });
                    _this.settingsWizardStep = new WizardStep("Settings", _this.settingsWizardStepForm);
                    steps.push(_this.settingsWizardStep);
                    if (_this.isSecurityWizardStepFormAllowed) {
                        steps.push(new WizardStep("Security", _this.securityWizardStepForm));
                    }
                    _this.setSteps(steps);
                    return mixins;
                });
            };
            ContentWizardPanel.prototype.preLayoutNew = function () {
                var deferred = wemQ.defer();
                this.saveChanges().then(function () {
                    deferred.resolve(null);
                }).catch(function (reason) {
                    deferred.reject(reason);
                }).done();
                return deferred.promise;
            };
            ContentWizardPanel.prototype.postLayoutNew = function () {
                var deferred = wemQ.defer();
                this.enableDisplayNameScriptExecution(this.contentWizardStepForm.getFormView());
                this.contentWizardHeader.initNames(this.getPersistedItem().getDisplayName(), this.getPersistedItem().getName().toString(), true);
                deferred.resolve(null);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.layoutPersistedItem = function (persistedContent) {
                var _this = this;
                this.thumbnailUploader.setValue(new ContentIconUrlResolver().setContent(persistedContent).resolve()).setEnabled(!persistedContent.isImage()).setParams({
                    id: persistedContent.getContentId().toString()
                });
                this.thumbnailUploader.toggleClass("invalid", !persistedContent.isValid());
                this.notifyValidityChanged(persistedContent.isValid());
                api.content.ContentSummaryAndCompareStatusFetcher.fetch(persistedContent.getContentId()).then(function (contentSummaryAndCompareStatus) {
                    var ignore = contentSummaryAndCompareStatus.getCompareStatus() !== 0 /* NEW */;
                    _this.contentWizardHeader.disableNameGeneration(ignore);
                    if (!ignore) {
                        var publishHandler = function (event) {
                            if (_this.getPersistedItem() && event.getContentId() && (_this.getPersistedItem().getId() === event.getContentId().toString())) {
                                _this.contentWizardHeader.disableNameGeneration(true);
                                api.content.ContentPublishedEvent.un(publishHandler);
                            }
                        };
                        api.content.ContentPublishedEvent.on(publishHandler);
                        _this.onClosed(function () {
                            api.content.ContentPublishedEvent.un(publishHandler);
                        });
                    }
                }).done();
                var viewedContent;
                if (!this.constructing) {
                    var deferred = wemQ.defer();
                    viewedContent = this.assembleViewedContent(persistedContent.newBuilder()).build();
                    if (viewedContent.equals(persistedContent)) {
                        if (this.liveFormPanel) {
                            this.liveFormPanel.loadPage();
                        }
                    }
                    else {
                        console.warn("Received Content from server differs from what's viewed:");
                        if (!viewedContent.getContentData().equals(persistedContent.getContentData())) {
                            console.warn(" inequality found in Content.data");
                            if (persistedContent.getContentData() && viewedContent.getContentData()) {
                                console.warn(" comparing persistedContent.data against viewedContent.data:");
                                new api.data.PropertyTreeComparator().compareTree(persistedContent.getContentData(), viewedContent.getContentData());
                            }
                        }
                        if (!api.ObjectHelper.equals(viewedContent.getPage(), persistedContent.getPage())) {
                            console.warn(" inequality found in Content.page");
                            if (persistedContent.getPage() && viewedContent.getPage()) {
                                console.warn(" comparing persistedContent.page.config against viewedContent.page.config:");
                                new api.data.PropertyTreeComparator().compareTree(persistedContent.getPage().getConfig(), viewedContent.getPage().getConfig());
                            }
                        }
                        if (!api.ObjectHelper.arrayEquals(viewedContent.getAllMetadata(), persistedContent.getAllMetadata())) {
                            console.warn(" inequality found in Content.meta");
                        }
                        if (!api.ObjectHelper.equals(viewedContent.getAttachments(), persistedContent.getAttachments())) {
                            console.warn(" inequality found in Content.attachments");
                        }
                        if (!api.ObjectHelper.equals(viewedContent.getPermissions(), persistedContent.getPermissions())) {
                            console.warn(" inequality found in Content.permissions");
                        }
                        console.warn(" viewedContent: ", viewedContent);
                        console.warn(" persistedContent: ", persistedContent);
                        ConfirmationDialog.get().setQuestion("Received Content from server differs from what you have. Would you like to load changes from server?").setYesCallback(function () { return _this.doLayoutPersistedItem(persistedContent.clone()); }).setNoCallback(function () {
                        }).show();
                    }
                    deferred.resolve(null);
                    return deferred.promise;
                }
                else {
                    return this.doLayoutPersistedItem(persistedContent.clone());
                }
            };
            ContentWizardPanel.prototype.doLayoutPersistedItem = function (content) {
                var _this = this;
                this.showLiveEditAction.setVisible(false);
                this.showLiveEditAction.setEnabled(false);
                this.previewAction.setVisible(false);
                this.contextWindowToggler.setVisible(false);
                this.cycleViewModeButton.setVisible(false);
                new IsRenderableRequest(content.getContentId()).sendAndParse().then(function (renderable) {
                    _this.showLiveEditAction.setVisible(renderable);
                    _this.showLiveEditAction.setEnabled(renderable);
                    _this.showSplitEditAction.setEnabled(renderable);
                    _this.previewAction.setVisible(renderable);
                    _this.contextWindowToggler.setVisible(renderable);
                    _this.cycleViewModeButton.setVisible(renderable);
                    if (_this.getEl().getWidth() > ResponsiveRanges._720_960.getMaximumRange() && renderable) {
                        _this.cycleViewModeButton.setCurrentAction(_this.wizardActions.getShowSplitEditAction());
                    }
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                var parallelPromises = [this.createSteps()];
                return wemQ.all(parallelPromises).spread(function (schemas) {
                    var formContext = _this.createFormContext(content);
                    var contentData = content.getContentData();
                    var formViewLayoutPromises = [];
                    formViewLayoutPromises.push(_this.contentWizardStepForm.layout(formContext, contentData, _this.contentType.getForm()));
                    _this.displayNameScriptExecutor.setFormView(_this.contentWizardStepForm.getFormView());
                    _this.settingsWizardStepForm.layout(content);
                    _this.settingsWizardStepForm.setModel(new _wizard.ContentSettingsModel(content));
                    if (_this.isSecurityWizardStepFormAllowed) {
                        _this.securityWizardStepForm.layout(content);
                    }
                    schemas.forEach(function (schema, index) {
                        var metadata = content.getMetadata(schema.getMixinName());
                        if (!metadata) {
                            metadata = new Metadata(schema.getMixinName(), new PropertyTree(api.Client.get().getPropertyIdProvider()));
                            content.getAllMetadata().push(metadata);
                        }
                        var metadataFormView = _this.metadataStepFormByName[schema.getMixinName().toString()];
                        var metadataForm = new api.form.FormBuilder().addFormItems(schema.getFormItems()).build();
                        formViewLayoutPromises.push(metadataFormView.layout(formContext, metadata.getData(), metadataForm));
                    });
                    return wemQ.all(formViewLayoutPromises).spread(function () {
                        console.log("ContentWizardPanel.doLayoutPersistedItem: all FormView-s layed out");
                        if (_this.liveFormPanel) {
                            if (!_this.liveEditModel) {
                                var site = content.isSite() ? content : _this.site;
                                _this.siteModel = new SiteModel(site);
                                return _this.initLiveEditModel(content, _this.siteModel, formContext).then(function () {
                                    _this.liveFormPanel.setModel(_this.liveEditModel);
                                    _this.liveFormPanel.loadPage();
                                    return wemQ(null);
                                });
                            }
                            else {
                                _this.liveFormPanel.loadPage();
                            }
                        }
                        if (!_this.siteModel && content.isSite()) {
                            _this.siteModel = new SiteModel(content);
                        }
                        if (_this.siteModel) {
                            _this.initSiteModelListeners();
                        }
                        return wemQ(null);
                    });
                });
            };
            ContentWizardPanel.prototype.initSiteModelListeners = function () {
                var _this = this;
                this.siteModel.onModuleAdded(function (event) {
                    _this.addMetadataStepForms(event.getModuleKey());
                });
                this.siteModel.onModuleRemoved(function (event) {
                    _this.removeMetadataStepForms();
                });
            };
            ContentWizardPanel.prototype.removeMetadataStepForms = function () {
                var _this = this;
                var moduleKeys = this.siteModel.getModuleKeys();
                var modulePromises = moduleKeys.map(function (key) { return new api.module.GetModuleRequest(key).sendAndParse(); });
                return wemQ.all(modulePromises).then(function (modules) {
                    var metadataMixinPromises = [];
                    modules.forEach(function (mdl) {
                        metadataMixinPromises = metadataMixinPromises.concat(mdl.getMetaSteps().map(function (name) {
                            return new GetMixinByQualifiedNameRequest(name).sendAndParse();
                        }));
                    });
                    return wemQ.all(metadataMixinPromises);
                }).then(function (mixins) {
                    var activeMixinsNames = api.schema.mixin.MixinNames.create().fromMixins(mixins).build();
                    var panelNamesToRemoveBuilder = MixinNames.create();
                    for (var key in _this.metadataStepFormByName) {
                        var mixinName = new MixinName(key);
                        if (!activeMixinsNames.contains(mixinName)) {
                            panelNamesToRemoveBuilder.addMixinName(mixinName);
                        }
                    }
                    var panelNamesToRemove = panelNamesToRemoveBuilder.build();
                    panelNamesToRemove.forEach(function (panelName) {
                        _this.removeStepWithForm(_this.metadataStepFormByName[panelName.toString()]);
                        delete _this.metadataStepFormByName[panelName.toString()];
                    });
                    return mixins;
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
            };
            ContentWizardPanel.prototype.addMetadataStepForms = function (moduleKey) {
                var _this = this;
                new api.module.GetModuleRequest(moduleKey).sendAndParse().then(function (currentModule) {
                    var mixinNames = currentModule.getMetaSteps();
                    var mixinNamesToAdd = mixinNames.filter(function (mixinName) {
                        return !_this.metadataStepFormByName[mixinName.toString()];
                    });
                    var getMixinPromises = mixinNamesToAdd.map(function (name) {
                        return new GetMixinByQualifiedNameRequest(name).sendAndParse();
                    });
                    return wemQ.all(getMixinPromises);
                }).then(function (mixins) {
                    mixins.forEach(function (mixin) {
                        if (!_this.metadataStepFormByName[mixin.getMixinName().toString()]) {
                            var stepForm = new _wizard.ContentWizardStepForm();
                            _this.metadataStepFormByName[mixin.getMixinName().toString()] = stepForm;
                            var wizardStep = new WizardStep(mixin.getDisplayName(), stepForm);
                            _this.insertStepBefore(wizardStep, _this.settingsWizardStep);
                            var metadata = new Metadata(mixin.getMixinName(), new PropertyTree(api.Client.get().getPropertyIdProvider()));
                            stepForm.layout(_this.createFormContext(_this.getPersistedItem()), metadata.getData(), mixin.toForm());
                        }
                    });
                    return mixins;
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
            };
            ContentWizardPanel.prototype.initLiveEditModel = function (content, siteModel, formContext) {
                this.initSiteModelListeners();
                this.liveEditModel = LiveEditModel.create().setParentContent(this.parentContent).setContent(content).setContentFormContext(formContext).setSiteModel(siteModel).build();
                return this.liveEditModel.init(this.defaultModels.getPageTemplate(), this.defaultModels.getPageDescriptor());
            };
            ContentWizardPanel.prototype.postLayoutPersisted = function (existing) {
                var deferred = wemQ.defer();
                this.contentWizardHeader.initNames(existing.getDisplayName(), existing.getName().toString(), false);
                this.enableDisplayNameScriptExecution(this.contentWizardStepForm.getFormView());
                deferred.resolve(null);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.persistNewItem = function () {
                return new _wizard.PersistNewContentRoutine(this).setCreateContentRequestProducer(this.produceCreateContentRequest).execute().then(function (content) {
                    api.notify.showFeedback('Content was created!');
                    return content;
                });
            };
            ContentWizardPanel.prototype.postPersistNewItem = function (persistedContent) {
                var deferred = wemQ.defer();
                if (persistedContent.isSite()) {
                    this.site = persistedContent;
                }
                deferred.resolve(null);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.produceCreateContentRequest = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var parentPath = this.parentContent != null ? this.parentContent.getPath() : api.content.ContentPath.ROOT;
                if (this.contentType.getContentTypeName().isMedia()) {
                    deferred.resolve(null);
                }
                else {
                    var formContext = this.createFormContext(null);
                    var form = this.contentType.getForm();
                    var data = new PropertyTree();
                    var formView = new FormView(formContext, form, data.getRoot());
                    formView.layout().then(function () {
                        deferred.resolve(new CreateContentRequest().setRequireValid(_this.requireValid).setName(api.content.ContentUnnamed.newUnnamed()).setParent(parentPath).setContentType(_this.contentType.getContentTypeName()).setDisplayName(_this.contentWizardHeader.getDisplayName()).setData(data).setMetadata([]));
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                }
                return deferred.promise;
            };
            ContentWizardPanel.prototype.updatePersistedItem = function () {
                var _this = this;
                var persistedContent = this.getPersistedItem();
                var viewedContent = this.assembleViewedContent(persistedContent.newBuilder()).build();
                var updatePersistedContentRoutine = new _wizard.UpdatePersistedContentRoutine(this, persistedContent, viewedContent).setUpdateContentRequestProducer(this.produceUpdateContentRequest);
                return updatePersistedContentRoutine.execute().then(function (content) {
                    if (persistedContent.getName().isUnnamed() && !content.getName().isUnnamed()) {
                        _this.notifyContentNamed(content);
                    }
                    api.notify.showFeedback('Content was updated!');
                    new api.content.ContentUpdatedEvent(content.getContentId()).fire();
                    return content;
                });
            };
            ContentWizardPanel.prototype.produceUpdateContentRequest = function (persistedContent, viewedContent) {
                var persistedContent = this.getPersistedItem();
                var updateContentRequest = new UpdateContentRequest(persistedContent.getId()).setRequireValid(this.requireValid).setContentName(viewedContent.getName()).setDisplayName(viewedContent.getDisplayName()).setData(viewedContent.getContentData()).setMetadata(viewedContent.getAllMetadata()).setOwner(viewedContent.getOwner()).setLanguage(viewedContent.getLanguage());
                return updateContentRequest;
            };
            ContentWizardPanel.prototype.hasUnsavedChanges = function () {
                var persistedContent = this.getPersistedItem();
                if (persistedContent == undefined) {
                    return true;
                }
                else {
                    var viewedContent = this.assembleViewedContent(new ContentBuilder(persistedContent)).build();
                    return !viewedContent.equals(persistedContent);
                }
            };
            ContentWizardPanel.prototype.enableDisplayNameScriptExecution = function (formView) {
                var _this = this;
                if (this.displayNameScriptExecutor.hasScript()) {
                    formView.onKeyUp(function (event) {
                        if (_this.displayNameScriptExecutor.hasScript()) {
                            _this.contentWizardHeader.setDisplayName(_this.displayNameScriptExecutor.execute());
                        }
                    });
                }
            };
            ContentWizardPanel.prototype.assembleViewedContent = function (viewedContentBuilder) {
                viewedContentBuilder.setName(this.resolveContentNameForUpdateRequest());
                viewedContentBuilder.setDisplayName(this.contentWizardHeader.getDisplayName());
                if (this.contentWizardStepForm) {
                    viewedContentBuilder.setData(this.contentWizardStepForm.getData());
                }
                var metadata = [];
                for (var key in this.metadataStepFormByName) {
                    if (this.metadataStepFormByName.hasOwnProperty(key)) {
                        metadata.push(new Metadata(new MixinName(key), this.metadataStepFormByName[key].getData()));
                    }
                }
                viewedContentBuilder.setMetadata(metadata);
                this.settingsWizardStepForm.getModel().apply(viewedContentBuilder);
                viewedContentBuilder.setPage(this.assembleViewedPage());
                return viewedContentBuilder;
            };
            ContentWizardPanel.prototype.assembleViewedPage = function () {
                if (!this.liveFormPanel) {
                    return null;
                }
                return this.liveFormPanel.getPage();
            };
            ContentWizardPanel.prototype.resolveContentNameForUpdateRequest = function () {
                if (api.util.StringHelper.isEmpty(this.contentWizardHeader.getName()) && this.getPersistedItem().getName().isUnnamed()) {
                    return this.getPersistedItem().getName();
                }
                else {
                    return ContentName.fromString(this.contentWizardHeader.getName());
                }
            };
            ContentWizardPanel.prototype.setRequireValid = function (requireValid) {
                this.requireValid = requireValid;
            };
            ContentWizardPanel.prototype.showLiveEdit = function () {
                this.getSplitPanel().addClass("toggle-live");
                this.getSplitPanel().removeClass("toggle-form toggle-split prerendered");
                ResponsiveManager.fireResizeEvent();
            };
            ContentWizardPanel.prototype.showSplitEdit = function () {
                if (this.getSplitPanel()) {
                    this.getSplitPanel().addClass("toggle-split");
                    this.getSplitPanel().removeClass("toggle-live toggle-form prerendered");
                }
                ResponsiveManager.fireResizeEvent();
            };
            ContentWizardPanel.prototype.showForm = function () {
                this.getSplitPanel().addClass("toggle-form");
                this.getSplitPanel().removeClass("toggle-live toggle-split prerendered");
                ResponsiveManager.fireResizeEvent();
            };
            ContentWizardPanel.prototype.isSplitView = function () {
                return this.getSplitPanel() && this.getSplitPanel().hasClass("toggle-split");
            };
            ContentWizardPanel.prototype.checkContentCanBePublished = function () {
                if (!this.isContentFormValid) {
                    this.contentWizardStepForm.displayValidationErrors(true);
                }
                var contentFormHasValidUserInput = this.contentWizardStepForm.getFormView().hasValidUserInput();
                var allMetadataFormsValid = true;
                var allMetadataFormsHasValidUserInput = true;
                for (var key in this.metadataStepFormByName) {
                    if (this.metadataStepFormByName.hasOwnProperty(key)) {
                        var form = this.metadataStepFormByName[key];
                        if (!form.isValid()) {
                            form.displayValidationErrors(true);
                            allMetadataFormsValid = false;
                        }
                        var formHasValidUserInput = form.getFormView().hasValidUserInput();
                        if (!formHasValidUserInput) {
                            allMetadataFormsHasValidUserInput = false;
                        }
                    }
                }
                return this.isContentFormValid && allMetadataFormsValid && contentFormHasValidUserInput && allMetadataFormsHasValidUserInput;
            };
            ContentWizardPanel.prototype.getContextWindowToggler = function () {
                return this.contextWindowToggler;
            };
            ContentWizardPanel.prototype.getCloseAction = function () {
                return this.wizardActions.getCloseAction();
            };
            ContentWizardPanel.prototype.onContentNamed = function (listener) {
                this.contentNamedListeners.push(listener);
            };
            ContentWizardPanel.prototype.unContentNamed = function (listener) {
                this.contentNamedListeners = this.contentNamedListeners.filter(function (curr) {
                    return curr != listener;
                });
                return this;
            };
            ContentWizardPanel.prototype.notifyContentNamed = function (content) {
                var _this = this;
                this.contentNamedListeners.forEach(function (listener) {
                    listener.call(_this, new api.content.ContentNamedEvent(_this, content));
                });
            };
            ContentWizardPanel.prototype.contentPermissionsUpdated = function (content) {
                var persistedContent = this.getPersistedItem();
                if (persistedContent && (content.getId() === persistedContent.getId())) {
                    var updatedContent = persistedContent.newBuilder().setInheritPermissionsEnabled(content.isInheritPermissionsEnabled()).setPermissions(content.getPermissions().clone()).build();
                    this.setPersistedItem(updatedContent);
                }
            };
            ContentWizardPanel.prototype.createFormContext = function (content) {
                var formContext = ContentFormContext.create().setSite(this.site).setParentContent(this.parentContent).setPersistedContent(content).setShowEmptyFormItemSetOccurrences(this.isItemPersisted()).build();
                return formContext;
            };
            ContentWizardPanel.prototype.checkSecurityWizardStepFormAllowed = function (loginResult) {
                var _this = this;
                var accessEntriesWithEditPermissions = this.getPersistedItem().getPermissions().getEntries().filter(function (item) {
                    if (item.isAllowed(6 /* WRITE_PERMISSIONS */))
                        return true;
                    else
                        return false;
                });
                loginResult.getPrincipals().some(function (principalKey) {
                    if (api.security.RoleKeys.ADMIN.equals(principalKey) || _this.isPrincipalAllowedToEditPermissions(principalKey, accessEntriesWithEditPermissions)) {
                        _this.isSecurityWizardStepFormAllowed = true;
                        return true;
                    }
                });
            };
            ContentWizardPanel.prototype.isPrincipalAllowedToEditPermissions = function (prinipalKey, accessEntriesWithEditPermissions) {
                var result = false;
                accessEntriesWithEditPermissions.some(function (entry) {
                    if (entry.getPrincipalKey().equals(prinipalKey)) {
                        result = true;
                        return true;
                    }
                });
                return result;
            };
            return ContentWizardPanel;
        })(api.app.wizard.WizardPanel);
        _wizard.ContentWizardPanel = ContentWizardPanel;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PersistedNewContentRoutineContext = (function () {
            function PersistedNewContentRoutineContext() {
                this.content = null;
            }
            return PersistedNewContentRoutineContext;
        })();
        wizard.PersistedNewContentRoutineContext = PersistedNewContentRoutineContext;
        var PersistNewContentRoutine = (function (_super) {
            __extends(PersistNewContentRoutine, _super);
            function PersistNewContentRoutine(thisOfProducer) {
                _super.call(this, thisOfProducer);
                this.doneHandledContent = false;
            }
            PersistNewContentRoutine.prototype.setCreateContentRequestProducer = function (producer) {
                this.createContentRequestProducer = producer;
                return this;
            };
            PersistNewContentRoutine.prototype.execute = function () {
                var context = new PersistedNewContentRoutineContext();
                return this.doExecute(context);
            };
            PersistNewContentRoutine.prototype.doExecuteNext = function (context) {
                var _this = this;
                if (!this.doneHandledContent) {
                    return this.doHandleCreateContent(context).then(function () {
                        _this.doneHandledContent = true;
                        return _this.doExecuteNext(context);
                    });
                }
                else {
                    return wemQ(context.content);
                }
            };
            PersistNewContentRoutine.prototype.doHandleCreateContent = function (context) {
                if (this.createContentRequestProducer != undefined) {
                    return this.createContentRequestProducer.call(this.getThisOfProducer()).then(function (createContentRequest) {
                        return createContentRequest.sendAndParse().then(function (content) {
                            context.content = content;
                        });
                    });
                }
                else {
                    return api.util.PromiseHelper.newResolvedVoidPromise();
                }
            };
            return PersistNewContentRoutine;
        })(api.util.Flow);
        wizard.PersistNewContentRoutine = PersistNewContentRoutine;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var CreatePageRequest = api.content.page.CreatePageRequest;
        var UpdatePageRequest = api.content.page.UpdatePageRequest;
        var DeletePageRequest = api.content.page.DeletePageRequest;
        var UpdatePersistedContentRoutineContext = (function () {
            function UpdatePersistedContentRoutineContext() {
                this.content = null;
            }
            return UpdatePersistedContentRoutineContext;
        })();
        wizard.UpdatePersistedContentRoutineContext = UpdatePersistedContentRoutineContext;
        var UpdatePersistedContentRoutine = (function (_super) {
            __extends(UpdatePersistedContentRoutine, _super);
            function UpdatePersistedContentRoutine(thisOfProducer, persistedContent, viewedContent) {
                _super.call(this, thisOfProducer);
                this.doneHandledContent = false;
                this.doneHandledPage = false;
                this.persistedContent = persistedContent;
                this.viewedContent = viewedContent;
            }
            UpdatePersistedContentRoutine.prototype.setUpdateContentRequestProducer = function (producer) {
                this.updateContentRequestProducer = producer;
                return this;
            };
            UpdatePersistedContentRoutine.prototype.execute = function () {
                var context = new UpdatePersistedContentRoutineContext();
                context.content = this.persistedContent;
                return this.doExecute(context);
            };
            UpdatePersistedContentRoutine.prototype.doExecuteNext = function (context) {
                var _this = this;
                if (!this.doneHandledContent) {
                    return this.doHandleUpdateContent(context).then(function () {
                        _this.doneHandledContent = true;
                        return _this.doExecuteNext(context);
                    });
                }
                else if (!this.doneHandledPage) {
                    return this.doHandlePage(context).then(function () {
                        _this.doneHandledPage = true;
                        return _this.doExecuteNext(context);
                    });
                }
                else {
                    return wemQ(context.content);
                }
            };
            UpdatePersistedContentRoutine.prototype.doHandleUpdateContent = function (context) {
                return this.updateContentRequestProducer.call(this.getThisOfProducer(), context.content, this.viewedContent).sendAndParse().then(function (content) {
                    context.content = content;
                });
            };
            UpdatePersistedContentRoutine.prototype.doHandlePage = function (context) {
                var pageCUDRequest = this.producePageCUDRequest(context.content, this.viewedContent);
                if (pageCUDRequest != null) {
                    return pageCUDRequest.sendAndParse().then(function (content) {
                        context.content = content;
                    });
                }
                else {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                }
            };
            UpdatePersistedContentRoutine.prototype.producePageCUDRequest = function (persistedContent, viewedContent) {
                if (persistedContent.isPage() && !viewedContent.isPage()) {
                    return new DeletePageRequest(persistedContent.getContentId());
                }
                else if (!persistedContent.isPage() && viewedContent.isPage()) {
                    var viewedPage = viewedContent.getPage();
                    return new CreatePageRequest(persistedContent.getContentId()).setController(viewedPage.getController()).setPageTemplateKey(viewedPage.getTemplate()).setConfig(viewedPage.getConfig()).setRegions(viewedPage.getRegions());
                }
                else if (persistedContent.isPage() && viewedContent.isPage()) {
                    var viewedPage = viewedContent.getPage();
                    return new UpdatePageRequest(persistedContent.getContentId()).setController((viewedPage.getController())).setPageTemplateKey((viewedPage.getTemplate())).setConfig(viewedPage.getConfig()).setRegions(viewedPage.getRegions());
                }
            };
            return UpdatePersistedContentRoutine;
        })(api.util.Flow);
        wizard.UpdatePersistedContentRoutine = UpdatePersistedContentRoutine;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var DefaultModelsFactory = app.wizard.page.DefaultModelsFactory;
        var ContentWizardPanelFactory = (function () {
            function ContentWizardPanelFactory() {
                this.createSite = false;
            }
            ContentWizardPanelFactory.prototype.setContentToEdit = function (value) {
                this.contentId = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setContentTypeName = function (value) {
                this.contentTypeName = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setParentContent = function (value) {
                this.parentContent = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setCreateSite = function (value) {
                this.createSite = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setAppBarTabId = function (value) {
                this.appBarTabId = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.createForNew = function () {
                var _this = this;
                this.creatingForNew = true;
                return this.loadContentType(this.contentTypeName).then(function (loadedContentType) {
                    _this.contentType = loadedContentType;
                    return _this.loadParentContent();
                }).then(function (loadedParentContent) {
                    _this.parentContent = loadedParentContent;
                    return _this.loadSite(loadedParentContent ? loadedParentContent.getContentId() : null);
                }).then(function (loadedSite) {
                    _this.siteContent = loadedSite;
                    return _this.loadDefaultModels(_this.siteContent, _this.contentTypeName);
                }).then(function (defaultModels) {
                    _this.defaultModels = defaultModels;
                    return _this.newContentWizardPanelForNew();
                });
            };
            ContentWizardPanelFactory.prototype.createForEdit = function () {
                var _this = this;
                this.creatingForNew = false;
                return this.loadContentToEdit().then(function (loadedContentToEdit) {
                    _this.contentToEdit = loadedContentToEdit;
                    return _this.loadContentType(_this.contentToEdit.getType());
                }).then(function (loadedContentType) {
                    _this.contentType = loadedContentType;
                    return _this.loadParentContent();
                }).then(function (loadedParentContent) {
                    _this.parentContent = loadedParentContent;
                    return _this.loadSite(_this.contentId);
                }).then(function (loadedSite) {
                    if (!!loadedSite) {
                        _this.siteContent = loadedSite;
                    }
                    return _this.loadDefaultModels(_this.siteContent, _this.contentToEdit.getType());
                }).then(function (defaultModels) {
                    _this.defaultModels = defaultModels;
                    return _this.newContentWizardPanelForEdit();
                });
            };
            ContentWizardPanelFactory.prototype.loadContentToEdit = function () {
                return new api.content.GetContentByIdRequest(this.contentId).sendAndParse();
            };
            ContentWizardPanelFactory.prototype.loadContentType = function (name) {
                return new api.schema.content.GetContentTypeByNameRequest(name).sendAndParse();
            };
            ContentWizardPanelFactory.prototype.loadSite = function (contentId) {
                return contentId ? new api.content.GetNearestSiteRequest(contentId).sendAndParse() : wemQ(null);
            };
            ContentWizardPanelFactory.prototype.loadDefaultModels = function (site, contentType) {
                if (site) {
                    return DefaultModelsFactory.create({
                        siteId: site.getContentId(),
                        contentType: contentType,
                        modules: site.getModuleKeys()
                    });
                }
                else {
                    return wemQ(null);
                }
            };
            ContentWizardPanelFactory.prototype.loadParentContent = function () {
                if (this.parentContent != null) {
                    return wemQ(this.parentContent);
                }
                else if (!this.creatingForNew && !this.contentToEdit.hasParent()) {
                    return wemQ(null);
                }
                else if (this.creatingForNew && this.parentContent == null) {
                    return wemQ(null);
                }
                return new api.content.GetContentByPathRequest(this.contentToEdit.getPath().getParentPath()).sendAndParse();
            };
            ContentWizardPanelFactory.prototype.newContentWizardPanelForNew = function () {
                var deferred = wemQ.defer();
                var wizardParams = new app.wizard.ContentWizardPanelParams().setAppBarTabId(this.appBarTabId).setContentType(this.contentType).setParentContent(this.parentContent).setSite(this.siteContent).setDefaultModels(this.defaultModels);
                wizardParams.setCreateSite(this.createSite);
                new app.wizard.ContentWizardPanel(wizardParams, function (wizard) {
                    deferred.resolve(wizard);
                });
                return deferred.promise;
            };
            ContentWizardPanelFactory.prototype.newContentWizardPanelForEdit = function () {
                var deferred = wemQ.defer();
                var wizardParams = new _wizard.ContentWizardPanelParams().setAppBarTabId(this.appBarTabId).setContentType(this.contentType).setParentContent(this.parentContent).setPersistedContent(this.contentToEdit).setSite(this.siteContent).setDefaultModels(this.defaultModels);
                new _wizard.ContentWizardPanel(wizardParams, function (wizard) {
                    deferred.resolve(wizard);
                });
                return deferred.promise;
            };
            return ContentWizardPanelFactory;
        })();
        _wizard.ContentWizardPanelFactory = ContentWizardPanelFactory;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ContentWizardPanelParams = (function () {
            function ContentWizardPanelParams() {
                this.createSite = false;
            }
            ContentWizardPanelParams.prototype.setAppBarTabId = function (value) {
                this.tabId = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setContentType = function (value) {
                this.contentType = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setParentContent = function (value) {
                this.parentContent = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setPersistedContent = function (value) {
                this.persistedContent = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setSite = function (value) {
                this.site = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setDefaultModels = function (value) {
                this.defaultModels = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setCreateSite = function (value) {
                this.createSite = value;
                return this;
            };
            return ContentWizardPanelParams;
        })();
        wizard.ContentWizardPanelParams = ContentWizardPanelParams;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ShowContentFormEvent = (function (_super) {
            __extends(ShowContentFormEvent, _super);
            function ShowContentFormEvent() {
                _super.apply(this, arguments);
            }
            ShowContentFormEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowContentFormEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowContentFormEvent;
        })(api.event.Event);
        wizard.ShowContentFormEvent = ShowContentFormEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ShowLiveEditEvent = (function (_super) {
            __extends(ShowLiveEditEvent, _super);
            function ShowLiveEditEvent() {
                _super.apply(this, arguments);
            }
            ShowLiveEditEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowLiveEditEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowLiveEditEvent;
        })(api.event.Event);
        wizard.ShowLiveEditEvent = ShowLiveEditEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ShowSplitEditEvent = (function (_super) {
            __extends(ShowSplitEditEvent, _super);
            function ShowSplitEditEvent() {
                _super.apply(this, arguments);
            }
            ShowSplitEditEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowSplitEditEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowSplitEditEvent;
        })(api.event.Event);
        wizard.ShowSplitEditEvent = ShowSplitEditEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var ContentSummaryViewer = api.content.ContentSummaryViewer;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var CompareContentGrid = (function (_super) {
            __extends(CompareContentGrid, _super);
            function CompareContentGrid(content) {
                var _this = this;
                _super.call(this, new TreeGridBuilder().setColumns([
                    new GridColumnBuilder().setName("Name").setId("displayName").setField("displayName").setFormatter(this.nameFormatter).build()
                ]).setPartialLoadEnabled(true).setLoadBufferSize(20).prependClasses("compare-content-grid"));
                this.content = content;
                this.onLoaded(function () {
                    _this.selectAll();
                });
            }
            CompareContentGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                var viewer = node.getViewer("name");
                if (!viewer) {
                    viewer = new ContentSummaryViewer();
                    viewer.setObject(node.getData().getContentSummary());
                    node.setViewer("name", viewer);
                }
                return viewer.toString();
            };
            CompareContentGrid.prototype.fetchChildren = function (parentNode) {
                var parentContentId = parentNode && parentNode.getData() ? parentNode.getData().getContentId() : null;
                return api.content.ContentSummaryAndCompareStatusFetcher.fetchChildren(parentContentId).then(function (data) {
                    return data.getContents();
                });
            };
            CompareContentGrid.prototype.hasChildren = function (elem) {
                return elem.hasChildren();
            };
            CompareContentGrid.prototype.getDataId = function (data) {
                return data.getId();
            };
            CompareContentGrid.prototype.refreshNodeData = function (parentNode) {
                var _this = this;
                return ContentSummaryAndCompareStatusFetcher.fetch(parentNode.getData().getContentId()).then(function (content) {
                    parentNode.setData(content);
                    _this.refreshNode(parentNode);
                    return parentNode;
                });
            };
            CompareContentGrid.prototype.sortNodeChildren = function (node) {
                var comparator;
                if (this.getRoot().getCurrentRoot() == node) {
                    comparator = new api.content.ContentNodeByDisplayNameComparator();
                }
                else {
                    comparator = new api.content.ContentNodeByModifiedTimeComparator();
                }
                var children = node.getChildren().sort(comparator.compare);
                node.setChildren(children);
                this.initData(this.getRoot().getCurrentRoot().treeToList());
            };
            return CompareContentGrid;
        })(TreeGrid);
        wizard.CompareContentGrid = CompareContentGrid;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PublishAction = (function (_super) {
            __extends(PublishAction, _super);
            function PublishAction() {
                _super.call(this, PublishAction.BASE_STRING);
            }
            PublishAction.prototype.setToBePublishedAmount = function (amount) {
                this.setEnabled(amount >= 1);
                this.setLabel(PublishAction.BASE_STRING + " (" + amount + ")");
            };
            PublishAction.BASE_STRING = "Publish now";
            return PublishAction;
        })(api.ui.Action);
        wizard.PublishAction = PublishAction;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ScheduleAction = (function (_super) {
            __extends(ScheduleAction, _super);
            function ScheduleAction() {
                _super.call(this, "Schedule");
            }
            return ScheduleAction;
        })(api.ui.Action);
        wizard.ScheduleAction = ScheduleAction;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var OpenEditPermissionsDialogEvent = (function (_super) {
            __extends(OpenEditPermissionsDialogEvent, _super);
            function OpenEditPermissionsDialogEvent(content) {
                _super.call(this);
                this.content = content;
            }
            OpenEditPermissionsDialogEvent.prototype.getContent = function () {
                return this.content;
            };
            OpenEditPermissionsDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            OpenEditPermissionsDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return OpenEditPermissionsDialogEvent;
        })(api.event.Event);
        wizard.OpenEditPermissionsDialogEvent = OpenEditPermissionsDialogEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ContentPermissionsAppliedEvent = (function (_super) {
            __extends(ContentPermissionsAppliedEvent, _super);
            function ContentPermissionsAppliedEvent(content) {
                _super.call(this);
                this.content = content;
            }
            ContentPermissionsAppliedEvent.prototype.getContent = function () {
                return this.content;
            };
            ContentPermissionsAppliedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ContentPermissionsAppliedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ContentPermissionsAppliedEvent;
        })(api.event.Event);
        wizard.ContentPermissionsAppliedEvent = ContentPermissionsAppliedEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var AccessControlComboBox = api.ui.security.acl.AccessControlComboBox;
        var AccessControlList = api.security.acl.AccessControlList;
        var EditPermissionsDialog = (function (_super) {
            __extends(EditPermissionsDialog, _super);
            function EditPermissionsDialog() {
                var _this = this;
                this.dialogTitle = new EditPermissionsDialogTitle('Edit Permissions', '');
                _super.call(this, {
                    title: this.dialogTitle
                });
                this.addClass('edit-permissions-dialog');
                this.inheritPermissionsCheck = new api.ui.Checkbox().setLabel('Inherit permissions');
                this.inheritPermissionsCheck.addClass('inherit-perm-check');
                this.appendChildToContentPanel(this.inheritPermissionsCheck);
                var section = new api.dom.SectionEl();
                this.appendChildToContentPanel(section);
                var form = new api.ui.form.Form();
                section.appendChild(form);
                this.comboBox = new AccessControlComboBox();
                this.comboBox.addClass('principal-combobox');
                form.appendChild(this.comboBox);
                var comboBoxChangeListener = function () {
                    var currentEntries = _this.getEntries().sort();
                    var permissionsModified = !api.ObjectHelper.arrayEquals(currentEntries, _this.originalValues);
                    var inheritCheckModified = _this.inheritPermissionsCheck.isChecked() !== _this.originalInherit;
                    _this.applyAction.setEnabled(permissionsModified || inheritCheckModified);
                };
                var changeListener = function () {
                    var inheritPermissions = _this.inheritPermissionsCheck.isChecked();
                    _this.comboBox.toggleClass('disabled', inheritPermissions);
                    if (inheritPermissions) {
                        _this.layoutInheritedPermissions();
                    }
                    _this.comboBox.getComboBox().setVisible(!inheritPermissions);
                    _this.comboBox.setEditable(!inheritPermissions);
                    comboBoxChangeListener();
                };
                this.inheritPermissionsCheck.onValueChanged(changeListener);
                this.comboBox.onOptionValueChanged(comboBoxChangeListener);
                this.comboBox.onOptionSelected(comboBoxChangeListener);
                this.comboBox.onOptionDeselected(comboBoxChangeListener);
                this.overwriteChildPermissionsCheck = new api.ui.Checkbox().setLabel('Overwrite child permissions');
                this.overwriteChildPermissionsCheck.addClass('overwrite-child-check');
                this.appendChildToContentPanel(this.overwriteChildPermissionsCheck);
                this.applyAction = new api.ui.Action('Apply');
                this.applyAction.onExecuted(function () {
                    _this.applyPermissions();
                });
                this.addAction(this.applyAction, true);
                api.dom.Body.get().appendChild(this);
                this.parentPermissions = [];
                wizard.OpenEditPermissionsDialogEvent.on(function (event) {
                    _this.content = event.getContent();
                    _this.getParentPermissions().then(function (parentPermissions) {
                        _this.parentPermissions = parentPermissions.getEntries();
                        _this.setUpDialog();
                        _this.open();
                    }).catch(function (reason) {
                        api.notify.showWarning('Could not read inherit permissions for content ' + _this.content.getPath().toString());
                    }).done();
                });
                this.addCancelButtonToBottom();
            }
            EditPermissionsDialog.prototype.applyPermissions = function () {
                var _this = this;
                var permissions = new AccessControlList(this.getEntries());
                var req = new api.content.ApplyContentPermissionsRequest().setId(this.content.getId()).setInheritPermissions(this.inheritPermissionsCheck.isChecked()).setPermissions(permissions).setOverwriteChildPermissions(this.overwriteChildPermissionsCheck.isChecked());
                var res = req.sendAndParse();
                res.done(function (updatedContent) {
                    new wizard.ContentPermissionsAppliedEvent(updatedContent).fire();
                    api.notify.showFeedback('Permissions applied to content ' + updatedContent.getPath().toString());
                    _this.close();
                });
            };
            EditPermissionsDialog.prototype.setUpDialog = function () {
                var _this = this;
                this.comboBox.clearSelection(true);
                this.overwriteChildPermissionsCheck.setChecked(false);
                var contentPermissions = this.content.getPermissions();
                var contentPermissionsEntries = contentPermissions.getEntries();
                this.originalValues = contentPermissionsEntries.sort();
                this.originalInherit = this.content.isInheritPermissionsEnabled();
                this.originalValues.forEach(function (item) {
                    if (!_this.comboBox.isSelected(item)) {
                        _this.comboBox.select(item);
                    }
                });
                this.inheritPermissionsCheck.setChecked(this.content.isInheritPermissionsEnabled());
                this.comboBox.giveFocus();
            };
            EditPermissionsDialog.prototype.layoutInheritedPermissions = function () {
                var _this = this;
                this.comboBox.clearSelection(true);
                this.parentPermissions.forEach(function (item) {
                    if (!_this.comboBox.isSelected(item)) {
                        _this.comboBox.select(item);
                    }
                });
            };
            EditPermissionsDialog.prototype.getEntries = function () {
                return this.comboBox.getSelectedDisplayValues();
            };
            EditPermissionsDialog.prototype.getParentPermissions = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var parentPath = this.content.getPath().getParentPath();
                if (parentPath && parentPath.isNotRoot()) {
                    new api.content.GetContentByPathRequest(parentPath).sendAndParse().then(function (content) {
                        deferred.resolve(content.getPermissions());
                    }).catch(function (reason) {
                        deferred.reject(new Error("Inherit permissions for [" + _this.content.getPath().toString() + "] could not be retrieved"));
                    }).done();
                }
                else {
                    new api.content.GetContentRootPermissionsRequest().sendAndParse().then(function (rootPermissions) {
                        deferred.resolve(rootPermissions);
                    }).catch(function (reason) {
                        deferred.reject(new Error("Inherit permissions for [" + _this.content.getPath().toString() + "] could not be retrieved"));
                    }).done();
                }
                return deferred.promise;
            };
            EditPermissionsDialog.prototype.show = function () {
                if (this.content) {
                    this.dialogTitle.setPath(this.content.getPath().toString());
                }
                else {
                    this.dialogTitle.setPath('');
                }
                _super.prototype.show.call(this);
            };
            return EditPermissionsDialog;
        })(api.ui.dialog.ModalDialog);
        wizard.EditPermissionsDialog = EditPermissionsDialog;
        var EditPermissionsDialogTitle = (function (_super) {
            __extends(EditPermissionsDialogTitle, _super);
            function EditPermissionsDialogTitle(title, path) {
                _super.call(this, title);
                this.pathEl = new api.dom.PEl('path');
                this.pathEl.setHtml(path);
                this.appendChild(this.pathEl);
            }
            EditPermissionsDialogTitle.prototype.setPath = function (path) {
                this.pathEl.setHtml(path);
            };
            return EditPermissionsDialogTitle;
        })(api.ui.dialog.ModalDialogHeader);
        wizard.EditPermissionsDialogTitle = EditPermissionsDialogTitle;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var components;
(function (components) {
    components.detailPanel;
})(components || (components = {}));
function startApplication() {
    var application = api.app.Application.getApplication();
    var body = api.dom.Body.get();
    var appBar = new api.app.bar.AppBar(application);
    var appPanel = new app.ContentAppPanel(appBar, application.getPath());
    body.appendChild(appBar);
    body.appendChild(appPanel);
    var contentDeleteDialog = new app.remove.ContentDeleteDialog();
    app.browse.ContentDeletePromptEvent.on(function (event) {
        contentDeleteDialog.setContentToDelete(event.getModels());
        contentDeleteDialog.open();
    });
    var newContentDialog = new app.create.NewContentDialog();
    app.browse.ShowNewContentDialogEvent.on(function (event) {
        var parentContent = event.getParentContent();
        if (parentContent != null) {
            new api.content.GetContentByIdRequest(parentContent.getContentId()).sendAndParse().then(function (newParentContent) {
                if (parentContent.hasParent() && parentContent.getType().isTemplateFolder()) {
                    new api.content.GetContentByPathRequest(parentContent.getPath().getParentPath()).sendAndParse().then(function (grandParent) {
                        newContentDialog.setParentContent(newParentContent);
                        newContentDialog.open();
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                }
                else {
                    newContentDialog.setParentContent(newParentContent);
                    newContentDialog.open();
                }
            }).catch(function (reason) {
                api.DefaultErrorHandler.handle(reason);
            }).done();
        }
        else {
            newContentDialog.setParentContent(null);
            newContentDialog.open();
        }
    });
    var sortDialog = new app.browse.SortContentDialog();
    var moveDialog = new app.browse.MoveContentDialog();
    var editPermissionsDialog = new app.wizard.EditPermissionsDialog();
    application.setLoaded(true);
    window.onmessage = function (e) {
        if (e.data.appLauncherEvent) {
            var eventType = api.app.AppLauncherEventType[e.data.appLauncherEvent];
            if (eventType == 0 /* Show */) {
                appPanel.activateCurrentKeyBindings();
            }
        }
    };
}
//# sourceMappingURL=_all.js.map